<?php
/**
*	Utility script to copy from one eGroupWare installation to another
*	
*	Copies all the interesting tables, and leaves the ones that have site-specific info.
*	The ones to not copy are specified in the $exclude_regex, below.
*	Doesn't take any command-line parameters.
*
*	@author Justin Hallett
*/


## User Editable options
$sourceuser = "";
$sourcepass = "";
$sourcehost = "";
$sourcedb   = "";

## if blank no export file
$exportpath = "./";

## 1 import automaticaly, 0 no import
$import = 0;

$targetuser = "";
$targetpass = "";
$targetuser = "";
$targetdb   = "";

$exclude_regex = ARRAY(
	'egw_access_log',
	'egw_admin_queue',
	'egw_admin_remote',
	'egw_api_content_history',
	'egw_app_sessions',
	'egw_config',
	'egw_etemplate',
	'egw_history_log',
	'egw_interserv',
	'egw_log',
	'egw_log_msg',
	'egw_sessions',
	'egw_vfs',
	'phpgw_*',
	'tmp_*'
);

### DO NOT EDIT BELOW THIS ###

## Get timestamp for file and filename
$timestamp = date(DATE_RFC822);
$filetimestamp = date(DATE_ISO8601);

## Create connection to the source db
$source = new mysqli($sourcehost, $sourceuser, $sourcepass, $sourcedb);
if (mysqli_connect_errno()) {
	printf("Connect failed: %s\n", mysqli_connect_error());
	exit();
}

/* change character set to utf8 */
if (!$source->set_charset("utf8")) {
	printf("Error loading character set utf8: %s\n", $source->error);
} else {
	printf("Current character set: %s\n", $source->character_set_name());
}

## Get tables
$sql_gettables = "SHOW TABLES";
if (!$result = $source->query($sql_gettables)) {
	printf("Error: %s\n", $source->error);
	$source->close();
	exit();
}

## Build new table array without excludes
$export_tables = ARRAY();
while ($row = $result->fetch_row()) {
	$match = 0;
	foreach ($exclude_regex as $regex) {
		if (preg_match('/'.$regex.'/', $row[0])) {
			$match = 1;
		}
	}
	if (!$match) {
		array_push($export_tables, $row[0]);
	}
}
$result->close();

## Build the export to save
$export  = "# Host: $sourcehost\n";
$export .= "# Database: $sourcedb\n";
$export .= "# Timestamp: $timestamp\n";
$export .= "\n";

foreach ($export_tables as $table) {
	## Start table creation with drop if exists
	$export .= "# Dump of table $table\n";
	$export .= "DROP TABLE IF EXISTS `$table`;\n";
	$export .= "\n";

	$sql_create = "SHOW CREATE TABLE `$table`";
	$sql_data   = "SELECT * FROM `$table`";

	## Create Table
	if (!$result = $source->query($sql_create)) {
		printf("Error: %s\n", $source->error);
		$source->close();
		exit();
	}

	$row = $result->fetch_row();
	$export .= $row[1].";\n";
	$export .= "\n";

	$result->close();

	## Insert data (records)
	if (!$result = $source->query($sql_data)) {
		printf("Error: %s\n", $source->error);
		$source->close();
		exit();
	}

	## Get field list
	$finfo = $result->fetch_fields();
	$fields = "";
	foreach ($finfo as $field) {
		$fields .= "`".$field->name."`, ";
	}
	$fields = rtrim($fields, ", ");

	while ($row = $result->fetch_row()) {
		$values = "";
		foreach ($row as $value) {
			$values .= "'".$source->real_escape_string($value)."', ";
		}
		$values = rtrim($values, ", ");
		$export .= "INSERT INTO `$table` ($fields) VALUES ($values);\n";
	}

	$result->close();

	$export .= "\n";
}

$source->close();

if ($import) {
	## Start Import
	print "Importing Export into ${targethost}/${targetdb}\n";

	## Create connection to the target db
	$target = new mysqli($targethost, $targetuser, $targetpass, $targetdb);
	if (mysqli_connect_errno()) {
		printf("Connect failed: %s\n", mysqli_connect_error());
		exit();
	}

	/* change character set to utf8 */
	if (!$target->set_charset("utf8")) {
		printf("Error loading character set utf8: %s\n", $target->error);
	} else {
		printf("Current character set: %s\n", $target->character_set_name());
	}

	if(!$target->query(utf8_encode($export))) {
		printf("Error: %s\n", $target->error);
	}

	$target->close();
}

if (is_dir($exportpath)) {
	## Remove trailing slash if exists
	$exportpath = rtrim($exportpath, "/");

	## Set filename
	$filename = "${sourcehost}-${sourcedb}-${filetimestamp}.sql";

	## filename + path
	$pathfilename = "${exportpath}/${filename}";

	print "Exporting to ${pathfilename}\n";
	$fh = fopen($pathfilename, "w+t");
	fwrite($fh, utf8_encode($export));
	fclose($fh);
}
?>
