<?php
	/**
	*	Ajax functions for editing a purchase order
	*
	*	@author Nathan Gray
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ajax_perp_order.inc.php');
	require_once('class.PurchaseOrder.inc.php');

	class ajax_perp_purchase_order extends ajax_perp_order {

		/**
		*	Reference to bo_perp_catalog for access to supplier catalog
		*/
		private $bostock;

		/**
		*	Constructor\n
		*	Creates references to bo_perp_catalog and xajax
		*/
		public function __construct() {
			$this->bostock = new bo_perp_catalog();
		}

		/**
		*	Set the line details when the user chooses an item to add to this Order
		*
		*	@param supplier_id	int	Supplier's ID
		*	@param supplier_code	string	Supplier's identification number for the selected catalog item
		*	@param address		int	The address that this purchase order will be shipped to
		*       @return xml response
		*/
		public function get_line_details($supplier_id, $supplier_code, $address) {
			$xml = parent::get_line_details($supplier_id, $supplier_code, $address);

			// Restrict the units of measure to our UOM for the item and their UOM for the item
			$item = $this->bostock->get_catalog_item($supplier_id, $supplier_code);

			$uom = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			// Clear all UOMs
			$this->response->addScript("removeOptions('exec[new][unit_of_measure]')");

			$their_uom_name = $uom[$item['supplier_uom']];

			// Add our unit of measure, if it's one of our stock items
			if($item['stock_id'] && ($item['quantity'] * $item['conversion_factor']) != 1) {
				$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $item['stock_id']);
				$their_uom_name .= ' (' . ($item['conversion_factor'] * $item['quantity']) . ' ' . $uom[$stock_item['unit_of_measure']] . ')';
			}
			

			/**
			*	Add options to selectbox 
			*	Their unit of measure has to go first, because that's what the 
			*	unit price (already set) is for.
			*/
			$this->response->addScript(
				"addOption('exec[new][unit_of_measure]', '{$item['supplier_uom']}', '" . addslashes($their_uom_name) . "')"
			);
			if($item['stock_id'] && ($item['quantity'] * $item['conversion_factor']) != 1) {
				$this->response->addScript(
					"addOption('exec[new][unit_of_measure]', '" . $stock_item['unit_of_measure'] . "', '" . addslashes($uom[$stock_item['unit_of_measure']]) . "')"
				);
			}

			return $this->response->getXML();
		}

		/**
		*	User changed the unit of measure, so we'll re-calculate the unit cost using the conversion
		*/
		public function change_unit_of_measure($supplier_id, $supplier_code, $unit_of_measure_id) {
			$this->response = new xajaxResponse();

			$item = $this->bostock->get_catalog_item($supplier_id, $supplier_code);
			if($item['stock_id']) {
				$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $item['stock_id']);
			}
			
			$cost = 0;
			if($unit_of_measure_id == $item['supplier_uom']) {
				$cost = $item['price'];
			} elseif ($unit_of_measure_id == $stock_item['unit_of_measure']) {
				$cost = round($item['price'] / ($item['conversion_factor'] * $item['quantity']), 2);
			}
			$this->response->addScript('
				var unit_price = document.getElementById("exec[new][unit_price]");
				if(unit_price) {
					unit_price.value = "' . $cost . '";
				}'
			);

			return $this->response->getXML();
		}
	}
?>
