<?php

	/**
	*	Ajax functions for receiving 
	*
	*	@package perp_ap
	*	@author Nathan Gray
	*/

	require_once(EGW_API_INC.'/xajax.inc.php');
	require_once(EGW_INCLUDE_ROOT.'/perp_ap/inc/class.bo_perp_catalog.inc.php');
	require_once(EGW_INCLUDE_ROOT.'/perp_ap/inc/class.PurchaseOrder.inc.php');

	class ajax_perp_receive {

		protected $response;

		public function __construct() {
			$this->response = new xajaxResponse();
		}

		/**
		*	If the user selects a Purchase Order to receive,
		*	check that the expected delivery location matches the 
		*	location they're about to receive it into.
		*
		*	@param location_id The location they're about to receive the order into
		*	@param order_id The system's ID for the purchase order being received
		*
		*	@return empty or error message (in XML from an xajaxResponse)
		*/
		public function check_order_location($location_id, $order_id) {
			
			$order = new PurchaseOrder($order_id);
			if(!$location_id) {
				$this->response->addAlert(
					lang('Please choose your location first')
				);
			} else if($location_id == $order->get_location_id()) {
				// Ok, do nothing
			} else {
				$this->response->addAlert(
					lang('That Purchase Order (%1) was supposed to be shipped to %2, not %3',
						$order->get_our_reference_number(),
						ExecMethod('perp_inventory.bo_perp_location.link_title', $order->get_location_id()),
						ExecMethod('perp_inventory.bo_perp_location.link_title', $location_id)
					)
				);
			}

			return $this->response->getXML();
		}

		/**
		*	When the user changes the stock code, change the unit of measure
		*/
		public function change_stock_code($supplier_id, $stock_code) {
			$item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item',$supplier_id, $stock_code);
			$uom = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			$uom_location = etemplate::form_name($cname='exec', 'unit_of_measure');

			$unit_of_measure = '';
	
			if($item['stock_id']) {
				$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $item['stock_id']);
				$unit_of_measure = $uom[$stock_item['unit_of_measure']] .
					' ('  . ($item['conversion_factor'] * $item['quantity']) . ' ' . 
					$uom[$stock_item['unit_of_measure']] .
					' / ' . $uom[$item['supplier_uom']] . ')';

				// If controlled, show serial numbers button
				$this->display_serial_button(($stock_item['controlled']), ($stock_item['serial_setup'] != bo_perp_stock::SERIAL_WHEN_NEEDED));
				$this->response->addAssign(etemplate::form_name('exec', 'serial_numbers[stock_id]'), 'value', $item['stock_id']);
			} else {
				$this->display_serial_button(false);
			}

			$this->response->addAssign($uom_location, 'innerHTML', $unit_of_measure);
			
			return $this->response->getXML();
		}

		protected function display_serial_button($display = true, $required = true) {
			$button = etemplate::form_name('exec', 'serial_numbers[button]');
			$this->response->addScript("document.getElementById('$button').parentNode.style.visibility = '" . ($display ? 'visible' : 'collapse') . "'");
			$this->response->addScript("document.getElementById('$button').parentNode.style.border = '" . ($required ? '1px solid red' : 'none') . "'");
			$this->response->addScript("document.getElementById(etemplate::form_name('exec', 'serial_numbers[required]').value = '$required'");
		}
	}

?>
