<?php
	/**
	*	pERP Accounts Payable
	*	Logical functions for Accounts Payable
	*	
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	require_once('class.PurchaseOrder.inc.php');

	class bo_perp_ap {
		/**
		*	Reference to storage layer object
		*/
		private $so;

		/**
		*	Constructor\n
		*	Create references
		*/
		public function bo_perp_ap() {
			$this->so = CreateObject('perp_ap.so_perp_ap');
		}

		/**
		* get title for a perp Accounts Payable entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param entry string/array	key to look up or array with supplier entry
		*	key is in the form {SUPPLIER | PO | SHIPMENT | PAYMENT}:id
		* @return string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				list($location, $id) = explode('-', $entry);
				if($id) {
					switch ($location) {
						case 'SUPPLIER':
							return ExecMethod('perp_ap.bo_perp_supplier.link_title', $id);
							break;
						case 'PO':
						case 'ORDER':
							return ExecMethod('perp_ap.bo_perp_purchase_order.link_title', $id);
							break;
						case 'SHIPMENT':
							return ExecMethod('perp_ap.bo_perp_incoming_shipment.link_title', $id);
							break;
						case 'INVOICE':
							return ExecMethod('perp_ap.bo_perp_payable_invoice.link_title', $id);
							break;
						case 'PAYMENT':
							return ExecMethod('perp_ap.bo_perp_payment.link_title', $id);
							break;
					}
				} else {
					return ExecMethod('perp_ap.bo_perp_supplier.link_title', $entry);
				}
			} else {
				if(isset($entry['supplier_id'])) {
					return ExecMethod('perp_ap.bo_perp_supplier.link_title', $entry);
				} elseif(isset($entry['purchase_order_id'])) {
					return ExecMethod('perp_ap.bo_perp_purchase_order.link_title', $entry);
				} elseif(isset($entry['shipment_id'])) {
					return ExecMethod('perp_ap.bo_perp_incoming_shipment.link_title', $entry);
				} elseif(isset($entry['invoice_id'])) {
					return ExecMethod('perp_ap.bo_perp_invoice.link_title', $entry);
				} elseif(isset($entry['payment_id'])) {
					return ExecMethod('perp_ap.bo_perp_payment.link_title', $entry);
				}
					
			}
			if (!$entry) {
				return $entry;
			}
		}

	
		/**
		* query perp clients for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param pattern string Pattern to search
		* @return array with client_id - title pairs of the matching entries
		*/
		function link_query( $pattern ) {
			list($location, $pattern) = explode('-', $pattern, 2);
			if(!$pattern) {
				$pattern = $location;
				unset($location);
			} else {
				$location = strtoupper($location);
			}
			$list = array();
			switch (strtoupper($location)) {
				case 'S':
				case 'SUPPLIER':
				case strtoupper(lang('supplier')):
					$list = ExecMethod('perp_ap.bo_perp_supplier.link_query', $pattern);
					break;
				case 'PO':
				case strtoupper(lang('purchase order')):
					$list = ExecMethod('perp_ap.bo_perp_purchase_order.link_query', $pattern);
					break;
				case 'I':
				case strtoupper(lang('invoice')):
					$list = ExecMethod('perp_ap.bo_perp_payable_invoice.link_query', $pattern);
					break;
				default:
					$list = ExecMethod('perp_ap.bo_perp_supplier.link_query', $pattern);
					$list += ExecMethod('perp_ap.bo_perp_purchase_order.link_query', $pattern);
					$list += ExecMethod('perp_ap.bo_perp_payable_invoice.link_query', $pattern);
					break;
			}
			return $list;
		}

		/**
		*	Add a line to an open PurchaseOrder.
		*	Create a new PurchaseOrder if there is no open PurchaseOrder for the supplier.
		*
		*	@param	supplier	int	Supplier ID
		*	@param	stock_code	string	Supplier's reference number for the item to add to the PurchaseOrder
		*	@param	quantity	float	Quantity to add to the order
		*	@return string Success message or false on failure
		*/
		public function add_to_po($supplier, $stock_code, $quantity) {
			// Get PO
			$order_id = ExecMethod('perp_ap.bo_perp_purchase_order.get_open_order',$supplier);
			if($order_id) {
				$po = new PurchaseOrder($order_id);
			} else {
				$po = new PurchaseOrder();
				$po->set_supplier_id((int)$supplier);
				$po->set_our_reference_number($po->suggest_order_number());
				if(!$po->get_ship_to_address()) {
					// Default ship_to address to the first location we can find
					$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
					if(key($locations)) {
						$contacts = ExecMethod2('phpgwapi.bolink.get_links', 'perp_inventory', 'LOCATION-' . key($locations), 'addressbook');
						$po->set_ship_to_address((int)current($contacts));
					}
				}
			}

			if($po) {
				$items = array();
				$bocatalog = CreateObject('perp_ap.bo_perp_catalog');
				$item = $bocatalog->get_catalog_item($supplier, $stock_code);
				$line_data = array(
					'stock_id'		=>	$item['stock_id'],
					'supplier_stock_code'     =>      $stock_code,
					'description'   =>      $item['description'],
					'unit_price'    =>      $item['price'],
					'quantity_ordered'	=>	$quantity,
				);
				$line = new PurchaseOrderLine($po, $line_data);
				$po->add_line($line);
				$po_id = $po->write();
				return lang('Item added to %1 purchase order %2', 
					($order_id ? lang('existing') : lang('new')), 
					html::a_href(
						$po->get_our_reference_number(),
						$GLOBALS['egw']->link('/index.php', array(
							'menuaction'	=>	'perp_ap.ui_perp_purchase_order.edit',
							'order_id'	=>	$po_id
						))
					)
				);
			}
			return False;
		}
	}
?>
