<?php
	/**
	*	Logic functions for dealing with supplier invoices
	*
	*	@ingroup perp_ap
	*	@author Nathan Gray
	*/
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_invoice.inc.php');
	require_once('class.so_perp_payable_invoice.inc.php');

	class bo_perp_payable_invoice extends bo_perp_invoice {

		/**
		*	boolean enable for debug messages
		*/
		private $debug = false;

		/**
		*	Constructor
		*/
		public function __construct() {
			$this->so_perp_invoice = new so_perp_payable_invoice();
		}

		/**
		*	Get a list of invoices
		*	Also returns the Invoice object in the results
		*
		*	@nextmatch
		*/
		public function get_invoice_list($query = array(), &$rows = array(), &$readonlys = array()) {
			if($query['filter2']) {
				$query['supplier_id'] = $query['filter2'];
			}

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($query['supplier_id'], $supplier_ids)) {
				$query['supplier_id'] = $supplier_ids;
			}


			// Only payable invoices, not receivable
			$query['client_id'] = 0;

			$read = $GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ap');
			if(!$read) {
				if(!$query['template']) {
					return array();
				} else {
					return 0;
				}
			}

			$count = parent::get_invoice_list($query, $rows, $readonlys);

 			$rows['no_actions'] = !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_EDIT, 'perp_ap');
			$rows['no_actions'] |= !$GLOBALS['egw']->acl->check('payment', EGW_ACL_ADD, 'perp_ap');

			// Copy disabled columns into the result
			foreach($query as $key => $value) {
				if(strpos($key, 'no_') === 0) {
					$rows[$key] = $value;
				}
			}
			return $count;
		}

		/**
		*	Get a list of possible statuses for Invoices
		*
		*	@return array of statuses
		*/
		public function get_status_list() {
			return array (
				'all'		=>	lang('All'),
				'unposted'	=>	lang('Not posted'),
				'due'		=>	lang('Due'),
				'overdue'	=>	lang('Overdue'),
				'open'		=>	lang('Open'),
				'closed'	=>	lang('Closed'),
			);
		}
	
		/**
		*	Get title for a perp invoice entry identified by $entry
		*                               
		* Is called as hook to participate in the linking                
		*
		* @param int/array $entry key to look up or array with invoice entry
		* @return string with title/boolean
		*/

		public function link_title($entry) {
			if(is_a($entry, 'Invoice')) {
				$title = $entry->get_our_reference_number();
				if($entry->get_their_reference_number()) {
					$title .= '(' . $entry->get_their_reference_number() . ')';
				}
				return $title;
			} elseif( is_array($entry)) {
				return $this->link_title($entry['invoice_id']);
			} elseif(is_numeric($entry) && $entry != 0) {
				$invoice = new PayableInvoice($entry);
				return $this->link_title($invoice);
			}
			return false;
		}

		/**
		*	Participate in linking
		*	
		*	@pattern is a pattern to search for in the Purchase Orders
		*	
		*	@return array of matching entries, with keys like ORDER-<id>
		*/
		public function link_query($pattern) {
			$query = array(
				'search'	=>	$pattern,
				'num_rows'	=>	-1
			);
			$list = array();
			// Need this swap for permission check to pass
			$old_app = $GLOBALS['egw_info']['flags']['currentapp'];
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ap';

			$this->get_invoice_list($query, $list);

			$GLOBALS['egw_info']['flags']['currentapp'] = $old_app;

			$result = array();
			foreach($list as $invoice) {
				$result['INVOICE-' . $invoice['invoice_id']] = $this->link_title($invoice['invoice_id']);
			}
			return $result;
		}
	}
?>
