<?php

	/**
	*	Logic level object for supplier
	*	@author Nathan gray
	*	@todo Supplier should have an associated object
	*	@ingroup perp_ap
	*/
	class bo_perp_supplier {
		/**
		*	Reference to storage layer object
		*/
		private $so;

		/**
		*	eGroupWare config object for site wide configuration
		*/
		private $config;

		/**
		*	Constructor
		*/
		public function bo_perp_supplier() {
			$this->so = CreateObject('perp_ap.so_perp_supplier');
			$this->config = config::read('perp_ap');
		}

		/**
		*	Get a supplier 
		*	@param supplier_id	int	ID of the supplier
		*	@return array of supplier information
		*/
		public function get_supplier($supplier_id) {
			if($supplier_id && $GLOBALS['egw']->acl->check('supplier', EGW_ACL_READ, 'perp_ap')) {
				$supplier = array();
				$supplier = $this->so->get_supplier($supplier_id);

				// Check if the current user is a supplier
				$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
				if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($supplier_id, $supplier_ids)) {
					return array();
				}

				if($supplier['supplier_address']) {
					$supplier['address'] = $supplier['supplier_address'];
				}
				return $supplier;
			} else {
				return array();
			}
		}
		
		
		/**
		*	Populate Status Select Box Values
		*/
		public function get_status_list() {
			return array(
				1	=>	lang('Active'),		// Normal supplier, can do stuff
				0	=>	lang('Disabled')	// No longer a supplier.
			);
		}

		/**
		*	Get a list of suppliers
		*
		*	@nextmatch
		*/
		public function get_suppliers($query = array(), &$rows = array(), &$readonly = array()) {
			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($query['supplier_id'], $supplier_ids)) {
				$query['supplier_id'] = $supplier_ids;
			}

			if(!$GLOBALS['egw']->acl->check('supplier', EGW_ACL_READ, 'perp_ap')) {
				$rows = array();
				if(count($query) == 0) {
					return array();
				} else {
					return 0;
				}
			}

			if(!$query['order']) {
				$query['order'] = 'supplier_num';
			}
			if(!$query['num_rows']) {
				$query['num_rows'] = -1;
			}
			$list = $this->so->get_suppliers($query);
			if(!isset($query['template'])) {
				// For a select box
				$rows = array();
				foreach($list as $row) {
					$rows[$row['supplier_id']] = $row['supplier_num'] . ' - ' . $row['supplier_name'];
				}
				return $rows;
			}
			
			$edit = $GLOBALS['egw']->acl->check('supplier', EGW_ACL_EDIT, 'perp_ap');
			$delete = $GLOBALS['egw']->acl->check('supplier', EGW_ACL_DELETE, 'perp_ap');
			$rows = array('action' => $query['filter'], 'edit' => $edit);
			foreach($list as $row) {
				$rows[] = $row;
				$readonly['edit[' . $row['supplier_id'] . ']'] = !$edit;
				$readonly['delete[' . $row['supplier_id'] . ']'] = !$delete;
			}

			return $query['count'];
		}

		/**
		*	Set a supplier's information
		*	@param supplier_id	int	ID of the supplier to set, or null for a new supplier
		*	@param supplier		array	Supplier's information
		*	@return int ID of the supplier
		*/
		public function set_supplier($supplier_id, $supplier) {
			$edit = $GLOBALS['egw']->acl->check('supplier', EGW_ACL_EDIT, 'perp_ap');

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($supplier_id, $supplier_ids)) {
				$edit = false;
			}
			if(!$edit) {
				return false;
			}


			if(!$supplier['supplier_name']) {
				return lang('Missing required information');
			}
			if(!$supplier['supplier_num']) {
				return lang('Missing supplier number');
			}
			if(!$supplier['supplier_since']) {
				$supplier['supplier_since'] = date('U');
			}
			if(!$supplier['supplier_currency']) {
				$supplier['supplier_currency'] = key(ExecMethod('perp_api.bo_perp_api.get_currencies'));
			}
			if(!$supplier['payment_terms']) {
				$supplier['payment_terms'] = key(ExecMethod('perp_api.bo_perp_api.get_payment_terms'));
			}
			if($this->config['supplier_user_accounts']) {
				if(!$supplier['user_account']) {
					// Create user account
					$this->new_account['account_lid'] = $supplier['supplier_num'];
					$this->new_account['account_firstname'] = $supplier['supplier_name'];
					$this->new_account['account_lastname'] = lang('Supplier');
					$supplier['user_account'] = $GLOBALS['egw']->accounts->create($this->new_account, True);
					$GLOBALS['egw']->acl->add_repository('phpgw_group', $this->new_account['account_primary_group'], $supplier['user_account'], EGW_ACL_READ);
				}
			} else {
				$this->disable_supplier_user_account($supplier_id);
			}

			$id = $this->so->set_supplier($supplier_id, $supplier);
			if(!$supplier_id) {
				$this->create_contact($id, $supplier['supplier_name']);
			}
			return $id;
		}

		/**
		*	Delete a supplier.
		*	@param supplier_id	int	ID of the supplier to delete
		*	
		*/
		public function delete_supplier($supplier_id) {
			if($GLOBALS['egw']->acl->check('supplier', EGW_ACL_DELETE, 'perp_ap')) {
				ExecMethod2('phpgwapi.bolink.unlink2', null, 'perp_ap', $supplier_id);
				return $this->so->delete_supplier($supplier_id);
			}
		}

		/**
		* get title for an perp supplier entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param entry int/array key to look up or array with supplier entry.
		*	key is in the form {SUPPLIER | PO | SHIPMENT | PAYMENT}:id
		* @return string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				$entry = $this->get_supplier($entry);
				return $entry['supplier_num'].' - '.$entry['supplier_name'];
			} else {
				if(isset($entry['supplier_id'])) {
					return $entry['supplier_num'].' - '.$entry['supplier_name'];
				}
			}
			if (!$entry) {
				return $entry;
			}
		}

	
		/**
		* query perp suppliers for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param pattern string pattern to search
		* @return array with client_id - title pairs of the matching entries
		*/
		function link_query( $pattern ) {		
			$list = $this->so->link_query($pattern);

			$result = array();
			foreach($list as $list_el) {
				$result['SUPPLIER-' .$list_el['supplier_id']] = $this->link_title($list_el);
			}
			return $result;
		}

		/**
		*	Get a supplier's account balance
		*
		*	@param supplier_id
		*/
		public function get_account_balance($supplier_id) {
			if(!$GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ap')) {
				return 0;
			}
			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($supplier_id, $supplier_ids)) {
				$supplier_id = $supplier_ids;
			}

			return $this->so->get_account_balance($supplier_id);
		}

		/**
		*	Get the Supplier Ledger entries for a particular supplier
		*
		*	@nextmatch
		*/
		public function get_supplier_ledger(&$query, &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ap')) {
				$rows = array();
				return 0;
			}
			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($query['supplier_id'], $supplier_ids)) {
				$query['supplier_id'] = $supplier_ids;
			}

			if(!$query['order']) {
				$query['order'] = 'trans_date';
			}
			$list = $this->so->get_supplier_ledger($query);

			$rows = $list;
			return count($list);
		}

		/**
		*	Disables the user account for the given supplier
		*/
		public function disable_supplier_user_account($supplier_id) {
			$supplier = $this->get_supplier($supplier_id);
			if($supplier['user_account']) {
				$account =& CreateObject('phpgwapi.accounts', $branch['user_account']);
				$account_data = $account->read();
				$account_data['account_status'] = '';
				$account->update_data($account_data);
			}
		}

		/**
		*	Get an array of supplier_ids associated with the given account_id(s)
		*
		*	An empty array means the account is not associated with a supplier.  That
		*	usually means a staff member, so access should not be restricted based on this.
		*	Use normal ACLs.
		*
		*	This is an example of how to check if the current user is allowed to access the $supplier:
		*	@code
		*	$supplier_ids = $bo_perp_supplier->get_supplier_by_user_account();
		*	if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($supplier['supplier_id'], $supplier_ids)) {
		*		<deny>
		*	}
		*	@endcode
		*	
		*	@param account_id Account IDs to check for (usually current user & user's groups), or false for current user
		*	@return array of supplier_ids that match
		*/
		public function get_supplier_by_user_account($account_id = false) {
			if($account_id) {
				return $this->so->get_supplier_by_user_account($account_id);
			} else {
				$account_id = $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'],true);
				foreach($account_id as &$id) {
					$id = abs($id);
				}
				$account_id[] = $GLOBALS['egw_info']['user']['account_id'];
				return $this->so->get_supplier_by_user_account($account_id);
			}
		}

		/**
		*	Create an contact, or link any existing contacts automatically
		*/
		protected function create_contact($supplier_id, $name) {
			// Look for an existing contact with the same name
			$bo_contacts = new addressbook_bo();

			$create_new = true;
			$contacts = $bo_contacts->link_query("'$name'");
			if(count($contacts) > 0) {
				foreach($contacts as $contact_id => $contact_name) {
					$contact = $bo_contacts->read($contact_id);
					if($contact['org_name'] == $name || stripos($contact['n_fileas'], $name) !== false) {
						ExecMethod2('phpgwapi.bolink.link', 'perp_ap', 'SUPPLIER-' . $supplier_id, 'addressbook', $contact_id);
						$create_new = false;
					}
				}
			}
			if($create_new) {
				$config = config::read('perp_ap');

				// Create new contact
				$contact = array(
					'cat_id'        =>      $config['address_category'],
					'org_name'      =>      $name,
					'fileas'        =>      $name,
					'owner'		=>	$bo_contacts->default_addressbook,
				);
				$contact_id = $bo_contacts->save($contact, true);
				$link = ExecMethod2('phpgwapi.bolink.link', 'perp_ap', 'SUPPLIER-' . $supplier_id, 'addressbook', $contact_id);

				// Open contact for editing (won't work if we redirect for a new supplier)
				if(!is_object($GLOBALS['egw']->js)) {
					$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
				}

				$href = $GLOBALS['egw']->link('/index.php',array(
					'menuaction'    =>      'addressbook.addressbook_ui.edit',
					'contact_id'    =>      $contact_id
				));
				$GLOBALS['egw']->js->set_onload("window.open('$href','_blank','dependent=yes,width=850x440,height=450,scrollbars=yes,status=yes'); ");
			}

			// Check if this is the only linked address & save it as the address
			$addresses = ExecMethod2('phpgwapi.bolink.get_links', 'perp_ap', 'SUPPLIER-' . $supplier_id, 'addressbook');
			if(count($addresses) == 1) {
				$supplier = $this->get_supplier($supplier_id);
				$supplier['supplier_address'] = $contact_id;
				$supplier['supplier_status'] = true;
				$this->set_supplier($supplier_id, $supplier);
			}
		}
	}
?>
