<?php
	/**
	*	Data storage layer for supplier catalogs
	*	@ingroup perp_ap
	*	@author Nathan Gray
	*/
	class so_perp_catalog {
		/**
		*	Data access object
		*/
		private $db;

		/**
		*	Constructor
		*/
		public function so_perp_catalog() {
			$this->db = clone($GLOBALS['egw']->db);
		}

		/**
		*	Get the information about a single catalog item.
		*	@param supplier_id	int	The ID of the supplier whose catalog we're looking in
		*	@param supplier_code	string	The supplier's reference for this item
		*	@param stock_id		int	Optional.  Our stock_id, if this item is one of our stock items
		*
		*	@return array Information on a matching catalog item
		*/
		public function get_catalog_item($supplier_id, $supplier_code, $stock_id = '') {

			if($supplier_code) {
				$where['supplier_code'] = $supplier_code;
			}
			if($stock_id) {
				$where[] = 'perp_stock.stock_id = ' . $stock_id;
			}
			$where['supplier_id'] = $supplier_id;

			$this->db->select('perp_supplier_stock', array(
				'perp_supplier_stock.stock_id',
				'stock_num',
				'perp_stock.unit_of_measure AS stock_uom',
				'supplier_code',
				'price',
				'average_indirect_cost',
				'description',
				'supplier_uom',
				'conversion_factor',
				'lead_time',
				'quantity',
				'preferred'),
				$where, __LINE__, __FILE__,
				False, null, 'perp_ap', 0,
				'LEFT JOIN perp_stock ON
					perp_supplier_stock.stock_id = perp_stock.stock_id
				'
			);
			$item = array();
			if($row = $this->db->row(True)) {
				$item = $row;
			}
			return $item;
		}

		/**
		*	Set the information about a single Supplier's catalog item.
		*	@param supplier_id	int	The ID of the supplier whose catalog we're looking in
		*	@param supplier_code	string	The supplier's reference for this item
		*	@param stock_id		int	Optional.  Our stock_id, if this item is one of our stock items
		*/
		public function set_catalog_item($supplier_id, $supplier_code, $item) {
			if(!$supplier_id) {
				return false;
			}
			$this->db->insert('perp_supplier_stock', array(
				'stock_id' 	=>	$item['stock_id'],
				'supplier_code'	=>	$supplier_code,
				'supplier_id'	=>	$supplier_id,
				'price'		=>	$item['price'],
				'average_indirect_cost'	=>	$item['average_indirect_cost'],
				'description'	=>	$item['description'],
				'supplier_uom'	=>	$item['supplier_uom'],
				'quantity'	=>	$item['quantity'],
				'conversion_factor'	=>	$item['conversion_factor'],
				'lead_time'	=>	$item['lead_time'],
				'preferred'	=>	$item['preferred']),
				array(
					'supplier_id'	=>	$supplier_id,
					'supplier_code'	=>	$supplier_code
				),
				__LINE__, __FILE__, 'perp_ap'
			);
		}

		/**
		*	Get a list of all items in a Supplier's catalog
		*
		*	@nextmatch
		*/
		public function get_supplier_catalog_list(&$query = array()) {
			$where = array();
			if($query['search']) {
				$where[] = "(supplier_code LIKE '%" . addslashes($query['search']) . "%' OR ".
					"perp_supplier_stock.description LIKE '%" . addslashes($query['search']) . "%' OR " .
					"stock_num LIKE '%" . addslashes($query['search']) . "%' OR " .
					"short_description LIKE '%" . addslashes($query['search']) . "%')";
			}
	
			if($query['stock_category']) {
				$where[] = 'stock_category = ' . $query['stock_category'];
			}
			if($query['supplier_id']) {
				$where[] = 'perp_supplier_stock.supplier_id = ' . (int)$query['supplier_id'];
			}

			// By default, don't show discontinued stock
			if(!$query['status_flag']) {
				$where[] = "(perp_stock.status_flag IN ('E', 'P') OR perp_stock.status_flag IS NULL )";
			} elseif ($query['status_flag']) {
				$where[] = '(perp_stock.status_flag IN (\'' . implode("','", $query['status_flag']) . "') OR perp_stock.status_flag IS NULL )";
			}

			// Get a list of un-received Purchase Orders
			$in_use_query = array(
				'supplier_id'	=>	$query['supplier_id'],
				'status'	=>	'not_received'
			);
			$list = array();
			$bo_po = CreateObject('perp_ap.bo_perp_purchase_order');
			$bo_po->get_order_list($in_use_query, $list);
			$open_orders = array();
			foreach($list as $po_info) {
				$open_orders[] = $po_info['order_id'];
			}

			$fieldlist = array(
				'perp_supplier.supplier_id',
				'perp_stock.stock_num',
				'perp_stock.short_description',
				'perp_stock.unit_of_measure AS stock_uom',
				'stock_uom.uom_name AS stock_uom_name',
				'perp_supplier_stock.stock_id',
				'supplier_code',
				'price',
				'supplier_currency',
				'perp_supplier_stock.description',
				'supplier_uom',
				'supplier_uom.uom_name AS supplier_uom_name',
				'conversion_factor',
				'lead_time',
				'quantity',
				'preferred',
				'total_on_hand',
				'decimal_places',
			);
			if(count($open_orders) > 0) {
				$fieldlist[] = 'perp_order_line.order_id IS NOT NULL AS in_use';
			}
			if($query['selectcols']) {
				$fieldlist[] = $query['selectcols'];
			}

			$category = $query['stock_category'];
			unset($query['stock_category']);
			$join = 'LEFT JOIN perp_stock 
					ON (perp_stock.stock_id = perp_supplier_stock.stock_id)
				LEFT JOIN perp_unit_of_measure AS stock_uom ON
					perp_stock.unit_of_measure = stock_uom.uom_id ' .
				(!$category ? 'LEFT ' : ' ') . 'JOIN egw_categories 
					ON (perp_stock.stock_category = egw_categories.cat_id ' .
				($category ? " AND perp_stock.stock_category = $category" : '') . ')
				LEFT JOIN perp_unit_of_measure AS supplier_uom ON
					perp_supplier_stock.supplier_uom = supplier_uom.uom_id 
				LEFT JOIN (
					SELECT 
						stock_id,
						sum(quantity_on_hand) AS total_on_hand
					FROM perp_location_stock
					GROUP BY stock_id
				) AS total_inventory ON
					total_inventory.stock_id = perp_supplier_stock.stock_id

				JOIN perp_supplier ON
					perp_supplier.supplier_id = perp_supplier_stock.supplier_id';
			
			// Quantities on hand
			if(strstr($query['selectcols'], 'quantity_') !== false || strstr($query['selectcols'], 'inventory') !== false) {
				$join .= ' LEFT JOIN (
					SELECT stock_id,
						sum(quantity_on_hand + quantity_reserved) AS inventory,
						sum(quantity_on_hand) AS quantity_on_hand,
						sum(quantity_reserved) AS quantity_reserved
					FROM perp_location_stock ';
				if($query['col_filter']['inventory']) {
					$join .= ' WHERE location_id = ' . (int)$query['col_filter']['inventory'];
				}
				$join .= '      GROUP BY stock_id
					) AS inventory ON
						inventory.stock_id = perp_supplier_stock.stock_id';
			}

			// Get number of rows
			$query['count'] = $GLOBALS['egw']->db->query(
				"SELECT count(supplier_code) AS count
				FROM perp_supplier_stock " . $join . 
				(count($where) > 0 ? ' WHERE ' . $GLOBALS['egw']->db->column_data_implode(' AND ',$where) : ''),
				__LINE__, __FILE__
			)->fetchSingle();

			/**
			*	Look for in-use items, these can't be changed
			*	This is added to the join after the count query because this join requires a GROUP BY,
			*	which changes the count
			*/
			if(count($open_orders) > 0) {
				$join .= ' LEFT JOIN perp_order_line ON
					perp_order_line.order_id IN (' . implode(',', $open_orders) . ') AND
					perp_order_line.supplier_stock_code = perp_supplier_stock.supplier_code ';
			}

			$result = $GLOBALS['egw']->db->query(
				'SELECT ' . implode(',', $fieldlist) . 
				' FROM perp_supplier_stock ' . $join .
				(count($where) > 0 ? ' WHERE ' . $GLOBALS['egw']->db->column_data_implode(' AND ',$where) : '') . 
				'GROUP BY perp_supplier_stock.supplier_code ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				__LINE__, __FILE__, 
				$query['start'], $query['num_rows']
			);

			$list = array();

			foreach($result as $row) {
				$row['escaped_supplier_code'] = urlencode($row['supplier_code']);
				$row['formatted_price'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $row['supplier_currency'], $row['price']);
				$row['quantity_our_uom'] = round($row['quantity'] * $row['conversion_factor'], $row['decimal_places']);
				if(strstr($query['selectcols'], 'quantity_') !== false) {
					$row['inventory'] = number_format($row['inventory'], $row['decimal_places']);
					$row['quantity_on_hand'] = number_format($row['quantity_on_hand'], $row['decimal_places']);
					$row['quantity_reserved'] = number_format($row['quantity_reserved'], $row['decimal_places']);
				}
				$row['add_prompt'] = htmlspecialchars(lang(
					"We order %5 in %3 (%2 / %1).  How many %5 to order (%3)?",
					$row['supplier_uom_name'],
					$row['quantity_our_uom'],
					$row['stock_uom_name'],
					$row['quantity'],
					addslashes($row['supplier_code'])
				));
				$list[] = $row;
			}
			return $list;
		}
		
		/**
		*	Delete an item from a supplier's catalog
		*	@param supplier_id	int	Supplier whose catalog we're editing
		*	@param item_code	string	Supplier's reference for the item to remove
		*/
		public function delete($supplier_id, $item_code) {
			$this->db->delete('perp_supplier_stock', array(
				'supplier_id'	=>	$supplier_id,
				'supplier_code'	=>	$item_code),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_supplier_ship_package', array(
				'supplier_id'	=>	$supplier_id,
				'supplier_code'	=>	$item_code),
				__LINE__, __FILE__
			);
		}

		/**
		*	Get the list of shipping packages.
		*	There's usually only 1 entry, but if something is shipped
		*	in several boxes, there will be 1 entry for each box.
		*
		*	@param supplier_id ID of the supplier
		*	@param supplier_code Supplier's stock code
		*/
		public function get_shipping_packages($supplier_id, $supplier_code) {
			$GLOBALS['egw']->db->select(
				'perp_supplier_ship_package',
				array(
					'*',
					'round(length * width * height, 3) AS volume_m'
				),
				array(
					'supplier_id'	=>	$supplier_id,
					'supplier_code'	=>	$supplier_code
				),
				__LINE__, __FILE__,0,
				'ORDER BY mass DESC',
				'perp_ap'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Set the list of shipping packages for a supplier's item.
		*	Clears the list, then sets it again.
		*/
		public function set_shipping_packages($supplier_id, $supplier_code, $packages) {
			$GLOBALS['egw']->db->delete(
				'perp_supplier_ship_package',
				array(
					'supplier_id'	=>	$supplier_id,
					'supplier_code'	=>	$supplier_code
				),
				__LINE__, __FILE__,
				'perp_ap'
			);
			foreach($packages as $package) {
				$GLOBALS['egw']->db->insert(
					'perp_supplier_ship_package',
					array(
						'supplier_id'	=>	$supplier_id,
						'supplier_code'	=>	$supplier_code
					) + $package,
					array(),
					__LINE__, __FILE__,
					'perp_ap'
				);
			}
		}

		/**
		*	Get an average of the indirect costs associated with buying a supplier's item.
		*
		*	This includes things such as shipping costs, duties and taxes.  The actual numbers
		*	are calclulated using all kinds of different methods from all kinds of unknown information.
		*	Here, we just average across historical data as recorded in perp_stock_cost over the past year.
		*	This requires the stock_id.
		*/
		public function calculate_average_indirect_cost($supplier_id, $stock_id) {
			$GLOBALS['egw']->db->select(
				'perp_supplier_stock',
				array(
					'AVG(perp_stock_cost.quantity * indirect_cost) / SUM(perp_stock_cost.quantity) AS indirect_cost'
				),
				array(
					'supplier_id'	=>	$supplier_id,
					'perp_supplier_stock.stock_id = ' . $GLOBALS['egw']->db->quote($stock_id),
					'date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp(strtotime('last year')))
				),
				__LINE__, __FILE__,0, '',
				'perp_ap', 1, 
				'JOIN perp_order_line ON
					perp_order_line.supplier_stock_code = perp_supplier_stock.supplier_code
				JOIN perp_shipment_line ON
					perp_shipment_line.shippable_class = "PurchaseOrder" AND
					perp_shipment_line.shippable_id = perp_order_line.order_id AND
					perp_shipment_line.shippable_item_line = perp_order_line.line_no
				JOIN perp_goods_received ON
					perp_goods_received.shipment_id = perp_shipment_line.shipment_id AND
					perp_goods_received.line_no = perp_shipment_line.line_no
				JOIN perp_stock_cost ON
					perp_stock_cost.stock_cost_id = perp_goods_received.stock_cost_id'
			);
			$row = array();
			if($row = $GLOBALS['egw']->db->row(true)) {
				return $row['indirect_cost'];
			} else {
				return 0;
			}
		}

		/**
		*	Determine if a supplier catalog item is in use on an open Purchase Order 
		*
		*	This is expensive (O(n^2)), so you're probably better off with the get_list function
		*/
		public function in_use(array $catalog) {
			// Get a list of un-received Purchase Orders
			$query = array(
				'supplier_id'	=>	$catalog['supplier_id'],
				'status'	=>	'not_received'
			);
			$bo_po = CreateObject('perp_ap.bo_perp_purchase_order');
			$bo_po->get_order_list($query, $list);
			if(!is_array($list)) {
				return false;
			}
			foreach($list as $po_info) {
				$order = new PurchaseOrder($po_info['order_id']);
				$lines = $order->get_lines();
				foreach($lines as $line) {
					if($line->get_supplier_stock_code() == $catalog['supplier_code']) {
						return true;
					}
				}
			}

			return false;
		}
	}
?>
