<?php
	/**
	*	Data storage layer object for shipments
	*
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.so_perp_shipment.inc.php');

	class so_perp_incoming_shipment extends so_perp_shipment {
	
		/**
		*	Constructor
		*/
		public function __construct() {
			parent::__construct();
		}

		/**
		*	Get a list of shipments
		*	@nextmatch
		*/
		public function get_shipment_list(&$query) {
			$where = array();

			if($query['supplier_id']) {
				if(is_array($query['supplier_id'])) {
					$where[] = 'perp_shipment.supplier_id IN (' . implode($query['supplier_id']) . ')';
				} else {
					$where[] = 'perp_shipment.supplier_id = ' . $query['supplier_id'];
				}
			}
			if(isset($query['client_id'])) {
				$where[] = 'perp_shipment.client_id = ' . $query['client_id'];
			}
			if($query['shipment_id']) {
				$where[] = 'perp_shipment.shipment_id = ' . $query['shipment_id'];
			}
			if($query['order_id']) {
				$where[] = 'order_reference.order_id = ' . (int)$query['order_id'];
			}
			if($query['status']) {
				switch($query['status']) {
					case 'pending':
						$where[] = 'perp_shipment.requires_approval IS true';
						break;
					case 'in-transit':
						$where[] = 'actual_delivery_date IS NULL';
						break;
					case 'arrived':
						$where[] = 'actual_delivery_date IS NOT NULL';
						break;
					case 'overdue':
						$where[] = "requested_delivery_date < '" . $GLOBALS['egw']->db->to_timestamp(date('U')) . "'";
						$where[] = 'actual_delivery_date IS  NULL';
						break;
				}
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(our_reference_number LIKE $search OR their_reference_number LIKE $search)";
			}

			$fields = array(
				'perp_shipment.shipment_id',
				'perp_shipment.our_reference_number',
				'perp_shipment.their_reference_number',
				'perp_shipment.supplier_id',
				'perp_shipment.requested_delivery_date',
				'perp_shipment.actual_delivery_date',
				'perp_shipment.approval_date',
				'perp_shipment.vessel',
				'perp_shipment.requires_approval',
				'perp_supplier.supplier_num AS supplier_number',
				'perp_shipment.shipper',
				'perp_shipment.originating_user',
				'perp_shipment.ship_to_address'
			);

			if($query['selectcols']) {
				$fields = array_merge($fields, explode(',', $query['selectcols']));
			}
			$join = 'JOIN perp_supplier ON
					perp_shipment.supplier_id = perp_supplier.supplier_id ';

			// Add one order reference number
			$fields[] = 'order_reference.order_reference_number';
			$fields[] = 'order_reference.order_id';
			$join .= 'JOIN (
				SELECT shipment_id, our_reference_number AS order_reference_number, perp_order.order_id
				FROM perp_shipment_line
				JOIN perp_order_line ON
					perp_shipment_line.shippable_id = perp_order_line.order_id AND
					perp_shipment_line.shippable_item_line = perp_order_line.line_no
				JOIN perp_order ON
					perp_order.order_id = perp_order_line.order_id
				GROUP BY perp_order.order_id, perp_shipment_line.shipment_id
			) AS order_reference ON
			    perp_shipment.shipment_id = order_reference.shipment_id ';

			// Get number for results
			$query['count'] = $GLOBALS['egw']->db->select('perp_shipment',
				array('count(perp_shipment.shipment_id) AS count'),
				$where,
				__LINE__, __FILE__, 
				false, '', 'perp_api', 0, 
				$join
			)->fetchSingle();

			$sql = 'SELECT ' . implode(', ', $fields) . '
				FROM perp_shipment
			' . $join;
			if(count($where) > 0) {
				$where = implode(' AND ',$where);
				$sql .= ' WHERE ' . $where;
			}
			$sql .= ' ORDER BY ' . $query['order'] . ' ' . $query['sort'];
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__, 
				$query['start'], $query['num_rows']
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$row['requested_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['requested_delivery_date']);
				$row['actual_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['actual_delivery_date']);
				$row['approval_date'] = $GLOBALS['egw']->db->from_timestamp($row['approval_date']);
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get a list of items ordered but not shipped
		*	@param supplier_id	Limit to supplier
		*	@param shipment_id	Limit to shipment
		*	@param order_id	Limit to PurchaseOrder
		*	@return list of items to be shipped
		*/
		public function get_shippable_items(&$query) {
			$where = array();
			if($query['supplier_id']) {
				if(is_array($query['supplier_id'])) {
					$where[] = 'perp_shipment.supplier_id IN (' . implode($query['supplier_id']) . ')';
				} else {
					$where[] = " perp_order.supplier_id = '" . $query['supplier_id'] . "'";
				}
			}
			if($query['order_id']) {
				$where[] = " perp_order.order_id = " . $query['order_id'];
			}

			$sql .= 'SELECT 
					perp_order_line.order_id,
					IF(purchase_order_id IS NULL, \'Order\', \'PurchaseOrder\') AS shippable_class,
					perp_order.our_reference_number,
					perp_order.ship_to_address,
					perp_order_line.line_no,
					perp_order_line.description,
					perp_order_line.quantity_ordered,
					shipment_total.quantity_shipped,
					shipment_total.quantity_received,
					shipment_total.quantity_shipped - shipment_total.quantity_received AS quantity_in_transit,
					perp_order_line.quantity_ordered - IF(
						shipment_total.quantity_shipped IS NULL,
						0,
						shipment_total.quantity_shipped
					) AS quantity_unshipped
				FROM 
					perp_order_line 
				LEFT JOIN perp_purchase_order_detail ON
					perp_order_line.order_id = perp_purchase_order_detail.purchase_order_id AND
					perp_order_line.line_no = perp_purchase_order_detail.line_no
				JOIN perp_order ON
					perp_order_line.order_id = perp_order.order_id
				LEFT JOIN (
					SELECT 
						SUM(IF(quantity_shipped IS NULL, 0, quantity_shipped)) AS quantity_shipped,
						IF(quantity_units IS NULL, 0, quantity_units) AS quantity_received,
						shippable_id,
						shippable_item_line
					FROM perp_shipment_line
					LEFT JOIN (
						SELECT 
							shipment_id,
							line_no,
							sum(quantity_units) AS quantity_units
						FROM perp_goods_received
						GROUP BY shipment_id, line_no
					) AS received_total ON
						received_total.shipment_id = perp_shipment_line.shipment_id AND
						received_total.line_no = perp_shipment_line.line_no
					GROUP BY shippable_id, shippable_item_line
				) AS shipment_total ON
					perp_order_line.order_id = shipment_total.shippable_id AND
					perp_order_line.line_no = shipment_total.shippable_item_line';
			if(count($where) > 0) {
				$sql .= ' WHERE ' . implode (' AND ', $where);
			}
			$sql .= ' HAVING quantity_unshipped != 0';
			$sql .= ' ORDER BY ' . $query['order'] . ' ' . $query['sort'];
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);

			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
	}
?>
