<?php
	/**
	*	Data Storage object for dealing with payments
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/
	require_once('class.PayableInvoice.inc.php');

	class so_perp_payment {
		/**
		*	Database object
		*/
		private $db;

		/**
		*	Constructor
		*/
		public function so_perp_payment() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_ap');
		}

		/**
		*	Get a list of payments that match the given query array
		*	@nextmatch
		*/
		public function get_payments($query) {
			$where = array();
			if($query['supplier_id']) {
				if(is_numeric($query['supplier_id'])) {
					$where[] = 'perp_supplier.supplier_id = ' . $GLOBALS['egw']->db->quote($query['supplier_id']);
				} elseif (is_array($query['supplier_id']) && count($query['supplier_id'] > 0)) {
					$where[] = 'perp_supplier.supplier_id IN (' . implode(',', $query['supplier_id']) . ')';
				}
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "perp_supplier_payment.payment_number LIKE $search";
			}
			if(isset($query['payment_number'])) {
				$where[] = 'perp_supplier_payment.payment_number = ' . $GLOBALS['egw']->db->quote($query['payment_number']);
			}
			if(isset($query['status'])) {
				switch ($query['status']) {
					// Outstanding
					case 1:
						//$where[] = 'SUM(amount_cleared) != perp_bank_ledger.amount';
						$having[] = 'status = 1';
						break;
					// Cleared
					case 2:
						//$where[] = 'SUM(amount_cleared) = perp_bank_ledger.amount';
						$having[] = 'status = 2';
						break;
					// Reversed
					case 3:
						//$where[] = 'perp_supplier_payment.payment_id = reversed.original_id';
						$having[] = 'status = 3';
						break;
				}

			}
			if($query['bank_account']) {
				$where[] = 'perp_bank_ledger.bank_account = ' . addslashes($query['bank_account']);
			}
			if($query['payment_method']) {
				$where[] = 'perp_bank_ledger.payment_method = ' . addslashes($query['payment_method']);
			}
			if($query['start_date']) {
				$where[] = 'perp_general_ledger.trans_date >= \'' . addslashes($query['start_date']) . "'";
			}
			if($query['end_date']) {
				$where[] = 'perp_general_ledger.trans_date <= \'' . addslashes($query['end_date']) . "'";
			}

			// Get number of rows 
			$sql = 'SELECT payment_id,
					CASE 
						WHEN reversed.original_id THEN 3
						WHEN SUM(amount_cleared) = perp_bank_ledger.amount THEN 2
						ELSE 1
					END as status
				FROM perp_supplier_payment
				JOIN perp_supplier_ledger ON
					perp_supplier_payment.trans_id = perp_supplier_ledger.trans_id
				JOIN perp_supplier ON
					perp_supplier_ledger.supplier_id = perp_supplier.supplier_id
				JOIN perp_general_ledger ON
					perp_supplier_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_supplier_ledger.line_no = perp_general_ledger.line_no
				JOIN perp_bank_ledger ON
					perp_bank_ledger.trans_id = perp_supplier_ledger.trans_id 
				LEFT JOIN perp_bank_reconcile ON
					perp_bank_ledger.trans_id = perp_bank_reconcile.trans_id
					AND perp_bank_ledger.line_no = perp_bank_reconcile.line_no
					AND perp_bank_ledger.bank_account = perp_bank_reconcile.account
				LEFT JOIN perp_payment_reversal AS reversed ON
					perp_supplier_payment.payment_id = reversed.original_id';
			if($where) {
				$sql .= ' WHERE ' . implode(' AND ', $where);
			}
			$sql .= ' GROUP BY payment_id, perp_bank_ledger.trans_id, perp_bank_ledger.bank_account';
			if($having) {
				$sql .= ' HAVING ' . implode(' AND ', $having);
			}
			$this->db->query($sql,__LINE__, __FILE__);
			$query['count'] = $this->db->num_rows();

			$sql = 'SELECT DISTINCT
					payment_id,
					payment_number,
					trans_date,
					payment_method,
					payment_method_name,
					perp_supplier_ledger.supplier_id,
					supplier_num,
					supplier_name,
					perp_supplier_payment.trans_id,
					abs(round(perp_bank_ledger.amount / perp_supplier_ledger.exchange_rate, 2)) AS amount,
					supplier_currency,
					CASE 
						WHEN reversed.original_id THEN 3
						WHEN SUM(amount_cleared) = perp_bank_ledger.amount THEN 2
						ELSE 1
					END as status
				FROM (
					SELECT 
						payment_id,
						payment_number,
						trans_id,
						sum(amount) AS amount
					FROM
						perp_supplier_payment
					GROUP BY payment_id
				) AS perp_supplier_payment
				JOIN perp_supplier_ledger ON
					perp_supplier_payment.trans_id = perp_supplier_ledger.trans_id
				JOIN perp_supplier ON
					perp_supplier_ledger.supplier_id = perp_supplier.supplier_id
				JOIN perp_general_ledger ON
					perp_supplier_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_supplier_ledger.line_no = perp_general_ledger.line_no
				JOIN perp_bank_ledger ON
					perp_supplier_ledger.trans_id = perp_bank_ledger.trans_id
				JOIN perp_payment_method ON
					perp_payment_method.payment_method_id = perp_bank_ledger.payment_method
				LEFT JOIN perp_bank_reconcile ON
					perp_bank_ledger.trans_id = perp_bank_reconcile.trans_id
					AND perp_bank_ledger.bank_account = perp_bank_reconcile.account
					AND perp_bank_ledger.line_no = perp_bank_reconcile.line_no
				LEFT JOIN perp_payment_reversal AS reversed ON
					perp_supplier_payment.payment_id = reversed.original_id
				';
			if($where) {
				$sql .= 'WHERE ' . implode(' AND ',  $where);
			}
			$sql .= " GROUP BY perp_bank_ledger.trans_id, perp_bank_ledger.bank_account ";
			if($having) {
				$sql .= ' HAVING ' . implode(' AND ', $having);
			}
			$sql .= ' ORDER BY ' . $query['order'] . ' ' . $query['sort'];
			$this->db->query($sql, __LINE__, __FILE__, $query['start'], $query['num_rows']);
			$list = array();

			while($row = $this->db->row(True)) {
				$row['formatted_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $row['supplier_currency'],  $row['amount']);
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Get some information on a payment given by payment_id
		*	@param payment_id	int	ID number of a payment to read
		*	@return array payment information
		*/
		public function get_payment($payment_id) {
			$sql = 'SELECT DISTINCT
					payment_id,
					payment_number,
					perp_supplier_payment.trans_id,
					trans_date,
					perp_supplier_payment.invoice_id,
					perp_supplier_payment.amount,
					perp_bank_ledger.bank_account,
					allocation_date,
					perp_supplier_ledger.supplier_id,
					payment_method_id,
					payment_method_name,
					supplier_name,
					supplier_currency,
					our_reference_number,
					their_reference_number,
					CASE 
						WHEN SUM(amount_cleared) = perp_bank_ledger.amount THEN 2
						ELSE 1
					END as status
				FROM perp_supplier_payment
				LEFT JOIN perp_invoice ON
					perp_supplier_payment.invoice_id = perp_invoice.invoice_id
				JOIN perp_supplier_ledger ON
					perp_supplier_payment.trans_id = perp_supplier_ledger.trans_id
				JOIN perp_general_ledger ON
					perp_supplier_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_supplier_ledger.line_no = perp_general_ledger.line_no
				JOIN perp_bank_ledger ON
					perp_supplier_ledger.trans_id = perp_bank_ledger.trans_id
				LEFT JOIN perp_bank_reconcile ON
					perp_bank_ledger.trans_id = perp_bank_reconcile.trans_id
					AND perp_bank_ledger.line_no = perp_bank_reconcile.line_no
					AND perp_bank_ledger.bank_account = perp_bank_reconcile.account
				JOIN perp_payment_method ON
					perp_payment_method.payment_method_id = perp_bank_ledger.payment_method
				JOIN perp_supplier ON
					perp_supplier_ledger.supplier_id = perp_supplier.supplier_id
				WHERE payment_id =' . $payment_id . '
				GROUP BY perp_supplier_payment.payment_id, 
					perp_bank_ledger.trans_id, 
					perp_supplier_payment.invoice_id, 
					perp_supplier_payment.allocated_from, 
					perp_bank_ledger.bank_account';
			$this->db->query($sql, __LINE__, __FILE__);

			$list = array(0 => false);
			$list['total'] = 0;
			$list['payment_id'] = $payment_id;

			while($row = $this->db->row(True)) {
				$list['supplier_id'] = $row['supplier_id'];
				$list['supplier_name'] = $row['supplier_name'];
				$list['date'] = $row['trans_date'];
				$list['payment_method_id'] = $row['payment_method_id'];
				$list['payment_method_name'] = $row['payment_method_name'];
				$list['payment_number'] = $row['payment_number'];
				$list['bank_account'] = $row['bank_account'];
				$list['trans_id'] = $row['trans_id'];
				$list['status'] = $row['status'];

				$list['currency'] = $list['currency'] ? $list['currency'] : $row['supplier_currency'];
				$list['total'] += $row['amount'];
				if($row['invoice_id']) {
					$row['formatted_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $list['supplier_currency'], $row['amount']);
				} else {
					$row['prepayment'] = lang('pre-payment');
					$row['formatted_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $list['supplier_currency'], $row['amount']);
				}
				$list[] = $row;
			}
			$list['formatted_total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $list['currency'], $list['total']);

			return $list;
		}

		/**
		*	Get amounts that have been paid, but not paying an invoice
		*	These are pre-payments and overpayments
		*	@nextmatch
		*/
		public function get_unallocated_amounts(&$query) {
			$sql = 'SELECT 
					perp_supplier_payment.payment_id AS reference_id,
					null AS their_reference_number,
					perp_supplier_payment.payment_id,
					perp_supplier_payment.payment_number,
					perp_supplier_payment.trans_id,
					perp_supplier_payment.allocation_date,
					null AS invoice_id,
					null AS due_date,
					-perp_supplier_payment.amount AS original_amount,
					sum(prepay.amount) AS amount_paid
				FROM perp_supplier_payment 
				JOIN perp_supplier_ledger ON
					perp_supplier_payment.trans_id = perp_supplier_ledger.trans_id
				JOIN perp_bank_ledger ON
					perp_supplier_payment.trans_id = perp_bank_ledger.trans_id
				LEFT JOIN perp_supplier_payment AS prepay ON
					perp_supplier_payment.payment_id = prepay.allocated_from
				WHERE perp_supplier_payment.allocation_date IS NULL
					AND perp_supplier_ledger.supplier_id = ' . $query['supplier_id'];
				if($query['bank_account']) {
					$sql .= ' AND perp_bank_ledger.bank_account = ' . $query['bank_account'];
				}
				$sql .= '
				GROUP BY perp_supplier_payment.invoice_id, perp_supplier_payment.payment_id

				UNION 

				SELECT 
					perp_invoice.our_reference_number AS reference_id,
					perp_invoice.their_reference_number,
					perp_supplier_payment.payment_id,
					perp_supplier_payment.payment_number,
					perp_supplier_payment.trans_id,
					perp_supplier_payment.allocation_date,
					perp_invoice.invoice_id,
					perp_invoice.due_date,
					amount.amount, 
					sum(perp_supplier_payment.amount) AS amount_paid
				FROM perp_invoice
				LEFT JOIN (
					SELECT 
						invoice_id,
						ROUND(SUM(ROUND(quantity * unit_price, 2)),2) AS amount
					FROM perp_invoice_line 
					GROUP BY invoice_id
				) AS amount ON
					perp_invoice.invoice_id = amount.invoice_id
				LEFT JOIN perp_supplier_payment ON
					perp_supplier_payment.invoice_id = perp_invoice.invoice_id ';

				if($query['bank_account']) {
					$sql .= ' JOIN perp_bank_account ON
						perp_bank_account.bank_account_id = ' . $this->db->quote($query['bank_account']) . ' AND
						perp_bank_account.currency = perp_invoice.invoice_currency ';
				}
				$sql .= 'WHERE perp_invoice.supplier_id = ' . $query['supplier_id'] . ' AND !perp_invoice.invoice_paid AND 
					perp_invoice.invoice_transaction_id IS NOT NULL ';

				$sql .= '
				GROUP BY perp_invoice.invoice_id, perp_supplier_payment.invoice_id';
		
			$this->db->query($sql, __LINE__, __FILE__);
			$list = array();

			while($row = $this->db->row(True)) {
				if($row['invoice_id']) {
					$row['invoice'] =& new PayableInvoice($row['invoice_id']);
					$row['currency'] = $row['invoice']->get_invoice_currency();
					$row['amount_owing'] = $row['original_amount'] - $row['amount_paid'] - $row['invoice']->get_discount_taken();
					$row['discount_available'] = $row['invoice']->get_discount_available($query['date']);
					$row['formatted_discount_available'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $row['discount_available']);
				} else {
					$row['amount_owing'] = $row['original_amount'] - $row['amount_paid'];
				}
				$row['formatted_amount_owing'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $row['amount_owing']);
				$row['formatted_original_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $row['original_amount']);

				if($row['amount_owing'] != 0) {
					$list[] = $row;
				}
			}

			$query['count'] = count($list);
			return $list;
		}

		/**
		*	Enter a payment, and possibly match up a payment with an invoice
		*	@param payment_id	int	ID of the payment being dealt with
		*	@param payment_number	int	Reference number for this payment type
		*	@param trans_id		int	General Ledger transaction ID
		*	@param invoice_id	int	If this payment is in payment of an invoice, the ID of the invoice being paid
		*	@param amount		float	Amount of payment
		*	@param date		date	Date of the payment (Y-m-d)
		*	@param pre_payment	float	Amount of extra payment, not against any invoice
		*/
		public function allocate_payment($payment_id, $payment_number, $trans_id, $invoice_id, $amount, $date, $pre_payment) {
			if(!$invoice_id) {
				$invoice_id = 0;
			}

			$date = (is_null($date) ? null : $GLOBALS['egw']->db->to_timestamp($date));

			return $this->db->insert('perp_supplier_payment', 
				array(
					'payment_id'	=>	$payment_id,
					'payment_number'	=>	$payment_number,
					'trans_id'	=>	$trans_id,
					'invoice_id'	=>	$invoice_id,
					'amount'	=>	$amount,
					'allocation_date'	=>	$date,
					'allocated_from'	=>	$pre_payment
				),
				$where,
				__LINE__, __FILE__
			);
		}

		/**
		*	Get the next payment_id
		*
		*	@return int Next payment ID
		*/
		public function get_next_payment_id() {
			return max(1, $GLOBALS['egw']->db->query(
				'SELECT max(payment_id) + 1 AS next FROM perp_supplier_payment'
			)->fetchSingle());
		}

		/**
		*	Reverse a payment
		*/
		public function reverse(Array $payment, $reason) {

			$GLOBALS['egw']->db->transaction_begin();

			// Generate reversing payment
			$reverse = $payment;
			unset($reverse['payment_id']);
			$reverse['payment_number'] = lang('Reverse %1', $reverse['payment_method_name'] . ' ' . $reverse['payment_number']);
			$reverse['payment_method'] = $payment['payment_method_id'];
			$reverse['payment_total'] = -$reverse['total'];
			$reverse['payment_narrative'] = $reverse['payment_number'];
			$reverse['allocation_date'] = $payment['date'];

			foreach($reverse as $key => &$row) {
				if(is_numeric($key) && is_array($row)) {
					unset($reverse[$key]);
					if($row['invoice_id']) {
						unset($row['payment_id']);
					}
					$row['payment_amount'] = -$row['amount'];

					$reverse['rows'][] = $row;
				}
			}

			$reverse_id = ExecMethod('perp_ap.bo_perp_payment.post_payment', $reverse);
			if(!is_numeric($reverse_id) || !$reverse_id) {
				$GLOBALS['egw']->db->transaction_abort();
			}
			
			// Record the reversal
			$result = $GLOBALS['egw']->db->insert(
				'perp_payment_reversal',
				array(
					'original_id'	=>	$payment['payment_id'],
					'reversal_id'	=>	$reverse_id,
					'account_id'	=>	$GLOBALS['egw_info']['user']['account_id'],
					'timestamp'	=>	time(),
					'reason'	=>	$reason
				), false,
				__LINE__, __FILE__, 'perp_ap'
			);
			if($result === false) {
				$GLOBALS['egw']->db->transaction_abort();
			}

			$GLOBALS['egw']->db->transaction_commit();

			return $reverse_id;
		}

		/**
		*	Read a reversal record
		*
		*	@param payment_id Either the reversed payment, or the reversal.
		*	@return array of reversal information
		*/
		public static function get_reversal($payment_id) {
			
			$where = array(
				'(original_id = ' . (int)$payment_id . ' OR reversal_id = ' . (int)$payment_id . ')'
			);

			$result = $GLOBALS['egw']->db->select(
				'perp_payment_reversal',
				array(
					'original_id',
					'reversal_id',
					'account_id',
					'timestamp',
					'reason'
				),
				$where,
				__LINE__, __FILE__, false,
				'', 'perp_ap'
			)->fetchRow();
			if($result) {
				$result['timestamp'] = $GLOBALS['egw']->db->from_timestamp($result['timestamp']);
			}
			return $result;
		}
	}
?>
