<?php
	/**
	*	Data storage object for dealing with PurchaseOrders
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*	@see PurchaseOrder
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.so_perp_order.inc.php');

	class so_perp_purchase_order extends so_perp_order {
		/**
		*	Constructor
		*/
		public function so_perp_po() {
			parent::__construct();
		}

		/**
		*	Get a list of backordered items we're waiting for
		*	@param supplier	int	Optional.  Limit list to one supplier
		*	@param start	int	Start results at this record number
		*/
		public function get_backorders(&$query) {
			$where = array();
			if($query['supplier_id']) { 
				if(is_array($query['supplier_id'])) {
					$where[] = 'perp_supplier.supplier_id IN (' . implode(',', $query['supplier_id']) . ')';
				} else {
					$where[] = 'perp_supplier.supplier_id = ' . $query['supplier_id'];
				}
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_order.our_reference_number LIKE $search OR
					perp_order_line.supplier_stock_code LIKE $search OR
					perp_order_line.description LIKE $search)";
			}

			$join = 'JOIN perp_order_line ON
					perp_order.order_id = perp_order_line.order_id
				JOIN perp_supplier ON
					perp_supplier.supplier_id = perp_order.supplier_id
				LEFT JOIN (
					SELECT
						perp_shipment_line.shipment_id,
						perp_shipment_line.line_no,
						perp_shipment_line.shippable_id,
						IF(SUM(quantity_shipped) IS NULL, 0, SUM(quantity_shipped)) AS quantity_shipped,
						IF(SUM(quantity_received) IS NULL, 0, SUM(quantity_received)) AS quantity_received
					FROM perp_shipment_line 
					LEFT JOIN (
						SELECT shipment_id,
							line_no,
							IF(SUM(quantity_units) IS NULL, 0, SUM(quantity_units)) AS quantity_received
						FROM perp_goods_received
						GROUP BY shipment_id, line_no
					) AS goods_received ON 
						perp_shipment_line.shipment_id = goods_received.shipment_id AND
						perp_shipment_line.line_no = goods_received.line_no
					GROUP BY shippable_id, line_no
					ORDER BY perp_shipment_line.shipment_id
				) AS in_transit ON
					in_transit.shippable_id = perp_order_line.order_id AND
					in_transit.line_no = perp_order_line.line_no ';
		
			$sql = 'SELECT 
				perp_order.supplier_id, 
				supplier_num, 
				perp_order.order_id, 
				our_reference_number, 
				perp_order_line.supplier_stock_code, 
				description, 
				if(perp_order_line.quantity_ordered IS NULL,0, perp_order_line.quantity_ordered) AS quantity_ordered,
				if(quantity_shipped IS NULL,0, quantity_shipped) AS quantity_shipped,
				if(quantity_received IS NULL,0, quantity_received) AS quantity_received,
				if(quantity_shipped IS NULL,0, quantity_shipped) - if(quantity_received IS NULL,0, quantity_received) AS quantity_in_transit,
				if(quantity_ordered IS NULL,0, quantity_ordered) - if(quantity_shipped IS NULL,0, quantity_shipped) AS quantity_backordered

				FROM perp_order ' .
				$join;

			if(count($where) > 0) {
				$where = ' WHERE ' . $GLOBALS['egw']->db->column_data_implode(' AND ', $where);
				$sql .= $where;
			}
			$sql .= ' HAVING quantity_backordered != 0
				ORDER BY ' . ($query['order'] ? $query['order'] : 'perp_order.order_id') . ' ' . $query['sort'];

			// Get number of rows
			$query['count'] = $GLOBALS['egw']->db->query("SELECT count(*) FROM ($sql) AS results", __LINE__, __FILE__)->fetchSingle();

			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__, $query['start'], $query['num_rows']);

			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Returns the quantity that should be ordered for the specified location
		*	@todo this should be in perp_inventory, I think
		*	@param location	int	Location the goods being ordered are for
		*	@param stock_id	int	ID of the stock being ordered
		*	@param supplier_id	int	Not used.
		*	@return float Quantity to order
		*/
		public function get_order_quantity($location, $stock_id, $supplier_id = '') {
			// Switch based on order type, default to min/max
			// TODO: Add other ordering types
			$quantity = $this->get_min_max_order_quantity($location, $stock_id);

			if($supplier_id) {
				$GLOBALS['egw']->db->select('perp_supplier_stock', array(
						'conversion_factor',
						'quantity'
					),
					array(
						'supplier_id'	=>	$supplier_id
					),
					__LINE__, __FILE__
				);
				if($row = $GLOBALS['egw']->db->row(True)) {
					$quantity = $quantity / $row['conversion_factor'] / $row['quantity'];
				}
			}

			return $quantity;
		}

		/**
		*	Calculate quantity to order using min & max on hand levels
		*	@param location Location the goods are for
		*	@param stock_id Stock being ordered
		*	@return float Quantity to order
		*/
		private function get_min_max_order_quantity($location, $stock_id) {
			$where = array(
				'location_id'	=>	$location,
				'stock_id'	=>	$stock_id
			);
			
			$GLOBALS['egw']->db->select('perp_location_stock', array(
					'min_on_hand',
					'max_on_hand',
					'quantity_on_hand'
				),
				$where,
				__LINE__, __FILE__
			);
			if($row = $GLOBALS['egw']->db->row(True)) {
				return ($row['quantity_on_hand'] < $row['min_on_hand']) ? $row['max_on_hand'] - $row['quantity_on_hand'] : 0;
			} else {
				return 0;
			}
		}

		/**
		*	Get a list of purchase orders that are not on a shipment
		*	@param owner int	Limit to PurchaseOrders created by this User id 
		*	@return array purchase order information
		*/
		public function get_unshipped($owner) {
			$sql = 'SELECT 
					perp_purchase_order.purchase_order_id,
					perp_purchase_order.purchase_order_number AS id,
					perp_purchase_order.initiator,
					perp_purchase_order.supplier AS supplier_id,
					perp_supplier.supplier_num,
					perp_supplier.supplier_currency,
					perp_purchase_order.order_date,
					sum(perp_purchase_order_detail.line_price) as total
				FROM perp_purchase_order
					JOIN perp_supplier ON
						perp_supplier.supplier_id = perp_purchase_order.supplier
					JOIN perp_purchase_order_detail ON
						perp_purchase_order.purchase_order_id = perp_purchase_order_detail.purchase_order_id
				WHERE 
					perp_purchase_order.approved_date IS NOT NULL AND
					perp_purchase_order.received_date IS NULL AND
					perp_purchase_order.purchase_order_id NOT IN (
						SELECT shippable_id
						FROM perp_shipment_line
				)';
			if($owner && is_numeric($owner)) {
				$sql .= ' AND perp_purchase_order.initiator = ' . $owner;
			}
			$sql .= ' GROUP BY perp_purchase_order.purchase_order_id';

			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$row['order_date'] = $GLOBALS['egw']->db->from_timestamp($row['order_date']);
				$row['total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $row['supplier_currency'], $row['total']);
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Find a purchase order to add lines to.
		*	This order must be approved (or no approval requred), not at all shipped, not at all invoiced.
		*
		*	@param query Array of search parameters
		*	@return order_id
		*/
		public function get_addable_order($query) {
			$sql = 'SELECT order_id
				FROM perp_order
				LEFT JOIN perp_shipment_line ON
					perp_shipment_line.shippable_id = order_id
				LEFT JOIN perp_invoice_line ON
					perp_invoice_line.invoicable_id = order_id
				WHERE
					shipment_id IS NULL AND
					invoice_id IS NULL';
			if($query['supplier_id']) {
				$sql .= ' AND supplier_id = ' . $query['supplier_id'];
			}
			$sql .= ' ORDER BY order_id DESC';
			return $GLOBALS['egw']->db->query($sql, __LINE__, __FILE__, 0, 1)->fetchSingle();
		}

		/**
		*	Get a list of purchase order lines that have been at least partially received,
		*	but the quantity received does not match the quantity ordered.
		*
		*/
		public function get_under_over_list(&$query) {
			$fields = array(
				'perp_order_line.order_id',
				'perp_order_line.line_no',
				'perp_order_line.quantity_ordered',
				'perp_order_line.supplier_stock_code',
				'perp_order_line.our_stock_code',
				'perp_order_line.stock_id',
				'perp_order_line.description',
				'perp_order.our_reference_number',
				'perp_shipment.shipment_id',
				'perp_shipment.our_reference_number AS shipment_reference_number',
				'total_received.quantity_received'
			);

			$where = array(
				'total_received.quantity_received != quantity_ordered'
			);
			if($query['supplier_id']) {
				$where[] = 'perp_order.supplier_id IN (' . implode(',', (array)$query['supplier_id']) . ')';
			}
			if($query['shipment_id']) {
				$where[] = 'perp_shipment.shipment_id IN (' . implode(',', (array)$query['shipment_id']) . ')';
			}
			if($query['order_id']) {
				$where[] = 'perp_order.order_id IN (' . implode(',', (array)$query['order_id']) . ')';
			}

			$join = 'JOIN perp_order ON
					perp_order.order_id = perp_order_line.order_id
				JOIN perp_shipment_line ON
					perp_shipment_line.shippable_id = perp_order_line.order_id AND
					perp_shipment_line.shippable_item_line = perp_order_line.line_no
				JOIN perp_shipment ON
					perp_shipment.shipment_id = perp_shipment_line.shipment_id
				JOIN (
					SELECT perp_shipment_line.shippable_id AS order_id,
						perp_shipment_line.shippable_item_line AS line_no,
						sum(quantity_units) AS quantity_received
					FROM perp_goods_received
					JOIN perp_shipment_line ON
						perp_shipment_line.shipment_id = perp_goods_received.shipment_id AND
						perp_shipment_line.line_no = perp_goods_received.line_no
					GROUP BY
						perp_shipment_line.shippable_id, perp_shipment_line.shippable_item_line
				) AS total_received ON
					total_received.order_id = perp_order_line.order_id AND
					total_received.line_no = perp_order_line.line_no';

			// Get total row count
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_order_line',
				array('count(*)'),
				$where,
				__LINE__, __FILE__, false, 'GROUP BY perp_order_line.order_id, perp_order_line.line_no',
				'perp_ap', 1, 
				$join
			)->fetchSingle();

			// Get limited results
			$result = $GLOBALS['egw']->db->select(
				'perp_order_line',
				$fields,
				$where, 
				__LINE__, __FILE__, $query['start'],
				'GROUP BY perp_order_line.order_id, perp_order_line.line_no ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_ap', $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			return $list;
		}
	}
?>
