<?php

	/**
	*	Data access object for receiving
	*
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	class so_perp_receive {

		public function __construct() {
		}

		/**
		*	Get a list of goods that can be received.
		*	This comes from the list of shipment items
		*	that have not been received yet.
		*
		*	@param query an array of query parameters
		*	@nextmatch
		*	@see bo_perp_receive::get_receivable_list()
		*/
		public function get_receivable_list(&$query) {
			$where = array();
			if(isset($query['supplier_id']) && is_numeric($query['supplier_id'])) {
				$where[] = 'perp_shipment.supplier_id = ' . $query['supplier_id'];
			}
			if(isset($query['shipment_id']) && is_numeric($query['shipment_id'])) {
				$where[] = 'perp_shipment_line.shipment_id = ' . $query['shipment_id'];
			}
			if(isset($query['location_id']) && is_numeric($query['location_id'])) {
				$where[] = 'perp_location.location_id = ' . $query['location_id'];
			}
			if(isset($query['search']) && trim($query['search']) != '') {
				$search = $GLOBALS['egw']->db->quote($query['search']);
				$where[] = '(supplier_stock_code LIKE \'%' . $search . '%\' OR
						description LIKE \'%' . $search . '%\')';
			}

			// Get row count
			$sql = 'SELECT 
				count(perp_shipment_line.shipment_id) AS count
			FROM perp_shipment_line
			LEFT JOIN (
				SELECT 
					shipment_id,
					line_no,
					sum(quantity_units) AS quantity_units
				FROM perp_goods_received
				GROUP BY shipment_id, line_no
			) AS received_total ON
				received_total.shipment_id = perp_shipment_line.shipment_id AND
				received_total.line_no = perp_shipment_line.line_no
			JOIN perp_shipment ON
				perp_shipment_line.shipment_id = perp_shipment.shipment_id
			JOIN perp_location ON
				perp_location.delivery_address = perp_shipment.ship_to_address
			WHERE quantity_shipped - IF(quantity_units IS NULL, 0, quantity_units) != 0 ';
			if(count($where) > 0) {
				$sql .= ' AND ' . implode(' AND ', $where);
			}
			$GLOBALS['egw']->db->query(
				$sql, __LINE__, __FILE__
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['count'] = $row['count'];
			}

			// Get data
			$sql = 'SELECT 
				perp_shipment_line.shipment_id,
				perp_shipment_line.line_no,
				perp_shipment_line.shippable_class,
				perp_shipment_line.shippable_id,
				perp_shipment_line.shippable_item_line,
				perp_location.location_id,
				perp_location.location_name,
				perp_shipment.our_reference_number AS shipment_reference,
				perp_shipment.shipper,
				perp_order.order_id,
				perp_order.our_reference_number AS order_reference,
				perp_order_line.supplier_stock_code,
				perp_order_line.description,
				perp_order_line.quantity_ordered,
				perp_order_line.unit_price,
				perp_stock.stock_id,
				perp_stock.stock_num,
				perp_stock.controlled,
				perp_stock.decimal_places,
				perp_shipment_line.quantity_shipped,
				perp_supplier_stock.conversion_factor,
				IF(quantity_units IS NULL, 0, quantity_units) AS quantity_received,
				quantity_shipped - IF(quantity_units IS NULL, 0, quantity_units) AS receivable,
				our_uom.uom_name AS our_unit_of_measure,
				supplier_uom.uom_name AS supplier_unit_of_measure
			FROM perp_shipment_line
			JOIN perp_shipment ON
				perp_shipment_line.shipment_id = perp_shipment.shipment_id
			LEFT JOIN (
				perp_order_line 
				JOIN perp_order ON
					perp_order.order_id = perp_order_line.order_id
			) ON
				perp_order_line.order_id = perp_shipment_line.shippable_id AND
				perp_order_line.line_no = perp_shipment_line.shippable_item_line
			LEFT JOIN (
				SELECT 
					shipment_id,
					line_no,
					sum(quantity_units) AS quantity_units
				FROM perp_goods_received
				GROUP BY shipment_id, line_no
			) AS received_total ON
				received_total.shipment_id = perp_shipment_line.shipment_id AND
				received_total.line_no = perp_shipment_line.line_no
			LEFT JOIN (
				perp_stock JOIN 
				perp_unit_of_measure AS our_uom ON
				our_uom.uom_id = perp_stock.unit_of_measure
			) ON
				perp_order_line.stock_id = perp_stock.stock_id
			LEFT JOIN (
				perp_supplier_stock
				JOIN perp_unit_of_measure AS supplier_uom ON
					supplier_uom.uom_id = perp_supplier_stock.supplier_uom
			) ON
				perp_supplier_stock.supplier_id = perp_shipment.supplier_id AND
				perp_supplier_stock.stock_id = perp_order_line.stock_id AND
				perp_supplier_stock.supplier_code = perp_order_line.supplier_stock_code
			JOIN perp_location ON
				perp_location.delivery_address = perp_shipment.ship_to_address';

			if(count($where) > 0) {
				$sql .= ' WHERE ' . implode(' AND ', $where);
			}
			$sql .= ' HAVING receivable != 0
			ORDER BY ' . $query['order'] . ' ' . $query['sort'];

			$GLOBALS['egw']->db->query(
				$sql, 
				__LINE__, __FILE__,
				$query['start'], $query['num_rows']
			);

			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				if(is_numeric($row['decimal_places'])) {
					$row['quantity_shipped'] = round($row['quantity_shipped'], $row['decimal_places'] + strlen(ceil($row['conversion_factor'])));
				}
				$list[] = $row;
			}
			return $list;
		}

		
		/**
		*	Get data requested, from the passed in $start_date and $end_date
		*	@nextmatch
		*/
		public function get_data_goods_received_report($start_date, $end_date) {
						
			$sql = 'SELECT 
					DAYNAME(perp_goods_received.receive_date) AS day_of_week,
					DATE(perp_goods_received.receive_date) AS receive_date,
					perp_goods_received.receive_date AS receive_date_time,
					perp_shipment.their_reference_number,
					perp_shipment.shipment_id,
					perp_location.location_name,
					perp_shipper.shipper_name,
					perp_shipment.our_reference_number,
					
					perp_order_line.supplier_stock_code, 
					perp_order_line.description, 
					perp_order_line.quantity_ordered AS ordered_units,
					perp_shipment_line.quantity_shipped AS shipped_units,
					perp_goods_received.quantity_units AS received_units,
					(perp_order_line.quantity_ordered - perp_goods_received.quantity_units) AS back_ordered,
					perp_order.our_reference_number AS p_order_num, 
					perp_stock.stock_num, 
					perp_supplier.supplier_num, 
					perp_unit_of_measure.uom_name 
				FROM 
					perp_goods_received 
				INNER JOIN perp_shipment_line ON
					perp_goods_received.shipment_id = perp_shipment_line.shipment_id AND
					perp_goods_received.line_no = perp_shipment_line.line_no
				INNER JOIN perp_shipment ON
					perp_shipment_line.shipment_id = perp_shipment.shipment_id
				LEFT OUTER JOIN perp_location ON
					perp_shipment.ship_to_address = perp_location.delivery_address
				LEFT OUTER JOIN perp_shipper ON
					perp_shipment.shipper = perp_shipper.shipper_id
				INNER JOIN perp_order_line ON
					perp_shipment_line.shippable_id = perp_order_line.order_id AND
					perp_shipment_line.shippable_item_line = perp_order_line.line_no
				INNER JOIN perp_order ON
					perp_order_line.order_id = perp_order.order_id
				LEFT OUTER JOIN perp_stock ON
					perp_order_line.stock_id = perp_stock.stock_id
				INNER JOIN perp_supplier ON
					perp_shipment.supplier_id = perp_supplier.supplier_id
				LEFT OUTER JOIN perp_unit_of_measure ON
					perp_stock.unit_of_measure = perp_unit_of_measure.uom_id
				WHERE perp_goods_received.receive_date BETWEEN \''.$GLOBALS['egw']->db->to_timestamp($start_date)."' AND '". 
					$GLOBALS['egw']->db->to_timestamp($end_date) . "' AND perp_order.supplier_id != 0
				ORDER BY perp_goods_received.receive_date ASC";		
				
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__, $offset=0, $num_rows=-1, $inputarr=false, $fetchmode=egw_db::FETCH_ASSOC);
			$goods_received = array();
			while($row = $GLOBALS['egw']->db->row(true)) {				
				$row['receive_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $GLOBALS['egw']->db->from_timestamp($row['receive_date']));
				$row['day_of_week'] = lang($row['day_of_week']);
				$goods_received[] = $row;
			}
			
			return $goods_received;			
		}

		/**
		*	Get a list of items that were received but not ordered
		*
		*	@nextmatch
		*/
		public function get_not_ordered_list(&$query) {
			$fields = array(
				'perp_receiving_not_ordered.*',
				'perp_shipment.our_reference_number',
				'perp_shipment.their_reference_number',
				'perp_shipment.supplier_id',
				'perp_stock.stock_num',
			);
			$where = array();
			$join = 'JOIN perp_shipment ON
				perp_shipment.shipment_id = perp_receiving_not_ordered.shipment_id
				LEFT JOIN perp_stock ON
					perp_receiving_not_ordered.stock_id = perp_stock.stock_id';
			
			if($query['shipment_id']) {
				$where['shipment_id'] = $query['shipment_id'];
			}

			// Get total number of records
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_receiving_not_ordered',
				array('count(id)'),
				$where,
				__LINE__, __FILE__, false, '', 'perp_ap', 1,
				$join
			)->fetchSingle();

			// Limited query
			$result = $GLOBALS['egw']->db->select(
				'perp_receiving_not_ordered',
				$fields,
				$where,
				__LINE__, __FILE__,$query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_ap', $query['num_rows'],
				$join
			);

			$list = array();
			foreach($result as $row) {
				$row['received_date'] = $GLOBALS['egw']->db->from_timestamp($row['received_date']);
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Record an item that was received but not ordered
		*/
		public function set_not_ordered($id, $data) {
			$data['received_date'] = $GLOBALS['egw']->db->to_timestamp($data['received_date']);

			$GLOBALS['egw']->db->insert(
				'perp_receiving_not_ordered',
				array(
					'shipment_id'		=>	(int)$data['shipment_id'],
					'supplier_stock_code'	=>	$data['supplier_stock_code'],
					'stock_id'		=>	$data['stock_id'],
					'description'		=>	$data['description'],
					'quantity'		=>	$data['quantity_received'],
					'unit_price'		=>	$data['unit_price'],
					'received_by'		=>	$data['received_by'],
					'received_date'		=>	$data['received_date']
				),
				array('id'	=>	$id),
				__LINE__, __FILE__, 'perp_ap'
			);
			$id = $id ? $id : $GLOBALS['egw']->db->get_last_insert_id('perp_receiving_not_ordered', 'id');
			return $id;
		}

		/**
		*	Retrieve an item that was received but not ordered
		*/
		public function get_not_ordered($id) {
			$data = $GLOBALS['egw']->db->select(
				'perp_receiving_not_ordered',
				array('*'),
				array('id'	=>	$id),
				__LINE__, __FILE__, false, '', 'perp_ap'
			)->fetch();
			$data['received_date'] = $GLOBALS['egw']->db->from_timestamp($data['received_date']);

			return $data;
		}

		/**
		*	Remove an item that was received but not ordered.
		*
		*	Either we want to forget about it, or we've processed it and want to
		*	remove the record.
		*
		*	@param id
		*/
		public function delete_not_ordered($id) {
			$GLOBALS['egw']->db->delete(
				'perp_receiving_not_ordered',
				array('id'	=>	$id),
				__LINE__, __FILE__, 'perp_ap'
			);
			return true;
		}

	}
?>
