<?php
	/**
	*	User Interface for common Accounts Payable functions
	*
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/
	class ui_perp_ap {
		
		private $bo;
		private $template;
		private $api;

		public $public_functions = array(
			'index'		=>	True,
			'link_view'	=>	True,
			'link_add'	=>	true,
		);

		private $debug = False;
		protected $messages = array();

		function ui_perp_ap() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_ap.bo_perp_ap');
			$this->api = CreateObject('perp_api.ui_perp_api');
		}

		function index($content = array()) {
			$supplier = $GLOBALS['egw']->acl->check('supplier', EGW_ACL_READ, 'perp_ap');
			$purchase_order = $GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_ap');
			$shipments = $GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, 'perp_ap');
			$receive = $GLOBALS['egw']->acl->check('receive', EGW_ACL_READ, 'perp_ap');
			$invoice = $GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ap');
			$payment = $GLOBALS['egw']->acl->check('payment', EGW_ACL_READ, 'perp_ap');

		
			if(isset($content['supplier_list']) && $supplier) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_supplier.supplier_list'));
			} elseif(isset($content['po_list']) && $purchase_order) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_purchase_order.order_list'));
			} elseif(isset($content['backorders']) && ($purchase_order || $shipments)) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_purchase_order.backorder_list'));
			} elseif(isset($content['shipments']) && $shipments) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_incoming_shipment.shipment_list'));
			} elseif(isset($content['receive']) && $receive) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_receive.receive_new'));
			} elseif(isset($content['invoice']) && $invoice) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_payable_invoice.invoice_list'));
			} elseif(isset($content['payment']) && $payment) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_payment.payment_list'));
			} elseif(isset($content['reports'])) {
				$GLOBALS['egw']->redirect_link('/index.php', 
					array(
						'menuaction'	=>	'perp_api.ui_perp_report.index',
						'appname'	=>	'perp_ap'
					)
				);
			}

			$readonly['supplier_list'] = !$supplier;
			$readonly['po_list']	=	!$purchase_order;
			$readonly['backorders']	=	!($purchase_order || $shipments);
			$readonly['shipments']	=	!$shipments;
			$readonly['receive']	=	!$receive;
			$readonly['invoice']	=	!$invoice;
			$readonly['payment']	=	!$payment;

			// Check for a valid setup
			// Check for inventory locations
			if($GLOBALS['egw_info']['apps']['perp_inventory']) {
				$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
				if(count($locations) == 0) {
					$this->messages[] = array(lang('No inventory locations set up.'), 'error');
				} else {
					$addresses = false;
					foreach($locations as $location_id => $name) {
						$contacts = ExecMethod2('phpgwapi.bolink.get_links', 'perp_inventory', 'LOCATION-' . $location_id, 'addressbook');
						if(count($contacts) > 0) {
							foreach($contacts as $link_id => $address_id) {
								if(ExecMethod('addressbook.addressbook_bo.read', $address_id)) {
									$addresses = $addresses || true;
									break;
								}
							}
						}
					}
					if(!$addresses) {
						$this->messages[] = array(lang('Inventory location(s) missing addresses.  No shipping or receiving can be done.'), 'warning');
					}
				}
			}

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$this->template->read('perp_ap.index');
			$this->template->exec('perp_ap.ui_perp_ap.index', $data, $select_box_options, $readonly, $preserve);
		}

		public function add_to_po($supplier, $stock_code, $qty) {
			$stock_code = (string) $stock_code;
			if($qty != 0) {
				return $this->bo->add_to_po($supplier, $stock_code, $qty);
			}
		}

		public function link_view() {
			$link = $_GET['link_id'];
			list($location, $id) = explode('-', $link);
			switch($location) {
				case 'SUPPLIER':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_supplier.edit_supplier',
						'supplier_id'	=>	$id
					));
					break;
				case 'PO':
				case 'ORDER':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_purchase_order.edit',
						'order_id'	=>	$id
					));
					break;
				case 'SHIPMENT':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_incoming_shipment.edit',
						'shipment_id'	=>	$id
					));
					echo $id;
					break;
				case 'INVOICE':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_payable_invoice.edit',
						'invoice_id'	=>	$id
					));
					break;
				case 'PAYMENT':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_payment.payment_view',
						'payment_id'	=>	$id
					));
					break;
				case 'RECEIVED':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_receive.view',
						'goods_received_id'	=>	$id
					));
					break;
			}
		}

		/**
		* get title for a perp Accounts Payable entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param int/array $entry key to look up or array with supplier entry
		*	key is in the form {SUPPLIER | PO | SHIPMENT | PAYMENT}:id
		* @param string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				list($location, $id) = explode('-', $entry);
				if($id) {
					switch ($location) {
						case 'SUPPLIER':
							return ExecMethod('perp_ap.bo_perp_supplier.link_title', $id);
							break;
						case 'PO':
							return ExecMethod('perp_ap.bo_perp_purchase_order.link_title', $id);
							break;
						case 'SHIPMENT':
							return ExecMethod('perp_ap.bo_perp_incoming_shipment.link_title', $id);
							break;
						case 'INVOICE':
							return ExecMethod('perp_ap.bo_perp_payable_invoice.link_title', $id);
							break;
						case 'PAYMENT':
							return ExecMethod('perp_ap.bo_perp_payment.link_title', $id);
							break;
					}
				} else {
					return ExecMethod('perp_ap.bo_perp_supplier.link_title', $id);
				}
			} else {
				if(isset($entry['supplier_id'])) {
					return $entry['supplier_num'].' - '.$entry['supplier_name'];
				} elseif(isset($entry['purchase_order_id'])) {
					return ExecMethod('perp_ap.bo_perp_payable_order.link_title', $entry);
				} elseif(isset($entry['shipment_id'])) {
					return ExecMethod('perp_ap.bo_perp_incoming_shipment.link_title', $entry);
				} elseif(isset($entry['invoice_id'])) {
					return ExecMethod('perp_ap.bo_perp_payable_invoice.link_title', $entry);
				} elseif(isset($entry['payment_id'])) {
					return ExecMethod('perp_ap.bo_perp_payment.link_title', $entry);
				}
					
			}
			if (!$entry) {
			}
		}

		/**
		*	Add a new entry in Accounts Payable
		*
		*
		*/
		public function link_add($content = array()) {
			if(!isset($content['link_add'])) {
				$content['link_add'] = $GLOBALS['egw_info']['user']['preferences']['perp_ap']['link_add'];
			}
			if($content['link_add']) {
				switch ($content['link_add']) {
					case 'SUPPLIER':
						$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_supplier.edit_supplier'));
						break;
					case 'PO':
						$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_purchase_order.new_order'));
						break;
					case 'SHIPMENT':
						$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_incoming_shipment.new_shipment'));
						break;
					case 'INVOICE':
						$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_payable_invoice.new_invoice'));
						break;
					case 'PAYMENT':
						$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_payment.payment_new'));
						break;
				}
			} else {
				// Display
				$this->template->read('perp_ap.link_add');
				$this->template->exec('perp_ap.ui_perp_ap.link_add', $data, $select_box_options, $readonly, $preserve, 2);
			}
		}
	}
?>
