<?php
	/**
	*	User Interface for a Supplier's Catalog
	*
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	class ui_perp_catalog {

		/**
		*	eTemplate Object for display
		*/
		private $template;

		/**
		*	Logic layer object for information
		*/
		private $bo;

		/**
		*	List of public functions that phpgwapi allows to be
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'edit'	=>	True,
		);
		
		/**
		*	Constructor
		*/
		public function ui_perp_catalog() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_ap.bo_perp_catalog');
		}

		/**
		*	Edit a catalog item
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function edit($content = array()) {
			$supplier_id = $_GET['supplier_id'] ? $_GET['supplier_id'] : $content['supplier_id'];
			$supplier_code = $_GET['supplier_code'] ? urldecode($_GET['supplier_code']) : $content['supplier_code'];
			$data = array();

			// Deal with content
			if($content['save'] || ($content['supplier_code'] && isset($content['stock_id']))) {
				if(!$content['quantity']) {
					$this->messages[] = array(lang('Quantity cannot be 0'), 'error');
					$data = $content;
				} else {
					$result = $this->bo->edit_catalog($supplier_id, $supplier_code, $content);
					if($content['save'] && $result === true) {
						if(!is_object($GLOBALS['egw']->js)) {
							$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
						}
						$GLOBALS['egw']->js->set_onload('self.close(); ');
					} elseif ($result !== true) {
						$this->messages[] = array($result, 'warning');
					}
				}
			}

			// Load catalog item
			if($supplier_code) {
				$data += $this->bo->get_catalog_item($supplier_id, $supplier_code);

				// Move packages to make room for heading
				if(is_array($data['packages'])) {
					array_unshift($data['packages'], false);
				}

			}

			if($content['stock_id']) {
				$data['stock_id'] = $content['stock_id'];
			}

			if($supplier_id) {
				$supplier = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $supplier_id);
				$data['supplier_id'] = $supplier_id;
				$data['currency'] = $supplier['supplier_currency'];
				$readonly['supplier_id'] = True;
			}
			$data['supplier_code'] = $supplier_code;
			if(!$data['stock_id']) {
				unset($data['stock_id']);
			} else {
				$data['stock_id'] = array(
					'stock_id'		=>	$data['stock_id'],
					'stock_type_flag'	=>	'R'
				);
				$data['conversion_factor'] = $data['conversion_factor'] ? $data['conversion_factor'] : 1;
			}
			if(!$data['quantity']) {
				$data['quantity'] = 1;
			}
			
			if(!is_object($GLOBALS['perp']['bo'])) {
				$GLOBALS['perp']['bo'] = CreateObject('perp_api.bo_perp_api');
			}
			$query = array('currency_id' => $data['currency'], 'template' => false);
			$GLOBALS['perp']['bo']->get_currencies($query, $currencies);
			$data['our_cost'] = ($data['price'] / ($data['conversion_factor'] == 0 ? 1 : $data['conversion_factor'])) * $currencies[0]['exchange_rate'];
			$data['our_cost'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $data['our_cost']);
			$data['calculated_indirect_cost'] = round($this->bo->calculate_average_indirect_cost($supplier_id, $data['stock_id']['stock_id']), 5);

			$select_box_options['supplier_id'] = ExecMethod('perp_ap.bo_perp_supplier.get_suppliers');
			$select_box_options['supplier_uom'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			$select_box_options['stock_uom'] = $select_box_options['supplier_uom'];

			// Packages only use supplier's UOM
			$data['packages']['no_units'] = true;

			$data['supplier_quantity_label'] = lang("(%1 quantity)", $select_box_options['supplier_uom'][$data['supplier_uom']]);
			/*
			* Tell user to actually pay attention to the conversion factor
			* This depends on the translation of 'each' to not be annoying
			* and warn when the unit of measure is each
			*/
			if($data['conversion_factor'] == 1 && (strtolower('each') != strtolower(lang($select_box_options['supplier_uom'][$data['supplier_uom']])) ||
					$data['supplier_uom'] != $data['stock_uom'])) {
				$this->messages[] = array(lang(
					'Errors in conversion factor can cause receiving and inventory discrepancies.  
					Please make sure the conversion factor %1 is correct',
					$data['supplier_quantity_label']
				), 'warning');
			}
			
			$preserve['supplier_id'] = $supplier_id;
			$preserve['currency'] = $data['currency'];
			$preserve['old_supplier_code'] = $supplier_code;

			// Permissions
			// Add can only edit until they click save 
			$add = $GLOBALS['egw']->acl->check('catalog', EGW_ACL_ADD, 'perp_ap');
			$edit = $GLOBALS['egw']->acl->check('catalog', EGW_ACL_EDIT, 'perp_ap');
			$ro = !(($add&& !$supplier_code) || $edit);

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0) {
				if(!in_array($supplier_id, $supplier_ids)) {
					$data = array();
				} else {
					unset($data['stock_id']);
				}
			}

			$readonly['save'] = $ro;
			$readonly['supplier_code'] = $ro;
			$readonly['description'] = $ro;
			$readonly['price'] = $ro;
			$readonly['quantity'] = $ro;
			$readonly['supplier_uom'] = $ro;
			$readonly['lead_time'] = $ro;
			$readonly['stock_id'] = $ro;
			$readonly['conversion_factor'] = $ro;
			$readonly['average_indirect_cost'] = $ro;
			$readonly['preferred'] = $ro;

			if($supplier_code && $this->bo->in_use($data)) {
				$this->messages[] = array(lang('Item should not be edited until all Purchase Orders using it have been received'), 'warning');
			}

			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->validate_file('catalog', 'edit', 'perp_ap');
			$GLOBALS['egw']->js->set_onload('set_stock_id(); ');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Edit Supplier Catalog');
			$this->template->read('perp_ap.edit_stock');
			return $this->template->exec('perp_ap.ui_perp_catalog.edit', $data, $select_box_options, $readonly, $preserve,2);
		}

		/**
		*       Get catalog list, keeping user's search criteria.
		*       This function (in the UI layer) is used in order to use user preferences for search and sort
		*	@nextmatch
		*/
		public function get_supplier_catalog_list(&$query, &$rows = array(), &$readonlys = array()) {
			$GLOBALS['egw']->session->appsession('supplier_catalog_list', 'perp_ap', $query);

			// The following is used if there are filters
			if($query['filter'] != $GLOBALS['egw_info']['user']['preferences']['perp_ap']['supplier_catalog_list']) {
				$GLOBALS['egw']->preferences->add('perp_ap', 'supplier_catalog_list', $query['filter']);
				$GLOBALS['egw']->preferences->save_repository(true);
			}
			return $this->bo->get_supplier_catalog_list($query, $rows, $readonlys);
		}
	}
?>
