<?php
	/**
	*	User Interface object for Shipments
	*	
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_shipment.inc.php');
	require_once('class.IncomingShipment.inc.php');
	require_once('class.PurchaseOrder.inc.php');
	require_once('class.bo_perp_incoming_shipment.inc.php');

	class ui_perp_incoming_shipment extends ui_perp_shipment {
		/**
		*	List of public functions that phpgwapi allows to be
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'new_shipment'	=>	True,
			'shipment_list'	=>	True,
			'add'		=>	True,
			'edit'		=>	True,
		);

		protected $shipment_list_template = 'perp_ap.incoming_shipment_list';
		protected $shipment_list_row_template = 'perp_ap.incoming_shipment_list.row';
		protected $shipment_line_template = 'perp_ap.incoming_shipment_edit.row';
		
		/**
		*	Constructor
		*/
		public function __construct() {
			parent::__construct();
			$this->bo_shipment = new bo_perp_incoming_shipment();
			$this->template = CreateObject('etemplate.etemplate');
		}

		/**
		*	Display a list of shipments
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function shipment_list($content = array()) {
			parent::shipment_list($content);
			
			// Get where to return to
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			$this->list_update($content);
			$data = $this->list_get_display_data();

			$select_box_options = $this->list_get_options();

			$readonly = $this->list_get_readonlys();

			if(count($this->messages) > 0 ) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Shipments');
			$this->template->read($this->shipment_list_template);
			$this->template->exec('perp_ap.ui_perp_incoming_shipment.shipment_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Create a new shipment.  Redirects to edit().
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function new_shipment($content = array()) {
			$shipment = new IncomingShipment();

			$order_id = reg_var('order_id');

			if($order_id) {
				$order = new PurchaseOrder($order_id);
				$shipment->read_from_object($order);
			}

			$GLOBALS['egw']->session->appsession('shipment', false, $shipment);
			return $this->edit($content);
		}

		/**
		*	Add items to a shipment.  This is a popup that displays a list of goods ordered from a supplier that 
		*	have not shipped yet, and can be added to the shipment.  The shipment to work with
		*	is pulled from the session.
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function add($content = array()) {
			if($this->debug) {
				echo 'Incoming data:';
				_debug_array($content);
			}

			$shipment = $GLOBALS['egw']->session->appsession('shipment');
			if(!$shipment || !$shipment->is_editor($GLOBALS['egw_info']['user']['account_id'])) {
				$GLOBALS['egw']->redirect($content['return'] ? $content['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ap.ui_perp_ap.index'));
			}
			
			// Add items to shipment
			if($content['order']) {
				foreach($content['order'] as $key => $data) {
					if(is_numeric($key)) {
						if(is_array($data['lines'])) {
							foreach($data['lines'] as $ship) {
								if(is_array($ship) && $ship['quantity_unshipped'] != 0) {
									$this->add_line($shipment, $ship);
								}
							}
						}
					}
				}
			}
			// Save to session
			$GLOBALS['egw']->session->appsession('shipment', 'perp_ap', $shipment);

			// Get items that can be added
			$items = array();
			$query = array(
				'supplier_id'	=>	$shipment->get_supplier_id(),
				'shipment_id'	=>	$shipment->get_shipment_id(),
			);
			$this->bo_shipment->get_shippable_items($query, $items);

			$data = $items;
			$preserve = $data;
			if(count($data) == 0) {
				$this->messages[] = array(lang('Nothing from this supplier to ship'), 'warning');
			}

			$select_box_options['into_stock_location'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list');


			if($msg) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $msg);
			}

			$this->template->read('perp_ap.edit_shipment.add');
			$this->template->exec('perp_ap.ui_perp_incoming_shipment.add', $data, $select_box_options, $readonly, $preserve, 2);
		}

		/**
		*	Edit a shipment (just the header info).  See add() to add goods.
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo 'perp_ap.ui_perp_shipment.edit incoming data:';
				_debug_array($content);
			}

			if(!$GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, 'perp_ap')) {
				$GLOBALS['egw']->redirect($_SERVER['HTTP_REFERER']);
			}
			// Get shipment
			$shipment_id = get_var('shipment_id');
			$shipment = $this->get_shipment($shipment_id, 'IncomingShipment');
			if(!$shipment) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_ap.ui_perp_incoming_shipment.shipment_list',
					'message'	=>	'Could not get shipment'
				));
			}

			// Get where to return to
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);


			if($content['cancel']) {
				// Clear from session
				$GLOBALS['egw']->session->appsession('shipment', false, null);
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ap.ui_perp_incoming_shipment.shipment_list'));
			}

			// Deal with incoming data
			try {
				$result = $this->update_shipment($shipment, $content);
			} catch (Exception $e) {
				$this->messages[] = array($e->getMessage(), 'error');
			}

			// Save to session
			$GLOBALS['egw']->session->appsession('shipment', false, $shipment);

			// Load data for display
			$data = $this->get_display_data($shipment);

			// Supporting data
			$select_box_options = $this->get_selectbox_options($shipment);
			
			// Readonlys
			$readonly = $this->get_readonlys($shipment);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang($shipment->get_shipment_id() ? 'Edit Shipment' : 'New Shipment');
			$this->template->read('perp_ap.incoming_shipment_edit');
			$this->template->exec('perp_ap.ui_perp_incoming_shipment.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Update shipment
		*/
		protected function update_shipment(IncomingShipment &$shipment, $content) {
			// Check for an integer shipper, add if possible 
			if($content['shipper'] && !is_numeric($content['shipper'])) {
				// User entered a new shipper
				if($GLOBALS['egw']->acl->check('shipper', EGW_ACL_ADD, 'perp_api')) {
					$result = ExecMethod2('perp_api.bo_perp_api.change_lookup',
						'perp_shipper',
						'perp_api',
						array('shipper_id' => null),
						array('shipper_name' => $content['shipper'])
					);
					if($result) {
						$this->messages[] = array(lang('New shipper "%1" added.', $content['shipper']), 'info');
						$content['shipper'] = $result;
					} else {
						$this->messages[] = array(lang('Unable to add shipper "%1"', $content['shipper']), 'error');
						$content['shipper'] = null;
					}
				} else {
					$content['shipper'] = null;
					$this->messages[] = array(lang('You must choose a shipper from the list.'), 'error');
				}
			}
			if(isset($content['indirect_cost']['allocate_indirect_cost'])) {
				$result = $this->bo_shipment->allocate_indirect_costs(
					$shipment, 
					$content['indirect_cost']['cost_allocation'], 
					$content['indirect_cost']['total_indirect_cost']
				);
			} else {
				return parent::update_shipment($shipment, $content);
			}
		}

		/**
		*	Get selectbox options for the shipment list
		*/
		protected function list_get_options() {
			$options = parent::list_get_options();
			$options['filter2'] = array('' => lang('All')) + ExecMethod('perp_ap.bo_perp_supplier.get_suppliers');
			return $options;
		}

		/**
		*       Handle an action on an Shipment
		*
		*       Actions can be one of new, edit, ship, receive,
		*       invoice, approve, deny, cancel
		*
		*       @param action   Action to take
		*       @param shipment_id      ID of the Shipment to act on
		*/
		public function handle_shipment_action($action, $shipment_id) {
			$result = parent::handle_shipment_action($action, $shipment_id);
			if($result) {
				return $result;
			}
			switch($action) {		
				case 'receive':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_receive.receive',
						'shipment_id'	=>	$shipment_id
					));
					break;
				case 'invoice':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_payable_invoice.new_invoice',
						'shipment_id'	=>	$shipment_id
					));
					break;
				default:
					return false;
			}
		}

		protected function get_display_data(&$shipment) {
			$data = parent::get_display_data($shipment);

			// No grouping or filtering
			$data['nextmatch']['no_cat'] = true;
			$data['nextmatch']['no_filter'] = true;
			$data['nextmatch']['no_filter2'] = true;
			$data['nextmatch']['no_search'] = true;

			/**
			*	Look for a indirect cost, if shipment is not approved
			*/
			if($shipment->get_actual_delivery_date() && !$shipment->get_approval_date() && array_key_exists($GLOBALS['egw_info']['user']['account_id'], $shipment->get_admin_list())) {
				$invoice_lines = array();
				$data['indirect_cost']['total_indirect_cost'] = $this->bo_shipment->get_indirect_cost($shipment, $invoice_lines);
				if(count($invoice_lines) > 0) {
					$data['indirect_cost']['source_lines'] =& $invoice_lines;
				}
			}
			
			return $data;
		}

		protected function get_selectbox_options(&$shipment) {
			$options = parent::get_selectbox_options($shipment);

			// Ship to addresses come from locations
                        $options['ship_to_address'] = array();
			$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
			foreach($locations as $location_id => $name) {
				$contacts = ExecMethod2('phpgwapi.bolink.get_links', 'perp_inventory', 'LOCATION-' . $location_id, 'addressbook');
				foreach($contacts as $link_id => $contact_id) {
					$options['ship_to_address'][$contact_id] = ExecMethod2('phpgwapi.bolink.title', 'addressbook', $contact_id);
				}
			}

			// Indirect cost allocation options
			$options['cost_allocation'] = array(
				'quantity'	=>	lang('Quantity - charges are divided equally amoung all items'),
				'cost'		=>	lang('Cost - charges are divided according to unit cost'),
				'size'		=>	lang('Size - charges are divided according to shipping dimensions in the supplier catalog'),
				'mass'		=>	lang('Mass - charges are divided according to shipping mass in the supplier catalog')
			);
			return $options;
		}

		/**
		*	Add a line to a shipment
		*/
		protected function add_line(Shipment &$shipment, &$line_data) {
			// Re-format line data
			$line_data = array_merge($line_data, array(
				'quantity_shipped'	=>	$line_data['quantity_unshipped'],
				'shippable_id'		=>	$line_data['order_id'],
				'shippable_item_line'	=>	$line_data['line_no']
			));
			$line = new ShipmentLine($shipment, $line_data);
			$shipment->add_line($line);
		}
	}
?>
