<?php
	require_once('class.PayableInvoice.inc.php');
	require_once('class.PurchaseOrder.inc.php');
	require_once('class.IncomingShipment.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_invoice.inc.php');

	class ui_perp_payable_invoice extends ui_perp_invoice {

		private $bopo;

		protected $list_template = 'perp_ap.payable_invoice_list';
		protected $list_row_template = 'perp_ap.payable_invoice_list.row';
		protected $invoice_line_template = 'perp_ap.payable_invoice_edit.row';
		protected $uninvoiced_line_template = 'perp_ap.payable_invoice_edit.uninvoiced_line';

		public function __construct() {
			$this->bo_invoice = CreateObject('perp_ap.bo_perp_payable_invoice');
			$this->bopo = CreateObject('perp_ap.bo_perp_purchase_order');
			$this->template = CreateObject('etemplate.etemplate');

			$this->config = config::read('perp_ap');

			$this->public_functions += array(
				'invoice_list'	=>	True,
				'new_invoice'	=>	True,
				'edit'		=>	True,
			);
		}

		public function invoice_list($content = array()) {
			parent::invoice_list($content);
			
			// Get where to return to
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			$this->list_update($content);

			$data = $this->list_get_display_data();

			if(!isset($data['nextmatch']['filter'])) {
				$data['nextmatch']['filter'] = 'open';
			}

			$select_box_options = $this->list_get_options();
			$readonly = $this->list_get_readonlys();

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Invoice List');
			$this->template->read($this->list_template);
			$this->template->exec('perp_ap.ui_perp_payable_invoice.invoice_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function new_invoice($content = array()) {
			if($content['invoice_id']) {
				return $this->edit($content);
			}
			$invoice = new PayableInvoice();
			$po_id = $_GET['po_id'] ? $_GET['po_id'] : $content['po_id'];
			$shipment_id = reg_var('shipment_id');

			if($po_id) {
				$order = new PurchaseOrder($po_id);
				$invoice->read_from_object($order);
			} elseif ($shipment_id) {
				$shipment = new IncomingShipment($shipment_id);
				$invoice->read_from_object($shipment);
				$lines =& $invoice->get_lines();

				// Swap the InvoicableItem out so the Order gets invoiced.
				foreach($lines as &$line) {
					if($line->get_invoicable_item() instanceof ShipmentLine) {
						$line->set_invoicable_item($line->get_invoicable_item()->get_shippable_item());
					}
				}
			}

			$GLOBALS['egw']->session->appsession('invoice', false, $invoice);
			$GLOBALS['egw']->redirect_link('/index.php', array(
				'menuaction'    => $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.edit',
			));
		}

		public function edit($content = array()) {
			if($this->debug) {
				echo 'perp_ap.ui_perp_invoice.edit incoming data:';
				_debug_array($content);
			}
			// Get where to return to
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $GLOBALS['egw']->link('/index.php', 
				array('menuaction' => 'perp_ap.ui_perp_payable_invoice.invoice_list')
			));
			$invoice_id = reg_var('invoice_id');

			// Get invoice
			$invoice = $this->get_invoice($invoice_id, 'PayableInvoice');
			if($invoice == null) {
				$this->messages[] = array('Invoice not found', 'error');
			} else {
				// Save / Update
				$admin = $invoice->is_admin($GLOBALS['egw_info']['user']['account_id']) === True;
				$edit = $invoice->is_editor($GLOBALS['egw_info']['user']['account_id']) || $admin;
			}

			$buttons = array(
				'save'		=>	true,
				'post'		=>	true,
				'add_line'	=>	true,
				'add_line_from'	=>	true,
				'calculate_taxes'	=>	true
			);

			if($edit && (count(array_intersect_key($buttons, $content)) > 0 || isset($content['nextmatch']['rows']['delete']))) {
				try {
					$this->update_invoice($invoice, $content);
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			}

			if(isset($content['cancel']) || !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ap')) {
				$GLOBALS['egw']->redirect($preserve['return']);
			} 
			
			if(($content['save'] || $content['post']) && $GLOBALS['egw']->acl->check('invoice', EGW_ACL_EDIT, 'perp_ap')) {
				$result = $this->check_result($invoice->check(), 'error');

				if($result === true) {
					$invoice->write();
					$GLOBALS['egw']->session->appsession('invoice', false, null);
					$this->messages[] = array(lang('Invoice saved'), 'success');
				}
			}
			if($content['post'] && $invoice->is_admin($GLOBALS['egw_info']['user']['account_id'])) {
				$result = null;
				try {
					$result = $invoice->post();
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
				if($result === true) {
					$this->messages[] = array(lang('Invoice posted'), 'success');
				} elseif (!is_null($result)) {
					$this->messages[] = array($result, 'error');
				}
			}

			// Save to session
			$GLOBALS['egw']->session->appsession('invoice', false, $invoice);

			// Reload is sometimes needed to avoid re-posting by popups
			if($GLOBALS['egw']->session->appsession('invoice_reload')) {
				$GLOBALS['egw']->session->appsession('invoice_reload', 'perp_ap', false);
				$GLOBALS['egw']->redirect_link('/index.php',
					array(
						'menuaction'    =>      'perp_ap.ui_perp_payable_invoice.edit',
					)
				);
			}

			// Get data
			if($invoice != null) {
				$data = $this->get_display_data($invoice);


				if($data['contact']) {
					$contact_info = ExecMethod('addressbook.addressbook_bo.read', array('id' => $data['contact']));
					if(is_array($contact_info)) {
						$data['contact_name'] = $contact_info[0]['fn'];
					}
				}

				// Supporting data
				$select_box_options = $this->get_select_box_options($invoice);

				$readonlys = $this->get_readonlys($invoice);

				// Format total if it's readonly
				if($readonlys['total']) {
					$data['total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $invoice->get_invoice_currency(), $data['total']);
				}
			}

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$this->get_javascript('perp_ap.ajax_perp_payable_invoice');

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = ($invoice_id) ? lang('Edit Invoice') : lang('New Invoice');
			$this->template->read('perp_ap.payable_invoice_edit');
			$this->template->exec('perp_ap.ui_perp_payable_invoice.edit', $data, $select_box_options, $readonlys, $preserve);

			if($this->debug) {
				echo '<pre>' . $invoice->__toString() . '</pre>';
			}
		}

		protected function update_invoice(PayableInvoice &$invoice, &$content) {
			$result = parent::update_invoice($invoice, $content);

			// Add tax lines to a blank invoice
			if($invoice->get_supplier_id() && count($invoice->get_lines()) == 0) {
				$invoice->add_tax_lines();
			}
		}

		/**
		*	Display data for PayableInvoice
		*/
		protected function get_display_data(PayableInvoice &$invoice) {
			$data = parent::get_display_data($invoice);

			// Payments
			$payments = $invoice->get_payments();
			if(count($payments) > 0) {
				foreach($payments as &$payment) {
					$row['our_reference_number'] = ExecMethod('perp_ap.bo_perp_payment.link_title', $payment);
				}
				array_unshift($payments, false);
				$data['payments'] = $payments;
			}

			// Hide quantity if this is a from scratch invoice
			$from_scratch = true;
			$lines =& $invoice->get_lines();
			foreach($lines as &$line) {
				$from_scratch &= (is_null($line->get_invoicable_item()));
			}
			$data['new']['hide_quantity'] = $from_scratch;
			$data['nextmatch']['rows']['hide_quantity'] = $from_scratch;

			return $data;
		}

		/**
		*	Get selectbox options for editing
		*/
		protected function get_select_box_options(PayableInvoice &$invoice) {
			$options = parent::get_select_box_options($invoice);
			if($invoice->get_supplier_id() != 0) {
				$contacts = egw_link::get_links('perp_ap', 'SUPPLIER-' . $invoice->get_supplier_id(), 'addressbook');
				foreach($contacts as $id => $contact) {
					$options['their_contact_person'][$contact] = egw_link::title('addressbook', $contact);
				}
				if(count($contacts) == 0) {
					$this->messages[] = array(lang('No addresses set for supplier'), 'error');
				}
			}

			// Bill to addresses
			$contacts = array();
			if($this->config['default_bill_to']) {
				$contacts[] = $this->config['default_bill_to'];
			}
			$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
			foreach($locations as $location_id => $name) {
				$contacts += egw_link::get_links('perp_inventory', 'LOCATION-' . $location_id, 'addressbook');
			}
			foreach($contacts as $id => $contact) {
				$options['bill_to_address'][$contact] = egw_link::title('addressbook', $contact);
			}
			$options['add_line_app']['perp_ap'] = lang('Purchase Order');

			return $options;
		}

		/**
		*	Add a line to a PayableInvoice
		*
		*	@param invoice PayableInvoice to add the line to
		*	@param line_data array of information for the new line
		*/
		public function add_line(PayableInvoice &$invoice, &$line_data) {
			$line =& new PayableInvoiceLine(&$invoice, $line_data);
			$invoice->add_line($line);
		}

		/**
		*	Add a line from another application.
		*	This exending function makes sure we only do PurchaseOrders.
		*	It may go away in time, as other things need invoicing.
		*/
		public function add_line_from($content = array()) {
			if(isset($content['add']) && is_array($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as $key => &$row) {
					$row['invoicable_class'] = 'PurchaseOrder';
				}
			}
			parent::add_line_from($content);
		}

		public function get_uninvoiced_lines(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$count =  $this->bo_invoice->get_uninvoiced_order_lines($query, $rows, $readonlys);
			return $count;
		}

		/**
		*	Handle actions on an invoice from the list
		*	Actions can be one of post, pay
		*/
		public function handle_action($action, $invoice_id) {
			if(is_numeric($invoice_id)) {
				$invoice_id = array($invoice_id);
			}
			foreach($invoice_id as $id) {
				$invoices[] =& new PayableInvoice($id);
			}
			$result = true;
			switch($action) {
				case pay:
					$supplier_id = $invoices[0]->get_supplier_id();
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ap.ui_perp_payment.payment_new',
						'supplier_id'	=>	$supplier_id
					));
					break;
				case post:
					foreach($invoices as &$invoice) {
						$result = $result && $invoice->post();
					}
					break;
				case cancel:
					foreach($invoices as &$invoice) {
						$result = $result && $invoice->cancel();
					}
					break;
				default:
					$result = parent::handle_action($action, $invoice_id);
			}
			return $result;
		}
	}
?>
