<?php
	require_once('class.PayableInvoice.inc.php');

	class ui_perp_payment {

		public $public_functions = array(
			'payment_list'	=>	true,
			'payment_new'	=>	true,
			'payment_view'	=>	true,
		);

		private $debug = false;

		private $bo;
		private $template;

		public $messages = array();

		public function ui_perp_payment() {
			$this->bo = CreateObject('perp_ap.bo_perp_payment');
			$this->template = CreateObject('etemplate.etemplate');
		}

		public function payment_list($content = array()) {
			if($this->debug) {
				echo "perp_ap.ui_perp_payment.payment_list incoming data:";
				_debug_array($content);
			}

			// Deal with incoming
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			if(isset($content['new_payment']) && $GLOBALS['egw']->acl->check('payment', EGW_ACL_ADD, 'perp_ap')) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'    =>      'perp_ap.ui_perp_payment.payment_new',
				));
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ap.ui_perp_ap.index'));
			} elseif ($content['nm']['rows']['reverse']) {
				$id = key($content['nm']['rows']['reverse']);
				try {
					$result = $this->bo->reverse($id, $content['reverse_reason']);
				} catch (egw_exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
				if(is_numeric($result) && $result) {
					$this->messages[] = array('Payment was reversed', 'success');
				}
			}

			// Set up outgoing
			$data['nm'] = array(
				'no_cat'        =>      True,
		//		'no_filter'	=>	True,
				'filter_label'  =>      'Supplier',
				'filter_no_lang'        =>      True,
		//		'no_filter2'	=>	True,
				'filter2_label' =>      'Status',
				'filter2_no_lang'       =>      True,
				'get_rows'      =>      'perp_ap.bo_perp_payment.get_payments',
				'template'      =>      'perp_ap.payment_list.row',
		//		'header_left'   =>      '',
		//		'header_right'  =>      '',
			);
			if(is_array($content['extra_filter'])) {
				$data['nm'] = array_merge($data['nm'], $content['extra_filter']);
			}
			if(!$data['nm']['end_date']) {
				$data['nm']['end_date'] = date('Y-m-d', mktime(0,0,0,date('m'), date('t'), date('Y')));
			}
			$data['extra_filter'] = array(
				'bank_account'	=>	$data['nm']['bank_account'],
				'payment_method'	=>	$data['nm']['payment_method'],
				'start_date'		=>	$data['nm']['start_date'],
				'end_data'		=>	$data['nm']['end_date']
			);

			// Check for bank accounts - need those
			$bank_accounts = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts');
			if(count($bank_accounts) == 0) {
				$this->messages[] = array(
					lang(
						'You do not have any %1 set up.  pERP has nowhere to get the money from.',
						'<a href="' . 
						$GLOBALS['egw']->link('/index.php', array(
							'menuaction'	=>	'perp_ledger.hook_perp_ledger.bank_account'
						)) .
						'">' . lang('bank accounts') . '</a>'
					),
					'error'
				);
			}

			// Select box options
			$select_box_options['filter'] = array('' => lang('all')) + ExecMethod('perp_ap.bo_perp_supplier.get_suppliers');
			$select_box_options['status'] = array(1 => lang('Outstanding'), 2 => lang('Cleared'), 3 => lang('Reversed'));
			$select_box_options['filter2'] = array('' => lang('All')) + $select_box_options['status'];
			$select_box_options['bank_account'] = array('' => lang('All')) + ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts');
			$select_box_options['payment_method'] = array('' => lang('All')) + ExecMethod('perp_api.bo_perp_api.get_payment_methods');

			// Readonlys
			$readonly['new_payment'] = !$GLOBALS['egw']->acl->check('payment', EGW_ACL_ADD, 'perp_ap') || (count($bank_accounts) == 0);

			// Display
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Payments');
			$this->template->read('perp_ap.payment_list');
			$this->template->exec('perp_ap.ui_perp_payment.payment_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function payment_new($content = array()) {
			if($this->debug) {
				echo "perp_ap.ui_perp_payment.payment_new incoming data:";
				_debug_array($content);
			}
			$data = array();

			if(!$GLOBALS['egw']->acl->check('payment', EGW_ACL_ADD, 'perp_ap')) {
				$this->messages[] = array(lang('access is not allowed'), 'error');
			} else {

				// Deal with incoming
				$supplier_id = $content['supplier_id'] ? $content['supplier_id'] : $_GET['supplier_id'];
			
				if (isset($content['cancel'])) {
					$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ap.ui_perp_payment.payment_list'));
				} elseif(isset($content['post'])) {
					$result = $this->bo->post_payment($content);
					if(is_numeric($result) && !$this->debug) {
						$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_ap.ui_perp_payment.payment_view&payment_id=' . $result);
					} elseif (is_numeric($result) && $this->debug) {
						$this->messages[] = array(lang('success'), 'success');
					} else {
						$this->messages[] = array($result, 'error');
					}
				} else {
					// They did something else, like changed the date.  Keep all the numbers they've entered.
					$data = $content;
				}

				// Set up outgoing
				$data['supplier_id'] = $supplier_id;
				$data['payment_method'] = $content['payment_method'];
				$data['bank_account'] = $content['bank_account'];
				$data['allocation_date'] = $content['allocation_date'] ? $content['allocation_date'] : date('U');
				$config = config::read('perp_ap');

	
				if($supplier_id) {
					$supplier = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $supplier_id);
					if(!$data['bank_account']) {
						$data['bank_account'] = key(ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts', array('currency' => $supplier['supplier_currency'])));
					}
					$bank = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_account', $data['bank_account']);
				}
				if($supplier_id && !$supplier['supplier_currency']) {
					$this->messages[] = array(
						lang("Supplier currency is not set.  Somebody probably forgot to click 'Save' after they entered the supplier's financial information."), 
						'error'
					);
				} elseif($supplier_id && !$bank) {
					$this->messages[] = array(lang('Could not find bank account information.  It could be not set, or you may lack the needed permissions.'), 'error');
				} elseif($supplier['supplier_currency'] != $bank['currency']) {
					$this->messages[] = array(lang('Supplier currency does not match bank currency.  This will cause posting problems'), 'warning');
				}

				if(!$data['payment_method'] || is_array($bank['payment_methods']) && !in_array($data['payment_method'], $bank['payment_methods'])) {
					$methods = ExecMethod('perp_api.bo_perp_api.get_payment_methods', array('payment_method_id' => $bank['payment_methods']));
					$data['payment_method'] = key($methods);
				}

				// Don't keep the payment number if they've changed the payment method
				if($content['old_payment_method'] != $data['payment_method']) {
					$data['payment_number'] = $config['payments'][$data['payment_method']]['next_payment_number'];
				}

				if($supplier_id) {
					$this->bo->get_unallocated_amounts(
						array(
							'supplier_id'	=>	$supplier_id,
							'bank_account'	=>	$data['bank_account'],
							'date'		=>	$data['allocation_date'],
							'template'	=>	false,
						), 
						$rows
					);
					if(count($rows) == 0) {
						$this->messages[] = array(lang('No invoices or prepayments found for the selected settings'), 'info');
					} else {
						array_unshift($rows, false);
						$data['rows'] = $rows;
					}
				}

				// Select box options
				$select_box_options['payment_method'] = ExecMethod('perp_api.bo_perp_api.get_payment_methods', array('payment_method_id' => $bank['payment_methods']));
				$select_box_options['bank_account'] = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts');

				$preserve = $data;
				$preserve['old_payment_method'] = $data['payment_method'];
			}
			// Readonlys

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// xajax
			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$autofill = $GLOBALS['egw_info']['user']['preferences']['perp_ap']['autofill_payment'];
			$GLOBALS['egw']->js->set_onload('payment_new_setup(' . $autofill . ');');
			$GLOBALS['egw']->js->validate_file('payment', 'new', 'perp_ap');

			if($this->debug) {
				_debug_array($data);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('New Payment');
			$this->template->read('perp_ap.payment_edit');
			$this->template->exec('perp_ap.ui_perp_payment.payment_new', $data, $select_box_options, $readonly, $preserve);
		}

		public function payment_view($content = array()) {
			$payment_id = $content['payment_id'] ? $content['payment_id'] : $_GET['payment_id'];

			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_ap.ui_perp_payment.payment_list');
			}

			// Get data from display
			$data = $this->bo->get_payment($payment_id);
			if(!is_array($data)) {
				$error = ExecMethod2('perp_api.ui_perp_api.make_message', $data, 'error');
				$data = array('message' => $error);
			}

			// Select box options

			// Readonlys

			$preserve['payment_id'] = $payment_id;

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('View %2 %1', $data['payment_number'], $data['payment_method_name']);
			$this->template->read('perp_ap.payment_view');
			$this->template->exec('perp_ap.ui_perp_payment.payment_view', $data, $select_box_options, $readonly, $preserve);
		}
	}
?>
