<?php
	/**
	*	User Interface object for editing PurchaseOrders
	*
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_order.inc.php');
	require_once('class.bo_perp_purchase_order.inc.php');

	require_once('class.PurchaseOrder.inc.php');
	require_once('class.PurchaseOrderLine.inc.php');

	class ui_perp_purchase_order extends ui_perp_order{

		/**
		*	eTemplate for displaying order lines
		*/
		protected $order_line_template = 'perp_ap.purchase_order_edit.order_line';
		protected $order_list_template = 'perp_ap.purchase_order_list';
		protected $order_list_row_template = 'perp_ap.purchase_order_list.row';

		/**
		*	List of public functions that phpgwapi allows to be
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'order_list'	=>	True,
			'edit'		=>	True,
			'new_order'	=>	True,
			'backorder_list'	=>	True,
		);

		/**
		*	Constructor
		*/
		public function __construct() {
			parent::__construct();
			$this->bo_order = new bo_perp_purchase_order();
		}

		/**
		*	Get a list of PurchaseOrders 
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function order_list($content = array()) {
			$this->order_list_update($content);
			
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);


			$data = $this->order_list_get_display_data(); 
			$select_box_options = $this->order_list_get_options();
			$readonly = $this->order_list_get_readonlys();

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Purchase Order List');
			$this->template->read($this->order_list_template);
			$this->template->exec('perp_ap.ui_perp_purchase_order.order_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Handle an action on an Order from the Order List 
		*
		*/
		public function handle_order_action($action, $order_id) {
			$result = $this->check_result(parent::handle_order_action($action, $order_id), 'warning');
			if($result === true) {
				return true;
			}

			switch($action) {
				case 'ship':
					$GLOBALS['egw']->redirect_link(
						'/index.php', array(
							'menuaction'	=>	'perp_ap.ui_perp_incoming_shipment.new_shipment', 
							'order_id'	=>	$order_id
						)
					);
					break;
				case 'receive':
					$GLOBALS['egw']->redirect_link(
						'/index.php', array(
							'menuaction'	=>	'perp_ap.ui_perp_receive.receive', 
							'order_id'	=>	$order_id
						)
					);
					break;
				case 'invoice':
					$GLOBALS['egw']->redirect_link(
						'/index.php', array(
							'menuaction'	=>	'perp_ap.ui_perp_payable_invoice.new_invoice', 
							'po_id'		=>	$order_id
						)
					);
					break;
				default:
					return false;
					break;
			}
		}

		/**
		*	Add an item from a Supplier's catalog to an open PurchaseOrder
		*	@param supplier	int	ID of the supplier 
		*	@param stock_code	string	Supplier's reference for the item to add
		*	@param qty		float	Quantity to add to the PurchaseOrder, in Supplier's units.
		*	@return Success message or false on failure.
		*/
		public function add_to_po($supplier, $stock_code, $qty = 0) {
			$stock_code = (string) $stock_code;
			return $this->bo_order->add_to_po($supplier, $stock_code, $qty);
		}

		/**
		*	Create a new PurchaseOrder
		*	Overwrites any PurchaseOrder the user is currently working on with a new, blank PurchaseOrder, then
		*	calls edit() on the new PurchaseOrder
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function new_order($content = array()) {
			$purchase_order = CreateObject('perp_ap.PurchaseOrder');
			$supplier_id = (int)reg_var('supplier_id');
			if($supplier_id) {
				$purchase_order->set_supplier_id($supplier_id);
				$purchase_order->set_our_reference_number($purchase_order->suggest_order_number());
			}
			$GLOBALS['egw']->session->appsession('order', 'perp_ap', $purchase_order);
			return $this->edit($content);
		}

		/**
		*	Edit a PurchaseOrder
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo 'ui_perp_purchase_order.edit Incoming data:';
				_debug_array($content);
			}
			if(!$GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_ap')) {
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ap.ui_perp_ap.index'));
			}

			$supplier_id = $_GET['supplier_id'] ? $_GET['supplier_id'] : $content['supplier_id'];
			$order_id = $_GET['order_id'] ? $_GET['order_id'] : $content['order_id'];

			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : '/index.php?menuaction=perp_ap.ui_perp_purchase_order.order_list');

			// Get PO
			$purchase_order = $this->get_order('PurchaseOrder', $order_id);
			if($purchase_order == null) {
				$this->messages[] = array(lang('Unable to restore Purchase Order %1', $order_id), 'error');
				return $this->order_list();
			}

			// Update the Purchase Order
			try {
				$this->update_order($purchase_order, $content);
			} catch (Exception $e) {
				$this->messages[] = array($e->getMessage(), 'error');
			}

			if($content['nextmatch']['rows'] && $edit) {
				foreach($content['nextmatch']['rows'] as $no => $line) {
					$l = $purchase_order->get_line($line['line_no']);
					if($l) {
						$this->update_line($l, $line);
					}
				}
			}

			// Save to db
			if(isset($content['save'])) {
				try {
					$result = $purchase_order->write();
				} catch (Exception $exception) {
					$this->messages[] = array($exception->getMessage(), 'error');
				}

				if(is_numeric($result)) {
					$order_id = $result;
					unset($result);
					$this->messages[] = array(lang('Purchase order saved'), 'success');
				} else {
					$this->check_result($result, 'error');
				}
			} elseif (isset($content['cancel'])) {
				// Remove from session
				$GLOBALS['egw']->session->appsession('order', false, false);
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ap.ui_perp_purchase_order.order_list'));
			}

			// Save to session
			$GLOBALS['egw']->session->appsession('order', false, serialize($purchase_order));

			$check = $this->check_result($purchase_order->check(), 'warning');
			if($check === true && $admin) {
				$approval_check = $this->check_result($purchase_order->approval_check(), 'warning');
			}

			// Read Purchase Order
			$data = $this->get_display_data($purchase_order);

			// Selectboxes
			$select_box_options = $this->get_selectbox_options($purchase_order);

			// Set stuff to read only
			$readonly = $this->get_readonlys($purchase_order);
 
 			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Set up javascript
			$this->get_javascript('perp_ap.ajax_perp_purchase_order');

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('perp_ap') .' - ' . ($order_id ? lang('Edit Order') : lang('New Order'));
			$this->template->read('perp_ap.purchase_order_edit');
			$this->template->exec('perp_ap.ui_perp_purchase_order.edit', $data, $select_box_options, $readonly, $preserve);

			if($this->debug) {
				echo '<pre>';
				echo $purchase_order;
				echo '</pre>';
			}
		}

		/**
		*	Display a list of goods backordered from suppliers.
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function backorder_list($content = array()) {
			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ap.ui_perp_ap.index'));
			}

			// Set up pagination
			$data['nm'] = array(
				'no_cat'        =>      True,
			//	'no_filter'	=>	True,
				'filter_label'  =>      'Supplier',
				'filter'        =>      '',
				'filter_no_lang'        =>      True,
				'no_filter2'	=>	True,
			//	'filter2_label' =>      'Purchase Order',
			//	'filter2_no_lang'       =>      True,
			//	'filter2'       =>      current($select_box_options['filter2']),
				'get_rows'      =>      'perp_ap.bo_perp_purchase_order.get_backorders',
				'template'      =>      'perp_ap.po_backorder.row',
			);
			$select_box_options['filter'] = array('' => lang('All'));
			$select_box_options['filter'] += ExecMethod('perp_ap.bo_perp_supplier.get_suppliers');

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Backorders');
			$this->template->read('perp_ap.po_backorder');
			$this->template->exec('perp_ap.ui_perp_purchase_order.backorder_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Update the Order with the given data
		*	
		*	@todo Adjust with template to better match Order
		*	@param order PurchaseOrder to update
		*	@param content array with key => value mapping of new values
		*
		*	@return boolean Successfully updated
		*/
		protected function update_order(PurchaseOrder &$order, &$content) {
			if(isset($content['into_stock_location'])) {
				$location = ExecMethod('perp_inventory.bo_perp_location.get_location', $content['into_stock_location']);
				$content['ship_to_address'] = (int)$location['delivery_address'];
			}
			if(isset($content['arrival_date'])) {
				$content['requested_delivery_date'] = strtotime($content['arrival_date']);
			}

			return parent::update_order($order, $content);
		}

		protected function add_line(PurchaseOrder &$order, &$line_data) {
			if($line_data['quantity_ordered'] == 0) {
				$this->messages[] = array(lang('Cannot add a line with 0 quantity'), 'error');
				return false;
			}

			if(!$line_data['supplier_stock_code'] && $line_data['stock_id']) {
				// User somehow didn't type in their code, but got the stock_id set
				$this->messages[] = array(lang('To order inventory items, the supplier code is required'), 'error');
				return false;
			}

			$catalog_item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $order->get_supplier_id(), $line_data['supplier_stock_code']);

			if($line_data['supplier_stock_code'] && !$line_data['stock_id']) {
				$line_data['stock_id'] = $catalog_item['stock_id'];
				$line_data['our_stock_code'] = $catalog_item['stock_num'];
			}

			/**
			*	Convert the quantity to OUR units of measure, because
			*	it makes things easier when the supplier ships us 4.75  
			*	boxes when we ordered 5 because they're short.
			*/
			if($catalog_item) {
				if($line_data['unit_of_measure'] == $catalog_item['supplier_uom']) {
					// User chose supplier's unit of measure, convert
					$convert = ($catalog_item['conversion_factor'] * $catalog_item['quantity']);
					if($convert == 0) {
						$convert = 1;
					}
					$line_data['quantity_ordered'] *= $convert;
					$line_data['unit_price'] /= $convert;
				}
			}
			

			$line = new PurchaseOrderLine($order, $line_data);
			$order->add_line($line);

			// Update supplier catalog with new price
			if($line_data['update_catalog']) {
				if($catalog_item) {
					$convert = ($catalog_item['conversion_factor'] * $catalog_item['quantity']);
					if($convert == 0) {
						$convert = 1;
					}
					$old_price = $catalog_item['price'];
					$catalog_item['price'] = $line_data['unit_price'] * $convert;
					$result = ExecMethod2('perp_ap.bo_perp_catalog.edit_catalog', $order->get_supplier_id(), $line_data['supplier_stock_code'], $catalog_item);
					if($result === true) {
						$this->messages[] = array(lang('Updated catalog price for %1 from %2 to %3', $catalog_item['supplier_code'], $old_price, $catalog_item['price']), 'success');
					} else {
						$this->messages[] = array($result, 'warning');
					}
				} else {
					$this->messages[] = array(lang('Unable to update supplier catalog for ' . $line_data['supplier_stock_code']), 'warning');
				}
			}
			return true;
		}

		protected function get_display_data(PurchaseOrder &$purchase_order) {
			$data = parent::get_display_data($purchase_order);
			$data['nextmatch']['po_id'] = $purchase_order->get_order_id();
			$data['nextmatch']['rows']['set_account'] = $purchase_order->is_admin($GLOBALS['egw_info']['user']['account_id']);

			// Set label here so if the user doesn't have permission, the label is missing too
			$data['update_catalog_label'] = ($GLOBALS['egw']->acl->check('catalog', EGW_ACL_EDIT, 'perp_ap') ? lang('Update catalog') . ' %s' : '');
			if($data['update_catalog_label']) {
				$data['new']['update_catalog'] = true;
			}

			return $data;
		}

		protected function get_selectbox_options(PurchaseOrder &$purchase_order) {
			$options = parent::get_selectbox_options($purchase_order);
			if($purchase_order->get_supplier_id() != 0) {
				$contacts = ExecMethod2('phpgwapi.bolink.get_links', 'perp_ap', 'SUPPLIER-' . $purchase_order->get_supplier_id(), 'addressbook');
				foreach($contacts as $id => $contact) {
					$options['their_contact_person'][$contact] = ExecMethod2('phpgwapi.bolink.title', 'addressbook', $contact);
				}
				if(count($contacts) == 0) {
					$this->messages[] = array(lang('No addresses set for supplier'), 'error');
				}
			}

			// Ship to addresses come from locations
			$options['ship_to_address'] = array();
			$config = config::read('perp_ap');
			$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
			if(count($locations) > 0) {
				foreach($locations as $location_id => $name) {
					$contacts = ExecMethod2('phpgwapi.bolink.get_links', 'perp_inventory', 'LOCATION-' . $location_id, 'addressbook');
					foreach($contacts as $link_id => $contact_id) {
						$options['ship_to_address'][$contact_id] = ExecMethod2('phpgwapi.bolink.title', 'addressbook', $contact_id);
					}
				}
			} else {
				$options['ship_to_address'][$config['default_bill_to']] = ExecMethod2('phpgwapi.bolink.title', 'addressbook', $config['default_bill_to']);
			}
			$options['bill_to_address'][$config['default_bill_to']] = ExecMethod2('phpgwapi.bolink.title', 'addressbook', $config['default_bill_to']);

			// Units of measure for conversion - start will all UOM
			$options['unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
				
			return $options;
		}

		protected function get_readonlys(PurchaseOrder &$purchase_order) {
			$readonly = parent::get_readonlys($purchase_order);
			$readonly[$this->order_line_template]	=	!$purchase_order->is_editor($GLOBALS['egw_info']['user']['account_id']);
			$readonly['add_to_catalogue']		=	!$GLOBALS['egw']->acl->check('catalog', EGW_ACL_ADD, 'perp_ap');
			$readonly['new[update_catalog]']	=	!$GLOBALS['egw']->acl->check('catalog', EGW_ACL_EDIT, 'perp_ap');

			// Readonly to false, until I figure out a good way of turning it off and on.
			$readonly['add_line_from']		=	false;
			return $readonly;
		}

		protected function get_javascript($ajax_class) {
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			if($GLOBALS['egw_info']['user']['preferences']['perp_ap']['order_print_prompt']) {
				$GLOBALS['egw']->js->set_onload('
					var print_prompt = \'' . lang('Remember to print') . '\'; 
					window.onbeforeunload = function print_confirm() {
						alert(print_prompt);
						return false;
					}
				');
			}

			parent::get_javascript($ajax_class);
			$GLOBALS['egw']->js->set_onload('purchase_order_edit_setup(); ');
		}

		/** 
		*	Callback for the nextmatch widget to get the order lines
		*
		*	This overrides the readonlys from the parent, because the templates are different
		*
		*	@nextmatch
		*/
		public function get_order_lines(&$query, &$rows, &$readonlys) {
			$count = parent::get_order_lines($query, $rows, $readonlys);

			if(isset($query['order']) && $query['order'] instanceof Order) {
				$order = $query['order'];
			} else {
				$order = $this->get_order('Order');
			}

			foreach($rows as $key => &$row) {
				if(is_array($row) && array_key_exists('line_number', $row)) {
					unset($readonlys[($key + 1)]);
					$line =& $order->get_line($row['line_number']);
					$readonlys[($key + 1) . '[unit_price]'] = ($line->get_quantity_invoiced() != 0);
					$readonlys[($key + 1) . '[unit_price]'] = ($line->get_quantity_invoiced() != 0);
				}
			}
			return $count;
		}
	}
?>
