<?php
	/**
	*	User Interface object for receiving incoming Shipments
	*
	*	@author Nathan Gray
	*	@ingroup perp_ap
	*/

	require_once('class.IncomingShipment.inc.php');
	require_once('class.PurchaseOrder.inc.php');
	require_once('class.ReceiveLine.inc.php');
	require_once('class.bo_perp_receive.inc.php');

	class ui_perp_receive{

		/**
		*	List of public functions that phpgwapi allows to be
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'view'		=>	True,
			'resolve'	=>	true,
			'receive_new'	=>	true,
			'receive'	=>	True,
			'not_ordered'	=>	true,
		);

		/**
		*	Reference to eTemplate for display
		*/
		private $template;

		/**
		*	boolean enable for debug messages
		*/
		private $debug = false;

		/**
		*	Array of messages for the user (errors, info, etc)
		*/
		private $messages = array();

		/**
		*	Business end of receiving
		*/
		private $bo;

		/**
		*	Constructor
		*/
		public function ui_perp_receive() {
			$this->bo = new bo_perp_receive();
			$this->template = CreateObject('etemplate.etemplate');
		}

		/**
		*	View a previous receiving.
		*
		*	Looks for received_goods_id.
		*	
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function view($content = array()) {
			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_ap.ui_perp_ap.index'
				));
			}

			$id = reg_var('goods_received_id', 'any');
			$receive = new ReceiveLine($id);

			$data = $receive->toArray();
			// Get units of measure
			$uom = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			if($data['stock_id']) {
				$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $data['stock_id']);
				$data['quantity'] = round($data['quantity'], $stock_item['decimal_places']);
		
				$data['our_unit_of_measure'] = $uom[$stock_item['unit_of_measure']];
			}

			// Get supplier's unit of measure
			if($data['shipment_id']) {
				$shipment = new IncomingShipment($data['shipment_id']);
				if($shipment->get_line($data['line_no'])) {
					$order_line = $shipment->get_line($data['line_no'])->get_shippable_item();
					if($order_line) {
						// Pull catalog entry for quantity conversion
						$catalog = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item',$shipment->get_supplier_id(), $order_line->get_supplier_stock_code());
						$data['quantity_units'] = $data['quantity_units'] / $catalog['conversion_factor'];

						// If we wound up with more than 1, round to our decimal places
						if($data['quantity_units'] > 1) {
							$data['quantity_units'] = round($data['quantity_units'], $stock_item['decimal_places']);
						} else {
							// We got less than 1 in supplier's unit of measure.  Something probably went wrong with conversion
							// Round to 3 decimal places
							$data['quantity_units'] = round($data['quantity_units'], 3);
						}
						$data['supplier_unit_of_measure'] = $uom[$catalog['supplier_uom']];
					}
				}
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Received');
			$this->template->read('perp_ap.receive_view');
			$this->template->exec('perp_ap.ui_perp_receive.view', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Deal with items that were received but not ordered and deferred, as well as underships
		*	and overships so the purchase order can be closed from a goods point of view.  It will
		*	still stay open if it has not been completely invoiced.
		*
		*
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function resolve($content = array()) {

			if($content['save']) {
				// Items received but not ordered
				if(is_array($content['not_ordered'])) {
					$shipment = null;
					foreach($content['not_ordered'] as $key => $action) {
						list($garbage, $id) = explode(':', $key);
						list($action, $target) = explode(':', $action);

						$item = $this->bo->get_not_ordered($id);

						if(is_null($shipment) || $shipment->get_shipment_id() != $item['shipment_id']) {
							$shipment = new IncomingShipment($item['shipment_id']);
						}

						$this->bo->not_ordered($shipment, $item, $action, $target);

						// Unless they specifically want to do nothing, remove the line after
						if($action != 'nothing') {
							$this->bo->not_ordered($shipment, $item, 'cancel', $id);
						}
					}
				}
				if(is_array($content['under_over'])) {
					$order = null;
					foreach($content['under_over'] as $key => $action) {
						list($garbage, $order_id, $line_no) = explode(':', $key);
						list($action, $target) = explode(':', $action);

						if($action == 'nothing') {
							continue;
						} elseif ($action == 'adjust') {
							if(is_null($order) || $order->get_order_id() != $order_id) {
								if($order) {
									$order->write();
								}
								$order = new PurchaseOrder($order_id);
							}
							if($order->is_editor($GLOBALS['egw_info']['user']['account_id'])) {
								$line =& $order->get_line($line_no);
								$line->set_quantity_ordered($target);
							} else {
								$this->messages[] = array(lang('You are not allowed to modify Purchase Order %1', 
									$order->get_our_reference_number()
								), 'error');
							}
						}
					}
					$order->write();
				}
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/perp_ap/index.php');
			}

			$query = array();

			// Items received but not ordered
			$this->bo->get_not_ordered_list($query, $not_ordered, $not_ordered_readonly);
			if(count($not_ordered) > 0) {
				foreach($not_ordered as $key => $item) {
					$actions = array(
						'nothing'		=>	lang('Do nothing'),
						'cancel:' . $item['id']	=>	lang('Forget this entry (do not receive)')
					);

					$item['stock_code'] = $item['supplier_stock_code'];
					$choices = $this->get_choices($item['shipment_id'], $item);

					foreach($choices as $choice) {
						if($choice['action'] == 'defer') {
							// Can't defer anymore
							continue;
						}
						$actions[$choice['action'] . ':' . $choice['target']] = $choice['label'];
					}

					$readonly['action:' . $item['id']] = $not_ordered_readonly[$key]['action'];
					$select_box_options['action:' . $item['id']] = $actions;
				}
				// Make room for headings
				array_unshift($not_ordered, false);
				array_unshift($not_ordered, false);
				$data['not_ordered'] = $not_ordered;
			}

			// Under / overship
			$query = array(
				'order'		=>	'order_id',
				'num_rows'	=>	-1
			);
			$bo_order = new bo_perp_purchase_order();
			$bo_order->get_under_over_list($query, $under_over, $under_over_readonly);
			if(count($under_over) > 0) {
				foreach($under_over as $key => $item) {
					$actions = array();
					if($item['quantity_ordered'] > $item['quantity_received']) {
						$actions['nothing']	=	lang('Leave backordered');
					} else {
						$actions['nothing']	=	lang('Do nothing');
					}
					$actions['adjust:' . $item['quantity_received']]	=	lang('Adjust order to match received');

					$select_box_options['under_over:' . $item['order_id'] . ':' . $item['line_no']] = $actions;
					$readonly['under_over:' . $item['order_id'] . ':' . $item['line_no']] = !$GLOBALS['egw']->acl->check('order', EGW_ACL_CUSTOM_1, 'perp_ap');
				}

				// Make room for headings
				array_unshift($under_over, false);
				array_unshift($under_over, false);
				$data['under_over'] = $under_over;
			}


			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Resolve Shipments');
			$this->template->read('perp_ap.resolve_shipment');
			$this->template->exec('perp_ap.ui_perp_receive.resolve', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Receive an item from a shipment.
		*	This may be a continuation of an existing receiving, or a new shipment.
		*	For this method of receiving, you do not need an IncomingShipment.  If you
		*	have one, it will be used.  If not, a new one will be created and updated.
		*
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function receive_new($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' Incoming data:';
				_debug_array($content);
			}
			if(!$GLOBALS['egw']->acl->check('receive', EGW_ACL_EDIT, 'perp_ap')) {
				$GLOBALS['egw']->redirect_link('/perp_ap/index.php');
			}
				

			// Deal with incoming
			$location_id		=	$content['location_id'] ? $content['location_id'] : reg_var('location_id');
			$supplier_id		=	$content['supplier_id'] ? $content['supplier_id'] : reg_var('supplier_id');
			$shipper_id		=	$content['shipper_id'] ? $content['shipper_id']: reg_var('shipper_id');
			$order_id		=	$content['order_id'] ? $content['order_id'] : reg_var('order_id');
			$shipment_reference	=	$content['shipment_reference'];

			// Check for a preference so user doesn't always have to select
			if(!$location_id) {
				$location_id = $GLOBALS['egw_info']['user']['preferences']['perp_ap']['receiving_location'];
			}

			// Check for an integer shipper, add if possible 
			if($shipper_id && !is_numeric($shipper_id)) {
				// User entered a new shipper
				if($GLOBALS['egw']->acl->check('shipper', EGW_ACL_ADD, 'perp_api')) {
					$result = ExecMethod2('perp_api.bo_perp_api.change_lookup',
						'perp_shipper',
						'perp_api',
						array('shipper_id' => null),
						array('shipper_name' => $shipper_id)
					);
					if($result) {
						$this->messages[] = array(lang('New shipper "%1" added.', $shipper_id), 'info');
						$shipper_id = $result;
						$content['shipper_id'] = $shipper_id;
					} else {
						$this->messages[] = array(lang('Unable to add shipper "%1"', $shipper_id), 'error');
						$shipper_id = null;
					}
				} else {
					$shipper_id = null;
					$this->messages[] = array(lang('You must choose a shipper from the list.'), 'error');
				}
			}

			if($location_id && $supplier_id && $order_id) {

				/**
				*	Need to get the ID of the shipment we're working with.
				*	If we don't have an ID, we'll search to see if there's an existing
				*	incoming shipment for the supplier we can build on
				*/
				$shipment_id	=	$content['shipment_id'];
				$shipment =& $this->get_shipment($shipment_id, $location_id, $supplier_id, $shipper_id, $order_id, $shipment_reference);

				if(isset($content['save'])) {
					unset($content['save']);
					try {
						// receive_item() expects serials in (S/N => $quantity) format
						if(is_array($content['serial_numbers'])) {
							$serial_array = array();
							foreach($content['serial_numbers'] as $values) {
								if(is_array($values)) {
									$serial_array[$values['serial_number']] = $values['quantity'];
								}
							}
							$content['serial_numbers'] = $serial_array;
						}

						// Add the line to the shipment
						$result = $this->bo->receive_item($shipment, $content);
					} catch (Exception $e) {
						$this->messages[] = array($e->getMessage(), 'error');
					}
					if($result === true) {
						$this->messages[] = array(lang('Received %1 of %2', $content['quantity_received'], $content['stock_code']), 'success');
					} elseif ($result === false) { 
						// Could not receive the line
						$GLOBALS['egw']->session->appsession('receive_shipment', 'perp_ap', $shipment);
						$GLOBALS['egw']->session->appsession('not_ordered', 'perp_ap', $content);
						$GLOBALS['egw']->redirect_link('/index.php', array(
							'menuaction'	=>	'perp_ap.ui_perp_receive.not_ordered'
						));
					} else {
						$this->messages[] = array(lang('Could not receive item.'), 'error');
						$this->messages[] = array($result, 'error');
					}

				}

				// Store shipment in session for handy reference
				$GLOBALS['egw']->session->appsession('receive_shipment', 'perp_ap', $shipment);
			} elseif(count($content) > 0) {
				// Missing required info
				if(!$location_id) {
					$this->messages[] = array(lang('Missing receiving location'), 'error');
				}
				if(!$supplier_id) {
					$this->messages[] = array(lang('Missing supplier'), 'error');
				} 
				if (!$order_id) {
					$this->messages[] = array(lang('Missing purchase order'), 'error');
				}
			}
			if(isset($content['done'])) {
				$GLOBALS['egw']->session->appsession('receive_shipment', 'perp_ap', null);
				$GLOBALS['egw']->session->appsession('not_ordered', 'perp_ap', null);
				$GLOBALS['egw']->redirect_link('/perp_ap/index.php');
			}

			// Carry over for display
			$data['location_id']		=	$location_id;
			$data['supplier_id']		=	$supplier_id;
			$data['order_id']		=	array(
				'order_id'	=>	$order_id,
				'filter'	=>	'not_received'
			);
			if($shipment && $shipment->get_shipment_id()) {
				$data['shipment_id']	=	$shipment->get_shipment_id();
			}
			$data['shipment_reference']	=	$shipment_reference;
			$data['stock_code']		=	array(
				'supplier_id'	=>	$supplier_id
			);
			$data['serial_numbers'] = array(
				'valid'		=>	false,
				'existing'	=>	false
			);

			$readonly['location_id']	=	($content['location_id']);
			$readonly['supplier_id']	=	($supplier_id);
			$readonly['shipment_id']	=	array('search' => true);

			// Selects
			$select_box_options['location_id'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list', 
				array('filter2' => 'shippable')
			);
			$preserve = $data;
			if($receiving_id) {
				$preserve['receiving_id']	=	$receiving_id;
			}
			if($shipment && $shipment->get_shipment_id()) {
				$preserve['shipment_id']	=	$shipment->get_shipment_id();
			}

			// Javascript
			$GLOBALS['egw']->js->set_onload("receive_setup(); ");
			$GLOBALS['egw']->js->validate_file('receive', 'new', 'perp_ap');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Receive Goods');
			$this->template->read('perp_ap.receive_new');
			$this->template->exec('perp_ap.ui_perp_receive.receive_new', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Find a shipment to receive against, or create a new one
		*
		*	@param shipment_id An existing shipment we know the ID for
		*	@param data Content from the etemplate
		*	
		*	@return IncomingShipment
		*/
		protected function &get_shipment($shipment_id, $location_id, $supplier_id, $shipper_id, $order_id, $shipment_ref) {
			if(!$shipment_id && !$supplier_id) {
				return  new IncomingShipment();
			}
			if(!$shipment_id && $supplier_id) {
				// Probably just created a new shipment
				$shipment =& $GLOBALS['egw']->session->appsession('receive_shipment');
				if($shipment instanceof IncomingShipment && $shipment->get_supplier_id() == $supplier_id) {
					return $shipment;
				}
			}

			if($shipment_id && $supplier_id) {
				$shipment =& $GLOBALS['egw']->session->appsession('receive_shipment');
				if(!($shipment instanceof IncomingShipment)) {
					$shipment = new IncomingShipment($shipment_id);
					if($shipment->get_supplier_id() != $supplier_id) {
					}
				}
				return $shipment;
			}

			// Look for a shipment already set up for this supplier & Purchase Order
			$query = array(
				'status'	=>	'in-transit',
				'supplier_id'	=>	$supplier_id,
				'order_id'	=>	$order_id,
				'order'		=>	'requested_delivery_date',
				'sort'		=>	'ASC',
				'template'	=>	false,
			);
			$bo_incoming_shipment = new bo_perp_incoming_shipment();
			$count = $bo_incoming_shipment->get_shipment_list($query, $shipments);
			if($count > 0) {
				return new IncomingShipment($shipments[0]['shipment_id']);
			}

			// No shipments set up, create a blank one
			$order = new PurchaseOrder($order_id);
			
			if(!($order instanceof PurchaseOrder) || !$order_id) {
				throw new egw_exception(lang('Missing purchase order'));
			}

			$shipment = new IncomingShipment();
			$shipment->read_from_object($order);

			$shipment->set_their_reference_number($shipment_ref);
			$shipment->set_shipper_id($shipper_id);

			// Remove lines, we'll add them as we go
			$lines = $shipment->get_lines();
			foreach($lines as $key => $line) {
				$shipment->remove_line($key);
			}
			
			return $shipment;
		}

		/**
		*	Receive a shipment
		*
		*	This is for receiving goods against a list of items that have been recorded
		*	as shipped.  This means that the goods received must already be on an IncomingShipment.
		*	Goods not on an incoming shipment will not be on the list.
		*
		*	@param content  array Data returned from eTemplate when the user submits the form.
		*/
		public function receive($content = array()) {
			if($this->debug) {
				echo 'ui_perp_receive.receive Incoming data:';
				_debug_array($content);
			}

			// get_var is supposed to search GET & POST, but doesn't seem to find these, so I provide the default from $content
			$shipment_id = get_var('shipment_id', 'any', $content['shipment_id']);
			$order_id = get_var('order_id', 'any', $content['order_id']);

			// Get where to return to
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			if ($shipment_id) {
				$shipment = new IncomingShipment($shipment_id);
				$preserve['shipment_id'] = $shipment_id;
				$supplier_id = $shipment->get_supplier_id();
			} elseif ($order_id) {
				$order = new PurchaseOrder($order_id);
				$preserve['order_id'] = $order_id;
				$supplier_id = $order->get_supplier_id();
			}

			if(!$shipment_id && !$order_id) {
				$this->messages[] = array(lang('Nothing available to receive using selected options'), 'info');
			}

			// Update
			if(isset($content['save'])) {
				$result = $this->update($content);
				if($result === True) {
					$this->messages[] = array(lang('Goods received'), 'success');
				} else {
					$this->messages[] = array(($result ? $result : lang("Error receiving.  Some goods not received.")), 'warning');
				}
			} elseif(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($content['return']);
			}

			// Set up pagination
			$data['nextmatch'] = array(
				'supplier_id'	=>	$supplier_id,
				'order_id'	=>	$order_id,
				'shipment_id'	=>	$shipment_id,
				'status'	=>	'in-transit',
				'get_lines'	=>	True,
				'no_cat'        =>      True,
				'no_filter'	=>	True,
				'filter_label'  =>      'Status',
				'filter'        =>      $po_id,
				'filter_no_lang'        =>      True,
				'no_filter2'	=>	True,
		//		'filter2_label' =>      'Show %s transactions',
		//		'filter2_no_lang'       =>      True,
		//		'filter2'       =>      current($select_box_options['filter2']),
				'template'      =>      'perp_ap.receive.row',
				'get_rows'	=> 	'perp_ap.bo_perp_receive.get_receivable_list',
			);
			$select_box_options['shipper'] = ExecMethod('perp_api.bo_perp_api.get_shippers');
			$select_box_options['destination'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list');

			// Readonly
			$readonly['save'] = !$GLOBALS['egw']->acl->check('receive', EGW_ACL_EDIT, 'perp_ap');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			
			// Keep shipment_id && order_id
			$preserve['shipment_id']	= $shipment_id;
			$preserve['order_id']		= $order_id;
			
			// xajax
//			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			if($GLOBALS['egw_info']['user']['preferences']['perp_ap']['order_print_prompt']) {
				$GLOBALS['egw']->js->set_onload('print_prompt = \'' . lang('Remember to print') . '\'; ');
			}
			$GLOBALS['egw']->js->set_onload("receive_setup('" . $GLOBALS['egw_info']['server']['webserver_url'] . "'); ");
			
			$GLOBALS['egw']->js->validate_file('receive', 'edit', 'perp_ap');

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Receive shipment %1', ($shipment ? $shipment->get_our_reference_number() : ''));
			$this->template->read('perp_ap.receive');
			$this->template->exec('perp_ap.ui_perp_receive.receive', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Deal with incoming data from receiving
		*	@param content array returned from the eTemplate
		*/
		protected function update(&$content) {
			if(!is_array($content['nextmatch']['rows'])) {
				return false;
			}
	
			$shipment = null;
			$success = false;

			foreach($content['nextmatch']['rows'] as $key => $row) {
				if(!(is_numeric($key) && isset($row['shipment_id'])) || $row['received'] == 0) {
					continue;
				}
				$row['serial_numbers'] = unserialize(stripslashes(urldecode($row['serial_numbers'])));
				if(is_null($shipment) || $shipment->get_shipment_id() != $row['shipment_id']) {
					if($shipment) {
						$shipment->write();
					}
					$shipment =& new IncomingShipment($row['shipment_id']);
				}
				$line = $shipment->get_line($row['line_no']);

				if(!($row['serial_numbers'])) {
					$row['serial_numbers'] = array();
				}
				$receipt = new ReceiveLine($line, $GLOBALS['egw_info']['user']['account_id'], $row['received'], $row['serial_numbers']);

				try {
					// receive() will throw an exception if the serial numbers are bad
					$receipt->receive();
					$success = true;
				} catch (Exception $e) {
					$this->messages[] = array( $e->getMessage(), 'error');
				}
				unset($content['nextmatch']['rows'][$key]);
			}
			return $success;
		}

		/**
		*	We received something we didn't order.
		*
		*	Since shipments refer back to their order, rather than copy the information, we can't receive
		*	something that wasn't on an order.  We'll deal with this item by:
		*	1.  Check for other Purchase Orders.  It could be that the user chose the wrong PO.  Offer to receive against that PO.
		*	2.  Add a line to an existing Purchase Order.  
		*	3.  Create a new Purchase Order for this item
		*	4.  Record but don't receive and defer to someone else
		*
		*	The first and last options can be done for anyone, but the other options are dependant on
		*	permissions.  If a user can't add or edit POs, they won't be offered those options.
		*/
		public function not_ordered($content = array()) {
			// Get the shipment from the session
			$shipment =& $GLOBALS['egw']->session->appsession('receive_shipment');

			// Get the details of the item received
			$details = $GLOBALS['egw']->session->appsession('not_ordered');

			if(!$details) {
				throw new egw_exception_wrong_parameter('Missing receiving information');
			}

			// User has made a choice
			if($content['save'] && $content['choice']) {
				if($content['choice']['action'] && $content['choice']['target']) {
					$action = $content['choice']['action'];
					$target = $content['choice']['target'];
				} else {
					list($action, $target) = explode(':', $content['choice']['action']);
				}
				$result = $this->bo->not_ordered($shipment, $details, $action, $target);
				if($result === true) {
					$this->messages[] = array(lang('Received %1 of %2', $details['quantity_received'], $details['stock_code']), 'success');
					$GLOBALS['egw']->session->appsession('not_ordered', 'perp_ap', null);
					$GLOBALS['egw']->session->appsession('receive_shipment', 'perp_ap', $shipment);
					return $this->receive_new($details);
				} elseif($result) {
					$this->messages[] = array($result, 'error');
				} else {
					$this->messages[] = array(lang('Your choice was not recorded.  I don\'t know why.'), 'error');
				}
				$GLOBALS['egw']->session->appsession('receive_shipment', 'perp_ap', $shipment);
			}

			// Get the choices available to the user
			$choices = $this->get_choices($shipment->get_shipment_id(), $details);

			// No choice?  Don't ask, just do it
			if(count($choices) == 1) {
				$content['choice'] = $choices[0];
				$content['save'] = true;
				return $this->not_ordered($content);
			}

			// Format choices to pass them to the etemplate
			foreach($choices as $choice) {
				$data['choice'][] = array(
					'action'	=>	$choice['action'] . ':' . $choice['target'],
					'label'		=>	$choice['label']
				);
			}

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Item not ordered - %1', $details['stock_code']);
			$this->template->read('perp_ap.not_ordered');
			$this->template->exec('perp_ap.ui_perp_receive.not_ordered', $data, $select_box_options, $readonly, $preserve);
		}

		protected function get_choices($shipment_id, $details) {
			$choices = array();

			// We need the shipment_id before we can defer
			if($shipment_id) {
				$choices[] = array(
					'action'	=>	'defer',
					'label'		=>	lang('Record it and let someone else deal with it')
				);
			}

			// Check for other purchase orders
			$other_orders = $this->bo->find_other_orders($details['supplier_id'], $details['stock_code']);
			foreach($other_orders as $order) {
				$choices[] = array(
					'action'	=>	'other_order',
					'target'	=>	$order['order_id'],
					'label'		=>	lang('Receive from Purchase Order %1 (%2 ordered) instead', 
						$order['our_reference_number'],
						$order['total_ordered']
					)
				);
			}

			// Check if we can add to existing Purchase Orders
			if($details['order_id'] && $GLOBALS['egw']->acl->check('order', EGW_ACL_EDIT, 'perp_ap')) {
				$choices[] = array(
					'action'	=>	'add_to_order',
					'target'	=>	$details['order_id'],
					'label'		=>	lang('Add to Purchase Order %1', ExecMethod('perp_ap.bo_perp_purchase_order.link_title', $details['order_id']))
				);
			}

			// Check if we can create a new Purchase Order for this item
			/* Not implemented yet
			if($GLOBALS['egw']->acl->check('order', EGW_ACL_ADD, 'perp_ap')) {
				$choices[] = array(
					'action'	=>	'new_order',
					'label'		=>	lang('Create a new Purchase Order for this item')
				);
			}
			*/

			return $choices;
		}
	}
?>
