<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.Invoice.inc.php');

	/**
	*	Report that lists detailed amounts payable by days overdue
	*	Grouped by currency then supplier
	*/
	class detailed_aged_accounts_payable extends perp_report {

		public function detailed_aged_accounts_payable (array $parameters = array()) {
			parent::perp_report($parameters);

			$this->template = 'detailed_aged_accounts_payable.odt';

			// Default parameters
			if(!$this->parameters['end_date']) {
				$this->parameters['end_date'] = date('U');
				$this->parameters['options-currencies'] = ExecMethod('perp_api.bo_perp_api.get_currencies');
				$this->parameters['start_supplier'];
				$this->parameters['end_supplier'];
			}
		}

		public function get_data($query = array()) {
			if($query['start_supplier']) {
				$start_supplier = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $query['start_supplier']);
				$query['start_supplier'] = $start_supplier['supplier_num'];
			}
			if($query['end_supplier']) {
				$end_supplier = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $query['end_supplier']);
				$query['end_supplier'] = $end_supplier['supplier_num'];
			}
			
			$data = array(
				'end_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date']),
				'currencies'	=>	(is_array($query['currencies']) || $query['currencies'] != '' ? $query['currencies'] : lang('All')),
			);
			if($query['start_supplier'] || $query['end_supplier']) {
				$data['supplier_range'] = lang('Supplier: %1 to %2',
					($query['start_supplier'] ? $query['start_supplier'] : lang('all')),
					($query['end_supplier'] ? $query['end_supplier'] : lang('all')) 
				);
			} else {
				$data['supplier_range'] = lang('all suppliers');
			}
			if($query['currencies']) {
				$query['currencies'] = explode(',', $query['currencies']);
			}

			$sql = 'SELECT
				perp_supplier_ledger.trans_id,
				perp_supplier_ledger.supplier_id,
				perp_supplier_ledger.exchange_rate,
				perp_general_ledger.trans_date AS bill_date,
				perp_general_ledger.narrative AS our_reference,
				perp_invoice.their_reference_number AS their_reference,
				perp_supplier_ledger.amount AS original_amount,
				invoice_total.total AS invoice_total,
				IF(payment_total.total IS NULL, perp_supplier_payment.amount, payment_total.total) AS payment_total,
				perp_invoice.invoice_paid,
				(
					IF(invoice_total.total IS NULL, 0, invoice_total.total) - 
					IF(payment_total.total IS NULL, IF(perp_supplier_payment.amount IS NULL, 0, perp_supplier_payment.amount), payment_total.total)
				) AS amount_owing,
				perp_invoice.discount_taken,
				perp_invoice.due_date,
				perp_supplier.supplier_num,
				perp_supplier.supplier_name,
				perp_supplier.supplier_currency,
				perp_currency.currency_name
				FROM perp_supplier_ledger
				JOIN perp_general_ledger ON
					perp_supplier_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_supplier_ledger.line_no = perp_general_ledger.line_no
				LEFT JOIN (
					perp_invoice JOIN (
						SELECT
							invoice_id,
							sum(ROUND(unit_price * quantity, 2)) AS total
						FROM perp_invoice_line
						GROUP BY invoice_id
					) AS invoice_total ON
						perp_invoice.invoice_id = invoice_total.invoice_id
				) ON
					perp_invoice.invoice_transaction_id = perp_supplier_ledger.trans_id
				LEFT JOIN (
					SELECT 
						invoice_id,
						sum(amount) AS total
					FROM perp_supplier_payment 
					GROUP BY invoice_id
				) AS payment_total ON
					payment_total.invoice_id = perp_invoice.invoice_id
				LEFT JOIN perp_supplier_payment ON
					!perp_supplier_payment.invoice_id AND
					perp_supplier_payment.trans_id = perp_supplier_ledger.trans_id
				JOIN perp_supplier ON
					perp_supplier_ledger.supplier_id = perp_supplier.supplier_id
				JOIN perp_currency ON
					perp_supplier.supplier_currency = perp_currency.currency_id
				WHERE (perp_invoice.invoice_id IS NULL OR !perp_invoice.invoice_paid) AND
					perp_supplier_ledger.trans_id NOT IN (
						SELECT trans_id 
						FROM perp_supplier_payment
						WHERE invoice_id != 0
					) ';
			if($query['currencies']) {
				$sql .= 'AND perp_currency.currency_id IN (\'' . implode("','", $query['currencies']) . '\') ';
			}
			if($query['start_supplier']) {
				$sql .= 'AND perp_supplier.supplier_num >= \'' . $query['start_supplier'] . '\' ';
			}
			if($query['end_supplier']) {
				$sql .= 'AND perp_supplier.supplier_num <= \'' . $query['end_supplier'] . '\' ';
			}
			if($query['end_date']) {
				$sql .= 'AND perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['end_date'])) . ' ';
			}
			$sql .= 'GROUP BY perp_invoice.invoice_id, perp_supplier_payment.invoice_id
				ORDER BY perp_currency.currency_order, perp_supplier.supplier_num';

			$invoices = array();
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			while($row = $GLOBALS['egw']->db->row(true)) {
				if($query['end_date']) {
					if(strtotime($row['bill_date']) > $query['end_date']) {
						continue;
					}
				}
				$row['amount_owing'] -= $row['discount_taken'];
				$invoices[] = $row;
			}

			$currency_total = array();
			$supplier = '';
			$currency = '';
			foreach($invoices as $key => &$invoice) {
				// Massage data to fit into report


				// Set defaults
				$invoice['more'] = 0;
				$invoice['60'] = 0;
				$invoice['30'] = 0;
				$invoice['current'] = 0;
				$invoice['total_overdue'] = 0;

				if($currency != $invoice['supplier_currency']) {
					if(count($currency_total)) {
						// Keep total currency
						$currency_total['currency'] = $currency;
						$total_currency[] = $currency_total;
					}
					$currency = $invoice['supplier_currency'];
					$currency_total = array(
						'current'	=>	0,
						'30'		=>	0,
						'60'		=>	0,
						'more'		=>	0,
						'overdue'	=>	0,
						'total'		=>	0
					);
				}
				if($supplier != $invoice['supplier_id']) {
					$supplier = $invoice['supplier_id'];
					$supplier_total = array(
						'current'	=>	0,
						'30'		=>	0,
						'60'		=>	0,
						'more'		=>	0,
						'overdue'	=>	0,
						'total'		=>	0
					);
				}

				if(is_null($invoice['due_date'])) {
					$overdue_days = 0;
				} else {
					$overdue_days = round((date('U') - $GLOBALS['egw']->db->from_timestamp($invoice['due_date'])) / (24*60*60), 0);
					$overdue_days = $overdue_days < 0 ? 0 : $overdue_days;
				}
				if($overdue_days == 0) {
					$range = 'current';
				} elseif ($overdue_days <= 30) {
					$range = '30';
				} elseif ($overdue_days <= 60) {
					$range = '60';
				} else {
					$range = 'more';
				}

				$invoice[$range] = $invoice['amount_owing'];
				$supplier_total[$range] += $invoice['amount_owing'];
				$currency_total[$range] += $invoice['amount_owing'];
				if($overdue_days > 0) {
					$invoice['total_overdue'] = $invoice['amount_owing'];
					$supplier_total['overdue'] += $invoice['amount_owing'];
					$currency_total['overdue'] += $invoice['amount_owing'];
				}
				$invoice['total_payable'] = $invoice['amount_owing'];
				$supplier_total['total'] += $invoice['amount_owing'];
				$currency_total['total'] += $invoice['amount_owing'];

				// Set totals
				foreach($supplier_total as $s_range => $s_total) {
					$invoice['supplier_total_' . $s_range] = $s_total;
				}
				foreach($currency_total as $c_range => $c_total) {
					$invoice['currency_total_' . $c_range] = $c_total;
				}
			}

			if(count($invoices) == 0) {
				// Define empty totals so TBS doesn't complain
				$total_currency[] = array(
					'currency'	=>	'',
					'currency_name'	=>	'',
					'current'	=>	0,
					'30'		=>	0,
					'60'		=>	0,
					'more'		=>	0,
					'overdue'	=>	0,
					'total'		=>	0,
				);
			} else {
				// Get last currency
				$currency_total['currency'] = $currency;
				$total_currency[] = $currency_total;
				// Get currency names
				$currency_list = ExecMethod('perp_api.bo_perp_api.get_currencies');
				foreach($total_currency as &$currency) {
					$currency['currency_name'] = $currency_list[$currency['currency']];
				}
			}
			$data['detail'] = $invoices;
			$data['total'] = $total_currency;

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}
	}
?>
