<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');

	/**
	*	Report that lists detailed amounts payable by days overdue
	*	One line per supplier, grouped by currency
	*/
	class summary_aged_accounts_payable extends perp_report {

		public function summary_aged_accounts_payable (array $parameters = array()) {
			parent::perp_report($parameters);

			$this->template = 'summary_aged_accounts_payable.odt';

			// Default parameters
			if(!$parameters['end_date']) {
				$this->parameters['end_date'] = date('U');
				$this->parameters['start_supplier'];
				$this->parameters['end_supplier'];
				$this->parameters['only_non_zero'] = true;
			}
			$this->parameters['options-currencies'] = ExecMethod('perp_api.bo_perp_api.get_currencies');
		}

		public function get_data($query = array()) {
			if($query['start_supplier']) {
				$start_supplier = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $query['start_supplier']);
				$query['start_supplier'] = $start_supplier['supplier_num'];
			}
			if($query['end_supplier']) {
				$end_supplier = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $query['end_supplier']);
				$query['end_supplier'] = $end_supplier['supplier_num'];
			}
			
			$data = array(
				'end_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date']),
				'currencies'	=>	(is_array($query['currencies']) || $query['currencies'] != '' ? $query['currencies'] : lang('All')),
			);
			if($query['start_supplier'] || $query['end_supplier']) {
				$data['supplier_range'] = lang('Supplier: %1 to %2',
					($query['start_supplier'] ? $query['start_supplier'] : lang('all')),
					($query['end_supplier'] ? $query['end_supplier'] : lang('all')) 
				);
			} else {
				$data['supplier_range'] = lang('all suppliers');
			}
			if($query['only_non_zero']) {
				$data['supplier_range'] .= ' - ' . lang('only non-zero accounts');
			}

			if($query['currencies']) {
				$query['currencies'] = explode(',', $query['currencies']);
			}

			$sql = 'SELECT
				perp_supplier_ledger.trans_id,
				perp_supplier.supplier_id,
				perp_supplier_ledger.exchange_rate,
				narrative,
				perp_supplier_ledger.amount AS original_amount,
				invoice_total.total AS invoice_total,
				IF(payment_total.total IS NULL, perp_supplier_payment.amount, payment_total.total) AS payment_total,
				perp_invoice.invoice_paid,
				ROUND(
					IF(invoice_total.total IS NULL, 0, invoice_total.total) - 
					(
						IF(payment_total.total IS NULL, IF(perp_supplier_payment.amount IS NULL, 0, perp_supplier_payment.amount), payment_total.total) +
						IF(discount_taken IS NULL, 0, discount_taken)
					)
				,2) AS amount_owing,
				IF(total_payable.total IS NULL, 0, total_payable.total) AS total_payable,
				perp_invoice.discount_taken,
				perp_invoice.due_date,
				perp_supplier.supplier_num,
				perp_supplier.supplier_name,
				perp_supplier.supplier_currency,
				perp_currency.currency_name
			FROM perp_supplier
			LEFT JOIN perp_supplier_ledger ON
				perp_supplier_ledger.supplier_id = perp_supplier.supplier_id
			LEFT JOIN perp_general_ledger ON
				perp_supplier_ledger.trans_id = perp_general_ledger.trans_no AND
				perp_supplier_ledger.line_no = perp_general_ledger.line_no
			LEFT JOIN (
				SELECT
					supplier_id,
					-SUM(ROUND(perp_supplier_ledger.amount / exchange_rate, 2)) AS total
				FROM perp_supplier_ledger 
				JOIN perp_general_ledger ON
					perp_general_ledger.trans_no = perp_supplier_ledger.trans_id 
					AND perp_general_ledger.line_no = perp_supplier_ledger.line_no ' .
				($query['end_date'] ? 'WHERE perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['end_date'])) : '') . '
				GROUP BY supplier_id
			) AS total_payable ON
				perp_supplier.supplier_id = total_payable.supplier_id
			LEFT JOIN (
				perp_invoice JOIN (
					SELECT
						invoice_id,
						sum(unit_price * quantity) AS total
					FROM perp_invoice_line
					GROUP BY invoice_id
				) AS invoice_total ON
				perp_invoice.invoice_id = invoice_total.invoice_id
			) ON
				perp_invoice.invoice_transaction_id = perp_supplier_ledger.trans_id
			LEFT JOIN (
				SELECT 
					invoice_id,
					sum(amount) AS total
				FROM perp_supplier_payment 
				GROUP BY invoice_id
			) AS payment_total ON
				payment_total.invoice_id = perp_invoice.invoice_id
			LEFT JOIN perp_supplier_payment ON
				!perp_supplier_payment.invoice_id AND
				perp_supplier_payment.trans_id = perp_supplier_ledger.trans_id
			JOIN perp_currency ON
				perp_supplier.supplier_currency = perp_currency.currency_id';

			if($query['currencies']) {
				$where[] = 'perp_currency.currency_id IN (\'' . implode("','", $query['currencies']) . '\') ';
			}
			if($query['start_supplier']) {
				$where[] = 'perp_supplier.supplier_num >= \'' . $query['start_supplier'] . '\' ';
			}
			if($query['end_supplier']) {
				$where[] = 'perp_supplier.supplier_num <= \'' . $query['end_supplier'] . '\' ';
			}
			if($query['end_date']) {
				$where[] = '(perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['end_date']))  . 
					' OR trans_date IS NULL)';
			}
			if(count($where) > 0) {
				$sql .= ' WHERE ' . implode(' AND ', $where);
			}

			if(is_numeric($query['start_balance'])) {
				$having[] = 'total_payable >= ' . $query['start_balance'] . ' ';
			}
			if(is_numeric($query['end_balance'])) {
				$having[] = 'total_payable <= ' . $query['end_balance'] . ' ';
			}
			if($query['only_non_zero']) {
				$having[] = 'total_payable != 0';
			}
			if(count($having) > 0) {
				$sql .= ' HAVING ' . implode(' AND ', $having);
			}
			$sql .= ' ORDER BY perp_currency.currency_order, perp_supplier.supplier_num';

			$invoices = array();
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['due_date'] = $GLOBALS['egw']->db->from_timestamp($row['due_date']);
				$invoices[] = $row;
			}
			$currency_total = array();
			$supplier = '';
			$currency = '';
			$detail = array();
			foreach($invoices as $key => &$invoice) {
				// Massage data to fit into report
				if($currency != $invoice['supplier_currency']) {
					if(count($currency_total)) {
						// Keep total currency
						$currency_total['currency'] = $currency;
						$total_currency[] = $currency_total;
					}
					$currency = $invoice['supplier_currency'];
					$currency_total = array(
						'current'	=>	0,
						'7'		=>	0,
						'14'		=>	0,
						'30'		=>	0,
						'more'		=>	0,
						'overdue'	=>	0,
						'total'		=>	0
					);
				}
				if($supplier != $invoice['supplier_id']) {
					$supplier = $invoice['supplier_id'];
					$detail[$invoice['supplier_id']] = array(
						'current'	=>	0,
						'7'		=>	0,
						'14'		=>	0,
						'30'		=>	0,
						'more'		=>	0,
						'total_overdue'	=>	0,
						'total_payable'	=>	0
					);
				}
				$detail[$invoice['supplier_id']]['supplier_num'] = $invoice['supplier_num'];
				$detail[$invoice['supplier_id']]['supplier_name'] = $invoice['supplier_name'];
				$detail[$invoice['supplier_id']]['supplier_currency'] = $invoice['supplier_currency'];
				$detail[$invoice['supplier_id']]['currency_name'] = $invoice['currency_name'];


				$overdue_days = !($invoice['due_date']) ? 0 : floor((date('U') - $invoice['due_date']) / (24*60*60));
				$overdue_days = $overdue_days < 0 ? 0 : $overdue_days;
				if($overdue_days == 0) {
					$range = 'current';
				} elseif ($overdue_days <= 7) {
					$range = '7';
				} elseif ($overdue_days <= 14) {
					$range = '14';
				} elseif ($overdue_days <= 30) {
					$range = '30';
				} else {
					$range = 'more';
				}

				$detail[$invoice['supplier_id']][$range] += $invoice['amount_owing'];
				$currency_total[$range] += $invoice['amount_owing'];
				if($overdue_days > 0) {
					$detail[$invoice['supplier_id']]['total_overdue'] += $invoice['amount_owing'];
					$currency_total['overdue'] += $invoice['amount_owing'];
				}
				$detail[$invoice['supplier_id']]['total_payable'] = $invoice['total_payable'];
				$currency_total['total'] += $invoice['amount_owing'];

				// Set totals
				foreach($currency_total as $c_range => $c_total) {
					$detail[$invoice['supplier_id']]['currency_total_' . $c_range] = $c_total;
				}
			}

			if(count($invoices) == 0) {
			} else {
				// Get last currency
				$currency_total['currency'] = $currency;
				$total_currency[] = $currency_total;

			}
			$total_currency = array();
			$currency_total = array();
			$data['detail'] = array();

			foreach($detail as $supplier_id => &$supplier_line) {
				$supplier_line['supplier_id'] = $supplier_id;
				if($query['start_balance'] != '' && $supplier_line['total_payable'] < $query['start_balance']) {
					continue;
				} elseif($query['end_balance'] != '' && $supplier_line['total_payable'] > $query['end_balance']) {
					continue;
				} else {
					$data['detail'][] = $supplier_line;
					$currency_total[$supplier_line['supplier_currency']]['current'] += $supplier_line['current'];
					$currency_total[$supplier_line['supplier_currency']]['7'] += $supplier_line['7'];
					$currency_total[$supplier_line['supplier_currency']]['14'] += $supplier_line['14'];
					$currency_total[$supplier_line['supplier_currency']]['30'] += $supplier_line['30'];
					$currency_total[$supplier_line['supplier_currency']]['more'] += $supplier_line['more'];
					$currency_total[$supplier_line['supplier_currency']]['total_overdue'] += $supplier_line['total_overdue'];
					$currency_total[$supplier_line['supplier_currency']]['total_payable'] += $supplier_line['total_payable'];
				}
			}
			// Get currency names
			$currency_list = ExecMethod('perp_api.bo_perp_api.get_currencies');
			foreach($currency_total as $currency_id => &$currency) {
				$currency['currency'] = $currency_id;
				$currency['currency_name'] = $currency_list[$currency['currency']];
				$total_currency[] = $currency;
			}
			$data['total'] = $total_currency;

//_debug_array($data); die();
			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}
	}
?>
