<?php
	/**
	*	In order for an Invoice to be automatically generated from a record of a non-pERP eGroupWare
	*	application, the eGroupWare application should create an object that implements Invoicable.
	*	Then create a new ReceivableInvoice, and call read_from_object(<your Invoicable>).  
	*
	*	This will create a new ReceivableInvoice, then display it for editing:
	*	@code
	*		$my_record = new MyInvoicable($my_record_id);
	*		$invoice = new ReceivableInvoice();
	*		$invoice->read_from_object($my_record);
	*		$GLOBALS['egw']->session->appsession('invoice', 'perp_ar', $invoice);
	*		$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ar.ui_perp_receivable_invoice.edit'));
	*	@endcode
	*
	*	@see InvoicableItem
	*
	*	@author Nathan Gray
	*	@ingroup perp_api
	*/

	interface Invoicable {

		/**
		*	For a ReceivableInvoice (they owe us), this should be 0.
		*	For a PayableInvoice (we owe them), this should be the supplier we need to pay.
		*
		*	@return int
		*/
		public function get_supplier_id();

		/**
		*	For a ReceivableInvoice (they owe us), this should be the ID of the client 
		*	who we are charging.
		*	For a PayableInvoice (we owe them), this should be 0.
		*
		*	@return int
		*/
		public function get_client_id();

		/**
		*	The contact_id (addressbook) of the person in our organization who should be 
		*	contacted regarding this invoice.  This should probably be the originator of the
		*	Invoicable, as they would know the most about it.
		*
		*	@return int
		*/
		public function get_our_contact_person();

		/**
		*	The contact_id (addressbook) of the person in their organization who should be 
		*	contacted regarding this invoice.  
		*
		*	@return int
		*/
		public function get_their_contact_person();

		/**
		*	contact_id (addressbook) of the address to send the payment to
		*
		*	@return int
		*/
		public function get_bill_from_address();

		/**
		*	contact_id (addressbook) of the address to send the invoice to
		*
		*	@return int
		*/
		public function get_bill_to_address();

		/**
		*	3 character currency code for the currency the invoice is in
		*
		*	@return string
		*/
		public function get_invoice_currency();

		/**
		*	Get the terms for this invoice.  
		*	
		*	@return int
		*/
		public function get_payment_terms();

		/**
		*	Get an array of InvoicableItem objects that will be used
		*	to create the lines on the Invoice.
		*
		*	@return array of InvoicableItems
		*/
		public function &get_invoicable_items();
	}
?>
