<?php

	/**
	*	Details on an ShipmentLine.  Used for specifying various things
	*	on an shipment line, such as which package it was shipped in.
	*	
	*	@author Nathan Gray
	*	@ingroup perp_api
	*/

	class ShipmentLineDetail {

		/**
		*	Database table for storage
		*/
		const detail_table = 'perp_shipment_detail';

		/**
		*	ShipmentLine this detail describes
		*/
		protected $shipment_line = null;

		/**
		*	Identifying number for this detail
		*/
		protected $detail_number = null;

		/**
		*	Name of the detail.
		*	For example: package
		*/
		protected $detail_name = null;

		/**
		*	Value of the detail.
		*	For example: A, B, 1, 2, Front
		*/
		protected $detail_value = null;

		/**
		*	Display this detail in the list of details?
		*/
		protected $display = true;
		

		public function __construct(ShipmentLine &$line, $data) {
			$this->set_shipment_line($line);
			$this->detail_number = $data['detail_number'];
			$this->detail_name = $data['detail_name'];
			$this->detail_value = $data['detail_value'];
			if(isset($data['display'])) {
				$this->display = $data['display'];
			}
		}

		/**
		*	This function is a stub to allow extending functions
		*	to read their own data.  Actual reading is done in
		*	ShipmentLine.
		*/
		public function read() {
		}

		/**
		*	Write this ShipmentLineDetail into the database for long term storage
		*/
		public function write() {
			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->insert(self::detail_table,
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number,
					'detail_class'	=>	get_class($this)
				) + $this->toArray(),
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number,
				),
				__LINE__, __FILE__, 'perp_api'
			);
		}

		/**
		*	Get the ShipmentLine this ShipmentLineDetail describes
		*/
		public function &get_shipment_line() {
			return $this->shipment_line;
		}
		/**
		*	Set the ShipmentLine this ShipmentLineDetail describes
		*/
		public function set_shipment_line(ShipmentLine &$shipment_line) {
			$this->shipment_line =& $shipment_line;
		}

		/**
		*	Get the identifying number for this ShipmentLineDetail
		*/
		public function get_number() {
			return $this->detail_number;
		}

		/**
		*	Set the identifying number for this ShipmentLineDetail.
		*	This should only be called from ShipmentLine.
		*	@param number New identifying number. 
		*/
		public function set_number($number) {
			if($this->shipment_line->get_detail($number) !== $this) {
				throw new Exception('Call $shipment_line->add_detail, don\'t call set_number directly');
			}
			$this->detail_number = $number;
		}

		/**
		*	Get the name of this detail
		*/
		public function get_name() {
			return $this->detail_name;
		}
		/**
		*	Set the name of this detail
		*/
		public function set_name($name) {
			$this->detail_name = $name;
		}

		/**
		*	Get the value of this detail
		*/
		public function get_value() {
			return $this->detail_value;
		}
		/**
		*	Set the value of this detail
		*/
		public function set_value($value) {
			$this->detail_value = $value;
		}

		/**
		*	Get whether or not to display this detail in a list
		*	@return boolean display
		*/
		public function get_display() {
			return $this->display;
		}
		/**
		*	Set whether or not to display this detail in a list
		*/
		public function set_display($display) {
			$this->display = (boolean)$display;
		}

		/**
		*	Function called when this ShipmentLineDetail is added to the
		*	ShipmentLine.
		*/
		public function add() {}

		/**
		*	Function called when this ShipmentLineDetail is removed from the
		*	ShipmentLine.
		*/
		public function remove() {}

		/** 
		*	Return an array of this ShipmentLineDetail's properties
		*	@return array
		*/
		public function toArray() {
			return array(
				'detail_number'	=>	$this->detail_number,
				'detail_name'	=>	$this->detail_name,
				'detail_value'	=>	$this->detail_value,
				'display'	=>	$this->display
			);
		}

		/**
		*	Return a string representation of this object.
		*
		*	@return string Representation of this ShipmentLineDetail
		*/
		public function __toString() {
			return sprintf("%-10s\t%-10s\t%-20s\n",
				get_class($this),
				$this->detail_name,
				$this->detail_value
			);
		}

		/**
		*	Return an array of properties to preserve across
		*	serialization.
		*/
		public function __sleep() {
			return( array_keys( get_object_vars( $this ) ) );
		}
	}
?>
