<?php
	require_once('class.ShipmentLine.inc.php');
	require_once('class.ReceiptLine.inc.php');

	interface ShippableItem {
		/**
		*	Mark this Shippable Item as shipped.  
		*	The ShipmentLine will take care of any General Ledger transactions and will
		*	adjust inventory counts if required.
		*	
		*	@param shipment_line	ShipmentLine that is moving this ShippableItem
		*/
		public function ship(ShipmentLine &$shipment_line);

		/**
		*	Mark this Shippable Item as received.  
		*	The ReceiptLine will take care of any General Ledger transactions and
		*	adjust inventory counts if required.
		*/
		public function receive(ReceiptLine &$receipt_line);

		/**
		*	Check to see if this ShippableItem can be shipped.  Depends on 
		*	status and permissions.  It can't be shipped if it has already been completely
		*	shipped, it can't ship if there is approval required.
		*	@throws Exception with error message if it can't be shipped.
		*	@return boolean 
		*/
		public function can_ship();

		/**
		*	Check to see if this ShippableItem can be received.  Depends on 
		*	status and permissions.  It can't be received if it has already been completely
		*	received.
		*	@throws Exception with error message if it can't be shipped.
		*	@return boolean 
		*/
		public function can_receive();

		/**
		*	Get the quantity to be shipped, in Supplier's units of measure.
		*	
		*	@return float quantity to be shipped
		*/
		public function get_ship_quantity();

		/**
		*	Get unit price for one unit of the ShippableItem being shipped.
		*	This is in the supplier's units of measure.
		*
		*	@return float the price of one unit 
		*/
		public function get_ship_unit_price();

		/**
		*	Return a unique identifier for the  Shippable object that this 
		*	ShipableItem is part of	for storage & recovery purposes.
		*/
		public function get_shippable_id();

		/**
		*	Return a unique (amongst the owning object's ShippableItems)
		*	identifier for this ShippableItem
		*/
		public function get_shippable_line_id();

		/**
		*	Return an array that will work for an eGroupWare style link.
		*	This is for allowing cross linking and jumping around.
		*	The array contain a sub-array, with keys app, id:
		*	return array(
		*		array(
		*			'app'	=>	$GLOBALS['egw_info']['flags']['currentapp'],
		*			'id'	=>	'ORDER-' . $this->get_owner->get__id()
		*		)
		*	);
		*
		*	This function can't actually be here, because of multiple inheritance, but it'll work.
		*	@see etemplate::inc::class.link_widget.inc.php
		*	@return array
		public function get_link();
		*/
	}
?>
