<?php
	/**
	*	Ajax functions for editing an invoice
	*
	*	@author Nathan Gray
	*/

	require_once('class.Invoice.inc.php');
	require_once(EGW_API_INC.'/xajax.inc.php');

	abstract class ajax_perp_invoice {

		/**
		*	xajax object for returning XML responses
		*/
		private $xajax;

		/**
		*	Constructor\n
		*	Creates references to xajax
		*/
		public function __construct() {
			$this->xajax = new xajax();
		}

		/**
		*       Change the information specific to suppliers when the user picks a different supplier:
		*       - Contact person
		*       - Currency
		*       - Payment terms
		*	- Catalog
		*       This is only possible for an new Invoice
		*       @param supplier_id int     Supplier ID
		*       @return xml response
		*/
		public function change_supplier($supplier_id) {

			$response = new xajaxResponse();

			$contacts = egw_link::get_links('perp_ap', 'SUPPLIER-' . $supplier_id, 'addressbook');
			$response->addScript("removeOptions('exec[their_contact_person]')");
			if(is_array($contacts)) {
				foreach($contacts as $key => $contact_id) {
					$contact = egw_link::title('addressbook', $contact_id);
					$response->addScript("addOption('exec[their_contact_person]', '" . $contact_id . "', '" . $contact . "')");
				}
			}

			// Default currency to suppliers
			$supplier_data = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $supplier_id);
			if(!is_object($GLOBALS['perp_api']['bo'])) {
				$GLOBALS['perp_api']['bo'] =& CreateObject('perp_api.bo_perp_api');
			}
			$GLOBALS['perp_api']['bo']->get_currencies( 
				array(
					'template'	=>	false,
					'currency_id'	=>	$supplier_data['supplier_currency']
				),
				$currencies
			);
			$response->addScript('
				var currency = document.getElementById("exec[invoice_currency]");
				if(currency) {
					currency.value ="'. $supplier_data['supplier_currency'] . '";
				}
				var exchange_rate = document.getElementById("exec[exchange_rate]");
				if(exchange_rate) {
					exchange_rate.value = "' . $currencies[0]['exchange_rate'] . '";
				}'
			);

			// Default payment terms
			$response->addScript('
				var payment_terms = document.getElementById("exec[payment_terms]");
				if(payment_terms) {
					payment_terms.value = "' . $supplier_data['payment_terms'] . '";
				}'
			);

			// Update Invoice
			$invoice = $GLOBALS['egw']->session->appsession('invoice');
			if($invoice instanceof Invoice) {
				$invoice->set_supplier_id((int)$supplier_id);
				$invoice->set_our_reference_number($invoice->suggest_invoice_number());
		
				$response->addScript("
					var our_reference = document.getElementById('exec[our_reference_number]');
					if(our_reference) {
						our_reference.value = '" . $invoice->get_our_reference_number() . "';
					}
				");
			}
			$GLOBALS['egw']->session->appsession('invoice', $GLOBALS['egw_info']['flags']['currentapp'], $invoice);
			return $response->getXML();
		}

		/**
		*	Update the due date based on the bill date and the payment terms
		*/
		public function update_due_date($bill_date, $terms) {
			$invoice = $GLOBALS['egw']->session->appsession('invoice');
			if(!($invoice instanceof Invoice)) {
				return;
			}
			$response = new xajaxResponse();

			$invoice->set_bill_date((int)strtotime($bill_date));
			$invoice->set_payment_terms((int)$terms);
			$due_date = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $invoice->get_due_date());

			$response->addAssign('exec[due_date][str]', 'value', $due_date);
			return $response->getXML();
		}

		/**
		*	Set the line details when the user chooses an item to add to this Invoice
		*
		*       @return xml response
		*/
		public function get_line_details($supplier_id, $supplier_code, $location) {
			$response = new xajaxResponse();

			return $response->getXML();
		}
		
		/** 
		*	Converts a php array to a javascript array 
		*	@todo Move this function.  This should go somewhere shared
		*
		*	@param name string javascript name of the array
		*	@param array	array	PHP array to convert
		*	@param prePend	string	String to prepend to the javascript array
		*	@return string Javascript array
		*/
		function phpArrayToJsArray($name,$array,$prePend='var ') {
			if (is_array($array)) { // Array recursion
				$result = $name.' = new Array();'."\n";
				foreach ($array as $key => $value) {
					$result .= $this->phpArrayToJsArray($name.'["'.$key.'"]',$value,'');
				}
			} else {  // Base case of recursion
				$result = $name.' = "'.addslashes($array).'";'."\n";
			}
			return $prePend.$result;
		}

	}
?>
