<?php
	/**
	*	Ajax functions for editing a order
	*
	*	@author Nathan Gray
	*/

	require_once('class.Order.inc.php');
	require_once(EGW_API_INC.'/xajax.inc.php');
	require_once(EGW_INCLUDE_ROOT.'/perp_ap/inc/class.bo_perp_catalog.inc.php');

	abstract class ajax_perp_order {

		/**
		*	xajax object for returning XML responses
		*/
		private $xajax;

		/**
		*	Response object
		*/
		protected $response;

		/**
		*	Constructor\n
		*	Creates references to xajax
		*/
		public function ajax_perp_edit() {
			$this->xajax = new xajax();
		}

		/**
		*       Change the information specific to suppliers when the user picks a different supplier:
		*       - Contact person
		*       - Currency
		*       - Payment terms
		*	- Catalog
		*       This is only possible for an new Order
		*       @param supplier_id int     Supplier ID
		*       @return xml response
		*/
		public function change_supplier($etemplate_exec_id, $supplier_id) {

			$this->response = new xajaxResponse();

			$unit_of_measure = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			$contacts = egw_link::get_links('perp_ap', 'SUPPLIER-' . $supplier_id, 'addressbook');
			$this->response->addScript("removeOptions('exec[their_contact_person]')");

			if(is_array($contacts)) {
				foreach($contacts as $key => $contact_id) {
					$contact = egw_link::title('addressbook', $contact_id);
					$this->response->addScript("addOption('exec[their_contact_person]', '" . $contact_id . "', '" . $contact . "')");
				}
			}

			// Default currency to suppliers
			$supplier_data = ExecMethod('perp_ap.bo_perp_supplier.get_supplier', $supplier_id);
			$this->response->addScript('
				var currency = document.getElementById("exec[order_currency]");
				if(currency) {
					currency.value ="'. $supplier_data['supplier_currency'] . '";
				}');

			// Default payment terms
			$this->response->addScript('
				var payment_terms = document.getElementById("exec[payment_terms]");
				if(payment_terms ) {
					payment_terms.value = "' . $supplier_data['payment_terms'] . '";
				}'
			);

			// Set supplier_id in supplier_code AJAX Select
			$this->response->addScript("
				var supplier_code = document.getElementById('exec[new][supplier_stock_code][results]');
				if(supplier_code) {
					supplier_code.options.supplier_id = '$supplier_id';
				}
			");

			// Update Order
			$order = unserialize($GLOBALS['egw']->session->appsession('order'));
			if($order instanceof Order) {
				$order->set_supplier_id((int)$supplier_id);
				if(!$order->get_our_reference_number()) {
					$order->set_our_reference_number($order->suggest_order_number());
		
					$this->response->addScript("
						var our_reference = document.getElementById('exec[our_reference_number]');
						if(our_reference) {
							our_reference.value = '" . $order->get_our_reference_number() . "';
						}
					");
				}
			}
			$GLOBALS['egw']->session->appsession('order', $GLOBALS['egw_info']['flags']['currentapp'], $order);
			return $this->response->getXML();
		}

		/**
		*       Change the information specific to clients when the user picks a different supplier:
		*       - Contact person
		*       - Currency
		*       - Payment terms
		*	- Catalog
		*       This is only possible for an new Order
		*/
		public function change_client($client_id) {
		}

		/**
		*	Suggest an order number for this order based on 
		*	the company's settings
		*       @return xml response
		*/
//		public abstract function suggest_order_number();

		/**
		*	Set the line details when the user chooses an item to add to this Order
		*
		*	@param supplier_id	int	Supplier's ID
		*	@param supplier_code	string	Supplier's identification number for the selected catalog item
		*	@param address		int	The address that this purchase order will be shipped to
		*       @return xml response
		*/
		public function get_line_details($supplier_id, $supplier_code, $address) {
			$this->response = new xajaxResponse();

			$item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $supplier_id, $supplier_code);
			$stock_id = $item['stock_id'];

			// Get location from address
			$locations = array();
			$bo_location = CreateObject('perp_inventory.bo_perp_location');
			$bo_location->get_location_list(array('delivery_address' => $address), $locations);
			$location = array_shift($locations);
			$location = $location['location_id'];

			// Get default order quantity
			$quantity = ExecMethod2('perp_ap.bo_perp_purchase_order.get_order_quantity', $location, $supplier_code);
			$this->response->addScript('
				var quantity = document.getElementById("exec[new][quantity_ordered]");
				if(quantity && quantity.value == "") {
					quantity.value = ' . $quantity . ';
				}
			');

			// Default description line
			$description = ExecMethod2('perp_ap.bo_perp_catalog.link_title', $supplier_id . '-' . $supplier_code);
			$description = addslashes($description);
			$this->response->addScript('
				var description = document.getElementById("exec[new][description]");
				if(description) {
					description.value = "' . $description . '";
				}'
			);

			// Unit Cost
			$this->response->addScript('
				var unit_price = document.getElementById("exec[new][unit_price]");
				if(unit_price) {
					unit_price.value = "' . $item['price'] . '";
				}'
			);

			// Focus to quantity field
			$this->response->addScript("
				var quantity = document.getElementById('exec[new][quantity_ordered]');
				quantity.select();
			");

			return $this->response->getXML();
		}

		/** 
		*	Converts a php array to a javascript array 
		*	@todo Move this function.  This should go somewhere shared
		*
		*	@param name string javascript name of the array
		*	@param array	array	PHP array to convert
		*	@param prePend	string	String to prepend to the javascript array
		*	@return string Javascript array
		*/
		function phpArrayToJsArray($name,$array,$prePend='var ') {
			if (is_array($array)) { // Array recursion
				$result = $name.' = new Array();'."\n";
				foreach ($array as $key => $value) {
					$result .= $this->phpArrayToJsArray($name.'["'.$key.'"]',$value,'');
				}
			} else {  // Base case of recursion
				$result = $name.' = "'.addslashes($array).'";'."\n";
			}
			return $prePend.$result;
		}

		/**
		*	The order could not be found in the session
		*
		*	Show an alert.
		*/
		protected function missing_order() {
			$this->response = new xajaxResponse();
			$this->response->addAlert(
				lang('Something went wrong with this order.  Your order may have been saved.  Please click Cancel at the bottom of the page, and try again.')
			);
			return $this->response->getXML();
		}
	}
?>
