<?php

	/**
	*	Logic layer for pERP API
	*
	*	@author Nathan Gray
	*	@ingroup perp_api
	*/
	class bo_perp_api {

		/**
		*       Reference to so_perp_api data storage object
		*/
		protected $so;

		/**
		*       boolean output extra debug information
		*/
		protected $debug = false;

		/**
		*	Cache for InvoicableItems to reduce duplication
		*	and make sure two lines from the same Invoicable point to the same object
		*/
		protected $invoicable_item_cache = array();

		/**
		*	Cache for ShippableItems to reduce duplication
		*	and make sure two lines from the same Shippable point to the same object
		*/
		protected $shippable_item_cache = array();

		/**
		*	Constructor
		*	Creates the data storage object
		*/
		public function bo_perp_api() {
			$this->so = CreateObject('perp_api.so_perp_api');
		}

		/**
		*	Destrutor
		*	Clears the caches so they don't get serialized
		*/
		public function __destruct() {
			$this->invoiceable_item_cache = array();
			$this->shippable_item_cache = array();
		}

		/**
		*	Change lookup table data
		*
		*	Changes data in lookup tables.  
		*
		*	@see perp_api::so_perp_api::change_lookup()
		*	@see perp_api::ui_perp_api::change_lookup()
		*	@param	table	string	Table name
		*	@param	app	string	Name of the application that owns the table
		*	@param	key	mixed	Key of the row to be changed, or null for a new entry
		*	@param	value	array	Values to set
		*/
		public function change_lookup($table, $app, $key, $value) {
			if($this->debug) {
				echo 'Key ' . print_r($key, true) . '<br />';
				echo 'Value ' . print_r($value, true) . '<br />';
			}
		//	if($GLOBALS['egw']->acl->check('run', EGW_ACL_EDIT)) {
				return $this->so->change_lookup($table, $app, $key ? $key : array(), $value);
			/*
			} else {
				return False;
			}
			*/
		}

		/**
		*	Get a list of shippers
		*
		*	@nextmatch
		*/
		public function get_shippers($query = array(), &$rows = array(), &$readonlys =array()) {
			if(!$GLOBALS['egw']->acl->check('shipper', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			// Make sure we get all shippers for a selectbox
			if(!isset($query['template'])) {
				if(!isset($query['num_rows'])) {
					$query['num_rows'] = -1;
				}
			}

			$list = $this->so->get_shippers($query);

			// Selectbox
			if(!isset($query['template'])) {
				foreach($list as $row) {
					$rows[$row['shipper_id']] = $row['shipper_name'];
				}
				return $rows;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = array();
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('shipper', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('shipper', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('shipper', EGW_ACL_DELETE);
				$rows = array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			$readonlys = array();
			foreach($list as $value) {
				$readonlys[] = $edit;
			}
			$rows += $list;
			return $query['count'];
		}

		/**
		*	Get the name of a shipper from its ID
		*/
		public function get_shipper_name($shipper_id) {
			$query = array(
				'shipper_id'	=>	$shipper_id,
			);
			$list = $this->get_shippers($query);
			return $list[$shipper_id];
		}

		/**
		*	Get a list of units of measures used in pERP
		*	
		*	@nextmatch
		*/
		public function get_unit_of_measures($query = array(), &$rows = array(), &$readonlys =array()) {

			if(!$GLOBALS['egw']->acl->check('unit_of_measure', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			if(!is_array($query) && is_numeric($query)) {
				$query = array(
					'uom_id'	=>	$query
				);
				$return_single = true;
			}
			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			if(!$query['order']) {
				$query['order'] = 'uom_id';
			}

			$list =  $this->so->get_unit_of_measures($query);

			if(!isset($query['template'])) {
				// Just wanted the name
				if($return_single && count($list) == 1) {
					return $list[0]['uom_name'];
				}

				// Selectbox
				foreach($list as $key => $value) {
					$rows[$value['uom_id']] = $value['uom_name'];
				}
				return $rows;
			} else {
				// Parameters given, return full data
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('unit_of_measure', EGW_ACL_ADD,'perp_api');
				$edit = $GLOBALS['egw']->acl->check('unit_of_measure', EGW_ACL_EDIT,'perp_api');
				$delete = $GLOBALS['egw']->acl->check('unit_of_measure', EGW_ACL_DELETE,'perp_api');
				$rows = array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			$readonlys = array();
			if(!$edit) {
				foreach($list as $key => $value) {
					$readonlys[$key + 1]['uom_name'] = true;
				}
			}

			$rows += $list;
			return $query['count'];
		}

		/**
		*	Get a list of currencies
		*	This function name is spelt wrong to fit with the ui_perp_api::change_lookup() scheme, use get_currencies() instead
		*/
		public function get_currencys(&$query = array(), &$rows = array(), &$readonlys = array()) {
			return $this->get_currencies($query, $rows, $readonlys);
		}

		/**
		*	Get a list of currencies used in pERP
		*	
		*	@nextmatch
		*/
		public function get_currencies($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('currency', EGW_ACL_READ, 'perp_api')) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_currencies($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['currency_id']] = $value['currency_name'];
				}
				return $select;
			}

			$edit = $GLOBALS['egw']->acl->check('currency', EGW_ACL_EDIT);
			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = array();
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('currency', EGW_ACL_ADD);
				$delete = $GLOBALS['egw']->acl->check('currency', EGW_ACL_DELETE);
				$rows = array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			foreach($list as $key => $value) {
				$readonlys[$key] = !$edit;
			}

			$rows += $list;
			return $query['count'];
		}

		/**
		*	Get a list of payment terms used in pERP
		*	This list is for both Accounts Receivable and Accounts Payable
		*
		*	@nextmatch
		*/
		public function get_payment_terms($query = array(), &$rows = array()) {
			$read = $GLOBALS['egw']->acl->check('payment_term', EGW_ACL_READ, 'perp_api');
			if(!$read) {
				if(!isset($query['rows'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_payment_terms($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['payment_term_id']] = $value['payment_term_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = array();
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('payment_term', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('payment_term', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('payment_term', EGW_ACL_DELETE);
				$rows = array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}
			$rows += $list;
			return $query['count'];
		}

		/**
		*	Get a list of payment methods used in pERP (Cash, Cheque, Visa, Mastercard)
		*	These payment methods are used in Accounts Payable and Accounts Receivable
		*
		*	@nextmatch
		*/
		public function get_payment_methods($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('payment_method', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_payment_methods($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['payment_method_id']] = $value['payment_method_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('payment_method', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('payment_method', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('payment_method', EGW_ACL_DELETE);
				$rows = array('add'=> $add, 'edit' => $edit, 'delete' => $delete) + $list;
			}

			return $query['count'];
		}

		/**
		*	Get a list of tax locations 
		*	These are not physical locations, although they could be, but areas a particular tax applies to.
		*
		*	@nextmatch
		*/
		public function get_tax_locations($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('tax_location', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			
			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_tax_locations($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['tax_location_id']] = $value['tax_location_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('currency', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('currency', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('currency', EGW_ACL_DELETE);
				$rows = $list + array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}
			return $query['count'];
		}
	
		/**
		*	Get a list of tax_categories
		*	This function name is spelt wrong to fit with the perp_api::change_lookup() scheme, use get_tax_categories() instead
		*/
		public function get_tax_categorys(&$query = array(), &$rows = array()) {
			return $this->get_tax_categories($query, $rows);
		}

		/**
		*	Get a list of tax categories taxable goods are put in \n
		*	For example, for Canadian GST, there's Luxury and Non-taxable.  Luxury is taxed at 6%.
		*
		*	@nextmatch
		*/
		public function get_tax_categories($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('tax_category', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_tax_categories($query);
			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['tax_category_id']] = $value['tax_category_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('currency', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('currency', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('currency', EGW_ACL_DELETE);
				$rows = $list + array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			return $query['count'];
		}
	
		/**
		*	Get a list of tax_authorities
		*	This function name is spelt wrong to fit with the perp_api::change_lookup() scheme, use get_tax_authorities() instead
		*/
		public function get_tax_authoritys($query = array(), &$rows = array()) {
			return $this->get_tax_authorities($query, $rows);
		}

		/**
		*	Get a list of Tax authorities that the taxes go to
		*	This is most often the actual name of the tax, such as Canadian GST, that the taxes go to
		*
		*	@nextmatch
		*/
		public function get_tax_authorities($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('tax_authority', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_tax_authorities($query);
			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['tax_authority_id']] = $value['tax_authority_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('currency', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('currency', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('currency', EGW_ACL_DELETE);
				$rows = $list + array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}
			return $query['count'];
		}
	
		/**
		*	Sometimes taxes are grouped together.  You may need to pay both GST and PST in some locations.
		*	Some taxes are charged before others, and sometimes tax is charged on taxes as well.
		*
		*	@nextmatch
		*/
		public function get_tax_groups($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('tax_group', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_tax_groups($query);
			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['tax_group_id']] = $value['tax_group_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('currency', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('currency', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('currency', EGW_ACL_DELETE);
				$rows = $list + array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			return $query['count'];
		}
	
		/**
		*	Get a list of grouped taxes
		*	This function name is spelt wrong to fit with the ui_perp_api::change_lookup() scheme, use get_tax_group_taxes() instead
		*/
		public function get_tax_group_taxs(&$query = array(), &$rows = array()) {
			return $this->get_tax_group_taxes($query, $rows);
		}

		/**
		*	Get a list of taxes grouped into tax groups
		*
		*	@nextmatch
		*/
		public function get_tax_group_taxes($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('tax_group_tax', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if($query['search']) {
				$where[] = 'tax_group_id LIKE \'%' . 
					$query['search'] . "%'";
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_tax_group_taxes($query);

			if(count($query) == 0) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['tax_group_id'] . ',' . $value['tax_authority_id']] = $value['tax_group_id'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('currency', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('currency', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('currency', EGW_ACL_DELETE);
				$rows = $list + array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			return $query['count'];
		}
	
		/**
		*	Get a list of the taxes and rates for tax authorities starting from a location for a category of item
		*
		*	@nextmatch
		*/
		public function get_tax_auth_rates($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('tax_auth_rate', EGW_ACL_READ, 'perp_api')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['num_rows'] && !isset($query['template'])) {
				// Need all the records for a selectbox
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_tax_auth_rates($query);

			if(!isset($query['template'])) {
				$rows = array();
				foreach($list as $row) {
					$rows[$row['tax_authority'] . ',' . $row['tax_source_location'] . ',' . $row['tax_category']] = $row;
				}
				return $rows;
			}

			if(!isset($query['rows'])) {
				// Parameters given, return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('tax_auth_rate', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('tax_auth_rate', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('tax_auth_rate', EGW_ACL_DELETE);
				$rows = $list + array('add'=> $add, 'edit' => $edit, 'delete' => $delete);
			}

			return $query['count'];
		}


		/**
		*	Get a list of ACL for the ui_perp_api::acl()
		*	Use the eGroupWare functions for actually determining access
		*
		*	@nextmatch
		*/
		public function get_acls($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('acl', EGW_ACL_READ, 'perp_api');
			if($read) {
				$app = $query['filter'];
				$list = $this->so->get_acls($app);
				$rows = array_slice($list, $query['start'], $query['num_rows']);
				foreach($rows as $line => $row) {
					$readonlys[($line+1).'[read]'] = !isset($row['read']);
					$readonlys[($line+1).'[add]'] = !isset($row['add']);
					$readonlys[($line+1).'[edit]'] = !isset($row['edit']);
					$readonlys[($line+1).'[delete]'] = !isset($row['delete']);
					$readonlys[($line+1).'[private]'] = !isset($row['private']);
					$readonlys[($line+1).'[admin]'] = !isset($row['admin']);
				}
				return count($list);
			}
			$rows = array();
			return 0;
		}

		/** 
		*	Formats a currency amount according to user preferences
		*	
		*	@param currency_id - 3 letter currency ID
		*	@param price - amount to format
		*	@author Christoph 
		*/
		public function get_formatted_price($currency_id, $price) {
			if($currency_id == null) {
				$config = config::read('perp_ledger');
				$currency_id = $config['gl_currency'];
			}
			return $this->so->get_formatted_price($currency_id, $price);
		}

		/**
		*	Changes a string in an international format into a float.
		*
		*	Accepted formats:
		*	- 12.345,67 (German)
		*	- 12,345.67 (English)
		*	- 12 345,67 (French)
		*	- 12'345,67 (Swiss)
		*
		*	Thanks to www.produkte24.com for publishing this on PHP.net docs
		*
		*	@param value A string that contains a number
		*	@return float
		*/
		public static function get_float_from_string($value) {
			$value = trim($value);
			if(preg_match("~^([-]?[0-9]+|(?:(?:[0-9]{1,3}([.,' ]))+[0-9]{3})*)(([.,])[0-9]{1,2})?$~", $value, $r)) {
				if(!empty($r['2'])) {
					$pre = preg_replace("~[".$r['2']."]~", "", $r['1']);
				} else {
					$pre = $r['1'];
				}
				if(!empty($r['4'])) {
					$post = ".".preg_replace("~[".$r['4']."]~", "", $r['3']);
				} else {
					$post = false;
				}
				return (float)$pre.$post;
			}
			return 0;
		}

		/**
		*	Autoloading for pERP objects, will start in perp_api
		*	and then check perp_* /inc/ *.inc.php
		*/
		public static function perp_autoload($class_name) {
			$prefix = 'class.';
			$suffix = '.inc.php';
			$class_file = $prefix.$class_name.$suffix;

			if(file_exists( $class_file)) {
				require_once($class_file);
				return true;
			}

			$dir = dir(EGW_INCLUDE_ROOT);
			while(($entry = $dir->read()) !== false) {
				if(strpos($entry, 'perp_') === 0) {
					if(is_dir(EGW_INCLUDE_ROOT . '/' . $entry)){
						if(file_exists(EGW_INCLUDE_ROOT . '/'. $entry .'/inc/' . $class_file)) {
							require_once(EGW_INCLUDE_ROOT . '/' . $entry . '/inc/' . $class_file);
							return true;
						}
					}
				}
			}

			// Try to let eGW handle it
			unset($GLOBALS['egw_info']['flags']['autoload']);
			__autoload($class_name);
		}

		/**
		*	Gets an InvoicableItem as referenced by an InvoiceLine.
		*	Uses caching to cut down on objects.
		*	@param data Array must contain invoicable_class, invoicable_id, line_number to identify the item
		*/
		public function get_invoicable_item(array &$data) {
			list($app, $class) = explode('.', $data['invoicable_class']);
			if($class) {
				$data['invoicable_class'] = $class;
			}
			if(!isset($this->invoicable_item_cache[$data['invoicable_class']][$data['invoicable_id']])) {
				$this->perp_autoload($data['invoicable_class']);
				$this->invoicable_item_cache[$data['invoicable_class']][$data['invoicable_id']] =& new $data['invoicable_class']($data['invoicable_id']);
			}
			$object =& $this->invoicable_item_cache[$data['invoicable_class']][$data['invoicable_id']];
			if($object instanceof InvoicableItem && $object instanceof $data['invoicable_class']) {
				return $object;
			} elseif (method_exists($object, 'get_line')) {
				return $object->get_line(isset($data['invoicable_item_line']) ? $data['invoicable_item_line'] : $data['line_number']);
			}
		}

		/**
		*	Gets a ShippableItem as referenced by a ShipmentLine.
		*	Uses caching to cut down on objects.
		*	@param data Array must contain shippable_class, shippable_id, shippable_item_line to identify the item
		*/
		public function get_shippable_item(array &$data) {
			if(!$data['shippable_class']) {
				return null;
			}
			if(!isset($this->shippable_item_cache[$data['shippable_class']][$data['shippable_id']])) {
				$this->perp_autoload($data['shippable_class']);
				$this->shippable_item_cache[$data['shippable_class']][$data['shippable_id']] =& new $data['shippable_class']($data['shippable_id']);
			}
			return $this->shippable_item_cache[$data['shippable_class']][$data['shippable_id']]->get_line($data['shippable_item_line']);
		}

		/**
		*	Reconnect to the database in a way that supports transactions
		*	so we can safely do atomic commits.
		*
		*	@return $GLOBALS['egw']->db, or a clone that supports transactions
		*/
		public function &reconnect_transactional() {
			if($GLOBALS['egw']->db->hasTransactions) {
				return $GLOBALS['egw']->db;
			} else {
				$db = clone($GLOBALS['egw']->db);
				$db->disconnect();
				$db->connect(
					$GLOBALS['egw_info']['server']['db_name'],
					$GLOBALS['egw_info']['server']['db_host'],
					$GLOBALS['egw_info']['server']['db_port'],
					$GLOBALS['egw_info']['server']['db_user'],
					$GLOBALS['egw_info']['server']['db_pass'],
					'mysqlt'
				);
				return $db;
			}
		}

		public static function static_init() {
			// SPL not used by eGW yet
			//spl_autoload_register(array(__CLASS__, 'perp_autoload'));
			$GLOBALS['egw_info']['flags']['autoload'] = array(__CLASS__, 'perp_autoload');
		}
	}

	bo_perp_api::static_init();
?>
