<?php

	/**
	*	Logic layer for Invoices.
	*	Provides common functioon for dealing with Invoices
	*
	*	@ingroup perp_api
	*	@author Nathan Gray
	*/

	require_once('class.so_perp_invoice.inc.php');
	require_once('class.Invoice.inc.php');

	class bo_perp_invoice {


		/**
		*	Link to storage layer object for database access
		*/
		protected $so_perp_invoice;

		/**
		*	Constructor
		*/
		public function __construct() {
			$this->so_perp_invoice = new so_perp_invoice();
		}

		/**
		*	Get a list of invoices
		*	@nextmatch
		*	Query options:
		*	- search:	Our reference number or their reference number
		*	- supplier_id:	Restrict to supplier
		*	- client_id:	Restrict to client
		*	- filter:	Filter by status, set to status
		*/
		public function get_invoice_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$query['order']) {
				$query['order'] = 'invoice_id';
			}
			if($query['filter']) {
				$query['status'] = $query['filter'];
			}

			$rows = $this->so_perp_invoice->get_invoice_list($query);

			foreach($rows as &$row) {
				if($row['due_date'] < time() && !$row['invoice_paid']) {
					$row['overdue'] = 'overdue';
				}
				$readonlys['pay[' . $row['invoice_id'] . ']'] = $row['invoice_paid'] || is_null($row['invoice_transaction_id']);
				$readonlys['post[' . $row['invoice_id'] . ']'] = !is_null($row['invoice_transaction_id']) || 
					!$GLOBALS['egw']->acl->check('invoice', EGW_ACL_CUSTOM_1, $GLOBALS['egw_info']['flags']['currentapp']);
				$readonlys['cancel[' . $row['invoice_id'] . ']'] = (bool)$row['invoice_transaction_id'] ||
					!$GLOBALS['egw']->acl->check('invoice', EGW_ACL_DELETE, $GLOBALS['egw_info']['flags']['currentapp']);
			}
			return $query['count'];
		}

		/**
		*	Get a list of order lines where the quantity ordered does not equal the quantity invoiced.
		*	This is used to add lines from multiple orders (on different shipments) to the same invoice.
		*
		*	@nextmatch
		*	Query options:
		*		supplier_id:	Restrict to supplier
		*		client_id:	Restrict to client
		*/
		public function get_uninvoiced_order_lines($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('order', EGW_ACL_READ, $GLOBALS['egw_info']['flags']['currentapp']);
			if(!$read) {
				$rows = array();
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'perp_order.order_id, perp_order_line.line_no';
			}

			$rows = $this->so_perp_invoice->get_uninvoiced_order_lines($query);
			return $query['count'];
		}
	}
?>
