<?php
	/**
	*	Logic layer for Orders
	*	Provides common functions for dealing with Orders.
	*	
	*	@ingroup perp_api
	*	@author Nathan Gray
	*/

	require_once('class.so_perp_order.inc.php');
	require_once('class.Order.inc.php');

	class bo_perp_order {
		
		/**
		*	Link to storage layer object for database access
		*/
		protected $so_perp_order;

		/**
		*	Constructor
		*/
		public function __construct() {
			$this->so_perp_order = new so_perp_order();
		}


		/**
		*	Get a list of Orders
		*	@nextmatch
		*	Query options:
		*		search:		Our reference number or their reference number
		*		supplier_id:	Restrict to supplier
		*		client_id:	Restrict to client
		*		filter:		Restrict to status
		*/
		public function get_order_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('order', EGW_ACL_READ, $GLOBALS['egw_info']['flags']['currentapp']);
			
			if(!$read) {
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'order_id';
			}
			if($query['filter']) {
				$query['status'] = $query['filter'];
			}
			if(!$query['order_class']) {
				$query['order_class'] = 'Order';
			}

			$list = $this->so_perp_order->get_order_list($query);

			// Permissions

			$user = $GLOBALS['egw_info']['user']['account_id'];

			foreach($list as &$row) {
				$row['formatted_total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $row['order_currency'], $row['total']);

				$rows[] =& $row;
				$order = new $query['order_class']($row['order_id']);
				$complete = $order->is_complete();


				$readonlys['read[' .$row['order_id'].']'] = !$read;
				$readonlys['edit[' .$row['order_id'].']'] = !($order->is_editor($user)) || $complete;
				$readonlys['approve[' . $row['order_id'] . ']'] = !$order->is_admin($user) || $order->get_approval_date() || $order->approval_check() !== true || !$order->get_approval_required();
				$readonlys['deny[' . $row['order_id'] . ']'] = $readonlys['approve[' . $row['order_id'] . ']'];
				$readonlys['ship[' . $row['order_id'] . ']'] = ($order->can_ship() !== true);
				$readonlys['receive[' . $row['order_id'] . ']'] =!$order->can_receive();
				$readonlys['invoice[' . $row['order_id'] . ']'] =!($order->can_invoice() === true);
				$readonlys['cancel[' . $row['order_id'] . ']'] = $order->can_cancel($user) !== true;

				if(!$complete && ($order->requires_approval() && $order->get_approval_date() || !$order->requires_approval())) {
					$ship_warnings = $order->can_ship();
					$readonlys['ship_warning_icon[' . $row['order_id'] . ']'] = !($order->is_editor($user)) || is_bool($ship_warnings);
					if(!is_bool($ship_warnings)) {
						$row['ship_warning'] = $ship_warnings;
					}
				} else {
					$readonlys['ship_warning_icon[' . $row['order_id'] . ']'] = true;
				}
			}
			return $query['count'];
		}

		public function get_status_list() {
			return array(
			);
		}
	}
?>
