<?php
	/**
	*	Logic layer for Shipments
	*	Provides common functions for dealing with Shipments.
	*	
	*	@ingroup perp_api
	*	@author Nathan Gray
	*/

	require_once('class.so_perp_shipment.inc.php');
	require_once('class.Shipment.inc.php');

	class bo_perp_shipment {
		
		/**
		*	Link to storage layer object for database access
		*/
		protected $so_perp_shipment;

		/**
		*	Constructor
		*/
		public function __construct() {
			$this->so_perp_shipment = new so_perp_shipment();
		}


		/**
		*	Get a list of Shipments
		*	@nextmatch
		*	Query options:
		*		search:		Our reference number or their reference number
		*		supplier_id:	Restrict to supplier
		*		client_id:	Restrict to client
		*		filter:		Restrict to status
		*/
		public function get_shipment_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, $GLOBALS['egw_info']['flags']['currentapp']);
			
			if(!$read) {
				$rows = array();
				return 0;
			}

			if(!isset($query['order'])) {
				$query['order'] = 'perp_shipment.shipment_id';
			}
			if(isset($query['filter'])) {
				$query['status'] = $query['filter'];
			}

			$list = $this->so_perp_shipment->get_shipment_list($query);

			// Permissions
			$ship = $GLOBALS['egw']->acl->check('ship', EGW_ACL_EDIT, $GLOBALS['egw_info']['flags']['currentapp']);
			$receive = $GLOBALS['egw']->acl->check('receive', EGW_ACL_EDIT, $GLOBALS['egw_info']['flags']['currentapp']);
			$invoice = $GLOBALS['egw']->acl->check('invoice', EGW_ACL_ADD, $GLOBALS['egw_info']['flags']['currentapp']);

			$user = $GLOBALS['egw_info']['user']['account_id'];

			$rows = array();
			foreach($list as &$row) {
				$shipment = new Shipment($row['shipment_id']);
				$complete = $shipment->is_complete();

				$row['overdue'] = ($row['requested_delivery_date'] < date('U') && !$row['actual_delivery_date'] && !$complete) ? 'overdue' : '';
				$rows[] =& $row;

				$readonlys['read[' .$row['shipment_id'].']'] = !$read;
				$readonlys['edit[' .$row['shipment_id'].']'] = !($shipment->is_editor($user)) || $complete;
				$readonlys['deny[' . $row['shipment_id'] . ']'] = !$shipment->is_admin($user) || ($shipment->get_approval_date());
				$readonlys['approve[' . $row['shipment_id'] . ']'] = !$shipment->is_admin($user) || ($shipment->requires_approval() && $shipment->get_approval_date()) || $shipment->approval_check() !== true;
				$readonlys['ship[' . $row['shipment_id'] . ']'] = $shipment->can_ship() !== true || !$ship;
				$readonlys['receive[' . $row['shipment_id'] . ']'] =!$shipment->can_receive() || !$receive;
				$readonlys['invoice[' . $row['shipment_id'] . ']'] =!$shipment->can_invoice() || !$invoice;
				$readonlys['cancel[' . $row['shipment_id'] . ']'] = ($shipment->can_cancel($user) !== true);

				if(!$complete && $ship && $shipment->get_approval_date() && 
						$readonlys['ship[' . $row['shipment_id'] . ']']) {
					$row['ship_warning'] = $shipment->can_ship();
				} else {
					$readonlys['ship_warning_icon[' . $row['shipment_id'] . ']'] = true;
				}
			}
			return $query['count'];
		}

		/** 
		*	Get a list of OrderLines that have not been fully shipped.
		*
		*	@nextmatch
		*	Query options:
		*		- supplier_id	- Restrict to just one supplier
		*		- client_id	- Restrict to just one client
		*		- order_id	- Restrict to just one order
		*/
		public function get_shippable_items(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, $GLOBALS['egw_info']['flags']['currentapp'])) {
				$rows = array();
				return 0;
			}

			if(!isset($query['order'])) {
				$query['order'] = 'order_id';
			}
			$list = $this->so_perp_shipment->get_shippable_items($query);

			$rows = $list;

			return $query['count'];
		}

		public function get_status_list() {
			return array(
			);
		}
	}
?>
