<?php
	/**
	*       pERP API - Common functions for pERP applications
	*
	*	Storage layer - access to the database only happens here
	*
	*       @author Nathan Gray
	*       @copyright (c) 2006 Nathan Gray - Blue Falls Manufacturing
	*       @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
	*	@ingroup perp_api
	*/

	class so_perp_api {

		/**
		*	Reference to a DB object
		*/
		protected $db;
		
		/**
		*	This is for dealing with user preferences\n
		*	all tables names added here addionality updating the repository
		*	@author Christoph
		*/
		protected $repchange = array('perp_currency' => True);

		/**
		*	Constructor
		*
		*	Creates reference to database object
		*/
		public function so_perp_api() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_api');			
		}

		/**
		*	Change a lookup value
		*	@param	table	string - Table that contains the value
		*	@param	app	string - application that owns the table
		*	@param	key	mixed  - Id of the lookup, or False for new (must be auto-incrementing)
		*	@param	value	mixed  - Value to change it to, or null to remove
		*/
		public function change_lookup($table, $app, $key, $value) {
			if($value != null) {
				$this->db->insert($table, $value, $key, __LINE__, __FILE__, $app);

				// Check for a new ID
				if(is_array($key) && $key_field = key($key)) {
					return $this->db->get_last_insert_id($table, $key_field);
				}
			} elseif($key && $value == null) {
				$this->db->delete($table, $key, __LINE__, __FILE__, $app);
				if($this->repchange[$table]) { $this->change_repository($table, $key); }
			}
		}
		
		/**
		*	... something to do with changing preferences
		*
		*	@author Christoph
		*	
		*	@param table	string - table - ?
		*	@param key	array - Array of preference names
		*/
		public function change_repository($table, $key) {			
			if(is_array($key)) {
				$id = $key[key($key)];
			} else {
				$id = $table.'_'.$key[substr($table,5).'_id'];
			}
			$GLOBALS['egw']->preferences->read_repository();			
			$GLOBALS['egw']->preferences->delete('perp_api',$id, 'user');
			$GLOBALS['egw']->preferences->delete('perp_api',$id, 'forced');
			$GLOBALS['egw']->preferences->delete('perp_api',$id, 'default');
			$GLOBALS['egw']->preferences->save_repository(True);
		}

		/**
		*	Get the units of measure used in pERP
		*
		*	@nextmatch
		*/
		public function get_unit_of_measures(&$query = array()) {
			if($query['search']) {
				$where[] = 'uom_id = \''. addslashes($query['search']) 
					.'\' OR uom_name LIKE \'%' .
					addslashes($query['search']) . "%'";
			}
			if($query['uom_id']) {
				$where['uom_id'] = $query['uom_id'];
			}

			// Get row count
			$this->db->select('perp_unit_of_measure',
				array('count(uom_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			if(!$query['order']) {
				$query['order'] = 'uom_id';
			}

			$this->db->select('perp_unit_of_measure', 
				array('uom_id', 'uom_name'), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 'perp_api',
				$query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get the shipping companies used in pERP
		*
		*	@nextmatch
		*/
		public function get_shippers(&$query = array()) {
			if($query['search']) {
				$where[] = 'shipper_id = \''. addslashes($query['search']) 
					.'\' OR shipper_name LIKE \'%' .
					addslashes($query['search']) . "%'";
			}
			if($query['shipper_id']) {
				$where['shipper_id'] = $query['shipper_id'];
			}
			if(!$query['order']) {
				$query['order'] = 'shipper_name';
			}

			// Get row count
			$this->db->select('perp_shipper',
				array('count(shipper_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_shipper', 
				array(
					'shipper_id', 
					'shipper_name', 
					'account_number'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], False,
				$query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
		
		/**
		*	Get the currencies used in pERP
		*
		*	@nextmatch
		*/
		public function get_currencies(&$query = array()) {
			if($query['currency_id']) {
				$where['currency_id'] = $query['currency_id'];
			}
			if($query['search']) {
				$where[] = '(currency_id = \'' . $query['search'] . '\' OR 
					currency_name LIKE \'%' . $query['search'] . '%\')';
			}
			if(!$query['order']) {
				$query['order'] = 'currency_order';
			}

			// Get row count
			$this->db->select('perp_currency',
				array('count(currency_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_currency', 
				array(
					'currency_id', 
					'currency_name', 
					'country', 
					'hundreds_name', 
					'exchange_rate', 
					'currency_order',
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				False, $query['num_rows']
			);

			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
		
		/**
		*	Get the payment terms used in pERP
		*
		*	@nextmatch
		*/
		public function get_payment_terms(&$query = array()) {
			$where = array();
			if($query['search']) {
				$where[] = '(payment_term_id = \'' . addslashes($query['search']) . '\' OR 
					payment_term_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}

			if($query['terms']) {
				$where['payment_term_id'] = $query['terms'];
			}

			if(!$query['order']) {
				$query['order'] = 'payment_term_order';
			}

			// Get row count
			$this->db->select('perp_payment_term',
				array('count(payment_term_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_payment_term', 
				array(
					'payment_term_id', 
					'payment_term_name', 
					'due_days', 
					'due_date_type',
					'discount_days',
					'discount_date_type',
					'discount_percent',
					'payment_term_order'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get the payment methods used in pERP
		*
		*	@nextmatch
		*/
		public function get_payment_methods(&$query = array()) {
			if($query['search']) {
				$where[] = '(payment_method_id = \'' . addslashes($query['search']) . '\' OR 
					payment_method_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}
			if($query['payment_method_id']) {
				$where['payment_method_id'] = $query['payment_method_id'];
			}
			if(!$query['order']) {
				$query['order'] = 'payment_method_id';
			}

			// Get row count
			$this->db->select('perp_payment_method',
				array('count(payment_method_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_payment_method', 
				array(
					'payment_method_id', 
					'payment_method_name'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get the tax locations used in pERP
		*
		*	@nextmatch
		*/
		public function get_tax_locations(&$query = array()) {
			if($query['search']) {
				$where[] = '(tax_location_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}
			if(!$query['order']) {
				$query['order'] = 'tax_location_name';
			}

			// Get row count
			$this->db->select('perp_tax_location',
				array('count(tax_location_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_tax_location', 
				array(
					'tax_location_id', 
					'tax_location_name'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				$query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
	
		/**
		*	Get the tax categories used in pERP
		*
		*	@nextmatch
		*/
		public function get_tax_categories(&$query = array()) {
			if($query['search']) {
				$where[] = '(tax_category_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}
			if(!$query['order']) {
				$query['order'] = 'tax_category_name';
			}

			// Get row count
			$this->db->select('perp_tax_category',
				array('count(tax_category_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_tax_category', 
				array(
					'tax_category_id', 
					'tax_category_name'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
	
		/**
		*	Get the tax authorities used in pERP
		*
		*	@nextmatch
		*/
		public function get_tax_authorities(&$query = array()) {
			if($query['search']) {
				$where[] = '(tax_authority_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}
			if($query['tax_authority_id']) {
				$where['tax_authority_id'] = $query['tax_authority_id'];
			}
			if(!$query['order']) {
				$query['order'] = 'tax_authority_name';
			}

			// Get row count
			$this->db->select('perp_tax_authority',
				array('count(tax_authority_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_tax_authority', 
				array(
					'tax_authority_id', 
					'tax_authority_name', 
					'registration_number',
					'tax_authority_in_account', 
					'tax_authority_out_account'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
	
		/**
		*	Get the tax groups used in pERP
		*
		*	@nextmatch
		*/
		public function get_tax_groups(&$query = array()) {
			if($query['search']) {
				$where[] = '(tax_group_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}
			if($query['tax_group_id']) {
				$where['tax_group_id'] = $query['tax_group_id'];
			}
			if(!$query['order']) {
				$query['order'] = 'tax_group_name';
			}

			// Get row count
			$this->db->select('perp_tax_group',
				array('count(tax_group_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_tax_group', 
				array(
					'tax_group_id', 
					'tax_group_name'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
	
		/**
		*	Get the tax authority groupings used in pERP
		*
		*	@nextmatch
		*/
		public function get_tax_group_taxes(&$query = array()) {
			if($query['search']) {
				$where[] = '(tax_group_name LIKE \'%' . addslashes($query['search']) . '%\')';
			}
			if($query['tax_group_id']) {
				$where['tax_group_id'] = $query['tax_group_id'];
			}

			// Get row count
			$this->db->select('perp_tax_group_tax',
				array('count(tax_group_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}
			$this->db->select('perp_tax_group_tax', 
				array(
					'tax_group_id', 
					'tax_authority_id', 
					'calc_order', 
					'tax_on_tax'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY tax_group_id', False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get the tax rates used in pERP
		*
		*	@nextmatch
		*/
		public function get_tax_auth_rates(&$query = array()) {
			if($query['search']) {
				$where[] = '(tax_authority = ' . addslashes($query['search']) . ' OR ' .
					'tax_source_location = ' . addslashes($query['search']) .' OR '.
					'tax_category = ' . addslashes($query['search']) . ')';
			}
			if($query['tax_authority']) {
				$where['tax_authority'] = $query['tax_authority'];
			}

			// Get row count
			$this->db->select('perp_tax_auth_rate',
				array('count(tax_authority) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_tax_auth_rate', 
				array(
					'tax_authority', 
					'tax_source_location', 
					'tax_category', 
					'tax_rate'), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY tax_authority, tax_source_location',
				False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get the pERP style Access Control Levels, formatted for use in ui_perp_api::acl()
		*	Locations tend to be things like 'supplier' and 'customer'.  Access to a location
		*	allows access to all records in that location.
		*	Returned array is in the form:
		*	@code
		*	Array( {
		*		<location>	=>	array(
		*			location	=>	<location>,
		*			read		=>	array(<accounts allowed to read the location>),
		*			add		=>	array(<accounts allowed to add to the location>),
		*			edit		=>	array(<accounts allowed to edit the location>),
		*			delete		=>	array(<accounts allowed to delete the location>),
		*			private		=>	array(<accounts allowed private in the location>),
		*			admin		=>	array(<accounts allowed to admin the location>),
		*		), 
		*		}+
		*	)
		*	@endcode
		*	@param	app	string	-	Application to get the ACLs for
		*	@return	array	of the form:\n
		*		
		*/
		public function get_acls($app) {
			$this->db->select('egw_acl', 
				array('acl_location', 'acl_account', 'acl_rights'),
				array('acl_appname' => $app),
				__LINE__, __FILE__, false,
				'ORDER BY acl_appname, acl_location');
			
			$rows = array();
			$locs = array();
			while($row = $this->db->row(True)) {
				$location = $row['acl_location'];
				$account = $row['acl_account'];
				$permission = $row['acl_rights'];
				if(is_numeric($location)) {
					continue;
				}
				$location = $location == 'run' ? lang($app) : $location;
				$locs[$location]['location'] = $location;
				if($permission & EGW_ACL_READ) {
					if($account) {
						$locs[$location]['read'][] = $account;
					} elseif(!is_array($locs[$location]['read'])) {
						$locs[$location]['read'] = array();
					}
				}
				if($permission & EGW_ACL_ADD) {
					if($account) {
						$locs[$location]['add'][] = $account;
					} elseif(!is_array($locs[$location]['add'])) {
						$locs[$location]['add'] = array();
					}
				}
				if($permission & EGW_ACL_EDIT) {
					if($account) {
						$locs[$location]['edit'][] = $account;
					} elseif(!is_array($locs[$location]['edit'])) {
						$locs[$location]['edit'] = array();
					}
				}
				if($permission & EGW_ACL_DELETE) {
					if($account) {
						$locs[$location]['delete'][] = $account;
					} elseif(!is_array($locs[$location]['delete'])) {
						$locs[$location]['delete'] = array();
					}
				}
				if($permission & EGW_ACL_PRIVATE) {
					if($account) {
						$locs[$location]['private'][] = $account;
					} elseif(!is_array($locs[$location]['private'])) {
						$locs[$location]['private'] = array();
					}
				}
				if($permission & EGW_ACL_CUSTOM_1) {
					if($account) {
						$locs[$location]['admin'][] = $account;
					} elseif(!is_array($locs[$location]['admin'])) {
						$locs[$location]['admin'] = array();
					}
				}
			}
			foreach($locs as $key => $row) {
				$row['location'] = $key;
				$rows[] = $row;
			}
			return $rows;
		}

		/**
		*	Formats a currency amount according to user preferences
		*	@note This function returns HTML, which must be displayed as such
		*	
		*	@author Christoph
		*	@param id string - 3 letter currency ID
		*	@param price 	double	- amount to format
		*	@return string HTML to display the formatted price
		*/
		public function get_formatted_price($id, $price) {
			$where['currency_id'] = $id;
			$this->db->select('perp_currency', array('currency_name'), $where, __LINE__, __FILE__, False);

			while($row = $this->db->row(True)) {
				$cform = $row['currency_format'];
				$cname = $row['currency_name'];
			}

			if($GLOBALS['egw_info']['user']['preferences']['perp_api']['perp_currency_'.$id]) {
				$cform = $GLOBALS['egw_info']['user']['preferences']['perp_api']['perp_currency_'.$id];
			}
			
			$cformparts = array_flip(explode(',',$cform)); 
						
			//Default
			$dec_point = ',';
			$thousands_sep = '';
			$rocpos = 2;
						
			if(isset($cformparts['3P'])) { 
				$rocpos = 3; 
			}
			if(isset($cformparts['TP'])) { 
				$thousands_sep = '.'; 
			} elseif(isset($cformparts['TC'])) {
				$thousands_sep = ',';
			}
			if(isset($cformparts['CP'])) {
				$dec_point = '.';
			}
			
			$fprice = number_format($price, $rocpos, $dec_point, $thousands_sep);

			if(isset($cformparts['CBEF'])) { $fprice = $cname.$fprice; }
			if(isset($cformparts['CAFT'])) { $fprice .= ' '.$cname; }
			if(isset($cformparts['R']) && floatval($price) < 0) { $fprice = '<span style="color: red;">'.$fprice.'<span>'; }
			
			return $fprice;
		}

		public static function change_table_engine(&$db, $appname, Array $tables = array(), $engine = 'InnoDB') {
			if($db->Type != 'mysql' && $db->m_odb->Type != 'mysql') {
				return false;
			}
			if(!method_exists($db, 'get_table_definitions') && property_exists($db, 'm_odb')) {
				$use_db = $db->m_odb;
			} else {
				$use_db =& $db;
			}

			if(count($tables) == 0) {
				// Get all tables for that app
				$definitions = $use_db->get_table_definitions($appname);
				$tables = array_keys($definitions);
			} elseif (count($tables) > 0) {
				$use_db = $db;
			} else {
				throw new egw_exception_wrong_parameter('No tables provided, and unable to determine tables from ' . get_class($db));
			}

			if(!$appname) {
				if(strpos($GLOBALS['egw_info']['flags']['currentapp'], 'perp_') === 0) {
					$appname = $GLOBALS['egw_info']['flags']['currentapp'];
				} else {
					throw new egw_exception_wrong_parameter('Invalid appname, change_table_engine only works with pERP applications');
				}
			}


			$alter = 'ALTER TABLE %s ENGINE = InnoDB';

			// If using MySQL, need to specify InnoDB tables
			foreach($tables as $table) {
				$alter_query = sprintf($alter, $table);
echo $alter_query . '<br />';
				$db->query($alter_query);
			}
		}
	}
?>
