<?php

	/**
	*	Data Access object for working with Invoices.
	*	Note that Invoice is a nice Object Oriented class that handles
	*	most things on its own, including reading and writing itself.
	*/
	class so_perp_invoice {
		public function __construct() {
		}

		public function get_invoice_list(&$query) {
			$where = array();
			if(isset($query['supplier_id']) && is_numeric($query['supplier_id'])) {
				$where[] = Invoice::invoice_table . '.supplier_id = ' . $query['supplier_id'];
			} elseif (isset($query['supplier_id']) && is_array($query['supplier_id'])) {
				$where[] = Invoice::invoice_table . '.supplier_id IN (' . implode(',', $query['supplier_id']) . ')';
			}
			if(isset($query['client_id']) && is_numeric($query['client_id'])) {
				$where[] = Invoice::invoice_table . '.client_id = ' . $query['client_id'];
			} elseif (isset($query['client_id']) && is_array($query['client_id'])) {
				$where[] = Invoice::invoice_table . '.client_id IN (' . implode(',', $query['client_id']) . ')';
			}

			if(isset($query['class'])) {
				$where[] = Invoice::invoice_table . '.class = ' . $GLOBALS['egw']->db->quote($query['class']);
			}

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = '(' . Invoice::invoice_table . ".our_reference_number LIKE $search OR " .
					Invoice::invoice_table . ".their_reference_number LIKE $search OR " . 
					" supplier_num LIKE $search OR client_num LIKE $search OR
					comments LIKE $search or description LIKE $search)";
			}
					
			// Filter on dates
			if(isset($query['issue_date'])) {
				if(is_array($query['issue_date'])) {
					if($query['issue_date']['start']) {
						$where[] = 'perp_invoice.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['start']));
					}
					if($query['issue_date']['end']) {
						$where[] = 'perp_invoice.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['end']));
					}
				} else {
					$where[] = 'perp_invoice.issue_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']));
				}
			}
			if(isset($query['originating_date'])) {
				if(is_array($query['originating_date'])) {
					if($query['originating_date']['start']) {
						$where[] = 'perp_invoice.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['start']));
					}
					if($query['originating_date']['end']) {
						$where[] = 'perp_invoice.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['end']));
					}
				} else {
					$where[] = 'perp_invoice.originating_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']));
				}
			}
			if(isset($query['due_date'])) {
				if(is_array($query['due_date'])) {
					if($query['due_date']['start']) {
						$where[] = 'perp_invoice.due_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['due_date']['start']));
					}
					if($query['due_date']['end']) {
						$where[] = 'perp_invoice.due_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['due_date']['end']));
					}
				} else {
					$where[] = 'perp_invoice.due_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['due_date']));
				}
			}

			if(isset($query['status'])) {
				switch($query['status']) {
					case 'unposted':
						$where[] = 'invoice_transaction_id IS NULL';
						break;
					case 'open':
						$where[] = '!invoice_paid';
						break;
					case 'closed':
						$where[] = 'invoice_paid';
						break;
					case 'due':
						$where[] = '(!invoice_paid AND 
							due_date <= \'' . $GLOBALS['egw']->db->to_timestamp(mktime(0,0,0,date('m'), date('d') + 7, date('Y'))) . '\')';
						break;
					case 'overdue':
						$where[] = '(!invoice_paid AND
							due_date < NOW())';
						break;
				}
			}

			$join = 'LEFT JOIN perp_client ON
					perp_client.client_id = ' . Invoice::invoice_table . '.client_id 
				LEFT JOIN perp_supplier ON
					perp_supplier.supplier_id = ' . Invoice::invoice_table . '.supplier_id 
				JOIN perp_invoice_line ON
					perp_invoice_line.invoice_id = perp_invoice.invoice_id 
				JOIN (
					SELECT 
						invoice_id,
						sum(unit_price * quantity) AS total
					FROM perp_invoice_line
					WHERE line_type != ' . InvoiceLine::TAX . '
					GROUP BY invoice_id
				) AS pretax ON
					perp_invoice.invoice_id = pretax.invoice_id 
				JOIN perp_payment_term ON
					perp_invoice.payment_terms = perp_payment_term.payment_term_id';

			// Get number of rows
			$query['count'] = $GLOBALS['egw']->db->query(
				'SELECT count(*) AS count
				FROM ' . Invoice::invoice_table . '
				' . $join . '
				WHERE ' . $GLOBALS['egw']->db->column_data_implode(' AND ', $where, true, false) .
				' GROUP BY perp_invoice.invoice_id'
			)->NumRows();
			$group_columns = array(
				Invoice::invoice_table . '.invoice_id',
				Invoice::invoice_table . '.our_reference_number',
				Invoice::invoice_table . '.their_reference_number',
				Invoice::invoice_table . '.supplier_id',
				'supplier_num',
				Invoice::invoice_table . '.client_id',
				'client_num',
				Invoice::invoice_table . '.originating_user',
				Invoice::invoice_table . '.our_contact_person',
				Invoice::invoice_table . '.their_contact_person',
				Invoice::invoice_table . '.issue_date',
				Invoice::invoice_table . '.due_date',
				Invoice::invoice_table . '.discount_date',
				Invoice::invoice_table . '.discount_taken',
				Invoice::invoice_table . '.invoice_currency',
				Invoice::invoice_table . '.invoice_paid',
				'pretax.total',
				'perp_payment_term.discount_percent'
			);
			$sql_group_by = implode(',', $group_columns);

			$GLOBALS['egw']->db->select(Invoice::invoice_table, array(
					Invoice::invoice_table . '.invoice_id',
					Invoice::invoice_table . '.our_reference_number',
					Invoice::invoice_table . '.their_reference_number',
					Invoice::invoice_table . '.supplier_id',
					'supplier_num AS supplier_number',
					'supplier_name',
					Invoice::invoice_table . '.client_id',
					'client_num AS client_number',
					'client_name',
					Invoice::invoice_table . '.originating_user',
					Invoice::invoice_table . '.our_contact_person',
					Invoice::invoice_table . '.their_contact_person',
					Invoice::invoice_table . '.issue_date',
					Invoice::invoice_table . '.due_date',
					Invoice::invoice_table . '.discount_date',
					Invoice::invoice_table . '.discount_taken',
					Invoice::invoice_table . '.invoice_currency',
					Invoice::invoice_table . '.invoice_paid',
					Invoice::invoice_table . '.invoice_transaction_id',
					'SUM(perp_invoice_line.unit_price * perp_invoice_line.quantity) AS invoice_total',
					'pretax.total AS pretax_subtotal',
					'perp_payment_term.discount_percent'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'GROUP BY ' . $sql_group_by . '
				ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_api', $query['num_rows'],
				$join
			);
		
			$list = array();
			if(!is_object($GLOBALS['perp_api']['bo'])) {
				$GLOBALS['perp_api']['bo'] =& CreateObject('perp_api.bo_perp_api');
			}

			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['issue_date'] = $GLOBALS['egw']->db->from_timestamp($row['issue_date']);
				$row['due_date'] = $GLOBALS['egw']->db->from_timestamp($row['due_date']);
				$row['discount_date'] = $GLOBALS['egw']->db->from_timestamp($row['discount_date']);
				if($row['discount_date'] >= date('U')) {
					$discount = round(($row['discount_percent'] / 100) * $row['pretax_subtotal'] - $row['discount_taken'], 2);
					if($discount != 0) {
						$row['discount_available'] = $GLOBALS['perp_api']['bo']->get_formatted_price($row['invoice_currency'], $discount);
					}
				}
				$row['formatted_pretax_subtotal'] = $GLOBALS['perp_api']['bo']->get_formatted_price($row['invoice_currency'], $row['pretax_subtotal']);
				$row['formatted_total'] = $GLOBALS['perp_api']['bo']->get_formatted_price($row['invoice_currency'], $row['invoice_total']);
				$list[] = $row;
			}
			return $list;
		}

		public function get_uninvoiced_order_lines(&$query) {
			$where = array('(quantity_invoiced != quantity_ordered OR
				quantity_invoiced IS NULL)');

			if(isset($query['client_id'])) {
				$where[] = 'perp_order.client_id = ' . $GLOBALS['egw']->db->quote($query['client_id']);
			}
			if(isset($query['supplier_id'])) {
				$where[] = 'perp_order.supplier_id = ' . $GLOBALS['egw']->db->quote($query['supplier_id']);
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = '(perp_order.our_reference_number LIKE ' . $search . ' OR 
					perp_order_line.supplier_stock_code LIKE ' . $search . ')';
			}
			
			$join = 'JOIN perp_order ON
					perp_order.order_id = perp_order_line.order_id
				LEFT JOIN (
					SELECT 
						invoicable_id,
						invoicable_item_line,
						SUM(quantity) AS quantity_invoiced
					FROM perp_invoice_line 
					GROUP BY invoicable_id, invoicable_item_line
				) AS perp_invoice_line ON
					perp_invoice_line.invoicable_id = perp_order_line.order_id AND
					perp_invoice_line.invoicable_item_line = perp_order_line.line_no';

			// Get number of rows
			$GLOBALS['egw']->db->select(
				'perp_order_line',
				array(
					'count(perp_order.order_id) AS count'
				),
				$where,
				__LINE__, __FILE__, 0,'',
				'perp_api', -1,
				$join
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$GLOBALS['egw']->db->select(
				'perp_order_line',
				array(
					'perp_order.order_id',
					'perp_order.order_id AS invoicable_id',
					'perp_order.our_reference_number',
					'perp_order_line.line_no',
					'perp_order_line.line_no AS invoicable_item_line',
					'perp_order_line.line_no AS line_number',
					'quantity_ordered - IF(
						quantity_invoiced IS NULL,
						0,
						quantity_invoiced
					) AS invoice_quantity',
					'stock_id',
					'supplier_stock_code',
					'our_stock_code',
					'description',
					'unit_price',
					'quantity_ordered',
					'quantity_invoiced'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_api', $query['num_rows'],
				$join
			);

			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}

			return $list;
		}
	}
?>
