<?php

	/**
	*	Data Access object for working with Orders.
	*	Note that Order is a nice Object Oriented class that handles
	*	most things on its own, including reading and writing itself.
	*/
	class so_perp_order {
		
		public function __construct() {
		}

		/**
		*	@todo	improve the efficiency of getting the number of rows
		*/
		public function get_order_list(&$query) {
			$where = array();
			$having = array();

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(our_reference_number LIKE $search OR 
					their_reference_number LIKE $search OR
					perp_supplier.supplier_num LIKE $search OR 
					perp_client.client_num LIKE $search)";
			}
			if(isset($query['supplier_id']) && is_numeric($query['supplier_id'])) {
				$where[] = Order::order_table. '.supplier_id = ' . $query['supplier_id'];
			} elseif (isset($query['supplier_id']) && is_array($query['supplier_id'])) {
				$where[] = Order::order_table . '.supplier_id IN (' . implode(',',$query['supplier_id']) . ')';
			}
			if(isset($query['client_id']) && is_numeric($query['client_id'])) {
				$where[] = Order::order_table. '.client_id = ' . (int)$query['client_id'];
			} elseif (isset($query['client_id']) && is_array($query['client_id'])) {
				$where[] = Order::order_table . '.client_id IN (' . implode(',',$query['client_id']) . ')';
			}
			if(isset($query['status'])) {
				switch($query['status']) {
					case 'request':
						$where[] = Order::order_table.'.approval_date IS NULL AND requires_approval';
						break;
					case 'approved':
						$where[] = Order::order_table. '.approval_date IS NOT NULL';
						$having[] = '((total_shipped = 0 OR total_shipped IS NULL) AND (total_invoiced = 0 OR total_invoiced IS NULL))';
						break;
					case 'ordered':
						$where[] = Order::order_table. '.issue_date <= NOW()';
						break;
					case 'shipped':
						$where[] = Order::order_table . '.order_id IN (
							SELECT shippable_id
							FROM perp_shipment_line
							WHERE shippable_id IS NOT NULL
						)';
						break;
					case 'not_shipped':
						$where[] = '(' . Order::order_table. '.approval_date IS NOT NULL OR !requires_approval OR requires_approval IS NULL)';
						$having[] = '(total_ordered != total_shipped OR total_shipped IS NULL)';
						break;
					case 'received':
						$where[] = Order::order_table . '.order_id IN (
							SELECT order_id
							FROM perp_order_line
							JOIN (
								perp_shipment_line JOIN
								perp_goods_received ON
								perp_goods_received.shipment_id = perp_shipment_line.shipment_id AND
								perp_goods_received.line_no = perp_shipment_line.line_no
							) ON
								perp_shipment_line.shippable_id = perp_order_line.order_id AND
								perp_shipment_line.shippable_item_line = perp_order_line.line_no
						)';
						break;
					case 'not_received':
						$where[] = '(' . Order::order_table. '.approval_date IS NOT NULL OR !requires_approval OR requires_approval IS NULL OR ( '.
							Order::order_table . '.order_id NOT IN (
								SELECT order_id
								FROM perp_order_line
								JOIN (
									perp_shipment_line JOIN
									perp_goods_received ON
									perp_goods_received.shipment_id = perp_shipment_line.shipment_id AND
									perp_goods_received.line_no = perp_shipment_line.line_no
								) ON
									perp_shipment_line.shippable_id = perp_order_line.order_id AND
									perp_shipment_line.shippable_item_line = perp_order_line.line_no
							)
						))';
						$having[] = '(total_ordered != total_shipped OR total_shipped IS NULL)';
						break;
						
					case 'invoiced':
						$where[] = Order::order_table . '.order_id IN (
							SELECT invoicable_id
							FROM perp_invoice_line
							WHERE invoicable_id IS NOT NULL
						)';
						break;
					case 'not_invoiced':
						$where[] = '(' . Order::order_table. '.approval_date IS NOT NULL OR !' . Order::order_table . '.requires_approval OR requires_approval IS NULL)';
						$where[] = Order::order_table . '.order_id NOT IN (
							SELECT invoicable_id
							FROM perp_invoice_line
							WHERE invoicable_id IS NOT NULL
						)';
						break;
					case 'closed':
						$having[] = '(total_shipped = total_ordered AND total_invoiced = total_ordered)';
						break;
				}
			}

			// Filter on dates
			if(isset($query['issue_date'])) {
				if(is_array($query['issue_date'])) {
					if($query['issue_date']['start']) {
						$where[] = 'perp_order.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['start']));
					}
					if($query['issue_date']['end']) {
						$where[] = 'perp_order.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['end']));
					}
				} else {
					$where[] = 'perp_order.issue_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']));
				}
			}
			if(isset($query['originating_date'])) {
				if(is_array($query['originating_date'])) {
					if($query['originating_date']['start']) {
						$where[] = 'perp_order.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['start']));
					}
					if($query['originating_date']['end']) {
						$where[] = 'perp_order.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['end']));
					}
				} else {
					$where[] = 'perp_order.originating_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']));
				}
			}

			// Query on order contents
			if(isset($query['stock_id'])) {
				$where[] = 'perp_order_line.stock_id = ' . (int)$query['stock_id'];
			}
			if(isset($query['supplier_stock_code'])) {
				$where[] = 'perp_order_line.supplier_stock_code = ' . $GLOBALS['egw']->db->quote($query['supplier_stock_code']);
			}
			if(isset($query['our_stock_code'])) {
				$where[] = 'perp_order_line.our_stock_code = ' . $GLOBALS['egw']->db->quote($query['our_stock_code']);
			}

			if(count($having) > 0) {
				$having = ' HAVING ' . implode(' AND ', $having);
			} else {
				$having = '';
			}

			$join = 'JOIN perp_order_line ON
					' . Order::order_table .'.order_id = perp_order_line.order_id '.
				'LEFT JOIN perp_client ON
					perp_client.client_id = ' . Order::order_table . '.client_id '.
				'LEFT JOIN perp_supplier ON
					perp_supplier.supplier_id = ' . Order::order_table . '.supplier_id ' .
				'LEFT JOIN (
					SELECT 
						sum(quantity_shipped) AS quantity_shipped,
						perp_shipment_line.shippable_id,
						perp_shipment_line.shippable_item_line
					FROM perp_shipment_line 
					GROUP by shippable_id, shippable_item_line
				) AS perp_shipment_line ON
					perp_shipment_line.shippable_id = ' . Order::order_table . '.order_id AND
					perp_shipment_line.shippable_item_line = ' . OrderLine::order_line_table . '.line_no
				LEFT JOIN (
					SELECT
						sum(quantity) AS quantity_invoiced,
						perp_invoice_line.invoicable_id,
						perp_invoice_line.invoicable_item_line
					FROM perp_invoice_line
					GROUP BY invoicable_id, invoicable_item_line
				) AS perp_invoice_line ON
					perp_invoice_line.invoicable_id = ' . Order::order_table . '.order_id AND
					perp_invoice_line.invoicable_item_line = ' . OrderLine::order_line_table . '.line_no';

			// Get number of rows
			$result =& $GLOBALS['egw']->db->select(Order::order_table,
				//array('count(' . Order::order_table . '.order_id) AS count'),
				array(
					'count(' . Order::order_table . '.order_id) AS count',

					// for checking for done
					'SUM(quantity_ordered) AS total_ordered',
					'SUM(quantity_shipped) AS total_shipped',
					'SUM(quantity_invoiced) AS total_invoiced',
				),
				$where,
				__LINE__, __FILE__, false, 'GROUP BY ' . Order::order_table . '.order_id ' . $having, 'perp_api', -1,
				$join
			);
			$query['count'] = $result->NumRows();

			$GLOBALS['egw']->db->select(Order::order_table, 
				array(
					Order::order_table.'.order_id',
					Order::order_table.'.our_reference_number',
					Order::order_table.'.their_reference_number',
					Order::order_table.'.supplier_id',
					'supplier_num AS supplier_number',
					Order::order_table.'.client_id',
					'client_num AS client_number',
					Order::order_table.'.originating_user',
					Order::order_table.'.our_contact_person',
					Order::order_table.'.their_contact_person',
					Order::order_table.'.issue_date',
					Order::order_table.'.order_currency',
					'sum(' . OrderLine::order_line_table . '.quantity_ordered * ' . OrderLine::order_line_table. '.unit_price) AS total',

					// for checking for done
					'SUM(quantity_ordered) AS total_ordered',
					'SUM(quantity_shipped) AS total_shipped',
					'SUM(quantity_invoiced) AS total_invoiced',
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'GROUP BY order_id ' . $having . 
				' ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_api', $query['num_rows'],
				$join
			);

			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['issue_date'] = $GLOBALS['egw']->db->from_timestamp($row['issue_date']);
				$list[] = $row;
			}
			return $list;
		}
	}
?>
