<?php
	/**
	*	pERP API - Common functions for pERP applications
	*
	*	@see http://projecterp.org
	*	@author Nathan Gray
	*	@copyright (c) 2006 Nathan Gray - Blue Falls Manufacturing
	*	@license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
	*	@ingroup perp_api
	*/

	class ui_perp_api {
		
		/**
		*	Reference to bo_perp_api business object
		*	@see perp_api::bo_perp_api
		*/
		protected $bo;

		/**
		*	Reference to eTemplate for display
		*/
		protected $template;

		/**
		*	List of public functions that phpgwapi allows to be 
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'acl'		=>	True,
			'change_lookup'	=>	True,
		);

		/**
		*	Messages to display to the user
		*	@see make_message()
		*/
		public $messages = array();

		/**
		*	boolean output extra debug information
		*/
		public $debug = false;

		/**
		*	Contstructor
		*	Creates the business object and template object
		*/
		function __construct() {
			$this->template = CreateObject('etemplate.etemplate');

			if(!$GLOBALS['perp_api']) {
				$GLOBALS['perp_api'] = array();
			}
			$GLOBALS['perp_api']['ui'] =& $this;
			if(!$GLOBALS['perp_api']['bo']) {
				$GLOBALS['perp_api']['bo'] =& CreateObject('perp_api.bo_perp_api');
			}
			$this->bo =& $GLOBALS['perp_api']['bo'];
		}

		/**
		*	Set configuration options
		*
		*	Reused for every pERP applications's Admin section, this function is used to set 
		*	configuration variables.  All the application has to do is create an appropriately
		*	named template (<appname>.admin.config) and call this function.
		*
		*	@param content	array Data returned from callback function when the user submits the form.
		*	@param app	string Name of the calling application.
		*	@param callback	string Function to call when the user submits the form.  This is usually the calling function,
		*			and you can use it to do special pre-processing on the data, if needed.
		*	@param select_box_options	array	Array of the form array( {field_name => array({key => value,}+ ) }+ ) for select box options
		*	@param template string If your template is for some reason not in the form <appname>.admin.config, the name of the template to use.
		*/
		public function config($content = array(), $app, $callback, $select_box_options = array(), $template = '', $data = array()) {

			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			}

			// Match up keys with rows
			if(!is_array($content['nm']['rows'])) {
				$content['nm']['rows'] = array();
			}

			// Save changes
			if($content['save']) {
				$config = new config($app);
				foreach($content['settings'] as $key => $value) {
					$config->save_value($key, $value, $app);
				}
				$this->messages[] = array(lang('Settings saved'), 'success');
			}
			if(!is_array($data['settings'])) {
				$data['settings'] = array();
			}
			// Get config
			$data['settings'] += config::read($app);
			$data['template'] = $template ? $template : $app . '.admin.config';

			$GLOBALS['egw_info']['flags']['app_header'] = lang($app) . ' - ' .lang('site configuration');

			if(count($this->messages) > 0) {
				$data['message'] = $this->make_message($this->messages);
			}
			$this->template->read('perp_api.config');
			$this->template->exec($callback, $data, $select_box_options, $read_only, array($app));
		}

		/**
		*	Set Access Control Levels for pERP applications
		*	
		*	pERP does not make a lot of use of record-level ACLs.  Instead, pERP uses location based
		*	controls.  This allows some people to see invoices, some to change stock, et cetera, based
		*	on the needs of your organization.  This fits well into eGroupWare's ACL system, but unfortunately
		*	in order to use eGroupWare's ACL user interface, you have to add your application into an array in 
		*	eGroupWare's code.  This is an alternative.
		*
		*	acl() uses change_lookup() to actually perform the changes.
		*
		*	@param	content	array Data returned from callback function when the user submits the form.
		*	@param	app	string Name of the calling application.
		*	@param	lookup	The table name and the function (called from hook)
		*	@param	prefix	The prefix for the fields (same for every field)
		*	@param	select_box_options	array	Array of the form array( {field_name => array({key => value,}+ ) }+ ) for select box options
		*	@param	filters Filters for the header, if desired
		*/

		public function acl($content = array(), $app, $lookup, $prefix, $select_box_options = array(), $filters = array()) {
			if($this->debug) {
				echo 'acl incoming data:';
				_debug_array($content);
				printf("App: %s Lookup: %s Prefix: %s<br />Select_box_options:", $app, $lookup, $prefix);
				_debug_array($select_box_options);
			}
			$edit = $GLOBALS['egw']->acl->check('acl', EGW_ACL_EDIT);
	
			// This needs special handling because it's not a regular lookup
			if((isset($content['save']) || isset($content['add'])) && $edit) {
				// Match up keys with rows
				$rows = array();
				$functionname = 'get_acls';
				$this->bo->$functionname($content['nm'], $rows);

				$count = 1;
				foreach($rows as $row) {
					$content['nm']['rows'][$count]['location'] = ($row['location'] == lang($app) ? 'run' : $row['location']);
					foreach($row as $perm => $accounts) {
						if($perm != 'location' && $row['location'] != lang($app)) {
							if(is_array($content['nm']['rows'][$count][$perm])) {
								$content['nm']['rows'][$count][$perm][] = 0;
							} else {
								$content['nm']['rows'][$count][$perm] = array(0);
							}
						}
					}
					$count++;
				}

				$rights = array();
				foreach($content['nm']['rows'] as $location) {
					$GLOBALS['egw']->acl->delete_repository($app, $location['location'], False);
					if($location['location'] == '') {
						// Skip blanks
						continue;
					}
					if($location['read']) {
						foreach($location['read'] as $account) {
							$rights[$location['location']][$account] += EGW_ACL_READ;
						}
					}
					if($location['add']) {
						foreach($location['add'] as $account) {
							$rights[$location['location']][$account] += EGW_ACL_ADD;
						}
					}
					if($location['edit']) {
						foreach($location['edit'] as $account) {
							$rights[$location['location']][$account] += EGW_ACL_EDIT;
						}
					}
					if($location['delete']) {
						foreach($location['delete'] as $account) {
							$rights[$location['location']][$account] += EGW_ACL_DELETE;
						}
					}
					if($location['private']) {
						foreach($location['private'] as $account) {
							$rights[$location['location']][$account] += PHPGW_ACL_PRIVATE;
						}
					}
					if($location['admin']) {
						foreach($location['admin'] as $account) {
							$rights[$location['location']][$account] += EGW_ACL_CUSTOM_1;
						}
					}
				}

				// Give run permission
				if(isset($content['add']) && $content['add_to_all']) {
					$GLOBALS['egw']->acl->add_repository($app, 'run', $content['add_to_all'], 1);
				}
				foreach($rights as $location=>&$accounts) {
					if(isset($content['add']) && $content['add_to_all']) {
						$accounts[$content['add_to_all']] = $accounts[0];
					}
					foreach($accounts as $account=>$perms) {
						$GLOBALS['egw']->acl->add_repository($app, $location, $account, $perms);
					}
				}
				
				$this->messages[] = array(lang('Permissions have been changed'), 'success');
				unset($content['add']);
				unset($content['save']);
				unset($content['nm']['rows']);
			} elseif (isset($content['save']) && !$edit) {
				$this->messages[] = array(lang('You do not have permission to make changes'), 'error');
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			}

			$content['nm']['filter'] = $app;
			$content['acl'] = true;
			$filters = array(array('field' => 'app', 'label' => 'Application'));
			$select_box_options['filter'] = array($app=> lang($app));

			return $this->change_lookup($content, $app, $lookup, $prefix, $select_box_options, $filters, $this->bo, 'perp_api.bo_perp_api.get_acls');
		}

		/**
		* 	function change_lookup
		*	Changes lookup table values, such as currencies and transaction types
		*	@param	content - The returned content of the form or empty array
		*	@param	app - The application (perp_api, perp_ledger, etc)
		*	@param	lookup - The table name and the function (called from hook)
		*	@param	prefix - The prefix for the fields (same for every field)
		*	@param	select_box_options - an array for select box options
		*	@param	filters - Filters for the header, if desired
		*	@param 	bo	Optional, the logic layer (bo_) object to use to get the data from
		*	@param	get_rows	if $bo is provided, the function to call to get row data from
		*	@param	preserve	array of data to preserve, passed to eTemplate
		*/
		public function change_lookup($content = array(), $app, $lookup, $prefix, $select_box_options = array(), $filters = array(), $bo = '', $get_rows = '', $preserve = null) {
			if($this->debug) {
				echo 'change_lookup incoming data:';
				_debug_array($content);
				printf("App: %s Lookup: %s Prefix: %s<br />Select_box_options:", $app, $lookup, $prefix);
				_debug_array($select_box_options);
			}

			// Check Permissions
			if(!$GLOBALS['egw']->acl->check($lookup, EGW_ACL_READ, $app)) {
				echo "<p>Access to $app $lookup denied.  Need at least EGW_ACL_READ</p>";
				return false;
			} else {
				$add = $GLOBALS['egw']->acl->check($lookup, EGW_ACL_ADD,$app);
				$edit = $GLOBALS['egw']->acl->check($lookup, EGW_ACL_EDIT,$app);
				$delete = $GLOBALS['egw']->acl->check($lookup, EGW_ACL_DELETE,$app);
			}

			if($this->debug) {
				printf("Access: %s : %s : %s<br />", $add ? 'Add' : 'No Add', $edit ? 'Edit' : 'No Edit', $delete ? 'Delete' : 'No Delete');
			}

			// Set up nextmatch widget
			$data['nm'] = array(
				'no_cat'	=>	True,
				'filter_no_lang'	=>	True,
				'filter2_no_lang'	=>	True,
				'get_row_data'	=>	$get_rows ? $get_rows : $app.'.bo_'.$app.'.get_'.$lookup.'s',
				'prefix'	=>	$prefix,
				'lookup'	=>	$lookup,
				'get_rows'	=>	'perp_api.ui_perp_api.get_rows',
				'template'	=>	$app.'.admin.'.$lookup.'.row',
			);

			if($filters[0]) {
				$data['nm']['filter_label'] = $filters[0]['label'];
			} else {
				$data['nm']['no_filter'] = True;
			}
			if($filters[1]) {
				$data['nm']['filter2_label'] = $filters[1]['label'];
			} else {
				$data['nm']['no_filter2'] = True;
			}

			// Keep user settings
			$data['nm'] = array_merge($data['nm'], isset($content['nm']) ? $content['nm'] : array());
			$data['nm']['rows'] = isset($data['nm']['rows']) ? $data['nm']['rows'] : array();

			if($this->debug) {
				echo 'Callback: ' . $data['nm']['get_rows'] . '<br />';
			}

			// Match up keys with rows
			if(!is_array($content['nm']['rows'])) {
				$content['nm']['rows'] = array();
			}

			if(is_array($_POST['exec']['nm']['rows'])) {
				foreach($_POST['exec']['nm']['rows'] as $key => &$row) {
					if($key == 'new' && !$content['nm']['rows'][$key]) {
						$content['nm']['rows'][$key] = $_POST['exec']['nm']['rows'][$key];
					}
					if(is_numeric($key)) {
						$content['nm']['rows'][$key][$prefix . '_id'] = $_POST['exec']['nm']['rows'][$key][$prefix . '_id'];
					}
				}
			}

			if($this->debug) {
				echo 'Matched with $_POST:';
				_debug_array($content);
			}

			// Deal with incoming data
			if(isset($content['save']) && ($add || $edit)) {
				// Check for changes
				foreach($content['nm']['rows'] as $key => $row) {
					if(($edit || ($key == 'new' && $add)) && $row[$prefix . '_id'] !== '') {
						$id = array($prefix . '_id' => $row[$prefix.'_id']);
					} else {
						$id = false;
						unset($row[$prefix.'_id']);
					}
					if((is_array($id) && $id[$prefix.'_id'] != '') || (!is_array($id) && $id != '')) {
						$this->bo->change_lookup('perp_'.$lookup, $app, $id, $row);
					}
				}
				$this->messages[] = array(lang('settings saved'), 'success');
			} elseif (isset($content['nm']['rows']['delete']) && $delete) {
				$id = key($content['nm']['rows']['delete']);
				if($content['nm']['rows']['delete']) {
					$this->bo->change_lookup('perp_'.$lookup, $app, array($prefix.'_id' => $id),null);
				}
				unset($sections[$id]);
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			}

			// Get data for the first time
			$data['nm']['rows'] = $rows;

			$data['edit'] = $edit;
			$data['acl'] = isset($content['acl']);
			
			if(count($this->messages) > 0) {
				$data['message'] = $this->make_message($this->messages);
			}

			if($this->debug) {
				$callback = $app.'.hook_'.$app.'.'.$lookup;
				echo 'Callback is ' . $callback . '<br />';
				echo 'Data:';
				_debug_array($rows);
			}

			// Javascript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->set_onload("keyHandlerSetup();");
			$GLOBALS['egw']->js->validate_file('api', 'keyhandler', 'perp_api');

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang($app) .' - ' .lang('Admin') . ' - ' .lang($lookup);
			$this->template->read('perp_api.lookups');
			$this->template->exec($app.'.hook_'.$app.'.'.$lookup, $data, $select_box_options, $read_only, $preserve);
		}

		/** 
		*	This function adds in a hidden field with the read-only key, so we know what record the row
		*	is referring to.  Otherwise, things can get mixed up by the nextmatch widget when the user
		*	moves through the list.  It also adds in permissions to disable various columns.
		*
		*	This is an additional layer between the change_lookup() function and the actual data source,
		*	which is stored in $query['get_row_data'] instead of the normal $query['get_rows'].
		*/
		public function get_rows(&$query, &$rows, &$readonlys) {
			$app_data = explode('.', $query['get_row_data']);
			$bo = $bo ? $bo : CreateObject($app_data[0] . '.' . $app_data[1]);
			$count = $bo->$app_data[2]($query, $rows, $readonlys);
			foreach($rows as $key => &$row) {
				if(is_numeric($key)) {
					$row['hidden_' . $query['prefix'] . '_id'] = $row[$query['prefix'] . '_id'] . 
					'<input type="hidden" name="exec[nm][rows]['. ($key + 1).'][' . $query['prefix'] . '_id]" id="exec[nm][rows]['.($key + 1).'][' . $query['prefix'] . '_id]" value="'.$row[$query['prefix'] . '_id'].'" />';
				}
			}

			// add in permissions
			$add = $GLOBALS['egw']->acl->check($query['lookup'], EGW_ACL_ADD,$app_data[0]);
			$edit = $GLOBALS['egw']->acl->check($query['lookup'], EGW_ACL_EDIT,$app_data[0]);
			$delete = $GLOBALS['egw']->acl->check($query['lookup'], EGW_ACL_DELETE,$app_data[0]);
			if(!isset($rows['add'])) {
				$rows['add'] = $add;
			}
			if(!isset($rows['edit'])) {
				$rows['edit'] = $edit;
			}
			if(!isset($rows['delete'])) {
				$rows['delete'] = $delete;
			}
			return $count;
		}

		/**
		*	Make nicely formatted messages for the user
		*
		*	Includes the required CSS file, creates a nicely formatted message, and returns the HTML
		*	It can take either a single message, or an array of messages.  The array is the preferred 
		*	method, because you can have more than one message, and they'll be grouped by type.
		*
		*	Available types are:
		*		@image html 	message_success.png		
		*		success	-	Everything ok \n
		*		@image html	message_info.png	""	info	-	Everything ok, some additional information given to the user \n
		*		@image html	message_warning.png	""	warning	-	Something might be wrong, but things went ahead / can go ahead anyway \n
		*		@image html	message_error.png	""	error	-	Somethings wrong, can't proceed until it's fixed \n
		*
		*	@param	message	array / string	Either message text or an array in the form array( <message_text> => <message_type> { [, <message_text> => <message_type> ] } )
		*	@param	type	string		If message is a string, type is they message type.
		*
		*	@return	string	HTML for the messages
		*/
		public function make_message($message, $type = '') {
			// Load perp_api css.. This is a bad hack
			// TODO: make this less hackey
			if(file_exists(EGW_SERVER_ROOT . '/' . $appname . '/' 
				. 'templates' . '/' . $GLOBALS['egw_info']['server']['template_set']
				. '/' . 'app.css')) {
				$GLOBALS['egw_info']['flags']['css'] .= 
				'</style><link href="'.$GLOBALS['egw_info']['server']['webserver_url']
				. "/perp_api/templates/".$GLOBALS['egw_info']['server']['template_set']
				. "/app.css".'" type=text/css rel=StyleSheet><style type="text/css">';
			} elseif (file_exists(EGW_SERVER_ROOT . '/' . 'perp_api' . '/'
				. 'templates' . '/' . 'default'
				. '/' . 'app.css')) {
				$GLOBALS['egw_info']['flags']['css'] .= '</style><link href="'.$GLOBALS['egw_info']['server']['webserver_url'] . "/perp_api/templates/default/app.css" . '" type=text/css rel=StyleSheet><style type="text/css">';
			}

			// Format messages
			if(is_array($message)) {
				$msg = '';
				$list = array();
				foreach($message as $text => $type) {
					if(is_array($type)) {
						$text = $type[0];
						$type = $type[1];
					} 
					if(trim($text) != '') {
						$list[$type][] = '<li>' . $text . '</li>';
					}
				}
				foreach($list as $type => $messages) {
					$msg .= "<ul class='message' type='$type'>";
					$msg .= implode(' ', $messages);
					$msg .= '</ul>';
				}
				return $msg;
			} else {
				return "<ul class='message' type='$type'><li>$message</li></div>";
			}
		}
	}
?>
