<?php

	/**
	*	User inverface for Invoices.
	*	This is the base class providing common UI functions for 
	*	dealing with Invoices.
	*
	*	@ingroup perp_api
	*	@author Nathan Gray
	*/

	require_once('class.Invoice.inc.php');
	require_once('class.bo_perp_invoice.inc.php');

	class ui_perp_invoice {

		public $public_functions = array(
			'invoice_list'		=>	true,
			'add_line_from'		=>	true,
		);

		/**
		*	boolean Enable for debug information
		*/
		protected $debug = false;

		/**
		*	Array of messages to the user.
		*       @see perp_api::ui_perp_api::make_message()
		*/
		protected $messages = array();

		/**
		*       eTemplate used for display
		*/
		protected $template;

		/**
		*	Name of eTemplate used for displaying row lines
		*/
		protected $invoice_line_template;

		/**
		*	Name of eTemplate used for displaying a list of Invoices
		*/
		protected $list_template;
		protected $list_row_template;

		/**
		*	Name of eTemplate used for displaying uninvoiced lines from other applications
		*/
		protected $uninvoiced_line_template;

		/**
		*	Configuration options for Invoices
		*/
		protected $config;

		/**
		*	Logic layer object
		*/
		protected $bo_invoice;

		/**
		*	Constructor
		*	initializes the helper objects
		*/
		public function __construct() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->config = config::read('perp_api');

			$this->bo_invoice = new bo_perp_invoice();
		}

		/**
		*	Display a list of Invoices
		*
		*	@param content Array of content returned from the eTemplate
		*/
		public function invoice_list($content = array()) {
		}

		/**
		*	Get data for displaying a list of Invoices
		*/
		protected function list_get_display_data() {
			$data = array();
			$data['nextmatch'] = $GLOBALS['egw']->session->appsession('invoice_list', $GLOBALS['egw_info']['flags']['currentapp']);
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array(
					'no_cat'		=>	true,
					'filter_label'		=>	'Status',
					'filter_no_lang'	=>	true,
					'no_filter2'		=>	true,
					'get_rows'		=>	$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.get_invoice_list',
					'template'		=>	$this->list_row_template,
				);
			}
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			return $data;
		}

		/**
		*	Handle incoming data from the eTemplate for a list of Invoices
		*	@param content Array of content returned from the eTemplate
		*/
		protected function list_update(&$content) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}
			
			if(isset($content['new_invoice'])) {
				$this->handle_action('new_invoice', null);
				if($result) {
					unset($content['new_invoice']);
				}
			}

			if(!isset($content['nextmatch']['rows'])) {
				return false;
			}

			if($content['action']) {
				$result = $this->handle_action($content['action'], $content['nextmatch']['rows']['checked']);
				unset($content['action']);
			}

			foreach($content['nextmatch']['rows'] as $action => $row) {
				if(!is_array($row)) {
					continue;
				}
				$invoice_id = key($row);

				$result = $this->handle_action($action, $invoice_id);
				if($result) {
					unset($content['nextmatch']['rows'][$action]);
				}
			}
		}

		/**
		*       Get selectbox options for displaying a list
		*/
		protected function list_get_options() {
			$options = array();
			$options['filter'] = $this->bo_invoice->get_status_list();
			return $options;
		}

		/**
		*	Get read only array for displaying a list
		*/
		protected function list_get_readonlys() {
			$readonly = array();

			$readonly['new_invoice'] = !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_ADD, $GLOBALS['egw_info']['flags']['currentapp']);

			return $readonly;
		}

		/**
		*	Handle an action from one item on the list
		*
		*	Actions can be one of new_invoice, post
		*/
		protected function handle_action($action, $invoice_id) {
			switch($action) {
				case 'new_invoice':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=> $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.new_invoice',
						'owner_id'	=>	$invoice_id
					));
					return true;
				case 'post':
					$invoice = new Invoice($invoice_id);
					$invoice->post();
					return true;
					break;
				default:
					return false;
			}
		}

		/**
		*	Store nextmatch parameters in the session
		*	
		*	@nextmatch
		*/
		public function get_invoice_list(&$query=array(), &$rows = array(), &$readonly = array()) {
			$GLOBALS['egw']->session->appsession('invoice_list', $GLOBALS['egw_info']['flags']['currentapp'], $query);
			return $this->bo_invoice->get_invoice_list($query, $rows, $readonly);
		}

		/**
		*	Update the Invoice with the given data.
		*	Removes lines too.  Adds errors into messages array.
		*	
		*	@param invoice	Invoice to pdate
		*	@param content	array with key => value mapping of new values
		*
		*	@return boolean Successfully updated
		*/
		protected function update_invoice(Invoice &$invoice, &$content) {
			if($this->debug) {
				echo get_class($this) . '.' . __FUNCTION__ . ' ' . get_class($invoice) . ': ';
				echo '<pre>' . $invoice->__toString() . '</pre>';
				echo 'New content: ';
				_debug_array($content);
			}

			$edit = $invoice->is_editor($GLOBALS['egw_info']['user']['account_id']);
			if($edit !== true) {
				$this->messages[] = array($edit, 'error');
				return false;
			}

			$result = true;

			// Parties involved (suppliers, clients)
			if(isset($content['supplier_id'])) {
				$result &= $this->check_result( $invoice->set_supplier_id((int)$content['supplier_id']));
			}
			if(isset($content['client_id'])) {
				$result &= $this->check_result( $invoice->set_client_id((int)$content['client_id']));
			}

			// Reference numbers
			if(isset($content['our_reference_number'])) {
				$invoice->set_our_reference_number($content['our_reference_number']);
			}
			if(isset($content['their_reference_number'])) {
				$invoice->set_their_reference_number($content['their_reference_number']);
			}

			// Contacts
			if(isset($content['our_contact_person'])) {
				$result &= $this->check_result($invoice->set_our_contact_person((int)$content['our_contact_person']));
			}
			if(isset($content['their_contact_person'])) {
				$result &= $this->check_result( $invoice->set_their_contact_person((int)$content['their_contact_person']));
			}

			// Dates
			if(isset($content['issue_date'])) {
				$result &= $this->check_result( $invoice->set_issue_date((int)$content['issue_date']));
			}
			if(isset($content['due_date'])) {
				$result &= $this->check_result($invoice->set_due_date((int)$content['due_date']));
			}
			if(isset($content['discount_date'])) {
				$result &= $this->check_result($invoice->set_discount_date((int)$content['discount_date']));
			}

			// Locations
			if(isset($content['bill_from_address'])) {
				$result &= $this->check_result( $invoice->set_bill_from_address((int)$content['bill_from_address']));
			}
			if(isset($content['bill_to_address'])) {
				$result &= $this->check_result($invoice->set_bill_to_address((int)$content['bill_to_address']));
			}

			// Financial
			if(isset($content['invoice_currency'])) {
				$result &= $this->check_result($invoice->set_invoice_currency($content['invoice_currency']));
			}
			if(isset($content['exchange_rate'])) {
				$result &= $this->check_result($invoice->set_exchange_rate($content['exchange_rate']));
			}
			if(isset($content['payment_terms'])) {
				$result &= $this->check_result($invoice->set_payment_terms((int)$content['payment_terms']));
			}

			// Comments
			if(isset($content['comments'])) {
				$invoice->set_comments($content['comments']);
			}

			if(isset($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as &$line_data) {
					$invoice_line =& $invoice->get_line($line_data['line_no']);
					if($invoice_line instanceof InvoiceLine) {
						$result &= $this->update_line($invoice_line, $line_data);
					}
				}
			}

			// Add line
			if(isset($content['add_line']) && isset($content['new']) && trim($content['new']['description']) != '') {
				$this->add_line($invoice, $content['new']);
			}

			// Add line from other apps needs a popup
			if($content['add_line_from']) {
				unset($content['add_line_from']);
				$GLOBALS['egw']->session->appsession('invoice_reload', $GLOBALS['egw_info']['flags']['currentapp'], true);
				$GLOBALS['egw']->session->appsession('invoice_popup', $GLOBALS['egw_info']['flags']['currentapp'], true);
				$GLOBALS['egw']->session->appsession('invoice_app', $GLOBALS['egw_info']['flags']['currentapp'], $content['add_line_app']);
			}

			// Remove lines
			if(isset($content['nextmatch']['rows']['delete'])) {
				$this->remove_line($invoice, key($content['nextmatch']['rows']['delete']));
			}

			// Approval
			if($invoice->is_admin($GLOBALS['egw_info']['user']['account_id']) && !$invoice->get_approval_date()) {

			}

			// Taxes
			if(isset($content['calculate_taxes'])) {
				$lines =& $invoice->get_lines();
				foreach($lines as $line) {
					if(!isset($location[$line->get_our_location()])) {
						$location[$line->get_our_location()] = ExecMethod('perp_inventory.bo_perp_location.get_location', $line->get_our_location());
					}
					$this->check_result($line->calculate_taxes($tax_group, $location[$line->get_our_location()]['tax_location']));
				}
				$invoice->add_tax_lines();
			}

			if($this->debug) {
				echo get_class($this) . ' After updating ' . get_class($invoice) . ': ';
				echo '<pre>' . $invoice->__toString() . '</pre>';
				echo 'Errors so far: ';
				_debug_array($this->messages);
			}
			return $result;
		}

		/**
		*	Add a new InvoiceLine to the provided Invoice.  This is 
		*	a stub that should be re-implemented and don't call parent::.
		*	This is so that the correct InvoiceLine extending class is 
		*	added, instead of the generic InvoiceLine.
		*	@param invoice Invoice to add the line to
		*	@param data InvoicableItem or array of data for the new line
		*/
		protected function add_line(Invoice &$invoice, &$data) {
			$this->messages[] = array('Developer should re-implement add_line', 'error');
		}

		/**
		*	Remove an InvoiceLine from the provided Invoice
		*	@param invoice Invoice to remove the line from
		*	@param line_no Line number to remove
		*/
		protected function remove_line(Invoice &$invoice, &$line_no) {
			return $this->check_result($invoice->remove_line($line_no));
		}

		/**
		*	Update an InvoiceLine with the provided line data
		*
		*	@param invoice_line InvoiceLine to update
		*	@param line_data array of line data
		*/
		protected function update_line(InvoiceLine &$invoice_line, &$line_data) {
			$result = true;
			if($line_data['quantity']) {
				$invoice_line->set_quantity($line_data['quantity']);
			}
			if($line_data['description']) {
				$invoice_line->set_description($line_data['description']);
			}
			$invoice_line->set_unit_price($line_data['unit_price']);
			$invoice_line->set_general_ledger_account($line_data['general_ledger_account']);
			return $result;
		}

		/**
		*       Internal function to get a Invoice object to work with.
		*       First, we check the session for an invoice in progress.  Then we
		*       check the provided invoice ID an read from the database, if required.
		*	Extending classes should override this function to create the correct
		*	Invoice type.
		*/
		protected function &get_invoice($invoice_id, $class = 'Invoice') {
			$inv = $GLOBALS['egw']->session->appsession('invoice');
			if(
				($invoice_id && $inv instanceof $class && $invoice_id != $inv->get_invoice_id()) ||
				$invoice_id && !($inv instanceof $class)
			) {
				$invoice = new $class($invoice_id);
			} elseif ($inv) {
				$invoice = $inv;
			} else {
				$invoice = null;
			}

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($invoice->get_supplier_id(), $supplier_ids)) {
				$invoice = null;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($invoice->get_client_id(), $client_ids)) {
				$invoice = null;
			}

			return $invoice;
		}

		/**
		*	Get an array of data suitable for displaying with an eTemplate
		*/
		protected function get_display_data(Invoice &$invoice) {
			$data = array();
			$data += $invoice->toArray(false);

			// Can add new lines
			$data['add'] = (is_null($invoice->get_transaction()) && $invoice->is_editor($GLOBALS['egw_info']['user']['account_id']));

			// Set up nextmatch widget
			$data['nextmatch'] = array(
				'invoice_id'		=>	$invoice->get_invoice_id(),
				'supplier_id'		=>	$invoice->get_supplier_id(),
				'client_id'		=>	$invoice->get_client_id(),
				'no_cat'		=>	true,
				'no_filter2'		=>	true,
				'get_rows'		=>	'perp_api.ui_perp_invoice.get_invoice_lines',
				'template'		=>	$this->invoice_line_template
			);
			// Invoice added below
			
			if($this->debug) {
				echo __METHOD__ .' display ' . get_class($invoice) . ' data: ';
				_debug_array($data);
				echo 'Errors so far: ';
				_debug_array($this->messages);
			}

			// This variable set after the debug, because it's pretty big
			$data['nextmatch']['invoice']	=&	$invoice;

			return $data;
		}

		/**
		*	Get selectbox arrays for editing 
		*/
		protected function get_select_box_options(Invoice &$invoice) {
			$options = array();
			$options['general_ledger_account']	=	array('' => lang('Select')) + ExecMethod('perp_ledger.bo_perp_ledger.get_accounts');
			$options['payment_terms']	=	array('' => lang('Select')) + ExecMethod('perp_api.bo_perp_api.get_payment_terms');
			$options['payment_method']	=	array('' => lang('Select')) + ExecMethod('perp_api.bo_perp_api.get_payment_methods');
			$options['invoice_currency']	=	ExecMethod('perp_api.bo_perp_api.get_currencies');
			$options['filter']		=	array(false => 'No details', 1 => 'Some details', 2 => 'Full details');

			return $options;
		}

		/**
		*	Set up array of widgets to set to readonly because of Invoice status or
		*	user permissions.  Array is in the form widget_name => true.
		*/
		protected function get_readonlys(Invoice &$invoice) {
			$posted = (boolean)$invoice->get_transaction();
			$edit = !$posted && $invoice->is_editor($GLOBALS['egw_info']['user']['account_id']);
			$admin = !$posted && $invoice->is_admin($GLOBALS['egw_info']['user']['account_id']);

			$readonly = array(
				'supplier_id'		=>	array('search' => !$edit || (boolean)$invoice->get_supplier_id()),
				'client_id'		=>	!$edit || (boolean)$invoice->get_client_id(),
				'our_reference_number'	=>	!$edit,
				'their_reference_number'	=>	!$edit,
				'our_contact_person'	=>	!$edit,
				'their_contact_person'	=>	!$edit,
				'payment_terms'		=>	!$edit,
				'invoice_currency'	=>	!$edit,
				'exchange_rate'		=>	!$edit,
				'issue_date'		=>	!$edit,
				'due_date'		=>	!$edit,
				'total'			=>	!$edit,
				'add_line'		=>	!$edit,

				// This button only available after client or supplier is set
				'add_line_from'		=>	!$edit,

				'save'			=>	!$edit,
				'post'			=>	!$admin,
			);

			return $readonly;
		}

		/**
		*	Set up javascript and ajax
		*	@param ajax_callback appname.ajax_object Object to use for ajax callbacks
		*/
		protected function get_javascript($ajax_callback) {
			// xajax
			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}       
			$GLOBALS['egw']->js->set_onload(" ajax_perp_callback = '$ajax_callback'; ");
			$GLOBALS['egw']->js->set_onload('invoice_edit_setup(); ');
			$GLOBALS['egw']->js->validate_file('invoice', 'edit', 'perp_api');
			$GLOBALS['egw']->js->validate_file('invoice', 'edit', $GLOBALS['egw_info']['flags']['currentapp']); 

			if($GLOBALS['egw']->session->appsession('invoice_popup', $GLOBALS['egw_info']['flags']['currentapp'])) {
				$popup_link = $GLOBALS['egw']->link('/index.php',
					array(
						'menuaction'    =>      $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.add_line_from',
						'appname'       =>      $GLOBALS['egw']->session->appsession('invoice_app', $GLOBALS['egw_info']['flags']['currentapp']),
					)
				);
				$GLOBALS['egw']->js->set_onload("window.open('$popup_link', '_blank','dependent=yes,width=700,height=600,scrollbars=yes,status=yes'); ");
				$GLOBALS['egw']->session->appsession('invoice_popup', $GLOBALS['egw_info']['flags']['currentapp'], false);
				$GLOBALS['egw']->session->appsession('invoice_app', $GLOBALS['egw_info']['flags']['currentapp'], false);
			}
		}


		/**
		*	Callback for the nextmatch widget to get the InvoiceLines while 
		*	viewing / editing an Invoice
		*	@nextmatch
		*/
		public function get_invoice_lines(&$query, &$rows, &$readonlys) {
			if(isset($query['invoice']) && $query['invoice'] instanceof Invoice) {
				$invoice =& $query['invoice'];
			} elseif ($invoice = $GLOBALS['egw']->session->appsession('invoice')) {
				// $invoice pulled from session
				$invoice = unserialize($invoice);
			}
			if (!$invoice instanceof Invoice) {
				return 0;
			}

			$lines = array();
			$rows = array();
			$readonlys = array();
			$lines = $invoice->get_lines();

			if(!isset($query['start'])) {
				$query['start'] = 0;
			}
			if(!isset($query['num_rows']) || $query['num_rows'] < 0) {
				$query['num_rows'] = count($lines);
			}

			// Save query settings to session
			$GLOBALS['egw']->session->appsession('invoice_' . $invoice->get_invoice_id() . '_lines', $GLOBALS['egw_info']['flags']['currentapp'], $query);

			/*
			* Kinda sucks to do this from the session instead of the database...
			* Have to sort by line number to clear the keys and keep everything in the right order,
			* and have the query count get the right records
			*/
			if(count($lines) > 0) {
				foreach($lines as $key => $line) {
					$line_number[$key] = $line->get_line_number();
				}
				array_multisort($line_number, $lines);
			} else {
				return 0;
			}
			$count = count($lines);

			for($i = $query['start']; count($rows) < $query['num_rows'] && $i < count($lines); $i++) {
				$line =& $lines[$i];
				if(!($line instanceof InvoiceLine)) {
					continue;
				}

				$row = $line->toArray();
				$row['line_no'] = $line->get_line_number();
				$row['line_price'] = $row['quantity'] * $row['unit_price'];

				if($line->get_invoicable_item() != null) {
					$row += $line->get_invoicable_item()->toArray();

					// Display extra details for each line
					if($query['filter'] && method_exists($line->get_invoicable_item(), 'get_invoice_details')) {
						$row['details'] = $line->get_invoicable_item()->get_invoice_details($query['filter']);
						$row['details']['details'] = count($row['details']) > 0;
					}
				}

				if($query['search']) {
					if(strpos($row['supplier_stock_code'], $query['search']) === False &&
					strpos($row['our_stock_code'], $query['search']) === False &&
					strpos($row['description'], $query['search']) === False &&
					strpos($row['our_reference_number'], $query['search']) === False) {
						unset($row);
						$count --;
						continue;
					}
				}

				$rows[] = $row;

				$readonlys['delete[' . $line->get_line_number() . ']'] = !$invoice->is_editor($GLOBALS['egw_info']['user']['account_id']);
			}

			$rows['remove_line'] = !$invoice->is_editor($GLOBALS['egw_info']['user']['account_id']);

			return $count;
		}


		/**
		*	Add lines from any invoicable application.  This lets an invoice be for more than just one order.
		*
		*	@param content Array of content returned from the eTemplate
		*/
		public function add_line_from($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			$invoice =& $this->get_invoice(null);
			$appname = get_var('appname', 'any');

			// Add lines to order
			if($invoice && isset($content['add'])) {
				$checked = array_flip($content['nextmatch']['rows']['checked']);
				unset($content['nextmatch']['rows']['checked']);
				foreach($content['nextmatch']['rows'] as $key => $row) {
					// Skip lines where line is not checked, or missing info
					if(!array_key_exists($key, $checked) || !$row['invoicable_class']) {
						continue;
					}

					$add_to_invoice = true;

					// Skip lines that are already on the invoice
					foreach($invoice->get_lines() as $line) {
						$invoicable_item =& $line->get_invoicable_item();
						if(!is_null($invoicable_item)) {
						// Skip lines that are already on the invoice
							if($invoicable_item->get_invoicable_id() == $row['invoicable_id'] &&
								$invoicable_item->get_invoicable_line_id() == $row['line_number']) {
								$add_to_invoice = false;
								if($this->debug) {
									echo 'Skipped ' . print_r($row, true) . ' because it is already on the invoice as ' . $line->__toString() . '<br />';
								}
								break;
							}
						}
					}
					if($add_to_invoice) {
						$line_count++;
						$row['quantity'] = $row['invoice_quantity'];
						$this->add_line($invoice, $row);
						if($this->debug) {
							echo 'Added ' . print_r($row, true) . ' to invoice<br />';
						}
					}
				}

				// Save to session
				$GLOBALS['egw']->session->appsession('invoice', false, $invoice);

				// Reload parent window
				$GLOBALS['egw']->js->set_onload('window.opener.location.reload(); ');
				// Close popup
				if(!$this->debug) {
					$GLOBALS['egw']->js->set_onload('window.close();');
				}

			} 

			// Get display data
			$data['nextmatch'] = array(
				'appname'	=>	$appname,
				'supplier_id'	=>	$invoice->get_supplier_id(),
				'client_id'	=>	$invoice->get_client_id(),
				'get_rows'	=>	$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.get_uninvoiced_lines',
				'template'	=>	is_array($this->uninvoiced_line_template) ? $this->uninvoiced_line_template[$appname] : $this->uninvoiced_line_template,
				'no_cat'	=>	true,
				'no_filter'	=>	true,
				'no_filter2'	=>	true
			);
			$select_box_options['filter'] = array('' => 'All');
			$select_box_options['filter2'] = array('' => 'All');

			$preserve['appname'] = $appname;

			$GLOBALS['egw_info']['flags']['app_header'] = lang('Add Line From %1', lang($appname));
			$this->template->read('perp_api.invoice.uninvoiced_order_lines');
			$this->template->exec($GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.' . __FUNCTION__, 
				$data, $select_box_options, $readonly, false, 2
			);
		}

		public function get_uninvoiced_lines(&$query = array(), &$rows = array(), &$readonly = array()) {
			return $this->bo_invoice->get_uninvoiced_lines($query, $rows, $readonly);
		}

		/**
		*       Check the result of a method call and add it into the messages array if needed.
		*	@todo This should go somewhere common
		*       @param result Boolean true or error message
		*       @param type Severity of message
		*/
		protected function check_result($result, $type = 'warning') {
			if($result === true) {
				return true;
			} elseif(strlen(trim($result)) > 0) {
				$this->messages[] = array($result, $type);
			}
			return false;
		}
	}
?>
