<?php

	/**
	*	User interface for Orders
	*
	*	Provides common UI functions for dealing with Orders
	*
	*	@ingroup perp_api
	*	@author Nathan Gray
	*/

	require_once('class.Order.inc.php');
	require_once('class.bo_perp_order.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_ap/inc/class.bo_perp_supplier.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_ar/inc/class.bo_perp_client.inc.php');

	class ui_perp_order {
		
		public $public_functions = array(
			'order_list'	=>	true,
			'new_order'	=>	true,
		);

		/**
		*	boolean Enable for debug information
		*/
		protected $debug = false;

		/**
		*	Array of messages to the user.
		*	@see perp_api::ui_perp_api::make_message()
		*/
		protected $messages = array();

		/**
		*	eTemplate used for display
		*/
		protected $template;

		/**
		*	Name of eTemplate used for displaying row lines
		*/
		protected $order_line_template;

		/**
		*	Name of eTemplate used for editing OrderLine details
		*/
		protected $order_line_detail_edit;

		/**
		*	Name of eTemplate used for displaying a list of Orders
		*/
		protected $order_list_template;
		protected $order_list_row_template;

		/**
		*	Configuration options for Orders
		*/
		protected $config;

		/**
		*	Logic layer object
		*/
		protected $bo_order;

		/**
		*	Constructor
		*	Initializes the helper objects
		*/
		public function __construct() {
			// I don't know where else to put this
			if(!$GLOBALS['perp_api']['ui']) {
				require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_api.inc.php');
				$GLOBALS['perp_api']['ui'] =& new ui_perp_api();
			}

			$this->template = CreateObject('etemplate.etemplate');
			$this->config = config::read('perp_api');

			$this->bo_order = new bo_perp_order();
		}

		/**
		*	Display a list of Orders
		*	
		*
		*	@param content Array of content returned from the eTemplate
		*/
		public function order_list($content = array()) {
		}

		/**
		*	Handle incoming data from the eTemplate for a list of orders
		*	@param content Array of content returned from the eTemplate
		*/
		protected function order_list_update(&$content) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			if(isset($content['new_order'])) {
				$result = $this->handle_order_action('new', null);
				if($result) {
					unset($content['new_order']);
				}
			}

			if(!isset($content['nextmatch']['rows'])) {
				return false;
			}
			foreach($content['nextmatch']['rows'] as $action => $row) {
				$order_id = key($row);
				try {
					$result = $this->handle_order_action($action, $order_id);
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
				if($result) {
					unset($content['nextmatch']['rows'][$action]);
				}
			}
		}

		/**
		*	Get data for displaying a list of Orders
		*/
		protected function order_list_get_display_data() {
			$data = array();
			$data['nextmatch'] = $GLOBALS['egw']->session->appsession($GLOBALS['egw_info']['flags']['currentapp'] . '.nextmatch');
			if(!$data['nextmatch']) {
				$data['nextmatch'] = array(
					'no_cat'	=>	true,
					'filter_label'		=>	'Status',
					'filter_no_lang'	=>	true,
					'no_filter2'		=>	true,
					'get_rows'		=>	$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.get_order_list',
					'template'		=>	$this->order_list_row_template,
					'filter'		=>	 $GLOBALS['egw_info']['user']['preferences'][$GLOBALS['egw_info']['flags']['currentapp']]['order_filter'],
					'order'			=>	 $GLOBALS['egw_info']['user']['preferences'][$GLOBALS['egw_info']['flags']['currentapp']]['order_order'],
				);
			}
			unset($data['nextmatch']['rows']);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			return $data;
		}

		/**
		*	Get selectbox options for displaying a list of Orders
		*/
		protected function order_list_get_options() {
			$options = array();
			$options['filter'] = $this->bo_order->get_status_list();
			return $options;
		}

		/**
		*	Get read only array for displaying a list of Orders
		*/
		protected function order_list_get_readonlys() {
			$readonly = array();

			$add = $GLOBALS['egw']->acl->check('order', EGW_ACL_ADD, $GLOBALS['egw_info']['flags']['current_app']);
			$edit = $GLOBALS['egw']->acl->check('order', EGW_ACL_EDIT, $GLOBALS['egw_info']['flags']['current_app']);
			$approve = $GLOBALS['egw']->acl->check('order', EGW_ACL_CUSTOM1, $GLOBALS['egw_info']['flags']['current_app']);
			$cancel = $GLOBALS['egw']->acl->check('order', EGW_ACL_DELETE, $GLOBALS['egw_info']['flags']['current_app']);
			$receive = $GLOBALS['egw']->acl->check('receive', EGW_ACL_READ, $GLOBALS['egw_info']['flags']['current_app']);

			$readonly['new_order'] = !$add;

			return $readonly;
		}

		/**
		*	Handle an action on an Order
		*
		*	Actions can be one of new, edit, 
		*	approve, deny, cancel
		*
		*	@param action	Action to take
		*	@param order_id	ID of the Order to act on
		*/
		protected function handle_order_action($action, $order_id) {
			switch($action) {
				case 'new':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=> $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.new_order',
					));
					return true;
					break;
				case 'edit':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=> $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.edit',
						'order_id'	=> $order_id
					));
					return true;
					break;
				case 'approve':
					$order = new Order($order_id);
					return $order->approve($GLOBALS['egw_info']['user']['account_id']);
					break;
				case 'deny':
					$order = new Order($order_id);
					return $order->deny($GLOBALS['egw_info']['user']['account_id']);
					break;
				case 'cancel':
					$order = new Order($order_id);
					return $order->cancel($GLOBALS['egw_info']['user']['account_id']);
					break;
				default:
					// Don't know how to handle this
					return false;
			}
		}
		
		/**
		*	Update the Order with the given data.
		*	Removes lines too.
		*	
		*	@param order	Order to update
		*	@param content	array with key => value mapping of new values
		*	
		*	@return boolean Successfully updated
		*/
		protected function update_order(Order &$order, &$content) {
			if($this->debug) {
				echo 'perp_api.ui_perp_order Update ' . get_class($order) . ': ';
				echo '<pre>' . $order->__toString() . '</pre>';
				echo 'New content: ';
				_debug_array($content);
			}
			$edit = $order->is_editor($GLOBALS['egw_info']['user']['account_id']);
			if($edit !== true) {
				$this->messages[] = array(lang('Edit is not allowed'), 'error');
				return false;
			}

			$result = true;

			// Parties involved (suppliers, clients)
			if(isset($content['supplier_id']) && is_numeric($content['supplier_id'])) {
				$result &= $this->check_result( $order->set_supplier_id((int)$content['supplier_id']));
			}
			if(isset($content['client_id']) && is_numeric($content['client_id'])) {
				$result &= $this->check_result( $order->set_client_id((int)$content['client_id']));
			}

			// Reference numbers
			if(isset($content['our_reference_number'])) {
				$order->set_our_reference_number($content['our_reference_number']);
			}
			if(isset($content['their_reference_number'])) {
				$order->set_their_reference_number($content['their_reference_number']);
			}

			// Contacts
			if(isset($content['our_contact_person'])) {
				$result &= $this->check_result($order->set_our_contact_person((int)$content['our_contact_person']));
			}
			if(isset($content['their_contact_person'])) {
				$result &= $this->check_result( $order->set_their_contact_person((int)$content['their_contact_person']));
			}

			// Dates
			if(isset($content['issue_date'])) {
				$result &= $this->check_result( $order->set_issue_date((int)$content['issue_date']));
			}
			if(isset($content['requested_delivery_date'])) {
				$result &= $this->check_result($order->set_requested_delivery_date((int)$content['requested_delivery_date']));
			}

			// Locations
			if(isset($content['ship_from_address'])) {
				$result &= $this->check_result( $order->set_ship_from_address((int)$content['ship_from_address']));
			}
			if(isset($content['ship_to_address'])) {
				$result &= $this->check_result($order->set_ship_to_address((int)$content['ship_to_address']));
			}
			if(isset($content['bill_to_address'])) {
				$result &= $this->check_result($order->set_bill_to_address((int)$content['bill_to_address']));
			}

			// Financial
			if(isset($content['order_currency'])) {
				$result &= $this->check_result($order->set_order_currency($content['order_currency']));
			}
			if(isset($content['payment_terms'])) {
				$result &= $this->check_result($order->set_payment_terms((int)$content['payment_terms']));
			}

			// Comments
			if(isset($content['comments'])) {
				$order->set_comments($content['comments']);
			}

			if(isset($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as &$line_data) {
					$order_line =& $order->get_line($line_data['line_number']);
					if(isset($line_data['delete'])) {
						$this->remove_line($order, key($line_data['delete']));
					} elseif($order_line instanceof OrderLine) {
						$result &= $this->update_line($order_line, $line_data);
					}

				}
			}

			// Add line
			if(isset($content['add_line']) && isset($content['new'])) {
				$this->add_line($order, $content['new']);
			}

			// Remove lines
			if(isset($content['nextmatch']['rows']['delete'])) {
				$this->remove_line($order, key($content['nextmatch']['rows']['delete']));
			}

			// Approval
			if(!$order->get_approval_date() && $content['approving_user']) {
				$order->set_approving_user((int)$content['approving_user']);
			}
			if(isset($content['approve']) && $order->is_admin($GLOBALS['egw_info']['user']['account_id']) && !$order->get_approval_date()) {
				$result &= $this->check_result($order->approve($GLOBALS['egw_info']['user']['account_id']));
			}
			if($this->debug) {
				echo 'perp_api.ui_perp_order After updating ' . get_class($order) . ': ';
				echo '<pre>' . $order->__toString() . '</pre>';
				echo 'Errors so far: ';
				_debug_array($this->messages);
			}
			return $result;
		}

		/**
		*	Add a new OrderLine to the provided Order
		*	@param order Order to add the line to
		*	@param line_data array of line data for the new line
		*
		*	@return the new line number
		*/
		protected function add_line(Order &$order, &$line_data) {
			$this->messages[] = array('Developer should re-implement add_line', 'error');
		}

		/**
		*	Remove an OrderLine from the provided Order
		*	@param order Order to remove the line from
		*	@param line_no Line number to remove
		*/
		protected function remove_line(Order &$order, &$line_no) {
			return $this->check_result($order->remove_line($line_no));
		}

		/**
		*	Update an OrderLine with the provided line data
		*
		*	@param order_line OrderLine to update
		*	@param line_data array of line data
		*/
		protected function update_line(OrderLine &$order_line, &$line_data) {
			$result = true;

			if(isset($line_data['stock_id'])) {
				$result &= $this->check_result($order_line->set_stock_id($line_data['stock_id']));
			}
			if(isset($line_data['supplier_stock_code'])) {
				$result &= $this->check_result($order_line->set_supplier_stock_code($line_data['supplier_stock_code']));
			}
			if(isset($line_data['our_stock_code'])) {
				$result &= $this->check_result($order_line->set_our_stock_code($line_data['our_stock-code']));
			}
			if(isset($line_data['description'])) {
				$result &= $this->check_result($order_line->set_description($line_data['description']));
			}
			if(isset($line_data['quantity_ordered'])) {
				$result &= $this->check_result($order_line->set_quantity_ordered($line_data['quantity_ordered']));
			}
			if(isset($line_data['unit_price'])) {
				$result &= $this->check_result($order_line->set_unit_price($line_data['unit_price']));
			}
			if(isset($line_data['ship_location'])) {
				$result &= $this->check_result($order_line->set_ship_location($line_data['ship_location']));
			}
			if(isset($line_data['gl_account'])) {
				$result &= $this->check_result($order_line->set_gl_account($line_data['gl_account']));
			}
			if(is_array($line_data['details'])) {
				foreach($line_data['details'] as $detail_class => $detail_data_list) {
					foreach($detail_data_list as $detail_data) {
						$detail =& $order_line->get_detail($detail_data['detail_number']);
						if ($detail_data['detail_number'] === '') {
							$current_details = $order_line->get_detail($detail_data['detail_name']);
							foreach($current_details as $current_detail) {
								if($current_detail->get_value() == '') {
									// Already an empty detail of this type, don't add another
									continue 2;
								}
							}
							$detail = new $detail_class($order_line, $detail_data);
							$order_line->add_detail($detail);
						}
						if($detail && $detail instanceof OrderLineDetail) {
							$result &= $this->check_result($this->update_line_detail($detail, $detail_data));
						}
					}
				}
			}

			return $result;
		}

		/**
		*	Get an array of data suitable for displaying with an eTemplate
		*/
		protected function get_display_data(Order &$order) {
			$data = array();
			$data += $order->toArray(false);

			// Addresses
			if($data['bill_to_address']) {
				$data['bill_to_full_address']	=	ExecMethod('addressbook.addressbook_bo.read', $order->get_bill_to_address());
				if($data['bill_to_full_address'] === false) {
					$this->messages[] = array(lang('Permission on billing address for %1 denied.  The address is there, but you are not allowed to see it.', lang('bill to')), 'warning');
				} elseif (is_null($data['bill_to_full_address'])) {
					$this->messages[] = array(lang('Unable to find address for %1.', lang('bill_to')), 'error');
				}
				$data['bill_to_full_address'] = array('address' => $data['bill_to_address']);
			}
			if($data['ship_from_address']) {
				$data['ship_from_full_address']	=	ExecMethod('addressbook.addressbook_bo.read', $order->get_ship_from_address());
				if($data['ship_from_full_address'] === false) {
					$this->messages[] = array(lang('Permission on shipping address for %1 denied.  The address is there, but you are not allowed to see it.', lang('ship from')), 'warning');
				} elseif (is_null($data['ship_from_full_address'])) {
					$this->messages[] = array(lang('Unable to find address for %1.', lang('ship_from')), 'error');
				}
				$data['ship_from_full_address'] = array('address' => $data['ship_from_address']);
			}
			if($data['ship_to_address']) {
				$data['ship_to_full_address']	=	ExecMethod('addressbook.addressbook_bo.read', $order->get_ship_to_address());
				if($data['ship_to_full_address'] === false) {
					$this->messages[] = array(lang('Permission on shipping address for %1 denied.  The address is there, but you are not allowed to see it.', lang('ship to')), 'warning');
				} elseif (is_null($data['ship_to_full_address'])) {
					$this->messages[] = array(lang('Unable to find address for %1.', lang('ship_to')), 'error');
				}
				$data['ship_to_full_address'] = array('address' => $data['ship_to_address']);
			}

			// Set up nextmatch widget
			// Check session for saved settings
			$data['nextmatch'] = $GLOBALS['egw']->session->appsession('order_lines', $GLOBALS['egw_info']['flags']['currentapp']);
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array();
			}
			$data['nextmatch'] += array(
				'order_id'	=>	$order->get_order_id(),
				'supplier_id'	=>	$order->get_supplier_id(),
				'client_id'	=>	$order->get_client_id(),
				'no_filter'	=>      True,
				'filter_label'	=>      'Status',
				'filter'	=>      $po_id,
				'filter_no_lang'	=>      True,
				'no_filter2'	=>      True,
			//	'filter2_label' =>      '',
			//	'filter2_no_lang'       =>      True, 
			//	'filter2'	=>	current($select_box_options['filter2']),
				'get_rows'      =>      $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.' . 'get_order_lines',
				'template'      =>      $this->order_line_template
			);

			// Set up add line select box
			$data['new']['supplier_stock_code']['supplier_id'] = (string)$order->get_supplier_id();

			// Total
			$data['total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $order->get_order_currency(), $order->get_total());

			// Check credit limits
			if($GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ) && 
					($order->can_invoice() === true || $order->get_approval_required())) {
				if($order->get_supplier_id()) {
					// Check supplier credit limit
					$bo_supplier = new bo_perp_supplier();
					$supplier = $bo_supplier->get_supplier($order->get_supplier_id());
					$ledger_query = array(
						'supplier_id'	=>	$order->get_supplier_id()
					);
					$bo_supplier->get_supplier_ledger($ledger_query, $ledger);
					if($supplier['credit_limit'] != '' && ($order->get_total() + $ledger_query['balance']) > $supplier['credit_limit']) {
						$this->messages[] = array(lang('This order will exceed the credit limit of %1.  The current balance, including this order, is %2.', 
							ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $supplier['supplier_currency'], $supplier['credit_limit']),
							ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $supplier['supplier_currency'], ($order->get_total() + $ledger['total']))
						), 'warning');
					}
				} elseif ($order->get_client_id()) {
					// Check client credit limit - will have to be overridden for client accounts
					$bo_client = new bo_perp_client();
					$client = $bo_client->get_client($order->get_client_id());
					$balance = $bo_client->get_account_balance($order->get_client_id());

					if($client['credit_limit'] != '' && ($order->get_total() + $balance) > $client['credit_limit']) {
						$this->messages[] = array(lang('This order will exceed the credit limit of %1.  The current balance, including this order, is %2.', 
							ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], $client['credit_limit']),
							ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $supplier['client_currency'], ($order->get_total() + $balance))
						), 'warning');
					}
				}
			}

			// Disabled variables
			$data['add_line'] = $order->is_editor($GLOBALS['egw_info']['user']['account_id']) || !$order->get_approval_date();

			if($this->debug) {
				echo 'perp_api.ui_perp_order display ' . get_class($order) . ' data: ';
				_debug_array($data);
				echo 'Errors so far: ';
				_debug_array($this->messages);
			}

			// This variable set after the debug, because it's pretty big
			$data['nextmatch']['order']	=&	$order;

			return $data;
		}

		/**
		*	Get selectbox arrays
		*/
		protected function get_selectbox_options(Order &$order) {
			$options = array();
			$our_contacts = ExecMethod2('addressbook.addressbook_bo.search', '', true, '', '', false, '', '', false, array('owner' => 0));
			foreach($our_contacts as $contact) {
				$options['our_contact_person'][$contact['id']] = egw_link::title('addressbook', $contact['id']);
			}
			$options['order_currency']	=	ExecMethod('perp_api.bo_perp_api.get_currencies');
			$options['payment_terms']	=	ExecMethod('perp_api.bo_perp_api.get_payment_terms');
	
			// Get list of who's allowed to approve this Order
			$options['approving_user'] = $order->get_admin_list();

			return $options;
		}

		/**
		*	Get readonlys
		*	Get an array of widgets to set to readonly because of Order status or permissions
		*/
		protected function get_readonlys(Order &$order) {
			$edit = $order->is_editor($GLOBALS['egw_info']['user']['account_id']) && !$order->is_complete();
			$admin = $order->is_admin($GLOBALS['egw_info']['user']['account_id']) && !$order->is_complete();

			$readonly = array(
				'approving_user'	=>	(!$admin || $order->get_approval_date()),
				'approve'		=>	!$admin || ($order->approval_check === true) || (boolean)$order->get_approval_date(),
				'deny'			=>	!$admin || (boolean)$order->get_approval_date(),
				'save'			=>	!($edit || $admin),

				'supplier_id'		=>	array('search' => !$edit || $order->get_supplier_id()),
				'client_id'		=>	array('search' => !$edit || $order->get_client_id()),
				'their_reference_number'	=>	!$edit,
				'their_contact_person'	=>	!$edit,

				'our_reference_number'	=>	!$edit || (boolean)$order->get_approval_date(),
				'our_contact_person'	=>	!$edit,
				'ship_to_address'	=>	!$edit || (boolean)$order->get_approval_date(),

				'issue_date'		=>	!$edit,
				'requested_delivery_date'	=>	!$edit,
				'order_currency'	=>	!$edit || (boolean)$order->get_approval_date(),
				'payment_terms'		=>	!$edit || (boolean)$order->get_approval_date(),

				'comments'		=>	!$edit,
			);
			return $readonly;
		}

		/**
		*	Set up javascript and ajax
		*	@param ajax_callback appname.ajax_object Object to use for ajax callbacks
		*/
		protected function get_javascript($ajax_callback) {
			// xajax
			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->set_onload(" ajax_perp_order = '$ajax_callback'; ");
			$GLOBALS['egw']->js->set_onload('order_edit_setup(); ');
			$GLOBALS['egw']->js->validate_file('order', 'edit', 'perp_api');
			$GLOBALS['egw']->js->validate_file('order', 'edit', $GLOBALS['egw_info']['flags']['currentapp']);
		}

		/**
		*	Callback for the nextmatch widget to get the OrderLines
		*
		*	@nextmatch
		*/
		public function get_order_lines(&$query, &$rows, &$readonlys) {
			if(isset($query['order']) && $query['order'] instanceof Order) {
				$order = $query['order'];
			} else {
				$order = $this->get_order('Order');
			}
			$config = config::read($GLOBALS['egw_info']['flags']['currentapp']);
			$lines = array();
			$rows = array();
			$readonlys = array();
			$lines =& $order->get_lines();

			if(!isset($query['start'])) {
				$query['start'] = 0;
			}
			if(!isset($query['num_rows']) || $query['num_rows'] < 0) {
				$query['num_rows'] = count($lines);
			}

			// Save query settings to session
			$GLOBALS['egw']->session->appsession('order_lines', $GLOBALS['egw_info']['flags']['currentapp'], $query);

			/* 
			* Kinda sucks to do this from the session instead of the database...
			* Have to sort by line number to clear the keys and keep everything in the right order,
			* and have the query count get the right records
			*/
			if(count($lines) > 0) {
				foreach($lines as $key => $line) {
					$line_number[$key] = $line->get_line_number();
				}
				array_multisort($line_number, $lines); 
			} else {
				return 0;
			}

			$count = count($lines);
			for($i = $query['start']; count($rows) < $query['num_rows'] && $i < count($lines); $i++) {
				$line =& $lines[$i];
				if(!($line instanceof OrderLine)) {
					continue;
				}
				// Get details
				$details = array();
				$line_details =& $line->get_details();
				foreach($line_details as $detail_key => &$detail) {
					if($detail->get_display()) {
						$detail_array = $detail->toArray();
						if(!$details[get_class($detail)]) {
							// Shift details to allow for title
							$details[get_class($detail)][] = array();
						}
						$details[get_class($detail)][] = $detail_array;
						
						// Readonly
						$readonly = !$detail->can_edit();

						if(is_array($detail_array['readonly'])) {
							foreach($detail_array['readonly'] as $field => $value) {
								$readonlys[ (count($rows) + 1) ]['details'][get_class($detail)][ ($detail_key ) . "[$field]"] = $value;
							}
						} else if($readonly) {
							foreach($detail_array as $field => $value) {
								$readonlys[ (count($rows) + 1) ]['details'][get_class($detail)][ ($detail_key ) . "[$field]"] = $readonly;
							}
						}
					}
				}

				$readonlys['ship[' . (count($rows) + 1) . ']'] = !$line->can_ship();
				$readonlys['receive[' . (count($rows) + 1) . ']'] = !$line->can_receive();

				$row = $line->toArray();
				if(count($details) > 0) {
					$row['details'] = $details;
				}

				// Special check to see if we use a per-line reference
				if($details['ClientReferenceDetail'] || $config['use_client_reference']) {
					$row['details']['ClientReferenceDetail']['use_client_reference'] = true;
				} elseif($row['details']) {
					$row['details']['ClientReferenceDetail']['use_client_reference'] = false;
				}

				// Get General Ledger account
				$row['gl_account'] = $line->get_gl_account();

				// Get history / status information
				$status = $line->get_status();
				if(count($status) > 0) {
					array_unshift($status, false);
					$row['status'] = $status;
				}

				// Make sure all the bases are covered
				$readonlys[count($rows) + 1]['unit_price'] = ($line->get_quantity_invoiced() != 0);
				$readonlys[(count($rows) + 1) . '[unit_price]'] = ($line->get_quantity_invoiced() != 0);
				$readonlys[(count($rows) + 1)]['delete[' . $line->get_line_number() .']'] = ($line->can_cancel() !== true);

				$readonlys['delete[' . $line->get_line_number() .']'] = ($line->can_cancel() !== true);

				if($query['search']) {
					if(strpos($row['supplier_stock_code'], $query['search']) === False && 
					strpos($row['our_stock_code'], $query['search']) === False &&
					strpos($row['description'], $query['search']) === False) {
						unset($row);
						$count--;
						continue;
					}
				}
				// TODO: Figure out how to make the quantity and unit cost readonly

				$row['line_no'] = $line->get_line_number();
				$row['extended_cost'] = sprintf("%.2f", $row['quantity_ordered'] * $row['unit_price']);
				$row['formatted_extended_cost'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $this->currency, $row['extended_cost']);

				// Get unit of measure

				// One of our items, use our UOM
				$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $row['stock_id']);
				$row['our_unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures', $stock_item['unit_of_measure']);

				$catalog_item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $order->get_supplier_id(), $row['supplier_stock_code']);
				if(count($catalog_item) > 0) {
					$row['supplier_quantity'] = $catalog_item['quantity'];
					$row['supplier_unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures', $catalog_item['supplier_uom']);
					if($row['stock_id'] && ($catalog_item['conversion_factor'] * $catalog_item['quantity']) != 1) {
						$row['supplier_unit_of_measure'] = '(' . ($catalog_item['conversion_factor'] * $catalog_item['quantity']) . ' ' . $row['our_unit_of_measure'] . ' / ' . $row['supplier_unit_of_measure'] . ')';
					}
				}


				$rows[] = $row;
			}

			// Permission to remove lines
			$rows['remove_line'] = $order->is_editor($GLOBALS['egw_info']['user']['account_id']);

			// Permission to set the GL account from the Order
			$rows['set_account'] = $order->is_admin($GLOBALS['egw_info']['user']['account_id']);;

			return $count;
		}


		/** 
		*	Pull an Order object from the session, or read from the database
		*	if there isn't one in the session.
		*	@param classname Name of the class of Order we're looking for
		*	@param order_id ID of the order to get (for db)
		*	@return Order object, or null if it can't be found
		*/
		public function get_order($classname = 'Order', $order_id = null) {
			// Make sure the class is defined first
			if(!class_exists($classname)) {
				if($this->debug) {
					echo "$classname is not defined.<br />";
				}
				__autoload($classname);
			} elseif ($this->debug) {
				echo "$classname is defined.<br />";
			}

			// $order pulled from session
			$order = $GLOBALS['egw']->session->appsession('order');

			if($order_id && (!($order instanceof $classname) || $order->get_order_id() != $order_id)) {
				if($this->debug) {
					echo 'Order loaded from database<br />';
				}
				$order = new $classname($order_id);
			} elseif ($this->debug) {
				echo 'Order loaded from session array<br />';
			}
			if($order && !is_object($order)) {
				// Class probably wasn't loaded, so try and load it
				list($garbage, $length, $classname) = explode(':', $order);
				$classname = substr($classname, 1, $length);
				if(!is_object($GLOBALS['perp_api']['bo'])) {
					$GLOBALS['perp_api']['bo'] = CreateObject('perp_api.bo_perp_api');
				}
				$GLOBALS['perp_api']['bo']->perp_autoload($classname);

				// Try the read again
				$order = $GLOBALS['egw']->session->appsession('order');

				if(!is_object($order)) {
					$order = unserialize($order);
				}

				if(!is_object($order)) {
					echo $classname . ' object not restored<br />';
					_debug_array($order);
				}
			}
			if(!($order instanceof $classname)) {
				if($this->debug) {
					echo 'Order in session is wrong type (found ' . get_class($order) . ',  should be ' . $classname . ')<br />';
				}
				$order = null;
			}

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($order->get_supplier_id(), $supplier_ids)) {
				// Supplier trying to create an Order
				if($order->get_supplier_id() == false && $order->get_client_id() == false) {
					// Sure, as long as its for them or a new order
					return $order;
				}
				$order = null;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($order->get_client_id(), $client_ids)) {
				// Client trying to create an Order
				if($order->get_supplier_id() == false && $order->get_client_id() == false) {
					// Sure, as long as its for them or a new order
					return $order;
				}
				$order = null;
			}

			return $order;
		}

		/**
		*	Check the result of a method call and add it into the messages array if needed.
		*	@param result Boolean true or error message
		*	@param type Severity of message
		*/
		protected function check_result($result, $type = 'warning') {
			if($result === true) {
				return true;
			} elseif(strlen(trim($result)) > 0) {
				$this->messages[] = array($result, $type);
				return false;
			}
		}

		/**
		*	Edit the OrderLineDetails on an OrderLine
		*	@param content Array of incoming data from the eTemplate
		*/
		public function edit_line_detail($content = array()) {
			$order =& $this->get_order(reg_var('order_id'));
			$line =& $order->get_line(reg_var('line_no'));

			// Update line
			try {
				$this->update_order_line($line, $content);
			} catch (Exception $e) {
				$this->messages[] = array($e->getMessage(), 'error');
			}

			// Save to session
			$GLOBALS['egw']->session->appsession('order', false, $order);

			// Get data for display
			$data = $this->get_line_detail_display_data($line);

			// Select box options
			$select_box_options = $this->get_line_detail_options($line);

			// Readonlys
			$readonly = $this->get_line_detail_readonlys($line);

			// Javascript
			$this->get_line_detail_javascript('perp_api.ajax_perp_order');

			// Display
			$this->template->exec($this->order_line_detail_edit, $data, $select_box_options, $readonly, $preserve, 2);
		}

		/**
		*	Update a detail line with data returned from the user
		*	@param detail OrderLineDetail to update
		*	@param data Array of data to update the detail with
		*/
		protected function update_line_detail(OrderLineDetail $detail, Array $data) {
			if(isset($data['detail_value'])) {
				$detail->set_value($data['detail_value']);
			}
		}

		/**
		*	Get order list, and preserve options in the session
		*/
		public function get_order_list(&$query, &$rows = array(), &$readonlys = array()) {
			$GLOBALS['egw']->session->appsession($GLOBALS['egw_info']['flags']['currentapp'] . '.nextmatch', false, $query);

			if($query['filter'] != $GLOBALS['egw_info']['user']['preferences'][$GLOBALS['egw_info']['flags']['currentapp']]['order_filter']) {
				$GLOBALS['egw']->preferences->add($GLOBALS['egw_info']['flags']['currentapp'], 'order_filter', $query['filter']);
				$GLOBALS['egw']->preferences->save_repository(true);
			}
			if($query['order'] != $GLOBALS['egw_info']['user']['preferences'][$GLOBALS['egw_info']['flags']['currentapp']]['order_order']) {
				$GLOBALS['egw']->preferences->add($GLOBALS['egw_info']['flags']['currentapp'], 'order_order', $query['order']);
				$GLOBALS['egw']->preferences->save_repository(true);
			}
			return $this->bo_order->get_order_list($query, $rows, $readonlys);
		}
	}
?>
