<?php

	/**
	*	User interface for pERP reports
	*
	*	Reports in pERP are generated using TinyButStrong's OpenOffice.org extension,
	*	which creates OpenOffice.org documents based on template OO.o documents.
	*
	*	This user interface allows a common interface to reporting for all pERP applications.
	*	
	*	@see http://tinybutstrong.com
	*	@ingroup perp_report
	*/
	class ui_perp_report {
		
		/**
		*	Reference to bo_perp_report logic object
		*/
		protected $bo;

		/**
		*	Reference to eTemplate object for display
		*/
		protected $template;

		/**
		*       List of public functions that phpgwapi allows to be
		*       called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'index'		=>	True,
			'get_report'	=>	True,
			'schedule'	=>	true,
		);

		/**
		*       boolean output extra debug information
		*/
		protected $debug = false;

		/**
		*	Constructor\n
		*	Creates the logic object and the template object
		*/
		function ui_perp_report() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_api.bo_perp_report');
		}

		/**
		*	Displays a list of reports the user has access to for the given application
		*	Application should be specified in $_GET['appname']
		*/
		function index($content = array()) {
			if($content['appname']) {
				$app = $content['appname'];
			} else {
				$app = reg_var('appname', 'any', 'alphanumeric', $GLOBALS['egw_info']['flags']['currentapp']);
			}

			// Admin might be changing settings
			if(($content['save'] || $content['nm']['rows']['adhoc'] || $content['nm']['rows']['schedule']) && $GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'admin')) {
				$config = config::read($app);
				foreach($content['nm']['rows'] as $report) {
					$config[$report['report_name']]['can_adhoc'] = $report['can_adhoc'];
					$config[$report['report_name']]['can_schedule'] = $report['can_schedule'];
					config::save_value($report['report_name'], $config[$report['report_name']], $app);
				}
			}
			if($content['nm']['rows']['adhoc']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_api.ui_perp_report.get_report',
					'appname'	=>	$app,
					'report_name'	=>	key($content['nm']['rows']['adhoc']),
				));
			}
			if($content['nm']['rows']['schedule']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_api.ui_perp_report.schedule',
					'appname'	=>	$app,
					'report_name'	=>	key($content['nm']['rows']['schedule'])
				));
			}

			// Not sure why the cancel doesn't come back.  Probably due to the complexity of the async_id.
			if(strpos($_POST['submit_button'], '[cancel][') !== false) {
				$start = strpos($_POST['submit_button'], '[cancel][');
				$async_id = substr($_POST['submit_button'], $start + strlen('[cancel]['), -1);
				ExecMethod('phpgwapi.asyncservice.cancel_timer', $async_id);
			}

			// Get list of reports
			$query = array(
				'appname'	=>	$app
			);
			$GLOBALS['egw']->translation->add_app($appname);

			$data['nm'] = array(
				'no_cat'	=>	true,
				'no_filter'	=>	true,
				'no_filter2'	=>	true,
				'no_search'	=>	true,
				//'no_columnselection'	=>	true,
				'csv_fields'	=>	false,
				'appname'	=>	$app,
				'template'	=>	'perp_api.report_list.row',
				'get_rows'	=>	'perp_api.bo_perp_report.get_report_list'
			);
			$GLOBALS['egw_info']['flags']['currentapp'] = $app;

			if(!$GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'admin')) {
				$readonly['save'] = true;
			}

			$preserve['appname'] = $app;

			// Display
			$this->template->read('perp_api.report_list');
			$this->template->exec('perp_api.ui_perp_report.index', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Generates a report.  If the report requires parameters from the user, a template named
		*	<appname>.report.<report_name> will be shown to get them.  A common header and footer are added to the template.\n
		*	The name of the report should be given in $content['report_name'].
		*
		*	The report will be output, and most browsers should know what to do with it.  Users should have 
		*	OpenOffice.org installed.
		*/
		function get_report($content = array()) {
			if($this->debug) {
				echo 'perp_api.ui_perp_report.get_report incoming data:';
				_debug_array($content);
			}

			$report = reg_var('report_name', 'any', 'alphanumeric', $content['report_name']);
			$datasource = reg_var('data_source', 'any', 'alphanumeric', $content['datasource']);
			list($app_name, $report_name) = explode('.', $report);
			if(!$report_name) {
				$report_name = $report;
				unset($app_name);
			}
			$app_name = reg_var('appname', 'any', 'alphanumeric', $app_name);

			unset($content['report_name']);

			if(!$report) {
				echo lang('No report specified');
				return false;
			} elseif($this->debug) {
				echo "Report: $report<br />";
			}
			$data = array();

			// Look to see if data was provided
			if(count($content) > 1 && !isset($content['datasource'])) {
				$data = $content;
			} elseif ($datasource) {
				unset($content['datasource']);
				$data = ExecMethod($datasource, $data);
			}
			if($content['cancel']) {
				// Redirect
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_api.ui_perp_report.index',
					'appname'	=>	$app_name
				));
			}

			// Check for an etemplate, we may need some user parameters
			$etemplate = $app_name . '.report.' . $report_name;
			if(ExecMethod('etemplate.boetemplate.read', $etemplate)) {
				$this->template->read('perp_api.report.parameters');
				$found_template = true;
			}

			// Load language for the app
			$GLOBALS['egw']->translation->add_app($app_name);

			// Load the report
			$perp_report = $this->bo->get_report($app_name . '.' . $report_name, $content, $data);

			// Output
			if(is_object($perp_report) && $found_template && !$content['generate_report']) {
				// Get parameters
				$preserve['report_name'] = $report;
				if($datasource) {
					$preserve['datasource'] = $datasource;
				}
				$data = $perp_report->get_parameters();
				$data['etemplate'] = $etemplate;
				$GLOBALS['egw_info']['flags']['app_header'] = lang('report_' . $report_name);
				$this->template->exec('perp_api.ui_perp_report.get_report', $data, $select_box_options, $readonly, $preserve);
			} elseif(is_object($perp_report) && $perp_report) {
				// output report
				unset($content['generate_report']);
				$result = $perp_report->display();
				if($result === false) {
					echo lang('Couldn\'t generate report %1', $report);
				} 
			} else {
				echo $perp_report;
			}
		}

		/**
		*	User interface for scheduling a report for generation later
		*/
		public function schedule($content = array()) {
			$appname = get_var('appname', 'any', $content['appname']);
			$report_name = get_var('report_name', 'any', $content['report_name']);
			$async_id = get_var('async_id', 'any', $content['async_id']);

			if($content['save']) {
				if($content['schedule_type'] == 'asap') {
					// Schedule for the next async run
					$installed = ExecMethod('phpgwapi.asyncservice.installed');
					$next_run = ExecMethod('phpgwapi.asyncservice.next_run', $installed);
					if($next_run == 0) {
						// Using fallback method.  Set it for 5 seconds in the future, or it will be rejected
						$next_run = time() + 5;
					}
					$content['schedule_time'] = $next_run;
				} elseif ($content['schedule_type'] == 'once') {
					$content['schedule_time'] = $content['schedule_once'];
				} elseif ($content['schedule_type'] == 'repeat') {
					foreach($content['schedule_time'] as $key => &$value) {
						if($value === '') {
							$value = '*';
						}
					}
				}
				if($async_id) {
					$content['settings']['async_id'] = $async_id;
				}
				$async_id = $this->bo->schedule_report(
					$appname,
					$report_name, 
					$content['schedule_time'], 
					$content['distribution'], 
					$content['settings'], 
					$GLOBALS['egw_info']['user']['account_id']
				);

				// Go back to the list when done
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_api.ui_perp_report.index',
					'appname'	=>	$appname
				));

			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_api.ui_perp_report.index',
					'appname'	=>	$appname
				));
			}

			$data = $content ? $content : array(
				'distribution'	=>	array(),
				'settings'	=>	array(
					'output_format'	=>	$GLOBALS['egw_info']['user']['preferences']['perp_api']['report_output_format']
				),
			);

			if($async_id) {
				list($appname, $report_name, $owner) = explode('.', $async_id);
				$schedules = ExecMethod('phpgwapi.asyncservice.read', $async_id);
				$schedule = current($schedules);

				$data['schedule_time'] = $schedule['times'];
				if(is_numeric($schedule['times'])) {
					$data['schedule_type'] = 'once';
					$data['schedule_once'] = $data['schedule_time'];
				} elseif(is_array($schedule['times'])) {
					$data['schedule_type'] = 'repeat';
					foreach($data['schedule_time'] as $interval => &$value) {
						if(is_array($value)) {
							$value = implode(',', array_keys($value));
						}
					}
				}
				$data['distribution'] = $schedule['data']['distribution'];
				$data['settings'] = $schedule['data']['settings'];

				// Schedule at the same time
				if($owner != $GLOBALS['egw_info']['user']['account_id']) {
					$owner = $GLOBALS['egw']->accounts->read($owner);
					$this->messages[] = array(lang('Someone is already getting this report.  Click save to get the same report at the same time, or 
						make any changes you wish to get your own version.'), 'info'
					);
					unset($async_id);
				}
			}

			if(strpos($report_name, 'report_') === 0) {
				$report_name = substr($report_name, 7);
			}
			$report_template = "$appname.report.$report_name";
			$schedule_template = $appname . '.schedule.' . $report_name;
			$class_file = EGW_INCLUDE_ROOT . '/' . $appname . '/inc/report/class.' . $report_name . '.inc.php';
			if(file_exists($class_file)) {
				include_once($class_file);
				$report = new $report_name($data['settings']);
				$data['settings'] = $report->get_parameters();
			}
			if(ExecMethod('etemplate.boetemplate.read', $schedule_template)) {
				$data['schedule_template'] = $schedule_template;
				if($report) {
					$data['settings'] = array_merge($data['settings'], $report->get_schedule_settings($async_id));
				}
			} elseif(ExecMethod('etemplate.boetemplate.read', $report_template)) {
				$data['report_template'] = $report_template;
				if($report) {
					$data['settings'] = array_merge($data['settings'], $report->get_parameters());
				}
			}

			$select_box_options['schedule_type'] = array(
				'asap'		=>	'as soon as possible',
				'once'		=>	'once at a certain time',
				'repeat'	=>	'at a regular time'
			);
			$select_box_options['type'] = array();
			if($GLOBALS['egw_info']['applications']['filemanager']) {
				$select_box_options['type']['filemanager'] = 'save it';
			}
			$select_box_options['type']['email'] = 'email it';
			$select_box_options['output_format'] = ExecMethod('perp_api.bo_perp_report.get_supported_formats');

			$preserve['appname'] = $appname;
			$preserve['async_id'] = $async_id;
			$preserve['report_name'] = $report_name;
			$preserve['schedule_time']['year'] = '*';

			// Javascript
			$GLOBALS['egw']->js->validate_file('report', 'schedule', 'perp_api');
			$GLOBALS['egw']->js->set_onload('change_schedule_type();');

			// Messages
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw']->translation->add_app($appname);
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Schedule %1', $report_name);
			$this->template->read('perp_api.report.schedule');
			$this->template->exec('perp_api.ui_perp_report.schedule', $data, $select_box_options, $readonly, $preserve);
		}
	}
?>
