<?php

	/**
	*	User interface for Shipments
	*
	*	Provides common UI functions for dealing with Shipments
	*
	*	@ingroup perp_api
	*	@author Nathan Gray
	*/

	require_once('class.Shipment.inc.php');
	require_once('class.bo_perp_shipment.inc.php');

	class ui_perp_shipment {
		
		public $public_functions = array(
			'shipment_list'	=>	true,
			'new_shipment'	=>	true,
		);

		/**
		*	boolean Enable for debug information
		*/
		protected $debug = false;

		/**
		*	Array of messages to the user.
		*	@see perp_api::ui_perp_api::make_message()
		*/
		protected $messages = array();

		/**
		*	eTemplate used for display
		*/
		protected $template;

		/**
		*	Name of eTemplate used for displaying row lines
		*/
		protected $shipment_line_template;

		/**
		*	Name of eTemplate used for displaying a list of Shipments
		*/
		protected $shipment_list_template;
		protected $shipment_list_row_template;


		/**
		*	Configuration options for Shipments
		*/
		protected $config;

		/**
		*	Logic layer object
		*/
		protected $bo_shipment;

		/**
		*	Constructor
		*	Initializes the helper objects
		*/
		public function __construct() {
			// I don't know where else to put this
			if(!$GLOBALS['perp_api']['ui']) {
				require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_api.inc.php');
				$GLOBALS['perp_api']['ui'] =& new ui_perp_api();
			}

			$this->template = CreateObject('etemplate.etemplate');
			$this->config = config::read('perp_api');

			$this->bo_shipment = new bo_perp_shipment();
		}

		/**
		*	Display a list of Shipments
		*	
		*
		*	@param content Array of content returned from the eTemplate
		*/
		public function shipment_list($content = array()) {
		}

		/**
		*	Handle incoming data from the eTemplate for a list of shipments
		*	@param content Array of content returned from the eTemplate
		*/
		protected function list_update(&$content) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}
			if(isset($content['new'])) {
				$result = $this->handle_shipment_action('new', null);
				if($result) {
					unset($content['new']);
				}
			}

			if(!isset($content['nextmatch']['rows'])) {
				return false;
			}
			foreach($content['nextmatch']['rows'] as $action => $row) {
				$shipment_id = key($row);
				$result = $this->handle_shipment_action($action, $shipment_id);
				if($result) {
					unset($content['nextmatch']['rows'][$action]);
				}
			}
		}

		/**
		*	Get data for displaying a list of Shipments
		*/
		protected function list_get_display_data() {
			$data = array();
			$data['nextmatch'] = array(
				'no_cat'	=>	true,
				'filter_label'		=>	'Status',
				'filter_no_lang'	=>	true,
				'no_filter2'		=>	true,
				'get_rows'		=>	$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this->bo_shipment) . '.get_shipment_list',
				'template'		=>	$this->shipment_list_row_template,
			);

			if(reg_var('order_id')) {
				$data['nextmatch']['order_id']	=	reg_var('order_id');
			}

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			return $data;
		}

		/**
		*	Get selectbox options for displaying a list of Shipments
		*/
		protected function list_get_options() {
			$options = array();
			$options['filter'] = $this->bo_shipment->get_status_list();
			$options['shipper'] = ExecMethod('perp_api.bo_perp_api.get_shippers');
			return $options;
		}

		/**
		*	Get read only array for displaying a list of Shipments
		*/
		protected function list_get_readonlys() {
			$readonly = array();

			$add = $GLOBALS['egw']->acl->check('shipment', EGW_ACL_ADD, $GLOBALS['egw_info']['flags']['current_app']);
			$edit = $GLOBALS['egw']->acl->check('shipment', EGW_ACL_EDIT, $GLOBALS['egw_info']['flags']['current_app']);
			$approve = $GLOBALS['egw']->acl->check('shipment', EGW_ACL_CUSTOM1, $GLOBALS['egw_info']['flags']['current_app']);
			$cancel = $GLOBALS['egw']->acl->check('shipment', EGW_ACL_DELETE, $GLOBALS['egw_info']['flags']['current_app']);
			$receive = $GLOBALS['egw']->acl->check('receive', EGW_ACL_READ, $GLOBALS['egw_info']['flags']['current_app']);

			$readonly['new'] = !$add;

			return $readonly;
		}

		/**
		*	Handle an action on an Shipment
		*
		*	Actions can be one of new, edit, ship, receive,
		*	invoice, approve, deny, cancel
		*
		*	@param action	Action to take
		*	@param shipment_id	ID of the Shipment to act on
		*/
		protected function handle_shipment_action($action, $shipment_id) {
			switch($action) {
				case 'new':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=> $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.new_shipment',
					));
					return true;
					break;
				case 'edit':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=> $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.edit',
						'shipment_id'	=> $shipment_id
					));
					return true;
					break;
				case 'approve':
					$shipment = new Shipment($shipment_id);
					return $shipment->approve($GLOBALS['egw_info']['user']['account_id']);
					break;
				case 'deny':
					$shipment = new Shipment($shipment_id);
					try {
						return $shipment->deny($GLOBALS['egw_info']['user']['account_id']);
					} catch (Exception $e) {
						$this->messages[] = array($e->getMessage(), 'error');
						return false;
					}
					break;
				case 'cancel':
					$shipment = new Shipment($shipment_id);
					return $shipment->cancel($GLOBALS['egw_info']['user']['account_id']);
					break;
				default:
					// Don't know how to handle this
					return false;
			}
		}
		
		/**
		*	Update the Shipment with the given data.
		*	Removes lines too.
		*	
		*	@param shipment	Shipment to update
		*	@param content	array with key => value mapping of new values
		*	
		*	@return boolean Successfully updated
		*/
		protected function update_shipment(Shipment &$shipment, &$content) {
			if($this->debug) {
				echo 'perp_api.ui_perp_shipment Update ' . get_class($shipment) . ': ';
				echo '<pre>' . $shipment->__toString() . '</pre>';
				echo 'New content: ';
				_debug_array($content);
			}
			$edit = $shipment->is_editor($GLOBALS['egw_info']['user']['account_id']);
			if($edit !== true) {
				$this->messages[] = array($edit, 'error');
				return false;
			}

			$result = true;

			// Parties involved (suppliers, clients)
			if(isset($content['supplier_id'])) {
				$result = $result && $this->check_result( $shipment->set_supplier_id((int)$content['supplier_id']));
			}
			if(isset($content['client_id'])) {
				$result = $result && $this->check_result( $shipment->set_client_id((int)$content['client_id']));
			}

			// Reference numbers
			if(isset($content['our_reference_number'])) {
				$shipment->set_our_reference_number($content['our_reference_number']);
			}
			if(isset($content['their_reference_number'])) {
				$shipment->set_their_reference_number($content['their_reference_number']);
			}

			// Contacts
			if(isset($content['our_contact_person'])) {
				$result = $result && $this->check_result($shipment->set_our_contact_person((int)$content['our_contact_person']));
			}
			if(isset($content['their_contact_person'])) {
				$result = $result && $this->check_result( $shipment->set_their_contact_person((int)$content['their_contact_person']));
			}

			// Dates
			if(isset($content['issue_date'])) {
				$result = $result && $this->check_result( $shipment->set_issue_date((int)$content['issue_date']));
			}
			if(isset($content['requested_delivery_date'])) {
				$result = $result && $this->check_result($shipment->set_requested_delivery_date((int)$content['requested_delivery_date']));
			}

			// Locations
			if(isset($content['ship_from_address'])) {
				$result = $result && $this->check_result( $shipment->set_ship_from_address((int)$content['ship_from_address']));
			}
			if(isset($content['ship_to_address'])) {
				$result = $result && $this->check_result($shipment->set_ship_to_address((int)$content['ship_to_address']));
			}

			// Shipment
			if(isset($content['shipper'])) {
				$result = $result && $this->check_result($shipment->set_shipper_id((int)$content['shipper']));
			}
			if(isset($content['vessel'])) {
				$result = $result && $this->check_result($shipment->set_vessel($content['vessel']));
			}

			// Comments
			if(isset($content['comments'])) {
				$shipment->set_comments($content['comments']);
			}

			if(isset($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as $key => &$line_data) {
					if(isset($line_data['line_no'])) {
						$shipment_line =& $shipment->get_line($line_data['line_no']);
						if($shipment_line) {
							$line_result = $this->update_line($shipment_line, $line_data);
							$result = $result && $line_result;
						}
					}
				}
			}

			// Add line
			if(isset($content['add_line']) && isset($content['new'])) {
				$this->add_line($shipment, $content['new']);
			}

			// Remove lines
			if(isset($content['nextmatch']['rows']['delete'])) {
				$this->remove_line($shipment, key($content['nextmatch']['rows']['delete']));
			}

			// Approval
			if(isset($content['approve']) && $shipment->is_admin($GLOBALS['egw_info']['user']['account_id']) && !$shipment->get_approval_date()) {
				$result = $result && $this->check_result($shipment->approve($GLOBALS['egw_info']['user']['account_id']));
			}

			// Ship / Receive
			if(isset($content['ship'])) {
				$this->handle_shipment_action('ship', $shipment->get_shipment_id());
			}
			if(isset($content['receive'])) {
				$this->handle_shipment_action('receive', $shipment->get_shipment_id());
			}

			// Save
			if(isset($content['save'])) {
				$test = $this->check_result($shipment->check(), 'error');
	
				if($test === true) {
					$shipment->write();
					$GLOBALS['egw']->session->appsession('shipment', false, null);
					if($shipment->get_shipment_id()) {
						$this->messages[] = array(lang('Shipment saved'), 'success');
					}
				} 
			}
			if($this->debug) {
				echo 'perp_api.ui_perp_shipment After updating ' . get_class($shipment) . ': ';
				echo '<pre>' . $shipment->__toString() . '</pre>';
				echo 'Errors so far: ';
				_debug_array($this->messages);
			}
			return ($result == true);
		}

		/**
		*	Add a new ShipmentLine to the provided Shipment
		*	@param shipment Shipment to add the line to
		*	@param line_data array of line data for the new line
		*/
		protected function add_line(Shipment &$shipment, &$line_data) {
			$this->messages[] = array('Developer should re-implement add_line', 'error');
		}

		/**
		*	Remove an ShipmentLine from the provided Shipment
		*	@param shipment Shipment to remove the line from
		*	@param line_no Line number to remove
		*/
		protected function remove_line(Shipment &$shipment, &$line_no) {
			return $this->check_result($shipment->remove_line($line_no));
		}

		/**
		*	Update an ShipmentLine with the provided line data
		*
		*	@param shipment_line ShipmentLine to update
		*	@param line_data array of line data
		*/
		protected function update_line(ShipmentLine &$shipment_line, &$line_data) {
			$result = true;

			if(isset($line_data['quantity_shipped'])) {
				$result &= $this->check_result($shipment_line->set_quantity_shipped($line_data['quantity_shipped']));
			}
			if(isset($line_data['unit_price'])) {
				$result &= $this->check_result($shipment_line->set_unit_price($line_data['unit_price']));
			}
			if(isset($line_data['details'])) {
				$result &= $this->check_result($shipment_line->set_details($line_data['details']));
			}

			return $result;
		}

		/**
		*	Get an array of data suitable for displaying with an eTemplate
		*/
		protected function get_display_data(Shipment &$shipment) {
			$data = array();
			$data += $shipment->toArray(false);

			// Addresses
			if($data['ship_from_address']) {
			//	$data['ship_from_address']	=	ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_from_address());
				if($data['ship_from_address'] === false) {
					$this->messages[] = array(lang('Permission on shipping address for %1 denied', lang('ship from')), 'info');
				}
			}
			if($data['ship_to_address']) {
			//	$data['ship_to_address']	=	ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_to_address());
				if($data['ship_to_address'] === false) {
					$this->messages[] = array(lang('Permission on shipping address for %1 denied', lang('ship to')), 'info');
				}
			}

			// Set up nextmatch widget
			$data['nextmatch'] = array(
				'shipment_id'	=>	$shipment->get_shipment_id(),
				'supplier_id'	=>	$data['supplier_id'],
				'client_id'	=>	$data['client_id'],
				'no_filter'	=>      True,
				'filter_label'	=>      'Status',
				'filter_no_lang'	=>      True,
				'filter2_label' =>      'Group By',
			//	'filter2_no_lang'       =>      True, 
				'get_rows'      =>      $GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.get_shipment_lines',
				'template'      =>      $this->shipment_line_template
			);

			// Set up add line select box
			$data['new']['supplier_stock_code']['supplier_id'] = (string)$shipment->get_supplier_id();

			// Save / Next button
			$data['save'] = ($shipment->get_shipment_id() ? lang('Save') : lang('Next'));

			// Disabled variables
			$data['add_line'] = $shipment->is_editor($GLOBALS['egw_info']['user']['account_id']) || !$shipment->get_approval_date();

			if($this->debug) {
				echo 'perp_api.ui_perp_shipment display ' . get_class($shipment) . ' data: ';
				_debug_array($data);
				echo 'Errors so far: ';
				_debug_array($this->messages);
			}

			// This variable set after the debug, because it's pretty big
			$data['nextmatch']['shipment']	=	$shipment;

			return $data;
		}

		/**
		*	Get selectbox arrays
		*/
		protected function get_selectbox_options(Shipment &$shipment) {
			$options = array();

			// Nextmatch group by
			$options['filter2'] = array(
				'none'		=>	'None',
				'order'		=>	'Order',
				'stock_category'=>	'Stock Category',
				'stock_code'	=>	'Stock Code'
			);
			$options['shipper'] = ExecMethod('perp_api.bo_perp_api.get_shippers');

			$our_contacts = ExecMethod2('addressbook.addressbook_bo.search', '', true, '', '', false, '', '', false, array('owner' => 0));
			foreach($our_contacts as $contact) {
				$options['our_contact_person'][$contact['id']] = egw_link::title('addressbook', $contact['id']);
			}
	
			// Get list of who's allowed to approve this Shipment
			$options['approving_user'] = $shipment->get_admin_list();

			return $options;
		}

		/**
		*	Get readonlys
		*	Get an array of widgets to set to readonly because of Shipment status or permissions
		*/
		protected function get_readonlys(Shipment &$shipment) {
			$shipped = (boolean)$shipment->get_actual_delivery_date();
			$edit = !$shipped && $shipment->is_editor($GLOBALS['egw_info']['user']['account_id']);
			$admin = $shipment->is_admin($GLOBALS['egw_info']['user']['account_id']);

			// Allow admin to edit shipment if it's not fully received
			$not_received = false;
			foreach($shipment->get_lines() as $line) {
				$not_received |= $line->can_receive();
			}
			$admin &= $not_received;

			$readonly = array(
				'approving_user'	=>	(!$admin || $shipment->get_approval_date()),
				'approve'		=>	!$admin || ($shipment->approval_check === true) || (boolean)$shipment->get_approval_date(),
				'deny'			=>	!$admin || (boolean)$shipment->get_approval_date(),
				'save'			=>	!($edit || $admin),
				'add'			=>	!($edit || $admin),

				'supplier_id'		=>	array('search' => (!$edit || $shipment->get_supplier_id())),
				'client_id'		=>	array('search' => (!$edit || $shipment->get_client_id())),
				'their_reference_number'	=>	!$edit,
				'their_contact_person'	=>	!$edit,

				'our_reference_number'	=>	!$edit || (boolean)$shipment->get_approval_date(),
				'our_contact_person'	=>	!$edit,
				'ship_to_address'	=>	!$edit || (boolean)$shipment->get_approval_date(),
				'ship_from_address'	=>	!$edit || (boolean)$shipment->get_approval_date(),

				'issue_date'		=>	!$edit,
				'requested_delivery_date'	=>	!$edit || (boolean)$shipment->get_actual_delivery_date(),

				'comments'		=>	!$edit,
				'shipper'		=>	!$edit,
				'pack'			=>	!$edit,
				'receive'		=>	!($edit && (boolean)$shipment->can_receive()),
				'ship'			=>	!($edit && $shipment->can_ship() === true),
			);

			/**
			*	Hide Add button until the shipment has been saved.
			*	Otherwise, the add dialog will show items already on
			*	this this shipment.
			*/
			$readonly['add'] |= !(boolean)$shipment->get_shipment_id();

			return $readonly;
		}

		/**
		*	Set up javascript and ajax
		*	@param ajax_callback appname.ajax_object Object to use for ajax callbacks
		*/
		protected function get_javascript($ajax_callback) {
			// xajax
			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->set_onload(" ajax_perp_shipment = '$ajax_callback'; ");
			$GLOBALS['egw']->js->set_onload('shipment_edit_setup(); ');
			$GLOBALS['egw']->js->validate_file('shipment', 'edit', 'perp_api');
			$GLOBALS['egw']->js->validate_file('shipment', 'edit', $GLOBALS['egw_info']['flags']['currentapp']);
		}

		/**
		*       Internal function to get a Shipment object to work with.
		*       First, we check the session for a shipment in progress.  Then we
		*       check the provided shipment ID an read from the database, if required.
		*	Extending classes should override this function to create the correct
		*	Shipment type.
		*/
		protected function &get_shipment($shipment_id, $class = 'Shipment') {
			$s = $GLOBALS['egw']->session->appsession('shipment');
			if(
				($shipment_id && $s instanceof $class && $shipment_id != $s->get_shipment_id()) ||
				$shipment_id && !($s instanceof $class)
			) {
				$shipment = new $class($shipment_id);
			} elseif ($s) {
				$shipment = $s;
			} else {
				$shipment = null;
			}

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($shipment->get_supplier_id(), $supplier_ids)) {
				$shipment = null;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($shipment->get_client_id(), $client_ids)) {
				$shipment = null;
			}

			return $shipment;
		}

		/**
		*	Callback for the nextmatch widget to get the ShipmentLines
		*
		*	@nextmatch
		*/
		public function get_shipment_lines(&$query, &$rows, &$readonlys) {
			if(isset($query['shipment']) && $query['shipment'] instanceof Shipment) {
				$shipment = $query['shipment'];
			} elseif ($shipment = $GLOBALS['egw']->session->appsession('shipment')) { 
				// $shipment pulled from session
			} else {
				return 0;
			}
			$lines = $shipment->get_lines();
			$rows = array();
			$readonlys = array();

			if(!isset($query['start'])) {
				$query['start'] = 0;
			}
			if(!isset($query['num_rows']) || $query['num_rows'] < 0) {
				$query['num_rows'] = count($lines);
			}

			// Save query settings to session
			$GLOBALS['egw']->session->appsession('shipment_lines', $GLOBALS['egw_info']['flags']['currentapp'], $query);

			/*
			* Kinda sucks to do this from the session instead of the database...
			* Have to sort by line number to clear the keys and keep everything in the right order,
			* and have the query count get the right records
			*/
			if(count($lines) > 0) {
				foreach($lines as $key => $line) {
					$line_number[$key] = $line->get_line_number();
				}
				array_multisort($line_number, $lines);
			} else {
				return 0;
			}

			$count = count($lines);
			$total = 0;
			$currency = '';
			$delete = $query['filter2'] == 'none' || !$query['filter2'];

			for($i = $query['start']; count($rows) < $query['num_rows'] && $i < count($lines); $i++) {
				$line =& $lines[$i];

				if(!($line instanceof ShipmentLine)) {
					$count--;
					continue;
				}
				if(is_null($line->get_shippable_item())) {
					$count--;
					continue;
				}
				if($query['search']) {
					if(strpos($line->get_shippable_item()->get_supplier_stock_code(), $query['search']) !== False ||
						strpos($line->get_shippable_item()->get_our_stock_code(), $query['search']) !== False ||
					strpos($line->get_shippable_item()->get_description(), $query['search']) !== False) {
						$count--;
						continue;
					}
				}
				if($query['cat_id'] && $line->get_shippable_item()->get_stock_id()) {
					$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $line->get_shippable_item()->get_stock_id());
					if($stock['stock_category'] != $query['cat_id']) {
						$count--;
						continue;
					}
				}
				$row = $line->toArray();
				$row += $line->get_shippable_item()->toArray();
				$row['line_no'] = $line->get_line_number();

				$readonlys['delete[' . $line->get_line_number() . ']'] = !$line->can_cancel();
				$delete |= $line->can_cancel();
				$readonlys[($i + 1) . '[quantity_shipped]'] = !$line->get_owner()->get_actual_delivery_date() && $line->get_quantity_received() == $line->get_quantity_shipped();
				$row['value'] = $row['unit_price'] * $row['quantity_shipped'];
				$total += $row['value'];
				$row['our_reference_number'] = $line->get_shippable_item()->get_owner()->get_our_reference_number();
				$row['currency'] = $line->get_shippable_item()->get_owner()->get_order_currency();
				$currency = $row['currency'];
				$row['formatted_value'] = $GLOBALS['perp_api']['bo']->get_formatted_price($row['currency'], $row['value']);
				// TODO: Figure out how to make the quantity and unit cost readonly


				// Get unit of measure
				$catalog_item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $shipment->get_supplier_id(), $row['supplier_stock_code']);
				$row['supplier_quantity'] = $catalog_item['quantity'];
				$row['supplier_unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures', $catalog_item['supplier_uom']);
				if($row['stock_id'] && ($catalog_item['conversion_factor'] * $catalog_item['quantity']) != 1) {
					// One of our items, use our UOM
					$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $row['stock_id']);
					$row['our_unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures', $stock_item['unit_of_measure']);
					$row['supplier_unit_of_measure'] = '(' . ($catalog_item['conversion_factor'] * $catalog_item['quantity']) . ' ' . $row['our_unit_of_measure'] . ' / ' . $row['supplier_unit_of_measure'] . ')';
				}

				$rows[] = $row;
			}

			$query['count'] = $count;

			$this->group_shipment_lines($query, $rows, $readonlys);
			$rows['total'] = $total;
			$rows['formatted_total'] = $GLOBALS['perp_api']['bo']->get_formatted_price($currency, $total);
			$rows['delete'] = $shipment->is_editor($GLOBALS['egw_info']['user']['account_id']) && $delete;
			return $query['count'];
		}

		/**
		*	Group the shipment lines according to the query parameter
		*	query['filter2'].  
		*/
		protected function group_shipment_lines(&$query, &$lines, &$readonlys) {
			if(!$query['filter2'] || $query['filter2'] == 'none') {
				return;
			} elseif(count($lines) == 0) {
				return;
			}

			$group_id = null;
			$group_count = -1;
			$groups = array();

			foreach($lines as &$line) {
				if(!is_array($line)) {
					continue;
				}
				if($query['filter2'] == 'order') {
					$new_group_id = $line['our_reference_number'];
					unset($line['stock_id']);
					unset($line['supplier_stock_code']);
					unset($line['our_stock_code']);
					unset($line['description']);
				} elseif($query['filter2'] == 'stock_category') {
					$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $line['stock_id']);
					$new_group_id = $stock['stock_category'];
					$stock_category = ExecMethod('perp_inventory.bo_perp_inventory.get_stock_category', $new_group_id);
					unset($line['stock_id']);
					unset($line['supplier_stock_code']);
					unset($line['our_stock_code']);
					unset($line['our_reference_number']);
					$line['description'] = $stock_category[0]['name'];
					$line['stock_category'] = $new_group_id;
				} elseif ($query['filter2'] == 'stock_code') {
					$new_group_id = $line['stock_id'];
					$line['our_reference_number'] = '{' . $line['our_stock_code'] . '}';
				}
				if($new_group_id != $group_id) {
					$group_count++;
					$group_id = $new_group_id;
					$groups[$group_id] = array();
				}
				$groups[$group_id]['quantity_shipped'] += $line['quantity_shipped'];
				$groups[$group_id]['quantity_ordered'] += $line['quantity_ordered'];
				$groups[$group_id]['value'] += $line['value'];
				$groups[$group_id] += $line;
			}
			$lines = array();
			foreach($groups as $group_id => &$group) {
				$group['formatted_value'] = $GLOBALS['perp_api']['bo']->get_formatted_price($group['currency'], $group['value']);
				$lines[] = $group;
			}
		}

		/**
		*	Check the result of a method call and add it into the messages array if needed.
		*	@param result Boolean true or error message
		*	@param type Severity of message
		*/
		protected function check_result($result, $type = 'warning') {
			if($result === true) {
				return true;
			} elseif(strlen(trim($result)) > 0) {
				$this->messages[] = array($result, $type);
				return false;
			}
		}
	}
?>
