<?php
	require_once('class.exchange_datasource.inc.php');

	/**
	*	Aggregate currency exchange rate datasource
	*
	*	Using all the other datasources, calculate the mean (average) exchange rate.  
	*	Do not use a currency if the datasource could not find it.
	*
	*	@see exchange_datasource
	*	@ingroup exchange_update
	*/
	class aggregate extends exchange_datasource {
	
		/**
		*	Array containing an array of all exchange rates for each currency
		*/
		protected $aggregate = array();

		/**
		*	Local list of datasources, we need to modify it and remove this class
		*/
		protected $datasource_list = array();

		/**
		*	Reference to datasource logic class to use bo_perp_exchange_update::create_datasource(),
		*	amoung other functions
		*/
		protected $bo_datasource;

		public function aggregate() {
			parent::__construct();

			// Get us a bo_datasource object to get the list from
			$this->bo_datasource =& new bo_perp_exchange_update();

			// Get the datasource list, and remove this class from it
			$this->datasource_list = $this->bo_datasource->get_datasource_list();
			unset($this->datasource_list[get_class($this)]);
		}

		/**
		*	Update all currency exchange rates by aggregating from all the other data sources
		*	If a datasource does not return a currency, don't use that datasource in the calculations
		*	for that currency.  That datasource will still be used for other currencies
		*/
		public function update_all() {
			foreach($this->datasource_list as $class_name => &$datasource) {
				$datasource = array();
				$datasource['class'] = $this->bo_datasource->get_datasource_class($class_name);
				if($datasource['class'] instanceof exchange_datasource) {
					$datasource['results'] = $datasource['class']->update_all();
				}
				$this->aggregate_data($datasource['results']);
			}

			// Calculate mean - Change the math here for a different calculation such as median or mode
			$results = array();
			foreach ($this->aggregate as $currency_id => $data) {
				$results[$currency_id] = round(array_sum($data) / count($data), 3);
			}
			return $results;
		}

		/**
		*	@param currency_id string Three letter currency ID
		*	@return float New exchange rate
		*/
		public function update($currency_id) {
			if($currency_id == $this->gl_currency) {
				return 1;
			}

			foreach($this->datasource_list as $class_name => &$datasource) {
				$datasource = array();
				$datasource['class'] = $this->bo_datasource->get_datasource_class($class_name);
				if($datasource['class'] instanceof exchange_datasource) {
					$rate = $datasource['class']->update($currency_id);
					$this->aggregate_data(array($currency_id => $rate));
				}
			}

			// Calculate mean - Change the math here for a different calculation such as median or mode
			$new_rate = round(array_sum($this->aggregate[$currency_id]) / count($this->aggregate[$currency_id]), 3);
			if($new_rate > 0) {
				return $this->update_currency($currency_id, $new_rate);
			}

			return false;
		}

		/**
		*	Add the results to the total set, average will be calulated elsewhere
		*	This is just collecting the raw data together.
		*	Classes extending this one will be able to access this->aggregate for other 
		*	calculations.
		*
		*	@param	results	array Results from another datasource
		*/
		protected function aggregate_data($results) {
			foreach($results as $currency_id => $rate) {
				if(!$rate) {
					continue;
				}
				$this->aggregate[$currency_id][] = $rate;
			}
		}
	}
?>
