<?php

	/**
	*	@brief Abstract class for automatic currency updating.  
	*
	*	Abstract class for automatic currency updating.
	*	To add another data source, extend exchange_datasource
	*
	*	@author Nathan Gray
	*	@ingroup perp_api
	*	@defgroup exchange_update Automatic update of exchange rates
	*/

	require_once( EGW_INCLUDE_ROOT . '/phpgwapi/inc/class.config.inc.php');

	abstract class exchange_datasource {

		/**
		*	The general ledger currency, which we convert from / to
		*/
		protected $gl_currency;	

		/**
		*	Constructor
		*	Reads the general ledger currency so we know what to convert to.
		*
		*	Don't forget to call parent::__construct()
		*/
		public function exchange_datasource() {
			$config =& new config('perp_ledger');
			$config->read_repository();
			$this->gl_currency = $config->config_data['gl_currency'];
		}

		/**
		*	Protected function to actually change the exchange rate
		*
		*	@param currency_id string Three letter currency code to update
		*	@param new_rate float New exchange rate for this currency
		*	@return float New exchange rate
		*/
		protected function update_currency($currency_id, $new_rate) {
			$GLOBALS['egw']->db->update('perp_currency', 
				array(
					'exchange_rate'	=>	$new_rate
				),
				array(
					'currency_id'	=>	$currency_id
				),
				__LINE__, __FILE__, 'perp_api'
			);
			return $new_rate;
		}

		/**
		*	Update the exchange rate for all pERP currencies.  
		*	No parameters
		*
		*	@return array New exchange rates in the form currency_id => [ new_rate | false]
		*/
		abstract function update_all();

		/**
		*	Update the exchange rate for one currency
		*
		*	@param	currency_id string Three letter currency code to update
		*	@return	float	New exchange rate for the currency or false if it could not be updated
		*/
		abstract function update($currency_id);
	}
?>
