<?php
	/**
	*	Class to encapsulate outgoing shipments (for goods sold).
	*	It extends perp_api::Shipment.
	*
	*	@author Nathan Gray
	*	@ingroup perp_ar
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.Shipment.inc.php');
	require_once('class.OutgoingShipmentLine.inc.php');

	class OutgoingShipment extends Shipment {

		protected $client_branch_id;

		public function __construct($shipment_id = null) {
			$this->set_supplier_id(0);
			parent::__construct($shipment_id);
		}

		public function read($shipment_id) {
			parent::read($shipment_id);

			$GLOBALS['egw']->db->select('perp_client_shipment',
				array(
					'client_branch_id'
				),
				array(
					'shipment_id'	=>	$shipment_id
				),
				__LINE__, __FILE__
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->client_branch_id = $row['client_branch_id'];
		}

		public function write() {
			parent::write();
			$GLOBALS['egw']->db->insert('perp_client_shipment',
				array(
					'shipment_id'		=>	$this->shipment_id,
					'client_branch_id'	=>	$this->client_branch_id
				),
				array(
					'shipment_id'	=>	$this->shipment_id
				),
				__LINE__, __FILE__, 'perp_ar'
			);
			return $this->shipment_id;
		}

		/**
		*	Construct a shipment from an object.
		*	If the object is a SalesOrder, the client branch is read from it.
		*/
		public function read_from_object(Shippable &$object) {
			parent::read_from_object($object);
			if(method_exists($object, 'get_client_branch_id')) {
				$this->set_client_branch_id($object->get_client_branch_id());
			}
		}

		public function get_client_branch_id() {
			return $this->client_branch_id;
		}

		/**
		*	Set the branch this shipment is going to.
		*	@todo Error checking
		*/
		public function set_client_branch_id($branch) {
			$this->client_branch_id = $branch;
		}

		public function __sleep() {
			$parameters = parent::__sleep();
			$parameters[] = 'client_branch_id';
			return $parameters;
		}
	}
?>
