<?php
	/**
	*	This class implements Invoicable 
	*	to allow invoicing of projects.
	*
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.Invoicable.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.InvoicableItem.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/projectmanager/inc/class.projectmanager_bo.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/projectmanager/inc/class.projectmanager_elements_bo.inc.php');

	class ProjectManagerInvoicable implements Invoicable {

		public static $bo_project;		// Initialized at the bottom
		public static $bo_project_elements;	// Initialized at the bottom
		public static $config;			// Initialized at the bottom
		private $project = array();


		/**
		*	Hooking functions
		*	@{
		*/

		/**
		*	Get a list of Project elements that are invoicable
		*	to support hooking
		*
		*	@param query An array of arguments, including client_id
		*	@nextmatch
		*/
		public static function &search(&$query, &$rows = array(), &$readonlys = array()) {
			if(!self::$config[__CLASS__]['enabled']) {
				return array();
			}

			$where = array(
				'(cat_id IN (' . implode(',', self::$config[__CLASS__]['allowed_categories']) . ')' . (self::$config[__CLASS__][0]['allowed'] ? ' OR cat_id IS NULL ' : '') . ')',
			);

			if($query['appname']) {
				$query['no_cat'] = false;
				$query['cat_app'] = 'projectmanager';
				$query['no_filter'] = false;
				$query['filter_label'] = 'project';
			
				$projects = self::$bo_project->search(
					$where, false, '', 
					'pm_extra_value AS client_id', 
					'', false, 'AND', 0, 
					array('pm_status' => 'active'), 
					'JOIN egw_pm_extra ON 
						egw_pm_projects.pm_id = egw_pm_extra.pm_id AND 
						pm_extra_name = "client_id"'
				);
				foreach($projects as $project) {
					$client_id = self::_get_client_id($project);
					if($client_id == null || $client_id != $query['client_id']) {
						continue;
					}
					$query['options-filter'][$project['pm_id']] = $project['pm_number'] . ': ' . $project['pm_title'];
				}
			}

			$start = $query['start'];
			$possibles = array();
			if(!$query['num_rows']) {
				$query['num_rows'] = $GLOBALS['egw_info']['user'][__CLASS__]['common']['maxmatchs'];
			}

			if($query['cat_id']) {
				$where['cat_id'] = (int)$query['cat_id'];
			}
			if($query['filter']) {
				$where['egw_pm_projects.pm_id'] = (int)$query['filter'];
			}

			if($query['order'] == 'unit_price') {
				$order = 'pe_unitprice';
			} elseif ($query['order'] == 'quantity') {
				$order = 'pe_used_quantity';
			} else {
				$order = 'pe_title';
			}
			
			$projects = self::$bo_project->search(
				$where, false, '', 
				'pm_extra_value AS client_id', 
				'', false, 'AND', 0, 
				array('pm_status' => 'active'), 
				'LEFT JOIN egw_pm_extra ON 
					egw_pm_projects.pm_id = egw_pm_extra.pm_id AND 
					pm_extra_name = "client_id"'
			);
			if(!is_array($projects) || count($projects) == 0) {
				return array();
			}
			foreach($projects as $project) {
				$client_id = self::_get_client_id($project);
				if($client_id == null || $client_id != $query['client_id']) {
					continue;
				}
				$elements = self::$bo_project_elements->search(
					$criteria,
					false,			// only keys
					$order . ' ' . $query['sort'], 
					'',			// extra columns
					'',			// wildcard
					false,			// empty
					'AND',			// criteria join
					array(0, -1),			// start, num_rows
					$filter=array(
						'pm_id'		=>	$project['pm_id']
					)
				);
				if(!is_array($elements) || count($elements) == 0) {
					break;
				}
				foreach($elements as $element) {
					if($element['pe_app'] != 'perp_ar') {
						$invoicable =& new ProjectManagerInvoicableItem($element);
						if($invoicable->can_invoice()) {
							$rows[] =& $invoicable;
						}
					}
				}
			}
		}

		/**
		*	Configure the invoicing of timesheets.
		*
		*	This is a hook into perp_ar Site Configuration to
		*	set up invoicing timesheets.
		*
		*	@param content Array containing location & data keys.
		*/
		public static function config($content = array()) {
			$category_object = CreateObject('phpgwapi.categories', '', 'projectmanager');

			// Save new settings
			if(is_array($content) && count($content) > 0) {
				self::$config[__CLASS__] = array();
				foreach($content as $key => $category) {
					if(is_numeric($key) && is_array($category)) {
						self::$config[__CLASS__][$category['category_id']] = $category;
						self::$config[__CLASS__]['allowed_categories'][] = $category['category_id'];
					}

				}
				self::$config[__CLASS__]['enabled'] = $content['enabled'];
				config::save_value(__CLASS__, self::$config[__CLASS__], 'perp_ar');
			}
			

			$categories = $category_object->return_array('all', 0, false, '', '', '', true);

			$settings = array();
			$settings[] = array(
				'category_id'	=>	0,
				'name'		=>	'No category',
				'allowed'	=>	self::$config[__CLASS__][0]['allowed'],
				'general_ledger_account'	=>	self::$config[__CLASS__][0]['general_ledger_account'],
			);
			if(is_array($categories)) {
				foreach($categories as $category) {
					$settings[] = array(
						'category_id'	=>	$category['id'],
						'name'		=>	$category['name'],
						'allowed'	=>	self::$config[__CLASS__][$category['id']]['allowed'],
						'general_ledger_account'	=>	self::$config[__CLASS__][$category['id']]['general_ledger_account']
					);
				}
			}
			$settings['enabled'] = self::$config[__CLASS__]['enabled'];
			// Get site configuration for invoicing timesheets
			return array(
				'template'	=>	'perp_ar.admin.timesheet_invoicable',
				'config'	=>	$settings,
				'help'		=>	'ProjectManager requires a custom field called client_id.  It should be of type \'Select entry - Accounts Receivable\', and that
							field must be set for every project you want to invoice.',
			);
		}

		/** 
		*	Determine the correct InvoicableItem object to use for the given element
		*
		*	@param element array
		*	@return string Class name
		*/
		public static function getInvoicableItemClass($element) {
			
			/**
			*	If you want to get fancy, and have a different class for a different app, 
			*	you can use this line:
			*	return str_replace(' ', '', lang($element['pe_app'])) . 'InvoicableItem';
			*	but you will have to verify that the search function isn't allowing you
			*	to invoice it as a project element, and as an element in pe_app.
			*/
			return 'ProjectManagerInvoicableItem';
		}

		/**
		* @}
		*/


		/**
		*	Invoicable bits
		*	
		*	This interface is so a new invoice can be automatically created from a
		*	project at the click of a button.
		*	The search function above is so lines can be added to an existing invoice.
		*/


		/**
		*	Constructor
		*/
		public function __construct($project_id = '') {
			if(is_numeric($project_id)) {
				$bopm = new projectmanager_bo($project_id);
				$this->project = $bopm->read($project_id);
			} elseif (is_array($project_id) && $project_id['pm_id']) {
				$this->project = $project_id;
			}
		}

		/**
		*	Get the supplier ID for the invoice.  Since we're the ones doing the invoicing,
		*	the ID is 0.
		*/
		public function get_supplier_id() {
			return 0;
		}

		/**
		*	Get the ID of the client 
		*
		*	This is a multi-step process, to determine which client
		*	- Check for a custom field, client_id
		*	- Check for a link to a client
		*	- Check for a link to a client's address
		*
		*	If a client can't be determined, it will return null.
		*/
		public function get_client_id() {
			return self::_get_client_id($this->project);
		}

		/**
		*	Find a client ID for a project, if it can be found
		*	
		*	@param project_id int/array Either a project ID number, or the array of project information
		*
		*	@return int client_id or null if a client cannot be identified.
		*/
		private static function _get_client_id($project_id) {
			if(is_array($project_id)) {
				$project = $project_id;
				$project_id = $project['pm_id'];
			} else {
				self::$bo_project = new projectmanager_bo($project_id);
				$project = self::$bo_project->read($project_id);
			}

			// Check for custom field
			if($project['#client_id']) {
				$project['client_id'] = $project['#client_id'];
			}
			if ($project['client_id']) {
				if(is_numeric($project['client_id'])) {
					return $project['client_id'];
				} else {
					return self::extract_client_id($project['client_id']);
				}
			}

			// Check links for a client
			$links = egw_link::get_links(
				'projectmanager', 
				$project_id, 
				'perp_ar'
			);
			foreach($links as $id => $link) {
				return self::extract_client_id($link);
			}
		}

		private static function extract_client_id($link) {

			if(strpos($link, 'CLIENT') === 0) {
				list($location, $c_id) = explode('-', $link);
				if($location == 'CLIENT') {
					return $c_id;
				} elseif ($location == 'CLIENT_BRANCH') {
					$branch = ExecMethod('perp_ar.bo_perp_client_branch.get_client_branch', $c_id);

					return $branch['client_id'];
				} elseif ($location == 'CLIENT_ACCOUNT') {
					$account = ExecMethod('perp_ar.bo_perp_client_account.get_client_account', $c_id);
					return $account['client_id'];
				}
			}
			return null;
		}

		public function get_our_contact_person() {
			return $this->project['pm_creator'];
		}
		public function get_their_contact_person() {
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $this->get_client_id());
			return $client['address'];
		}

		public function get_bill_from_address() {
			return self::$config['default_bill_from'];
		}
		public function get_bill_to_address() {
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $this->get_client_id());
			return $client['address'];
		}
		public function get_invoice_currency() {
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $this->get_client_id());
			return $client['client_currency'];
		}
		public function get_payment_terms() {
			return null;
		}

		public function &get_invoicable_items() {
			$elements = self::$bo_project_elements->search(
				array(),
				false,			// only keys
				'',
				'',			// extra columns
				'',			// wildcard
				false,			// empty
				'AND',			// criteria join
				array(0, -1),			// start, num_rows
				$filter=array(
					'pm_id'		=>	$this->project['pm_id']
				)
			);
			foreach($elements as $element) {
				if($element['pe_app'] != 'perp_ar') {
					$invoicable =& new ProjectManagerInvoicableItem($element);
					if($invoicable->can_invoice()) {
						$results[] =& $invoicable;
					}
				}
			}
			return $results;
		}

		public function &get_line($line_number) {
			$items = $this->get_invoicable_items();
			if(is_array($items)) {
				foreach($items as $key => &$item) {
					if($item->get_invoicable_line_id() == $line_number) {
						return $item;
					}
				}
			}
			return null;
		}
	}
	ProjectmanagerInvoicable::$bo_project = new projectmanager_bo();
	ProjectmanagerInvoicable::$bo_project_elements = new projectmanager_elements_bo();
	ProjectmanagerInvoicable::$config = config::read('perp_ar');
?>
