<?php

	/**
	*	Make project elements invoicable
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.InvoicableItem.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/projectmanager/inc/class.projectmanager_elements_bo.inc.php');

	class ProjectManagerInvoicableItem implements InvoicableItem {

		public static $config; 		// Initialized at the bottom
		public static $invoicable_class = 'ProjectManagerInvoicable';

		private $element = array();

		public function __construct($element_array) {
			if($element_array && is_array($element_array)) {
				if($element_array['pe_id']) {
					$this->element = $element_array;
				} elseif ($element_array['invoicable_id'] && $element_array['line_number']) {
					$bope = new projectmanager_elements_bo();
					$keys = array(
						'pm_id'	=>	$element_array['invoicable_id'],
						'pe_id'	=>	$element_array['line_number']
					);
					$bope->read($keys);
					$this->element = $bope->data;
				}
			}
		}

		/**
		*	Mark this InvoicableItem as invoiced.  The InvoiceLine will take care of
		*	any General Ledger transactions.
		*	@param invoice_line	InvoiceLine that is part of the Invoice
		*/
		public function invoice(InvoiceLine &$invoice_line) {

		}

		/**
		*	Return the Invoicable object that this InvoicableItem is part of
		*/
		public function get_owner() {
			return new ProjectManagerInvoicable($this->element['pm_id']);
		}

		/**
		*	Return a unique identifier for the owner of this InvoicableItem
		*/
		public function get_invoicable_id() {
			return $this->element['pm_id'];
		}

		/**
		*	Return a unique (amongst the owning object's InvoicableItems)
		*	identifier for this InvoicableItem 
		*/
		public function get_invoicable_line_id() {
			return $this->element['pe_id'];
		}

		/**
		*	Check that this InvoicableItem can be invoiced.  It can't be
		*	invoiced if it has already been completely invoiced.
		*
		*	@throws Exception with error message if it can't be invoiced.
		*	@return boolean 
		*/
		public function can_invoice() {
			if($this->element['pe_unitprice'] == null) {
				return false;
			} else {
				return ($this->get_invoice_quantity() != 0);
			}
		}

		public function get_quantity_invoiced() {
			return (float)$GLOBALS['egw']->db->select(
				'perp_invoice_line',
				array(
					'SUM(quantity) AS invoiced_quantity'
				),
				array(
					'invoicable_class'	=>	self::$invoicable_class,
					'invoicable_id'		=>	$this->get_invoicable_id(),
					'invoicable_item_line'	=>	$this->get_invoicable_line_id()
				),
				__LINE__, __FILE__, 0, 
				'GROUP BY invoicable_class, invoicable_id, invoicable_item_line',
				'perp_ar', 0,
				'JOIN egw_pm_elements ON 
					perp_invoice_line.invoicable_class = \'' . self::$invoicable_class . '\' AND
					perp_invoice_line.invoicable_id = egw_pm_elements.pm_id AND 
					perp_invoice_line.invoicable_item_line = egw_pm_elements.pe_id'
			)->fetchSingle();
		}

		/**
		*	Get the quantity to be invoiced, in the supplier's units of measure.
		*	
		*	@return float quantity to be invoiced
		*/
		public function get_invoice_quantity() {
			$quantity = $this->element['pe_used_quantity'] ? round($this->element['pe_used_quantity'],2) : 1;
			return (float)($quantity - $this->get_quantity_invoiced());
		}

		/**
		*	Get unit price for one unit of the InvoicableItem being invoiced.
		*	This is in the supplier's units of measure.
		*
		*	@return float the price of one unit 
		*/
		public function get_unit_price() {
			return $this->element['pe_unitprice'] ? $this->element['pe_unitprice'] : $this->element['pe_used_budget'] / $this->get_invoice_quantity();
		}

		/**
		*	Get the description for the InvoicableItem
		*	@return string description
		*/
		public function get_description() {
			return $this->element['pe_title'];
		}

		/**
		*	Get the general ledger account to be used on the invoice.
		*	May return null if the gl account is not known.
		*	@return int general ledger account number
		*/
		public function get_gl_account() {
			if($this->element['#gl_account']) {
				return $this->element['#gl_account'];
			} else {
				if(!self::$config) {
					self::$config = config::read('perp_ar');
				}
				return self::$config['ProjectManagerInvoicable'][($this->element['cat_id'] == null ? 0 : $this->element['cat_id'])]['general_ledger_account'];
			}
			
		}

		/**
		*       Return an array that will work for an eGroupWare style link.
		*       This is for allowing cross linking and jumping around.
		*       The array contain a sub-array, with keys app, id:
		*       return array(
		*               array(
		*                       'app'   =>      $GLOBALS['egw_info']['flags']['currentapp'],
		*                       'id'    =>      'ORDER-' . $this->get_owner->get__id()
		*               )
		*       );
		*
		*       @return array
		*/
		public function get_link() {
			return array(
				array(
					'app'	=>	'projectmanager',
					'id'	=>	$this->element['pe_id']
				)
			);
		}


		/**
		*	Return a list of extra details to be displayed on the invoice, below
		*	the line.  This list cannot be edited, and is an array of details in
		*	the form ('name' => 'Detail name', 'value' => '123')
		*
		*	@param filter The filter parameter from the display select (usually true)
		*	@return array
		*/
		public function get_invoice_details($filter) { 
			// Basic details
			$details = array(
			);

			if($filter == 2) {
				// More details
			}
			return $details;
		}

		/**
		*	Return a string representation of this object.
		*       Very useful for debugging purposes, not much else.
		*
		*       @return string Representation of this InvoicableItem
		*/
		public function __toString() {
			return sprintf('%-15s %-20s',
				__CLASS__,
				$this->element['pe_title']
			);
		}

		/**
		*	Return an array representation of this object
		*
		*	@return array
		*/
		public function toArray() {
			return $this->element + array(
				'link'	=>	$this->get_link()
			);
		}

		/**
		*	Prepare this object for serialization
		*
		*	@return array Properties to preserve across serialization
		*/
		public function __sleep() {
			return array('element');
		}
	}

?>
