<?php
	require_once(EGW_SERVER_ROOT . '/perp_api/inc/class.InvoiceLine.inc.php');

	class ReceivableInvoiceLine extends InvoiceLine {
		
		public function __construct(ReceivableInvoice $invoice, $info = null) {
			parent::__construct($invoice, $info);
			if($info instanceof SalesOrderLine) {
				$this->add_line_discount();
			}
		}

		/**
		*	Add this line to a GLTransaction.  Re-implemented
		*	to add the negative amount, as per accounting rules
		*/
		public function post(GLTransaction $transaction) {
			$line =& $transaction->add_line($this->general_ledger_account, $this->description,
				-$this->unit_price * $this->quantity * $this->invoice->get_exchange_rate()
			);
			$this->gl_transaction_line = end(array_keys($transaction->get_lines()));
			return $line->get_amount();
		}

		/**
		*	Try to find a General Ledger account for this line
		*	Looks in Sales Categories for Sales account
		*/
		public function find_general_ledger_account() {
			if($this->general_ledger_account) {
				return false;
			}
			if(!$this->stock_id) {
				if(!is_null($this->invoicable_item) && method_exists($this->invoicable_item, 'get_stock_id')) {
					$this->stock_id = $this->invoicable_item->get_stock_id();
					if(!$this->stock_id) {
						return false;
					}
				}
			}
			// Find sales category
			$sales_category = null;
			if(!is_null($this->invoicable_item) && method_exists($this->invoicable_item, 'get_price_list_id')) {
				$price_list = ExecMethod('perp_orders.bo_perp_price_list.get_price_list', $this->invoicable_item->get_price_list_id());
				$sales_category = ExecMethod('perp_orders.bo_perp_orders.get_sales_category', $price_list['sales_category_id']);
			}

			if(is_null($sales_category)) {
				$sales_categories = ExecMethod('perp_inventory.bo_perp_stock.get_sales_categories', $this->stock_id);
				if(count($sales_categories) != 1) {
					$this->invoice->set_comments($this->invoice->get_comments() . "\n" .
					lang('Not sure about the general ledger account for %1', $this->description));
				}
				if(count($sales_categories) > 0) {
					$sales_category = $sales_categories[0];
				}
			}
			if(!is_null($sales_category)) {
				$this->general_ledger_account = $sales_category['sales_gl_account'];
			}
		}

		/**
		*	Add a check to prevent mixing client accounts on the invoice
		*	This is in addition to the checks done by the parent.
		*
		*	@return true or error message
		*/
		public function check() {
			$message = parent::check();
			if($message !== true) {
				return $message;
			}
			
			if(is_object($this->get_invoicable_item())) {
				$owner =& $this->get_invoicable_item()->get_owner();
				if($owner && method_exists($owner, 'get_client_account_id') && $owner->get_client_account_id() != $this->invoice->get_client_account_id()) {
					return lang('Client account mismatch: Line %1 uses account %2 but the invoice is set to %3.  <br />Change the invoice, or remove the line.',
						($this->line_number ? $this->line_number : $this->get_description()),
						ExecMethod('perp_ar.bo_perp_client_account.link_title', $owner->get_client_account_id()),
						ExecMethod('perp_ar.bo_perp_client_account.link_title', $this->invoice->get_client_account_id())
					);
				}
			}
			return true;
		}

		/**
		*	Calculate per-line sales category discounts, as set per client account.
		*
		*	If the item being invoiced is in a sales category that is given a discount,
		*	add an additional line to the invoice to give the specific discount.
		*/
		protected function add_line_discount() {
			if(!$this->invoice->get_client_account_id() || !method_exists($this->invoicable_item, 'get_stock_id') || !$this->invoicable_item->get_stock_id()) {
				return false;
			}

			$sales_categories = ExecMethod('perp_inventory.bo_perp_stock.get_sales_categories', $this->invoicable_item->get_stock_id());
			foreach($sales_categories as $category) {
				$cats[] = $category['sales_category_id'];
			}
			if(count($cats) == 0) {
				return false;
			}
			$discounts = ExecMethod2('perp_ar.bo_perp_client_account.get_discounts', $this->invoice->get_client_account_id(), $cats);
			$line_class = get_class($this);
			foreach($discounts as $discount) {
				if($discount['discount_amount'] == 0 && $discount['discount_percent'] == 0) {
					continue;
				}
				$discount_info = array(
					'quantity'		=>	1,
					'description'		=>	$discount['narrative'],
					'general_ledger_account'=>	$discount['account_id']
				);
				if($discount['discount_amount'] != 0) {
					$discount_info['unit_price'] = -$this->get_quantity() * $discount['discount_amount'];
					$discount_line = new $line_class($this->invoice, $discount_info);
					$this->invoice->add_line($discount_line);
				}
				if($discount['discount_percent'] != 0) {
					$discount_info['unit_price'] = -($this->get_unit_price() * $this->get_quantity()) * $discount['discount_percent'];
					$discount_line = new $line_class($this->invoice, $discount_info);
					$this->invoice->add_line($discount_line);
				}
				
			}
		}
	}
?>
