<?php

	/**
	*	Extends ShipmentLineDetail to allow grouping OutgoingShipments into 
	*	multiple packages.  This allows a large order with several small parts that
	*	can be packed into two boxes generates accurate packing lists.
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ShipmentLineDetail.inc.php');

	class ShipmentPackage extends ShipmentLineDetail {

		/**
		*	In case the ShipmentLine gets split up between several containers,
		*	we record the number of the stock item's package (1,2,3...) that should be 
		*	in this container.  This is mainly for reference ( 1 of 3 style), as 
		*	they could change after the shipment is done, invalidating this as historical
		*	data.  For example, if something was shipped in 3 pieces, and 6 months later
		*	it's shipped in only 2 pieces, package #3 will cease to exist.
		*/
		protected $package_id;

		/**
		*	This is the particulars of the package, used for historical reasons, because
		*	package_id may change in the future.
		*/
		protected $package_details;

		public function __construct(ShipmentLine &$line, $data) {
			parent::__construct($line, $data);

			$this->package_id	=	$data['package_id'];
			$this->package_details	=	$data['package_details'];
		}
		
		public function read() {
			parent::read();
			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->select(
				'perp_outgoing_shipment_package',
				array(
					'*'
				),
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number
				),
				__LINE__, __FILE__, 0,
				'', 1,
				'perp_ar'
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->package_id	=	$row['package_id'];
			$this->package_details	=	$row['package_details'];
		}

		public function write() {
			parent::write();

			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->insert(
				'perp_outgoing_shipment_package',
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number,
					'package_id'	=>	$this->package_id,
					'package_details'	=>	$this->package_details,
				),
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number
				),
				__LINE__, __FILE__, 'perp_ar'
			);
		}

		public function remove() {
			parent::remove();
			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->delete(
				'perp_outgoing_shipment_package',
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number
				),
				__LINE__, __FILE__, 'perp_ar'
			);

		}

		public function get_container_id() {
			return $this->get_name();
		}
		public function set_container_id($id) {
			return $this->set_name($id);
		}
		public function get_quantity() {
			return $this->get_value();
		}
		public function set_quantity($qty) {
			return $this->set_value((float)$qty);
		}
		public function get_package_id() {
			return $this->package_id;
		}
		public function set_package_id($p_id) {
			$this->package_id = $p_id;
		}
		public function get_package_details() {
			return $this->package_details;
		}
		public function set_package_details($p_details) {
			$this->package_details = $p_details;
		}


		public function toArray() {
			return parent::toArray() +
			array(
				'package_id'		=>	$this->package_id,
				'package_details'	=>	$this->package_details,
			);
		}

		public function __toString() {
			return sprintf('%s Container ID: %s %s of Package # %d %s', 
				__CLASS__,
				$this->detail_name,
				$this->detail_value,
				$this->package_id,
				$this->package_details
			);
		}
	}
?>
