<?php

	/**
	*	Extends ShipmentLineDetail to record which serial numbers wind up in
	*	which package when grouping OutgoingShipments into 
	*	multiple packages.  
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ShipmentLineDetail.inc.php');

	class ShipmentSerial extends ShipmentLineDetail {

		/**
		*	In case the ShipmentLine gets split up between several containers,
		*	we record the serial number of the stock item in each container
		*/
		protected $serial_number;

		public function __construct(ShipmentLine &$line, $data) {
			parent::__construct($line, $data);

			$this->serial_number	=	$data['serial_number'];
		}
		
		public function read() {
			parent::read();
			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->select(
				'perp_outgoing_shipment_package',
				array(
					'*'
				),
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number
				),
				__LINE__, __FILE__, 0,
				'', 1,
				'perp_ar'
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->serial_number	=	$row['package_details'];
		}

		public function write() {
			parent::write();

			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->insert(
				'perp_outgoing_shipment_package',
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number,
					'package_details'	=>	$this->serial_number
				),
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number
				),
				__LINE__, __FILE__, 'perp_ar'
			);
		}

		public function remove() {
			parent::remove();
			$shipment_id = $this->get_shipment_line()->get_owner()->get_shipment_id();
			$line_number = $this->get_shipment_line()->get_line_number();
			$GLOBALS['egw']->db->delete(
				'perp_outgoing_shipment_package',
				array(
					'shipment_id'	=>	$shipment_id,
					'line_no'	=>	$line_number,
					'detail_id'	=>	$this->detail_number
				),
				__LINE__, __FILE__, 'perp_ar'
			);

		}

		public function get_container_id() {
			return $this->get_name();
		}
		public function set_container_id($id) {
			return $this->set_name($id);
		}
		public function get_quantity() {
			return $this->get_value();
		}
		public function set_quantity($qty) {
			return $this->set_value((float)$qty);
		}
		public function get_serial_number() {
			return $this->serial_number;
		}
		public function set_serial_number($sn) {
			$this->serial_number = $sn;
		}

		public function toArray() {
			return parent::toArray() +
			array(
				'serial_number'		=>	$this->serial_number,
			);
		}

		public function __toString() {
			return sprintf('%s Container ID: %s %s of S/N # %s', 
				__CLASS__,
				$this->detail_name,
				$this->detail_value,
				$this->serial_number
			);
		}

		public function __sleep() {
			return parent::__sleep() + array('serial_number');
		}
	}
?>
