<?php
	/**
	*	This class implements both Invoicable and InvoicableItem
	*	to allow invoicing of timesheet entries.
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.Invoicable.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.InvoicableItem.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/timesheet/inc/class.timesheet_bo.inc.php');

	class TimeSheetInvoicable implements Invoicable, InvoicableItem {

		public static $timesheet_bo; // Created just before use
		public static $config = array();
		private $timesheet = array();


		/**
		*	Get a list of TimeSheet entries that are invoicable
		*	to support hooking
		*
		*	@param query An array of arguments, including client_id
		*	@param rows Array of TimeSheetInvoicable objects
		*/
		public static function &search(&$query, &$rows) {
			$count = 0;
			$start = $query['start'];
			$possibles = array();
			if(!self::$config) {
				self::$config = config::read('perp_ar');
			}
			if(!self::$config[__CLASS__]['enabled']) {
				return array();
			}

			$already_invoiced = implode(',', self::get_invoiced_timesheets());

			if(!$query['num_rows']) {
				$query['num_rows'] = $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs'];
			}

			$where = array(
				'(cat_id IN (' . implode(',', self::$config[__CLASS__][allowed_categories]) . ')' . (self::$config[__CLASS__][0][allowed] ? ' OR cat_id IS NULL ' : '') . ')',
			);
			if(strlen($already_invoiced) > 0) {
				$where[] = 'ts_id NOT IN (' . $already_invoiced . ')';
			}
			if(!is_object(self::$timesheet_bo)) {
				self::$timesheet_bo = new timesheet_bo();
			}
			do {
				$results = self::$timesheet_bo->search(
					$where, 
					true, 
					$query['order'] + ' ' + $query['sort'],
					'', '', false, 'AND', array($start, $query['num_rows']),
					null, '', true
				);
				$count += $query['num_rows'];
				foreach($results as $key => &$result) {
					$ts_client_id = self::_get_client_id($result['ts_id']);
					if($ts_client_id == null || $ts_client_id != $query['client_id']) {
						unset($results[$key]);
					} else {
						$possibles[] = $result['ts_id'];
					}
				}
				$start = $count;
			} while($count < self::$timesheet_bo->total);
			$results = array();
			foreach($possibles as $id) {
				$object =& new TimeSheetInvoicable($id);
				if($object->can_invoice() && $object->get_client_id() == $query['client_id']) {
					$results[] =& $object;
				}
			}
			$rows = $results;
			return $results;
		}

		/**
		*	Configure the invoicing of timesheets.
		*
		*	This is a hook into perp_ar Site Configuration to
		*	set up invoicing timesheets.
		*
		*	@param content Array containing location & data keys.
		*/
		public static function config($content = array()) {
			$category_object = CreateObject('phpgwapi.categories', '', 'timesheet');
			self::$config = config::read('perp_ar');

			// Save new settings
			if(is_array($content) && count($content) > 0) {
				self::$config[__CLASS__] = array();
				foreach($content as $key => $category) {
					if(is_numeric($key) && is_array($category)) {
						self::$config['TimeSheetInvoicable'][$category['category_id']] = $category;
						self::$config['TimeSheetInvoicable']['allowed_categories'][] = $category['category_id'];
					}

				}
				self::$config[__CLASS__]['enabled'] = $content['enabled'];
			}
			config::save_value(__CLASS__, self::$config[__CLASS__], 'perp_ar');

			$categories = $category_object->return_array('all', 0, false, '', '', '', true);

			$settings = array();
			$settings[] = array(
				'category_id'	=>	0,
				'name'		=>	'No category',
				'allowed'	=>	self::$config['TimeSheetInvoicable'][0]['allowed'],
				'general_ledger_account'	=>	self::$config['TimeSheetInvoicable'][0]['general_ledger_account'],
			);
			if(is_array($categories)) {
				foreach($categories as $category) {
					$settings[] = array(
						'category_id'	=>	$category['id'],
						'name'		=>	$category['name'],
						'allowed'	=>	self::$config['TimeSheetInvoicable'][$category['id']]['allowed'],
						'general_ledger_account'	=>	self::$config['TimeSheetInvoicable'][$category['id']]['general_ledger_account']
					);
				}
			}
			$settings['enabled'] = self::$config[__CLASS__]['enabled'];

			// Get site configuration for invoicing timesheets
			return array(
				'template'	=>	'perp_ar.admin.timesheet_invoicable',
				'config'	=>	$settings,
			);
		}

		/**
		*	Utility function used to get a list of timesheets that 
		*	have already been invoiced, according to perp_invoice_line.invoicable_class
		*	in case silly users delete the link
		*
		*	@todo Move this into the SQL directly, it adds O(n) to a search
		*	@return array of timesheet ids that have been invoiced
		*/
		public static function get_invoiced_timesheets() {
			$GLOBALS['egw']->db->select(
				'perp_invoice_line',
				array('invoicable_id'),
				array('invoicable_class' => __CLASS__),
				__LINE__, __FILE__, false, '', 'perp_ar'
			);
			$ids = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$ids[] = $row['invoicable_id'];
			}
			return $ids;
		}

		public function __construct($timesheet_id = '') {
			if(!is_object(self::$timesheet_bo)) {
				self::$timesheet_bo = new timesheet_bo();
			}
			if($timesheet_id) {
				// second parameter is to ignore acl
				$this->timesheet = self::$timesheet_bo->read($timesheet_id, true);
			}
		}


		// Invoicable

		public function get_supplier_id() {
			return 0;
		}

		/**
		*	Get the ID of the client 
		*
		*	This is a multi-step process, to determine which client
		*	- Check for a custom field, client_id
		*	- Check for a link to a client
		*	- Check for a link to a client's address
		*
		*	If a client can't be determined, it will return null.
		*/
		public function get_client_id() {
			return TimeSheetInvoicable::_get_client_id($this->timesheet['ts_id']);
		}

		private static function _get_client_id($timesheet_id) {
			if(!is_object(self::$timesheet_bo)) {
				self::$timesheet_bo = new timesheet_bo();
			}
			$timesheet = self::$timesheet_bo->read($timesheet_id,true);

			// Check for custom field
			if($timesheet['#client_id']) {
				$links[] = $timesheet['#client_id'];
			} else {

				// Check links for a client
				$links = egw_link::get_links(
					'timesheet', 
					$timesheet_id, 
					'perp_ar'
				);
			}

			foreach($links as $id => $link) {
				if(strpos($link, 'CLIENT') === 0) {
					list($location, $c_id) = explode('-', $link);
					if($location == 'CLIENT') {
						return $c_id;
					} elseif ($location == 'CLIENT_BRANCH') {
						$branch = ExecMethod('perp_ar.bo_perp_client_branch.get_client_branch', $c_id);

						return $branch['client_id'];
					} elseif ($location == 'CLIENT_ACCOUNT') {
						$account = ExecMethod('perp_ar.bo_perp_client_account.get_client_account', $c_id);
						return $account['client_id'];
					}
				}
			}
			return null;
		}

		public function get_our_contact_person() {
			return $this->timesheet['ts_owner'];
		}
		public function get_their_contact_person() {
			return null;
		}

		public function get_bill_from_address() {
			return null;
		}
		public function get_bill_to_address() {
			return null;
		}
		public function get_invoice_currency() {
			return null;
		}
		public function get_payment_terms() {
			return null;
		}

		public function &get_invoicable_items() {
			return array($this);
		}

		// InvoicableItem

		/**
		*	Mark this InvoicableItem as invoiced.  The InvoiceLine will take care of
		*	any General Ledger transactions.
		*
		*	This is implemented using eGroupWare's linking system.
		*
		*	@param invoice_line	InvoiceLine that is part of the Invoice
		*/
		public function invoice(InvoiceLine &$invoice_line) {
			egw_link::link(
				'timesheet', 
				$this->timesheet['ts_id'],
				'perp_ar', 
				'INVOICE-' . $invoice_line->get_invoice()->get_invoice_id() . ':' . $invoice_line->get_line_number()
			);
		}

		/**
		*	Return the Invoicable object that this InvoicableItem is part of
		*
		*	For Timesheets, this is the timesheet entry itself.
		*
		*	@return Invoicable
		*/
		public function get_owner() {
			return $this;
		}

		/**
		*	Return a unique identifier for the owner of this InvoicableItem
		*
		*	This is the timesheet entry's ID, ts_id.
		*/
		public function get_invoicable_id() {
			return $this->timesheet['ts_id'];
		}

		/**
		*	Return a unique (amongst the owning object's InvoicableItems)
		*	identifier for this InvoicableItem 
		*
		*	In this case, it's the same as get_invoicable_id() because
		*	timesheets don't have sub-items.  Timesheet doesn't use this,
		*	so we're OK as long as it doesn't return null.
		*/
		public function get_invoicable_line_id() {
			return $this->timesheet['ts_id'];
		}

		/**
		*	Check that this InvoicableItem can be invoiced.  It can't be
		*	invoiced if it has already been completely invoiced.
		*
		*	@throws Exception with error message if it can't be invoiced.
		*	@return boolean 
		*/
		public function can_invoice() {
			$links = egw_link::get_links(
				'timesheet', 
				$this->timesheet['ts_id'], 
				'perp_ar'
			);
			if (count($links) != 0) {
				foreach($links as $id => $link) {
				// Timesheet entry has a link to an invoice
					if(strpos($link, 'INVOICE') === 0) {
						return false;
					}
				}
			}
			// Check if naughty users have tried to unlink the invoice
			if(in_array($this->timesheet['ts_id'], self::get_invoiced_timesheets())) {
				return false;
			}
			return true;
		}

		/**
		*	Get the quantity to be invoiced, in the supplier's units of measure.
		*	
		*	@return float quantity to be invoiced
		*/
		public function get_invoice_quantity() {
			return $this->timesheet['ts_quantity'];
		}

		/**
		*	Get unit price for one unit of the InvoicableItem being invoiced.
		*	This is in the supplier's units of measure.
		*
		*	@return float the price of one unit 
		*/
		public function get_unit_price() {
			return $this->timesheet['ts_unitprice'];
		}

		/**
		*	Get the description for the InvoicableItem
		*	@return string description
		*/
		public function get_description() {
			return $this->timesheet['ts_title'];
		}

		/**
		*	Get the general ledger account to be used on the invoice.
		*	May return null if the gl account is not known.
		*	@return int general ledger account number
		*/
		public function get_gl_account() {
			if($this->timesheet['#gl_account']) {
				return $this->timesheet['#gl_account'];
			} else {
				if(!self::$config) {
					self::$config = config::read('perp_ar');
				}
				return self::$config[__CLASS__][($this->timesheet['cat_id'] == null ? 0 : $this->timesheet['cat_id'])]['general_ledger_account'];
			}
		}

		/**
		*       Return an array that will work for an eGroupWare style link.
		*       This is for allowing cross linking and jumping around.
		*       The array contain a sub-array, with keys app, id:
		*       return array(
		*               array(
		*                       'app'   =>      $GLOBALS['egw_info']['flags']['currentapp'],
		*                       'id'    =>      'ORDER-' . $this->get_owner->get__id()
		*               )
		*       );
		*
		*       @return array
		*/
		public function get_link() {
			return array(
				array(
					'app'	=>	'timesheet',
					'id'	=>	$this->timesheet['ts_id'],
				)
			);
		}

		/**
		*	Return a list of extra details to be displayed on the invoice, below
		*	the line.  This list cannot be edited, and is an array of details in
		*	the form ('name' => 'Detail name', 'value' => '123')
		*
		*	@param filter The filter parameter from the display select (usually true)
		*	@return array
		*/
		public function get_invoice_details($filter) { 
			// Basic details
			$details = array(
				array(
					'name'	=>	lang('Start'),
					'value'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' ' . ($GLOBALS['egw_info']['user']['preferences']['common']['timeformat']==24 ? 'H:i':'g:i a'), $this->timesheet['ts_start'])
				),
			);
			if($filter == 2) {
				// More details
				$details[] = array(
					'name'	=>	lang('Duration'),
					'value'	=>	lang('%1 minutes', $this->timesheet['ts_duration'])
				);
				$details[] = array(
					'name'	=>	lang('Notes'),
					'value'	=>	$this->timesheet['ts_description']
				);
			}
			return $details;
		}

		/**
		*	Return an array representation of this object
		*
		*	@return array
		*/
		public function toArray() {
			return $this->timesheet + array(
				'link'	=>	$this->get_link()
			);
		}

		/**
		*	Return a string representation of this object.
		*       Very useful for debugging purposes, not much else.
		*
		*       @return string Representation of this InvoicableItem
		*/
		public function __toString() {
			return sprintf('%-15s %-20s',
				__CLASS__,
				$this->element['pe_title']
			);
		}

		public function __sleep() {
			return array('timesheet');
		}

	}
?>
