<?php

	require_once('class.bo_perp_client_branch.inc.php');

	class bo_perp_client {

		private $so;
		public $config;

		private $client_read;
		private $client_add;
		private $client_edit;
		private $client_delete;	
		
		public $link;

		public function bo_perp_client() {
			$this->so = CreateObject('perp_ar.so_perp_client');
			$bo_ar = CreateObject('perp_ar.bo_perp_ar');
			$this->config = &$bo_ar->config;

			$this->client_read = $GLOBALS['egw']->acl->check('client', EGW_ACL_READ, 'perp_ar');
			$this->client_add = $GLOBALS['egw']->acl->check('client', EGW_ACL_ADD, 'perp_ar');
			$this->client_edit = $GLOBALS['egw']->acl->check('client', EGW_ACL_EDIT, 'perp_ar');
			$this->client_delete = $GLOBALS['egw']->acl->check('client', EGW_ACL_DELETE, 'perp_ar');
		}

		public function get_client_list($query = array(), &$rows = array(), &$readonlys = array()) {	
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			if(!$this->client_read) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			
			$query['status'] = $query['status'] ? $query['status'] : $query['filter'];
			$query['area'] = $query['area'] ? $query['area'] : $query['filter2'];
			$query['order'] = $query['order'] ? $query['order'] : 'client_name';
			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';

			if(!isset($query['template']) && !isset($query['num_rows'])) {
				// Get all records for a select box
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_client_list($query);

			if(!isset($query['template'])) {
				foreach($list as $key => $row) {
					$rows[$row['client_id']] = $row['client_num'] . ' - ' . $row['client_name'];
				}
				return $rows;
			} else {
				if($query['include_branches']) {
					$bo_branch = new bo_perp_client_branch();
					foreach($list as $client) {

						$sub_query = $query;
						$branches = array();
						$sub_query['client_id'] = $client['client_id'];
						$bo_branch->get_client_branch_list($sub_query, $client['branches'], $branch_readonlys);
						$rows[] = $client;
					}
				} else {
					$rows = $list;
				}
				$rows['edit'] = $this->client_edit;
				$rows['delete'] = $this->client_delete;
			}

			return $query['count'];
		}

		public function get_client($client_id) {
			$client = array();
			if($this->client_read) {
				$client = $this->so->get_client($client_id);
			}
			return $client;
		}

		public function set_client($client_id, $client) {
		
			if(!($this->client_edit || $this->client_add)) {
				return False;
			}

			// TODO: data checks
			if(!$client['client_name']) {
				return lang('Name is required');
			}
			if(!$client['client_since']) {
				$client['client_since'] = date('U');
			}

			// Need to create default (main) account
			if(!$client_id) {
				$account = array();
			}
			// Need to create default (main) branch
			if(!$client_id) {
				$branch = array();
			}

			// If client doesn't have billing address, disable them
			if(!$client['address'] || $client['address'] == 0) {
				$client['status'] = 3;
			}
			$client_id =  $this->so->set_client($client_id, $client);

			// Create default (main) account
			if(isset($account)) {
				$account_name = ($this->config->config_data['default_client_account']) ? $this->config->config_data['default_client_account'] : $client['client_name'];
				$account = array(
					'client_id'	=>	$client_id,
					'client_account_name'	=>	$account_name
				);
				ExecMethod2('perp_ar.bo_perp_client_account.set_client_account',
					false,
					$account
				);
			}

			// Create default (main) branch
			if(isset($branch)) {
				$branch = array(
					'client_id'	=>	$client_id,
					'branch_name'	=>	$client['client_name'],
					'sales_group'	=>	$client['sales_group'],
				);
				ExecMethod2('perp_ar.bo_perp_client_branch.set_client_branch',
					false,
					$branch
				);
			}
			return $client_id;
		}

		public function delete_client($client_id) {
			if(!$this->client_delete) {
				return False;
			}

			$client = $this->get_client($client_id);
			
			// Remove client user account
			if($client['user_account']) {
				$GLOBALS['egw']->accounts->delete($client['user_account']);
			}

			egw_link::unlink2(null, 'perp_ar', $client_id);
			ExecMethod('addressbook.addressbook_bo.delete', $client['address']);

			$bo_branch = CreateObject('perp_ar.bo_perp_client_branch');
			$bo_branch->get_client_branch_list(array('client_id' => $client_id), $branch_list);
			if(is_array($branch_list)) {
				foreach($branch_list as $branch) {
					$bo_branch->delete_client_branch($branch['client_branch_id']);
				}
			}
			return $this->so->delete_client($client_id);
		}

		public function suggest_client_number() {
			return $this->so->suggest_client_number();
		}

		public function get_shipment_list($query = array(), &$rows = array(), &$readonlys = array()) {
			if($GLOBALS['egw']->acl->check('client_shipment', EGW_ACL_READ, 'perp_ar')) {
				$bo_shipment = CreateObject('perp_ar.bo_perp_client_shipment');
				return $bo_shipment->get_shipment_list($query, $rows, $readonlys);
			} else {
				return lang('Access not permitted');
			}
		}

		public function get_account_balance($client_id, $client_account_id = '', $as_at_date = '') {
			if($this->client_read && $GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar')) {
				if(!$as_at_date){
					$as_at_date = time();
				}
				return $this->so->get_account_balance($client_id, $client_account_id, $as_at_date);
			} else {
				return lang('Access not permitted');
			}
		}

		public function get_ledger($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ar')) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
	
			$query['client_id'] = $query['filter'] ? $query['filter'] : $query['client_id'];
			$query['client_account_id'] = $query['filter2'] ? $query['filter2'] : $query['client_account_id'];
			$query['order'] = $query['order'] ? $query['order'] : 'trans_date, trans_id';
			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';
			$rows = $this->so->get_ledger($query);
			
			if(!isset($query['template'])) {
				// TODO: if needed, write this
			} else {
				return $query['count'];
			}
		}

		public function get_status_list() {
			return array(
				1	=>	lang('Active'),		// Normal client, can do stuff
				2	=>	lang('Restricted'),	// 
				3	=>	lang('Disabled')	// No longer a client, can't order
			);
		}

		/**
		*	Get an array of client_ids associated with the given account_id(s)
		*
		*	An empty array means the account is not associated with a client.  That
		*	usually means a staff member, so access should not be restricted based on this.
		*	Use normal ACLs.
		*
		*	@example if(is_array($client_ids) && count($client_ids) && !in_array($branch['client_id'], $client_ids)) <deny>
		*	
		*	@param account_id Account IDs to check for (usually current user & user's groups), or false for current user
		*	@return array of client_ids that match
		*/
		public function get_client_by_user_account($account_id = false) {
			if($account_id) {
				return $this->so->get_client_by_user_account($account_id);
			} else {
				$account_id = $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'],true);
				foreach($account_id as &$id) {
					$id = abs($id);
				}
				$account_id[] = $GLOBALS['egw_info']['user']['account_id'];
				return $this->so->get_client_by_user_account($account_id);
			}
		}

		/**
		* get title for an perp client entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param int/array $entry int client_id or array with client entry
		* @param string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				return $this->so->link_title($entry);
			}
			if (!$entry) {
				return $entry;
			}
			return $entry['client_num'].' - '.$entry['client_name'];
		}

		
		/**
		* query perp clients for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param string $pattern pattern to search
		* @return array with client_id - title pairs of the matching entries
		*/
		function link_query( $pattern ) {		

			$list = $this->so->link_query($pattern);
			
			$result = array();
			foreach($list as $list_el) {
				$result['CLIENT-' . $list_el['client_id']] = $this->link_title($list_el);
			}
						
			return $result;
		}	


		/**
		*	List of product the client has ordered that we have not shipped yet.
		*	
		*	This list is pretty much the same as what is shippable (bo_perp_outgoing_shipment::get_shippable(),
		*	with some extra information added in, such as reference tags and options.
		*/
		public function get_on_order(&$query, &$rows = array(), &$readonlys = array()) {
			$bo_shipment = new bo_perp_outgoing_shipment();
			$rows = array();
			$count = $bo_shipment->get_shippable_items($query, $rows, $readonlys);

			if(!$query['include_details']) {
				return $count;
			}

			// Cache the orders to speed it up
			$order_cache = array();
			
			// For some reason, this query doesn't limit
			foreach($rows as &$group) {
				if(!is_array($group['stock_category'])) {
					continue;
				}
				foreach($group['stock_category'] as &$category) {
					if(!is_array($category['stock'])) {
						continue;
					}
					foreach($category['stock'] as &$item) {
						if(!isset($item['stock_id'])) {
							continue;
						}

						// Add in some extra numbers
						if(!isset($order_cache[$item['order_id']])) {
							$order_cache[$item['order_id']] = new SalesOrder($item['order_id']);
						}
						$details = $order_cache[$item['order_id']]->get_line($item['line_no'])->get_details();
						foreach($details as $detail) {
							if($detail->get_display()) {
								$detail_array = $detail->toArray();
								switch(get_class($detail)) {
									case 'WorkOrderDetail':
										$detail_array['detail_name'] = lang('Work Order');
										$detail_array['detail_value'] = $detail_array['work_order_reference'];
										break;
									case 'ClientReferenceDetail':
										$detail_array['detail_name'] = lang('Client Reference');
										break;
								}
								$item['details'][] = $detail_array;
							}
						}
					}
				}
			}
			return $count;
		}
			
	}
?>
