<?php

	class bo_perp_client_account {

		private $so;

		public function bo_perp_client_account() {
			$this->so = CreateObject('perp_ar.so_perp_client_account');
		}

		public function get_client_account_list($query = array(), &$rows = array(), &$readonly = array()) {
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0) {
				if($query['client_id'] && !in_array($query['client_id'], $client_ids)) {
					return array();
				}
				$query['client_id'] = $client_ids;
			}
			
			$query['order'] = $query['order'] ? $query['order'] : 'client_account_name';

			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';
			if(!isset($query['template'])) {
				$query['num_rows'] = -1;
			}

			// Default to only active accounts
			if(!isset($query['filter'])) {
				$query['filter'] = 'current';
			}

			if($query['filter']) {
				$query['status'] = $query['filter'];
			}
			
			$rows = $this->so->get_client_account_list($query);

			if(!$GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar')){
				// Users creating sales orders need at least the account_id & name
				if(!$GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_orders')) {
					if(isset($query['template'])) {
						$rows = array();
						return 0;
					} else {
						return array();
					}
				}
			}

			if(!isset($query['template'])) {
				$list = array();
				foreach($rows as &$row) {
					$list[$row['client_account_id']] = $this->link_title($row['client_account_id']);
				}
				return $list;
			} else {
				// Users creating sales orders need at least the account_id & name
				if(!$GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar') && $GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_orders')) {
					foreach($rows as $key => $row) {
						$rows[$key] = array(
							'client_id'		=>	$row['client_id'],
							'client_account_id'	=>	$row['client_account_id'],
							'client_account_name'	=>	$row['client_account_name']
						);
					}
				}
				if($query['get_price_lists']) {
					foreach($rows as $key => $row) {
						$rows[$key]['price_lists'] = implode(',', $this->get_price_lists($row['client_account_id']));
					}
				}
				return $query['count'];
			}
		}

		public function get_client_account($client_account_id) {
			// Users creating sales orders need at least the account_id & name
			if($GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar') || $GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_orders')) {

				$client_account =  $this->so->get_client_account($client_account_id);

				if(!$GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar')){
					$client_account = array(
						'client_id'		=>	$client_account['client_id'],
						'client_account_id'	=>	$client_account['client_account_id'],
						'client_account_name'	=>	$client_account['client_account_name']
					);
				}

				// Check if the current user is a client
				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) > 0 && !in_array($client_account['client_id'], $client_ids)) {
					// Tried to access an account that they have no rights to
					return array();
				}

				return $client_account;
			} else {
				return array();
			}
		}

		public function set_client_account($client_account_id, $data) {
			if(!is_array($data)) {
				return False;
			}
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($data['client_id'], $client_ids)) {
				// Tried to access an account that they have no rights to
				return lang('Access not permitted');
			}
			if($GLOBALS['egw']->acl->check('client_account', EGW_ACL_EDIT, 'perp_ar')){
				return $this->so->set_client_account($client_account_id, $data);
			} else {
				return lang('Access not permitted');
			}
		}

		public function disable_account($client_account_id) {
			if($GLOBALS['egw']->acl->check('client_account', EGW_ACL_EDIT, 'perp_ar')){
				return $this->so->disable_account($client_account_id);
			} else {
				return lang('Access not permitted');
			}
		}

		public function delete_client_account($client_account_id) {
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			$client_account =  $this->so->get_client_account($client_account_id);
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($client_account['client_id'], $client_ids)) {
				// Tried to access an account that they have no rights to
				return lang('Access not permitted');
			}
			if($GLOBALS['egw']->acl->check('client_account', EGW_ACL_DELETE, 'perp_ar')){
				return $this->so->delete_client_account($client_account_id);
			} else {
				return lang('Access not permitted');
			}
		}

		public function get_price_lists($client_account_id, $date = null) {
			if(is_null($date)) {
				$date = time();
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			
			// Check to make sure client doesn't try to look into the future
			if($client_ids && $date > time()) {
				$date = time();
			}

			$client_account =  $this->so->get_client_account($client_account_id);
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($client_account['client_id'], $client_ids)) {
				// Tried to access an account that they have no rights to
				return lang('Access not permitted');
			}
			if($GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar') || $GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				return $this->so->get_price_lists($client_account_id, $date);
			} else {
				return lang('Access not permitted');
			}
		}

		public function get_discounts($client_account_id, $sales_category = '') {
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			$client_account =  $this->so->get_client_account($client_account_id);
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($client_account['client_id'], $client_ids)) {
				// Tried to access an account that they have no rights to
				return lang('Access not permitted');
			}

			if($GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar')) {
				return $this->so->get_discounts($client_account_id, $sales_category);
			} else {
				return lang('Access not permitted');
			}
		}

		public function get_status_list() {
			return array(
				1	=>	lang('Active'),		// Normal account, can do stuff
				2	=>	lang('Restricted'),	// 
				3	=>	lang('Disabled')	// Account closed, can't order
			);
		}

		/** 
		*	Get title for linking
		*	Takes an account id
		*/
		public function link_title($id) {
			list($location, $id) = explode('-', $id);
			if(!$id) {
				$id = $location;
			}
			$account = $this->get_client_account($id);
			return ExecMethod('perp_ar.bo_perp_client.link_title', $account['client_id']) . ' - ' . $account['client_account_name'];
		}
	}
?>
