<?php
	class bo_perp_client_branch {
		
		private $so;
		public $config;

		// Auto-create client user accounts
		private $new_account = array(
			'account_status'	=>	'A',
			'account_expires'	=>	-1,
			'account_type'		=>	'u',
		);

		public function bo_perp_client_branch() {
			$this->so = CreateObject('perp_ar.so_perp_client_branch');

			// Use config from bo_ar
			$bo_ar = CreateObject('perp_ar.bo_perp_ar');
			$this->config = &$bo_ar->config;
		}

		public function get_client_branch_list($query = array(), &$rows = array(), &$readonlys = array()) {
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}
			if(!$GLOBALS['egw']->acl->check('client_branch', EGW_ACL_READ, 'perp_ar')) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			$query['filter'] = isset($query['client_id']) ? $query['client_id'] : $query['filter'];
			$query['order'] = $query['order'] ? $query['order'] : 'client_name';
			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';

			$list = $this->so->get_client_branch_list($query);

			if(!isset($query['template'])) {
				foreach($list as $row) {
					$rows[$row['client_branch_id']] = $row['client_branch_id'] . ' - ' . $row['branch_name'];
				}
				return $rows;
			} else {
				$rows = array_slice($list, $query['start'], $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs']);
				$rows['edit'] = $GLOBALS['egw']->acl->check('client_branch', EGW_ACL_READ, 'perp_ar');
				$rows['delete'] = $GLOBALS['egw']->acl->check('client_branch', EGW_ACL_DELETE, 'perp_ar');
				return count($list);
			}
		}

		public function get_client_branch($client_branch_id) {
			if($GLOBALS['egw']->acl->check('client_branch', EGW_ACL_READ, 'perp_ar')) {
				$branch =  $this->so->get_client_branch($client_branch_id);

				// Check if the current user is a client
				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) && !in_array($branch['client_id'], $client_ids)) {
					return array();
				}

				if($branch['delivery_address'] == $branch['postal_address']) {
					$branch['same_address'] = True;
				}
				return $branch;
			} else {
				return array();
			}
		}

		public function set_client_branch($client_branch_id, $branch) {
			if($GLOBALS['egw']->acl->check('client_branch', EGW_ACL_EDIT, 'perp_ar')) {
				// Check if the current user is a client
				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) && !in_array($branch['client_id'], $client_ids)) {
					return false;
				}

				if($this->config->config_data['client_user_accounts']) {
					if(!$branch['user_account']) {
						// Create user account
						$client = ExecMethod('perp_ar.bo_perp_client.get_client', $branch['client_id']);
						$this->new_account['account_lid'] = $client['client_num'] .
							'-' . $branch['branch_name'];
						$this->new_account['account_firstname'] = $branch_name;
						$this->new_account['account_lastname'] = lang('Client');
						$branch['user_account'] = $GLOBALS['egw']->accounts->create($this->new_account, True);
						$GLOBALS['egw']->acl->add_repository('phpgw_group', $this->new_account['account_primary_group'], $content['account'], EGW_ACL_READ);
					} else {
						$account =& CreateObject('phpgwapi.accounts', $branch['user_account']);
						$account_data = $account->read();

						if($branch['status'] > 2) {
						// If branch is disabled, disable user
							$account_data['account_status'] = '';
						} else {
						// Branch is enabled, enable user
							$account_data['account_status'] = 'A';
						}
						$account->update_data($account_data);
						$account->save_repository();
					}
				}
				if(trim($branch['tax_number']) == '') {
					$branch['tax_number'] = null;
				}

				return $this->so->set_client_branch($client_branch_id, $branch);
			} else {
				return False;
			}
		}

		public function delete_client_branch($client_branch_id) {
			if($GLOBALS['egw']->acl->check('client', EGW_ACL_DELETE, 'perp_ar')) {
				$branch = $this->get_client_branch($client_branch_id);

				// Check if the current user is a client
				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) && !in_array($branch['client_id'], $client_ids)) {
					return false;
				}

				// Remove client user account
				if($branch['user_account']) {
					$GLOBALS['egw']->accounts->delete($branch['user_account']);
				}

				ExecMethod('addressbook.addressbook_bo.delete', $branch['postal_address']);
				ExecMethod('addressbook.addressbook_bo.delete', $branch['delivery_address']);
				$link_id = $branch['client_id'] . ':' . $client_branch_id;
				egw_link::unlink2(null, 'perp_ar', $link_id);
				return $this->so->delete_client_branch($client_branch_id);
			}
		}

		/**
		* get title for an perp client branch entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param int/array $entry int client_branch_id or array with client branch entry
		* @param string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				return $this->so->link_title($entry);
			}
			if (!$entry) {
				return $entry;
			}
			return $entry['client_num'] . '(' . $entry['client_branch_id'].') ' . 
				$entry['client_name'] . ' - '.$entry['branch_name'];
		}

		
		/**
		* query perp client branches for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param string $pattern pattern to search
		* @return array with client_id - title pairs of the matching entries
		*/
		function link_query( $pattern ) {		

			$list = $this->so->link_query($pattern);
			
			$result = array();
			foreach($list as $list_el) {
				$result[ 'CLIENT_BRANCH-' . $list_el['client_branch_id']] = $this->link_title($list_el);
			}
			
			return $result;
		}
	}
?>
