<?php

	/**
	*	Logic layer for outgoing shipments, this extends perp_api::bo_perp_shipment
	*
	*	@author Nathan Gray
	*	@ingroup perp_ar
	*/

	require_once('class.OutgoingShipment.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_shipment.inc.php');
	require_once('class.so_perp_outgoing_shipment.inc.php');

	class bo_perp_outgoing_shipment extends bo_perp_shipment {
	

		public function bo_perp_outgoing_shipment() {
			$this->so_perp_shipment = new so_perp_outgoing_shipment();
		}

		/**
		*	Get a list of outgoing shipments
		*	@nextmatch
		*	Query options:
		*		- client_id	-	Restrict to just one client
		*		- status (filter)	Filter by status
		*/
		public function get_shipment_list($query = array(), &$rows = array(), &$readonlys = array()) {

			$query['supplier_id'] = 0;
			$query['client_id'] = $query['client_id'] ? $query['client_id'] : $query['filter2'];
			$query['status'] = $query['status'] ? $query['status'] : $query['filter'];

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			/* 
				Parent function checks permissions on the current app, so we need to make sure
				that perp_ar is the current app.  It gets set back at the end of this function.
				Noticed because of ajax calls from the Ajax select widget.
			*/
			$old_app = $GLOBALS['egw_info']['flags']['currentapp'];
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';

			$query['count'] = parent::get_shipment_list($query, $rows, $readonlys);
			foreach($rows as $key => $row) {
				// Items in shipment
				if(!$row['pending'] && ($query['get_lines'] || $query['show_sales_orders'])) {
					$lines = array();
					$sales_orders = array();
					$sub_query = $row;
					if($query['sales_order_id']) {
						$sub_query['sales_order_id'] = $query['sales_order_id'];
					}
					$this->get_shipment_items($sub_query, $lines, $line_readonlys);
					foreach($lines as $l_key=>$line) {
						if(is_array($line) && isset($line['sales_order_id'])) {
							$sales_orders[$line['sales_order_id']] = $line['sales_order_num'];
						}
					}
					array_unshift($lines, false);
					if($query['get_lines']) {
						$rows[$key]['lines'] = $lines;
					} 

					if($query['show_sales_orders']) {
						$row_so = array();
						foreach($sales_orders as $so_id => $so_num) {
							$row_so[] = array(
								'sales_order_id' => $so_id,
								'sales_order_number' => $so_num
							);
						}
						$rows[$key]['sales_orders'] = $row_so;
					}
				}

				// Add in permissions on packing
				$readonlys['pack[' . $row['shipment_id'] . ']'] = !$GLOBALS['egw']->acl->check('pack', EGW_ACL_EDIT, 'perp_ar') || $readonlys['ship[' . $row['shipment_id'] . ']'] || ($row['actual_delivery_date'] != 0);
			}

			if($query['hide_client']) {
				// All the same client, hide client column
				$rows['client_id'] = True;
			}

			$GLOBALS['egw_info']['flags']['currentapp'] = $old_app;

			return $query['count'];
		}

		public function get_shipment($shipment_id) {
			if(!$GLOBALS['egw']->acl->check('client_shipment', EGW_ACL_READ, 'perp_ar')) {
				$rows = array();
				return 0;
			}

			if(!$shipment_id) {
				return False;
			}

			$shipment = $this->so->get_shipment($shipment_id);
			if($shipment[0]) {
				$shipment = $shipment[0];
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($shipment['client_id'], $client_ids)) {
				return false;
			}

			if($shipment) {
				$shipment['items'] = $this->so->get_shipment_items($shipment_id, false, false, 'client_shipment_id', 'ASC');
			}
			return $shipment;
		}

		public function get_shipment_items($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('client_shipment', EGW_ACL_READ, 'perp_ar')) {
				$rows = array();
				return 0;
			}
			
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			$query['order'] = $query['order'] ? $query['order'] : 'client_shipment_id';
			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';

			$list = $this->so->get_shipment_items($query['client_shipment_id'], $query['sales_order_id'], $query['search'], $query['order'], $query['sort'], $query['group_by']);

			if($query['group_by'] == 'stock_category') {
				$cat = '';
				$count = -1;
				$total = 0;
				foreach($list as $item) {
					if($item['stock_category_name'] != $cat) {
						$cat = $item['stock_category_name'];
						$count++;
						$rows[$count]['stock_category_name'] = $item['stock_category_name'];
						$rows[$count]['items'][] = false;
					}
					$rows[$count]['items'][] = $item;
					$total += $item['value'];
				}
				$rows['total'] = $total;
			} else {
				$rows = array_slice($list, $query['start'], $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs']);
				array_unshift($rows, false);
				if(isset($rows[1])) {
					$rows['total'] = $rows[1]['value'];
				}
			}

			return count($list);
		}

		public function get_shippable_items($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, 'perp_ar')) {
				$rows = array();
				return 0;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			if($query['shipment_id']) {
				$shipment = $this->get_shipment($query['shipment_id']);
				$query['location_id'] = $shipment['from_location'];
			} elseif ($query['sales_order_id']) {
				$so = ExecMethod('perp_orders.bo_perp_sales_order.get_sales_order', $query['sales_order_id']);
				$query['location_id'] = $so->get_from_location();
			}

			$list = $this->so_perp_shipment->get_shipable_items($query);

			$items = array();
			$count = array(0 => -1, 1 => -1);
			$stock_category = -1;
			$sales_order = 0;
			$rows[0]['group'] = lang('In stock:');
			$rows[1]['group'] = lang('Not in stock:');
			foreach($list as $row) {
				$group = ($row['quantity_on_hand'] <= 0);
				if(!$rows[$group]['stock_category'][$row['stock_category_id']] && ($row['stock_category_id'] != $stock_category || !$rows[$group]['stock_category'])) {
					$count[$group]++;
					$stock_category = $row['stock_category_id'];

					$rows[$group]['stock_category'][$row['stock_category_id']]['stock_category_id'] = $row['stock_category_id'];
					$rows[$group]['stock_category'][$row['stock_category_id']]['stock_category_name'] = $row['stock_category_name'];
					$rows[$group]['stock_category'][$row['stock_category_id']]['stock'][] = false;
				}
				if($row['eta'] && $row['quantity_on_hand'] <= 0) {
					$rows[$group]['stock_category'][$row['stock_category_id']]['stock']['eta'] = true;
				} else {
					$rows[$group]['stock_category'][$row['stock_category_id']]['stock']['on_hand'] = True;
				}
				if($row['stock_type_flag'] == 'A' && $GLOBALS['egw_info']['apps']['perp_manufacturing']) {
					$row['status'] = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order_status', $row['incoming_order_id']);
				}
				$rows[$group]['stock_category'][$row['stock_category_id']]['stock'][] = $row;
			}
			foreach($rows as $group => &$group_categories) {
				if(is_array($group_categories['stock_category'])) {
					sort($group_categories['stock_category']);
				}
			}
			return count($rows);
		}

		public function get_packing_slip($shipment_id) {
			if($GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, 'perp_ar')) {
				
				// Check if the current user is a client
				$shipment = $this->get_shipment($shipment_id);
				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) > 0 && in_array($shipment['client_id'], $client_ids)) {
					return array();
				}

				return $this->so_perp_shipment->get_packing_slip($shipment_id);
			}
		}

		public function get_status_list() {
			return array(
				''		=>	lang('All'),
				'pending'	=>	lang('pending'),
				'approved'	=>	lang('approved'),
				'in-transit'	=>	lang('in-transit'),
				'arrived'	=>	lang('arrived'),
				'overdue'	=>	lang('overdue'),
			);
		}

		/**
		* get title for a perp supplier shipment entry identified by $entry
		*                               
		* Is called as hook to participate in the linking                
		*
		* @param entry int/array key to look up or array with shipment entry
		* @return string  with title/boolean
		*/

		public function link_title($entry) {
			if(is_array($entry)) {
				return $entry['our_reference_number'];
			} elseif(is_numeric($entry)) {
				$shipment = new OutgoingShipment($entry);
				return $shipment->get_our_reference_number();
			}
			return false;
		}
	}
?>
