<?php
	require_once('class.ReceivableInvoice.inc.php');

	class bo_perp_receipt {

		private $so;
		private $debug = false;

		public function bo_perp_receipt() {
			$this->so = CreateObject('perp_ar.so_perp_receipt');
		}

		public function get_receipts($query = array(), &$rows = array(), &$readonlys = array()) {
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_READ, 'perp_ar')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			$query['client_id'] = $query['client_id'] ? $query['client_id'] : $query['filter'];
			$query['payment_method'] = $query['payment_method'] ? $query['payment_method'] : $query['filter2'];
			$query['sort'] = $query['sort'] ? $query['sort'] : 'DESC';
			$query['order'] = $query['order'] ? $query['order'] : 'receipt_id';

			$rows = $this->so->get_receipts($query);

			if(!isset($query['template'])) {
				// TODO: write this
			} else {
				if($GLOBALS['egw']->acl->check('receipt', EGW_ACL_DELETE, 'perp_ar')) {
					foreach($rows as &$row) {
						$readonlys['reverse[' . $row['receipt_id'] . ']'] = ($row['status'] != 1);
					}
				} else {
					$rows['no_reverse'] = true;
				}
				return $query['count'];
			}
		}

		public function get_receipt($receipt_id) {
			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_READ, 'perp_ar')) {
				return lang('access is not allowed');
			}
			if(!$receipt_id) {
				return false;
			}
			$receipt = $this->so->get_receipt($receipt_id);

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($receipt['client_id'], $client_ids)) {
				return lang('access is not allowed');
			}
			// Check for reversal information
			$reversal = self::get_reversal($receipt_id);
			if($reversal) {
				// Copy the receipt ID for UI convenience
				$reversal['receipt_id'] = $receipt_id;
				$receipt['reversal'] = $reversal;
			}

			return $receipt;
		}

		public function get_unallocated_amounts($query = array(), &$rows = array(), &$readonlys = array()) {
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_READ, 'perp_ar')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			$list = $this->so->get_unallocated_amounts($query);

			if(!isset($query['template'])) {
				// TODO: write this
			} else {
				$rows = $list;
				return $query['total'];
			}
		}

		/**
		*	Post receipt 
		*	Processes a receipt and:
		*	1. Posts it to the General Ledger, Bank Ledger and Client Ledger
		*	2. Marks pre-payments as used (if used)
		*	3. Marks invoices as (partially) paid
		*
		*	@param array $receipt_information in the form
		*		array(
		*			'client_id'		=>	Client ID
		*			'client_account_id'	=>	Client account id
		*			'receipt_id'		=>	The reference of the receipt (Receipt number)
		*			'payment_method'	=>	Payment Method
		*			'payment_number'	=>	Cheque or CC number
		*			'bank_account'		=>	Bank account
		*			'allocation_date'	=>	Date this happens on
		*			'pre_payment'		=>	Additional amount that was pre-paid
		*			'receipt_total'		=>	Total amount received
		*			row# => array(
		*				'reference_id'		=>	either a receipt_id or an invoice_number
		*				'trans_id'		=>	If a pre-payment, the transaction_id of the payment
		*				'allocation_date'	=>	The date the receipt was allocated
		*				'invoice_id'		=>	If an invoice, the id of the invoice
		*				'original_amount'	=>	Original amount of the payment / invoice
		*				'amount_owing'		=>	Amount of the invoice left to pay, or amount of pre-payment left un-allocated
		*				'discount_taken'	=>	Discount used from the Invoice
		*				'receipt_amount'	=>	The amount received
		*			)
		*		)
		*/
		public function post_receipt($receipt_information) {
			// Permission check
			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar')) {
				throw new egw_exception_no_permission(lang('You do not have permission to add receipts'));
			}

			$GLOBALS['egw']->db->transaction_begin();

			$client_id 	= $receipt_information['client_id'];
			$client_account_id = $receipt_information['client_account_id'];
			$receipt_id	= $receipt_information['receipt_id'];
			$payment_method	= $receipt_information['payment_method'];
			$payment_number = $receipt_information['payment_number'];
			$bank_account	= $receipt_information['bank_account'];
			$bank		= ExecMethod('perp_ledger.bo_perp_ledger.get_bank_account', $bank_account);
			$pre_payment	= (float)$receipt_information['pre_payment'];
			$allocation_date= $receipt_information['allocation_date'];
			$receipt_total	= (float)$receipt_information['receipt_total'];

			// Check for double-post
			$check = $this->get_receipt($receipt_id);
			if(isset($check[1]) && $check['receipt_id'] == $receipt_id) {
				throw new egw_exception(lang('Duplicate receipt number, Receipt %1 has already been posted.', $receipt_id));
			}

			// Get site settings
			$config = config::read('perp_ar');
			$ar_account = $config['ar_account'];
			$discount_account = $config['discount_account'];
			$receipt_narrative = $receipt_information['receipt_narrative'] ? $receipt_information['receipt_narrative'] : $config['receipt_narrative'];
			$discount_narrative = $config['discount_narrative'];
			$trans_type = $config['receipt_transaction_type'];
			if(!$receipt_id) {
				$receipt_id = $config['next_receipt_number'];
			} 
			$config['next_receipt_number'] = $receipt_id + 1;
			config::save_value('next_receipt_number', $config['next_receipt_number'], 'perp_ar');

			$config = config::read('perp_api');
			$diff_on_exchange_account = $config['unrealized_currency_loss_account'];

			// Check Client currency against bank currency
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $client_id);
			$bo_api = CreateObject('perp_api.bo_perp_api');
			if($client['currency'] == $bank['currency']) {
				$query = array(
					'currency_id' => $bank['currency']
				);
			} else {
				$query = array(
					'currency_id' => $client['currency']
				);
			}
			$query['template'] = false;
			$bo_api->get_currencies( $query, $currency);
			if($currency[0]) {
				$currency = $currency[0];
			} else {
				return lang('Could not find currency');
			}
			$exchange = $currency['exchange_rate'];

			if(!$receipt_narrative) {
				$receipt_narrative = lang('Receipt narrative not set') . " %2$s";
			}
			$receipt_narrative = sprintf($receipt_narrative,
				$client['client_num'],
				$receipt_id,
				($invoice ? $invoice->get_our_reference_number() : '')
			);

			// Set up precision, 2 decimal places unless otherwise specified
			// bc might not be the right choice here, since scale
			// chops and doesn't round.  And it's compairing against
			// rounded values.  Just going to round everything
			// before bcMath will Nathan returns.
			bcscale(2);

			// Set up transaction
			$transaction = CreateObject('perp_ledger.GLTransaction');
			$transaction->set_transaction_type($trans_type);
			$transaction->set_date($allocation_date);

			$gl_currency_total = bcmul(round($receipt_total, 2), round($exchange, 2));

			// Bank account line
			$bank_trans = $transaction->add_line($bank_account, $receipt_narrative, $gl_currency_total);
			$bank_trans = $bank_trans->get_associated_line(0);
			$bank_trans->set_payment_method($payment_method);
			$bank_trans->set_currency($bank['currency']);

			$check_total = 0;
			$client_ledger = 0;
			$difference_on_exchange = 0;
			$receipts = array();
			$last_invoice = null;

			foreach($receipt_information as $key => $row) {
				if(is_numeric($key) && is_array($row)) {
					if((is_null($row['discount_taken']) || $row['discount_taken'] === '') &&
							(is_null($row['receipt_amount']) || $row['receipt_amount'] === '')) {
						// User didn't actually enter anything here, skip it.
						continue;
					}

					if($row['invoice_id'] && !($row['invoice'] && $row['invoice'] instanceof ReceivableInvoice)) {
						$row['invoice'] =& new ReceivableInvoice($row['invoice_id']);
					}
					if($row['invoice'] instanceof Invoice) {
						$last_invoice =& $row['invoice'];
					}

					if($row['discount_taken']) {

						// Discount line
						$discount_amount = bcmul(round($row['discount_taken'], 2), round($exchange, 2));

						$transaction->add_line($discount_account, $discount_narrative, $discount_amount);
						$ar_line = $transaction->add_line($ar_account, $discount_narrative, -$discount_amount);
						$client_trans = CreateObject('perp_ar.ClientTransactionLine', $ar_line);
						$client_trans->set_account($ar_account);
						$client_trans->set_amount(-$discount_amount);
						$client_trans->set_narrative($discount_narrative);
						$client_trans->set_client($client_id);
						$client_trans->set_client_account($client_account_id);
						$client_trans->set_exchange_rate($exchange);
						$client_trans->set_link('perp_ar', 'INVOICE-' . $row['invoice_id']);
					}

					/**
					*	We don't want people allocating more money to the invoice then is allowed,
					*	that'll probably cause more problems that it's worth, as it's probably a
					*	data entry error.  Any overpayments are moved into the prepayment.  It
					*	gets tricky if we're not completely consuming a previous prepayment or a
					*	credit, because those are negative, so we need to do some work with the
					*	numbers
					*/
					$credit = $row['amount_owing'] < 0 ? -1 : 1;
					if(($row['discount_taken'] + $row['receipt_amount']) * $credit > $credit * $row['amount_owing']) {
						$overpay = bcmul(round($credit, 2), bcsub( round(bcadd(round($row['discount_taken'], 2), round($row['receipt_amount']), 2), 2), round(bcmul(round($credit, 2), round($row['amount_owing'], 2)), 2)));

						// Debug 
						if($this->debug && $overpay != 0) {
							echo 'Tried to overpay ' . $row['invoice_id'] . ' by ' . $overpay . ' <br />';
						}
						$row['receipt_amount'] = bcsub(round($row['receipt_amount'], 2), round($overpay, 2));
						$pre_payment = bcadd($pre_payment, $overpay);
					}

					if($row['invoice'] && $row['invoice']->get_transaction()) {
						$trans_id = $row['invoice']->get_transaction();
						$inv_trans = CreateObject('perp_ledger.GLTransaction', $trans_id);
						if(!$inv_trans) {
							continue;
						}
						$invoiced_trans_line = CreateObject('perp_ar.ClientTransactionLine', $inv_trans);
						if($this->debug) {
							echo '<pre>' . $invoiced_trans_line->__toString() . '</pre>';
							echo 'Difference on exchange. ' . $invoiced_trans_line->get_exchange_rate() . ' -> ' . $exchange;
						}
						if($exchange && $invoiced_trans_line->get_exchange_rate() != $exchange && $diff_on_exchange_account) {
						        $difference_on_exchange = bcadd(round($difference_on_exchange, 2), round(bcmul(round(bcsub(round($invoiced_trans_line->get_exchange_rate(), 2), round($exchange, 2)), 2), round(bcadd(round($row['receipt_amount'], 2), round($row['discount_taken'], 2), 2)))), 2);
						}
					}
					$line_no = count($transaction->get_lines()) - 1;

					$receipts[] = array(
						'invoice_id'	=>	$row['invoice_id'],
						'receipt_amount'	=>	$row['receipt_amount'],
						'allocated_from'	=>	$row['receipt_id'],
						'line_no'		=>	$line_no
					);
					$check_total = bcadd(round($check_total, 2), round($row['receipt_amount'], 2));
				}
			}

			// Crazy comparison due to floating point imprecision
			if(($check_total + $pre_payment - $receipt_total) > 0.001) {
				return lang("Lines don't add up to total (%1 + %2 != %3)", $check_total, $pre_payment, $receipt_total);
			}

			// Use the currency of the last invoice
			if($difference_on_exchange != 0) {
				$transaction->add_line($diff_on_exchange_account, $last_invoice->get_invoice_currency() . ' ' . $last_invoice->get_exchange_rate() . ' -> ' . $exchange, $difference_on_exchange);
			}
			unset($last_invoice);

			// AR line
			$ar_line = $transaction->add_line($ar_account, $receipt_narrative, -bcadd(round($gl_currency_total, 2), round($difference_on_exchange, 2)));
			$line_no = count($transaction->get_lines()) - 1;
			$client_trans = CreateObject('perp_ar.ClientTransactionLine', $ar_line);
			$client_trans->set_account($ar_account);
			$client_trans->set_amount(-$gl_currency_total);
			$client_trans->set_narrative(lang('Receipt %1', $receipt_id));
			$client_trans->set_client($client_id);
			$client_trans->set_client_account($client_account_id);
			$client_trans->set_client_reference($payment_number);
			$client_trans->set_exchange_rate($exchange);
			$client_trans->set_link('perp_ar', 'RECEIPT-' . $receipt_id);

			if($this->debug) {
				echo '<pre>';
				echo $transaction;
				echo '</pre>';
			}

			if($GLOBALS['egw']->db->Link_ID->transCnt == 0) {
				/*
				* Database doesn't support transactions.
				* Reconnect to do transactions, and shove it in GLOBALS[egw]->db so
				* everywhere uses the same connection
				*/
				$old_db =& $GLOBALS['egw']->db;
				$GLOBALS['egw']->db =& ExecMethod('perp_api.bo_perp_api.reconnect_transactional');
				$GLOBALS['egw']->db->transaction_begin();
			}
			$trans_id = $transaction->write();

			// Update Invoices
			foreach($receipt_information as $key => $row) {
				if(is_numeric($key) && is_array($row)) {
					if($row['invoice_id']) {
						// Paying an Invoice
						if(!($row['invoice'] instanceof ReceivableInvoice)) {
							$row['invoice'] =& new ReceivableInvoice($row['invoice_id']);
						}
						
						$row['invoice']->take_discount($row['discount_taken']);

						$row['invoice']->set_paid(($row['discount_taken'] + $row['receipt_amount'] == $row['amount_owing']));

						$row['invoice']->write();
					}
				}
			}

			foreach($receipts as $receipt) {
				// Allocate receipts
				$result = $this->allocate_receipt($receipt_id, $trans_id, $receipt['line_no'] ? $receipt['line_no'] : $line_no, $receipt['invoice_id'], $receipt['receipt_amount'], $allocation_date, $receipt['allocated_from'], $payment_number);
				if($result === false) {
					// Error, rollback transaction
					$GLOBALS['egw']->db->transaction_abort();
					$receipt_id = 0;
					break;
				}
			}

			// Pre-payment
			if($pre_payment != 0 && !$this->allocate_receipt($receipt_id, $trans_id, $line_no, null, $pre_payment, null, null, $payment_number)) {
				$GLOBALS['egw']->db->transaction_abort();
				$GLOBALS['egw']->db->transaction_commit();
				return lang('Error allocating the prepayment');
			}

			$GLOBALS['egw']->db->transaction_commit();
			if($old_db) {
				// Swap the original database back, so it doesn't cause problems elsewhere
				$GLOBALS['egw']->db = $old_db;
			}

			return $receipt_id;
		}

		public function allocate_receipt($receipt_id, $trans_id, $line_no, $invoice_id, $amount, $date, $pre_payment, $payment_number='') {
			if(is_null($pre_payment)) {
				$pre_payment = 0;
			}
			if($GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar') && $amount != 0) {
				return $this->so->allocate_receipt($receipt_id, $trans_id, $line_no, $invoice_id, $amount, $date, $pre_payment, trim($payment_number));
			} else {
				return False;
			}
		}

		/**
		* get title for a perp receipt entry identified by $entry
		*
		* Is called as hook to participate in the link
		*
		* @param int/array $entry key to look for or array with receipt entry
		* @return string the title
		*/
		public function link_title($entry) {
			if(is_array($entry)) {
				return lang('Receipt %1', $entry['receipt_id']);
			} elseif(is_numeric($entry)) {
				return lang('Receipt %1', $entry);
			}
			return false;
		}

		/**
		*	Undo a receipt
		*
		*	The receipt is still recorded, and all the ledger entries remain, but a reversing transaction is
		*	done to reverse the effects.
		*
		*	@param receipt_id The ID number of the receipt to reverse
		*	@param reason The reason for the reversal
		*	@return int Reversing receipt ID
		*	@throws egw_exception_no_permission if the user does not have access to reverse receipts
		*	@throws egw_exception_wrong_userinput if you try to reverse a receipt that has cleared
		*/
		public function reverse($receipt_id, $reason) {
			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_DELETE, 'perp_ar')) {
				throw new egw_exception_no_permission(lang('You do not have permission to reverse receipts'));
			}
			$receipt = $this->get_receipt($receipt_id);

			if($receipt['status'] != 1) {
				throw new egw_exception_wrong_userinput(lang('Receipt %1 cannot be reversed', $receipt_id));
			}
			
			if(trim($reason) == '') {
				throw new egw_exception_wrong_userinput(lang('You must provide a reason for the reversal'));
			}

			return $this->so->reverse($receipt, $reason);
		}

		/**
		*	Read a reversal record
		*
		*	@param receipt_id Either the original receipt, or the reversal
		*	@return Array of reversal information
		*/
		public static function get_reversal($receipt_id) {
			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_READ, 'perp_ar')) {
				return array();
			}
			return so_perp_receipt::get_reversal($receipt_id);
		}
	}
?>
