<?php

	/**
	*	Functions for dealing with multiple invoices.  Note that an Invoice is 
	*	an Object Oriented object, and handles most things itself.
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_invoice.inc.php');

	require_once('class.ReceivableInvoice.inc.php');

	class bo_perp_receivable_invoice extends bo_perp_invoice {

		private $debug = false;
		
		public function __construct() {
			parent::__construct();
		}

		public function get_invoice_list($query, &$rows = array(), &$readonly = array()) {
			$query['supplier_id'] = 0;
			$query['client_id'] = $query['filter2'] ? $query['filter2'] : $query['client_id'];
			$query['order'] = $query['order'] ? $query['order'] : 'client_name';
			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';
			$read = $GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ar');
			if(!$read) {
				if(!$query['template']) {
					return array();
				} else {
					return 0;
				}
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			parent::get_invoice_list($query, $rows, $readonly);

			if(!isset($query['template'])) {
				foreach($rows as $key => $row) {
					$rows[$row['invoice_id']] = $row['client_num'] . ' - ' . $row['client_name'] . ' ' . $this->link_title($row['invoice_id']);
				}
				return $rows;
			}

			// Additional permission check on receipt
			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar')) {
				foreach($rows as $row) {
					$readonly['pay[' . $row['invoice_id'] . ']'] = true;
				}
			}
				
			return $query['count'];
		}

		public function get_status_list() {
			return array(
				'all'		=>	lang('All'),
				'unposted'	=>	lang('Not posted'),
				'due'		=>	lang('Due'),
				'overdue'	=>	lang('Overdue'),
				'open'		=>	lang('Open'),
				'closed'	=>	lang('Closed'),
			);
		}

		/**
		* Query receivable invoices for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param string $pattern pattern to search
		* @return array with invoice_id - title pairs of the matching entries
		*/
		public function link_query( $pattern ) {		

			$query = array(
				'search'	=>	$pattern,
				'template'	=>	false
			);
			$this->get_invoice_list($query, $list);

			$result = array();
			foreach($list as $invoice) {
				$result['INVOICE-' . $invoice['invoice_id']] = $this->link_title($invoice['invoice_id']);
			}
			return $result;
		}

		public function link_title($id) {
			list($invoice, $id) = explode('-', $id);
			if(!$id) {
				$id = $invoice;
			}
			$invoice = new ReceivableInvoice($id);
			return lang('Invoice %1', $invoice->get_our_reference_number());
		}

		/**
		*	Get a list of items from other eGroupWare applications that can be invoiced.
		*
		*	Supporting applications should define a hook to the location 
		*	'perp_ar:invoicable_search'.  An array with search parameters (standard nextmatch) 
		* 	and the client_id will be passed to the hook function, which 
		*	should return an array of InvoicableItem objects.  If invoicing from that
		*	app is disabled, it will not be called.  The app should check its config
		*	parameters when searching.
		*
		*	Optionally, supporting applications may define a hook to the location
		*	'perp_ar:invoicable_config'.  It should return an array with two keys:
		*
		*	-	'template'	=>	the name of an etemplate for configuring the app
		*	-	'config'	=>	Array of name => value pairs,
		*			one of which should be 'enabled' => (boolean)
		*	The hook may be passed an array when called containing new values.
		*
		*	In addition to the standard nextmatch parameters available, 
		*	client_id must be set.
		*
		*	@nextmatch
		*/
		public function get_uninvoiced_lines(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$query['client_id'] || !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ar')) {
				$rows = array();
				return false;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}

			$args = array(
				'location'	=>	'perp_ar:invoicable',
			);
			
			if($query['appname']) {
				$hooks[$query['appname']] =& $GLOBALS['egw']->hooks->single($args, $query['appname']);
			} else {
				$hooks =& $GLOBALS['egw']->hooks->process($args);
			}

			$count = 0;
			foreach($hooks as $app => $hook) {
				if($hook['search']) {
					// Do it nextmatch style
					list($search_app, $class, $method) = explode('.', $hook['search']);
					$object = CreateObject($search_app . '.' . $class);
					$object->$method($query, $rows[$app], $readonlys[$app]);
				}
			}
			foreach($rows as $app => &$results) {
				if(is_array($results) && count($results) > 0) {
					$count += count($results);
				} else {
					unset($rows[$app]);
				}
			}
			return $count;
		}

		/**
		*	Get the preferred report format for an invoice
		*
		*	This is based on the preferences of the users involved.  If the client has a preference, return that,
		*	otherwise, the current user's preference.
		*/
		public function get_preferred_report_format(ReceivableInvoice &$invoice) {
			// If client has an account use their preference
			if($client['user_account']) {
				$preferences = CreateObject('phpgwapi.preferences', $client['user_account']);
				$prefs = $preferences->read(); 
				return $prefs['perp_api']['report_output_format'];
			} else {
				// Use current user's preference
				return $GLOBALS['egw_info']['user']['preferences']['perp_api']['report_output_format'];
			}
		}

		/**
		*	Email an invoice to a client
		*
		*	A single report will be generated using the given invoice IDs, converted if needed, and then
		*	emailed to the requested destination.  The email will be logged into the Infolog.
		*
		*	If format is not specified, get_preferred_report_format() will be used.
		*	If to is not provided, the client's billing email will be used.
		*	You cannot mix invoices for more than one client, and send client A any of client B's invoices.
		*
		*	@param invoice_id An array of invoice IDs.  All the invoices will be sent in one email
		*	@param format The format of the report.  Must be a format the report generator can handle, or it won't work.
		*	@param to The destination email address
		*
		*	@throws egw_exception_wrong_userinput if you try and mix invoices from two clients together
		*
		*	@return boolean
		*/
		public function email_invoice(Array $invoice_id, $format = null, $to = null) {
			if(!$invoice_id || count($invoice_id) == 0) {
				return false;
			}

			// Need to check to make sure user doesn't accidentally try to email client A any of client B's invoices
			$invoice = null;
			$client_id = null;
			$references = array();
			$bill_from_address = null;
			$bill_to_address = null;

			foreach($invoice_id as $id) {
				$invoice =& new ReceivableInvoice($id);
				$references[] = $invoice->get_our_reference_number();
				if(is_null($client_id)) {
					$client = ExecMethod('perp_ar.bo_perp_client.get_client', $invoice->get_client_id());
					$bill_to_address = ExecMethod('addressbook.addressbook_bo.read', $client['address']);
					$bill_from_address = ExecMethod('addressbook.addressbook_bo.read', $invoice->get_bill_from_address());
				} elseif ($invoice->get_client_id() != $client_id) {
					// Mixing clients!
					throw new egw_exception_wrong_userinput(lang('You cannot email invoices from different clients together.'));
				}
			}

			if(!$format) {
				$format = get_preferred_report_format(new ReceivableInvoice($invoice_id[0]));
			}
			
			// Get report settings
			$settings = array(
				'output_format'	=>	$format,
				'invoice_id'	=>	$invoice_id
			);

			$distribution = array(
				'type'		=>	'email',
				'from'		=>	$bill_from_address['email'],
				'from_name'	=>	$bill_from_address['fileas'],
				'to'		=>	array($bill_to_address['email'] ? $bill_to_address['email'] : $bill_to_address['home_email']),
				'subject'	=>      lang('Invoices') . ' ' . implode(', ', $references),
				'body'		=>	lang('Please find attached the following invoices: <br />%1', implode("\n", $references)),
			);

			// Create infolog record
			$infolog = array(
				'info_subject'  =>      $distribution['subject'],
				'info_type'     =>      'email',
				'info_addr'	=>	$distribution['to'],
				'info_from'	=>	$distribution['from'],
				'info_cat'      =>      null,
				'info_des'      =>      str_replace('<br />', "\n", lang('client_invoice_email<br />%1',
					implode("\n", $references)
				)),
				'info_owner'    =>      $GLOBALS['egw_info']['user']['account_id'],
				'info_contact'  =>      'addressbook:' . $client['address'],
				'info_link'	=>	array(
					'app'	=>	'perp_ar'
				),
				'info_startdate'	=>	time() + 5,
				'info_status'   =>      'ongoing',
				'info_percent'  =>      0,
			);
			$distribution['infolog_id'] = ExecMethod('infolog.infolog_bo.write', $infolog);

			// Link infolog to invoices

			// Schedule report
			$async_id = ExecMethod2('perp_api.bo_perp_report.schedule_report', 
				'perp_ar', 'invoice_report', $infolog['info_startdate'], 
				$distribution, 
				$settings
			);
			
			return ($async_id != 0);
		}
	}
?>
