<?php
	class so_perp_client {
		
		private $db;

		public function so_perp_client() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_ar');
		}

		public function get_client_list(&$query = array()) {
			$where = array();
			if($query['status']) {
				$where['status'] = $query['status'];
			}
			if($query['area']) {
				$where['sales_area'] = $query['area'];
			}
			if($query['search']) {
				$where[] = "(perp_client.client_num LIKE " . $GLOBALS['egw']->db->quote('%'. $query['search'] . '%') . " OR
					lower(perp_client.client_name) LIKE lower('%" . addslashes($query['search']) . "%'))";
			}
			if($query['searchletter']) {
				$where[] = 'client_name LIKE \'' . $query['searchletter'] . '%\'';
			}
			if($query['client_id']) {
				$where['client_id'] = $query['client_id'];
			}

			// A range of clients, by client number.  But start & end are client_id.
			if($query['client_start']) {
				$start_client = $this->get_client($query['client_start']);
				$where[] = 'client_num >= ' . $GLOBALS['egw']->db->quote($start_client['client_num']);
			}
			if($query['client_end']) {
				$end_client = $this->get_client($query['client_end']);
				$where[] = 'client_num <= ' . $GLOBALS['egw']->db->quote($end_client['client_num']);
			}

			// Get record count
			$this->db->select('perp_client',
				array('count(client_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_client', array(
					'perp_client.client_id',
					'perp_client.client_num',
					'perp_client.client_name',
					'perp_client.address'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				"ORDER BY " . $query['order'] . " " . $query['sort'], False, $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
		
		public function get_client($client_id) {
			$this->db->select('perp_client', array(
					'perp_client.client_id',
					'client_num',
					'client_name',
					'address',
					'currency',
					'status',
					'sales_type',
					'sales_area',
					'client_since',
					'hold_reason',
					'payment_terms',
					'discount',
					'credit_limit',
					'invoice_branch',
					'user_account'
				),
				array('client_id' => $client_id),
				__LINE__, __FILE__
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list = $row;
			}
			return $list;
		}

		public function set_client($client_id, $client) {
			$insert_data = array(
				'client_num'	=>	$client['client_num'],
				'client_name'	=>	$client['client_name'],
				'address'	=>	$client['address'],
				'currency'	=>	$client['currency'],
				'status'	=>	$client['status'],
				'sales_type'	=>	$client['sales_type'],
				'sales_area'	=>	$client['sales_area'],
				'client_since'	=>	$client['client_since'],
				'hold_reason'	=>	$client['hold_reason'],
				'payment_terms'	=>	$client['payment_terms'],
				'discount'	=>	$client['discount'],
				'credit_limit'	=>	$client['credit_limit'],
				'invoice_branch'	=>	$client['invoice_branch'],
				'user_account'	=>	$client['user_account']
			);

			if(!$client_id) {
				$this->db->select('perp_client',
					array('client_name'),
					array(
						'client_name'  =>      $client['client_num'],
					),
					__LINE__, __FILE__
				);
				if($this->db->num_rows()) {
					return lang('Duplicate Client Number');
				}
			} else {
				$insert_data['client_id'] = $client_id;
			}

			$this->db->insert('perp_client', 
				$insert_data,
				array(
					'client_id'	=>	$client_id
				),
				__LINE__, __FILE__
			);
			$client_id = $client_id ? $client_id : $this->db->get_last_insert_id('perp_client', 'client_id');
			return $client_id;
		}

		public function delete_client($client_id) {
			$this->db->delete('perp_client_ledger', 
				array('client_id' => $client_id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_client_branch', 
				array('client_id' => $client_id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_client', 
				array('client_id' => $client_id),
				__LINE__, __FILE__
			);
		}

		public function suggest_client_number() {
			$config = config::read('perp_ar');
			$format = $config['client_number_format'];

			$sql = "SHOW TABLE STATUS LIKE 'perp_client'";
			$this->db->query($sql, __LINE__, __FILE__);
			$row = $this->db->row(True);
			$next_id = $row['Auto_increment'];
			$number = sprintf($format,
				$next_id
			);
			return $number;
		}

		public function get_ledger(&$query) {
			$where = array();
			if($query['client_id']) {
				if(is_array($query['client_id'])) {
					$where[] = 'client_id IN (' . $query['client_id'] . ')';
				} else {
					$where[] = 'client_id = ' . $query['client_id'];
				}
			}
			if($query['client_account_id']) {
				$where[] = 'client_account_id = ' . $query['client_account_id'];
			}
			if($query['start_date']) {
				$where[] = 'trans_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['start_date']));
			}
			if($query['end_date']) {
				$where[] = 'trans_date < ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp(strtotime('+1 day',$query['end_date'])));
			}

			$join = 'JOIN perp_general_ledger ON
					perp_general_ledger.trans_no = perp_client_ledger.trans_id AND
					perp_general_ledger.line_no = perp_client_ledger.line_no
				JOIN perp_account ON
					perp_general_ledger.account = perp_account.account_id
				JOIN perp_account_group ON
					perp_account.account_group = perp_account_group.group_id';

			$GLOBALS['egw']->db->select(
				'perp_client_ledger',
				array('count(trans_id) as count'),
				$where,
				__LINE__, __FILE__, 0, '','perp_ar', -1, $join
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['count'] = $row['count'];
			}
			$this->db->select('perp_client_ledger', array(
					'trans_id',
					'client_id',
					'our_reference',
					'their_reference',
					'sales_type',
					'trans_date',
					'account',
					'round(amount / exchange_rate * (abs(asset_or_liability) / asset_or_liability), 2) AS amount',
					'narrative',
					'exchange_rate',
					'link'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				"ORDER BY " . $query['order'] . ", trans_id " . $query['sort'], False, $query['num_rows'],
				$join
			);

			$list = array();
			while($row = $this->db->row(True)) {
				$row[($row['amount'] > 0 ? 'credit' : 'debit')] = abs($row['amount']);
				$row['formatted_' . ($row['amount'] > 0 ? 'credit' : 'debit')] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $query['currency'], abs($row['amount']));
				list($app, $id) = explode(':', $row['link']);
				$title = egw_link::title($app, $id);
				$row['reference'] = array(
					'app'   =>      $app,
					'id'    =>      $id,
					'title' =>      $title
				);
				$list[] = $row;
			}
			return $list;
		}

		public function get_account_balance($client_id, $client_account_id, $as_at_date) {
			$where = array();
			if($client_id) {
				$where[] = 'perp_client_ledger.client_id = ' . $client_id;
			}
			if($client_account_id) {
				$where[] = 'perp_client_ledger.client_account_id = ' . $client_account_id;
			}
			
			$where[] = 'perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($as_at_date));

			$this->db->select('perp_client_ledger', array(
					'-sum(round(amount / exchange_rate * (abs(asset_or_liability) / asset_or_liability), 2)) AS amount',
				),
				$where,
				__LINE__, __FILE__, False,
				"", False, 0,
				'JOIN perp_general_ledger ON
					perp_general_ledger.trans_no = perp_client_ledger.trans_id AND
					perp_general_ledger.line_no = perp_client_ledger.line_no
				JOIN perp_account ON
					perp_general_ledger.account = perp_account.account_id
				JOIN perp_account_group ON
					perp_account.account_group = perp_account_group.group_id'
			);
			$balance = 0;
			if($row = $this->db->row(True)) {
				$balance = $row['amount'];
			}
			return $balance ? $balance : 0;
		}

		public function link_title($client_id) {
			$this->db->select('perp_client',
				array('client_id', 'client_num', 'client_name'),
				array('client_id' => $client_id),
				__LINE__, __FILE__
			);
			
			if($row = $this->db->row(True)) {
				return $row['client_num'].' - '.$row['client_name'];
			}
			return false;
		}

		public function link_query($pattern) {
			$query_str = 'SELECT 
					client_id, client_num, client_name 
				FROM perp_client 
				WHERE client_num LIKE '.$this->db->quote('%'.$pattern.'%').' OR 
					client_name LIKE '.$this->db->quote('%'.$pattern.'%');  
				
			$this->db->query($query_str);	
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function get_client_by_user_account($account_id) {
			$GLOBALS['egw']->db->select(
				'perp_client',
				array('client_id'),
				array('user_account'	=>	$account_id),
				__LINE__, __FILE__, false,
				'', 'perp_ar'
			);
			$client_ids = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$client_ids[] = $row['client_id'];
			}
			$GLOBALS['egw']->db->select(
				'perp_client_branch',
				array('client_id'),
				array('user_account'	=>	$account_id),
				__LINE__, __FILE__, false,
				'', 'perp_ar'
			);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$client_ids[] = $row['client_id'];
			}
			return array_unique($client_ids);
		}
	}
?>
