<?php
	class so_perp_client_account {
		
		private $db;

		public function so_perp_client_account() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_ar');
		}

		public function get_client_account_list(&$query) {
			$where = array();
			if($query['search']) {
				$where[] = "(lower(client_account_name) LIKE lower('%" . $query['search'] . "%') OR
						lower(client_name) LIKE lower('%" . $query['search'] . "%') OR
						lower(client_num) LIKE lower('%" . $query['search'] . "%')
					)";
			}
			if($query['client_id']) {
				if(is_array($query['client_id'])) {
					$where[] = 'perp_client_account.client_id IN (' . $GLOBALS['egw']->db->quote(implode(',', $query['client_id'])) . ')';
				} else {
					$where[] = 'perp_client_account.client_id = ' . $query['client_id'];
				}
			}
			if($query['status']) {
				if($query['status'] == 'current') {
					$where[] = 'disabled_date is null';
				} elseif ($query['status'] == 'disabled') {
					$where[] = 'disabled_date is not null';
				}
			}

			$fields = array(
				'perp_client_account.client_id',
				'perp_client_account.client_account_id',
				'client_account_name',
				'disabled_date',
				'perp_client_account.payment_terms',
				'perp_client_account.credit_limit',
				'perp_client_account.hold_reason',
				'discount_percent'
			);
			if($account_balance) {
				$fields[] = 'sum(amount) AS account_balance';
			}

			// Get number of rows
			$this->db->select('perp_client_account',
				array('count(perp_client_account.client_account_id) AS count'),
				$where, __LINE__, __FILE__, 0, '',
				'perp_ar', -1,
				'JOIN perp_client ON
					perp_client_account.client_id = perp_client.client_id'
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}


			$this->db->select('perp_client_account', 
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				($query['account_balance'] ? 
					' GROUP BY perp_client_account.client_account_id ' : '') . 
				"ORDER BY " . $query['order'] . ' ' .  $query['sort'], 
				'perp_ar', $query['num_rows'],
				'JOIN perp_client ON
					perp_client_account.client_id = perp_client.client_id' .
				($query['account_balance'] ?
				'LEFT JOIN perp_client_ledger ON 
					perp_client_ledger.client_account_id = perp_client_account.client_account_id
				LEFT JOIN perp_general_ledger ON
					perp_client_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_client_ledger.line_no = perp_general_ledger.line_no' : '')
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$row['disabled_date'] = $GLOBALS['egw']->db->from_timestamp($row['disabled_date']);
				$list[] = $row;
			}

			return $list;
		}

		public function get_client_account($client_account_id) {
			$this->db->select('perp_client_account', 
				array('*'),
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);
			$list = array();
			if($row = $this->db->row(True)) {
				$list = $row;
			}
			$list['disabled_date'] = $GLOBALS['egw']->db->from_timestamp($list['disabled_date']);
			return $list;
		}

		public function set_client_account($client_account_id, $data) {
			$insert_data = array(
				'client_id'		=>	$data['client_id'],
				'client_account_name'	=>	$data['client_account_name'],
				'payment_terms'		=>	$data['payment_terms'],
				'credit_limit'		=>	$data['credit_limit'],
				'hold_reason'		=>	$data['hold_reason'],
				'discount_percent'	=>	$data['discount_percent']
			);
			$where = array();
			
			if($client_account_id) {
				$insert_data['client_account_id'] = $client_account_id;
				$where['client_account_id'] = $client_account_id;
			}

			$this->db->insert('perp_client_account', 
				$insert_data,
				$where,
				__LINE__, __FILE__
			);
			$id = $client_account_id ? $client_account_id : $this->db->get_last_insert_id('perp_client_account', 'client_account_id'); 

			if($data['price_lists']) {
				if(!is_array($data['price_lists'])) {
					$data['price_lists'] = explode(',', $data['price_lists']);
				}
			}
			$this->set_price_lists($id, $data['price_lists']);
			if($data['discounts']) {
				$this->set_discounts($id, $data['discounts']);
			}
			return $id;
		}

		public function disable_account($client_account_id) {
			$GLOBALS['egw']->db->update(
				'perp_client_account',
				array(
					'disabled_date'		=>	$GLOBALS['egw']->db->to_timestamp(time())
				),
				array(
					'client_account_id'	=>	$client_account_id
				),
				__LINE__, __FILE__, 'perp_ar'
			);
		}

		public function delete_client_account($client_account_id) {
			// Check for Sales Orders
			$orders = ExecMethod('perp_orders.so_perp_sales_order.get_order_list', array(
				'order'		=>	'order_id',
				'client_account_id' => $client_account_id,
				'num_rows'	=>	1
			));
			if(count($orders) > 0) {
				return lang('Unable to delete.  Account has orders.');
			}
			$this->db->select('perp_client_ledger', 
				array('count(trans_id) AS count'),
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);
			$row = $this->db->row(True);
			if($row['count']) {
				$account = $this->get_client_account($client_account_id);
				$account['status'] = 3;
				return lang('Unable to delete.  Account has transactions.');
			}
			$this->db->delete('perp_client_account',
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_client_ledger',
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_client_account_price_list',
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_client_account_discount',
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);
			return True;
		}

		public function get_price_lists($client_account_id, $date) {
			$where = array(
				'client_account_id = ' . (int)$client_account_id,
				'(effective_to IS NULL OR effective_to > \'' . $GLOBALS['egw']->db->to_timestamp($date) . '\')'
			);

			// Restrict price lists explicitly, or by sales area and sales type
			$config = config::read('perp_ar');
			if($config['individual_client_discounts']) {
				$join = 'JOIN perp_client_account_price_list ON
					perp_client_account_price_list.price_list_id = perp_price_list.price_list_id ';
			} else {
				$join = 'JOIN perp_client ON
						perp_client.status = 1 AND (
							perp_price_list.sales_area_id = perp_client.sales_area OR
							perp_client.sales_area = 0
						) AND (
							perp_price_list.sales_type_id = perp_client.sales_type OR
							perp_client.sales_type = 0
						)
					JOIN perp_client_account ON
						perp_client.client_id = perp_client_account.client_id ';
			}
				

			$result = $GLOBALS['egw']->db->select(
				'perp_price_list',
				array('perp_price_list.price_list_id'),
				$where,
				__LINE__, __FILE__, false,
				'', 'perp_ar', -1,
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row['price_list_id'];
			}
			return $list;
		}

		public function set_price_lists($client_account_id, $price_lists) {
			$this->db->delete('perp_client_account_price_list',
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);

			if(!is_array($price_lists)) {
				return;
			}

			foreach($price_lists as $id) {
				$this->db->insert('perp_client_account_price_list',
					array(
						'client_account_id'	=>	$client_account_id,
						'price_list_id'		=>	$id
					),
					array(),
					__LINE__, __FILE__
				);
			}
		}

		public function get_discounts($client_account_id, $sales_category = '') {
			$sql = 'SELECT 
					egw_categories.cat_name AS sales_category_name,
					perp_sales_category.sales_category_id,
					discount_amount,
					discount_percent,
					narrative,
					account_id
				FROM perp_sales_category
				JOIN egw_categories ON
					perp_sales_category.sales_category_id = egw_categories.cat_id
				LEFT JOIN perp_client_account_discount ON 
					perp_client_account_discount.sales_category_id = perp_sales_category.sales_category_id AND
					perp_client_account_discount.client_account_id=' . $client_account_id;

			if($sales_category) {
				$sql .= ' WHERE perp_sales_category.sales_category_id ';
				if(is_array($sales_category)) {
					$sql .= 'IN ( ' . implode(',',$sales_category) . ')';
				} else {
					$sql .= ' = ' . $sales_category;
				}
			}

			$this->db->query($sql, __LINE__, __FILE__);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function set_discounts($client_account_id, $discounts) {
			$this->db->delete('perp_client_account_discount',
				array('client_account_id' => $client_account_id),
				__LINE__, __FILE__
			);

			foreach($discounts as $discount) {
				if($discount['discount_amount'] || $discount['discount_percent']) {
					$this->db->insert('perp_client_account_discount',
						array(
							'client_account_id'	=>	$client_account_id,
							'sales_category_id'	=>	$discount['sales_category_id'],
							'discount_amount'	=>	$discount['discount_amount'],
							'discount_percent'	=>	$discount['discount_percent'],
							'narrative'		=>	$discount['narrative'],
							'account_id'		=>	$discount['account_id']
						),
						array(),
						__LINE__, __FILE__
					);
				}
			}
		}
	}
?>
