<?php

	/**
	*	Data access layer for outgoing shipments.  This extends perp_api::so_perp_shipment.
	*
	*	@author Nathan Gray
	*	@ingroup perp_ar
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_api/inc/class.so_perp_shipment.inc.php');

	class so_perp_outgoing_shipment extends so_perp_shipment {

		public function __construct() {
			parent::__construct();
		}

		public function get_shipment_list(&$query) {
			$where = array();
			if(isset($query['supplier_id']) && is_numeric($query['supplier_id'])) {
				$where[] = 'perp_shipment.supplier_id = ' . $query['supplier_id'];
			}
			if($query['client_id'] && is_numeric($query['client_id'])) {
				$where[] = 'perp_shipment.client_id = ' .$query['client_id'];
			} elseif($query['client_id'] && is_array($query['client_id'])) {
				$where[] = 'perp_shipment.client_id IN (' . implode(',', $query['client_id']) . ')';
			}
			if($query['client_branch_id'] && is_numeric($query['client_branch_id'])) {
				$where[] = 'perp_client_shipment.client_branch_id = ' . $query['client_branch_id'];
			}
			if($query['order_id'] && is_numeric($query['order_id'])) {
				$where[] = 'perp_shipment_line.shippable_id = ' . $query['order_id'];
			}
			if(trim($query['search'])) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(
					our_reference_number LIKE $search OR
					actual_delivery_date = $search)";
			}

			// Filter on dates
			if(isset($query['issue_date'])) {
				if(is_array($query['issue_date'])) {
					if($query['issue_date']['start']) {
						$where[] = 'perp_shipment.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['start']));
					}
					if($query['issue_date']['end']) {
						$where[] = 'perp_shipment.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['end']));
					}
				} else {
					$where[] = 'perp_shipment.issue_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']));
				}
			}
			if(isset($query['originating_date'])) {
				if(is_array($query['originating_date'])) {
					if($query['originating_date']['start']) {
						$where[] = 'perp_shipment.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['start']));
					}
					if($query['originating_date']['end']) {
						$where[] = 'perp_shipment.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['end']));
					}
				} else {
					$where[] = 'perp_shipment.originating_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']));
				}
			}
			if(isset($query['requested_delivery_date'])) {
				if(is_array($query['requested_delivery_date'])) {
					if($query['requested_delivery_date']['start']) {
						$where[] = 'perp_shipment.requested_delivery_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['requested_delivery_date']['start']));
					}
					if($query['requested_delivery_date']['end']) {
						$where[] = 'perp_shipment.requested_delivery_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['requested_delivery_date']['end']));
					}
				} else {
					$where[] = 'perp_shipment.requested_delivery_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['requested_delivery_date']));
				}
			}

			if($query['status']) {
				switch($query['status']) {
				case 'pending':
					$where[] = 'perp_shipment.approval_date IS NULL';
					break;
				case 'approved':
					$where[] = 'perp_shipment.approval_date IS NOT NULL AND
						perp_shipment.actual_delivery_date IS NULL';
					break;
				case 'shipped':
					$where[] = 'perp_shipment.actual_delivery_date IS NOT NULL';
					break;
				case 'overdue':
					$where[] = 'perp_shipment.approval_date IS NOT NULL';
					$where[] = "requested_delivery_date < NOW() ";
					$where[] = 'actual_delivery_date IS NULL';
					break;
				}
			}

			// Get number for results
			$GLOBALS['egw']->db->select('perp_shipment',
				array('count(perp_shipment.shipment_id) AS count'),
				$where,
				__LINE__, __FILE__, false,
				false, false, 0,
				'JOIN perp_client ON
					perp_shipment.client_id = perp_client.client_id
				LEFT JOIN perp_client_shipment ON
					perp_client_shipment.shipment_id = perp_shipment.shipment_id'
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$GLOBALS['egw']->db->select('perp_shipment',
				array(
					'perp_shipment.shipment_id',
					'perp_shipment.our_reference_number',
					'perp_shipment.their_reference_number',
					'perp_shipment.client_id',
					'perp_shipment.requested_delivery_date',
					'perp_shipment.actual_delivery_date',
					'perp_shipment.vessel',
					'perp_shipment.requires_approval',
					'perp_client.client_num AS client_number',
					'perp_client.client_name',
					'perp_shipment.shipper',
					'perp_shipment.ship_to_address',
					'perp_client.currency',
					'sum(quantity_ordered * unit_price) AS value'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				"GROUP BY perp_shipment.shipment_id
				ORDER BY ". $query['order'] . ' ' .  $query['sort'], 'perp_ar', $query['num_rows'],
				'JOIN perp_client ON 
					perp_client.client_id = perp_shipment.client_id
				JOIN perp_shipment_line ON
					perp_shipment.shipment_id = perp_shipment_line.shipment_id
				JOIN perp_order_line ON
					perp_shipment_line.shippable_id = perp_order_line.order_id AND
					perp_shipment_line.shippable_item_line = perp_order_line.line_no
				LEFT JOIN perp_client_shipment ON
					perp_client_shipment.shipment_id = perp_shipment.shipment_id'
			);
			if(!isset($GLOBALS['perp_api'])) {
				$GLOBALS['perp_api']['bo'] = CreateObject('perp_api.bo_perp_api');;
			}
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['requested_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['requested_delivery_date']);
				$row['actual_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['actual_delivery_date']);
				$row['formatted_value'] = $GLOBALS['perp_api']['bo']->get_formatted_price($row['currency'], $row['value']);
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Get a list of items that can be shipped for a given client.
		*	The list includes all items on order, seperated into in stock and
		*	not in stock, so shipments can be created and scheduled before 
		*	production is finished.
		*	This takes several sub-queries, which are joined up into one
		*	query.
		*
		*	query parameters:
		*		client_id	=>	Restrict to a client
		*		order_id	=>	Restrict to an order
		*		shipment_id	=>	Restrict to a shipment
		*		location_id	=>	Restrict to a source location
		*/
		public function get_shipable_items(&$query) {
			// Keep a list of sales order IDs that have something unshipped 
			$unshipped_orders = array();
			
			// List of stock IDs that have not been shipped
			$unshipped_stock_ids = array();


			$where = array();
			if($query['client_id'] && is_numeric($query['client_id'])) {
				$where[] = 'shippable_items.client_id = ' . $query['client_id'];
			} 
			if($query['shipment_id']) {
				$where[] = 'shipment_id = ' . $query['shipment_id'];
			}
			if($query['order_id']) {
				$where[] = 'order_id = ' . $query['order_id'];
			}

			// Items that have been ordered but not shipped
			$shippable_items_sql = 'SELECT                                        
				perp_order_line.order_id,                                        
				perp_order.our_reference_number,
				perp_order.ship_to_address,
				perp_order.client_id,
				shipment_total.shipment_id,
				perp_order.requested_delivery_date,
				perp_order_line.line_no,
				perp_order_line.description,
				perp_order_line.ship_location,
				perp_stock.stock_id,
				perp_stock.stock_num,
				perp_stock.stock_type_flag,
				perp_stock.stock_category AS stock_category_id,
				egw_categories.cat_name AS stock_category_name,
				perp_stock.decimal_places,
				perp_order_line.quantity_ordered,
				shipment_total.quantity_shipped,
				perp_order_line.quantity_ordered - IF(
					shipment_total.quantity_shipped IS NULL,
					0,
					shipment_total.quantity_shipped
				) AS quantity_unshipped
			FROM perp_order_line
			JOIN perp_order ON
				perp_order_line.order_id = perp_order.order_id
			JOIN perp_sales_order ON
				perp_order.order_id = perp_sales_order.order_id
			LEFT JOIN perp_order_detail ON
				perp_order_detail.order_id = perp_order_line.order_id AND
				perp_order_detail.line_no = perp_order_line.line_no	AND
			        perp_order_detail.detail_name = \'work_order\'
			LEFT JOIN (
				SELECT
					shipment_id,
					SUM(IF(quantity_shipped IS NULL, 0, quantity_shipped)) AS quantity_shipped,
					shippable_id,
					shippable_item_line
				FROM perp_shipment_line
				GROUP BY shippable_id, shippable_item_line
			) AS shipment_total ON
				perp_order_line.order_id = shipment_total.shippable_id AND
				perp_order_line.line_no = shipment_total.shippable_item_line
			JOIN perp_stock ON
				perp_order_line.stock_id = perp_stock.stock_id
			LEFT JOIN egw_categories ON
				perp_stock.stock_category = egw_categories.cat_id ';

			if($query['location_id']) {
				$shippable_items_sql .= 'WHERE perp_order_line.ship_location IN (0, ' . $query['location_id'] .' )';
			}
			$shippable_items_sql .= ' HAVING quantity_unshipped != 0';

			$result =& $GLOBALS['egw']->db->query($shippable_items_sql, __LINE__, __FILE__);
			$shippable_items = array();
			foreach($result as $row) {
				$shippable_items[] = $row;
				$unshipped_orders[] = $row['order_id'];
				$unshipped_stock_ids[] = $row['stock_id'];
			}

			$unshipped_orders = array_unique($unshipped_orders);
			$unshipped_stock_ids = array_unique($unshipped_stock_ids);

			// Items that have been ordered and are in stock
			$in_stock_sql = 'SELECT
				perp_order_line.order_id,
				perp_order_line.line_no,
				perp_location.location_id,
				perp_location.location_name,
				perp_location.delivery_address AS ship_from_address,
				null AS serial_number, 
				' . ($query['location_id'] ? 'perp_location_stock.quantity_on_hand' : 'SUM(perp_location_stock.quantity_on_hand) AS quantity_on_hand') . '
			FROM perp_order_line
			JOIN perp_stock ON
				perp_order_line.stock_id = perp_stock.stock_id AND
				perp_stock.stock_type_flag = \'R\'
			JOIN (
				perp_location 
				JOIN perp_location_stock ON 
					perp_location_stock.location_id = perp_location.location_id AND
					perp_location_stock.quantity_on_hand != 0
			) ON 
				perp_order_line.stock_id = perp_location_stock.stock_id AND (
					(perp_order_line.ship_location = perp_location.location_id ';
			if($query['location_id']) {
				$in_stock_sql .= ' OR perp_order_line.ship_location = 0) AND perp_location.location_id = ' . $query['location_id'];
			} else {
				$in_stock_sql .= ')
					OR (
						perp_location_stock.quantity_on_hand > 0
					) ';
			}
			$in_stock_sql .= '
				)
			';
			if(!$query['location_id']) {
				$in_stock_sql .= 'GROUP BY perp_order_line.order_id, perp_order_line.line_no ';
			}

			if($GLOBALS['egw_info']['apps']['perp_manufacturing']) {
				// Items involving a work order that are in stock
				$work_order_in_stock_sql = 'SELECT
					perp_order_line.order_id,
					perp_order_line.line_no,
					perp_location.location_id,
					perp_location.location_name,
					perp_location.delivery_address AS ship_from_address,
					perp_stock_serial.serial_number,
					perp_stock_serial.quantity AS quantity_on_hand
				FROM perp_order_line
				JOIN perp_order_detail ON
					perp_order_detail.order_id = perp_order_line.order_id AND
					perp_order_detail.line_no = perp_order_line.line_no
				JOIN perp_sales_order_work_order ON
					perp_order_detail.order_id = perp_sales_order_work_order.order_id AND
					perp_order_detail.line_no = perp_sales_order_work_order.line_no AND
					perp_order_detail.detail_no = perp_sales_order_work_order.detail_no
				LEFT JOIN perp_stock_serial ON
					perp_order_detail.detail_value = perp_stock_serial.work_order_id AND
					perp_sales_order_work_order.work_order_detail_id = perp_stock_serial.detail_id
				LEFT JOIN perp_location ON
					perp_stock_serial.location_id = perp_location.location_id';
			}


			// Items that are not in stock and are purchased
			$purchase_eta_sql = 'SELECT
				perp_order_line.order_id,
				perp_order_line.line_no,
				purchase_order_line.order_id AS incoming_order_id,
				purchase_order.our_reference_number AS incoming_order_reference,
				purchase_order.requested_delivery_date AS eta,
			/**
			*	Depending on how you want your expected calculated, use either
			*	ordered or shipped
				(perp_shipment_line.quantity_shipped - perp_goods_received.quantity_units) * perp_supplier_stock.conversion_factor AS quantity_expected,
			*/
				(purchase_order_line.quantity_ordered - IF(
					received.quantity IS NULL, 
					0, 
					received.quantity
				)) * perp_supplier_stock.conversion_factor AS quantity_expected,
				perp_location.location_id,
				perp_location.location_name,
				purchase_order.ship_to_address AS ship_from_address
			FROM perp_order_line
			JOIN perp_supplier_stock ON
				perp_supplier_stock.stock_id = perp_order_line.stock_id
			JOIN (
				perp_order_line AS purchase_order_line 
				LEFT JOIN (
					SELECT 
						shippable_id, shippable_item_line,
						SUM(quantity) AS quantity
					FROM
						perp_goods_received 
					LEFT JOIN perp_stock_cost ON
						perp_goods_received.stock_cost_id = perp_stock_cost.stock_cost_id
					/* If you only want goods that have shipped, use a JOIN
					* JOIN perp_shipment_line ON
					*/
					LEFT JOIN perp_shipment_line ON
						perp_shipment_line.shipment_id = perp_goods_received.shipment_id AND
						perp_shipment_line.line_no = perp_goods_received.line_no
					GROUP BY shippable_id, shippable_item_line
				) AS received ON
					purchase_order_line.order_id = received.shippable_id AND
					purchase_order_line.line_no = received.shippable_item_line
			) ON
				perp_supplier_stock.supplier_code = purchase_order_line.supplier_stock_code AND
				(received.quantity < purchase_order_line.quantity_ordered OR
				received.quantity IS NULL)
			JOIN perp_order AS purchase_order ON
				purchase_order.order_id = purchase_order_line.order_id
			JOIN perp_location ON
				purchase_order.ship_to_address = perp_location.delivery_address
			WHERE perp_order_line.order_id IN (' . implode(',', $unshipped_orders) . ')
			HAVING quantity_expected > 0';

			if($GLOBALS['egw_info']['apps']['perp_manufacturing']) {
				// Items that are not in stock and are manufactured
				$work_order_eta_sql = 'SELECT
					perp_order_line.order_id,
					perp_order_line.line_no,
					perp_work_order.work_order_id AS incoming_order_id,
					perp_work_order.work_order_reference AS incoming_order_reference,
					perp_work_order.required_date AS eta,
					perp_work_order_detail.quantity AS quantity_expected,
					perp_location.location_id,
					perp_location.location_name,
					perp_location.delivery_address AS ship_from_address
				FROM perp_order_line
				JOIN perp_order_detail ON
					perp_order_detail.order_id = perp_order_line.order_id AND
					perp_order_detail.line_no = perp_order_line.line_no
				JOIN perp_sales_order_work_order ON
					perp_order_detail.order_id = perp_sales_order_work_order.order_id AND
					perp_order_detail.line_no = perp_sales_order_work_order.line_no AND
					perp_order_detail.detail_no = perp_sales_order_work_order.detail_no
				JOIN perp_work_order_detail ON
					perp_work_order_detail.work_order_id = perp_order_detail.detail_value AND
					perp_work_order_detail.detail_id = perp_sales_order_work_order.work_order_detail_id 
				JOIN perp_work_order ON
					perp_work_order.work_order_id = perp_work_order_detail.work_order_id
				JOIN perp_location ON
					perp_work_order.location = perp_location.location_id
				WHERE perp_order_line.order_id IN (' . implode(',', $unshipped_orders) . ')';
			}

			// Join it all up
			$sql = "SELECT 
				shippable_items.client_id,
				perp_client.client_name,
				perp_client.client_num,
				shippable_items.order_id,
				shippable_items.our_reference_number,
				IF(in_stock.ship_from_address IS NULL, eta.ship_from_address, in_stock.ship_from_address) AS ship_from_address,
				shippable_items.ship_to_address,
				shippable_items.line_no,
				shippable_items.stock_id,
				shippable_items.stock_num,
				shippable_items.description,
				in_stock.serial_number,
				shippable_items.decimal_places,
				shippable_items.stock_category_id,
				shippable_items.stock_category_name,
				shippable_items.stock_type_flag,
				shippable_items.quantity_ordered,
				shippable_items.quantity_shipped, 
				shippable_items.stock_type_flag,
				IF(in_stock.location_id IS NULL, eta.location_id, in_stock.location_id) AS location_id,
				IF(in_stock.location_name IS NULL, eta.location_name, in_stock.location_name) AS location_name,
				in_stock.quantity_on_hand,
				eta.incoming_order_id,
				eta.incoming_order_reference,
				eta.quantity_expected,
				eta.eta,
				shippable_items.quantity_unshipped
			FROM ($shippable_items_sql) AS shippable_items
			JOIN perp_client ON
				shippable_items.client_id = perp_client.client_id
			LEFT JOIN (
				$in_stock_sql";
			if($work_order_in_stock_sql) {
				$sql .= "
				UNION
				$work_order_in_stock_sql";
			}
			$sql .= "
			) AS in_stock ON
				in_stock.order_id = shippable_items.order_id AND
				in_stock.line_no = shippable_items.line_no
			LEFT JOIN (
				SELECT *
				FROM (
					$purchase_eta_sql";
			if($work_order_eta_sql) {
				$sql .= "
					UNION
					$work_order_eta_sql";
			}
			$sql .="
				) as incoming
				ORDER BY eta
			) AS eta ON
				eta.order_id = shippable_items.order_id AND
				eta.line_no = shippable_items.line_no ";

			if(count($where) > 0) {
				$sql .= 'WHERE ' . implode(' AND ', $where);
			}
			$sql .= "
			ORDER BY shippable_items.requested_delivery_date, shippable_items.line_no";

			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__, $query['start'], $query['num_rows']);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				if($row['work_order_id'] && !$row['completed_date']) {
					$row['to_ship'] = 0;
				}
				$row['eta'] = $GLOBALS['egw']->db->from_timestamp($row['eta']);

				// Only display the appropriate number of decimals
				$row['quantity_ordered'] = round($row['quantity_ordered'], $row['decimal_places']);
				$row['quantity_shipped'] = round($row['quantity_shipped'], $row['decimal_places']);
				$row['quantity_on_hand'] = round($row['quantity_on_hand'], $row['decimal_places']);
				$row['quantity_expected'] = round($row['quantity_expected'], $row['decimal_places']);
				$row['quantity_unshipped'] = round($row['quantity_unshipped'], $row['decimal_places']);
				$list[] = $row;
			}
			return $list;
		}

		/* The shipment goes out the door
		*	Stock is taken out of inventory, shipment is updated & locked
		*/
		public function ship($shipment_id, $shipped_by) {
			$shipment = $this->get_shipment($shipment_id);
			$config = config::read('perp_ar');
			$cogs_account = $config['cogs_account'];

			$GLOBALS['egw']->db->transaction_begin();
			$transaction = CreateObject('perp_ledger.GLTransaction');
			$complete = True;

			$items = $this->get_shipment_items($shipment_id, '', 'client_shipment_id');

			foreach($items as $item) {
				// Take stock out of inventory
				$cost = ExecMethod2('perp_inventory.bo_perp_inventory.ship', $item['stock_id'], $shipment['from_location'], $item['quantity']);

				// Update shipment
				$GLOBALS['egw']->db->update('perp_client_shipment_detail', 
					array('shipped_date' => date('Y-m-d')),
					array(
						'shipment_id'	=>	$shipment_id,
						'sales_order_id'	=>	$item['sales_order_id'],
						'line_no'	=>	$item['line_no']
					),
					__LINE__, __FILE__
				);

				// Update Sales Order
				$so = CreateObject('perp_orders.SalesOrder', $item['sales_order_id']);
				// TODO: this will commit the transaction, which is not good
				$so->ship($item['sales_order_id'], $item['line_no'], $item['quantity']);

				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $item['stock_id']);
				$category = ExecMethod('perp_inventory.bo_perp_location.get_gl_accounts', array('location_id' => $shipment['from_location'], 'stock_category_id' => $stock['stock_category']));

				// Financial 
				// Reduce inventory assets
				$transaction->add_line($category['stock_gl_account'], lang('Shipment %1 - shipped %2 of %3 for SO %4', $shipment['shipment_num'], $item['quantity'], $stock['stock_num'], $item['sales_order_num']), $cost);

				// Increase cost of sales
				$transaction->add_line($cogs_account, lang('Shipment %1 - shipped %2 of %3 for SO %4', $shipment['shipment_num'], $item['quantity'], $stock['stock_num'], $item['sales_order_num']), -$cost);
				
			}

			if($this->debug) {
				echo 'Transaction before commit: <pre>';
				echo $transaction;
				echo '</pre>';
			}
			if($complete) {
				$GLOBALS['egw']->db->update('perp_client_shipment',
					array(
						'actual_ship_date' => date('Y-m-d'),
						'shipped_by'	=>	$shipped_by
					),
					array('client_shipment_id' => $shipment_id),
					__LINE__, __FILE__
				);
			}

			// TODO: Trying to nest transactions again, which will actually commit the everything until now.
			$result = $transaction->write();
			
			if(is_numeric($result)) {
				return True;
			} else {
				return $result;
			}
		}

		public function get_packing_slip($shipment_id) {
			$shipment = new OutgoingShipment($shipment_id);
			if(!$shipment) {
				return array();
			}
			$data = $shipment->toArray();
			$data['ship_date'] = $data['actual_delivery_date'] ? $data['actual_delivery_date'] : $data['requested_delivery_date'];

			$data['client'] = ExecMethod('perp_ar.bo_perp_client.get_client', $shipment->get_client_id());

			// Get items
			$sql = "SELECT 
					perp_order.our_reference_number,
					perp_order.their_reference_number,
					egw_categories.cat_name AS stock_category_name,
					perp_shipment_line.line_no,
					perp_stock.stock_id,
					perp_stock.stock_num,
					perp_stock.short_description AS description,
					ROUND(perp_stock_ship_package.piece_count * perp_shipment_line.quantity_shipped, perp_stock.decimal_places) AS piece_count,
					perp_stock_ship_package.mass,
					perp_stock_ship_package.volume,
					perp_stock_serial.serial_number,
					ROUND(perp_shipment_line.quantity_shipped, perp_stock.decimal_places) AS quantity_shipped
				FROM perp_shipment_line
				JOIN perp_order_line ON
				    perp_shipment_line.shippable_id = perp_order_line.order_id AND
				    perp_shipment_line.shippable_item_line = perp_order_line.line_no
				JOIN perp_order ON
				    perp_order.order_id = perp_order_line.order_id
				LEFT JOIN (
					perp_sales_order_work_order JOIN
					perp_order_detail ON
						perp_sales_order_work_order.order_id = perp_order_detail.order_id AND
						perp_sales_order_work_order.line_no = perp_order_detail.line_no AND
						perp_sales_order_work_order.detail_no = perp_order_detail.detail_no
					) ON
					perp_order_line.order_id = perp_sales_order_work_order.order_id AND
					perp_order_line.line_no = perp_sales_order_work_order.line_no
				LEFT JOIN perp_stock_serial ON
				    perp_order_detail.detail_value = perp_stock_serial.work_order_id AND
				    perp_order_line.stock_id = perp_stock_serial.stock_id
				JOIN perp_stock ON
				    perp_order_line.stock_id = perp_stock.stock_id
				JOIN (
					SELECT
						stock_id,
						COUNT(stock_id) AS piece_count,
						SUM(mass) AS mass,
						SUM(length * width * height) AS volume
					FROM perp_stock_ship_package
					GROUP BY stock_id
				) AS perp_stock_ship_package ON
					perp_stock.stock_id = perp_stock_ship_package.stock_id
				JOIN egw_categories ON
					perp_stock.stock_category = egw_categories.cat_id
				WHERE perp_shipment_line.shipment_id = $shipment_id
				ORDER BY perp_order.our_reference_number, perp_stock.stock_num, perp_stock_serial.serial_number
			";
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			$items = array();
			$item_count = 0;
			$piece_count = 0;
			$ship_mass = 0;
			$ship_volume = 0;
			$category = 0;
			$count = -1;
			$data = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				if($category !== $row['stock_category_name']) {
					$count++;
					$items[$count]['stock_category_name'] = $row['stock_category_name'];
					$category = $row['stock_category_name'];
					$items[$count]['items'][] = false;
				}
				$items[$count]['items'][] = $row;
				$item_count += $row['quantity_shipped'];
				$piece_count += $row['piece_count'];
				$ship_mass += $row['mass'] * $row['quantity_shipped'];
				$ship_volume += $row['volume'] * $row['quantity_shipped'];
			}



			$data['category'] = $items;
			$data['item_count'] = $item_count;
			$data['piece_count'] = $piece_count;
			$data['ship_mass'] = $ship_mass;
			$data['ship_volume'] = $ship_volume;

			$data['container_count'] = $GLOBALS['egw']->db->query(
				'SELECT
					true
				    FROM perp_shipment_detail
				    WHERE shipment_id = ' . (int)$shipment_id . '
					AND detail_class = \'ShipmentPackage\'
				    GROUP BY shipment_id, detail_name',
				__LINE__, __FILE__
			)->numRows();

			return $data;
		}
	}
?>
