<?php
	require_once('class.bo_perp_outgoing_shipment.inc.php');

	class ui_perp_client {

		public $public_functions = array(
			'client_list'	=>	True,
			'client_edit'	=>	True,
			'client_new'	=>	True,
			'on_order'	=>	true,
		);

		private $debug = false;
		private $template;
		private $bo;
		private $api;

		private $messages = array();

		private static $tabs = 'details|accounts|locations|links|orders|shipments|invoices|warranty|ledger';

		private $client_read;
		private $client_add;
		private $client_edit;
		private $client_delete;

		public function ui_perp_client() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_ar.bo_perp_client');
			$this->api = CreateObject('perp_api.ui_perp_api');

			$this->client_read = $GLOBALS['egw']->acl->check('client', EGW_ACL_READ, 'perp_ar');
			$this->client_add = $GLOBALS['egw']->acl->check('client', EGW_ACL_ADD, 'perp_ar');
			$this->client_edit = $GLOBALS['egw']->acl->check('client', EGW_ACL_EDIT, 'perp_ar');
			$this->client_delete = $GLOBALS['egw']->acl->check('client', EGW_ACL_DELETE, 'perp_ar');
			$this->account_read = $GLOBALS['egw']->acl->check('client_account', EGW_ACL_READ, 'perp_ar');
			$this->account_add = $GLOBALS['egw']->acl->check('client_account', EGW_ACL_ADD, 'perp_ar');
			$this->account_edit = $GLOBALS['egw']->acl->check('client_account', EGW_ACL_EDIT, 'perp_ar');
			$this->account_delete = $GLOBALS['egw']->acl->check('client_account', EGW_ACL_DELETE, 'perp_ar');
			$this->branch_read = $GLOBALS['egw']->acl->check('client_branch', EGW_ACL_READ, 'perp_ar');
			$this->branch_add = $GLOBALS['egw']->acl->check('client_branch', EGW_ACL_ADD, 'perp_ar');
			$this->ledger_read = $GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ar');

			if($this->debug) {
				printf("Client permissions: READ: %s, ADD: %s, EDIT: %s, DELETE: %s<br />", 
					($this->client_read ? 'Yes' : 'No'),
					($this->client_add ? 'Yes' : 'No'),
					($this->client_edit ? 'Yes' : 'No'),
					($this->client_delete ? 'Yes' : 'No')
				);
			}
		}

		public function client_list($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_client.client_list incoming data:';
				_debug_array($content);
			}

			// Deal with incoming
			if(isset($content['cancel']) || !$this->client_read) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ar.ui_perp_ar.index'));
			}
			if(isset($content['new_client']) && $this->client_add) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ar.ui_perp_client.client_new'));
			} elseif (isset($content['nm']['rows']['edit']) && $this->client_edit) {
				$id = key($content['nm']['rows']['edit']);
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ar.ui_perp_client.client_edit', 'client_id'=>$id));
			} elseif (isset($content['nm']['rows']['delete']) && $this->client_delete) {
				$id = key($content['nm']['rows']['delete']);
				$this->bo->delete_client($id);
			}

			// Set up outgoing
			$data['nm'] = $GLOBALS['egw']->session->appsession('client.nextmatch');
			if(!$data['nm']) {
				$data['nm'] = array(
					'no_cat'        =>      True,
					'filter_label'  =>      lang('Status'),
					'filter_no_lang'        =>      True,
					// Default status is active
					'filter'	=>	1,
					'filter2_label' =>      lang('Sales Area'),
					'filter2_no_lang'       =>      True,
					'lettersearch'	=>	true,
					'get_rows'      =>      'perp_ar.ui_perp_client.get_client_list',
					'template'      =>      'perp_ar.client_list.row',
				);
			}
			$select_box_options['filter'] = array('' => lang('All')) + $this->bo->get_status_list();
			$select_box_options['filter2'] = array('' => lang('All')) + ExecMethod('perp_orders.bo_perp_orders.get_sales_areas');

			// Hide areas user has no permission to
			$readonly['new_client'] = !$this->client_add;

			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->set_onload("
				var check = document.getElementById('exec[nm][include_branches]');  
				set_style_by_class('*', 'branch', 'display', (check.checked ? 'block' : 'none'));"
			);

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Client List');
			$this->template->read('perp_ar.client_list');
			$this->template->exec('perp_ar.ui_perp_client.client_list', $data, $select_box_options, $readonly, $preserve); 
		}

		public function client_new($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_client.client_new incoming data:';
				_debug_array($content);
			}

			$content['client_num'] = $this->bo->suggest_client_number();
			$content['new_client'] = true;
			$config = config::read('perp_ledger');
			$content['currency'] = $config['gl_currency'];
			return $this->client_edit($content);
		}

		public function client_edit($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_client.client_edit incoming data:';
				_debug_array($content);
			}
			
			$client_id = $content['client_id'] ? $content['client_id'] : (isset($_GET['client_id']) ? $_GET['client_id'] : False);
			$data = $content;


			/**********************
			* Deal with incoming
			**********************/

			if(isset($content['save']) && $this->client_edit) {
				unset($content['save']);
				$result = $this->bo->set_client($client_id, $content);
				if(is_numeric($result)) {
					$client_id = $result;
					$this->messages[] = array(lang('Saved'), 'success');

					if($content['accounts']['rows']) {
						foreach($content['accounts']['rows'] as $row) {
							$result = ExecMethod2('perp_ar.bo_perp_client_account.set_client_account', $row['client_account_id'], $row);
							if(!is_numeric) {
								$this->messages[] = array($result, 'error');
							}
						}
					}
				} else {
					$this->messages[] = array($result, 'error');
				}
				
			} elseif(isset($content['locations']['rows']['delete']) && is_array($content['locations']['rows']['delete'])) {
				foreach($content['locations']['rows']['delete'] as $id => $value) {
					if($id) {
						ExecMethod('perp_ar.bo_perp_client_branch.delete_client_branch', $id);
					}
				}
			} elseif(isset($content['accounts']['rows']['delete']) && is_array($content['accounts']['rows']['delete'])) {
				foreach($content['accounts']['rows']['delete'] as $id => $value) {
					if($id) {
						$check = ExecMethod('perp_ar.bo_perp_client_account.delete_client_account', $id);
						if($check === true) {
							$this->messages[] = array(lang('Account deleted'), 'success');
						} else {
							$this->messages[] = array($check, 'error');
						}
					}
				}

			// New Contact
			} elseif(isset($content['new_contact'])) {
				$this->create_contact($client_id, $content['client_name']);
			}
			// Warranty
			if($content['warranty'] && ($this->client_edit || $GLOBALS['egw']->acl->check('client', EGW_ACL_EDIT, 'perp_warranty'))) {
				bo_perp_client_warranty::set_client_settings($client_id, $content['warranty']);
			}

			// New client, they've filled in the name, we can create a contact
			if($content['new_client'] && trim($content['client_name']) != '') {
				unset($content['new_client']);
				$this->create_contact($client_id, $content['client_name']);
				$GLOBALS['egw']->redirect_link(
					'/index.php', 
					array(
						'menuaction'	=>	'perp_ar.ui_perp_client.client_edit',
						'client_id'	=>	$client_id
					)
				);
			}

			// Tab actions
			foreach($content as $tab_name => $tab_data) {
				if(!is_array($tab_data) || !is_array($tab_data['rows'])) {
					continue;
				}
				foreach($tab_data['rows'] as $action => $id) {
					if(!is_numeric($action)) {
						if(is_array($id)) {
							if(count($id) == 1) {
								$id = key($id);
							} elseif ($content[$tab_name]['action']) {
								$action = $content[$tab_name]['action'];
							}
						} else {
							$id = $client_id;
						}
						$this->handle_action($tab_name . '.' .  $action, $id);
					}
				}
			}
			if(isset($content['cancel']) || !$this->client_read) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ar.ui_perp_client.client_list'));
			}
				
			/********************
			* Set up outgoing
			*********************/
			

			// Read outgoing
			if($client_id !== False) {
				$client = $this->bo->get_client($client_id);
			} else {
				$client = array();
			}
			if($client_id && !$client['address']) {
				$this->messages[] = array(lang('Please link a billing address for this client'), 'info');
				$data['links']['app'] = 'addressbook';
			}
			$select_box_options = array();

			// Accounts - Bill to
			if($this->account_read && $client_id !== False) {
				$data['accounts'] = array(
					'client_id'	=>	$client_id,	
					//'account_balance'	=>	True,
					'get_price_lists'	=>	True,
					'no_cat'	=>	true,
					'filter'	=>	$content['filter'] ? $content['filter'] : 'current',
					'no_filter2'	=>	true,
					'no_search'	=>	true,
					'template'	=>	'perp_ar.client_edit.accounts.row',
					'get_rows'	=>	'perp_ar.bo_perp_client_account.get_client_account_list',
					'options-filter'=>	array(
						''		=>	'all',
						'current'	=>	'current',
						'disabled'	=>	'disabled'
					)
				);
				
				$readonly['accounts']['add_account'] = !$this->account_add;
			} else {
				$readonly[self::$tabs]['accounts'] = True;
			}
			

			// Branches  / Locations - Ship to
			if($this->branch_read && $client_id !== False) {
				$data['locations'] = $content['locations'] ? $content['locations'] : array();
				$data['locations'] = array(
					'client_id'	=>	$client_id,
					'template'	=>	'perp_ar.client_edit.locations.row', 
                                        'get_rows'      =>      'perp_ar.bo_perp_client_branch.get_client_branch_list',
					'no_filter'	=>	true,
					'no_filter2'	=>	true
				);
				$readonly['locations']['add_location'] = !$this->branch_add;
			} else {
				$readonly[self::$tabs]['locations'] = True;
			}		
			

			// Orders - Needs order entry module	
			if($client_id) {
				// Get Orders
				$data['orders'] = array(
					'client_id'	=>	$client_id,
					'template'	=>	'perp_orders.sales_order_list.row',
					'get_rows'      =>      'perp_ar.ui_perp_client.get_order_list',
					'order'		=>	'issue_date',
					'sort'		=>	'DESC',
					'no_cat'	=>	true,
					'no_filter2'	=>	true,
					'default_cols'	=>	'!client_name',
					'options-filter'=>	ExecMethod('perp_orders.bo_perp_sales_order.get_status_list')
				);
			}
			$readonly['orders']['rows']['new'] != $data['new_order'] = $GLOBALS['egw']->acl->check('order', EGW_ACL_ADD, 'perp_orders');
			$readonly[self::$tabs]['orders'] = !$GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_orders');

			// Shipments
				if(isset($content['shipments'])) {
					$data['shipments'] = $content['shipments'];
				} else {
					$data['shipments'] = array(
						'no_cat'	=>	true,
						'no_filter'	=>	true,
						'no_filter2'	=>	true,
						'client_id'	=>	$client_id,
						'template'	=>	'perp_ar.outgoing_shipment_list.row',
						'get_rows'      =>      'perp_ar.bo_perp_outgoing_shipment.get_shipment_list',
					);
				}
			$readonly[self::$tabs]['shipments'] = !$GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, 'perp_ar');

			// Invoices
				if(isset($content['invoices'])) {
					$data['invoices'] = $content['invoices'];
				} else {
					$data['invoices'] = array(
						'client_id'	=>	$client_id,
						'template'	=>	'perp_ar.receivable_invoice_list.row',
						'get_rows'      =>      'perp_ar.bo_perp_receivable_invoice.get_invoice_list',
						'status'	=>	'open',
						'no_cat'	=>	true,
						'no_filter'	=>	true,
						'no_filter2'	=>	true,
						'order'		=>	'due_date',
						'sort'		=>	'ASC'
					);
				}
				if($client_id) {
					$data['invoices']['rows']['client_id'] = $client_id;
				}
			$readonly[self::$tabs]['invoices'] = !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ar');

			// Warranty 
			$data['client_id'] = $client_id;
			$this->setup_warranty_tab($data, $select_box_options, $readonly);

			// Ledger
			if($this->ledger_read && $client_id !== False) {
				$data['ledger'] = $content['ledger'] ? $content['ledger'] : array();
				$data['ledger'] += array(
					'client_id'	=>	$client_id,
					'client_account_id'	=>	$content['ledger']['client_account_id'],
					'currency'	=>	$client['currency'],
					'template'	=>	'perp_ar.client_edit.ledger.row',
                                        'get_rows'      =>      'perp_ar.bo_perp_client.get_ledger',
					'no_cat'	=>	true,
					'no_filter'	=>	true,
					'no_filter2'	=>	true,
					'order'		=>	'trans_date',
					'sort'		=>	'DESC'
				);

				$data['ledger']['total'] = $this->bo->get_account_balance($client_id, $data['ledger']['client_account_id']);
				$data['ledger']['total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], $data['ledger']['total']);
			} else {
				$readonly[self::$tabs]['ledger'] = True;
			}
			
			// Links
			$data['links']['to_app'] = 'perp_ar';
			$data['links']['to_id'] = 'CLIENT-' . $client_id;

			$data = array_merge($data, $client);
			unset($data['save']);

			// Selectboxes
			$link_id = 'CLIENT-' . $client_id;
			$addresses = egw_link::get_links('perp_ar', $link_id, 'addressbook');
			foreach($addresses as $id => $address_id) {
				$select_box_options['address'][$address_id] = egw_link::title('addressbook', $address_id);
			}
			if($client['address'] && count($select_box_options['address']) == 0) {
				$this->messages[] = array(lang('Please link a billing address for this client'), 'warning');
			}
			$select_box_options['status'] = $this->bo->get_status_list();
			$select_box_options['currency'] = ExecMethod('perp_api.bo_perp_api.get_currencies');
			$select_box_options['payment_terms'] = ExecMethod('perp_api.bo_perp_api.get_payment_terms');
			$select_box_options['hold_reason'] = array('' => lang('Not Held')) + ExecMethod('perp_ar.bo_perp_ar.get_hold_reasons');
			$select_box_options['sales_area'] = ExecMethod('perp_orders.bo_perp_orders.get_sales_areas');
			$select_box_options['sales_type'] = array('' => lang('All')) + ExecMethod('perp_orders.bo_perp_orders.get_sales_types');
			$select_box_options['price_lists'] = ExecMethod2('perp_orders.bo_perp_price_list.get_price_list_list', 
				array(
					'sales_area_id' => $data['sales_area'], 
					'sales_type_id' => $data['sales_type'],
					'status'	=> 'current'
				)
			);
			$select_box_options['client_account_id'] = array('' => lang('All')) + ExecMethod('perp_ar.bo_perp_client_account.get_client_account_list', array('client_id' => $client_id));
			if($GLOBALS['egw']->acl->check('shipment', EGW_ACL_ADD, 'perp_ar')) {
				$options = ExecMethod('perp_ar.ui_perp_outgoing_shipment.get_selectbox_options', new Shipment());
				$select_box_options['ship_from_address'] = $options['ship_from_address'];
				$select_box_options['ship_to_address'] = array();
				$contacts = $addresses;
				if(is_array($data['locations'])) {
					foreach($data['locations'] as $location) {
						$contacts += egw_link::get_links('perp_ar', 'CLIENT_BRANCH-' . $location['client_branch_id'], 'addressbook');
					}
				}
				foreach($contacts as $id => $contact) {
					$select_box_options['ship_to_address'][$contact] = egw_link::title('addressbook', $contact);
				}
			}

			// Invoice actions from the invoice list
			$options = ExecMethod('perp_ar.ui_perp_receivable_invoice.list_get_options');
			$select_box_options['action'] = $options['action'];
	
			// Readonlys
			if( count($client) == 0 || !$client['address']) {
				// Hide all tabs until client_num is set
				foreach(explode('|', self::$tabs) as $tab) {
					$readonly[self::$tabs][$tab] = True;
				}
				if(count($client)) {
					unset($readonly[self::$tabs]['details']);
				}
			}
			if($client_id) {
				$readonly['client_num'] = True;
				$readonly[self::$tabs]['links'] = False;
			}
			if((!$client_id && !$this->client_add) || $client_id && !$this->client_edit) {
				$readonly['address']		=	true;
				$readonly['currency']		=	true;
				$readonly['client_since']	=	true;
				$readonly['status']		=	true;
				$readonly['sales_area']		=	true;
				$readonly['sales_type']		=	true;
				$readonly['save']		=	true;
			}

			$preserve = $data;

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = ($client_id && $data['client_num']) ? lang('Edit Client') : lang('New Client');
			$this->template->read('perp_ar.client_edit');
			$this->template->exec('perp_ar.ui_perp_client.client_edit', $data, $select_box_options, $readonly, $preserve, 0); 
		}

		public function handle_action($action, $document_id) {
			list($tab, $action) = explode('.', $action);
			if($tab == 'orders') {
				switch ($action) {
					case 'new':
						$GLOBALS['egw']->redirect_link('/index.php', array(
							'menuaction'	=>	'perp_orders.ui_perp_sales_order.new_order',
							'client_id'	=>	$document_id
						));
						break;
					default:
						ExecMethod2('perp_orders.ui_perp_sales_order.handle_order_action', $action, $document_id);
						break;
				}
			} elseif ($tab == 'shipments') {
				switch ($action) {
					case 'new':
						$GLOBALS['egw']->redirect_link('/index.php', array(
							'menuaction'	=>	'perp_ar.ui_perp_outgoing_shipment.new_shipment',
							'client_id'	=>	$document_id,
							'client_branch_id'	=>	$_POST['exec']['client_branch_id'],
							'ship_from_address'	=>	$_POST['exec']['ship_from_address'],
							'ship_to_address'	=>	$_POST['exec']['ship_to_address'],
						));
						break;
					default:
						ExecMethod2('perp_ar.ui_perp_outgoing_shipment.handle_shipment_action', $action, $document_id);
						break;
				}
			} elseif ($tab == 'invoices') {
				ExecMethod2('perp_ar.ui_perp_receivable_invoice.handle_action', $action, $document_id);
			}
			return false;
		}

		public function create_contact($client_id, $client_name) {
			// Look for an existing contact with the same name
			$link_id = 'CLIENT-' . $client_id;
			$contacts = ExecMethod('addressbook.addressbook_bo.link_query', "'$client_name'");
			$create_new = true;
			if(count($contacts) > 0) {
				foreach($contacts as $contact_id => $name) {
					$contact = ExecMethod('addressbook.addressbook_bo.read', $contact_id);
					if($contact['org_name'] == $client_name || stripos($contact['n_fileas'], $client_name) !== false) {
						egw_link::link('perp_ar', $link_id, 'addressbook', $contact_id);
						$create_new = true;
					}
				}
			}
			if($create_new) {
				// Create new contact
				$config = config::read('perp_ar');
				$contact = array(
					'cat_id'        =>      $config['address_category'],
					'org_name'      =>      $client_name,
					'fileas'        =>      $client_name,
				);
				$contact_id = ExecMethod('addressbook.addressbook_bo.save', $contact);
				$link = egw_link::link('perp_ar', $link_id, 'addressbook', $contact_id);

				// Open contact for editing (won't work if we redirect for a new client)
				if(!is_object($GLOBALS['egw']->js)) {
					$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
				}

				$href = $GLOBALS['egw']->link('/index.php',array(
					'menuaction'    =>      'addressbook.addressbook_ui.edit',
					'contact_id'    =>      $contact_id
				));
				$GLOBALS['egw']->js->set_onload("window.open('$href','_blank','dependent=yes,width=850x440,height=450,scrollbars=yes,status=yes'); ");
			}

			// Check if this is the only linked address & save it as the billing address
			$addresses = egw_link::get_links('perp_ar', $client_id, 'addressbook');
			if(count($addresses) == 1) {
				$client = $this->bo->get_client($client_id);
				$client['address'] = $contact_id;
				$this->bo->set_client($client_id, $client);
			}
		}

		/**
		*       Get client list, and preserve options in the session
		*/
		public function get_client_list(&$query, &$rows = array(), &$readonlys = array()) {
			$GLOBALS['egw']->session->appsession('client.nextmatch', false, $query);
			return $this->bo->get_client_list($query, $rows, $readonlys);
		}

		/**
		*	View the items a client has on order that have not shipped yet
		*/
		public function on_order($content = array()) {
			$client_id = reg_var('client_id', 'any', 'alphanumeric', $content['client_id']);

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($client_id, $client_ids)) {
				$client_id = $client_ids;
			}


			if(!$client_id) {
				return false;
			}

			// Get On Order
			$query = array(
				'client_id'		=>	$client_id,
				'include_details'	=>	$content['on_order_details'],
				'num_rows'		=>	-1,
			);
			$bo_shipment = new bo_perp_outgoing_shipment();
			$this->bo->get_on_order( $query, $data['on_order']);
			foreach($data['on_order'] as $group_key => &$group) {
				if(is_array($group['stock_category'])) {
					array_unshift($group['stock_category'], false);
				}
			}
			array_unshift($data['on_order'], false);

			$preserve['client_id'] = $client_id;
			$data['on_order_details'] = $content['on_order_details'];

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Items On Order');
			$this->template->read('perp_ar.client_edit.on_order');
			$this->template->exec('perp_ar.ui_perp_client.on_order', $data, $select_box_options, $readonly, $preserve); 
			
		}

		/**
		*	Set up the Warranty for display.
		*	
		*	This sets display data, adds selectbox options, and sets readonlys.
		*/
		protected function setup_warranty_tab(&$data, &$options = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				$readonly[self::$tabs]['warranty'] = true;
				return;
			}
			$data['warranty'] = bo_perp_client_warranty::get_client_settings($data['client_id']);
			if(!$data['warranty']['warranty_category']) {
				$data['warranty'] = array(
					'warranty_category'	=>	'',
					'claim'		=>	array(
						'credit_rate'	=>	100
					),
					'pdi'		=>	array(
						'travel_rate'	=>	0,
						'credit_rate'	=>	100
					)
				);
			}

			$data['warranty']['warranty_claims'] = $GLOBALS['egw']->session->appsession('warranty_claims_' . $data['client_id']);
			if(!is_array($data['warranty']['warranty_claims'])) {
				$data['warranty']['warranty_claims'] = array(
					'col_filter'	=>	array(
						'client_id'	=>	$data['client_id'],
					),
					'cat_app'		=>	'perp_inventory',
					'cat_id'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['cat_id'],
					'filter'		=>	'client',
					'no_filter'		=>	true,
					'template'		=>	ui_perp_claim::$list_row_template,
					'get_rows'		=>	'perp_warranty.ui_perp_claim.get_claim_list',
					'hide_parent'		=>	true,
					'no_actions'		=>	true,
					'no_select'		=>	true,
				);
			}
	
			$list_options = ui_perp_claim::list_get_options();
			unset($list_options['supplier_id']);
			$list_options['client_id'] = array(
				$data['client_id'] => $list_options['client_id'][$data['client_id']]
			);
			$options += $list_options;
			$options['warranty_category'] = bo_perp_client_warranty::get_category_list();

			// Add in exception so the selectbox returns a good value
			if($data['warranty']['is_exception']) {
				$options['warranty_category'][$data['warranty']['id']] = ExecMethod('perp_ar.bo_perp_client.link_title', $data['client_id']);
			} else {
				$options['warranty_category'] = array('' => lang('Custom')) + $options['warranty_category'];
			}
			$options['travel_units'] = array(
				'km'	=>	'km',
				'mi'	=>	'mi'
			);

			if($data['warranty']['id'] && !$data['warranty']['is_exception'] || !$GLOBALS['egw']->acl->check('client', EGW_ACL_EDIT, 'perp_warranty')) {
				$readonly += array(
					'claim'			=>	true,
					'pdi'			=>	true,
					'travel_units'		=>	true,
					'return_to_address'	=>	true
				);
			}

			$config = config::read('perp_warranty');

			// Hide PDI if we don't use it
			if(!$config['use_pdi']) {
				$data['warranty']['pdi']['no_pdi'] = true;
			}

		}

		/**
		*	Need to keep this in perp_ar, otherwise the currentapp gets reset to perp_orders, and 
		*	causes problems with the sidebox
		*/
		public function get_order_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			// Flip current app to perp_orders to use permissions from there.  Otherwise, we get no read permission
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_orders';
			$bo = new bo_perp_sales_order();
			$count = $bo->get_order_list($query, $rows, $readonlys);
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';
		}
	}
?>
