<?php

	/**
	*	User interface for outgoing shipments
	*
	*	@author Nathan Gray
	*	@ingroup perp_ar
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_shipment.inc.php');
	require_once('class.OutgoingShipment.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/perp_orders/inc/class.SalesOrder.inc.php');

	class ui_perp_outgoing_shipment extends ui_perp_shipment {

		public $public_functions = array(
			'shipment_list'	=>	True,
			'new_shipment'	=>	True,
			'add'		=>	True,
			'edit'		=>	True,
			'ship'		=>	True,
		);
		
		protected $shipment_list_template = 'perp_ar.outgoing_shipment_list';
		protected $shipment_list_row_template = 'perp_ar.outgoing_shipment_list.row';
		protected $shipment_line_template = 'perp_ar.outgoing_shipment_edit.stock_category';

		public function __construct() {
			parent::__construct();
			$this->bo_shipment = CreateObject('perp_ar.bo_perp_outgoing_shipment');
		}

		public function shipment_list($content = array()) {
			parent::shipment_list($content);

			// Get where to return to
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			$this->list_update($content);

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ar.ui_perp_ar.index'));
			}

			$select_box_options = $this->list_get_options();

			$readonly = $this->list_get_readonlys();

			$data = $this->list_get_display_data();

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = ($so ? lang("Shipments for %1", $so->get_sales_order_number()) : lang('Shipments'));
			$this->template->read($this->shipment_list_template);
			$this->template->exec('perp_ar.ui_perp_outgoing_shipment.shipment_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function new_shipment($content = array()) {
			$shipment = new OutgoingShipment();

			$order_id = reg_var('order_id');

			if($order_id) {
				$order = new SalesOrder($order_id);
				$shipment->read_from_object($order);
			}
			$client_id = (int)reg_var('client_id');
			if($client_id) {
				$shipment->set_client_id($client_id);
			}

			$client_branch_id = (int)reg_var('client_branch_id');
			if($client_branch_id) {
				$shipment->set_client_branch_id($client_branch_id);
			}

			$ship_from_address = (int)reg_var('ship_from_address');
			if($ship_from_address) {
				$shipment->set_ship_from_address($ship_from_address);
			}

			$ship_to_address = (int)reg_var('ship_to_address');
			if($ship_to_address) {
				$shipment->set_ship_to_address($ship_to_address);
			}
			if(!$order_id && $client_id && $ship_from_address) {
				// For a new shipment, add everything that's on order and in stock
				$query = array(
					'client_id'	=>	$client_id,
					'location_id'	=>	$shipment->get_location_id()
				);
				if($client_branch_id) {
					$query['client_branch_id'] = $client_branch_id;
				}
				if($ship_to_address) {
					$query['ship_to_address'] = $ship_to_address;
				}
				$this->bo_shipment->get_shippable_items($query, $items);
				if(is_array($items[0]['stock_category'])) {
					foreach($items[0]['stock_category'] as $category_key => &$category) {
						foreach($category['stock'] as $stock_key => &$stock_info) {
							if(is_array($stock_info)) {
								$stock_info['to_ship'] = min($stock_info['quantity_on_hand'], $stock_info['quantity_ordered']);
								$this->add_line($shipment, $stock_info);
							}
						}
					}
				}
		
			}

			$shipment->set_our_reference_number($shipment->suggest_shipment_number());
			$GLOBALS['egw']->session->appsession('shipment', false, $shipment);

			return $this->edit($content);
		}

		public function add($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_outgoing_shipment.add incoming data:';
				_debug_array($content);
			}
			if(!$GLOBALS['egw']->acl->check('shipment', EGW_ACL_EDIT, 'perp_ar')) {
				$GLOBALS['egw']->redirect($content['return'] ? $content['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ar.ui_perp_ar.index'));
			}
				
			$shipment = $GLOBALS['egw']->session->appsession('shipment');

			// Add items to shipment
			$items = array();
			if($content['save']) {
				foreach($content as $group_num => $group) {
					if(is_numeric($group_num) && is_array($group['stock_category'])) {
						foreach($group['stock_category'] as $category_num => $stock_category) {
							foreach($stock_category['stock'] as $key => $stock) {
								if(is_array($stock) && $stock['to_ship'] != 0) {
									// Look in the $_POST array, because etemplate doesn't return the hidden values
									$line = $_POST['exec'][$group_num]['stock_category'][$category_num]['stock'][$key];
									$this->add_line($shipment, $line);
								}
							}
						}
					}
				}
			}
			try { 
				$shipment->write();
			} catch (Exception $e) {
				$this->messages[] = array($e->getMessage(), 'error');
			}
			$GLOBALS['egw']->session->appsession('shipment', false, $shipment);

			// Get items that can be added
			$items = array();
			$query = array(
				'client_id'	=>	$shipment->get_client_id(),
				'num_rows'	=>	-1
			);
			if($shipment->get_ship_from_address()) {
				$query['location_id'] = $shipment->get_location_id();
			}
			$this->bo_shipment->get_shippable_items($query, $items);

			// eTemplate won't preserve the data because of numeric keys, so we have to make sure it comes back
			$preserve_fields = array(
				'order_id',
				'line_no'
			);
			foreach($items as $group_num => &$group) {
				if(!is_array($group['stock_category'])) {
					continue;
				}
				foreach($group['stock_category'] as $category_num => &$category) {
					if(!is_array($category['stock'])) {
						continue;
					}
					foreach($category['stock'] as $key => &$stock) {
						if(!is_numeric($key) || !is_array($stock)) {
							continue;
						}
						
						// Generate some hidden html inputs for this
						foreach($preserve_fields as $field_name) {
							$stock['preserve'] .= $GLOBALS['egw']->html->input_hidden(
								"exec[$group_num][stock_category][$category_num][stock][$key][$field_name]", 
								$stock[$field_name],
								false
							);
						}
					}
				}
			}

			$data = $items;
			if(count($data) == 0) {
				$this->messages[] = array(lang('Nothing for this client to ship'), 'info');
			}

			$preserve = $data;
			$preserve['client_id'] = $shipment->get_client_id();
			$preserve['shipment_id'] = $shipment->get_shipment_id();

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			} elseif ($content['save']) {
				// Close the popup
				if(!is_object($GLOBALS['egw']->js)) {
					$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
				}
				$GLOBALS['egw']->js->set_onload('self.close();');
			}
			
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Add to shipment');
			$this->template->read('perp_ar.outgoing_shipment_add');
			$this->template->exec('perp_ar.ui_perp_outgoing_shipment.add', $data, $select_box_options, $readonly, $preserve, 2);
		}

		/**
		*	Edit a shipment (just the header information).  See add to add lines.
		*	@param content Incoming data from eTemplate.
		*/
		public function edit($content = array()) {
			if(!$GLOBALS['egw']->acl->check('shipment', EGW_ACL_READ, 'perp_ar')) {
				$GLOBALS['egw']->redirect($_SERVER['HTTP_REFERER']);                        
			}

			// Get shipment
			$shipment_id = reg_var('shipment_id');
			$shipment = $this->get_shipment($shipment_id, 'OutgoingShipment');
			if(!$shipment) {
				$GLOBALS['egw']->redirect($preserve['return']);
			}
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_ar.ui_perp_outgoing_shipment.shipment_list');
			}

			// Deal with incoming data
			try {
				$test = $this->update_shipment($shipment, $content);
			} catch (Exception $e) {
				$this->messages[] = array($e->getMessage(), 'error');
			} 

			// Save to session
			$GLOBALS['egw']->session->appsession('shipment', 'perp_ar', $shipment);

			// Load data for display
			$data = $this->get_display_data($shipment);

			// Supporting data
			$select_box_options = $this->get_selectbox_options($shipment);
			
			// Readonlys
			$readonly = $this->get_readonlys($shipment);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			
			//Javascript
			$this->get_javascript(null);

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang($shipment_id ? 'Edit Shipment' : 'New Shipment');
			$this->template->read('perp_ar.outgoing_shipment_edit');
			$this->template->exec('perp_ar.ui_perp_outgoing_shipment.edit', $data, $select_box_options, $readonly, $preserve);
		}

		public function ship($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_outgoing_shipment.ship incoming:';
				_debug_array($content);
			}
			$shipment_id = $content['shipment_id'] ? $content['shipment_id'] : reg_var('shipment_id');
			$shipment = new OutgoingShipment($shipment_id);
			
			$data = array();

			// Incoming
			if(isset($content['ship'])) {
				ExecMethod('perp_ar.ui_perp_pack_shipment.initialize', $shipment);
				$shipment->write();
				$result = $shipment->ship();
				if($result === True && !$this->debug) {
					$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_ar.ui_perp_outgoing_shipment.shipment_list');
				} else {
					$this->messages[] = array($result, 'error');
				}
			} else if (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_ar.ui_perp_outgoing_shipment.shipment_list');
			}

			// Data
			$data += $shipment->toArray();
			$data['client'] = ExecMethod('perp_ar.bo_perp_client.get_client', $shipment->get_client_id());
			$packing_slip = $this->bo_shipment->get_packing_slip($shipment_id);
			if(is_array($packing_slip)) {
				$data += $packing_slip;
			}

			$test = $shipment->can_ship();
			if($test !== true) {
				$this->messages[] = array($test, 'error');
			}

			// Select box options
			$select_box_options['shipper'] = ExecMethod('perp_api.bo_perp_api.get_shippers');

			// Preserve
			$preserve['shipment_id'] = $shipment_id;

			// Readonly
			$readonly['ship'] = (boolean)$shipment->get_actual_delivery_date() || !$GLOBALS['egw']->acl->check('shipment', EGW_ACL_EDIT, 'perp_ar') || count($this->messages) > 0;
			
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Ship %1', $shipment->get_our_reference_number());
			$this->template->read('perp_ar.outgoing_shipment_ship');
			$this->template->exec('perp_ar.ui_perp_outgoing_shipment.ship', $data, $select_box_options, $readonly, $preserve);
		}

		protected function list_get_display_data() { 
			$data = parent::list_get_display_data();
			return $data;
		}

		protected function list_get_options() {
			$options = parent::list_get_options();
			unset($options['filter']['in-transit']);	// We don't know when it arrives
			unset($options['filter']['arrived']);		// We don't know when it arrives
			$options['filter']['shipped'] = lang('Shipped');
			$options['filter2'] = array('' => lang('All')) + ExecMethod('perp_ar.bo_perp_client.get_client_list');
			return $options;
		}

		/**
		*       Handle an action on an Shipment
		*
		*       Actions can be one of new, edit, ship, receive,
		*       invoice, approve, deny, cancel
		*
		*       @param action   Action to take
		*       @param shipment_id      ID of the Shipment to act on
		*/
		public function handle_shipment_action($action, $shipment_id) {
			$result = parent::handle_shipment_action($action, $shipment_id);
			if($result) {
				return $result;
			}
			switch($action) {
				case 'pack':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ar.ui_perp_pack_shipment.package',
						'shipment_id'	=>	$shipment_id
					));
				case 'ship':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ar.ui_perp_outgoing_shipment.ship',
						'shipment_id'	=>	$shipment_id
					));
					break;
				case 'receive':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ar.ui_perp_receive.receive',
						'shipment_id'	=>	$shipment_id
					));
					break;
				case 'invoice':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ar.ui_perp_receivable_invoice.new_invoice',
						'shipment_id'	=>	$shipment_id
					));
					break;
				default:
					return false;
			}
		}

		/**
		*	Update an OutgoingShipment with data from the eTemplate
		*	We deal with delete commands because they're not where they're expected to be,
		*	nextmatch[rows][delete][#]
		*/
		protected function update_shipment(&$shipment, &$content) {

			$result = parent::update_shipment($shipment, $content);

			if(isset($content['client_branch_id'])) {
				if(is_array($content['client_branch_id'])) {
					$shipment->set_client_branch_id((int)$content['client_branch_id']['client_branch_id']);
				} else {
					$shipment->set_client_branch_id((int)$content['client_branch_id']);
				}
			}
			if(is_array($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as $stock_category) {
					if(is_array($stock_category['items'])) {
						foreach($stock_category['items'] as $item => $data) {
							if($item == 'delete') {
								$this->remove_line($shipment, key($data));
							}
						}
					}
				}
			}
			if(isset($content['pack'])) {
				$this->handle_shipment_action('pack', $shipment->get_shipment_id());
			}
			return $result;
		}

		/**
		*	Get display data for an OutgoingShipment
		*	@param shipment Shipment for data 
		*/
		protected function get_display_data(&$shipment) {
			$data = parent::get_display_data($shipment);

			// Status warnings - check each line
			if(!$shipment->get_actual_delivery_date()) {
				$lines =& $shipment->get_lines();
				foreach($lines as &$line) {
					$test = $line->can_ship();
					if($test !== true) {
						$this->messages[] = array($test, 'warning');
					}
				}
			}

			$data['client_branch_id'] = array(
				'client_branch_id'	=>	$shipment->get_client_branch_id(),
				'client_id'		=>	(string)$shipment->get_client_id()
			);
			$data['nextmatch']['group_by'] = 'stock_category';
			return $data;
		}

		/**
		*	Get options for selectboxes
		*	@param shipment Shipment for data 
		*/
		public function get_selectbox_options(&$shipment) {
			$options = parent::get_selectbox_options($shipment);

			// Ship from addresses come from locations
                        $options['ship_from_address'] = array();
			$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
			foreach($locations as $location_id => $name) {
				$contacts = egw_link::get_links('perp_inventory', 'LOCATION-' . $location_id, 'addressbook');
				foreach($contacts as $link_id => $contact_id) {
					$options['ship_from_address'][$contact_id] = egw_link::title('addressbook', $contact_id);
				}
			}
			if($shipment->get_client_id()) {
				$contacts = array();
				$contacts += egw_link::get_links('perp_ar', 'CLIENT-' . $shipment->get_client_id(), 'addressbook');
				$branch_list = ExecMethod(
					'perp_ar.bo_perp_client_branch.get_client_branch_list', 
					array('client_id'	=>	$shipment->get_client_id())
				);
				foreach($branch_list as $branch_id => $branch) {
					$contacts += egw_link::get_links('perp_ar', 'CLIENT_BRANCH-' . $branch_id, 'addressbook');
				}
				foreach($contacts as $id => $contact) {
					$options['ship_to_address'][$contact] = egw_link::title('addressbook', $contact);
				}
			}
			return $options;
		}
				
		protected function get_readonlys(OutgoingShipment $shipment) {
			$readonlys = parent::get_readonlys($shipment);

			$readonlys['client_branch_id']['search'] = $readonlys['client_id'] && $shipment->get_client_branch_id() != 0;

			return $readonlys;
		}


		/**
		*	Add a line to a shipment
		*/
		protected function add_line(Shipment &$shipment, &$line_data) {
			if( !(isset($line_data['order_id']) && isset($line_data['line_no']))) {
				$this->messages[] = array(lang('Unable to find reference line from order.') . ' ' . lang('Line not added.'), 'warning');
				return false;
			}

			// Re-format line data
			$line_data += array(
				'shippable_class'	=>	'SalesOrder',
				'shippable_id'		=>	$line_data['order_id'],
				'shippable_item_line'	=>	$line_data['line_no']
			);
			$line_data['quantity_shipped']	=	$line_data['to_ship'];
			if($line_data['to_ship'] != 0) {
				$line = new ShipmentLine($shipment, $line_data);
				$shipment->add_line($line);
			}
		}

		/**
		*	Callback for the nextmatch widget to get the ShipmentLines
		*	@nextmatch
		*/
		public function get_shipment_lines(&$query, &$rows, &$readonlys) {
			$rows = array();
			$query['cat_app'] = 'perp_inventory';
			$query['count'] = parent::get_shipment_lines($query, $list, $readonlys);

			if($query['filter2'] == 'none' || !$query['filter2']) {
//				$rows =& $list;
//				return $query['count'];
			}

			// Group items by stock category
			$cat = '~ no category ~';
			$count = 0;
			foreach($list as $key => $item) {
				if(!is_array($item)) {
					$rows[$key] = $item;
					$rows[$count]['items'][$key] = $item;
					continue;
				}
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $item['stock_id']);
				if($stock) {
					$item['stock_category'] = $stock['stock_category'];
				}

				if($item['stock_category'] != $cat) {
					$cat = $item['stock_category'];
					$count++;
					$rows[$count]['stock_category'] = $item['stock_category'];
					$rows[$count]['delete'] = $list['delete'];
					$rows[$count]['items']['delete'] = $list['delete'];
					$rows[$count]['items']['quantity_header'] = lang('%1 / %2',
						$query['shipment']->get_actual_delivery_date() ? lang('Quantity Shipped') : lang('Quantity Scheduled'),
						lang('Quantity Ordered')
					);

					$rows[$count]['items'][] = false;
				}

				// Look for an ETA 
				$line =& $query['shipment']->get_line($item['line_number'])->get_shippable_item();
				$item['serial_number'] = array();
				$work_order_details =& $line->get_details('WorkOrderDetail');
				if(is_array($work_order_details) && count($work_order_details) > 0) {
					foreach($work_order_details as &$detail) {
						// If work order is not complete, get the completion date.
						$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $detail->get_work_order_id());
						$work_order_info = array(
							'app'	=>	'perp_manufacturing',
							'id'	=>	'WORK_ORDER-' . $detail->get_work_order_id(),
						);
						if(!$work_order['completed']) {
							$work_order_info['status']	=	ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order_status', $work_order['work_order_id']);
							$work_order_info['eta_date']	=	$work_order['required_date'];
						}
						$item['work_order'][] = $work_order_info;
						$item['eta_date'] = max($item['eta_date'], $work_order['completed_date']);
						$item['serial_number'] = array_merge(
							$item['serial_number'], 
							ExecMethod2('perp_manufacturing.bo_perp_work_order.get_serial_number', $detail->get_work_order_id(), $item['stock_id'])
						);
					}
				} elseif ($stock['controlled']) {
					$serial_numbers = $query['shipment']->get_line($item['line_number'])->get_serial_numbers();
					foreach($serial_numbers as $serial => $quantity) {
						$item['serial_number'][]['serial_number'] = $serial;
					}
				}
				$rows[$count]['items'][] = $item;
			}
			return $query['count'];
		}

	}
?>
