<?php
	require_once(EGW_INCLUDE_ROOT . '/etemplate/inc/class.etemplate.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');
	require_once('class.OutgoingShipment.inc.php');
	require_once('class.ShipmentPackage.inc.php');
	require_once('class.ShipmentSerial.inc.php');

	class ui_perp_pack_shipment {

		public $public_functions = array(
			'package'	=>	true,
		);

		/**
		*	We don't want all details to show on the form, so hide details 
		*	with the following names.
		*/
		protected $hide_details = array(
			'price_override',
			'Total',
		);

		private $template;
		private $debug = false;
		private $messages = array();

		public function __construct() {
			$this->template = new etemplate();
			$this->bo_inventory = new bo_perp_inventory();
		}

		public function package($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			$shipment = $this->get_shipment($content);
			if(is_null($shipment)) {
				return false;
			}

			// Deal with incoming
			$this->update_packaging($shipment, $content);

			if(count($content) == 0) {
				// First time?  Try to initialize
				$this->initialize($shipment);
			}

			// Read data for display
			$data = $this->get_display_data($shipment);
			$data['app_header'] = lang('Packing for Shipment %1', $shipment->get_our_reference_number());

			$preserve['rows'] = $data['items'];
			$preserve['shipment_id'] = $shipment->get_shipment_id();

			// Javascript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->validate_file('ship', 'serials', 'perp_ar');
			$GLOBALS['egw']->js->set_onload("serial_setup('" . $GLOBALS['egw_info']['server']['webserver_url'] . "'); ");

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$this->template->read('perp_ar.pack_shipment');
			$this->template->exec('perp_ar.ui_perp_pack_shipment.package', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Set up the packing with reasonable defaults
		*
		*	Reasonable being variable, of course, we'll try to assign container quantities (and serial numbers)
		*
		*	@param shipment The shipment to initialize.
		*/
		public function initialize($shipment) {
			$lines =& $shipment->get_lines();
			foreach($lines as &$line) {
				if($this->debug) {
					echo 'Initializing line ' . $line->get_line_number() . '<br />';
				}
				if(!is_object($line->get_shippable_item())) {
					// No item to ship?  How'd that happen?
					if($this->debug) {
						echo 'No item for line ' . $line->get_line_number() . '<br />';
					}
					continue;
				}
				if(count($line->get_details()) > 0) {
					// Already have some details,
					if($this->debug) {
						echo 'Already have details, skipping initialization on line ' . $line->get_line_number();
					}
					continue;
				}
				if(!method_exists($line->get_shippable_item(), 'get_details')) {
					// No get_details() function for the shippable item
					continue;
				}

				// Initialize packing
				$package_count = 0;
				$packages = array();
				if(method_exists($line->get_shippable_item(), 'get_details')) {
					$work_order_details =& $line->get_shippable_item()->get_details('WorkOrderDetail');
					foreach($work_order_details as $detail) {
						$packages = array_merge($packages, ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', 
							$line->get_shippable_item()->get_stock_id(), $detail->get_work_order_id(),
							$detail->get_work_order_detail_id())
						);
					}
				}
				if(count($packages) == 0) {
					$packages = ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', $line->get_shippable_item()->get_stock_id());
				}
				foreach($packages as $id => &$package) {
					$package['package_id'] = $id;
					$package['detail_value'] = $line->get_quantity_shipped();
					$this->set_packages($shipment, $line, $package);
				}

				$details =& $line->get_shippable_item()->get_details();
				foreach($details as $detail) {
					if($detail->get_name() == 'serial number') {
						$this->set_serials(
							$shipment, 
							$line,
							null,
							array(
								$detail->get_value() => (int)$detail->quantity
							)
						);
					}
				}
			}
		}

		/**
		*	Retrieve the shipment being worked on
		*/
		protected function get_shipment($content) {
			// Get shipment
			$shipment_id = reg_var('shipment_id', 'any', 'alphanumeric', $content['shipment_id']);

			$ship = $GLOBALS['egw']->session->appsession('shipment');
			if($shipment_id) {
				$shipment = new OutgoingShipment($shipment_id);
			} elseif ($ship) {
				$shipment = $ship;
			} else {
				$shipment = null;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($shipment->get_client_id(), $client_ids)) {
				return null;
			}

			return $shipment;
		}

		/**
		*	Update packaging for this shipment
		*/
		protected function update_packaging(OutgoingShipment &$shipment, $data) {
			if($data['cancel']) {
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/index.php', array(
					'menuaction'	=>	'perp_ar.ui_perp_outgoing_shipment.edit',
					'shipment_id'	=>	$shipment->get_shipment_id()
				)));
			}
			if($data['items']) {
				foreach($data['items'] as $key => $row) {
					$line_check[$data['rows'][$key]['line_no']] += $row['container_quantity'];
					$shipment_line =& $shipment->get_line($data['rows'][$key]['line_no']);
					if(is_null($shipment_line)) {
						continue;
					}

					$package_count = 0;
					if(method_exists($shipment_line->get_shippable_item(), 'get_details')) {
						$work_order_details =& $shipment_line->get_shippable_item()->get_details('WorkOrderDetail');
						foreach($work_order_details as $detail) {
							$package_count += count(ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', 
								$row['stock_id'], $detail->get_work_order_id(),
								$detail->get_work_order_detail_id())
							);
						}
					}
					if($package_count == 0) {
						$package_count = count(ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', $data['rows'][$key]['stock_id']));
					}
					if($line_check[$data['rows'][$key]['line_no']] > $shipment_line->get_quantity_shipped()) {
						$this->messages[] = array(lang(
							'Container %1 has too many %2.  It brings the total to %3 but only %4 were shipped.',
							$row['container_id'],
							$data['rows'][$key]['stock_num'],
							$line_check[$data['rows'][$key]['line_no']],
							$shipment_line->get_quantity_shipped() * $package_count
						), 'warning');
					} else {
						$this->set_packages($shipment, $shipment_line, array(
								'detail_name'	=>	$row['container_id'],
								'detail_value'	=>	($row['delete'] ? 0 : $row['container_quantity']),
								'package_id'	=>	$data['rows'][$key]['package_id'],
								'package_details'	=>	$data['rows'][$key]['package_details']
							)
						);
						if($row['serial_numbers']) {
							$this->set_serials(
								$shipment, 
								$shipment_line,
								$row['container_id'],
								unserialize(urldecode($row['serial_numbers']))
							);
						}
					}
				}
			}
			$GLOBALS['egw']->session->appsession('shipment', 'perp_ar', $shipment);
			
			if($data['save']) {
				$shipment->write();
				$GLOBALS['egw']->session->appsession('shipment', 'perp_ar', null);
			}
		}

		/**
		*	Set packages for shipping
		*/
		protected function set_packages(Shipment &$shipment, ShipmentLine &$shipment_line, $pack) {
			$details = $shipment_line->get_details('ShipmentPackage');
			foreach($details as &$detail) {
				if($pack['package_id'] == $detail->get_package_id()) {
					if($pack['detail_name'] == $detail->get_container_id()) {
						$detail->set_container_id($pack['detail_name']);
						$detail->set_quantity($pack['detail_value']);
						
						if($this->debug) {
							echo 'Updated package: <br /><pre>' . $detail->__toString() . '</pre><br />';
						}
						return true;
					}
				}
				if($detail->get_quantity() == 0) {
					$shipment_line->remove_detail($detail->get_number());
					if($this->debug) {
						echo 'Removed from package: <br /><pre>' . $detail->__toString() . '</pre><br />';
					}
				}
			}
			if($pack['detail_value'] > 0) {
				$package = new ShipmentPackage($shipment_line, $pack);
				$shipment_line->add_detail($package);
				if($this->debug) {
					echo 'New package: <br /><pre>' . $package->__toString() . '</pre><br />';
				}
			}
		}

		/**
		*	Set serial numbers on goods
		*/
		protected function set_serials(Shipment &$shipment, &$shipment_line, $package_name, $serials) {
		
			if($this->debug) {
				echo "set_serials(Shipment, ShipmentLine {$shipment_line->get_line_number()}, $package_name, ...)<br />";
				_debug_array($serials);
			}
			if(is_null($shipment_line->get_shippable_item())) {
				// Something probably went wrong, because there's no item to ship
				$this->messages[] = array(lang('Unable to find the item being shipped on line %1', $line_no), 'error');
				return;
			}

			$item = $shipment_line->get_shippable_item();

			$details = $shipment_line->get_details('ShipmentSerial');
			foreach($details as $detail) {
				if($detail->get_name() == $package_name) {
					$shipment_line->remove_detail($detail->get_number());
				}
			}

			$data = array('detail_name' => $package_name);
			foreach($serials as $serial => $quantity) {
				// Good serial number, but never been recorded before
				if(bo_perp_stock::check_serial($shipment_line->get_shippable_item()->get_stock_id(), $serial) == bo_perp_stock::FORMAT_OK) {
					bo_perp_inventory::add_serial($shipment_line->get_shippable_item()->get_stock_id(), $serial, $quantity, $shipment->get_location_id());
				}
				$data['serial_number'] = $serial;
				$data['detail_value'] = (int)$quantity;
				$detail =& new ShipmentSerial($shipment_line, $data);
				if($this->debug) {
					echo 'New ShipmentSerial():<br />';
					_debug_array($detail->toArray());
				}
				$shipment_line->add_detail($detail);

				// Add it to the item, too
	//			$detail =& new OrderLineDetail($item, $data);
		//		$item->add_detail($detail);
			}
		}

		/**
		*	Get data for display
		*/
		protected function get_display_data(OutgoingShipment $shipment) {
			$data = array();
			
			$shipment_lines = $shipment->get_lines();
			foreach($shipment_lines as $shipment_line) {
				$stock_id = $shipment_line->get_shippable_item()->get_stock_id();
				$line = $this->make_stock_line($stock_id, $shipment_line);
				$details = $shipment_line->get_details();

				// Get some arrays we can index into
				$container = array();
				$serials = array();
				foreach($details as &$detail) {
					if($detail instanceof ShipmentPackage) {
						$container[$detail->get_package_id()][] = $detail;
					} elseif ($detail instanceof ShipmentSerial) {
						$serials[$detail->get_name()][] = $detail;
					}
				}

				$quantity_unpacked = $shipment_line->get_quantity_shipped();
				$sub_line = array();

				// Get packaging for shipment lines (pieces per item)
				foreach($line['packing'] as $package_id => $package_details) {
					$sub_line = $line;
					unset($sub_line['packing']);
					$sub_line['package_id']		=	$package_id;
					$sub_line['package_details']	=	$package_details;
					$sub_line['package_name']	=	lang('Package %1 of %2', $package_id + 1, $sub_line['piece_count']);

					if(!is_array($container[$package_id])) {
						// First time
						$data['items'][] = $sub_line;
						
						continue;
					}
					foreach($container[$package_id] as $detail) {
						if($detail instanceof ShipmentPackage && $package_id == $detail->get_package_id()) {
							$sub_line['container_id'] = $detail->get_container_id();
							$sub_line['container_quantity'] = $detail->get_quantity();
							unset($sub_line['serial_numbers']);
							$quantity_unpacked -= $detail->get_quantity();

							// Assign serials
							$serials_list = $serials[$detail->get_container_id()];
							if(is_array($serials_list)) {
								foreach($serials_list as $serial) {
									$sub_line['serial_numbers'][$serial->get_serial_number()] = $serial->get_value();
									$got_serials++;
								}
							}
							if(count($sub_line['serial_numbers']) > 0) {
								foreach($sub_line['serial_numbers'] as $serial => $quantity) {
									$serial_number_details[count($data['items']) ][]		=	array(
										'detail_name'		=>	lang('serial number'),
										'detail_value'		=>	$serial
									);
								}
								$sub_line['serial_numbers'] = urlencode(serialize($sub_line['serial_numbers']));
							}
							$data['items'][] = $sub_line;
							unset($sub_line['serial_numbers']);
						}
					}

					// First time
					if(count($details) == 0) {
						$data['items'][] = $sub_line;
						
					}
				}
				if($quantity_unpacked > 0 && $quantity_unpacked < $shipment_line->get_quantity_shipped()) {
					$sub_line['container_id'] = '';
					$sub_line['container_quantity'] = $quantity_unpacked;
					$data['items'][] = $sub_line;
				}

			}
			return $data;
		}

		public function make_stock_line($stock_id, ShipmentLine $shipment_line, $quantity = 0) {
			$line = array();
			$unit_of_measure = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);

			$line['line_no']		=	$shipment_line->get_line_number();
			$line['stock_id']		=	$stock_id;
			$line['stock_num']		=	$stock['stock_num'];
			$line['description']		=	$shipment_line->get_description();
			$line['unit_of_measure']	=	$unit_of_measure[$stock['unit_of_measure']];
			if($quantity == 0) {
				$line['quantity']	=	(float)$shipment_line->get_quantity_shipped();
			} else {
				$line['quantity']	=	(float)$quantity;
			}
			if($stock['controlled']) {
				$line['controlled']	=	true;
			}

			$details = $shipment_line->get_shippable_item()->get_details();
			$got_serials = 0;
			$packing = array();
			foreach($details as $detail) {
				if($detail->get_display() && $detail->get_value()) {
					if(!in_array($detail->get_name(), $this->hide_details) && $detail->get_display()) {
						$line['details'][]		=	$detail->toArray();
					}
				}
				if($detail instanceof WorkOrderDetail) {
					$packing = ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id, $detail->get_work_order_id(), $detail->get_work_order_detail_id());
				}
			}

			if(count($packing) == 0) {
				$packing = ExecMethod('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id);
			}
			foreach($packing as $package) {
				$line['packing'][] = (float)$package['length'] . ' x ' . 
					(float)$package['width'] . ' x ' . 
					(float)$package['height'] . ' ' . 
					$unit_of_measure[$package['unit_of_measure']];
			}
			$line['piece_count']		=	count($packing);

			if(count($line['details']) > 0) {
				// Move 1 to allow for title
				array_unshift($line['details'], false);
			} else {
				unset($line['details']);
			}

			return $line;
		}
	}
?>
