<?php
	require_once('class.ReceivableInvoice.inc.php');

	class ui_perp_receipt {

		public $public_functions = array(
			'receipt_list'	=>	true,
			'receipt_new'	=>	true,
			'receipt_view'	=>	true,
		);

		private $debug = false;
		private $messages = array();

		private $bo;
		private $template;

		public function ui_perp_receipt() {
			$this->bo = CreateObject('perp_ar.bo_perp_receipt');
			$this->template = CreateObject('etemplate.etemplate');
		}

		public function receipt_list($content = array()) {
			if($this->debug) {
				echo "perp_ar.ui_perp_receipt.receipt_list incoming data:";
				_debug_array($content);
			}

			// Deal with incoming
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			if(isset($content['new_receipt']) && $GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar')) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'    =>      'perp_ar.ui_perp_receipt.receipt_new',
				));
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ar.ui_perp_ar.index'));
			} elseif ($content['nm']['rows']['reverse']) {
				$id = key($content['nm']['rows']['reverse']);
				try {
					$result = $this->bo->reverse($id, $content['reverse_reason']);
				} catch (egw_exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
				if(is_numeric($result) && $result) {
					$this->messages[] = array('Receipt was reversed', 'success');
				}
			}

			// Set up outgoing
			$data['nm'] = array(
				'no_cat'        =>      True,
				'filter_label'  =>      'Client',
				'filter_no_lang'        =>      True,
				'filter2_label'	=>	'Payment Method',
				'get_rows'      =>      'perp_ar.bo_perp_receipt.get_receipts',
				'template'      =>      'perp_ar.receipt_list.row',
			);

			// Check for bank accounts - need those
			$bank_accounts = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts');
			if(count($bank_accounts) == 0 && $GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar')) {
				$this->messages[] = array(
					lang(
						'You do not have any %1 set up.  pERP has nowhere to put the money.',
						'<a href="' . 
						$GLOBALS['egw']->link('/index.php', array(
							'menuaction'	=>	'perp_ledger.hook_perp_ledger.bank_account'
						)) .
						'">' . lang('bank accounts') . '</a>'
					), 
					'error'
				);
			}

			// Select box options
			$select_box_options['filter'] = array('' => lang('all')) + ExecMethod('perp_ar.bo_perp_client.get_client_list');
			$select_box_options['filter2'] = array('' => lang('all')) + ExecMethod('perp_api.bo_perp_api.get_payment_methods');
			$select_box_options['status'] = array(
				1	=>	lang('Outstanding'),
				2	=>	lang('Cleared'),
				3	=>	lang('Reversed')
			);
			$select_box_options['client_id'] = $select_box_options['filter'];
			$select_box_options['payment_method'] = $select_box_options['filter2'];

			// Readonlys
			$readonly['new_receipt'] = !$GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar') || (count($bank_accounts) == 0);

			// Display
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Receipts');
			$this->template->read('perp_ar.receipt_list');
			$this->template->exec('perp_ar.ui_perp_receipt.receipt_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function receipt_new($content = array()) {
			if($this->debug) {
				echo "perp_ar.ui_perp_receipt.receipt_new incoming data:";
				_debug_array($content);
			}
			$data = array();

			if(!$GLOBALS['egw']->acl->check('receipt', EGW_ACL_ADD, 'perp_ar')) {
				$this->messages[] = array(lang('You do not have permission to add receipts'), 'error');
				return $this->receipt_list();
			}

			// Deal with incoming
			$client_id = $content['client_id'] ? $content['client_id'] : $_GET['client_id'];
			
			if (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ar.ui_perp_receipt.receipt_list'));
			} elseif(isset($content['post'])) {
				// Posting to the GL can throw an exception
				try {
					$result = $this->bo->post_receipt($content);
				} catch (Exception $e) {
					$result = $e->getMessage();
				}
				if(is_numeric($result) && !$this->debug) {
					$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_ar.ui_perp_receipt.receipt_view&receipt_id=' . $result);
				} else {
					$this->messages[] = array($result, 'error');
				}
			}

			// Set up outgoing
			if($client_id) {
				$this->bo->get_unallocated_amounts(
					array(
						'client_id'	=>	$client_id,
						'template'	=>	'Get all data',
						'num_rows'	=>	-1
					), 
					$rows
				);
				array_unshift($rows, false);
				$data += $rows;
				$data['rows'] = True;
			}

			$data['client_id'] = $client_id;
			$data['payment_method'] = $content['payment_method'];
			$data['bank_account'] = $content['bank_account'];
			$data['allocation_date'] = $content['allocation_date'] ? $content['allocation_date'] : date('Y-m-d');
			$config = config::read('perp_ar');
			$data['receipt_id'] = $config['next_receipt_number'];

			$banks = array();
			if($client_id) {
				$client = ExecMethod('perp_ar.bo_perp_client.get_client', $client_id);
				if($data['bank_account'] && $content['old_client_id'] == $client_id) {
					$bank = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_account', $data['bank_account']);
				} else {
					$banks = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts', array('currency' => $client['currency']));
					$bank = array();
					foreach($banks as $account => $name) {
						$bank = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_account', $account);
						if($bank['payment_methods']) {
							$data['bank_account'] = $account;
							break;
						} else {
							unset($banks[$account]);
						}
					}
				}
				if($client['currency'] != $bank['currency']) {
					$this->messages[] = array(lang('Client currency does not match bank currency.  It should be %1.  This will cause posting problems', $client['currency']), 'warning');
				}
			}

			// Select box options
			$select_box_options['client_id'] = array('' => lang('Select')) + ExecMethod('perp_ar.bo_perp_client.get_client_list');
			if($client_id) {
				$select_box_options['client_account_id'] = ExecMethod2('perp_ar.bo_perp_client_account.get_client_account_list', array('client_id' => $client_id));
			}

			$select_box_options['bank_account'] = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts', array('bank_account_id' => array_keys($banks)));
			if($bank['payment_methods']) {
				$select_box_options['payment_method'] = ExecMethod('perp_api.bo_perp_api.get_payment_methods', array('payment_method_id' => $bank['payment_methods']));
			}

			$preserve = $data;
			$preserve['old_client_id'] = $client_id;

			// Readonlys

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->set_onload('receipt_new_setup(' . $GLOBALS['egw_info']['user']['preferences']['perp_ar']['autofill_receipt'] . '); ');
			$GLOBALS['egw']->js->validate_file('receipt', 'new', 'perp_ar');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('New Receipt');
			$this->template->read('perp_ar.receipt_edit');
			$this->template->exec('perp_ar.ui_perp_receipt.receipt_new', $data, $select_box_options, $readonly, $preserve);
		}

		public function receipt_view($content = array()) {
			$receipt_id = $content['receipt_id'] ? $content['receipt_id'] : $_GET['receipt_id'];

			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($preserve['return'] ? $preserve['return'] : $GLOBALS['egw']->link('/index.php?menuaction=perp_ar.ui_perp_receipt.receipt_list'));
			}

			// Get data from display
			$data = $this->bo->get_receipt($receipt_id);
			if(!is_array($data)) {
				$error = ExecMethod2('perp_api.ui_perp_api.make_message', $data, 'error');
				$data = array('message' => $error);
			}

			// Select box options
			$select_box_options['payment_method'] = ExecMethod('perp_api.bo_perp_api.get_payment_methods');

			// Readonlys

			$preserve['receipt_id'] = $receipt_id;

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('View Receipt %1', $receipt_id);
			$this->template->read('perp_ar.receipt_view');
			$this->template->exec('perp_ar.ui_perp_receipt.receipt_view', $data, $select_box_options, $readonly, $preserve);
		}
	}
?>
