<?php
	/**
	*	User interface for receivable invoice.  
	*
	*	@author Nathan Gray
	*	@ingroup perp_ar
	*/
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_invoice.inc.php');
	require_once('class.bo_perp_receivable_invoice.inc.php');
	
	require_once('class.ReceivableInvoice.inc.php');
	require_once('class.OutgoingShipment.inc.php');

	// Required to invoice a SalesOrder
	require_once(EGW_INCLUDE_ROOT . '/perp_orders/inc/class.SalesOrder.inc.php');

	class ui_perp_receivable_invoice extends ui_perp_invoice {

		/**
		*	List of functions callable by the browser.
		*	All functions not in this list are denied access from the browser
		*/
		public $public_functions = array(
			'invoice_list'	=>	True,
			'new_invoice'	=>	True,
			'edit'		=>	True,
			'add_line_from'	=>	True,
		);

		protected $list_template = 'perp_ar.receivable_invoice_list';
		protected $list_row_template = 'perp_ar.receivable_invoice_list.row';
		protected $invoice_line_template = 'perp_ar.receivable_invoice_edit.row';
		protected $uninvoiced_line_template = 'perp_ar.receivable_invoice.uninvoiced_lines';

		public function __construct() {
			parent::__construct();
			$this->bo_invoice = new bo_perp_receivable_invoice();
		}

		/**
		*	Display a list of already entered invoices
		*/
		public function invoice_list($content = array()) {
			parent::invoice_list($content);

			// Deal with incoming
			$this->list_update($content);

			if(isset($content['cancel']) || !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ar')) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ar.ui_perp_ar.index'));
			}

			$data = $this->list_get_display_data();
			if(!$data['nextmatch']['filter']) {
				$data['nextmatch']['filter'] = ($content['nextmatch']['filter'] ? $content['nextmatch']['filter'] : 'due');
			}
			
			$select_box_options = $this->list_get_options();
	
			$readonly = $this->list_get_readonlys();

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Receivable Invoice List');
			$this->template->read($this->list_template);
			$this->template->exec('perp_ar.ui_perp_receivable_invoice.invoice_list', $data, $select_box_options, $readonly, $preserve); 
		}

		/**
		*	Get a list of options for selectboxes
		*/
		public function list_get_options() {
			$options = parent::list_get_options();

			// Add in actions
			$options['action']['invoice_report'] = lang('report_invoice_report');

			return $options;
		}

		/**
		*	Create a new blank invoice
		*/
		public function new_invoice($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_receivable_invoice.new_invoice incoming data:';
				_debug_array($content);
			}
                        $invoice = new ReceivableInvoice();

                        $order_id = reg_var('order_id');
                        $shipment_id = reg_var('shipment_id');
			$client_id = reg_var('owner_id');

                        if($order_id) {
				$order =& new SalesOrder($order_id);
                                $invoice->read_from_object($order);
                        } elseif ($shipment_id) {
				$shipment = new OutgoingShipment($shipment_id);
                                $invoice->read_from_object($shipment);
				$lines = $invoice->get_lines();
				// Swap the InvoicableItem out so the Order gets invoiced.
				foreach($lines as &$line) {
					if($line->get_invoicable_item() instanceof ShipmentLine) {
						$line->set_invoicable_item($line->get_invoicable_item()->get_shippable_item());
					}
				}
			} elseif ($client_id) {
				$invoice->set_client_id((int)$client_id);
				$invoice->set_our_reference_number($invoice->suggest_invoice_number());

				$branch_list = ExecMethod('perp_ar.bo_perp_client_branch.get_client_branch_list', array('client_id' => $client_id));
				if(count($branch_list) == 1) {
					$invoice->set_client_branch_id((int)key($branch_list));
				}
				$account_list = ExecMethod('perp_ar.bo_perp_client_account.get_client_account_list', array('client_id' => $client_id));
				if(count($account_list) == 1) {
					$invoice->set_client_account_id((int)key($account_list));
					$account = ExecMethod('perp_ar.bo_perp_client_account.get_client_account', key($account_list));
					$invoice->set_payment_terms((int)$account['payment_terms']);
				}
			}

                        $GLOBALS['egw']->session->appsession('invoice', false, $invoice);
			return $this->edit($content);
		}

		/**
		*	Edit an invoice in progress.  It is not possible to change an invoice after it has been posted.
		*	The invoice is stored in the session variable 'invoice' before it is posted.
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo 'perp_ar.ui_perp_receivable_invoice.edit incoming data:';
				_debug_array($content);
			}

			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $GLOBALS['egw']->link('/index.php', 
				array('menuaction' => 'perp_ar.ui_perp_receivable_invoice.invoice_list')
			));
			
			// Get invoice
			$invoice_id = reg_var('invoice_id');
			                                
			// Get invoice
			$invoice =& $this->get_invoice($invoice_id);
			if(!$invoice) {
				$GLOBALS['egw']->redirect($preserve['return']);
			}

			// Save / Update
			$admin = $invoice->is_admin($GLOBALS['egw_info']['user']['account_id']) === True;                               
			$edit = $invoice->is_editor($GLOBALS['egw_info']['user']['account_id']) || $admin;

			$buttons = array(
				'save'		=>	true, 
				'post'		=>	true,
				'add_line'	=>	true, 
				'add_line_from'	=>	true,
				'calculate_taxes'	=>	true
			);
			if($edit && (count(array_intersect_key($buttons, $content)) > 0 || isset($content['nextmatch']['rows']['delete']))) {
				try {
					$this->update_invoice($invoice, $content);
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			}

			if(isset($content['cancel']) || !$GLOBALS['egw']->acl->check('invoice', EGW_ACL_READ, 'perp_ar')) {
				$GLOBALS['egw']->redirect($preserve['return']);
			}
			if($content['save']) {
				if($invoice->is_editor($GLOBALS['egw_info']['user']['account_id'])) {
					try {
						$result = $invoice->write();
					} catch (Exception $ex) {
						$this->messages[] = array($ex->getMessage(), 'error');
					}
					if(!is_numeric($result )) {
						$this->messages[] = array($result, 'error');
					}
						
					$GLOBALS['egw']->session->appsession('invoice', false, null);
				} else {
					$this->messages[] = array(lang('Cannot save invoice'), 'warning');
				}
			}
			if($content['post'] && $invoice->is_admin($GLOBALS['egw_info']['user']['account_id'])) {
				unset($content['post']);

				if($invoice->get_transaction()) {
					$this->messages[] = array(lang('Invoice has already been posted.'), 'warning');
				} else {
					try {
						$result = $invoice->post();
					} catch (Exception $ex) {
						$this->messages[] = array($ex->getMessage(), 'error');
					}

					if($result === true) {
						$this->messages[] = array(lang('Invoice posted'), 'success');

						// Any action to be taken?
						try {
							if($content['post_action']) {
								$this->handle_action($content['post_action'], $invoice->get_invoice_id());
							}
						} catch (Exception $ex) {
							$this->messages[] = array($ex->getMessage(), 'error');
						}
					} else {
						$this->messages[] = array(lang('Cannot post invoice'), 'error');
					}
				}
			}
			
			// Save to session
			if($invoice->is_editor($GLOBALS['egw_info']['user']['account_id'])) {
				$GLOBALS['egw']->session->appsession('invoice', false, $invoice);
			}

			// Reload is sometimes needed to avoid re-posting by popups
			if($GLOBALS['egw']->session->appsession('invoice_reload')) {
				$GLOBALS['egw']->session->appsession('invoice_reload', 'perp_ar', false);
				$GLOBALS['egw']->redirect_link('/index.php',
					array(
						'menuaction'    =>      'perp_ar.ui_perp_receivable_invoice.edit',
					)
				);
			}


			// Get data
			$data = array();
			$this->get_display_data($invoice, $data, $preserve);

			// Supporting data
			$select_box_options = $this->get_select_box_options($invoice);

			// Read onlys
			$readonly = $this->get_readonlys($invoice);
			if(!$readonly['add_line_from']) {
				$data['add_line_label'] = lang('Add Line From');
			}

			// Actions on post
			$this->get_post_actions($invoice, $data, $select_box_options, $readonly);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$this->get_javascript('perp_ar.ajax_perp_receivable_invoice');

			if($this->debug) {
				echo 'perp_ar.ui_perp_receivable_invoice.edit outgoing data:';
				_debug_array($data);
			}

			// Display
			if($invoice->get_invoice_id()) {
				if(!(boolean)$invoice->get_transaction()) {
					// Invoice has been saved, but not posted
					$GLOBALS['egw_info']['flags']['app_header'] = lang('Edit Invoice %1', $invoice->get_our_reference_number());
				} else {
					// Invoice has been posted, can't be edited
					$GLOBALS['egw_info']['flags']['app_header'] = lang('Invoice %1', $invoice->get_our_reference_number());
				}
			} else {
				// Invoice hasn't been saved yet
				$GLOBALS['egw_info']['flags']['app_header'] = lang('New Invoice');
			}
			$this->template->read('perp_ar.receivable_invoice_edit');
			$this->template->exec('perp_ar.ui_perp_receivable_invoice.edit', $data, $select_box_options, $readonly, $preserve); 

			/**
			*	Actions on post done here, so the page gets refreshed first
			*/
		}

		/**
		*	Internal function to get a ReceivableInvoice object to work with.
		*	First, we check the session for an invoice in progress.  Then we
		*	check the provided invoice ID an read from the database, if required.
		*/
		protected function &get_invoice($invoice_id) {
			$inv = $GLOBALS['egw']->session->appsession('invoice');
			if(
				($invoice_id && $inv instanceof ReceivableInvoice && $invoice_id != $inv->get_invoice_id()) || 
				$invoice_id && !($inv instanceof ReceivableInvoice)
			) {
				$inv = new ReceivableInvoice($invoice_id);
				// Check if the current user is a client
				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) > 0 && !in_array($inv->get_client_id(), $client_ids)) {
					return null;
				} else {
					return $inv;
				}
			} elseif ($inv) {
				return $inv;
			} else {
				return null;
			}
		}

		/**
		*	Handle an action from one or more Invoices on the list.
		*	Actions can be one of pay, new_invoice, post, ...
		*/
		public function &handle_action($action, $invoice_id) {
			if(!is_array($invoice_id)) {
				$invoice_id = array($invoice_id);
			}
			foreach($invoice_id as $id) {
				$invoices[] =& new ReceivableInvoice($id);
			}
			$result = true;
			switch($action) {
				case 'pay':
					$client_id = $invoices[0]->get_client_id();
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_ar.ui_perp_receipt.receipt_new',
						'client_id'	=>	$client_id
					));
					break;
				case 'post':
					foreach($invoices as &$invoice) {
						$result = $result && $invoice->post();
					}
					break;
				case 'cancel':
					foreach($invoices as &$invoice) {
						$result = $result && $invoice->cancel();
					}
					break;
				case 'invoice_report':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_api.ui_perp_report.get_report',
						'report_name'	=>	'perp_ar.invoice_report',
						'invoice_id'	=>	implode(',', $invoice_id)
					));
					break;
				default:
					// Check for email to client
					if(strpos($action, 'email') === 0) {
						$format = substr($action, 6);
						$result = $this->bo_invoice->email_invoice($invoice_id, $format);
					} else {
						foreach($invoice_id as $id) {
							$result = parent::handle_action($action, $id);
						}
					}
			}
			return $result;
		}

		/**
		*	Internal function to update a ClientInvoice with the information
		*	returned from eTemplate when the user submits the form
		*
		*	@param invoice	ClientInvoice to update
		*	@param content	array of information
		*/
		protected function update_invoice(&$invoice, &$content) {

			// Update header information
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $invoice->get_client_id());
			if($client) {
				$invoice->set_bill_to_address((int)$client['address']);
			}
			if($content['client_account_id']) {
				$invoice->set_client_account_id($content['client_account_id']);
			}
			if($content['client_branch_id']) {
				$invoice->set_client_branch_id($content['client_branch_id']);
			}

			parent::update_invoice($invoice, $content);
		}

		/**
		*	Add a new line to the invoice
		*	@param invoice ReceivableInvoice to add the line to
		*	@param line_data array of information for the new line
		*/
		protected function add_line(ReceivableInvoice &$invoice, $line_data) {
			$line =& new ReceivableInvoiceLine($invoice, $line_data);
			$invoice->add_line($line);
		}

		/**
		*	Internal function to get the data required for display
		*	
		*	@param invoice ClientInvoice to retrieve data from
		*	@param data	array to fill 
		*	@param preserve	array of data to keep across form submit
		*/
		protected function get_display_data(&$invoice, &$data, &$preserve) {
			$data = parent::get_display_data($invoice);

			$data['total'] = 0;
			$data['new']['no_actions'] = True;

			$data['client_account_id']	=	array(
				'client_id'		=>	(string)$invoice->get_client_id(),
				'client_account_id'	=>	$invoice->get_client_account_id()
			);
			$data['client_branch_id']	=	array(
				'client_id'		=>	(string)$invoice->get_client_id(),
				'client_branch_id'	=>	$invoice->get_client_branch_id()
			);
			$data['client_id']	= 	array(
				'client_id'		=>	(string)$invoice->get_client_id(),
				'status'		=>	'1'
			);
			$data['total']          =       sprintf("%.2f", $invoice->get_total());

			// Receipts
			$receipts =& $invoice->get_receipts();
			if(count($receipts) > 0) {
				array_unshift($receipts, false);
				$data['receipts'] =& $receipts;
			}

			return $data;
		}

		/**
		*	Internal function to set the widgets read only as required
		*
		*	@param invoice	ReceivableInvoice to retrieve data from
		*/
		protected function get_readonlys($invoice) {
			$readonly = parent::get_readonlys($invoice);

			if($invoice->get_invoice_id() || $read_only) {
				$readonly['client_account_id'] = array(
					'search'	=>	true
				);
				$readonly['client_branch_id'] = array(
					'search'	=>	true
				);
				$readonly['perp_ar.receivable_invoice_edit.row'] = True;
				$readonly['add'] = True;
			}

			if($invoice->get_paid_date || $read_only) {
				$readonly['hold']		=	True;
				$readonly['perp_ar.invoice_edit.row'] =	True;
			}

			$readonly['paid_date'] = true;
			$readonly['bank_account'] = True;
			$readonly['payment_method'] = True;

//			$readonly['comments'] = !$read_only;

			// Hide Add Line From button if no apps are allowed in Admin -> AR -> Site config
			$readonly['add_line_app'] = true;
			$readonly['add_line_from'] = true;
			$invoicable = $GLOBALS['egw']->hooks->process('perp_ar:invoicable');
			foreach($invoicable as $app => $settings) {
				if($settings['config']) {
					$config = ExecMethod($settings['config']);
					if($config['config']['enabled']) {
						$readonly['add_line_app'] = false;
						$readonly['add_line_from'] = false;
						break;
					}
				}
			}

			return $readonly;
		}

		/**
		*	Internal function to populate the selectbox options
		*
		*	@param invoice Invoice to get the options for
		*/
		protected function get_select_box_options(&$invoice) {
			$options = parent::get_select_box_options($invoice);

			$options['bank_account'] = array('' => lang('Not set')) + ExecMethod('perp_ledger.bo_perp_ledger.get_bank_accounts');

			// Client contacts
			if($invoice->get_client_id() != 0) {
				$contacts = ExecMethod2('egw_link::get_links', 'perp_ar', 'CLIENT-' . $invoice->get_client_id(), 'addressbook');
				foreach($contacts as $id => $contact) {
					$options['their_contact_person'][$contact] = egw_link::title('addressbook', $contact);
				}
				if(count($contacts) == 0) {
					$this->messages[] = array(lang('No addresses set for client'), 'error');
				}
			}

			// Bill from addresses
			$contacts = array();
			$config = config::read('perp_ar');
			if($config['default_bill_from']) {
				$contacts[] = $config['default_bill_from'];
			}
			$locations = ExecMethod2('perp_inventory.bo_perp_location.get_location_list');
			foreach($locations as $location_id => $name) {
				$contacts += egw_link::get_links('perp_inventory', 'LOCATION-' . $location_id, 'addressbook');
			}
			foreach($contacts as $id => $contact) {
				$options['bill_from_address'][$contact] = egw_link::title('addressbook', $contact);
			}

			// Get allowed apps for adding InvoicableItems
			$invoicable = $GLOBALS['egw']->hooks->process('perp_ar:invoicable');
			foreach($invoicable as $app => $settings) {
				if($settings['config']) {
					$config = ExecMethod($settings['config']);
					if($config['config']['enabled']) {
						$options['add_line_app'][$app] = lang($app);
					}
				}
			}

			return $options;
		}

		/** 
		*	Add a setup function specific to receivable invoices
		*/
		protected function get_javascript($ajax_callback) {
			parent::get_javascript($ajax_callback);

			$GLOBALS['egw']->js->set_onload('
				if (\'function\' == typeof window.receivable_invoice_edit_setup) {
					receivable_invoice_edit_setup(); 
				}
			');

			if($GLOBALS['egw']->session->appsession('invoice_popup')) {
				$popup_link = $GLOBALS['egw']->link('/index.php',
					array(
						'menuaction'    =>      'perp_ar.ui_perp_receivable_invoice.add_line_from',
						'appname'	=>	$GLOBALS['egw']->session->appsession('invoice_app'),
					)
				);
				$GLOBALS['egw']->js->set_onload("window.open('$popup_link', '_blank','dependent=yes,width=700,height=600,scrollbars=yes,status=yes'); ");
				$GLOBALS['egw']->session->appsession('invoice_popup', false, false);
				$GLOBALS['egw']->session->appsession('invoice_app', false, false);
			}
		}

		/**
		*	Set up a list of actions that can be done when the invoice is posted
		*
		*/
		protected function get_post_actions(&$invoice, &$data, &$options, &$readonly) {
			// Get allowed actions on post
			if((boolean)$invoice->get_transaction()) {
				$readonly['post_action'] = true;
				return;
			}

			$data['post_action_label'] = lang('Action on post:');

			// Reports
			/**
			* This causes problems because the report generation is a redirect, which makes the invoice
			* look like it hasn't been posted because the page doesn't refresh.
			if($GLOBALS['egw']->acl->check('report_invoice_report', EGW_ACL_READ, 'perp_ar')) {
				$options['post_action'][lang('Create Report')]['invoice_report'] = lang('report_invoice_report');
			}
			*/

			// Email
			if($invoice->get_client_id() && $invoice->get_bill_from_address()) {
				$client = ExecMethod('perp_ar.bo_perp_client.get_client', $invoice->get_client_id());
				$bill_to_address = ExecMethod('addressbook.addressbook_bo.read', $client['address']);
				$bill_from_address = ExecMethod('addressbook.addressbook_bo.read', $invoice->get_bill_from_address());
				
				// Automatically email on post
				if($bill_from_address['email'] && ($bill_to_address['email'] || $bill_to_address['email_home'])) {
					$formats = ExecMethod('perp_api.bo_perp_report.get_supported_formats');
					foreach($formats as $format => $name) {
						$options['post_action'][lang('Email to client')]['email_' . $format] = $name;
					}
				}
			}

			// Allow a 'Do nothing' option
			if(count($options['post_action'])) {
				$options['post_action'] = array_merge(array('' => lang('Just post')), $options['post_action']);

				// Set a default
				$data['post_action'] = '';
				if($GLOBALS['egw_info']['user']['preferences']['perp_ar']['post_action'] == 'email') {
					$data['post_action'] = 'email_' . $this->bo_invoice->get_preferred_report_format($invoice);
				}
			} else {
				$data['post_action_label'] = '';
				$readonly['post_action'] = true;
			}
		}

		/**
		*	Get a list of items from other eGroupWare applications that can be invoiced.
		*
		*	Supporting applications should define a hook to the location 
		*	'perp_ar:invoicable'.  An array with the client_id will be passed to the 
		*	hook function, which should return an array of InvoicableItem objects.
		*
		*	In addition to the standard nextmatch parameters available, 
		*	client_id must be set.  Setting appname will only search that app, which is good.
		*
		*	@nextmatch
		*/
		public function get_uninvoiced_lines(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$data = array();
			$count = $this->bo_invoice->get_uninvoiced_lines($query, $data, $readonlys);

			if($count == 0) {
				$rows = array();
				return $count;
			}

			/* The data comes in the form
			*	app_name => array(Invoicable [, Invoicable]* )
			* We're going to unshift it and add in application headers here
			*/

			$invoice =& $this->get_invoice(null);

			if($invoice == null) {
				return 0;
			}

			$invoicable_hooks = $GLOBALS['egw']->hooks->process('perp_ar:invoicable');

			// Add in application headers
			$rows = array();
			$result_count = 0;
			foreach($data as $app_name => $results) {
				if(!is_array($results)) {
					continue;
				}
				$rows[] = array(
					'description'	=>	lang($app_name),
					'class'		=>	'app_heading',
				);
				$result_count++;
				foreach($results as $item) {
					if(!($item instanceof InvoicableItem)) {
						continue;
					}
					$class = $invoicable_hooks[$app_name]['Invoicable'];
					if(!$class) {
						$class = get_class($item);
						if(property_exists($class, 'invoicable_class')) {
							$prop = new ReflectionProperty($class, 'invoicable_class');
							$obj = new $class($row);
							$class = $prop->getValue($obj);
						}
					}
					$rows[] = array(
						'invoicable_class'	=>	$class,
						'invoicable_id'		=>	$item->get_invoicable_id(),
						'line_number'		=>	$item->get_invoicable_line_id(),
						'our_reference_number'	=>	'',
						'description'	=>	$item->get_description(),
						'invoice_quantity'	=>	$item->get_invoice_quantity(),
						'unit_price'		=>	$item->get_unit_price(),
						'formatted_unit_price'	=>	ExecMethod2(
							'perp_api.bo_perp_api.get_formatted_price', 
							$invoice->get_invoice_currency(), 
							$item->get_unit_price()
						),
						'general_ledger_account'		=>	$item->get_gl_account()
					);
				}
			}
			$rows = array_slice($rows, $query['start'], $query['num_rows'] + $result_count);
			array_unshift($rows, false);
			return $count;
		}
	}
?>
