<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.OutgoingShipment.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');

	/**
	*	Packing Slip
	*/
	class bill_of_lading extends perp_report {

		/**
		*	A list of details to remove from the detail list, by name.
		*/
		protected $hide_details = array(
			'price_override',
			'work_order',
			'Total',
		);

		public function bill_of_lading(array $parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';
			parent::perp_report($parameters);

			$this->template = 'bill_of_lading.odt';

			$this->parameters = $parameters;

			// Default parameters
			if(!isset($parameters['shipment_id'])) {
				$this->parameters['shipment_id']['status'] = 'approved';
			}
		}

		public function get_data($query = array()) {
			if(!is_numeric($query['shipment_id'])) {
				return false;
			}

			$this->bo_inventory = new bo_perp_inventory();
			$this->unit_of_measure = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			$shipment = new OutgoingShipment($query['shipment_id']);

			// Get containers
			$GLOBALS['egw']->db->select(
				'perp_shipment_line',
				array(
					'perp_shipment_line.line_no',
					'detail_name',
					'detail_value',
					'display',
					'package_id',
					'package_details'
				),
				array(
					'perp_shipment_line.shipment_id='.$query['shipment_id']
				),
				__LINE__, __FILE__,0,
				'ORDER BY detail_name, package_id', 'perp_ar', -1,
				'LEFT JOIN (
					perp_shipment_detail 
					JOIN perp_outgoing_shipment_package ON
						perp_outgoing_shipment_package.shipment_id = perp_shipment_detail.shipment_id AND
						perp_outgoing_shipment_package.line_no = perp_shipment_detail.line_no AND
						perp_outgoing_shipment_package.detail_id = perp_shipment_detail.detail_id AND
						detail_class=\'ShipmentPackage\'
				) ON
					perp_shipment_line.shipment_id = perp_shipment_detail.shipment_id AND
					perp_shipment_line.line_no = perp_shipment_detail.line_no'
			);
			$content_list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$content_list[$row['detail_name']][] = $row;
			}

			$item_index = 0;
			foreach($content_list as $name => &$contents) {
				$container = $this->make_container($contents, $shipment);
				for($item_index; $item_index < count($this->items); $item_index++) {
					$this->items[$item_index]['container_mass'] = $container['mass'];
					// Stupid imperial measurement system...
					$this->items[$item_index]['container_mass_lb'] = $container['mass'] * 2.20462262;

					if(!$this->items[$item_index]['container_quantity']) {

						// Unpackaged or loose packed items
						$this->items[$item_index]['container_quantity'] = $container['quantity'];
						foreach($this->items[$item_index]['packing'] as $key => $package) {
							$this->items[$item_index]['package_details'] .= $package . "\t" . $this->items[$item_index]['mass'][$key] . " kg\n";
						}
					} else {
						$this->items[$item_index]['package_details'] = ''; //implode("\n", $this->items[$item_index]['packing']);
					}
				}
				$this->total_mass += $container['mass'];
				$this->total_pieces += $container['quantity'];

				$containers[$container['container_name']] = $container;
			}

			$data['our_reference_number']	=	$shipment->get_our_reference_number();
			$data['comments']		=	$shipment->get_comments();
			$data['ship_from'][]		=	ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_from_address());
			$data['ship_to'][]		=	ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_to_address());
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $shipment->get_client_id());
			$shippers = ExecMethod('perp_api.bo_perp_api.get_shippers', array('shipper_id' => $shipment->get_shipper_id()));
			$data['shipper_name']		=	$shippers[$shipment->get_shipper_id()];
			$data['customer_number']	=	$client['client_num'];
			$data['originating_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $shipment->get_originating_date());
			$data['requested_delivery_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $shipment->get_requested_delivery_date());
			if($shipment->get_actual_delivery_date()) {
				$data['actual_delivery_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $shipment->get_actual_delivery_date());
			} else {
				$data['actual_delivery_date']	=	'';
			}
			$data['total_mass']		=	$this->total_mass;
			$data['total_pieces']		=	$this->total_pieces;
			$data['declared_value']		=	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $shipment->get_invoice_currency(), $this->declared_value);


			$data['item'] = $this->items;
			$data['container'] = $containers;

			$this->filename = lang('BOL') . ' ' . $shipment->get_our_reference_number();
//_debug_array($data); die();
			return $data;
		}

		/**
		*	@param contents array of container contents
		*	@param shipment OutgoingShipment
		*/
		protected function make_container($contents, $shipment) {
			$container = array(
				'container_name'	=>	$contents[0]['detail_name'],
				'container_quantity'	=>	0,
			);
			foreach($contents as $package) {
				$shipment_line = $shipment->get_line($package['line_no']);
				$item = $this->make_line($shipment_line);
				foreach($shipment_line->get_details() as $detail) {
					if($detail instanceof ShipmentPackage) {
						continue;
					} elseif ($detail->get_name() != $package['detail_name']) {
						continue;
					} elseif ($detail instanceof ShipmentSerial) {
						$item['details'][] = lang('serial number') . ': ' . $detail->get_serial_number();
					} else {
						$item['details'][] = $detail->get_name() . ': ' . $detail->get_value();
					}
				}
				$item['package_count'] = lang("(%1 of %2 pieces)", $package['package_id'] + 1, $item['piece_count']);
				if($container['container_name']) {
					$container['container_quantity'] += $package['detail_value'];
					$item['quantity'] = $package['detail_value'];
				} else {
					// Not packaged, but on their own
					$container['quantity'] = $item['quantity'];
					$container['mass'] = array_sum($item['mass']) * $container['quantity'];
					$container['pieces'] = $package['package_details'];
				}
				if(is_array($item['mass'])) {
					$container['mass'] += array_sum($item['mass']) * $package['detail_value'];
				} else {
					$container['mass'] += $item['mass'] * $package['detail_value'];
				}
				$item += $container;
				$item['details'] 		=	implode("\n", $item['details']);
				$this->items[] = $item;
			}
			return $container;
		}

		private function make_line(ShipmentLine $shipment_line) {
			$line = array();
			$packing = array();
			$stock_id = $shipment_line->get_shippable_item()->get_stock_id();
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);

			$line['stock_num']		=	$stock['stock_num'];
			$line['description']		=	$shipment_line->get_description();
			$line['details']		=	array();
			$details =& $shipment_line->get_shippable_item()->get_details();
			foreach($details as $detail) {
				if($detail->get_display() && $detail->get_value()) {
					
					// Get serial number(s)
					if($detail instanceof WorkOrderDetail) {
						$packing = ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id, $detail->get_work_order_id(), $detail->get_work_order_detail_id());
						$controlled_query = array(
							'stock_id'	=>	$stock_id,
							'location'	=>	$location_id,
							'work_order_id'	=>	$detail->get_work_order_id(),
							'detail_id'	=>	$detail->get_work_order_detail_id(),
						);
						$this->bo_inventory->get_controlled_inventory_list($controlled_query, $controlled_list);
						if($serial = $controlled_list[0]) {
							// Serial number needed, set from work order
							$line['details'][]	=	lang('serial number') . ': ' . $serial['serial_number'];
						} elseif ($stock['controlled']) {
							// Serial number needed, but not provided
							$line['details'][]	=	lang('serial number') . ': ' . '____________';
						} 
					}
					if(!in_array($detail->get_name(), $this->hide_details) ) {
						$line['details'][]		=	lang($detail->get_name()) . ': ' . $detail->get_value();
					}
				}
			}
			$line['unit_of_measure']	=	$this->unit_of_measure[$stock['unit_of_measure']];
			if($quantity == 0) {
				$line['quantity']	=	(float)$shipment_line->get_quantity_shipped();
			} else {
				$line['quantity']	=	(float)$quantity;
			}

			$this->declared_value 		+=	$shipment_line->get_unit_price() * $shipment_line->get_quantity_shipped();

			if(count($packing) == 0) {
				$packing = ExecMethod('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id);
			}
			foreach($packing as $package) {
				$line['packing'][] = (float)$package['length'] . ' x ' . 
					(float)$package['width'] . ' x ' . 
					(float)$package['height'] . ' ' . 
					$this->unit_of_measure[$package['unit_of_measure']];
				$line['mass'][] = $package['mass'];
			}
			$line['piece_count']		=	count($packing);

			return $line;
		}
	}
?>
