<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.InvoiceLine.inc.php');

	/**
	*	Report that lists client transactions 
	*	One line per transaction, grouped by currency then client
	*/
	class client_transactions extends perp_report {

		public function client_transactions (array $parameters = array()) {
			parent::perp_report($parameters);

			$this->template = 'client_transactions.odt';

			// Default parameters
			$config = CreateObject('phpgwapi.config', 'perp_ledger');
			$config->read_repository();
			if(!$parameters['start_date']) {
				$this->parameters['start_date'] = (isset($config->config_data['last_year_end']) ? $config->config_data['last_year_end'] : 0);
				$this->parameters['end_date'] = date('U');
				$this->parameters['options-currencies'] = ExecMethod('perp_api.bo_perp_api.get_currencies');
				$this->parameters['start_client'];
				$this->parameters['end_client'];
				$this->parameters['only_non_zero'] = true;
				$this->parameters['only_unpaid_invoices'] = false;
			}
		}

		public function get_data($query = array()) {

			// Set defaults and parameter strings for report
			if(!$query['start_date']) {
				$query['start_date'] = 0;
			}
			if($query['start_client']) {
				$start_client = ExecMethod('perp_ar.bo_perp_client.get_client', $query['start_client']);
				$query['start_client'] = $start_client['client_num'];
			}
			if($query['end_client']) {
				$end_client = ExecMethod('perp_ar.bo_perp_client.get_client', $query['end_client']);
				$query['end_client'] = $end_client['client_num'];
			}
			
			$data = array(
				'date_range'	=>	lang('date range %1 to %2', 
								date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['start_date']),
								date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date'])
							),
				'currencies'	=>	(is_array($query['currencies']) || $query['currencies'] != '' ? $query['currencies'] : lang('All')),
			);
			if($query['start_client'] || $query['end_client']) {
				$data['client_range'] = lang('Supplier: %1 to %2',
					($query['start_client'] ? $query['start_client'] : lang('all')),
					($query['end_client'] ? $query['end_client'] : lang('all')) 
				);
			} else {
				$data['client_range'] = lang('all clients');
			}
			if($query['only_unpaid_invoices']) {
				$data['include_paid'] .= lang('Only unpaid invoices');
			} else {
				$data['include_paid'] .= lang('all transactions');
			}

			if($query['currencies']) {
				$query['currencies'] = explode(',', $query['currencies']);
			}

			// Big nasty SQL
			$sql = 'SELECT
				perp_client_ledger.trans_id,
				perp_client.client_id,
				perp_client_ledger.exchange_rate,
				narrative AS our_reference,
				perp_invoice.their_reference_number AS their_reference,
				- ( perp_general_ledger.amount / perp_client_ledger.exchange_rate) AS original_amount,
				invoice_total.total AS invoice_total,
				IF(receipt_total.total IS NULL, perp_client_receipt.amount, receipt_total.total) AS receipt_total,
				perp_invoice.invoice_paid,
				(
					IF(invoice_total.total IS NULL, 0, invoice_total.total) - 
					(
						IF(receipt_total.total IS NULL, IF(perp_client_receipt.amount IS NULL, 0, perp_client_receipt.amount), receipt_total.total) +
						IF(discount_taken IS NULL, 0, discount_taken)
					)
				) AS amount_owing,
				-IF(total_payable.total IS NULL, 0, total_payable.total) AS total_payable,
				perp_invoice.discount_taken,
				perp_invoice.due_date,
				perp_general_ledger.trans_date AS bill_date,
				perp_invoice.discount_date,
				perp_client.client_num,
				perp_client.client_name,
				perp_client.currency,
				perp_currency.currency_name
			FROM perp_client
			LEFT JOIN perp_client_ledger ON
				perp_client_ledger.client_id = perp_client.client_id
			LEFT JOIN perp_general_ledger ON
				perp_client_ledger.trans_id = perp_general_ledger.trans_no AND
				perp_client_ledger.line_no = perp_general_ledger.line_no
			LEFT JOIN (
				SELECT
					client_id,
					sum(amount / exchange_rate) AS total
				FROM perp_client_ledger
				JOIN perp_general_ledger ON
					perp_general_ledger.trans_no = perp_client_ledger.trans_id AND
					perp_general_ledger.line_no = perp_client_ledger.line_no
				GROUP BY client_id
			) AS total_payable ON
				perp_client.client_id = total_payable.client_id
			LEFT JOIN (
				perp_invoice JOIN (
					SELECT
						invoice_id,
						sum(unit_price * quantity) AS total
					FROM perp_invoice_line
					GROUP BY invoice_id
				) AS invoice_total ON
					perp_invoice.invoice_id = invoice_total.invoice_id
				JOIN (
					SELECT
						invoice_id,
						sum(unit_price * quantity) AS total
					FROM perp_invoice_line
					WHERE line_type != ' . InvoiceLine::TAX . '
					GROUP BY invoice_id
				) AS pretax ON
					perp_invoice.invoice_id = pretax.invoice_id 
				JOIN perp_payment_term ON
					perp_invoice.payment_terms = perp_payment_term.payment_term_id
			) ON
				perp_invoice.invoice_transaction_id = perp_client_ledger.trans_id
			LEFT JOIN (
				SELECT 
					invoice_id,
					sum(amount) AS total
				FROM perp_client_receipt 
				GROUP BY invoice_id
			) AS receipt_total ON
				receipt_total.invoice_id = perp_invoice.invoice_id
			LEFT JOIN perp_client_receipt ON
				!perp_client_receipt.invoice_id AND
				perp_client_receipt.trans_id = perp_client_ledger.trans_id
			JOIN perp_currency ON
				perp_client.currency = perp_currency.currency_id ';

			if($query['only_unpaid_invoices']) {
				$where[] = '!perp_invoice.invoice_paid';
			}
			if($query['currencies']) {
				$where[] = 'perp_currency.currency_id IN (\'' . implode("','", $query['currencies']) . '\') ';
			}
			if($query['start_client']) {
				$where[] = 'perp_client.client_num >= \'' . $query['start_client'] . '\' ';
			}
			if($query['end_client']) {
				$where[] = 'perp_client.client_num <= \'' . $query['end_client'] . '\' ';
			}
			if($query['start_date']) {
				$where[] = 'perp_general_ledger.trans_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['start_date']));
			}
			if($query['end_date']) {
				$where[] = 'perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['end_date']));
			}
			if(count($where) > 0) {
				$sql .= ' WHERE ' . implode(' AND ', $where);
			}

			if($query['only_non_zero']) {
				$having[] = 'total_payable != 0';
			}
			if(is_numeric($query['start_balance'])) {
				$having[] = 'total_payable >= ' . $query['start_balance'] . ' ';
			}
			if(is_numeric($query['end_balance'])) {
				$having[] = 'total_payable <= ' . $query['end_balance'] . ' ';
			}
			if(count($having) > 0) {
				$sql .= ' HAVING ' . implode(' AND ', $having);
			}

			$sql .= ' ORDER BY perp_currency.currency_order, perp_client.client_num, perp_general_ledger.trans_date';

			$invoices = array();
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['balance'] = $row['total_payable'];
				$row['current_amount'] = $row['amount_owing'];

				$row['discount_amount'] = 0;
				if($row['discount_date'] >= date('U')) {
					$discount = round(($row['discount_percent'] / 100) * $row['pretax_subtotal'] - $row['discount_taken'], 2);
					if($discount != 0) {
						$row['discount_amount'] = $GLOBALS['perp_api']['bo']->get_formatted_price($row['invoice_currency'], $discount);
					}
				}

				// Format dates according to user's preferences
				$row['bill_date'] = $GLOBALS['egw']->db->from_timestamp($row['bill_date']);
				if($row['bill_date']) {
					$row['bill_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['bill_date']);
				}
				$row['due_date'] = $GLOBALS['egw']->db->from_timestamp($row['due_date']);
				if($row['due_date']) {
					$row['due_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['due_date']);
				}
				$row['discount_date'] = $GLOBALS['egw']->db->from_timestamp($row['discount_date']);
				if($row['discount_date']) {
					$row['discount_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['discount_date']);
				}
				$invoices[] = $row;
			}

			$total_currency = array();	// Summaries at the bottom
			$currency_total = array();	// Running totals
			$currency = '';
			foreach($invoices as $key => &$detail_line) {
				// Calculate currency totals
				if($currency != $detail_line['currency']) {
					if(count($currency_total)) {
						// Keep total currency
						$currency_total['currency'] = $currency;
						$total_currency[] = $currency_total;
					}
					$currency = $detail_line['currency'];
					$currency_total = array(
						'original_amount'	=>	0,
						'current_amount'	=>	0,
						'discount_amount'	=>	0,
					);
				}

				// Set totals
				foreach($currency_total as $c_column => &$c_total) {
					$c_total += $detail_line[$c_column];
					$detail_line['currency_total_' . $c_column] = $c_total;
				}
			}

			if(count($invoices) > 0) {
				// Get last currency
				$currency_total['currency'] = $currency;
				$total_currency[] = $currency_total;
			}
			$currency_total = array();

			$data['detail'] = $invoices;

			// Get currency names
			$currency_list = ExecMethod('perp_api.bo_perp_api.get_currencies');
			foreach($total_currency as &$currency) {
				$currency['currency_name'] = $currency_list[$currency['currency']];
			}
			$data['total'] = $total_currency;

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}
	}
?>
