<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');

	/**
	*	A report that lists accounts receivable by days overdue
	*	Grouped by currency then client
	*/
	class detailed_aged_accounts_receivable extends perp_report {

		public function detailed_aged_accounts_receivable (array $parameters = array()) {
			parent::perp_report($parameters);

			$this->template = 'detailed_aged_accounts_receivable.odt';

			// Default parameters
			if(!$parameters['end_date']) {
				$this->parameters['end_date'] = date('U');
				$this->parameters['options-currencies'] = ExecMethod('perp_api.bo_perp_api.get_currencies');
				$this->parameters['start_client'];
				$this->parameters['end_client'];
			}
		}

		public function get_data($query = array()) {
			if($query['start_client']) {
				$start_client = ExecMethod('perp_ar.bo_perp_client.get_client', $query['start_client']);
				$query['start_client'] = $start_client['client_num'];
			}
			if($query['end_client']) {
				$end_client = ExecMethod('perp_ar.bo_perp_client.get_client', $query['end_client']);
				$query['end_client'] = $end_client['client_num'];
			}
			
			$data = array(
				'end_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date']),
				'currencies'	=>	(is_array($query['currencies']) || $query['currencies'] != '' ? $query['currencies'] : lang('All')),
			);
			if($query['start_client'] || $query['end_client']) {
				$data['client_range'] = lang('Client: %1 to %2',
					($query['start_client'] ? $query['start_client'] : lang('all')),
					($query['end_client'] ? $query['end_client'] : lang('all')) 
				);
			} else {
				$data['client_range'] = lang('all clients');
			}
			if($query['currencies']) {
				$query['currencies'] = explode(',', $query['currencies']);
			}

			$sql = 'SELECT
				perp_client_ledger.trans_id,
				perp_client_ledger.client_id,
				perp_client_ledger.exchange_rate,
				perp_general_ledger.trans_date AS bill_date,
				perp_general_ledger.narrative AS our_reference,
				perp_invoice.their_reference_number AS their_reference,
				(perp_general_ledger.amount / perp_client_ledger.exchange_rate) AS original_amount,
				invoice_total.total AS invoice_total,
				IF(receipt_total.total IS NULL, perp_client_receipt.amount, receipt_total.total) AS receipt_total,
				perp_invoice.invoice_paid,
				(
					IF(invoice_total.total IS NULL, 0, invoice_total.total) - 
					IF(receipt_total.total IS NULL, IF(perp_client_receipt.amount IS NULL, 0, perp_client_receipt.amount), receipt_total.total)
				) AS amount_owing,
				perp_invoice.discount_taken,
				perp_invoice.due_date,
				case 
					when perp_invoice.due_date IS NULL then \'current\'
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 0 then \'current\'
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 30 then 30
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 60 then 60
					else \'more\'
				end AS age,
				perp_client.client_num,
				perp_client.client_name,
				perp_client.currency AS client_currency,
				perp_currency.currency_name
				FROM perp_client_ledger
				JOIN perp_general_ledger ON
					perp_client_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_client_ledger.line_no = perp_general_ledger.line_no
				LEFT JOIN (
					perp_invoice JOIN (
						SELECT
							invoice_id,
							sum(unit_price * quantity) AS total
						FROM perp_invoice_line
						GROUP BY invoice_id
					) AS invoice_total ON
						perp_invoice.invoice_id = invoice_total.invoice_id
				) ON
					perp_invoice.invoice_transaction_id = perp_client_ledger.trans_id
				LEFT JOIN (
					SELECT 
						invoice_id,
						sum(amount) AS total
					FROM perp_client_receipt 
					GROUP BY invoice_id
				) AS receipt_total ON
					receipt_total.invoice_id = perp_invoice.invoice_id
				LEFT JOIN perp_client_receipt ON
					!perp_client_receipt.invoice_id AND
					perp_client_receipt.trans_id = perp_client_ledger.trans_id
				JOIN perp_client ON
					perp_client_ledger.client_id = perp_client.client_id
				JOIN perp_currency ON
					perp_client.currency = perp_currency.currency_id
				WHERE (perp_invoice.invoice_id IS NULL OR !perp_invoice.invoice_paid) ';
			if($query['currencies']) {
				$sql .= 'AND perp_currency.currency_id IN (\'' . implode("','", $query['currencies']) . '\') ';
			}
			if($query['start_client']) {
				$sql .= 'AND perp_client.client_num >= \'' . $query['start_client'] . '\' ';
			}
			if($query['end_client']) {
				$sql .= 'AND perp_client.client_num <= \'' . $query['end_client'] . '\' ';
			}
			if($query['end_date']) {
				$sql .= ' AND perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['end_date']));
			}

			$sql .= 'GROUP BY perp_invoice.invoice_id, perp_client_receipt.invoice_id
				HAVING amount_owing != 0
				ORDER BY perp_currency.currency_order, perp_client.client_num, perp_general_ledger.trans_date';

			$invoices = array();
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['amount_owing'] -= $row['discount_taken'];
				$invoices[] = $row;
			}

			$total_currency = array();
			$currency_total = array();
			$client = '';
			$currency = '';

			foreach($invoices as $key => &$invoice) {
				// Massage data to fit into report


				// Set defaults
				$invoice['more'] = 0;
				$invoice['60'] = 0;
				$invoice['30'] = 0;
				$invoice['current'] = 0;
				$invoice['total_overdue'] = 0;

				if($currency != $invoice['client_currency']) {
					if(count($currency_total)) {
						// Keep total currency
						$currency_total['currency'] = $currency;
						$total_currency[] = $currency_total;
					}
					$currency = $invoice['client_currency'];
					$currency_total = array(
						'current'	=>	0,
						'30'		=>	0,
						'60'		=>	0,
						'more'		=>	0,
						'overdue'	=>	0,
						'total'		=>	0
					);
				}
				if($client != $invoice['client_id']) {
					$client = $invoice['client_id'];
					$client_total = array(
						'current'	=>	0,
						'30'		=>	0,
						'60'		=>	0,
						'more'		=>	0,
						'overdue'	=>	0,
						'total'		=>	0
					);
				}

				$invoice[$invoice['age']] = $invoice['amount_owing'];
				$client_total[$invoice['age']] += $invoice['amount_owing'];
				$currency_total[$invoice['age']] += $invoice['amount_owing'];
				if($invoice['age'] != 'current') {
					$invoice['total_overdue'] = $invoice['amount_owing'];
					$client_total['overdue'] += $invoice['amount_owing'];
					$currency_total['overdue'] += $invoice['amount_owing'];
				}
				$invoice['total_payable'] = $invoice['amount_owing'];
				$client_total['total'] += $invoice['amount_owing'];
				$currency_total['total'] += $invoice['amount_owing'];

				// Set totals
				foreach($client_total as $s_range => $s_total) {
					$invoice['client_total_' . $s_range] = $s_total;
				}
				foreach($currency_total as $c_range => $c_total) {
					$invoice['currency_total_' . $c_range] = $c_total;
				}
			}

			// Get last currency
			$currency_total['currency'] = $currency;
			if(count($invoices) == 0) {
				// Define empty totals so TBS doesn't complain
				$total_currency[] = array(
					'currency'	=>	'',
					'currency_name'	=>	'',
					'current'	=>	0,
					'30'		=>	0,
					'60'		=>	0,
					'more'		=>	0,
					'overdue'	=>	0,
					'total'		=>	0
				);
			} else {
				$total_currency[] = $currency_total;

				// Get currency names
				$currency_list = ExecMethod('perp_api.bo_perp_api.get_currencies');
				foreach($total_currency as &$currency) {
					$currency['currency_name'] = $currency_list[$currency['currency']];
				}
			}

			$data['detail'] = $invoices;
			$data['total'] = $total_currency;

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}
	}
?>
