<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.ReceivableInvoice.inc.php');

	/**
	*	Accounts receivable invoice
	*/
	class invoice_report extends perp_report {

		public function invoice_report(array &$parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';
			parent::perp_report($parameters);

			$this->template = 'invoice.odt';

			// Default parameters
			if(isset($parameters['client_id']) && !isset($parameters['invoice_id'])) {
				$this->parameters['invoice_id']['client_id'] = (string)$parameters['client_id'];
			}

			// These options are copied from perp_api.ui_perp_invoice
			$this->parameters['options-filter'] = array(false => 'No details', 1 => 'Some details', 2 => 'Full details');

			$this->parameters['invoice_id'] = $parameters['invoice_id'] ? $parameters['invoice_id'] : reg_var('invoice_id');
			$this->parameters['filter'] = $parameters['filter'] ? $parameters['filter'] : reg_var('filter');

			if(!$this->parameters['invoice_id']) {
				$this->parameters['issue_date'] = 'now';
			}

			// If invoice_id is provided, just generate the report
			if($this->parameters['invoice_id']) {
				if(strpos($this->parameters['invoice_id'], ',') === false || $this->parameters['filter']) {
					// Just one invoice, or filter parameter is set
					$parameters['generate_report'] = true;
				}
				// Otherwise, let user change filter
			}
		}

		public function get_data($query = array()) {
			if(!$query['invoice_id'] && $query['issue_date']) {
				// Get a list of invoices dated on or after the invoice date
				$invoice_query = array(
					'client_id'   =>      $query['client_id'],
					'issue_date'    =>      array(
						'start' =>      $query['issue_date']
					)
				);
				$bo = new bo_perp_receivable_invoice();
				$invoices = array();
				$count = $bo->get_invoice_list($invoice_query, $invoices);
				if($count == 0) {
					return false;
				}
				foreach($invoices as $invoice) {
					$query['invoice_id'][] = $invoice['invoice_id'];
				}
			}
			if(!$query['invoice_id']) {
				return false;
			}

			$query['invoice_id'] = explode(',', $query['invoice_id']);

			$data = array();
			$this->terms = ExecMethod('perp_api.bo_perp_api.get_payment_terms');

			foreach($query['invoice_id'] as $invoice_id) {
				
				$data['invoice'][] = $this->get_header($invoice_id, $query);
				$lines = $this->get_lines($invoice_id, $query);
				$data[$invoice_id . '_line'] = $lines['line'];
				$data += $lines['details'];
				$data[$invoice_id . '_taxes'] = $lines['taxes'];
			}

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			if(count($query['invoice_id']) == 1) {
				$this->filename = lang('Invoice') . ' ' . $query['invoice']->get_our_reference_number();
			} else {
				$this->filename = lang('Multiple receivable invoices');
			}
//_debug_array($data); die();
			return $data;
		}

		protected function get_header($invoice_id, &$query) {
			$invoice =& new ReceivableInvoice($invoice_id);
			$query['invoice'] =& $invoice;

			$data = array(
				'invoice_id'		=>	$invoice_id,
				'our_reference'		=>	$invoice->get_our_reference_number(),
				'their_reference'	=>	$invoice->get_their_reference_number(),
				'comment'		=>	$invoice->get_comments(),
				'bill_date'		=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $invoice->get_bill_date()),
				'due_date'		=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $invoice->get_due_date()),
				'subtotal'		=>	$invoice->get_total(InvoiceLine::NORMAL) + $invoice->get_total(InvoiceLine::DISCOUNT),
				'total'			=>	$invoice->get_total(),
				'paid'			=>	'',
				'owing'			=>	'',
			);
			$data['subtotal']	=	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $invoice->get_invoice_currency(), $data['subtotal']);
			$data['total']		=	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $invoice->get_invoice_currency(), $data['total']);
			$data['terms']		=	$this->terms[$invoice->get_payment_terms()];

			// Get any payments
			$receipts = $invoice->get_receipts();
			foreach($receipts as &$row) {
				$data['paid'] += $row['amount'];
			}
			$data['owing'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $invoice->get_invoice_currency(), $invoice->get_total() - $data['paid']);
			$data['paid'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $invoice->get_invoice_currency(), $data['paid']);

			$client_id = $invoice->get_client_id();
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $client_id);

			// Client's billing address
			$data['bill'] = ExecMethod('addressbook.addressbook_bo.read', $invoice->get_bill_to_address());

			// Payable address
			if($invoice->get_bill_from_address()) {
				$data['pay'] = ExecMethod('addressbook.addressbook_bo.read', $invoice->get_bill_from_address());
			}

			return $data;
		}

		/**
		*	Get the lines for the invoice
		*/
		protected function get_lines($invoice_id, $query) {

			$data = array();
			$data['taxes'] = array();

			// Use the UI function to get all the nice details
			$ui = new ui_perp_receivable_invoice();
			$ui->get_invoice_lines($query, $data['line'], $readonlys);

			foreach($data['line'] as $key => &$line) {
				if(!is_array($line)) {
					unset($data['line'][$key]);
					continue;
				}
				$line['line_price'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $query['invoice']->get_invoice_currency(), $line['unit_price'] * $line['quantity']);
				$line['unit_price'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $query['invoice']->get_invoice_currency(), $line['unit_price']);

				if($line['line_type'] == InvoiceLine::TAX) {
					$data['taxes'][] = $line;
					unset($data['line'][$key]);
					continue;
				}

				// Move details into a seperate array so they can be merged
				unset($line['details']['details']);
				$data['details'][$invoice_id . '_' . $line['line_number'] . '_detail'] = (array)$line['details'];
				unset($line['details']);
			}

			return $data;
		}
	}

?>
