<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.OutgoingShipment.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.bo_perp_outgoing_shipment.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');

	/**
	*	Packing Slip
	*/
	class packing_slip extends perp_report {

		private $hide_details = array(
			'Total',
		);

		public function packing_slip(array $parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';

			// Default parameters
			if(!isset($parameters['shipment_id'])) {
				$parameters['status'] = 'approved';
				if(!$parameters['requested_delivery_date']) {
					$parameters['requested_delivery_date'] = 'now';
				}
			}

			parent::perp_report($parameters);

			$this->template = 'packing_slip.odt';

		}

		public function get_data($query = array()) {
			$data = array(
				'shipment'	=>	array(),
			);
			if($query['requested_delivery_date']) {
				// Get a list of orders dated on or after the order date
				$shipment_query = array(
					'supplier_id'   =>      $query['supplier_id'],
					'status'	=>	$query['status'],
					'requested_delivery_date'    =>      array(
						'start' =>      $query['requested_delivery_date']
					)
				);
				$bo_ship = new bo_perp_outgoing_shipment();
				$shipments = array();
				$count = $bo_ship->get_shipment_list($shipment_query, $shipments);
				if($count == 0) {
					return $data;
				}
				foreach($shipments as $shipment) {
					$query['shipment_id'][] = $shipment['shipment_id'];
				}
			}
			if(!$query['shipment_id']) {
				return $data;
			} elseif(!is_array($query['shipment_id'])) {
				$query['shipment_id'] = array($query['shipment_id']);
			}

			$this->bo_inventory = new bo_perp_inventory();
			$this->unit_of_measure = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			foreach($query['shipment_id'] as $shipment_id) {
				$shipment = $this->get_shipment($shipment_id);

				// Move things around so TBS can find and access them without globals
				$data[$shipment_id . '_container'] = $shipment['container'];
				$data += $shipment['line'];
				$data += $shipment['details'];
				$data += $shipment['package_detail'];
				unset($shipment['container']);
				unset($shipment['line']);
				unset($shipment['details']);
				unset($shipment['package_detail']);
				$data['shipment'][] = $shipment;
			}

			if(count($query['shipment_id']) == 1) {
				$this->filename = lang('Packing slip') . ' ' . $data['shipment'][0]['our_reference_number'];
			} else {
				$this->filename = lang('Multiple packing slips');
			}

//_debug_array($data);die();
			return $data;

		}

		protected function get_shipment($shipment_id) {
			$shipment = new OutgoingShipment($shipment_id);
			$data = $shipment->toArray();

			$data['ship_from']		=	ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_from_address());
			$data['ship_to']		=	ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_to_address());
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $shipment->get_client_id());
			$data['bill_to']		=	ExecMethod('addressbook.addressbook_bo.read', $client['address']);
			$shippers = ExecMethod('perp_api.bo_perp_api.get_shippers', array('shipper_id' => $shipment->get_shipper_id()));
			$data['shipper_name']		=	$shippers[$shipment->get_shipper_id()];
			$data['customer_number']	=	$client['client_num'];
			$data['originating_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $shipment->get_originating_date());
			$data['requested_delivery_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $shipment->get_requested_delivery_date());
			if($shipment->get_actual_delivery_date()) {
				$data['actual_delivery_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $shipment->get_actual_delivery_date());
			} else {
				$data['actual_delivery_date']	=	'';
			}

			// Get containers
			$GLOBALS['egw']->db->select(
				'perp_shipment_detail',
				array(
					'detail_name AS container_name',
					'detail_value AS container_quantity',
					'perp_shipment_detail.line_no'
				),
				array(
					'perp_shipment_detail.shipment_id='.$shipment_id,
					'perp_shipment_detail.detail_class=\'ShipmentPackage\'',
				),
				__LINE__, __FILE__,0,
				'ORDER BY detail_name', 'perp_ar', -1,
				'JOIN perp_outgoing_shipment_package ON
					perp_outgoing_shipment_package.shipment_id = perp_shipment_detail.shipment_id AND
					perp_outgoing_shipment_package.line_no = perp_shipment_detail.line_no AND
					perp_outgoing_shipment_package.detail_id = perp_shipment_detail.detail_id'
			);
			$container_list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$container_list[$row['container_name']][] = $row;
			}
			if(count($container_list) < count($shipment->get_lines())) {
				foreach($shipment->get_lines() as $line) {
					if(count($line->get_details('ShipmentPackage')) == 0) {
						$stock_id = $line->get_shippable_item()->get_stock_id();
						$container_list[''][] = array(
							'line_no'		=>	$line->get_line_number(),
							'container_name'	=>	'',
							'container_quantity'	=>	(float)$line->get_quantity_shipped(),
						);
					}
				}
			}
			foreach($container_list as $name => $container) {
				$container_data = $this->make_container($container, $shipment);
				foreach($container_data['lines'] as &$line) {
					// Line details
					$data['details'][$shipment_id . '_' . $line['line_number'] . '_detail'] = $line['details'];
					unset($line['details']);

					// Package details
					$data['package_detail'][$shipment_id . '_' . $line['line_number'] . '_package_detail'] = $line['package_details'];
					unset($line['package_details']);

					$data['line'][$shipment_id . '_' . $container_data['name'] . '_line'][] = $line;
				}
				unset($container_data['lines']);
				$data['container'][] = $container_data;

			}

			//$data['container'] = $containers;
			return $data;
		}

		/**
		*	@param contents array of container contents
		*	@param shipment OutgoingShipment
		*/
		protected function make_container($contents, &$shipment) {
			$container = array();
			$container['name'] = $contents[0]['container_name'];

			foreach($contents as $key => $line) {
				$shipment_line = $shipment->get_line($line['line_no']);
				$item = $this->make_line($shipment_line);
				$item['container'] = $line['container_name'];
				$item['container_quantity'] = $line['container_quantity'];
				$item['package_count'] = lang("\n(%1 of %2 pieces)", $item['package_id'] + 1, $item['piece_count']);
				$container['lines'][] = $item;
			}

			return $container;
		}

		private function make_line(ShipmentLine $shipment_line) {
			$line = $shipment_line->toArray();

			$stock_id = $shipment_line->get_shippable_item()->get_stock_id();
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);

			$line['description']		=	$shipment_line->get_description();
			$line['stock_num']		=	$stock['stock_num'];
			$line['unit_of_measure']	=	$this->unit_of_measure[$stock['unit_of_measure']];
			$line['quantity']		=	(float)$shipment_line->get_quantity_shipped();
			$serials_needed = ($stock['controlled'] == 0) ? 0 : $line['quantity'];
			$serials_found = 0;
			$serials = array();

			$line['details'] = array();
			$details =& $shipment_line->get_details();
			foreach($details as $detail) {
				if($detail->get_display() && $detail->get_value()) {
					
					// Get serial number(s)
					if($detail instanceof ShipmentSerial) {
						// Serial number needed, set from packing
						$serials[] = $detail->get_serial_number();
						$serials_found += $detail->get_value();
					} elseif($detail instanceof ShipmentPackage) {
						continue;
					} elseif($detail instanceof WorkOrderDetail) {
						$controlled_query = array(
							'stock_id'	=>	$stock_id,
							'location'	=>	$location_id,
							'work_order_id'	=>	$detail->get_work_order_id(),
							'detail_id'	=>	$detail->get_work_order_detail_id(),
						);
						$this->bo_inventory->get_controlled_inventory_list($controlled_query, $controlled_list);
						if($controlled_list[0]) {
							// Serial number needed, set from work order
							$serials[]	=	$serial['serial_number'];
						} 
						$line['details'][]		=	lang($detail->get_name()) . ': ' . $detail->get_value();

						// Get packing that may include options that ship separately
						$line['package_details']	=	$this->make_package_details($stock_id, $detail->get_work_order_id(), $detail->get_work_order_detail_id());

					} elseif ($detail->get_name() && $detail->get_display() && !in_array($detail->get_name(), $this->hide_details)) {
						$line['details'][]		=	$detail->toArray();
					}
				}
			}

			$serials = array_unique($serials);
			foreach($serials as $serial) {
				$line['details'][]	=	array(
					'detail_name'	=>	lang('serial number'),
					'detail_value'	=>	$serial
				);
			}
			// Serial number needed, but not provided
			while($serials_found < $serials_needed) {
				$line['details'][]	=	array(
					'detail_name'	=>	lang('serial number'),
					'detail_value'	=>	'____________'
				);
				$serials_found++;
			}

			// Get packing, if not set by a work order
			if(!$line['package_details']) {
				$line['package_details']	=	$this->make_package_details($stock_id);
			}
			$line['piece_count']		=	count($line['package_details']);

			return $line;
		}

		protected function make_package_details($stock_id, $work_order_id = null, $detail_id = null) {
			$packing = ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id, $work_order_id, $detail_id);
			foreach($packing as $id => $package) {
				$line['packing'][$id] = (float)$package['length'] . ' x ' . 
					(float)$package['width'] . ' x ' . 
					(float)$package['height'] . ' ' . 
					$this->unit_of_measure[$package['unit_of_measure']];
			}
			return $line['packing'];
		}
	}
?>
