<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.OutgoingShipment.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_location.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');

	/**
	*	Pick List
	*
	*	Tells stock pickers where to find the inventory required to fulfill the shipment.
	*	It will start with the location the shipment is leaving from, and look down into the 
	*	location tree to find the first location that matches the shipment line.  
	*	Serialized goods go for the exact location, if the serial number is provided.
	*/
	class pick_list extends perp_report {

		private $hide_details = array(
			'price_list',
			'Total'
		);

		public function pick_list(array $parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';
			parent::perp_report($parameters);

			$this->template = 'pick_list.odt';

			$this->parameters = $parameters;

			// Default parameters
			if(isset($parameters['client_id']) && !isset($parameters['shipment_id'])) {
				$this->parameters['shipment_id']['client_id'] = (string)$parameters['client_id'];
			}
			if(!$this->parameters['shipment_id']) {
				$this->parameters['shipment_id']['status'] = 'approved';
			}
			$this->parameters['options-location'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list');
		}

		public function get_data($query = array()) {
			if(!$query['shipment_id']) {
				return false;
			}

			$bo_location = new bo_perp_location();
			$this->bo_inventory = new bo_perp_inventory();

			// Set these to empty here, so they're in the right order
			$data = array(
				'lines'		=>	array(),
			);

			$shipment = new OutgoingShipment($query['shipment_id']);

			if($shipment->get_actual_delivery_date()) {
				return false;
			}
			$this->unit_of_measure = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			// Get list of locations to draw stock from
			$location_list = $bo_location->get_location_list(
				array(
					'location_parent'	=>	$shipment->get_location_id(),
			//		'template'		=>	false,
				),
				$location_list
			);
			if(!is_array($location_list)) {
				$location_list = $bo_location->get_location_list();
			}
			foreach($location_list as $location_id => &$location) {
				$location = $bo_location->get_location($location_id);
			}
			$location_list[$shipment->get_location_id()] = $bo_location->get_location($shipment->get_location_id());
			ksort($location_list);
			$lines = array();

			$shipment_lines = $shipment->get_lines();

			foreach($location_list as $location_id => &$location) {
				$bo_location->get_inventory(array('num_rows' => -1, 'location_id' => $location_id), $inventory);

				$location_stock = array();
				$location_item = array();

				// Refactor inventory array so we can index into it
				foreach($inventory as $inventory_item) {
					$location_stock[$inventory_item['stock_id']] = $inventory_item;
				}

				foreach($shipment_lines as $key => $shipment_line) {
					$shippable_item = $shipment_line->get_shippable_item();
					if(is_null($shippable_item)) {
						continue;
					}
					try {
						$stock_id = $shippable_item->get_stock_id();
					} catch (Exception $e) {
						continue;
					}
					$location_path = array();

					// Tell pickers to take stock from here
					if($location_stock[$stock_id]) {
						// If it's serialized, we may need to use a specific one
						$work_orders = $shipment_line->get_shippable_item()->get_details('WorkOrderDetail');
						if(count($work_orders) > 0) {
							$controlled_query = array(
								'stock_id'	=>	$stock_id,
								'location'	=>	$location_id
							);
							$is_here = false;
							foreach($work_orders as $work_order) {
								$controlled_query['work_order_id'] = $work_order->get_work_order_id();
								$controlled_query['detail_id'] = $work_order->get_work_order_detail_id();
								$this->bo_inventory->get_controlled_inventory_list($controlled_query, $controlled_list);
								if(count($controlled_list) > 0) {
									// Found completed work order
									$is_here = true;
									break;
								} else {
									// Check in progress work orders
									$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $work_order->get_work_order_id());
									if(!$work_order['completed'] && $work_order['location'] == $location_id) {
										$is_here = true;
										$location_path[] = '(' . trim(ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order_status', $work_order['work_order_id'])) . ')';
										break;
									}
								}
							}
							if(!$is_here) {
								continue;
							} else {
								$item = $this->make_stock_line($stock_id, $shipment_line);
							}
						} elseif($location_stock[$stock_id]['quantity_on_hand'] >= $shipment_line->get_quantity_shipped()) {
							$item = $this->make_stock_line($stock_id, $shipment_line);
							unset($shipment_lines[$key]);
						} elseif ($location_stock[$stock_id]['quantity_on_hand'] > 0 ) {
							$item = $this->make_stock_line($stock_id, $shipment_line, $location_stock[$stock_id]['quantity_on_hand']);
							// This will change the shipment, so make sure we don't save!
							$shipment_line->set_quantity_shipped($shipment_line->get_quantity_shipped() - $location_stock[$stock_id]['quantity_on_hand']);
						}
						if($item) {
							$item += $location;
							$item += $location_stock[$stock_id];
							
							$location_path[] = $location['location_name'];
							$location_parent = $location_list[$location['location_parent']];
							while($location_parent) { // && $location_parent['location_parent'] != 0) {
								$location_path[] = $location_parent['location_name'];
								$location_parent = $location_list[$location_parent['location_parent']];
							}
							$item['location_path'] = implode(' / ', array_reverse($location_path));
							$data[count($lines) . '_detail'] = $item['details'];

							/*	Set up packages
							*	We only want the #_package array to have content if there's more than one package
							*	otherwise, the 'Multiple packages' table is there all the time, and it takes up too much space
							*/
							if($item['piece_count'] == 1) {
								$item['packages'] = $item['packing'][0]['dimensions'] . ' ' . $item['packing'][0]['unit_of_measure'];
								$data[count($lines) . '_package'] = array();
							} else {
								$data[count($lines) . '_package'] = $item['packing'];
							}
							$lines[] = $item;
							$item = null;
						}
					}
				}
				if(count($location_item) > 0) {
				} else {
				//	unset($location_list[$number]);
				}
			}

			if(count($lines) == 0) {
				$lines[] = array(
					'quantity'	=>	0,
					'serial_number'	=>	'',
					'description'	=>	'',
					'stock_num'	=>	'',
					'location_id'	=>	'',
					'location_name'	=>	lang('No location'),
					'location_path'	=>	lang('No location'),
					'packing'	=>	'',
					'piece_count'	=>	0,
					'details'	=>	''
				);
			}

			$data['our_reference_number']	=	$shipment->get_our_reference_number();
			$data['ship_from']		=	$location_list[$shipment->get_location_id()]['location_name'];
			$data['ship_date']		=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $shipment->get_requested_delivery_date());

			$data['lines'] = $lines;

			$this->filename = lang('Pick list') . ' ' . $shipment->get_our_reference_number();

//_debug_array($data); die();
			return $data;
		}

		private function make_stock_line($stock_id, ShipmentLine $shipment_line, $quantity = 0) {
			$line = array();
			$packing = array();
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);

			$line['stock_num']		=	$stock['stock_num'];
			$line['description']		=	$shipment_line->get_description();

			if($stock['stock_type_flag'] != 'R') {
				// Look for a work order number
				$work_orders = $shipment_line->get_shippable_item()->get_detail('work_order');
			}
			if($stock['controlled']) {
				$serials = $shipment_line->get_shippable_item()->get_detail('serial_number');
			}
			$line['details'] = array();

			$serials = array();
			$serials_found = 0;
			if($stock['controlled']) {
				$serials_needed = $shipment_line->get_quantity_shipped();
			}

			$details =& $shipment_line->get_details();
			foreach($details as $detail) {
				if($detail->get_display() && $detail->get_value()) {
					
					// Get serial number(s)
					if($detail instanceof ShipmentSerial) {
						// Serial number needed, set from packing
						$serials[] = $detail->get_serial_number();
						$serials_found += $detail->get_value();
					} elseif ($detail instanceof ShipmentPackage) {
						continue;
					} elseif($detail instanceof WorkOrderDetail) {
						$packing = ExecMethod2('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id, $detail->get_work_order_id(), $detail->get_work_order_detail_id());
						$controlled_query = array(
							'stock_id'	=>	$stock_id,
							'location'	=>	$location_id,
							'work_order_id'	=>	$detail->get_work_order_id(),
							'detail_id'	=>	$detail->get_work_order_detail_id(),
						);
						$serials = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_serial_number', $detail->get_work_order_id(), $stock_id);
						if($serials) {
							// Serial number needed, set from work order
							foreach($serials as $serial) {
								$serials[] = $serial['serial_number'];
								$serials_found += $serial['quantity'];
							}
						} elseif ($stock['controlled']) {
						} 
					} elseif ($this->parameters['include_options'] && !in_array($detail->get_name(), $this->hide_details)) {
						$line['details'][]		=	array(
							'name'	=>	lang($detail->get_name()),
							'value'	=>	$detail->get_value()
						);
					}
				}
			}

			$serials = array_unique($serials);
			while($serials_needed > $serials_found) {
				// Serial number needed, but not provided
				$serials[]	=	'___________________';
				$serials_found++;
			}
			foreach($serials as $serial) {
				$line['details'][] = array(
					'name'	=>	lang('serial number'),
					'value'	=>	$serial
				);
			}

			$line['unit_of_measure']	=	$this->unit_of_measure[$stock['unit_of_measure']];
			if($quantity == 0) {
				$line['quantity']	=	(float)$shipment_line->get_quantity_shipped();
			} else {
				$line['quantity']	=	(float)$quantity;
			}

			if(count($packing) == 0) {
				$packing = ExecMethod('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id);
			}
			$line['packing'] = array();
			foreach($packing as $id => $package) {
				$name = '';
				if($package['stock_id'] != $stock_id) {
					// Packing something that isn't what the line is, probably an option that ships separate
					$name = ExecMethod('perp_inventory.bo_perp_stock.link_title', $package['stock_id']);
				} else {
					$name = ($id + 1) . '. ' . $line['description'];
				}
				$line['packing'][] = array(
					'title'			=>	$name,
					'dimensions'		=>	(float)$package['length'] . ' x ' . 
						(float)$package['width'] . ' x ' . 
						(float)$package['height'],
					'unit_of_measure'	=>	$this->unit_of_measure[$package['unit_of_measure']]
				);
			}
			$line['piece_count']		=	count($packing);
			return $line;
		}
	}
?>
