<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.bo_perp_client.inc.php');

	/**
	*	Accounts receivable statement of account activity over a given time period
	*/
	class statement extends perp_report {

		public function __construct(array $parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';
			parent::perp_report($parameters);

			$this->template = 'statement.odt';

			$this->parameters = $parameters;

			// Default parameters
			if(!$this->parameters['end_date']) {
				$this->parameters['end_date'] = date('U', mktime(0,0,0,date('m') - 1, date('t', mktime(0,0,0,date('m') - 1, date('Y'))), date('Y')));
			}
			if(!$this->parameters['start_date']) {
				$this->parameters['start_date'] = date('U', mktime(0,0,0,date('m', $this->parameters['end_date']), 1, date('Y', $this->parameters['end_date'])));
			}
		}

		public function get_data($query = array()) {
			$bo_client = new bo_perp_client();
			$query['template'] = false;
			$query['num_rows'] = -1;
			$query['order'] = 'client_id, trans_date';
			$rows = array();

			$data['start_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['start_date']);
			$data['end_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date']);

			$client_query = array(
				'template'	=>	false,
				'num_rows'	=>	-1,
				'order'		=>	'client_num',
				'client_start'	=>	$query['client_start'],
				'client_end'	=>	$query['client_end'],
			);

			$data['client'] = array();

			// Get client list
			$bo_client->get_client_list($client_query, $clients);
			foreach($clients as $key => &$client) {
				if(!is_array($client)) {
					unset($clients[$key]);
					continue;
				}

				// Get addresses
				$address = ExecMethod('addressbook.addressbook_bo.read', $client['address']);
				$bill_to[$client['client_id']] = (is_array($address) ? $address : array());


				// Get client ledger entries
				$query['client_id'] = $client['client_id'];
				$bo_client->get_ledger($query, $ledger);
				if($query['skip_no_activity'] && count($ledger) == 0) {
					continue;
				}
				$rows[$client['client_id']] = $ledger;

				$client = $bo_client->get_client($client['client_id']);

				// Get sales orders
				$this->get_order_data($client['client_id'], $data);
				
				//Get opening balance
				$closing_date  = strtotime("-1 day",$query['start_date']);
				$client['opening_balance'] = $bo_client->get_account_balance($client['client_id'], $client_account_id, $closing_date);
				$client['opening_balance_text'] = lang('Opening Balance %1', ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], $client['opening_balance']));

				// Get account balance
				$client['account_balance'] = $bo_client->get_account_balance($client['client_id']);
				if($client['account_balance'] > 0) {
					$client['owe_text'] = lang('Please pay %1', ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], abs($client['account_balance'])));
				} elseif ($client['account_balance'] == 0) {
					$client['owe_text'] = lang('No payment is required');
				} else {
					$client['owe_text'] = lang('You have a credit of %1', ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], abs($client['account_balance'])));
				}

				$data['client'][] = $client;

			}

			// Process ledger lines and add details as needed
			$details = array();
			foreach($rows as $client_id => &$client_ledger) {
				if(is_array($client_ledger)) {
					foreach($client_ledger as &$entry) {
						$details[$entry['reference']['id']] = array();
						if($query['show_details']) {
							list($type, $id) = explode(':', $entry['reference']['id']);
							switch($type) {
								case 'INVOICE':
									$details[$entry['reference']['id']] = $this->invoice_details($id, $entry);
									break;
							}
						}
					}
					$data['ledger' . $client_id] = $client_ledger;
				}
			}
			$data += $details;
			$GLOBALS['bill_to'] = $bill_to;

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}

		protected function get_order_data($client_id, &$data) {
			$data['orders' . $client_id] = array();
		}

		/**
		*	Expand the an invoice into its lines
		*/
		protected function invoice_details($invoice_id, &$entry) {
			$invoice = new Invoice($invoice_id);

			if($invoice == null || !($invoice instanceof Invoice)) {
				return array();
			}

			// Clear the entry out, it will become a heading
		//	unset($entry['formatted_debit']);
		//	unset($entry['formatted_credit']);

			foreach($invoice->get_lines() as $line) {
				$amount = $line->get_unit_price() * $line->get_quantity();
				$details[] = array(
					'description'	=>	$line->get_description(),
					'formatted_' . ($amount > 0 ? 'debit' : 'credit')	=>	ExecMethod2(
						'perp_api.bo_perp_api.get_formatted_price', 
						$invoice->get_invoice_currency(), 
						abs($amount)
					),
				);
			}
			return $details;
		}
	}
?>
