<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');

	/**
	*	Report that lists detailed amounts receivable by days overdue
	*	One line per client, grouped by currency
	*/
	class summary_aged_accounts_receivable extends perp_report {

		public function summary_aged_accounts_receivable (array $parameters = array()) {
			parent::perp_report($parameters);

			$this->template = 'summary_aged_accounts_receivable.odt';

			// Default parameters
			if(!$parameters['end_date']) {
				$this->parameters['end_date'] = date('U');
				$this->parameters['options-currencies'] = ExecMethod('perp_api.bo_perp_api.get_currencies');
				$this->parameters['start_client'];
				$this->parameters['end_client'];
				$this->parameters['only_non_zero'] = true;
			}
		}

		public function get_data($query = array()) {
			if($query['start_client']) {
				$start_client = ExecMethod('perp_ar.bo_perp_client.get_client', $query['start_client']);
				$query['start_client'] = $start_client['client_num'];
			}
			if($query['end_client']) {
				$end_client = ExecMethod('perp_ar.bo_perp_client.get_client', $query['end_client']);
				$query['end_client'] = $end_client['client_num'];
			}
			
			$data = array(
				'end_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date']),
				'currencies'	=>	(is_array($query['currencies']) || $query['currencies'] != '' ? $query['currencies'] : lang('All')),
			);
			if($query['start_client'] || $query['end_client']) {
				$data['client_range'] = lang('Client: %1 to %2',
					($query['start_client'] ? $query['start_client'] : lang('all')),
					($query['end_client'] ? $query['end_client'] : lang('all')) 
				);
			} else {
				$data['client_range'] = lang('all clients');
			}
			if($query['only_non_zero']) {
				$data['client_range'] .= ' ' . lang('only non-zero accounts');
			}

			if($query['currencies']) {
				$query['currencies'] = explode(',', $query['currencies']);
			}

			$sql = 'SELECT
				perp_client_ledger.trans_id,
				perp_client.client_id,
				perp_client_ledger.exchange_rate,
				narrative,
				ROUND(perp_general_ledger.amount / perp_client_ledger.exchange_rate, 2) AS original_amount,
				invoice_total.total AS invoice_total,
				IF(receipt_total.total IS NULL, perp_client_receipt.amount, receipt_total.total) AS receipt_total,
				perp_invoice.invoice_paid,
				SUM(
					IF(invoice_total.total IS NULL, 0, invoice_total.total) - 
					(
						IF(receipt_total.total IS NULL, IF(perp_client_receipt.amount IS NULL, 0, perp_client_receipt.amount), receipt_total.total) +
						IF(discount_taken IS NULL, 0, discount_taken)
					)
				) AS amount_owing,
				IF(total_receivable.total IS NULL, 0, total_receivable.total) AS total_receivable,
				perp_invoice.discount_taken,
				perp_invoice.due_date,
				case 
					when perp_invoice.due_date IS NULL then \'current\'
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 0 then \'current\'
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 7 then 7
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 14 then 14
					when TO_DAYS(\'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . '\') - TO_DAYS(perp_invoice.due_date) <= 30 then 30
					else \'more\'
				end AS age,
				perp_client.client_num,
				perp_client.client_name,
				perp_client.currency,
				perp_currency.currency_name
			FROM perp_client
			LEFT JOIN perp_client_ledger ON
				perp_client_ledger.client_id = perp_client.client_id
			LEFT JOIN perp_general_ledger ON
				perp_client_ledger.trans_id = perp_general_ledger.trans_no AND
				perp_client_ledger.line_no = perp_general_ledger.line_no
			LEFT JOIN (
				SELECT
					client_id,
					sum(ROUND(amount / exchange_rate, 2)) AS total
				FROM perp_client_ledger
				JOIN perp_general_ledger ON
					perp_client_ledger.trans_id = perp_general_ledger.trans_no AND
					perp_client_ledger.line_no = perp_general_ledger.line_no
				GROUP BY client_id
			) AS total_receivable ON
				perp_client.client_id = total_receivable.client_id
			LEFT JOIN (
				perp_invoice JOIN (
					SELECT
						invoice_id,
						sum(ROUND(unit_price * quantity, 2)) AS total
					FROM perp_invoice_line
					GROUP BY invoice_id
				) AS invoice_total ON
				perp_invoice.invoice_id = invoice_total.invoice_id
			) ON
				perp_invoice.invoice_transaction_id = perp_client_ledger.trans_id
			LEFT JOIN (
				SELECT 
					invoice_id,
					sum(amount) AS total
				FROM perp_client_receipt 
				GROUP BY invoice_id
			) AS receipt_total ON
				receipt_total.invoice_id = perp_invoice.invoice_id
			LEFT JOIN perp_client_receipt ON
				!perp_client_receipt.invoice_id AND
				perp_client_receipt.trans_id = perp_client_ledger.trans_id
			JOIN perp_currency ON
				perp_client.currency = perp_currency.currency_id';

			if($query['currencies']) {
				$where[] = 'perp_currency.currency_id IN (\'' . implode("','", $query['currencies']) . '\') ';
			}
			if($query['start_client']) {
				$where[] = 'perp_client.client_num >= \'' . $query['start_client'] . '\' ';
			}
			if($query['end_client']) {
				$where[] = 'perp_client.client_num <= \'' . $query['end_client'] . '\' ';
			}
			if($query['end_date']) {
				$where[] = '(perp_general_ledger.trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['end_date']))  . 
					' OR trans_date IS NULL)';
			}
			if(count($where) > 0) {
				$sql .= ' WHERE ' . implode(' AND ', $where);
			}

			$sql .= ' GROUP BY client_id, age ';
			if(is_numeric($query['start_balance'])) {
				$having[] = 'total_receivable >= ' . $query['start_balance'] . ' ';
			}
			if(is_numeric($query['end_balance'])) {
				$having[] = 'total_receivable <= ' . $query['end_balance'] . ' ';
			}
			if($query['only_non_zero']) {
				$having[] = 'total_receivable != 0';
			}
			if(count($having) > 0) {
				$sql .= ' HAVING ' . implode(' AND ', $having);
			}
			$sql .= ' ORDER BY perp_currency.currency_order, perp_client.client_num';
			$invoices = array();
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['due_date'] = $GLOBALS['egw']->db->from_timestamp($row['due_date']);
				$invoices[] = $row;
			}

			$currency_total = array();
			$client = '';
			$currency = '';
			$detail = array();
			foreach($invoices as $key => &$invoice) {
				// Massage data to fit into report
				if($currency != $invoice['currency']) {
					if(count($currency_total)) {
						// Keep total currency
						$currency_total['currency'] = $currency;
						$total_currency[] = $currency_total;
					}
					$currency = $invoice['currency'];
					$currency_total = array(
						'current'	=>	0,
						'7'		=>	0,
						'14'		=>	0,
						'30'		=>	0,
						'more'		=>	0,
						'overdue'	=>	0,
						'total'		=>	0
					);
				}
				if($client != $invoice['client_id']) {
					$currency_total['total'] += $invoice['total_receivable'];
					$client = $invoice['client_id'];
					$detail[$invoice['client_id']] = array(
						'current'	=>	0,
						'7'		=>	0,
						'14'		=>	0,
						'30'		=>	0,
						'more'		=>	0,
						'total_overdue'	=>	0,
						'total_receivable'	=>	0
					);
				}
				$detail[$invoice['client_id']]['client_num'] = $invoice['client_num'];
				$detail[$invoice['client_id']]['client_name'] = $invoice['client_name'];
				$detail[$invoice['client_id']]['currency'] = $invoice['currency'];
				$detail[$invoice['client_id']]['currency_name'] = $invoice['currency_name'];

				$detail[$invoice['client_id']][$invoice['age']] += $invoice['amount_owing'];
				$currency_total[$invoice['age']] += $invoice['amount_owing'];
				if($invoice['age'] != 'current') {
					$detail[$invoice['client_id']]['total_overdue'] += $invoice['amount_owing'];
					$currency_total['overdue'] += $invoice['amount_owing'];
				}
				$detail[$invoice['client_id']]['total_receivable'] = $invoice['total_receivable'];

				// Set totals
				foreach($currency_total as $c_range => $c_total) {
					$detail[$invoice['client_id']]['currency_total_' . $c_range] = $c_total;
				}
			}

			if(count($invoices) == 0) {
			} else {
				// Get last currency
				$currency_total['currency'] = $currency;
				$total_currency[] = $currency_total;

			}
			$total_currency = array();
			$currency_total = array();
			$data['detail'] = array();

			foreach($detail as $client_id => &$client_line) {
				$client_line['client_id'] = $client_id;
				if($query['start_balance'] != '' && $client_line['total_receivable'] < $query['start_balance']) {
					continue;
				} elseif($query['end_balance'] != '' && $client_line['total_receivable'] > $query['end_balance']) {
					continue;
				} else {
					$data['detail'][] = $client_line;
					$currency_total[$client_line['currency']]['current'] += $client_line['current'];
					$currency_total[$client_line['currency']]['7'] += $client_line['7'];
					$currency_total[$client_line['currency']]['14'] += $client_line['14'];
					$currency_total[$client_line['currency']]['30'] += $client_line['30'];
					$currency_total[$client_line['currency']]['more'] += $client_line['more'];
					$currency_total[$client_line['currency']]['total_overdue'] += $client_line['total_overdue'];
					$currency_total[$client_line['currency']]['total_receivable'] += $client_line['total_receivable'];
				}
			}
			// Get currency names
			$currency_list = ExecMethod('perp_api.bo_perp_api.get_currencies');
			foreach($currency_total as $currency_id => &$currency) {
				$currency['currency'] = $currency_id;
				$currency['currency_name'] = $currency_list[$currency['currency']];
				$total_currency[] = $currency;
			}
			$data['total'] = $total_currency;

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}
	}
?>
