<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.OutgoingShipment.inc.php');

	/**
	*	Commercial invoice
	*/
	class us_customs_invoice extends perp_report {

		public function us_customs_invoice(array $parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ar';
			parent::perp_report($parameters);

			$this->template = 'us_customs_invoice.odt';

			$this->parameters = $parameters;

			// Default parameters
			if(isset($parameters['client_id']) && !isset($parameters['shipment_id'])) {
				$this->parameters['shipment_id']['client_id'] = (string)$parameters['client_id'];
			}
//			$this->parameters['shipment_id']['status'] = 'approved';
			$this->parameters['options-mode_of_transport'] = array(
				lang('Road')	=>	lang('Road'), 
				lang('Rail')	=>	lang('Rail'), 
				lang('Water')	=>	lang('Water'), 
				lang('Air')	=>	lang('Air')
			);
			$this->parameters['options-status'] = array(
				lang('Owner / Prop.')	=>	lang('Owner / Prop.'),
				lang('Agent')		=>	lang('Agent')
			);
		}

		public function get_data($query = array()) {
			if(!$query['shipment_id']) {
				return false;
			}


			$shipment = new OutgoingShipment($query['shipment_id']);
			$carriers = ExecMethod('perp_api.bo_perp_api.get_shippers');
			$data = array(
				'our_reference_number'	=>	$shipment->get_our_reference_number(),
				'their_reference_number'	=>	$shipment->get_their_reference_number(),
				'carrier_name'		=>	$carriers[$shipment->get_shipper_id()],
				'vessel'		=>	$shipment->get_vessel(),
				'item_count'		=>	0,
				'item_total_value'	=>	0,
				'item_total_weight'	=>	0,
				'currency'		=>	$shipment->get_invoice_currency(),
				'exporter_no'		=>	$query['exporter_no'],
				'mode_of_transport'	=>	$query['mode_of_transport'],
				'containerized'		=>	$query['containerized'],
				'status'		=>	$query['status'],
			);

			$data['shipper'][] = ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_from_address());
			$data['user'][] = ExecMethod('addressbook.addressbook_bo.read', $GLOBALS['egw_info']['user']['person_id']);

			$ship_to = ExecMethod('addressbook.addressbook_bo.read', $shipment->get_ship_to_address());
			$bill_to = ExecMethod('addressbook.addressbook_bo.read', $client['address']);

			$client_id = $shipment->get_client_id();
			$client = ExecMethod('perp_ar.bo_perp_client.get_client', $client_id);
			$client_branch = ExecMethod('perp_ar.bo_perp_client_branch.get_client_branch', $shipment->get_client_branch_id());
			$data['importer_tax_no']	=	$client_branch['tax_number'];
			$data['purchaser_tax_no']	=	$client_branch['tax_number'];
			$data['final_destination']	=	$ship_to['adr_one_region'] . ' ' . $ship_to['adr_one_countryname'];
			$data['province_of_origin']	=	$data['shipper'][0]['adr_one_region'];


			$sql = "SELECT sum(count) 
				FROM (
					SELECT 1 AS count
					FROM perp_shipment_detail
					WHERE perp_shipment_detail.shipment_id = {$shipment->get_shipment_id()} AND 
					detail_class = 'ShipmentPackage' AND
					detail_name != ''
					GROUP BY detail_name
					UNION 

					SELECT count(*)  AS count
					FROM perp_shipment_detail
					WHERE perp_shipment_detail.shipment_id = {$shipment->get_shipment_id()} AND 
						detail_class = 'ShipmentPackage' AND
						detail_name = ''
					GROUP BY detail_name
				) AS package_count";
			$data['package_count'] = $GLOBALS['egw']->db->query($sql, __LINE__, __FILE__)->fetchSingle();

			$lines = array();
			$unit_of_measure = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			$shipment_lines = $shipment->get_lines();
			foreach($shipment_lines as $shipment_line) {
				$line = $shipment_line->toArray();

				$shippable_item = $shipment_line->get_shippable_item();
				if(is_null($shippable_item)) {
					continue;
				}
				try {
					$stock_id = $shippable_item->get_stock_id();
				} catch (Exception $e) {
					continue;
				}
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
				$packing = ExecMethod('perp_inventory.bo_perp_stock.get_shipping_packages', $stock_id);

				
				$line['description']		=	$shipment_line->get_description();
				$line['serial_number']		=	'';
				$line['country_of_origin']	=	$stock['country_of_origin'];
				$line['hs_code']		=	$stock['usa_inbound_code'];
				$line['unit_of_measure']	=	$unit_of_measure[$stock['unit_of_measure']];

				$line['value']			=	$shipment_line->get_unit_price() * $shipment_line->get_quantity_shipped();
				$data['item_total_value']	+=	$line['value'];
				$line['value'] 			= 	ExecMethod2(
										'perp_api.bo_perp_api.get_formatted_price',
										$shipment->get_invoice_currency(),
										$line['value']
									);
				$data['item_count']		+=	$shipment_line->get_quantity_shipped();

				foreach($packing as $package) {
					$data['item_total_weight']	+=	$package['mass'] * $shipment_line->get_quantity_shipped();
				}
				$lines[] = $line;
			}
			$data['item_total_value']		=	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $shipment->get_invoice_currency(), $data['item_total_value']);

			if(is_array($bill_to)) {
				$data['bill_to'][] = $bill_to;

				// If ship_to not set, use bill to
				if(!is_array($ship_to)) {
					$ship_to = $bill_to;
				}
			}
			if(is_array($ship_to)) {
				$data['consignee'][] = $ship_to;
				// If bill_to not set, use ship to
				if(!is_array($bill_to)) {
					$data['bill_to'][] = $ship_to;
				}
			}
			$data['items'] = $lines;
//	_debug_array($data);

			$this->filename = lang('US Customs Invoice') . ' ' . $shipment->get_our_reference_number();
			return $data;
		}
	}
?>
