<?php

	/**
	*	Object for dealing with organizational departments
	*/

	class Department {

		protected $department_id;
		protected $parent_id = 0;

		/**
		*	Reference number for accounting purposes
		*/
		protected $department_number = null;

		protected $department_name;
		protected $department_manager = -1;

		/**
		*	account_id of egroupware group account
		*/
		protected $group_id;

		public function __construct($id = null) {
			if($id) {
				$this->read($id);
			}
		}

		public function read($id) {
			$GLOBALS['egw']->db->select(
				'perp_staff_department',
				array('*'),
				array(
					'department_id'		=>	$id
				),
				__LINE__, __FILE__
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->department_id	=	$row['department_id'];
			$this->department_number	=	$row['department_number'];
			$this->department_name	=	$row['department_name'];
			$this->department_manager	=	$row['department_manager'];
			$this->group_id		=	$row['group_id'];
			$this->parent_id	=	$row['department_parent'];
		}

		public function write() {
			$GLOBALS['egw']->db->insert(
				'perp_staff_department',
				array(
					'department_id'		=>	$this->department_id,
					'department_number'	=>	$this->department_number,
					'department_name'	=>	$this->department_name,
					'department_manager'	=>	$this->department_manager,
					'group_id'		=>	$this->group_id,
					'department_parent'	=>	$this->parent_id,
				),
				array(
					'department_id'		=>	$this->department_id
				),
				__LINE__, __FILE__,
				'perp_human_resources'
			);

			$this->department_id = $this->department_id ? $this->department_id : $GLOBALS['egw']->db->get_last_insert_id('perp_staff_department', 'department_id');

			return $this->department_id;
		}

		public function get_department_id() {
			return $this->department_id;
		}

		public function set_parent($parent_id) {
			if(is_numeric($parent_id) && $parent_id != $this->department_id) {
				$this->parent_id = $parent_id;
			}
		}
		public function get_parent_id() {
			return $this->parent_id;
		}


		/**
		*	Get the name of this department
		*/
		public function get_name() {
			return $this->department_name;
		}
		/**
		*	Set the name of this department
		*/
		public function set_name($name) {
			if(trim($name) != '') {
				$this->department_name = $name;
			}
		}

		/**
		*	Set reference number for this department
		*/
		public function set_number($num) {
			$this->department_number = $num;
		}

		/**
		*	Get the reference number for this department
		*/
		public function get_number() {
			return $this->department_number;
		}

		/**
		*	Get the manager of this department
		*
		*/
		public function get_manager() {
			return $this->department_manager;
		}
		/**
		*	Set the manager of this department
		*	@param user_id ID of the staff record for the manager
		*/
		public function set_manager($user_id) {
			$this->department_manager = $user_id;
		}

		/**
		*	Get the eGroupWare account group associated with this
		*	department.  Mostly used for permissions for the staff
		*	in this department if they have no user account, and to
		*	set the default group for new staff if they have a 
		*	user account.
		*/
		public function get_group_id() {
			return $this->group_id;
		}

		/**
		*	Set the eGroupWare account group associated with this
		*	department.
		*	$param group_id ID number of the eGroupWare account group
		*/
		public function set_group_id($group_id) {
			$this->group_id = $group_id;
		}

		public function toArray() {
			$data = get_object_vars($this);
			return $data;
		}

		public function __toString() {
			$string = sprintf("%s (%s)", 
				$this->department_name,
				$this->department_manager
			);
			return $string;
		}

		public function __sleep() {
			$properties = array(
				'department_id',
				'parent_id',
				'department_name',
				'department_number',
				'department_manager',
				'group_id'
			);

			return $properties;
		}
	}
?>
