<?php
	require_once('class.Department.inc.php');

	require_once('class.StaffSkill.inc.php');

	/**
	*	Class for dealing with Staff people
	*/

	class Staff {

		// Class constants
		/**
		*	Status constants
		*/
		const ACTIVE		= 1;
		const ON_LEAVE		= 2;
		const SUSPENSION	= 3;
		const GONE		= 4;

		public static $status_list = array(
			1	=>	'active',
			2	=>	'on_leave',
			3	=>	'suspension',
			4	=>	'gone'
		);

		protected $staff_id = null;
		protected $addressbook_id;
		protected $job_description;

		protected $start_date;
		protected $end_date = null;
		
		protected $status = Staff::ACTIVE;
		
		protected $department_id = null;

		protected $reports_to;

		protected $employee_number = null;

		protected $gender = null;

		// When and how often this employee is paid
		protected $pay_schedule = null;

		/**
		*	A list of custom region or company specific fields
		*/
		protected $extra_fields = array();

		/**
		*	A list of StaffSkills for skills or certificates or
		*	training this staff person has
		*/
		protected $skills = array();

		protected static $payroll_config;
		protected static $timeclock_config;


		public function __construct($staff_id = null) {

			// Get shared object for payroll config options
			if(!is_array($this->payroll_config)) {
				$this->payroll_config = config::read('perp_payroll');
			}

			// Get shared object for timeclock config options
			if(!is_array($this->timeclock_config)) {
				$this->timeclock_config = config::read('perp_timeclock');
			}

			// Init Staff
			$start_date = time();
			if($staff_id) {
				$this->read($staff_id);
			}
		}

		public function read($staff_id) {
			$GLOBALS['egw']->db->select(
				'perp_staff',
				array('*'),
				array('staff_id'	=>	$staff_id),
				__LINE__, __FILE__
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$this->staff_id		=	$row['staff_id'];
				$this->addressbook_id	=	$row['addressbook_id'];
				$this->job_description	=	$row['job_description'];
				
				$this->status		=	$row['status'];

				$this->department_id	=	$row['department_id'];
				$this->reports_to	=	$row['reports_to'];

				$this->employee_number	=	$row['employee_number'];
				$this->pay_schedule	=	$row['pay_schedule'];

				$this->gender		=	$row['gender'];

			}

			// Get extra fields
			$result = $GLOBALS['egw']->db->select(
				'perp_staff_extra',
				array('*'),
				array('staff_id'	=>	$staff_id),
				__LINE__, __FILE__, 0, '', 'perp_human_resources'
			);
			foreach($result as $row) {
				$this->extra_fields[$row['staff_extra_name']] = $row['staff_extra_value'];
			}

			// Get most recent start / end date
			$GLOBALS['egw']->db->select(
				'perp_record_of_employment',
				array(
					'start_date',
					'end_date',
				),
				array(
					'staff_id'	=>	$this->staff_id,
				),
				__LINE__, __FILE__, 0,
				'ORDER BY start_date DESC', 'perp_human_resources',
				1
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->start_date = is_null($row['start_date']) ? null : $GLOBALS['egw']->db->from_timestamp($row['start_date']);
			$this->end_date = is_null($row['end_date']) ? null : $GLOBALS['egw']->db->from_timestamp($row['end_date']);

			// Read skills
			$GLOBALS['egw']->db->select(
				'perp_staff_skill',
				array('*'),
				array('staff_id'	=>	$this->staff_id),
				__LINE__, __FILE__
			);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['skill_start_date'] = $GLOBALS['egw']->db->from_timestamp($row['skill_start_date']);
				$row['skill_expiry_date'] = $GLOBALS['egw']->db->from_timestamp($row['skill_expiry_date']);
				$this->add_skill(new StaffSkill($this, $row), $row['skill_id']);
			}
		}

		public function write() {
			if(!$this->start_date) {
				$this->set_start_date(time());
			}

			// Write staff record
			$GLOBALS['egw']->db->insert('perp_staff',
				array(
					'staff_id'		=>	$this->staff_id,
					'addressbook_id'	=>	$this->addressbook_id,
					'job_description'	=>	$this->job_description,
					'status'		=>	$this->status,
					'department_id'		=>	$this->department_id,
					'reports_to'		=>	$this->reports_to,
					'employee_number'	=>	$this->employee_number,
					'pay_schedule'		=>	$this->pay_schedule,
					'gender'		=>	$this->gender,
				),
				array('staff_id'	=>	$this->staff_id),
				__LINE__, __FILE__,
				'perp_human_resources'
			);

			// Set extra fields
			foreach($this->extra_fields as $key => $value) {
				$GLOBALS['egw']->db->insert(
					'perp_staff_extra',
					array(
						'staff_id'		=>	$this->staff_id,
						'staff_extra_name'	=>	$key,
						'staff_extra_value'	=>	$value
					),
					array(
						'staff_id'		=>	$this->staff_id,
						'staff_extra_name'	=>	$key
					),
					__LINE__, __FILE__,
					'perp_human_resources'
				);

			}

			$GLOBALS['egw']->db->insert(
				'perp_record_of_employment',
				array(
					'staff_id'		=>	$this->staff_id,
					'start_date'		=>	$GLOBALS['egw']->db->to_timestamp($this->start_date),
					'end_date'		=>	(is_null($this->end_date) ? null : $GLOBALS['egw']->db->to_timestamp($this->end_date)),
				),
				array(
					'staff_id'		=>	$this->staff_id,
					'start_date'		=>	$GLOBALS['egw']->db->to_timestamp($this->start_date),
				), __LINE__, __FILE__, 'perp_human_resources'
			);

			// Update contact info
			$department = $this->get_department();
			if (!is_object($GLOBALS['egw']->contacts))
			{
				$GLOBALS['egw']->contacts =& CreateObject('phpgwapi.contacts');
			}
			$contact = $GLOBALS['egw']->contacts->read($this->addressbook_id);
			$contact['org_unit'] = $department->get_name();
			$GLOBALS['egw']->contacts->save($contact);

			// Save skills
			foreach($this->skills as $skill) {
				if(is_object($skill)) {
					$skill->write();
				}
			}
		}

		/**
		*	Disable the staff person.
		*	Sets their user account to expire, sets the end date and reason for leaving
		*	
		*	@param end_date Effective end date
		*	@param reason Reason for leaving
		*
		*	@todo Implement reason
		*
		*/
		public function delete($end_date, $reason) {
			if(!$GLOBALS['egw']->acl->check('staff', EGW_ACL_DELETE, 'perp_human_resources')) {
				throw new Exception(lang('You do not have permission to delete staff'));
			}

			if($this->staff_id) {
				// Disable users rather than delete them, preferrably
				$account =& CreateObject('phpgwapi.accounts',(int)$this->staff_id,'u');
				$userData = $account->read_repository();
				$userData['account_expires'] = (int)$end_date;
				$account->save($userData);
			}

			$this->set_end_date((int)$end_date);
			$this->set_status(Staff::GONE);
			$this->write();
		}

		public function get_employee_number() {
			return $this->employee_number;
		}
		public function set_employee_number($number) {
			$this->employee_number = $number;
		}
		

		/**
		*	Get the ID of the addressbook entry for this person
		*	@return int addressbook_id
		*/
		public function get_addressbook_id() {
			return $this->addressbook_id;
		}

		/**
		*	Set the ID of the addressbook entry for this person
		*	@parm id ID of the existing addressbook entry
		*	@return boolean of success
		*	@throws Exception if it's a bad addressbook id
		*/
		public function set_addressbook_id($id) {
			$test = ExecMethod('addressbook.addressbook_bo.read', $id);
			if(!is_array($test) || !$test['id']) {
				throw new Exception(lang('Unknown contact'));
				return false;
			}
			$this->addressbook_id = $id;
			return true;
		}

		/**
		*	Get the job description of this person
		*	@return String job description
		*/
		public function get_job_description() {
			return $this->job_description;
		}

		/**
		*	Set the job description of this person
		*	@param job_description String
		*/
		public function set_job_description($job_description) {
			$this->job_description = $job_description;
		}

		/**
		*	Get the ID of the department this person is in
		*/
		public function get_department_id() {
			return $this->department_id;
		}
		/**
		*	Set the ID of the department this person is in
		*/
		public function set_department_id($dept_id) {
			$this->department_id = $dept_id;
		}

		/**
		*	Get the Department this person belongs to
		*
		*	@return Department
		*/
		public function get_department() {
			return new Department($this->department_id);
		}

		/**
		*	Set this person's immediate supervisor
		*
		*	@param super id of staff person
		*/
		public function set_reports_to($super) {
			$this->reports_to = (int)str_replace('STAFF-', '', $super);
		}

		public function get_reports_to() {
			return $this->reports_to;
		}

		/**
		*	Set the start date
		*	@param date timestamp
		*/
		public function set_start_date($date) {
			if($date == 0) {
				throw new egw_exception_wrong_userinput(lang('Missing start date'));
			}
			$this->start_date = $date;
		}

		/**
		*	Get the start date
		*/
		public function get_start_date() {
			return $this->start_date;
		}

		/**
		*	Set the end of employment date
		*	@param date timestamp
		*/
		public function set_end_date($date) {
			if($date == 0) {
				$date = null;
			}
			$this->end_date = $date;
		}

		/**
		*	Get the end date
		*/
		public function get_end_date() {
			return $this->end_date;
		}

		/**
		*	Set the employee's gender
		*
		*	Should be 1 for Male, or 2 for Female
		*
		*	@param sex_change Employee's new gender
		*/
		public function set_gender($sex_change) {
			$this->gender = $sex_change;
		}

		/**
		*	Get the employee's gender
		*/
		public function get_gender() {
			return $this->gender;
		}

		/**
		*	Get the employee's age
		*/
		public function get_age($today = null) {
			if(!$today) {
				$today = time();
			}
			$today = getdate($today);

			$address = ExecMethod('addressbook.addressbook_bo.read', $this->addressbook_id);
			if(!$address['bday']) {
				return 0;
			}
			// Why is bday stored as Y-m-d?
			list($birthday['year'], $birthday['mon'], $birthday['mday']) = explode('-', $address['bday']);

			// find the difference in the years of the two dates
			$yeardiff = $today['year'] - $birthday['year'];

			// if the current date occurs before the birthday, subtract one
			$birth_jd = gregoriantojd($birthday['mon'],$birthday['mday'],$today['year']);
			$today_jd = gregoriantojd($today['mon'],$today['mday'],$today['year']);
			if ($today_jd < $birth_jd) {
				$yeardiff--;
			}
			return($yeardiff);
		}

		/**
		*	Add a skill / certificate
		*
		*	@param skill StaffSkill object to add
		*	@param skill_id Optional ID number for the StaffSkill
		*
		*	@return int skill_id
		*/
		public function add_skill(StaffSkill &$skill, $skill_id = null) {
			if($skill != null && $skill->get_name() != '') {
				if(is_null($skill_id)) {
					if(count($this->skills) > 0) {
						end($this->skills);
						$skill_id = key($this->skills);
					}
					$skill_id++;
				}
				$skill->set_id((int)$skill_id);
				$this->skills[$skill_id] =& $skill;
			}
			return $skill_id;
		}

		/**
		*	Retrieve a particular StaffSkill object
		*
		*	@param skill_id
		*/
		public function &get_skill($skill_id) {
			return $this->skills[$skill_id];
		}

		/**
		*	Remove and return a particular StaffSkill
		*
		*	@return StaffSkill
		*/
		public function &remove_skill($skill_id) {
			$skill =& $this->get_skill($skill_id);
			unset($this->skills[$skill_id]);
			$skill->delete();
			return $skill;
		}

		/**
		*	Get payment schedule
		*	
		*	@param date Optional, defaults to today
		*	@return an array of calendar information about payment schedule
		*/
		public function get_payment_schedule($date = null) {
			if(is_null($date)) {
				$date = time();
			}

			$calendar = ExecMethod2('calendar.calendar_bo.read', $this->pay_schedule, $date, true);
			if($this->pay_schedule && !$calendar) {
				// Calendar event probably needs more recurrence entries added
				ExecMethod('calendar.calendar_bo.check_move_horizont', strtotime('+2 months', $date));
				$calendar = ExecMethod2('calendar.calendar_bo.read', $this->pay_schedule, $date, true);
			}
			return $calendar;
		}

		/**
		*	Set payment schedule
		*
		*	@param schedule_id ID of pay schedule for this person
		*	@return boolean success
		*/
		public function set_payment_schedule($schedule_id) {
			$this->pay_schedule = $schedule_id;
		}

		/**
		*	Get payroll information
		*
		*	@return associative array
		*/
		public function get_payroll_info() {
			if(!$GLOBALS['egw']->acl->check('staff_data', EGW_ACL_READ, 'perp_payroll')) {
				throw new Exception(lang('Access to payroll denied'));
			}
			return ExecMethod('perp_payroll.bo_perp_payroll.get_staff_data', $this);
		}

		/**
		*	Get work schedule
		*	
		*	@param date Optional, defaults to today
		*	@return an array of calendar information about work schedule
		*/
		public function get_work_schedule($date = null) {
			// Work shift schedules
			$query = array(
				'start'		=>	$date,
				'date_format'	=>	'ts',
				'enum_recuring'	=>	false,
				'ignore_acl'	=>	true,
				'cat_id'	=>	$this->timeclock_config['work_shift_category'],
			);
			$work_shifts    =       ExecMethod('calendar.calendar_bo.search', $query);
			$schedule_ids = array();
			foreach($work_shifts as $work_shift) {
				if(isset($work_shift['participants'][$this->get_staff_id()]) && $work_shift['participants'][$this->get_staff_id()] == 'A') {
					$schedule_ids[] = $work_shift['id'];
				}
			}
			$work_shifts = null;
			$schedules = array();
			if(count($schedule_ids) == 0) {
				// Fall back to calendar hours
				$schedules[] = array(
					'title'	=>	lang('Default work shift'),
					'start'	=>	mktime($GLOBALS['egw_info']['user']['preferences']['calendar']['workdaystarts'], 0, 0, date('m', $date), date('d', $date), date('Y', $date)),
					'end'	=>	mktime($GLOBALS['egw_info']['user']['preferences']['calendar']['workdayends'], 0, 0, date('m', $date), date('d', $date), date('Y', $date)),
				);
				return $schedules;
			}
			$events = ExecMethod2('calendar.calendar_bo.read', $schedule_ids, $date, true);
			if(is_array($events)) {
				foreach($events as $calendar) {
					if(is_array($calendar) && date('Ymd', $calendar['start']) <= date('Ymd',$date) && date('Ymd', $calendar['start']) >= date('Ymd', $date) || is_null($date)) {
						$schedules[] = $calendar;
					}
				}
			}
			$events = null;
			return $schedules;
		}

		/**
		*	Set work schedule
		*
		*	@param cal_id ID of calendar entry that will be the work schedule
		*	@param date Optional, defaults to today.  Sets schedule after date
		*	@return boolean success
		*/
		public function set_work_schedule($cal_ids, $date = null) {
			if(!is_array($cal_ids)) {
				$cal_ids = array($cal_ids);
			}
			$current = $this->get_work_schedule($date);
			foreach($current as $schedule) {
				if(!in_array($schedule['id'], $cal_ids)) {
					if(is_array($schedule) && $schedule['id']) {
						unset($schedule['participants'][$this->get_staff_id()]);
						$result = ExecMethod2(
							'calendar.calendar_so.participants',
							$schedule['id'],
							$schedule['participants']
						);
					}
				}
			}

			foreach($cal_ids as $cal_id) {
				$cal = ExecMethod('calendar.calendar_bo.read', $cal_id);
				$cal['participants'][$this->get_staff_id()] = 'A';

				// Set new pay schedule
				$result = ExecMethod2(
					'calendar.calendar_so.participants',
					$cal_id,
					$cal['participants'],
					$date
				);
			}
		}


		/**
		*	Set the employment status
		*/
		public function set_status($status) {
			if(array_key_exists($status, Staff::get_status_list())) {
				$this->status = $status;
				return true;
			} else {
				throw new Exception(lang('Invalid parameter: %1', $status));
			}
		}
		
		public function get_staff_id() {
			return $this->staff_id;
		}
		public function set_staff_id($id) {
			if(is_numeric($id)) {
				$this->staff_id = $id;

				// Get addressbook ID of the user account
				$account_contact = ExecMethod('addressbook.addressbook_bo.read', 'account:'.$id);
				if(is_array($account_contact)) {
					if($this->get_addressbook_id() != $account_contact['id']) {
						ExecMethod('addressbook.addressbook_bo.merge', array($this->get_addressbook_id(), $account_contact['id']));
					}
					$this->set_addressbook_id($account_contact['id']);
				}
			} elseif ($this->staff_id) {
				// Clear user id
			} else {
				throw new Exception(lang('Invalid parameter: %1', $id));
			}
		}

		/**
		*	Access for custom variables.
		*	Custom variable names start with #
		*
		*	@param extra_name Name of the variable
		*	@return mixed
		*/
		public function __get($extra_name) {
			if(strpos($extra_name, '#') === 0) {
				return $this->extra_fields[substr($extra_name, 1)];
			}
			return null;
		}

		/**
		*	Access for custom variables.
		*	Custom variable names start with #
		*
		*	@param extra_name Name of the variable
		*	@param value New value for the variable
		*/
		public function __set($extra_name, $value) {
			if(strpos($extra_name, '#') === 0) {
				$this->extra_fields[substr($extra_name, 1)] = $value;
			}
		}

		public static function get_status_list() {
			return array(
				Staff::ACTIVE		=> lang('active'),
				Staff::ON_LEAVE		=> lang('on_leave'),
				Staff::SUSPENSION	=> lang('suspension'),
				Staff::GONE		=> lang('gone')
			);
		}

		public function toArray() {
			$data = get_object_vars($this);
			unset($data['payroll_config']);
			unset($data['timeclock_config']);

			foreach($data['extra_fields'] as $key => $value) {
				unset($data['extra_fields'][$key]);
				$data['extra_fields']['#' . $key] = $value;
			}
			foreach($data['skills'] as $key => $skill) {
				if(is_object($skill)) {
					$data['skills'][$key] = $skill->toArray();
				} else {
					// How did this happen?
					unset($this->skills[$key]);
				}
			}
			return $data;
		}

		public function __toString() {
			return "Staff - " . $this->staff_id;
		}

		public function __sleep() {
			$properties = get_object_vars($this);
			return array_keys($properties);
		}

	}
?>
