<?php
	/**
	*	Ajax functions for editing a staff
	*
	*	@author Nathan Gray
	*/

	require_once(EGW_API_INC.'/xajax.inc.php');

	class ajax_perp_staff {

		/**
		*	xajax object for returning XML responses
		*/
		private $xajax;

		/**
		*	Response object
		*/
		protected $response;

		/**
		*	Constructor\n
		*	Creates references to xajax
		*/
		public function __construct() {
			$this->xajax = new xajax();
		}

		/**
		*       Change the information specific to a department when the user picks a different department:
		*	- Reports to
		*       @param department_id int
		*       @return xml response
		*/
		public function change_department($etemplate_exec_id, $department_id) {

			$response = new xajaxResponse();

			$department = new Department((int)$department_id);

			if($department->get_manager()) {
				$element_id = etemplate::form_name('exec', 'reports_to[id]');

				// Set reports to  
				$script = "var select = document.getElementById('$element_id');
					var old_option = select.options[select.selectedIndex];
					select.options.length=0;
					select.options[0] = old_option;\n";
				$option = array('label' => bo_perp_staff::link_title($department->get_manager()));
				$script .= "if(old_option.value != " . $department->get_manager() . ") {
					opt = select.options[select.options.length] = new Option('".addslashes($option['label'])."','".addslashes('STAFF-' . $department->get_manager())."');
					select.selectedIndex = opt.index;
				}\n";
				$script .= "select.options[select.options.length] = new Option('".addslashes(lang('New search').' ...')."','');\n";

				$response->addScript($script);

				// store new allowed id's in the eT request
				if ($etemplate_exec_id && ($et_request = etemplate_request::read($etemplate_exec_id))) {
					$data = $et_request->get_to_process($element_id);
					//error_log($id_res.'='.array2string($data));
					$data['allowed'][] = 'STAFF-' . $department->get_manager();
					$et_request->set_to_process($element_id,$data);
				}
			}

			return $response->getXML();
		}

		/**
		*       Change the information specific to clients when the user picks a different supplier:
		*       - Contact person
		*       - Currency
		*       - Payment terms
		*	- Catalog
		*       This is only possible for an new Order
		*/
		public function change_client($client_id) {
		}

		/**
		*	Suggest an order number for this order based on 
		*	the company's settings
		*       @return xml response
		*/
//		public abstract function suggest_order_number();

		/**
		*	Set the line details when the user chooses an item to add to this Order
		*
		*	@param supplier_id	int	Supplier's ID
		*	@param supplier_code	string	Supplier's identification number for the selected catalog item
		*	@param address		int	The address that this purchase order will be shipped to
		*       @return xml response
		*/
		public function get_line_details($supplier_id, $supplier_code, $address) {
			$this->response = new xajaxResponse();

			$item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $supplier_id, $supplier_code);
			$stock_id = $item['stock_id'];

			// Get location from address
			$locations = array();
			$bo_location = CreateObject('perp_inventory.bo_perp_location');
			$bo_location->get_location_list(array('delivery_address' => $address), $locations);
			$location = array_shift($locations);
			$location = $location['location_id'];

			// Get default order quantity
			$quantity = ExecMethod2('perp_ap.bo_perp_purchase_order.get_order_quantity', $location, $supplier_code);
			$this->response->addScript('
				var quantity = document.getElementById("exec[new][quantity_ordered]");
				if(quantity && quantity.value == "") {
					quantity.value = ' . $quantity . ';
				}
			');

			// Default description line
			$description = ExecMethod2('perp_ap.bo_perp_catalog.link_title', $supplier_id . '-' . $supplier_code);
			$description = addslashes($description);
			$this->response->addScript('
				var description = document.getElementById("exec[new][description]");
				if(description) {
					description.value = "' . $description . '";
				}'
			);

			// Unit Cost
			$this->response->addScript('
				var unit_price = document.getElementById("exec[new][unit_price]");
				if(unit_price) {
					unit_price.value = "' . $item['price'] . '";
				}'
			);

			// Focus to quantity field
			$this->response->addScript("
				var quantity = document.getElementById('exec[new][quantity_ordered]');
				quantity.select();
			");

			return $this->response->getXML();
		}

		/** 
		*	Converts a php array to a javascript array 
		*	@todo Move this function.  This should go somewhere shared
		*
		*	@param name string javascript name of the array
		*	@param array	array	PHP array to convert
		*	@param prePend	string	String to prepend to the javascript array
		*	@return string Javascript array
		*/
		function phpArrayToJsArray($name,$array,$prePend='var ') {
			if (is_array($array)) { // Array recursion
				$result = $name.' = new Array();'."\n";
				foreach ($array as $key => $value) {
					$result .= $this->phpArrayToJsArray($name.'["'.$key.'"]',$value,'');
				}
			} else {  // Base case of recursion
				$result = $name.' = "'.addslashes($array).'";'."\n";
			}
			return $prePend.$result;
		}

		/**
		*	The order could not be found in the session
		*
		*	Show an alert.
		*/
		protected function missing_order() {
			$this->response = new xajaxResponse();
			$this->response->addAlert(
				lang('Something went wrong with this order.  Your order may have been saved.  Please click Cancel at the bottom of the page, and try again.')
			);
			return $this->response->getXML();
		}
	}
?>
