<?php
	require_once('class.Staff.inc.php');
	require_once('class.so_perp_staff.inc.php');

	class bo_perp_staff {
		
		private $so = null;


		public function __construct() {
			$this->so = new so_perp_staff();
		}

		/**
		*	Get a list of staff in the usual nextmatch widget fashion.
		*	If no query['template'] is false or not set, an array in the form
		*	staff_id => staff_name will be returned.  Otherwise, a list of information
		*	for each matching staff member will be passed to the $rows parameter
		*
		*	@nextmatch
		*/
		public function get_staff_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('staff', EGW_ACL_READ, 'perp_human_resources');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!$query['order']) {
				$query['order'] = 'staff_id';
			}
			if($query['filter']) {
				$query['department_id'] = $query['filter'];
			}
			if($query['filter2']) {
				$query['status'] = $query['filter2'];
			}
			if(!isset($query['template']) && !isset($query['num_rows'])) {
				// Get all rows for a select box
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_staff_list($query);

			if(!isset($query['template']) && !$query['get_all_data']) {
				$rows = array();
				foreach($list as $row) {
					$rows[$row['staff_id']] = $this->link_title($row);
				}
				return $rows;
			}
			$rows = $list;

			return $query['count'];
		}

		/**
		*	Factory method to create a new staff member.
		*
		*	This creates a new staff member, complete with user account, not a new Staff object.
		*
		*	@param staff Staff object to populate
		*	@param userData array of user data
		*	
		*	@return string the new account's password.
		*/
		public function create_new_staff(Staff &$staff, Array $userData) {
			if(!$GLOBALS['egw']->acl->check('staff', EGW_ACL_ADD, 'perp_human_resources')) {
				throw new Exception(lang('Need ADD permission to %1', 'perp_human_resources:staff'));
			}
			if($userData['user_account'] && $userData['user_account'] != 'popup') {
				$account = ExecMethod('phpgwapi.accounts.read', $userData['user_account']);
				if($account) {
					$staff->set_staff_id($userData['user_account']);
					$staff->set_employee_number($userData['account_lid']);

					// Contact info
					$contact = ExecMethod('addressbook.addressbook_bo.read', $account['person_id']);

					if($contact['org_unit']) {
						// Look for matching department
						$departments = ExecMethod('perp_human_resources.bo_perp_department.get_department_list', array('search' => $contact['org_unit']));
						if(count($departments)) {
							$staff->set_department_id(key($departments));
						}
					}

					return true;
				}
			} 
			if($userData['create_user_account']) {
				// We need to create the eGW account, set the address, and create & save the Staff
				// Create eGW account
				$account_id = null;
				if($this->validate_user($userData)) {
					$account_id = ExecMethod('phpgwapi.accounts.save', $userData);
				}
				if($account_id) {
					$staff->set_staff_id($account_id);
					$staff->set_employee_number($userData['account_lid']);
				}
			}

			// File a record in the addressbook
			$config = config::read('perp_human_resources');
			$contact = array(
				'n_fileas'	=>	$userData['last_name'] . ', ' . $userData['first_name'],
				'n_family'	=>	$userData['last_name'],
				'n_given'	=>	$userData['first_name'],
				'n_middle'	=>	$userData['middle_name'],
				'contact_title'	=>	$userData['title'],
				'owner'		=>	$config['staff_addressbook']
			);

			$contact_id = ExecMethod('addressbook.addressbook_bo.save', $contact);
			$staff->set_addressbook_id($contact_id);
			$staff->set_staff_id($contact_id);
			$staff->set_department_id($userData['department_id']);
			if($staff->get_department_id()) {
				$department = new Department($staff->get_department_id());
				$staff->set_reports_to($department->get_manager());
			}

			return $userData['account_passwd'];
		}

		/**
		*	Fix up an array of userData to create a new staff person
		*/
		protected function validate_user(Array &$userData) {
			$config = config::read('perp_human_resources');

			$userData['account_firstname']	=	$userData['first_name'];
			$userData['account_lastname']	=	$userData['last_name'];
			$userData['account_type']	=	'u';
			$userData['account_expires']	=	-1;

			if(!$userData['account_lid']) {
				$query = array(
					'type'		=>	'accounts',
				);
				$results = ExecMethod('phpgwapi.accounts.search', $query);
				
				$userData['account_lid'] = sprintf(
					$config['user_name'],
					$userData['first_name'],
					$userData['last_name'],
					$userData['department'],
					count($results)
				);


				// Invalidate account cache
				$GLOBALS['egw']->accounts->cache_invalidate();

				// Check for duplicates
				$query['query_type'] = 'lid';
				$pad = '';
				$query['query'] = $userData['account_lid'];
				$results = ExecMethod('phpgwapi.accounts.search', $query);
				while(count($results) > 0) {
					$query['query'] = $userData['account_lid'] . ++$pad;
					$results = ExecMethod('phpgwapi.accounts.search', $query);
				}
				$userData['account_lid'] = $userData['account_lid'].$pad;
			}
			if(!$userData['account_passwd']) {
				// Create password
				$userData['account_passwd'] = $userData['account_passwd_2'] = substr(md5(time()), 0, 8);
			}
			if(!$userData['account_status']) {
				// User account disabled by default
				$userData['account_status'] = 'D';
			}

			if(!$userData['account_primary_group']) {
				$userData['account_primary_group'] = $config['default_group'];
			}
			if(!$userData['account_groups']) {
				$userData['account_groups'] = array($userData['account_primary_group']);
			}
			// Returns false if valid, errors otherwise
			$errors = ExecMethod('admin.boaccounts.validate_user', $userData);
			if(is_array($errors)) {
				throw new Exception(print_r($userData, true) . "\n" . lang(current($errors)));
			}
			if($errors !== false) {
				throw new Exception(lang('Could not create user account'));
			}
			return true;
		}

		/**
		*	Get a list of authentication types registered for a staff person.
		*
		*	@param staff_id
		*/
		public function get_authentication_list($staff_id) {
			if($GLOBALS['egw']->acl->check('staff_authentication', EGW_ACL_READ, 'perp_human_resources')) {
				return $this->so->get_authentication_list($staff_id);
			} else {
				return array();
			}
		}

		/**
		*	Add some form of authentication to a staff ID
		*
		*	@param staff_id
		*	@param auth_type Type of authentication, such as PalmScan, Fingerprint, RFID
		*	@param auth_string The authentication data
		*
		*	@return boolean success or String error message
		*/
		public function add_authentication($staff_id, $auth_type, $auth_string) {
			if($GLOBALS['egw']->acl->check('staff_authentication', EGW_ACL_ADD, 'perp_human_resources')) {
				return $this->so->add_authentication($staff_id, $auth_type, $auth_string);
			} else {
				return false;
			}
		}

		/**
		*	Remove some form of authentication from a staff ID
		*	Removes all entries of the given type.
		*
		*	@param staff_id
		*	@param auth_type Type of authentication, such as PalmScan, Fingerprint, RFID
		*/
		public function remove_authentication($staff_id, $auth_type) {
			if($GLOBALS['egw']->acl->check('staff_authentication', EGW_ACL_DELETE, 'perp_human_resources')) {
				return $this->so->remove_authentication($staff_id, $auth_type);
			} else {
				return false;
			}
		}

		/**
		*	Get a staff ID by some form of authentication
		*
		*	@param auth_type Type of authentication, such as PalmScan, Fingerprint, RFID
		*	@param auth_string The authentication data
		*
		*	@return int staff_id
		*/
		public function identify($auth_type, $auth_string) {
			if($GLOBALS['egw']->acl->check('staff_authentication', EGW_ACL_READ, 'perp_human_resources')) {
				return $this->so->authenticate($auth_type, $auth_string);
			} else {
				return -1;
			}
		}

		/**
		*	Get staff list for xmlrpc 
		*/
		public function xmlrpc_get_staff_list($query) {
			return $this->get_staff_list($query);
		}

		/**
		*	add_authentication for xmlrpc (which can only handle 1 parameter) 
		*/
		public function xmlrpc_add_authentication($params) {
			return $this->add_authentication($params['staff_id'], $params['authentication_type'], $params['authentication_data']);
		}

		/**
		*	identify for xmlrpc (which can only handle 1 parameter)
		*/
		public function xmlrpc_identify($params) {
			return $this->identify($params['authentication_type'], $params['authentication_data']);
		}


		/**
		*	How should Staff be displayed for links, select boxes, etc
		*/
		public static function link_title($entry) {
			if($entry instanceof Staff) {
				$contact = ExecMethod('addressbook.addressbook_bo.read', $entry->get_addressbook_id());
				return ExecMethod('addressbook.addressbook_bo.fileas', $contact);
			} elseif (is_array($entry)) {
				return $entry['n_fn'];
			} elseif (is_numeric($entry)) {
				return self::link_title(new Staff($entry));
			}
			return null;
		}

		public function link_query($search) {
			$query = array(
				'search'	=>	$search
			);
			$staff = $this->get_staff_list($query);

			$results = array();
			foreach($staff as $staff_id => $name) {
				$results['STAFF-' . $staff_id] = $name;
			}

			return $results;
		}

		/**
		 * handles introspection or discovery by the logged in client,
		 *  in which case the input might be an array.  The server always calls
		 *  this function to fill the server dispatch map using a string.
		 *
		 * @param string $_type='xmlrpc' xmlrpc or soap
		 * @return array
		 */
		function list_methods($_type='xmlrpc')
		{
			if (is_array($_type))
			{
				$_type = $_type['type'] ? $_type['type'] : $_type[0];
			}

			switch($_type)
			{
				case 'xmlrpc':
					$xml_functions = array(
						'get_staff_list'	=>	array(
							'function'	=>	'xmlrpc_get_staff_list',
							'signature'	=>	array(array(xmlrpcStruct,xmlrpcStruct)),
							'docstring'	=>	lang('Get a list of staff people')
						),
						'add_authentication' => array(
							'function'  => 'xmlrpc_add_authentication',
							'signature' => array(array(xmlrpcStruct,xmlrpcStruct)),
							'docstring' => lang('Add a type of authentication for a staff person.  Used for the TimeClock.')
						),
						'identify' => array(
							'function'  => 'xmlrpc_identify',
							'signature' => array(array(xmlrpcStruct,xmlrpcStruct)),
							'docstring' => lang('Get a staff_id by passing an authentication type and the authentication data.  Used for the TimeClock.')
						),
					);
					return $xml_functions;
					break;
				case 'soap':
					return $this->soap_functions;
					break;
				default:
					return array();
					break;
			}
		}
	}
?>
