<?php

	/**
	*	Storage object for working with staff
	*
	*	The Staff object itself does operations on itself, so 
	*	most of what is here is for operations on many staff.
	*/

	class so_perp_staff {

		public function __construct() {
		}

		public function get_staff_list(&$query) {
			$where = array();
			$join = 'LEFT JOIN egw_addressbook ON
					perp_staff.addressbook_id = egw_addressbook.contact_id
			LEFT JOIN perp_staff_authentication ON
				perp_staff.staff_id = perp_staff_authentication.staff_id';


			if($query['search']) {
				$where[] = '(employee_number LIKE ' . $GLOBALS['egw']->db->quote('%'.$query['search'].'%') . ' OR ' .
					'n_fn LIKE ' . $GLOBALS['egw']->db->quote('%'.$query['search'].'%') . ')';
			}
			if($query['no_authentication']) {
				$where[] = 'authentication_type IS NULL';
			}
			if($query['department_id']) {
				$where['department_id'] = $query['department_id'];
			} 
			if($query['pay_type']) {
				$where[] = 'pay_type = ' . $GLOBALS['egw']->db->quote($query['pay_type']);
				$join .= ' LEFT JOIN perp_staff_pay_detail ON
					perp_staff.staff_id = perp_staff_pay_detail.staff_id';
			}
			if($query['status']) {
				$where[] = 'status = ' . $GLOBALS['egw']->db->quote($query['status']);
			}

			// Get number of records
			$GLOBALS['egw']->db->select(
				'perp_staff',
				array('count(perp_staff.staff_id) AS count'),
				$where,
				__LINE__, __FILE__, 0, 'GROUP BY perp_staff.staff_id',
				'perp_human_resources', -1,
				$join
			);
			$query['count'] = 0;
			while($row = $GLOBALS['egw']->db->row(true)) {
				$query['count'] ++;
			}

			$GLOBALS['egw']->db->select(
				'perp_staff',
				array(
					'perp_staff.staff_id',
					'n_fn',
					'n_family',
					'n_given',
					'addressbook_id',
					'egw_addressbook.*',
					'job_description',
					'employee_number',
					'department_id',
					'reports_to',
					'CONCAT("STAFF-", reports_to) AS reports_to_link'
				),
				$where,
				__LINE__, __FILE__,
				$query['start'],
				'GROUP BY staff_id ' . 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_human_resources',
				$query['num_rows'],
				$join
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Get a list of registered authentication types for the given staff member
		*/
		public function get_authentication_list($staff_id) {
			$GLOBALS['egw']->db->select(
				'perp_staff_authentication',
				array('authentication_type'),
				array('staff_id'	=>	$staff_id),
				__LINE__, __FILE__, 0,
				'',
				'perp_human_resources'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Add a method of authentication for a staff member
		*/
		public function add_authentication($staff_id, $type, $data) {
			$result = $GLOBALS['egw']->db->insert(
				'perp_staff_authentication',
				array(
					'staff_id'	=>	$staff_id,
					'authentication_type'	=>	$type,
					'authentication_data'	=>	$data
				),
				array(),
				__LINE__, __FILE__,
				'perp_human_resources'
			);

			return ($result == true);
		}

		public function remove_authentication($staff_id, $auth_type) {
			$result = $GLOBALS['egw']->db->delete(
				'perp_staff_authentication',
				array(
					'staff_id'	=>	$staff_id,
					'authentication_type'	=>	$auth_type,
				),
				__LINE__, __FILE__,
				'perp_human_resources'
			);
		}

		/**
		*	Authenticate a staff member and return the staff id
		*	@param type Type of authentication 
		*	@param data Authentication data
		*/
		public function authenticate($auth_type, $auth_data) {
			$GLOBALS['egw']->db->select(
				'perp_staff_authentication',
				array('staff_id'),
				array(
					'authentication_type'	=>	$auth_type,
					'authentication_data'	=>	$auth_data
				),
				__LINE__, __FILE__, 0,
				'GROUP BY staff_id',
				'perp_human_resources'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			if(count($list) != 1) {
				return false;
			} else {
				return (int)$list[0]['staff_id'];
			}
		}
	}
?>
