<?php

	require_once('class.Department.inc.php');
	require_once('class.bo_perp_department.inc.php');

	class ui_perp_department {

		public $public_functions = array(
			'department_list'	=>	true,
			'department_new'	=>	true,
			'edit'			=>	true
		);

		private $debug = false;

		/**
		*	eTemplate object used for display
		*/
		private $template = null;

		/**
		*	Object used for data
		*/
		private $bo_department = null;
		
		/**
		*       eTemplates used to display the list
		*/
		protected $list_template = 'perp_human_resources.department_list';
		protected $list_row_template = 'perp_human_resources.department_list.row';

		public function __construct() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo_department = new bo_perp_department();
		}
		
		public function department_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			// Deal with incoming
			$this->list_update($content);

			// Set up outgoing
			$data = $this->list_get_display_data();
			$select_box_options = $this->list_get_options();
			$readonly = $this->list_get_readonlys();
			
			// Display
			$this->display(
				'department list', 
				'perp_human_resources.ui_perp_department.department_list', 
				$this->list_template, 
				$data, $select_box_options, $readonly, $preserve
			);
		}

		/**
		*	Create a new department
		*/
		public function department_new($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			$department = new Department();
			$GLOBALS['egw']->session->appsession('department', false, $department);
			$GLOBALS['egw']->redirect_link('/index.php', array(
				'menuaction'	=>	'perp_human_resources.ui_perp_department.edit'
			));
			return true;
		}

		public function edit($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			$department_id = reg_var('department_id');
			$department = $this->get_department($department_id);
			$department_id = $department->get_department_id();

			
			// Deal with incoming
			$result = $this->edit_update($department, $content);
			if($result) {
				$this->messages[] = array(lang('success'), 'success');
			} else {
			}

			// Set up outgoing
			$data = $this->edit_get_display_data($department);
			$select_box_options = $this->edit_get_options($department);
			$readonly = $this->edit_get_readonlys($department);
			
			// Display
			$this->display(
				($department_id ? 'Edit ' . $this->bo_department->link_title($department) : 'new department'), 
				'perp_human_resources.ui_perp_department.edit', 
				'perp_human_resources.department_edit', 
				$data, $select_box_options, $readonly, $preserve
			);
		}

		/**
		*       Internal function to get a Department object to work with.
		*       First, we check the session for a Department object being edited.  Then we
		*       check the provided Department ID and read from the database, if required.
		*       Extending classes should override this function.
		*
		*	@param department_id Optional department_id to read
		*/
		protected function &get_department($department_id = null) {
			$o = $GLOBALS['egw']->session->appsession('department');
			if(
				($department_id && $o instanceof Department && $department_id != $o->get_department_id()) ||
				$department_id && !($o instanceof Department)
			) {
				$department = new Department($department_id);
			} elseif ($o) {
				$department = $o;
			} else {
				$department = new Department();
			}

			if($this->debug) {
				echo __METHOD__;
				_debug_array($department);
			}
			$GLOBALS['egw']->session->appsession('department', 'perp_human_resources', $department);
			return $department;
		}


		/**
		*	Update the list of displayed departments, or act on a 
		*	user's actions
		*/
		protected function list_update(&$content) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			if(isset($content['new_department'])) {
				$this->handle_action('department_new', null);
			} elseif(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_human_resources.ui_perp_human_resources.index'
				));
			}
			if(!isset($content['nm']['rows'])) {
				return false;
			}
			foreach($content['nm']['rows'] as $action => $row) {
				$department_id = key($row);
				$result = $this->handle_action($action, $department_id);
				if($result) {
					unset($content['nm']['rows'][$action]);
				}
			}
		}

		/**
		*	Get data for display, set up the nextmatch list
		*/
		protected function list_get_display_data() {
			$data = array();

			$data['nm'] = array(
				'template'	=>	$this->list_row_template,
				'get_rows'	=>	'perp_human_resources.ui_perp_department.get_department_list',		// Comes from UI to provide memory
				'no_filter'	=>	true,
				'no_filter2'	=>	true
			);

			return $data;
		}

		/**
		*	Get data for selectbox lists
		*/
		protected function list_get_options() {
			$options['filter'] = $this->bo_department->get_department_list();
			return $options;
		}

		/**
		*	Get array of fields to set readonly
		*	(doesn't get the nextmatch, which comes from bo_staff)
		*/
		protected function list_get_readonlys() {
			return $readonly;
		}


		/**
		*	Update a Department object with the data provided
		*	@param department Department object to update
		*	@param content Data to update with
		*
		*	@return boolean success
		*/
		protected function edit_update(Department $department, Array $content) {
			
			$department->set_parent($content['department_parent']);


			if($content['department_name']) {
				$department->set_name($content['department_name']);
			}
			if($content['department_number']) {
				$department->set_number($content['department_number']);
			}
			if($content['department_manager']) {
				$department->set_manager($content['department_manager']);
			}
			if($content['group_id']) {
				$department->set_group_id($content['group_id']);
			}

			if(isset($content['save'])) {
				$result = is_numeric($department->write());
			}
			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_human_resources.ui_perp_department.department_list'
				));
			}

			return ($result === true);
		}

		/**
		*	Get data for displaying a Department object
		*
		*	@param department Department object to display
		*	@return array of data for eTemplate
		*/
		protected function edit_get_display_data(Department $department) {

			$data = $department->toArray();
			$data['department_parent']	=	$department->get_parent_id();

			return $data;
		}

		/**
		*	Get selectbox options for displaying a Department object
		*	@param department Department object to display
		*	@return array of options for eTemplate
		*/
		protected function edit_get_options(Department $department) {
			$options = array();

			$options['department_parent']	=	array(0 => lang('none')) + ExecMethod('perp_human_resources.bo_perp_department.get_department_list');
			$options['department_manager']	=	ExecMethod('perp_human_resources.bo_perp_staff.get_staff_list', array('order' => 'n_family,n_given'));
			unset($options['parent_id'][$department->get_department_id()]);

			return $options;
		}

		/**
		*	Get readonly fields for displaying a Department object
		*	@param department Department object to display
		*	@return array of fields to be readonly for eTemplate
		*/
		protected function edit_get_readonlys(Department $department) {
			$readonly = array();

			return $readonly;
		}

		/**
		*	Get a list of departments, and remember position and filter settings
		*/
		public function get_department_list(&$query, &$rows = array(), &$readonly = array()) {
			$stored_query = $GLOBALS['egw']->session->appsession('department_list', 'perp_human_resources');
			if(!is_array($stored_query)) {
				$stored_query = array();
			}
			$query = array_merge($stored_query, $query);
			$GLOBALS['egw']->session->appsession('departmen_list', 'perp_human_resources', $query);

			return $this->bo_department->get_department_list(&$query, &$rows, &$readonlys);
		}

		/**
		*	Display a template
		*	Utility wrapper to set the title, read the template & show it
		*/
		protected function display($title, $callback, $template, $data, $options, $readonly, $preserve) {
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = lang($title);
			$this->template->read($template);
			$this->template->exec($callback, $data, $options, $readonly, $preserve);
		}

		/**
		*	Handle an action on a department (from a list)
		*/
		protected function handle_action($action, $department_id) {
			switch($action) {
				case 'department_new':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_human_resources.ui_perp_department.department_new'
					));
					return true;
					break;
				case 'delete':
					$this->bo_department->delete($department_id);
					break;
				default:
					return false;
					break;
			}
		}
	}
?>
