<?php
	/**
	*	User Interface class for dealing with Staff.
	*/

	require_once('class.Staff.inc.php');
	require_once('class.bo_perp_staff.inc.php');
	require_once('class.bo_perp_department.inc.php');

	if($GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_payroll')) {
		include_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Paystub.inc.php');
		include_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.EarningType.inc.php');
	}

	class ui_perp_staff {

		public $public_functions = array(
			'staff_list'	=>	true,
			'edit'		=>	true,
			'staff_new'	=>	true,
			'delete'	=>	true,
		);

		protected $tabs = 'details|extra_fields|pay|skills|resources|links';

		/**
		*	eTemplates used to display the list
		*/
		protected $list_template = 'perp_human_resources.staff_list';
		protected $list_row_template = 'perp_human_resources.staff_list.row';

		private $debug = false;

		/**
		*	business object used for data
		*/
		private $bo_staff;

		/**
		*	Logic level object for payroll
		*/
		private $bo_payroll;

		/**
		*	eTemplate object used for display
		*/
		private $template;

		/**
		*	Array of messages to the user
		*/
		private $messages = array();

		/**
		*	Payroll config options
		*/
		private $payroll_config;

		public function __construct() {
			$this->bo_staff = new bo_perp_staff();
			
			$this->bo_payroll = CreateObject('perp_payroll.bo_perp_payroll');
			$this->template = CreateObject('etemplate.etemplate');

			$this->payroll_config = config::read('perp_payroll');

			// Get messages from session
			$messages = $GLOBALS['egw']->session->appsession('messages');
			$GLOBALS['egw']->session->appsession('messages', false, array());
			if(is_array($messages)) {
				$this->messages = $messages;
			}
		}

		public function staff_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			// Deal with incoming
			$this->list_update($content);

			// Set up outgoing
			$data = $this->list_get_display_data();
			$select_box_options = $this->list_get_options();
			$readonly = $this->list_get_readonlys();
			
			// Display
			$this->display(
				'staff list', 
				'perp_human_resources.ui_perp_staff.staff_list', 
				$this->list_template, 
				$data, $select_box_options, $readonly, $preserve
			);
		}

		/**
		*	Create a new staff member, then edit it.
		*/
		public function staff_new($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			// Deal with incoming

			// Keep department to speed up new hires
			$GLOBALS['egw']->preferences->add('perp_human_resources', 'new_staff_department', $content['department_id']);
			$GLOBALS['egw']->preferences->save_repository(true);


			$staff = new Staff();
			if(isset($content['save'])) {
				$staff = $this->new_update($content);
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_human_resources.ui_perp_staff.staff_list'
				));
			}
			$GLOBALS['egw']->session->appsession('staff', false, $staff);
			if($staff->get_staff_id()) {
				// We need to keep the messages
				$GLOBALS['egw']->session->appsession('messages', false, $this->messages);

				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_human_resources.ui_perp_staff.edit'
				));
			}

			$staff = new Staff();

			// Set up outgoing
			$data['create_user_account'] = true;
			$data['department_id'] = $GLOBALS['egw_info']['user']['preferences']['perp_human_resources']['new_staff_department'];
			$select_box_options = $this->edit_get_options($staff);
			$readonly = $this->edit_get_readonlys($staff);
			
			// Display
			$this->display(
				'new staff', 
				'perp_human_resources.ui_perp_staff.staff_new', 
				'perp_human_resources.staff_new', 
				$data, $select_box_options, $readonly, $preserve
			);

		}

		/**
		*	Edit a staff member
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}
			
			$staff_id = get_var('staff_id', 'any', $content['staff_id']);
			$staff =& $this->get_staff($staff_id);
			if(is_object($staff) && !is_null($staff) ) {
				$staff_id = $staff->get_staff_id();
			} else {
				$content['cancel'] = true;
			}
		
			// Deal with incoming
			if(!isset($content['cancel'])) {
				try {
					$this->edit_update($staff, $content);
				} catch (egw_exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->session->appsession('staff', false, null);
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_human_resources.ui_perp_staff.staff_list'
				));
			}

			// Set up outgoing
			$options = array();
			$data = $this->edit_get_display_data($staff, $options);
			$select_box_options = $this->edit_get_options($staff, $options);
			$readonly = $this->edit_get_readonlys($staff);
			if($GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_payroll')) {
				$this->get_payroll_readonly($staff, $data, $readonly);
			}
			$preserve['staff_id'] = $data['staff_id'];
			$preserve['skills'] = $data['skills'];
			$preserve['pay']['paystub_entries'] = $data['pay']['paystub_entries'];

			// Load javascript
			$GLOBALS['egw']->js->validate_file('staff', 'edit', 'perp_human_resources');
			
			// Display
			$this->display(
				($staff_id ? 'Edit ' . $this->bo_staff->link_title($staff) : 'new staff'), 
				'perp_human_resources.ui_perp_staff.edit', 
				'perp_human_resources.staff_edit', 
				$data, $select_box_options, $readonly, $preserve
			);
		}

		/**
		*	Delete (fire / terminate) a staff person.  
		*
		*	Firing keeps all the information for historical purposes.  You still need to keep all the records.
		*	The user account will be set to deactivate on the effective date.
		*/
		public function delete($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}
			
			$staff_id = reg_var('staff_id');
			if(!$staff_id) {
				$staff_id = $content['staff_id'];
			}
			$staff =& $this->get_staff($staff_id);
			if(!is_null($staff) ) {
				$staff_id = $staff->get_staff_id();
				if($staff_id && $content['end_date']) {
					$staff->delete($content['end_date'], $content['reason']);
					$GLOBALS['egw']->session->appsession('staff', false, null);
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_human_resources.ui_perp_staff.staff_list'
					));
				}
			}
			$data = array('end_date' => time());
			
			$this->display(
				lang('Delete %1', $this->bo_staff->link_title($staff)),
				'perp_human_resources.ui_perp_staff.delete',
				'perp_human_resources.staff_delete',
				$data, $select_box_options, array(), array('staff_id' => $staff_id)
			);
		}

		/**
		*       Internal function to get a Staff object to work with.
		*       First, we check the session for a Staff object being edited.  Then we
		*       check the provided Staff ID and read from the database, if required.
		*       Extending classes should override this function.
		*/
		protected function &get_staff($staff_id) {
			$s = $GLOBALS['egw']->session->appsession('staff');
			if(
				($staff_id && $s instanceof Staff && $staff_id != $s->get_staff_id()) ||
				$staff_id && !($s instanceof Staff)
			) {
				$staff = new Staff($staff_id);
			} elseif ($s) {
				$staff = $s;
			} else {
				$staff = new Staff();
			}

			if($this->debug) {
				echo __METHOD__;
				_debug_array($staff);
			}
			return $staff;
		}

		/**
		*	Update the list of displayed staff people, or act on a 
		*	user's actions
		*/
		protected function list_update(&$content) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_human_resources.ui_perp_human_resources.index'
				));
			}
			if(isset($content['new_staff'])) {
				$this->handle_action('staff_new', null);
			}
			if(!isset($content['nm']['rows'])) {
				return false;
			}
			foreach($content['nm']['rows'] as $action => $row) {
				$staff_id = key($row);
				$result = $this->handle_action($action, $staff_id);
				if($result) {
					unset($content['nm']['rows'][$action]);
				}
			}

		}

		/**
		*	Get data for display, set up the nextmatch list
		*/
		protected function list_get_display_data() {
			$data = array();

			$data['nm'] = $GLOBALS['egw']->session->appsession('staff_list', 'perp_human_resources');
			if(!$data['nm']) {
				$data['nm'] = array(
					'template'	=>	$this->list_row_template,
					'get_rows'	=>	'perp_human_resources.ui_perp_staff.get_staff_list',		// Comes from UI to provide memory
				);
				$data['nm']['sort'] = $GLOBALS['egw_info']['user']['preferences']['perp_human_resources']['staff_list_query']['sort'];
				$data['nm']['order'] = $GLOBALS['egw_info']['user']['preferences']['perp_human_resources']['staff_list_query']['order'];
			}


			return $data;
		}

		/**
		*	Get data for selectbox lists
		*/
		protected function list_get_options() {
			$bo_department = new bo_perp_department();

			$options['filter'] = array('' => 'all') + $bo_department->get_department_list();
			$options['filter2'] = Staff::$status_list;
			return $options;
		}

		/**
		*	Get array of fields to set readonly
		*	(doesn't get the nextmatch, which comes from bo_staff)
		*/
		protected function list_get_readonlys() {
			return $readonly;
		}

		/**
		*	Get a list of staff, and remember position and filter settings
		*/
		public function get_staff_list(&$query, &$rows = array(), &$readonly = array()) {
			$stored_query = $GLOBALS['egw']->session->appsession('staff_list', 'perp_human_resources');
			if(!is_array($stored_query)) {
				$stored_query = array();
			}
			$query = array_merge($stored_query, $query);
			$GLOBALS['egw']->session->appsession('staff_list', 'perp_human_resources', $query);

			// Store it in preferences too, so it gets preserved across sessions
			$GLOBALS['egw']->preferences->add(
				'perp_human_resources', 
				'staff_list_query', 
				array('order' => $query['order'], 'sort' => $query['sort'])
			);
			$GLOBALS['egw']->preferences->save_repository(true);

			return $this->bo_staff->get_staff_list(&$query, &$rows, &$readonlys);
		}

		/**
		*	Create a new Staff object with the data provided
		*	@param content Data to update with
		*	@return staff New Staff object
		*/
		protected function new_update(Array $content) {
			$staff = new Staff();
			$password = $this->bo_staff->create_new_staff($staff, $content);

			if($staff->get_staff_id()) {
				// Don't reveal info about an existing account
				if(is_string($password)) {
					$account = ExecMethod('phpgwapi.accounts.read', $staff->get_staff_id());
					$this->messages[] = array(lang('User account login is %1', $account['account_lid']), 'info');
					$this->messages[] = array(lang('User account password is %1', $password), 'info');
					$this->messages[] = array(lang('User account is %1', $account['account_status'] == 'A' ? lang('active') : lang('disabled')), 'info');
				}
				$staff->write();
			}

			return $staff;
		}

		/**
		*	Update a Staff object with the data provided
		*	@param staff Staff object to update
		*	@param content Data to update with
		*/
		protected function edit_update(Staff $staff, Array $content) {
			if(!$GLOBALS['egw']->acl->check('staff', EGW_ACL_EDIT, 'perp_human_resources') && $content['save']) {
				$this->messages[] = array(lang('You are not allowed to edit staff'), 'warning');
				return false;
			}
			if($content['employee_number']) {
				$this->check_result($staff->set_employee_number($content['employee_number']));
			}
			if($content['department_id']) {
				$this->check_result($staff->set_department_id($content['department_id']));
			}
			if($content['reports_to']) {
				$this->check_result($staff->set_reports_to($content['reports_to']));
			}
			if($content['job_description']) {
				$this->check_result($staff->set_job_description($content['job_description']));
			}
			
			if(isset($content['start_date'])) {
				$this->check_result($staff->set_start_date($content['start_date']));
			}
			if(isset($content['end_date'])) {
				$staff->set_end_date($content['end_date']);
				if($content['end_date'] > 0) {
					$this->check_result($staff->delete($content['end_date'], null));
				}
			}
			if($content['status']) {
				$this->check_result($staff->set_status($content['status']));
			}
			if($content['gender']) {
				$this->check_result($staff->set_gender($content['gender']));
			}
			try {
				$this->check_result($staff->set_staff_id($content['user_id']));
			} catch (Exception $e) {
				$this->check_result($e->getMessage(), 'error');
			}

			if(is_array($content['extra_fields'])) {
				foreach($content['extra_fields'] as $key => $value) {
					$staff->$key = $value;
				}
			}

			// Payroll
			if(is_array($content['pay']) && $GLOBALS['egw']->acl->check('staff_data', EGW_ACL_EDIT, 'perp_payroll')) {
				if(is_array($content['pay']['paystub_entries'])) {
					foreach($content['pay']['paystub_entries'] as $entry_data) {
						$entry = PayStubEntryType::factory($entry_data['type_id']);
						if(!$entry) {
							continue;
						}
						if($entry_data['on_paystub']) {
							$staff_data = array(
								'staff_id'	=>	$staff->get_staff_id(),
								'type_id'	=>	$entry_data['type_id']
							);
							$staff_data += $entry_data['override_value'];
							$entry->set_staff(array($staff_data));
						} elseif (!$entry_data['on_paystub']) {
							$entry->remove_staff($staff);
						}
					}
				}

				if(isset($content['pay']['work_shift'])) {
					$this->check_result($staff->set_work_schedule(explode(',',$content['pay']['work_shift'])));
				}
				if(isset($content['pay']['payment_schedule'])) {
					$this->check_result($staff->set_payment_schedule($content['pay']['payment_schedule']));
				}
				if($content['pay']['type_id']) {
					unset($content['pay']['paystub_entries']);

					$content['pay']['value'] = array(
						'amount'	=>	$content['pay']['pay_rate'],
						'per'		=>	$content['pay']['pay_unit']
					);
					$this->check_result( $this->bo_payroll->set_staff_data($staff, $content['pay']));
				}
			}

			if($content['authentication']['delete']) {
				$auth_type = key($content['authentication']['delete']);
				$this->bo_staff->remove_authentication($staff->get_staff_id(), $auth_type);
			}

			// Skills
			if(is_array($content['new_skills'][1]) && $content['new_skills'][1]['skill_name'] != '') {
				$staff->add_skill(new StaffSkill($staff, $content['new_skills'][1]));
			}
			if(is_array($content['skills'])) {
				unset($content['skills'][0]);	// Remove title row
				foreach($content['skills'] as $key => $skill_array) {
					if($key == 'delete') {
						$skill =& $staff->remove_skill(key($skill_array));
						continue;
					}
					$skill =& $staff->get_skill($skill_array['skill_id']);
					if(is_null($skill) || !is_object($skill)) {
						continue;
					}
					$skill->set_name($skill_array['skill_name']);
					$skill->set_start_date($skill_array['skill_start_date']);
					$skill->set_expiry_date($skill_array['skill_expiry_date']);
					$skill->set_skill_level($skill_array['skill_level']);
				}
			}

			$GLOBALS['egw']->session->appsession('staff', false, $staff);

			if($content['save']) {
				$this->check_result($staff->write());
			}
		}

		/**
		*	Get data for displaying a Staff object
		*
		*	@param staff Staff object to display
		*	@return array of data for eTemplate
		*/
		protected function edit_get_display_data(Staff $staff, &$options) {

			$data = array();

			$data += $staff->toArray();

			// Default employee number to staff_id
			if(!$data['employee_number']) {
				$data['employee_number'] = $data['staff_id'];
			}

			if($data['end_date'] == 0) {
				$data['end_date'] = '';
			}

			// Turn into a link key
			$data['reports_to'] = 'STAFF-' . $data['reports_to'];

			// Calculate age
			$data['age'] = $staff->get_age();

			if($GLOBALS['egw']->acl->check('staff_authentication', EGW_ACL_READ, 'perp_human_resources')) {
				$data['authentication'] = $this->bo_staff->get_authentication_list($staff->get_staff_id());
				// Shift 1 to leave room for title
				array_unshift($data['authentication'], false);
				$data['authentication']['delete'] = $GLOBALS['egw']->acl->check('staff_authentication', EGW_ACL_DELETE, 'perp_human_resources');
			}

			if($GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_payroll')) {
				$this->get_payroll_data($staff, $data, $options);
			}

			if(!$GLOBALS['egw']->acl->check('staff', EGW_ACL_EDIT, 'perp_human_resources')) {
				$data['no_add_skills'] = true;
			}
			// Move skills to allow for title row
			if(count($data['skills']) > 0) {
				array_unshift($data['skills'], false);
			} else {
				unset($data['skills']);
			}
			$data['link'] = array(
				'to_app'	=>	'perp_human_resources',
				'to_id'		=>	'STAFF-' . $data['staff_id']
			);
			return $data;
		}

		/**
		*	Get selectbox options for displaying a Staff object
		*	@param staff Staff object to display
		*	@return array of options for eTemplate
		*/
		protected function edit_get_options(Staff &$staff, &$options = array()) {
			$options['location_id']		=	ExecMethod('perp_inventory.bo_perp_location.get_location_list', array('filter2' => 'shippable'));
			$options['department_id']	=	ExecMethod('perp_human_resources.bo_perp_department.get_department_list');
			$query = array('order' => 'n_family,n_given');
			$options['reports_to']		=	array('' => lang('nobody')) + $this->bo_staff->get_staff_list($query);
			$options['status']		=	Staff::get_status_list();

			// Some checks to make sure there's data
			if(count($options['department_id']) == 0) {
				$this->messages[] = array(lang('No departments found.  Create Departments in Human Resources -> Departments.'), 'warning');
			}
			if($GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_payroll')) {
				$this->get_payroll_options($staff, $options);
			}

			$options['gender']		=	array(
				'1'	=>	lang('Male'),
				'2'	=>	lang('Female')
			);

			return $options;
		}

		/**
		*	Get readonly fields for displaying a Staff object
		*	@param staff Staff object to display
		*	@return array of fields to be readonly for eTemplate
		*/
		protected function edit_get_readonlys(Staff $staff) {
			$readonly = array();

			if(!$GLOBALS['egw']->acl->check('staff', EGW_ACL_EDIT, 'perp_human_resources')) {
				$readonly = array(
					'employee_number'	=>	true,
					'department_id'		=>	true,
					'reports_to'		=>	true,
					'job_description'	=>	true,
					'start_date'		=>	true,
					'end_date'		=>	true,
					'status'		=>	true,
					'authentication'	=>	true,

					'save'			=>	true,
				);
			}

			// Payroll
			if(!$GLOBALS['egw']->acl->check('staff_data', EGW_ACL_READ, 'perp_payroll') || !$GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_payroll')) {
				$readonly[$this->tabs]['pay'] = true;
			}
			if(!$GLOBALS['egw']->acl->check('staff_data', EGW_ACL_EDIT, 'perp_payroll')) {
				$pay = array(
					'earning_type'	=>	true,
					'pay_amount'	=>	true,
					'pay_unit'	=>	true,
					'payment_schedule'	=>	true,
					'work_shift'	=>	true
				);
				foreach($pay as $key => $value) {
					$readonly['pay[' . $key . ']'] = $value;
				}
			}

			// Skills
			$readonly['new_skills']['delete[]'] = true;
			if(!$GLOBALS['egw']->acl->check('staff', EGW_ACL_EDIT, 'perp_human_resources')) {
				$readonly['perp_human_resources.staff_skill'] = true;
			}

			return $readonly;
		}

		/**
		*	Display a template
		*/
		protected function display($title, $callback, $template, $data, $options, $readonly, $preserve) {
			if(count($this->messages) > 0) {
				$data['message'] .= ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = $title;
			$this->template->read($template);
			$this->template->exec($callback, $data, $options, $readonly, $preserve);
		}

		/**
		*	Handle an action on a staff person (from a list)
		*/
		protected function handle_action($action, $staff_id) {
			switch($action) {
				case 'staff_new':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_human_resources.ui_perp_staff.staff_new'
					));
					return true;
					break;
				case 'delete':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_human_resources.ui_perp_staff.delete',
						'staff_id'	=>	$staff_id
					));
					break;
				default:
					return false;
					break;
			}
		}

		/**
		*	Check a result, if it's not true or a number, add the result to the 
		*	message array
		*/
		protected function check_result($result, $type = 'warning') {
			if($result === true || is_numeric($result)) {
				return true;
			} elseif (trim($result) != '') {
				$this->messages[] = array($result, $type);
			}
			return false;
		}

		/**
		*	Set up payroll tab
		*/
		protected function get_payroll_data(Staff &$staff, &$data, &$options) {
			if(!$GLOBALS['egw']->acl->check('staff_data', EGW_ACL_READ, 'perp_payroll') && 
				$staff->get_staff_id() != $GLOBALS['egw_info']['user']['account_id']) {
				return;
			}

			$data['pay'] = $this->bo_payroll->get_staff_data($staff);

			// Pay period schedules
			$pay_schedule = $staff->get_payment_schedule();
			$data['pay']['payment_schedule'] = $pay_schedule['id'];
			$data['pay']['payment_schedule_start'] = $pay_schedule['start'];
			$data['pay']['payment_schedule_end'] = $pay_schedule['end'];
			$scheduled_time = ExecMethod2('perp_payroll.PayrollUtility.get_scheduled_time', $staff, $pay_schedule['start'], $pay_schedule['end']);

			$data['pay']['period_hours'] = $scheduled_time['seconds'] / 3600;
			$data['pay']['pay_frequency'] = ExecMethod2('perp_payroll.PayrollUtility.get_pay_frequency', $staff);
			$data['pay']['pay_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, ExecMethod2('perp_payroll.PayrollUtility.get_salaried_rate', $staff, null));

			$data['pay']['pay_frequency'] = Paystub::$pay_frequency[$data['pay']['pay_frequency']];
			$work_shift = $staff->get_work_schedule();
			foreach($work_shift as $shift) {
				$data['pay']['work_shift'][] = $shift['id'];
			}

			// Pay stub entries
			$query = array(
				'order'		=>	'sort_order',
				'template'	=>	false,
				'num_rows'	=>	-1,
				'staff_id'	=>	$staff->get_staff_id()
			);
			$this->bo_payroll->get_line_types($query, $paystub_entries, $readonlys);
			foreach($paystub_entries as $key => &$entry) {
				$obj = PayStubEntryType::factory($entry['type_id']);
				if(!($obj instanceof PayStubEntryType)) {
					unset($data['pay']['paystub_entries'][$key]);
				}
				$entry['default_value'] = array(
					'value'	=>	$entry['default_value']
				);
				list($entry['class']) = explode('Type', get_class($obj));
				switch (get_class($obj)) {
					case 'DeductionType':
						$entry['class'] .= ': ' . DeductionType::$type_array[$obj->get_type()];
						break;
					case 'AccrualType':
						$entry['override_template'] = 'perp_payroll.AccrualValue.rate';
						break;
					case 'TotalType':
						$entry['on_paystub'] = true;
						break;
				}
				if($obj->get_calculation_name()) {
					$calculation = CalculationFactory::getCalculation($obj->get_calculation_name());
					$entry['template'] = $calculation->getTemplate();
					$calc_options = $calculation->getOptions();
					foreach($calc_options as $name => $type_options) {
						if(is_array($options)) {
							$options[$name] = $type_options;
						}
					}
					$entry['override_template'] = $calculation->getOverrideTemplate();
				} 
				$entry['override_value']['value'] = $obj->get_value($staff->get_staff_id());
				// See if this line is on the paystub
				$entry_staff = $obj->get_staff();
				foreach($entry_staff as $staff_entry) {
					if($staff_entry['staff_id'] == $staff->get_staff_id()) {
						$entry['on_paystub'] = true;
						break;
					}
				}
	
			}
			array_unshift($paystub_entries, false);
			$data['pay']['paystub_entries'] = $paystub_entries;
		}

		/**
		*	Get selectbox options for payroll tab
		*/
		protected function get_payroll_options(Staff &$staff, &$options) {
			// Payroll
			$options['type_id']	=	ExecMethod('perp_payroll.bo_perp_payroll.get_earning_types');
			$options['pay_unit']		=	Paystub::$pay_unit;

			// Pay period schedules
			$options['payment_schedule'] = bo_perp_payroll::get_pay_schedules();

			// Work shift schedules
			$config = config::read('perp_timeclock');
			$query = array(
				'cat_id'	=>	$config['work_shift_category'],
				'ignore_acl'	=>	true,
				'end'		=>	strtotime('+1 month', time())
			);
			$work_shifts	=	ExecMethod('calendar.calendar_bo.search', $query);
			foreach($work_shifts as $shift) {
				$options['work_shift'][$shift['id']] = $shift['title'];
			}

			// Some checks to make sure there's data
			if(count($options['type_id']) == 0) {
				$this->messages[] = array(lang('No earning types found.  You will not be able to pay anyone.  Create earning types in Admin -> Payroll.'), 'warning');
			}
			if(count($options['payment_schedule']) == 0) {
				$this->messages[] = array(lang('No payment schedules found.  You will have to generate paystubs manually, or create payment schedule in Admin -> Payroll -> Pay Schedules.'), 'warning');
			}
			if(count($options['work_shift']) == 0) {
				$this->messages[] = array(lang('No work shifts found.  You might want at least one.  Work shifts are a calendar event, in a particular category.  Set which category in Admin -> Timeclock.'), 'warning');
			}
		}

		protected function get_payroll_readonly(Staff &$staff, array &$data, array &$readonly) {
			if(is_array($data['pay']['paystub_entries'])) {
				foreach($data['pay']['paystub_entries'] as $key => $entry) {
					if($entry['class'] == 'Earning' || $entry['class'] == 'Total') {
						$readonly[$key . '[on_paystub]'] = true;
					}
				}
			} else {
				$this->messages[] = array(lang('No paystub entries found.  Payroll may not be set up correctly.'), 'warning');
			}
		}
	}
?>
