<?php

	require_once( 'class.so_perp_count.inc.php' );
	require_once( 'class.bo_perp_location.inc.php' );
	require_once( 'class.bo_perp_stock.inc.php' );
	require_once( 'class.bo_perp_inventory.inc.php' );

	class bo_perp_count {
		
		private $so = null;

		public function __construct() {
			$this->so = new so_perp_count();
		}
		
		public function get_count_list(&$query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('stock_count', EGW_ACL_READ, 'perp_inventory')) {
				if($query['template']) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if($query['filter']) {
				$query['status'] = $query['status'] ? $query['status'] : $query['filter'];
			}
			if(!$query['order']) {
				$query['order'] = 'count_date';
			}
			
			

			$rows = $this->so->get_count_list($query);			
			$delete = $GLOBALS['egw']->acl->check('stock_count', EGW_ACL_DELETE, 'perp_inventory');
			$rows['delete'] = $delete;

			return $query['count'];
		}

		/**
		*	Set up a new count, and create the minumum data we need
		*	
		*	@param info is an array with supervisor, date, count_by, locations & categories
		*	
		*/
		public function new_count(array $info) {

			if(!$GLOBALS['egw']->acl->check('stock_count', EGW_ACL_ADD, 'perp_inventory')) {
				return false;
			}

			if(!isset($info['locations'])) {
				return false;
			} elseif (!is_array($info['locations'])) {
				$info['locations'] = explode(',', $info['locations']);
			}

			if(!isset($info['categories'])) {
				return false;
			} elseif (!is_array($info['categories'])) {
				$info['categories'] = explode(',', $info['categories']);
			}

			$info['count_id'] = $this->so->set_count(array(
				'count_date'	=>	$info['count_date'],
				'supervisor'	=>	$info['supervisor'],
				'count_by'	=>	$info['count_by'],
			));

			// Create count sheets
			$this->create_sheets($info);

			return $info['count_id'];
		}

		/**
		*	Get a count instance
		*
		*	@param count_id
		*/
		public function get_count($count_id) {
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_READ, 'perp_inventory')) {
				return $this->so->get_count((int)$count_id);
			} else {
				return array();
			}
		}
			
		/**
		*	Set the count instance
		*	
		*	This is the top level, not the actual numbers of stock items
		*/
		public function set_count(array $data) {
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_EDIT, 'perp_inventory')) {
				if(!$data['supervisor']) {
					$data['supervisor'] = $GLOBALS['egw_info']['user']['account_id'];
				}
				return $this->so->set_count($data);
			} else {
				return false;
			}
		}

		public function get_sheet_list($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$query['order']) {
				$query['order'] = 'perp_stock_count_sheet.sheet_id';
			}
			$rows = $this->so->get_sheet_list($query);
			return $query['count'];
		}

		/**
		*	Get the info for a count sheet
		*/
		public function get_sheet($sheet_id) {
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_READ, 'perp_inventory')) {
				return $this->so->get_sheet($sheet_id);
			} else {
				return false;
			}
		}

		/**
		*	Set the info for a count sheet
		*/
		public function set_sheet(array $data) {
			if(!$data['count_id']) {
				return false;
			}
			
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_EDIT, 'perp_inventory')) {
				if(is_array($data['details'])) {
					foreach($data['details'] as &$detail) {
						if(is_null($detail['count']) || trim($detail['count']) === '') {
							$detail['count'] = null;
						}
						if(is_null($detail['unit_cost']) || $detail['unit_cost'] == 0) {
							$detail['unit_cost'] = null;
						}
					}
				}
				return $this->so->set_sheet($data);
			}
		}


		/**
		*	Get a list of serial numbers that are expected at a location
		*	provided by the sheet_id
		*
		*	@param sheet_id
		*	@param stock_id
		*	
		*	@return Array of serial numbers
		*/
		public function get_serial_numbers($sheet_id, $stock_id) {
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_READ, 'perp_inventory')) {
				return $this->so->get_serial_numbers((int)$sheet_id, (int)$stock_id);
			} else {
				return array();
			}
		}

		/**
		*	Set a list of serial numbers that are found at a location
		*	provided by the sheet_id.  
		*
		*	The list of serial numbers can contain serial numbers that were not
		*	expected to be found at the location.  In this case, the 'quantity_expected'
		*	value is 0, and the 'quantity_counted' value is > 0.
		*
		*	@param sheet_id 
		*	@param stock_id
		*	@param serials An array of serial number information, including serial_number and quantity counted
		*/
		public function set_serial_numbers($sheet_id, $stock_id, $serials) {
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_EDIT, 'perp_inventory')) {
				return $this->so->set_serial_numbers($sheet_id, $stock_id, $serials);
			} else {
				return false;
			}
		}
			
		/**
		*
		*/
		protected function create_sheets($info) {
			//set_time_limit(120);		
			$data = array(
				'count_id'	=>	$info['count_id'],
				'location_id'	=>	$info['locations'],
				'categories'	=>	$info['categories'],
				'count_date'	=>	$info['count_date'],
				'pre_fill'	=>	$info['pre_fill'],
				'include_items'	=>	$info['include_items'],
			);
			$this->so->create_sheets($data);		
		}
		
		/**
		*
		*/
		public function delete_count($count_id) {
			if($GLOBALS['egw']->acl->check('stock_count', EGW_ACL_DELETE, 'perp_inventory')) {
				return $this->so->delete_count($count_id);
			}
		}

		public function get_change_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('stock_count', EGW_ACL_READ, 'perp_inventory')) {
				$rows = array();
				return 0;
			}
			

			if(!$query['order']) {
				$query['order'] = 'stock_num';
			}

			$rows = $this->so->get_change_list($query);

			$category = array();

			// Needed for error messages
			if(!isset($query['options-location_id'])) {
				$query['options-location_id'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list');
			}
			foreach($rows as $key => &$row) {
				$gl_query['location_id'] = $row['location_id'];
				$gl_query['stock_id'] = $row['stock_id'];
				$adjustment_account = bo_perp_stock::get_gl_accounts($row['stock_id'], bo_perp_inventory::ADJUSTMENT);

				// If it's not completed, do some integrity checks
				if(!$row['finish_date']) {
					if($row['expected'] != $row['current']) {
						$row['row_message'] = lang('Inventory counts have changed by %1 since count started', ($row['current'] - $row['expected']));
					}
					if(!$adjustment_account) {
						// We can't continue with this error, set it in the query so make_changes() can find it
						$query['error'] = $key;

						$row['row_message'] = lang('Missing adjustment general ledger account for %1.  If you have permission, you can %3.',
							ExecMethod('perp_inventory.bo_perp_stock.link_title', $row['stock_id']),
							'',
							'<a href="' . $GLOBALS['egw']->link('/index.php', array(
								'menuaction'	=>	'perp_inventory.ui_perp_stock.stock_edit',
								'stock_id'	=>	$row['stock_id'],
								'tab'		=>	'financial'
							)) . '">' . lang('fix it') . '</a>'
						);
					}
				}
			}
			return $query['count'];
		}

		/**
		*	Make the changes needed to bring the stock levels into agreement
		*	with the counted numbers.
		*
		*	@param count_id The ID of the stock count to apply
		*	@param changes Optional array of changes to make
		*	
		*	@return true or error message
		*/
		public function make_changes($count_id, $changes = null) {
			if(!$GLOBALS['egw']->acl->check('stock_count', EGW_ACL_EDIT, 'perp_inventory')) {
				return lang('You do not have permission to make these changes');
			}
			
			$count = $this->get_count($count_id);

			if($count['finish_date']) {
				// Already done
				return lang('Changes have already been made.');
			}

			if(!$changes) {
				$query = array(
					'count_id'	=>	$count_id,
					'num_rows'	=>	-1
				);
				$changes = array();
				$this->get_change_list($query, $changes);
			}

			// Check for errors
			if($query['error']) {
				return lang('Error in changes: %1',
					$changes[$query['error']]['row_message']
				);
			}
			return $this->so->make_changes($count_id, $changes);

		}
	}
?>
