<?php
	/**
	*	Record damage done to controlled (serialized / batch) goods while in our control.
	*
	*	You can record what the damage was, how many (for batched, 1 for serialized) were
	*	damaged, and set a discount off sale price because of the damage.  You can link
	*	additional info, such as infologs or pictures.
	*
	*	If you want to write off damaged goods, you should record the damage, then 
	*	move the correct quantity to the garbage.
	*
	*	@author Nathan Gray
	*	@package perp_inventory
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.so_perp_damage.inc.php');

	class bo_perp_damage {
		
		public static function get_damage_list(&$query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('damage', EGW_ACL_READ, 'perp_inventory')) {
				$rows = array();
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'perp_stock_damage.stock_history_id';
			}

			$rows = so_perp_damage::get_damage_list($query);

			return $query['total'];
		}

		/**
		*	Read a record of damage done to some quantity of a work order.
		*
		*	@param damage_id
		*
		*	@return array of information about the damage.  Does not include links.
		*/
		public static function get_damage($damage_id) {
			if(!$GLOBALS['egw']->acl->check('damage', EGW_ACL_READ, 'perp_inventory')) {
				return array();
			} else {
				return so_perp_damage::get_damage($damage_id);
			}
		}
		
		/**
		*	Record damage done to some quantity of a work order.
		*
		*	@param data array of information containing stock_id, serial_number, quantity_damatged, description, discount_percent
		*
		*	@return damage_id
		*/
		public static function set_damage(array $data) {
			if((!$damage_id) && $GLOBALS['egw']->acl->check('damage', EGW_ACL_ADD, 'perp_inventory') || 
					($damage_id && $GLOBALS['egw']->acl-check('damage', EGW_ACL_EDIT, 'perp_inventory'))) {

				if(!$data['timestamp']) {
					$data['timestamp'] = time();
				}
				$damage_id = so_perp_damage::set_damage($data);
				
				// Link uploaded pictures
				if($data['link'] && is_array($data['link']['to_id'])) {
					$link_id = 'DAMAGE-' . $damage_id;
					egw_link::link('perp_inventory', $link_id, $data['link']['to_id']);
				}

				return $damage_id;
			} else {
				// No permission
				return false;
			}
		}

		/**
		*       Participate in linking
		*/
		public static function link_title($entry) {
			list($damage, $entry) = explode('-', $entry);
			if(!$entry) {
				$entry = $damage;
			}
			$damage = self::get_damage($entry);
			return lang('Damage') . ': ' . $damage['comment'];
		}
	}

?>
