<?php
	include_once(EGW_INCLUDE_ROOT.'/perp_inventory/inc/class.bo_perp_stock.inc.php');

	class bo_perp_inventory {

		/**
		*	Constants for dealing with the purpose of a general ledger account
		*
		*	These will be used by other areas to get the appropriate general ledger 
		*	account for the stock item, based on what is being done with it.
		*
		*	@{
		*/
		const ON_HAND		=	1;
		const WORK_IN_PROGRESS	=	2;
		const SHRINKAGE		=	3; // Also known as write-off
		const ADJUSTMENT	=	4;

		public static $account_uses = array(
			self::ON_HAND		=>	'On hand',
			self::WORK_IN_PROGRESS	=>	'Work in progress',
			self::SHRINKAGE		=>	'Shrinkage / write-off',
			self::ADJUSTMENT	=>	'Adjustment'
		);

		/**
		*	@}
		*/

		private static $so;
		private $stock_id_cache;

		public function bo_perp_inventory() {
			$this->so = CreateObject('perp_inventory.so_perp_inventory');
		}

		public function get_inventory($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('inventory', EGW_ACL_READ, 'perp_inventory')) {
				$rows = array();
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'stock_num';
			}
			$rows = $this->so->get_inventory($query);

			return $query['count'];
		}

		public function get_controlled_inventory_list($query = array(), &$rows = array(), &$readonly = array()) {
			// TODO: ACL
			if(!isset($query['template'])) {
				$query['num_rows'] = -1;
			}
			if(!isset($query['order'])) {
				$query['order'] = 'serial_number';
			}
			if($query['filter']) {
				$query['location'] = $query['filter'];
			}
			if($query['filter2']) {
				$query['stock_id'] = $query['filter2'];
			}

			// Make it so this method can be called static
			if(!is_object(self::$so)) {
				self::$so = new so_perp_inventory();
			}
			$rows = self::$so->get_controlled_inventory_list($query);
			foreach($rows as $key => &$row) {
				$readonly['history[' . $row['serial_number'] . ']'] = !($row['work_order_id']);
			}
			return $query['count'];
		}

		public function reserve_stock($location, $stock_id, $quantity, $reference) {
			if($location) {
				return $this->so->reserve_stock($location, $stock_id, $quantity, $reference);
			} else {
				throw new egw_exception_wrong_parameter('Missing location id');
			}
		}

		/**
		*	Receive stock into inventory
		*
		*	@param stock_id int - The id of the stock to be received
		*	@param location int - The location to receive the stock (it goes into on_hand)
		*	@param quantity float - Quantity received
		*	@param stock_cost_id int - the id of the stock_cost entry for costing
		*	@param serials array - if the stock is controlled, an array of serial numbers of the form serial => quantity 
		*	@param reference String - Text reference for the source of the stock (Shipment, stock count, etc)
		*	@param move_type int - one of the stock movement types provided by get_stock_move_types.  Default (3) is normal goods received.
		*/
		public function receive($stock_id, $location, $quantity, $stock_cost_id, $serials = '', $reference = '', $move_type = 3) {
			$receive = $GLOBALS['egw']->acl->check('receive', EGW_ACL_READ, 'perp_ap') || $GLOBALS['egw']->acl->check('receive', EGW_ACL_READ, 'perp_inventory');

			if($receive) {
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
				if($stock['controlled']) {
					if(!is_array($serials)) {
						return lang('Missing serial number for controlled stock');
					}
					foreach($serials as $serial => $qty) {
						$this->so->add_serial($stock_id, $serial, $qty, $location, $stock_cost_id);
					}
				}
						
				return $this->so->receive($stock_id, $location, $quantity, $stock_cost_id, $reference);
			} else {
				return lang('Access is not allowed');
			}
		}


		public function ship($stock, $location, $quantity, $serials = array(), $shipment_id = null) {
			$ship = $GLOBALS['egw']->acl->check('ship', EGW_ACL_EDIT, 'perp_ar') || $GLOBALS['egw']->acl->check('ship', EGW_ACL_EDIT, 'perp_inventory');
			if($ship) {
				return $this->so->ship($stock, $location, $quantity, $serials, $shipment_id);
			} else {
				return False;
			}
		}

		/**
		*	Add a serial number for an item already in inventory.  
		*	This just records the serial number so it can get a history, it doesn't change inventory counts.
		*	You should probably use receive() for most cases.
		*
		*	@param stock_id int - The id of the stock you are recording the serial number for
		*	@param serial string - The serial or batch number to record
		*	@param quantity float - 1 for serialized items, > 1 for batched
		*	@param location int - The id of the location the item is currently at.
		*	@param stock_cost_id int - The record of cost when this item was received
		*/
		public static function add_serial($stock_id, $serial, $quantity, $location, $stock_cost_id = null) {
			so_perp_inventory::add_serial($stock_id, trim($serial), $quantity, $location, $stock_cost_id);
		}

		public function get_movable_stock($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('inventory', EGW_ACL_EDIT, 'perp_inventory')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!isset($query['order'])) {
				$query['order'] = 'source.stock_id';
			}

			$rows = $this->so->get_movable_stock($query);
			
			if(!isset($query['template'])) {
				// TODO:
			} else {
				return $query['count'];
			}
		}

		/**
		*	Move stock from one location to another
		*
		*	@param int from From location id
		*	@param mixed from From work center id or one of 'on_hand', 'reserved', 'garbage'
		*	@param int to To location id
		*	@param mixed to To work center id or one of 'on_hand', 'reserved', 'garbage'
		*	@param numeric quantity number to move
		*	@param array optional array of serial numbers in the form (serial => quantity)
		*/
		public function move($from, $from_wc, $to, $to_wc, $stock_id, $quantity, $serial_numbers = array()) {
			if(!$stock_id) {
				return lang('Missing stock number');
			}
			if($GLOBALS['egw']->acl->check('inventory', EGW_ACL_EDIT, 'perp_inventory')) {
				if($from_wc == 'garbage') {
					// Found some stock
					$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
					$cost = $this->get_cost($stock_id, $to, $quantity);
					$stock_cost_id = $this->create($stock_id, $quantity, $to, $cost, $narrative = lang('moved from garbage'));
					return true;
				} elseif ($to_wc == 'garbage') {
					// Lost some stock
					$this->reserve_stock($from, $stock_id, $quantity, lang('To be garbaged'));
					$this->consume($stock_id, 'garbaged', $from, $quantity, lang('Garbaged'), $serial_numbers);
					return true;
				} else {
					return $this->so->move($from, $from_wc, $to, $to_wc, $stock_id, $quantity, $serial_numbers);
				}
			}
		}

		/*
		*	Some stock was used / garbaged / eaten / returned
		*	Removes from inventory, adjusts GL accounts.
		*
		*	Takes from RESERVED stock.
		*	The inventory is one-sided, you should deal with the other side (where
		*	it went) on your own.  The GL transaction is complete.
		*	$consumption is one of 'used', 'garbaged', 'adjustment'
		*	Returns the cost of the stock consumed
		*/
		public function consume($stock_id, $consumption, $location, $quantity, $reference, $serials, $work_in_progress_stock_id = null) {
			//TODO: ACLs
			$cost = $this->so->consume($stock_id, $consumption, $location, $quantity, $reference, $serials, $work_in_progress_stock_id);

			// Check stock, if it's discontinued and out of stock, retire it.
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			if($stock['status_flag'] == 'D') {
				$query = array(
					'stock_id'	=>	$stock_id,
					'num_rows'	=>	-1,
				);
				$this->get_inventory($query, $locations);
				$out_of_stock = true;
				foreach($locations as $location) {
					$out_of_stock = $out_of_stock && ($location['quantity_in_stock'] == 0);
				}
				if($out_of_stock) {
					$stock['status_flag'] = 'R';
					ExecMethod2('perp_inventory.bo_perp_stock.set_stock_item', $stock_id, $stock);
				}
			}

			return $cost;
		}

		public function create($stock_id, $quantity, $location, $value, $narrative = '') {
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			$stock_accounts = bo_perp_stock::get_gl_accounts($stock_id);

			$stock_cost_id = $this->so->add_stock_cost(
				$stock_id,
				$quantity,
				$value
			);

			$this->so->receive($stock_id, $location, $quantity, $stock_cost_id, $narrative, 4);

			$config = config::read('perp_manufacturing');
			$trans_type = $config['creation_transaction_type'];

			$trans = CreateObject('perp_ledger.GLTransaction');
			
			$trans->set_transaction_type($trans_type);
			$trans->add_line($stock_accounts[bo_perp_inventory::WORK_IN_PROGRESS], $stock['stock_num'] . ' ' . lang('created') . ($narrative ? ' ' . $narrative : ''), -$value);
			$trans->add_line($stock_accounts[bo_perp_inventory::ON_HAND], $stock['stock_num'] . ' ' . lang('created') . ($narrative ? ' ' . $narrative : ''), $value);
			$trans->write();

			return $stock_cost_id;
		}

		public function add_stock_cost($stock_id, $quantity, $direct_cost, $indirect_cost = 0) {
			// TODO: ACL
			if($quantity != 0) {
				return $this->so->add_stock_cost($stock_id, $quantity, $direct_cost, $indirect_cost);
			} else {
				return False;
			}
		}

		/**
		*	Get the cost of a certain number of stock.
		*
		*	This is the actual cost right now of the goods in stock.  If the goods are not in stock,
		*	the supplier's catalog price is used instead.  The cost returned is in the
		*	General Ledger currency.
		*	Raw & assembly stock should both work, as long as there is inventory.  Assembly stock
		*	will not work if there's no inventory because there is no other way to figure out costs.
		*	Pseudo stock won't work, because it is never in inventory, and never purchased.
		*	
		*
		*	@param	int	$stock_id - the id of the stock
		*	@param	int	$location_id - the id of the location
		*	@param	double	$quantity - the quantity needed
		*
		*	@return	double	Cost 
		*/
		public function get_cost($stock_id, $location_id, $quantity) {
			$config = config::read('perp_inventory');

			// Check for in stock
			$in_stock = ExecMethod2('perp_inventory.bo_perp_stock.get_inventory', $stock_id, $location_id);

			if($in_stock['total'] <= 0) {
				// Not in stock, check for supplier pricing
				$suppliers = ExecMethod('perp_inventory.bo_perp_stock.get_suppliers', $stock_id);
				if(count($suppliers)) {
					return $quantity * ($suppliers[0]['price'] / $suppliers[0]['conversion_factor']);
				}
			}

			// FIFO only works if there's stock on hand
			if($config['costing'] == 'fifo') {
				return $this->so->fifo_cost($stock_id, $location_id, $quantity, false);
			}
		}

		/**
		*	Get an estimate for the unit cost of a stock item.
		*	
		*	This is an estimate of how much it would cost for a given stock item, on a certain date.
		*	If the date is today, the estimate should be pretty close to get_cost(), but this function
		*	does not take into account FIFO costing, it works on guessing.
		*
		*	Raw and assembly stock get their costing from actual costs, as recorded on the nearest (previous)
		*	date in the stock_cost table.  Pseudo raw stock return 0, pseudo assembly stock, and real assemblies
		*	with no stock_cost history use the BOM for the specified date.
		*
		*	If possible, the original invoice currency is used, otherwise the general ledger currency is used.
		*	No correction is done for varying exchange rates.
		*
		*	@param stock stock_id, or array of stock item information
		*	@param date timestamp
		*	@param original_currency If true, returns an array of costs for each currency.  If false, the cost in the GL currency
		*		is returned.
		*
		*	@return an array of currency_id => cost
		*/
		public function get_cost_estimate($stock, $date, $original_currency = true) {
			if(!is_array($stock)) {
				if((int)$stock) {
					$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock);
				}
			}
			if(!isset($stock['stock_id'])) {
				return false;
			}
			
			//cache the cost to be returned, if a call for the same stock_id is made then returned the cached cost
			if($this->stock_id_cache[$stock['stock_id']]){
				return $this->stock_id_cache[$stock['stock_id']];
			}	
			
			// Get a list of currencies
			$currencies = ExecMethod('perp_api.bo_perp_api.get_currencies');
			$config = config::read('perp_ledger');

			// Default
			if($original_currency) {
				$cost = array(
					$config['gl_currency']	=>	array(
						'cost'	=>	0,
						'date'	=>	strtotime('today')
					)
				);
			} else {
				$cost = 0;
			}

			// Pseudo raw stock has no cost.  Easy.
			if($stock['stock_type_flag'] == 'R' && $stock['pseudo']) {
				$this->stock_id_cache[$stock['stock_id']] = $cost;
				return $cost;
			}

			$bo_stock = new bo_perp_stock();

			// Look for historical data on raw parts
			if (!$stock['pseudo'] && $stock['stock_type_flag'] == 'R') {
				$query = array(
					'stock_id'	=>	$stock['stock_id'],
					'as_of'		=>	$date,
					'order'		=>	'date',
					'sort'		=>	'DESC'
				);
				$bo_stock->get_cost_details($query, $cost_history);
				if($cost_history[0] && $cost_history[0]['shipment_id']) {
					if($original_currency) {
						/**
						* Get the shipment line, so we can get the order line, 
						* so we can get the invoice line, so we can get the 
						* original currency and exchange, so we can get the 
						* unit cost for this item.
						*/
						$shipment =& CreateObject('perp_ap.IncomingShipment', $cost_history[0]['shipment_id']);
						$line =& $shipment->get_line($cost_history[0]['line_no']);
						if(is_object($line)) {
							$order_line =& $line->get_shippable_item();
							$status_list =& $order_line->get_status();
						
							// Got details, get invoice
							foreach($status_list as $info) {
								if($info['type'] == 'INVOICE') {
									$invoice = CreateObject('perp_api.Invoice', $info['reference_id']);
									$cost[$invoice->get_invoice_currency()] = array(
										'cost'	=>	bcmul($invoice->get_exchange_rate(), bcadd($cost_history[0]['direct_cost'],$cost_history[0]['indirect_cost'],3),3),
										'date'	=>	$cost_history[0]['date']
									);
									$this->stock_id_cache[$stock['stock_id']] = $cost;
									return $cost;
								}
							}
						}
					} else {
						$cost = bcadd($cost_history[0]['direct_cost'],$cost_history[0]['indirect_cost'],3);
						$this->stock_id_cache[$stock['stock_id']] = $cost;
						return $cost;
					}
				}

				// Never been received & invoiced, look in the catalog.  This is the current price.
				$suppliers = ExecMethod('perp_inventory.bo_perp_stock.get_suppliers', $stock['stock_id']);
				if(count($suppliers) && $suppliers[0]) {
					$catalog_cost = bcdiv($suppliers[0]['price'], $suppliers[0]['conversion_factor'], 3);
					if($original_currency) {
						$cost[$suppliers[0]['supplier_currency']] = array(
							'cost'	=>	$catalog_cost,
							'date'	=>	strtotime('today')
						);
						$this->stock_id_cache[$stock['stock_id']] = $cost;
						return $cost;
					} else {
						$currencies = array();
						$query = array(
							'currency_id'   =>      $this->invoice_currency,
							'template'      =>      false,
							'num_rows'      =>      1
						);
						if(!is_object($GLOBALS['perp_api']['bo'])) {
							$GLOBALS['perp_api']['bo'] =& CreateObject('perp_api.bo_perp_api');
						}
						$GLOBALS['perp_api']['bo']->get_currencies(
							$query,
							$currencies
						);
						if($currencies[0]) {
							$cost = bcmul($currencies[0]['exchange_rate'], $catalog_cost, 3);
							$this->stock_id_cache[$stock['stock_id']] = $cost;
							return $cost;
						}
					}
				}
			} 

			/**
			*	Now we look for costs on assembly parts.  We can't use the historical
			*	actual recorded cost in our currency,
			*	because we want to have currencies seperate.
			*/
			if($stock['stock_type_flag'] !== 'R') {
				$bom_query = array(
					'stock_id'      =>      $stock['stock_id'],
					'as_of_date'    =>      $date
				);
				$bo_bom = CreateObject('perp_manufacturing.bo_perp_bill_of_material');
				$bo_bom->get_bill_of_material_list($bom_query, $bom_list);

				if(count($bom_list) > 0 && isset($bom_list[0])) {
					$bom = $bo_bom->get_bill_of_material($bom_list[0]['bill_of_material_id']);
				} else {
					$this->stock_id_cache[$stock['stock_id']] = $cost;
					return $cost;
				}
				foreach($bom['details'] as $detail) {
					$component_cost = $this->get_cost_estimate($detail['component'], $date, $original_currency);
					if(is_array($component_cost)) {
						foreach($component_cost as $currency => $costs) {
							$cost[$currency]['cost'] = bcadd( $cost[$currency]['cost'], bcmul($detail['quantity'], $costs['cost'], 3), 3 );
							$cost[$currency]['date'] = max($cost[$currency]['date'], $costs['date']);
						}
					} elseif (!$original_currency) {
						$cost = bcadd( $cost, bcmul($detail['quantity'], $component_cost, 3), 3);
					}
				}
				$this->stock_id_cache[$stock['stock_id']] = $cost;
				return $cost;
			}
			$this->stock_id_cache[$stock['stock_id']] = $cost;
			return $cost;
		}


		/** 
		*	get title for part of perp_inventory for linking
		*
		*	Location is:
		*		STOCK for stock items
		*		LOCATION for production locations
		*		SERIAL for a particular serialized stock item
		*
		*	@param string/array $entry string <location>:<id> or array with data
		*	@return string with title or false if not valid
		*/
		public function link_title($entry) {
			if(!is_array($entry)) {
				list($location, $id) = explode('-', $entry);
				
				if($id) {
					switch($location) {
						case 'STOCK':
							return ExecMethod('perp_inventory.bo_perp_stock.link_title', $id);
							break;
						case 'LOCATION':
							return ExecMethod('perp_inventory.bo_perp_location.link_title', $id);
							break;
						case 'SERIAL':
							return ExecMethod('perp_inventory.bo_perp_stock_history.link_title', $id);
							break;
						case 'DAMAGE':
							return ExecMethod('perp_inventory.bo_perp_damage.link_title', $id);
							break;
					}
				}
			} else {
				if(isset($entry['stock_id'])) {
					return ExecMethod('perp_inventory.bo_perp_stock.link_title', $entry['stock_id']);
				} elseif (isset($entry['location_id'])) {
					return ExecMethod('perp_inventory.bo_perp_location.link_title', $entry['location_id']);
				}
			}
		}

		/**
		*	Query perp_inventory for entries matching the given pattern
		*
		*	Called as hook to participate in linking
		*	@param string $pattern pattern to search
		*	@return array with key => title pairs of entries
		*/
		public function link_query($pattern) {
			$list = array();
			$list += ExecMethod('perp_inventory.bo_perp_stock.link_query', $pattern);
			$list += ExecMethod('perp_inventory.bo_perp_location.link_query', $pattern);

			return $list;
		}

		/**
		*	Functions available for XML-RPC
		*	@param type xmlrpc or soap
		*/
		public function list_methods($type) {
			if(is_array($type)) {
				$type = $type['type'] ? $type['type'] : $type[0];
			}
			switch($type) {
				case 'xmlrpc':
				default:
					$xml_functions = array(
						'list_methods'  =>      array(
							'function'      =>      'list_methods',
							'signature'     =>      array(array(xmlrpcStruct, xmlrpcString)),
							'docstring'     =>      lang('Read this list of methods')
						),
						'move'     =>      array(
							'function'      =>      'xml_rpc_move',
							'signature'     =>      array(
//								array(xmlrpcStruct, xmlrpcInt, xmlrpcString, xmlrpcInt, xmlrpcString, xmlrpcInt, xmlrpcDouble),
								array(xmlrpcStruct, xmlrpcStruct)
							),
							'docstring'     =>      lang('Get a list of stock locations')
						),
					);
					return $xml_functions;
					break;
			}
		}

		/**
		*	Move for xml-rpc, since eGroupWare's xmlrpc server only passes 1 parameter
		*	@param parameters Array of parameters to be passed to move()
		*/
		public function xml_rpc_move(array $parameters) {
			return $this->move($parameters[0], $parameters[1], $parameters[2], $parameters[3], $parameters[4], $parameters[5], $parameters[6]);
		}

		/**
		*	Get stock movement types
		*
		*/
		public function get_stock_move_types() {
			return array(
				1	=>	lang('moved'),
				2	=>	lang('reserved'),

				3	=>	lang('received'),
				4	=>	lang('created'),

				-1	=>	lang('shipped'),
				-2	=>	lang('consumed'),
				-3	=>	lang('garbaged'),
				-4	=>	lang('adjustment'),
			);
		}

		/**
		*	Get stock movement records
		*
		*	@nextmatch
		*/
		public function get_stock_movement(&$query, &$rows=array(), &$readonly=array()) {
			// ACL check
			if(!$GLOBALS['egw']->acl->check('inventory', EGW_ACL_READ, 'perp_inventory')) {
				$rows = array();
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'timestamp';
				$query['sort'] = 'DESC';
			}
			if(array_key_exists('filter', $query)) {
				$query['stock_id'] = $query['filter'];
			}
			if($query['col_filter']) {
				foreach($query['col_filter'] as $name => $value) {
					$query[$name] = $value;
				}
			}

			$rows =& $this->so->get_stock_movement($query);
			return $query['total'];
		}
	}
?>
