<?php

	class bo_perp_location {
	
		private $so;

		/**
		*	Logical locations that don't map to a physical
		*	location.  These are used for serialized goods.
		*/
		public static $pseudo_locations = array(
			-1	=>	'shipped out',
			-2	=>	'used',
			-3	=>	'garbaged'
		);

		public function bo_perp_location() {
			$this->so = CreateObject('perp_inventory.so_perp_location');
		}

		public function get_location_list($query = array(), &$rows = array(), &$readonly = array()) {
			$read = $GLOBALS['egw']->acl->check('location', EGW_ACL_READ, 'perp_inventory');
			if(!$read) {
				if(!$query['template']) {
					// For select box
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!$query['order']) {
				$query['order'] = 'location_id';
			}

			$list = $this->so->get_locations($query);

			if(!isset($query['template']) && !$query['get_all_data']) {
				foreach($list as $location) {
					$rows[$location['location_id']] = $location['indented_name'];
				}
				return $rows;
			}

			// Slice instead of restricting the query because of nesting
			if(isset($query['num_rows']) && $query['num_rows'] > 0) {
				$rows = array_slice($list, $query['start'], $query['num_rows']);
			} else {
				$rows = $list;
			}

			$delete = !$GLOBALS['egw']->acl->check('location', EGW_ACL_DELETE, 'perp_inventory');
			foreach($rows as &$row) {
				$readonly['delete'][ $row['location_id']  ] = $delete;
				$readonly['delete[' . $row['location_id'] .']' ] = $delete;
			}

			if($query['get_all_data']) {
				$query['rows'] =& $rows;
				return $query;
			} else {
				return $query['count'];
			}
		}

		public function get_location($location_id) {
			$read = $GLOBALS['egw']->acl->check('location', EGW_ACL_READ, 'perp_inventory');
			if(!$read) {
				return array();
			}

			$query = array(
				'location_id'	=>	$location_id,
				'num_rows'	=>	1,
				'template'	=>	false
			);
			$count = $this->get_location_list($query, $locations);
			if($count == 1) {
				$location = $locations[0];
			} else {
				return array();
			}
		
			return $location;
		}

		public function set_location($location_id, $location) {
			$edit = $GLOBALS['egw']->acl->check('location', EGW_ACL_EDIT, 'perp_inventory');

			if(!$edit) {
				return lang('Not allowed to edit locations');
			}
			
			if($this->so->check_location_name($location['location_name']) > 0 && !$location['location_id']) {
				return lang('Location name already exists. Please enter a new one.');
			}

			// Check if the delivery address is already used
			$query = array(
				'order'			=>	'location_id',
				'delivery_address'	=>	$location['delivery_address']
			);
			$list = $this->so->get_locations($query);
			if($location['delivery_address'] && count($list) > 0) {
				foreach($list as $duplicate_address) {
					if($location_id != $duplicate_address['location_id']) {
						return lang('Delivery address is already used by %1', $duplicate_address['location_name']);
					}
				}
			}
			
			// Inventory
			if(count($location['inventory']['rows']) > 0) {
				foreach($location['inventory']['rows'] as $row) {
					$this->set_inventory($location_id, $row['stock_id'], $row['min_on_hand'], $row['max_on_hand']);
				}
			}

			if(isset($location['accounts']['rows'])) {
				foreach($location['accounts']['rows'] as $accounts) {
					if(is_numeric($accounts)) {
						$this->set_gl_accounts($location_id, $accounts);
					}
				}
			}
			
			return $this->so->set_location($location_id, $location);
		}

		/** 
		*	Disables or deletes a stock location
		*	Disabled if it has children 
		*	Deleted if it has no children and no inventory
		*
		*/
		public function delete_location($location_id) {
			if(!$GLOBALS['egw']->acl->check('location', EGW_ACL_DELETE, 'perp_inventory')) {
				return lang('Access not allowed');
			}
			
			// Check location
			$query = array(
				'location_parent'	=>	$location_id,
				'template'		=>	false,
			);
			$children = $this->get_location_list($query);
			if($children > 0) {
				$return = lang('Cannot delete location, it has sub-locations');
			}

			$query = array('location_id' => $location_id);
			$inventory = array();
			$this->get_inventory($query, $inventory);
			$no_inventory = true;
			foreach($inventory as $stock_item) {
				$no_inventory &= ($stock_item['quantity_on_hand'] == 0);
				$no_inventory &= ($stock_item['quantity_reserved'] == 0);		
			}
			if(!$no_inventory) {
				$return = lang('Cannot delete location, it has inventory');
			}
			$really_delete = $no_inventory && $children == 0;
				
			$this->so->delete_location($location_id, $really_delete);
			return $return;
		}

		public function get_inventory($query = array(), &$rows = array(), &$readonly = array()) {
			$read = $GLOBALS['egw']->acl->check('location', EGW_ACL_READ, 'perp_inventory');
			$read = $read && $GLOBALS['egw']->acl->check('inventory', EGW_ACL_READ, 'perp_inventory');
			$edit = $GLOBALS['egw']->acl->check('location', EGW_ACL_EDIT, 'perp_inventory') && $GLOBALS['egw']->acl->check('inventory', EGW_ACL_EDIT, 'perp_inventory');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			if(!$query['order']) {
				$query['order'] = 'stock_num';
			}
			$list = $this->so->get_inventory($query);

			$rows = $list;
			foreach($rows as $key => $row) {
				$readonlys[$key . '[minimum_on_hand]'] = !$edit;
			}
			return $query['count'];
		}

		public function set_inventory($location, $stock_id, $min_on_hand, $max_on_hand) {
			// TODO: ACL
			$this->so->set_inventory($location, $stock_id, $min_on_hand, $max_on_hand);
		}
		
		public function get_gl_accounts($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('account', EGW_ACL_READ, 'perp_ledger')) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			if(!isset($query['location_id'])) {
				return false;
			}

			$count = $this->so->get_gl_accounts($query, $rows, $readonlys);

			if(!isset($query['template'])) {
				unset($query['start']);
				unset($query['num_rows']);

				/**
				*	Returns array, not for selectboxes
				*/
				if($count == 1) {
					$rows = $rows[0];
					if((!$rows['stock_gl_account'] || !$rows['work_in_progress_gl_account'] || !$rows['shrinkage_gl_account']) &&
							isset($query['stock_category_id']) && isset($query['location_id'])) {
						// We really need to find gl accounts, there's production at stake
						// Try parent category
						$category = bo_perp_stock_category::get_category($query['stock_category_id']);
						$stock_category = $query['stock_category_id'];
						$query['stock_category_id'] = $category[0]['parent'];
						$parent_rows = array();
						$this->get_gl_accounts($query, $parent_rows);
						if(!isset($parent_rows['stock_gl_account']) && count($parent_rows) > 1) {
							$parent_rows = $parent_rows[0];
						}
						$this->merge_gl_accounts($rows, $parent_rows);

						// Try parent location
						$query['stock_category_id'] = $stock_category;
						$location = ExecMethod('perp_inventory.bo_perp_location.get_location', $query['location_id']);
						if($location['location_parent']) {
							$query['location_id'] = $location['location_parent'];
							$parent_rows = array();
							$this->get_gl_accounts($query, $parent_rows);
							if(!isset($parent_rows['stock_gl_account']) && count($parent_rows) > 1) {
								$parent_rows = $parent_rows[0];
							}
							$this->merge_gl_accounts($rows, $parent_rows);
						}

					}
				}
				return $rows;
			} else {
				$count = $this->so->get_gl_accounts($query, $rows, $readonlys);
			}
			return $count;
		}

		public function set_gl_accounts($location_id, $accounts) {
			if($GLOBALS['egw']->acl->check('location', EGW_ACL_EDIT, 'perp_inventory')) {
				return $this->so->set_gl_accounts($location_id, $accounts);
			}
		}

		public function get_location_status() {
			return array(
				0	=>	lang('disabled location'),
				1	=>	lang('current location')
			);
		}

		/**
		* get title for an perp location entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param int/array $entry int location_id or array with location entry
		* @param string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				return $this->so->link_title($entry);
			}
			if (!$entry) {
				return $entry;
			}
			return $entry['location_name'];
		}

		
		/**
		* query perp locations for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param string $pattern pattern to search
		* @return array with location_id - title pairs of the matching entries
		*/
		function link_query( $pattern ) {		

			$list = $this->so->link_query($pattern);
			
			$result = array();
			foreach($list as $list_el) {
				$result['LOCATION-'.$list_el['location_id']] = $this->link_title($list_el);
			}
						
			return $result;
		}
		

		/**
		*	Merge $new gl accounts into $base
		*	Keep $base if != 0, else set to $new
		*/
		private function merge_gl_accounts(&$base, $new) {
			if(!$base['stock_gl_account']) {
				$base['stock_gl_account'] = $new['stock_gl_account'];
			}
			if(!$base['work_in_progress_gl_account']) {
				$base['work_in_progress_gl_account'] = $new['work_in_progress_gl_account'];
			}
			if(!$base['shrinkage_gl_account']) {
				$base['shrinkage_gl_account'] = $new['shrinkage_gl_account'];
			}
		}

		/**
		*	List XML-RPC methods available
		*	@param type String, either xmlrpc or soap
		*/
		public function list_methods($type) {
			if(is_array($type)) {
				$type = $type['type'] ? $type['type'] : $type[0];
			}
			switch($type) {
				case 'xmlrpc':
				default:
					$xml_functions = array(
						'list_methods'	=>	array(
							'function'	=>	'list_methods',
							'signature'	=>	array(array(xmlrpcStruct, xmlrpcString)),
							'docstring'	=>	lang('Read this list of methods')
						),
						'get_location_list'	=>	array(
							'function'	=>	'get_location_list',
							'signature'	=>	array(array(xmlrpcStruct, xmlrpcStruct)),
							'docstring'	=>	lang('Get a list of stock locations')
						),
					);
					return $xml_functions;
					break;
			}
		}
	}
?>
