<?php
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_order.inc.php');

	class bo_perp_stock {

		private $so;
		private $config;

		/**
		*	Settings for how we control serial numbers
		*	This setting is stored in the key 'serial_setup'.  Values greater than 0 refer
		*	to a pERP generated numbering scheme set in perp_manufacturing.
		*
		*	@{
		*/
		const SERIAL_ALWAYS	 = -1;
		const SERIAL_WHEN_NEEDED = -2;
		/**
		*	@}
		*/

		/**
		*	Return values from check_serial()
		*	@{
		*/
		const NOT_CONTROLLED = 0;
		const NOT_VALID = 1;
		const FORMAT_OK = 2;
		const SERIAL_OK = 3;
		/**
		*	@}
		*/
		
		/**
		*	API object to handle tracking changes
		*/
		private $history_log;

		/**
		*	A mapping of stock item field names to history codes
		*
		*	This array defines the fields we want to keep history on.
		*	history_log uses two character key codes, so we map the 
		*	field names to the codes
		*/
		public $field2history = array(
			'stock_num'		=>	'SC',
			'short_description'	=>	'sd',
			'stock_category'	=>	'Ca',
			'long_description'	=>	'ld',
			'unit_of_measure'	=>	'um',
			'decimal_places'	=>	'dp',
			'status_flag'		=>	'st',
			'stock_type_flag'	=>	'ty',
			'pseudo'		=>	'Ps',
			'controlled'		=>	'cn',
			'serial_setup'		=>	'sl',
			'bar_code'		=>	'bc',

			'country_of_origin'	=>	'co',
			'international_code'	=>	'IC',
			'canadian_inbound_code'	=>	'CI',
			'canadian_returning_code'	=>	'CR',
			'usa_inbound_code'	=>	'UI',
			'usa_returning_code'	=>	'UR',
			'ship_notes'		=>	'SN',
		);

		/**
		*	A mapping of history log codes to human labels
		*/
		public $history2field = array(
			'SC'	=>	'stock code',
			'sd'	=>	'description',
			'Ca'	=>	'stock category',
			'ld'	=>	'long description',
			'um'	=>	'unit of measure',
			'dp'	=>	'decimal places',
			'st'	=>	'status',
			'ty'	=>	'stock type',
			'Ps'	=>	'pseudo',
			'cn'	=>	'controlled',
			'sl'	=>	'serial setup',
			'bc'	=>	'bar code',

			'co'	=>	'country of origin',
			'IC'	=>	'international',
			'CI'	=>	'canadian inbound',
			'CR'	=>	'canadian returning',
			'UI'	=>	'USA Inbound',
			'UR'	=>	'USA returning',
			'SN'	=>	'Shipping notes',

			// Suppliers
			'sp'	=>	'supplier',
			'ps'	=>	'preferred supplier',

			// Build Options - dealt with in perp_manufacturing
			'Bv'	=>	'build option allowed',
			'Bc'	=>	'build option component',
			'Bw'	=>	'build option work center',

			// Sales
			'sc'	=>	'sales category',
		);

		/**
		*	A mapping of history log codes to the widgets used for display
		*	
		*	Some more are set in the constructor, where we're allowed to call functions
		*/
		public $history_widgets = array(
			// Status 
			'st'	=>	null, 			// Set in get_history_widgets()

			// Unit of measure
			'um'	=>	null, 			// Set in get_history_widgets()

			// Stock type
			'ty'	=>	null, 			// Set in get_history_widgets()

			// Stock category
			'Ca'	=>	'select-cat',

			// Country of origin
			'co'	=>	'select-country',

			// Pseudo
			'Ps'	=>	array(
				0	=>	'no',
				1	=>	'yes'
			),

			// Serialized
			'sl'	=>	array(), 			// Set in get_history_widgets()

			// Controlled
			'cn'	=>	array(
				0	=>	'no',
				1	=>	'yes'
			),

			// Supplier
			'sp'	=>	array(), 			// Set in get_history_widgets()
			// Preferred supplier
			'ps'	=>	array(), 			// Set in get_history_widgets()

			// Build Options
			'Bw'	=>	array(), 			// Set in get_history_widgets()

			// Sales category
			'sc'	=>	array(), 			// Set in get_history_widgets()
		);


		public function bo_perp_stock() {
			$this->so = CreateObject('perp_inventory.so_perp_stock');
			$this->config = config::read('perp_inventory');

			// Keeping history of changes
			$this->history_log =& CreateObject('phpgwapi.historylog', 'perp_inventory');
		}

		/**
		*	stock list
		*	@nextmatch
		*	query:
		*		filter	=>	stock_category
		*		filter2	=>	status
		*		search	=>	searches stock_num & short_description
		*/
		public function get_stock_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!isset($query['template']) && !isset($query['num_rows'])) {
				// Get all rows for a select box
				$query['num_rows'] = -1;
			}
			if(!isset($query['selectcols'])) {
				$query['selectcols'] = 'stock_id,stock_num,short_description';
			}

			$list = $this->so->get_stock_list($query);

			$edit = $GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory');
			$delete = $GLOBALS['egw']->acl->check('stock', EGW_ACL_DELETE, 'perp_inventory');

			if(!isset($query['template']) && !$query['get_all_data']) {
				$rows = array();
				foreach($list as $row) {
					$rows[$row['stock_id']] = $row['stock_num'] . ' - ' . $row['short_description'];
				}
				return $rows;
			}
			$rows = $list;

			$rows['edit'] = $edit;
			$rows['delete'] = $delete;
			if($query['get_all_data']) {
				$query['rows'] =& $rows;
				return $query;
			} else {
				return $query['count'];
			}
		}

		public function get_stock_item($stock_id) {
			$read = $GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory');
			if($read) {
				$stock_item = $this->so->get_stock_item($stock_id);

				// Get change timestamps
				$history = array('add', 'modify', 'delete');
				foreach($history as $action) {
					$stock_item['history'][$action] = $GLOBALS['egw']->contenthistory->getTSforAction('perp_inventory:STOCK', $stock_id, $action);
				}
				return $stock_item;
			} else {
				return array();
			}
		}

		public function set_stock_item($stock_id, &$stock_item) {

			// Check permission on stock item AND prices, user could have permission to change prices
			$edit = $GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory');
			if(!$edit && !$GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				return lang('Permission denied');
			}

			// Pricing
			if($stock_id && $stock_item['pricing'] && $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				$result = $this->set_stock_prices($stock_id, $stock_item['pricing']);
				if($result !== true) {
					$error = $result;
				}
				if(!$edit) {
					return $result;
				}
			}

			$stock_item['stock_num'] = trim($stock_item['stock_num']);
			$stock_item['short_description'] = trim($stock_item['short_description']);

			if(!$stock_item['stock_num']) {
				return lang('Missing stock number');
			} elseif (!$stock_item['stock_category']) {
				return lang('Missing stock category');
			} elseif (!$stock_item['short_description']) {
				return lang('Missing description');
			}

			// Automatically update stock number?
			if($this->config['allow_stock_number_changes'] == 1) {
				$stock_item['stock_num'] = $this->suggest_stock_number($stock_item);
			}

			// Check for duplicate stock number
			if(!$stock_id) {
				$query = array(
					'search'	=>	$stock_item['stock_num'],
					'selectcols'	=>	'stock_num'
				);
				$stock_list = $this->so->get_stock_list($query);
				if(count($stock_list)) {
					foreach($stock_list as $item) {
						if($item['stock_num'] == $stock_item['stock_num']) {
							$stock_item['stock_num'] = $this->suggest_stock_number($stock_item);
							return lang('Stock code already exists.  Please enter a new one. (%1)', $stock_item['stock_num']);
						}
					}
				}
			}

			if(!$stock_item['status_flag']) {
				$stock_item['status_flag'] = $this->config['default_stock_status'];
			}
			if(!$stock_item['stock_type_flag']) {
				$stock_item['stock_type_flag'] = $this->config['default_stock_type'];
			}

			// Kits must be pseudo, they'll be exploded into individual components later
			if($stock_item['stock_type_flag'] == 'K') {
				$stock_item['pseudo'] = True;
			}

			// Shipping (packages)
			if($stock_id) {
				$this->set_shipping_packages($stock_id, $stock_item);
			}


			// Preferred supplier
			if($stock_item['suppliers']['preferred'] != $stock_item['suppliers']['old_preferred']) {
				$suppliers = $this->get_suppliers($stock_id);
				// Log change
				if($stock_id) {
					$this->history_log->add('ps', $stock_id, $stock_item['suppliers']['preferred'], $stock_item['suppliers']['old_preferred']);
				}
				foreach($suppliers as $supplier) {
					if($supplier['supplier_id'] == $stock_item['suppliers']['preferred']) {
						$catalog_item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $supplier['supplier_id'], $supplier['item_code']);
						$catalog_item['preferred'] = True;
						ExecMethod2('perp_ap.bo_perp_catalog.edit_catalog', $supplier['supplier_id'], $supplier['item_code'], $catalog_item);
					} elseif($supplier['supplier_id'] == $stock_item['suppliers']['old_preferred']){
						$catalog_item = ExecMethod2('perp_ap.bo_perp_catalog.get_catalog_item', $supplier['supplier_id'], $supplier['item_code']);
						$catalog_item['preferred'] = false;
						ExecMethod2('perp_ap.bo_perp_catalog.edit_catalog', $supplier['supplier_id'], $supplier['item_code'], $catalog_item);
					}
				}
			}

			// Bill of material is not editable here

			// Options
			if($stock_item['options']) {
				foreach($stock_item['options']['rows'] as $option) {
					if(is_array($option)) {
						if(is_array($option['component'])) {
							// Component needs to be an int
							// Search part to avoid keeping the old value if the user is trying to clear the field
							if($option['component']['value'] && $option['component']['search']) {
								$option['component'] = (int)$option['component']['value'];
							} else {
								unset($option['component']);
							}
						}
						$option['effective_from'] = $stock_item['options_edit']['effective_from'];
						$result = ExecMethod2('perp_manufacturing.bo_perp_build_option.set_allowed_build_option', $stock_id, $option);
						if($result !== true) {
							// Something went wrong
							return $result;
						}
					}
				}
			}

			// Workflow
			if($GLOBALS['egw_info']['apps']['perp_manufacturing']) {
				$config = config::read('perp_manufacturing');
				if($config['use_workflow']) {
					$result = ExecMethod2('perp_manufacturing.bo_perp_workflow.set_process_id', $stock_item['stock_id'], $stock_item['workflow_process']);
					if(is_array($result)) {
						$message = array();
						foreach($result as $error) {
							$message[] = array($error, 'error');
						}
						return $message;
					}
				}
			}

			// Financial
			self::set_gl_accounts($stock_id, $stock_item['general_ledger_accounts']);

			// Inventory

			// Taxes
			if(isset($stock_item['taxes'])) {
				$this->set_tax_categories($stock_id, $stock_item['taxes']);
			}

			$action = $stock_id ? 'modify' : 'add';

			// History on stock item itself
			if($stock_id) {
				$old_item = $this->get_stock_item($stock_id);
				$change = array_intersect_key(array_diff_assoc($stock_item, $old_item), $this->field2history);
				foreach($change as $field => $value) {
					$this->history_log->add($this->field2history[$field], $stock_id, $stock_item[$field], $old_item[$field]);
				}

				if($old_item['stock_category'] != $stock_item['stock_category']) {
					$this->so->move_stock_value($stock_id, $old_item['stock_category'], $stock_item['stock_category']);
				}
			}
			$stock_id = $this->so->set_stock_item($stock_id, $stock_item);
			
			// Serial Patterns
			if($stock_item['controlled_settings']['serial_setup'] == self::SERIAL_WHEN_NEEDED) {
				if($stock_item['patterns']){
					$this->set_serial_patterns($stock_id, $stock_item['patterns']);	
				}
			}

			// Barcodes
			if($stock_item['barcodes']) {
				$this->set_barcodes($stock_id, $stock_item['barcodes']);
			}

			// Update timestamp
			$GLOBALS['egw']->contenthistory->updateTimeStamp('perp_inventory:STOCK', $stock_id, $action, time());

			return $error ? $error : $stock_id;
		}

		/**
		*	Set the price for a stock item on [several] price list
		*
		*	@param stock_id
		*	@param pricing An array of price information, the most important are the price_list_id and price for each price list,
		*		as well as an effective date when the new prices start.
		*/
		public function set_stock_prices($stock_id, $pricing) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				return false;
			}

			$effective_date = $pricing['effective_date'];
			unset($pricing['effective_date']);
			foreach($pricing as $price) {
				if(!is_array($price)) {
					continue;
				}
				$result = ExecMethod2('perp_orders.bo_perp_price.set_price', $price['price_list_id'], $stock_id, $price['price'], $effective_date);
				if($result !== true) {
					$error = $result;
				}
			}
			return $error;
		}

		public function delete($stock_id) {
			if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_DELETE, 'perp_inventory')) {
				return lang('Access is not allowed');
			}
			$GLOBALS['egw']->contenthistory->updateTimeStamp('perp_inventory:STOCK', $stock_id, 'delete', time());
			return $this->so->delete($stock_id);
		}

		/** 
		*	stock_clone
		*	Creates a copy of the given stock item and returns the new stock_id
		*	suggest_stock_number() is used to create a new stock_num
		*
		*	@param int stock_id - the stock item to clone
		*	@return int stock_id - the new stock item
		*/
		public function stock_clone($stock_id) {
			$clone = $this->get_stock_item($stock_id);
			if(!is_array($clone)) {
				return lang('Invalid stock item');
			}
			unset($clone['stock_id']);
			unset($clone['stock_num']);
			unset($clone['bar_code']);
			$clone['stock_num'] = $this->suggest_stock_number($clone);
			$clone['short_description'] = lang('%1 copy', $clone['short_description']);
			$clone_id = $this->set_stock_item(null, $clone);
			if(!is_numeric($clone_id)) {
				// Error trying to save;
				return $clone_id;
			}
			$clone['stock_id'] = $clone_id;

			// Can't copy suppliers, because we need to know the suppliers's code for the new item

			// Don't copy links, but create a link from the new to the old
			$id_1 = 'STOCK-' . $stock_id;
			$id_2 = 'STOCK-' . $clone_id;
			$link = egw_link::link('perp_inventory', $id_1, 'perp_inventory', $id_2, lang('Copied from %1', $this->link_title($stock_id)));

			// Clone Bill of Material
			if($clone['stock_type_flag'] != 'R') {
				$bom_id = $this->get_current_bom($stock_id);
				$clone_bom_id = ExecMethod2('perp_manufacturing.bo_perp_bill_of_material.clone_bill_of_material', $bom_id, $clone_id);
			}

			// Clone options
			$bo_option = CreateObject('perp_manufacturing.bo_perp_build_option');
			if($bo_option) {
				$allowed_build_option_list_query['stock_id'] = $stock_id;
				$clone['options']['rows'] = $bo_option->get_allowed_build_option_list($allowed_build_option_list_query);
			}

			// Clone Taxes
			$clone['taxes'] = $this->get_tax_categories($stock_id);

			// Clone Sales categories
			$clone['sales_categories'] = $this->get_sales_categories($stock_id);

			// Clone Pricing
			$clone['pricing'] = $this->get_pricing($stock_id);
			
			// Option pricing
			$query = array(
				'bill_of_material_id'	=>	$clone_bom_id,
				'stock_id'		=>	$clone_id,
				'hide_supplier'		=>	True,
			);
			$this->get_option_price_list($query, $clone['option_price']);

			$this->set_stock_item($clone_id, $clone);
			return $clone_id;
		}

		public function get_suppliers($stock_id, $include_orders = false) {
			$read = $GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory');
			$read = $read && $GLOBALS['egw']->acl->check('supplier', EGW_ACL_READ, 'perp_ap');
			if(!$read) {
				return array();
			} else {
				$list = $this->so->get_suppliers($stock_id);
				if($include_orders) {
					// Required because api checks permissions on current app, which will fail for perp_inventory
					$temp = $GLOBALS['egw_info']['flags']['currentapp'];
					$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ap';

					$query = array(
						'stock_id'	=>	$stock_id,
						'status'	=>	'not_received'
					);
					$bo_orders = new bo_perp_order();
					foreach($list as &$supplier) {
						$query['supplier_id'] = $supplier['supplier_id'];
						$query['supplier_stock_code'] = $supplier['item_code'];
						$bo_orders->get_order_list($query, $supplier['on_order']);
						if(is_array($supplier['on_order'])) {
							foreach($supplier['on_order'] as &$order) {
								$order['total_ordered'] *= $supplier['conversion_factor'];
								$order['our_unit_of_measure'] = $supplier['our_unit_of_measure'];
							}
						}
					}
					$GLOBALS['egw_info']['flags']['currentapp'] = $temp;
				}
				return $list;
			}
		}

		public function get_inventory($stock_id, $location_id = '') {
			$read = $GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory');
			if(!$read) {
				return array();
			}
			return $this->so->get_inventory($stock_id, $location_id);
		}

		// Returns id of the current BoM
		public function get_current_bom($stock_id, $effective_date = null) {
			if(!$stock_id) {
				return null;
			}
			$query = array(
				'stock_id'	=>	$stock_id,
				'filter2'	=>	'current'
			);
			if($effective_date) {
				unset($query['filter2']);
				$query['as_of_date'] = $effective_date;
			}
			$bom = array();
			$bo_bom = CreateObject('perp_manufacturing.bo_perp_bill_of_material');
			$bo_bom->get_bill_of_material_list($query, $bom);
			if(count($bom)) {
				return $bom[0]['bill_of_material_id'];
			} else {
				return null;
			}
		}

		// These are hard coded because they have significance in the code
		public function get_status_flags() {
			return array(
				'E'	=>	lang('experimental'),
				'P'	=>	lang('in production'),
				'D'	=>	lang('discontinued'),
				'R'	=>	lang('retired'),
			);
		}

		/**
		* Changes the status of a stock item and updates:
		*	disables BOM for the stock item
		*	Creates a matching BOM for the replacement
		*	Creates matching options for replacement
		*
		*	@param stock_id ID of the stock_id to change the status of
		*	@param old_status String Current status of the stock item
		*	@param new_status String New status of the stock item
		*	@param replacement Optional ID of the stock item replacing this one
		*/
		public function change_status($stock_id, $old_status, $new_status, $replacement) {
			// Update status
			if($old_status == $new_status) {
				return false;
			}

			$stock = $this->get_stock_item($stock_id);
			$stock['status_flag'] = $new_status;

			// Create a link to replacement
			if($replacement) {
				$original_key = 'STOCK-' . $stock_id;
				$replacement_key = 'STOCK-' . $replacement;
				$link = egw_link::link('perp_inventory', $original_key, 'perp_inventory', $replacement_key, 
					lang('%1 was replaced by %2', $this->link_title($stock_id), $this->link_title($replacement)));
			}
			$this->set_stock_item($stock_id, $stock);

			// Disable BOMs
			$result = ExecMethod2('perp_manufacturing.bo_perp_bill_of_material.disable_stock', $stock_id, $replacement);

			// Create matching options for replacement
			$allowed_build_option_list_query['stock_id'] = $stock_id;
			$options = ExecMethod('perp_manufacturing.bo_perp_build_option.get_allowed_build_option_list', $allowed_build_option_list_query);
			foreach($options as $option) {
				ExecMethod2('perp_manufacturing.bo_perp_build_option.set_allowed_build_option', $replacement, $option);
			}

			// Disable and replace any options that use this stock item
			$count = ExecMethod2('perp_manufacturing.bo_perp_build_option.disable_stock', $stock_id, $replacement);
		}

		// These are hard coded because they have significance in the code
		public function get_stock_types() {
			return array(
				'R'	=>	lang('raw'),
				'A'	=>	lang('assembly'),
				'K'	=>	lang('stock_kit'),
			//	'P'	=>	lang('package')
			);
		}

		/**
		*	Get the next serial number for the given settings.  Data not given will be 0 or blank,
		*	if used in the serial number.
		*
		*	@param stock_id		The ID of the stock item
		*	@param work_order_id	The ID of the work order building the stock item
		*	@param detail_id	The ID of the detail line of the work order.  1 is the default.
		*	@param consume		Use this serial number, making it unavailable for any other item.
		*
		*	@return A number uniquely (depending on settings) identifying either a single serialized item,
		*		or a group of batched items.
		*/
		public function get_next_serial($stock_id, $work_order_id = false, $detail_id = 1, $consume = false) {
			$stock_item = $this->get_stock_item($stock_id);
			return ExecMethod2('perp_manufacturing.so_perp_manufacturing.get_next_serial',
				$stock_item['serial_setup'],
				$stock_id,
				$work_order_id,
				$detail_id,
				$consume
			);
		}

		public function make_serial($stock_id, $work_order = '', $detail_id = '', $quantity = 1, $location_id = '', $stock_cost_id) {
			return $this->so->make_serial($stock_id, $work_order, $detail_id, $quantity, $location_id, $stock_cost_id);
		}

		public function get_cost_summary($stock_id, $start_date = '', $end_date = '') {
			// TODO: ACLs
			if($stock_id) {
				return $this->so->get_cost_summary($stock_id, $start_date, $end_date);
			}
		}
	
		public function get_cost_details($query = array(), &$rows = array(), &$readonlys = array()) {
			// TODO: ACLs
			if(!$query['order']) {
				$query['order'] = 'date';
			}
			$rows = $this->so->get_cost_details($query);
			$count = $query['count'];
			$rows['date'] = True;
			$rows['edit'] = $GLOBALS['egw']->acl->check('receive', EGW_ACL_EDIT, 'perp_ap') && $query['stock_type_flag'] == 'R';

			return $count;
		}

		public function set_cost_details($stock_id, $details) {
			// TODO: ACLS
			foreach($details as $detail) {
				$this->so->set_cost_details($stock_id, $detail);
			}
		}

		public function add_sales_category($stock_id, $category) {
			if(!$category) {
				return lang('Missing category to add');
			}
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory') || $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				$this->so->add_sales_category($stock_id, $category);
				$this->history_log->add('sc', $stock_id, $category, 'added');
			}
		}

		public function delete_sales_category($stock_id, $category) {
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory')) {
				$this->so->delete_sales_category($stock_id, $category);
				$this->history_log->add('sc', $stock_id, $category, 'removed');
			}
		}

		public function get_tax_categories($stock_id) {
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
				return $this->so->get_tax_categories($stock_id);
			}
		}

		/**
		*	Set a tax category for a tax authority
		*	For example, Computers in Canada are Luxuries (and taxed at 6%)
		*
		*	@param int $stock_id	Stock_id of the stock item 
		*	@param int/array $tax_authority_id	Tax authorities to set the category for or (tax_authority_id => $tax_category_id) mapping
		*	@param int	$tax_category_id (optional)
		*/
		public function set_tax_categories($stock_id, $tax_authority_id, $tax_category_id='') {
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory')) {
				if(!is_array($tax_authority_id)) {
					return $this->so->set_tax_category($stock_id, $tax_authority_id, $tax_category_id);
				} else {
					foreach($tax_authority_id as $tax_auth_id => $tax_category_id) {
						if(is_array($tax_category_id)) {
							$tax_category_id = current($tax_category_id);
						}
						$this->so->set_tax_category($stock_id, $tax_auth_id, $tax_category_id);
					}
				}
			}
		}

		public function get_sales_categories($stock_id) {
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
				return $this->so->get_sales_categories($stock_id);
			}
		}

		public function get_pricing($stock_id, $as_of_date = null) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				$bo_price = CreateObject('perp_orders.bo_perp_price');
				$query = array(
					'stock_id'	=>	$stock_id,
					'template'	=>	false,
					'date'		=>	is_null($as_of_date) ? time() : $as_of_date,
					'format'	=>	true,
					'include_not_allowed'	=>	true,
					'include_options'	=>	false,
				);
				$bo_price->get_price_list($query, $list);
				foreach($list as &$row) {
					if(!is_array($row)) {
						continue;
					}
					$price_list = ExecMethod('perp_orders.bo_perp_price_list.get_price_list', $row['price_list_id']);
					$row = array_merge($row, $price_list);
				}
				return $list;
			} else {
				return array();
			}
		}

		public function get_option_price_list($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			
			$price_list = ExecMethod('perp_orders.bo_perp_price_list.get_price_list', $query['option_price_list']);
			if($price_list) {
				$rows = ExecMethod2('perp_orders.bo_perp_price_list.get_stock_price_list', 
					$query['stock_id'], 
					$price_list['sales_category_id'], 
					$price_list['sales_area_id'], 
					$price_list['sales_type_id'],
					$query['date']
				);
			}
			return count($rows['options']);
		}

		public function get_full_option_list($stock_id, $bom_id = '') {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				return $this->so->get_full_option_list($stock_id, $bom_id);
			}
		}

		public function get_shipping_packages($stock_id, $work_order_id = null, $detail_id = null) {
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
				$packages = $this->so->get_shipping_packages($stock_id);

				// Check for options that ship separately
				if(!is_null($work_order_id) && !is_null($detail_id)) {

					// Options that ship separately
					$query = array(
						'ships_separate'	=>	true,
						'by_category'		=>	true,
						'option_category'	=>	array(),
					);
					$bo_work_order = CreateObject('perp_manufacturing.bo_perp_work_order');
					$bo_options = CreateObject('perp_manufacturing.bo_perp_build_option');
					$separate_options = array();
					$bo_options->get_build_option_category_list($query, $separate_options);

					// Get build options
					foreach($separate_options as $option_category_id => $name) {
						$option = $bo_work_order->get_work_order_detail_option($work_order_id, $detail_id, $option_category_id);
						if(is_array($option) && $option['component']) {
							$packages = array_merge($packages, $this->so->get_shipping_packages($option['component']));
						}
					}
				}
				return $packages;
			} else {
				return array();
			}
		}

		public function set_shipping_packages($stock_id, $stock) {
			if(!isset($stock['package_count']) && !$GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory')) {
				return;
			}

			if(!is_array($stock['packages'])) {
				$stock['packages'] = array();
			}

			// Make sure there's 1 line for every package there's supposed to be
			$packages = array_pad($stock['packages'], $stock['package_count'], 
				array(
					'unit_of_measure'	=>	1,
					'length'		=>	1,
					'width'			=>	1,
					'height'		=>	1,
					'mass'			=>	0.1,
				)
			);

			$this->so->set_shipping_packages($stock_id, $packages);
		}

		public function get_price_list_list($stock_id, $date = '') {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders') &&
				is_numeric($stock_id)) {
				if(!$date) {
					$date = time();
				}
				return $this->so->get_price_list_list($stock_id, $date);
			}
		}
		
		/**
		*	Generates a list of viewable build options for a particular stock id
		*	
		*	@todo MUST ADD EFFECTIVE DATES
		*/
		public function stock_edit_options_view($query = array(), &$rows = array(), &$readonlys = array()) {
			$bo_option = CreateObject('perp_manufacturing.bo_perp_build_option');
			
			if($bo_option) {
				
				if(strtolower($query['filter2']) == 'current'){
					$query['build_option_category'] = $query['filter'];
					$rows = $bo_option->get_allowed_build_option_list($query);
					$count = count($rows);
				} else{
					$count = $bo_option->get_build_option_time_period_list($query, $rows, $readonlys);
				}
			}
			return $count;
		}
		
		
		/**
		*	Generates a list of editable build options for a particular stock id
		*
		*	
		*/
		public function stock_edit_options_edit($query = array(), &$rows = array(), &$readonlys = array()) {
			
			$bo_option = CreateObject('perp_manufacturing.bo_perp_build_option');			
			$rows = array();
			$count = $bo_option->get_build_option_list($query, $rows);
	
			if(is_array($rows)) {
				$readonlys[] = '';
				foreach($rows as $key => &$option) {
					if($option['build_option_id']) {
						$option += $bo_option->get_allowed_build_option($query['stock_id'], $option['build_option_id'], $query['effective_from']);
						$option['component'] = array(
							'stock_id'	=>	$option['component'],
							'filter2'	=>	'E,P',
						);
					} else {
						unset($rows[$key]);
					}
					if($query['effective_from'] < mktime(0,0,0, date('m'), date('d'), date('Y'))){
                                                $readonlys[] = true;
                                        }
				}
			}
			return $count;			
		}
		
		/**
		*	Get title for a stock item identified by $entry
		*	Called as hook to participate in linking
		*
		*	@param int/array $entry stock_id or array with stock item
		*	@return string Title or false
		*/
		public function link_title($entry) {
			if(!is_array($entry)) {
				$entry = $this->get_stock_item($entry);
			}

			$title = $entry['stock_num'] . ' - ' . $entry['short_description'];
			egw_link::set_cache('perp_inventory', 'STOCK-' . $entry['stock_id'], $title);

			return $title;
		}

		/**
		*	Query stock items for entries matching $pattern
		*
		*	Is called as a hook to participate in linking
		*	@param pattern string Pattern to search
		*	@return array with link key => title pairs of matching entries
		*/
		public function link_query($pattern) {
			$query = array(
				'search'	=>	$pattern,
				'selectcols'	=>	'stock_num,short_description'
			);
			$list = $this->get_stock_list($query);
			$result = array();
			foreach($list as $stock_id => $stock_name) {
				$result['STOCK-' . $stock_id] = $stock_name;
			}
			return $result;
		}

		/** 
		*	Suggest a stock number
		*	Formatting comes from Site Configuration.  Possible values:
		*		%1$d	Stock ID
		*		%2$d	Category ID
		*		%3$s	Category Name
		*		%4$s	Stock Status Code - blank for production or E, R, or D
		*		%5$s	Stock Status Code - E, P, R, or D
		*	All the sprintf() formatting specifiers are allowed
		*
		*	Some examples:
		*		%5$s%1$d		= E1, P5658, D7
		*		%2$d-%1$d		= 5-1, 19-5659
		*		%3$.3s-%1$05s%4$'-2s 	= CAL-031351, DEL-00001-E, IBM-00186-D
		*
		*	@param stock_item array containing (category_id, status_flag)
		*	
		*	@return string the suggested stock number
		*/
		public function suggest_stock_number($stock_item) {
			$stock_num = $this->so->suggest_stock_number($stock_item);
			$this->get_stock_list(array('search' => $stock_num), $duplicates);
			foreach($duplicates as $stock) {
				if($stock['stock_num'] == $stock_num) {
					// Found a duplicate
					$stock_num .= '-' . substr(date('U'), -2);
					break;
				}
			}
			return $stock_num;
		}

		/**
		*	List XML-RPC methods available
		*	@param type String, either xmlrpc or soap
		*/
		public function list_methods($type) {
			if(is_array($type)) {
				$type = $type['type'] ? $type['type'] : $type[0];
			}
			switch($type) {
				case 'xmlrpc':
				default:
					$xml_functions = array(
						'list_methods' => array(
							'function'  => 'list_methods',
							'signature' => array(array(xmlrpcStruct,xmlrpcString)),
							'docstring' => lang('Read this list of methods.')
						),
						'get_stock_list'	=>	array(
							'function'	=>	'get_stock_list',
							'signature'	=>	array(array(xmlrpcStruct,xmlrpcStruct)),
							'docstring'	=>	lang('Get a list of stock items')
						),
						'get_stock_item'	=>	array(
							'function'	=>	'get_stock_item',
							'signature'	=>	array(array(xmlrpcStruct,xmlrpcInt)),
							'docstring'	=>	lang('Get a stock item')
						),
						'get_barcodes'	=>	array(
							'function'	=>	'get_barcodes',
							'signature'	=>	array(array(xmlrpcStruct,xmlrpcInt)),
							'docstring'	=>	lang('Get the barcodes for a stock item')
						),
					);
					return $xml_functions;
					break;
			}
		}

		/**
		*	Get an array of data for a history log widget
		*
		*	This is done as a function instead of a static array
		*	because some of the things the history log tracks are 
		*	suppliers and assemblies, which tend to be huge.  The
		*	history_log widget wants a list of all possible values,
		*	which causes memory errors.
		*
		*	@param stock_id ID of the stock item 
		*	@return array suitable for a history_log widget
		*/
		public function get_history_widgets($stock_id) {
			$stock = $this->get_stock_item($stock_id);
			$history = $this->history_log->search($stock_id);
			
			$this->history_widgets['um'] =& ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			$this->history_widgets['st'] =& $this->get_status_flags();
			$this->history_widgets['ty'] =& $this->get_stock_types();
			$this->history_widgets['sl'] = array(0 => lang('Number not generated'));
			if($GLOBALS['egw_info']['apps']['perp_manufacturing']) {
				$this->history_widgets['sl'] += ExecMethod('perp_manufacturing.bo_perp_manufacturing.get_serial_setup');
			}

			foreach($history as $history_record) {
				switch ($history_record['status']) {
				case 'ps':
				case 'sp':
					if(is_numeric($history_record['new_value'])) {
						$this->history_widgets[$history_record['status']][$history_record['new_value']] = ExecMethod('perp_ap.bo_perp_supplier.link_title', $history_record['new_value']);
					}
					if(is_numeric($history_record['old_value'])) {
						$this->history_widgets[$history_record['status']][$history_record['old_value']] = ExecMethod('perp_ap.bo_perp_supplier.link_title', $history_record['old_value']);
					}
					break;
				case 'Bc':
					$this->history_widgets['Bc'][$history_record['new_value']] =& $this->link_title($history_record['new_value']);
					$this->history_widgets['Bc'][$history_record['old_value']] =& $this->link_title($history_record['old_value']);
					
					break;

				}
			}
			$this->history_widgets['sp'] += array(
				'added'		=>	lang('added'),
				'removed'	=>	lang('removed')
			);
			$this->history_widgets['sc'] = array(
				'added'		=>	lang('added'),
				'removed'	=>	lang('removed')
			) + ExecMethod('perp_orders.bo_perp_orders.get_sales_categories');
			$this->history_widgets['Bw'] = array('' => lang('Any')) + ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center_list');
			$this->history_widgets['Bv'] =& ExecMethod('perp_manufacturing.bo_perp_build_option.get_allowed_option_value_list');

			return $this->history_widgets;
		}

		/**
                *       Check for valid serial
                *
		*       @param stock_id
		*	@param serial_number
                */
                public static function check_serial($stock_id, $serial_number) {
			if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
                                return array();
                        } else {
				if($stock_id && $serial_number) {
                        		$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
					
					if($stock_item['stock_id']) {
                        			if($stock_item['controlled'] == 0) {
							return self::NOT_CONTROLLED;
						} elseif($stock_item['serial_setup'] == self::SERIAL_ALWAYS || $stock_item['serial_setup'] >= 0) {
						
							return (so_perp_stock::match_serial($stock_id, $serial_number))?self::SERIAL_OK:self::NOT_VALID;

						}elseif($stock_item['serial_setup'] == self::SERIAL_WHEN_NEEDED) {
                                			$regular_expressions = so_perp_stock::get_serial_patterns($stock_id);
                                			foreach($regular_expressions as $regular_expression ){
                                        			if (preg_match('/^'.$regular_expression['pattern'].'$/', $serial_number)) {
                                                			$matched = true;
                                        			}
                                			}	

                                			return ($matched)?self::FORMAT_OK:self::NOT_VALID;

						} else {
							return self::NOT_VALID;
						}
					}else {
						return array();
					}	
                        	
				}else{
                                	return array();
                        	}	
                        }
			
                }		

		/**
		*	Get a list of patterns for a stock item
		*
		*	@param stock_id
		*
		*	@return array with one sub-array per pattern.
		*/
		public function get_serial_patterns($stock_id) {
			if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
				return array();
			} else {
				$patterns = $this->so->get_serial_patterns($stock_id);
				foreach($patterns as &$pattern){
					$pattern['pattern'] = str_replace($this->so->regex_legend, $this->so->pattern_legend, $pattern['pattern']);
				}
				return $patterns;
			}
		}

		/**
                *       Set the defined patterns for the stock item
                *
                *       @param stock_id The ID of the stock item
                *       @param patterns An array of patterns for the stock item  
                */
                public function set_serial_patterns($stock_id, $patterns) {
                        if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory')) {
                                return;
                        }
			$this->so->set_serial_patterns($stock_id, $patterns);
                }		


		/**
		*	Get a list of barcodes for a stock item
		*
		*	@param stock_id
		*
		*	@return array with one sub-array per barcode.  Each sub array has the keys barcode, encoding,
		*		and preferred, which indicates which barcode should be used when printing & referencing
		*/
		public function get_barcodes($stock_id) {
			if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
				return array();
			} else {
				return $this->so->get_barcodes($stock_id);
			}
		}

		/**
		*	Set the recognised barcodes for a stock item
		*
		*	@param stock_id The ID of the stock item
		*	@param barcodes An array of barcode information, or an array of barcodes.  
		*		Each barcode array should have barcode, 
		*		encoding and preferred keys.
		*/
		public function set_barcodes($stock_id, Array $barcodes) {
			if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory')) {
				return;
			}
			$old_barcodes = $this->get_barcodes($stock_id);
			$this->so->set_barcodes($stock_id, $barcodes);
			foreach($old_barcodes as $key => $change) {
				$this->history_log->add($this->field2history['bar_code'], $stock_id, $barcodes[$key]['barcode'], $change['barcode']);
			}
		}

		/**
		*	Update a stock item's stock number
		*
		*	@param stock_id
		*	@param new_number
		*
		*	@return true on successful update
		*/
		public function update_number($stock_id, $new_number) {
			
			$new_number = trim($new_number);

			// Check for duplicate stock number
			$query = array(
				'search'	=>	$new_number
			);
			$stock_list = $this->so->get_stock_list($query);
			if(count($stock_list)) {
				foreach($stock_list as $item) {
					if($item['stock_num'] == $new_number) {
						return false;
						return lang('Stock code already exists.  Please enter a new one.');
					}
				}
			}
			$old_item = $this->get_stock_item($stock_id);
			if($old_item['stock_num'] != $new_number) {
				$this->history_log->add($this->field2history['stock_num'], $stock_id, $new_number, $old_item['stock_num']);
				$old_item['stock_num'] = $new_number;
				$this->so->set_stock_item($stock_id, $old_item);

				// Update timestamp
				$GLOBALS['egw']->contenthistory->updateTimeStamp('perp_inventory:STOCK', $stock_id, 'modify', time());
			}
			return true;
		}

		/**
		*	Get a list of general ledger accounts associated with a stock item
		*
		*	The list contents vary depending on certain stock item settings.  Pseudo
		*	stock items don't have an On Hand account, because there never is any
		*	inventory on hand.  Only assemblies have a work in progress GL account,
		*	because raw stock items are never in progress.
		*
		*	If the associated account has not been set for that particular stock item,
		*	the default for the stock category will be used.  If the default for the 
		*	stock category has not been set, null will be returned.
		*
		*	@param stock_id
		*	@param account_use Optional.  One or more of bo_perp_inventory::$account_uses.  If not
		*		provided, all applicable accounts will be returned
		*
		*	@return array of (use => GL account)
		*/
		public static function get_gl_accounts($stock_id, $account_use = array()) {
			if(!$stock_id) {
				throw new egw_exception_wrong_parameter('Missing stock_id');
			}
			if(!$account_use) {
				$account_use = array_keys(bo_perp_inventory::$account_uses);
			}
			$gl_accounts = so_perp_stock::get_gl_accounts($stock_id, (array)$account_use);

			// Only asked for one
			if(!is_array($account_use)) {
				return $gl_accounts[$account_use];
			} else {
			// Wanted them all
				return $gl_accounts;
			}
		}

		/**
		*	Set a list of general ledger accounts associated with a stock item.
		*
		*	@param stock_id
		*	@param accounts Array of (use => GL account)
		*/
		public static function set_gl_accounts($stock_id, $accounts) {
			if($GLOBALS['egw']->acl->check('stock', EGW_ACL_EDIT, 'perp_inventory')) {
				so_perp_stock::set_gl_accounts($stock_id, $accounts);
			}
		}
	}
?>
