<?php
	/**
	*	Record damage done to controlled (serialized / batched) goods
	*
	*	@see bo_perp_damage
	*
	*	@author Nathan Gray
	*	@package perp_inventory
	*/

	class so_perp_damage {
		
		public static function get_damage_list(&$query) {
			$fields = array(
				'perp_stock_history.*',
				'perp_stock_damage.*',
			);
			$join = 'JOIN perp_stock_history ON
				perp_stock_damage.stock_history_id = perp_stock_history.history_id
			JOIN perp_stock_serial ON
				perp_stock_serial.stock_id = perp_stock_history.stock_id AND
				perp_stock_serial.serial_number = perp_stock_history.serial_number
			LEFT JOIN perp_work_order_detail ON
				perp_stock_serial.work_order_id = perp_work_order_detail.work_order_id AND
				perp_stock_serial.detail_id = perp_work_order_detail.detail_id
			';

			$where = array();
			if($query['stock_id']) {
				$where[] = 'perp_work_order_detail.stock_id = ' . (int)$query['stock_id'];
			}
			if($query['serial_number']) {
				$where['serial_number'] = $query['serial_number'];
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(serial_number LIKE $search OR comment LIKE $search)";
			}
			if($query['work_order_id']) {
				$where[] = 'perp_work_order_detail.work_order_id = ' . (int)$query['work_order_id'];
				if($query['detail_id']) {
					$where[] = 'perp_work_order_detail.detail_id = ' . (int)$query['detail_id'];
				}
			}
			if($query['unclaimed']) {
				$where[] = '(perp_work_order_detail.work_order_id, perp_work_order_detail.detail_id) NOT IN (
					SELECT 
						detail_value AS work_order_id,
						work_order_detail_id AS detail_no
					FROM perp_sales_order_work_order
					JOIN perp_order_detail ON
						perp_order_detail.order_id = perp_sales_order_work_order.order_id AND
						perp_order_detail.line_no = perp_sales_order_work_order.line_no AND
						perp_order_detail.detail_no = perp_sales_order_work_order.detail_no
					GROUP BY work_order_id, detail_no
					HAVING sum(quantity) > 0
				)
				';

			}

			// Get total
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_stock_damage',
				array('count(*)'),
				$where, 
				__LINE__, __FILE__, false, '', 'perp_inventory',1,
				$join
			)->fetchSingle();

			$result = $GLOBALS['egw']->db->select(
				'perp_stock_damage',
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_inventory', $query['num_rows'], 
				$join
			);

			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			return $list;
		}

		public static function get_damage($id) {
			$join = 'JOIN perp_stock_damage ON
				perp_stock_damage.stock_history_id = perp_stock_history.history_id
				JOIN perp_stock_serial ON
					perp_stock_serial.stock_id = perp_stock_history.stock_id AND
					perp_stock_serial.serial_number = perp_stock_history.serial_number
				LEFT JOIN perp_work_order_detail ON
					perp_stock_serial.work_order_id = perp_work_order_detail.work_order_id AND
					perp_stock_serial.detail_id = perp_work_order_detail.detail_id
			';
			$data = $GLOBALS['egw']->db->select(
				'perp_stock_history',
				array('*'),
				array('perp_stock_history.history_id = ' . (int)$id),
				__LINE__, __FILE__, false, '', 'perp_inventory', 1,
				$join
			)->fetch();
			$data['timestamp'] = $GLOBALS['egw']->db->from_timestamp($data['timestamp']);

			return $data;
		}

		public static function set_damage($damage) {
			$GLOBALS['egw']->db->transaction_begin();

			if(!$damage['stock_history_id']) {
				// Create stock history record
				$damage['stock_history_id'] = so_perp_stock_history::add_history(
					$damage['stock_id'],
					$damage['serial_number'],
					$damage['comment'],
					array(),
					$damage['timestamp']
				);

				// Set the link, now that we know the history ID
				$GLOBALS['egw']->db->update(
					'perp_stock_history',
						array(
						'link_app'      =>      'perp_inventory',
						'link_id'       =>      'DAMAGE-' . $damage['stock_history_id']
						),
					array(
						'history_id'              =>      $damage['stock_history_id'],
					),
					__LINE__, __FILE__, 'perp_inventory'
				);
			}

			if(!$damage['stock_history_id']) {
				$GLOBALS['egw']->db->transaction_abort();
				$GLOBALS['egw']->db->transaction_commit();
				return lang('Could not add entry');
			}

			$GLOBALS['egw']->db->insert(
				'perp_stock_damage',
				array(
					'stock_history_id'	=>	$damage['stock_history_id'],
					'quantity_damaged'	=>	$damage['quantity_damaged'],
					'discount_percent'	=>	$damage['discount_percent'],
				),
				array(
					'stock_history_id'	=>	$damage['stock_history_id'],
				),
				__LINE__, __FILE__, 'perp_inventory'
			);
			$GLOBALS['egw']->db->transaction_commit();

			return $damage['stock_history_id'];
		}
	}
?>
