<?php

	class so_perp_inventory {

		private $db;
		private $config;

		public function so_perp_inventory() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_inventory');
			
			$this->config = config::read('perp_inventory');
		}

		public function get_inventory(&$query) {
			// Set up where array
			$where = array();

			if($query['location_id']) {
				if(is_array($query['location_id'])) {
					$where[] = 'perp_location_stock.location_id IN (' . implode(',', $query['location_id']) . ')';
				} else {
					$where[] = 'perp_location_stock.location_id = ' . (int)$query['location_id'];
				}
			}
			if($query['stock_id']) {
				if(is_array($query['stock_id'])) {
					$where[] = 'perp_stock.stock_id IN (' . implode(',', $query['stock_id']) . ')';
				} else {
					$where[] = 'perp_stock.stock_id = ' . $query['stock_id'];
				}
			}
			if($query['stock_category']) {
				$where[] = 'stock_category IN (' . implode(',', $query['stock_category']) . ')';
			}
			if($query['to_order']) {
				$where[] = 'quantity_on_hand < min_on_hand';
			}
			if($query['stock_type_flag']) {
				$where[] = 'stock_type_flag = \'' . $query['stock_type_flag'] . '\'';
			}
			if(!$query['status_flag']) {
				$where[] = "status_flag IN ('E', 'P', 'D')";
			} elseif($query['status_flag']) {
				$where[] = 'status_flag IN (' . implode(',', $query['status_flag']) . ')';
			}

			// Get record count
			$this->db->select('perp_stock',
				array('count(perp_stock.stock_id) AS count'),
				$where,
				__LINE__, __FILE__, false,
				'', 'perp_inventory', 1,
				'LEFT JOIN perp_location_stock ON
					perp_location_stock.stock_id = perp_stock.stock_id'
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			// Get results
			$where[] = 'egw_categories.cat_appname = \'perp_inventory\'';
			$this->db->select('egw_categories',
				array(
					'perp_stock.stock_id',
					'stock_num',
					'short_description',
					'stock_category',
					'cat_name AS stock_category_name',
					'perp_location.location_id',
					'perp_location.location_name',
					'IF(quantity_on_hand IS NULL, 0, quantity_on_hand) AS quantity_on_hand',
					'quantity_reserved',
					'IF(min_on_hand IS NULL, 0, min_on_hand) AS min_on_hand',
					'max_on_hand',
					'(quantity_on_hand + quantity_reserved) AS quantity_in_stock'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 'perp_inventory', $query['num_rows'],
				'JOIN perp_stock ON
					perp_stock.stock_category = egw_categories.cat_id
				LEFT JOIN perp_location_stock ON
					perp_location_stock.stock_id = perp_stock.stock_id
				LEFT JOIN perp_location ON
					perp_location_stock.location_id = perp_location.location_id'
			);
			$list = array();
			while($row = $this->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		public static function add_serial($stock_id, $serial, $quantity, $location_id, $stock_cost_id) {
			$GLOBALS['egw']->db->insert('perp_stock_serial', 
				array(
					'stock_id'	=>	$stock_id,
					'serial_number'	=>	$serial,
					'quantity'	=>	$quantity,
					'location_id'	=>	$location_id,
					'stock_cost_id'	=>	$stock_cost_id,
				),
				array(
					'stock_id'	=>	$stock_id,
					'serial_number'	=>	$serial
				), __LINE__, __FILE__, 'perp_inventory'
			);
		}

		public function get_controlled_inventory_list(&$query) {
			if($query['location']) {
				$where[] = 'perp_stock_serial.location_id = ' . $query['location'];
			}
			if($query['cat_id']) {
				$where[] = 'stock_category = ' . $query['cat_id'];
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_stock_serial.serial_number LIKE $search OR
					perp_stock.stock_num LIKE $search OR
					perp_stock.short_description LIKE $search OR
					perp_work_order.work_order_reference LIKE $search OR
					perp_order.our_reference_number LIKE $search
				)";
			}
			if(isset($query['stock_id'])) {
				$where[] = 'perp_stock_serial.stock_id = ' . (int)$query['stock_id'];
			}
			if(isset($query['serial_number'])) {
				$where[] = 'perp_stock_serial.serial_number = ' . $GLOBALS['egw']->db->quote($query['serial_number']);
			}
			if(isset($query['work_order_id']) && $GLOBALS['egw_info']['apps']['perp_manufacturing']) {
				$where[] = 'perp_stock_serial.work_order_id = ' . $GLOBALS['egw']->db->quote($query['work_order_id']);
				if($query['detail_id']) {
					$where[] = 'perp_stock_serial.detail_id = ' . $GLOBALS['egw']->db->quote($query['detail_id']);
				}
			}

			$columns = array(
				'perp_stock_serial.stock_id',
				'stock_num',
				'short_description',
				'perp_work_order.work_order_id',
				'work_order_reference',
				'serial_number',
				'perp_stock_serial.location_id',
				'detail_id',
				'quantity',
				'short_description',
				'stock_category'
			);


			$join = 'JOIN perp_stock ON
					perp_stock_serial.stock_id = perp_stock.stock_id
				LEFT JOIN perp_location ON
					perp_stock_serial.location_id = perp_location.location_id';
			if($GLOBALS['egw_info']['apps']['perp_manufacturing']) {
				$columns[] = 'perp_work_order.work_order_id';
				$columns[] = 'perp_order.order_id';
				$columns[] = 'perp_order.our_reference_number';
				$join .= ' LEFT JOIN perp_work_order ON
					perp_work_order.work_order_id = perp_stock_serial.work_order_id
					LEFT JOIN perp_order_detail ON
						perp_work_order.work_order_id = perp_order_detail.detail_value AND
						perp_order_detail.detail_class = \'WorkOrderDetail\'
					LEFT JOIN perp_order ON
						perp_order_detail.order_id = perp_order.order_id';
			}

			// Get number of results
			$this->db->select('perp_stock_serial', 
				array('count(serial_number) AS count'),
				$where, __LINE__, __FILE__, 0, '',
				'perp_inventory', 1,
				$join
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_stock_serial', 
				$columns,
				$where, __LINE__, __FILE__,
				$query['start'], 'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_inventory', $query['num_rows'],
				$join
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function add_stock_cost($stock_id, $quantity, $direct_cost, $indirect_cost = 0) {
			$this->db->insert('perp_stock_cost',
				array(
					'stock_id'	=>	$stock_id,
					'date'		=>	date('U'),
					'quantity'	=>	$quantity,
					'direct_cost'	=>	$direct_cost,
					'indirect_cost'	=>	$indirect_cost,
				),
				array(),
				__LINE__, __FILE__
			);
			return $this->db->get_last_insert_id('perp_stock_cost', 'stock_cost_id');
		}

		public function receive($stock_id, $location, $quantity, $stock_cost_id, $reference = '', $move_type = 3) {
			if(!$stock_id) {
				// Only interested in stock items here
				throw new egw_exception_wrong_parameter(lang('Missing required field: %1', 'stock_id'));
			}
			if(!$location) {
				throw new egw_exception_wrong_parameter(lang('Missing required field: %1', 'location_id'));
			}

			$where = array(
				'stock_id'	=>	$stock_id,
				'location_id'	=>	$location
			);
			$this->db->select('perp_location_stock', array(
				'location_id',
				'stock_id',
				'quantity_on_hand'),
				$where, __LINE__, __FILE__
			);
			$row = $this->db->row(True);
			if($row) {
				$this->db->update('perp_location_stock', array(
					'quantity_on_hand = quantity_on_hand + ' . $quantity),
					$where, __LINE__, __FILE__
				);
			} else {
				$this->db->insert('perp_location_stock', array(
					'location_id'	=> 	$location,
					'stock_id'	=>	$stock_id,
					'quantity_on_hand'	=>	$quantity),
					$where, __LINE__, __FILE__
				);
			}
			if($stock_id && $this->config['costing'] == 'fifo') {
				$this->db->insert('perp_fifo_costing', array(
					'stock_id'	=>	$stock_id,
					'quantity'	=>	$quantity,
					'location'	=>	$location,
					'stock_cost_id'	=>	$stock_cost_id,
					),
					array(), __LINE__, __FILE__
				);
			}
			if(!$reference) {
				$reference = lang('Received');
			}
			$this->add_stock_movement($stock_id, $location, $location, $move_type, $quantity, $reference);
			return true;
		}


		/* Only deals inventory counts, because it may be just a stock transfer
		*	Returns total (direct + indirect) cost
		*/
		public function ship($stock_id, $location, $quantity, $serials, $shipment_id) {
			if(!$stock_id) {
				return;
			}

			// Controlled stock needs numbers
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			if($stock['controlled']) {
				$serial_count = 0;
				if(is_array($serials)) {
					foreach($serials as $serial => $serial_quantity) {
						$serial_count += $serial_quantity;
					}
				}
					
				if($serial_count != $quantity) {
					throw new Exception(lang('Tried to ship %1 controlled goods, but only provided numbers for %2', $quantity, $serial_count));
				}
			}

			$where = array(
				'stock_id'	=>	$stock_id,
				'location_id'	=>	$location
			);
			$this->db->select('perp_location_stock', array(
				'location_id',
				'stock_id',
				'quantity_reserved'),
				$where, __LINE__, __FILE__
			);
			$row = $this->db->row(True);
			if($row) {
				$this->db->update('perp_location_stock', array(
					'quantity_reserved = quantity_reserved - ' . $quantity),
					$where, __LINE__, __FILE__
				);
			} else {
				$this->db->insert('perp_location_stock', array(
					'location_id'	=> 	$location,
					'stock_id'	=>	$stock_id,
					'quantity_reserved'	=>	-$quantity),
					$where, __LINE__, __FILE__
				);
			}
			$this->add_stock_movement($stock_id, $location, $location, -1, $quantity, lang('Shipped'));

			// Controlled records
			if($stock['controlled']) {
				foreach($serials as $serial => $s_quantity) {
					// Add history
					$this->db->select('perp_stock_serial',
						array('*'),
						array(
							'stock_id'	=>	$stock_id,
							'serial_number'	=>	$serial
						),
						__LINE__, __FILE__
					);
					$row = $this->db->row(true);
					if($row['work_order_id']) {
						ExecMethod2('perp_inventory.bo_perp_stock_history.add_history', 
							$stock_id,
							$serial,
							lang('Shipped out'),
							array(
								'app'	=>	'perp_ar',
								'id'	=>	'SHIPMENT-' . $shipment_id
							),
							__LINE__, __FILE__
						);
					}

					$sql = "UPDATE perp_stock_serial
						SET location_id = -1
						WHERE stock_id = '$stock_id' AND
							serial_number = '$serial' AND
							quantity = $s_quantity AND
							location_id = $location";
					$this->db->query($sql, __LINE__, __FILE__);
					if($this->db->affected_rows() == 0) {
						// Only moving some of a batch
						// TODO: Make this work
					}

					
				}
			}
			// Update financial

			$cost = 0;
			if($stock_id && $this->config['costing'] == 'fifo') {
				$cost = $this->fifo_cost($stock_id, $location, $quantity, true);
			}

			return $cost;
		}

		public function reserve_stock($location_id, $stock_id, $quantity, $reference = '') {
			if(!$location_id) {
				throw new egw_exception_wrong_parameter('Missing required field: ' . 'location_id');
			} elseif (!$stock_id) {
				throw new egw_exception_wrong_parameter('Missing required field: ' . 'stock_id');
			}

			$query = array(
				'stock_id'	=>	$stock_id,
				'location_id'	=>	$location_id
			);
			$inventory = array();
			$bo_loc = CreateObject('perp_inventory.bo_perp_location');
			$bo_loc->get_inventory($query, $inventory);
			if($quantity > 0 && (!count($inventory) || $inventory[0]['quantity_on_hand'] == 0) && $this->config['no_negative_stock_count']) {
				return lang('No stock on hand');
			}
			$sql = 'SELECT quantity_on_hand
				FROM perp_location_stock
				WHERE
					location_id = ' . $location_id . '
					AND stock_id = ' . $stock_id;
			
			$this->db->query($sql, __LINE__, __FILE__);
			if(!($row = $this->db->row(True))) {
				$this->db->insert('perp_location_stock', 
					array(
						'location_id'	=>	$location_id,
						'stock_id'	=>	$stock_id
					),
					array(
						'location_id'	=>	$location_id,
						'stock_id'	=>	$stock_id
					),
					__LINE__, __FILE__
				);
			}
			$sql = 'UPDATE perp_location_stock 
				SET
					quantity_on_hand = (quantity_on_hand - ' . $quantity . '),
					quantity_reserved = (quantity_reserved + ' . $quantity . ')
				WHERE
					location_id = ' . $location_id . '
					AND stock_id = ' . $stock_id;

			$this->db->query($sql, __LINE__, __FILE__);
			$this->add_stock_movement($stock_id, $location_id, $location_id, 2, $quantity, $reference);
		}

		/**
		*	Get a list of stock that can be moved from one work center to another
		*	The list is restricted based on the stock available at each work center and
		*	General Ledger accounts to receive it.
		*	
		*	@param $query		array an associative array containing standard 
		*				nextmatch variables as well as the following:
		*	@param $from_location	int the location the stock is at
		*	@param $from_work_center int the work center (or on_hand) the stock is at
		*	@param $to_location 	int the location to move the stock to
		*	@param $to_work_center 	int the work center the stock will go to
		*/
		public function get_movable_stock(&$query) {
			if(!$query['from_work_center']) {
				$query['from_work_center'] = 'on_hand';
			}
				
			$select = 'SELECT 
					perp_stock.stock_id,
					perp_stock.stock_num,
					perp_stock.short_description, 
					source.quantity_on_hand,
					source.quantity_reserved, ';
			if($query['from_work_center'] != 'garbage') {
				$select .= 'if(
					source.quantity_' . $query['from_work_center'] . ' > destination.max_on_hand, 
					source.quantity_' . $query['from_work_center'] . ' , 
					destination.max_on_hand
				) AS maximum ';
			} else {
				$select .= '\' - \' AS maximum ';
			}
			$select .= ' FROM ';
			$count = 'SELECT count(perp_stock.stock_id) AS count FROM';
			$sql = ' perp_location_stock AS source
				LEFT JOIN perp_location_stock AS destination ON
					source.stock_id = destination.stock_id ';
			if ($query['to_location']) {
				$sql .= '	AND destination.location_id = ' . $query['to_location'];
			}

			$sql .= ' JOIN perp_stock ON 
					perp_stock.stock_id = source.stock_id
				WHERE ';
			if($query['from_work_center'] != 'garbage') {
				$sql .= ' source.quantity_' . $query['from_work_center'] . ' != 0 ';
			} else {
				$sql .= ' true = true ';
			}
			if ($query['from_location']) {
				$sql .= ' AND source.location_id = ' . $query['from_location'];
			} 
			if($query['search']) {
				$search = $this->db->quote('%' . $query['search'] . '%');
				$sql .= " AND (stock_num LIKE $search OR short_description LIKE $search)";
			}
			if($query['stock_id']) {
				$sql .= ' AND perp_stock.stock_id = ' . (int)$query['stock_id'];
			}

			if(isset($query['order'])) {
				$sql .= ' ORDER BY ' . $query['order'] . ' ' . $query['sort'] . ' ';
			}
			if(isset($query['start'])) {
				$sql .= 'LIMIT ' . $query['start'];
				if(isset($query['num_rows'])) {
					$sql .= ', ' . $query['num_rows'];
				}
			}
	
			$this->db->query($count . $sql, __LINE__, __FILE__);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->query($select . $sql, __LINE__, __FILE__);

			$list = array();
			while($row = $this->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		public function move($from, $from_wc, $to, $to_wc, $stock_id, $quantity, $serials) {
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			if($stock['controlled'] && $stock['serial_setup'] != bo_perp_stock::SERIAL_WHEN_NEEDED && !count($serials)) {
				return lang('Need serial numbers for controlled stock item %1', $stock['stock_num']);
			}

			$this->db->transaction_begin();

			$sql = "UPDATE perp_location_stock
				SET quantity_{$from_wc} = quantity_{$from_wc} - $quantity
				WHERE location_id = $from AND
				stock_id = $stock_id";
			$this->db->query($sql, __LINE__, __FILE__);
			if($this->db->affected_rows() != 1) {
				$this->db->transaction_abort();
				return lang('Could not find stock to move (%1)', $stock['stock_num']);
			}

			$sql = '';
			if(!is_numeric($to_wc) && $to_wc != 'garbage') {
				$sql = "UPDATE perp_location_stock
					SET quantity_{$to_wc} = quantity_{$to_wc} + $quantity
					WHERE location_id = $to AND
					stock_id = $stock_id";
				$this->db->query($sql, __LINE__, __FILE__);
			}

			if($this->db->affected_rows() != 1) {
				// No record for that stock item at that location
				$GLOBALS['egw']->db->insert(
					'perp_location_stock',
					array(
						'location_id'		=>	$to,
						'stock_id'		=>	$stock_id,
						"quantity_{$to_wc}"	=>	$quantity
					),
					array(),
					__LINE__, __FILE__,
					'perp_inventory'
				);
			}

			if($serials) {
				$location_check = array(
					'stock_id'	=>	$stock_id,
					'order'		=>	'stock_id',
				);

				foreach($serials as $serial => $quantity) {
					// Don't check if it's serial when needed, it might not be there
					if($stock['serial_setup'] != bo_perp_stock::SERIAL_WHEN_NEEDED) {
						// Check to make sure item is where we think it is
						$location_check['serial_number'] = $serial;
						$list = $this->get_controlled_inventory_list($location_check);
						if(count($list) != 1) {
							$this->db->transaction_abort();
							return lang(
								'Could not find %1 serial number %2 - may have been entered wrong', 
								ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id), 
								$serial
							);
						} elseif ($list[0]['location_id'] != $from) {
							$this->db->transaction_abort();
							return lang(
								'Could not find %1 serial number %2 at %3.  It is at %4.', 
								ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id), 
								$serial,
								ExecMethod('perp_inventory.bo_perp_location.link_title', $from), 
								ExecMethod('perp_inventory.bo_perp_location.link_title', $list[0]['location_id'])
							);
						}
					}
					$sql = "UPDATE perp_stock_serial
						SET location_id = $to
						WHERE stock_id = '$stock_id' AND
							serial_number = '$serial' AND
							quantity = $quantity AND
							location_id = $from";
					$this->db->query($sql, __LINE__, __FILE__);
					if($this->db->affected_rows() == 0) {
						// Only moving some of a batch
						// TODO: Make this work
					}
				}
			}

			// Update financial
			if($stock_id && $this->config['costing'] == 'fifo') {
				$GLOBALS['egw']->db->select('perp_fifo_costing', array(
					'quantity',
					'stock_cost_id',
					), array(
						'stock_id'	=>	$stock_id,
						'location'	=>	$from,
					),
					__LINE__, __FILE__, 'perp_inventory'
				);
				$move_quantity = $quantity;
				while(($row = $GLOBALS['egw']->db->row(true)) && $move_quantity > 0) {
					$row['quantity'] = min($row['quantity'], $move_quantity);
					$move_quantity -= $row['quantity'];

					$this->db->insert('perp_fifo_costing', array(
						'stock_id'	=>	$stock_id,
						'quantity'	=>	$row['quantity'],
						'location'	=>	$to,
						'stock_cost_id'	=>	$row['stock_cost_id'],
						),
						array(), __LINE__, __FILE__
					);
				}
				$cost = $this->fifo_cost($stock_id, $from, $quantity, True);
			}
			$this->add_stock_movement($stock_id, $from, $to, 1, $quantity);

			$this->db->transaction_commit();
			return True;
		}

		/**
		*	Consume some stock
		*	Adjusts stock counts, returns cost of stock consumed.
		*	Takes from RESERVED stock.
		*/
		public function consume($stock_id, $consumption, $location_id, $quantity, $work_order_ref, $serials, $work_in_progress_stock_id = null) {
		
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			$work_center = ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center', $location_id);

			// Used for something, but for what?
			if($consumption == 'used' && !$work_in_progress_stock_id) {
				trigger_error(
					lang('Tried to consume %1 as used, but no work in progress stock ID was provided', 
					ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id)), E_USER_WARNING
				);
				$consumption = 'adjustment';
			}
			
			$qty = $quantity;

			// Location is a work center
			$bo_inventory = CreateObject('perp_inventory.bo_perp_inventory');
			$query = array(
				'stock_id'	=>	$stock_id,
				'location_id'	=>	$location_id
			);
			$bo_inventory->get_inventory($query, $inventory);
			if(count($inventory) == 0) {
				// No bins at the work center
				$stock_name = ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id);
				echo lang('No inventory set up at %1 for %2', $work_center['work_center_name'], $stock_name);
			}
			foreach($inventory as $bin) {
				if($bin['stock_id'] == $stock_id) {
					if($bin['quantity_reserved'] > $qty) {
						$change = $qty;
						$qty = 0;
					} else {
						$change = $bin['quantity_reserved'];
						$qty -= $bin['quantity_reserved'];
					}
					$this->db->update('perp_location_stock', 
						array(
							'quantity_reserved = quantity_reserved - ' . $change
						), 
						array(
							'location_id'	=>	$bin['location_id'],
							'stock_id'	=>	$bin['stock_id']
						), __LINE__, __FILE__
					);
					if($qty == 0) {
						break;
					}
				}
			}

			switch($consumption) {
			case 'used':
				$account = bo_perp_stock::get_gl_accounts($work_in_progress_stock_id, bo_perp_inventory::WORK_IN_PROGRESS);
				$use_type = -2;
				break;
			case 'garbaged':
				$account = bo_perp_stock::get_gl_accounts($stock_id, bo_perp_inventory::SHRINKAGE);
				$use_type = -3;
				break;
			case 'adjustment':
				$account = bo_perp_stock::get_gl_accounts($stock_id, bo_perp_inventory::ADJUSTMENT);
				$use_type = -4;
				break;
			}

			$GLOBALS['egw']->db->transaction_begin();

			if($serials) {
				foreach($serials as $serial => $serial_quantity) {
					$sql = "UPDATE perp_stock_serial
						SET location_id = " . $use_type . "
						WHERE stock_id = '$stock_id' AND
							serial_number = '$serial' AND
							quantity = $serial_quantity AND
							location_id = $location_id";
					$this->db->query($sql, __LINE__, __FILE__);
					if($this->db->affected_rows() == 0) {
						// Only moving some of a batch
						// TODO: Make this work
					}
				}
			}

			$this->add_stock_movement($stock_id, $location_id, $location_id, $use_type, $quantity, $work_order_ref);

			if($this->config['costing'] == 'fifo') {
				$cost = $this->fifo_cost($stock_id, $location_id, $quantity, True);
			}

			$narrative = lang(
				'consume_stock_narrative %1 %2 %3 - %4',
				abs($quantity),
				$stock['stock_num'],
				(($quantity < 0 && $use_type != -4) ? lang('recovered') : lang($consumption)),
				$work_order_ref
			);
			$trans = CreateObject('perp_ledger.GLTransaction');
			$trans->add_line($account, $narrative, $cost);
			$trans->add_line(bo_perp_stock::get_gl_accounts($stock_id, bo_perp_inventory::ON_HAND), $narrative, -$cost);
			$trans->write();

			$GLOBALS['egw']->db->transaction_commit();

			return $cost;
		}

		public function fifo_cost($stock_id, $location_id, $quantity, $consume = False) {
			$cost = 0;
			$cost_quantity = $quantity;
			$kill_fifos = array();

			$sql = 'SELECT 
					perp_fifo_costing.stock_cost_id,
					perp_fifo_costing.quantity,
					perp_fifo_costing.location,
					(direct_cost + indirect_cost) AS price
				FROM
					perp_fifo_costing
				JOIN perp_stock_cost ON
					perp_fifo_costing.stock_cost_id = perp_stock_cost.stock_cost_id
				WHERE
					perp_stock_cost.stock_id = ' . $stock_id . ' AND
					perp_fifo_costing.location = ' . $location_id . ' 
				ORDER BY date ASC'; 
			$this->db->query($sql, __LINE__, __FILE__);

				
			while($cost_quantity != 0 && $row = $this->db->row(True)) {
				if($quantity < 0) {
					// Put some back at the current cost
					$cost += $quantity * $row['price'];
					if($consume) {
						$GLOBALS['egw']->db->update('perp_fifo_costing', array(
								'quantity'	=>	'quantity + ' . abs($quantity)
							),
							$row, __LINE__, __FILE__, 'perp_inventory'
						);
					}
					break;
				}
				if($cost_quantity >= $row['quantity']) {
					// Use the entire row
					$kill_fifos[] = $row['stock_cost_id'];
					$cost += $row['quantity'] * $row['price'];
					$cost_quantity -= $row['quantity'];
				} else {
					// Use part of the row
					$cost += $cost_quantity * $row['price'];
					$new_quantity = $row['quantity'] - $cost_quantity;
					if($consume) {
						$GLOBALS['egw']->db->update('perp_fifo_costing', array(
								'quantity'	=>	$new_quantity
							),
							$row, __LINE__, __FILE__, 'perp_inventory'
						);
					}
					$cost_quantity = 0;
				}
			}
			if($consume) {
				foreach($kill_fifos as $received_id) {
					$this->db->delete('perp_fifo_costing', array(
						'stock_id'	=>	$stock_id,
						'location'	=>	$location_id,
						'stock_cost_id'	=>	$received_id),
						__LINE__, __FILE__
					);
				}
			}
			return $cost;
		}

		/**
		*	Add a stock movement record
		*
		*	Keep track of a history of where things were
		*/
		public function add_stock_movement($stock_id, $from_location_id, $to_location_id, $type, $quantity, $narrative = '') {
			if(!$stock_id || !$from_location_id || !$to_location_id || !$type || !$quantity) {
				if(!$stock_id) {
					$missing = 'stock_id';
				} elseif (!$from_location_id) {
					$missing = 'from_location_id';
				} elseif (!$to_location_id) {
					$missing = 'to_location_id';
				} elseif (!$type) {
					$missing = 'type';
				} elseif (!$quantity) {
					$missing = 'quantity';
				} else {
					$missing = lang('Unknown');
				}
				throw new egw_exception_wrong_parameter('Missing required field: ' . $missing);
			}

			$narrative = trim($narrative);
			$GLOBALS['egw']->db->insert(
				'perp_stock_move',
				array(
					'stock_id'		=>	(int)$stock_id,
					'timestamp'		=>	$GLOBALS['egw']->db->to_timestamp(date('U')),
					'from_location_id'	=>	(int)$from_location_id,
					'to_location_id'	=>	(int)$to_location_id,
					'move_type'		=>	(int)$type,
					'quantity'		=>	(double)$quantity,
					'narrative'		=>	$narrative
				),
				array(),
				__LINE__, __FILE__, 'perp_inventory'
			);
		}


		/**
		*	Get a list of stock movement records for the given parameters
		*/
		public function get_stock_movement(&$query) {
			$where = array();
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_stock.stock_num LIKE $search OR 
					perp_stock.short_description LIKE $search OR
					perp_stock_move.timestamp LIKE $search OR
					perp_stock_move.narrative LIKE $search)";
			}
			if($query['stock_id']) {
				$where[] = 'perp_stock_move.stock_id = '. (int)$query['stock_id'];
			}
			if($query['from_location_id']) {
				$where['from_location_id'] = (is_array($query['from_location_id']) ? $query['from_location_id'] : (int)$query['from_location_id']);
			}
			if($query['to_location_id']) {
				$where['to_location_id'] = (is_array($query['to_location_id']) ? $query['to_location_id'] : (int)$query['to_location_id']);
			}
			if($query['location_id']) {
				$location_id = (is_array($query['location_id']) ? ' IN (' . implode(',', $query['location_id']) . ')' : '=' . (int)$query['location_id']);
				$where[] = '(to_location_id ' . $location_id . ' OR from_location_id ' . $location_id . ')';
			}
			if($query['date']) {
				$where[] = 'timestamp BETWEEN ' . 
					$GLOBALS['egw']->db->to_timestamp(mktime(0,0,0, date('d', $query['date']), date('m', $query['date']), date('Y', $query['date']))) . ' AND ' .
					$GLOBALS['egw']->db->to_timestamp(mktime(23,59,59, date('d', $query['date']), date('m', $query['date']), date('Y', $query['date'])));
			}
			if($query['start_time']) {
				$where[] = 'timestamp >= ' . $GLOBALS['egw']->db->to_timestamp($query['start_time']);
			}
			if($query['end_time']) {
				$where[] = 'timestamp <= ' . $GLOBALS['egw']->db->to_timestamp($query['end_time']);
			}
			if($query['move_type']) {
				$where['move_type'] = (int)$query['move_type'];
			}
			if($query['cat_id']) {
				$where[] = 'perp_stock.stock_category = ' . (int)$query['cat_id'];
			}

			// Get count
			$GLOBALS['egw']->db->select(
				'perp_stock_move',
				array('count(perp_stock_move.stock_id) AS count'),
				$where,
				__LINE__, __FILE__, 0,
				'', 'perp_inventory', 1,
				'JOIN perp_stock ON
					perp_stock.stock_id = perp_stock_move.stock_id'
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['total'] = $row['count'];
			}

			// Get results
			$list = array();
			$GLOBALS['egw']->db->select(
				'perp_stock_move',
				array('*'),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_inventory', $query['num_rows'],
				'JOIN perp_stock ON
					perp_stock.stock_id = perp_stock_move.stock_id
				JOIN perp_location AS from_location ON
					from_location.location_id = perp_stock_move.from_location_id
				JOIN perp_location AS to_location ON
					to_location.location_id = perp_stock_move.to_location_id'
			);
			while($row = $GLOBALS['egw']->db->row(true)) {
				$row['timestamp'] = $GLOBALS['egw']->db->from_timestamp($row['timestamp']);
				$list[] = $row;
			}
			return $list;
		}
	}
?>
