<?php
	include_once(EGW_INCLUDE_ROOT . '/phpgwapi/inc/class.categories.inc.php');

	class so_perp_location {

		private $db;

		public function so_perp_location() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_inventory');
		}

		/**
		*	Get a list of locations
		*	@nextmatch
		*/
		public function get_locations(&$query = array()) {
			$where = array();
			if($query['location_id']) {
				// We're looking for a specific location
				$where['location_id'] = $query['location_id'];
			} else {
				if($query['search']) {
					$where[] = "location_name LIKE '%" . addslashes($query['location_name']) ."%'";
				}
				if(isset($query['filter']) && $query['filter'] !== '') {
					$where['status'] = $query['filter'];
				} elseif (!isset($query['filter'])) {
					$where['status'] = 1;
				}
				if(isset($query['filter2'])) {
					if($query['filter2'] == 'shippable') {
						$where[] = 'delivery_address != 0';
					}
				}
				if(isset($query['delivery_address'])) {
					$where['delivery_address'] = (int)($query['delivery_address']);
				}
				if(!$query['location_parent']) {
					// Start at the top
					$query['location_parent'] = 0;
				}
				if(isset($query['location_parent'])) {
					$where['location_parent'] = (int)($query['location_parent']);
				}
			}

			// Get number of rows 
			$this->db->select('perp_location', 
				array('count(location_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}
			$this->db->select('perp_location', array(
				'location_id',
				'location_parent',
				'location_name',
				'delivery_address',
				'tax_location',
				'status'),
				$where,
				__LINE__, __FILE__, 0, 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_inventory', -1
			);
			$lines = array();
			while($row = $this->db->row(True)) {
				$lines[] = $row;
			}
			if($query['location_id']) {
				// Looking for a specific location
				return $lines;
			}
			$locations = array();
			$sub_query = $query;
			$sub_query['level'] = $query['level'] + 1;

			// Get sub locations
			foreach($lines as &$line) {
				// Indent 3 spaces per level
				$line['indent'] = str_repeat('&nbsp;', max(0,$sub_query['level'] - 1) * 3);
				$line['indented_name'] = $line['indent'] . $line['location_name'];

				$locations[] = $line;
				$sub_query['location_parent'] = $line['location_id'];
				$sub_locations = $this->get_locations($sub_query);
				$query['count'] += $sub_query['count'];
				if(count($sub_locations) > 0) {
					$locations = array_merge($locations, $sub_locations);
				}
			}
			return $locations;
		}

		public function set_location($location_id, $location) {
			$where = array();
			if($location_id != '') {
				$where['location_id'] = $location_id;
			}

			$this->db->insert('perp_location', array(
					'location_id'	=>	$location_id,
					'location_parent'	=>	$location['location_parent'],
					'location_name'	=>	$location['location_name'],
					'delivery_address'	=>	$location['delivery_address'],
					'tax_location'	=>	$location['tax_location'],
					'status'	=>	$location['status']
				),
				$where, __LINE__, __FILE__
			);
			$location_id = $location_id ? $location_id : $this->db->get_last_insert_id('perp_location', 'location_id');

			return $location_id;
		}

		public function delete_location($location_id, $really_delete) {
			if($really_delete) {
				$tables = array(
					'perp_location',
					'perp_location_gl_account',
					'perp_location_stock',
				);
				foreach($tables as $table) {
					$this->db->delete(
						$table, 
						array('location_id' => $location_id), 
						__LINE__, __FILE__
					);
				}
				$this->db->delete(
					'perp_location_transfer',
					array('ship_location = ' . $location_id . ' OR 
						receive_location = ' . $location_id 
					),
					__LINE__, __FILE__
				);
			} else {
				$this->db->update('perp_location',
					array(
						'status'	=>	0
					),
					array(
						'location_id'	=>	$location_id
					),
					__LINE__, __FILE__
				);
			}
		}

		public function check_location_name($location_name)
		{
			$this->db->select('perp_location', 
				array('location_name'),
				array('location_name' => $location_name),
				__LINE__, __FILE__
			);
			return $this->db->num_rows();
		}

		public function get_inventory(&$query) {
			$where = array(
				'!pseudo',
				"(status_flag != 'R' OR status_flag = 'R' AND (quantity_on_hand != 0 OR quantity_reserved != 0))",
			);
			if($query['cat_id'] && $query['cat_id'] != 1) {
				if(!is_object($this->categories)) {
					$this->categories =& new categories();
				}
				$where[] = "stock_category IN (" . implode(',', $this->categories->return_all_children($query['cat_id'])) . ')';
			}
			if($query['filter2']) {
				if($query['filter2'] == 'on_hand') {
					$where[] ='(perp_location_stock.quantity_on_hand > 0)';
				} elseif ($query['filter2'] == 'not_on_hand') {
					$where[] ='(perp_location_stock.quantity_on_hand = 0 OR
						perp_location_stock.quantity_on_hand IS NULL)';
				} elseif ($query['filter2'] == 'reserved') {
					$where[] = 'perp_location_stock.quantity_reserved > 0';
				}
			}
			if($query['search']) {
				$search = $this->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_stock.stock_num LIKE $search OR perp_stock.short_description LIKE $search)";
			}
			// $query['location_id'] handled in JOIN clause

			$join = 'LEFT JOIN perp_location_stock ON
					perp_location_stock.stock_id = perp_stock.stock_id ' .
				( $query['location_id'] ? 'AND perp_location_stock.location_id = ' . (int)$query['location_id'] : '') . '
				JOIN perp_unit_of_measure ON
					perp_stock.unit_of_measure = perp_unit_of_measure.uom_id ';
			if( $query['location_id']) {
				$join .= 'JOIN perp_location_gl_account ON
					perp_location_gl_account.stock_category_id = perp_stock.stock_category AND
					perp_location_gl_account.stock_gl_account != 0 AND
					perp_location_gl_account.location_id = ' . (int)$query['location_id'];
			}
			// Get row count
			$this->db->select('perp_stock',
				array('count(perp_stock.stock_id) AS count '),
				$where,
				__LINE__, __FILE__, 0,
				'', false, 1,
				$join
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_stock',
				array(
					'perp_stock.stock_id',
					'perp_stock.stock_num',
					'perp_stock.short_description',
					'perp_stock.decimal_places',
					'perp_unit_of_measure.uom_name',
					'perp_location_stock.quantity_on_hand',
					'perp_location_stock.quantity_reserved',
					'perp_location_stock.min_on_hand',
					'perp_location_stock.max_on_hand'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				false, $query['num_rows'], 
				$join
			);

			$list = array();
			while($row = $this->db->row(True)) {
				$row['quantity_on_hand'] = round($row['quantity_on_hand'], $row['decimal_places']);
				$row['quantity_reserved'] = round($row['quantity_reserved'], $row['decimal_places']);
				$row['min_on_hand'] = round($row['min_on_hand'], $row['decimal_places']);
				$row['max_on_hand'] = round($row['max_on_hand'], $row['decimal_places']);
				$list[] = $row;
			}

			return $list;
		}

		public function set_inventory($location, $stock, $min_on_hand, $max_on_hand) {
			$where = array(
				'location_id'	=>	$location,
				'stock_id'	=>	$stock,
			);

			// Get current values
			$this->db->select('perp_location_stock', 
				array('*'),
				$where,
				__LINE__, __FILE__
			);
			$row = $this->db->row(true);

			// Update min / max, set others if they're not set yet
			$row['location_id']	=	$location;
			$row['stock_id']	=	$stock;
			$row['min_on_hand']	=	$min_on_hand;
			$row['max_on_hand']	=	$max_on_hand;

			// Update info
			$this->db->insert('perp_location_stock', 
				$row,
				$where,
				__LINE__, __FILE__
			);
		}

		public function get_gl_accounts($query, &$rows, &$readonlys) {
			$sql = 'SELECT 
					cat_id AS stock_category_id,
					cat_name AS stock_category_name,
					location_id,
					stock_gl_account,
					work_in_progress_gl_account,
					shrinkage_gl_account
			FROM egw_categories
			LEFT JOIN perp_location_gl_account ON
			    egw_categories.cat_id = perp_location_gl_account.stock_category_id ';
			if(isset($query['location_id']) && $query['location_id']) {
				$sql .= 'AND perp_location_gl_account.location_id = ' . $query['location_id'];
			}
			if($query['stock_id']) {
				$sql .= ' JOIN perp_stock ON
				perp_stock.stock_category = perp_location_gl_account.stock_category_id AND 
				perp_stock.stock_id = ' . $query['stock_id'];
			}
			$sql .= ' WHERE egw_categories.cat_appname = \'perp_inventory\' ';
			if(isset($query['stock_category_id'])) {
				$sql .= ' AND cat_id = ' . $query['stock_category_id'];
			}

			$this->db->query($sql, __LINE__, __FILE__);
			$count = $this->db->num_rows();
			$this->db->query($sql, __LINE__, __FILE__, $query['start'], $query['num_rows']);

			$rows = array();
			while($row = $this->db->row(True)) {
				$rows[] = $row;
			}
			return $count;
		}

		public function set_gl_accounts($location_id, $accounts) {
			$this->db->insert('perp_location_gl_account',
				array(
					'location_id'		=>	$location_id,
					'stock_category_id'	=>	$accounts['stock_category_id'],
					'stock_gl_account'	=>	$accounts['stock_gl_account'],
					'work_in_progress_gl_account'	=>	$accounts['work_in_progress_gl_account'],
					'shrinkage_gl_account'	=>	$accounts['shrinkage_gl_account']
				),
				array(
					'location_id'		=>	$location_id,
					'stock_category_id'	=>	$stock_category_id
				),
				__LINE__, __FILE__
			);
		}
		
		public function link_title($location_id) {
		
			$this->db->select('perp_location',
				array('location_id', 'location_name'),
				array('location_id' => $location_id),
				__LINE__, __FILE__
			);
			
			if($row = $this->db->row(True)) {
				return $row['location_name'];
			}
			return false;
		}

		public function link_query($pattern) {
		
			$query_str = '	SELECT 	location_id, location_name 
							FROM 	perp_location 
							WHERE 	location_name LIKE '.$this->db->quote('%'.$pattern.'%');  
				
			$this->db->query($query_str);	
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
		
		
	}
?>
