<?php
	require_once(EGW_INCLUDE_ROOT . '/phpgwapi/inc/class.categories.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_order.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_stock_category.inc.php');

	class so_perp_stock {
		private $db;
		
		/**
                *       Serial pattern legend and regex equivalent
                */
                public $pattern_legend = array("#", "@", ".");
                public $regex_legend  = array("[0-9]", "[a-zA-Z]", "[a-zA-Z0-9]");
		
		public function so_perp_stock() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_inventory');
		}

		public function get_stock_list(&$query) {
			$where = array();
			if($query['cat_id'] && $query['cat_id'] != 1) {
				$where['stock_category'] = $query['cat_id'];
			}
			if($query['filter2']) {
				$query['status'] = $query['filter2'];
			}
			if($query['status']) {
				$where['status_flag'] = $query['status'];
			}

			if($query['stock_type_flag'] || $query['col_filter']['stock_type_flag']) {
				$where['stock_type_flag'] = $query['stock_type_flag'] ? $query['stock_type_flag'] : $query['col_filter']['stock_type_flag'];
			}
			if($query['controlled'] || (
					is_array($query['col_filter']) && array_key_exists('controlled', $query['col_filter']) && $query['col_filter']['controlled'] !== '')) {
				if($query['col_filter']['controlled']) {
					$query['controlled'] = $query['col_filter']['controlled'];
				}
				if(is_bool($query['controlled'])) {
					$where[] = 'controlled';
				} else {
					$where['controlled'] = $query['controlled'];
				}
			}

			if($query['stock_id']) {
				$where[] = 'perp_stock.stock_id IN (' . implode(',', (array)$query['stock_id']) . ')';
			}

			if($query['search']) {
				$search = $this->db->quote('%' . $query['search'] . '%');
				$search_clause = "(stock_num LIKE $search OR short_description LIKE $search OR
					perp_stock_barcode.barcode = " . $this->db->quote($query['search']);
				if(strpos($query['selectcols'], 'supplier_code') !== false || strpos($query['selectcols'], 'preferred') !== false) {
					$query['selectcols'] .= ',perp_supplier_stock.supplier_id,supplier_name';
					$search_clause .= ' OR ' .
					"perp_supplier_stock.supplier_code LIKE $search";
				}
				$search_clause .= ")";
				$where[] = $search_clause;
			}
			if($query['searchletter']) {
				$where[] = '(stock_num LIKE \'' . $query['searchletter'] . '%\' OR
					short_description LIKE \'' . $query['searchletter'] . '%\')';
			}
			if(array_key_exists('pseudo', $query)) {
				$where['pseudo'] = (boolean)$query['pseudo'];
			}

			// Use eGW's content history to get only changed since
			if($query['history']) {
				if(!is_array($query['history']['action'])) {
					$query['history']['action'] = array($query['history']['action']);
				}
				$history = array();
				foreach($query['history']['action'] as $action) {
					$history += $GLOBALS['egw']->contenthistory->getHistory('perp_inventory:STOCK', $action, (int)$query['history']['timestamp']);
				}
				$stock_ids = array();
				foreach($history as $guid) {
					$stock_ids[] = $GLOBALS['egw']->common->get_egwId($guid);
				}
				if(count($stock_ids) > 0) {
					$where[] = 'perp_stock.stock_id IN (' . implode(',', $stock_ids) . ')';
				} else {
					$query['count'] = (int)0;
					return array();
				}
			}

			$fields = array(
				'perp_stock.stock_id'
			);
			if($query['selectcols']) {
				$fields = array_merge($fields, explode(',', str_replace('stock_id,', '', $query['selectcols'])));
			} elseif ($query['default_cols']) {
				$fields = array_merge($fields, explode(',', $query['default_cols']));
			}

			if(!in_array($query['order'], $fields) || !$query['order']) {
				$query['order'] = 'stock_num';
			}
			if(!$query['sort']) {
				$query['sort'] = 'ASC';
			}

			// Special handling for files
			if(in_array('has_attachments', $fields)) {
				$fields = array_diff($fields, array('has_attachments'));
				$has_attachments = true;
			}

			$join = '';
			if(strpos($query['selectcols'], 'supplier_code') !== false || strpos($query['selectcols'], 'preferred') !== false) {
				$fields[] = 'perp_supplier.supplier_id';
				$fields[] = 'perp_supplier.supplier_name';
				$join .= ' LEFT JOIN (
					perp_supplier_stock JOIN perp_supplier ON
						perp_supplier_stock.supplier_id = perp_supplier.supplier_id
					) ON 
					perp_stock.stock_id = perp_supplier_stock.stock_id';
			}

			// Barcode search
			if(in_array('barcode', $fields) || isset($query['barcode']) || $query['search'] != '') {
				$join .= ' LEFT JOIN perp_stock_barcode ON 
					perp_stock.stock_id = perp_stock_barcode.stock_id AND
					barcode = ' . $this->db->quote($query['search']);
			}
			
			// Inventory search
			if(in_array('inventory', $fields) || in_array('quantity_on_hand', $fields) || in_array('quantity_reserved', $fields)) {
				$fields[] = 'perp_stock.decimal_places';
				$join .= ' LEFT JOIN ( 
						SELECT stock_id,
							sum(quantity_on_hand + quantity_reserved) AS inventory,
							sum(quantity_on_hand) AS quantity_on_hand,
							sum(quantity_reserved) AS quantity_reserved
						FROM perp_location_stock ';
				if($query['col_filter']['inventory']) {
					$join .= ' WHERE location_id = ' . (int)$query['col_filter']['inventory'];
				}
				$join .= '	GROUP BY stock_id 
					) AS inventory ON
						inventory.stock_id = perp_stock.stock_id';
			}

			// Last cost
			if(in_array('last_cost', $fields) || in_array('unit_cost', $fields)) {
				$join .= ' LEFT JOIN (
					SELECT stock_id,
						quantity,
						direct_cost,
						indirect_cost,
						(direct_cost + indirect_cost) AS unit_cost,
						ROUND((direct_cost + indirect_cost) * quantity,3) AS total_cost
					FROM (perp_stock_cost JOIN (
						SELECT stock_cost_id 
						FROM perp_stock_cost
						ORDER BY date DESC
					) AS date_sort ON
						perp_stock_cost.stock_cost_id = date_sort.stock_cost_id
					)
					GROUP BY stock_id
				) AS cost ON
					cost.stock_id = perp_stock.stock_id';
			}

			// On Order
			if(in_array('on_order', $fields)) {
				unset($fields[array_search('on_order', $fields)]);
				$on_order = true;
			}

			$join .= ' JOIN egw_api_content_history ON 
					sync_appname = \'perp_inventory:STOCK\' AND 
					sync_contentid = perp_stock.stock_id';

			// Get row count
			$this->db->select('perp_stock',
				array('count(perp_stock.stock_id) AS count'),
				$where, 
				__LINE__, __FILE__, 0,
				'', 'perp_inventory', 1,
				$join
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}
	
			$this->db->select('perp_stock', 
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				"ORDER BY " . $query['order'] . ' ' . $query['sort'],
				'perp_inventory', $query['num_rows'],
				$join
			);

			$list = array();
			while($row = $this->db->row(True)) {
				if(in_array('inventory', $fields) || in_array('quantity_on_hand', $fields) || in_array('quantity_reserved', $fields)) {
					$row['inventory'] = number_format($row['inventory'], $row['decimal_places']);
					$row['quantity_on_hand'] = number_format($row['quantity_on_hand'], $row['decimal_places']);
					$row['quantity_reserved'] = number_format($row['quantity_reserved'], $row['decimal_places']);
				}

				if($query['barcodes']) {
					$row['barcodes'] = $this->get_barcodes($row['stock_id']);
				}
				if($row['sync_added']) {
					$row['sync_added'] = $GLOBALS['egw']->db->from_timestamp($row['sync_added']);
				}
				if($row['sync_modified']) {
					$row['sync_modified'] = $GLOBALS['egw']->db->from_timestamp($row['sync_modified']);
				}
				$list[] = $row;
			}

			if($has_attachments || $on_order) {
				if($has_attachments) {
					$bolink = CreateObject('phpgwapi.bolink');
				}
				if($on_order) {
					// Required because api checks permissions on current app, which will fail for perp_inventory
					$temp = $GLOBALS['egw_info']['flags']['currentapp'];
					$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_ap';

					$bo_order = new bo_perp_order();

					$on_order_query = array(
						'status'	=>	'not_received',
						'client_id'	=>	0
					);
				}
				foreach($list as $key => &$stock) {
					if($has_attachments) {
						$stock['has_attachments'] = (count($bolink->get_links('perp_inventory', 'STOCK-' . $stock['stock_id'], 'file')) > 0);
					}
					if($on_order) {
						$on_order_query['stock_id'] = $stock['stock_id'];
						$bo_order->get_order_list($on_order_query, $stock['on_order']);
					}
				}
			}

			return $list;
		}

		public function get_stock_item($stock_id) {
			$this->db->select('perp_stock', array('*'),
				array('stock_id' => $stock_id),
				__LINE__, __FILE__
			);
			$item = array();
			$item = $this->db->row(True);
			if($item['controlled'] == 2) {
				$item['serialized'] = true;
			}
			return $item;
		}

		public function set_stock_item($stock_id, $item) {
			$where = array();
			if($stock_id) {
				$where['stock_id'] = $stock_id;
			} else {
				// Check for duplicates
				$this->db->select('perp_stock', 
					array('short_description'),
					array('stock_num' => $item['stock_num']),
					__LINE__, __FILE__
				);
				$dupes = array();
				while($row = $this->db->row(True)) {
					$dupes[] = $row['short_description'];
				}
				if(count($dupes)) {
					return lang('Duplicate stock code ( %1 )', implode(', ', $dupes));
				}
			}
			$this->db->insert('perp_stock', array(
					'stock_id'		=>	$stock_id,
					'stock_num'		=>	$item['stock_num'],
					'stock_category'		=>	$item['stock_category'],
					'short_description'		=>	$item['short_description'],
					'long_description'	=>	$item['long_description'],
					'unit_of_measure'	=>	$item['unit_of_measure'],
					'status_flag'		=>	$item['status_flag'],
					'stock_type_flag'	=>	$item['stock_type_flag'],
					'pseudo'		=>	$item['pseudo'],
					'controlled'		=>	$item['controlled'],
					'serial_setup'		=>	$item['serial_setup'],
					'discount_category'	=>	$item['discount_category'],
					'tax_category'		=>	$item['tax_category'],
					'decimal_places'	=>	$item['decimal_places'],
					'ship_notes'		=>	$item['ship_notes'],
					'country_of_origin'	=>	$item['country_of_origin'],
					'usa_inbound_code'	=>	$item['usa_inbound_code'],
					'usa_returning_code'	=>	$item['usa_returning_code'],
					'canadian_inbound_code'	=>	$item['canadian_inbound_code'],
					'canadian_returning_code'	=>	$item['canadian_returning_code'],
					'international_code'	=>	$item['international_code'],
				),
				$where,
				__LINE__, __FILE__
			);
			$stock_id = $stock_id ? $stock_id : $this->db->get_last_insert_id('perp_stock', 'stock_id');
			return $stock_id;
		}

		public function delete($stock_id) {
			$tables = array(
				'perp_stock',
				'perp_stock_ship_package',
				'perp_allowed_build_option',
				'perp_bill_of_material',
				'perp_build_option_price',
				'perp_location_stock',
				'perp_price',
				'perp_order_line',
				'perp_stock_barcode',
				'perp_stock_cost',
				'perp_stock_sales_category',
				'perp_stock_serial',
				'perp_supplier_stock',
				'perp_work_order_detail',
			);

			foreach($tables as $table) {
				$this->db->delete($table,
					array($table.'.stock_id = ' . $stock_id),
					__LINE__, __FILE__
				);
			}
		}

		public function get_suppliers($stock_id) {
			$sql = 'SELECT 
					perp_supplier.supplier_id, 
					perp_supplier.supplier_num,
					perp_supplier.supplier_name, 
					perp_supplier_stock.supplier_code AS item_code,
					perp_supplier_stock.description,
					perp_supplier_stock.price,
					perp_supplier_stock.lead_time,
					perp_supplier_stock.preferred,
					perp_supplier_stock.conversion_factor,
					perp_supplier_stock.quantity,
					perp_supplier_stock.average_indirect_cost,
					perp_supplier.supplier_currency,
					perp_supplier_stock.price / (
						if(perp_supplier_stock.conversion_factor = 0,
							1,
							perp_supplier_stock.conversion_factor
						)
					) * perp_currency.exchange_rate AS our_unit_cost,
					perp_stock.unit_of_measure AS our_unit_of_measure
				FROM perp_supplier
				    JOIN perp_supplier_stock 
				        ON perp_supplier.supplier_id = perp_supplier_stock.supplier_id
				    JOIN perp_stock
				        ON perp_supplier_stock.stock_id = perp_stock.stock_id
				JOIN perp_currency ON
					perp_supplier.supplier_currency = perp_currency.currency_id
			';
			$sql .= "WHERE perp_stock.stock_id = '$stock_id'";
			$sql .= ' ORDER BY perp_supplier_stock.preferred DESC';
			$this->db->query($sql,
				__LINE__, __FILE__);
			$list = array();
			while($row = $this->db->row(True)) {
				$row['escaped_item_code'] = urlencode($row['item_code']);
				$list[] = $row;
			}
			return $list;
		}
		
		public function get_inventory($stock_id = '', $location = '') {
			$where = array();
			if($stock_id) {
				$where[] = "stock_id = '$stock_id'";
				$where[] = '(perp_location.status OR quantity_on_hand != 0 OR quantity_reserved != 0)';
			}
			if($location) {
				$where[] = "perp_location.location_id = $location";
			}
			$sql = 'SELECT 	perp_location.location_id,
					perp_location.location_name,
					perp_unit_of_measure.uom_name,
					perp_location_stock.location_id,
					perp_location_stock.quantity_on_hand,
					perp_location_stock.min_on_hand,
					perp_location_stock.max_on_hand,
					perp_location_stock.quantity_reserved
				FROM 
					perp_location 
				JOIN perp_location_stock ON 
					perp_location.location_id = perp_location_stock.location_id ';
			if(count($where)) {
				$sql .= ' AND ' . implode(' AND ', $where);
			}
			$sql .= ' JOIN perp_stock ON
						perp_location_stock.stock_id = perp_stock.stock_id
					JOIN perp_unit_of_measure ON
						perp_stock.unit_of_measure = perp_unit_of_measure.uom_id';

			$this->db->query($sql, __LINE__, __FILE__);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}

			foreach($list as $key => $location) {
				$list[$key]['total'] = $location['quantity_on_hand'] + $location['quantity_reserved'];
				$list[$key]['total'] = $list[$key]['quantity_on_hand'];
				while($row = $this->db->row(True)) {
					$list[$key]['total'] += $row['quantity_on_hand'];
				}
				$list['total'] += $list[$key]['total'];
				$list['uom_name'] = $location['uom_name'];
			}
			return $list;
		}

		public function make_serial($stock_id, $work_order, $detail_id, $quantity = 1, $location_id = 0, $stock_cost_id = null) {
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			if(!$stock['controlled']) {
				return;
			} elseif ($stock['serialized'] && $quantity > 1) {
				// Serialized items are 1 number per item
				for($i = 0; $i < $quantity; $i++) {
					$this->make_serial($stock_id, $work_order, $detail_id, 1, $location_id);
				}
			}
			$serial = ExecMethod2('perp_manufacturing.so_perp_manufacturing.get_next_serial',
				$stock['serial_setup'],
				$stock_id,
				$work_order['work_order_id'],
				$detail_id,
				true
			);

			$this->db->insert('perp_stock_serial', array(
				'stock_id'	=>	$stock_id,
				'serial_number'	=>	$serial,
				'work_order_id'	=>	$work_order['work_order_id'],
				'detail_id'	=>	$detail_id,
				'quantity'	=>	$quantity,
				'location_id'	=>	($work_order) ? $work_order['location'] : $location_id,
				'stock_cost_id'	=>	$stock_cost_id
				),
				array(
					'stock_id'	=>	$stock_id,
					'serial_number'	=>	$serial,
					'work_order_id'	=>	$work_order['work_order_id'],
					'detail_id'	=>	$detail_id,
					'quantity'	=>	$quantity
				),
				__LINE__, __FILE__
			);
			return $serial;
		}

		/**
                *	Match up the passed serial with one in table
		* 	
                *       @param stock_id
		*	@param serial_number
                */
                public function match_serial($stock_id, $serial_number) {
			$GLOBALS['egw']->db->select(
                                'perp_stock_serial',
                                array('*'),
                                array('stock_id' => $stock_id,
					'serial_number' => $serial_number),
                                __LINE__, __FILE__, 0,
                                '', 'perp_inventory'
                        );
                        $list = array();
                        while($row = $GLOBALS['egw']->db->row(true)) {
                                $list[] = $row;
                        }
                        return $list;
                }

		public function get_cost_summary($stock_id, $start_date, $end_date) {
			$where[] = 'stock_id = ' . $stock_id;
			if($start_date) {
				$where[] = 'receive_date >= ' . $GLOBALS['egw']->db->to_timestamp($start_date);
			}
			if($end_date) {
				$where[] = 'receive_date <= ' . $GLOBALS['egw']->db->to_timestamp($end_date);
			}
			$sql = 'SELECT 
					date as date,
					min(date) as start,
					max(date) as end,
					FORMAT(direct_cost, 2) as direct_cost,
					FORMAT(indirect_cost, 2) as indirect_cost,
					sum(quantity) as quantity,
					sum((direct_cost + indirect_cost) * quantity) as total_cost
				FROM
					perp_stock_cost';
			if(count($where)) {
				$sql .= ' WHERE ' . implode(' AND ', $where) . ' ' ;
			}
			$sql .= 'GROUP BY direct_cost, indirect_cost
			ORDER BY date DESC';

			$this->db->query($sql, __LINE__, __FILE__);
			$list = array();
			while($row = $this->db->row(True)) {
				$row['start'] = $GLOBALS['egw']->db->from_timestamp($row['start']);
				$row['end'] = $GLOBALS['egw']->db->from_timestamp($row['end']);
				$list['t_cost'] += $row['total_cost'];
				$list['t_quantity'] += $row['quantity'];
				$row['total_cost'] = sprintf("%.2f", $row['total_cost']);
				$list[] = $row;
			}
			$list['t_cost'] = sprintf("%.2f", $list['t_cost']);
			return $list;
		}

		public function get_cost_details(&$query) {
			if(!$query['stock_id']) {
				return array();
			}

			$where['stock_id'] = $query['stock_id'];
			if($query['date']) {
				$where['date'] = $GLOBALS['egw']->db->to_timestamp($query['date']);
			}
			if($query['as_of']) {
				$where[] = 'date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['as_of']) . "'";
			}

			// Get number of rows
			$this->db->select('perp_stock_cost',
				array('count(stock_cost_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_stock_cost', 
				array(
					'date',
					'quantity',
					'FORMAT(direct_cost, 2) AS direct_cost',
					'FORMAT(indirect_cost, 2) AS indirect_cost',
					'FORMAT(quantity * (direct_cost + indirect_cost), 2) as total_cost',
					'shipment_id',
					'line_no'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_inventory', $query['num_rows'],
				'LEFT JOIN perp_goods_received ON
					perp_goods_received.stock_cost_id = perp_stock_cost.stock_cost_id'
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$row['date'] = $GLOBALS['egw']->db->from_timestamp($row['date']);
				$list[] = $row;
				$list['total'] += $row['total_cost'];
			}
			// Has to be done seperate to avoid killing the results of the previous query
			foreach($list as &$row) {
				if($row['shipment_id']) {
					$row['reference'] = array(
						'app'	=>	'perp_ap',
						'id'		=>	'SHIPMENT-' . $row['shipment_id'],
						'title'	=>	ExecMethod('perp_ap.bo_perp_ap.link_title', 'SHIPMENT-' . $row['shipment_id'])
					);
				}
			}
			return $list;
		}

		public function set_cost_details($stock_id, $details) {
			$fields = array();
			if(isset($details['direct_cost'])) {
				$fields['direct_cost'] = $details['direct_cost'];
			}
			if(isset($details['indirect_cost'])) {
				$fields['indirect_cost'] = $details['indirect_cost'];
			}
			$where = array(
				'stock_id'	=>	$stock_id
			);
			if(isset($details['date'])) {
				$where['date'] = $details['date'];
			}
			if(isset($details['stock_cost_id'])) {
				$where['stock_cost_id'] = $details['stock_cost_id'];
			}
			if(count($where) == 1) {
				throw new egw_exception(lang('Missing enough information to change the cost.  Need either a date or stock_cost_id.'));
			}
			$this->db->update('perp_stock_cost', 
				$fields,
				$where,
				__LINE__, __FILE__
			);
		}

		public function add_sales_category($stock_id, $category) {
			$GLOBALS['egw']->db->transaction_begin();

			// Add the sales category
			$GLOBALS['egw']->db->insert('perp_stock_sales_category', 
				array(
					'sales_category_id'	=>	$category,
					'stock_id'		=>	$stock_id
				),
				array(
					'sales_category_id'	=>	$category,
					'stock_id'		=>	$stock_id
				),
				__LINE__, __FILE__, 'perp_orders'
			);

			/**
			*	Effective immediately, add this item to all active price lists
			*	that use this sales category, but set it N/A (not allowed)
			*	by default
			*/
			$query = array(
				'status'		=>	'current',
				'status_date'		=>	time(),
				'sales_category_id'	=>	$category
			);
			$prices = ExecMethod('perp_orders.bo_perp_price_list.get_price_list_list', $query);
			foreach($prices as $price_list_id => $name) {
				ExecMethod2('perp_orders.bo_perp_price.set_price', $price_list_id, $stock_id, so_perp_price::NOT_ALLOWED, time());
			}
			$GLOBALS['egw']->db->transaction_commit();
		}

		public function delete_sales_category($stock_id, $category) {
			$GLOBALS['egw']->db->transaction_begin();
			$this->db->delete('perp_stock_sales_category',
				array(
					'sales_category_id'	=>	$category,
					'stock_id'		=>	$stock_id
				),
				__LINE__, __FILE__, 'perp_orders'
			);

			/**
			*	Effective immediately, set this item to N/A (not allowed on
			*	all active price lists that use this sales category
			*/
			$query = array(
				'status'		=>	'current',
				'status_date'		=>	time(),
				'sales_category_id'	=>	$category
			);
			$prices = ExecMethod('perp_orders.bo_perp_price_list.get_price_list_list', $query);
			foreach($prices as $price_list_id => $name) {
				ExecMethod2('perp_orders.bo_perp_price.set_price', $price_list_id, $stock_id, so_perp_price::NOT_ALLOWED, time());
			}
			$GLOBALS['egw']->db->transaction_commit();
		}

		public function get_sales_categories($stock_id) {
			$this->db->select('perp_stock_sales_category', array(
					'perp_stock_sales_category.sales_category_id',
					'cat_name AS sales_category_name',
					'cat_data',
					'cost_of_sales_gl_account',
					'sales_gl_account'
				),
				array('stock_id' => $stock_id),
				__LINE__, __FILE__, 'perp_orders',
				'', False, 0,
				'JOIN perp_sales_category ON
					perp_sales_category.sales_category_id = perp_stock_sales_category.sales_category_id
				JOIN egw_categories ON
					perp_sales_category.sales_category_id = egw_categories.cat_id'
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function get_tax_categories($stock_id) {
			$sql = 'SELECT 
					perp_tax_authority.tax_authority_id,
					perp_tax_authority.tax_authority_name,
					perp_stock_tax_category.tax_category_id
				FROM perp_tax_authority
				LEFT JOIN perp_stock_tax_category ON
					perp_stock_tax_category.tax_authority_id = perp_tax_authority.tax_authority_id AND
					perp_stock_tax_category.stock_id = ' . $stock_id;
			$this->db->query($sql, __LINE__, __FILE__);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function set_tax_category($stock_id, $tax_authority_id, $tax_category_id) {
			$this->db->insert('perp_stock_tax_category', 
				array(
					'stock_id'		=>	$stock_id,
					'tax_authority_id'	=>	$tax_authority_id,
					'tax_category_id'	=>	$tax_category_id
				),
				array(
					'stock_id'		=>	$stock_id,
					'tax_authority_id'	=>	$tax_authority_id,
				),
				__LINE__, __FILE__
			);
		}

		public function get_price_list_list($stock_id, $date) {
			$date = $GLOBALS['egw']->db->to_timestamp($date);

			$result =& $GLOBALS->db->select('perp_price_list', array(
					'perp_price_list.price_list_id',
					'cat_name AS sales_category_name',
					'sales_area_name',
					'sales_type_name',
					'price_list_name',
				),
				array(
					'stock_id = ' . $stock_id,
					"effective_from <= '" . $date . "'"
				),
				__LINE__, __FILE__, 'perp_orders',
				'', False, 0,
				'NATURAL JOIN perp_sales_category
				JOIN egw_categories ON
					perp_sales_category.sales_category_id = egw_categories.cat_id
				NATURAL JOIN perp_sales_type
				NATURAL JOIN perp_sales_area
				NATURAL JOIN perp_stock_sales_category'
			);
			
			$list = array();
			foreach($result as $row) {
				$key = $row['price_list_id'];
				$name = $row['price_list_name'];
				unset($row['price_list_id']);
				unset($row['price_list_name']);
				$list[$key] = implode(' ', $row) . ' - ' . $name;
			}
			return $list;
		}

		public function get_full_option_list($stock_id, $bom_id = '') {
			$sql = '
				SELECT build_option_id,
					perp_allowed_build_option.stock_id,
					component,
					build_option_category_name,
					build_option_name
				FROM perp_allowed_build_option
					NATURAL JOIN perp_build_option
					JOIN perp_build_option_category ON
						perp_build_option_category.build_option_category_id = perp_build_option.build_option_category
					JOIN perp_stock ON
						perp_stock.stock_id = perp_allowed_build_option.component
				WHERE perp_allowed_build_option.stock_id = ' . $stock_id;
			$this->db->query($sql, __LINE__, __FILE__);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}

			foreach($list as $key => $row) {
				$list = array_merge($list, $this->get_full_option_list($row['component']));
			}

			// Look for options on components
			if(!$bom_id) {
				// Get current bom
				$bom_id = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $stock_id);
			}

			$bom = ExecMethod('perp_production.bo_perp_bill_of_material.get_bill_of_material', $bom_id);
			if($bom) {
				foreach($bom['details'] as $detail) {
					$list += $this->get_full_option_list($detail['component']);
				}
			}
			return $list;
		}

		public function suggest_stock_number($stock_item) {
			$config = config::read('perp_inventory');
			$number_format = $config['stock_number_format'];

			// Get next stock id
			if($stock_item['stock_id']) {
				$stock_id = $stock_item['stock_id'];
			} else {
				$stock_id = 0;
				$sql = "show table status LIKE 'perp_stock'";
				$this->db->query($sql, __LINE__, __FILE__);
				if($row = $this->db->row(true)) {
					$stock_id = $row['Auto_increment'];
				}
			}

			// Stock category
			if($stock_item['stock_category']) {
				$category = bo_perp_stock_category::get_category($stock_item['stock_category']);
				if(count($category) == 1) {
					$category = $category[0];
				}
			}

			// Status flag is hidden if P
			if($stock_item['status_flag'] == 'P') {
				preg_match('/%4\$.*s/', $number_format, $regs);
				if(count($regs)) {
					$number_format = str_replace($regs[0], '', $number_format);
				}
			}
			$stock_number = sprintf($number_format,
				$stock_id,
				$stock_item['stock_category'],
				$category['name'],
				$stock_item['status_flag'],
				$stock_item['status_flag']
			);
			return $stock_number;
		}

		/**
		*	Get the list of shipping packages.
		*	There's usually only 1 entry, but if something is shipped
		*	in several boxes, there will be 1 entry for each box.
		*/
		public function get_shipping_packages($stock_id) {
			$GLOBALS['egw']->db->select(
				'perp_stock_ship_package',
				array(
					'*',
					'round(length * width * height, 3) AS volume_m'
				),
				array('stock_id'	=>	$stock_id),
				__LINE__, __FILE__,0,
				'ORDER BY mass DESC',
				'perp_inventory'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				// Calculate volume
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Set the list of shipping packages for a stock item.
		*	Clears the list, then sets it again.
		*/
		public function set_shipping_packages($stock_id, $packages) {
			$GLOBALS['egw']->db->delete(
				'perp_stock_ship_package',
				array('stock_id'	=>	$stock_id),
				__LINE__, __FILE__,
				'perp_inventory'
			);
			foreach($packages as $package) {
				$GLOBALS['egw']->db->insert(
					'perp_stock_ship_package',
					array('stock_id' => $stock_id) + $package,
					array(),
					__LINE__, __FILE__,
					'perp_inventory'
				);
			}
		}

		/**
		*	Get a list of patterns for a stock item
		*
		*	@param stock_id
		*/
		public function get_serial_patterns($stock_id) {
			$GLOBALS['egw']->db->select(
				'perp_stock_serial_pattern',
				array('*'),
				array('stock_id' => $stock_id),
				__LINE__, __FILE__, 0,
				'', 'perp_inventory'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Set the list of patterns for a stock item.
		*	Clears the list, then sets it again.
		*/
		public function set_serial_patterns($stock_id, $patterns) {
			
			$GLOBALS['egw']->db->delete(
                                'perp_stock_serial_pattern',
                                array('stock_id' => $stock_id),
                                __LINE__, __FILE__,
                                'perp_inventory'
                        );
			
			foreach($patterns as $key => $pattern) {
                                if(trim($pattern['pattern']) == '' || $pattern['pattern'] == lang('new pattern')) {
                                        unset($pattern[$key]);
                                        continue;
                                }
				$pattern_regular_expression = str_replace($this->pattern_legend, $this->regex_legend, $pattern['pattern']);		
				$GLOBALS['egw']->db->insert(
					'perp_stock_serial_pattern',
					array('stock_id' => $stock_id,'pattern' => $pattern_regular_expression),
					array(),
					__LINE__, __FILE__,
					'perp_inventory'
				);
			}
		}

		/**
		/**
		*	Get a list of barcodes for a stock item
		*
		*	@param stock_id
		*/
		public function get_barcodes($stock_id) {
			$GLOBALS['egw']->db->select(
				'perp_stock_barcode',
				array('*'),
				array('stock_id' => $stock_id),
				__LINE__, __FILE__, 0,
				'', 'perp_inventory'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Set the list of barcodes for a stock item.
		*	Clears the list, then sets it again.
		*/
		public function set_barcodes($stock_id, &$barcodes) {
			$GLOBALS['egw']->db->delete(
				'perp_stock_barcode',
				array('stock_id'	=>	$stock_id),
				__LINE__, __FILE__,
				'perp_inventory'
			);
			foreach($barcodes as $key => $barcode) {
				if(trim($barcode['barcode']) == '' || $barcode['barcode'] == lang('new barcode')) {
					unset($barcodes[$key]);
					continue;
				}
				$GLOBALS['egw']->db->insert(
					'perp_stock_barcode',
					array('stock_id' => $stock_id) + $barcode,
					array(),
					__LINE__, __FILE__,
					'perp_inventory'
				);
			}
		}

		/**
		*	When a stock item's category changes, and there is inventory in stock, it is possible
		*	that the new stock category uses a different general ledger account than the old
		*	stock category.  The value of the inventory needs to be moved from the old general
		*	ledger account to the new general ledger account so the value of stock according to
		*	the general ledger is correctly maintained.  This needs to be done for every 
		*	inventory location, because different inventory locations may use different general
		*	ledger accounts for the same stock category.
		*
		*	@param stock_id
		*	@param old_category
		*	@param new_category
		*
		*/
		public static function move_stock_value($stock_id, $old_category, $new_category) {
			
			$GLOBALS['egw']->db->transaction_begin();

			// First we need the GL accounts for the categories at every location
			$result =& $GLOBALS['egw']->db->query(
				'SELECT perp_location.location_id, perp_location.location_name,
					old.stock_gl_account AS "old", new.stock_gl_account AS "new",
					perp_location_stock.quantity_on_hand + perp_location_stock.quantity_reserved AS "quantity"
				FROM perp_location
				JOIN perp_location_gl_account AS old ON
					perp_location.location_id = old.location_id 
					AND old.stock_category_id = ' . (int)$old_category . '
				LEFT JOIN perp_location_gl_account AS new ON
					perp_location.location_id = new.location_id
					AND new.stock_category_id = ' . (int)$new_category . '
				JOIN perp_location_stock ON
					perp_location.location_id = perp_location_stock.location_id
					AND perp_location_stock.stock_id = ' . (int)$stock_id . '
				WHERE old.stock_gl_account 
					AND new.stock_gl_account
					/* Do not bother if the accounts are the same */
					AND old.stock_gl_account != new.stock_gl_account
					/* Do not bother if there is no inventory */
					AND perp_location_stock.quantity_on_hand + perp_location_stock.quantity_reserved != 0'
			);
			$accounts = array();
			foreach($result as $row) {
				if(!$row['new']) {
					throw new egw_exception(lang('Missing general ledger account for %1 at %2',
						$GLOBALS['egw']->categories->id2name($new_category),
						$row['location_name']
					));
				}
				$accounts[] = $row;
			}

			// A GL transaction to move the value
			$trans = CreateObject('perp_ledger.GLTransaction');

			// Next we need the total value of the stock item at each location
			$bo_inventory = CreateObject('perp_inventory.bo_perp_inventory');
			foreach($accounts as &$location) {
				$location['cost'] = $bo_inventory->get_cost($stock_id, $location['location_id'], $location['quantity']);
				$narrative = lang('%1 changed category from %2 to %3, moved %4 @ %5 at %6',
					ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id),
					$GLOBALS['egw']->categories->id2name($old_category),
					$GLOBALS['egw']->categories->id2name($new_category),
					$location['quantity'],
					ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $location['cost']),
					$location['location_name']
				);
				$trans->add_line($location['old'], $narrative, -$location['cost']);
				$trans->add_line($location['new'], $narrative, $location['cost']);
			}

			$trans->write();

			$GLOBALS['egw']->db->transaction_commit();
		}

		/**
		*	Get GL accounts associated with a particular stock item
		*/
		public static function get_gl_accounts($stock_id, Array $account_uses) {
			$gl_accounts = array();
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			$defaults = bo_perp_stock_category::get_category($stock_item['stock_category']);

			$result = $GLOBALS['egw']->db->select(
				'perp_stock_gl_account',
				array(
					'account_use',
					'gl_account'
				),
				array(
					'stock_id'	=>	$stock_id,
					'account_use'	=>	$account_uses,
				),
				__LINE__, __FILE__, false,
				'', 'perp_inventory'
			);
			$stock_accounts = array();
			foreach($result as $row) {
				$stock_accounts[$row['account_use']] = $row['gl_account'];
			}

			// Set the ones that are needed for this item
			if(!$stock_item['pseudo']) {
				$gl_accounts[bo_perp_inventory::ON_HAND] = $stock_accounts[bo_perp_inventory::ON_HAND] ? 
					$stock_accounts[bo_perp_inventory::ON_HAND] : $defaults['on_hand_gl_account'];
				$gl_accounts[bo_perp_inventory::SHRINKAGE] = $stock_accounts[bo_perp_inventory::SHRINKAGE] ? 
					$stock_accounts[bo_perp_inventory::SHRINKAGE] : $defaults['shrinkage_gl_account'];
				$gl_accounts[bo_perp_inventory::ADJUSTMENT] = $stock_accounts[bo_perp_inventory::ADJUSTMENT] ? 
					$stock_accounts[bo_perp_inventory::ADJUSTMENT] : $defaults['adjustment_gl_account'];
			}
			if($stock_item['stock_type_flag'] == 'A') {
				$gl_accounts[bo_perp_inventory::WORK_IN_PROGRESS] = $stock_accounts[bo_perp_inventory::WORK_IN_PROGRESS] ? 
					$stock_accounts[bo_perp_inventory::WORK_IN_PROGRESS] : $defaults['work_in_progress_gl_account'];
			}
			

			return $gl_accounts;
		}

		/**
		*	Set GL accounts associated with a particular stock item
		*/
		public static function set_gl_accounts($stock_id, $accounts) {

			// Clear out old ones
			$GLOBALS['egw']->db->delete(
				'perp_stock_gl_account',
				array('stock_id' => $stock_id),
				__LINE__, __FILE__, 'perp_inventory'
			);
			foreach($accounts as $use => $gl_account) {
				$GLOBALS['egw']->db->insert(
					'perp_stock_gl_account',
					array(
						'stock_id'	=>	$stock_id,
						'account_use'	=>	$use,
						'gl_account'	=>	$gl_account
					),
					array(),
					__LINE__, __FILE__, 'perp_inventory'
				);
			}
		}
	}
?>
