<?php

	/**
	*	User interface for counting and ajusting inventory quantities.
	*
	*
	*	@author Nathan Gray
	*/

	require_once(EGW_INCLUDE_ROOT . '/etemplate/inc/class.etemplate.inc.php');
	require_once('class.bo_perp_count.inc.php');

	class ui_perp_count {
		
		public $public_functions = array(
			'count_list'	=>	true,
			'count_new'	=>	true,
			'edit'		=>	true,
			'edit_sheet'	=>	true,
			'serial_numbers'	=>	true
		);

		private $template;
		private $bo;

		private $debug = false;

		private $messages = array();

		public function __construct() {
			$this->template =& new etemplate();
			$this->bo =& new bo_perp_count();
		}

		public function count_list(Array $content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}

			$can_add = $GLOBALS['egw']->acl->check('stock_count', EGW_ACL_ADD, 'perp_inventory');

			if(isset($content['new_count']) && $can_add) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.count_new'
				));
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/perp_inventory/index.php');
			} elseif(isset($content['nm']['rows']['delete'])) {
				$this->bo->delete_count(key($content['nm']['rows']['delete']));
			}

			$data['nm'] = array(
				'template'	=>	'perp_inventory.count_list.row',
				'get_rows'	=>	'perp_inventory.bo_perp_count.get_count_list',
				'filter_label'	=>	lang('Status'),
				'no_cat'	=>	true,
				'no_filter2'	=>	true,
				'filter'	=>	'current'
			);
			if($content['nm']) {
				$data['nm'] += $content['nm'];
			} 

			$select_box_options['filter'] = array(
				''		=>	'all',
				'current'	=>	'current',
				'previous'	=>	'previous'
			);

			$readonly['new_count'] = !$can_add;

			$GLOBALS['egw_info']['flags']['app_header'] = lang('Count List');
			$this->template->read('perp_inventory.count_list');
			$this->template->exec('perp_inventory.ui_perp_count.count_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function count_new(Array $content = array()) {
			// JavaScript
                        if(!is_object($GLOBALS['egw']->js)) {
                                $GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
                        }
                        //add in suport for a select all button for a multi-selectbox
                        $GLOBALS['egw']->js->validate_file('api', 'multi_select', 'perp_api');
			
			if($this->debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}
			$GLOBALS['egw']->session->appsession('count_changes', 'perp_inventory', null);

			// Deal with incoming
			if($content && !$content['locations']) {
				$this->messages[] = array(lang('You should choose at least one location to count'), 'warning');
				unset($content['save']);
			}
			if($content && !$content['categories']) {
				$this->messages[] = array(lang('You should choose at least one stock category to count'), 'warning');
				unset($content['save']);
			}

			// Create count
			if($content['save'] && $GLOBALS['egw']->acl->check('stock_count', EGW_ACL_ADD, 'perp_inventory')) {
				if($content['locations']) {
					$content['locations'] = explode(',', $content['locations']);
				}
				$count_id = $this->bo->new_count($content);
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.edit',
					'count_id'	=>	$count_id
				));
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.count_list'
				));
			}

			// Set up outgoing (defaults)
			$data['supervisor'] = $GLOBALS['egw_info']['user']['account_id'];
			$data['count_date'] = date('U');

			$select_box_options['locations'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list');
			$select_box_options['pre_fill'] = array(
				''		=>	lang('Do not pre-fill'),
				'0'		=>	lang('Fill sheets with 0'),
				'expected'	=>	lang('Fill sheets with expected quantity')
			);
			$select_box_options['include_items'] = array(
				'currently_there'	=>	lang('that are currently in stock'),
			//	'ever_there'		=>	lang('that were ever in stock'),
			//	'last_year'		=>	lang('that were in stock in the last year'),
				'in_categories'		=>	lang('all items'),
			);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$GLOBALS['egw']->js->validate_file('stock', 'new_count', 'perp_inventory');

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('New Inventory Count');
			$this->template->read('perp_inventory.count_new');
			$this->template->exec('perp_inventory.ui_perp_count.count_new', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Edit a stock counting
		*/
		public function edit(array $content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}
			$count_id = get_var('count_id', 'any', $content['count_id']);

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->session->appsession('count_changes', 'perp_inventory');
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.count_list'
				));
			} elseif (isset($content['save'])) {
				$result = $this->bo->make_changes($count_id);
				$GLOBALS['egw']->session->appsession('count_changes', 'perp_inventory', null);
				if($result !== true) {
					$this->messages[] = array($result, 'error');
				} else {
					$this->messages[] = array(lang('Inventory has been adjusted'), 'success');
				}
			}

			$data = $this->bo->get_count($count_id);
			$data['status'] = isset($content['status']) ? $content['status'] : 'not_completed';
			$sheets = array();
			$this->bo->get_sheet_list(array(
				'count_id'	=>	$count_id,
				'status'	=>	$data['status'],
				'num_rows'	=>	-1
			), $sheets);
			if(count($sheets)) {
				$data['sheets'] = $sheets;

				// make room for column headers
				array_unshift($data['sheets'], false);
			}

			$changes = $GLOBALS['egw']->session->appsession('count_changes', 'perp_inventory');
			if($changes) {
				$data['changes'] = $changes;
			} else {
				$data['changes'] = array(
					'get_rows'	=>	'perp_inventory.ui_perp_count.get_change_list',
					'template'	=>	'perp_inventory.count_edit.row',
					'no_filter'	=>	true,
					'no_filter2'	=>	true,
					'count_id'	=>	$count_id
				);
			}

			$select_box_options['location_id'] =& ExecMethod('perp_inventory.bo_perp_location.get_location_list');
			$data['changes']['options-location_id'] =& $select_box_options['location_id'];
			$select_box_options['status'] = array(
				''		=>	lang('all'),
				'completed'	=>	lang('completed'),
				'not_completed'	=>	lang('not completed')
			);

			$preserve['count_id'] = $count_id;
			$preserve['status'] = $content['status'];

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Readonlys
			$sheet_check = $this->bo->get_sheet_list(array('count_id' => $count_id, 'status' => 'not_completed'));
			$readonly['save'] = ($sheet_check != 0) || 
				!$GLOBALS['egw']->acl->check('stock_count', EGW_ACL_EDIT, 'perp_inventory') || 
				$data['finish_date'] ||
				$GLOBALS['egw_info']['user']['account_id'] != $data['supervisor'];

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Inventory Count #%1 %2', $count_id, date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $data['count_date']));
			$this->template->read('perp_inventory.count_edit');
			$this->template->exec('perp_inventory.ui_perp_count.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Get a list of changes that need to be made to correct the stock counts
		*/
		public function get_change_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$result_count = $this->bo->get_change_list($query, $rows, $readonlys);

			$count = $this->bo->get_count($query['count_id']);

			$messages = array();
			foreach($rows as &$row) {
				if(is_null($count['finish_date'])) {
					if($row['message']) {
						$messages[] = $row['message'];
						$row['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $row['message'], 'error');
					}
				} else {
					unset($row['message']);
				}
			}
			$messages = array_unique($messages);
			foreach($messages as &$message) {
				$message = array($message, 'error');
			}
			$this->messages += $messages;

			return $result_count;
		}

		/**
		*	Edit a count sheet
		*
		*	Actually enter the numbers of items found
		*/
		public function edit_sheet(array $content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}
			$sheet_id = get_var('sheet_id', 'any', $content['sheet_id']);

			// Update
			if(isset($content['save'])) {
				$this->bo->set_sheet($content);
			}

			// User wants to check / set serials?
			if(isset($content['details']['set_serial_numbers'])) {
				// Save work first
				$this->bo->set_sheet($content);

				$stock_id = key($content['details']['set_serial_numbers']);
				
				// Redirect to serial numbers page
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.serial_numbers',
					'sheet_id'	=>	$content['sheet_id'],
					'stock_id'	=>	$stock_id
				));
			}

			$sheet = $this->bo->get_sheet($sheet_id);
			if(isset($content['save']) || isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.edit',
					'count_id'	=>	$sheet['count_id']
				));
			}

			if(is_array($sheet['details'])) {
				// Make room for row heading
				array_unshift($sheet['details'], false);
			}
			$count = $this->bo->get_count($sheet['count_id']);
			$location_name = ExecMethod('perp_inventory.bo_perp_location.link_title', $sheet['location_id']);
			$data = $sheet;

			// This is important: Inventory valuation is in your GL currency, NOT your supplier's currency
			$config = config::read('perp_ledger');
			$data['details']['gl_currency'] = $config['gl_currency'];
	
			$preserve = $data;

			// Readonlys
			$readonly['save'] = !$GLOBALS['egw']->acl->check('stock_count', EGW_ACL_EDIT, 'perp_inventory') || 
				$count['finish_date'];
			// Hide serial numbers button for non-controlled items
			foreach($data['details'] as $row => $detail) {
				if($detail['stock_id'] && !$detail['need_serials']) {
					$readonly['set_serial_numbers[' . $detail['stock_id'] . ']'] = !$detail['need_serials'];
				}
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Inventory Count #%1 %2 - %4 - Sheet #%3', 
				$sheet['count_id'], 
				date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $count['count_date']),
				$sheet_id,
				$location_name
			);
			$this->template->read('perp_inventory.sheet_edit');
			$this->template->exec('perp_inventory.ui_perp_count.edit_sheet', $data, $select_box_options, $readonly, $preserve);
		}

		public function serial_numbers(array $content = array()) {
			$sheet_id = get_var('sheet_id', 'any', $content['sheet_id']);
			$stock_id = get_var('stock_id', 'any', $content['stock_id']);
			
			// Deal with incoming
			if($content['save']) {
				if(is_array($content['unexpected'])) {
					$serials = array_merge($content['serial_numbers'], $content['unexpected']);
				} else {
					$serials = $content['serial_numbers'];
				}

				$result = $this->bo->set_serial_numbers($sheet_id, $stock_id, $serials);
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_inventory.ui_perp_count.edit_sheet',
					'sheet_id'	=>	$sheet_id
				));
			}

			// Get data for display
			$data = array(
				'sheet_id'	=>	$sheet_id
			);

			// Get serial numbers
			$serial_numbers = $this->bo->get_serial_numbers($sheet_id, $stock_id);
			
			// Move some sheet info
			$data['location_id'] = $serial_numbers[0]['location_id'];
			$data['quantity_expected'] = (float)$serial_numbers[0]['total_expected'];
			$data['quantity_counted'] = (float)$serial_numbers[0]['total_counted'];
			$data['serial_numbers'] = $serial_numbers;
		
			// Get the stock item info
			$data += ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			
			// Set up checkboxes for serialized items
			if($data['controlled'] == 2 && is_array($data['serial_numbers'])) {
				foreach($data['serial_numbers'] as &$serial) {
					
					$serial['quantity_counted'] = ($serial['quantity_counted'] > 0) ? 1 : 0;
				}
			}
			
			// Shift 1 to allow for title
			array_unshift($data['serial_numbers'], false);

			// Need to move this in 1 level to disable columns
			$data['serial_numbers']['controlled'] = $data['controlled'];


			// Counted more than expected, or different serial from expected?  Add some lines to record the unexpected
			$quantity_unexpected = max(1, $data['quantity_counted'] - $data['quantity_expected']);
			
			// Be reasonable here, don't show hundreds of empty rows for a batch.
			$quantity_unexpected = min($quantity_unexpected, $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs']);

			$data['unexpected'] = array_fill(1, $quantity_unexpected, 
				array(
					'serial_number'		=>	'',
					'quantity_expected'	=>	0,
					'quantity_counted'	=>	($data['controlled'] == 2) ? 1 : ''
				)
			);
			// Need to move this in 1 level to disable columns
			$data['unexpected']['controlled'] = $data['controlled'];

			$preserve = $data;

			// Display
			$location_name = ExecMethod('perp_inventory.bo_perp_location.link_title', $data['location_id']);
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Sheet #%1 - %2 Serial Numbers for %3', 
				$sheet_id,
				$location_name,
				$data['short_description']
			);
			$this->template->read('perp_inventory.count_serial_numbers');
			$this->template->exec('perp_inventory.ui_perp_count.serial_numbers', $data, $select_box_options, $readonly, $preserve);
		}
	}
?>
