<?php
	/**
	*	Record interesting actions done to serialized stock items.
	*	We can only do this for items with a serial or batch number, because
	*	without it it's impossible to say which item something happened to
	*/
	
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_damage.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_stock_history.inc.php');

	@include_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_build_option.inc.php');
	include_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_retrofit.inc.php');

	class ui_perp_stock_history {
		
		public $public_functions = array(
			'stock_history_list'	=>	True,
		);

		private $debug = false;
		private $template;
		private $messages = array();

		protected $tabs = 'retrofit|damage';

		/**
		*	Buttons to take action on
		*/
		protected $action_buttons = array(
			'add_retrofit'	=>	true,
			'add_damage'	=>	true,
		);

		public function ui_perp_stock_history() {
			$this->template = CreateObject('etemplate.etemplate');
		}

		public function stock_history_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php?menuaction=perp_inventory.ui_perp_inventory.controlled_list');
			} elseif ($action = array_intersect_key($this->action_buttons, $content)) {
				try {
					self::handle_action(key($action), $content);
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			} else {
				foreach(explode('|', $this->tabs) as $tab) {
					try {
						if(is_array($content[$tab]) && $action = array_intersect_key($this->action_buttons, $content[$tab])) {
							self::handle_action(key($action), $content);
						}
					} catch (Exception $e) {
						$this->messages[] = array($e->getMessage(), 'error');
					}
				}
			}
			
			$stock_id = $content['stock_id'] ? $content['stock_id'] : $_GET['stock_id'];
			$serial = $content['serial_number'] ? $content['serial_number'] : $_GET['serial'];

			$data['nm'] = array(
				'stock_id'	=>	$stock_id,
				'serial_number'	=>	$serial,

				'no_cat'        =>      True,
				'no_filter'     =>      True,
				'filter_label'  =>      'Location',
				'filter_no_lang'        =>      True,
			//	'no_filter2'    =>      True,
				'filter2_label' =>      'Work Center',
				'filter2_no_lang'       =>      True,
				'get_rows'      =>      'perp_inventory.bo_perp_stock_history.get_stock_history_list',
				'template'      =>      'perp_inventory.stock_history_list.row',
				'order'		=>	'timestamp',
				'sort'		=>	'DESC'
			);
			$data['filter'] = $content['filter'];
			$data['filter2'] = $content['filter2'];

			$serial_data = array();
			$query = array(
				'stock_id'	=>	$stock_id,
				'serial_number'	=>	$serial,
			);
			$bo_inventory = CreateObject('perp_inventory.bo_perp_inventory');
			$bo_inventory->get_controlled_inventory_list($query, $serial_data);
			if(count($serial_data) == 1) {
				$serial_data = $serial_data[0];
				$data += $serial_data;

				$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $serial_data['work_order_id']);
				$data += $work_order;
			} else {
				unset($serial_data);
			}

			$select_box_options = self::get_selectbox_options($data);

			$readonly['add_retrofit'] = !$GLOBALS['egw']->acl->check('retrofit', EGW_ACL_ADD, 'perp_manufacturing');
			$readonly[$this->tabs]['retrofit'] = $readonly['add_retrofit'] || count($select_box_options['build_option_id']) == 0;

			$this->setup_damage($data, $select_box_options, $readonly);


			$preserve['stock_id']		=	$stock_id;
			$preserve['serial_number']	=	$serial;
			$preserve['work_order_id']	=	$data['work_order_id'];
			$preserve['detail_id']		=	$data['detail_id'];

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Stock History %1 - %2', $serial_data['stock_num'], $serial);
			$this->template->read('perp_inventory.stock_history_list');
			$this->template->exec('perp_inventory.ui_perp_stock_history.stock_history_list', $data, $select_box_options, $readonly, $preserve);
		}

		protected static function get_selectbox_options(Array &$data) {
			$options = array();

			$work_centers = ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center_list');
			$options['filter2'] = array('' => 'All') + $work_centers;
			$options['work_center'] = $work_centers;

			// Retrofits
			$bo_build_option = new bo_perp_build_option();
			$options['build_option_category_id'] = $bo_build_option->get_build_option_category_list();
			$query = array(
				'num_rows'	=>	-1,
				'template'	=>	true,
				'by_category'	=>	true,
				'can_retrofit'	=>	true,
			);
			$bo_build_option->get_build_option_list($query, $build_options);
			foreach($build_options as $category) {
				if(!is_array($category['options'])) {
					continue;
				}
				foreach($category['options'] as $build_option) {
					$options['build_option_id'][$category['build_option_category_name']][$build_option['build_option_id']] = $build_option['build_option_name'];
				}
			}

			return $options;
		}

		/**
		*	Handle actions
		*	Actions must be listed in action_buttons to get here
		*/
		protected static function handle_action($action, $content) {
			switch ($action) {
				case 'add_retrofit':
					$retrofit = array(
						'stock_id'		=>	$content['stock_id'],
						'serial_number'		=>	$content['serial_number'],
						'build_option_id'	=>	$content['build_option_id'],
						'work_order_id'		=>	$content['work_order_id'],
						'detail_id'		=>	$content['detail_id'],
						'reference'		=>	$content['reference'],
						'account_id'		=>	$GLOBALS['egw_info']['user']['account_id'],
						'link'			=>	$content['link'],
					);
					$result = bo_perp_retrofit::set_retrofit($retrofit);
					break;
				case 'add_damage':
					$damage = array(
						'work_order_id'		=>	$content['work_order_id'],
						'detail_id'		=>	$content['detail_id'],
						'stock_id'		=>	$content['stock_id'],
						'serial_number'		=>	$content['serial_number'],
						'quantity_damaged'	=>	$content['damage']['quantity_damaged'],
						'comment'		=>	$content['damage']['description'],
						'discount_percent'	=>	$content['damage']['discount_percent'] / 100.0,
						'link'			=>	$content['damage']['link'],
					);
					$damage_id = bo_perp_damage::set_damage($damage);

					break;
			}
		}

		protected function setup_damage(&$data, &$options, &$readonly) {
			if(!$GLOBALS['egw']->acl->check('damage', EGW_ACL_READ, 'perp_inventory')) {
				$readonly[$this->tabs]['damage'] = true;
				return;
			} elseif(!$GLOBALS['egw']->acl->check('damage', EGW_ACL_ADD, 'perp_inventory')) {
				$readonly['damage']['add_damage'] = true;
			}

			$data['damage']['link']['app'] = 'infolog';
			$data['damage']['link']['to_app'] = 'perp_inventory';
			$data['damage']['quantity_damaged'] = 1;
		}
	}
?>
