<?php

	/**
	*	Business or Logic layer for General Ledger.
	*	@author Nathan Gray
	*	@note GLTransaction skips this and takes care of itself, in accordance with OO programming practices.
	*/
	class bo_perp_ledger {
		
		/**
		*	Reference to the Storage Object for database action
		*/
		var $so;

		/**
		*	boolean output extra debug information
		*/
		var $debug = False;

		/**
		*	Cache account names to try to speed up AJAX lists
		*/
		protected static $account_name_cache = array();

		/**
		*	Constructor\n
		*	Creates reference to storage object
		*/
		function bo_perp_ledger() {
			$this->so = CreateObject('perp_ledger.so_perp_ledger');
		}

		/**
		*	Get a list of General Ledger transaction types
		*
		*	@nextmatch
		*/
		public function get_transaction_types($query = array(), &$rows = array(), &$readonlys =array()) {
			$read = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_READ, 'perp_ledger');

			$row_start = $query['start'];
			if($query['search']) {
				$where[] = 'trans_id = \''.$query['search'] .'\' OR trans_name LIKE \'%' . 
					$query['search'] . "%'";
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}
			$list =  $this->so->get_transaction_types($where);
			$count = count($list);

			// No parameters, used for a select box.  Give only key => name pairs
			if(count($query) == 0) {
				return $read ? $list : array();
			} 
			
			$rows = array();

			if(!isset($query['template'])) {
				foreach($list as $key => $value) {
					$rows[$value['trans_id']] = $value['trans_name'];
				}
				return $rows;
			}
			$rows = $read ? array_slice($list, $query['start'], $query['num_rows']) : array();

			if (!isset($query['rows'])) {
				// Parameters, return full data
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_DELETE);
				$rows += array('add' => $add, 'edit'=>$edit, 'delete'=>$delete);
			}

			return $count;
		}

		/**
		*	Get a list of General Ledger account sections 
		*	Sections are the second highest level of account grouping
		*	@nextmatch
		*/
		public function get_account_sections($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_READ, 'perp_ledger');
			if(!$read) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_account_sections($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['section_id']] = $value['section_name'];
				}
				return $select;
			}
			
			if(!isset($query['rows'])) {
				// Parameters, return full data
				$rows = array();
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_DELETE);
				$rows = array('add' => $add, 'edit'=>$edit, 'delete'=>$delete);
			}
			$rows += $list;

			return $query['count'];
		}

		/**
		*	Get a list of General Ledger account groups.  Groups are the third highest, or lowest,
		*	level of account grouping.
		*
		*	@nextmatch
		*/
		public function get_account_groups($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('account_group', EGW_ACL_READ, 'perp_ledger');
			if(!$read) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_account_groups($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $row) {
					$select[$row['group_id']] = $row['group_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters, return full data
				$rows = array();
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_DELETE);
				$rows = array('add' => $add, 'edit'=>$edit, 'delete'=>$delete);
			}

			$rows += $list;

			return $query['count'];
		}

		/**
		*	Get a list of General Ledger accounts
		*
		*	@nextmatch
		*/
		public function get_accounts($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('account', EGW_ACL_READ, 'perp_ledger');
			if(!$read) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}
			$list = $this->so->get_accounts($query);

			// For select boxes etc.
			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['account_id']] = $value['account_id'] . ' - ' . $value['account_name'];
				}
				return $select;
			}


			foreach($list as &$row) {
				$row['current_balance'] = ExecMethod2(
					'perp_api.bo_perp_api.get_formatted_price', null, 
					$this->get_balance(time(), $row['account_id'])
				);
			}

			if(!isset($query['rows'])) {
				// return full data
				$rows = $list;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('account', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('account', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('account', EGW_ACL_DELETE);
				$rows = array('add' => $add, 'edit'=>$edit, 'delete'=>$delete);
				$rows += $list;
			}
			
			return $query['count'];
		}

		public function get_account_name($account_id) {
			if(!self::$account_name_cache[$account_id]) {
				$data = $this->get_accounts(array('account_id' => $account_id));
				if(count($data)) {
					self::$account_name_cache[$account_id] = $data[$account_id];
				}
			}
			return self::$account_name_cache[$account_id];
		}

		/**
		*	Get a list of bank accounts
		*
		*	@nextmatch
		*/
		public function get_bank_accounts($query = array(), &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('bank_account', EGW_ACL_READ, 'perp_ledger')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_bank_accounts($query);

			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['bank_account_id']] = $value['bank_account_name'];
				}
				return $select;
			}

			if(!isset($query['rows'])) {
				// Parameters, return full data
				$rows = array();
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('transaction_type', EGW_ACL_DELETE);
				$rows = array('add' => $add, 'edit'=>$edit, 'delete'=>$delete);
			}
			$rows += $list;
			return $query['count'];
		}

		/**
		*	Get a list of posted General Ledger transactions.
		*	This function returns only an array of transaction information, which contains an array of transaction line data.
		*	It does not return GLTransaction or TransactionLine.
		*
		*	@nextmatch
		*/
		public function get_transactions($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ledger')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}
			if(isset($query['trans_type']) || $query['filter'] != '') {
				$query['trans_type'] = isset($query['trans_type']) ? $query['trans_type'] : $query['filter'];
			}
			if(!isset($query['sort'])) {
				$query['sort'] = 'DESC';
			}
			if(!isset($query['order'])) {
				$query['order'] = 'trans_no';
			}

			$rows = $this->so->get_transactions($query);
			$i = 0;
			$trans_no = 0;

			$config = config::read('perp_ledger');
			$currency = $config['gl_currency'];
			foreach($rows as &$trans) {
				foreach($trans['lines'] as &$line) {
					$line['formatted_debit'] = ($line['debit']) ? ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, $line['debit']) : null;
					$line['formatted_credit'] = ($line['credit']) ? ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, $line['credit']) : null;
				}
				$i++;
			}
			return $query['count'];
		}

		/**
		*	Get the balance of a General Ledger account as at a certain date.
		*
		*	@param as_at_date	The date to get the balance for
		*	@param	account	int	The General Ledger account to get the balance for
		*	@return float The balance of the account on that date
		*/
		public function get_balance($as_at_date, $account) {
			if($GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_ledger')) {
				return $this->so->get_balance($as_at_date, $account);
			} else {
				return 0;
			}
		}

		/**
		*	Set the opening balance of a General Ledger account.
		*	The account must be empty.
		*
		*	@param  account_id 	int	The General Ledger account to set the balance for
		*	@param	amount		float	Amount to set the balance to
		*/
		public function set_opening_balance($account_id, $amount) {
			if(!$GLOBALS['egw']->acl->check('account', EGW_ACL_EDIT, 'perp_ledger')) {
				return lang('You do not have permission to set the opening balance');
			} elseif($this->get_balance(time(), $account_id) != 0) {
				return lang('General Ledger account %1 is not empty.', $account_id);
			} elseif($amount == 0) {
				return true;
			}

			return $this->so->set_opening_balance($account_id, $amount);
		}

		/**
		*	Get the information about a bank account
		*	@param account	int	General Ledger account number for the bank account
		*	@return array Bank account information
		*/
		public function get_bank_account($account) {
			$read = $GLOBALS['egw']->acl->check('bank_account', EGW_ACL_READ, 'perp_ledger');
			if(!$read) {
				return array();
			}

			$where['bank_account_id'] = $account;
			$list = $this->so->get_bank_accounts($where);
			return $list[0];
		}

		/**
		*	Get the balance of a bank account at a certain date
		*
		*	@param as_at_date	The date to get the balance for
		*	@param	account		The General Ledger account to get the balance for
		*	@return float The balance of the account on that date
		*/
		public static function get_bank_balance($as_at_date, $account) {
			if($GLOBALS['egw']->acl->check('bank_account', EGW_ACL_READ, 'perp_ledger')) {
				return so_perp_ledger::get_bank_balance((int)$as_at_date, $account);
			} else {
				return 0;
			}
		}

		/**
		*	Get a list of General Ledger account sequences
		*	Sections are the highest level of account grouping
		*	@nextmatch
		*/
		public function get_sequences($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('sequence', EGW_ACL_READ, 'perp_ledger')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}

			$query['order'] = $query['order'] ? $query['order'] : 'sequence_id';

			$list = $this->so->get_sequences($query);
			if(!isset($query['template'])) {
				$select = array();
				foreach($list as $key => $value) {
					$select[$value['sequence_id']] = $value['sequence_name'];
				}
				return $select;
			} else {
				// Next-match, add in permissions
				$add = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_ADD);
				$edit = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_EDIT);
				$delete = $GLOBALS['egw']->acl->check('account_section', EGW_ACL_DELETE);
				$rows = array('add' => $add, 'edit'=>$edit, 'delete'=>$delete) + $list;

				return $query['count'];
			}
		}
	}
?>
