<?php
	/**
	*	Logic layer for reconciling pERP's bank ledger to 
	*	your bank's statements.
	*
	*	@author Nathan Gray
	*	@package perp_ledger
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_ledger/inc/class.so_perp_reconcile.inc.php');

	class bo_perp_reconcile {

		/**
		*	get_reconciliation
		*
		*  Returns data for reconciliation report.  Probably won't work well (or 
		*  be useful) for lookups
		*
		*	@nextmatch
		*/
		public function get_reconciliation($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('reconcile', EGW_ACL_READ, 'perp_ledger')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			
			if(!isset($query['num_rows'])) {
				$query['num_rows'] = -1;
			}
			if(isset($query['filter'])) {
				$query['bank_account_id'] = $query['filter'];
			}

			$rows = so_perp_reconcile::get_bank_transactions($query);
			
			$bank_account = ExecMethod('perp_ledger.bo_perp_ledger.get_bank_account', $query['bank_account_id']);
			$currency = $bank_account['currency'];
		
			foreach($rows as $key => &$row) {
				if($row['statement_date'] && $row['statement_date'] < $query['statement_date'] && $row['amount'] == $row['amount_cleared']) {
					$readonlys[$key + 1]['clear'] = true;
				}
				$row['bank_account_id'] = $row['bank_account'];
				$row['outstanding'] = $row['amount'] - $row['amount_cleared'];
				$row['formatted_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, $row['amount']);
				$row['formatted_outstanding'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, $row['outstanding']);
				$row['clear'] = ($row['outstanding'] == 0);
				$row['cleared'] = $row['clear'] ? 'cleared' : '';
			}
			$rows['formatted_amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, $rows['amount']);
			$rows['difference'] = $rows['outstanding']; //ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, $rows['outstanding']);
		
			return $query['count'];
		}

		/**
		*	Mark a Bank transaction as reconciled
		*
		*	@param trans_id	int	The ID of the transaction
		*	@param clear_amount	float	The amount reconciled
		*/
		public static function clear_bank_transaction($statement_date, $trans_id, $account, $clear_amount) {
			if($GLOBALS['egw']->acl->check('bank_account', EGW_ACL_EDIT)) {
				so_perp_reconcile::clear_bank_transaction((int)$statement_date, $trans_id, $account, $clear_amount);
			}
		}

		/**
		*	Mark a Bank transaction as reconciled
		*
		*	@param trans_id	int	The ID of the transaction
		*	@param clear_amount	float	The amount reconciled
		*/
		public static function get_reconciled_balance($statement_date, $account) {
			if($GLOBALS['egw']->acl->check('bank_account', EGW_ACL_READ)) {
				return so_perp_reconcile::get_reconciled_balance((int)$statement_date, $account);
			} else {
				return false;
			}
		}

		/**
		*	Get reconciliation date list
		*
		*	Get a list of bank statement dates that have been reconciled.
		*/
		public static function get_date_list($account_id = '') {
			if($GLOBALS['egw']->acl->check('reconcile', EGW_ACL_READ, 'perp_ledger')) {
				return so_perp_reconcile::get_date_list($account_id);
			} else {
				return false;
			}
		}
			
	}
?>
