<?php
	/**
	*       Hooks for eGroupWare
	*       The hooks allow pERP to participate closely in most of eGroupWare's wonderful functions
	*       @ingroup perp_ledger
	*/
	class hook_perp_ledger {

		/**     
		*	List of public functions that phpgwapi allows to be
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'sidebox'	=>	True,
			'config'	=>	True,
			'acl'	=>	True,
			'account_section'	=>	True,
			'account_group'	=>	True,
			'change_lookup'	=>	True,
			'account'	=>	True,
			'sequence'	=>	True,
			'bank_account'	=>	True,
			'transaction_type'	=>	True,
			'year_end'	=>	True,
		);

		/**
		*	Name of this applicaton
		*/
		public $appname = 'perp_ledger';

		/**
		*	Reference to eTemplate object for display
		*/
		var $template;

		/**
		*	Reference to bo_perp_ledger logic object for data
		*/
		var $bo;

		/**
		*	Reference to ui_perp_api for access to common pERP functions
		*/
		var $erp;

		/**
		*	Constructor\n
		*	Creates references to eTemplate for display, bo_perp_ledger logic object, and ui_perp_api for 
		*	common UI type functions
		*/
		public function hook_perp_ledger() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_ledger.bo_perp_ledger');
			$this->erp = CreateObject('perp_api.ui_perp_api');
		}

		/**
		*  List the entries in the Admin module for this app
		*/
		public function admin_hook() {
			$file = array(
				'Site configuration'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.config',
				)),
				'Configure Access Permissions'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.acl',
				)),
				'sequences'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.sequence',
				)),
				'general ledger account sections'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.account_section',
				)),
				'general ledger account groups'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.account_group',
				)),
				'general ledger accounts'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.account',
				)),
				'bank Accounts'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.bank_account',
				)),
				'transaction Types'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.transaction_type',
				)),
				'Electronic Fund Transfer'	=>	$GLOBALS['egw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.ui_perp_eft.admin',
				)),
				'Year End'	=> $GLOBALS['phpgw']->link('/index.php', array(
					'menuaction'	=> 'perp_ledger.hook_perp_ledger.year_end',
				)),
			);

			display_section($this->appname, $file);
		}

		/**
		*	Add entries into the sidebox menu
		*/
		public function sidebox() {
			$menu_title = $GLOBALS['egw_info']['apps'][$this->appname]['title'] . ' ' . lang('Menu');
			if($GLOBALS['egw']->acl->check('account', EGW_ACL_READ, 'perp_ledger')) {
				$menu['Chart of Accounts']	=	$GLOBALS['egw']->link('/index.php','menuaction=perp_ledger.hook_perp_ledger.account');
				$menu['_NewLine_']	=	''; // give a newline
			}
			if($GLOBALS['egw']->acl->check('journal', EGW_ACL_ADD, 'perp_ledger')) {
				$menu['New Journal Entry']	=	$GLOBALS['egw']->link('/index.php','menuaction=perp_ledger.ui_perp_ledger.new_transaction');
			}
			if($GLOBALS['egw']->acl->check('bank_trans', EGW_ACL_ADD, 'perp_ledger')) {
				$menu['New Bank Transaction']	=	$GLOBALS['egw']->link('/index.php', 'menuaction=perp_ledger.ui_perp_ledger.bank');
			}
			if($GLOBALS['egw']->acl->check('reconcile', EGW_ACL_READ, 'perp_ledger')) {
				$menu['Reconciliation']	=	$GLOBALS['egw']->link('/index.php', 'menuaction=perp_ledger.ui_perp_reconcile.reconcile');
			}
			if($GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ledger')) {
				$menu['_NewLine_']	=	''; // give a newline
				$menu['View Transactions']	=	$GLOBALS['egw']->link('/index.php','menuaction=perp_ledger.ui_perp_ledger.list_transaction');
			}
			display_sidebox($this->appname, $menu_title, $menu);
		}

		/**
		*	Set up site configuration - application wide preferences and settings.
		*	@param content	array	Incoming data from the eTemplate
		*/
		public function config($content = array()) {
			$selects['gl_currency'] = ExecMethod('perp_api.bo_perp_api.get_currencies');

			$config = config::read($this->appname);
			if(!$config['next_year_end']) {
				$config['next_year_end'] = date('U', mktime(0,0,0, 13, 0, date('Y')));
				config::save_value('next_year_end', $config['next_year_end'], $this->appname);
			}

			// validation & correction
			//...

			$this->erp->config($content, $this->appname, 'perp_ledger.hook_perp_ledger.config',$selects);
		}

		/**
		*	Set ACLs for perp_ledger
		*
		*	@param content	array	Incoming data from the eTemplate
		*/
		public function acl($content = array()) {
			$lookup = 'acl';
			$data = $this->erp->acl($content, 'perp_ledger', $lookup, 'acl', $selects, $filters);
		}

		/**
		*	Set account sequences, which are the highest level of account grouping
		*	@param content	array	Incoming data from the eTemplate
		*	@see perp_api::ui_perp_api::change_lookup()
		*/
		public function sequence($content = array()) {
			$lookup = 'sequence';

			$data = $this->erp->change_lookup($content, 'perp_ledger', $lookup, 'sequence', $selects, $filters);
		}
	
		/**
		*	Set account sections, which are the second highest level of account grouping
		*	@param content	array	Incoming data from the eTemplate
		*	@see perp_api::ui_perp_api::change_lookup()
		*/
		public function account_section($content = array()) {
			$selects['section_sequence'] = ExecMethod('perp_ledger.bo_perp_ledger.get_sequences');
			$selects['filter'] = array('' => 'Any');
			$selects['filter'] += $selects['section_sequence'];
			$filters = array(
				array('field'	=>	'section_sequence',
					'label'	=>	'Sequence'),
			);
			$lookup = 'account_section';
			$data = $this->erp->change_lookup($content, 'perp_ledger', $lookup, 'section', $selects, $filters);
		}

		/**	
		*	Set account groups, which is the lowest level of account grouping
		*	@param content	array	Incoming data from the eTemplate
		*	@see perp_api::ui_perp_api::change_lookup()
		*/
		public function account_group($content = array()) {
			// Get sections for select boxes
			$selects['group_section'] = $this->bo->get_account_sections();
			$selects['asset_or_liability'] = array(
				'-1'	=>	'Credit Decreases', 
				'1'	=>	'Credit Increases',
				);
			foreach($selects['group_section'] as $key => $value) {
				$selects['group_section'][$key] = $key . ' - ' . $value;
			}
			$selects['filter'] = array('' => 'Any');
			$selects['filter'] += $selects['group_section'];
			$filters = array(
				array('field'	=>	'group_section',
					'label'	=>	'Sections'),
			);
			$lookup = 'account_group';
			$data = $this->erp->change_lookup($content, 'perp_ledger', $lookup, 'group', $selects, $filters);
		}

		/**
		*	Set general ledger accounts
		*	@param content	array	Incoming data from the eTemplate
		*	@see perp_api::ui_perp_api::change_lookup()
		*/
		public function account($content = array()) {
			$selects['group_section'] = $this->bo->get_account_sections();
			$selects['account_group'] = $this->bo->get_account_groups();
			foreach($selects['account_group'] as $key => $value) {
				$selects['account_group'][$key] = $key . ' - ' . $value;
			}
			
			$selects['filter'] = array('' => 'Any') + $selects['group_section'];
			$selects['filter2'] = array('' => 'Any') + $selects['account_group'];
			$filters = array(
				0 => array('field'	=>	'group_section',
					'label'	=>	'Section'),
				1 => array('field'	=>	'account_group',
					'label'	=>	'Groups'),
			);
			$selects['asset_or_liability'] = array(
				'-1'	=>	'Credit Decreases', 
				'1'	=>	'Credit Increases',
			);

			// Add new account, and set opening balance
			if(isset($content['nm']['rows']['new']) && $GLOBALS['egw']->acl->check('account', EGW_ACL_ADD, 'perp_ledger') &&
				($content['nm']['rows']['new']['account_id'] && $content['nm']['rows']['new']['account_group'] && $content['nm']['rows']['new']['account_name'])) {
				$new = $content['nm']['rows']['new'];
					unset($content['nm']['rows']['new']);
					ExecMethod2('perp_api.bo_perp_api.change_lookup', 'perp_account', 'perp_ledger', $new['account_id'], $new);

					if($new['opening_balance'] != 0) {
						// Set opening balance
						$result = ExecMethod2('perp_ledger.bo_perp_ledger.set_opening_balance', $new['account_id'], $new['opening_balance']);
						if($result !== true) {
							$this->erp->messages[] = array($result, 'warning');
						}
					}
			} 
			if (isset($content['nm']['rows']['delete'])) {
				// Check account balance before deleting
				$id = key($content['nm']['rows']['delete']);
				if(ExecMethod2('perp_ledger.bo_perp_ledger.get_balance', time(), $id) != 0) {
					$this->erp->messages[] = array(lang('General Ledger account %1 is not empty', $id), 'error');
					unset($content['nm']['rows']['delete']);
				}
			}
			$lookup = 'account';
			$data = $this->erp->change_lookup($content, 'perp_ledger', $lookup, 'account', $selects, $filters);
		}

		/**
		*	Set bank accounts
		*	@param content	array	Incoming data from the eTemplate
		*	@see perp_api::ui_perp_api::change_lookup()
		*/
		public function bank_account($content = array()) {
			if(isset($content['nm']) && isset($content['nm']['rows'])) {
				if(isset($content['nm']['rows']['new']) && $content['nm']['rows']['new']['bank_account_name']) {
					// check for address link
					// We have to change $_POST, because change_lookup() will overwrite $content with $_POST
					if($_POST['exec']['nm']['rows']['new']['bank_address']['id']) {
						$_POST['exec']['nm']['rows']['new']['bank_address'] = $_POST['exec']['nm']['rows']['new']['bank_address']['id'];
					} else {
						$this->erp->messages[] = array(lang('Invalid address'), 'error');
						unset($_POST['exec']['nm']['rows']['new']);
						unset($content['nm']['rows']['new']);
					}
				}

				// Copy bank_address from $_POST into $content, otherwise it gets lost
				foreach($content['nm']['rows'] as $key => &$row) {
					if(is_numeric($key)) {
						$row['bank_address'] = $_POST['exec']['nm']['rows'][$key]['bank_address'];
					}
				}
			}

			$boerp = CreateObject('perp_api.bo_perp_api');
			$selects['currency'] = $boerp->get_currencies();
			$selects['payment_methods'] = ExecMethod('perp_api.bo_perp_api.get_payment_methods');

			// Some javascript to default the link to addressbook
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
		
			$lookup = 'bank_account';
			$data = $this->erp->change_lookup($content, 'perp_ledger', $lookup, 'bank_account', $selects, $filters);
		}

		/**
		*	Set transaction types
		*	@param content	array	Incoming data from the eTemplate
		*	@see perp_api::ui_perp_api::change_lookup()
		*/
		public function transaction_type($content = array()) {
			$lookup = 'transaction_type';
			$data = $this->erp->change_lookup($content, 'perp_ledger', $lookup, 'trans', $selects, $filters);
		}

		/**	
		*	Year end 
		*	Locks the General Ledger before the year end date.  Transactions cannot be posted with dates earlier
		*	than the last year end.
		*	@param content	array	Incoming data from the eTemplate
		*/
		public function year_end($content = array()) {

			$config =& config::read($this->appname);

			// Deal with incoming
			if(isset($content['save'])) {
				config::save_value('last_year_end', $content['year_end'], $this->appname);
				config::save_value('next_year_end', date('U', mktime(0,0,0, date('m', $content['year_end']), date('d', $content['year_end']), date('Y', $content['year_end'])+1)), $this->appname);
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			}

			$data['year_end'] = $config['next_year_end'];

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Year End');
			$this->template->read('perp_ledger.year_end');
			$this->template->exec('perp_ledger.hook_perp_ledger.year_end', $data, $select_box_options, $readonly, $preserve);
		}
	}
?>
