<?php
	require_once('class.GLTransaction.inc.php');

	/**
	*	Data access object
	*
	*	@ingroup perp_ledger
	*	@author Nathan Gray
	*/
	class so_perp_ledger {
		
		/**
		*	Reference to database access object
		*/
		var $db;

		/**
		*	Constructor
		*	Clones the eGroupWare database access object
		*/
		public function so_perp_ledger() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_ledger');
		}
	
		/**
		*	Get a list of General Ledger account sections
		*	@param query array	Query from nextmatch style function
		*/
		public function get_account_sections(&$query = array()) {
			if($query['search']) {
				$where[] = 'section_id = \'' . $this->db->db_addslashes($query['search']) . '\' OR section_name LIKE \'%' . $this->db->db_addslashes($query['search']) . "%'";
			}
			if(isset($query['filter']) && $query['filter'] !== '') {
				$where['section_sequence'] = $query['filter'];
			}
			if(!$query['order']) {
				$query['order'] = 'section_id';
			}
			
			$this->db->select('perp_account_section',
				array('count(section_id) AS count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(True)) {
				$query['count'] = $row['count'];
			}

			$join .= 'LEFT JOIN perp_account_group ON
					perp_account_section.section_id = perp_account_group.group_section
				LEFT JOIN perp_account ON
					perp_account_group.group_id = perp_account.account_group
				LEFT JOIN perp_general_ledger ON
					perp_account.account_id = perp_general_ledger.account';

			$this->db->select('perp_account_section', 
				array('section_id', 'section_name', 'section_sequence',
					'SUM(amount) AS balance'
				), 
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'GROUP BY section_id ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_ledger', $query['num_rows'],
				$join
			);
			$sections = array();
			while($row = $this->db->row(True)) {
				$row['balance'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $row['balance']);
				$sections[] = $row;
			}
			return $sections;
		}

		/**
		*	Get a list of General Ledger account groups
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_account_groups(&$query = array()) {
			if($query['search']) {
				$where[] = 'group_id = \'' . $this->db->db_addslashes($query['search']) . '\' OR group_name LIKE \'%' . $this->db->db_addslashes($query['search']) . "%'";
			}
			if(isset($query['filter']) && $query['filter'] !== '') {
				$where['group_section'] = $query['filter'];
			}
			if(isset($query['sequence'])) {
				if(is_array($query['sequence'])) {
					$where[] = 'section_sequence IN (' . implode(',', $query['sequence']) . ')';
				} else {
					$where[] = 'section_sequence  = ' . (int)$query['sequence'];
				}
			}

			$join = 'JOIN perp_account_section ON
					perp_account_section.section_id = perp_account_group.group_section';

			// Get number of rows
			$this->db->select('perp_account_group',
				array('count(group_id) AS count'),
				$where,
				__LINE__, __FILE__, 0,
				'', 'perp_ledger', -1,
				$join
			);
			if($row = $this->db->row(True)) {
				$query['count'] = $row['count'];
			}

			$join .='	LEFT JOIN perp_account ON
					perp_account_group.group_id = perp_account.account_group
				LEFT JOIN perp_general_ledger ON
					perp_account.account_id = perp_general_ledger.account';

			$this->db->select('perp_account_group',
				array('group_id', 'group_name', 'group_section', 'asset_or_liability',
					'SUM(amount) AS balance'
				),
				$where,
				__LINE__, __FILE__, $query['start'], 
				'GROUP BY group_id ORDER BY group_id', 'perp_ledger',
				$query['num_rows'], $join
			);
			$groups = array();
			while($row = $this->db->row(True)) {
				$row['balance'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $row['balance']);
				$groups[] = $row;
			}
			return $groups;
		}

		/**
		*	Get a list of General Ledger accounts
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_accounts(&$query = array()) {
			if($query['search']) {
				$where[] = '(account_id LIKE \'' . $this->db->db_addslashes($query['search']) . '%\' OR account_name LIKE \'%' . $this->db->db_addslashes($query['search']) . "%')";
			}
			if(isset($query['filter']) && $query['filter'] !== '') {
				$query['group_section'] = $query['filter'];
			} 
			if (isset($query['group_section'])) {
				if(is_array($query['group_section'])) {
					$where[] = 'perp_account_group.group_section IN (' . implode(',', $query['group_section']) . ')';
				} else {
					$where[] = 'perp_account_group.group_section = ' . (int)$query['group_section'];
				}
			}

			if(isset($query['filter2']) && $query['filter2'] !== '') {
				$query['account_group'] = $query['filter2'];
			}
			if(isset($query['account_group']) && $query['account_group'] != '') {
				$where['account_group'] = $query['account_group'];
			}
			if(!$query['order']) {
				$query['order'] = 'account_id';
			}

			$fields = array('account_id', 'account_name', 'account_group', 'group_name', 'asset_or_liability');

			$join = 'JOIN perp_account_group ON account_group = group_id';

			// Get number of rows
			$this->db->select('perp_account',
				array('count(account_id) AS count'),
				$where,
				__LINE__, __FILE__,
				False, '', False, 0,
				$join
			);
			if($row = $this->db->row(True)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_account',
				$fields,
				$where,
				__LINE__, __FILE__, 
				$query['start'], 'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_ledger', $query['num_rows'],
				$join
			);

			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get a list of bank accounts
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_bank_accounts(&$query = array()) {
			$fields = array(
				'bank_account_id', 'bank_account_name', 
				'bank_account_number', 'bank_address', 
				'currency', 'payment_methods',
				'sort_order'
			);
			if(isset($query['account_group'])) {
				$join = 'JOIN perp_account ON bank_account_id = account_id';
				unset($query['account_group']);
			}
			if(isset($query['bank_account_id'])) {
				$where['bank_account_id'] = $query['bank_account_id'];
			}
			if(isset($query['currency'])) {
				$where['currency'] = $query['currency'];
			}
			if(isset($query['payment_method'])) {
				if(is_array($query['payment_method'])) {
					$methods = $query['payment_method'];
				} else {
					$methods = explode(',', $query['payment_method']);
				}
				$methods = implode('%', $methods);
				$where[] = 'payment_methods LIKE %' . $methods . '%';
			}
			if(!isset($query['order'])) {
				$query['order'] = 'sort_order, bank_account_id';
			}

			if($query['search']) {
				$where[] = 'bank_account_id = \'' . $this->db->db_addslashes($query['search']) . '\' OR bank_account_name LIKE \'%' . $this->db->db_addslashes($query['search']) . "%'";
			}
			if(isset($query['filter']) && $query['filter'] !== '') {
				$where['account_group'] = $query['filter'];
			}

			// Get number of rows
			$query['count'] = $GLOBALS['egw']->db->select('perp_bank_account',
				array('count(bank_account_id) AS count'),
				$where,
				__LINE__, __FILE__
			)->fetchSingle();

			$result = $GLOBALS['egw']->db->select('perp_bank_account',
				$fields,
				$where,
				__LINE__, __FILE__, 
				$query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				False, $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				if($row['payment_methods']) {
					$row['payment_methods'] = explode(',', $row['payment_methods']);
				}
				$list[] = $row;
			}
			foreach($list as &$row) {
				if($row['bank_address']) {
					$row['address_link'] = array(
						'app'	=>	'addressbook',
						'id'	=>	$row['bank_address'],
						'title'	=>	ExecMethod('addressbook.addressbook_bo.link_title', $row['bank_address'])
					);
					if(!$row['address_link']['title']) {
						unset($row['address_link']);
					}
				}
			}
			return $list;
		}

		/**
		*	Get a list of transaction types
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_transaction_types($query = array(), $start = 0) {
			$start = $start == 0 ? False : $start;
			$this->db->select('perp_transaction_type', array('trans_id', 'trans_name'), array(), __LINE__, __FILE__, False, 'ORDER BY trans_id');
			$types = array();
			while($row = $this->db->row(True)) {
				$types[] = $row;
			}
			return $types;
		}

		/**
		*	Get a list of General Ledger transactions
		*	Only returns information, not objects
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_transactions(&$query = array()) {
			$where = array();
			if(isset($query['trans_type'])) {
				$where['trans_type'] = $query['trans_type'];
			}
			if($query['filter']) {
				$where['trans_type'] = (int)$query['filter'];
			}
			if($query['account']) {
				$where['account'] = $query['account'];
			}
			if($query['filter2']) {
				$where['account'] = $query['filter2'];
			}
			if($query['start_date']) {
				$where[] = "trans_date >= '" . $GLOBALS['egw']->db->to_timestamp($query['start_date'] - 1) . "'";
			}
			if($query['end_date']) {
				$where[] = "trans_date <= '" . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . "'";
			}
			if(!$query['order']) {
				$query['order'] = 'trans_date, trans_no';
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(narrative LIKE $search OR trans_no = " . (int)$query['search'] . ')';
			}

			// Get number of rows
			$this->db->select('perp_general_ledger',
				array('trans_no'),
				$where,
				__LINE__, __FILE__, False,
				'GROUP BY trans_no'
			);

			$query['count'] = $this->db->num_rows();

			// Get transaction numbers
			$this->db->select('perp_general_ledger',
				array('trans_no'),
				$where,
				__LINE__, __FILE__, $query['start'], 
				'GROUP BY trans_no ORDER BY ' . $query['order'] . ' ' . $query['sort'], False, $query['num_rows'],
				'JOIN perp_account ON
					perp_account.account_id = perp_general_ledger.account
				JOIN perp_account_group ON
					perp_account.account_group = perp_account_group.group_id'
			);
			$trans_list = array();
			while($row = $this->db->row(true)) {
				$transaction = new GLTransaction($row['trans_no']);
				$trans = array(
					'transaction'	=>	array(
					'trans_no'	=>	$transaction->get_transaction_number(),
					'trans_date'	=>	$transaction->get_date(),
					'trans_type'	=>	$transaction->get_transaction_type()
					)
				);
				$trans['lines'] = $transaction->get_lines();
				$trans_list[] = $trans;
			}
			return $trans_list;
		}

		/**
		*	Get a list of bank transactions
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_bank_transactions(&$query) {
			if($query['search']) {
				$where[] = 'bank_account = \''. $this->db->db_addslashes($query['search']) .
					'\' OR perp_bank_ledger.amount LIKE \'%' . $this->db->db_addslashes($query['search']) . "%'";
			}
			if(isset($query['filter'])) {
				$where['bank_account'] = $query['filter'];
			}
			if(isset($query['filter2']) && $query['filter2'] == '0') {
				$where[] = 'amount_cleared != perp_bank_ledger.amount';
			}               
			if($query['statement_date']) {
				$where[] = 'trans_date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['statement_date']) . "'";
			}

			// Get number of rows
			$this->db->select('perp_bank_ledger',
				array('count(trans_id) AS count'),
				$where,
				__LINE__, __FILE__, 0,
				'', 'perp_ledger', False,
				'JOIN perp_general_ledger ON trans_id = trans_no AND bank_account = account'
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_bank_ledger',
				array('DISTINCT trans_id', 'bank_account', 'amount_cleared', 
					'payment_method', 'trans_date',
					'perp_bank_ledger.amount', 'narrative'),
				$where, 
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY trans_date, trans_id', 'perp_ledger', $query['num_rows'], 
				'JOIN perp_general_ledger ON 
					trans_id = trans_no 
					AND bank_account = account 
					AND perp_general_ledger.line_no = perp_bank_ledger.line_no'
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$row['trans_date'] = $GLOBALS['egw']->db->from_timestamp($row['trans_date']);
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get the balance of a General Ledger account at a certain date
		*	@param	as_at_date	Date of the balance
		*	@param	account		ID of the account to check
		*	@return float Account balance
		*/
		public function get_balance($as_at_date, $account) {
			$this->db->select('perp_general_ledger',
				array('asset_or_liability * -1 * sum(amount) AS amount'),
				array('account' => $account, 'trans_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($as_at_date))),
				__LINE__, __FILE__, false, 
				'', 'perp_ledger', 1,
				'JOIN perp_account ON
					perp_account.account_id = perp_general_ledger.account
				JOIN perp_account_group ON
					perp_account_group.group_id = perp_account.account_group'
			);
			$row = $this->db->row(True);
			return $row['amount'];
		}

		/**
		*	Set the balance of a General Ledger account
		*
		*	This creates an unbalanced transaction to set the opening balance.
		*
		*	@param account_id int ID of the account to set
		*	@param amount 	float	Balance for the account
		*/
		public function set_opening_balance($account_id, $amount) {
			// Get transaction number
			$this->db->select('perp_general_ledger', 'IFNULL(max(trans_no),0) + 1 AS trans_no', array(), __LINE__, __FILE__, 'perp_ledger');
			$row = $this->db->row(true);
			if($row) {
				$transaction_number = $row['trans_no'];
			} else {
				$transaction_number = 0;
			}
			$this->db->insert(
				'perp_general_ledger',
				array(
					'trans_no'	=>	$transaction_number,
					'trans_type'	=>	0,
					'trans_date'	=>	$GLOBALS['egw']->db->to_timestamp(time()),
					'line_no'	=>	0,
					'account'	=>	$account_id,
					'amount'	=>	$amount,
					'narrative'	=>	lang('Opening balance')
				),
				array(),
				__LINE__, __FILE__
			);
			return true;
		}
		
		/**
		*	Get the balance of a bank account at a certain date
		*	@param	as_at_date	Date of the balance
		*	@param	account		ID of the account to check
		*	@return float Account balance
		*/
		public static function get_bank_balance($as_at_date, $account) {
			return $GLOBALS['egw']->db->select('perp_bank_ledger',
				array('sum(perp_bank_ledger.amount) AS amount'),
				array('bank_account' => $account, 'trans_date <= \'' . $GLOBALS['egw']->db->to_timestamp($as_at_date) . "'"),
				__LINE__, __FILE__, False, 
				'', False, 0, 
				'JOIN perp_general_ledger ON trans_id = trans_no 
					AND account = bank_account
					AND perp_general_ledger.line_no = perp_bank_ledger.line_no'
			)->fetchSingle();
		}

		/**
		*	Get a list of General Ledger sequences
		*	@param query array	Query from nextmatch style function
		*	@return array of results
		*/
		public function get_sequences(&$query) {
			if($query['search']) {
				$where[] = 'sequence_name LIKE \'%' . add_slashes($query['search']) . '\'%';
			}
			if(isset($query['sequence_id'])) {
				$where['sequence_id'] = $query['sequence_id'];
			}

			// Get number of rows
			$this->db->select('perp_sequence',
				array('count(sequence_id) AS count'),
				$where,
				__LINE__, __FILE__, 0,
				'', 'perp_ledger', False
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_sequence',
				array(
					'sequence_id',
					'sequence_name'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' .  $query['sort'],
				'perp_ledger', $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}
	}
?>
