<?php

	/**
	*	Functions for getting report data out of the database.
	*	
	*	@author Nathan Gray
	*	@ingroup perp_ledger
	*/
	class so_perp_ledger_report {

		/**
		*	Reference to database abstraction object
		*/
		private $db;

		/**
		*	Reference to bo_perp_ledger
		*/
		private $bo;

		/**
		*	Constructor
		*/
		public function so_perp_ledger_report() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->bo = CreateObject('perp_ledger.bo_perp_ledger');
		}

		/**
		*	Get data for the trial balance
		*
		*	This function is used by the trial balance report.
		*
		*	@param	startDate	start date
		*	@param	endDate		end date
		*	@return array of account balances
		*/
		public function trial_balance($startDate, $endDate) {
			$data = array();

			$sql = 'SELECT 
					account_id,
					account_name,
					sum(amount) AS amount
				FROM perp_account 
					LEFT JOIN perp_general_ledger ON 
						perp_account.account_id = perp_general_ledger.account AND
						trans_date BETWEEN \'' . $GLOBALS['egw']->db->to_timestamp($startDate - 1) . "' AND '" . $GLOBALS['egw']->db->to_timestamp($endDate) . '\'
					LEFT JOIN perp_account_group ON 
						perp_account.account_group = perp_account_group.group_id 
				GROUP BY account_id
				ORDER BY account_id';
			$this->db->query($sql, __LINE__, __FILE__);
			$data = array();
			while($row = $this->db->row(True)) {
				$row[$row['amount'] >= 0 ? 'credit' : 'debit'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, abs($row['amount']));
				$data['data'][] = $row;
				$data[$row['amount'] >= 0 ? 'credit' : 'debit'] += abs($row['amount']);
			}
			$data['credit'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $data['credit']);
			$data['debit'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $data['debit']);
			return $data;
		}

		/**
		*	Get data for the balance sheet
		*	Replaced by report_balance_sheet
		*
		*	@depricated 
		*	@param	startDate	start date
		*	@param	endDate		end date
		*	@return array of balance sheet info
		*/
		/*
		public function balance_sheet($startDate, $endDate) {
			$line = 0;
			$sequence = ExecMethod('perp_ledger.bo_perp_ledger.get_sequences');
			foreach($sequence as $sequence_id => $seq) {
				$this->bo->get_account_sections(array('filter' => $sequence_id, 'template' => false), $sections);
				$groups = array();
				$section_count = 1;
				$data = array();
				$report['sequence'][$sequence_id] = array();
				foreach($sections as $section) {
					$data['sections'][$section_count] = array(
						'section_id'	=>	$section['section_id'],
						'section_name'	=>	$section['section_name'],
						'debit'	=>	0,
						'credit'	=>	0,
					);
					$this->balance_sheet_section($sequence_id, $section['section_id'], $section_count, $startDate, $endDate, $data);
					$report['debit'] += abs($data['sections'][$section_count]['debit']);
					$report['credit'] += abs($data['sections'][$section_count]['credit']);
					$report['sequence'][$sequence_id]['debit'] += abs($data['sections'][$section_count]['debit']);
					$report['sequence'][$sequence_id]['credit'] += abs($data['sections'][$section_count]['credit']);
					$section_count++;
				}
				$report['sequence'][$sequence_id] += $data;
				$report['sequence'][$sequence_id]['name'] = $seq;
			}
			$report['debit'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, abs($report['debit']));
			$report['credit'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, abs($report['credit']));

			return $report;
		}

		private function balance_sheet_section($sequence_id, $section_id, $section_count, $startDate, $endDate, &$data) {
			$sql = 'SELECT 
					group_id,
					group_name,
					group_section,
					sum(amount) AS amount 
				FROM perp_account_group 
					LEFT JOIN perp_account ON 
						perp_account.account_group = perp_account_group.group_id 
					LEFT JOIN perp_general_ledger ON 
						perp_account.account_id = perp_general_ledger.account AND
						trans_date <= \'' . $endDate . '\' 
				WHERE perp_account_group.group_section = ' . $section_id . '
				GROUP BY group_id';
			$this->db->query($sql, __LINE__, __FILE__);
			while($row = $this->db->row(True)) {
				$row[$row['amount'] <= 0 ? 'credit' : 'debit'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, abs($row['amount']));
				$data['sections'][$section_count][$row['amount'] <= 0 ? 'credit' : 'debit'] +=  abs($row['amount']);
				$data['sections'][$section_count]['groups'][] = $row;
			}
		}
		*/

		/**
		*	Probably replaced by general_ledger_listing report
		*	@depricated
		*/
		/*
		public function account_inquiry($account, $start_date, $end_date) {
			$sql = 'SELECT
					trans_no,
					trans_date, 
					amount * (asset_or_liability / abs(asset_or_liability)) AS amount,
					narrative
				FROM perp_general_ledger
				JOIN perp_account ON 
					perp_account.account_id = perp_general_ledger.account
				JOIN perp_account_group ON
					perp_account_group.group_id = perp_account.account_group
				WHERE account = ' . $this->db->quote($account, 'int') . ' AND
					trans_date >= ' . $this->db->quote($start_date, 'date') . ' AND
					trans_date <= ' . $this->db->quote($end_date, 'date') . '
				ORDER BY trans_date';

			$this->db->query($sql, __LINE__, __FILE__);

			$data = array();
			$count = 1;
			while($row = $this->db->row(True)) {
				$data[$count] = $row;
				$count++;
			}
			return $data;
		}
		*/
	}
?>
