<?php
	/**
	*	Storage layer for dealing with reconciling pERP's bank ledger to
	*	a bank's account statement.
	*
	*	@author Nathan Gray
	*/

	class so_perp_reconcile {

		/**
		*	Get a list of bank transactions with reconciliation information
		*	according to the query parameters given
		*
		*	@nextmatch
		*
		*	Extra keys:
		*	account_id
		*	
		*/
		public static function get_bank_transactions(&$query) {
			if($query['search']) {
				$where[] = 'bank_account = \''. $this->db->db_addslashes($query['search']) .
					'\' OR perp_bank_ledger.amount LIKE \'%' . $this->db->db_addslashes($query['search']) . "%'";
			}
			if(isset($query['bank_account_id'])) {
				$where['bank_account'] = $query['bank_account_id'];
			}
			if(isset($query['filter2']) && $query['filter2'] == '0') {
				$having[] = 'amount_cleared != perp_bank_ledger.amount';
				if($query['statement_date']) {
					$where[] = 'trans_date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['statement_date']) . "'";
				}
			}

			if($query['start_date']) {
				$where[] = 'trans_date >= \'' . $GLOBALS['egw']->db->to_timestamp($query['start_date']) . "'";
			}
			if($query['end_date']) {
				$where[] = 'trans_date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['end_date']) . "'";
			}

			if(count($having) > 0) {
				$having = 'HAVING ' . implode(' AND ', $having);
			}
			
			$join = 'JOIN perp_general_ledger ON 
					perp_bank_ledger.trans_id = trans_no AND bank_account = account
					AND perp_general_ledger.line_no = perp_bank_ledger.line_no
				LEFT JOIN perp_bank_reconcile ON
					perp_bank_reconcile.trans_id = perp_bank_ledger.trans_id
					AND perp_bank_reconcile.line_no = perp_bank_ledger.line_no
					AND perp_bank_reconcile.account = perp_bank_ledger.bank_account';

			// Get number of rows
			$query['count'] = $GLOBALS['egw']->db->select(
				'perp_bank_ledger',
				array(
					'perp_bank_ledger.amount',
					'IF(SUM(amount_cleared) IS NULL, 0, SUM(amount_cleared)) AS amount_cleared'
				),
				$where,
				__LINE__, __FILE__, 0,
				'GROUP BY perp_bank_ledger.trans_id, perp_bank_ledger.line_no, perp_bank_ledger.bank_account ' . $having,
				'perp_ledger', -1,
				$join
			)->NumRows();

			$result = $GLOBALS['egw']->db->select('perp_bank_ledger',
				array(
					'perp_bank_ledger.trans_id', 
					'perp_general_ledger.account', 
					'IF(SUM(amount_cleared) IS NULL, 0, SUM(amount_cleared)) AS amount_cleared',
					'payment_method', 'trans_date',
					'statement_date',
					'perp_bank_ledger.amount', 'narrative'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'GROUP BY perp_bank_ledger.trans_id, perp_bank_ledger.line_no, perp_bank_ledger.bank_account ' . $having . '
				ORDER BY trans_date, trans_id', 'perp_ledger', $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				$row['trans_date'] = $GLOBALS['egw']->db->from_timestamp($row['trans_date']);
				$row['statement_date'] = $GLOBALS['egw']->db->from_timestamp($row['statement_date']);
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Reconcile a bank transaction
		*	@param trans_id ID number of the transaction to reconcile
		*	@param amount	amount to reconcile
		*/
		public static function clear_bank_transaction($statement_date, $trans_id, $account, $amount) {
			$statement_date = $GLOBALS['egw']->db->to_timestamp($statement_date);

			if($amount == 0) {
				$GLOBALS['egw']->db->delete(
					'perp_bank_reconcile', 
					array(
						'trans_id'		=>	$trans_id,
						'account'		=>	$account,
						'statement_date'	=>	$statement_date
					), 
					__LINE__, __FILE__, 'perp_ledger'
				);
			} else {
				$GLOBALS['egw']->db->insert(
					'perp_bank_reconcile', 
					array(
						'trans_id'		=>	$trans_id,
						'account'		=>	$account,
						'statement_date'	=>	$statement_date,
						'amount_cleared'	=>	$amount
					),
					array(
						'trans_id'		=>	$trans_id,
						'account'		=>	$account,
						'statement_date'	=>	$statement_date
					), 
					__LINE__, __FILE__, 'perp_ledger'
				);
			}
		}

		/**
		*	Get the unreconciled balance as at a given date
		*/
		public static function get_reconciled_balance($statement_date, $account) {
			$statement_date = $GLOBALS['egw']->db->to_timestamp($statement_date);

			return $GLOBALS['egw']->db->select(
				'perp_bank_reconcile',
				array(
					'SUM(amount_cleared) AS amount'
				),
				array(
					'account = ' . $GLOBALS['egw']->db->quote($account),
					'statement_date <= ' . $GLOBALS['egw']->db->quote($statement_date)
				),
				__LINE__, __FILE__, 0,
				'', 'perp_ledger', 1
			)->fetchSingle();
		}

		/**
		*	Get a list of statement dates for which reconciliation has been done.
		*
		*	@param account_id Optional.  Restrict to one account.
		*/
		public static function get_date_list($account_id) {
			$result =& $GLOBALS['egw']->db->select(
				'perp_bank_reconcile',
				array('statement_date'),
				$where,
				__LINE__, __FILE__, 0,
				'GROUP BY statement_date ORDER BY statement_date DESC',
				'perp_ledger'
			);
			$list = array();
			foreach($result as $row) {
				$date = $GLOBALS['egw']->db->from_timestamp($row['statement_date']);
				$list[$date] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $date);
			}
			return $list;
		}
	}
?>
