<?php
	/**
	*	User Interface for General Ledger 
	*
	*	@author Nathan Gray
	*	@ingroup perp_ledger
	*/

	require_once('class.GLTransaction.inc.php');

	class ui_perp_ledger {
		/**
		*	Reference to logic layer
		*/
		protected $bo;

		/**
		*	Reference to eTemplate object for display
		*/
		protected $template;

		/**
		*	List of public functions that phpgwapi allows to be
		*	called from the browser.  All other functions are denied.
		*/
		public $public_functions = array(
			'index'		=>	True,
			'new_transaction'	=>	True,
			'list_transaction'	=>	True,
			'journal'	=>	True,
			'bank'		=>	True,
			'admin'		=>	True,
		);

		/**
		*	boolean output extra debug information
		*/
		protected $debug = false;

		/**
		*	Reference to a GLTransaction to use for building & display
		*/
		var $transaction;

		/**
		*	Constructor\n
		*	Creates the business object and the eTemplate object.
		*/
		function ui_perp_ledger() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_ledger.bo_perp_ledger');
		}

		/**
		*	Index of General Ledger displays a list of available functions.
		*	If the user does not have access to a location, it is not shown on the index.
		*
		*	@param content	array	Data returned from eTemplate when the user submits the form
		*/
		function index($content = array()) {
			$journal = $GLOBALS['egw']->acl->check('journal', EGW_ACL_READ, 'perp_ledger');
			$journal_add = $GLOBALS['egw']->acl->check('journal', EGW_ACL_ADD, 'perp_ledger');
			$bank_trans = $GLOBALS['egw']->acl->check('bank_trans', EGW_ACL_READ, 'perp_ledger');
			$reconcile = $GLOBALS['egw']->acl->check('reconcile', EGW_ACL_READ, 'perp_ledger');

			if($content['journal'] && $journal_add) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ledger.ui_perp_ledger.journal'));
			} elseif($content['bank'] && $bank_trans) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ledger.ui_perp_ledger.bank'));
			} elseif($content['reconcile'] && $reconcile) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ledger.ui_perp_reconcile.reconcile'));
			} elseif($content['view_trans'] && $journal) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ledger.ui_perp_ledger.list_transaction'));
			}

			$readonly['view_trans'] = !$journal;
			$readonly['journal'] = !$journal_add;
			$readonly['bank'] = !$bank_trans;
			$readonly['reconcile'] = !$reconcile;

			// Display
			$this->template->read('perp_ledger.index');
			$this->template->exec('perp_ledger.ui_perp_ledger.index', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	List posted transactions
		*	@param content	array	Data returned from eTemplate when the user submits the form
		*/
		function list_transaction($content = array()) {

			if(!$GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ, 'perp_ledger')) {
				return lang('Access not allowed');
			}

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_ledger.ui_perp_ledger.index'));
			}

			$data['trans_type'] = $content['trans_type'] ? $content['trans_type'] : $trans_types[0];

			$data['nm'] = array(
				'no_cat'	=>	True,
				'filter_label'	=>	'Transaction Type',
				'filter_no_lang'	=>	True,
				//'no_filter'	=>	True,
				'filter2_label'	=>	'Account',
				'filter2_no_lang'	=>	True,
				'get_rows'      =>      'perp_ledger.bo_perp_ledger.get_transactions',
				'template'	=>	'perp_ledger.list_transaction.row',
				'header_left'	=>	'perp_ledger.list_transaction.start_date',
				'header_right'	=>	'perp_ledger.list_transaction.end_date',
			);
			$select_box_options['filter'] = array('' => lang('All')) + $this->bo->get_transaction_types();
			$select_box_options['filter2'] = array('' => lang('All')) + $this->bo->get_accounts();
			$select_box_options['account'] = $select_box_options['filter2'];
			$select_box_options['trans_type'] = $select_box_options['filter'];

			//$data['nm']['start_date'] = $content['nm']['start_date'] ? $content['nm']['start_date'] : date('U',mktime(0,0,0,date('m') - 1, date('d'), date('Y')));

			// Permissions
			//$readonly['new_transaction'] = !$GLOBALS['egw']->acl->check('run', EGW_ACL_ADD);
			//$readonly['search'] = !$GLOBALS['egw']->acl->check('run', EGW_ACL_READ);

			// Display
			$this->template->read('perp_ledger.list_transaction');
			$this->template->exec('perp_ledger.ui_perp_ledger.list_transaction', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Create a new transaction.  This will overwrite any other transaction the user was working on.
		*/
		function new_transaction() {
			$this->transaction = CreateObject('perp_ledger.GLTransaction');
			$GLOBALS['egw']->session->appsession('transaction', false, serialize($this->transaction));
			return $this->journal();
		}

		/**
		*	Create a new bank transaction.  This will overwrite any other bank transaction the user was working on.
		*/
		function new_bank_transaction() {
			$transaction = CreateObject('perp_ledger.BankTransaction');
			$GLOBALS['egw']->session->appsession('bank_transaction', false, $transaction);
			return $this->bank();
		}

		/**
		*	Create a General Ledger transaction by manually adding lines to a GLTransaction.
		*	This doesn't work with accounts that need associated TransactionLines for other Ledgers.
		*
		*	@param content	array	Data returned from eTemplate when the user submits the form
		*/
		function journal($content = array()) {
			if($this->debug) {
				echo 'Incoming data';
				_debug_array($content);
			}
			if(!$GLOBALS['egw']->acl->check('journal', EGW_ACL_READ, 'perp_ledger')) {
				return lang('Access not allowed');
			}
			$edit = $GLOBALS['egw']->acl->check('journal', EGW_ACL_EDIT, 'perp_ledger');

			// Get the transaction
			$trans_no = $content['trans_no'] ? $content['trans_no'] : $_GET['trans_no'];

			$trans =& unserialize($GLOBALS['egw']->session->appsession('transaction'));
			if(isset($trans_no) ) {
				$this->transaction = CreateObject('perp_ledger.GLTransaction', $trans_no);
				unset($content);
			} else if($trans) {
				$this->transaction =& $trans;
			} else {
				$this->transaction = CreateObject('perp_ledger.GLTransaction');
			}

			// Update the transaction
			if($content && $edit) {
				if($content['new_transaction']) {
					return $this->new_transaction();
				}

				$this->transaction->set_transaction_type($content['trans_type']);
				$this->transaction->set_date($content['date']);
				if($content['new']) {
					if($content['account'] && ($content['debit'] || $content['credit']) && $content['narrative']) {
						if($content['debit']) {
							$amount = bo_perp_api::get_float_from_string($content['debit']);
						} else {
							$amount = - bo_perp_api::get_float_from_string($content['credit']);
						}
						$this->transaction->add_line($content['account'], 
							$content['narrative'], 
							$amount);
					} else {
						$msg[] = array(lang('Missing required field'), 'error');
					}
				}
				if($content['lines'] && !$this->transaction->get_transaction_number()) {
					$id = key($content['lines']['delete']);
					$this->transaction->remove_line( $id);
				}
			}
			
			// Save the transaction to the db
			if($content['post'] && !$this->transaction->get_transaction_number() && $edit && $this->transaction->check() === True) {
				$result = $this->transaction->write();
				if (is_numeric($result)) {
					$msg[] = array(lang('transaction number %1 posted', $result),'success');
					$GLOBALS['egw']->session->appsession('transaction', false, false);
					$this->transaction = CreateObject('perp_ledger.GLTransaction', $result);
				} elseif ($result !== True) {
					$msg[] = array($result, 'error');
				}
			} elseif($content['post']) {
				$msg[] = array(lang('Unable to post transaction'), 'error');
			}
			if($content && $edit) {
				$GLOBALS['egw']->session->appsession('transaction', 'perp_ledger', serialize($this->transaction));
			}

			// Get data for the template
			try {
				$balance = $this->transaction->check();
			} catch (Exception $e) {
				$msg[] = array($e->getMessage(), 'info');
			}
			if($balance === True) {
				$balance = lang('~ Balanced ~');
			} else {
				$balance = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $balance);
			}
			$data = array(
				'trans_no'	=>	$this->transaction->get_transaction_number(),
				'trans_type'	=>	$this->transaction->get_transaction_type(),
				'date'		=>	$this->transaction->get_date() ? $this->transaction->get_date() : date('U'),
				'off'		=>	$balance,
				'credit_total'	=>	0,
				'debit_total'	=>	0,
			);

			if(count($this->transaction->get_lines())) {
				$data['lines'] = $this->transaction->get_lines();
			
				foreach($data['lines'] as $key => $line) {
					$data['credit_total'] += $line['credit'];
					$data['debit_total'] += $line['debit'];
					$data['lines'][$key][($line['debit'] ? 'debit' : 'credit')] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, abs($line['amount']));
				}
				array_unshift($data['lines'], 'Heading');
			}
			$data['credit_total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $data['credit_total']);
			$data['debit_total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $data['debit_total']);

			$select_box_options = array(
				'trans_type'	=>	$this->bo->get_transaction_types(),
				'account'	=>	array('' => lang('Select')) + $this->bo->get_accounts(),
			);

			// Transaction number means it's been posted, and read only
			if($this->transaction->get_transaction_number() || !$edit) {
				$readonly['trans_type']	=	True;
				$readonly['date']	=	True;
				$readonly['account']	=	True;
				$readonly['debit']	=	True;
				$readonly['credit']	=	True;
				$readonly['narrative']	=	True;
				$readonly['post']	=	True;
				$readonly['perp_ledger.journal.line']	=	True;
				$data['lines']['read_only']	=	True;
				$readonly['new']	=	True;
			}
			$readonly['new_transaction']	=	!$edit;

			if(count($data['lines']) == 1 || $this->transaction->check() !== true) {
				$readonly['post'] 	=	True;
			}

			if($msg) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $msg);
			}

			$this->template->read('perp_ledger.journal');
			$this->template->exec('perp_ledger.ui_perp_ledger.journal', $data, $select_box_options, $readonly, $preserve);

			if($this->debug) {
				echo '<pre>';
				echo $this->transaction;
				echo '</pre>';
				_debug_array($this->transaction);
			}
		}

		/**
		*	Create a manual bank transaction by adding TransactionLines to a BankTransaction.
		*	pERP will add the balancing BankTransactionLine.
		*	@param content	array	Data returned from eTemplate when the user submits the form
		*/
		function bank($content = array()) {
			if($this->debug) {
				echo 'Incoming data';
				_debug_array($content);
			}

			// Get the transaction
			CreateObject('perp_ledger.BankTransaction');
			$trans = $GLOBALS['egw']->session->appsession('bank_transaction');
			if(isset($content['trans_no'])) {
				$transaction = CreateObject('perp_ledger.BankTransaction', $content['trans_no']);
				unset($content);
			} else if($trans) {
				$transaction = $trans;
			} else {
				$transaction = CreateObject('perp_ledger.BankTransaction');
			}

			// Update the transaction
			if($content) {
				if($content['new_transaction']) {
					return $this->new_bank_transaction();
				}
				$old_exchange = $transaction->get_exchange_rate();
				if ($content['currency'] && $content['currency'] != $transaction->get_currency()) {
					$transaction->set_currency($content['currency']);
				} 
				if(isset($content['bank_account']) && $content['bank_account'] != $transaction->get_bank_account()) {
					$transaction->set_bank_account($content['bank_account']);
				} 
				if($content['exchange_rate'] && $content['exchange_rate'] != $old_exchange) {
					$transaction->set_exchange_rate($content['exchange_rate']);
				}
				$transaction->set_transaction_type($content['trans_type']);
				$transaction->set_payment_method($content['payment_method']);
				$transaction->set_narrative($content['bank_narrative']);
				$transaction->set_date($content['date']);
				if($content['debit'] || $content['credit']) {
					if($content['debit']) {
						$amount = $content['debit'];
					} else {
						$amount = -$content['credit'];
					}
					$line = $transaction->add_line($content['account'], 
						$content['narrative'], 
						$amount);
				}
				if($content['lines'] && !$transaction->get_transaction_number()) {
					$id = key($content['lines']['delete']);
					$transaction->remove_line( $id);
				}
			}
		
			// Save the transaction to the db
			if($content['post']) {
				try {
					$transaction->write();
				} catch (Exception $e) {
					$msg[] = array(lang($e->getMessage()), 'error');
				}
			}

			$GLOBALS['egw']->session->appsession('bank_transaction', false, $transaction);

			// Get data for the template
			$balance = $transaction->check();
			if($balance === True) {
				$off = ' - ';
				$balance = 0; 
			} else { 
				$off = sprintf("%3.2f", -$balance);
			}
			$converted_balance = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $transaction->get_currency(),  abs($balance) / $transaction->get_exchange_rate());
			
			$config = config::read('perp_ledger');
			$currency = $config['gl_currency'];
			$accounts = $this->bo->get_accounts();
			$bank_accounts = $this->bo->get_bank_accounts();

			$data = array(
				'trans_no'	=>	$transaction->get_transaction_number(),
				'trans_type'	=>	$transaction->get_transaction_type(),
				'bank_account'	=>	$transaction->get_bank_account(),
				'currency'	=>	$transaction->get_currency(),
				'exchange_rate'	=>	$transaction->get_exchange_rate(),
				'payment_method'	=>	$transaction->get_payment_method(),
				'date'		=>	$transaction->get_date() ? $transaction->get_date() : date('U'),
				'off'		=>	abs($off),
				'default_currency'	=>	$currency,
			);

			// Fake line to balance
			if(!$transaction->get_transaction_number()) {
				$data += array(
					'bank_account_name'	=>	$accounts[$transaction->get_bank_account()],
					'bank_' . (-$balance > 0 ? 'debit' : 'credit')	=>	$converted_balance,
					'bank_narrative'	=>	$transaction->get_narrative(),
				);
				
				// Remove bank account from account list so user doesn't add another line
				$accounts = array_diff_key($accounts, $bank_accounts);
			}

			$data['lines'] = $transaction->get_lines();
//			$data['lines'] = $transaction->get_corrected_lines();
			foreach($data['lines'] as $key => $line) {
				$data['lines'][$key][($line['amount'] > 0 ? 'debit' : 'credit')] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $currency, abs($line['amount']));
			}
			array_unshift($data['lines'], 'Heading');
			
			$select_box_options = array(
				'trans_type'	=>	$this->bo->get_transaction_types(),
				'account'	=>	$accounts,
				'bank_account'	=>	array('' => lang('Select')) + $bank_accounts,
				'currency'	=>	ExecMethod('perp_api.bo_perp_api.get_currencies'),
				'payment_method'=>	ExecMethod('perp_api.bo_perp_api.get_payment_methods'),
			);

			// Transaction number means it's been posted, and read only
			if($transaction->get_transaction_number()) {
				$readonly['trans_type']	=	True;
				$readonly['date']	=	True;
				$readonly['account']	=	True;
				$readonly['debit']	=	True;
				$readonly['credit']	=	True;
				$readonly['narrative']	=	True;
				$readonly['post']	=	True;
				$readonly['bank_account']=	True;
				$readonly['currency']	=	True;
				$readonly['exchange_rate'] =	True;
				$readonly['payment_method'] = 	True;
				$data['lines']['read_only']	=	True;
			}

			if($msg) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $msg);
			}

			$GLOBALS['egw_info']['flags']['app_header'] = lang('perp_ledger') . ' - ' . lang('%1 %2 %3', $data['dep_withdraw'], $data['direction'], $select_box_options['bank_account'][$data['bank_account']]);
			$this->template->read('perp_ledger.bank');
			$this->template->exec('perp_ledger.ui_perp_ledger.bank', $data, $select_box_options, $readonly, $preserve);
			if($this->debug) {
				echo '<pre>';
				echo $transaction;
				echo '</pre>';
				_debug_array($transaction);
			}
		}
	}
?>
