<?php

	/**
	*	Balance Sheet Report
	*
	*	The balance sheet shows Assets against Liabilities and Equity.
	*	This report has two sections, the first for showing all Assets, the second for Liabilities and Equity.  These 
	*	are the sequence level groups.  Each of these groups shows all General Ledger account groups, grouped by General
	*	Ledger account sections, one line per account group.  Each line shows the group's balance for the current year
	*	and the previous year.
	*
	*	@author Nathan Gray
	*	@ingroup perp_report
	*/

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');

	class balance_sheet extends perp_report {
		/**
		*	@param parameters array Optional.  Report parameters.
		*/
		function balance_sheet(array $parameters = array()) {
			parent::perp_report($parameters);
			$this->template = 'balance_sheet.ods';

			$config = config::read('perp_ledger');

			if(!$this->parameters['end_date']) {
				$this->parameters = array(
					'end_date'		=>	date('U'),
					'assets'		=>	$config['balance_sheet']['assets'],
					'liabilities'		=>	$config['balance_sheet']['liabilities']
				);
			} else {
				$config = new config('perp_ledger');
				$config->read_repository();
				$config->save_value('balance_sheet', array(
					'assets'	=>	$parameters['assets'],
					'liabilities'	=>	$parameters['liabilities']
				), 'perp_ledger');

			}
			$this->parameters['options-assets']		=	ExecMethod('perp_ledger.bo_perp_ledger.get_sequences');
			$this->parameters['options-liabilities']	=	ExecMethod('perp_ledger.bo_perp_ledger.get_sequences');
		}

		/**
		*	Get the data from the report
		*
		*	@param query	array	Optional.  Report parameters, as set by the user.  These are
		*				passed from the template for this report.
		*	@return array Data for the report, which is passed to TBSOOo.
		*/
		public function get_data($query = array()) {
			if(!$query['assets'] || !$query['liabilities']) {
				return false;
			}
			$query['assets'] = explode(',', $query['assets']);
			$query['liabilities'] = explode(',', $query['liabilities']);

			// Set defaults
			if(!$query['end_date']) {
				$query['end_date'] = date('U');
			} else {
				// We want the end date to be inclusive, so we move it to the end of the day.
				$query['end_date'] = strtotime('+1 day', $query['end_date']) - 1;
			}
			$query['start_date'] = mktime(0,0,0,1,1,date('Y', $query['end_date']));
			$this->get_calculated_parameters($query);

			$bo_ledger = CreateObject('perp_ledger.bo_perp_ledger');
			$this->parameters['assets'] = $bo_ledger->get_sequences(array('sequence_id'=>explode(',',$this->parameters['assets'])));
			$this->parameters['liabilities'] = $bo_ledger->get_sequences(array('sequence_id'=>explode(',',$this->parameters['liabilities'])));

			$data = array(
				'asset'		=>	array(
				),
				'liability'	=>	array(
				),
				'current'	=>	array($query['current']),
				'previous'	=>	array($query['previous']),
			);
			$this->get_labels($query, $data);

			// Get data
			$sql = "SELECT sequence_id, sequence_name, 
				section_id, section_name, 
				group_id, group_name,

				current_year.amount as this_year,
				previous_year.amount as last_year
			FROM perp_sequence
			JOIN perp_account_section ON
				perp_account_section.section_sequence = perp_sequence.sequence_id
			JOIN perp_account_group ON
				perp_account_group.group_section = perp_account_section.section_id
			LEFT JOIN perp_account ON
				perp_account.account_group = perp_account_group.group_id
			LEFT JOIN (
				SELECT account,
					sum(amount) AS amount
				FROM perp_general_ledger 
				WHERE trans_date between '" . $GLOBALS['egw']->db->to_timestamp($query['previous']['start_date']) . 
					"' AND '" . $GLOBALS['egw']->db->to_timestamp($query['previous']['end_date']) . "'
				GROUP BY account
			) AS previous_year ON
				perp_account.account_id = previous_year.account 

			LEFT JOIN (
				SELECT account,
					sum(amount) AS amount
				FROM perp_general_ledger 
				WHERE trans_date between '" . $GLOBALS['egw']->db->to_timestamp($query['current']['start_date']) . 
					"' AND '" . $GLOBALS['egw']->db->to_timestamp($query['current']['end_date']) . "'
				GROUP BY account
			)AS current_year ON
				perp_account.account_id = current_year.account 
			WHERE (sequence_id IN (1,2,3))

			GROUP BY sequence_id, section_id, group_id";

			$result =& $GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);

			foreach($result as $row) {
				if(in_array($row['sequence_id'], $query['assets'])) {
					$sequence = 'asset';
				} elseif (in_array($row['sequence_id'], $query['liabilities'])) {
					$sequence = 'liability';
				} else {
					$sequence = 'equity';
				}
				
				$data[$sequence][] = $row;

				// Strange array structure is to make it easy on the template side
				$data['total_' . $row['sequence_id'] . '_' . $row['section_id']][0]['this_year'] += $row['this_year'];
				$data['total_' . $row['sequence_id'] . '_' . $row['section_id']][0]['last_year'] += $row['last_year'];

				$data['total_' . $sequence][0]['this_year'] += $row['this_year'];
				$data['total_' . $sequence][0]['last_year'] += $row['last_year'];

				$data['total'][0]['this_year'] += ($sequence == 'asset' ? 1 : -1) * $row['this_year'];
				$data['total'][0]['last_year'] += ($sequence == 'asset' ? 1 : -1) * $row['last_year'];
			}


//_debug_array($data); die();
			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			$this->filename = lang('report_balance_sheet') . ' ' . $data['lang'][0]['date_range'];

			return $data;
		}

		/**
		*	Determine parameters that can be calculated:
		*	- Current Year
		*	- Previous Year
		*	- Start Date (current year & previous year)
		*	- End Date (current year & previous year)
		*	- Other income & expense groups
		*	- % column total
		*
		*	@param query Array of report parameters
		*/
		protected function get_calculated_parameters(&$query) {
			$query['current']['year'] 	= date('Y', $query['start_date']);
			$query['current']['start_date']	= $query['start_date'];
			$query['current']['end_date']	= $query['end_date'];

			$query['previous']['year']	= date('Y', $query['start_date'] - 1);
			$query['previous']['start_date']= strtotime('-1 year', $query['start_date']);
			$query['previous']['end_date']	= strtotime('-1 year', $query['end_date']);
		}

		/**
		*       Build some translated headings, so this report is language independent
		*
		*       @param query Array of report parameters
		*       @param data Array of report data
		*/
		protected function get_labels(&$query, &$data) {
	
			$data['lang'][] = array(
				'balance_sheet'	=>	lang('report_balance_sheet'),
				'asset'		=>	implode(', ', $this->parameters['assets']),
				'liability'	=>	implode(', ', $this->parameters['liabilities']),
				'balance'	=>	lang('Balance'),
				'date'		=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], time()),
			);
			$data['lang'][0]['total_asset'] = lang('Total %1', $data['lang'][0]['asset']);
			$data['lang'][0]['total_liability'] = lang('Total %1', $data['lang'][0]['liability']);
			$data['lang'][0]['date_range'] = lang('As at %2',
				date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['start_date']),
				date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date'])
			);
		}

		/**
		*	Settings specific to being scheduled
		*/
		public function get_schedule_settings($async_id) {
			$settings = parent::get_schedule_settings($async_id);

			// Add in more months
			foreach($this->parameters as $name => $value) {
				if(strpos($name, 'date') !== false) {
					$date = mktime(0,0,0,date('m'),1,date('Y'));
					for($i = 2; $i < 12; $i++) {
						$this_time = strtotime('-' . $i . ' month', $date);
						$settings['options-' . $name][$this_time] = "-$i months (" . date('Y-m-d', $this_time) . ')';
					}
				}
			}

			$config = config::read('perp_ledger');
			$settings['assets']			=	$config['balance_sheet']['assets'];
			$settings['liabilities']		=	$config['balance_sheet']['liabilities'];
			$settings['options-assets']		=	ExecMethod('perp_ledger.bo_perp_ledger.get_sequences');
			$settings['options-liabilities']	=	ExecMethod('perp_ledger.bo_perp_ledger.get_sequences');

			return $settings;
		}

	}

?>
