<?php

	/**
	*	General Ledger Listing Report
	*	This report lists all transaction lines in a date range grouped by account, one line per transaction line.
	*	For each account, the opening balance, closing balance and net change are shown.
	*
	*	@author Nathan Gray
	*	@ingroup perp_ledger
	*	@ingroup perp_report
	*/
	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	include_once(EGW_INCLUDE_ROOT . '/perp_ledger/inc/class.bo_perp_ledger.inc.php');

	class general_ledger_listing extends perp_report {

		private static $bank_accounts;
		private static $ap_account;
		private static $ar_account;

		/**
		*	Constructor\n
		*	@param parameters array Optional.  Report parameters (start_date, end_date, start_account, end_account, include accounts with no activity)
		*/
		function general_ledger_listing(array $parameters = array()) {
			parent::perp_report($parameters);
			$this->template = 'general_ledger_listing.odt';

			if(!$this->parameters['start_date']) {
				$this->parameters = array(
					'start_date'	=>	date('U', mktime(0,0,0, date('m') - 1, 1, date('Y'))),
					'end_date'	=>	date('U')
				);
			}

		}

		/**
		*	Get the data for the report
		*
		*	@param query    array   Optional.  Report parameters, as set by the user.  These are passed from the template for this report.
		*	@return Data for TinyButStrong to generate the report
		*/
		public function get_data($query = array()) {
			// Set defaults
			$bo_ledger = new bo_perp_ledger();

			// Get list of accounts
			$where = array();
			if($query['start_account']) {
				$where[] = 'account_id >= ' . $query['start_account'];
			}
			if($query['end_account']) {
				$where[] = 'account_id <= ' . $query['end_account'];
			}

			$result = $GLOBALS['egw']->db->select(
				'perp_account',
				array('account_id'),
				$where,
				__LINE__, __FILE__, false, '', 'perp_ledger'
			);
			$account_list = array();
			foreach($result as $row) {
				$account_list[] = $row['account_id'];
			}


			// Get list of bank accounts - we'll use these to try to find who the money went to
			self::$bank_accounts = array_keys($bo_ledger->get_bank_accounts());

			// Get transactions
			$transactions = array();
	
			$bo_ledger->get_transactions(
				array(
					'account'	=>	$account_list,
					'order_by'	=>	'account, trans_date',
					'sort'		=>	'ASC',
					'start_date'	=>	$query['start_date'],
					'end_date'	=>	$query['end_date'],
				), 
				$transactions
			);

			$transaction_types = $bo_ledger->get_transaction_types();
			$account_names = $bo_ledger->get_accounts();

			foreach($transactions as &$transaction) {

				// Format date
				foreach($transaction['lines'] as $key => &$line) {
					if(in_array($line['account'], self::$bank_accounts)) {
						// Try to add in the payor / payee
						$this->add_payor_payee($transaction);
					}
					if(in_array($line['account'], $account_list)) {
						$transaction['transaction']['date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $transaction['transaction']['trans_date']);
						$transaction['transaction']['type'] = $transaction_types[$transaction['transaction']['trans_type']];
						$account_transactions[$line['account']][] = $transaction['transaction'] + $line;
						$account_totals[$line['account']]['total_credit'] += $line['credit'];
						$account_totals[$line['account']]['total_debit'] += $line['debit'];
					}
				}
			}
			//take the date the user entered and push back a day to include the day they entered
			$opening_balance_date = $query['start_date'] - 86400;
			foreach($account_list as $account_id) {
				$opening_balance	=	$bo_ledger->get_balance($opening_balance_date, $account_id);
				$closing_balance	=	$bo_ledger->get_balance($query['end_date'], $account_id);
				$net_change		=	$closing_balance - $opening_balance;
				
				$account = array(
					'name'				=>	$account_names[$account_id],
					'account_id'			=>	$account_id,
					'opening_balance'		=>	$opening_balance,
					'formatted_opening_balance'	=>	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $opening_balance),
					'closing_balance'		=>	$closing_balance,
					'formatted_closing_balance'	=>	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $closing_balance),
					'net_change'			=>	$net_change,
					'formatted_net_change'		=>	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $net_change),
					'total_debit'			=>	$account_totals[$account_id]['total_debit'],
					'formatted_total_debit'		=>	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $account_totals[$account_id]['total_debit']),
					'total_credit'			=>	$account_totals[$account_id]['total_credit'],
					'formatted_total_credit'	=>	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $account_totals[$account_id]['total_credit']),
				);
				//$account['transactions'] = $account_transactions[$account_id];
				//$accounts[] = $account;
				if(!is_array($account_transactions[$account_id]) && $query['no_activity']) {
					$accounts[] = $account;
				}
				if(is_array($account_transactions[$account_id])) {
					foreach($account_transactions[$account_id] as $trans) {
						$accounts[] = $account + $trans;
					}
					unset($account_transactions[$account_id]);
				}
			}
			unset($account_list);

			// Get data
			$data = array(
				'start_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['start_date']),
				'end_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['end_date']),
				'start_account'	=>	$query['start_account'],
				'end_account'	=>	$query['end_account'],
				'no_activity'	=>	($query['no_activity'] ? lang('Including accounts with no activity') : lang('Excluding accounts with no activity')),

				'account'	=>	$accounts,
				'listing'	=>	array(),
			);

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];
			
			return $data;
		}

		/**
		*	Settings specific to being scheduled
		*/
		public function get_schedule_settings($async_id) {
			$settings = parent::get_schedule_settings($async_id);

			if(!$settings['start_date']) {
				$settings['start_date'] = 'this_month_start';
			}

			return $settings;
		}

		/**
		*	For bank transactions, try to figure out where the money came from / went
		*/
		private function add_payor_payee(Array &$transaction) {
			if(!self::$ap_account) {
				if(!$GLOBALS['egw_info']['apps']['perp_ap']) {
					// No AP?  
					return;
				}
				$config = config::read('perp_ap');
				self::$ap_account = $config['ap_account'];
			}
			if(!self::$ar_account) {
				if(!$GLOBALS['egw_info']['apps']['perp_ar']) {
					// No AR?
					return;
				}
				$config = config::read('perp_ar');
				self::$ar_account = $config['ar_account'];
			}
			$payor_payee = null;
			$bank_account = null;
			foreach($transaction['lines'] as $key => &$line) {
				if(in_array($line['account'], self::$bank_accounts)) {
					$bank_account = $key;
				} elseif($line['account'] == self::$ap_account) {
					$trans = new SupplierTransactionLine(null);
					$trans->read($transaction['transaction']['trans_no'], $line['line_no']);
					if($trans->get_supplier()) {
						$payor_payee = ExecMethod('perp_ap.bo_perp_supplier.link_title', $trans->get_supplier());
						$line['payor_payee'] = $payor_payee;
					}
				} elseif ($line['account'] == self::$ar_account) {
					$trans = new ClientTransactionLine(null);
					$trans->read($transaction['transaction']['trans_no']);
					if($trans->get_client()) {
						$payor_payee = ExecMethod('perp_ar.bo_perp_client.link_title', $trans->get_client());
						$line['payor_payee'] = $payor_payee;
					}
				}
			}
			if(!is_null($bank_account)) {
				$transaction['lines'][$bank_account]['payor_payee'] = $payor_payee;
			}
		}
	}

?>
