<?php

	/**
	*	Reconciliation Report
	*	This report lists the bank transactions that have been reconciled to a bank's statement, and
	*	the bank transactions that have yet to be reconciled.
	*
	*	@author Nathan Gray
	*	@ingroup perp_ledger
	*	@ingroup perp_report
	*/
	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	include_once(EGW_INCLUDE_ROOT . '/perp_ledger/inc/class.bo_perp_reconcile.inc.php');
	include_once(EGW_INCLUDE_ROOT . '/perp_ledger/inc/class.bo_perp_ledger.inc.php');

	class reconciliation extends perp_report {

		private static $bo_ledger;
		private static $ap_account;
		private static $ar_account;

		/**
		*	Constructor\n
		*	@param parameters array Optional.  Report parameters (statement_date, bank_account,summary)
		*/
		function __construct(array $parameters = array()) {
			parent::perp_report($parameters);
			$this->template = 'reconciliation.odt';

			self::$bo_ledger = new bo_perp_ledger();

			if(!$this->parameters['date']) {
				$config = config::read('perp_ledger');
				$this->parameters = array(
					'date'			=>	date('U', mktime(0,0,0, date('m') - 1, date('t'), date('Y'))),
					'account_id'		=>	$config[__CLASS__]['account_id'],
					'options-account_id'	=>	self::$bo_ledger->get_bank_accounts(),
				);

			} else {
				config::save_value(__CLASS__, $this->parameters['account_id'], 'perp_ledger');
			}

			$this->parameters['options-date'] = bo_perp_reconcile::get_date_list($this->parameters['account_id']);
		}

		/**
		*	Get the data for the report
		*
		*	@param query    array   Optional.  Report parameters, as set by the user.  These are passed from the template for this report.
		*	@return Data for TinyButStrong to generate the report
		*/
		public function get_data($query = array()) {
			// Set defaults
			config::save_value(__CLASS__, array('account_id'=>$query['account_id']), 'perp_ledger');


			$data = array(
				'details'	=>	$query['details'] ? true : null,

				'opening_date'	=>	0,
				'closing_date'	=>	$query['date'],
				
				'opening_balance'	=>	0,
				'closing_balance'	=>	self::$bo_ledger->get_bank_balance($query['date'], $query['account_id']),

				'account_name'	=>	self::$bo_ledger->get_account_name($query['account_id']),
			);
			$data += self::$bo_ledger->get_bank_account($query['account_id']);

			// Try to find our start date
			// Dates are sorted descending
			while( list($date, $display) = each($this->parameters['options-date'])) {
				if($date == $data['closing_date']) {
					list($data['opening_date']) = each($this->parameters['options-date']);
					break;
				}
			}

			// If we got a start date, we can find the opening balance
			if($data['opening_date']) {
				$data['opening_balance'] = self::$bo_ledger->get_bank_balance($data['opening_date'], $query['account_id']);
			}

	
			// Get the transactions
			$transaction_query = array(
				'bank_account_id'	=>	$query['account_id'],
				'start_date'		=>	$data['opening_date'],
			);
			// No closing date will get any transactions posted after as well
			// $transaction_query['end_date'] = $data['closing_date'];

			bo_perp_reconcile::get_reconciliation(
				$transaction_query,
				$transactions
			);

			// Sort transactions
			$cleared = array(
				'withdrawal'	=>	array(),
				'deposit'	=>	array()
			);
			$uncleared = array(
				'withdrawal'	=>	array(),
				'deposit'	=>	array()
			);
			$new = array(
				'withdrawal'	=>	array(),
				'deposit'	=>	array()
			);
			
			foreach($transactions as &$transaction) {
				if(!is_array($transaction)) {
					continue;
				}
				
				// Determine if money went in or came out
				$direction = ($transaction['amount'] < 0 ? 'withdrawal' : 'deposit');

				// Format dates to user's preference
				$transaction['formatted_trans_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $transaction['trans_date']);

				// Sort transactions
				if($transaction['clear'] && $transaction['trans_date'] < $data['closing_date']) {
					$cleared[$direction]['total'] += $transaction['amount'];
					$cleared[$direction][] =& $transaction;
				} elseif($transaction['trans_date'] > $data['closing_date']) {
					$new[$direction]['total'] += $transaction['amount'];
					$new[$direction][] =& $transaction;
				} else {
					$uncleared[$direction]['total'] += $transaction['amount'];
					$uncleared[$direction][] =& $transaction;
				}
			}

			foreach(array('cleared', 'uncleared', 'new') as $group) {
				foreach($$group as $direction => &$transactions) {
					$data["{$group}_{$direction}_total"] = $transactions['total'];
					$data["formatted_{$group}_{$direction}_total"] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $account['currency'], $transactions['total']);
					unset($transactions['total']);
					$data[$group . '_' . $direction] = $query['details'] ? $transactions : array();
				}
			}
			$data['cleared_balance'] = $data['opening_balance'] + $data['cleared_withdrawal_total'] + $data['cleared_deposit_total'];
			
			foreach($data as $key => $value) {
				if(strpos($key, 'date') !== false) {
					if($value) {
						$data['formatted_' . $key] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $value);
					} else {
						$data['formatted_' . $key] = '';
					}
				} elseif (strpos($key, 'balance') !== false) {
					$data['formatted_' . $key] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $account['currency'], $value);
				}
			}

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			$this->filename = lang('Reconciliation %1: %2', $data['account_name'], $data['formatted_closing_date']);
 //_debug_array($data); die();
			return $data;
		}

		/**
		*	Settings specific to being scheduled
		*/
		public function get_schedule_settings($async_id) {
			$settings = parent::get_schedule_settings($async_id);

			if(!$settings['start_date']) {
				$settings['start_date'] = 'this_month_start';
			}

			return $settings;
		}

		/**
		*	For bank transactions, try to figure out where the money came from / went
		*/
		private function add_payor_payee(Array &$transaction) {
			if(!self::$ap_account) {
				if(!$GLOBALS['egw_info']['apps']['perp_ap']) {
					// No AP?  
					return;
				}
				$config = config::read('perp_ap');
				self::$ap_account = $config['ap_account'];
			}
			if(!self::$ar_account) {
				if(!$GLOBALS['egw_info']['apps']['perp_ar']) {
					// No AR?
					return;
				}
				$config = config::read('perp_ar');
				self::$ar_account = $config['ar_account'];
			}
			$payor_payee = null;
			$bank_account = null;
			foreach($transaction['lines'] as $key => &$line) {
				if(in_array($line['account'], self::$bank_accounts)) {
					$bank_account = $key;
				} elseif($line['account'] == self::$ap_account) {
					$trans = new SupplierTransactionLine(null);
					$trans->read($transaction['transaction']['trans_no'], $line['line_no']);
					if($trans->get_supplier()) {
						$payor_payee = ExecMethod('perp_ap.bo_perp_supplier.link_title', $trans->get_supplier());
						$line['payor_payee'] = $payor_payee;
					}
				} elseif ($line['account'] == self::$ar_account) {
					$trans = new ClientTransactionLine(null);
					$trans->read($transaction['transaction']['trans_no']);
					if($trans->get_client()) {
						$payor_payee = ExecMethod('perp_ar.bo_perp_client.link_title', $trans->get_client());
						$line['payor_payee'] = $payor_payee;
					}
				}
			}
			if(!is_null($bank_account)) {
				$transaction['lines'][$bank_account]['payor_payee'] = $payor_payee;
			}
		}
	}

?>
