<?php

	/**
	*	Trial balance report
	*
	*	This report lists every General Ledger account and its current balance, as calculated
	*	by the sum of transaction amounts between the start date and the end date, one line per account.
	*
	*	@author Nathan Gray
	*
	*	@ingroup perp_ledger
	*	@ingroup perp_report
	*/
	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');

	class trial_balance extends perp_report {
		/**
		*	Report parameters - start date
		*/
		private $start_date = null;

		/**
		*	Report parameters - start date
		*/
		private $end_date = null;

		/**
		*	Constructor\n
		*	@param parameters array Optional.  Report parameters (start_date and end_date)
		*/
		function trial_balance(array $parameters = array()) {
			parent::perp_report($parameters);
			$this->template = 'trial_balance.odt';

			// These are defaults
			if(!$this->parameters['end_date'] && !$this->parameters['start_date']) {
				$this->parameters = array(
					'start_date'	=>	date('U', mktime(0,0,0, date('m'), 1, date('Y'))),
					'end_date'	=>	date('U'),
				);
				$parameters = $this->parameters;
			}

		}

		/**
		*	Get the data for the report
		*
		*	@param query	array	Optional.  Report parameters, as set by the user.  These are
		*				passed from the template for this report.
		*	@return array Account balances
		*/
		public function get_data($query = array()) {
			// Set defaults
			if(!$this->start_date) {
				$this->start_date = $query['start_date'] ? $query['start_date'] : 0;
			}
			if(!$this->end_date) {
				$this->end_date = $query['end_date'] ? $query['end_date'] : date('U');
			}

			// Get data
			$data = ExecMethod2('perp_ledger.bo_perp_ledger_report.trial_balance', $this->start_date, $this->end_date);

			// Massage data
			$data['content'] = $data['data'];
			if(!is_array($data['content'])) {
				$data['content'] = array();
			}
			array_shift($data['content']);
			unset($data['data']);
			foreach($data['content'] as &$row) {
				if(!isset($row['debit'])) {
					$row['debit'] = '';
				} elseif (!isset($row['credit'])) {
					$row['credit'] = '';
				}
			}

			// Convert dates to user's format
			$data['end_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $this->end_date);

			if($this->start_date != 0) {
				$data['date_range'] = lang('Between %1 and %2', date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $this->start_date), $data['end_date']);
			} else { 
				$data['date_range'] = lang('as at %1', $data['end_date']);
			}
			
			$data['format'] = '0.00';

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			return $data;
		}

		/**
		*	Settings specific to being scheduled
		*/
		public function get_schedule_settings($async_id) {
			$settings = parent::get_schedule_settings($async_id);

			// Add in more months
			foreach($this->parameters as $name => $value) {
				if(strpos($name, 'date') !== false) {
					$date = mktime(0,0,0,date('m'),1,date('Y'));
					for($i = 2; $i < 12; $i++) {
						$this_time = strtotime('-' . $i . ' month', $date);
						$settings['options-' . $name][$this_time] = "-$i months (" . date('Y-m-d', $this_time) . ')';
					}
				}
			}
			$settings['options-start_date'][0] = lang('No start date');

			if(!$settings['start_date']) {
				$settings['start_date'] = 0;
			}
			if(!$settings['end_date']) {
				$settings['end_date'] = 'now';
			}
			return $settings;
		}
	}

?>
