<?php
	/**************************************************************************\
	* eGroupWare Workflow - pERP Manufacturing build agent			   *
	* ------------------------------------------------------------------------ *
	* This program is free software; you can redistribute it and/or modify it  *
	* under the terms of the GNU General Public License as published           *
	* by the Free Software Foundation; either version 2 of the License, or     *
	* any later version.                                                       *
	\**************************************************************************/

	/**
	 *	pERP Build Agent : business layer
	 *
	 *	This class connects the workflow agent to pERP Manufacturing, allowing
	 *	workflow based manufacturing.
	 *
	 *	Copied from smtp_mail by regis.leroy@glconseil.com
	 *
	 *	@package workflow
	 *	@author nathan@goarctic.com
	 */

	// Needed in case of exceptions
	require_once(EGW_INCLUDE_ROOT . '/phpgwapi/inc/common_functions.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/phpgwapi/inc/class.egw_exception.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/workflow/inc/class.bo_agent.inc.php');
	require_once('class.so_agent_perp_build.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_manufacturing.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_order.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_center.inc.php');
	
	// For checking serial numbers
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_stock.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');
	
	class bo_agent_perp_build extends bo_agent
	{
		var $public_functions = array(
			'bo_agent_perp_build'		=> true,
			'load'				=> true,
			'save'				=> true,
			'getAdminActivityOptions'	=> true,
			'decode_fields_in_final_array' 	=> true,
		);
			
		// Object for handling the actual building
		private $bo_perp_manufacturing;

		// Get some info about the work order
		protected $bo_perp_work_order;

		// For checking serial numbers
		protected $bo_invenvory;
		protected $bo_work_center;

		protected $part_list = array();

		private $debug = false;

		function __construct()
		{
			parent::__construct();

			$this->so_agent =& new so_agent_perp_build();

			$this->bo_perp_manufacturing =& new bo_perp_manufacturing();
			$this->bo_perp_work_order =& new bo_perp_work_order();
			$this->bo_inventory =& new bo_perp_inventory();
			$this->bo_work_center =& new bo_perp_work_center();

			$this->showProcessConfigurationFields = array();

			$this->ProcessConfigurationFieldsdefault = array(
				'work_center_id' 	=>	false,
			);
			
			$this->title = lang('pERP Manufacturing Build Agent');
			$this->description = lang('This agent gives the activity the ability to perform part of a build, as defined by a Bill of Materials');
			$this->help = lang('Use <a href="%1">Work Centers</a> to create work centers.', $GLOBALS['egw']->link('/index.php',array('menuaction' => 'perp_manufacturing.ui_perp_work_center.work_center_list')));
			$this->help .= "<br />\n".lang('You should have one work center agent per activity, and one activity per work center in your Bill of Materials.  ');
			$this->help .= "<br />\n".lang('If you want to use this agent with an interactive activity, your template should include {parts} to display the parts and get serial numbers.  You may also need a submit button so the serial numbers can be processed.  This agent will complete the activity when done, so be careful if you want to mix agents.  If it is not interactive, you better be sure that any needed serial numbers are provided with the set_serials() function, but I do not know how to do that yet.  It is better to have any activities that could use serialized stock interactive.');

			$this->fields = array(
				'display_parts'		=>	array(
					'type'		=>	'select',
					'label'		=>	lang('Display parts:'),
					'value'		=>	0,
					'values'	=>	array(
						0	=>	lang('None'),
						1	=>	lang('All'),
						2	=>	lang('Controlled / Serialized')
					)
				),
				'work_center_id'	=> array(
					'type'		=> 'select',
					'label'	=> lang('Build location:'),
					'value' => '',
					'values'=>  array(
						0	=>	'* ' . lang('None'),
						-1	=>	'* ' . lang('Build complete')
					) + ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center_list')
				),
			);
			
		}

		/**
		 * * Factory: Load the agent values stored somewhere in the agent object and retain the agent id
		*  * @param $agent_id is the agent id
		*  * @param $really_load boolean, true by default, if false the data wont be loaded from database and
		*  * the only thing done by this function is storing the agent_id (usefull if you know you wont need actual data)
		*  * @return false if the agent cannot be loaded, true else
		 */
		function load($agent_id, $really_load=true)
		{
			//read values from the so_object
			if ($really_load)
			{
				$values =& $this->so_agent->read($agent_id);
				if(is_array($values)) {
					foreach($values as $key => $value)
					{
						//load only known fields
						if (isset($this->fields[$key]))
						{
							$this->fields[$key]['value'] = $value;
							//echo "<br> DEBUG loading value $value for $key";
						}
					}
				} else {
					echo 'Could not load values for agent';
					return false;
				}
			}
			//store the id
			$this->agent_id = $agent_id;
		}

		/**
		 * * Save the agent
		*  * @return false if the agent cannot be saved, true else
		 */
		function save()
		{
			//make a simplified version of $this->fields with just values
			$simplefields = Array();
			foreach ($this->fields as $field => $arrayfield)
			{
				$simplefields[$field] = $arrayfield['value'];
			}
			return $this->so_agent->save($this->agent_id, $simplefields);
		}

		
		/**
		 * * this function lists activity level options avaible for the agent
		*  * @return an associative array which can be empty
		 */
		function getAdminActivityOptions ()
		{
			return $this->fields;
		}
		
		//initialize objects we will need for the build and retrieve the conf
		function init()
		{
			//set the $this->conf
			$this->getProcessConfigurationFields($this->activity->getProcessId());
			$query = array();
			$this->part_list = array();
			$this->get_part_list($query, $this->part_list);
		}

		/**
		*	Set the serial numbers for serialized components
		*	that are used at this work center
		*
		*	@param serials an array of component arrays, each with keys stock_id, serial, quantity
		*/
		public function set_serials($serials) {
			$query = array();
			$this->get_part_list($query, $parts, $readonlys);
			if(count($parts) == 0) {
				// No controlled parts
				if($this->debug) {
					echo 'Setting serials, but no parts found.<br />';
				}
				return true;
			}

			foreach($parts as &$component) {
				foreach($serials as $key => $serial_array) {
					if($serial_array['stock_id'] == $component['stock_id']) {
						$component['serial_number'] = array(
							$serial_array['serial_number']	=>	$serial_array['quantity']
						);
						unset($serials[$key]);
						break;
					}
				}
			}
			if(count($serials) > 0) {
				$this->error[] = lang('Some serials left without parts');
				return false;
			} else {
				$this->part_list = $parts;
				return true;
			}
		}
		
		function build() {
			$work_center_id = $this->fields['work_center_id']['value'];
			$work_order_id = $this->instance->get('work_order_id');
			$detail_id = $this->instance->get('detail_id');

			if(!$this->validate($this->part_list)) {
				return false;
			}

			if ($this->debug) {
				echo "Building work order ID $work_order_id:$detail_id at Work Center ID $work_center_id<br />";
				_debug_array($this->fields);
			}
			if($work_center_id > 0) {
				$result = $this->bo_perp_manufacturing->build($work_order_id, $detail_id, $work_center_id, time(), '', $this->part_list);
			} elseif ($work_center_id = -1) {
				$work_order = $this->bo_perp_work_order->get_work_order($work_order_id);
				$result = $this->bo_perp_manufacturing->finish_build($work_order, $detail_id, $work_center_id, time());
			}
			if($result === true) {
				// Interactive activities should not call complete
				//if($this->instance->activity->isInteractive) {
					$this->instance->complete();
				//}
			} else {
				$this->error[] = lang('Build of %1:%2 did not complete at Work Center \'%3\'', 
					$work_order_id,
					$detail_id,
					$work_center_id
				);

				if($result) {
					$this->error[] = $result;
				}
			}
			return ($result === true);
		}

		/**
		*	Checks to make sure we're good to do this build, and all needed
		*	information, such as location and serial numbers, is provided.
		*
		*	@param components Optional array of components to validate.  If not provided, the parts list as specified will be used.
		*
		*	@return true / false
		*/
		public function validate($components = null) {
			$work_center_id = $this->fields['work_center_id']['value'];
			$work_order_id = $this->instance->get('work_order_id');
			$detail_id = $this->instance->get('detail_id');

			if(!$work_order_id) {
				$this->error[] = "Missing work order number";
				return false;
			}
			if(!$detail_id) {
				$this->error[] = "Missing work order detail number";
				return false;
			}

			// Check to make sure any controlled stock has a number
			$query = array();
			if(is_null($components)) {
				$components = $this->part_list;
			}

			if(count($this->part_list) > 0) {
				foreach($components as $key => $component) {
					if($component['controlled']) {
						if(!is_array($component['serial_number'])) {
							$this->error[] = lang('Missing serial number for %1', ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['stock_id']));
							return false;
						} else {
							$count = 0;
							foreach($component['serial_number'] as $sn => $qty) {
								if($this->check_serial($component['stock_id'], $sn, $qty)) {
									$count += $qty;
								} else {
									$this->error[] = lang('Invalid serial number in validate(): %1', $sn);
									return false;
								}
							}
							if($count != $component['quantity']) {
								$this->error[] = lang('Count does not match quantity: %1 != %2', $count, $component['quantity']);
								return false;
							}
						}
					}
				}
			}
			if($this->debug) {
				echo 'Good to go?  ' . (count($this->error) == 0 ? 'Yes' : 'No');
			}
			return (count($this->error) == 0);
		}

		/**
		*	Load the parts list for this work center
		*/
		public function get_part_list(&$query, &$part_list, &$readonlys = array()) {
			$work_center_id = $this->fields['work_center_id']['value'];
			$work_order_id = $this->instance->get('work_order_id');
			$detail_id = $this->instance->get('detail_id');

			$agent_query = array(
				'work_order_id'		=>	$work_order_id,
				'detail_id'		=>	$detail_id,
				'work_center_id'	=>	$work_center_id,
				'num_rows'		=>	-1
			);
			$query = array_merge($query, $agent_query);

			if($this->debug) {
				echo 'Looking for parts using these parameters:';
				_debug_array($query);
			}
			$this->bo_perp_work_order->get_work_order_detail_bom($query, $part_list, $readonlys);
			if($this->debug) {
				echo 'Found these parts:';
				_debug_array($part_list);
			}
		}

		/**
		*	Check provided components for serials
		*
		*	@param components An array of (component_id => (int), serial_number => (string), quantity => (double) for each component
		*/
		public function check_serials(Array $components) {
			$good = true;
			foreach($components as $component) {
				$good = $good && $this->check_serial($component['stock_id'], $component['serial_number'], $component['quantity']);
			}
			return $good;
		}

		/**
		*	Check for valid serial
		*	@todo Check quantities too, not just if some are at this location
		*/
		protected function check_serial($component_id, &$serial_number, $quantity) {
			if(!$component_id) {
				// Why check?
echo "check_serial($component_id, &$serial_number, $quantity)<br />";
				return true;
			}

			$serial_number = strtoupper(trim($serial_number));
			if($serial_number == '') {
				$this->error[] = lang('Missing serial number for %1', ExecMethod('perp_inventory.bo_perp_stock.link_title', $component_id));
				return false;
			}

			$valid_check = array(
				'stock_id'		=>	$component_id,
				'serial_number'		=>	$serial_number
			);
			$used_check = array(
				'component_stock_id'	=>	$component_id,
				'serial_number'		=>	$serial_number
			);
			$location_check = array(
				'work_center_id'	=>	$this->fields['work_center_id']['value'],
				'template'		=>	false,
			);


			// Check if the component is serialized.  If not, it's good.
			$result = bo_perp_stock::check_serial($component_id, $serial_number);
		
			if($this->debug) {
				echo "Checking serial $serial_number for stock_id $component_id: $result<br />";
			}
			switch ($result) {
				case bo_perp_stock::NOT_CONTROLLED:
					return true;
					break;
				case bo_perp_stock::NOT_VALID:
					$this->error[] = lang('%1 is not a valid serial number for %2', $serial_number, ExecMethod('perp_inventory.bo_perp_stock.link_title', $component_id));
					return false;
					break;
				case bo_perp_stock::FORMAT_OK:
					// Format looks good, but we need to create the serial number before we use it

					// Figure out inventory location
					$this->bo_work_center->get_work_center_location_list($location_check, $location_list);
					$location_id = $location_list[0]['location_id'];

					bo_perp_inventory::add_serial($component_id, $serial_number, $quantity, $location_id);
					return true;
					break;
				case bo_perp_stock::SERIAL_OK:
					// Serial is OK, need to check that it's here
					break;
				default:
					// Not a valid response
					throw new egw_exception('Not a valid response from check_serial: ' . print_r($response, true));
					break;
			}


			$serials = array();
			$available_count = 0;
			$this->bo_inventory->get_controlled_inventory_list($valid_check, $serials);

			if(count($serials) == 0) {
				return false;
			} else {
				// Check location.  We do it like this, instead of as a parameter in the query, so we can find out where it really is.
				$here = false;
				$locations = array();
				$this->bo_work_center->get_work_center_location_list($location_check, $location_list);
				$location_id = $location_list[0]['location_id'];

				foreach($serials as $info) {
					$available_count += $info['quantity'];
					if($info['location_id'] == $location_id) {
						// OK, it's here
						$here = true;
						break;
					} else {
						$locations[$info['location_id']] = ExecMethod('perp_inventory.bo_perp_location.link_title', $info['location_id']);
					}
				}
			}

			$assemblies =  ExecMethod('perp_manufacturing.so_perp_serial_assembly.get_serial_assemblies', $used_check);
			if( count($assemblies) > 0) {
				// Serialized components there's only 1 of
				if($stock['controlled'] == 2) {
					$this->error[] = lang('%2 serial number %1 has already been used', $serial_number, ExecMethod('perp_inventory.bo_perp_stock.link_title', $component_id));
					return false;
				} else {
					if($available_count < $quantity) {
						$this->error[] = lang('Not enough %2 in batch %1.  There is %3 left.', 
							$serial_number, 
							ExecMethod('perp_inventory.bo_perp_stock.link_title', $component_id),
							$available_count
						);
						return false;
					}
				}
			}

			if(!$here && count($serials) == 1) {
				/**
				*	Found it, but it's not here.  But it is, because someone scanned it.  We'll trust the guy that
				*	has the part in his hand over the computer, but only if this is a unique serial accross all parts. 
				*	Could be he scanned s/n 002 for part B (which is somewhere else) when he should have scanned s/n 002
				*	for part A, which is in his hand, but recorded as being somewhere else.
				*/
				$result = $this->bo_inventory->move(
					$serials[0]['location_id'], 'reserved', 
					$location_id, 'reserved', 
					$component_id, $quantity, array($serial_number, $quantity)
				);
				if($result === true) {
					return true;
				} else {
					$this->error[] = $result;
					return false;
				}
			} elseif (!$here) {
				$this->error[] = lang(
					'%1 number %2 is not here.  I have it recorded at %3, but I cannot move it automatically.',
					ExecMethod('perp_inventory.bo_perp_stock.link_title', $component_id),
					$serial_number,
					implode(', ', $locations)
				);
				return false;
			}
			return true;
		}
	}
?>
