<?php

	class bo_perp_bill_of_material {
		
		private $so;
		
		public function bo_perp_bill_of_material() {
			$this->so = CreateObject('perp_manufacturing.so_perp_bill_of_material');
		}

		/*
		*	query (
		*		stock_id	=>	stock_id
		*		filter 		=>	category
		*		filter2		=>	status
		*		as_of_date	=>	effective date for historical searching
		*	)
		*/
		public function get_bill_of_material_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_READ, 'perp_manufacturing');
			$edit = $GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_EDIT, 'perp_manufacturing');
			$delete = $GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_DELETE, 'perp_manufacturing');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			if(!isset($query['order'])) {
				$query['order'] = 'stock_id, bill_of_material_id DESC';
			}
			if($query['cat_id'] || $query['stock_category']) {
				$query['stock_category'] = $query['cat_id'] ? $query['cat_id'] : $query['stock_category'];
			}
			$query['status'] = $query['filter2'] ? $query['filter2'] : $query['status'];
			$list = $this->so->get_bill_of_material_list($query);

			if(count($query) == 0) {
				// TODO: Something goes here?  Probably not...
			} else {
				$count = count($list);
				$rows = array_slice($list, $query['start'], $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs']);
				$rows['edit'] = $edit;
				$rows['delete'] = $delete;
				return $count;
			}
		}

		public function get_bill_of_material($bom_id) {
			if($GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_READ, 'perp_manufacturing')) {
				$bom = $this->so->get_bill_of_material($bom_id);
				
				return $bom;
			}
		}

		public function get_bill_of_material_details($bom_id, $category) {
			//TODO: write this, if it's needed
		}

		/**
		*	Sets the bill of materials
		*	Things get a little tricky here, because if a bom for a stock item currently
		*	exists, we actually create a copy, set the effective to date of the old one 
		*	to the effective after date of the 'new' bom (or today if not set).
		*	This means I can't save to the db until all changes are made.
		*
		*	Comments and time changes don't count as "changes", so won't cause a new bill of materials
		*/
		public function set_bill_of_material($bom_id, $bom) {
			$edit = $GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_EDIT, 'perp_manufacturing');

			if(!$edit) {
				return lang('You are not allowed to edit bills of materials');
			} elseif (!isset($bom['stock_id'])) {
				return lang('Missing stock item bill of materials is for');
			} elseif (!isset($bom['details'])) {
				return lang('Missing details for bill of materials');
			}

			$loops = $this->check_for_loops($bom_id, $bom);
			if($loops) {
				return $loops;
			}

			if(!is_numeric((int)$bom['effective_to_date'])) {
				$bom['effective_to_date'] = null;
			}

			$config = config::read('perp_manufacturing');
			
			if($config['auto_history_bom'] && $bom['effective_after_date'] <= time()) {
				$old_bom = $this->get_bill_of_material($bom_id);
				if($old_bom['bill_of_material_id']) {
					$same = True;
					if($old_bom['details']) {
						$unchecked = $bom['details'];
						foreach($old_bom['details'] as $key => $detail) {
							$same = $same && $this->check_bill_of_material_detail($bom['details'][$detail['bill_of_material_detail_id']], $detail);
							unset($unchecked[$detail['bill_of_material_detail_id']]);
						}
						if(count($unchecked) > 0) {
							// New item on BOM
							$same = false;
						}
					}
					$same = $same && ($bom['effective_after_date'] == $old_bom['effective_after_date']);
					$same = $same && ($bom['stock_id'] == $old_bom['stock_id']);
		
					if($same) {
						return $this->so->set_bill_of_material($bom_id, $bom);
					}
					if($bom['effective_after_date'] == $old_bom['effective_after_date']) {
						$bom['effective_after_date'] = $old_bom['effective_to_date'];
					}
					if(!$bom['effective_after_date']) {
						$bom['effective_after_date'] = date('U');
					}
					if(!$old_bom['effective_to_date']) {
						$old_bom['effective_to_date'] = $bom['effective_after_date'];
					}
					
					// Disable previous bill of materials
					$this->so->set_bill_of_material($old_bom['bill_of_material_id'], $old_bom);

					// Update timestamp
					$GLOBALS['egw']->contenthistory->updateTimeStamp('perp_manufacturing:BOM', $old_bom['bill_of_material_id'], 'delete', time());

					foreach($bom['details'] as &$detail) {
						unset($detail['bill_of_material_detail_id']);
					}
				}
				return $this->so->set_bill_of_material(null, $bom);
			} elseif(!$bom_id) {
				// Creating a new BOM, possibly replacing an existing one
				$query = array(
					'stock_id'	=>	$bom['stock_id'],
					'as_of_date'	=>	$bom['effective_after_date'],
				);
				$count = $this->get_bill_of_material_list($query, $bom_list);
				if($count > 0) {
					// Set the effective to date of the old BOM to the day before the start date of the new BOM
					$old_bom = $this->get_bill_of_material($bom_list[0]['bill_of_material_id']);
					$old_bom['effective_to_date'] = $bom['effective_after_date'] - 1;
					foreach($bom['details'] as &$detail) {
						unset($detail['bill_of_material_detail_id']);
					}
					$this->set_bill_of_material($old_bom['bill_of_material_id'], $old_bom);

					// Update timestamp
					$GLOBALS['egw']->contenthistory->updateTimeStamp('perp_manufacturing:BOM', $old_bom['bill_of_material_id'], 'delete', time());
				}
			}
			$new_bom = !$bom_id;
			$bom_id = $this->so->set_bill_of_material($bom_id, $bom);
			
			// Update timestamp
			$GLOBALS['egw']->contenthistory->updateTimeStamp('perp_manufacturing:BOM', $bom_id, ($new_bom ? 'add' : 'modify'), time());

			return $bom_id;
		}

		/**
		*	Check to see if two bill of material detail lines are equivalent (not identical)
		*/
		private function check_bill_of_material_detail($new_detail, $old_detail) {
			$same = True;
			$same = $same && ($new_detail['work_center'] == $old_detail['work_center']);
			$same = $same && ($new_detail['component'] == $old_detail['component']);
			$same = $same && ($new_detail['quantity'] == $old_detail['quantity']);
			return $same;
		}

		/* 
		*	Actually deletes the bom, not disables
		*/
		public function delete_bill_of_material($bom_id = '', $stock_id = '') {
			$delete = $GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_DELETE, 'perp_manufacturing');
			if(!$delete) {
				return false;
			}
			return $this->so->delete_bill_of_material($bom_id, $stock_id);
		}


		/** 
		*	Checks the bill of materials for loops where a stock item is a component of itself
		*	by taking each component and walking up the build tree (post-order) to see if it
		*	is used anywhere else.
		*
		*	 A
		*	|- B
		*	|  |- G
		*	|  |- H
		*	|  |  |- K
		*	|  |  |- L
		*	|  |  |  |- N
		*	|  |  |  |- O
		*	|  |  |  |- B <-- Should not be added to L's BOM
		*	|  |  |
		*	|  |  |- M
		*	|  |
		*	|  |- F
		*	|
		*	|- C
		*	|  |- I
		*	|  |- J
		*	|     |- B <-- This is OK
		*	|
		*	|- D
		*	|- E
		*	|- F
		*
		*	This will be discovered and removed when checking the bill of materials for L.
		*
		* @param int $bom_id - the ID of the bill of materials
		* @param array $bom - the bill of materials to check, must include $bom['details'] with the components - optional
		*	if $bom is not provided, it will be read from the database, which is probably not what you want
		*/
		public function check_for_loops($bom_id, &$bom = '') {
			if(!$bom) {
				$bom = $this->get_bill_of_materials($bom_id);
			}
		
			$loop = false;
			foreach($bom['details'] as $key => &$detail) {
				if($detail['stock_type_flag'] == 'R') {
					// Raw stock items have no BOM, so there's no loop danger
					continue;
				}
				if($detail['component'] != $bom['stock_id']) {
					// Check for hidden loops deep inside the tree
					$loop = $this->so->check_for_loops($bom['stock_id'], $detail['component']);
				} else {
					// BOM requires itself directly - fix it
					$loop = $bom['stock_id'];
					unset($bom['details'][$key]);
					return false;
				}
				if($loop) {
					return lang(
						'Loop in bill of materials (a part requires itself), check bill of materials #%2 for "%1", it requires %3 so it cannot be added here.', 
						ExecMethod('perp_inventory.bo_perp_stock.link_title', $loop['component_id']),
						$loop['bill_of_material_id'],
						ExecMethod('perp_inventory.bo_perp_stock.link_title', $bom['stock_id'])
					);
				}
			}
			return false;
		}

		/** 
		* Disables current BoM for listed stock_id, creates new Bom for replacement
		* Updates any BoM that uses the stock_id with $replacement
		*/
		public function disable_stock($stock_id, $replacement) {
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);

			$count = 0;
			$status_flags = ExecMethod('perp_inventory.bo_perp_stock.get_status_flags');

			// Create matching BOM for replacement
			$bom_id = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $stock_id);
			if($bom_id) {
				$bom = $this->get_bill_of_material($bom_id);
				$bom['effective_to_date'] = date('U');
				$bom['comment'] .= "\n" . $status_flags[$stock['status_flag']];
				$this->so->set_bill_of_material($bom_id, $bom);

				$replacement_bom_id = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $replacement);
				if(!$replacement_bom_id) {
					$bom['stock_id'] = $replacement;
					unset($bom['comment']);
					$this->so->set_bill_of_material($bom_id, $bom);
				}
				$count++;
			}

			// Discontinue any future BOMs as well
			$future_query = array(
				'stock_id'	=>	$stock_id,
				'status'	=>	'future',
				'order'		=>	'bill_of_material_id'
			);
			$future_boms = $this->so->get_bill_of_material_list($future_query);
			foreach($future_boms as $data) {
				$bom = $this->get_bill_of_material($data['bill_of_material_id']);
				$bom['effective_to_date'] = date('U');
				$bom['comment'] .= "\n" . $status_flags[$stock['status_flag']];
				$this->so->set_bill_of_material($data['bill_of_material_id'], $bom);
			}

			$using = $this->so->get_used_by($stock_id);
			foreach($using as $bom_id) {
				$bom = $this->get_bill_of_material($bom_id);

				// Skip old BOMs
				if($bom['effective_to_date'] && $bom['effective_to_date'] < date('U') || !is_array($bom['details'])) {
					continue;
				}
				$changed = false;
				foreach($bom['details'] as $key => &$detail) {
					if($detail['component'] == $stock_id) {
						if($replacement) {
							$detail['component'] = $replacement;
						} else {
							unset($bom['details'][$key]);
						}
						$changed = true;
					}
				}
				if($changed) {
					unset($bom['bom_id']);
					$bom['effective_after_date'] = strtotime('tomorrow', date('U'));
					$this->set_bill_of_material(false, $bom);
				}
				$count++;
			}
			return $count;
		}

		public function get_bill_of_material_status_list() {
			return array(
				''	=>	lang('all'),
				'current'	=>	lang('Current'),
				'previous'	=>	lang('previous'),
				'future'	=>	lang('Future'),
			);
		}

		/**
		*	Copy a bill of materials
		*
		*	Copies a bill of materials, along with all its components.  The 
		*	original BOM will be unchanged.  The effectived date for the new BOM will
		*	be tomorrow.
		*/
		public function clone_bill_of_material($bom_id, $new_stock_id = null) {
			$bom = $this->get_bill_of_material($bom_id);

			unset($bom['bill_of_material_id']);
			$bom['effective_after_date'] = strtotime('tomorrow');

			if($new_stock_id) {
				$bom['stock_id'] = $new_stock_id;
			}
			if(is_array($bom['details'])) {
				foreach($bom['details'] as $key => $detail) {
					unset($bom['details'][$key]['bill_of_material_detail_id']);
				}
			}
			$new_bom_id = $this->set_bill_of_material(null, $bom);
			return $new_bom_id;
		}

		/**
		*	Get a list of bill of material IDs that use the provided stock item
		*	as a component.
		*	
		*	@param stock_id
		*	
		*	@return array of bill of material IDs
		*/
		public function get_used_by($stock_id) {
			if($GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_READ, 'perp_manufacturing')) {
				return $this->so->get_used_by($stock_id);
			} else {
				return array();
			}
		}

		/**
		*	Get a list of bill of material information about BOMs that use a
		*	given stock item
		*
		*	@nextmatch
		*	stock_id	=>	stock id to look for
		*/
		public static function get_used_by_list(&$query, &$rows = array()) {
			if(!$GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_READ, 'perp_manufacturing')) {
				$rows = array();
				return 0;
			}
			if($query['filter2']) {
				$query['status'] = $query['filter2'];
			}

			$rows = so_perp_bill_of_material::get_used_by_list($query);

			return $query['total'];
		}
	}
?>
