<?php

	class bo_perp_build_option {

		private $so;

		public function bo_perp_build_option() {
			$this->so = CreateObject('perp_manufacturing.so_perp_build_option');
		}
		
		/*
		*	query:
		*		filter => stock_category
		*		filter2 => option_category
		*/
		public function get_build_option_category_list($query = array(), &$rows = array(), &$readonly = array()) {
			$read = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			$delete = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_DELETE, 'perp_manufacturing');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			$query['option_category'] = $query['option_category'] ? $query['option_category'] : $query['filter'];
			$query['status'] = $query['status'] ? $query['status'] : $query['filter2'];

			if(!$query['order']) {
				$query['order'] = 'build_option_category_name';
			}
			if(!$query['num_rows']) {
				$query['num_rows'] = -1;
			}

			// TODO: make this a little more efficient
			$list = $this->so->get_build_option_list($query);
			if(!isset($query['get_rows'])) {
				foreach($list as $row) {
					$rows[$row['build_option_category_id']] = $row['build_option_category_name'];
				}
				return $rows;
			} else {
				$rows['delete'] = $delete;
				return $query['count'];
			}
		}

		/**
		*	Get a list of build options
		*
		*	The result is an array of build options.  If the query parameter 'by_category' is true,
		*	the options will be nested inside their parent categories.
		*
		*	cat_id	=>	Stock category
		*	filter	=>	Option Category
		*	filter2	=>	Status (Current, past, future)
		*
		*	@nextmatch
		*/
		public function get_build_option_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			$delete = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_DELETE, 'perp_manufacturing');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			$query['option_category'] = $query['option_category'] ? $query['option_category'] : $query['filter'];
			$query['status'] = $query['status'] ? $query['status'] : $query['filter2'];

			if(!$query['order']) {
				$query['order'] = 'build_option_category_name, build_option_name';
			}
			if(!$query['template'] && !$query['num_rows']) {
				$query['num_rows'] = -1;
			}

			$list = $this->so->get_build_option_list($query);

			if(!$query['template']) {
				foreach($list as $row) {
					$rows[$row['build_option_id']] = $row['build_option_name'];
				}
				return $rows;
			}

			if($query['by_category']) {
				$rows = array();
				$category = array();
				foreach($list as $row) {
					if(!$row['build_option_id']) {
						$query['total']--;
						continue;
					}
					$category[$row['build_option_category_id']]['build_option_category_id'] = $row['build_option_category_id'];
					$category[$row['build_option_category_id']]['build_option_category_name'] =  $row['build_option_category_name'];
					$category[$row['build_option_category_id']]['stock_category'] =  $row['stock_category'];
					$category[$row['build_option_category_id']]['options'][] = $row;
					if($row['build_option_id']){
						$readonlys['delete['.$row['build_option_id'].']'] = !($delete && $this->so->build_option_valid_delete($row['build_option_id']));
					}else{
						$readonlys['delete[]'] = true;
					}					
				}
				foreach($category as $cat) {
					$readonlys['delete_cat['.$cat['build_option_category_id'].']'] = !($delete && $this->so->build_option_category_valid_delete($cat['build_option_category_id']));
					$rows[] = $cat;
				}
				//$rows['delete'] = $delete;
				return $query['total'];
			} else {
				$rows = $list;
				return $query['total'];
			}
		}
		
		public function get_build_option_time_period_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			if($read) {
				$count = $this->so->get_build_option_time_period_list($query, $rows, $readonlys);
				return $count;
			}
		}

		public function get_build_option_category($id) {
			$read = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			if($read) {
				return $this->so->get_build_option_category($id);
			}
		}

		public function set_build_option_category($id, $category) {
			if($GLOBALS['egw']->acl->check('build_option', EGW_ACL_EDIT, 'perp_manufacturing')) {

				// Check for a duplicate category name
				if(!$id) {
					$query = array('search' => $category['build_option_category_name']);
					$list = $this->get_build_option_category_list($query);
					foreach($list as $id => $name) {
						if($category['build_option_category_name'] == $name) {
							throw new egw_exception(lang('Duplicate option category name.  '));
						}
					}
				}

				// If stock category changes, we need to delete any options that are set
				$old_category = $this->so->get_build_option_category($id);

				// User changed the stock category? 
				if($old_category && $old_category['stock_category'] != $category['stock_category']) {
					$stock_list = ExecMethod('perp_inventory.bo_perp_stock.get_stock_list', array(
						'cat_id'	=>	$old_category['stock_category']
					));
					$query = array(
						'cat_id'		=>	$old_category['stock_category'],
						'option_category'	=>	$id,
						'template'		=>	true
					);
					$this->get_build_option_list($query,  $options);
					foreach($options as $option) {
						$option['value'] = 0;
						unset($option['component']);
						foreach($stock_list as $stock_id => $stock_name) {
							$this->set_allowed_build_option($stock_id, $option);
						}
					}
					
				}
				$this->so->set_build_option_category($id, $category);
			}
		}

		public function delete_build_option_category($id) {
			$delete = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_DELETE, 'perp_manufacturing');
			if($delete) {
				$this->so->delete_build_option_category($id);
			}
		}
		
		public function delete_build_option($id) {
			$delete = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_DELETE, 'perp_manufacturing');
			if($delete) {
				$this->so->delete_build_option($id);
			}
		}

		public function get_build_option($id) {
			$read = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			if($read) {
				return $this->so->get_build_option($id);
			}
		}

		public function set_build_option($id, $option) {
			if($GLOBALS['egw']->acl->check('build_option', EGW_ACL_EDIT, 'perp_manufacturing')) {
				if(!$option['effective_after_date']) {
					$option['effective_after_date'] = date('U');
				}
				if($option['old_effective_after_date']){
                                	if($option['old_effective_after_date'] != $option['effective_after_date']){
                                        	if($option['effective_after_date'] < time()) {
							return lang('Options cannot come into effect pre-dated, you must choose a date that is not in the past');
						}
                                       	}
                                }			

				if($id) {
					$old_option = $this->get_build_option($id);
					if($option['effective_to_date'] != '' && !$old_option['effective_to_date'] && $option['effective_to_date'] < mktime(0,0,0,date('m'), date('d'), date('Y'))) {
						return lang('Options cannot be retired pre-dated, you must choose a date that is not in the past');
					}
				}
				return $this->so->set_build_option($id, $option);
			}
		}

		public function get_build_option_status_list() {
			return array(
				''	=>	lang('all'),
				'current'	=>	lang('Current'),
				'previous'	=>	lang('previous'),
				'future'	=>	lang('Future'),
			);
		}

		public function get_allowed_option_value_list() {
			return array(
				0	=>	lang('Not allowed'),
				-1	=>	lang('Required'),
				-2	=>	lang('Default'),
				-3	=>	lang('Optional'),
			);
		}
		
		public function get_allowed_build_option_list($query = array()) {
			if($GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing')) {
				if($query['stock_id']) {
					$query['build_option_category'] = $query['filter'];
					return $this->so->get_allowed_build_option_list($query);
				} else {
					return array();
				}
			}
		}

		public function get_allowed_build_option($stock_id, $option, $as_of_date = null) {
			if(is_null($as_of_date)) {
				$as_of_date = time();
			}
			if($GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing')) {
				return $this->so->get_allowed_build_option($stock_id, $option, $as_of_date);
			}
		}

		public function set_allowed_build_option($stock_id, $option) {
			if($GLOBALS['egw']->acl->check('build_option', EGW_ACL_EDIT, 'perp_manufacturing') && $option['build_option_id']) {
				/**
				*	Log changes into stock history log
				*	This is not the best, as it doesn't record which option changed,
				*	just the change itself.
				*/
				if (!$option['effective_from']) {
					$option['effective_from'] = mktime(0,0,0,date('d'), date('m'), date('Y'));
				}
				$old_option = $this->get_allowed_build_option($stock_id, $option['build_option_id'], $option['effective_from']);
				if($old_option) {
					$change_map = array(
						'value'		=>	'Bv',
						'component'	=>	'Bc',
						'work_center'	=>	'Bw',
					);
					$change = array();
					if((int)$option['component'] != (int)$old_option['component']) {
						$change['component'] = true;
					}
					if((int)$option['work_center'] != (int)$old_option['work_center']) {
						$change['work_center'] = true;
					}
					if((int)$option['value'] != (int)$old_option['value']) {
						$change['value'] = true;
					}
					if(count($change) > 0) {
						$history_log =& CreateObject('phpgwapi.historylog', 'perp_inventory');
						foreach($change as $field => $value) {
							$history_log->add($change_map[$field], $stock_id, $option[$field], $old_option[$field]);
						}
					} else {
						// No changes
						$option['effective_from'] = $old_option['effective_from'];
						return true;
					}
				}

				// Check for loops where an option component's BOM requires the stock item
				if($option['component']) {
					$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $option['component']);
					if($stock_item['stock_type_flag'] != 'R') {
						$loop = ExecMethod2('perp_manufacturing.so_perp_bill_of_material.check_for_loops', $stock_id, $option['component']);
						if($loop) {
							$build_option = $this->get_build_option($option['build_option_id']);
							return lang(
								'Loop in options (a part requires itself), check bill of materials #%2 for "%1", it requires %3 so it cannot be the component for %4.',
								ExecMethod('perp_inventory.bo_perp_stock.link_title', $loop['component_id']),
								$loop['bill_of_material_id'],
								ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id),
								$build_option['build_option_name']
							);
						}
					}
				}
				$this->so->set_allowed_build_option($stock_id, $option);
				return true;
			}
		}

		/**
		*	Get a list of options where the option has been discontinued but
		*	we have some items in inventory with that option.
		*/
		public function get_in_stock_options($query) {
			if($GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing')) {
				return $this->so->get_in_stock_options($query);
			} else {
				return array();
			}
		}
		
		/**
		*	
		*	
		*/
		public function get_stock_item_stock_options($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			return $this->so->get_stock_item_stock_options($query, $rows, $readonlys);
			
		}

		public function get_price_list(&$query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			$query['order'] = $query['order'] ? $query['order'] : 'price';
			$query['sort'] = $query['sort'] ?  $query['sort'] : 'DESC';
			$query['status'] = $query['filter2'] ?  $query['filter2'] : $query['status'];

			$list = $this->so->get_price_list($query, $readonly);
			
			if(!isset($query['template'])) {
				
			} else {
				$rows = $list;
				return $query['total'];
			}
		}

		/**
		*	Disables any current allowed build options that use the provided stock_id as the component,
		*	and optionally replaces them with the provided replacement item.
		*/
		public function disable_stock($stock_id, $replacement = null) {
			$used_by = $this->so->get_used_by($stock_id);
			$count = 0;
			foreach($used_by as $option) {
				$option['component'] = $replacement;
				$this->set_allowed_build_option($option['stock_id'], $option);
				$count++;
			}
			return $count;
		}

		/**
		*	Get the build option exclusion matrix.
		*	
		*	This is a list of build options that cannot co-exist.  This is applied on top of the 
		*	normal tree structure to help prevent crazy contortions for interdependencies.  The list
		*	is a two way link (commutative).
		*
		*	@return array of build options that cannot co-exist.  One build option, then a list of others that it excludes.
		*/
		public static function get_exclusion_matrix() {
			return so_perp_build_option::get_exclusion_matrix();
		}

		/**
		*	Set the build option exclusion matrix.
		*	
		*	This is a list of build options that cannot co-exist.  This is applied on top of the 
		*	normal tree structure to help prevent crazy contortions for interdependencies.
		*
		*	@param list Array of build options that cannot be together
		*/
		public static function set_exclusion_matrix(Array $list) {
			if(!$GLOBALS['egw']->acl->check('build_option', EGW_ACL_EDIT, 'perp_manufacturing')) {
				throw new egw_exception_no_permission();
			}
			so_perp_build_option::set_exclusion_matrix($list);
		}
	}
?>
