<?php

	if($GLOBALS['egw_info']['user']['apps']['workflow']) {
		require_once(EGW_INCLUDE_ROOT . '/workflow/inc/class.workflow_instance.inc.php');
	}
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_stock_history.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_center.inc.php');

	class bo_perp_manufacturing {

		private $so;
		private $so_inv;
		private $so_wo;

		private $config;
		private $debug = false;

		public function bo_perp_manufacturing() {
			$this->so = CreateObject('perp_manufacturing.so_perp_manufacturing');
			$this->so_inv = CreateObject('perp_inventory.so_perp_inventory');
			$this->so_wo = CreateObject('perp_manufacturing.so_perp_work_order');
	
			$this->config = config::read('perp_inventory');
		}

		/* Adds to a work order
		*	This will add cost of components to cost of work order and remove components from inventory
		*/
		public function build($work_order_id, $detail_id, $work_center_id, $start_time = '', $end_time = '', $components = array()) {
			if($this->debug) {
				echo "build($work_order_id, $detail_id, $work_center_id, $start_time, $end_time, " . print_r($components, true) . ')<br />';
			}

			$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $work_order_id);
			if($work_order['completed']) {
				return true;
			}

			/* Workflow method */
			if(isset($GLOBALS['egw_info']['apps']['workflow'])) {
				$workflow_instance_id = ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $work_order_id, $detail_id);
			}
			if($workflow_instance_id) {
				$instance = new workflow_instance();
				$instance->loadInstance($workflow_instance_id);
				if($instance && $instance->getEnded()) {
					// Already done
					if($this->debug) {
						echo 'Instance already complete<br />';
					}
					return false;
				}
				if(!$instance || !$workflow_instance_id) {
					if($this->debug) {
						if(!$instance) {
							echo 'Could not find instance<br />';
						}
						if(!$workflow_instance_id) {
							echo 'Could not find workflow_instance_id<br />';
						}
					}
					return false;
				}
				return $this->build_workflow($work_order_id, $work_order, $detail_id, $work_center_id, $start_time, $end_time, $components, $instance);
			} else {
				/* Straight build method */
				return $this->build_straight($work_order_id, $work_order, $detail_id, $work_center_id, $start_time, $end_time, $components);
			}

		}

		/**
		*	Build using the workflow
		*/
		protected function build_workflow($work_order_id, &$work_order, $detail_id, $work_center_id, $start_time, $end_time, $components, $instance) {
			if($this->debug) {
				echo "build_workflow($work_order_id, &$work_order, $detail_id, $work_center_id, $start_time, $end_time, $components, instance)<br />";
			}
			$work_order_detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail', $work_order_id, $detail_id);
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $work_order_detail['stock_id']);
			$serial_number = $this->so_wo->get_serial_number($work_order_id, $stock_id);

			// Check for beginning of build
			if($work_order_detail) {
				$process_id = $instance->getProcessId();
				$activities = $instance->getActivities();
				foreach($activities as $activity) {
					if($this->config['wf_processes']['production_start_activity'][$process_id] == $activity['wf_activity_id'] || $activity['wf_type'] == 'start' ) {
						if($this->debug) {
							echo 'Build started at ' . $activity['wf_name'] . '<br />';
						}
						$this->start_build($work_order, $detail_id, $work_center_id, $start_time);
					}
				}
			}

			if($this->debug && is_array($activities)) {
				foreach($activities as $activity) {
					echo $activity['wf_name'] . ' is now ' . $instance->getActivityStatus($activity['wf_activity_id']) . '<br />';
				}
			}

			if($work_center_id) {
				$cost = $this->add_components($work_order_id, $detail_id, $work_center_id, $end_time, $components);
			}
			$time = $end_time - $start_time;
			//$indirect_cost = $time * $work_center['overhead_per_hour'];
			$this->so->update_cost($work_order_id, $detail_id, $cost, $indirect_cost);

			// Check is build complete?
			if(!$work_order['complete']) {
				$activities = $instance->getActivities();
				foreach($activities as $activity) {
					if($activity['wf_type'] == 'end') {
						// The build has finished
						return $this->finish_build($work_order, $detail_id, $work_center_id, $end_time ? $end_time + 1: null);
					}
				}
			}

			return true;
		}

		/**
		*	Build without the workflow
		*/
		protected function build_straight($work_order_id, &$work_order, $detail_id, $work_center_id, $start_time, $end_time, $components) {
			$work_order_detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail', $work_order_id, $detail_id);
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $work_order_detail['stock_id']);
			$serial_number = $this->so_wo->get_serial_number($work_order_id, $work_order_detail['stock_id']);
			$work_center = ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center', $work_center_id);
			$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $work_order_detail['bill_of_material_id']);

			if($this->debug) {
				echo 'Direct build at ' . $work_center_id ? $work_center['work_center_name'] : 'No work center' . '<br />';
			}

			// Check for beginning of build
			if(!$work_order_detail) {
				throw new egw_exception(lang('Could not find work order detail %2 for work order %1',
					$work_order['our_reference'],
					$detail_id
				));
			}
			if($work_center_id == $bom['start_work_center']) {
				// Build just started
				if($this->debug) {
					echo 'Build started<br />';
				}

				$this->start_build($work_order, $detail_id, $work_center_id, $start_time);
			}

			// Add in components
			$cost = $this->add_components($work_order_id, $detail_id, $work_center_id, $end_time, $components);

			$time = $end_time - $start_time;
			//$indirect_cost = $time * $work_center['overhead_per_hour'];
			$this->so->update_cost($work_order_id, $detail_id, $cost, $indirect_cost);

			// Check for end of build
			if($work_center_id == $bom['finished_work_center']) {
				// The build has finished 	 
				if($this->debug) {
					echo 'Build finished<br />';
				}

				$this->finish_build($work_order, $detail_id, $work_center_id, $end_time ? $end_time + 1: null);
			}
			return true;
		}

		/** 
		*	Add components to the build, pulling them out of the reserved inventory at the work center
		*
		*	@return the cost of the components
		*/
		public function add_components($work_order_id, $detail_id, $work_center_id, $end_time, &$components = null) {
			// Add in components
			if(!$components) {
				$query = array(
					'work_order_id' =>      $work_order_id,
					'detail_id'     =>      $detail_id,
					'work_center_id'=>      $work_center_id,
					'num_rows'	=>	-1
				);
				$bo_wo = CreateObject('perp_manufacturing.bo_perp_work_order');
				$bo_wo->get_work_order_detail_bom($query, $components);
			}
			if($this->debug) {
				echo "add_components($work_order_id, $detail_id, $work_center_id, $end_time, ...)<br />Components:";
				_debug_array($components);
			}
		
			$cost = 0;
			$work_order = $this->so_wo->get_work_order($work_order_id);
			$detail = $this->so_wo->get_work_order_detail($work_order_id, $detail_id);
			$location_list = ExecMethod('perp_inventory.bo_perp_location.get_location_list', array('location_parent' => $work_order['location']));
			$bo_work_center = new bo_perp_work_center();
			$work_center_query = array(
				'template'		=>	false,
				'location_id'		=>	array_keys($location_list),
			);
			foreach($components as &$component) {
				if($component['stock_id']) {
					$component += ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $component['stock_id']);
				}
				if(!$component['pseudo'] && $component['stock_id'] && $component['quantity']) {
					// Need to find out the physical location for the work center
					if(!$work_center_id) {
						$location = $work_order['location'];
					} else {
						$work_center_query['work_center_id'] = $work_center_id;
						$bo_work_center->get_work_center_location_list($work_center_query, $locations);
						$location = $locations[0]['location_id'];
						if(!$location) {
							$location = $work_order['location'];
						}
					}
					if(isset($component['controlled']) && $component['controlled']) {
						// serial_number is an array with serial_number => quantity
						if($component['serial_number']) {
							foreach($component['serial_number'] as $sn => $quantity) {
								ExecMethod2(
									'perp_manufacturing.bo_perp_serial_assembly.serial_assembly',
									$work_order_id, 
									$detail_id, 
									$component['stock_id'], 
									$sn, 
									$quantity,
									$end_time
								);
							}
							$item_cost = $this->so_inv->consume($component['stock_id'], 'used', $location, $component['quantity'], $work_order['work_order_reference'], $component['serial_number'], $detail['stock_id']);
							$cost = bcadd($cost, $item_cost, 3); 
						} else {
							// Error
							//return false;
						}
					} else {
						$item_cost = $this->so_inv->consume($component['stock_id'], 'used', $location, $component['quantity'], $work_order['work_order_reference'], array(), $detail['stock_id']);
						$cost = bcadd($cost, $item_cost, 3);
					}
					if($this->debug) {
						printf("Consumed %1$10s %2$30s  %3$4.3f @ %4$10.3f from %5$3d<br />\n",
							$component['stock_num'],
							$component['short_description'],
							$component['quantity'],
							$item_cost,
							$location
						);
					}
				}
			}
			return $cost;
		}

		/**
		*	Reserve stock for the work order
		*/
		public function reserve_stock($work_order, $detail_id, $components = false) {
			// Move stock from on hand to reserved
			// TODO: ACL
			$wo_detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail', $work_order['work_order_id'], $detail_id);

			if(!$components) {
				$query = array(
					'work_order_id'	=>	$work_order['work_order_id'],
					'detail_id'	=>	$detail_id,
					'num_rows'	=>	-1,
					'order'		=>	'stock_num'
				);
				$components = $this->so_wo->get_work_order_detail_bom($query);
			}

			$work_centers = array();
			$start_wc = null;
			$end_wc = null;

			// Need to find out the physical location for the work center
			$location_list = ExecMethod('perp_inventory.bo_perp_location.get_location_list', array('location_parent' => $work_order['location']));

			$bo_work_center = new bo_perp_work_center();
			$work_center_query = array(
				'template'		=>	false,
				'location_id'		=>	array_keys($location_list),
			);

			foreach($components as $detail) {
				if(!$detail['work_center_id']) {
					$location = $work_order['location'];
				} else {
					$work_center_query['work_center_id'] = $detail['work_center_id'];
					$bo_work_center->get_work_center_location_list($work_center_query, $locations);
					$location = $locations[0]['location_id'];
				}
				if($location && $detail['quantity'] != 0) {
					if($this->debug) {
						printf("<pre>Reserving %8s %-20.20s : %5.2f @ $location</pre><br />", $detail['stock_num'], $detail['short_description'], $detail['quantity'], $location);
					}
					$this->so_inv->reserve_stock($location, $detail['component'], $detail['quantity'], $work_order['work_order_reference']);
				} elseif ($this->debug) {
					printf("<b><pre>Not reserving %8s %-20.20s : %5.2f @ $location - no quantity or no location</pre></b><br />", $detail['stock_num'], $detail['short_description'], $detail['quantity']);
				}
			}
		}

		/** Reserve what it takes to build from the work order
		*  This will take care of:
		*	moving stock from on hand to reserved
		*  * Does not recurse *
		*/
		public function schedule_build($work_order, $detail_id) {
			$this->reserve_stock($work_order, $detail_id);

			//$work_centers = $this->so->order_by_production($start_wc, $end_wc, $work_centers);
			if($this->config['avg_hours']) {
				$this->schedule_avg_hours($wo_detail['stock_id']);
			} elseif($this->config['scheduling'] == 'work_center') {
				$start_time = $work_order['start_date'];
				$next = null;
				foreach($work_centers as $work_center) {
					$entry = $this->schedule_work_center($work_center['work_center_id'], $work_order, $bom_details, $start_time);
					$start_time = $entry['start'];
				}
			}
		}

		/* Start building an assembly from a work order
		* Should be called when an assembly hits its start_work_center
		* This will take care of:
		*	Accounting - moving value to work in progress
		*/
		public function start_build(&$work_order, $detail_id, $work_center_id, &$start_time = '') {
			if(!$start_time) {
				$start_time = time();
			}
			$work_order['start_date'] = $start_time;
			$this->so_wo->set_work_order($work_order['work_order_id'], $work_order);

			$start_time++;
		}

		/* Finish building an assembly from a work order
		* Should be called when an assembly hits its finished_work_center
		* This will take care of:
		*	Accounting - moving value to finished goods
		*	Stock adjustment - Adds to stock count
		*	Record end of manufacturing in history
		*	Record used serial numbers in history
		*/
		public function finish_build($work_order, $detail_id, $work_center_id, $end_time = '') {
			if(!$end_time) {
				$end_time = time() + 2;
			}
			$detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail', $work_order['work_order_id'], $detail_id);
			if(!$detail) {
				return false;
			}

			/**
			*	Some components don't have a work center associated with them because they're used in 
			*	multiple places.  Include them here, and take the stock from the work order's final 
			*	location.
			*/
			$query = array(
				'work_order_id'	=>	$work_order['work_order_id'],
				'detail_id'	=>	$detail_id,
				'num_rows'	=>	-1,
				'work_center_id'=>	0
			);
			$bo_wo = CreateObject('perp_manufacturing.bo_perp_work_order');
			$bo_wo->get_work_order_detail_bom($query, $components);
			if(count($components) > 0) {
				if($this->debug) {
					echo 'Adding components with no work center<br />';
				}
				$cost = $this->add_components($work_order['work_order_id'], $detail_id, false, $end_time, $components);
				$this->so->update_cost($work_order['work_order_id'], $detail_id, $cost, $indirect_cost);
				$detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail', $work_order['work_order_id'], $detail_id);
			}

			$reference = lang('%1 created by Work Order %2',
				ExecMethod2('perp_inventory.bo_perp_stock.get_next_serial', $detail['stock_id'], $work_order['work_order_id'], $detail['detail_id']),
				$work_order['work_order_reference']
			);
			if($this->debug) {
				echo 'Creating new stock ' . $reference . '<br />';
			}
			$stock_cost_id = ExecMethod2('perp_inventory.bo_perp_inventory.create', $detail['stock_id'], $detail['quantity'], $work_order['location'], $detail['direct_cost'] + $detail['indirect_cost'], $reference);

			// Make sure we have a serial number
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $detail['stock_id']);
			if($stock['controlled']) {
				if($this->debug) {
					echo 'Making serial number<br />';
				}
				$new_serial = ExecMethod2('perp_inventory.bo_perp_stock.make_serial', $detail['stock_id'], $work_order, $detail['detail_id'], $detail['quantity'], $work_order['location'], $stock_cost_id);
				if($this->debug) {
					echo "New serial number: $new_serial<br />";
				}
			}
			
			// TODO: Don't assume 1 detail per work order
			$work_order['completed_date'] = date('U');
			$work_order['completed'] = True;
			
			if($this->debug) {
				echo 'Completing work order<br />';
			}

			// Record end of manufacturing in history
			if($new_serial) {
				bo_perp_stock_history::add_history(
					$detail['stock_id'], $new_serial, 
					lang('Build Complete'), 
					array('app' => 'perp_manufacturing', 'id' => 'WORK_ORDER-' . $work_order['work_order_id'] . '-' . $detail_id),
					__LINE__, __FILE__
				);

				// Record use of serialized components in history
				$query = array(
					'work_order_id'	=>	$work_order['work_order_id'],
					'detail_id'	=>	$detail_id,
				);
				bo_perp_serial_assembly::get_serial_assemblies($query, $serial_components);
				foreach($serial_components as $component) {
					bo_perp_stock_history::add_history(
						$detail['stock_id'], $new_serial, 
						lang('Serialized component %1 added (s/n %2)', 
							ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['component_stock_id']), 
							$component['component_serial_number']
						),
						array('app' => 'perp_manufacturing', 'id' => 'WORK_ORDER-' . $work_order['work_order_id'] . '-' . $detail_id),
						__LINE__, __FILE__
					);
				}
			}

			$this->so_wo->set_work_order($work_order['work_order_id'], $work_order);
			return true;
		}

		/* Stop building an assembly from the work order
		*  This will take care of moving stock from reserved back to on hand
		*  * Does not recurse *
		*/
		public function cancel_build($work_order, $detail_id) {
			// Move stock from reserved to on hand
			// TODO: ACL
			$query = array(
				'work_order_id'	=>	$work_order['work_order_id'],
				'detail_id'	=>	$detail_id,
				'order'		=>	'stock_num',
				'num_rows'	=>	-1
			);
			$bom = $this->so_wo->get_work_order_detail_bom($query);
			if($bom) {
				// Need to find out the physical location for the work center
				$location_list = ExecMethod('perp_inventory.bo_perp_location.get_location_list', array('location_parent' => $work_order['location']));
				$work_center_query = array(
					'template'		=>	false,
					'location_id'		=>	array_keys($location_list),
				);
				$bo_work_center = new bo_perp_work_center();

				foreach($bom as $detail) {
					if(!$detail['work_center_id']) {
						$location = $work_order['location'];
					} else {
						$work_center_query['work_center_id'] = $detail['work_center_id'];
						$bo_work_center->get_work_center_location_list($work_center_query, $locations);
						$location = $locations[0]['location_id'];
					}
					if($location && $detail['quantity'] != 0) {
						if($this->debug) {
							printf("<pre>Reserving %8s %-20.20s : %5.2f @ $location</pre><br />", 
								$detail['stock_num'], 
								$detail['short_description'], 
								-$detail['quantity'], 
								$location
							);
						}
						$this->so_inv->reserve_stock($location, $detail['component'], -$detail['quantity'], $work_order['work_order_reference']);
					}
				}
			}
		}

		/* Schedule the work center to build the work order
		*	returns calender_id
		*/
		private function schedule_work_center($work_center_id, $work_order, $bom_list, $start_time) {
			$work_center = ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center', $work_center_id);
			$duration = 0; // in minutes
			foreach($bom_list as $bom_info) {
				$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $bom_info['component']);
				if(!$bom || !$bom['details']) {
					continue;
				}
			
				foreach($bom['details'] as $detail) {
					if($detail['work_center'] == $work_center_id) {
						$duration += $detail['time'];
					}
				}
			}
			$start = strtotime($start_time);
			$event = array(
				'start'		=>	$start,
				'end'		=>	$start + ($duration * 60),
				'title'		=>	lang('Work Order %1', $work_order['work_order_reference']),
				'owner'		=>	abs($this->config['production_group']),
				'participants'	=>	array($work_center['account'] => 'A'),
			);
			
			if($work_center['account']) {
				$result = ExecMethod2('calendar.calendar_boupdate.update', $event);
			}
			return $event;
		}

		/**
		*	Get the settings for creating serial numbers
		*
		*	@param setup_id ID of setting to get data for
		*	@param all_data If true, returns all data, if false, returns array for selectbox
		*/
		public function get_serial_setup($setup_id = false, $all_data = false) {
			$serial = $this->so->get_serial_setup($setup_id, $all_data);
			if(!$all_data) {
				$select = array();
				foreach($serial as $key => $data) {
					$select[$data['setup_id']] = $data['name'];
				}
				return $select;
			} else {
				return $serial;
			}
		}

		/**
		*	Set the settings for creating serial numbers
		*
		*	You can't set the next ID below what it currently is
		*
		*	@param setup_id ID of the setting
		*	@param data Array of data to set
		*/
		public function set_serial_setup($setup_id, $data) {
			return $this->so->set_serial_setup($setup_id, $data);
		}

		/**
		*	Participate in linking
		*/
		public static function link_title($entry) {
			list($location, $id) = explode('-', $entry);
			switch ($location) {
				case 'RETROFIT':
					return bo_perp_retrofit::link_title($id);
					break;
				case 'WORK_ORDER':
					return ExecMethod('perp_manufacturing.bo_perp_work_order.link_title', $id);
					break;
			}
		}
	}
?>
