<?php

	/**
	*	Logic layer for changing the options and components on a work order after the work order 
	*	has been completed.  This is called a retrofit.  It is important to be able to do this
	*	for warranty purposes.
	*
	*	@package perp_manufacturing
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.so_perp_retrofit.inc.php');

	class bo_perp_retrofit {

		public function __construct() {
		}

		/**
		*	Get a list of retrofits, and information about them
		*
		*	@nextmatch
		*
		*	Extra query parameters:
		*	- stock_id
		*	- work_order_id
		*	- detail_id
		*	- account_id - user who entered the retrofit
		*	- date
		*/
		public static function get_list(&$query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('retrofit', EGW_ACL_READ, 'perp_manufacturing')) {
				$rows = array();
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'stock_history_id';
			}

			$rows = so_perp_retrofit::get_list($query);

			return $query['total'];
		}

		/**
		*	Read all information related to one retrofit
		*
		*	@param id ID number of the retrofit
		*
		*	@return array of retrofit information
		*/
		public static function get_retrofit($id) {
			if(!$GLOBALS['egw']->acl->check('retrofit', EGW_ACL_READ, 'perp_manufacturing')) {
				return array();
			}
			return so_perp_retrofit::get_retrofit($id);
		}

		/**
		*	Set information for a retrofit, or create a new
		*	record if the stock_history_id is not set.
		*
		*	@param retrofit Array of retrofit information
		*
		*	@return stock_history_id
		*/
		public static function set_retrofit(Array $retrofit) {
			if(!($GLOBALS['egw']->acl->check('retrofit', EGW_ACL_ADD, 'perp_manufacturing') ||
					$GLOBALS['egw']->acl->check('retrofit', EGW_ACL_EDIT, 'perp_manufacturing'))) {
				// No permission
				return lang('No permission');
			}
			if(!$retrofit['timestamp']) {
				$retrofit['timestamp'] = time();
			}
			if(!$retrofit['account_id']) {
				$retrofit['account_id'] = $GLOBALS['egw_info']['user']['account_id'];
			}
			if(!$retrofit['build_option_id']) {
				return lang('Missing build option');
			}
			$build_option = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option', $retrofit['build_option_id']);
			if(!$retrofit['build_option_category_id']) {
				$retrofit['build_option_category_id'] = $build_option['build_option_category'];
			}
			if(!$retrofit['comment']) {
				$build_option_category = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option_category', $retrofit['build_option_category_id']);
				$retrofit['comment'] = lang('Retrofit %1: %2', 
					$build_option_category['build_option_category_name'],
					$build_option['build_option_name']
				);
			}
			$history_id = so_perp_retrofit::set_retrofit($retrofit);
			if(is_numeric($history_id) && is_array($retrofit['link'])) {
				$link_id = 'RETROFIT-' . $history_id;
				egw_link::link('perp_manufacturing',$link_id, $retrofit['link']['to_id'], '', $retrofit['remark']); 
			}
			return $history_id;
		}

		/** 
		*	Participate in linking
		*/
		public static function link_title($entry) {
			$retrofit = self::get_retrofit($entry);
			return $retrofit['comment'] . ' (' . $retrofit['reference'] . ')';
		}
	}
?>
