<?php

	class bo_perp_serial_assembly {
		
		public function __construct() {
		}

		/**
		*	Record one serialized item being put into another serialized item 
		*
		*	@todo ACL
		*	@todo error checking
		*
		*	@param work_order_id
		*	@param detail_id
		*	@param c_stock_id The stock ID of the component being added
		*	@param c_serial The serial number of the component being added
		*	@param quantity The quantity of the controlled component used.  This is 1 for serialized goods.
		*	@param timestamp
		*	@param work_center_id
		*/
		public static function serial_assembly($work_order_id, $detail_id, $c_stock_id, $c_serial, $quantity, $timestamp) {
			if(!$timestamp) {
				$timestamp = date('U');
			}

			// TODO: More error checking
			so_perp_serial_assembly::serial_assembly($work_order_id, $detail_id, $c_stock_id, $c_serial, $quantity, $timestamp);
		}

		/**
		*	Get a list of serial assemblies
		*
		*/
		public static function get_serial_assemblies($query, &$rows = array()) {
			$rows = so_perp_serial_assembly::get_serial_assemblies($query);
			return $query['count'];
		}

		/**
		*	Change a recorded serial number as created by serial_assembly()
		*
		*	This is here in the case of typos.
		*/
		public static function change_serial_assembly($stock_history_id, $work_order_id, $detail_id, $new_serial_number) {
			if(!$GLOBALS['egw']->acl->check('production', EGW_ACL_EDIT, 'perp_manufacturing')) {
				throw new egw_exception_no_permission();
			}
			if(!$stock_history_id) {
				throw new egw_exception_wrong_parameter('Missing stock_history_id');
			}
			if(!$work_order_id) {
				throw new egw_exception_wrong_parameter('Missing work_order_id');
			}
			if(!$detail_id) {
				throw new egw_exception_wrong_parameter('Missing detail_id');
			}
			if(!$new_serial_number) {
				throw new egw_exception_wrong_parameter('Missing new_serial_number');
			}

			// Check for a valid serial number
			$query = array(
				'stock_history_id'	=>	$stock_history_id
			);
			$old_serial = so_perp_serial_assembly::get_serial_assemblies($query);
			if(!$old_serial) {
				throw new egw_exception('Could not find old record');
			}
			$stock_id = $old_serial[0]['component_stock_id'];
			
			$valid = bo_perp_stock::check_serial($stock_id, $new_serial_number);

			switch ($valid) {
				case bo_perp_stock::NOT_VALID:
					throw new egw_exception_wrong_parameter(lang(
						'%1 is not a valid number for %2', 
						$new_serial_number, 
						ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id))
					);
				case bo_perp_stock::FORMAT_OK:
					// Create record of this serial number
					$query = array(
						'stock_id'	=>	$stock_id,
						'serial_number'	=>	$old_serial[0]['component_serial_number']
					);
					bo_perp_inventory::get_controlled_inventory_list($query, $list);
					if(strtoupper($list[0]['serial_number']) != strtoupper($old_serial[0]['component_serial_number'])) {
						throw new egw_exception('Could not find old record');
					}
					bo_perp_inventory::add_serial($stock_id, $new_serial_number, $old_serial[0]['quantity'], $list[0]['location_id']);

					// FALL THROUGH
				case bo_perp_stock::SERIAL_OK:
					so_perp_serial_assembly::change_serial_assembly($stock_history_id, $work_order_id, $detail_id, $new_serial_number, $old_serial[0]);
					return true;
					break;
			}

			return false;
		}
	}
?>
