<?php

	class bo_perp_work_center {

		private $so;
		
		public function bo_perp_work_center() {
			$this->so = CreateObject('perp_manufacturing.so_perp_work_center');
		}

		public function get_work_center_list($query = array(), &$rows = array(), &$readonly = array()){
			$read = $GLOBALS['egw']->acl->check('work_center', EGW_ACL_READ, 'perp_manufacturing');
			if(!$read) {
				if(!$query['template']) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			if(!$query['order']) {
				$query['order'] = 'work_center_id';
			}
			if(!$query['template']) {
				// This is for a selectbox, so make sure all results are returned
				$query['num_rows'] = -1;
			}

			$rows = $this->so->get_work_centers($query);
			if(!$query['template']) {
				$list = array();
				foreach($rows as $wc) {
					$list[$wc['work_center_id']] = $wc['name'];
				}
				return $list;
			}

			return $query['count'];
		}

		public function get_work_center($work_center_id) {
			$read = $GLOBALS['egw']->acl->check('work_center', EGW_ACL_READ, 'perp_manufacturing');
			if(!$read) {
				return array();
			}
			if(!$work_center_id) {
				return array();
			}
			$query = array(
				'work_center_id'=>	$work_center_id,
				'order'		=>	'work_center_id'
			);
			$list = $this->so->get_work_centers($query);
			if(count($list)) {
				$work_center = $list[0];
			} else {
				return array();
			}
			return $work_center;
		}

		public function set_work_center($work_center_id, $work_center) {
			$edit = $GLOBALS['egw']->acl->check('work_center', EGW_ACL_EDIT, 'perp_manufacturing');
			if(!$edit) {
				return lang('Unable to save work center.') . ' ' . lang('Permission denied');
			} elseif (count($work_center) == 0) {
				return lang('Missing required information');
			}
			
			$id = $this->so->set_work_center($work_center_id, $work_center);

			return $id;
		}

		public function delete_work_center($work_center_id) {
			$delete = $GLOBALS['egw']->acl->check('work_center', EGW_ACL_DELETE, 'perp_manufacturing');
			if($delete) {
				return $this->so->delete_work_center($work_center_id);
			} else {
				return lang('Could not delete') . ' ' . lang('Permission denied');
			}
		}

		public function get_history($query = array(), &$rows = array(), &$readonlys = array()) {
			// TODO: ACLs
			$list = $this->so->get_history($query['work_center_id'], $query['filter'], $query['search']);
			
			$rows = array_slice($list, $query['start'], $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs']);
			return count($list);
		}

		/**
		*	Get a list of physical inventory locations that are work centers
		*/
		public function get_work_center_location_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('work_center', EGW_ACL_READ, 'perp_manufacturing') && 
				!$GLOBALS['egw']->acl->check('location', EGW_ACL_READ, 'perp_inventory')) {

				// No read permission
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['order']) {
				$query['order'] = 'location_id';
			}
			if(!isset($query['template'])) {
				$query['num_rows'] = -1;
			}
			$rows = $this->so->get_work_center_location_list($query);

			// For a selectbox
			if(!isset($query['template'])) {
				$list = array();
				foreach($rows as $row) {
					$list[$row['work_center_id']] = $row['location_name'];
				}
				return $list;
			}
			return $query['total'];
		}
		
		/**
		*	Get the information about work centers for a given physical 
		*	inventory location
		*
		*	@param location_id ID of the location
		*	@return array of information
		*/
		public function get_work_center_location($location_id) {
			$query = array(
				'location_id'	=>	$location_id,
				'template'	=>	false,
			);
			$this->get_work_center_location_list($query, $list);
			return $list[0];
		}

		/**
		*	Set a certain physical inventory location to be a certain type of work center.
		*
		*	This is mostly a convenience function.
		*
		*	@param work_center_id
		*	@param location_id
		*/
		public function add_work_center_location($work_center_id, $location_id) {
			if(!$GLOBALS['egw']->acl->check('work_center', EGW_ACL_EDIT, 'perp_manufacturing')) {
				throw new egw_exception_no_permission();
			}

			// Get work center for defaults
			$work_center = $this->get_work_center($work_center_id);
			$work_center_location = array(
				'work_center_id'	=>	$work_center_id,
				'location_id'		=>	$location_id,
				'overhead_per_hour'	=>	$work_center['default_overhead_per_hour'],
				'overhead_account'	=>	$work_center['default_overhead_account'],
				'user_account'		=>	$work_center['default_user_account']
			);
			return $this->set_work_center_location($work_center_location);
		}

		/**
		*	Un-set a certain physical inventory location as a work center
		*
		*	This is mostly a convenience function.
		*
		*	@param location_id
		*/
		public function remove_work_center_location($work_center_id, $location_id) {
			if($GLOBALS['egw']->acl->check('location', EGW_ACL_EDIT, 'perp_inventory')) {
				return $this->so->remove_work_center_location($work_center_id, $location_id);
			}
		}

		/**
		*	Set the location specific information for a physical work center
		*
		*	@param work_center_location Array of info
		*/
		public function set_work_center_location($work_center_location) {
			if(!$work_center_location['work_center_id']) {
				return lang('Missing work center');
			} elseif (!$work_center_location['location_id']) {
				return lang('Missing location');
			} elseif (!$GLOBALS['egw']->acl->check('work_center', EGW_ACL_EDIT, 'perp_manufacturing') &&
				!$GLOBALS['egw']->acl->check('location', EGW_ACL_EDIT, 'perp_inventory')) {
				throw new egw_exception_no_permission();
			}
			
			return $this->so->set_work_center_location($work_center_location);
		}

		/**
		*	Get a list of stock items that are used at a work center
		*
		*	@nextmatch
		*	- work_center_id
		*/
		public static function get_work_center_stock_list(&$query = array(), &$rows = array(), &$readonly = array()) {

			if(!$GLOBALS['egw']->acl->check('stock', EGW_ACL_READ, 'perp_inventory')) {
				$rows = array();
				return 0;
			}

			if(!$query['order']) {
				$query['order'] = 'stock_id';
			}

			$stock_ids = so_perp_work_center::get_work_center_stock_list($query);
			$query['stock_id'] = $stock_ids;
			$bo_stock = new bo_perp_stock();
			$count = $bo_stock->get_stock_list($query, $rows, $readonly);

			return $count;
		}
	}
?>
