<?php
	require_once( EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_manufacturing.inc.php');

	class bo_perp_work_order {

		private $so;
		private $config;

		private $debug = false;

		/**
		*	Cache for exclusion matrix, it gets referenced several times
		*/
		private static $exclusion_matrix = null;

		/**
		*	Work orders with these statuses can be deleted
		*/
		public static $delete_statuses = array(
			'temporary',
			'pending',
			'scheduled',
			'pre-production',
			'aborted'
		);

		/**
		*	Work orders with these statuses can be built
		*/
		public static $build_statuses = array(
			'pending',
			'scheduled',
		);

		/**
		*	Work orders with these statuses will not have build option changes recorded
		*/
		public static $no_history_statuses = array(
			'pending'
		);

		public function bo_perp_work_order() {
			$this->so = CreateObject('perp_manufacturing.so_perp_work_order');
			$this->config = config::read('perp_manufacturing');
		}

		/** 
		*	Get a list of work orders
		*	
		*	Follows standard nextmatch format
		*	$query['filter']	=>	Status or work center
		*	$query['status']	=>	Status or work center
		*	$query['filter2']	=>	Physical location / plant
		*	$query['search']	=>	Work order reference
		*	$query['client_id']	=>	Only work orders sold to a certain client
		*
		*	@return array/int	Returns the total number of results, or an array 
		*				suitable for a selectbox, depending on if 
		*				$query['template'] is set or not.
		*/
		public function get_work_order_list($query = array(), &$rows = array(), &$readonly = array()){
			$read = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');
			if(!$read) {
				if(!isset($query['template'])) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}
			$query['status'] = $query['status'] ? $query['status'] : $query['filter'];
			if($query['col_filter']['stock_id']) {
				$query['stock_id'] = (int)$query['col_filter']['stock_id'];
			}

			$list = $this->so->get_work_order_list($query);

			if(!isset($query['template'])) {
				foreach($list as $row) {
					$rows[$row['work_order_id']] = $row['work_order_reference'];
				}
				return $rows;
			}

			$rows = array();
			$config = config::read('perp_manufacturing');
			if($config['use_workflow']) {
				$instance = new Instance($GLOBALS['egw']->db->Link_ID);
			}

			foreach($list as $key => $row) {
				$row['status'] = $this->get_work_order_status($row['work_order_id']);

				if($query['include_details']) {
					$row_query = array(
						'work_order_id'	=>	$row['work_order_id'],
						'template'	=>	false
					);

					$this->get_work_order_detail_list($row_query, $row['details']);
				}

				$rows[] = $row;

				$readonly['move_to[' . $row['work_order_id'] . ']'] = true;

				if($row['status']) {
					$readonly['delete[' . $row['work_order_id'] . ']'] = !$GLOBALS['egw']->acl->check('work_order', EGW_ACL_DELETE, 'perp_manufacturing') || $row['completed'];
					if(!in_array($row['status'], bo_perp_work_order::$delete_statuses)) {
						$readonly['delete[' . $row['work_order_id'] . ']'] = true;
					} 
					$readonly['build[' . $row['work_order_id'] . ']'] = true;
					if(!$row['location']) {
						// No end location set, can't build;
						continue;
					}
					if(in_array($row['status'], bo_perp_work_order::$build_statuses) || !$row['completed']) {
						// Status is OK, check to see if it is supposed to use workflow:
						$details = array();
						if(!$GLOBALS['egw']->acl->check('production', EGW_ACL_EDIT, 'perp_manufacturing')) {
							continue;
						}
						$readonly['build[' . $row['work_order_id'] . ']'] = false;
						if($row['details'] ) {
							$details =& $row['details'];
						} else {
							$this->get_work_order_detail_list(array('work_order_id' => $row['work_order_id']), $details);
						}

						if(isset($GLOBALS['egw_info']['apps']['workflow']) && $config['use_workflow']) {
							foreach($details as $detail) {
								$instance_id = ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $row['work_order_id'], $detail['detail_id']);
								if($instance_id = ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $row['work_order_id'], $detail['detail_id'])) {
									$instance->loadInstance($instance_id);
									$too_late_activity = $config['wf_processes']['production_start_activity'][$instance->getProcessId()];
									$readonly['delete[' . $row['work_order_id'] . ']'] = false;

									foreach($instance->workitems as $workitem) {
										if($workitem['wf_activity_id'] == $too_late_activity) {
											$readonly['delete[' . $row['work_order_id'] . ']'] = true;
											break;
										}
									}

									// Got a workflow ID, use workflow to build it
									$readonly['build[' . $row['work_order_id'] . ']'] = true;

									// Check simple things to see if user can move production around
									$readonly['move_to[' . $row['work_order_id'] . ']'] = $row['completed'] || 
										!$GLOBALS['egw']->acl->check('production', EGW_ACL_EDIT, 'perp_manufacturing') ||
										!$GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'workflow');

									if(!$readonly['move_to[' . $row['work_order_id'] . ']']) {
										// Check workflow permissions
										$workflow_permissions = ExecMethod2(
											'workflow.workflow_gui.gui_list_user_instances', 
											$GLOBALS['egw_info']['user']['account_id'], 
											0, -1, '', '', 
											"gi.wf_instance_id = $instance_id"
										);
										$readonly['move_to[' . $row['work_order_id'] . ']'] = ($workflow_permissions['cant'] == 0) &&
											($instance->getOwner() != $GLOBALS['egw_info']['user']['account_id'] || $instance->getOwner() != 0) &&
											($instance->getNextUser() != '' && $instance->getNextUser() != $GLOBALS['egw_info']['user']['account_id']) ;
									}
									continue;
								}
							}
						}
					}
				}
			}
			return $query['count'];
		}

		public function get_work_order($work_order_id) {
			$read = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');
			if($read) {
				return $this->so->get_work_order($work_order_id);
			}
		}

		public function set_work_order($work_order_id, $work_order) {
			$edit = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_EDIT, 'perp_manufacturing');
			// TODO: validation
			// $work_order_id < 0 is a temp work order for sales orders
			if($edit || $work_order_id < 0) {
				if(!$work_order['created_date']) {
					$work_order['created_date'] = date('U');
				}

				if(!$work_order['details']) {
					$this->get_work_order_detail_list(array('template' => false, 'work_order_id' => $work_order_id), $work_order['details']);
				}

				if(isset($work_order['details'])) {
					$serialized = False;
					foreach($work_order['details'] as $detail) {
						$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $detail['stock_id']);
						$serialized |= ($stock['serialized']);
					}
				} else {
					return lang('Empty work order');
				}

				$work_order['completed'] = ($work_order['completed_date']);

				if($this->config['one_serialized_per']) {
					if($serialized && count($work_order['details']) > 1) {
						return lang('Only one serialized item per work order');
					}
				}
				if(!$work_order['location'] && $this->config['default_location']) {
					$work_order['location'] = $this->config['default_location'];
				}

				$un_reserve_stock = false;
				if($work_order_id) {
					$old_work_order = $this->get_work_order($work_order_id);
					$un_reserve_stock = ($old_work_order['location']);
				}

				$work_order_id = $this->so->set_work_order($work_order_id, $work_order);
				$count = 0;

				foreach($work_order['details'] as $detail) {
					$this->set_work_order_detail($work_order_id, $detail['detail_id'] ? $detail['detail_id'] : ++$count, $detail, $un_reserve_stock);

				}
				return $work_order_id;
			}
		}

		/**
		*	Delete a work order
		*
		*	A work order may be deleted, if its status allows.
		*
		*	@todo Partially complete work orders that get aborted
		*/
		public function delete_work_order($work_order_id) {
			$delete = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_DELETE, 'perp_manufacturing');

			// $work_order_id < 0 is a temp work order for sales orders
			if(!$delete && $work_order_id >= 0) {
				return;
			}
			$status = $this->get_work_order_status($work_order_id);
			if($status && !in_array($status, bo_perp_work_order::$delete_statuses)) {
				return lang("You're not allowed to delete this work order anymore, it's status is %1", $status);
			}
			if($work_order_id >= 0) {
				$work_order = $this->get_work_order($work_order_id);
				$this->get_work_order_detail_list(array('work_order_id' => $work_order_id), $details);
				$instance = CreateObject('workflow.workflow_instance');
				foreach($details as $detail) {
					// Remove workflow
					$instance->loadInstance(ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $work_order_id, $detail['detail_id']));
					$instance->abort();

					// Deal with accumulated cost
					if($detail['direct_cost'] + $detail['indirect_cost'] != 0) {
						// TODO
					}
				}
			}

			return $this->so->delete_work_order($work_order_id);
		}

		/**
		*	Take a work order through its build process directly and immediately.
		*
		*	This is mostly intended for companies that do not have any sort of production
		*	monitoring, just build it.  This will fail if the work order is supposed to use
		*	the workflow (as indicated by an associated workflow instance).
		*/
		public function build_work_order($work_order_id, $detail_id = null) {
			if(!$GLOBALS['egw']->acl->check('production', EGW_ACL_EDIT, 'perp_manufacturing')) {
				return lang('Access is not allowed');
			}
			if(is_null($detail_id)) {
				$details = array();
				$this->get_work_order_detail_list(array('work_order_id' => $work_order_id), $details);
				$result = true;
				$message = '';
				foreach($details as $detail_id => $stock_name) {
					$test = $this->build_work_order($work_order_id, $detail_id);
					if($test !== true) {
						$message .= "\n".$test;
						$result = false;
					}
				}
				return $message;
			}
			if(isset($GLOBALS['egw_info']['apps']['workflow'])) {
				if(ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $work_order_id, $detail_id)) {
					return lang('This work order is supposed to use the workflow');
				}
			}

			$bo_manufacturing = new bo_perp_manufacturing();
			$work_order_detail = $this->get_work_order_detail($work_order_id, $detail_id);

			// Step through the bill of materials, and build this work order [detail]
			$work_centers = ExecMethod2('perp_manufacturing.bo_perp_work_center.get_work_center_list');

			// BOMs for workflow don't need start / end work centers, it's defined by the workflow, so we'll fake it
			$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $work_order_detail['bill_of_material_id']);
			if(!$bom['start_work_center'] || !$bom['finished_work_center']) {
				// Set start work center to the first work center we found, set end to the last
				$ids = array_keys($work_centers);
				$bom['start_work_center'] = $bom['start_work_center'] ? $bom['start_work_center'] : current($ids);
				$bom['finished_work_center'] = $bom['finished_work_center'] ? $bom['finished_work_center'] : end($ids);
				ExecMethod2('perp_manufacturing.so_perp_bill_of_material.set_bill_of_material', $work_order_detail['bill_of_material_id'], $bom);
			}

			foreach($work_centers as $work_center_id => $work_center_name) {
				$success = $bo_manufacturing->build($work_order_id, $detail_id, $work_center_id, $start, time(), $components);
				$start = time();
			}
			
			return $success;
		}

		public function get_work_order_status_list() {
			$status_list = array(
				'temporary'	=>	lang('temporary'),
				'pending'	=>	lang('pending'),
				'scheduled'	=>	lang('scheduled'),
			);
			$config = config::read('perp_manufacturing');
			if($config['use_workflow']) {
				$status_list['pre-production'] = lang('pre-production');
			}
			$status_list += array(
				'in-production'	=>	lang('in production'),
				'completed'	=>	lang('finished'),
			);
			return $status_list;
		}

		/**
		*	Get status of a work order from workflow
		*/
		public function get_work_order_status($work_order_id, $detail_id = null) {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				return lang('access is not allowed');
			}

			if($work_order_id <= 0) {
				return 'temporary';
			}
			if(is_null($detail_id)) {
				$detail_list = $this->get_work_order_detail_list(array('work_order_id' => $work_order_id));
			} else {
				$detail_list = array($detail_id => null);
			}

			if(isset($GLOBALS['egw_info']['apps']['workflow'])) {
				$instance =& CreateObject('workflow.workflow_instance');
				foreach($detail_list as $detail_id => $stock_name) {
					$instance_id = ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $work_order_id, $detail_id);
					if($instance_id) {
						$instance->loadInstance($instance_id);
						$activities =& $instance->getActivities();
						foreach($activities as $activity) {
							$status .= ' ' . $activity['wf_name'];
						}
						if(!$status) {
							$status = lang($instance->getStatus());
						}
					}
				}
			}
			if(!$status) {
				// Non-workflow
				$work_order = $this->get_work_order($work_order_id);

				if(!$work_order['start_date']) {
					$status = 'pending';
				} elseif ($work_order['start_date'] > time()) {
					$status = 'scheduled';
				} elseif ($work_order['start_date'] <= time() && !$work_order['required_date']) {
					$status = 'in production';
				} elseif ($work_order['completed_date']) {
					$status = 'completed';
				}
			}
			return $status;
		}

		public function get_work_order_detail_list($query = array(), &$rows = array(), &$readonly = array()) {
			$read = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');
			if(!$read) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			$list = $this->so->get_work_order_detail_list($query);
			if(!isset($query['template'])) {
				$rows = array();
				foreach($list as $row) {
					$rows[$row['detail_id']] = $row['detail_id'] . ' ' . ExecMethod('perp_inventory.bo_perp_stock.link_title', $row['stock_id']);
				}
				return $rows;
			}
			$rows = $list;

			$serial_query = array(
				'work_order_id'	=>	$query['work_order_id'],
			);

			foreach($rows as $key => &$row) {
				$readonly['delete['. $row['detail_id'] . ']'] = ($row['direct_cost'] + $row['indirect_cost'] != 0);
				$readonly['option_history['. $row['detail_id'] . ']'] = count(so_perp_work_order::get_option_history_list($query['work_order_id'], $row['detail_id'])) == 0;

				$row['instance_id'] = ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $query['work_order_id'], $row['detail_id']);
				$readonly['build_history['. $row['detail_id'] . ']'] = !($row['instance_id']);
			}

			return $query['count'];
		}

		/**
		*	Read a work order detail
		*
		*	@param wo_id
		*	@param detail_id
		*	@param for_edit Boolean.  Pass true if you intend to make changes, so the reserved stock
		*		can be correctly updated.
		*/
		public function get_work_order_detail($wo_id, $detail_id, $for_edit = false) {
			$read = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');
			if(!$read) {
				return array();
			}
			$detail = $this->so->get_work_order_detail($wo_id, $detail_id, $for_edit);

			return $detail;
		}

		public function set_work_order_detail($work_order_id, $detail_id, $detail, $un_reserve_stock = true) {
			$edit = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_EDIT, 'perp_manufacturing');
			if(!$edit && $work_order_id >= 0) {
				return array();
			}
			$work_order = $this->get_work_order($work_order_id);

			$result = $this->so->set_work_order_detail($work_order, $detail_id, $detail, $un_reserve_stock);

			// Don't create workflow for temporary work orders
			if($work_order_id > 0) {
				ExecMethod2('perp_manufacturing.bo_perp_workflow.create_new_instance', $work_order_id, $detail_id);
			}

			return $result;
		}

		/**
		*	Get the different items between two bills of materials
		*
		*	Use direction to set +quantity (added) or -quantity (removed)
		*/
		public static function bom_diff($bom_1, $bom_2, $direction = 1) {
			return so_perp_work_order::bom_diff($bom_1, $bom_2, $direction);
		}

		public function get_by_serial_number($stock_id, $serial) {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				return false;
			}
			return $this->so->get_by_serial_number($stock_id, $serial);
		}

		public function get_serial_number($work_order_id, $stock_id) {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				return false;
			}
			return $this->so->get_serial_number($work_order_id, $stock_id);
		}

		public function get_unclaimed_work_order($stock_id, $location = '', $required_date = '', $work_order_id = '', $detail_id = '') {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				return false;
			}

			if($work_order_id && is_numeric($work_order_id)) {
				if($detail_id) {
					$match_to = $this->get_option_search_string($work_order_id, $detail_id);
				} else {
					$this->get_work_order_detail_list(
						array(
							'work_order_id'	=>	$work_order_id,
							'template'	=>	false
						),
						$details
					);
					foreach($details as $detail) {
						if($detail['stock_id'] == $stock_id) {
							$match_to = $this->get_option_search_string($work_order_id, $detail['detail_id']);
							break;
						}
					}
				}
			}

			return $this->so->get_unclaimed_work_order($stock_id, $location, $required_date, $match_to);
		}

		public function get_option_search_string($match_to_wo_id, $detail_id) {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				return false;
			}

			return $this->so->get_option_search_string($match_to_wo_id, $detail_id);
		}

		/*
		*	The options (selected)
		*	Gets all the options (categories) available for the stock_id, and all the values (options) available for each category
		*	If the option has not been set, it is set to the default for the category.  Then, we check for options on the options
		*	recursively.  For example, you have a phone that comes in white or black, but the black phone can have silver or grey
		*	trim.
		*	
		*	query:
		*		'work_order_id',
		*		'detail_id',
		*		'stock_id',
		*		'effective_date',
		*		'no_status' (boolean) -Internal use- Avoid a call to get_work_order_status
		*		'no_readonlys'(boolean)	- No readonlys,
		*		'no_options' (boolean) - Skip all the recursion and just return the current settings
		*		'include_discontinued_options' (boolean) 
		*		
		*/
		public function get_work_order_detail_option_list($query = array(), &$rows = array(), &$readonlys = array()) {
			$read = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');

			if ($this->debug) {
				echo 'Incoming query:';
				_debug_array($query);
			}

			if(!$read || !$query['stock_id']) {
				if(count($query) == 0) {
					return array();
				} else {
					$rows = array();
					return 0;
				}
			}

			// No options for raw stock, don't even bother
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $query['stock_id']);
			if($stock['stock_type_flag'] == 'R') {
				return (count($query) == 0 ? array() : 0);
			}

			/**
			*	Check work order status, if it's complete, it's really easy
			*	Just load its options, format the results as needed and return
			*/
			if(!$query['work_order']) {
				$query['work_order'] = $this->get_work_order($query['work_order_id']);

				if($query['work_order']['completed'] || $query['no_options']) {
					$rows = array();
					$options = $this->get_work_order_detail_option($query['work_order_id'], $query['detail_id']);
					foreach($options as $option) {
						$category = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option_category', $option['option_category_id']);
						$build_option = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option', $option['build_option_id']);
						$rows[] = array(
							'build_option_category_id'	=>	$option['option_category_id'],
							'build_option_category_name'	=>	$category['build_option_category_name'],
							'value'				=>	$option['build_option_id'],
							'options-value'	=>	array(
								$option['build_option_id']	=>	array(
									'build_option_id'	=>	$option['build_option_id'],
									'build_option_name'	=>	$build_option['build_option_name'],
									'component'		=>	$option['component'],
									'value'			=>	$option['build_option_id']
								)
							)
						);
						$readonlys[ count($rows) . '[value]'] = true;
					}
					return count($options);
				}
			}

			// Get allowed options
			$query['include_discontinued_options'] = (array_key_exists('include_discontinued_options',$query))?$query['include_discontinued_options']:false;
			$allowed_build_option_list_query['stock_id'] = $query['stock_id'];
			$allowed_build_option_list_query['as_of_date'] = $query['effective_date'];
			$allowed_build_option_list_query['include_discontinued_options'] = $query['include_discontinued_options'];
			$allowed_options = ExecMethod2('perp_manufacturing.bo_perp_build_option.get_allowed_build_option_list', $allowed_build_option_list_query);

			if($this->debug) {
				echo 'Allowed options for stock_id ' . $query['stock_id'];
				_debug_array($allowed_options);
			}

			// I'd like to use this, but you also need to check the BoM too
/*
			if(count($allowed_options) == 0) {
				// No options, return now
				$rows = array();
				return 0;
			}
*/

			$readonlys = array();

			// Reformat allowed options
			$list = array();
			$current_option = null;
			$value = array();
			foreach($allowed_options as $key => $option) {
				if($option['value'] == 0) {
					// Not allowed
					continue;
				}
				if($current_option != $option['build_option_category']) {
					$current_option = $option['build_option_category'];
				
					$value = $this->so->get_work_order_detail_option($query['work_order_id'], $query['detail_id'], $option['build_option_category']);
				}
				if($option['value'] || ($value['component'] || $value['option_bom_id'])) {
					$list[$option['build_option_category']]['build_option_category_id'] = $option['build_option_category'];
					$list[$option['build_option_category']]['build_option_category_name'] = $option['build_option_category_name'];
					$list[$option['build_option_category']]['required'] = $option['required'];
					$list[$option['build_option_category']]['stock_id'] = $query['stock_id'];
					$list[$option['build_option_category']]['options-value'][$option['build_option_id']] = array(
						'build_option_id'	=>	$option['build_option_id'],
						'build_option_name'	=>	$option['build_option_name'],
						'component'		=>	$option['component'],
						'value'			=>	$option['value']
					);
				}
				/**
				* Try to find the most preferred option value for a category.
				* If the work order has the option set already, we take that.
				* Otherwise, we take the most preferred option.  Note that $list[build_option_category]['value'] is
				* the build_option_id of the current most preferred option (12, 34, etc) while 
				* $list[build_option_category][options-value][$list[build_option_category]['value']]['value'] is how preferred it is (-1, -2, -3)
				*/
				if(is_array($value) && array_key_exists('build_option_id', $value)) {
					$list[$option['build_option_category']]['value'] = $value['build_option_id'];
				} elseif($option['value'] < 0) {
					// Higher values ( < 0) are more preferred (-1=required, -2=default, -3=option)
					if(!$list[$option['build_option_category']]['value'] || $list[$option['build_option_category']]['options-value'][$option['build_option_id']]['value'] < 0) {
						if($option['value'] > $list[$option['build_option_category']]['options-value'][$list[$option['build_option_category']]['value']]['value']) {
							// Found a more preferred option
							$list[$option['build_option_category']]['value'] = $option['build_option_id'];
						} else {
							if(is_array($value) && $value['build_option_id']) {
								// If the work order has this option set, go with it
								$list[$option['build_option_category']]['value'] = $value['build_option_id'];
							} elseif (!$list[$option['build_option_category']]['value']) {
								// Haven't picked one yet
								$list[$option['build_option_category']]['value'] = $option['build_option_id'];
							}
						}
					}
				}

				// Check for readonly
				if($query['no_status']) {
					continue;	// Don't check status
				}

				$status = $this->get_work_order_status($query['work_order_id'], $query['detail_id']);
				if($status == 'pending' || $status == 'scheduled' || $status == 'pre-production') {
					continue;	// change allowed
				}
				if($this->config['last_minute_option_changes'] && !$option['required']) {
					// Look for a workflow instance that tracks this
					$instance_id = ExecMethod2('perp_manufacturing.bo_perp_workflow.get_instance_id', $query['work_order_id'], $query['detail_id']);
					if(!ExecMethod2('perp_manufacturing.bo_perp_workflow.instance_was_at_work_center', $instance_id, $option['work_center'])) {
						continue;	// Hasn't been there yet, change allowed
					}
				}
				// Readonly
				if(!$query['no_readonlys'] && $list[$option['build_option_category']]) {
					$list[$option['build_option_category']]['readonly'] = True;
				}
			}

			$rows = array();
		
			foreach($list as $option) {
				// Only 1 option is readonly, but this can cause problems comparing those options
				/*
				if(count($option['options-value']) <= 1) {
					$option['value'] = key($option['options-value']);
					$option['readonly'] = true;
				}
				*/

				$rows[] = $option;

				// Check for options on options
				if(!$option['options-value'][$option['value']]['component'] || $option['options-value'][$option['value']]['component'] == $query['stock_id']) {
					continue;
				}
				$sub_query = array(
					'work_order_id'	=>	$query['work_order_id'],
					'detail_id'	=>	$query['detail_id'],
					'stock_id'	=>	$option['options-value'][$option['value']]['component'],
					'effective_date'	=>	$query['effective_date'],
					'no_status'	=>	$query['no_status'],
					'no_readonlys'	=>	$query['no_readonlys'],
					'include_discontinued_options' => $query['include_discontinued_options']
				);
				$sub_options = array();
				$sub_ro = array();
				if($this->debug) {
					echo 'Recurse on option component ' . $sub_query['stock_id'] , '<br />';
					echo '<b>-------------------------------------</b><br />';
				}
				$this->get_work_order_detail_option_list($sub_query, $sub_options, $sub_ro);
				if($this->debug) {
					echo '<b>=====================================</b><br />end recursion.  Here\'s the sub-options:';
					_debug_array($sub_options);
				}
				$rows = array_merge($rows, $sub_options);
			}
			
			$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $query['stock_id'], $query['effective_date']));
			if(isset($bom['details']) && is_array($bom['details']) && $stock['stock_type_flag'] == 'A') {
				foreach($bom['details'] as $detail) {
					// Check for recursive bill of materials - just one level, won't look for multi-level
					if($detail['component'] == $query['stock_id']) {
						$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $query['stock_id']);
						echo lang('Error: bill of materials for %1 contains itself!', $stock['stock_num']);
						continue;
					}

					// Check for options on build components
					$sub_query = array(
						'work_order_id'	=>	$query['work_order_id'],
						'detail_id'	=>	$query['detail_id'],
						'stock_id'	=>	$detail['component'],
						'effective_date'	=>	$query['effective_date'],
						'no_status'	=>	$query['no_status'],
						'no_readonlys'	=>	$query['no_readonlys'],
						'include_discontinued_options' => $query['include_discontinued_options']
					);
					$sub_options = array();
					$sub_ro = array();
					if($this->debug) {
						echo 'Recurse on build component ' . $sub_query['stock_id'] , '<br />';
						echo '<b>-------------------------------------</b><br />';
					}
					$this->get_work_order_detail_option_list($sub_query, $sub_options, $sub_ro);
					if($this->debug) {
						echo '<b>=====================================</b><br />end recursion on ' . $sub_query['stock_id'] . '.  Here\'s the sub-options:';
						_debug_array($sub_options);
					}
					$rows = array_merge($rows, $sub_options);
				}
			}

			// Check exclusion matrix
			if(is_null(self::$exclusion_matrix)) {
				self::$exclusion_matrix = bo_perp_build_option::get_exclusion_matrix();
			}
			for( $i = 0; $i < count($rows); $i++) {
				// Check to make sure the current value is still allowed, sometimes orphaned option sub-branches
				// will try to disallow the new sub-branches.
				if(in_array($rows[$i]['value'], array_keys($rows[$i]['options-value'])) && in_array($rows[$i]['value'], array_keys(self::$exclusion_matrix))) {
					// Found something that excludes other options, look through other options and remove
					foreach($rows as $row_id => $category) {
						if($remove = array_intersect(array_keys($category['options-value']), self::$exclusion_matrix[$rows[$i]['value']])) {
							// An option here is not compatible with $rows[$i]
							foreach($remove as $option_id) {
								if($this->debug) {
									echo 'Removing: ';
									_debug_array($rows[$row_id]['options-value'][$option_id]);
									echo 'because ' . $rows[$i]['options-value'][$rows[$i]['value']]['build_option_name'] . ' says so.<br />';
								}
								unset($rows[$row_id]['options-value'][$option_id]);
							}
						}
					}
				}
				
			}

			foreach($rows as $key => $row) {
				$readonlys[ $key + 1 . '[value]'] = $row['readonly'];
			}
			if($this->debug) {
				echo 'Which gives, when added to parent:';
				_debug_array($rows);
				_debug_array($readonlys);
			}
			return count($rows);
		}


		public function get_work_order_detail_option($wo_id, $detail_id, $category = '') {
			$read = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');
			if($read) {
				return $this->so->get_work_order_detail_option($wo_id, $detail_id, $category);
			}
		}

		/**
		*	Set an option on a work order detail
		*
		*	@param wo_id ID number of the work order
		*	@param detail_id ID number of the detail
		*	@param stock_id ID of the stock item that the option is on.  This isn't always the item being made.
		*	@param category Build option category id
		*	@param option Build option id
		*/
		public function set_work_order_detail_option($wo_id, $detail_id, $stock_id, $category, $option) {
			$edit = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_EDIT, 'perp_manufacturing');
			if($edit || $wo_id < 0) {
				// Check to make sure this is a valid option at the time this work order was created
				if($wo_id > 0) {
					$work_order = $this->get_work_order($wo_id);
					
					$allowed_build_option_list_query['stock_id'] = $stock_id;
					$allowed_build_option_list_query['as_of_date'] = $work_order['created_date'];
					$allowed_options = ExecMethod2('perp_manufacturing.bo_perp_build_option.get_allowed_build_option_list', $allowed_build_option_list_query);
					
					$allowed = false;
					foreach($allowed_options as $key => $allowed_option) {
						if($allowed_option['build_option_category'] == $category &&
							$allowed_option['build_option_id'] == $option
						) {
							// Record changes
							if(!in_array($this->get_work_order_status($wo_id, $detail_id), self::$no_history_statuses)) {
								$old_option = $this->so->get_work_order_detail_option($wo_id, $detail_id, $category);
								bo_perp_work_order::add_option_history($wo_id, $detail_id, $category, $old_option['build_option_id'], $option);
							}
							return $this->so->set_work_order_detail_option($wo_id, $detail_id, $stock_id, $category, $option);
						}
					}
					$disallowed_option = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option', $option);
					if($disallowed_option['effective_to_date'] != false && $disallowed_option['effective_to_date'] < time()) {
						return lang('%1 has been discontinued, effective %2',
							$disallowed_option['build_option_name'],
							date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $disallowed_option['effective_to_date'])
						);
					} else {
						return lang('You have chosen an invalid option:  %1 ',
							$disallowed_option['build_option_name']
						);
					}
				}

				return $this->so->set_work_order_detail_option($wo_id, $detail_id, $stock_id, $category, $option);
			}
		}

		public function delete_work_order_detail_option($work_order_id, $detail_id, $category = '') {
			$edit = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_EDIT, 'perp_manufacturing');
			if($edit || $wo_id < 0) {
				$this->so->delete_work_order_detail_option($work_order_id, $detail_id, $category);
			}
		}

		/**
		*	Flush the options and add any new options as well as get rid of any leftovers.
		*
		*	This can be called explicitly, but will be called automatically when this object is destroyed.
		*/
		public function flush_options() {
			$this->so->flush_options();
		}

		/* Make the work order reference number
		*
		* @depricated Use suggest_work_order_number instead
		*/
		public function get_reference($stock_id = '') {
			return $this->suggest_work_order_number($stock_id);
		}

		public function suggest_work_order_number($stock_id = '') {
			$format = $this->config['work_order_reference'];
			if($stock_id) {
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			}

			// Next WO_id
			$wo_num = $this->so->get_next_id();

			// Next stock serial
			if($stock) {
				$stock_category = $stock['stock_category'];
			}

			$ref = sprintf($format,
				$wo_num,
				$stock_id,
				$stock_category
			);
			return $ref;
		}

		/**
		*	Get the complete bill of materials for a work order detail.
		*
		*	Includes options, pseudo sub-assemblies, etc.
		*
		*	Query keys:
		*		- work_order_id
		*		- detail_id
		*		- work_center_id
		*
		*	@todo Readonly permission could be more accurate, especially if 
		*		the config option to allow last minute option changes 
		*		is used, currently it just locks based on status.
		*/
		public function get_work_order_detail_bom(&$query = array(), &$rows = array(), &$readonlys = array()) {
			//TODO: ACLs
			if(!is_array($query)) {
				throw new egw_exception("get_work_order_detail_bom() should have a query parameter");

				$query = array(
					'work_order_id'	=>	$query,
					'detail_id'	=>	$rows,
					'order'		=>	'stock_num'
				);
				return $this->so->get_work_order_detail_bom($query);
			}

			if($query['filter']) {
				$query['work_center_id'] = $query['filter'];
			}
			if(!$query['order']) {
				$query['order'] = 'stock_num';
			}
			
			$list = $this->so->get_work_order_detail_bom($query);

			$status = $this->get_work_order_status($work_order_id);
			$edit = !in_array($status, bo_perp_work_order::$build_statuses);
			foreach($list as $key => &$item) {
				if($item['controlled']) {
				} else {
					$readonlys[$key + 1 . '[serial_number]'] = True;
				}
				$readonlys[$key + 1 . '[quantity]'] = !$edit;
			}
			$rows = $list;
			return $query['count'];
		}

		/**
		*	Change the bill of materials for a work order
		*
		*	This is for changing one component of the BOM for a work order detail,
		*	it does not change the original BOM.  
		*
		*	@todo ACLs
		*
		*	@param work_order_id
		*	@param detail_id
		*	@param component	Stock ID of the changed component
		*	@param delta_quantity	Change in quantity from the original BOM
		*	@param bom_detail_id If this is a change on an existing component, 0 for an added part
		*	@param work_center_id	ID of the work center this component is used at
		*/
		public function change_bill_of_material($work_order_id, $detail_id, $component, $delta_quantity, $bom_detail_id = 0, $work_center_id = 0) {
			if(!$work_order_id) {
				throw new egw_exception_wrong_parameter("Missing work_order_id");
			} elseif (!$detail_id) {
				throw new egw_exception_wrong_parameter("Missing detail_id");
			}

			$query = array(
				'work_order_id'	=>	$work_order_id,
				'detail_id'	=>	$detail_id,
				'component'	=>	$component,
				'work_center_id'	=>	$work_center_id,
				'bill_of_material_detail_id'	=>	is_null($bom_detail_id) ? 0 : $bom_detail_id
			);

			// Set change
			$query['delta_quantity'] = $delta_quantity;

			$this->so->change_bill_of_materials($query);
			return true;
		}

		/**
		*	Get a list of changes to the original bill(s) of material for a work order
		*
		*	@nextmatch
		*
		*	@todo ACLs
		*/
		public function get_bill_of_material_changes(&$query = array(), &$rows = array(), &$readonlys) {
			
			if(!$query['order']) {
				$query['order'] = 'stock_num';
			}
			if(!$query['num_rows']) {
				$query['num_rows'] = -1;
			}

			$rows = $this->so->get_bill_of_material_changes($query);

			return $query['count'];
		}
	
		/**
		*	Get a text reference to display for a work order.
		*	Used to participate in linking.
		*	@param entry work order ID
		*	@return String reference
		*/
		public function link_title($entry) {
			list($location, $id) = explode('-', $entry);
			if(!$id) {
				$id = $location;
			}
			$work_order = $this->get_work_order($id);
			return $work_order['work_order_reference'];
		}

		/**
		*	Record a change in work order option
		*/
		public static function add_option_history($work_order_id, $detail_id, $build_option_category_id, $old_option, $new_option) {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_EDIT, 'perp_manufacturing')) {
				return false;
			}
			return so_perp_work_order::add_option_history($work_order_id, $detail_id, $build_option_category_id, $old_option, $new_option);
		}

		/**
		*	Get a list of option changes
		*/
		public static function get_option_history($work_order_id, $detail_id, $build_option_category_id = '') {
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				return array();
			}
			return so_perp_work_order::get_option_history_list($work_order_id, $detail_id, $build_option_category_id);
		}
	
		/**
		*	Get a list of finish locations used, in the form ID => NAME
		*/
		public static function get_end_locations() {
			$query = array();
			return so_perp_work_order::get_end_locations($query);
		}
	}
?>
