<?php

	/**
	*	Integration with the workflow app for builds.
	*
	*	It is expected that each workflow is made of Work Centers,
	*	and that the appropriate functions are called.  For actual assembly,
	*	(goods put into the item being built), use the perp_build agent and set the 
	*	work center.
	*	
	*/

	require_once('class.bo_perp_work_order.inc.php');
	require_once('class.so_perp_workflow.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/workflow/inc/class.workflow_instance.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/workflow/inc/class.workflow_process.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/workflow/inc/class.workflow_instancemanager.inc.php');

	class bo_perp_workflow {

		/**
		*	Storage layer object for accessing database
		*/
		private $so_perp_workflow;

		private static $debug = false;

		public function __construct() {
			$this->so_perp_workflow = new so_perp_workflow();
		}

		/**
		*	Get the process ID of the workflow process to be used when
		*	manufacturing the given stock item
		*
		*	@param stock_id	ID number of the Assemby stock item being manufactured
		*
		*	@return process_id	ID number of the workflow process
		*/
		public function get_process_id($stock_id) {
			// Check for workflow
			if(!isset($GLOBALS['egw_info']['apps']['workflow']) || !$stock_id) {
				return false;
			}
			return $this->so_perp_workflow->get_process_id($stock_id);
		}

		/**
		*	Set the process ID of the workflow process to be used when
		*	manufacturing the given stock item
		*
		*	@param stock_id	ID number of the Assemby stock item being manufactured
		*	@param process_id	ID number of the workflow process
		*/
		public function set_process_id($stock_id, $process_id) {
			if(!$stock_id || !isset($GLOBALS['egw_info']['apps']['workflow'])) {
				return false;
			}
			$old_process_id = $this->get_process_id($stock_id);
			if($process_id == $old_process_id) {
				return true;
			}

			$result = true;
			if($process_id) {
				// Check the process for this stock item
				$result = $this->check_process($process_id, $stock_id);
			}

			$this->so_perp_workflow->set_process_id($stock_id, $process_id);
			return $result;
		}

		/**
		*	Check the process to try to avoid any unpleasant surprises while actually building things
		*
		*	@param process_id
		*	@param stock_id
		*/
		public function check_process($process_id, $stock_id, $work_order_id = null, $detail_id = null) {
			$messages = array();

			// Get a map of work center => activity
			$work_center_activities = array();
			$activities = ExecMethod('workflow.workflow_activitymanager.get_process_activities', $process_id);
			foreach($activities as $activity) {
				$activity = ExecMethod('workflow.workflow_activitymanager.get_activity', $activity['wf_activity_id']);

				if(!is_array($activity['wf_agents']) || $activity['wf_type'] == 'end') {
					// Just a logical step, no inventory involved
					continue;
				}
				foreach($activity['wf_agents'] as &$agent) {
					if($agent['wf_agent_type'] == 'perp_build') {
						$agent = ExecMethod2('workflow.workflow_activitymanager.get_activity_agent_data', $activity['wf_activity_id'], $agent['wf_agent_type']);
						$work_center_activities[$agent['work_center_id']] = $activity;
					}
				}
			}

			if(!$work_order_id) {
				// Check to make sure any serialized components are added at a work center that 
				$bom_id = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $stock_id);
				$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $bom_id);
				if($bom['details']) {
					foreach($bom['details'] as $detail) {
						if($detail['controlled'] && (
								$work_center_activities[$detail['work_center']]['wf_is_interactive'] != 'y' || 
								$work_center_activities[$detail['work_center']]['wf_agents'][0]['display_parts'] == 0)
						) {
							$messages[] = lang('Component %1 is controlled.  %2 must be interactive, and display at least Controlled / Serialized parts',
								$detail['stock_num'],
								$work_center_activities[$detail['work_center']]['wf_name']
							);
						} 
						if($detail['stock_type_flag'] != 'R') {
							$sub_result = $this->check_process($process_id, $detail['component']);
							if(is_array($sub_result)) {
								$messages = array_merge($messages, $sub_result);
							}
						}
					}
				}
			} else {
				$bo_work_order = new bo_perp_work_order();
				$query = array(
					'work_order_id'	=>	$work_order_id,
					'detail_id'	=>	$detail_id,
					'num_rows'	=>	-1
				);
				$bo_work_order->get_work_order_detail_bom($query, $components);
				foreach($components as $detail) {
					if($detail['controlled'] && (
							$work_center_activities[$detail['work_center_id']]['wf_is_interactive'] != 'y' || 
							$work_center_activities[$detail['work_center_id']]['wf_agents'][0]['display_parts'] == 0)
					) {
						$messages[] = lang('Component %1 is controlled.  %2 must be interactive, and display at least Controlled / Serialized parts',
							$detail['stock_num'],
							$work_center_activities[$detail['work_center_id']]['wf_name']
						);
					}
					if(!isset($work_center_activities[$detail['work_center_id']])) {
						$work_center = ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center', $detail['work_center_id']);
						if($work_center) {
							$messages[] = lang('Component %1 is added at work center %2, but the workflow does not go there',
								$detail['stock_num'],
								$work_center['name']
							);
						} else {
							$messages[] = lang('Component %1 is not added at any work center.', $detail['stock_num']);
						}
					}
				}
			}

			if(count($messages) ) {
				return $messages;
			} else {
				return true;
			}
		}

		/**
		*	Get a list of workflow process to be used when
		*	manufacturing the given stock item
		*
		*	@param stock_id	ID number of the Assemby stock item being manufactured
		*
		*	@return array of the workflow process information
		*/
		public function get_process_list($stock_id) {
			if(!isset($GLOBALS['egw_info']['apps']['workflow'])) {
				return array();
			}
			return $this->so_perp_workflow->get_process_list($stock_id);
		}

		/**
		*	Get the instance ID of the workflow instance to be used when
		*	manufacturing the given work order.
		*
		*	@param work_order_id	ID number of the work order being manufactured
		*	@param detail_id	ID number of the work order detail line
		*
		*	@return instance_id	ID number of the workflow instance
		*/
		public function get_instance_id($work_order_id, $detail_id) {
			if(!isset($GLOBALS['egw_info']['apps']['workflow'])) {
				return false;
			}
			$list = so_perp_workflow::get_instance_id($work_order_id, $detail_id);

			$instance_manager = new workflow_instancemanager();
			foreach($list as $instance_info) {
				$instance_properties = $instance_manager->get_instance_properties($instance_info['wf_instance_id']);
				if(is_array($instance_properties) && $instance_properties['work_order_id'] == $work_order_id && $instance_properties['detail_id'] == $detail_id) {
					return $instance_info['wf_instance_id'];
				}
			}
			return false;
		}

		/**
		*	Create a new workflow instance for a work order
		*
		*	@param work_order_id	ID number of the work order being manufactured
		*	@param detail_id	ID number of the work order detail line
		*
		*	@return instance_id	ID number of the new workflow instance
		*/
		public function create_new_instance($work_order_id, $detail_id) {
			if($instance_id = $this->get_instance_id($work_order_id, $detail_id)) {
				return false;
			}

			$settings = config::read('perp_manufacturing');
			if(!$settings['use_workflow']) {
				return false;
			}
			$bo_work_order = new bo_perp_work_order();

			$work_order = $bo_work_order->get_work_order($work_order_id);
			$detail = $bo_work_order->get_work_order_detail($work_order_id, $detail_id);

			// Get process
			$process_id = $this->get_process_id($detail['stock_id']);
			$process = new workflow_process();
			$process->getProcess($process_id);
			if(!$process_id || !$process) {
				throw new egw_exception(lang('Could not find workflow process.'));
			}

			// Get start activity
			$start_activity = $process->getActivityByName('start');
			$GLOBALS['egw_flags']['currentapp'] = 'workflow';
			$GLOBALS['egw_flags']['currentapp'] = 'perp_manufacturing';

			$new_instance = new workflow_instance();
			if(!$instance_id) {
				$returned_data = $new_instance->executeAutomaticActivity($start_activity['wf_activity_id'], false);
				$instance_id = $GLOBALS['workflow']['wf_instance_id'];
			}

			$new_instance->loadInstance($instance_id);
			$new_instance->setName($work_order['work_order_reference'] . ':' . $detail_id);
			$new_instance->set('work_order_id', $work_order_id);
			$new_instance->set('detail_id', $detail_id);
			$result = $new_instance->sync();

			return $instance_id;
		}

		/**
		*	Check the workflow to see if the given instance has passed through the given 
		*	work center.
		*
		*	@param instance_id
		*	@param work_center_id
		*
		*	@param boolean True if the instance has been at the work center, false if not
		*/
		public function instance_was_at_work_center($instance_id, $work_center_id) {
			if($instance_id) {
				return $this->so_perp_workflow->instance_was_at_work_center($instance_id, $work_center_id);
			} else {
				return false;
			}
		}

		/**
		*	Move a workflow instance along the workflow path, through any activity it would normally
		*	go through.  If you're dealing with multiple simultaneous paths, all paths are traversed 
		*	on the way from the source to the destination.
		*	
		*	This differs from workflow's set_instance_destination(), which jumps directly to the 
		*	specified destination and skips all activities in between.
		*
		*	We walk back up the graph (assume DAG, otherwise we're in trouble) and record the activities
		*	between the destination and the current location(s), then we walk forward from the 
		*	current location(s) and visit each node.
		*
		*	Of special note are splits and joins.  If we walk through a join, we still have to wait
		*	for all activities to get there.  If we walk through a split and a join, we need to visit
		*	all paths.  If we walk through a split, but not a join, parallel paths should NOT be 
		*	followed.
		*
		*	@param work_order_id The unique identifier for the work order to be moved
		*	@param detail_id Which item on the work order to be moved.  You can have multiple details / work order
		*	@param destination The system ID of the work center (not inventory location or activity) to
		*		move the work order to.  
		*
		*	@throws egw_exception if the destination cannot be reached from the current activity
		*/
		public static function move_to($work_order_id, $detail_id, $destination_activity) {
			// Identify work order instance & process
			$instance_id = self::get_instance_id($work_order_id, $detail_id);
			$instance =& new Instance($GLOBALS['egw']->db->Link_ID);
			$instance->loadInstance($instance_id);
			
			// Identify current location(s) (workflow activities)
			$current = ExecMethod('workflow.workflow_instancemanager.get_instance_activities', $instance_id);

			// Extract process_id
			foreach($current as $activity) {
				if($activity['wf_p_id']) {
					$process_id = $activity['wf_p_id'];
					break;
				}
			}
			if(!$process_id) {
				throw new egw_exception('Unable to determine workflow process');
			}
			
			// Load all activities and index by ID, we'll need them later
			$process_activities = ExecMethod('workflow.workflow_activitymanager.get_process_activities', $process_id);
			$activities = array();
			foreach($process_activities as $p_activity) {
				$activities[$p_activity['wf_activity_id']] = $p_activity;
			}

			// Walk back from destination to current location(s)
			foreach($current as $activity) {
				// it's already there...
				if($activity['wf_activity_id'] == $destination_activity && $activity['wf_act_status'] == 'complete') {
					return true;
				}

				// Find the steps we need to follow to get from here to the destination
				$back_path = so_perp_workflow::graph_walk($process_id, $destination_activity, $activity['wf_activity_id'], 'backward');
				if($back_path) {
					if(self::$debug) {
						echo "Path from {$activity['wf_name']} to {$activities[$destination_activity]['wf_name']}: " .print_r($back_path, true) . '<br />';
					}
					array_unshift($back_path, $activity['wf_activity_id']);
					$last_node = $activity['wf_activity_id'];
					foreach($back_path as $node) {
						if(self::$debug) {
							echo "Walking from $last_node -> $node<br />";
						}
						$current_node = $activities[$node];

						// Need to check the status _right_now_, but can't change the $current variable
						$temporary_current = ExecMethod('workflow.workflow_instancemanager.get_instance_activities', $instance_id);
						foreach($temporary_current as $t_activity) {
							// Update list of activities
							$activities[$t_activity['wf_activity_id']] = array_merge($activities[$t_activity['wf_activity_id']], $t_activity);
							if($t_activity['wf_activity_id'] == $node) {
								$current_node = $t_activity;

								// If it's not done, try to run it
								
								if(/*$current_node['wf_type'] != 'join' &&*/ $t_activity['wf_act_status'] != 'completed') { /* ||
										$activities[$node]['wf_type'] == 'join' && $activities[$node]['wf_act_status'] != 'waiting') {*/
									if(self::$debug) {
										echo "Trying to run $node<br />";
									}
									if($activities[$node]['wf_is_interactive'] == 'y') {
										$GLOBALS['egw']->redirect_link('/index.php', array(
											'menuaction'	=>	'workflow.run_activity.go',
											'iid'		=>	$instance_id,
											'activity_id'	=>	$node
										));
									} else {
										// Delete the current node if it's not a split
										$delete_current = ($current_node['wf_type'] != 'split');
										$result = $instance->sendTo($last_node, $node, $delete_current);
									}
									if(self::$debug) {
										_debug_array($result);
									}
									if($result === false) {
										// Execution failed
										break 2;
									}
								}
								break;
							}
						}

						// Transitions and cleanup
						if($current_node['wf_type'] == 'join') {
							if($current_node['wf_act_status'] == 'waiting') {
								if(self::$debug) {
									echo $current_node['wf_name'] . ' is a join that needs to wait for other activities.<br />';
								}
								// Hit a join that needs to wait
								break;
							} elseif ($current_node['wf_act_status'] == 'completed') {
								// Join is done, remove any complete prerequisites
								$prerequisites = so_perp_workflow::get_process_transitions($process_id, $node, 'backward');
								if(self::$debug) {
									echo 'Join is done, removing any complete prerequisites.  All prerequisites:<br />';
									_debug_array($prerequisites);
								}
								$prerequisite_check_activities = ExecMethod('workflow.workflow_instancemanager.get_instance_activities', $instance_id);
								foreach($prerequisite_check_activities as $check_activity) {
									if(in_array($check_activity['wf_activity_id'], $prerequisites) && $check_activity['wf_act_status'] == 'completed') {
										so_perp_workflow::delete_instance_activity($instance_id, $check_activity['wf_activity_id']);
									}
								}
							}
						}
						if($node != $destination_activity) {
							if(in_array($current_node['wf_act_status'], array('completed')) && $current_node['wf_is_autorouted'] == 'n') {
								$transitions = so_perp_workflow::get_process_transitions($process_id, $node, 'forward');
								$transition = true;
								foreach($transitions as $new_node) {
									if($activities[$new_node]['wf_act_status'] == 'completed') {
										// Already there
										$transition = false;
										
										// Remove activity from list
										so_perp_workflow::delete_instance_activity($instance_id, $node);
									}

								}
								if($transition) {
									// Force it
									$result = $instance->sendAutorouted($node, true);
								}
							}
						}
						$last_node = $node;
						$current_node = null;
					}
				}
			}
			
			// Check to make sure we're there
			$new_current = ExecMethod('workflow.workflow_instancemanager.get_instance_activities', $instance_id);
			foreach($new_current as $activity) {
				if($new_current['wf_activity_id'] == $destination_activity) {
					return true;
				}
			}

			// Not there yet.  Did we move at all?
			if(count(array_diff($new_current, $current)) > 0) {
				if(self::$debug) {
					echo 'Didn\'t make it to the destination.  Trying again.<br />';
				}

				// Some progress was made.  Probably got stopped at a split / join.  Try again.
				self::move_to($work_order_id, $detail_id, $destination_activity);
			}
			return false;
		}
	}
?>
