<?php

	class so_perp_bill_of_material {

		public function so_perp_bill_of_material() {
		}

		public function get_bill_of_material_list(&$query) {
			$where = array();
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%'.$query['search'].'%');
				$where[] = '(perp_stock.stock_num like ' . $search . ' OR perp_stock.short_description LIKE ' . $search . ')';
			}
			if($query['stock_id']) {
				$where[] = "perp_bill_of_material.stock_id = " . (int)$query['stock_id'];
			}
			if(isset($query['stock_category'])) {
				$where[] = "perp_stock.stock_category = " . (int)$query['stock_category'];
			}
			if($query['status']) {
				if($query['status'] == 'current') {
					$where[] = "(NOW() >= effective_after_date AND (NOW() <= effective_to_date OR effective_to_date IS NULL))";
				} elseif($query['status'] == 'previous') {
					$where[] = "(effective_to_date <= NOW())";
				} elseif($query['status'] == 'future') {
					$where[] = "(effective_after_date >= NOW())";
				}
			}
			if($query['as_of_date']) {
				$where[] = "'" . $GLOBALS['egw']->db->to_timestamp($query['as_of_date']) . '\' >= effective_after_date AND (\'' .
					$GLOBALS['egw']->db->to_timestamp($query['as_of_date']) . '\' <= effective_to_date OR effective_to_date IS NULL
				)';
			}
			if($query['bill_of_material_id']) {
				$where['bill_of_material_id'] = $query['bill_of_material_id'];
			}

			$GLOBALS['egw']->db->select('perp_bill_of_material', array(
				'bill_of_material_id',
				'effective_after_date',
				'effective_to_date',
				'perp_bill_of_material.stock_id AS stock_id',
				'stock_num',
				'short_description',
				'comment'),
				$where,
				__LINE__, __FILE__,
				False, 'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_manufacturing', 0,
				'JOIN perp_stock ON perp_stock.stock_id = perp_bill_of_material.stock_id'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$row['effective_after_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_after_date']);
				$row['effective_to_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_to_date']);
				$list[] = $row;
			}
			return $list;
		}

		public function get_bill_of_material($bom_id) {
			$GLOBALS['egw']->db->select('perp_bill_of_material', array(
				'bill_of_material_id',
				'effective_after_date',
				'effective_to_date',
				'stock_id',
				'start_work_center',
				'finished_work_center',
				'comment'),
				array('bill_of_material_id' => $bom_id),
				__LINE__, __FILE__
			);
			$bom = $GLOBALS['egw']->db->row(True);
			$bom['effective_after_date'] = $GLOBALS['egw']->db->from_timestamp($bom['effective_after_date']);
			$bom['effective_to_date'] = $GLOBALS['egw']->db->from_timestamp($bom['effective_to_date']);

			if($bom['bill_of_material_id']) {
				$GLOBALS['egw']->db->select('perp_bill_of_material_detail', 
					array(
						'bill_of_material_detail_id',
						'component',
						'stock_type_flag',
						'pseudo',
						'stock_num',
						'controlled',
						'work_center',
						'quantity',
						'perp_unit_of_measure.uom_name AS uom',
						'comment',
						'time',
					),
					array('bill_of_material_id' => $bom['bill_of_material_id']),
					__LINE__, __FILE__,
					False, 'ORDER BY bill_of_material_detail_id',
					'perp_manufacturing', 0,
					'JOIN perp_stock ON 
						perp_stock.stock_id = perp_bill_of_material_detail.component
					JOIN perp_unit_of_measure ON
						perp_stock.unit_of_measure = perp_unit_of_measure.uom_id'
				);
				$bom['details'] = array();
				while($row = $GLOBALS['egw']->db->row(True)) {
					$bom['details'][] = $row;
				}
			}

			// Get change timestamps
			$history = array('add', 'modify', 'delete');
			foreach($history as $action) {
				$bom['history'][$action] = $GLOBALS['egw']->contenthistory->getTSforAction('perp_manufacturing:BOM', $bom_id, $action);
			}

			// Find out who's responsible
			$bom['history']['changed_by'] = $GLOBALS['egw']->db->select(
				contenthistory::TABLE, 
				'sync_changedby', 
				array(
					'sync_appname'	=>	'perp_manufacturing:BOM',
					'sync_contentid'	=>	$bom_id,
				),
				__LINE__, __FILE__
			)->fetchSingle();

			return $bom;
		}

		public function set_bill_of_material($bom_id, $bom) {
			// Make sure to date is null, not blank.
			if(is_null($bom['effective_to_date']) || $bom['effective_to_date'] == '') {
				$to_date = null;
			} else {
				$to_date = $GLOBALS['egw']->db->to_timestamp($bom['effective_to_date']);
			}

			$data = array(
				'effective_after_date'	=>	$GLOBALS['egw']->db->to_timestamp($bom['effective_after_date']),
				'effective_to_date'	=>	$to_date,
				'stock_id'		=>	$bom['stock_id'],
				'start_work_center'	=>	$bom['start_work_center'],
				'finished_work_center'	=>	$bom['finished_work_center'],
				'comment'		=>	$bom['comment']
			);

			$GLOBALS['egw']->db->transaction_begin();
			$GLOBALS['egw']->db->insert('perp_bill_of_material', 
				$data,
				array('bill_of_material_id' => $bom_id),
				__LINE__, __FILE__, 'perp_manufacturing'
			);

			// If there's no bom_id, it's a 'new' bom, so we add the details
			if(!$bom_id) {
				$bom_id = $GLOBALS['egw']->db->get_last_insert_id('perp_bill_of_material', 'bill_of_material_id');
			}
			if(isset($bom['details'])) {
				$GLOBALS['egw']->db->delete(
					'perp_bill_of_material_detail',
					array(
						'bill_of_material_id'	=>	$bom_id
					),
					__LINE__, __FILE__
				);
				foreach($bom['details'] as $key => $detail) {
					$GLOBALS['egw']->db->insert('perp_bill_of_material_detail', array(
							'bill_of_material_id'	=>	$bom_id,
							'bill_of_material_detail_id'	=>	$detail['bill_of_material_detail_id'],
							'component'		=>	$detail['component'],
							'work_center'		=>	$detail['work_center'],
							'quantity'		=>	$detail['quantity'],
							'comment'		=>	$detail['comment'],
							'time'			=>	$detail['time'],
						),
						array(
							'bill_of_material_id'	=>	$bom_id,
							'bill_of_material_detail_id'	=>	$detail['bill_of_material_detail_id'],
						), __LINE__, __FILE__, 'perp_manufacturing'
					);
				}
			}
			$GLOBALS['egw']->db->transaction_commit();
			return $bom_id;
		}

		public function delete_bill_of_material($bom_id) {
			$GLOBALS['egw']->db->transaction_begin();
			$GLOBALS['egw']->db->delete('perp_bill_of_material', 
				array('bill_of_material_id' => $bom_id),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			$GLOBALS['egw']->db->delete('perp_bill_of_material_detail', 
				array('bill_of_material_id' => $bom_id),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			$GLOBALS['egw']->db->transaction_commit();
		}

		public function check_for_loops($stock_id, $search_stock_id) {
			// Get  a list of bills of materials that use the item we're building

			$sql = 'SELECT perp_bill_of_material.bill_of_material_id,
					stock_id
				FROM perp_bill_of_material
				JOIN perp_bill_of_material_detail ON
					perp_bill_of_material.bill_of_material_id = perp_bill_of_material_detail.bill_of_material_id
				WHERE (NOW() >= effective_after_date AND (NOW() <= effective_to_date OR effective_to_date IS NULL))
					AND perp_bill_of_material_detail.component = ';
		
			$GLOBALS['egw']->db->query($sql . $stock_id, __LINE__, __FILE__);
			$boms = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				if($row['stock_id'] == $search_stock_id) {
					// Loop found, return array of information
					return array(
						'bill_of_material_id'		=>	$row['bill_of_material_id'],
						'bill_of_material_detail_id'	=>	$row['bill_of_material_detail_id'],
						'component_id'			=>	$row['stock_id']
					);
				}
				$boms[] = $row;
			}

			foreach($boms as $bom) {
				// Get a list of items that use this item
				return $this->check_for_loops($bom['stock_id'], $search_stock_id);
			}
			return false;
		}

		public function get_used_by($stock_id) {
			$GLOBALS['egw']->db->select('perp_bill_of_material_detail',
				array('bill_of_material_id'),
				array('component'	=>	$stock_id),
				__LINE__, __FILE__,
				false, '', 'perp_manufacturing'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row['bill_of_material_id'];
			}
			return $list;
		}

		public function get_used_by_list(&$query) {
			$select_fields = array(
				'perp_bill_of_material.bill_of_material_id',
				'perp_bill_of_material.stock_id'
			);
			if($query['selectcols']) {
				if(is_array($query['selectcols'])) {
					$select_fields = array_merge($select_fields, $query['selectcols']);
				} else {
					$select_fields[] = $query['selectcols'];
				}
			}
			
			$where = array(
				'component'	=>	$query['stock_id']
			);
			if($query['cat_id']) {
				$where[] = 'stock_category = ' . (int)$query['cat_id'];
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(stock_num LIKE $search OR
					short_description LIKE $search OR
					perp_bill_of_material.bill_of_material_id LIKE $search)";
			}
			if($query['status']) {
				if($query['status'] == 'current') {
					$where[] = "(NOW() >= effective_after_date AND (NOW() <= effective_to_date OR effective_to_date IS NULL))";
				} elseif($query['status'] == 'previous') {
					$where[] = "(effective_to_date <= NOW())";
				} elseif($query['status'] == 'future') {
					$where[] = "(effective_after_date >= NOW())";
				}
			}

			$join = 'JOIN perp_bill_of_material ON
				perp_bill_of_material.bill_of_material_id = perp_bill_of_material_detail.bill_of_material_id
			JOIN perp_stock ON
				perp_stock.stock_id = perp_bill_of_material.stock_id';

			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_bill_of_material_detail',
				array('count(perp_bill_of_material.bill_of_material_id)'),
				$where,
				__LINE__, __FILE__, false, '', 
				'perp_manufacturing', -1,
				$join
			)->fetchSingle();

			$result = $GLOBALS['egw']->db->select(
				'perp_bill_of_material_detail',
				$select_fields,
				$where,
				__LINE__, __FILE__,
				$query['start'],
				"ORDER BY {$query['order']} {$query['sort']}",
				'perp_manufacturing',
				$query['num_rows'],
				$join
			);

			$list = array();
			foreach($result as $row) {
				$row['effective_after_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_after_date']);
				$row['effective_to_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_to_date']);
				$list[] = $row;
			}
			return $list;
		}
	}
?>
