<?php
	/**
	*	Storage layer for dealing with retrofits
	*
	*	@package perp_manufacturing
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_order.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.so_perp_stock_history.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_ledger/inc/class.GLTransaction.inc.php');

	class so_perp_retrofit {

		/**
		*	Get a list of retrofit records
		*/
		public static function get_list(&$query) {
			
			// Fields to retrieve
			$fields = array(
				'perp_stock_history.*',
				'perp_work_order_retrofit.*',
			);

			// Set up WHERE clause
			$where = array();
			if($query['stock_id']) {
				$where[] = 'perp_work_order_detail.stock_id IN (' . implode(',', $query['stock_id']) . ')';
			}
			if($query['work_order_id']) {
				$where[] = 'perp_work_order_detail.work_order_id IN (' . implode(',', $query['work_order_id']) . ')';

				if($query['detail_id']) {
					$where[] = 'perp_work_order_detail.detail_id IN (' . implode(',', $query['detail_id']) . ')';
				}
			}
			if($query['account_id']) {
				$where[] = 'perp_work_order_retrofit.account_id = ' . (int)$query['account_id'];
			}
			if($query['date']) {
				$where[] = 'DATE(perp_stock_history.timestamp) = ' . 
					$GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['date']));
			}

			// Set up JOIN clause
			$join = 'JOIN perp_stock_history ON
				perp_work_order_retrofit.stock_history_id = perp_stock_history.history_id
			JOIN perp_stock_serial ON
                                perp_stock_serial.stock_id = perp_stock_history.stock_id AND
                                perp_stock_serial.serial_number = perp_stock_history.serial_number
                        JOIN perp_work_order_detail ON
                                perp_stock_serial.work_order_id = perp_work_order_detail.work_order_id AND
                                perp_stock_serial.detail_id = perp_work_order_detail.detail_id
			';

			// Get total number of records
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_work_order_retrofit',
				array('count(*)'),
				$where,
				__LINE__, __FILE__, false,
				'', 1, 'perp_manufacturing',
				$join
			)->fetchSingle();

			// Get results
			$result = $GLOBALS['egw']->db->select(
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				"ORDER BY {$query['order']} {$query['sort']}", 
				'perp_manufacturing', $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				$row['timestamp'] = $GLOBALS['egw']->db->from_timestamp($row['timestamp']);
				$list[] = $row;
			}
			return $list;
		}

		/**
		*	Get information on one retrofit
		*/
		public static function get_retrofit($id) {
			$join = 'JOIN perp_work_order_retrofit ON
				perp_work_order_retrofit.stock_history_id = perp_stock_history.history_id';

			$retrofit = $GLOBALS['egw']->db->select(
				'perp_stock_history',
				array('*'),
				array('perp_stock_history.history_id = ' . (int)$id),
				__LINE__, __FILE__, false, '', 'perp_inventory', 1,
				$join
			)->fetch();
			$retrofit['timestamp'] = $GLOBALS['egw']->db->from_timestamp($retrofit['timestamp']);
			return $retrofit;
		}

		/**
		*	Set a retrofit
		*/
		public static function set_retrofit($retrofit) {
			$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order',
				$retrofit['work_order_id']
			);
			$detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail',
				$retrofit['work_order_id'],
				$retrofit['detail_id']
			);
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $detail['stock_id']);

			$GLOBALS['egw']->db->transaction_begin();

			if(!$retrofit['stock_history_id']) {
				// Create stock_history record
				$retrofit['stock_history_id'] = so_perp_stock_history::add_history(
					$retrofit['stock_id'],
					$retrofit['serial_number'],
					$retrofit['comment'],
					array(),
					$retrofit['timestamp']
				);

				// Set the link, now that we know the history ID
				$GLOBALS['egw']->db->update(
					'perp_stock_history',
					array(
						'link_app'	=>	'perp_manufacturing',
						'link_id'	=>	'RETROFIT-' . $retrofit['stock_history_id']
					),
					array(
						'history_id'		=>	$retrofit['stock_history_id'],
					),
					__LINE__, __FILE__, 'perp_inventory'
				);
			}

			if(!$retrofit['stock_history_id']) {
				$GLOBALS['egw']->db->transaction_abort();
				$GLOBALS['egw']->db->transaction_commit();
				return lang('Could not add entry');
			}

			$GLOBALS['egw']->db->insert(
				'perp_work_order_retrofit',
				array(
					'build_option_category_id'	=>	$retrofit['build_option_category_id'],
					'build_option_id'		=>	$retrofit['build_option_id'],
					'reference'			=>	$retrofit['reference'],
					'stock_history_id'		=>	$retrofit['stock_history_id'],
					'account_id'			=>	$retrofit['account_id']
				),
				array(
					'stock_history_id'		=>	$retrofit['stock_history_id'],
				),
				__LINE__, __FILE__, 'perp_manufacturing'
			);

			// Record option history
			$old_option = ExecMethod2('perp_manufacturing.so_perp_work_order.get_work_order_detail_option', 
				$retrofit['work_order_id'],
				$retrofit['detail_id'], 
				$retrofit['build_option_category_id']
			);
			bo_perp_work_order::add_option_history($retrofit['work_order_id'], $retrofit['detail_id'], $retrofit['build_option_category_id'], 
				$old_option['build_option_id'], 
				$retrofit['build_option_id']
			);

			// Change work order option
			ExecMethod2('perp_manufacturing.so_perp_work_order.set_work_order_detail_option', 
				$retrofit['work_order_id'],
				$retrofit['detail_id'],
				$detail['stock_id'],
				$retrofit['build_option_category_id'],
				$retrofit['build_option_id']
			);
			$new_option = ExecMethod2('perp_manufacturing.so_perp_work_order.get_work_order_detail_option', 
				$retrofit['work_order_id'],
				$retrofit['detail_id'], 
				$retrofit['build_option_category_id']
			);

			// Update search string
			$GLOBALS['egw']->db->update(
				'perp_work_order_detail',
				array(
					'option_search'	=>	ExecMethod2('perp_manufacturing.so_perp_work_order.get_option_search_string', $retrofit['work_order_id'], $retrofit['detail_id'])
				),
				array(
					'work_order_id'	=>	$retrofit['work_order_id'],
					'detail_id'	=>	$retrofit['detail_id']
				),
				__LINE__, __FILE__, 'perp_manufacturing'
			);

			// The work order has already been completed, we may need to mess with inventory
			if($work_order['completed']) {
				/**
				* Check to see if this item has been shipped.
				* If it's not controlled, we can't tell, so hope we're not retrofitting something that
				* is not here.  Location_id < 0 are special 'not here' locations, -1 is shipped out.
				*/
				$query = array(
					'work_order_id'	=>	$retrofit['work_order_id'],
					'detail_id'	=>	$retrofit['detail_id'],
				);
				$serials = array();
				$bo_inventory = CreateObject('perp_inventory.bo_perp_inventory');
				$bo_inventory->get_controlled_inventory_list(
					$query,
					$serials
				);
				if(count($serials) == 0 || $serials[0]['location_id'] > 0) {

					// Put inventory for the old option back (BOM has already been adjusted)
					if($old_option['option_bom_id']) {
						$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $old_option['option_bom_id']);
						$components = $bom['details'];
						foreach($components as &$component) {
							$component['quantity'] *= -1;
							$component['stock_id'] = $component['component'];
						}
					} elseif ($old_option['component']) {
						$components = array(
							'stock_id'	=>	$old_option['component'],
							'quantity'	=>	-1
						);
					}
					if(count($components)) {
						$recovered_cost = ExecMethod2(
							'perp_manufacturing.bo_perp_manufacturing.add_components', 
							$retrofit['work_order_id'], 
							$retrofit['detail_id'], 
							$retrofit['work_center_id'], 
							time(), 
							$components
						);
						$components = array();
					}

					// Pull new inventory
					if($new_option['option_bom_id']) {
						$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $new_option['option_bom_id']);
						$components = $bom['details'];
						foreach($components as &$component) {
							$component['quantity'] *= -1;
							$component['stock_id'] = $component['component'];
						}
					} elseif ($new_option['component']) {
						$components = array(
							'stock_id'	=>	$new_option['component'],
							'quantity'	=>	-1
						);
					}
					if(count($components)) {
						$part_cost = ExecMethod2(
							'perp_manufacturing.bo_perp_manufacturing.add_components', 
							$retrofit['work_order_id'], 
							$retrofit['detail_id'], 
							$retrofit['work_center_id'], 
							time(), 
							$components
						);
					}

					// Change cost of work order
					$cost_change = bcadd($recovered_cost, bcadd($part_cost, $retrofit['labour_cost']));
					ExecMethod2(
						'perp_manufacturing.so_perp_manufacturing.update_cost', 
						$retrofit['work_order_id'], 
						$retrofit['detail_id'], 
						bcadd($recovered_cost, $part_cost), 
						$retrofit['labour_cost']
					);
					
					// Charge to parent location that the work order was made at.
					$location_id = $work_order['location'];

					// Get Labour account for the work center
					$work_center = ExecMethod('perp_manufacturing.bo_perp_work_center.get_work_center_location', $location_id);
					if($work_center['overhead_account']) {
						$labour_gl_account = $work_center['overhead_account'];
					} else {
						$labour_gl_account = bo_perp_stock::get_gl_accounts($retrofit['stock_id'], bo_perp_inventory::ADJUSTMENT);
					}

					// GL Transaction
					$trans = new GLTransaction();
					$trans->add_line(bo_perp_stock::get_gl_accounts($retrofit['stock_id'], bo_perp_inventory::ADJUSTMENT), $retrofit['comment'], -bcadd($recovered_cost, $part_cost));
					if($retrofit['labour_cost']) {
						$trans->add_line($labour_gl_account, lang('Labour Cost'), -($retrofit['labour_cost']));
					}
					$trans->add_line(bo_perp_stock::get_gl_accounts($retrofit['stock_id'], bo_perp_inventory::ON_HAND), $retrofit['comment'], $cost_change);
					$trans->write();
				}
			}

			$GLOBALS['egw']->db->transaction_commit();

			return $retrofit['stock_history_id'];
		}
	}
?>
