<?php

	class so_perp_work_order {

		private $debug = false;

		/**
		*	This cache is used to store the previous build option for a work order detail when
		*	changing the options, so we can tell if it actually changed.  It's used to reduce
		*	the number of stock reservation / unreservations when changing only one option.
		*
		*	Sometimes changing one option may enable or disable other options.  Because the 
		*	order of the options may not be in order of dependency (usually not), we need
		*	to check after changing the options to see if anything else was enabled or disabled.
		*	After the check, this cache will be cleared.
		*/
		private static $old_option_cache = array();

		/**
		*	Cache for work order bill of materials, we only want to reserve
		*	stock once, no matter how many option changes there are.
		*/
		private static $detail_bom_cache = null;

		public function so_perp_work_order() {
			if(!is_array(self::$detail_bom_cache)) {
				self::$detail_bom_cache =& $GLOBALS['egw']->session->appsession('detail_bom_cache', 'perp_warranty');
				
				if(!is_array(self::$detail_bom_cache)) {
					self::$detail_bom_cache = array();
					$GLOBALS['egw']->session->appsession('detail_bom_cache', 'perp_warranty', self::$detail_bom_cache);
				}
			}
		}

		public function get_work_order_list(&$query) {
			$where = array();
			$config = config::read('perp_manufacturing');

			$join = 'JOIN perp_work_order_detail ON
					perp_work_order_detail.work_order_id = perp_work_order.work_order_id
				LEFT JOIN perp_order_detail ON
					perp_order_detail.detail_value = perp_work_order.work_order_id AND
					perp_order_detail.detail_class = \'WorkOrderDetail\'
				LEFT JOIN perp_order ON
					perp_order.order_id = perp_order_detail.order_id
				LEFT JOIN perp_client ON
					perp_order.client_id = perp_client.client_id
				LEFT JOIN perp_stock ON 
					perp_stock.stock_id = perp_work_order_detail.stock_id ';
			if($config['use_workflow']) {
				$join .= 'LEFT JOIN egw_wf_instances ON
					egw_wf_instances.wf_name = CONCAT(perp_work_order.work_order_reference, \':\', perp_work_order_detail.detail_id) ';
			}

			if($query['location_id']) {
				$where['location'] = $query['location_id'];
			}
			if($query['stock_id']) {
				$where[] = 'perp_work_order_detail.stock_id = ' . (int)$query['stock_id'];
			}
			if($query['cat_id']) {
				$where[] = 'perp_stock.stock_category = ' . (int)$query['cat_id'];
			}

			// Restrict to a certain client
			if($query['client_id']) {
				if($query['client_id'] == 'unclaimed') {
					// Pass it off to unclaimed check, below
					$query['order_id'] = null;
				} else {
					$where[] = 'perp_order.client_id IN (' . implode(',', (array)$query['client_id']) . ')';
				}
			}

			// Use array_key_exists() for this one so we can search for un-claimed work orders
			if(array_key_exists('order_id', $query)) {
				if(!is_null($query['order_id'])) {
					$where[] = 'perp_order_detail.order_id IN (' . implode(',', (array)$query['order_id']) . ')';
				} else {
					$where[] = 'perp_order_detail.order_id IS NULL';
				}
			}
			if($query['status']) {
				switch ($query['status']) {
				case 'pending':
					$where[] = 'start_date IS NULL';
					break;
				case 'scheduled':
					$where[] = 'start_date IS NOT NULL';
					$where[] = '!completed';
					break;
				case 'pre-production':
					$where[] = '!completed';
					if($config['use_workflow'] && count($config['wf_processes']['production_start_activity']) > 0) {
						$where[] = 'start_date IS NULL';
						/**
						*	Check for workflow instances where the flow has not passed through
						*	the activity flagged as where production actually starts.
						*/
						foreach($config['wf_processes']['production_start_activity'] as $p_id => $act) {
							$wf_where[] = 'wf_p_id = ' . $p_id . ' AND egw_wf_instances.wf_instance_id NOT IN (
									SELECT wf_instance_id   
									FROM egw_wf_workitems
									WHERE wf_activity_id = ' . $act . '
									GROUP BY wf_instance_id
								)';
						$where[] = '( ' . implode(' OR ', $wf_where) . ')';
						}
					} else {
						$where[] = 'false';
					}
					break;
				case 'in-production':
					$where[] = 'start_date IS NOT NULL';
					$where[] = '!completed';
					if($config['use_workflow'] && count($config['wf_processes']['production_start_activity']) > 0) {
						/**
						*	Check for workflow instances where the flow has passed through
						*	the activity flagged as where production actually starts.
						*/
						foreach($config['wf_processes']['production_start_activity'] as $p_id => $act) {
							$filter = "(egw_wf_instances.wf_p_id = $p_id ";
							if($act) {
								$filter .= "AND egw_wf_workitems.wf_activity_id = $act";
							}
							$wf_where[] = $filter . ')';
						}
						$where[] = '( ' . implode(' OR ', $wf_where) . ')';
						$join .= 'JOIN egw_wf_workitems ON
								egw_wf_workitems.wf_instance_id = egw_wf_instances.wf_instance_id ';

					} else {
						$join .= 'JOIN (
							SELECT work_order_id, detail_id
							FROM perp_stock_history
							GROUP BY work_order_id, detail_id
						) AS perp_stock_history ON
							perp_stock_history.work_order_id = perp_work_order_detail.work_order_id AND
							perp_stock_history.detail_id = perp_work_order_detail.detail_id';
					}
					break;
				case 'completed':
					$where[] = '(completed || completed_date IS NOT NULL)';
					break;
				case 'temporary':
					$where[] = 'perp_work_order.work_order_id <= 0';
					break;
				default:
					if($config['use_workflow']) {

						// Filter by a workflow activity
						$where[] = 'egw_wf_instance_activities.wf_activity_id = ' . (int)$query['status'];
						$join .= 'JOIN egw_wf_instance_activities ON
								egw_wf_instances.wf_instance_id = egw_wf_instance_activities.wf_instance_id ';
					} else {
						// Filter by a work center
						// No, if you're not using workflow, it all happens at once.
					}
				}
			}

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_work_order.work_order_reference LIKE $search OR
					perp_order.our_reference_number LIKE $search OR
					perp_stock.stock_num LIKE $search)";
			}
			if(!$query['order']) {
				$query['order'] = 'work_order_reference';
			}
			if(!$query['sort']) {
				$query['sort'] = 'DESC';
			}

			$GLOBALS['egw']->db->select('perp_work_order', array(
				'count(perp_work_order.work_order_id) AS count'),
				$where, __LINE__, __FILE__, 
				false, '', 'perp_manufacturing', -1,
				$join
			);
			$row = $GLOBALS['egw']->db->row(True);
			$query['count'] = $row['count'];

			$GLOBALS['egw']->db->select('perp_work_order', array(
				'perp_work_order.work_order_id',
				'work_order_reference',
				'created_date',
				'start_date',
				'required_date',
				'completed_date',
				'location',
				'perp_order.order_id',
				'perp_client.client_name',
				'our_reference_number',
				'completed', 
				'sum(direct_cost + indirect_cost) AS cost'
				),
				$where,
				__LINE__, __FILE__,
				$query['start'], 'GROUP BY perp_work_order.work_order_id ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_manufacturing', $query['num_rows'], 
				$join
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$row['created_date'] = $GLOBALS['egw']->db->from_timestamp($row['created_date']);
				$row['start_date'] = $GLOBALS['egw']->db->from_timestamp($row['start_date']);
				$row['required_date'] = $GLOBALS['egw']->db->from_timestamp($row['required_date']);
				$row['completed_date'] = $GLOBALS['egw']->db->from_timestamp($row['completed_date']);
				$list[] = $row;
			}
			foreach($list as &$row) {
				$row['formatted_cost'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', false, $row['cost']);
			}
			return $list;
		}

		public function get_work_order($id) {
			$GLOBALS['egw']->db->select('perp_work_order', array(
				'work_order_id',
				'work_order_reference',
				'created_date',
				'start_date',
				'required_date',
				'completed_date',
				'sales_order',
				'our_reference_number',
				'location',
				'completed'),
				array('work_order_id' => $id),
				__LINE__, __FILE__,
				0, '', 'perp_manufacturing', 1,
				'LEFT JOIN perp_order_detail ON
					perp_order_detail.detail_value = perp_work_order.work_order_id AND
					perp_order_detail.detail_class = \'WorkOrderDetail\'
				LEFT JOIN perp_order ON
					perp_order.order_id = perp_order_detail.order_id'
			);
			$list = array();
			if($row = $GLOBALS['egw']->db->row(True)) {
				$row['created_date'] = $GLOBALS['egw']->db->from_timestamp($row['created_date']);
				$row['start_date'] = $GLOBALS['egw']->db->from_timestamp($row['start_date']);
				$row['required_date'] = $GLOBALS['egw']->db->from_timestamp($row['required_date']);
				$row['completed_date'] = $GLOBALS['egw']->db->from_timestamp($row['completed_date']);
				$list = $row;	// Only one
			}
			return $list;
		}

		public function set_work_order($id, $work_order) {
			$where = array();
			if($id) {
				$where['work_order_id'] = $id;
			}

			if(!$work_order['work_order_reference']) {
				$work_order['work_order_reference'] = ExecMethod('perp_manufacturing.bo_perp_work_order.get_reference');
			}
			$GLOBALS['egw']->db->insert('perp_work_order', array(
				'work_order_id'		=>	$work_order['work_order_id'],
				'work_order_reference'	=>	$work_order['work_order_reference'],
				'created_date'		=>	$GLOBALS['egw']->db->to_timestamp($work_order['created_date']),
				'start_date'		=>	($work_order['start_date'] ? $GLOBALS['egw']->db->to_timestamp($work_order['start_date']) : null),
				'required_date'		=>	($work_order['required_date'] ? $GLOBALS['egw']->db->to_timestamp($work_order['required_date']) : null),
				'completed_date'	=>	($work_order['completed_date'] ? $GLOBALS['egw']->db->to_timestamp($work_order['completed_date']) : null),
				'sales_order'		=>	$work_order['sales_order'],
				'location'		=>	$work_order['location'],
				'completed'		=>	$work_order['completed']),
				$where,
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			$id = $id ? $id : $GLOBALS['egw']->db->get_last_insert_id('perp_work_order', 'work_order_id');
			return $id;
		}

		public function delete_work_order($id) {
			$work_order = $this->get_work_order($id);
			$GLOBALS['egw']->db->select('perp_work_order_detail',
				array('detail_id'),
				array('work_order_id' => $id),
				__LINE__, __FILE__,
				false, '', 'perp_manufacturing'
			);
			$details = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$details[] = $row['detail_id'];
			}
			foreach($details as $detail) {
				// Temporary (id < 0) work orders are not scheduled, so no need to cancel them
				if($work_order['location'] && $work_order['work_order_id'] > 0) {
					ExecMethod2('perp_manufacturing.bo_perp_manufacturing.cancel_build', $work_order, $detail);
				}
				$this->delete_work_order_detail_bom($id, $detail);
				$this->delete_work_order_detail_option($id, $detail);
			}
			$GLOBALS['egw']->db->delete('perp_work_order',
				array('work_order_id' => $id),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			$GLOBALS['egw']->db->delete('perp_work_order_detail',
				array('work_order_id' => $id),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			return true;
		}

		public function get_work_order_detail_list(Array &$query) {
			$where = array('work_order_id' => $query['work_order_id']);

			if($query['status']) {
				$where[] = '(
	NOW() >= perp_bill_of_material.effective_after_date AND perp_bill_of_material.effective_to_date IS NULL OR
	NOW() BETWEEN perp_bill_of_material.effective_after_date AND perp_bill_of_material.effective_to_date
	)';
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_stock.stock_num  LIKE $search OR perp_stock.short_description LIKE $search)";
			}

			$query['count'] = $GLOBALS['egw']->db->query('SELECT count(detail_id) 
				FROM perp_work_order_detail WHERE ' . $GLOBALS['egw']->db->column_data_implode(' AND ', $where), 
				__LINE__, __FILE__
			)->fetchSingle();
			$GLOBALS['egw']->db->select('perp_work_order_detail', array(
				'work_order_id',
				'detail_id',
				'perp_stock.stock_id',
				'stock_num',
				'short_description',
				'quantity',
				'direct_cost',
				'indirect_cost'),
				$where,
				__LINE__, __FILE__,
				$query['start'], 'ORDER BY detail_id',
				'perp_manufacturing', $query['num_rows'],
				'JOIN perp_stock
					ON perp_stock.stock_id = perp_work_order_detail.stock_id'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$list[] = $row;
			} 
			return $list;
			foreach($list as &$row) {
				// Get sales orders
				$GLOBALS['egw']->db->select('perp_order',
					array(
						'our_reference_number',
						'perp_order.order_id'
					),
					array(
						'perp_order_detail.detail_value = ' . $row['work_order_id'],
						'perp_order_detail.detail_class = \'WorkOrderDetail\''
					), __LINE__, __FILE__, 0, '', 'perp_orders', -1,
					'LEFT JOIN perp_order_detail ON
						perp_order.order_id = perp_order_detail.order_id'
				);
				while($order = $GLOBALS['egw']->db->row(true)) {
					$row['orders'][] = $order;
				}
			}
			return $list;
		}

		public function get_by_serial_number($stock_id, $serial) {
			$GLOBALS['egw']->db->select('perp_stock_serial', array(
					'stock_id',
					'serial_number',
					'work_order_id',
					'quantity'
				),
				array(
					'stock_id'	=>	$stock_id,
					'serial_number'	=>	$serial
				),
				__LINE__, __FILE__,
				false, '', 'perp_manufacturing'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		public function get_serial_number($work_order_id, $stock_id) {
			$GLOBALS['egw']->db->select('perp_stock_serial', array(
					'stock_id',
					'serial_number',
					'work_order_id',
					'quantity'
				),
				array(
					'stock_id'	=>	$stock_id,
					'work_order_id'	=>	$work_order_id,
					'serial_number != \'\''
				),
				__LINE__, __FILE__,
				false, '', 'perp_manufacturing'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		/*
		* Get a work order that's been scheduled but not completed or sold
		*  If provided, match_to gives an option work order to search against
		*  Returns the ID and detail_no of work orders that match.
		*/
		public function get_unclaimed_work_order($stock_id, $location = '', $required_date = '', $match_to = '') {
			$where = array();
			$where[] = 'perp_work_order.work_order_id > 0';
			$where[] = 'perp_work_order_detail.stock_id = ' . $stock_id;
			$where[] = '(perp_work_order_detail.quantity - IF(perp_sales_order_work_order.quantity IS NULL, 0, perp_sales_order_work_order.quantity)) > 0';

			if($location) {
				$where[] = 'location = ' . $location;
			}

			if($match_to) {
				/* Find a product in inventory with the same options, 
				* or one that hasn't started production
				* We use REGEXP instead of some fancy SQL because it's much easier to understand,
				* and REGEXP instead of LIKE to handle options not existing or set after work order was created
				*/
				$where[] = "option_search REGEXP '$match_to'";
			}
			if($required_date) {
				$date = $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($required_date));
				$where[] = "(( completed_date IS NULL AND required_date <= $date ) OR completed_date <= $date) ";
			}

			$sql = 'SELECT 
				perp_work_order.work_order_id,
				perp_work_order_detail.detail_id,
				(perp_work_order_detail.quantity - IF(perp_sales_order_work_order.quantity IS NULL, 0, perp_sales_order_work_order.quantity)) AS quantity,
				serial_number 
			FROM perp_work_order 
			JOIN perp_work_order_detail ON 
				perp_work_order_detail.work_order_id = perp_work_order.work_order_id 
			LEFT JOIN (
				SELECT sum(quantity - quantity_returned) AS quantity,
				detail_value AS work_order_id,
				work_order_detail_id AS detail_no
				FROM perp_sales_order_work_order 
				JOIN perp_order_detail ON
					perp_order_detail.order_id = perp_sales_order_work_order.order_id AND
					perp_order_detail.line_no = perp_sales_order_work_order.line_no AND
					perp_order_detail.detail_no = perp_sales_order_work_order.detail_no
				GROUP BY work_order_id, detail_no
			) AS perp_sales_order_work_order ON 
				perp_sales_order_work_order.work_order_id = perp_work_order_detail.work_order_id AND 
				perp_sales_order_work_order.detail_no = perp_work_order_detail.detail_id 
			LEFT JOIN perp_stock_serial ON 
				perp_stock_serial.work_order_id = perp_work_order.work_order_id AND 
				perp_stock_serial.stock_id = perp_work_order_detail.stock_id 
			WHERE ';
			$sql .= implode(' AND ', $where);
			$sql .= ' ORDER BY start_date, completed_date, required_date, work_order_id ASC';

			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);

			$wo = array();
			$wo['total'] = 0;
			while($row = $GLOBALS['egw']->db->row(true)) {
				$wo['total'] += $row['quantity'];
				$wo[] = $row;
			}
			return $wo;
		}

		public function get_option_search_string($work_order_id, $detail_id) {
			$search = array();

			$options = $this->get_work_order_detail_option($work_order_id, $detail_id);
			foreach($options as $option) {
				$search[$option['option_category_id']] = null;
			}
			ksort($search);
			$where = array(
				'work_order_id'	=>	$work_order_id,
				'detail_id'	=>	$detail_id,
			);
			if(count($search) > 0) {
				$where['option_category_id'] = array_keys($search);
			}

			$GLOBALS['egw']->db->select('perp_work_order_option',
				array(
					'option_category_id AS build_option_category',
					'perp_work_order_option.build_option_id AS value',
				),
				$where,
				__LINE__, __FILE__, false, 
				'', 'perp_manufacturing', -1, ''
			);
			$string = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				if($row['value']) {
					$string[] = $row['build_option_category'] . ':' . $row['value'] . ($work_order_id < 0 ? ';?' : ';');
				} else {
					$string[] = '(' . $row['build_option_category'] . ':[0-9]+)?.?';
				}
			}
			return implode('', $string);
		}

		public function get_work_order_option_list($wo_id, $detail_id) {
			$where = array(
				'work_order_id'	=>	$wo_id,
				'detail_id'	=>	$detail_id,
			//	'value < 0'
			);

			$GLOBALS['egw']->db->select('perp_work_order_option', array(
				'perp_work_order_option.work_order_id',
				'perp_work_order_option.detail_id',
				'perp_work_order_option.build_option_id',
				'perp_work_order_option.option_category_id',
				'perp_work_order_option.component'),
				$where,
				__LINE__, __FILE__, 
				False, 'ORDER BY option_category_id, build_option_id',
				'perp_manufacturing', 0, ''
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function get_work_order_detail($work_order_id, $detail_id, $for_edit = false) {
			$GLOBALS['egw']->db->select('perp_work_order_detail', array(
					'perp_work_order_detail.work_order_id',
					'perp_work_order_detail.detail_id',
					'perp_work_order_detail.stock_id',
					'perp_work_order_detail.quantity',
					'perp_work_order_detail.direct_cost',
					'perp_work_order_detail.indirect_cost',
					'option_search',
					'perp_work_order_bill_of_material.bill_of_material_id'
				),
				array(
					'perp_work_order_detail.work_order_id = ' . $work_order_id,
					'perp_work_order_detail.detail_id = ' . $detail_id
				),
				__LINE__, __FILE__,
				false, '', 'perp_manufacturing', -1,
				'JOIN perp_bill_of_material ON
					perp_bill_of_material.stock_id = perp_work_order_detail.stock_id
				JOIN perp_work_order_bill_of_material ON
					perp_bill_of_material.bill_of_material_id = perp_bill_of_material.bill_of_material_id AND
					perp_work_order_bill_of_material.work_order_id = perp_work_order_detail.work_order_id AND
					perp_work_order_bill_of_material.detail_id = perp_work_order_detail.detail_id'
			);
			$list = array();
			if($row = $GLOBALS['egw']->db->row(True)) {
				$list = $row;
			}

			if($work_order_id > 0 && $for_edit && !is_array(self::$detail_bom_cache[$work_order_id][$detail_id])) {
				// Record the current BOM in the cache, so we know if it changes
				$query = array(
					'work_order_id'	=>	$work_order_id,
					'detail_id'	=>	$detail_id,
					'num_rows'	=>	-1,
					'order'		=>	'stock_id'
				);
				self::$detail_bom_cache[$work_order_id][$detail_id] = $this->get_work_order_detail_bom($query);
			}
			return $list;
		}

		public function set_work_order_detail($work_order, $detail_id, $detail, $un_reserve_stock) {
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $detail['stock_id']);
			if($stock['controlled']) {
				if($stock['serialized'] && $detail['quantity'] > 1) {
					$detail['quantity'] = 1;
				}
			}

			$GLOBALS['egw']->db->transaction_begin();

			// Clear old bill of materials
			$this->delete_work_order_detail_bom($work_order['work_order_id'], $detail_id);

			// Set detail
			if($detail['quantity'] == 0) {
				$GLOBALS['egw']->db->delete('perp_work_order_detail',
					array(
						'work_order_id'	=>	$work_order['work_order_id'],
						'detail_id'	=>	$detail_id
					),
					__LINE__, __FILE__, 'perp_manufacturing'
				);
				// Clear options
				$this->delete_work_order_detail_option($work_order['work_order_id'], $detail_id);

				$GLOBALS['egw']->db->transaction_commit();
				return;
			}

			$GLOBALS['egw']->db->insert('perp_work_order_detail', array(
					'work_order_id'	=>	$work_order['work_order_id'],
					'detail_id'	=>	$detail_id,
					'stock_id'	=>	$detail['stock_id'],
					'quantity'	=>	$detail['quantity'],
					'direct_cost'		=>	$detail['direct_cost'],
					'indirect_cost'	=>	$detail['indirect_cost'],
					'option_search'	=>	$this->get_option_search_string($work_order['work_order_id'], $detail_id)
				), array(
					'work_order_id'	=>	$work_order['work_order_id'],
					'detail_id'	=>	$detail_id,
				),
				__LINE__, __FILE__, 'perp_manufacturing'
			);

			// Get new bill of materials
			$new_bom = ExecMethod2('perp_inventory.bo_perp_stock.get_current_bom', $detail['stock_id'], $work_order['created_date']);

			if($new_bom) {
				$this->set_work_order_detail_bom($work_order['work_order_id'], $detail_id, $detail['quantity'], $new_bom);
			}

			// Make sure default options are set
			$query = array(
				'work_order_id'		=>	$work_order['work_order_id'],
				'detail_id'		=>	$detail_id,
				'stock_id'		=>	$detail['stock_id'],
				'effective_date'	=>	$work_order['created_date'],
				'no_status'		=>	true,
				'no_readonlys'		=>	true,
			);

			$bo = new bo_perp_work_order();
			$bo->get_work_order_detail_option_list($query, $allowed);
			unset(self::$old_option_cache[$work_order_id][$detail_id]);
			foreach($allowed as $key => $row) {
				$bo->set_work_order_detail_option(
					$work_order['work_order_id'], $detail_id,
					$row['stock_id'], 
					$row['build_option_category_id'], 
					$row['value']
				);
			}

			$this->flush_options();


			// Update reserved stock
			if($work_order['work_order_id'] > 0) {
				// Get the current BOM
				$query = array(
					'work_order_id'	=>	$work_order['work_order_id'],
					'detail_id'	=>	$detail_id,
					'num_rows'	=>	-1,
					'order'		=>	'stock_id'
				);
				$current_bom = $this->get_work_order_detail_bom($query);

				if(self::$detail_bom_cache[$work_order['work_order_id']][$detail_id]) {
					$removed_items = self::bom_diff(self::$detail_bom_cache[$work_order['work_order_id']][$detail_id], $current_bom, -1);
					$added_items = self::bom_diff($current_bom, self::$detail_bom_cache[$work_order['work_order_id']][$detail_id]);
				} else {
					$added_items = $current_bom;
					$removed_items = array();
				}

				// Update stock reservation
				ExecMethod2('perp_manufacturing.bo_perp_manufacturing.reserve_stock', $work_order, $detail_id, $removed_items);
				ExecMethod2('perp_manufacturing.bo_perp_manufacturing.reserve_stock', $work_order, $detail_id, $added_items);

				unset(self::$detail_bom_cache[$work_order['work_order_id']][$detail_id]);
			}

			$GLOBALS['egw']->db->transaction_commit();
		}

		/**
		*	Get the different items between two bills of materials
		*/
		public static function bom_diff($bom_1, $bom_2, $direction = 1) {
			$result = array();
			$different_quantities = array();
			
			foreach($bom_1 as $item_1) {
				foreach($bom_2 as $item_2) {
					if($item_1['component'] == $item_2['component'] && 
						$item_1['quantity'] == $item_2['quantity'] &&
						$item_1['work_center_id'] == $item_2['work_center_id'] &&
						$item_1['bill_of_material_detail_id'] == $item_2['bill_of_material_detail_id']
					) {
						// Exactly the same
						continue 2;
					} elseif($item_1['component'] == $item_2['component'] && 
						$item_1['work_center_id'] == $item_2['work_center_id'] &&
						$item_1['bill_of_material_detail_id'] == $item_2['bill_of_material_detail_id']
					) {
						// Different quantity
						if($item_1['quantity'] > $item_2['quantity']) {
							$item_1['quantity'] -= $item_2['quantity'] * $direction;
							$result[] = $item_1;
						}
						continue 2;
					}
				}
				// Didn't find item_1 in bom_2
				$item_1['quantity'] *= $direction;
				$result[] = $item_1;
			}
			return $result;
		}

		public static function get_work_order_detail_option($work_order_id, $detail_id, $category = '', $stock_id = '') {
			$where = array(
				'work_order_id'		=>	$work_order_id,
				'detail_id'		=>	$detail_id,
			);
			if($category) {
				$where['option_category_id'] = $category;
			}
			if($stock_id) {
				$where['stock_id'] = $stock_id;
			}

			$result =& $GLOBALS['egw']->db->select(
				'perp_work_order_option', 
				array(
					'work_order_id',
					'detail_id',
					'option_category_id',
					'perp_work_order_option.build_option_id',
					'component',
					'work_center_id',
					'option_bom_id'
				),
				$where,
				__LINE__, __FILE__, false, 
				'', 'perp_manufacturing'
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			if(count($list) == 1) {
				return $list[0];
			}
			return $list;
		}
	
		public function set_work_order_detail_option($work_order_id, $detail_id, $stock_id, $category, $option_id) {
			if($this->debug) {
				echo "set_work_order_detail_option($work_order_id, $detail_id, $stock_id, $category, $option_id)<br />";
			}
			if(!$work_order_id || !$detail_id || !$category || (!$option_id && $work_order_id > 0)) {
				throw new egw_exception("Missing required parameter");
				return;
			}
			$where = array(
				'work_order_id'		=>	$work_order_id,
				'detail_id'		=>	$detail_id,
				'option_category_id'	=>	$category
			);

			// We need the work order's creation date so we know when to take the option from
			$work_order = $this->get_work_order($work_order_id);

			$option_data = ExecMethod2('perp_manufacturing.bo_perp_build_option.get_allowed_build_option', $stock_id, $option_id, $work_order['created_date']);

			$option = $this->get_work_order_detail_option($work_order_id, $detail_id, $category);

			if(self::$old_option_cache[$work_order_id][$detail_id][$category] == $option_data) {
				//unchanged
				if($this->debug) {
					echo "Unchanged, skipping<br />";
				}
				return true;
			}
			
			self::$old_option_cache[$work_order_id][$detail_id][$category] = $option_data;
			if($option_data) {
				self::$old_option_cache[$work_order_id][$detail_id][$category]['value'] = $option_id;
			}

			$GLOBALS['egw']->db->transaction_begin();

			$this->delete_work_order_detail_option($work_order_id, $detail_id, $category);

			if($option_data['component']) {
				$component = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $option_data['component']);
				if(isset($component['stock_id']) && $component['stock_type_flag'] != 'R') {
					$bom_id = ExecMethod2('perp_inventory.bo_perp_stock.get_current_bom', $component['stock_id'], $work_order['created_date']);
				}
			} else {
				$option_data['component'] = null;
			}
			$GLOBALS['egw']->db->insert('perp_work_order_option', array(
					'work_order_id'	=>	$work_order_id,
					'detail_id'	=>	$detail_id,
					'build_option_id'	=>	$option_id,
					'option_category_id'	=>	$category,
					'work_center_id'	=>	$option_data['work_center'],
					'option_bom_id'		=>	$bom_id,
					'component'	=>	$option_data['component']
				),
				$where,
				__LINE__, __FILE__, 'perp_manufacturing'
			);

			// Record that we use this BOM in the BOM list
			if($bom_id) {
				$this->set_work_order_detail_bom($work_order_id, $detail_id, 1, $bom_id);
			}
			$GLOBALS['egw']->db->transaction_commit();

			return true;
		}

		/**
		*	Sometimes changing one option enables or disables another.  This is a check to make sure
		*	there's no leftovers that are set, or newly enabled options that aren't set.
		*/
		public function flush_options() {
			if($this->debug) {
				echo __METHOD__;
				echo '<br />Option cache:'; _debug_array(self::$old_option_cache);
			}

			$GLOBALS['egw']->db->transaction_begin();
			
			$bo = new bo_perp_work_order();

			foreach(self::$old_option_cache as $work_order_id => &$details) {

				// We need the work order's creation date so we know when to take the option from
				$work_order = $this->get_work_order($work_order_id);

				foreach($details as $detail_id => &$old_categories) {
					// We need the detail so we know which stock item is being made
					$detail = $this->get_work_order_detail($work_order_id, $detail_id);

					$query = array(
						'work_order_id'		=>	$work_order_id,
						'work_order'		=>	$work_order,
						'detail_id'		=>	$detail_id,
						'stock_id'		=>	$detail['stock_id'],
						'effective_date'	=>	$work_order['created_date'],
						'no_status'		=>	true,
						'no_readonlys'		=>	true,
					);
					$bo->get_work_order_detail_option_list($query, $allowed);

					if($this->debug) {
						echo 'Allowed options:';
						_debug_array($allowed);
					}
					foreach($allowed as $key => $option_category) {
						if($old_categories[$option_category['build_option_category_id']]) {
							// Option set, OK
						} else {
							// New option
							$this->set_work_order_detail_option(
								$work_order_id, $detail_id, 
								$option_category['stock_id'], 
								$option_category['build_option_category_id'], 
								$option_category['value']
							);
						}
						$this->flush_option($work_order_id, $detail_id, $stock_id, $category);
						unset($old_categories[$option_category['build_option_category_id']]);
					}

					// Get rid of leftovers that were disallowed by an option change
					foreach($old_categories as $category_id => $option) {
						$this->delete_work_order_detail_option($work_order_id, $detail_id, $category_id);
						unset($old_categories[$category_id]);
					}

					if(count($old_categories) == 0) {
						unset($details[$detail_id]);
					}
				}
				if(count($details) == 0) {
					unset(self::$old_option_cache[$work_order_id]);
				}
			}

			$GLOBALS['egw']->db->transaction_commit();
		}

		private function flush_option($work_order_id, $detail_id, $category) {
			// Do we need to reserve stock?
			$reserve_stock = false;
			if($work_order_id > 0) {
				$work_order = $this->get_work_order($work_order_id);
				$reserve_stock = ($work_order['location']);
			}

			// Update search string
			$GLOBALS['egw']->db->update(
				'perp_work_order_detail',
				array(
					'option_search'	=>	$this->get_option_search_string($work_order_id, $detail_id)
				),
				array(
					'work_order_id'		=>	$work_order_id,
					'detail_id'		=>	$detail_id,
				),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			$GLOBALS['egw']->db->transaction_commit();

			return true;
		}

		public function delete_work_order_detail_option($work_order_id, $detail_id, $category = '') {
			if($this->debug) {
				echo "delete_work_order_detail_option($work_order_id, $detail_id, $category)<br />";
			}

			$GLOBALS['egw']->db->transaction_begin();

			$option = $this->get_work_order_detail_option($work_order_id, $detail_id, $category);
			if($option['option_bom_id']) {
				$this->delete_work_order_detail_bom($work_order_id, $detail_id, false, $option['option_bom_id']);

				// Remove any sub-components
				$work_order = $this->get_work_order($work_order_id);
				$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $option['option_bom_id']);
				foreach($bom['details'] as $detail) {
					if($detail['stock_type_flag'] != 'R' && $detail['pseudo']) {
						$component_bom = ExecMethod2('perp_inventory.bo_perp_stock.get_current_bom', $detail['component'], $work_order['created_date']);
						if($component_bom) {
							$this->delete_work_order_detail_bom($work_order_id, $detail_id, false, $component_bom);
						}
					}
				}
			}

			$where = array(
				'work_order_id'	=>	$work_order_id,
				'detail_id'	=>	$detail_id,
			);
			if($category) {
				$where[] = 'option_category_id = ' . (int)$category;
			}
			$GLOBALS['egw']->db->delete('perp_work_order_option',
				$where, __LINE__, __FILE__, 'perp_manufacturing'
			);
			
			$GLOBALS['egw']->db->transaction_commit();
		}

		/**
		*	Actual list of components used in for a work order detail
		*
		*	This is determined by:
		*	1. Components in bills of materials in work_order_bill_of_material
		*	2. Changes to the above specified in work_order_bom_change
		*	3. Single components included by options (pseudos included above)
		*	4. Changes (additions) to the BOM in work_order_bom_change
		*
		*	@param work_order_id
		*	@param detail_id
		*	@param build_option_category_id (optional)
		*
		*	@return array of components, quantities and work centers they are added in
		*/
		public function get_work_order_detail_bom(&$query) {
			$where = array();
			if($query['category']) {
				$where[] = 'stock_category = ' . (int)$query['category'];
			}
			if($query['cat_id']) {
				$where[] = 'stock_category IN (' . implode(',', (array)$query['cat_id']) . ')';
			}
			if(isset($query['controlled'])) {
				if(is_numeric($query['controlled'])) {
					$where[] = 'controlled = ' . (int)$query['controlled'];
				} elseif (is_array($query['controlled'])) {
					$where[] = 'controlled IN (' . implode(',', $query['controlled']) . ')';
				}
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(stock_num LIKE $search OR short_description LIKE $search)";
			}

			$selects = array(
				// Base BOMs + sub assemblies
				'base'	=>	array(
					'table'		=>	'perp_work_order_bill_of_material',
					'cols'		=>	array(
						'perp_bill_of_material_detail.component',
						'perp_bill_of_material_detail.work_center AS work_center_id',
						'SUM(perp_bill_of_material_detail.quantity + IF(delta_quantity IS NULL, 0, delta_quantity)) * perp_work_order_bill_of_material.quantity AS quantity',
						'perp_bill_of_material_detail.bill_of_material_detail_id',
						'perp_bill_of_material_detail.bill_of_material_id',
						'perp_stock.stock_id',
						'stock_num',
						'short_description',
						'controlled',
						'stock_type_flag',
						'pseudo',
						'unit_of_measure',
					),
					'where'		=>	array_merge($where, array(
						'perp_work_order_bill_of_material.work_order_id = ' . (int)$query['work_order_id'],
						'perp_work_order_bill_of_material.detail_id = ' . (int)$query['detail_id'],
						'perp_stock.pseudo = false'
					)),
					'append'	=>	'GROUP BY bill_of_material_detail_id',
					'app'		=>	'perp_manufacturing',
					'join'		=>	'JOIN (
						perp_bill_of_material_detail LEFT JOIN
							perp_work_order_bom_change ON
								perp_bill_of_material_detail.bill_of_material_detail_id = perp_work_order_bom_change.bill_of_material_detail_id
						) ON
							perp_work_order_bill_of_material.bill_of_material_id = perp_bill_of_material_detail.bill_of_material_id
						JOIN perp_stock ON
							perp_bill_of_material_detail.component = perp_stock.stock_id'
				),
				// Options - single component
				'options-single'	=>	array(
					'table'		=>	'perp_work_order_option',
					'cols'		=>	array(
						'component',
						'work_center_id',
						'1 AS quantity',
						'NULL AS bill_of_material_detail_id',
						'NULL AS bill_of_material_id',
						'perp_stock.stock_id',
						'stock_num',
						'short_description',
						'controlled',
						'stock_type_flag',
						'pseudo',
						'unit_of_measure',
					),
					'where'		=>	array_merge($where, array(
						'perp_work_order_option.work_order_id = ' . (int)$query['work_order_id'],
						'perp_work_order_option.detail_id = ' . (int)$query['detail_id'],
						'component IS NOT NULL',
						'option_bom_id IS NULL'
					)),
					'append'	=>	'',
					'app'		=>	'perp_manufacturing',
					'join'		=>	'JOIN perp_stock ON
							component = perp_stock.stock_id'
				),

				// Options - sub-assembly
				'options-sub'	=>	array(
					'table'		=>	'perp_work_order_option',
					'cols'		=>	array(
						'perp_bill_of_material_detail.component',
						'perp_bill_of_material_detail.work_center AS work_center_id',
						'SUM(perp_bill_of_material_detail.quantity + IF(delta_quantity IS NULL, 0, delta_quantity)) AS quantity',
						'perp_bill_of_material_detail.bill_of_material_detail_id',
						'perp_bill_of_material_detail.bill_of_material_id',
						'perp_stock.stock_id',
						'stock_num',
						'short_description',
						'controlled',
						'stock_type_flag',
						'pseudo',
						'unit_of_measure',
					),
					'where'		=>	array_merge($where, array(
						'perp_work_order_option.work_order_id = ' . (int)$query['work_order_id'],
						'perp_work_order_option.detail_id = ' . (int)$query['detail_id'],
						'option_bom_id IS NOT NULL',
						'perp_stock.pseudo = false'
					)),
					'append'	=>	'GROUP BY bill_of_material_detail_id',
					'app'		=>	'perp_manufacturing',
					'join'		=>	'JOIN (
						perp_bill_of_material_detail LEFT JOIN
							perp_work_order_bom_change ON
								perp_bill_of_material_detail.bill_of_material_detail_id = perp_work_order_bom_change.bill_of_material_detail_id
						)ON
							perp_work_order_option.option_bom_id = perp_bill_of_material_detail.bill_of_material_id
						JOIN perp_stock ON
							perp_bill_of_material_detail.component = perp_stock.stock_id'
				),

				// Custom changes
				'changes'	=>	array(
					'table'		=>	'perp_work_order_bom_change',
					'cols'		=>	array(
						'component',
						'work_center_id',
						'delta_quantity AS quantity',
						'NULL AS bill_of_material_detail_id',
						'NULL AS bill_of_material_id',
						'perp_stock.stock_id',
						'stock_num',
						'short_description',
						'controlled',
						'stock_type_flag',
						'pseudo',
						'unit_of_measure',
					),
					'where'		=>	array_merge($where, array(
						'perp_work_order_bom_change.work_order_id = ' . (int)$query['work_order_id'],
						'perp_work_order_bom_change.detail_id = ' . (int)$query['detail_id'],
						'bill_of_material_detail_id'	=>	0
					)),
					'append'	=>	'',
					'app'		=>	'perp_manufacturing',
					'join'		=>	'JOIN perp_stock ON
							component = perp_stock.stock_id'
				)
			);
			if(array_key_exists('work_center_id', $query)) {
				$selects['base']['where'][] = 'perp_bill_of_material_detail.work_center = ' . (int)$query['work_center_id'];
				$selects['options-single']['where'][] = 'perp_work_order_option.work_center_id = ' . (int)$query['work_center_id'];
				$selects['options-sub']['where'][] = 'perp_bill_of_material_detail.work_center = ' . (int)$query['work_center_id'];
				$selects['changes']['where'][] = 'perp_work_order_bom_change.work_center_id = ' . (int)$query['work_center_id'];
			}
			if(isset($query['build_option_category_id'])) {
				unset($selects['base']);
				$selects['options-single']['where'][] = 'perp_work_order_option.option_category_id = ' . (int)$query['build_option_category_id'];
				$selects['options-sub']['where'][] = 'perp_work_order_option.option_category_id = ' . (int)$query['build_option_category_id'];
			}

			// Get number of rows
			$query['count'] = $GLOBALS['egw']->db->union($selects, __LINE__, __FILE__)->NumRows();

			$list = array();
			foreach($GLOBALS['egw']->db->union($selects, __LINE__, __FILE__, 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				$query['start'], $query['num_rows']
			) as $row) {
				if($row['controlled'] > 1) {
					$quantity = $row['quantity'];
					for($count = 0; $count < $quantity; $count++) {
						$row['quantity'] = 1;
						$list[] = $row;
					}
				} else {
					$list[] = $row;
				}
			}
			return $list;
		}


		/**
		*	Set the Bill of Materials for this work order item
		*	Records the bill of materials ID for the detail stock item, and recurses through sub-assemblies and options
		*	recording the bill of materials IDs for any pseudo assemblies and option assemblies.
		*
		*	@param	int	$work_order_id - the id of the work order
		*	@param	int	$detail_id	The id of the detail line on the work order
		*	@param	int	$bom_id		The id of the bill of materials to use (optional)
		*	@param	int	$stock_id	The id of the stock_item to build (optional)
		*/
		public function set_work_order_detail_bom($work_order_id, $detail_id, $quantity, $bom_id = '', $stock_id = '') {
			if($this->debug) {
				echo "set_work_order_detail_bom($work_order_id, $detail_id, $quantity, $bom_id, $stock_id)<br />";
			}
			if(!$work_order_id || $quantity == 0) {
				return;
			};

			if(!$bom_id && !$stock_id) {
				$detail = $this->get_work_order_detail($work_order_id, $detail_id);
				$stock_id = $detail['stock_id'];
			}
				
			if($stock_id) {
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			}

			// Bill of Material
			if(!$bom_id && $stock_id) {
				// Get bom_id from stock_id
				$bom_id = ExecMethod2('perp_inventory.bo_perp_stock.get_current_bom', $stock_id, $work_order['created_date']);
			}
			$work_order = $this->get_work_order($work_order_id);

			if(!$bom_id) {
				throw new egw_exception('Need a BOM ID');
			}
			// Check for (sub)BOM already used somewhere, we'll add to the quantity
			$bom_quantity = $quantity;
			$result = $GLOBALS['egw']->db->select('perp_work_order_bill_of_material',
				array(
					'quantity'
				),
				array(
					'work_order_id'		=>	$work_order_id,
					'detail_id'		=>	$detail_id,
					'bill_of_material_id'	=>	$bom_id
				),
				__LINE__, __FILE__, false, '', 'perp_manufacturing'
			);
			foreach($result as $row) {
				$bom_quantity += $row['quantity'];
			}
				
			$GLOBALS['egw']->db->insert('perp_work_order_bill_of_material',
				array(
					'work_order_id'		=>	$work_order_id,
					'detail_id'		=>	$detail_id,
					'quantity'		=>	$bom_quantity,
					'bill_of_material_id'	=>	$bom_id
				),
				array(
					'work_order_id'		=>	$work_order_id,
					'detail_id'		=>	$detail_id,
					'bill_of_material_id'	=>	$bom_id
				),
				__LINE__, __FILE__, 'perp_manufacturing'
			);

			$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $bom_id);
			foreach($bom['details'] as $detail) {
				if($detail['stock_type_flag'] != 'R' && $detail['pseudo']) {
					$component_bom = ExecMethod2('perp_inventory.bo_perp_stock.get_current_bom', $detail['component'], $work_order['created_date']);
					if($component_bom) {
						$this->set_work_order_detail_bom($work_order_id, $detail_id, $quantity * $detail['quantity'], $component_bom);
					}
				}
			}

			if($stock['stock_type_flag'] == 'R') {
				return;
			}
		}

		public function delete_work_order_detail_bom($work_order_id, $detail_id, $component = '', $bom_id = '') {
			if($this->debug) {
				echo "delete_work_order_detail_bom($work_order_id, $detail_id, $bom_id)<br />";
			}
			if($component != '') {
				throw new Exception('Component parameter is deprecated and should be removed');
			}
			$where = array(
				'work_order_id'	=>	$work_order_id,
				'detail_id'	=>	$detail_id
			);
			if($component) {
				$where['component'] = $component;
			}
			if($bom_id) {
				$where['bill_of_material_id'] = $bom_id;
			}

			$GLOBALS['egw']->db->delete('perp_work_order_bill_of_material',
				$where,
				__LINE__, __FILE__, 'perp_manufacturing'
			);
		}

		/**
		*	Change a component of a work order's bill of materials
		*/
		public function change_bill_of_materials($data) {
			$where = $data;
			unset($where['delta_quantity']);

			// Check if this change will reset the BOM
			$change_query = $where + array(
				'order'	=>	'component'
			);
			$changes = $this->get_bill_of_material_changes($change_query);
			if(count($changes) > 0) {
				$net = 0;
				foreach($changes as $change) {
					$net += $change['delta_quantity'];
				}
				if($net + $data['delta_quantity'] == 0) {
					$GLOBALS['egw']->db->delete(
						'perp_work_order_bom_change',
						$where,
						__LINE__, __FILE__, 
						'perp_manufacturing'
					);
					return true;
				}
				// Take into account other changes
				$data['delta_quantity'] += $net;
			}

			$GLOBALS['egw']->db->insert(
				'perp_work_order_bom_change',
				$data,
				$where,
				__LINE__, __FILE__, 
				'perp_manufacturing'
			);

			return true;
		}

		/**
		*	Get a list of changes to a work order's BOM
		*/
		public function get_bill_of_material_changes(Array &$query) {
			$where = array(
				'work_order_id'	=>	(int)$query['work_order_id'],
				'detail_id'	=>	(int)$query['detail_id'],
			);
			if($query['component']) {
				$where['component'] = (int)$query['component'];
			}
			if(isset($query['work_center_id'])) {
				$where['work_center_id'] = $query['work_center_id'];
			}
			if(isset($query['bill_of_material_detail_id'])) {
				$where['bill_of_material_detail_id'] = (int)$query['bill_of_material_detail_id'];
			}

			// Get count
			$query['count'] = $GLOBALS['egw']->db->select('perp_work_order_bom_change', 
				array('count(*)'), $where, __LINE__, __FILE__, 0, '', 'perp_manufacturing'
			)->fetchSingle();

			$result = $GLOBALS['egw']->db->select(
				'perp_work_order_bom_change',
				array('*'),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 'perp_manufacturing',
				$query['num_rows'],
				''
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			return $list;
		}


		public function get_next_id() {
			$sql = 'SHOW TABLE STATUS LIKE \'perp_work_order\'';
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
			if($row = $GLOBALS['egw']->db->row(True)) {
				$id = $row['Auto_increment'];
			}
			return $id;
		}

		/**
		*	Record a change in a work order detail's build option.
		*
		*	We only care about changes if the work order's status is not in
		*	bo_perp_work_order::$no_history_statuses.
		*
		*	@param work_order_id
		*	@param detail_id
		*	@param build_option_category_id
		*	@param old_option The build_option_id of the current build option
		*	@param new_option The build_option_id of the new build option
		*/
		public static function add_option_history($work_order_id, $detail_id, $build_option_category_id, $old_option, $new_option) {
			// We have to use our own table, because category_id is an integer, and egw's history limits us to varchar(2)
			if($old_option == $new_option) {
				return true;
			}
			
			$GLOBALS['egw']->db->insert(
				'perp_work_order_option_history',
				array(
					'work_order_id'			=>	(int)$work_order_id,
					'detail_id'			=>	(int)$detail_id,
					'build_option_category_id'	=>	(int)$build_option_category_id,
					'old_option'			=>	(int)$old_option,
					'new_option'			=>	(int)$new_option,
					'timestamp'			=>	$GLOBALS['egw']->db->to_timestamp(time()),
					'changed_by'			=>	$GLOBALS['egw_info']['user']['account_id']
				),
				array(),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
			return true;
		}

		/**
		*	Get a list of changes to a work order detail's build options
		*
		*/
		public static function get_option_history_list($work_order_id, $detail_id, $build_option_category_id = '') {
			$where = array(
				'perp_work_order_option_history.work_order_id = ' . (int)$work_order_id,
				'perp_work_order_option_history.detail_id = ' . (int)$detail_id
			);

			if($build_option_category_id) {
				$where['perp_work_order_option_history.build_option_category_id'] = $build_option_category_id;
			}

			$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $work_order_id);
			$manufactured_option = array();
			$list = array();
			foreach($GLOBALS['egw']->db->select('perp_work_order_option_history', '*', $where, __LINE__, __FILE__, false, '', 'perp_manufacturing') as $row) {
				$row['timestamp'] = $GLOBALS['egw']->db->from_timestamp($row['timestamp']);
				if($work_order['completed_date'] && $work_order['completed_date'] >= $row['timestamp']) {
					$manufactured_option[$row['build_option_category_id']] = count($list);
				}
				$list[] = $row;
			}
			foreach($manufactured_option as $key) {
				$list[$key]['manufactured'] = true;
			}
			return $list;
		}

		/**
		*	Get a list of inventory locations that are used as end locations for work orders.
		*
		*	@return an array of inventory locations, in the form ID => NAME
		*/
		public static function get_end_locations(&$query) {
			$result = $GLOBALS['egw']->db->select(
				'perp_work_order',
				array(
					'location_id',
					'location_name',
				),
				array(),
				__LINE__, __FILE__, 
				0, '', -1, 'perp_manufacturing',
				'JOIN perp_location ON
					perp_work_order.location = perp_location.location_id'
			);
			$list = array();
			foreach($result as $row) {
				$list[$row['location_id']] = $row['location_name'];
			}
			return $list;
		}

		/**
		*	Make sure the options have been flushed, preserve old BOMs 
		*/
		public function __destruct() {
			if(count(self::$old_option_cache) > 0) {
				$this->flush_options();
			}
			// Store cache into session
			$GLOBALS['egw']->session->appsession('detail_bom_cache', 'perp_warranty', self::$detail_bom_cache);
		}
	}
?>
