<?php

	/**
	*	Class for interacting with the workflow app's tables
	*	
	*	This is probably really bad, you should never interact with
	*	another app's tables directly, but it's abstracted from the
	*	logic layer, and we're only reading.
	*/

	class so_perp_workflow {

		/**
		*	Get the process to use to manufacture the given stock item.  
		*	This is determined by looking for a record in the
		*	perp_workflow_stock
		*/
		public function get_process_id($stock_id) {
			return $GLOBALS['egw']->db->select(
				'perp_workflow_stock',
				array('workflow_id'),
				array('stock_id' => $stock_id),
				__LINE__, __FILE__
			)->fetchSingle();
		}

		/**
		*	Get the process to use to manufacture the given stock item.  
		*	This is determined by looking for a record in the
		*	perp_workflow_stock
		*/
		public function set_process_id($stock_id, $process_id) {
			$GLOBALS['egw']->db->insert(
				'perp_workflow_stock',
				array(
					'stock_id'	=>	$stock_id,
					'workflow_id'	=>	$process_id,
					'last_change'	=>	$GLOBALS['egw']->db->to_timestamp(time())
				),
				array('stock_id' => $stock_id),
				__LINE__, __FILE__, 'perp_manufacturing'
			);
		}

		/**
		*	Get a list of available processes to use to manufacture the given stock item
		*/
		public function get_process_list($stock_id) {
			$where = array(
				"wf_is_active = 'y'",
				"wf_config_name = 'stock_id'"
			);

			if($stock_id) {
				$where['wf_config_value'] = $stock_id;
			}
			$GLOBALS['egw']->db->select(
				'egw_wf_process_config',
				array(
					'*'
				),
				$where,
				__LINE__, __FILE__,false,
				'ORDER BY egw_wf_processes.wf_p_id DESC', 'workflow', -1,
				'JOIN egw_wf_processes ON
					egw_wf_processes.wf_p_id = egw_wf_process_config.wf_p_id'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row['wf_p_id'];
			}
			return $list;
		}

		/**
		*	Get the workflow instance responsible for a work order detail
		*
		*	@param work_order_id
		*	@param detail_id
		*/
		public function get_instance_id($work_order_id, $detail_id) {
			// We do it by work order reference...
			$work_order = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $work_order_id);
			// The instance name should be $work_order['work_order_reference'] . ':' . $detail_id
			$GLOBALS['egw']->db->select('egw_wf_instances',
				array(
					'wf_instance_id',
					'wf_properties'
				),
				array(
					'wf_name'	=>	$work_order['work_order_reference'] . ':' . $detail_id
				),
				__LINE__, __FILE__
			);
			$list = array();

			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Check the workflow to see if the given instance has passed through the given 
		*	work center.
		*
		*	@param instance_id
		*	@param work_center_id
		*
		*	@param boolean True if the instance has been at the work center, false if not
		*/
		public function instance_was_at_work_center($instance_id, $work_center_id) {
			$join = 'JOIN egw_wf_activity_agents ON
			    egw_wf_workitems.wf_activity_id = egw_wf_activity_agents.wf_activity_id
			JOIN egw_wf_agent_perp_build ON
			    egw_wf_activity_agents.wf_agent_id = egw_wf_agent_perp_build.wf_agent_id';
			$result = $GLOBALS['egw']->db->select(
				'egw_wf_workitems',
				array('true'),
				array(
					'egw_wf_workitems.wf_instance_id = ' . (int)$instance_id,
					'egw_wf_agent_perp_build.work_center_id = ' . (int)$work_center_id
				),
				__LINE__, __FILE__, false,
				'', false, 1, 
				$join
			)->fetchSingle();
			return (boolean)$result;
		}

		/**
		*	Walk the workflow activity graph from one activity to another.
		*
		*	Returns the path that needs to be taken to get from start to end.
		*/
		public static function graph_walk($process_id, $start_activity, $end_activity, $direction = 'forward') {
			$to_visit = array($start_activity);
			$visited = array();
			$path = array();
			$parent = array();
			while(count($to_visit) > 0) {
				$node = array_shift($to_visit);
				if(in_array($node, $visited)) {
					continue;
				}
				$visited[] = $node;
				if($node == $end_activity) {
					// Yay!
					break;
				} else {
					$neighbours = self::get_process_transitions($process_id, $node, $direction);
					foreach($neighbours as $neighbour) {
						$parent[$neighbour] = $node;
						$to_visit[] = $neighbour;
					}
				}
			}
			$node = $end_activity;
			while($parent[$node]) {
				$node = $parent[$node];
				$path[] = $node;
			}
			return $path;
		}

		/**
		*	Helper function for graph walking, finds edges in the given direction for a node
		*
		*	@param process_id
		*	@param node Activity ID
		*	@param direction either 'forward' or 'backward'
		*
		*	@return array of edges
		*/
		public static function get_process_transitions($process_id, $node, $direction) {
			$list = array();
			$select_column = ($direction == 'backward') ? 'wf_act_from_id' : 'wf_act_to_id';
			$where_column = ($direction != 'backward') ? 'wf_act_from_id' : 'wf_act_to_id';

			$result = $GLOBALS['egw']->db->select(
				'egw_wf_transitions',
				array('*'),
				array(
					"$where_column = $node",
					"egw_wf_transitions.wf_p_id = $process_id"
				),
				__LINE__, __FILE__, false, '', 
				'perp_manufacturing', -1,
				'JOIN egw_wf_activities ON
					egw_wf_activities.wf_activity_id = egw_wf_transitions.'.$select_column
			);
			foreach($result as $row) {
				$list[] = $row[$select_column];
			}
			return $list;
		}

		/**
		*	Remove an instance's activity from the list of places it currently is
		*
		*	@param instance_id
		*	@param activity_id
		*/
		public static function delete_instance_activity($instance_id, $activity_id) {
			$GLOBALS['egw']->db->delete(
				'egw_wf_instance_activities',
				array(
					'wf_instance_id'	=>	$instance_id,
					'wf_activity_id'	=>	$activity_id
				),
				__LINE__, __FILE__, 'workflow'
			);
		}
	}
?>
