<?php
	/**************************************************************************\
	* eGroupWare Workflow - pERP Manufacturing Workflow Build - interface layer        *
	* ------------------------------------------------------------------------ *
	* This program is free software; you can redistribute it and/or modify it  *
	* under the terms of the GNU General Public License as published           *
	* by the Free Software Foundation; either version 2 of the License, or     *
	* any later version.                                                       *
	\**************************************************************************/



	/**
	*	Workflow build Agent : interface layer
	*
	*	This class connects the workflow agents calls to the perp manufacturing business layer
	*	Copied from smtp_mail by regis.leroy@glconseil.com
	*
	*	The user interface template is required if you want to display any parts.  
	*	See the agent help in the Activity's configuration.
	*/

	require_once(EGW_INCLUDE_ROOT . SEP . 'workflow' . SEP . 'inc' . SEP . 'class.ui_agent.inc.php');
	require_once('class.bo_agent_perp_build.inc.php');
		
	class ui_agent_perp_build extends ui_agent
	{
		public function __construct()
		{
			parent::ui_agent();
			$this->agent_type = 'perp_build';
			$this->bo_agent =& new bo_agent_perp_build();
		}


		function showAdminActivityOptions ($template_block_name)
		{
			$admin_name = 'admin_agent_'.$this->agent_type;
			$this->t->set_file($admin_name, EGW_INCLUDE_ROOT . SEP . 'perp_manufacturing' . SEP . 'templates' . SEP . 'default' . SEP . 'admin_agent_perp_build.tpl');
			$this->t->set_block($admin_name, 'block_ag_config_option_select_option', 'ag_option_select_option');
			$this->t->set_block($admin_name, 'block_ag_config_option_select', 'ag_option_select');
			$options =& $this->bo_agent->getAdminActivityOptions();
			foreach ($options as $option_name => $option_conf)
			{
				if ($option_conf['type'] == 'select')
				{	
					$this->t->set_var(array(
						'ag_config_name_s'	=> "wf_agent[".$this->agent_type."][".$option_name."]",
						'ag_config_label_s'	=> $option_conf['label'],
					));
					foreach($option_conf['values'] as $key => $value)
					{
						$this->t->set_var(array(
							'ag_config_value_s_key'		=> $key,
							'ag_config_value_s_value'	=> $value,
							'ag_config_value_s_selected'	=> ($option_conf['value']==$key)? 'selected': '',
						));
						$this->t->parse('ag_option_select_option','block_ag_config_option_select_option',true);
					}
					$this->t->parse('ag_option_select','block_ag_config_option_select',true);
					$this->t->set_var('ag_option_select_option', null);
				}
			}
			//show the shared part handled by parent object
			parent::showAdminActivityOptions('shared_part');
			$this->translate_template($admin_name);
			$this->t->parse($template_block_name, $admin_name);
		}
		
		/**
		 * * Function called by the running object (run_activity) after the activity_pre code
		 * * and before the user code. This code is runned only if the $GLOBALS['workflow']['__leave_activity']
		 * * IS NOT set (i.e.: the user is not cancelling his form in case of interactive activity)
		 * * WARNING : on interactive queries the user code is parsed several times and this function is called
		 * * each time you reach the begining of the code, this means at least the first time when you show the form
		 * * and every time you loop on the form + the last time when you complete the code (if the user did not cancel).
		*  * @return true or false, if false the $this->error array should contains error messages
		 */
		function run_activity_pre()
		{
			//load agent data from database
			$this->bo_agent->init();
			if($this->bo_agent->get_error()) {
				$this->error[] = $this->bo_agent->get_error();
			}

			// Get form data
			$values = get_var('exec', array('POST', 'GET'));

			// Make sure we have all the serials we need
			$query['controlled'] = array(1,2);
			$this->bo_agent->get_part_list($query, $parts, $readonlys);
			$needed_serials = count($parts);

			// Check provided serial numbers
			if(is_array($values) && count($values) && array_key_exists(1, $values)) {
				unset($_POST['exec']);
				if( $this->bo_agent->check_serials($values)) {
					if($this->bo_agent->set_serials($values)) {
						$this->bo_agent->build();
					} else {
						$this->error[] = $this->bo_agent->get_error();
						$this->error[] = lang('Missing or invalid serial numbers');
						return false;
					}
				} else {
					$this->error[] = $this->bo_agent->get_error();
					$this->error[] = lang('Missing or invalid serial numbers');
					return false;
				}
			} elseif (!$this->interactivity || $_POST['wf_submit']) {
				// Not interactive, they better have set serials or no parts displayed
				if($this->bo_agent->validate()) {
					$build_success = $this->bo_agent->build();
					if(!$build_success) {
						$this->error[] = lang('Build failed');
						$this->error[] = $this->bo_agent->get_error();
					}
				} else {
					$this->error[] = $this->bo_agent->get_error();
					return false;
				}
			}
			return true;
		}
		
		/**
		 * * Function called by the running object (run_activity) after the activity_pre code
		 * * and before the user code. This code is runned only if the $GLOBALS['workflow']['__leave_activity']
		 * * IS set (i.e.: the user is cancelling his form in case of interactive activity)
		*  * @return true or false, if false the $this->error array should contains error messages
		 */
		function run_leaving_activity_pre()
		{
			return true;
		}
		
		/**
		 * * Function called by the running object (run_activity) after the user code
		 * * and after the activity_pos code. This code is runned only if the $GLOBALS['__activity_completed']
		 * * IS set (i.e.: the user has completing the activity)
		*  * @return true or false, if false the $this->error array should contains error messages
		 */
		function run_activity_completed_pos()
		{
			return true;
		}
		
		/**
		 * * Function called by the running object (run_activity) after the user code
		 * * and after the activity_pos code. This code is runned only if the $GLOBALS['__activity_completed']
		 * * IS NOT set (i.e.: the user is not yet completing the activity)
		 * * WARNING : on interactive queries the user code is parsed several times and this function is called
		 * * each time you reach the end of the code without completing, this means at least the first time
		 * * and every time you loop on the form.
		*  * @return true or false, if false the $this->error array should contains error messages
		 */
		function run_activity_pos() {

			if($this->interactivity) {
				$etemplate = CreateObject('etemplate.etemplate');

				// Load work order info for display
				
				// Load detail info for display

				// Display [ all | controlled ] components
				$parts = array();
				$query = array();
				switch ($this->bo_agent->fields['display_parts']['value']) {
					case 2: // Show controlled parts
						$query['controlled'] = array(1,2);
						// fall through
					case 1:	// Show all parts
						$this->bo_agent->get_part_list($query, $parts, $readonlys);
						break;
					default: // Show no parts
						$parts = array();
						break;
				}
				if(count($parts)) {
					array_unshift($parts, false);	// Allow for row header
					$etemplate->read('perp_manufacturing.build_agent.row');
					$part_display = $etemplate->exec('', $parts, $select_box_options, $readonlys, array(), 1);
				}
				$this->t->set_var('parts', $part_display);

				return true;
			} else {
				return true;
			}
		}
		
	}
?>
