<?php

	class ui_perp_manufacturing {

		public $public_functions = array(
			'index'		=>	true,
			'serials'	=>	true,
			'link_view'	=>	true,
		);

		private $debug = false;
		private $template;
		private $bo;
		private $wo_bo;

		public function ui_perp_manufacturing() {
			$this->template = CreateObject('etemplate.etemplate');
			$this->bo = CreateObject('perp_manufacturing.bo_perp_manufacturing');
			$this->wo_bo = CreateObject('perp_manufacturing.bo_perp_work_order');
		}

		public function index($content = array()) {
			if($this->debug) {
				echo '<b>perp_manufacturing.ui_perp_manufacturing.index</b> incoming data:';
				_debug_array($content);
			}

			$work_center = $GLOBALS['egw']->acl->check('work_center', EGW_ACL_READ, 'perp_manufacturing');
			$bom = $GLOBALS['egw']->acl->check('bill_of_material', EGW_ACL_READ, 'perp_manufacturing');
			$build_option = $GLOBALS['egw']->acl->check('build_option', EGW_ACL_READ, 'perp_manufacturing');
			$work_order = $GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing');

			if (isset($content['work_center_list']) && $work_center) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_manufacturing.ui_perp_work_center.work_center_list'));
			} elseif (isset($content['bom_list']) && $bom) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_manufacturing.ui_perp_bill_of_material.bill_of_material_list'));
			} elseif (isset($content['build_option_list']) && $build_option) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_manufacturing.ui_perp_build_option.build_option_category_list'));
			} elseif (isset($content['work_order_list']) && $work_order) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_manufacturing.ui_perp_work_order.work_order_list'));
			} elseif (isset($content['batch_work_order']) && $work_order) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_manufacturing.ui_perp_work_order.work_order_batch'));
			}

			$readonly['work_center'] = !$work_center;
			$readonly['bom_list'] = !$bom;
			$readonly['build_option_list'] = !$build_option;
			$readonly['work_order_list'] = !$work_order;
			$readonly['batch_work_order'] = !$work_order;

			// Display
			$this->template->read('perp_manufacturing.index');
			$this->template->exec('perp_manufacturing.ui_perp_manufacturing.index', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Set up and adjust numbering schemes for serialized goods.
		*/
		public function serials($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}


			// Deal with incoming
			if(isset($content['save'])) {
				$test = true;
				foreach($content['serials'] as $key => $row) {
					if($row['name'] == lang('New') || !$row['name']) {
						continue;
					}
					$result = $this->bo->set_serial_setup($row['setup_id'], $row);
					if(!is_numeric($result)) {
						$this->messages[] = array($result, 'error');
						$test = false;
					}
				}
				if($test) {
					$this->messages[] = array(lang('Settings saved'), 'success');
				}
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			}

			// Set up display data
			$data['serials'] = $this->bo->get_serial_setup(null,true);
			array_unshift($data['serials'], false);
			$data['serials'][] = array(
				'name'		=>	lang('New'),
				'pattern'	=>	'%1$05d',
				'next_id'	=>	1,
				'next_serial'	=>	'00001'
			);


			$data['serial_number_params'] = array(
				lang('%1 is replaced with %2', '&#37;1$d', lang('Next serial ID')),
				lang('%1 is replaced with %2', '&#37;2$d', lang('Stock ID number')),
				lang('%1 is replaced with %2', '&#37;3$d', lang('Stock Code')),
				lang('%1 is replaced with %2', '&#37;4$d', lang('Stock category ID')),
				lang('%1 is replaced with %2', '&#37;5$s', lang('Work order reference number')),
				lang('%1 is replaced with %2', '&#37;6$s', lang('Work order option enumeration')),
				lang('%1 is replaced with %2', 'date(Y.m.d)', date('Y.m.d')),
			);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$preserve['serials'] = $data['serials'];

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Serial Number Setup');
			$this->template->read('perp_manufacturing.admin.serial_setup');
			return $this->template->exec('perp_manufacturing.ui_perp_manufacturing.serials', $data, $select_box_options, $readonly, $preserve);
		}


		/**
		*	This function is designed to be called from a workflow action, not anywhere else.
		*	It may work elsewhere, but the workflow will probably not follow
		*
		*	This is an ugly hack to work with workflow interactive activity.  It really should be replaced by something nice.
		*
		*	pass it an array (
		*		'work_order_id'		=>
		*		'detail_id'		=>
		*		'work_center_id'	=>
		*	)
		*/
		public function enter_serial_numbers($content = array()) {
			if(!is_array($content['nm']['rows'])) {
				$content['nm']['rows'] = array();
			}
			foreach($content['nm']['rows'] as &$component) {
				$query = array(
					'stock_id'	=>	$component['stock_id'],
					'serial_number'	=>	$component['serial_number'],
//					'location'	=>	$content['work_center_id']
				);
				if($component['serial_number']) {
					// Check for valid serial number
					$assemblies =  ExecMethod('perp_manufacturing.so_perp_stock_history.get_serial_assemblies', $query + array('component_stock_id' => $part['stock_id']));
					if( count($assemblies) > 0) {
						$this->messages[] = lang('%2 serial number %1 has already been used', $component['serial_number'], ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['stock_id']));
						unset($component['serial_number']);
						break;
					}
					if( ExecMethod('perp_inventory.bo_perp_inventory.get_controlled_inventory_list', $query) != 1) {
						$this->messages[] = lang('%1 is not a valid serial number for %2', $component['serial_number'], ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['stock_id']));
						unset($component['serial_number']);
					}
				}
			}
			foreach($content['nm']['rows'] as &$component) {
				if($component['serial_number']) {
					ExecMethod2(
						'perp_manufacturing.bo_perp_serial_assembly.serial_assembly',
						$content['work_order_id'], 
						$content['detail_id'], 
						$component['stock_id'], 
						$component['serial_number'], 
						time(),
						$content['work_center_id']
					);
				}
			}

			$query = array(
				'work_order_id'	=>	$content['work_order_id'],
				'detail_id'	=>	$content['detail_id'],
				'work_center'	=>	$work_center_id,
				'template'      =>      'perp_manufacturing.production.build.row',
			);
			$this->wo_bo->get_work_order_detail_bom( $query, $parts);
			foreach($parts as $key => &$part) {
				if(!$part['controlled']) {
					unset($parts[$key]);
				} else {
					$assemblies =  ExecMethod('perp_manufacturing.so_perp_stock_history.get_serial_assemblies', $query + array('component_stock_id' => $part['stock_id']));
					if( count($assemblies) != 0) {
						unset($parts[$key]);
					}
				}
			}
			if(count($parts) > 0) {
				array_unshift($parts, false);
				$data['nm']['rows'] = $parts;
			} else {
				$GLOBALS['perp']['serial_entry_complete'] = true;
			}
			$preserve['nm']['rows'] = $parts;

			$select_box_options['unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');

			if(count($this->messages) > 0) {
				$data['message'] = '<ul><li>' . implode('</li><li>', $this->messages) . '</li></ul>'; //ExecMethod('perp_api.ui_perp_api.make_message', $messages);
			}

			$this->template->read('perp_manufacturing.enter_serials');
			return $this->template->exec('perp_manufacturing.ui_perp_manufacturing.enter_serial_numbers', $data, $select_box_options, $readonly, $preserve, 1);
		}

		public function link_view() {
			$link_id = $_GET['link_id'];
			list($location, $id) = explode('-', $link_id);

			switch($location) {
				case 'RETROFIT':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_manufacturing.ui_perp_retrofit.view',
						'stock_history_id'	=>	$id
					));
					break;
				case 'WORK_ORDER':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_manufacturing.ui_perp_work_order.work_order_edit',
						'work_order_id'	=>	$id
					));
					break;
					
			}
		}
	}
?>
