<?php

	/**
	*	User interface for dealing with generic (not tied to a specific location) work centers
	*/

	require_once(EGW_INCLUDE_ROOT . '/etemplate/inc/class.etemplate.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_center.inc.php');

	class ui_perp_work_center {
		
		public $public_functions = array(
			'work_center_list'	=>	true,
			'new_work_center'	=>	true,
			'edit'			=>	true
		);

		protected static $debug = false;

		protected static $template;

		protected static $bo;

		public function __construct() {
			if(!is_object(self::$template)) {
				self::$template = new etemplate();
			}
			if(!is_object(self::$bo)) {
				self::$bo = new bo_perp_work_center();
			}
		}

		/**
		*	User interface for viewing a list of generic 
		*	(not tied to a specific location) work centers
		*
		*	@param content Array of information returned from the eTemplate
		*/
		public function work_center_list($content = array()) {

			// Deal with incoming
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_manufacturing.ui_perp_manufacturing.index'
				));
			}
			$this->handle_actions($content);

			// Set up outgoing

			// Pull nextmatch from the session
			if($session = $GLOBALS['egw']->session->appsession('work_center_list')) {
				$data['nm'] = $session;
			} else {
				// First time, set it up
				$data['nm'] = array(
					'get_rows'	=>	'perp_manufacturing.ui_perp_work_center.get_work_center_list',
					'template'	=>	'perp_manufacturing.work_center_list.row',
					'filter_label'	=>	'location',
					'no_cat'	=>	true,
					'no_filter2'	=>	true,
				);
			}

			$readonly['new'] = !$GLOBALS['egw']->acl->check('work_center', EGW_ACL_ADD, 'perp_manufacturing');

			$select_box_options = $this->get_list_options();

			$GLOBALS['egw_info']['flags']['app_header'] = lang('Work Centers');
			self::$template->read('perp_manufacturing.work_center_list');
			self::$template->exec('perp_manufacturing.ui_perp_work_center.work_center_list', $data, $select_box_options, $readonly);
		}

		/**
		*	Get the select box options for the work center list
		*
		*	@return array of option arrays
		*/
		public function get_list_options() {
			return array(
				'filter'	=>	array('' => lang('All')) + ExecMethod('perp_inventory.bo_perp_location.get_location_list'),
			);
		}

		/**
		*	Store current nextmatch settings in the session 
		*	so user doesn't have to page through them constantly.
		*
		*	@nextmatch
		*/
		public function get_work_center_list(&$query = array(), &$rows = array(), &$readonly = array()) {
			$GLOBALS['egw']->session->appsession('work_center_list', 'perp_manufacturing', $query);
			
			return self::$bo->get_work_center_list($query, $rows, $readonly);
		}

		/**
		*	Handle actions for work centers
		*
		*	@param content Array returned from etemplate with actions
		*/
		public function handle_actions($content = array()) {
			// Cancel, return to Work Center List
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_manufacturing.ui_perp_work_center.work_center_list'
				));
			}

			// Create a new generic Work Center
			if($content['new']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_manufacturing.ui_perp_work_center.new_work_center'
				));
			}

			// Delete an existing work center
			if($content['nm']['rows']['delete']) {
				$id = key($content['nm']['rows']['delete']);
				return self::$bo->delete_work_center($id);
			}

			// Unset a location from a work center
			if($content['locations']['delete']) {
				$id = key($content['locations']['delete']);
				return self::$bo->remove_work_center_location($content['work_center_id'], $id);
			}

			// Save the work center
			if($content['save']) {
				return self::$bo->set_work_center($content['work_center_id'], $content);
			}

			// Set a location to be a certain type of work center
			if($content['add_location']) {
				return self::$bo->add_work_center_location($content['work_center_id'], $content['add_location']);
			}

		}

		/**
		*	Create a new work center
		*
		*/
		public function new_work_center($content = array()) {
			// Just a wrapper, until more is needed
			return $this->edit($content);
		}

		/**
		*	Edit a work center
		*
		*	@param content Array provided by eTemplate
		*/
		public function edit($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming data:<br />';
				_debug_array($content);
			}

			$work_center_id = reg_var('work_center_id', 'any', 'number', $content['work_center_id']);

			// Deal with incoming
			$result = $this->handle_actions($content);
			if(is_numeric($result)) {
				$work_center_id = $result;
				$this->messages[] = array(lang('Work Center saved.'), 'success');
			} elseif ($result !== true) {
				$this->messages[] = array($result, 'error');
			}

			// Set up outgoing
			$data = $this->get_edit_data($work_center_id);

			$select_box_options = $this->get_edit_options();

			$preserve['work_center_id'] = $work_center_id;
			$readonly = !$GLOBALS['egw']->acl->check('work_center', EGW_ACL_EDIT, 'perp_manufacturing') && !$GLOBALS['egw']->acl->check('work_center', EGW_ACL_EDIT, 'perp_manufacturing');
			if($readonly) {
				$readonly = array(
					'name'	=>	true,
					'perp_manufacturing.work_center_edit.details'	=>	true,
					'add_location'	=>	true,
					'save'	=>	true,
				);
				unset($data['work_center_id']);
			}

			$GLOBALS['egw_info']['flags']['app_header'] = lang('Edit Work Center - %1', $data['name']);
			self::$template->read('perp_manufacturing.work_center_edit');
			self::$template->exec('perp_manufacturing.ui_perp_work_center.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Get data for display when editing a work center
		*
		*	@param work_center_id ID number of the work center
		*/
		public function get_edit_data($work_center_id) {
			$data = array();
			
			if($work_center_id) {
				$data += self::$bo->get_work_center($work_center_id);

				$query = array(
					'work_center_id'	=>	$work_center_id,
					'template'		=>	false
				);
				self::$bo->get_work_center_location_list($query, $data['locations']);

				$data['stock'] = array(
					'work_center_id'	=>	$work_center_id,
					'cat_app'	=>	'perp_inventory',
					'selectcols'	=>	$GLOBALS['egw_info']['user']['preferences']['perp_inventory']['nextmatch-perp_inventory.stock_list.row'],
					'get_rows'	=>	'perp_manufacturing.bo_perp_work_center.get_work_center_stock_list',
					'template'	=>	'perp_inventory.stock_list.row',
					'no_filter'	=>	true,
					'filter2_label'	=>	lang('Status'),
				);
				$data['stock']['cat_id'] = $GLOBALS['egw_info']['user']['preferences']['perp_inventory']['stock_category'];
				$data['stock']['filter2'] = $GLOBALS['egw_info']['user']['preferences']['perp_inventory']['stock_status'];
			}

			return $data;
		}


		/**
		*	Get the select box options for editing a work center
		*
		*	@return array of option arrays
		*/
		public function get_edit_options() {
			$options = array(
				'add_location'	=>	array(false => lang('Select')) + ExecMethod('perp_inventory.bo_perp_location.get_location_list'),
			);

			// Options for stock list, copied from ui_perp_stock
			$options['filter2'] = array('' => lang('All')) + ExecMethod('perp_inventory.bo_perp_stock.get_status_flags');
			$options['stock_type_flag'] = ExecMethod('perp_inventory.bo_perp_stock.get_stock_types');
			$options['controlled'] = array(0 => lang('No'), 1 => lang('Batched'), 2 => lang('Serialized'));
			$options['has_attachments'] = array('' => lang('Any'), 1=> lang('yes'), 2=> lang('no'));
			$options['unit_of_measure'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			$options['status_flag'] = ExecMethod('perp_inventory.bo_perp_stock.get_status_flags');
			$options['inventory'] = ExecMethod('perp_inventory.bo_perp_location.get_location_list');

			return $options;
		}

		/**
		*	Get readonlys for editing a work center
		*
		*	@return array of fields to be read only
		*/
		protected function get_edit_readonlys() {
			return array(
			);
		}

	}
?>
