<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_stock.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_bill_of_material.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_build_option.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_order.inc.php');

	/**
	*	Costs of a bill of materials, in the supplier's currency.
	*/

	class bill_of_material_costing extends perp_report {

		/**
		*	These parameters relate to the spreadsheet, so formulas can be set up
		*/
		protected $currency_column = 0;


		public function __construct(array $parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_manufacturing';
			parent::perp_report($parameters);

			$this->template = 'bill_of_materials_costing.ods';

			$this->parameters = $parameters;

		// First Currency total column = I (65 = A)
			$this->currency_column = 65 + 8;

			$bo_options = new bo_perp_build_option();
			$this->bo_work_order = new bo_perp_work_order();

			// Default parameters
			$this->parameters['work_order_id'] = -1;
			$this->parameters['detail_id'] = $GLOBALS['egw_info']['user']['account_id'];

			$this->parameters['options-cost_method'] = array(
				'fifo'		=>	lang('First In, First Out'),
				'supplier'	=>	lang('Current Supplier Cost'),
			);
			if(!$parameters['stock_id']) {
				$this->parameters['stock_id'] = array(
					'stock_id'		=>	($parameters['stock_id'] ? $parameters['stock_id'] : null),
					'stock_type_flag'	=>	'A,K'
				);
				$this->parameters['effective_date'] = strtotime('today');
			}

			if($parameters['stock_id'] && $parameters['include_options']) {

				// Query to get the options
				$option_query = array(
					'work_order_id'		=>	$this->parameters['work_order_id'],
					'detail_id'		=>	$this->parameters['detail_id'],
					'stock_id'		=>	$parameters['stock_id'],
					'effective_date'	=>	$parameters['effective_date']
				);

				// Use a static temporary work order
				$detail = $this->bo_work_order->get_work_order_detail($this->parameters['work_order_id'], $this->parameters['detail_id'], true);
				if(!$detail) {
					// Create a temp order so options can be selected
					$wo['work_order_reference'] = lang('Temporary for cost report %1', $this->parameters['work_order_id']);
					$wo['created_date'] = time();
					$wo['required_date'] = $this->parameters['effective_date'];
					$wo['location'] = 0;
					$wo['details'] = array();
					$detail = array(
						'stock_id'      =>      $parameters['stock_id'],
						'quantity'      =>      1
					);
					$wo_id =  $this->bo_work_order->set_work_order($this->parameters['work_order_id'], $wo);
					$this->bo_work_order->set_work_order_detail($this->parameters['work_order_id'], $this->parameters['detail_id'], $detail);
					$detail = $this->bo_work_order->get_work_order_detail($this->parameters['work_order_id'], $this->parameters['detail_id']);

					// Get default options, they'll be set below
					$this->bo_work_order->get_work_order_detail_option_list($option_query, $this->parameters['options']['rows']);
				}

				// Update work order
				if(is_array($this->parameters['options']['rows'])) {
					foreach($this->parameters['options']['rows'] as $key => $row) {
						$this->bo_work_order->set_work_order_detail_option($this->parameters['work_order_id'], $this->parameters['detail_id'], 
							$row['stock_id'], $row['build_option_category_id'], $row['value']
						);
					}
				}

				// Get options for display
				$this->bo_work_order->get_work_order_detail_option_list($option_query, $this->parameters['options']['rows']);
				array_unshift($this->parameters['options']['rows'], false);
				if(is_array($this->parameters['options']['rows'])) {
					foreach($this->parameters['options']['rows'] as $key => &$row) {
						if(!$row['options-value']) {
							continue;
						}
						foreach($row['options-value'] as $value => $option) {
							$this->parameters['options']['rows']["options-{$key}[value]"][$value] = $option['build_option_name'];
						}
					}
				}
			}
		}

		public function get_data($query = array()) {
			if(!(int)$query['stock_id']) {
				return false;
			}
			// Prevent possible injection
			$query['stock_id'] = (int)$query['stock_id'];

			$this->bo_stock = new bo_perp_stock();
			$this->bo_bom = new bo_perp_bill_of_material();

			$stock_item = $this->bo_stock->get_stock_item($query['stock_id']);

			$this->filename = lang('Bill of materials costing') . ' ' . ExecMethod('perp_inventory.bo_perp_stock.link_title', $query['stock_id']);

			$data = array(
				'stock_num'		=>	$stock_item['stock_num'],
				'short_description'	=>	$stock_item['short_description'],
				'effective_date'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $query['effective_date']),
				'options'		=>	array(),
			);

			$data['lang'][] = $this->get_titles();

			$currency = $this->get_currencies();
			$data['currency0,currency1,currency2,currency3'] = $currency;

			foreach($currency as $key => $c) {
				$c['column'] = chr($this->currency_column + count($currency) - 1 + $key);
				$data['currency_total'][] = $c;
			}

			if(!$query['include_options']) {
				$data = array_merge($data, $this->base_bom($query));
			} else {
				$data = array_merge($data, $this->work_order_bom($query));
			}

			// Variables used in creating the spreadsheet formulas
			$data['option_count'] = max(ceil((float)count($data['options']) / 2), 1);
			$data['currency_count'] = count($currency);
			$data['currency_start'] = 6 + $data['option_count'] + 1;
			$data['currency_end'] = $data['currency_start'] + $data['currency_count'];
			$data['component_count'] = count($data['stock']);
			$data['component_start'] = $data['currency_end'] + 6;
			$data['row_count'] = $data['currency_count'] + $data['component_count'] + $data['option_count'];
			$data['component_end'] = $data['component_start'] + $data['component_count'];
//_debug_array($data);die();

			return $data;
		}

		protected function base_bom($query) {
			$bom_query = array(
				'stock_id'	=>	$query['stock_id'],
				'as_of_date'	=>	$query['effective_date']
			);
			$this->bo_bom->get_bill_of_material_list($bom_query, $bom_list);
			if(count($bom_list) > 0 && isset($bom_list[0])) {
				$bom = $this->bo_bom->get_bill_of_material($bom_list[0]['bill_of_material_id']);
			} else {
				return false;
			}

			if($bom['effective_to_date']) {
				$data['effective_to_date'] =	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $bom['effective_to_date']);
			} else {
				$data['effective_to_date'] = 	lang('Current');
			}
			$data['effective_after_date']	=	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $bom['effective_after_date']);
			$data['bill_of_materials']	=	$bom['bill_of_material_id'];
			$data['stock'] = $this->get_components($bom['details'], $query['effective_date']);
			foreach($data['stock'] as &$component) {
				$component['bill_of_material_id'] = $bom['bill_of_material_id'];
			}
			
			return $data;
		}

		protected function work_order_bom($query) {
			// Save detail to make it flush everything
			$detail = $this->bo_work_order->get_work_order_detail($query['work_order_id'], $query['detail_id'], true);
			$this->bo_work_order->set_work_order_detail($query['work_order_id'], $query['detail_id'], $detail, false);

			$data['effective_to_date'] = 	lang('Current');
			$data['effective_after_date'] = '';

			$data['options'] = array();
			if(is_array($query['options']['rows'])) {
				foreach($query['options']['rows'] as $row) {
					if(!$row['build_option_category_name']) {
						continue;
					}
					$data['options'][] = array(
						'build_option_category_name'	=>	$row['build_option_category_name'],
						'build_option_name'		=>	$row['options-value'][$row['value']]['build_option_name']
					);
				}
			}
			
			$bom_query = array(
				'stock_id'	=>	$query['stock_id'],
				'work_order_id'	=>	$query['work_order_id'],
				'detail_id'	=>	$query['detail_id'],
				'num_rows'	=>	-1,
				'order'		=>	'stock_num'
			);
			$data['bill_of_materials'] = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $query['stock_id']);
			$this->bo_work_order->get_work_order_detail_bom($bom_query, $components);
			$uom = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			$data['stock'] = $this->get_components($components, $query['effective_date']);
			foreach($data['stock'] as &$stock) {
				$stock['unit_of_measure_name'] = $uom[$stock['unit_of_measure']];
			}

			// Cancel temporary work order
			$detail = $this->bo_work_order->get_work_order_detail($query['work_order_id'], $query['detail_id']);
			$detail['quantity'] = 0;
			$this->bo_work_order->set_work_order_detail($query['work_order_id'], $query['detail_id'], $detail);
			$count = $this->bo_work_order->get_work_order_detail_list(array('work_order_id' => $query['work_order_id'], 'template' => false));
			if($count == 0) {
				$this->bo_work_order->delete_work_order($query['work_order_id']);
			}
			return $data;
		}

		protected function get_titles() {
			return array(
				'bill_of_material_costing'	=>	lang('bill of materials costing'),
				'bill_of_materials'		=>	lang('bill of materials'),
				'bill_of_materials_id'		=>	lang('BOM ID'),
				'as_at'				=>	lang('as at'),
				'currency_name'			=>	lang('currency name'),
				'exchange_rate'			=>	lang('exchange rate'),
				'total'				=>	lang('total'),
				'total_cost'			=>	lang('total cost'),
				'stock_code'			=>	lang('stock code'),
				'short_description'		=>	lang('description'),
				'quantity'			=>	lang('quantity'),
				'unit_of_measure'		=>	lang('unit of measure'),
				'no_options'			=>	lang('No options')
			);
		}

		protected function get_currencies() {
			$query = array(
				'template'	=>	false
			);

			$bo_api = new bo_perp_api();
			$bo_api->get_currencies($query, $currencies);
			return $currencies;
		}

		protected function get_components($details, $date) {

			foreach($details as $key => &$stock) {
				// Is this a pseudo assembly that got in here some how?
				if($stock['stock_type_flag'] != 'R' && $stock['pseudo']) {
					unset($details[$key]);
					$query = array(
						'stock_id'		=>	$stock['stock_id'] ? $stock['stock_id'] : $stock['component'],
						'effective_date'	=>	$date
					);
					$stock = $this->base_bom($query);
					$details = array_merge($stock['stock'], $details);
				}
			}

			foreach($details as $key => &$stock) {
				$stock['unit_of_measure_name'] = $stock['uom'];
				$stock_item = $this->bo_stock->get_stock_item($stock['component'] ? $stock['component'] : $stock['stock_id']);
				$stock += $stock_item;

				switch($this->parameters['cost_method']) {
					case 'fifo':
						$stock += ExecMethod2('perp_inventory.bo_perp_inventory.get_cost_estimate', $stock, $date);
						break;
					case 'supplier':
						// Get a list of suppliers, and prices, for this item
						$suppliers = $this->bo_stock->get_suppliers($stock['stock_id']);
						if(is_array($suppliers) && count($suppliers) > 0) {
							// Figure out which supplier to use.  We'd prefer the preferred supplier.
							$use_supplier = $suppliers[0];
							foreach($suppliers as $supplier) {
								if($supplier['preferred']) {
									$use_supplier = $supplier;
									break;
								}
							}
							$stock += array(
								$use_supplier['supplier_currency'] => array(
									'cost'	=>	$use_supplier['price'] / ($use_supplier['conversion_factor'])
								)
							);
						}
						break;
				}
			}
			return $details;
		}


	}
?>
