<?php

	/**
	*	This report lists a whole bunch of information about work orders.
	*	
	*	The information includes one line for each work order detail with the 
	*	following information:
	*	- Work order reference
	*	- Status
	*	- Item
	*	- Options
	*	- Required Date
	*	- Client Reference tag, from the Sales Order
	*	- Client Name
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_api/inc/barcode/barcode.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_order.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_orders/inc/class.SalesOrder.inc.php');

	class build_list extends perp_report {
		
		public function __construct(&$parameters) {
			parent::__construct($parameters);

			$this->template = 'build_list.odt';
			$this->bo = new bo_perp_work_order();

			$this->config = config::read('perp_manufacturing');

			// Set up filters
			$this->parameters['options-status'] = array('' => lang('All')) + $this->bo->get_work_order_status_list();
			if($this->config['use_workflow']) {
				// Add workflow activities into the Status filter
				$activity_list = ExecMethod('workflow.workflow_activitymanager.get_process_activities', 1 );

				if(is_array($activity_list) && count($activity_list) > 0) {
					// Sort activities so they're in approximate order
					foreach($activity_list as $key => $act) {
						$activity_number[$key] = $act['wf_flow_num'];
					}
					array_multisort($activity_number, $activity_list);
					foreach($activity_list as $act) {
						$activities[$act['wf_activity_id']] = $act['wf_name'];
					}
				}
				$this->parameters['options-status'] += array(lang('Activities') => $activities);
			}
			$this->parameters['options-location_id'] = array('' => lang('All')) + ExecMethod('perp_manufacturing.bo_perp_work_order.get_end_locations');

			// Copied directly from nextmatch
			$row_options = array();
			if (!($max = (int)$GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs'])) $max = 15;
			// Can't handle more than 50 rows ( > 60 seconds)
			foreach(array(5,12,25,50,100,200,500,999) as $n) {
				if ($n-5 <= $max && $max <= $n+5) $n = $max;
				$row_options[$n] = $n;
			}
			if (!isset($row_options[$max])) {
				$row_options[$max] = $max;
				ksort($row_options);
			}
			$this->parameters['options-num_rows'] =& $row_options;
			if(!$this->parameters['num_rows']) {
				$this->parameters['num_rows'] = $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs'];
			}

			$this->parameters['options-order'] = array(
				'work_order_reference'			=>	lang('Work Order Reference'),
				'created_date'				=>	lang('Created Date'),
				'start_date'				=>	lang('Start Date'),
				'completed_date'			=>	lang('Completed Date'),
				'required_date'				=>	lang('Required Date'),
				'perp_stock.stock_num'			=>	lang('Stock Code'),
				'perp_client.client_name'		=>	lang('Client Name'),
			);

			// Add in clients
			$this->parameters['options-client_id'] = array(
				''		=>	lang('All'),
				'unclaimed'	=>	lang('Stock')
			);
			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($query['client_id'], $client_ids)) {
				$query['client_id'] = $client_ids;
			}
			$this->parameters['options-client_id'] += ExecMethod('perp_ar.bo_perp_client.get_client_list', $query);
		}

		public function get_data($params) {

			$data = array(
					'location'	=>	$params['location_id'] ? $params['options-location_id'][$params['location_id']] : lang('All'),
					'status_name'	=>	$params['status'] ? (is_numeric($params['status']) ? $params['options-status'][lang('Activities')][$params['status']] : $params['status']) : lang('All'),
					'category_name'	=>	$params['cat_id'] ? $GLOBALS['egw']->categories->id2name($params['cat_id']) : lang('all'),
				'line'	=>	array(),
			);

			// Need this to get full array back
			//$params['template'] = false;

			// This will get details as well
			//$params['include_details'] = true;

			// Sorting
			if(!$params['order']) {
				$params['order'] = 'required_date';
				$params['sort'] = ' ';
				if($this->config['use_workflow']) {
					$params['order'] .= ', wf_priority ASC';
				}
				$params['order'] .= ', created_date';
			}

/**
*	This works, and is correct, in that it calls the appropriate pre-existing functions
*	to get all the needed information, but it takes too long.
*
			$data['count'] = $this->bo->get_work_order_list($params, $work_orders);
	
			foreach($work_orders as $work_order) {
				// Format dates in user's preferred format
				$work_order['created_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $work_order['created_date']);
				$work_order['start_date'] = ($work_order['start_date'] ? date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $work_order['start_date']) : ''); 
				$work_order['completed_date'] = ($work_order['completed_date'] ? date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $work_order['completed_date']) : ''); 
				$work_order['required_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $work_order['required_date']);

				$details = $work_order['details'];
				unset($work_order['details']);
				if($work_order['order_id']) {
				//	$sales_order = new SalesOrder($work_order['order_id']);
				//	$work_order['client_name'] = ExecMethod('perp_ar.bo_perp_client.link_title', $sales_order->get_client_id());

					// Get client reference
				} else {
					$work_order['client_name'] = lang('On Hand');
				}
				foreach($details as $detail) {
					$detail['stock_item'] = ExecMethod('perp_inventory.bo_perp_stock.link_title', $detail['stock_id']);
					$data['line'][] = $work_order + $detail;


					$option_query = array(
						'work_order_id'		=>      $work_order['work_order_id'],
						'detail_id'		=>      $detail['detail_id'],
						'stock_id'		=>      $detail['stock_id'],
						'effective_date'	=>      $work_order['created_date'],
						'no_status'		=>	true,
						'no_readonlys'		=>	true,
						'no_options'		=>	true
					);
					$options = array();
					$this->bo->get_work_order_detail_option_list($option_query, $options);

					foreach($options as &$option) {
						$option['build_option_name'] = $option['options-value'][$option['value']]['build_option_name'];
					}
					$data[ (count($data['line']) - 1) . '_option'] = $options;
				}
			}

*/

			$work_orders = $this->bo->get_work_order_list($params);
			$data['count'] = count($work_orders);
			
			if($data['count'] > 0) {
				$join = 'JOIN perp_work_order_detail ON
					perp_work_order_detail.work_order_id = perp_work_order.work_order_id
					JOIN perp_stock ON 
						perp_stock.stock_id = perp_work_order_detail.stock_id
					LEFT JOIN (
						SELECT perp_order_detail.detail_value AS work_order_id,
							perp_sales_order_work_order.work_order_detail_id AS detail_id,
							perp_order.client_id,
							reference.detail_value AS client_reference
						FROM perp_sales_order_work_order 
						JOIN perp_order_detail ON
							perp_order_detail.order_id = perp_sales_order_work_order.order_id AND
							perp_order_detail.line_no = perp_sales_order_work_order.line_no AND
							perp_order_detail.detail_no = perp_sales_order_work_order.detail_no
						JOIN perp_order ON
							perp_order_detail.order_id = perp_order.order_id
						LEFT JOIN perp_order_detail AS reference ON
							reference.order_id = perp_order_detail.order_id AND
							reference.line_no = perp_sales_order_work_order.line_no AND
							reference.detail_class = \'ClientReferenceDetail\'
					) AS sales ON
						sales.work_order_id = perp_work_order_detail.work_order_id AND
						sales.detail_id = perp_work_order_detail.detail_id
					LEFT JOIN perp_client ON
						perp_client.client_id = sales.client_id ';
				if($this->config['use_workflow']) {
					$join .= 'LEFT JOIN egw_wf_instances ON
						egw_wf_instances.wf_name = CONCAT(perp_work_order.work_order_reference, \':\', perp_work_order_detail.detail_id) ';
				}

				$result = $GLOBALS['egw']->db->select(
					'perp_work_order',
					array(
						'perp_work_order.work_order_id',
						'work_order_reference',
						'created_date',
						'start_date',
						'completed_date',
						'required_date',
						'perp_work_order_detail.detail_id',
						'perp_work_order_detail.stock_id',
						'perp_work_order_detail.quantity',
						'perp_stock.stock_num',
						'perp_stock.short_description',
						'decimal_places',
						'perp_client.client_name',
						'sales.client_reference'
					),
					array(
						'perp_work_order.work_order_id IN (' . implode(',', array_keys($work_orders)) . ')'
					),
					__LINE__, __FILE__, false,
					'ORDER BY ' . $params['order'] . ' ' . $params['sort'], 
					'perp_manufacturing', $params['num_rows'],
					$join
				);
				foreach($result as $row) {
					// Convert from database specific timestamp
					$row['created_date'] = $GLOBALS['egw']->db->from_timestamp($row['created_date']);
					$row['start_date'] = $GLOBALS['egw']->db->from_timestamp($row['start_date']);
					$row['completed_date'] = $GLOBALS['egw']->db->from_timestamp($row['completed_date']);
					$row['required_date'] = $GLOBALS['egw']->db->from_timestamp($row['required_date']);

					// Convert to user specific timestamp
					$row['created_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['created_date']);
					$row['start_date'] = ($row['start_date'] ? date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['start_date']) : ''); 
					$row['completed_date'] = ($row['completed_date'] ? date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['completed_date']) : ''); 
					$row['required_date'] = date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['required_date']);

					// Round quantity to item-specific precision
					$row['quantity'] = round($row['quantity'], $row['decimal_places']);

					$row['stock_item'] = $row['stock_num'] . ' - ' . $row['short_description'];

					$data['line'][] = $row;
				}
			}

			// Need to be done in a seperate loop, as these calls use the DB and will destroy the results
			foreach($data['line'] as $key => &$detail) {
				$detail['status'] = $this->bo->get_work_order_status($detail['work_order_id']);
				$data[ $key . '_option'] = array();

				$result = $GLOBALS['egw']->db->select(
					'perp_work_order_option',
					array(
						'build_option_category_name',
						'build_option_name'
					),
					array(
						'work_order_id'	=>	$detail['work_order_id'],
						'detail_id'	=>	$detail['detail_id']
					), __LINE__, __FILE__, false,
					'', 'perp_manufacturing', -1,
					'JOIN perp_build_option ON
						perp_work_order_option.build_option_id = perp_build_option.build_option_id
					JOIN perp_build_option_category ON
						perp_work_order_option.option_category_id = perp_build_option_category.build_option_category_id'
				);
				foreach($result as $option) {
					$data[ $key . '_option'][] = $option;
				}
			}

			//if($data['count'] > $params['num_rows']) {
			//	$data['count_limit'] = lang('Limited to the first %1 of %2 work orders', $params['num_rows'], $data['count']);
			if($data['count']) {
				$data['count_limit'] = lang('Limited to the first %1 work orders', $params['num_rows'], $data['count']);
			} else {
				$data['count_limit'] = '';
			}

			if($params['client_id']) {
				$data['client_limit'] = lang('Only %1', ExecMethod('perp_ar.bo_perp_client.link_title', $params['client_id']));
				if($data['count_limit']) {
					$data['client_limit'] .= "\n";
				}
			} else {
				$data['client_limit'] = '';
			}

			// Set the logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			// The report generation process takes a long time
			set_time_limit(0);

			return $data;
		}
	}
?>
