<?php

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.OrderLine.inc.php');
	require_once('class.WorkOrderDetail.inc.php');
	require_once('class.bo_perp_sales_order.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.bo_perp_inventory.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_inventory/inc/class.so_perp_damage.inc.php');

	class SalesOrderLine extends OrderLine {

		const sales_order_line_table = 'perp_sales_order_detail';

		private $shipped;		// Shipped this time
		private $shipment_id = null;

		public function SalesOrderLine(SalesOrder $so, $info) {
			parent::__construct($so, $info);

			// Try for missing info
			$stock = ExecMethod2('perp_inventory.bo_perp_stock.get_stock_item', $this->stock_id);
			if($this->stock_id && !$this->supplier_stock_code) {
				$this->supplier_stock_code = $stock['stock_num'];
				$this->our_stock_code = $stock['stock_num'];
			}
			if($this->stock_id && !(
				$this->description && 
				$this->get_gl_account() && 
				$this->unit_price)) {
				if(!$this->description) {
					$this->description = $stock['short_description'];
				}
				if($stock && !($this->get_gl_account())) {
					if(count($stock)) {
						if(!$this->get_gl_account()) {
							$categories = ExecMethod('perp_inventory.bo_perp_stock.get_sales_categories', $this->stock_id);
							if(count($categories) == 1) {
						 		$this->set_gl_account($categories[0]['sales_gl_account']);
							} else {
								// Multiple sales categories, pick the one that matches the price list
								$price_list = ExecMethod('perp_orders.bo_perp_price_list.get_price_list', $this->get_price_list_id());
								if($price_list) {
									foreach($categories as $category) {
										if($price_list['sales_category_id'] == $category['sales_category_id']) {
											$this->set_gl_account($category['sales_gl_account']);
										}
									}
								}
							}
						}
					}
				}
			}

		}

		public function read() {
			parent::read();

			$GLOBALS['egw']->db->select(self::sales_order_line_table,
				array('*'),
				array(
					'order_id'	=>	$this->order->get_order_id(),
					'line_no'	=>	$this->line_number
				),
				__LINE__, __FILE__
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->discount_percent = $row['discount_percent'];
		}

		/**
		*	Reserve stock from on hand, save order line.
		*/
		public function write() {
			parent::write();

			$GLOBALS['egw']->db->insert(self::sales_order_line_table, 
				array(
					'order_id'		=>	$this->order->get_order_id(),
					'line_no'		=>	$this->line_number,
					'discount_percent'	=>	$this->dicount_percent,
				),
				array(
					'order_id'		=>	$this->order->get_order_id(),
					'line_no'		=>	$this->line_number
				),
				__LINE__, __FILE__, 'perp_orders'
			);
		}


		/**
		*	Un-reserve stock, put it back into on hand inventory
		*/
		public function remove() {
			parent::remove();

			if($this->ship_location) {
				ExecMethod2('perp_inventory.bo_perp_inventory.reserve_stock', $this->ship_location, $this->stock_id, -$this->quantity_ordered, lang('Removed from Sales Order %1', $this->order->get_our_reference_number()));
			}

			// Remove price override infolog entry
			$this->clear_price_override();

			// Need to delete this too
			$GLOBALS['egw']->db->delete(self::sales_order_line_table, 
				array(
					'order_id'		=>	$this->order->get_order_id(),
					'line_no'		=>	$this->line_number
				),
				__LINE__, __FILE__, 'perp_orders'
			);
		}

		/**
		*	Check to make sure people aren't trying to order fractions of things they're not allowed to.
		*/
		public function set_quantity_ordered($quantity) {
			if($this->stock_id) {
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $this->stock_id);
				$multiple = ($stock['economic_order_quantity'] == 0 ? 1 : $stock['economic_order_quantity']);
				if($quantity == 0) {
					$quantity = 1;
				}

				if(abs($quantity) < $multiple) {
					$new_quantity = ($quantity / abs($quantity)) * $multiple;
				} elseif ($quantity % $multiple != 0)  {
					$new_quantity = ceil($quantity / $multiple) * $multiple;
				} else {
					$new_quantity = $quantity;
				}
				if($new_quantity != $quantity) {
					$message = lang('%1 is sold in quantities of %2', $stock['stock_num'], $multiple);
					$quantity = $new_quantity;
				}
			}

			parent::set_quantity_ordered($quantity);

			if($message) {
				return $message;
			} else {
				return true;
			}
		}

		public function set_unit_price($price) {
			if($this->quantity_shipped || $this->quantity_invoiced) {
				return lang('Unable to change unit price, order has been shipped or invoiced.');
			}

			// Unchanged
			if($price == $this->unit_price && !$this->get_price_override()) {
				return true;
			}

			// Check desired price against what it should be
			if($this->order->get_client_account_id() ) {
				$list_price = $this->get_list_price();
				if($list_price == $price) {
					$this->unit_price = $price;
					$this->clear_price_override();
				} elseif ($this->unit_price != $price && $GLOBALS['egw']->acl->check('order', EGW_ACL_CUSTOM_1, 'perp_orders')) {
					// Price override
					// Add an infolog entry for this override
					$infolog_id = $this->create_price_override($list_price, $override_price);
					if($infolog_id) {
						$infolog = ExecMethod('infolog.infolog_bo.read', $infolog_id);
					}
					if(!$infolog['info_des']) {
						$edit_override = True;
					}

					$this->set_price_override($infolog_id);
					$this->unit_price = $price;
					return $edit_override ? true : false;
				} elseif (!$GLOBALS['egw']->acl->check('order', EGW_ACL_CUSTOM_1, 'perp_orders')) {
					return lang('Can\'t override unit price of %1 with %2.  You do not have permission to override prices.', $this->unit_price, $price);
				}
			} else {
				return lang('Client account not set');
			}
		}

		/**
		*	Set the price list used
		*	
		*	@param price_list ID of the price list to use.  Null will remove.
		*/
		public function set_price_list_id($price_list = null) {
			$details = $this->get_details('OrderLineDetail');
			$price_list_detail = null;
			foreach($details as $detail) {
				if($detail->get_name() == 'price_list') {
					$price_list_detail =& $detail;
					break;
				}
			}
			if(is_null($price_list_detail)) {
				$data = array(
					'detail_name'	=>	'price_list',
					'detail_value'	=>	$price_list,
					'display'	=>	false,
				);
				$price_list_detail = new OrderLineDetail($this, $data);
				$this->add_detail($price_list_detail);
			} elseif(!is_null($price_list)) {
				$price_list_detail->set_value($price_list);
			} else {
				$this->remove_detail($price_list_detail);
			}
		}

		/**
		*	Get the price list used for this line.
		*	If not set, it will return the current price list.
		*
		*	@return int price_list_id
		*/
		public function get_price_list_id() {
			$details = $this->get_details('OrderLineDetail');
			$price_list = null;
			foreach($details as $detail) {
				if($detail->get_name() == 'price_list') {
					return $detail->get_value();
				}
			}
			if(!$this->order->get_client_account_id() ) {
				return null;
			}

			$price_lists = ExecMethod2('perp_ar.bo_perp_client_account.get_price_lists', $this->order->get_client_account_id(), $this->order->get_issue_date());

			if(count($price_lists) == 0) {
				return null;
			}
			$query = array(
				'stock_id'	=>	$this->stock_id,
				'price_list_id'	=>	$price_lists,
				'client_account_id'	=>	$this->order->get_client_account_id(),
				'date'		=>	$this->order->get_issue_date(),
				'template'	=>	false,
				'no_readonlys'	=>	True,
				'no_status'	=>	True,
			);

			$bo = CreateObject('perp_orders.bo_perp_price');
			$bo->get_price_list($query, $price_list);

			/*
			*	If there's more than one match, take the first price list where the 
			*	price matches what was entered.  If the price does not match any price list,
			*	take the price list ID of the lowest price.  We should generate an override 
			*	elsewhere.
			*/
			$lowest_price = PHP_INT_MAX;
			$price_list_id = null;
			if(is_array($price_list)) {
				foreach($price_list as $price) {
					if($price['price_list_id'] && $price['price'] == $this->get_unit_price()) {
						$price_list_id = $price['price_list_id'];
						break;
					}
					// Got some price lists, but the price doesn't match.  Take the one with the lowest price
					if($price['price_list_id'] && $price['price'] < $lowest_price) {
						$lowest_price = $price['price'];
						$price_list_id = $price['price_list_id'];
					}
				}
			}
			$this->set_price_list_id($price_list_id);
			return $price_list_id;
		}

		/**
		*	Get the unit price of this item according to the price list
		*	@return float list price
		*/
		public function get_list_price() {
			$query = array(
				'stock_id'	=>	$this->stock_id,
				'price_list_id'	=>	$this->get_price_list_id(),
				'client_account_id'	=>	$this->order->get_client_account_id(),
				'date'		=>	$this->order->get_issue_date(),
				'template'	=>	false,
				'no_readonlys'	=>	True,
				'no_status'	=>	True,
			);

			$bo = CreateObject('perp_orders.bo_perp_price');
			$bo->get_merged_price_list($query, $price_list);

			if(is_array($price_list[0]) && isset($price_list[0]['price'])) {
				$base_price = $price_list[0]['price'];
			}

			// All work orders for this line have to have the same options, so we grab the first one
			$work_orders = $this->get_details('WorkOrderDetail');
			
			if(is_array($work_orders) && count($work_orders) > 0) {
				// Get detail pricing
				$query['work_order_id']	=	$work_orders[0]->get_work_order_id();
				$query['detail_id']	=	$work_orders[0]->get_work_order_detail_id();

				$so_bo = CreateObject('perp_orders.bo_perp_sales_order');
				$so_bo->get_detail_details($query, $details);
				
				// Total is the last line
				$total = end($details);
				$price = $total['price'];

				// Check for damage
				$controlled_query = array(
					'stock_id'      =>      $this->stock_id,
					'location'      =>      $this->ship_from,
					'work_order_id'	=>	$query['work_order_id'],
					'detail_id'	=>	$query['detail_id'],
					'order'		=>	'perp_stock_damage.stock_history_id',
					'sort'		=>	'DESC',
					'num_rows'	=>	1,
				);

				// Skip the ACL check
				$damage = so_perp_damage::get_damage_list($controlled_query);
				if(is_array($damage[0]) && $damage[0]['discount_percent'] != 0) {
					// Item was damaged, discount the price
					$this->discount_percent = $damage[0]['discount_percent'];
					$discount_price = $price - ($price * $damage[0]['discount_percent']);

					// Record the price change
					$infolog_id = $this->create_price_override($price, $discount_price, ($damage[0]['discount_percent'] * 100) . "%\n". $damage[0]['comment']);
					$this->set_price_override($infolog_id);

					// Link to damage record
					if($this->order->get_order_id()) {
						$link_id = 'DAMAGE-' . $damage[0]['stock_history_id'];
					}
					egw_link::link('perp_inventory', $link_id, 'infolog', $infolog_id);

					$price = $discount_price;
				}
				return $price;
			} else {
				// No options
				return $base_price;
			}
		}

		/** 
		*	Using WorkOrderDetail information, add in OrderLineDetail objects for 
		*	work order options.  These shouldn't get written.
		*
		*	@param classname Restrict to only classes of the given name
		*/
		public function &get_details($classname = null) {
			$details =& parent::get_details($classname);
			if($classname) {
				return $details;
			}

			// All work orders should have the same options
			$work_orders =& $this->get_work_orders();
			if(isset($work_orders[0])) {
				// Grab the first work order and use it to get the options
				$work_order =& $work_orders[0];
			} else {
				// No work orders, so nothing special needed
				return $details;
			}

			// Look for serial numbers for the work orders
			$controlled_query = array(
				'stock_id'      =>      $this->stock_id,
				'location'      =>      $this->ship_from,
			);
			
			foreach($work_orders as $wo) {
				$controlled_query['work_order_id'] = $wo->get_work_order_id();
				$controlled_query['detail_id'] = $wo->get_work_order_detail_id();
				$controlled_list = array();
				
				bo_perp_inventory::get_controlled_inventory_list($controlled_query, $controlled_list);
				foreach($controlled_list as $serial) {
					// Serial number needed, set from work order
					$serial = new OrderLineDetail($this, array(
						'detail_name'   =>      'serial number',
						'detail_value'  =>      $serial['serial_number']
					));
					// Bad.  This variable isn't declared
					$serial->quantity = $wo->get_work_order_quantity();

					// Check for duplicates, don't add them
					$duplicate = false;
					foreach($details as $detail) {
						if($detail->get_name() == $serial->get_name() &&
							$detail->get_value() == $serial->get_value()) {
								continue 2;
						}
					}
					$details[] = $serial;
				}
			}

			// Check for options to see if we need to display them
			$allowed_build_option_list_query['stock_id'] = $this->get_stock_id();
			$allowed_build_option_list_query['value'] = array(-2,-3);
			$allowed_build_option_list_query['as_of_date'] = $this->order->get_issue_date();
			$options = ExecMethod2('perp_manufacturing.bo_perp_build_option.get_allowed_build_option_list', $allowed_build_option_list_query);
			
			if(count($options) == 0) {
				// No options
				return $details;
			}

			// Check for options, and add them in so they can be displayed
			$query = array(
				'client_account_id'	=>	$this->get_owner()->get_client_account_id(),
				'work_order_id'	=>	$work_order->get_work_order_id(),
				'detail_id'	=>	$work_order->get_work_order_detail_id(),
				'stock_id'	=>	$this->get_stock_id(),
				'date'		=>	$this->order->get_issue_date(),
			);
			if($this->get_price_list_id()) {
				$query['price_list_id'] = $this->get_price_list_id();
			}

			$bo_order = new bo_perp_sales_order();
			$bo_order->get_detail_details($query, $rows);
			foreach($rows as $option) {
				$option['detail_name'] = $option['build_option_category_name'];
				$option['detail_value'] = $option['options-value'][$option['value']]['build_option_name'];
				if($option['build_option_category_id'] == bo_perp_sales_order::TOTAL_LINE) {
					$option['detail_value'] = $option['price'];
				} elseif ($option['build_option_category_id'] == bo_perp_sales_order::DAMAGE_LINE) {
				}

				// Check for duplicates, don't add them
				$duplicate = false;
				foreach($details as $detail) {
					if($detail->get_name() == $option['detail_name'] &&
						$detail->get_value() == $option['detail_value']) {
							$duplicate = true;
					}
				}
				if(!$duplicate) {
					$details[] =& new OrderLineDetail($this, $option);
				}
			}
			return $details;
		}

		/**
		*	Get the number of days required before this item can be
		*	shipped out.  Does not include any time required to actually
		*	get it ready for shipping, such as picking or packing.
		*
		*	@todo Check WorkOrderDetails.  They need to be at least started...
		*/
		public function get_lead_time() {
			// Check inventory
			$inventory = ExecMethod2('perp_inventory.bo_perp_stock.get_inventory', $this->stock_id);
			if($inventory['total'] <= $this->quantity_ordered) {
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $this->stock_id);
				if($stock['pseudo']) {
					return 0;
				}

				// Backordered
				// TODO: Do some figuring and look at incoming shipments and in-progress work orders and get a better number
				return 30; // Days
			} else {
				return 1;
			}
		}

		public function &get_work_orders() {
			return parent::get_details('WorkOrderDetail');
		}

		public function add_work_order($work_order_id, $detail_no, $quantity) {
			$detail = new WorkOrderDetail($this, array(
				'detail_name'	=>	'work_order', 
				'detail_value'	=>	$work_order_id
			));
			$detail->set_work_order_detail_id($detail_no);
			$result = $detail->set_work_order_quantity($quantity);
			if($result !== true) {
				return $result;
			}

			$this->add_detail($detail);
		}

		public function remove_work_order($work_order_id, $detail_no) {
			$work_orders =& $this->get_details('WorkOrderDetail');

			// Remove from work order list
			foreach($work_orders as $work_order) {
				if($work_order->get_work_order_id() == $work_order_id && $work_order->get_work_order_detail_id() == $detail_no) {
					$this->remove_detail($work_order->get_number());
					break;
				}
			}
		}

		/**
		*	Create a price override
		*/
		protected function create_price_override($list_price, $override_price, $reason = '') {
			$config = config::read('perp_orders');

			$price_override_id = $this->get_price_override();
			if($price_override_id) {
				$infolog = ExecMethod('infolog.infolog_bo.read', $price_override_id);
			}
			if(!is_array($infolog)) {
				$infolog = array();
			}
			$override = array(
				'info_subject'	=>	lang('Price Override on %1', $this->order->get_our_reference_number()),
				'info_type'	=>	$config->config_data['price_override_type'],
				'info_cat'	=>	$config->config_data['price_override_category'],
				'info_des'	=>	($infolog['info_des']) ? $infolog['info_des'] : str_replace('<br />', "\n", lang('price_override_desc', 
					$this->order->get_our_reference_number(), 
					$this->line_number + 1,
					ExecMethod('perp_inventory.bo_perp_stock.link_title', $this->stock_id),
					ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $this->order->get_order_currency(), $list_price),
					ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $this->order->get_order_currency(), $override_price),
					$this->get_price_list_id()
				)) . $reason,
				'info_owner'	=>	$GLOBALS['egw_info']['user']['account_id'],
				'info_link'	=>	isset($infolog['info_link']) ? $infolog['info_contact'] : 'addressbook:' . $GLOBALS['egw_info']['user']['person_id'],
				'info_contact'	=>	isset($infolog['info_link']) ? $infolog['info_contact'] : 'addressbook:' . $GLOBALS['egw_info']['user']['person_id'],
				'info_status'	=>	'done',
				'info_price'	=>	$price,
				'info_percent'	=>	100,
			);
			$infolog = array_merge($override, $infolog);
			$infolog_id = ExecMethod('infolog.infolog_bo.write', $infolog);
			$infolog = ExecMethod('infolog.infolog_bo.read', $infolog_id);
		
			if($this->order->get_order_id()) {
				$link_id = 'SALES_ORDER-' . $this->order->get_order_id() . '-' . $this->line_number;
			}

			egw_link::link('perp_orders', $link_id, 'infolog', $infolog_id);
			if(!$infolog['info_link_id']) {
				egw_link::link('addressbook', $GLOBALS['egw_info']['user']['person_id'], 'infolog', $infolog_id);
				$infolog['info_link_id'] = $contact;
				$infolog_id = ExecMethod('infolog.infolog_bo.write', $infolog);
			}

			return $infolog_id;
		}

		/**
		*	Get any price override(s) on this OrderLine.
		*	There should be at most 1.
		*	@return ID number of the infolog entry for the override.
		*/
		public function get_price_override() {
			// This call will get us a list of all price overrides.  Should only be one
			$price_override = $this->get_detail('price_override');
			
			// Grab the current detail from the list
			$price_override = array_pop($price_override);

			if($price_override) {
				return $price_override->get_value();
			} else {
				return null;
			}
		}

		/**
		*	Set a price override as the given infolog_id.
		*	Creates the OrderLineDetail to preserve the information
		*
		*	@param infolog_id ID of the infolog entry for the override
		*/
		public function set_price_override($infolog_id) {
			if(is_null($this->get_price_override())) {
				$price_override = new OrderLineDetail($this, 
					array(
						'detail_name'	=>	'price_override',
						'detail_value'	=>	$infolog_id,
						'display'	=>	false
					)
				);
				$this->add_detail($price_override);
			}
		}

		/**
		*	Clear any price overrides on this line.  This will 
		*	remove the OrderLineDetail, and delete the infolog entry
		*/
		public function clear_price_override() {
			$price_override = $this->get_detail('price_override');
			foreach($price_override as $detail_id => $override) {

				// Clear the infolog type so it will be allowed to be deleted.
				$infolog = ExecMethod('infolog.infolog_bo.read', $override->get_value());
				$infolog['info_type'] = null;
				ExecMethod('infolog.infolog_bo.write', $infolog);

				// Delete override
				ExecMethod2('infolog.infolog_bo.delete', $override->get_value(), True);
				$this->remove_detail($detail_id);
			}
		}

		/**
		*	Can this SalesOrderLine be shipped?
		*	We need to check on stock levels and any
		*	work orders.  We only care about partially
		*	shipping right now, not if the whole thing can go.
		*
		*	@return boolean true or reason why not
		*/
		public function can_ship() {
			$test = parent::can_ship();
			if(!is_bool($test)) {
				return $test;
			}

			// Do we know where we're selling from?
			if($this->ship_location) {
				// We should have reserved stock there
				$inventory = ExecMethod2('perp_inventory.bo_perp_stock.get_inventory', $this->stock_id, $this->ship_location);
				if($inventory[0]['quantity_reserved'] < ($this->get_quantity_ordered() - $this->get_quantity_shipped())) {
					return lang('Not enough inventory reserved at %1 to fill this order.', $inventory[0]['location_name']);
				}
			} else {
				// Check for inventory (everywhere)
				$inventory = ExecMethod2('perp_inventory.bo_perp_stock.get_inventory', $this->stock_id);
				if($inventory['total'] < ($this->get_quantity_ordered() - $this->get_quantity_shipped())) {
					return lang('Not enough inventory to fill this order.');
				}
			}

			// Check work order status
			$details =& $this->get_details('WorkOrderDetail');
			$work_orders = (count($details) == 0);
			foreach($this->get_details('WorkOrderDetail') as $detail) {
				$wo = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order', $detail->get_work_order_id());
				if($wo['start_date']) {
					$work_orders = $work_orders || true;
				}
			}
			return $test && $work_orders;
		}

		public function ship(ShipmentLine &$shipment_line) {
			if(!$qty) {
				return;
			}

			// Set quantity shipped. 
			$this->quantity_shipped += $qty;
			$this->shipped = $qty;
			$this->shipment_id = $shipment_id;
			
			// Close this line if we've received & invoiced all of them
			if($this->get_quantity_shipped() == $this->quantity_ordered) {
				$this->ship_date = date('U');
				if($this->get_quantity_invoiced() == $this->quantity_ordered) {
					$this->completed = True;
				}
			}
		}

		public function get_status() {
			$list = parent::get_status();

			$status = array();
			foreach($list as $item) {
				$status[] = $item + array(
					'history_name'	=>	ucfirst(strtolower($item['type'])),
					'reference'	=>	array(
						'app'	=>	'perp_ar',
						'id'	=>	$item['type'] . '-' . $item['reference_id'],
						'title'	=>	$item['our_reference_number']
					)
				);
			}
			return $status;
		}

		/**
		*	Return a list of extra details to be displayed on the invoice, below
		*	the line.  This list cannot be edited, and is an array of details in
		*	the form ('name' => 'Detail name', 'value' => '123')
		*
		*	@param filter The filter parameter from the display select (usually true)
		*	@return array
		*/
		public function get_invoice_details($filter) {
			$details = array();
			
			// Work orders
			$work_orders =& $this->get_details('WorkOrderDetail');
			foreach($work_orders as $wo) {
				$wo_array = $wo->toArray();
				$details[] = array(
					'name'	=>	lang('Work Order'),
					'value'	=>	$wo_array['work_order_reference'] . ($wo->get_work_order_detail_id() != 1 ? $wo->get_work_order_detail_id() : '')
				);
			}
			if($filter > 1) {
				$detail_list =& $this->get_details();
				foreach($detail_list as &$detail) {
					if($detail instanceof WorkOrderDetail || !$detail->get_display()) {
						continue;
					}
					$details[] = array(
						'name'	=>	$detail->get_name(),
						'value'	=>	$detail->get_value()
					);
				}
			}
			return $details;
		}

		public function get_link() {
			return array(array(
				'app'	=>	'perp_orders',
				'id'	=>	'ORDER-' . $this->order->get_order_id()
				)
			);
		}

		public function toArray() {
			$data = parent::toArray() + array(
				'discount_percent'	=>	$this->discount_percent,
			);
			$price_override_id = $this->get_price_override();
			if($price_override_id) {
				$data += array(
					'price_override_id'	=>	$price_override_id,
					'price_override_icon'	=>	'flag'
				);
			}
			return $data;
		}
	}
?>
