<?php

	/**
	*	OrderLineDetail to store work order options for a SalesOrder.
	*
	*	@ingroup perp_orders
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.OrderLineDetail.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_order.inc.php');

	class WorkOrderDetail extends OrderLineDetail {
		
		/**
		*	ID number of the detail on the work order
		*/
		protected $work_order_detail_id = 0;

		/**
		*	Quantity of the work order detail that the owning
		*	SalesOrderLine uses
		*/
		protected $work_order_quantity = 0;

		/**
		*	Quantity of the work order detail that has been returned 
		*	for some reason
		*/
		protected $returned_quantity = 0;

		/**
		*	Static bo_work_order for checking
		*/
		protected static $bo_work_order;

		/**
		*	Constructor.  
		*	Name is set to 'work_order' (regardless of what is passed in).
		*	work_order_id is set to value
		*/
		public function __construct(OrderLine $line, array $data) {
			if(!is_object(self::$bo_work_order)) {
				self::$bo_work_order = new bo_perp_work_order();
			}
			$data['detail_name'] = 'work_order';

			parent::__construct($line, $data);

			$this->set_work_order_id((int)$data['detail_value']);
		}
		
		/**
		*	Function called when this WorkOrderDetail is added to the OrderLine
		*/
		public function add() {
		}

		/**
		*	Remove the reservation on the work order
		*/
		public function remove() {
			parent::remove();
			$GLOBALS['egw']->db->delete('perp_sales_order_work_order',
				array(
					'order_id'		=>	$this->order_line->get_owner()->get_order_id(),
					'line_no'		=>	$this->order_line->get_line_number(),
					'detail_no'		=>	$this->detail_number
				),
				__LINE__, __FILE__
			);
		}

		/**
		*	Read extra WorkOrderDetail specific data from the database
		*/
		public function read() {
			parent::read();
			$GLOBALS['egw']->db->select('perp_sales_order_work_order',
				array('*'),
				array(
					'order_id'		=>	$this->order_line->get_owner()->get_order_id(),
					'line_no'		=>	$this->order_line->get_line_number(),
					'detail_no'		=>	$this->detail_number
				),
				__LINE__, __FILE__
			);
			$row = $GLOBALS['egw']->db->row(true);
			$this->work_order_detail_id	=	$row['work_order_detail_id'] ? $row['work_order_detail_id'] : 1;
			$this->work_order_quantity	=	$row['quantity'];
			$this->returned_quantity	=	$row['quantity_returned'];
		}

		/**
		*	Write extra WorkOrderDetail specific data into the database
		*/
		public function write() {
			if($this->get_work_order_quantity() <= 0) {
				return;
			}

			parent::write();
			$GLOBALS['egw']->db->insert('perp_sales_order_work_order', 
				array(
					'order_id'		=>	$this->order_line->get_owner()->get_order_id(),
					'line_no'		=>	$this->order_line->get_line_number(),
					'detail_no'		=>	$this->detail_number,
					'work_order_detail_id'	=>	$this->work_order_detail_id,
					'quantity'		=>	$this->work_order_quantity,
					'quantity_returned'	=>	$this->returned_quantity
				),
				array(
					'order_id'		=>	$this->order_line->get_owner()->get_order_id(),
					'line_no'		=>	$this->order_line->get_line_number(),
					'detail_no'		=>	$this->detail_number
				),
				__LINE__, __FILE__, 'perp_orders'
			);
		}

		/**
		*	Can this Detail be edited?
		*
		*	@return boolean
		*/
		public function can_edit() {
			$can_edit = parent::can_edit();
			return $can_edit;
		}

		/**
		*	Get the ID of the work order this Detail references
		*/
		public function get_work_order_id() {
			return $this->detail_value;
		}

		/**
		*	Set the ID of the work order this detail references
		*	@param id ID number of the work order
		*/
		public function set_work_order_id($id) {
			if($this->check($id)) {
				$this->detail_value = $id;
				return true;
			} else {
				$work_order = self::$bo_work_order->get_work_order($id);
				return lang('Unable to set work order.  Given work order %1 is not for the correct item.', $work_order['work_order_reference']);
			}
		}

		/**
		*	Check to make sure the work_order_id & detail_id are good
		*
		*	@param work_order_id
		*	@param detail_id
		*
		*	@return boolean
		*/
		protected function check($work_order_id, $detail_id = null) {
			// Check to make sure this work order actually contains a detail for this line's stock_id
			if(!is_object(self::$bo_work_order)) {
				self::$bo_work_order = new bo_perp_work_order();
			}
			self::$bo_work_order->get_work_order_detail_list(
				array(
					'work_order_id'	=>	$work_order_id,
					'detail_id'	=>	$detail_id,
					'template'	=>	false,
				), 
				$details
			);
			foreach($details as $detail) {
				if((!$detail_id || $detail['detail_id'] == $detail_id) && $detail['stock_id'] == $this->order_line->get_stock_id()) {
					return true;
				}
			}
			return false;
		}

		/**
		*	Get the ID of the work order detail line this Detail references
		*/
		public function get_work_order_detail_id() {
			return $this->work_order_detail_id;
		}

		/**
		*	Set the ID of the work order detail line this detail references
		*	@param id ID number of the work order detail line
		*/
		public function set_work_order_detail_id($id) {
			if($this->check($this->get_work_order_id(), $id)) {
				$this->work_order_detail_id = $id;
				return true;
			} else {
				$work_order = self::$bo_work_order->get_work_order($id);
				return lang(
					'Unable to set work order.  Given work order %1 detail %2 is not for the correct item.', 
					$work_order['work_order_reference'],
					$id
				);
			}
		}

		/**
		*	Get the quantity of the work order detail the owning SalesOrderLine
		*	consumes.
		*/
		public function get_work_order_quantity() {
			return $this->work_order_quantity;
		}

		/**
		*	Set the quantity of the work order detail the owning SalesOrderLine
		*	consumes.
		*	@param quantity Quantity of the work order detail line used by the owning SalesOrderLine
		*	@return True or error message
		*/
		public function set_work_order_quantity($quantity) {
			$work_order_detail = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_detail', $this->get_work_order_id(), $this->get_work_order_detail_id());
			if(!$work_order_detail) {
				return lang('Could not find a matching work order line');
			} elseif($work_order_detail['stock_id'] != $this->order_line->get_stock_id()) {
				// Check that stock matches
				return lang('Work order is for a different stock item (%1)', ExecMethod('perp_inventory.bo_perp_stock.link_title', $work_order_detail['stock_id']));
			} elseif ($quantity <= 0) {
				return lang('Tried to add 0 from work order');
			} elseif ($quantity > $work_order_detail['quantity']) {
				return lang('Tried to take more than the work order detail had (%1 of %2)', $quantity, $work_order_detail['quantity']);
			}

			$this->work_order_quantity = $quantity;
			return true;
		}

		/**
		*	Get the quantity that has been returned, and freed for resale
		*/
		public function get_returned_quantity() {
			return $this->returned_quantity;
		}

		/**
		*	Set the quantity that has been returned, and freed for resale
		*	consumes.
		*	@param quantity Quantity of the work order detail line returned
		*	@return True or error message
		*/
		public function set_returned_quantity($qty) {
			if($qty > $this->work_order_quantity) {
				return lang('Tried to return too many.  Originally sold %1, tried to return %2', $this->work_order_quantity, $qty);
			}
			$this->returned_quantity = $qty;
			return true;
		}

		/**
		*	Get the reference string for this work order
		*
		*	@return string
		*/
		public function get_work_order_reference() {
			return ExecMethod('perp_manufacturing.bo_perp_work_order.link_title', $this->detail_value);
		}

		/**
		*	Return an array of this object's public properties
		*/
		public function toArray() {
			$data = parent::toArray();
			$data += array(
				'work_order_id'		=>	$this->detail_value,
				'work_order_reference'	=>	$this->get_work_order_reference(),
				'work_order_detail_id'	=>	$this->work_order_detail_id,
				'work_order_quantity'	=>	$this->work_order_quantity,
			);

			if($GLOBALS['egw']->acl->check('work_order', EGW_ACL_READ, 'perp_manufacturing')) {
				$data['status'] = ExecMethod2('perp_manufacturing.bo_perp_work_order.get_work_order_status', $this->detail_value, $this->work_order_detail_id);
			}

			$data['readonly'] = array();
			if(!$GLOBALS['egw']->acl->check('work_order', EGW_ACL_EDIT, 'perp_manufacturing')) {
				$data['readonly']['edit'] = true;
			}
			if($this->get_order_line()->get_quantity_invoiced() != 0) {
				$data['readonly']['work_order_reference'] = true;
			}
			return $data;
		}

		public function __toString() {
			$string = parent::__toString();
			$string .= 'WorkOrder Line: ' . $this->work_order_detail_id . "\tQuantity: " . $this->work_order_quantity . "\n";
			return $string;
		}

		/**
		*	Prepare this object for serialization
		*/
		public function __sleep() {
			return( array_keys( get_object_vars( $this ) ) );
		}
	}
?>
