<?php

	/**
	*	Business / Logic layer for dealing with pricing on stock kits
	*
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_orders/inc/class.so_perp_kit_price.inc.php');

	class bo_perp_kit_price {

		public static function get_price($price_list_id, $stock_id, $component_id, $effective_date) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				return so_perp_kit_price::get_price($price_list_id, $stock_id, $component_id, $effective_date);
			} else {
				return false;
			}
		}

		public static function set_price($price_list_id, $stock_id, $kit_stock_id, $component_id, $price, $effective_date) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {

				/**
				*	Allow 5 minutes into the past to ease user interface issues, 
				*	but below we'll still set it to time(), not in the past
				*/
				if(time() - $effective_date >= 5 * 60) {
					return lang('Effective date for price changes (%1) must be in the future',
						date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $effective_date));
				}
				if($effective_date == null || $effective_date < time()) {
					$effective_date = time();
				}

				if(strtoupper($price) === strtoupper(so_perp_price::NOT_ALLOWED) || trim($price) == '') {
				      $price = null;
				} elseif(!is_null($price)) {
					$price = ExecMethod('perp_api.bo_perp_api.get_float_from_string', $price);
				}

				if(!$kit_stock_id) {
					$kit_stock_id = $stock_id;
				}

				/**
				*	Check current price, don't change it if it's the same
				*/
				$current_price = so_perp_kit_price::get_price($price_list_id, $stock_id, $kit_stock_id, $component_id, $effective_date);
				if(isset($current_price) && ($current_price == $price ||
					($current_price == so_perp_price::NOT_ALLOWED && $price == null)
				)) {
					// Price hasn't changed
					return true;
				}
				return so_perp_kit_price::set_price($price_list_id, $stock_id, $kit_stock_id, $component_id, $price, $effective_date);
			}
		}
	}
?>
