<?php

	class bo_perp_orders {

		private $so;

		public function bo_perp_orders() {
			$this->so = CreateObject('perp_orders.so_perp_orders');
		}

		public function get_sales_categorys(&$query = array(), &$rows = array(), &$readonly = array()) {
			return $this->get_sales_categories($query, $rows, $readonly);
		}

		public function get_sales_categories($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('sales_category', EGW_ACL_READ, 'perp_orders')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';

			$list = $this->so->get_sales_categories($query);

			if(!isset($query['template'])) {
				foreach($list as $row) {
					$rows[$row['sales_category_id']] = $row['abbreviation'] . $row['indent'] . '-' . $row['sales_category_name'];
				}
				return $rows;
			} else {
				$rows = $list;
				$rows['add'] = $GLOBALS['egw']->acl->check('sales_category', EGW_ACL_ADD, 'perp_orders');
				$rows['edit'] = $GLOBALS['egw']->acl->check('sales_category', EGW_ACL_EDIT, 'perp_orders');
				$rows['delete'] = $GLOBALS['egw']->acl->check('sales_category', EGW_ACL_DELETE, 'perp_orders');
				return $query['count'];
			}
		}

		/**
		*	Get a single sales category
		*	Includes eGroupWare category info
		*	@param category_id ID of the category to get
		*/
		public function get_sales_category($category_id) {
			if(!is_object($this->categories)) {
				$this->categories =& CreateObject('phpgwapi.categories', -1, 'perp_orders');
			}

			$category = array();

			// Get eGW category data
			list($category) = $this->categories->return_single($category_id);
			$category['data'] = unserialize($category['data']);

			// Get list of parents, excluding this category
			$categories = $this->get_sales_categories();
			unset($categories[$category_id]);
			$category['options-parent'] = $categories;

			// Get pERP category data
			$perp_category = $this->so->get_sales_category($category_id);

			return $category + $perp_category;
		}

		/**
		*	Set a sales category
		*	This will also update the eGroupWare category
		*
		*	@param category array of category information
		*/
		public function set_sales_category($category) {
			if(!is_object($this->categories)) {
				$this->categories =& CreateObject('phpgwapi.categories', -1, 'perp_orders');
			}
		
			// Update eGroupWare category
			$category['id'] = $category['category_id'];
			$category['data'] = serialize($category['data']);
			$category['descr'] = $category['description'];
			$category['access'] = 'public';
			if($category['id']) {
				$category['category_id'] = $this->categories->edit($category);
			} else {
				$category['category_id'] = $this->categories->add($category);
			}

			// Update pERP category
			$this->so->set_sales_category($category);

			return $category['category_id'];
		}

		public function delete_sales_category($category_id) {
			if($this->get_sales_category_stock(array('sales_category_id' => $category_id), $stock) > 0) {
				foreach($stock as $stock_item) {
					$use[] = $stock_item['stock_num'];
				}
				return lang('Could not delete, some stock uses this category (%1)', implode(', ', $use));
			}
			if(!is_object($this->categories)) {
				$this->categories =& CreateObject('phpgwapi.categories', -1, 'perp_orders');
			}
			$this->categories->delete($category_id);
			$this->so->delete_sales_category($category_id);
		}

		/**
		*	Get a list of stock items in a given sales category
		*	
		*	This is kind of the opposite view of perp_inventory::bo_perp_stock::get_sales_categories.
		*	@nextmatch
		*/
		public function get_sales_category_stock($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('sales_category', EGW_ACL_READ, 'perp_orders')) {
				$rows = array();
				return 0;
			}
			
			if(!isset($query['order'])) {
				$query['order'] = 'stock_num';
			}

			$rows = $this->so->get_sales_category_stock($query);
			$rows['delete'] = $GLOBALS['egw']->acl->check('sales_category', EGW_ACL_EDIT, 'perp_orders');
			return $query['total'];
		}

		public function get_sales_areas($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('sales_area', EGW_ACL_READ, 'perp_orders')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
					if (is_array($client_ids) && count($client_ids)) {
						// Make sure clients can get their own sales area, it'll cause problems with pricing with no sales areas
						$query['sales_area_id'] = array();
						foreach($client_ids as $client_id) {
							$client = ExecMethod('perp_ar.bo_perp_client.get_client', (int)$client_id);
							$query['sales_area_id'][] = $client['sales_area'];
						}
					} else {
						return array();
					}
				}
			}

			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';
			$query['order'] = $query['order'] ? $query['order'] : 'sales_area_name';

			$list = $this->so->get_sales_areas($query);

			if(!isset($query['template'])) {
				foreach($list as $row) {
					$rows[$row['sales_area_id']] = $row['sales_area_name'];
				}
				return $rows;
			} else {
				$rows = array_slice($list, $query['start'], $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs']);
				$rows['add'] = $GLOBALS['egw']->acl->check('sales_area', EGW_ACL_ADD, 'perp_orders');
				$rows['edit'] = $GLOBALS['egw']->acl->check('sales_area', EGW_ACL_EDIT, 'perp_orders');
				$rows['delete'] = $GLOBALS['egw']->acl->check('sales_area', EGW_ACL_DELETE, 'perp_orders');
				return count($list);
			}
		}

		public function get_sales_types($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('sales_type', EGW_ACL_READ, 'perp_orders')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';
			$query['order'] = $query['order'] ? $query['order'] : 'sales_type_name';

			$list = $this->so->get_sales_types($query);

			if(!isset($query['template'])) {
				foreach($list as $row) {
					$rows[$row['sales_type_id']] = $row['sales_type_name'];
				}
				return $rows;
			} else {
				$rows = $list;
				$rows['add'] = $GLOBALS['egw']->acl->check('sales_type', EGW_ACL_ADD, 'perp_orders');
				$rows['edit'] = $GLOBALS['egw']->acl->check('sales_type', EGW_ACL_EDIT, 'perp_orders');
				$rows['delete'] = $GLOBALS['egw']->acl->check('sales_type', EGW_ACL_DELETE, 'perp_orders');
				return $query['count'];
			}
		}

		/**
		* get title for a perp Orders entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param int/array $entry key to look up or array with supplier entry
		*	key is in the form {SALES_ORDER | PRICE_LIST }:id[:SALES_ORDER_LINE]
		* @param string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				list($location, $id, $line) = explode('-', $entry);
				if($id) {
					switch ($location) {
						case 'SALES_ORDER':
						case 'ORDER':
							return ExecMethod('perp_orders.bo_perp_sales_order.link_title', $entry);
							break;
						case 'PRICE_LIST':
							return ExecMethod('perp_orders.bo_perp_price_list.link_title', $entry);
							break;
					}
				}
			} else {
			}
		}


		/**
		* query perp orders for entries matching $pattern
		*
		* Is called as hook to participate in the linking
		*
		* @param string pattern pattern to search
		* @return array with sales_order_id - title pairs of the matching entries
		*/
		function link_query( $pattern ) {
			$list = ExecMethod('perp_orders.bo_perp_sales_order.get_sales_order_list', array('search' => $pattern));

			$result = array();
			foreach($list as $id => $list_el) {
				$result['ORDER-' . $id] = $this->link_title("ORDER-$id");
			}
			return $result;
		}
	}
?>
