<?php
	class bo_perp_price {
		
		private $so;

		public function bo_perp_price() {
			$this->so = CreateObject('perp_orders.so_perp_price');
		}
		
		/**
                *       Get a list of stock items availible to a price list
                *
                *       query = array(
                *               'price_list_id'         =>      Search a particular price list
                *               'stock_id'              =>      Search for prices for this stock item
                *               'search'                =>      Search on stock_code & short_desc
                *               'cat_id'     =>      Restrict search to sales category (same as cat_id)
                *               'sales_area_id'         =>      Restrict search to sales area
                *               'sales_type_id'         =>      Restrict search to sales type
                *               'date'                  =>      Restrict to effective date
                *               'include_not_allowed'   =>      true = include all stock in sales category, false = include only stock with a price
                *               'filter2'               =>      Same as include_not_allowed
                *       )
                */	
		public function get_price_list_stock_items($query= array()) {
			$list = array();	
			$price_list_stock_items = $this->so->get_price_list_stock_items($query);
			foreach($price_list_stock_items as $stock_item) {
				$list[$stock_item['stock_id']] = ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_item['stock_id']);;
			}
			return $list;	
		}


		/**
		*	Get a list of prices for saleable stock items
		*
		*	This function may return more than one price per stock item.  Use
		*	get_merged_price_list to get a list of prices with only one price
		*	per stock item.
		*	
		*	@nextmatch
		*
		*	query = array(
		*		'price_list_id'		=>	Search a particular price list
		*		'stock_id'		=>	Search for prices for this stock item
		*		'search'		=>	Search on stock_code & short_desc
		*		'sales_category_id'	=>	Restrict search to sales category (same as cat_id)
		*		'filter'		=>	Show out of stock discontinued items (set to true to display them)
		*		'sales_area_id'		=>	Restrict search to sales area
		*		'sales_type_id'		=>	Restrict search to sales type
		*		'date'			=>	Restrict to effective date
		*		'include_not_allowed'	=>	true = include all stock in sales category, false = include only stock with a price
		*		'filter2'		=>	Same as include_not_allowed
		*		'include_options'	=>	Include build options.  Defaults to true.
		*	)
		*/
		public function get_price_list(&$query= array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			$query['include_not_allowed'] = $query['filter'] ? $query['filter'] : $query['include_not_allowed'];

			if(array_key_exists('filter2',$query)){	
				$price_list_stock_items_query = array(
                                                        'price_list_id' => $query['price_list_id'],
                                                        'date' => $query['date'],
                                                        'order' => 'short_description',
                                                        'include_not_allowed' => $query['include_not_allowed']
                                );
				$price_list_stock_items = $this->get_price_list_stock_items($price_list_stock_items_query);
				$filter2['none'] = 'Select Stock(none)';
                        	//$filter2['all'] = 'All Stock Items';
                        	$filter2['raw'] = 'All Raw Stock';
                        	$filter2 = $filter2 + $price_list_stock_items;

				$query['options-filter2'] = $filter2;
			}
			$query['sales_category_id'] = $query['cat_id'] ? $query['cat_id'] : $query['sales_category_id'];
			$query['sort'] = $query['sort'] ? $query['sort'] : 'DESC';
			$query['order'] = $query['order'] ? $query['order'] : 'price';
			if(!array_key_exists('include_options', $query)) {
				$query['include_options'] = true;
			}
			$list_view = $this->so->get_created_price_list($query, $readonly);
			$list = $this->so->get_price_list($query, $readonly);
			if(!isset($query['template'])) {
				foreach($list as $key => $row) {
					$rows[$row['stock_id']] = $row['stock_num'] . ' - ' . $row['description'];
				}
				return $rows;
			} else {
				$edit = $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders');
				if (!$query['start']) {
					$query['start'] = 0;
				}
				if(!$query['num_rows']) {
					$query['num_rows'] = $GLOBALS['egw_info']['user']['preferences']['common']['maxmatchs'];
				}
				$rows = array_slice($list, $query['start'], $query['num_rows']);

				$rows['edit'] = $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders');
				$rows['delete'] = $GLOBALS['egw']->acl->check('price_list', EGW_ACL_DELETE, 'perp_orders');
			}
			return count($list);	
		}

		/**
		*	Get a list of prices for build options
		*/
		public function get_option_price_history(&$query = array(), &$rows = array(), $readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				$rows = array();
				return 0;
			}

			$rows = $this->so->get_option_price_history($query);

			return $query['total'];
		}

		/**
		*	Set option template if an option has sub-options.
		*	Works recursively.
		*/
		public function add_sub_option_template($query, &$options, &$readonly) {
			foreach($options as &$option_category) {
				if(!is_array($option_category['build_option_prices'])) {
					continue;
				}
				foreach($option_category['build_option_prices'] as &$option) {
					if(is_array($option['options'])) {
						$option['option_template'] = $query['option_template'];
						$this->add_sub_option_template($query, $option['options'], $readonly);
					}
				}
			}
		}

		/**
		*	Get a list of prices with only one entry per stock item.
		*	
		*	Calls get_price_list to get a list of prices, then filters them
		*	so there is only one price per stock item.  If there is more than
		*	one price per stock item, the lowest price is kept.
		*
		*	@see get_price_list()
		*/
		public function get_merged_price_list($query = array(), &$rows = array(), &$readonlys = array()) {
			if($query['format']) {
				// Formatting causes problems when we compare
				$format = true;
				unset($query['format']);
			}
			
			// Shuffle around query keys, filter2 is now reserved for filtering
			if(array_key_exists('filter2', $query)) {
				$query['include_not_allowed'] = $query['filter2'];
				unset($query['filter2']);
			}
			$query['count'] = $this->get_price_list($query, $rows, $readonlys);

			$merged = array();
			$keep = array();
			foreach($rows as $key => $price) {
				if(!is_array($price)) {
					$keep[$key] = $price;
					continue;
				}
				if(is_numeric($price['price']) && (!isset($merged[$price['stock_id']]) || $merged[$price['stock_id']]['price'] > $price['price'])) {
					$merged[$price['stock_id']] = $price;
				} else {
					$query['count']--;
				}
			}

			if($format) {
				foreach($merged as &$item) {
					$item['price'] = number_format($item['price'], 2);
					if(is_array($item['components'])) {
						foreach($item['components'] as &$component) {
							$component['price'] = number_format($component['price'], 2);
						}
					}
					if(is_array($item['options'])) {
						foreach($item['options'] as &$category) {
							foreach($category['build_option_prices'] as &$option) {
								$option['price'] = number_format($option['price'], 2);
							}
						}
					}
				}
			}
			$rows = $keep + array_values($merged);
			return $query['count'];
		}

		/**
		*	Set the price for a stock item, build option, or default price for a build option
		*
		*	@param price_list_id ID of the price list this price is for.
		*	@param stock_id ID of the stock item the price is for, or null if this is a
		*		default price for a build option
		*	@param price New price, or so_perp_price::NOT_ALLOWED if the stock item or option is not
		*		permitted on this price list.
		*	@param effective_date The date the new price comes into effect.  This must be in the future.
		*	@param build_option_id If this is the price for a build option, the ID of the build option
		*/
		public function set_price($price_list_id, $stock_id = null, $price = null, $effective_date = null, $build_option_id='', $parent_component_id = 0) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {

				/**
				*	Allow 5 minutes into the past to ease user interface issues, 
				*	but below we'll still set it to time(), not in the past
				*/
				if(time() - $effective_date >= 5 * 60) {
					return lang('Effective date for price changes (%1) must be in the future',
						date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $effective_date));
				}
				if($effective_date == null || $effective_date < time()) {
					$effective_date = time();
				}

				if(strtoupper($price) === strtoupper(so_perp_price::NOT_ALLOWED) || trim($price) == '') {
				      $price = null;
				} elseif(!is_null($price) && $price != 0) {
					$price = ExecMethod('perp_api.bo_perp_api.get_float_from_string', $price);
				}

				/**
				*	Check current price, don't change it if it's the same
				*/
				$query = array(
					'price_list_id'	=>	$price_list_id,
					'stock_id'	=>	$stock_id,
					'parent_component_id'	=>	$parent_component_id,
					'date'		=>	$effective_date,
					'include_not_allowed'	=>	true,
					'template'	=>	false
				);
				if($build_option_id) {
					$query['build_option_id'] = $build_option_id;
					$query['parent_component_id'] = $parent_component_id;
					$current = $this->so->get_build_option_price($query);
					
					if(isset($current[0]['price'])) {
						$current_price = $current[0]['price'];
					}

				} else {
					$this->get_price_list($query, $current);
					if(isset($current[0]['price'])) {
						$current_price = $current[0]['price'];
					}
				}
				
				if(isset($current_price) && ($current_price == $price || ($current_price == so_perp_price::NOT_ALLOWED && $price == null))) {
					// Price hasn't changed
					return true;
				}
				return $this->so->set_price($price_list_id, $stock_id, $price, $effective_date, $build_option_id, $current_price, $parent_component_id);
			}
		}
	}
?>
