<?php

	class bo_perp_price_list {
		
		private $so;

		public function bo_perp_price_list() {
			$this->so = CreateObject('perp_orders.so_perp_price_list');
		}

		public function get_price_list_list($query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')){
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0) {
				// Restrict clients to only their price lists
				$query['client_id'] = $client_ids;
			}

			$query['sales_category_id'] = $query['sales_category_id'] ? $query['sales_category_id'] : $query['cat_id'];
			$query['status'] = $query['status'] ? $query['status'] : $query['filter2'];
			$query['sort'] = $query['sort'] ? $query['sort'] : 'ASC';
			$query['order'] = $query['order'] ? $query['order'] : 'price_list_id';
			if($query['status']) {
				$query['status_date'] = $query['date'] ? $query['date'] : date('U');
			}
			$list = $this->so->get_price_list_list($query);

			if(!isset($query['template'])) {				
				foreach($list as $price_list) {
					if(is_array($price_list)) {
						if(isset($query['client_id'])) {
							// Hide some details from clients
							$rows[$price_list['price_list_id']] = 
								$price_list['cat_name'] . '-' . 
								$price_list['price_list_title'];
						} else {
							// Full details for non-client users
							$rows[$price_list['price_list_id']] = 
								$price_list['cat_name'] . '-' . 
								$price_list['sales_area_name'] . '-' .
								$price_list['sales_type_name'] . '-' .
								$price_list['currency'] . ' (' .
								$price_list['price_list_name'] . ')';
						}
					}
				}					
				return $rows;
			} else {
				$rows = $list;
				
				// Hide some details from clients
				if(is_array($client_ids) && count($client_ids) > 0) {
					foreach($rows as &$price_list) {
						unset($price_list['price_list_name']);
						unset($price_list['sales_area_name']);
						unset($price_list['sales_type_name']);
					}
				}
				if(count($rows) < 1) {
					unset($rows);
				} else {
					// This seems very strange, but it's what gets the results
					$rows['edit'] = $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders');
					$rows['delete'] = $GLOBALS['egw']->acl->check('price_list', EGW_ACL_DELETE, 'perp_orders');
				}
				
			}
			return $query['count'];	
		}

		public function get_price_list($id) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				$list = $this->so->get_price_list($id);

				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');

				if(is_array($client_ids) && count($client_ids) > 0) {
					// Restrict clients to only their price lists
					$client_accounts = ExecMethod('perp_ar.bo_perp_client_account.get_client_account_list');
					foreach($client_accounts as $client_account_id => $name) {
						$lists = ExecMethod('perp_ar.bo_perp_client_account.get_price_lists', $client_account_id);
						if(is_array($lists) && in_array($id, $lists)) {
							unset($list['price_list_name']);
							unset($list['sales_area_name']);
							unset($list['sales_type_name']);
							return $list;
						}
					}
					return array();
				} else {
					return $list;
				}
			}
		}

		public function set_price_list($price_list_id, $price_list) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				if($price_list['price_list_name']) {
					return $this->so->set_price_list($price_list_id, $price_list);
				} else {
					return lang('Name is required');
				}
			}
		}

		public function delete_price_list($id) {
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_DELETE, 'perp_orders')) {
				return $this->so->delete_price_list($id);
			}
		}

		/**
		*	View a list of prices for the provided parameters
		*/
		public function view_price_list($price_list_id, $sales_category_id, $sales_area_id = '', $sales_type_id = '', $date = '', $edit = False) {
			trigger_error('view_price_list is deprecated, use bo_perp_price->get_price_list() instead');
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				$list = $this->so->view_price_list($price_list_id, $sales_category_id, $sales_area_id, $sales_type_id, $date, $edit);

				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if(is_array($client_ids) && count($client_ids) > 0) {
					// Restrict clients to only their price lists
					$client_accounts = ExecMethod('perp_ar.bo_perp_client_account.get_client_account_list');
					foreach($client_accounts as $client_account_id => $name) {
						$lists = ExecMethod('perp_ar.bo_perp_client_account.get_price_lists', $client_account_id);
						if(is_array($lists) && in_array($price_list_id, $lists)) {
							return $list;
						}
					}
					return array();
				}
			} else {
				return array();
			}
		}
	
		public function get_stock_price_list($stock_id, $category_id, $area_id, $type_id, $date, $edit = False, $price_list = False, $flatten = False) {
			trigger_error('get_stock_price_list is deprecated, use bo_perp_price->get_price_list() instead');
			if($GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				$price_list = $this->so->get_stock_price_list($stock_id, $category_id, $area_id, $type_id, $date, $edit, $price_list);
				if($flatten) {
					$price_list = $this->flatten_price_list($price_list);
					return $price_list;
				} else {
					return $price_list;
				}
			} else {
				return array();
			}
		}

		private function flatten_price_list($price_list) {
			$new_list = array(
				'stock_id'	=>	$price_list['stock_id'],
				'stock_num'	=>	$price_list['stock_num'],
				'price'		=>	$price_list['price'],
				'edit'		=>	$price_list['edit']
			);
			if(!$price_list['options']){
				foreach($price_list as $key => &$option) {
					$new_list[$option['build_option_category_id']] = array();
					if(is_array($option['options'])) {
						foreach($option['options'] as $sub_key => $sub_option) {
							$new_list[$option['build_option_category_id']]['options'][$sub_option['build_option_id']] = $sub_option;
						}
					}
				}
				return $new_list;
			}
			foreach($price_list['options'] as $key => $option) {
				if(!$option['options']) {
					if(is_array($option)) {
						$new_list[$option['build_option_category_id']]['options'][$option['build_option_id']] = $option;
					}
					continue;
				}
				$new_list[$option['build_option_category_id']] = array();
				foreach($option['options'] as $sub_key => $sub_option) {
					$new_list[$option['build_option_category_id']]['options'][$sub_option['build_option_id']] = $sub_option;
					if($sub_option['options']) {
						$flat = $this->flatten_price_list($sub_option['options']);
						foreach($flat as $id => $opt) {
							if(is_array($opt)) {
								$new_list[$id]['options'] = $opt['options'];
							}
						}
						unset($new_list[$option['build_option_category_id']]['options'][$sub_option['build_option_id']]['options']);
					}
				}
			}
			return $new_list;
		}


		// These are hard coded because they have significance in other places in the code
		public function get_status_list() {
			return array(
				''		=>	lang('All'),
				'current'	=>	lang('Current'),
				'previous'	=>	lang('Previous'),
			);
		}

		/**
		*	Get a list of client accounts that have access to a price list
		*
		*	@nextmatch
		*	- price_list_id int Which price list you're interested in
		*/
		public static function get_client_accounts(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_READ, 'perp_orders')) {
				$rows = array();
				return 0;
			}

			// Limit clients to only themselves
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0) {
				$query['client_id'] = $client_ids;
			}

			if(!$query['order']) {
				$query['order'] = 'perp_client_account.client_id';
				$query['sort'] = 'ASC';
			}

			$rows =& so_perp_price_list::get_client_accounts($query);

			return $query['total'];
		}
	}
?>
