<?php
	/**
	*	Logic object for dealing with several SalesOrders.
	*
	*	@author Nathan Gray
	*	@ingroup perp_orders
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_order.inc.php');
	require_once('class.so_perp_sales_order.inc.php');

	class bo_perp_sales_order extends bo_perp_order {

		const TOTAL_LINE = -1;
		const DAMAGE_LINE = -2;

		public function __construct() {
			parent::__construct();
			$this->so_perp_order = new so_perp_sales_order();
		}

		/**
		*       Get a list of Orders
		*       @nextmatch
		*       Query options:
		*               search:         Our reference number or their reference number
		*               supplier_id:    Restrict to supplier
		*               client_id:      Restrict to client
		*               filter:         Restrict to status
		*/
		public function get_order_list(&$query = array(), &$rows = array(), &$readonly = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_orders';

			$query['order_class'] = 'SalesOrder';
			$query['supplier_id'] = 0;
			$query['sales_category'] = $query['sales_category'] ? $query['sales_category'] : $query['filter2'];
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0) {
				// Restrict clients to only their price lists
				$query['client_id'] = $client_ids;
			}

			parent::get_order_list($query, $rows, $readonly);

			if(!isset($query['template'])) {
				$list = array();
				foreach($rows as $row) {
					$list[$row['order_id']] = $row['our_reference_number'];
				}
				return $list;
			} else {
				$user = $GLOBALS['egw_info']['user']['account_id'];
			}

			return $query['count'];
		}

		/**
		*	Get the lines off a SalesOrder.
		*
		*	@todo This function may be deprecated, and replaced by ui_perp_order::get_order_lines()
		*
		*	@nextmatch
		*/
		public function get_order_details($query, &$rows = array(), &$readonlys = array()) {
			$sales_order = $GLOBALS['egw']->session->appsession('sales_order');
			if($query['so_id'] && (!$sales_order || $sales_order->get_sales_order_id() != $query['so_id'])) {
				$sales_order = $this->get_sales_order($query['so_id']);
			}
			if(!$sales_order) {
				return 0;
			}
			if($GLOBALS['egw']->acl->check('sales_order', EGW_ACL_READ, 'perp_orders')) {
				$rows = $sales_order->get_lines(True);
			}
			$count = count($rows);

			if(!$sales_order->is_editor($GLOBALS['egw_info']['user']['account_id'])) {

				$rows['no_actions'] = True;
				foreach($rows as $key => &$row) {
					$readonlys[$key + 1]['perp_orders.sales_order_edit.work_orders'] = True;
					$readonlys[$key + 1]['unit_price'] = True;
					$readonlys[$key + 1]['quantity_ordered'] = True;
					$readonlys[$key + 1]['description'] = True;
					$readonlys[$key + 1]['delete[' . $row['line_no'] . ']'] = True;
				}
			}
			return $count;
		}

		/**
		*	Get a list of options and option values for user to choose.
		*
		*	This mess of complexity takes into account the build options that are allowed
		*	for the given stock item, at the time the order was made.  This means that
		*	if you don't offer an option on that product, the client can't order it.
		*
		*	It also takes into account the build options that are allowed by the price list.
		*	If a client cannot purchase an option because the price for that option on their
		*	current price list is set to N/A, that option is not offered.
		*
		*	Option pricing is added (or subtracted) from the base price, and the total
		*	price is used as the unit price for the line.
		*
		*	A 'Not important' selection is added to handle old stock that doesn't have all
		*	the options you currently offer.  For example, if you add an 'Extra Fan' with
		*	Yes and No this year to an existing product, choosing 'Not important' for 
		*	Extra Fan will allow the client to purchase stock that was manufactured 
		*	before you added the Extra Fan option.  The price will be 0 for that option.
		*	
		*	@nextmatch
		*/
		public function get_detail_details(&$query, &$rows = array(), &$readonlys = array()) {

			// Get the allowed options 
			$bo_wo = CreateObject('perp_manufacturing.bo_perp_work_order');
			$bo_wo->get_work_order_detail_option_list($query, $rows, $readonlys);
			$bo_options = CreateObject('perp_manufacturing.bo_perp_build_option');

			if(!is_array($rows)) {
				$rows = array();
				return 0;
			}

			$bo_price = CreateObject('perp_orders.bo_perp_price');
			if(!$query['price_list_id']) {
				$price_lists = ExecMethod2('perp_ar.bo_perp_client_account.get_price_lists', $query['client_account_id'], $query['effective_date']);
				$query['price_list_id'] = $price_lists;
			}
			// Still couldn't find a price list.  Give up, no pricing.
			if(!$query['price_list_id']) {
				return 0;
			}

			// Need to set template, otherwise we get an array for a selectbox
			$query['template'] = $query['template'] ? $query['template'] : false;

			// Get option pricing
			// Don't pass readonlys, because we're not interested in if the prices can be changed
			$bo_price->get_merged_price_list($query, $price_list);
			if(is_array($price_list[0]['options'])) {
				// Remember the actual price list that won the merge (lowest price), we'll use it later
				$query['price_list_id'] = $price_list[0]['price_list_id'];
				$total = $price_list[0]['price'];
				$price_list = $price_list[0]['options'];
			} else {
				// No option pricing
				$price_list = array();
				$total = 0;
			}

			/**
			*	Need to be able to index by category_id,
			*	so build an array with category_id as the key
			*/
			$prices = array();
			foreach($price_list as $category) {
				$prices[$category['build_option_category_id']] = $category;
				unset($prices[$category['build_option_category_id']]['build_option_prices']);
				foreach($category['build_option_prices'] as $option) {
					$prices[$category['build_option_category_id']]['build_option_prices'][$option['build_option_id']] = $option;
				}
			}
			// TODO: Set currency to the currency of the price list

			/**
			*	Step through the rows, remove build options forbidden by the price list, and 
			*	match up pricing to build option.
			*	If the default option is not sold to this client, set the default to
			*	the next most preferred option.
			*/
			foreach($rows as $key => &$row) {
				if(!is_array($row['options-value'])) {
					$row['options-value'] = array();
				}
				$preferred_option = $row['value'];

				// Required option?
				if(count($row['options-value']) == 1 && $row['options-value'][key($row['options-value'])]['value'] == -1) {
					// No choice, and there won't be a price, so remove it
					unset($rows[$key]);
				}

				// Check to see if we have prices
				if(!is_array($prices[$row['build_option_category_id']])) {
					/**
					* No prices found.  It could be that all of the options were set to N/A, which is not advised.
					* It could be there's some nesting going on.  Let's try nesting.  It's already in the array,
					* but it's easier to just get what we want than to look through the entire price list.
					*/
					$missing_price_query = array(
						'price_list_id'	=>	$query['price_list_id'],
						'date'		=>	$query['effective_date'],
						'component_id'	=>	$row['stock_id'],
						'stock_id'	=>	$query['stock_id']
					);
					$missing_prices = ExecMethod('perp_orders.so_perp_price.get_option_price_list', $missing_price_query);

					// Re-index by id, as above
					if(is_array($missing_prices[$row['build_option_category_id']])) {
						foreach($missing_prices[$row['build_option_category_id']]['build_option_prices'] as $option) {
							$prices[$row['build_option_category_id']]['build_option_prices'][$option['build_option_id']] = $option;
						}
					}
				}
				foreach($row['options-value'] as $id => $option) {
					if($option['value'] == 0) {
						// Not an allowed option.  How'd it get here?
						unset($row['options-value'][$id]);
						continue;
					}

					$price = $prices[$row['build_option_category_id']]['build_option_prices'][$id]['price'];

					// If we're not looking at a particular work order, and there's no price, remove the option
					// If we are looking at a particular work order, we need the option if the client can buy it or not
					if((!$query['work_order_id'] || ($query['work_order_id'] && $query['work_order_id'] < 0)) && 
							(is_null($price) || $price == so_perp_price::NOT_ALLOWED)) {
						unset($row['options-value'][$id]);
						if($row['value'] == $id) {
							$row['value'] = '--MARKER--';
						}
						continue;
					}
					$row['options-value'][$id]['price'] = $price;

					/*
					If the user selects 'Not important' for the build option, $row['value'] will be empty.
					If the default option has been denied by the price list, $row['value'] will be --MARKER--
					Setting $row['value'] to $preferred option if $row['value'] is empty could 
					possibly prevent the sale of old stock.  
					*/
					if(!$row['value'] && $option['value'] > $row['options-value'][$preferred_option]['value']) {
						$preferred_option = $id;
					}
				}
				if(count($row['options-value']) > 0) {
					$row['price'] = $row['options-value'][$row['value']]['price'];
					$total += $row['options-value'][$row['value']]['price'];
					if($row['value'] == '--MARKER--') {
						$row['value'] = $preferred_option;
					}
				} else {
					// Option has no allowed values.  Somebody screwed up the price list.
					$row['options-value'][0] = array(
						'build_option_name' => lang('Error on price list(s) #%1 - no options for %2 for sale anywhere',
							is_array($query['price_list_id']) ? implode(',', $query['price_list_id']) : $query['price_list_id'],
							$row['build_option_category_name']
						),
						'price'	=>	so_perp_price::NOT_ALLOWED
					);
					$row['value'] = 0;
				}
			}

			// Check for damage
			$controlled_query = array(
				'stock_id'      =>      $query['stock_id'],
				'order'         =>      'perp_stock_damage.stock_history_id',
				'sort'          =>      'DESC',
				'num_rows'      =>      1,
				'unclaimed'	=>	true,
			);
			if($query['work_order_id'] > 0) {
				$controlled_query['work_order_id']	=	$query['work_order_id'];
				$controlled_query['detail_id']		=	$query['detail_id'];
				$controlled_query['unclaimed']		=	false;
			}
			$count = bo_perp_damage::get_damage_list($controlled_query, $damage);

			if($count > 0 && is_array($damage)) {
				if($query['work_order_id'] > 0 && $damage[0]['work_order_id'] == $query['work_order_id'] && $damage[0]['detail_id'] == $query['detail_id']) {
					$query[self::DAMAGE_LINE] = $damage[0]['stock_history_id'];
				}

				// Damaged goods available
				$damage_row = array(
					'build_option_category_id'	=>	self::DAMAGE_LINE,
					// Give it a nice, friendly name instead of Damaged
					'build_option_category_name'	=>	lang('Blemish'),
					'options-value'			=>	array('' => array(
						'build_option_id'	=>	null,
						'build_option_name'	=>	lang('Unblemished'),
						'value'			=>	'',
						'price'			=>	'0.00'
					)),
					'value'			=>	$query[self::DAMAGE_LINE],
				);
					
				foreach($damage as $damaged) {
					$discount = -($total * $damage[0]['discount_percent']);
					$damage_row['options-value'][$damaged['stock_history_id']] = array(
						'build_option_id'	=>	null,
						'build_option_name'	=>	$damaged['comment'],
						'value'			=>	$damaged['stock_history_id'],
						'price'			=>	sprintf("%10.2f", $discount),
					);
				}

				$damage_row['price'] = $damage_row['options-value'][$damage_row['value']]['price'];
				$total += $damage_row['price'];

				// Add a line for this
				$rows[] = $damage_row;
			}

			// Add a line total for this item 
			$rows[] = array(
				'build_option_category_id'	=>	-1,
				'build_option_category_name'	=>	lang('total'),
				'price'				=>	sprintf("%10.2f", $total)
			);
			$readonlys[count($rows) . '[value]'] = True;
			return count($rows);
		}

		/**
		*	List of available statuses for a Sales Order
		*
		*	@return array
		*/
		public function get_status_list() {
			return array(
				''		=>	lang('All'),
				'open'		=>	lang('Open'),
				'not_shipped'	=>	lang('Not Shipped'),
				'shipped'	=>	lang('Shipped'),
				'not_invoiced'	=>	lang('Not Invoiced'),
				'closed'	=>	lang('Closed')
			);
		}

		/**
		* get title for an perp sales order entry identified by $entry
		* 
		* Is called as hook to participate in the linking
		*
		* @param int/array $entry key to look up or array with sales_order entry
		*	key is in the form SALES_ORDERid[:SALES_ORDER_LINE_#]
		* @return string  with title/boolean
		*/
		function link_title( $entry ) {	
			if (!is_array($entry)) {
				list($id, $line) = explode('-', $entry);
				$order = new SalesOrder($id);
				return $order->get_our_reference_number() . (isset($line) ? ':' . $line : '');
			} else {
				if(is_a($entry, 'SalesOrder')) {
					return $entry->get_our_reference_number();
				} else {
					$entry = new SalesOrder($id);
					return $entry->get_our_reference_number() . (isset($line) ? ':' . $line : '');
				}
			}
			if (!$entry) {
				return $entry;
			}
		}

		/**
		*	Called as a hook to participate in invoicing
		*
		*	@param query An array of arguments, including client_id
		*	@nextmatch
		*/
		public function invoicable_search(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$config = config::read('perp_orders');
			if(!$config['invoicable']) {
				$rows = array();
				return array();
			}
			// Need to flip the app to get correct permissions checking
			$temp_app = $GLOBALS['egw_info']['flags']['currentapp'];
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_orders';


			// Copy the query so we don't modify it
			$order_query = $query;

			// Add some parameters 
			$order_query['filter'] = 'open';
			$order_query['start'] = 0;
			$order_query['num_rows'] = -1;
			unset($order_query['template']);

			// Get a list of Sales Orders to choose from
			$query['options-filter'] = $this->get_order_list($order_query);
			$query['no_filter'] = false;
			$query['filter_label'] = 'sales order';

			// We want all the data now
			$order_query['template'] = $query['template'] ? $query['template'] : false;

			$rows = array();
			if($query['filter']) {
				$orders[]['order_id'] = $query['filter'];
			} else {
				$count = $this->get_order_list($order_query, $orders, $readonlys);
			}


			// Pull out the lines into the list of results
			if(is_array($orders)) {
				foreach($orders as $order_row) {
					$order = new SalesOrder($order_row['order_id']);
					if($order->can_invoice()) {
						$invoicable_items = $order->get_invoicable_items();
						foreach($invoicable_items as &$invoicable) {
							if($invoicable->get_invoice_quantity() != 0) {
								$rows[] = $invoicable;
							}
						}
					}
				}
			}

			// Put the current application back
			$GLOBALS['egw_info']['flags']['currentapp'] = $temp_app;
		}

		/**
		*	Given an OrderLine, find and reserve as many Work Orders as needed to make up the
		*	ordered quantity.  If there aren't enough already scheduled (or completed) work orders,
		*	create a work order based on the provided temporary work order.  The provided temporary
		*	work order will be deleted.
		*
		*	@param line OrderLine that we're working with
		*	@param required_date WorkOrders that will be completed after this date are not included in the list of possibilities
		*	@param work_order_id If provided, this work order will be used to match against.  Only work orders with the same
		*		options will be considered
		*	@param detail_id Together with work_order_id gives a work order detail whose options must match any work orders to claim
		*	@param options Options to set for any work orders claimed
		*	
		*	@return true on success, or array that contains array(error message, severity) suitable for passing directly to pERP's
		*		message maker.
		*/
		public function claim_work_orders(&$line, $required_date, &$work_order_id = null, &$detail_id = null, $options = array()) {
			$bo_wo = CreateObject('perp_manufacturing.bo_perp_work_order');
			$order =& $line->get_owner();
			$unclaimed = $bo_wo->get_unclaimed_work_order($line->get_stock_id(), null, $required_date, $work_order_id, $detail_id);

			// Look for extra restrictions (damage, etc)
			foreach($options as $key => $option) {
				switch ($option['build_option_category_id']) {
					case self::DAMAGE_LINE:
						// They want damaged goods, value is the damage ID
						$damage = bo_perp_damage::get_damage($option['value']);
						foreach($unclaimed as $row => &$work_order) {
							if(!is_array($work_order)) {
								continue;
							} elseif (!$option['value']) {
								// Don't want damaged goods, make sure it isn't
								$query = array(
									'work_order_id'	=>	$work_order['work_order_id'],
									'detail_id'	=>	$work_order['detail_id'],
								);
								$count = bo_perp_damage::get_damage_list($query, $check_list);
								if($count > 0) {
									unset($claimed[$row]);
									$unclaimed['total']--;
								}
							} elseif ($work_order['work_order_id'] != $damage['work_order_id']) {
								unset($unclaimed[$row]);
								$unclaimed['total']--;
							} elseif ($work_order['quantity'] > $damage['quantity_damaged']) {
								// Limit the quantity to those damaged.  Undamaged will probably have
								// a different price, and will have to be a different line
								$work_order['quantity'] = $damage['quantity_damaged'];
							}
						}
						break;
				}
			}

			// Remove total from results so it doesn't get in the way
			$unclaimed_total = $unclaimed['total'];
			unset($unclaimed['total']);
			reset($unclaimed);

			$quantity_left = $line->get_quantity_ordered();
			while($quantity_left > 0) {
				if($unclaimed_total - ($line->get_quantity_ordered() - $quantity_left) <= 0) {
					// Not enough unclaimed matching work orders.  Make a new one.
					if($work_order_id && $work_order_id < 0) {
						// A temporary was provided, make it a real work order

						$work_order = $bo_wo->get_work_order($work_order_id);
						$detail = $bo_wo->get_work_order_detail($work_order_id, $detail_id);
						$bo_wo->delete_work_order($work_order_id);
					} else {
						// No temporary was given as an example, create a new one
						$work_order = array(
							'required_date'	=>	$line->get_owner()->get_requested_delivery_date(),
						);
						$detail = array(
							'detail_id'	=>	1,
							'stock_id'	=>	$line->get_stock_id(),
							'quantity'	=>	$line->get_quantity_ordered()
						);
					}

					$work_order['sales_order'] = $order->get_order_id();
					$work_order['work_order_reference'] = $bo_wo->suggest_work_order_number($line->get_stock_id());
					unset($work_order['work_order_id']);

					$current_work_order_id = $bo_wo->set_work_order(null, $work_order);
					$current_detail_id = $detail['detail_id'];
					$quantity = $quantity_left;
					$detail['work_order_id'] = $work_order_id;
					$detail['quantity'] = $quantity;
					$bo_wo->set_work_order_detail($current_work_order_id, $current_detail_id, $detail);
				} else {
					$work_order = current($unclaimed);
					$current_work_order_id = $work_order['work_order_id'];
					$current_detail_id = $work_order['detail_id'];
					$quantity = min($quantity_left, $work_order['quantity']);

					next($unclaimed);
				}

				// Set options
				foreach($options as $key => $row) {
					if($row['value']) {
						if(is_numeric($row['build_option_category_id'])  && $row['build_option_category_id'] > 0) {
							$result = ExecMethod2('perp_manufacturing.so_perp_work_order.set_work_order_detail_option',
								$current_work_order_id, $current_detail_id, $row['stock_id'], $row['build_option_category_id'], $row['value']
							);
							if($result !== true) {
								$this->messages[] = array($result, 'warning');
							}
						}
					}
				}

				// Create the WorkOrderDetail that records the pairing of Work Order and Sales Order
				$new_work_order =& new WorkOrderDetail($line, array('detail_value' => $current_work_order_id));
				$result = $new_work_order->set_work_order_detail_id($current_detail_id);
				if($result !== true) {
					$this->messages[] = array($result, 'error');
					break;
				} else {
					/* This causes issues, it should return true, not an info message
					$this->messages[] = array(lang('Using work order %1:%2', 
						$new_work_order->get_work_order_reference(), 
						$current_detail_id
					), 'info');
					*/
				}

				$result = $new_work_order->set_work_order_quantity($quantity);

				if($result !== true) {
					$this->messages[] = array($result, 'error');
					break;
				} else {
					$line->add_detail($new_work_order);
					$quantity_left -= $quantity;
				}
				if($quantity_left > 0) {
					/*
					*	Need to save the order here, so unclaimed will be updated 
					*	to remove the one we just used from the available work orders.
					*	This prevents others from reserving it also, and it prevents
					*	it from showing up again in the list of unclaimed work
					*	orders.
					*/
					$line->get_owner()->write();

					$current_work_order_id = null;
					$current_detail_id = null;
					$unclaimed = $bo_wo->get_unclaimed_work_order($line->get_stock_id(), null, $required_date, $work_order_id, $detail_id);
				}
			}

			if(count($this->messages) > 0) {
				return $this->messages;
			}
			return true;
		}

		/**
		*	Mark a quantity from a work order as being returned, so it can be resold.
		*
		*	It doesn't much matter if we get the right sales order, just that we make some available
		*	again so it can be sold.
		*/
		public static function return_work_order($client_id, $work_order_id, $detail_id, $quantity) {
			so_perp_sales_order::return_work_order($client_id, $work_order_id, $detail_id, $quantity);
		}
	}
?>
