<?php
	/**
	* eGroupWare eTemplate Extension - Price Edit Widget
	*
	* Due to the complexities of nested stock options, I'm going to try creating a widget to 
	* edit the price of an item.  A regular nested grid won't work because of the unlimited
	* nesting potential with options on options [on options]*.
	* 
	* If you want to include stock items whose base price is N/A, you have to pass the stock_id
	* and include_not_allowed = true.  
	*	
	* This widget expects the price list array for a stock item's pricing, which looks like:
	* @verbatim
	* Array	(
	*	[price_list_id]		=>	36
	*	[stock_id]		=>	195
	*	[stock_num]		=>	00195
	*	[stock_type_flag]	=>	K
	*	[description]		=>	Widget (Kit)
	*	[price]			=>	1,234.56
	*	[effective_date]	=>	1218744000
	*	[modified_date]		=>	1218742990
	*	[modified_by]		=>	15
	*	[components]	=>	Array (
	*		[0] => Array(
	*			[stock_id]		=>	95
	*			[stock_num]		=>	00095
	*			[stock_type_flag]	=>	K
	*			[description]		=>	Widget
	*			[price]			=>	1,134.56
	*			[effective_date]	=>	1218744000
	*			[modified_date]		=>	1218742990
	*			[options]	=> Array (
	*				[0] => Array (
	*					[build_option_category_id] => 12
	*					[build_option_category_name] => Color
	*					[build_option_prices] => Array (
	*						[0] => Array (
	*							[build_option_id] => 77
	*							[build_option_name] => Red
	*							[component] => 140
	*							[price] => 
	*						)
	*						[1] => Array (
	*							[build_option_id] => 75
	*							[build_option_name] => White
	*							[component] => 130
	*							[price] => 10.000
	*							[options] => Array (
	*								[0] => Array (
	*									[build_option_category_id] => 13
	*									[build_option_category_name] => Trim
	*									[build_option_prices] => Array (
	*										[0] => Array (
	*											[build_option_id] => 79
	*											[build_option_name] => White
	*											[component] => 106
	*											[price] => 
	*										)
	*									)
	*								)
	*							)
	*							[option_template] => perp_orders.price_list.option_price
	*						)
	*						[2] => Array (
	*							[build_option_id] => 76
	*							[build_option_name] => Blue
	*							[component] => 131
	*							[price] => 15.000
	*						)
	*					)
	*				)
	*			)
	*		)
	*		[1] => Array(
	*			[stock_id]		=>	96
	*			[stock_num]		=>	00096
	*			[stock_type_flag]	=>	R
	*			[description]		=>	Widget Maintenance Kit
	*			[price]			=>	100.00
	*			[effective_date]	=>	1218744000
	*			[modified_date]		=>	1218742990
	*		)
	*	)
	*)
	* @endverbatim
	* 
	* A Raw stock item cannot have options or components.  An assembly can have options, but not components.  A Kit can have
	* both options and components.  If an item does not have options or components, that key is not set.  
	* Components can have options.  Options can have components, but these should be brought out into the component array.
	*
	* @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
	* @package perp_orders
	* @link http://projecterp.org
	* @author Nathan Gray <nathangray@sourceforge.net>
	*/

	class price_edit_widget {

		public $public_functions = array(
			'pre_process' => True,
			'post_process' => True,
		);
		var $human_name = 'pERP Price Edit';	// this is the name for the editor

		private $debug = false;

		function __construct($ui='')
		{
			switch($ui)
			{
				case '':
				case 'html':
					$this->ui = 'html';
					break;
				default:
					echo "UI='$ui' not implemented";
			}
			return 0;
		}

		/**
		* pre-processing of the extension
		*
		* This function is called before the extension gets rendered
		*
		* @param string $name form-name of the control
		* @param mixed &$value value / existing content, can be modified
		* @param array &$cell array with the widget, can be modified for ui-independent widgets 
		* @param array &$readonlys names of widgets as key, to be made readonly
		* @param mixed &$extension_data data the extension can store persisten between pre- and post-process
		* @param object &$tmpl reference to the template we belong too
		* @return boolean true if extra label is allowed, false otherwise
		*/
		function pre_process($name,&$value,&$cell,&$readonlys,&$extension_data,&$tmpl)
		{
			if($this->debug) {
				echo __METHOD__ . '<br />';
				printf("Name:%20s<br />", $name);
			//	echo 'Backtrace: <pre>';
			//	debug_print_backtrace();
			//	echo '</pre>';
				echo 'Value:';
				_debug_array($value);
				echo 'Cell:';
				_debug_array($cell);
				echo 'Readonlys: ';
				_debug_array($readonlys);
				echo 'Extension_data:';
				_debug_array($extension_data);
			}

			// Get options
			$options = array();
			if(!is_array($cell['size'])) {
				list(
				) = explode(',', $cell['size']);
			} else {
				$options = $cell['size'];
			}

			// Check session variable to see if user wants to hide options
			if($GLOBALS['egw']->session->appsession('hide_options', 'perp_orders')) {
				$options['hide_options'] = true;
			}

			if(is_array($value)) {
		//		$options = array_merge($options, $value);
			} else {
				$value = array();
			}
			if(!$options['template']) {
				$options['template'] = 'perp_orders.price_widget.option_category';
			}
			$onchange = ($cell['onchange'] ? $cell['onchange'] : 'false');
			
			if(is_array($readonlys)) {
				foreach($readonlys as $key => $value) {
					$cell['readonly'] = $cell['readonly'] || $value;
				}
			} else {
				$cell['readonly'] = $cell['readonly'] || $readonlys;
			}

			// Set current value
			if($value['effective_date']) {
				$value['formatted_effective_date'] = date(
					$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
					($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
					$value['effective_date']
				);
			}
			if($value['effective_to']) {
                                $value['formatted_effective_to'] = date(
                                        $GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
                                        ($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
                                        $value['effective_to']
                                );
                        }			


			// Set up widget
			$readonlys = false;
			$cell['type'] = 'template';

			$template = new etemplate('perp_orders.price_edit');
			$template->read('perp_orders.price_edit');
			$template->no_onclick = true;


			if(is_array($value['components'])) {
				$readonlys['price'] = true;
				self::make_components($name, $cell, $options, $value);
			}
			if(is_array($value['options']) && !$options['hide_options']) {
				$option_html = self::make_options($name, $cell, $options, $value);
			}

			$value['options'] = $option_html;

			if(!$value['stock_type_flag']) {
				$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $value['stock_id']);
				$value['stock_type_flag'] = $stock['stock_type_flag'];
			}
			if($value['stock_type_flag']) {
				list($span, $class) = explode(',', $cell['span']);
				if($class) {
					$class .= ' ';
				}
				$class .= 'stock_type_' . $value['stock_type_flag'];
				$cell['span'] = "$span,{$class}";
			}
			$cell['size'] = $cell['name'];
			$cell['obj'] =& $template;
			$cell['name'] = $template->name;

			return true;
		}

		function post_process($name,&$value,&$extension_data,&$loop,&$tmpl,$value_in) {
			$value = $value_in;
			return true;
		}

		/**
		*	Currently components don't need much special handling, just format the date
		*	so it can be used as a link
		*/
		protected static function make_components($name, $cell, Array $cell_options, Array &$value) {
			$components =& $value['components'];
			foreach($components as $key => &$component) {
				if($component['effective_date']) {
					$component['formatted_effective_date'] = date(
						$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
						($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
						$component['effective_date']
					);
				} else {
					$component['formatted_effective_date'] = '';
				}
				if($component['effective_to']) {
                                        $component['formatted_effective_to'] = date(
                                                $GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
                                                ($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
                                                $component['effective_to']
                                        );
                                } else {
                                        $component['formatted_effective_to'] = '';
                                }
			}
		}

		/**
		*	Generate the HTML needed to display options
		*/
		protected static function make_options($name, $cell, Array $cell_options, Array $value) {
			$options =& $value['options'];
			$result = '';

			if(!is_array($options)) {
				return $result;
			}
			foreach($options as $row => $category) {
				if(count($category['build_option_prices']) == 0) {
					continue;
				}

				$template = new etemplate($cell_options['template']);
				$template->read($cell_options['template']);
				$cell_name = $name . "[options][$row]";
				$category['build_option_category_id'] = $GLOBALS['egw']->html->input_hidden($cell_name . '[build_option_category_id]', $category['build_option_category_id']);
				$readonly = array();
				if(!is_array($category['build_option_prices'])) {
					continue;
				}

				foreach($category['build_option_prices'] as $option_key => &$option) {
					$option_name = $cell_name . '[build_option_prices][' . ($option_key + 1) . ']';
					$option['price_list_id']	=	$value['price_list_id'];
					$option['stock_id']		=	$value['stock_id'];
					$option['build_option']		=	$option['build_option_id'];
					
					$option['component_id'] = $GLOBALS['egw']->html->input_hidden($option_name . '[component_id]', $option['component_id']);
					$option['build_option_id'] = $GLOBALS['egw']->html->input_hidden($option_name . '[build_option_id]', $option['build_option_id']);
					if($option['effective_date']) {
						$option['formatted_effective_date'] = date(
							$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
							($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
							$option['effective_date']
						);
					} else {
                                                $option['formatted_effective_date'] = '';
                                                $option['no_option_effective_date'] = true;
                                        }
					if($option['effective_to']) {
                                		$option['formatted_effective_to'] = date(
                                                	$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
                                                	($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
                                                	$option['effective_to']
                                        	);
					} else {
                                        	$option['formatted_effective_to'] = '';
					}
					if(!$option['price_is_override']) {
						$option['no_override'] = true;
					}
					// eTemplate needs a key to properly pass the sub-elements, so we use 'recurse', and pull it out later
					if($option['options']) {
						$option['recurse']['options'] = $option['options'];

						// Copy over price_list_id & stock id for links
						$option['recurse']['price_list_id']	=	$value['price_list_id'];
						$option['recurse']['stock_id']		=	$value['stock_id'];
					}
					if($option['components']) {
						$option['recurse']['components'] = $option['components'];
					}
					$readonly['__ALL__'] = $cell['readonly'] || isset($option['components']);
				}
				// Leave room for header
				array_unshift($category['build_option_prices'], false);
				$html = $template->show($category, '', $readonly, $cell_name);
				$result .= $html;
			}
			return $result;
		}
	}
?>
