<?php

	/**
	*	Data access layer for backorders shipments.  
	*
	*	@author Nathan Gray
	*	@ingroup perp_orders
	*/


	class so_perp_backorder {

		public function __construct() {
		}

		/**
		*	Get a list of items that cannot be shipped.
		*	This takes several sub-queries, which are joined up into one
		*	query.
		*	This is very similar to perp_ar::so_perp_outgoing_shipment::get_shipable_items(), but unfortunately slightly different, in that this query only cares about not in stock goods, and is for all clients.
		*/
		public function get_backorder_list(&$query) {
			$where = array();
			if($query['client_id'] && is_numeric($query['client_id'])) {
				$where[] = 'client_id = ' . $query['client_id'];
			} 
			if($query['shipment_id']) {
				$where[] = 'shipment_id = ' . $query['shipment_id'];
			}
			if($query['order_id']) {
				$where[] = 'order_id = ' . $query['order_id'];
			}

			// Items that have been ordered but not shipped
			$shippable_items_sql = 'SELECT                                        
				perp_order_line.order_id,                                        
				perp_order.our_reference_number,
				perp_order.ship_from_address,
				perp_order.ship_to_address,
				perp_order.client_id,
				shipment_total.shipment_id,
				perp_order.requested_delivery_date,
				perp_order_line.line_no,
				perp_order_line.description,
				perp_stock.stock_id,
				perp_stock.stock_num,
				perp_stock.stock_type_flag,
				perp_stock.stock_category AS stock_category_id,
				perp_stock.decimal_places,
				perp_unit_of_measure.uom_name AS unit_of_measure_name,
				perp_order_line.quantity_ordered,
				shipment_total.quantity_shipped,
				perp_order_line.quantity_ordered - IF(
					shipment_total.quantity_shipped IS NULL,
					0,
					shipment_total.quantity_shipped
				) AS quantity_unshipped
			FROM perp_order_line
			JOIN perp_order ON
				perp_order_line.order_id = perp_order.order_id
			JOIN perp_sales_order ON
				perp_order.order_id = perp_sales_order.order_id
			LEFT JOIN perp_order_detail ON
				perp_order_detail.order_id = perp_order_line.order_id AND
				perp_order_detail.line_no = perp_order_line.line_no
			LEFT JOIN (
				SELECT
					shipment_id,
					SUM(IF(quantity_shipped IS NULL, 0, quantity_shipped)) AS quantity_shipped,
					shippable_id,
					shippable_item_line
				FROM perp_shipment_line
				GROUP BY shippable_id, shippable_item_line
			) AS shipment_total ON
				perp_order_line.order_id = shipment_total.shippable_id AND
				perp_order_line.line_no = shipment_total.shippable_item_line
			JOIN perp_stock ON
				perp_order_line.stock_id = perp_stock.stock_id
			JOIN perp_unit_of_measure ON
				perp_stock.unit_of_measure = perp_unit_of_measure.uom_id
			WHERE (true) ';  // Cheat and make sure the WHERE clause isn't empty

			// Add in filters here
			if($query['cat_id']) {
				$shippable_items_sql .= ' AND perp_stock.stock_category = ' . $query['cat_id'];
			}
			if($query['location_id']) {
				$shippable_items_sql .= ' AND perp_order_line.ship_location = ' . (int)$query['location_id'];
			}
			if(count($query['col_filter']) > 0) {
				foreach($query['col_filter'] as $key => $value) {
					if($value !== '') {
						$shippable_items_sql .= ' AND ' . $key . ' = ' . 
							$GLOBALS['egw']->db->quote($value);
					}
				}
			}

			$shippable_items_sql .= ' HAVING quantity_unshipped != 0';

			// Items that are not in stock and are purchased
			$purchase_eta_sql = 'SELECT
				perp_order_line.order_id,
				perp_order_line.line_no,
				purchase_order_line.order_id AS incoming_order_id,
				purchase_order.our_reference_number AS incoming_order_reference,
				purchase_order.requested_delivery_date AS eta,
				IF(perp_shipment_line.quantity_shipped IS NULL, purchase_order_line.quantity_ordered, perp_shipment_line.quantity_shipped) - 
					IF(perp_goods_received.quantity_units IS NULL, 0, perp_goods_received.quantity_units) AS quantity_expected,
				perp_location.location_name,
				\'perp_ap\' AS app,
				CONCAT(\'ORDER-\', purchase_order.order_id) AS link_id
			FROM perp_order_line
			JOIN (
				perp_order_line AS purchase_order_line 
				JOIN perp_order AS purchase_order ON
					purchase_order.client_id = 0 AND
					purchase_order.order_id = purchase_order_line.order_id 
				LEFT JOIN perp_shipment_line ON
					purchase_order_line.order_id = perp_shipment_line.shippable_id AND
					purchase_order_line.line_no = perp_shipment_line.shippable_item_line
				LEFT JOIN perp_goods_received ON
					perp_shipment_line.shipment_id = perp_goods_received.shipment_id AND
					perp_shipment_line.line_no = perp_goods_received.line_no
				LEFT JOIN perp_stock_cost ON
					perp_goods_received.stock_cost_id = perp_stock_cost.stock_cost_id
			) ON
				perp_order_line.stock_id = purchase_order_line.stock_id AND
				(perp_stock_cost.quantity < purchase_order_line.quantity_ordered OR
				perp_stock_cost.quantity IS NULL)
			JOIN perp_supplier_stock ON
				perp_supplier_stock.supplier_code = purchase_order_line.supplier_stock_code AND
				perp_supplier_stock.supplier_id = purchase_order.supplier_id				
			JOIN perp_location ON 
				purchase_order.ship_to_address = perp_location.delivery_address ';

			// Items that are not in stock and are manufactured
			$work_order_eta_sql = 'SELECT
				perp_order_line.order_id,
				perp_order_line.line_no,
				perp_work_order.work_order_id AS incoming_order_id,
				perp_work_order.work_order_reference AS incoming_order_reference,
				perp_work_order.required_date AS eta,
				perp_work_order_detail.quantity AS quantity_expected,
				perp_location.location_name,
				\'perp_manufacturing\' AS app,
				CONCAT(\'WORK_ORDER-\', perp_work_order.work_order_id) AS link_id
			FROM perp_order_line
			JOIN perp_order_detail ON
				perp_order_detail.order_id = perp_order_line.order_id AND
				perp_order_detail.line_no = perp_order_line.line_no
			JOIN perp_sales_order_work_order ON
				perp_order_detail.order_id = perp_sales_order_work_order.order_id AND
				perp_order_detail.line_no = perp_sales_order_work_order.line_no AND
				perp_order_detail.detail_no = perp_sales_order_work_order.detail_no
			JOIN perp_work_order_detail ON
				perp_work_order_detail.work_order_id = perp_order_detail.detail_value AND
				perp_work_order_detail.detail_id = perp_sales_order_work_order.work_order_detail_id 
			JOIN perp_work_order ON
				perp_work_order.work_order_id = perp_work_order_detail.work_order_id
			JOIN perp_location ON 
				perp_work_order.location = perp_location.location_id ';

			// Join it all up
			$fields = 'shippable_items.client_id,
				perp_client.client_name,
				perp_client.client_num,
				shippable_items.order_id,
				shippable_items.our_reference_number,
				shippable_items.ship_from_address,
				shippable_items.ship_to_address,
				shippable_items.requested_delivery_date,
				shippable_items.line_no,
				shippable_items.stock_id,
				shippable_items.stock_num,
				shippable_items.description,
				shippable_items.decimal_places,
				shippable_items.unit_of_measure_name,
				shippable_items.stock_category_id,
				shippable_items.stock_type_flag,
				shippable_items.quantity_ordered,
				shippable_items.quantity_shipped, 
				shippable_items.stock_type_flag,
				shippable_items.quantity_unshipped ';
			$from = "FROM ($shippable_items_sql) AS shippable_items ";
			$join = "JOIN perp_client ON
				shippable_items.client_id = perp_client.client_id
			";

			if(count($where) > 0) {
				$where_string .= 'WHERE ' . implode(' AND ', $where);
				$sql .= $where_string;
			}
			$group = "GROUP BY shippable_items.order_id, shippable_items.line_no";
			$sql = "SELECT $fields 
				$from 
				$join 
				$where_string
				$group
				ORDER BY " . $query['order'] . ' ' . $query['sort'];

			// Get the total number of items on backorder (one per item, quantity on back order isn't important here)
			$query['count'] = $GLOBALS['egw']->db->query(
				"SELECT SUM(row) FROM (SELECT 1 AS row FROM ($shippable_items_sql) AS shippable_items $join $where_string $group) AS count", 
			__LINE__, __FILE__)->fetchSingle();

			$GLOBALS['egw']->db->limit_query($sql, $query['start'], __LINE__, __FILE__, $query['num_rows']);
	
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				if($row['work_order_id'] && !$row['completed_date']) {
					$row['to_ship'] = 0;
				}
				$row['requested_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['requested_delivery_date']);

				// Only display the appropriate number of decimals
				$row['quantity_ordered'] = round($row['quantity_ordered'], $row['decimal_places']);
				$row['quantity_shipped'] = round($row['quantity_shipped'], $row['decimal_places']);
				$row['quantity_on_hand'] = round($row['quantity_on_hand'], $row['decimal_places']);
				$row['quantity_expected'] = round($row['quantity_expected'], $row['decimal_places']);
				$row['quantity_unshipped'] = round($row['quantity_unshipped'], $row['decimal_places']);

				$list[] = $row;
			}

			// Look for ETA
			foreach($list as &$item) {
				$where_string = "WHERE perp_order_line.order_id = {$item['order_id']} AND
						perp_order_line.line_no = {$item['line_no']}";
				$sql = "SELECT *
					FROM (
						$purchase_eta_sql 
						$where_string
						HAVING quantity_expected > 0
						UNION 
						$work_order_eta_sql
						$where_string
					) as eta";
				$result = $GLOBALS['egw']->db->query($sql, __LINE__, __FILE__);
				foreach($result as $eta) {
					$eta['eta'] = $GLOBALS['egw']->db->from_timestamp($eta['eta']);
					$eta['unit_of_measure_name'] = $item['unit_of_measure_name'];
					$eta['link'] = array(
						'app'	=>	$eta['app'],
						'id'	=>	$eta['link_id'],
						'title'	=>	$eta['incoming_order_reference']
					);
					$item['incoming'][] = $eta;
				}
			}
			return $list;
		}
	}
?>
