<?php

	class so_perp_orders {

		private $db;

		public function so_perp_orders() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_orders');
		}

		public function get_sales_categories(&$query) {
			$where = array();
			if($query['sales_category_id']) {
				$where['sales_category_id'] = $query['sales_category_id'];
			}
			if($query['parent']) {
				$where['parent_category_id'] = $query['parent'];
			}

			$this->db->select('perp_sales_category',
				array('count(sales_category_id) as count'),
				$where,
				__LINE__, __FILE__
			);
			if($row = $this->db->row(true)) {
				$query['count'] = $row['count'];
			}

			// Get egw categories
			if(!is_object($this->categories)) {
				$this->categories =& CreateObject('phpgwapi.categories', -1, 'perp_orders');
			}
			$categories =& $this->categories->return_sorted_array(
				$query['start'], false, $query['search'],
				$query['sort'], $query['order']
			);
			if($query['num_rows']) {
				$categories = array_slice($categories, (int)$query['start'], $query['num_rows']);
			}

			foreach($categories as &$category) {
				$this->db->select('perp_sales_category', 
					array(
						'*',
					),
					array('sales_category_id' => $category['id']),
					__LINE__, __FILE__
				);
				$row = $this->db->row(true);
				$row['category_id'] = $row['id'];
				$row['sales_category_name'] = $category['name'];
				$row['indent'] = str_repeat('&nbsp;', $category['level'] * 3);
				$row['indented_name'] = $row['indent'] . $category['name'];
				$category['data'] = unserialize($category['data']);
				if($category['data']['color']) {
					$category['color'] = '<div style="width: 100%; height: 100%; background-color:' . $category['data']['color'] . ';">&nbsp;</div>';
				}
				$category += $row;
			}
			return $categories;
		}

		public function get_sales_category($category_id) {
			$this->db->select('perp_sales_category', 
				array('*'),
				array('sales_category_id'	=>	$category_id),
				__LINE__, __FILE__
			);
			$row = $this->db->row(true);
			return is_array($row) ? $row : array();
		}

		public function set_sales_category($category) {
			$this->db->insert('perp_sales_category',
				$category,
				array('sales_category_id' => $category['category_id']),
				__LINE__, __FILE__
			);
		}	
		
		public function delete_sales_category($category_id) {
			$this->db->delete('perp_sales_category',
				array('sales_category_id'	=>	$category_id),
				__LINE__, __FILE__
			);
		}

		public function get_sales_category_stock(&$query = array()) {
			$where = array();
			if(isset($query['sales_category_id'])) {
				$where[] = 'sales_category_id = ' . $GLOBALS['egw']->db->quote($query['sales_category_id']);
			}
			if($query['cat_id']) {
				$where[] = 'stock_category = ' . (int)$query['cat_id'];
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_stock.stock_num LIKE $search OR 
					short_description LIKE $search)";
			}

			// Get number of rows
			$GLOBALS['egw']->db->select('perp_stock_sales_category',
				array('count(perp_stock.stock_id) AS count'),
				$where,
				__LINE__, __FILE__, false,
				'', 'perp_inventory', 1,
				'JOIN perp_stock ON
					perp_stock.stock_id = perp_stock_sales_category.stock_id'
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['total'] = $row['count'];
			}

			// Get rows
			$GLOBALS['egw']->db->select('perp_stock_sales_category',
				array(
					'perp_stock.stock_id',
					'stock_num',
					'short_description',
					'stock_category'
				),
				$where,
				__LINE__, __LINE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'],
				'perp_inventory', $query['num_rows'],
				'JOIN perp_stock ON
					perp_stock.stock_id = perp_stock_sales_category.stock_id'
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$list[] = $row;
			}
			return $list;
		}

		public function get_sales_areas(&$query) {
			$where = array();
			if(isset($query['sales_area_id'])) {
				$where['sales_area_id'] = $query['sales_area_id'];
			}
			$this->db->select('perp_sales_area', 
				array('*'),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_orders', $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}

		public function get_sales_types(&$query) {
			// Set up WHERE clauses
			$where = array();
			if($query['search']) {
				$where[] = 'sales_type_name LIKE ' . $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
			}

			// Get count
			$GLOBALS['egw']->db->select('perp_sales_type',
				array('count(sales_type_id) AS count'),
				$where,
				__LINE__, __FILE_
			);
			if($row = $GLOBALS['egw']->db->row(true)) {
				$query['count'] = $row['count'];
			}

			$this->db->select('perp_sales_type', 
				array('*'),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_orders', $query['num_rows']
			);
			$list = array();
			while($row = $this->db->row(True)) {
				$list[] = $row;
			}
			return $list;
		}
	}
?>
