<?php

	class so_perp_price {

		private $db;

		const NOT_ALLOWED = 'N/A';

		public function so_perp_price() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_orders');
		}
		
		public function get_price_list_stock_items($query) {
			// For filtering prices
			$where = array();

			if(isset($query['stock_id'])) {
				if(is_array($query['stock_id'])) {
					$where[] = 'perp_price.stock_id IN (' . implode(',', $query['stock_id']) . ')';
				} else {
					$where[] = 'perp_price.stock_id = ' . (int)$query['stock_id'];
				}
			}
			if($query['search']) {
				$where[] = '(perp_stock.stock_num LIKE ' . $GLOBALS['egw']->db->quote('%' .$query['search'] . '%') . ' OR
					perp_stock.short_description LIKE ' . $GLOBALS['egw']->db->quote('%' . $query['search'] . '%' ) . ')';
			}
			if(isset($query['price_list_id'])) {
				if(is_array($query['price_list_id'])) {
					$where[] = 'perp_price_list.price_list_id IN (' . implode(',', $query['price_list_id']) . ')';
				} else {
					$where[] = 'perp_price_list.price_list_id = ' . $this->db->quote($query['price_list_id']);
				}
			}
			if(isset($query['date'])) {
				$date = $GLOBALS['egw']->db->to_timestamp($query['date']);
				$where[] = '(perp_price_list.effective_to IS NULL OR perp_price_list.effective_to > ' . $this->db->quote($date) . ')';
				$where[] = "(perp_price.effective_date <= '$date' OR perp_price.effective_date IS NULL)";
			}
			
			// Control return with filter2 
			if(array_key_exists('filter2',$query)) {
				if($query['filter2'] == 'none' || !$query['filter2']){
					$none = array();
					return $none;
				}
				if($query['filter2'] == 'raw'){
					$where[] = 'perp_stock.stock_type_flag = \'R\''; 
				}	
				if(is_numeric($query['filter2'])){
					$where[] = 'perp_stock.stock_id = ' . $this->db->quote($query['filter2']);
                                }
			}
			
			// Exclude retired stock
			$where[] = 'perp_stock.status_flag != \'R\'';
		
			if($query['cat_id']) {
				if(is_array($query['cat_id'])) {
					$where[] = 'perp_price_list.sales_category_id IN (' . implode(',', $query['cat_id']) . ')';
				} else {
					$where[] = 'perp_price_list.sales_category_id = ' . (int)$query['cat_id'];
				}
			}
			if($query['sales_area_id']) {
				if(is_array($query['sales_area_id'])) {
					$where[] = 'perp_price_list.sales_area_id IN (' . implode(',', $query['sales_area_id']) . ')';
				} else {
					$where[] = 'perp_price_list.sales_area_id = ' . (int)$query['sales_area_id'];
				}
			}
			if($query['sales_type_id']) {
				if(is_array($query['sales_type_id'])) {
					$where[] = 'perp_price_list.sales_type_id IN (' . implode(',', $query['sales_type_id']) . ')';
				} else {
					$where[] = 'perp_price_list.sales_type_id = ' . (int)$query['sales_type_id'];
				}
			}
			if($query['order']){
                                $order_string = 'ORDER BY '.$query['order'].' '.$query['sort'];
                        }
			$fields = implode(',', 	array(
				'perp_price.price_list_id',
				'perp_price.stock_id',
				'stock_num',
				'stock_type_flag',
				'short_description AS description',
				'price',
				'effective_date',
				'modified_by',
				'modified_date'
			));
			if(count($where)) {
				$where_string = 'WHERE ' . implode(' AND ', $where);
			}
			if($date) {
				$join = "JOIN (
						SELECT * 
						FROM (
							SELECT perp_price_list.price_list_id,
								perp_stock.stock_id,
								perp_price.price,
								perp_price.effective_date,
								perp_price.modified_by,
								perp_price.modified_date
							FROM perp_price_list
							JOIN perp_stock_sales_category ON
								perp_price_list.sales_category_id = perp_stock_sales_category.sales_category_id
							JOIN perp_stock ON
								perp_stock.stock_id = perp_stock_sales_category.stock_id
							LEFT JOIN perp_price ON
								perp_price.price_list_id = perp_price_list.price_list_id AND
								perp_price.stock_id = perp_stock_sales_category.stock_id
							$where_string
							ORDER BY effective_date DESC, stock_id DESC
						) as dated_prices 
						GROUP BY stock_id, price_list_id 
					) AS perp_price ON	
						perp_price_list.price_list_id = perp_price.price_list_id ";

				if(!$query['include_not_allowed']) {
					$where_string = 'WHERE perp_price.price IS NOT NULL';
				} else {
					$where_string = '';
				}
			} else {
				$join = 'JOIN perp_price ON
					perp_price_list.price_list_id = perp_price.price_list_id ';

				if(!$query['include_not_allowed']) {
					$where_string .= ($where_string ? ' AND ' : 'WHERE ') . 'perp_price.price IS NOT NULL';
				}
			}
			$sql = "SELECT $fields FROM perp_price_list $join 
				JOIN perp_stock ON
					perp_stock.stock_id = perp_price.stock_id
				$where_string
				$order_string";
			$result = $GLOBALS['egw']->db->query($sql, __LINE__, __FILE__, $query['start']);
			
			$price_list_stock_items = array();	
			foreach($result as $row) {
				$price_list_stock_items[] = $row;
			}
			
			return $price_list_stock_items;
		}



		public function get_price_list(&$query, &$readonly) {
			$result = $this->get_price_list_stock_items($query);
			$list = array();
			$not_raw = array();
			$read_only = ($query['date'] < time()) || !$GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders');
			if(is_array($result)){
				foreach($result as $row) {
					$row['effective_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_date']);
					$row['modified_date'] = $GLOBALS['egw']->db->from_timestamp($row['modified_date']);
					if($row['stock_type_flag'] != 'R') {
						$not_raw[count($list)] = $row;
					}
					if(is_null($row['price'])) {
						$row['price'] = self::NOT_ALLOWED;
					} elseif ($query['format']) {
						$row['price'] = number_format($row['price'], 2);
					}
					$list[] = $row;
					$readonly[ count($list)  . '[price]'] = $read_only;
					$readonly[ count($list) ] = $read_only;
				}
			}
			// Stock items that are not raw, and may have options or need recursion
			$option_query = $query;
			$kit_query = $query;
			unset($option_query['rows']);
			unset($kit_query['rows']);
			foreach($not_raw as $key => $row) {
				if($row['stock_type_flag'] == 'K'){     
                                	// Look for components (Kits)
                                	$kit_query['price_list_id']     =       $row['price_list_id'];
                                	$kit_query['stock_id']          =       $row['stock_id'];
                                	$kit_query['kit_stock_id']      =       $row['stock_id'];
                                	$kit_query['effective_date']    =       $query['date'];
                                	$kit_query['stock_type_flag']   =       $row['stock_type_flag'];
                                
                                	$components =& so_perp_kit_price::get_kit_price_list($kit_query);
                                	if(count($components) > 0) {
                                        	$list[$key]['components'] = $components;
                                        	$readonly[ ($key + 1)  . '[components]'] = $read_only;
                                	}
                                }elseif($row['stock_type_flag'] == 'A'){
                                	// Look for options (Assemblies and Kits)
                                	$option_query['price_list_id']  =       $row['price_list_id'];
                                	$option_query['stock_id']       =       $row['stock_id'];
                                	$option_query['component_id']   =       $row['stock_id'];
                                	$option_query['stock_type_flag']        =       $row['stock_type_flag'];
                               		$option_query['parent_component_id']        =       $row['stock_id']; 
                                	if($query['include_options']) {
                                        	$options =& $this->get_option_price_list($option_query);
                                        	if(count($options) > 0) {
                                                	$list[$key]['options'] = $options;
                                                	$readonly[ ($key + 1)  . '[options]'] = $read_only;
						}
                                	}
                        	}
                        }
			return $list;
		}
		/**
                *       Use later for view
                */
	
		public function get_created_price_list(&$query, &$readonly) {
			//_debug_array($query);
			
		
		}
		
		/**
		*	Get a price for a build option
		*/
		public function get_build_option_price($query){
			$where = array();
                        if($query['price_list_id']) {
                                $where['price_list_id'] = $query['price_list_id'];
                        }
                        if($query['stock_id']) {
                                $where['stock_id'] = $query['stock_id'];
                        }
                        if($query['build_option_id']) {
                                $where['build_option_id'] = $query['build_option_id'];
                        }
                        if($query['parent_component_id']) {
                                $where['parent_component_id'] = $query['parent_component_id'];
                        }
                        if($query['date']) {
                        	$date = $GLOBALS['egw']->db->to_timestamp($query['date']);
                                $where[] = '(perp_build_option_price.effective_date <= ' . $this->db->quote($date) . ')';
			}

			//$GLOBALS['egw']->db->Debug = true;
                        $result = $GLOBALS['egw']->db->select(
                                'perp_build_option_price',
                                array('*'),
                                $where,
                                __LINE__, __FILE__,
				$offset = 0,
				'ORDER BY effective_date DESC',
				1
                        );
			//$GLOBALS['egw']->db->Debug = false;
                        $list = array();
                        foreach($result as $row) {
                                if($row['price'] == null){
					$row['price'] = self::NOT_ALLOWED;
				}
				$row['effective_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_date']);
                                $row['modified_date'] = $GLOBALS['egw']->db->from_timestamp($row['modified_date']);
				
                                $list[] = $row;
                        }
			return $list;


		}


		/**
		*	Get a price list for options
		*	@param price_list_id The ID of the price list to look on
		*	@param date Effective date for the price 
		*	@param component_id the stock_id of the component the option is on
		*	@param build_option_id The ID of a build option
		*	@param stock_id The top-most stock item that the customer is actually buying
		*	@param inclue_not_allowed Include options that are not on this price list
		*/
		public function get_option_price_list(&$query) {
			
			if($query['date']) {
				$date = $GLOBALS['egw']->db->to_timestamp($query['date']);
			}

			/**
			* Get allowed build options for the given parameters
			* The function called takes into account remaining stock on hand, so we can
			* sell out of discontinued options
			*/
			$allowed_build_option_list_query['stock_id'] = $query['component_id'];
			$allowed_build_option_list_query['as_of_date'] = $query['date'];
			$allowed_build_option_list_query['build_option_category'] = $query['build_option_category'];
			$allowed_options = ExecMethod2('perp_manufacturing.so_perp_build_option.get_allowed_build_option_list', $allowed_build_option_list_query);
			$option_categories = array();
			if(array_key_exists('stock_id', $query) && !$query['stock_id'] && $query['build_option_id']) {
				// Looking for default option pricing
				$build_option = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option', $query['build_option_id']);
				$option_categories[] = array(
					'build_option_category_id'	=>	$build_option['build_option_category']
				);
			} else {
				foreach($allowed_options as $option) {
					if(!isset($option_categories[$option['build_option_category']])) {
						$option_categories[$option['build_option_category']] = array(
							'build_option_category_id'	=>	$option['build_option_category'],
							'build_option_category_name'	=>	$option['build_option_category_name']
						);
					}
					$options[$option['build_option_id']] = $option;
					$option_categories[$option['build_option_category']]['allowed_option_ids'][] = $option['build_option_id'];
				}
			}
			
			// Get the prices
			foreach($option_categories as &$category) {
				$category['build_option_prices'] = array();
				if(count($category['allowed_option_ids']) == 0 && !$query['build_option_id']) {
					continue;
				}

				// This has to be done as SQL, because the WHERE in the nested SELECT confuses
				// egw_db into putting all the WHERE together, in the nested SELECT.

				$sql = 'SELECT
						perp_build_option.build_option_id,
						perp_build_option.build_option_name,
						if (perp_build_option_price.price is null,default_price.price,perp_build_option_price.price) AS price,
						if (perp_build_option_price.price is null,false,if(default_price.price is not null,if(perp_build_option_price.price != default_price.price,true,false ),true)) AS price_is_override,
						
						if (perp_build_option_price.effective_date is null,default_price.effective_date,perp_build_option_price.effective_date) AS effective_date,
						CASE
                				WHEN perp_build_option_price.effective_date THEN future_overrides.future_effective_date
                				WHEN perp_build_option_price.effective_date is null AND future_overrides.future_effective_date is not null THEN future_overrides.future_effective_date
                				WHEN perp_build_option_price.effective_date is null AND future_overrides.future_effective_date is null THEN default_price.effective_to
            					END AS effective_to,
						
						perp_build_option_price.modified_date,
						perp_build_option_price.modified_by,
						perp_build_option_price.parent_component_id,
						perp_stock.stock_type_flag
					FROM perp_allowed_build_option 
					JOIN perp_build_option ON
						perp_allowed_build_option.build_option_id = perp_build_option.build_option_id
					JOIN perp_build_option_category ON
						perp_build_option.build_option_category = perp_build_option_category.build_option_category_id
					LEFT JOIN (
						SELECT * FROM (
							SELECT perp_build_option_price.*, MIN(next_option_price.effective_date) AS effective_to
							FROM perp_build_option_price
							LEFT JOIN (SELECT * FROM perp_build_option_price
                                     				   ORDER by effective_date DESC
                                    				  ) AS next_option_price
                          				ON next_option_price.build_option_id = perp_build_option_price.build_option_id 
                          				AND next_option_price.stock_id = perp_build_option_price.stock_id 
                          				AND next_option_price.effective_date > perp_build_option_price.effective_date 
							
							WHERE perp_build_option_price.effective_date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['date']) . '\' 
							AND perp_build_option_price.price_list_id = ' . (int)$query['price_list_id'] . '
							AND perp_build_option_price.parent_component_id = ' . (int)$query['parent_component_id'] . '
							GROUP BY perp_build_option_price.build_option_id, perp_build_option_price.stock_id, perp_build_option_price.effective_date
							ORDER BY effective_date DESC
						) as dated_prices
						GROUP BY stock_id, build_option_id
						ORDER BY stock_id DESC
					) AS perp_build_option_price ON
						perp_allowed_build_option.build_option_id = perp_build_option_price.build_option_id AND
						(perp_allowed_build_option.stock_id = perp_build_option_price.stock_id OR (perp_build_option_price.stock_id = ' . (int)$query['stock_id'] . ' OR perp_build_option_price.stock_id = 0)) AND
						perp_build_option_price.price_list_id = ' . (int)$query['price_list_id'] . '
					LEFT JOIN (
						SELECT perp_build_option_price.*, MIN(next_option_price.effective_date) AS effective_to
                                                FROM perp_build_option_price
           
           					LEFT JOIN (SELECT * FROM perp_build_option_price
                      					   ORDER by effective_date DESC
                     					  ) AS next_option_price
           					ON next_option_price.build_option_id = perp_build_option_price.build_option_id 
           					AND next_option_price.stock_id = perp_build_option_price.stock_id
           					AND next_option_price.effective_date > perp_build_option_price.effective_date
						AND next_option_price.price_list_id = perp_build_option_price.price_list_id
						WHERE perp_build_option_price.effective_date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['date']) . '\' 
                                                AND perp_build_option_price.price_list_id = ' . (int)$query['price_list_id'] . '
           					AND perp_build_option_price.stock_id = 0
           					GROUP BY perp_build_option_price.build_option_id, perp_build_option_price.stock_id, perp_build_option_price.effective_date
						ORDER BY effective_date DESC
					) AS default_price ON
                                        perp_allowed_build_option.build_option_id = default_price.build_option_id AND
                                        default_price.price_list_id = ' . (int)$query['price_list_id'] . ' AND
                                        default_price.effective_date <= \'' . $GLOBALS['egw']->db->to_timestamp($query['date']) . '\' AND
                                        default_price.stock_id = 0					
					LEFT JOIN perp_stock ON
						perp_stock.stock_id = perp_allowed_build_option.component
					LEFT JOIN (
           					SELECT *, MIN(effective_date) AS future_effective_date
           					FROM perp_build_option_price
            					WHERE perp_build_option_price.effective_date > \'' . $GLOBALS['egw']->db->to_timestamp($query['date']) . '\' 
            					AND perp_build_option_price.price_list_id = ' . (int)$query['price_list_id'] . ' 
            					AND perp_build_option_price.stock_id = ' . (int)$query['stock_id'] . '
            					AND perp_build_option_price.parent_component_id = ' . (int)$query['parent_component_id'] . '
            					GROUP BY perp_build_option_price.build_option_id, perp_build_option_price.stock_id
    						) AS future_overrides
    						ON perp_allowed_build_option.build_option_id = future_overrides.build_option_id
					WHERE 
						perp_build_option.build_option_category = ' . $category['build_option_category_id'];
				if($query['build_option_id']) {
					$sql .= ' AND perp_build_option.build_option_id IN (' . implode(',', (array)$query['build_option_id']) . ') ';
				} else {
					$sql .= ' AND perp_build_option.build_option_id IN (' . implode(',', $category['allowed_option_ids']) . ') ';
				}
				$sql .= 'GROUP BY perp_build_option.build_option_id';
				foreach($GLOBALS['egw']->db->query($sql, __LINE__, __FILE__) as $row) {
					
					$row['effective_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_date']);
					$row['effective_to'] = $GLOBALS['egw']->db->from_timestamp($row['effective_to']);
					$row['modified_date'] = $GLOBALS['egw']->db->from_timestamp($row['modified_date']);
					$row['build_option_name'] = $options[$row['build_option_id']]['build_option_name'];
					$row['component'] = $options[$row['build_option_id']]['component'];
					if(is_null($row['price'])) {
						if($query['include_not_allowed']) {
							$row['price'] = self::NOT_ALLOWED;
						} else {
							continue;
						}
					} else {
						$row['price'] = number_format($row['price'], 2);
					}
					$query['count']++;
					$category['build_option_prices'][] = $row;
				}

				$kit_query = $query;
				unset($kit_query['component_id']);
				foreach($category['build_option_prices'] as &$option_price) {
					if($option_price['stock_type_flag'] == 'K') {
						// Look for components (Kits)
						$kit_query['price_list_id']	=	$query['price_list_id'];
						$kit_query['stock_id']		=	$query['stock_id'];
						$kit_query['kit_stock_id']	=	$option_price['component'];
						$kit_query['effective_date']	=	$query['date'];
						$kit_query['stock_type_flag']	=	$option_price['stock_type_flag'];
						
						$components =& so_perp_kit_price::get_kit_price_list($kit_query);
						if(count($components) > 0) {
							$option_price['components'] = $components;
							$readonly[ ($key + 1)  . '[components]'] = $read_only;
						}
					}

					if($option_price['component']) {
						$sub_query = $query;
						$sub_query['component_id'] = $option_price['component'];
						$sub_query['parent_component_id'] = $option_price['component'];
						$sub_options = $this->get_option_price_list($sub_query);
						if(count($sub_options) > 0) {
							$option_price['options'] = $sub_options;
						}
					}
				}
			}
			reset($option_categories);

			// Look for options on components
			// Get bill of materials
			if($query['component_id'] && $query['stock_type_flag'] != 'K') {
				$bom_id = ExecMethod2('perp_inventory.bo_perp_stock.get_current_bom', $query['component_id'], $query['date']);
				$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $bom_id);
				if(is_array($bom['details'])) {
					foreach($bom['details'] as $detail) {
						if($detail['stock_type_flag'] == 'R') {
							continue;
						}
						$sub_query = $query;
						$sub_query['component_id'] = $detail['component'];
						$bom_options = $this->get_option_price_list($sub_query);
						if(count($bom_options) > 0) {
							$option_categories = array_merge($option_categories, $bom_options);
						}
					}
				}
			}
			return $option_categories;
		}

		public function set_price($price_list_id, $stock_id, $price, $date, $build_option_id, $current_price = null, $parent_component_id = null) {
			if(strtoupper($price) === strtoupper(self::NOT_ALLOWED)) {
				$price = null;
			}

			// Convert date to DB specific timestamp format
			$date = $GLOBALS['egw']->db->to_timestamp($date);

			$GLOBALS['egw']->db->transaction_begin();
			if($stock_id && !$build_option_id) {
				// Price for a stock item
				$GLOBALS['egw']->db->insert('perp_price', array(
						'price_list_id'		=>	$price_list_id,
						'stock_id'		=>	$stock_id,
						'price'			=>	$price,
						'effective_date'	=>	$date,
						'modified_by'		=>	$GLOBALS['egw_info']['user']['account_id'],
					),
					array(
						'price_list_id'		=>	$price_list_id,
						'stock_id'		=>	$stock_id,
						'effective_date'	=>	$date,
					),
					__LINE__, __FILE__, 'perp_orders'
				);
			} elseif ($build_option_id) {
				// Price for an option
				if(!$stock_id) {
					// $stock_id can't be null
					$stock_id = '';
				}
				if(!$stock_id) {
					$GLOBALS['egw']->db->insert('perp_build_option_price', array(
							'build_option_id'	=>	$build_option_id,
							'stock_id'		=>	$stock_id,
							'price_list_id'		=>	$price_list_id,
							'price'			=>	$price,
							'effective_date'	=>	$date,
							'modified_by'		=>	$GLOBALS['egw_info']['user']['account_id'],
							'parent_component_id'	=>	$parent_component_id
						),
						array(
							'build_option_id'	=>	$build_option_id,
							'stock_id'		=>	$stock_id,
							'price_list_id'		=>	$price_list_id,
							'effective_date'	=>	$date,
							'parent_component_id'	=>	$parent_component_id
						),
						__LINE__, __FILE__, 'perp_orders'
					);
					
					if($current_price == self::NOT_ALLOWED) {
						$current_price = null;
					}
					/**
					*	This is a default price.
					*	We'll update all the options that match the 
					*	current default price to the new default price
					*/
					// Current
					$result = $GLOBALS['egw']->db->select(
						'perp_build_option_price',
						array(
							'stock_id',
							'build_option_id',
							'parent_component_id'
						),
						array(
							'build_option_id'	=>	$build_option_id,
							'price_list_id'		=>	$price_list_id,
							'price'			=>	$current_price,
						),
						__LINE__, __FILE__
					);
					$current = array();
					foreach($result as $row) {
						$current[] = $row;
					}
					foreach($current as $item) {
						$GLOBALS['egw']->db->insert(
							'perp_build_option_price',
							array(
								'build_option_id'	=>	$build_option_id,
								'stock_id'		=>	$item['stock_id'],
								'price_list_id'		=>	$price_list_id,
								'price'			=>	$price,
								'effective_date'	=>	$date,
								'modified_by'		=>	$GLOBALS['egw_info']['user']['account_id'],
							),
							array(
								'build_option_id'	=>	$item['build_option_id'],
								'stock_id'		=>	$item['stock_id'],
								'price_list_id'		=>	$price_list_id,
								'effective_date'	=>	$date,
								'parent_component_id'	=>	$item['parent_component_id']
							),
							__LINE__, __FILE__, 'perp_orders'
						);
					}
				} else {
					/**
					*	This is the price for one particular option for one particular
					*	stock item.  Insert the new option price.
					*/
					$GLOBALS['egw']->db->insert('perp_build_option_price', array(
							'build_option_id'	=>	$build_option_id,
							'stock_id'		=>	$stock_id,
							'price_list_id'		=>	$price_list_id,
							'price'			=>	$price,
							'effective_date'	=>	$date,
							'modified_by'		=>	$GLOBALS['egw_info']['user']['account_id'],
							'parent_component_id'	=>	$parent_component_id
						),
						array(
							'build_option_id'	=>	$build_option_id,
							'stock_id'		=>	$stock_id,
							'price_list_id'		=>	$price_list_id,
							'effective_date'	=>	$date,
							'parent_component_id'	=>	$parent_component_id
						),
						__LINE__, __FILE__, 'perp_orders'
					);
				}
			}
			$GLOBALS['egw']->db->transaction_commit();
			return true;
		}

		/**
		*	Get option price history
		*/
		public function get_option_price_history(&$query) {
			$where = array();
			if($query['price_list_id']) {
				$where['price_list_id'] = $query['price_list_id'];
			}
			if($query['stock_id']) {
				$where['stock_id'] = $query['stock_id'];
			}
			if($query['build_option_id']) {
				$where['build_option_id'] = $query['build_option_id'];
			}
			if($query['parent_component_id']) {
				$where['parent_component_id'] = $query['parent_component_id'];
			}

			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_build_option_price',
				array('count(*)'),
				$where,
				__LINE__, __FILE__, false, '', 'perp_orders'
			)->fetchSingle();

			$result = $GLOBALS['egw']->db->select(
				'perp_build_option_price',
				array('*'),
				$where,
				__LINE__, __FILE__, $query['start'],
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 'perp_orders',
				$query['num_rows']
			);
			$list = array();
			foreach($result as $row) {
				$row['effective_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_date']);
				$row['modified_date'] = $GLOBALS['egw']->db->from_timestamp($row['modified_date']);

				if(is_null($row['price'])) {
					$row['price'] = self::NOT_ALLOWED;
				} elseif ($query['format']) {
					$row['price'] = number_format($row['price'], 2);
				}

				$list[] = $row;
			}

			return $list;
		}
	}
?>
