<?php

	/**
	*	Storage for calculations that automatically adjust prices
	*
	*	@author Nathan Gray
	*	@package perp_orders
	*/

	require_once( EGW_INCLUDE_ROOT . '/etemplate/inc/class.so_sql.inc.php');

	require_once( EGW_INCLUDE_ROOT . '/perp_orders/inc/class.bo_perp_price.inc.php');

	class so_perp_price_calculation extends so_sql {

		public function __construct() {
			parent::__construct('perp_orders', 'perp_price_calculation');
		}


		public static function run(Array $parameters, $make_changes = true) {
			
			$bo_price = new bo_perp_price();

			$query = array(
				'price_list_id'	=>	$parameters['price_list_id'],
				'date'		=>	time(),
				'template'	=>	false,
				'num_rows'	=>	-1,
				'order'		=>	'price_list_id, stock_id',
			);
			if($parameters['stock_id']) {
				$query['stock_id'] = $parameters['stock_id'];
			}
			if($parameters['num_rows']) {
				$query['num_rows'] = $parameters['num_rows'];
			}

			// Needed for some costing, just use the first one
			$location_id = key(ExecMethod('perp_inventory.bo_perp_location.get_location_list'));

			// Caching to speed up 
			$current_price_list = null;

			$prices = array();

			$GLOBALS['egw']->db->transaction_begin();

			// Buffer any output, it's probably an error
			ob_start();

			try {
				// Get current prices
				$bo_price->get_price_list($query, $prices);

				// Calculate new prices
				foreach($prices as &$item) {
					if(!$item['stock_id']) {
						continue;
					}

					// Set the available variables

					// FIFO Cost
					$current_fifo = ExecMethod2('perp_inventory.bo_perp_inventory.get_cost', $item['stock_id'], $location_id, 1);

					// Catalog cost
					$current_catalog = 0;
					$suppliers = ExecMethod('perp_inventory.bo_perp_stock.get_suppliers', $item['stock_id']);
					foreach($suppliers as $supplier) {
						if(count($suppliers) == 1 || $supplier['preferred']) {
							$current_catalog = $supplier['price'] * $supplier['quantity'] / $supplier['conversion_factor'] + $supplier['average_indirect_cost'];
						}
					}

					// Average cost
					$current_average = $GLOBALS['egw']->db->select(
						'perp_fifo_costing',
						array('sum((direct_cost + indirect_cost) * perp_fifo_costing.quantity) /sum(perp_fifo_costing.quantity)'),
						array('perp_fifo_costing.stock_id = ' . (int)$item['stock_id']),
						__LINE__, __FILE__, 0, '', 'perp_inventory',1,
						'JOIN perp_stock_cost ON
							perp_fifo_costing.stock_cost_id = perp_stock_cost.stock_cost_id'
					)->fetchSingle();

					// Exchange rate
					$exchange_rate = 1;
					if($item['price_list_id'] != $current_price_list['price_list_id']) {
						$current_price_list = ExecMethod('perp_orders.bo_perp_price_list.get_price_list', $item['price_list_id']);
					}
					$currencies = ExecMethod('perp_api.bo_perp_api.get_currencies', array('currency_id' => $current_price_list['currency']));
					$exchange_rate = $currencies[0]['exchange_rate'];
					
					// Current price
					$current_price = $item['price'];


					// Calculate the new price
					eval('$price = ' . $parameters['calculation']. ';');
					if(is_numeric($price)) {
						// Force rounding, or when we set the price the 3 decimal places will screw up the conversion from string
						$price = round($price, 2);
					}

					// Set the new price
					if($make_changes) {
						$bo_price->set_price($item['price_list_id'], $item['stock_id'], $price, $parameters['effective_date']);
					} else {
						$item['new_price'] = $price;
					}

				}
			} catch (Exception $e) {
				$GLOBALS['egw']->db->transaction_abort();

				// Log exception
				$GLOBALS['egw']->log->error(array(
					'text'	=>	'E-PriceCalculation, ID# %1 <br />Error was: %2 <br />Parameters: %3',
					'p_1'	=>	$parameters['id'],
					'p_2'	=>	$e->getMessage(),
					'p_3'	=>	print_r($parameters, true),
					'line'	=>	__LINE__,
					'file'	=>	__FILE__
				));
				$GLOBALS['egw']->log->commit();
				return $e->getMessage();
			}

			$output = ob_get_clean();
			if(trim($output)) {
				$GLOBALS['egw']->db->transaction_abort();
				$GLOBALS['egw']->log->error(array(
					'text'	=>	'E-PriceCalculation, ID# %1 <br />Error was: %2 <br />Parameters: %3',
					'p_1'	=>	$parameters['id'],
					'p_2'	=>	$output,
					'p_3'	=>	print_r($parameters, true),
					'line'	=>	__LINE__,
					'file'	=>	__FILE__
				));
				$GLOBALS['egw']->log->commit();
				return $output;
			}

			$GLOBALS['egw']->db->transaction_commit();
			
			return $make_changes ? true : $prices;
		}
	}
?>
