<?php

	class so_perp_price_list {

		private $db;

		public function so_perp_price_list() {
			$this->db = clone($GLOBALS['egw']->db);
			$this->db->set_app('perp_orders');
		}

		/* Get list of price lists
		*	query = array(
		*		'sales_category_id',
		*		'sales_area_id',
		*		'sales_type_id',
		*		'currency',
		*		
		*		'status',
		*		'date', 
		*	
		*		'sort',
		*		'order'
		*	)
		*/
		public function get_price_list_list(&$query) {

			// Get egw categories
			if(!is_object($this->categories)) {
				$this->categories =& CreateObject('phpgwapi.categories', -1, 'perp_orders');
			}
			$categories =& $this->categories->return_sorted_array(
				$query['start'], false, '',
				$query['sort'], false,
				true, $query['cat_id']
			);
			if($query['sales_category_id']) {
				$parent = $this->categories->return_single($query['sales_category_id']);
				if(is_array($parent)) {
					$categories[] = $parent[0];
				}
			}

			$where = array();
			switch($query['status']) {
			case 'current':
				$where_status = "(effective_to >= '" . $this->db->to_timestamp($query['status_date']) . "' OR effective_to IS NULL)";
				break;
			case 'previous':
				$where_status = "effective_to <= '" . $this->db->to_timestamp($query['status_date']) . "'";
				break;
			}

			// Get total number of rows
			$where = array();
			if($where_status) {
				$where[] = $where_status;
			}
			if($query['sales_category_id']) {
				$where[] = 'perp_price_list.sales_category_id = ' . (int)$query['sales_category_id'];
			}
			if($query['search']) {
				$search = $this->db->quote('%' . $query['search'] . '%');
				$where[] = "perp_price_list.price_list_name LIKE $search OR perp_price_list.price_list_title LIKE $search";
			}
			if($query['client_id']) {
				$where[] = 'perp_client_account.client_id IN (' . implode(',', $query['client_id']) . ')';
			}
			if($query['date']) {
				$date = $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['date']));
				$where[] = "(perp_price_list.effective_to > $date OR perp_price_list.effective_to IS NULL)";
			}

			$query['count'] = $GLOBALS['egw']->db->select('perp_price_list', 
				array('count(perp_price_list.price_list_id) AS count'),
				$where,
				__LINE__, __FILE__, false,
				'', False, 0,
				'JOIN perp_sales_category ON
					perp_sales_category.sales_category_id = perp_price_list.sales_category_id
				JOIN perp_sales_area ON
					perp_sales_area.sales_area_id = perp_price_list.sales_area_id
				JOIN perp_sales_type ON
					perp_sales_type.sales_type_id = perp_price_list.sales_type_id ' . 
				($query['client_id'] ? 'JOIN perp_client_account_price_list ON 
					perp_client_account_price_list.price_list_id = perp_price_list.price_list_id
				JOIN perp_client_account ON
					perp_client_account.client_account_id = perp_client_account_price_list.client_account_id' : '')
			)->fetchSingle();
			
			if($where_status) {
				$where[] = $where_status;
			}
			if($query['sales_area_id']) {
				$where[] = 'perp_price_list.sales_area_id = ' . $query['sales_area_id'];
			}
			if($query['sales_type_id']) {
				$where[] = 'perp_price_list.sales_type_id = ' . $query['sales_type_id'];
			}
			if($query['search']) {
				$search = $this->db->quote('%' . $query['search'] . '%');
				$where[] = "perp_price_list.price_list_name LIKE $search";
			}
			if($query['client_id']) {
				$where[] = 'perp_client_account.client_id IN (' . implode(',', $query['client_id']) . ')';
			}
			if($query['date']) {
				// $date set above
				$where[] = "(perp_price_list.effective_to > $date OR perp_price_list.effective_to IS NULL)";
			}
			
			$this->db->select('perp_price_list', array(
				'perp_price_list.price_list_id',
				'price_list_name',
				'price_list_title',
				'currency',
				'perp_price_list.sales_category_id',
				'egw_categories.cat_name',
				'sales_area_name',
				'sales_type_name',
				'order_by',
				'sort',	
				'effective_to',
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				"ORDER BY ${query['order']} ${query['sort']}", False, $query['num_rows'],
				'JOIN egw_categories ON
					egw_categories.cat_id = perp_price_list.sales_category_id
				JOIN perp_sales_area ON
					perp_sales_area.sales_area_id = perp_price_list.sales_area_id
				JOIN perp_sales_type ON
					perp_sales_type.sales_type_id = perp_price_list.sales_type_id ' . 
				($query['client_id'] ? 'JOIN perp_client_account_price_list ON 
					perp_client_account_price_list.price_list_id = perp_price_list.price_list_id
				JOIN perp_client_account ON
					perp_client_account.client_account_id = perp_client_account_price_list.client_account_id' : '')
			);
			$price_lists = array();
			while($row = $this->db->row(True)) {
				$row['effective_to'] = $this->db->from_timestamp($row['effective_to']);
				if($row['sort'] == 1) {
					$row['sort'] = 'ASC';
				} elseif ($row['sort'] == -1) {
					$row['sort'] = 'DESC';
				}
				$price_lists[] = $row;
			}
			
			return $price_lists;
		}

		public function get_price_list($id) {
			$this->db->select('perp_price_list', 
				array('*'),
				array('price_list_id' => $id),
				__LINE__, __FILE__
			);
			$list = array();
			if($row = $this->db->row(True)) {
				$row['effective_to'] = $this->db->from_timestamp($row['effective_to']);
				if($row['sort'] == 1) {
					$row['sort'] = 'ASC';
				} elseif ($row['sort'] == -1) {
					$row['sort'] = 'DESC';
				}
				$list = $row;
			}

			return $list;
		}

		public function set_price_list($id, $price_list) {
			$this->db->insert('perp_price_list', array(
					'price_list_id'		=>	$id,
					'price_list_name'	=>	$price_list['price_list_name'],
					'price_list_title'	=>	$price_list['price_list_title'],
					'currency'		=>	$price_list['currency'],
					'sales_category_id'	=>	$price_list['sales_category_id'],
					'sales_area_id'		=>	$price_list['sales_area_id'],
					'sales_type_id'		=>	$price_list['sales_type_id'],
					'order_by'		=>	$price_list['order_by'],
					'sort'			=>	(int)($price_list['sort']),
					'effective_to'		=>	$price_list['effective_to'],
				), 
				array(
					'price_list_id'	=>	$id,
				),
				__LINE__, __FILE__
			);
			if(!$id) {
				$new_price_list = true;
			}
			$id = $id ? $id : $this->db->get_last_insert_id('perp_price_list', 'price_list_id'); 
			/*
			//Nathan had this code in here. As far as i'm concerned it's not needed. Maybe it was here for he was going to do
			//who knows, but it is commented out to speed up the creation of a price list.
			if($new_price_list) {
				// Add all the stock in the sales category to this price list
				$bo_orders = CreateObject('perp_orders.bo_perp_orders');
				$query = array('sales_category_id' => $price_list['sales_category_id'], 'num_rows' => -1);
				$all_stock = array();
				$bo_orders->get_sales_category_stock($query, $all_stock);

				foreach($all_stock as &$new_item) {
					if(!isset($new_item['stock_id'])) {
						continue;
					}
					// Get options for the new item, need to set them, too
					$option_query = array(
						'date'			=>	$price_list['effective_date'],
						'component_id'		=>	$new_item['stock_id'],
						'stock_id'		=>	$new_item['stock_id'],
						'include_not_allowed'	=>	true
					);
					$options = ExecMethod2('perp_orders.so_perp_price.get_option_price_list', $option_query);
					if(count($options)) {
						$new_item['options'] = $options;
					}
						
					$price_list['prices'][] = $new_item;
				}
			}
			*/
			// No prices, (that's wierd) just return the ID.
			if(!is_array($price_list['prices'])) {
				$bo_orders = CreateObject('perp_orders.bo_perp_orders');
                               	$query = array('sales_category_id' => $price_list['sales_category_id'], 'num_rows' => -1);
                               	$all_stock = array();
                               	$bo_orders->get_sales_category_stock($query, $all_stock);
				unset($all_stock['delete']);
				foreach($all_stock as $new_item) {
					 ExecMethod2('perp_orders.bo_perp_price.set_price', $id, $new_item['stock_id'], so_perp_price::NOT_ALLOWED, time());
				}
				return $id;
			}
			foreach($price_list['prices'] as $stock_item) {
				if(is_array($stock_item) && $stock_item['stock_id']) {
					if(is_array($stock_item['components'])) {
						$this->set_component_price($id, $price_list['effective_date'], $stock_item);
					}
					ExecMethod2('perp_orders.bo_perp_price.set_price', $id, $stock_item['stock_id'], $stock_item['price'], $price_list['effective_date']);
					if(is_array($stock_item['options'])) {
						$this->set_option_price($id, $price_list['effective_date'], $stock_item['options'], $stock_item['stock_id'], $stock_item['stock_id']);
					}
				}
			}
			return $id;
		}

		/**
		*	Recursive function to set the price for an item with components
		*/
		private function set_component_price($price_list_id, $effective_date, &$stock_item) {
			// Overall price is the sum of the components
			$stock_item['price'] = null;

			// Set price for components
			foreach($stock_item['components'] as $component) {
				if(strtoupper($component['price']) === strtoupper(so_perp_price::NOT_ALLOWED) || trim($component['price']) == '') {
					$component['price'] = null;
				} elseif(!is_null($component['price']) && $component['price'] != 0) {
					$component['price'] = ExecMethod('perp_api.bo_perp_api.get_float_from_string', $component['price']);
				}
				$result = bo_perp_kit_price::set_price(
					$price_list_id, $stock_item['stock_id'], 
					$component['kit_stock_id'],
					$component['component_id'], $component['price'], $effective_date
				);
				if($result && !is_null($component['price'])) {
					$stock_item['price'] += $component['price'];
				}
			}
		}

		/**
		*	Recursive function to set options on a stock item
		*/
		private function set_option_price($price_list_id, $effective_date, $options, $stock_id = 0, $parent_component_id = 0) {
			if($options['date']) {
				$effective_date = $options['date'];
				unset($options['date']);
			}
			$result = true;
			foreach($options as $option_category) {
				if($option_category['date']) {
					$effective_date = $option_category['date'];
					unset($option_category['date']);
				}
				foreach($option_category['build_option_prices'] as $option) {
					if($option_category['date']) {
						$effective_date = $option['date'];
						unset($option['date']);
					}

					// Check for recurse sub-array, put there as a requirement of eTemplate
					// Left to come back to so_ instead of being dealt with in ui_ because of performance
					if(is_array($option['recurse'])) {
						$option = array_merge($option, $option['recurse']);
						unset($option['recurse']);
					}
					if(is_array($option['options'])) {
						if($option['component']) {
                                        		$new_parent_component_id = $option['component'];	
						}
						$this->set_option_price($price_list_id, $effective_date, $option['options'], $stock_id, $new_parent_component_id);
					}
					if(is_array($option['components'])) {
						$option['stock_id'] = $stock_id;
						$this->set_component_price($price_list_id, $effective_date, $option);
					}
					$result = ExecMethod2('perp_orders.bo_perp_price.set_price', $price_list_id, $stock_id, $option['price'], $effective_date, $option['build_option_id'], $parent_component_id);
					if($result !== true) {
						return $result;
					}
				}
			}
			return $result;
		}

		public function delete_price_list($id) {
			$this->db->delete('perp_price',
				array('price_list_id' => $id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_build_option_price',
				array('price_list_id' => $id),
				__LINE__, __FILE__
			);
			$this->db->delete('perp_price_list', 
				array('price_list_id' => $id),
				__LINE__, __FILE__
			);
		}

		public function view_price_list($price_list_id, $category_id, $area_id, $type_id, $date, $edit) {
			$edit &= $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders');
			$where = array();
			
			if($price_list_id) {
				$where['price_list_id'] = $price_list_id;
			}
			if($category_id) {
				$where['sales_category_id'] = $category_id;
			}
			if($area_id) {
				$where['sales_area_id'] = $area_id;
			}
			if($type_id) {
				$where['sales_type_id'] = $type_id;
			}
			if(!$date) {
				$date = time();
			}
			
			$list = array();
			$category = array();

			// Get Category information
			$this->db->select('perp_sales_category', array(
					'sales_category_id',
					'cat_name AS sales_category_name',
				),
				array('sales_category_id' => $category_id),
				__LINE__, __FILE__, false,
				'', false, 1,
				'JOIN egw_categories ON
					perp_sales_category.sales_category_id = egw_categories.cat_id'
			);
			$list = array();
			if($row = $this->db->row(True)) {
				$category['category'] = $row;
			} else {
				return array();
			}
			
			// Get stock in this category
			$stock_list = array();
			$stock_where = array(
				'sales_category_id' => $category_id,
			);
			if(!$edit) {
				// Only production & discontinued stock
				$stock_where[] = "status_flag IN ('P','D')";
			}
			$this->db->select('perp_stock_sales_category', array(
					'stock_id'
				),
				$stock_where,
				__LINE__, __FILE__, False,
				"ORDER BY short_description ASC", False, 0,
				'NATURAL JOIN perp_stock'
			);
			while($row = $this->db->row(True)) {
				$stock_list[] = $row;
			}

			foreach($stock_list as $key => $stock) {
				$category['stock'][$key] = $this->get_stock_price_list($stock['stock_id'], $category_id, $area_id, $type_id, $date, $edit, $price_list_id);
			}

			$list[] = $category;

			// Check for category-wide options?


			// Check for sub-categories
			$categories = array();
			$this->db->select('perp_sales_category', array(
					'sales_category_id',
					'cat_name AS sales_category_name'
				),
				array('cat_parent = ' . $category_id),
				__LINE__, __FILE__, False,
				'ORDER BY sales_category_name ASC', false, -1,
				'JOIN egw_categories ON
					perp_sales_category.sales_category_id = egw_categories.cat_id'
			);
			while($row = $this->db->row(True)) {
				$categories[] = $row;
			}

			if($categories) {
				foreach($categories as $key => $cat) {
					$c = $this->view_price_list($cat['sales_category_id'], $area_id, $type_id, $date, $edit);
					$c['category']	=	$c[0]['category'];
					if($c[0]['stock']) {
						$c['stock']	=	$c[0]['stock'];
					}
					unset($c[0]);
					$list[] = $c;
				}
			}
			
			return $list;
		}

		public function get_stock_price_list($stock_id, $category_id, $area_id, $type_id, $date, $edit, $price_list_id = false) {
			trigger_error('get_stock_price_list() is depricated, use bo_perp_price->get_price_list() instead');

			$edit &= $GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders');
			$this->db->select('perp_stock', array(
					'stock_id',
					'stock_num',
					'short_description'
				),
				array('stock_id = '.$stock_id),
				__LINE__, __FILE__
			);
			if($row = $this->db->row(True)) {
				$list = $row;
			} else {
				return array();
			}
			if(!$price_list_id) {
				$where = array(
					"(effective_to >= '" . $GLOBALS['egw']->db->to_timestamp($date) . "' OR effective_to IS NULL)",
				);
				if($category_id) {
					$where['sales_category_id'] = $category_id;
				}
				if($type_id) {
					$where['sales_type_id'] = $type_id;
				}
				if($area_id) {
					$where['sales_area_id'] = $area_id;
				}
				$this->db->select('perp_price_list', array(
						'price_list_id'
					),
					$where,
					__LINE__, __FILE__, False
				);
				if($row = $this->db->row(True)) {
					$price_list_id = $row['price_list_id'];
				} else {
					$list = array_merge($list, array('price' => lang('No price list')));
					return $list;
				}
			}

			// Get prices for this stock item
			if(is_array($price_list_id)) {
				$price_list_where = 'price_list_id IN (' . implode(',', $price_list_id) . ')';
			} else {
				$price_list_where = 'price_list_id = ' . $price_list_id;
			}

			$this->db->select('perp_price_list', array(
					'stock_id',
					'price_list_id',
					'price'
				),
				array(
					'stock_id = '.$stock_id,
					$price_list_where
				),
				__LINE__, __FILE__, False,
				"ORDER BY price DESC", False, 0,
				'NATURAL JOIN perp_price
				NATURAL JOIN perp_stock'
			);

			while($row = $this->db->row(True)) {
				$list = array_merge($list, $row);
				$list['edit'] = $edit;
			}
			if($edit) {
				$list['price_list_id'] = $price_list_id;
				$list['price'] = $list['price'] ? $list['price'] : '';
				$list['override_price'] = $list['price'];
				$list['edit'] = $edit;
			}
	
			return $list;
		}

		/**
		*	Get price list for options
		*	@todo Check to see if bo_perp_price->get_price_list won't cover this (or call this)
		*/
		public function get_option_price_list($stock_id, $bom_id, $price_list_id, $date, $edit = false) {
			if(!$price_list_id) {
				return array();
			} elseif (!$bom_id) {
				return array();
			}

			$list = array();

			// Get options on the current stock id
			$this->db->select('perp_allowed_build_option', array(
					'build_option_category_id',
					'build_option_category_name',
					'perp_build_option.build_option_id',
					'build_option_name',
					'component'),
				array(
					'stock_id'	=>	$stock_id,
					'value != 0'
				),
				__LINE__, __FILE__, False,
				"", False, 0,
				'JOIN perp_build_option ON
					perp_build_option.build_option_id = perp_allowed_build_option.build_option_id
				JOIN perp_build_option_category ON
					perp_build_option.build_option_category = perp_build_option_category.build_option_category_id'
			);
			$options = array();
			while($row = $this->db->row(True)) {
				$row['edit'] = $edit;
				$options[] = $row;
			}
			foreach($options as $row) {
				$sql = '
				SELECT DISTINCT
					prices.stock_id,
					prices.price_list_id,
					prices.build_option_category,
					prices.build_option_category_name,
					prices.build_option_name, 
					prices.build_option_id,
					price_list_name,
					currency,
					cat_name AS sales_category_name,
					sales_area_name,';
				if($edit) {
					$sql .= 'IF(default_price.price IS NULL, 0, default_price.price) AS default_price,
					override_price.price AS override_price, ';
				}
				$sql .= ' IF(override_price.price IS NULL, default_price.price, override_price.price) AS price
				FROM (
					SELECT * 
					FROM perp_allowed_build_option 
						NATURAL JOIN perp_build_option
						JOIN perp_price_list ON
							perp_price_list.price_list_id IN (' . (is_array($price_list_id) ? implode(',', $price_list_id) : $price_list_id) .')
						JOIN perp_build_option_category ON 
							perp_build_option.build_option_category = perp_build_option_category.build_option_category_id
					WHERE stock_id = ' . $stock_id . ' AND
						build_option_id = ' . $row[build_option_id] . '
				) AS prices
					LEFT JOIN perp_build_option_price ON
						prices.build_option_id = perp_build_option_price.build_option_id
					LEFT JOIN perp_build_option_price AS default_price ON 
						default_price.price_list_id = prices.price_list_id AND
						default_price.build_option_id = prices.build_option_id AND
						(default_price.stock_id = 0 OR 
						default_price.stock_id IS NULL)
					LEFT JOIN perp_build_option_price AS override_price ON
						override_price.build_option_id = prices.build_option_id AND
						override_price.price_list_id = prices.price_list_id AND
						override_price.stock_id = ' . $stock_id . '
					LEFT JOIN perp_sales_category ON prices.sales_category_id = perp_sales_category.sales_category_id 
					JOIN egw_categories ON perp_sales_category.sales_category_id = egw_categories.cat_id
					LEFT JOIN perp_sales_area ON perp_sales_area.sales_area_id = prices.sales_area_id 
					WHERE IF(override_price.price IS NULL, default_price.price, override_price.price) IS NOT NULL
					ORDER BY price ASC';

				$this->db->query($sql, __LINE__, __FILE__);
				while($r = $this->db->row(True)) {
					$row += $r;
				}

				$list[$row['build_option_category_id']]['build_option_category_id']	=	$row['build_option_category_id'];
				$list[$row['build_option_category_id']]['build_option_category_name']	=	$row['build_option_category_name'];
				$options = array(
					'build_option_id'	=>	$row['build_option_id'],
					'build_option_name'	=>	$row['build_option_name'],
					'stock_id'		=>	$stock_id,
					'price'			=>	$row['price'],
					'edit'			=>	$edit
				);
				if($row['default_price']) {
					$options['stock_id']		=	$row['stock_id'] ? $row['stock_id'] : $stock_id;
					$options['price_list_id']	=	$row['price_list_id'];
					$options['default_price'] = $row['default_price'];
					$options['override_price'] = $row['override_price'];
				}

				// Look for options on options
				$opt_bom_id = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $row['component']);
				if($opt_bom_id) {
					$o = $this->get_option_price_list($row['component'], $opt_bom_id, $row['price_list_id'], $date, $edit);
					$options['options'] = $o;
				}
				$list[$row['build_option_category_id']]['options'][] = $options;
				$list[$row['build_option_category_id']]['edit'] = $edit;
			}
			sort($list); // Re-key array

			// Look through BOM
			$bom = ExecMethod('perp_manufacturing.bo_perp_bill_of_material.get_bill_of_material', $bom_id);
			if(is_array($bom['details'])) {
				foreach($bom['details'] as $detail) {
					$bom_id = ExecMethod('perp_inventory.bo_perp_stock.get_current_bom', $detail['component']);
					$option = $this->get_option_price_list($detail['component'], $bom_id, $price_list_id, $date, $edit);
					if(is_array($option) && count($option)) {
						$list += $option;
					}
				}
			}

			return $list;
		}

		/**
		*	Get a list of client accounts that have access to a price list
		*/
		public static function get_client_accounts(Array &$query) {
			
			$where = array();
			$join = '';

			if($query['price_list_id']) {
				$where[] = 'perp_price_list.price_list_id IN (' . implode(',', (Array)$query['price_list_id']) . ')';
			}
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(client_name LIKE $search OR client_account_name LIKE $search OR
					client_num LIKE $search OR client_account_id LIKE $search)";
			}
			if($query['searchletter']) {
				$search = "'" . $query['searchletter'] . "%'";
				$where[] = "(client_name LIKE $search OR client_account_name LIKE $search)";
			}

			// Check to see how client accounts are restricted: explicitly, or by sales area
			$config = config::read('perp_ar');
			if($config['individual_client_discounts']) {
				// explicit
				$join .= 'JOIN perp_client_account_price_list ON
					perp_client_account_price_list.price_list_id = perp_price_list.price_list_id ';
			} else {
				// By sales area
				$join = 'JOIN perp_client ON
						perp_client.status = 1 AND (
							perp_price_list.sales_area_id = perp_client.sales_area OR
							perp_client.sales_area = 0
						) AND (
							perp_price_list.sales_type_id = perp_client.sales_type OR
							perp_client.sales_type = 0
						)
					JOIN perp_client_account ON
						perp_client.client_id = perp_client_account.client_id ';
			}

			// Total number of results
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_price_list',
				array('count(*)'),
				$where,
				__LINE__, __FILE__, false,
				"ORDER BY {$query['order']} {$query['sort']}", 
				'perp_orders', 1,
				$join
			)->fetchSingle();
			
			$result =& $GLOBALS['egw']->db->select(
				'perp_price_list',
				array(
					'price_list_id',
					'perp_client.*',
					'perp_client_account.*'
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				"ORDER BY {$query['order']} {$query['sort']}", 
				'perp_orders', $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			return $list;
		}
	}
?>
