<?php
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.so_perp_order.inc.php');

	require_once('class.SalesOrder.inc.php');

	class so_perp_sales_order extends so_perp_order {


		public function __construct() {
			parent::__construct();
		}

		public function get_order_list(&$query) {
			$where = array();

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(our_reference_number LIKE $search OR
					their_reference_number LIKE $search OR
					perp_client.client_num LIKE $search OR
					perp_client.client_name LIKE $search)";
			}
			if(isset($query['supplier_id']) && is_numeric($query['supplier_id'])) {
				$where[] = Order::order_table . '.supplier_id = ' . $query['supplier_id'];
			}
			if($query['client_id']) {
				if(is_array($query['client_id'])) {
					$where[] = Order::order_table . '.client_id IN (' . implode(',', $query['client_id']) . ')';
				} else {
					$where[] = Order::order_table . '.client_id = ' . (int)$query['client_id'];
				}
			}
			if($query['client_branch_id']) {
				$where[] = 'perp_sales_order.client_branch_id = ' . $query['client_branch_id'];
			}
			if($query['client_account_id']) {
				$where[] = 'perp_sales_order.client_account_id = ' . $query['client_account_id'];
			}
			if($query['status']) {
				switch($query['status']) {
					case 'request':
						$where[] = Order::order_table.'.approval_date IS NULL';
						break;
					case 'approved':
						$where[] = Order::order_table. '.approval_date IS NOT NULL';
						break;
					case 'ordered':
						$where[] = Order::order_table. '.issue_date <= NOW';
						break;
					case 'shipped':
						$where[] = Order::order_table . '.order_id IN (
							SELECT shippable_id
							FROM perp_shipment_line
							WHERE shippable_id IS NOT NULL
						)';
						break;
						
					case 'not_shipped':
						$where[] = Order::order_table . '.order_id NOT IN (
							SELECT shippable_id
							FROM perp_shipment_line
							WHERE shippable_id IS NOT NULL
						)';
						break;
						
					case 'invoiced':
						$where[] = Order::order_table . '.order_id IN (
							SELECT invoicable_id
							FROM perp_invoice_line
							WHERE invoicable_id IS NOT NULL
						)';
						break;

					case 'not_invoiced':
						$where[] = '(' . Order::order_table. '.approval_date IS NOT NULL OR !' . Order::order_table . '.requires_approval OR requires_approval IS NULL)';
						$where[] = Order::order_table . '.order_id NOT IN (
							SELECT invoicable_id
							FROM perp_invoice_line
							WHERE invoicable_id IS NOT NULL
						)';
						break;
					
					case 'open':
						$having[] = '(total_shipped IS NULL OR total_shipped != total_ordered) OR
							(total_invoiced IS NULL OR total_invoiced != total_ordered)';
						break;

					case 'closed':
						$having[] = '(total_shipped = total_ordered AND
							total_invoiced = total_ordered)';
						break;
				}
			}

			// Filter on dates
			if(isset($query['issue_date'])) {
				if(is_array($query['issue_date'])) {
					if($query['issue_date']['start']) {
						$where[] = 'perp_order.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['start']));
					}
					if($query['issue_date']['end']) {
						$where[] = 'perp_order.issue_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']['end']));
					}
				} else {
					$where[] = 'perp_order.issue_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['issue_date']));
				}
			}
			if(isset($query['originating_date'])) {
				if(is_array($query['originating_date'])) {
					if($query['originating_date']['start']) {
						$where[] = 'perp_order.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['start']));
					}
					if($query['originating_date']['end']) {
						$where[] = 'perp_order.originating_date >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']['end']));
					}
				} else {
					$where[] = 'perp_order.originating_date = ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['originating_date']));
				}
			}

			// Query on order contents
			if(isset($query['stock_id'])) {
				$where[] = 'perp_order_line.stock_id = ' . (int)$query['stock_id'];
			}

			if($query['sales_category']) {
				$where[] = 'perp_stock_sales_category.sales_category_id = ' . $query['sales_category'];
			}

			// Large and complicated join
			$join = 'LEFT JOIN ' . OrderLine::order_line_table . ' ON ' .
					Order::order_table . '.order_id = ' . OrderLine::order_line_table . '.order_id 
				LEFT JOIN ' . SalesOrder::sales_order_table . ' ON ' . 
					SalesOrder::sales_order_table . '.order_id = ' . Order::order_table . '.order_id 
				LEFT JOIN ' . SalesOrderLine::sales_order_line_table . ' ON ' .
					OrderLine::order_line_table . '.order_id = ' . SalesOrderLine::sales_order_line_table . '.order_id AND ' .
					OrderLine::order_line_table . '.line_no = ' . SalesOrderLine::sales_order_line_table . '.line_no
				LEFT JOIN perp_client_branch ON ' . 
					SalesOrder::sales_order_table . '.client_branch_id = perp_client_branch.client_branch_id
				LEFT JOIN perp_client_account ON '.
					SalesOrder::sales_order_table . '.client_account_id = perp_client_account.client_account_id
				LEFT JOIN perp_client ON
					perp_client_branch.client_id = perp_client.client_id OR
					perp_client_account.client_id = perp_client.client_id' . 
				($sales_category ? 
					' LEFT JOIN perp_stock ON ' .
						SalesOrder::$detail_table . '.stock_id = perp_stock.stock_id
					LEFT JOIN perp_stock_sales_category ON
						perp_stock.stock_id = perp_stock_sales_category.stock_id' : '') . '
				LEFT JOIN (
					SELECT
						sum(quantity_shipped) AS quantity_shipped,
						perp_shipment_line.shippable_id,
						perp_shipment_line.shippable_item_line
					FROM perp_shipment_line
					GROUP by shippable_id, shippable_item_line
				) AS perp_shipment_line ON
					perp_shipment_line.shippable_id = ' . Order::order_table . '.order_id AND
					perp_shipment_line.shippable_item_line = ' . OrderLine::order_line_table . '.line_no
				LEFT JOIN (
					SELECT
						sum(quantity) AS quantity_invoiced,
						perp_invoice_line.invoicable_id,
						perp_invoice_line.invoicable_item_line
					FROM perp_invoice_line
					GROUP BY invoicable_id, invoicable_item_line
				) AS perp_invoice_line ON
					perp_invoice_line.invoicable_id = ' . Order::order_table . '.order_id AND
					perp_invoice_line.invoicable_item_line = ' . OrderLine::order_line_table . '.line_no';


			if(count($having) > 0) {
				$having = ' HAVING ' . implode(' AND ', $having);
			}

			// Get number of rows
			$result =& $GLOBALS['egw']->db->select(Order::order_table,
				//array('count(' . Order::order_table . '.order_id) AS count'),
				array(
					'count(' . Order::order_table . '.order_id) AS count',

					// for checking for done
					'SUM(quantity_ordered) AS total_ordered',
					'SUM(quantity_shipped) AS total_shipped',
					'SUM(quantity_invoiced) AS total_invoiced',
				),
				$where,
				__LINE__, __FILE__, false, 'GROUP BY ' . Order::order_table . '.order_id ' . $having, 'perp_api', -1,
				$join
			);
			$query['count'] = $result->NumRows();

			$GLOBALS['egw']->db->select(Order::order_table, array(
					'our_reference_number',
					'client_name',
					Order::order_table . '.client_id',
					'client_account_name',
					'perp_client_account.client_account_id',
					'their_reference_number',
					'perp_client_branch.client_branch_id',
					'branch_name',
					Order::order_table.'.order_id',
					'issue_date',
					'requested_delivery_date',
					'actual_delivery_date',
					'originating_user',
					'sum(quantity_ordered * (perp_order_line.unit_price - (perp_order_line.unit_price * IF(' . 
						SalesOrderLine::sales_order_line_table . '.discount_percent IS NULL, 0, ' . SalesOrderLine::sales_order_line_table . '.discount_percent
					) / 100))) AS subtotal',

					// for checking for done
					'SUM(quantity_ordered) AS total_ordered',
					'SUM(quantity_shipped) AS total_shipped',
					'SUM(quantity_invoiced) AS total_invoiced',
				), 
				$where,
				__LINE__, __FILE__, $query['start'],
				"GROUP BY " . Order::order_table . ".order_id
				$having
				ORDER BY " . $query['order'] . ' ' .  $query['sort'], 'perp_api', $query['num_rows'],
				$join
			);
			$list = array();
			while($row = $GLOBALS['egw']->db->row(True)) {
				$row['subtotal'] = sprintf("%.2f", $row['subtotal']);
				$row['issue_date'] = $GLOBALS['egw']->db->from_timestamp($row['issue_date']);
				$row['requested_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['requested_delivery_date']);
				$row['actual_delivery_date'] = $GLOBALS['egw']->db->from_timestamp($row['actual_delivery_date']);
				$list[] = $row;
			}
			foreach($list as &$row) {
				$row['formatted_subtotal'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $row['order_currency'], $row['subtotal']);
			}
			return $list;
		}

		public function get_sales_order($sales_order_id) {

		}

		public function set_sales_order($sales_order) {

		}

		public function delete_sales_order($sales_order) {

		}

		/**
		*	Return a certain amount of a work order, so it can be resold
		*/
		public static function return_work_order($client_id, $work_order_id, $detail_id, $quantity) {
			$sql = 'UPDATE perp_sales_order_work_order
				JOIN perp_order ON
				    perp_sales_order_work_order.order_id = perp_order.order_id
				JOIN perp_order_detail ON
				    perp_sales_order_work_order.order_id = perp_order_detail.order_id
				    AND perp_sales_order_work_order.line_no = perp_order_detail.line_no
				    AND perp_sales_order_work_order.detail_no = perp_order_detail.detail_no

				SET quantity_returned = quantity_returned + ' . (float)$quantity . '
				WHERE
					perp_order_detail.detail_value = ' . (int)$work_order_id . '
					AND perp_sales_order_work_order.work_order_detail_id = ' . (int)$detail_id . '
					AND perp_order.client_id = ' . (int)$client_id;
			$GLOBALS['egw']->db->query($sql, __LINE__, __FILE__); 
		}
	}
?>
