<?php

	/**
	*	User interface for dealing with calculations for automatically updating prices.
	*
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_orders/inc/class.bo_perp_price_calculation.inc.php');

	require_once( EGW_INCLUDE_ROOT . '/etemplate/inc/class.etemplate.inc.php');

	class ui_perp_price_calculation {
		
		public $public_functions = array(
			'calculation_list'	=>	true,
			'edit'		=>	true,
			'run'		=>	true,
			'schedule'	=>	true,
		);

		/**
		*	Logic object to get all the info from
		*/
		protected $bo;

		/**
		*	eTemplate object for display
		*/
		protected $template;

		/**
		*	A list of (message, severity) to be displayed to the user
		*/
		protected $messages = array();


		public function __construct() {
			$this->bo = new bo_perp_price_calculation();
			$this->template = new etemplate();
		}

		public function calculation_list(Array $content = array()) {
			if(isset($content['new']) && $GLOBALS['egw']->acl->check('calculation', EGW_ACL_ADD, 'perp_orders')) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_orders.ui_perp_price_calculation.edit'
				));
			} elseif (isset($content['calculation'])) {
				$action = key($content['calculation']);
				$id = key($content['calculation'][$action]);

				switch ($action) {
					case 'delete':
						$this->bo->delete($id);
						break;
					case 'run':
						$GLOBALS['egw']->redirect_link('/index.php', array(
							'menuaction'	=>	'perp_orders.ui_perp_price_calculation.run',
							'id'		=>	$id
						));
						break;
					case 'schedule':
						$GLOBALS['egw']->redirect_link('/index.php', array(
							'menuaction'	=>	'perp_orders.ui_perp_price_calculation.schedule',
							'id'		=>	$id
						));
						break;
					default:
						if($content['calculation'][$action][$id]['cancel']) {
							$id = key($content['calculation'][$action][$id]['cancel']);
							ExecMethod('phpgwapi.asyncservice.cancel_timer', urldecode($id));
						}
				}
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			}
			
			$this->bo->search($query, $data['calculation'], $readonly);

			// Move to allow for heading
			if(is_array($data['calculation'][0])) {
				array_unshift($data['calculation'], false);
				unset($data['calculation'][0]);
			}

			$select_box_options['price_list_id'] = ExecMethod('perp_orders.bo_perp_price_list.get_price_list_list');

			$readonly['new'] = !$GLOBALS['egw']->acl->check('calculation', EGW_ACL_ADD, 'perp_orders');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Price Calculations');
			$this->template->read('perp_orders.price_calculation_list');
			$this->template->exec('perp_orders.ui_perp_price_calculation.calculation_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function edit(Array $content = array()) {

			// Deal with incoming
			if(isset($content['save']) && $GLOBALS['egw']->acl->check('calculation', EGW_ACL_EDIT, 'perp_orders')) {
				try {
					$result = $this->bo->save($content);
					if(is_numeric($result)) {
						$content['id'] = $result;
						$this->messages[] = array(lang('Calculation saved'), 'success');
					} else {
						$this->messages[] = array($result, 'error');
					}
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			} 
			if (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_orders.ui_perp_price_calculation.calculation_list'
				));
			}

			$id = isset($content['id']) ? $content['id'] : reg_var('id');
			if($content) {
				$data = $content;
			} elseif($id) {
				try {
					$data = $this->bo->read($id);
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			}

			// Add allowed variables
			$data['variables'] = array();
			foreach(bo_perp_price_calculation::$allowed_variables as $name => $description) {
				$data['variables'][] = array(
					'name'		=>	$name,
					'description'	=>	$description
				);
			}

			// Add allowed functions
			$data['functions'] = array();
			foreach(bo_perp_price_calculation::$allowed_functions as $name => $description) {
				$data['functions'][] = array(
					'name'		=>	$name,
					'description'	=>	$description
				);
			}

			$preserve = $data;

			$readonly['__ALL__'] = !$GLOBALS['egw']->acl->check('calculation', EGW_ACL_EDIT, 'perp_orders') ||
				(!$GLOBALS['egw']->acl->check('calculation', EGW_ACL_ADD, 'perp_orders') && !$id);

			$GLOBALS['egw']->js->set_onload('price_calculation(); ');
			$GLOBALS['egw']->js->validate_file('price_list', 'calculation', 'perp_orders');
			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Price Calculation %1', $data['name']);
			$this->template->read('perp_orders.price_calculation_edit');
			$this->template->exec('perp_orders.ui_perp_price_calculation.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	User interface for running a certain calculation on certain price lists
		*
		*/
		public function run(Array $content = array()) {
			$id = isset($content['id']) ? $content['id'] : reg_var('id');
			if(!$id) {
				$this->messages[] = array(lang('Missing calculation ID'), 'error');
				return $this->calculation_list($content);
			}
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				$this->messages[] = array(lang('You do not have permission to change prices'), 'error');
				return $this->calculation_list($content);
			}
			if($content && !$content['price_list_id']) {
				$this->messages[] = array(lang('Missing price lists'), 'warning');
			}

			if(isset($content['run']) && $content['price_list_id']) {
				$content['price_list_id'] = explode(',', $content['price_list_id']);
				$result = $this->bo->run($content);
				if($result === true) {
					$formatted_date = date(
						$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
						($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
						$content['effective_date']
					);
					$this->messages[] = array(lang('Prices have been changed, effective %1', $formatted_date), 'success');
				} elseif($result) {
					$this->messages[] = array($result, 'error');
				} else {
					$this->messages[] = array('Something went wrong, but I do not know what.  Prices are unchanged.', 'error');
				}
			}
			if (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_orders.ui_perp_price_calculation.calculation_list'
				));
			}

			// Load the calculation
			$data = $this->bo->read($id);
			$data['cat_id'] = $content['cat_id'];

			$tomorrow = strtotime('tomorrow');
			$default_date = mktime($GLOBALS['egw_info']['user']['preferences']['calendar']['workdaystarts'], 0, 0, date('m', $tomorrow), date('d', $tomorrow), date('Y', $tomorrow));
			$data['effective_date'] = $content['effective_date'] ? $content['effective_date'] : $default_date;


			$select_box_options['price_list_id'] = ExecMethod('perp_orders.bo_perp_price_list.get_price_list_list', 
				array('status' => 'current', 'cat_id' => $data['cat_id'])
			);

			$preserve['id'] = $id;
		
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Adjust prices using %1', $data['name']);
			$this->template->read('perp_orders.price_calculation_run');
			$this->template->exec('perp_orders.ui_perp_price_calculation.run', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Schedule a price adjustment calculation for a future (recurring) time
		*/
		public function schedule(Array $content = array()) {
			$id = isset($content['id']) ? $content['id'] : reg_var('id');
			$async_id = isset($content['async_id']) ? $content['async_id'] : reg_var('async_id');
			if(!$id && !$async_id) {
				$this->messages[] = array(lang('Missing calculation ID'), 'error');
				return $this->calculation_list($content);
			}
			if(!$GLOBALS['egw']->acl->check('price_list', EGW_ACL_EDIT, 'perp_orders')) {
				$this->messages[] = array(lang('You do not have permission to change prices'), 'error');
				return $this->calculation_list($content);
			}
			if($content && !$content['price_list_id']) {
				$this->messages[] = array(lang('Missing price lists'), 'warning');
			}
			if($content && !$content['schedule']) {
				$this->messages[] = array(lang('Missing schedule times'), 'warning');
			}

			// Schedule the adjustment
			if($content['save']) {
				$content['price_list_id'] = explode(',', $content['price_list_id']);
				foreach($content['schedule'] as $key => &$value) {
					if($value === '') {
						$value = '*';
					}
				}
				$content['schedule']['year'] = '*';

				$async_id = $this->bo->schedule($content);

				if($async === true) {
					// Something good happened, but it wasn't scheduling a report...
				} else {
					// Read it back and confirm
					$async = ExecMethod('phpgwapi.asyncservice.read', $async_id);
					$async = $async[$async_id];
					if($async) {
						$next = date(
							$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
							($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
							$async['next']
						);
						$effective = date(
							$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] .
							($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == '24' ? ' H:i' : ' g:i a'),
							ExecMethod2('perp_api.perp_report.convert_date', $async['data']['effective_date'], $async['next'])
						);
						$this->messages[] = array(lang('Prices will be changed on %1, effective %2', $next, $effective), 'success');
					} else {
						$this->messages[] = array('Something went wrong, but I do not know what.  Prices will not be changed by this calculation.', 'error');
					}
				}
			}
			if (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_orders.ui_perp_price_calculation.calculation_list'
				));
			}

			$data = $content;
			if($async_id) {
				// Load an existing schedule
				$async = ExecMethod('phpgwapi.asyncservice.read', $async_id);
				$data = array_merge($async[$async_id]['data'], $data);
				$data['times'] = $async[$async_id]['times'];
				$id = $data['id'];
			}
			if($id) {
				// Load the calculation
				$data = array_merge($this->bo->read($id), $data);
			}

			if(is_array($data['times'])) {
				foreach($data['times'] as $interval => &$value) {
					if(is_array($value)) {
						$value = implode(',', array_keys($value));
					}
					$data['schedule'][$interval] = $value;
				}
			}

			$data['cat_id'] = $data['cat_id'] ? $data['cat_id'] : $content['cat_id'];

			$select_box_options['price_list_id'] = ExecMethod('perp_orders.bo_perp_price_list.get_price_list_list', 
				array('status' => 'current', 'cat_id' => $data['cat_id'])
			);
			$select_box_options['effective_date'] = array(
				'tomorrow'		=>	'tomorrow',
				'next Monday'		=>	'Next Monday',
				'next Friday'		=>	'Next Friday',
				'this_month_start'      =>	'Start of this month',
				'this_month_end'	=>	'End of this month',
				'next_month_start'	=>	'Start of next month',
				'next_month_end'	=>	'End of next month',
				'this_year_end'		=>	'End of this year',

			);
			foreach($select_box_options['effective_date'] as $key => &$option) {
				$option .= ' (' . date(
					$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i',
					ExecMethod2('perp_api.perp_report.convert_date', $key, $data['next'])
				) . ')';

			}

			$preserve['id'] = $id;
		
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Recurring price adjustment using %1', $data['name']);
			$this->template->read('perp_orders.price_calculation_schedule');
			$this->template->exec('perp_orders.ui_perp_price_calculation.schedule', $data, $select_box_options, $readonly, $preserve);
		}
	}
?>
