<?php
	/**
	*	User interface for Sales Orders.
	*
	*	@author Nathan Gray
	*	@ingroup perp_orders
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_order.inc.php');

	require_once('class.bo_perp_sales_order.inc.php');
	require_once('class.SalesOrder.inc.php');

	// Used for taxes
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.ReceivableInvoice.inc.php');

	// For credit limit check
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.bo_perp_client.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_ar/inc/class.bo_perp_client_account.inc.php');


	class ui_perp_sales_order extends ui_perp_order {

		public $public_functions = array(
			'order_list'	=>	True,
			'new_order'	=>	True,
			'edit'	=>	True,
			'edit_line_detail'	=>	True,
		);

		/**
		*       eTemplate for displaying order lines
		*/
		protected $order_line_template = 'perp_orders.sales_order_edit.row';
		protected $order_line_detail_edit = 'perp_orders.sales_order_detail_edit';
		protected $order_list_template = 'perp_orders.sales_order_list';
		protected $order_list_row_template = 'perp_orders.sales_order_list.row';

		public function ui_perp_sales_order() {
			parent::__construct();
			$this->bo_order = new bo_perp_sales_order();
		}

		/**
		*	Get a list of Sales Orders.
		*	@param content Array of data returned from the eTemplate when the user submits a form
		*/
		public function order_list($content = array()) {
			$this->order_list_update($content);
			
			$preserve['return'] = $_GET['return'] ? $_GET['return'] : ($content['return'] ? $content['return'] : $_SERVER['HTTP_REFERER']);

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_orders.ui_perp_orders.index'));
			}

			$data = $this->order_list_get_display_data();
			$select_box_options = $this->order_list_get_options();
			$readonly = $this->order_list_get_readonlys();

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Sales Order List');
			$this->template->read('perp_orders.sales_order_list');
			$this->template->exec('perp_orders.ui_perp_sales_order.order_list', $data, $select_box_options, $readonly, $preserve);
		}

		public function new_order($content = array()) {
			if($this->debug) {
				echo 'perp_orders.ui_perp_sales_order.new_sales_order incoming data:';
				_debug_array($content);
			}       
			if($content['order_id']) {
				return $this->edit($content);
			}
			
			$sales_order = CreateObject('perp_orders.SalesOrder');	
			$client_id = (int)reg_var('client_id');
			if($client_id) {
				$sales_order->set_client_id($client_id);

				$bo_client_branch = new bo_perp_client_branch();
				$bo_client_account = new bo_perp_client_account();

				// Check for only 1 account & branch
				//Query the client branch list with client id filter
				$query = array(
					'client_id'     =>      $client_id,
					'template'      =>      false,
				);
				$branch_count = $bo_client_branch->get_client_branch_list($query, $branch_list);
				if($branch_count == 1) {
					$sales_order->set_client_branch_id($branch_list[0]['client_branch_id']);
				}
				$account_count = $bo_client_account->get_client_account_list($query, $account_list);
				if($account_count == 1) {
					$sales_order->set_client_account_id($account_list[0]['client_account_id']);
					$account = ExecMethod('perp_ar.bo_perp_client_account.get_client_account', $sales_order->get_client_account_id());
					if($account['payment_terms']) {
						$sales_order->set_payment_terms((int)$account['payment_terms']);
					}
				}

			}
			$client_branch_id = (int)reg_var('client_branch_id');
			if($client_branch_id) {
				$sales_order->set_client_branch_id($client_branch_id);
			}

	
			$GLOBALS['egw']->session->appsession('order', false, $sales_order);

			$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_orders.ui_perp_sales_order.edit'));
		}

		public function edit($content = array()) {
			if($this->debug) {
				echo '<strong>perp_orders.ui_perp_sales_order.sales_order_edit</strong> incoming data:';
				_debug_array($content);
			}

			$order_id = reg_var('order_id');

			// Get sales order
			$sales_order =& $this->get_order('SalesOrder', $order_id);
			if(!$sales_order) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_orders.ui_perp_sales_order.new_order'));
			}

			// Update
			try {
				$this->update_order($sales_order, $content);
			} catch (Exception $e) {
				$this->messages[] = array($e->getMessage(), 'error');
			}

			if(isset($content['cancel']) || !$GLOBALS['egw']->acl->check('order', EGW_ACL_READ, 'perp_orders')) {
				$GLOBALS['egw']->session->appsession('order', false, false);
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_orders.ui_perp_sales_order.order_list'));
			}


			// Save to database
			if($content['save'] && $GLOBALS['egw']->acl->check('order', EGW_ACL_EDIT, 'perp_orders')) {
				try {
					$result = $sales_order->write();
				} catch (Exception $exception) {
					$this->messages[] = array($exception->getMessage(), 'error');
				}

				if(is_numeric($result)) {
					$order_id = $result;
					unset($result);
					$this->messages[] = array(lang('Sales order saved'), 'success');
				} else {
					$this->check_result($result, 'error');
				}
			}       

			// Save to session
			$GLOBALS['egw']->session->appsession('order', false, $sales_order);

			// Reload to avoid reposting from popup dialogs
			if($GLOBALS['egw']->session->appsession('order_reload')) {
				$sales_order->write();
				$js = $GLOBALS['egw']->session->appsession('order_javascript');
				if($GLOBALS['egw']->js->body->onLoad) {
					$js[] = $GLOBALS['egw']->js->body->onLoad;
				}
				$GLOBALS['egw']->session->appsession('order_javascript', false, $js);
				$GLOBALS['egw']->session->appsession('order_reload', false, false);
				$GLOBALS['egw']->redirect_link('/index.php',
					array(
						'menuaction'	=>	'perp_orders.ui_perp_sales_order.edit',
						'order_id'	=>	$sales_order->get_order_id()
					)
				);
			}

			// Check order
			$check = $this->check_result($sales_order->check(), 'warning');
			if($check === true && $admin) {
				$approval_check = $this->check_result($sales_order->approval_check(), 'warning');
			}

			// Get data for display
			$data = $this->get_display_data($sales_order);

			// READ TEMPLATE HERE
			$this->template->read('perp_orders.sales_order_edit');

			// Select box options
			$select_box_options = $this->get_selectbox_options($sales_order);

			// Readonlys
			$readonly = $this->get_readonlys($sales_order);
			$readonly['nextmatch']['rows'] = $data['nextmatch']['readonlys'];

			// JavaScript
			$this->get_javascript('perp_ap.ajax_perp_sales_order', $sales_order);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = $order_id ? lang('Edit Sales Order %1', $sales_order->get_our_reference_number()) : lang('New Sales Order');
			// Template read above
			$this->template->exec('perp_orders.ui_perp_sales_order.edit', $data, $select_box_options, $readonly, $preserve);

			// Clear javascript variables
			$GLOBALS['egw']->session->appsession('order_javascript', false, false);
			$GLOBALS['egw']->session->appsession('order_reload', false, false);
		}

		/**
		*	Edit the details of a SalesOrderLine.
		*	This calls up options for the stock id, presents them, and tries to find a matching 
		*	Work Order to what the user wants.  It creates a temporary work order with a negative 
		*	work order id to work with.  If a suitable work order already in stock or scheduled
		*	is found, it will use that one.  Otherwise, a new work order will be created with
		*	a status of pending.
		*
		*	@todo Clean this mess up.
		*/
		public function edit_line_detail($content = array()) {
			if($this->debug) {
				echo 'perp_orders.ui_perp_sales_order.edit_line_detail incoming data:';
				_debug_array($content);
			}
			
			// Get sales order
			$order = $this->get_order('SalesOrder');
			if(!$order) {
				return false;
			}

			$line_no = isset($content['line_no']) ? $content['line_no'] : $_GET['line_no'];
			$line =& $order->get_line($line_no);
			if(!$line) {
				return false;
			}

			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $line->get_stock_id());

			$total = 0;
			$required_date = isset($content['required_date']) ? $content['required_date'] : $order->get_requested_delivery_date();

			// Quantity
			if(isset($content['quantity'])) {
				$line->set_quantity_ordered($content['quantity']);
			}

			// Save changes to temp work order
			if($line && $line->get_stock_id() && isset($content['options'])) {
				foreach($content['options'] as $key => $row) {
					$total += $price_list[$row['build_option_category_id']]['options'][$row['value']]['price'];
					if(is_numeric($row['build_option_category_id']) && $row['build_option_category_id'] > 0) {
						ExecMethod2('perp_manufacturing.bo_perp_work_order.set_work_order_detail_option',
							$content['work_order_id'], $content['detail_id'], $row['stock_id'], $row['build_option_category_id'], $row['value']
						);
					} else {
						switch ($row['build_option_category_id']) {
							case bo_perp_sales_order::DAMAGE_LINE:
								// They want damaged goods, value is the damage ID
								// Put it in the session
								$GLOBALS['egw']->session->appsession("SalesOrder-" . $order->get_order_id() . '-' . $line_no . '-DAMAGE', 'perp_orders', $row['value']);
								break;
						}
					}
				}
			}

			$work_order_id = null;
			$bo_wo = CreateObject('perp_manufacturing.bo_perp_work_order');
			$unclaimed = $bo_wo->get_unclaimed_work_order($line->get_stock_id(), null, $required_date, $content['work_order_id']);
			if(isset($content['save'])) {
				if($content['unclaimed']['claim']) {
					foreach($content['unclaimed']['claim'] as $work_order_id => $detail) {
						foreach($detail as $detail_id => $claim) {
							if($claim) {
								$content['work_order_id'] = $work_order_id;
								$content['detail_id'] = $detail_id;
								break;
							}
						}
					}
				}
					
				if($stock['stock_type_flag'] == 'A') {
					$result = $this->bo_order->claim_work_orders(
						$line, 
						$required_date,
						$content['work_order_id'],
						$content['detail_id'],
						$content['options']
					);


					$line->set_ship_location($work_order['location']);
					$bo_wo->delete_work_order(-$order->get_order_id());

					if($result !== true) {
						if(is_array($result)) {
							$this->messages += $result;
						} else {
							$this->messages[] = array($result, 'info');
						}
					}

					$work_orders = $line->get_details('WorkOrderDetail');
					$work_order_id = $work_orders[0]->get_work_order_id();
					$detail_id = $work_orders[0]->get_work_order_detail_id();

					// Get pricing for the work order we claimed (might be different from the temp)
					$query = array(
						'work_order_id'	=>	$work_order_id,
						'detail_id'	=>	$detail_id,
						'stock_id'	=>	$line->get_stock_id(),
						'required_date'	=>	$order->get_issue_date(),
						'template'	=>	false
					);
					if($line->get_price_list_id()) {
						$query['price_list_id'] = $line->get_price_list_id();
					}
					$this->bo_order->get_detail_details($query, $pricing);
				} elseif ($stock['stock_type_flag'] == 'K') {
					$associated_lines = $line->get_detail('associated_line');

					// Get pricing for components
					$query = array(
						'work_order_id'	=>	$content['work_order_id'],
						'detail_id'	=>	$content['detail_id'],
						'stock_id'	=>	$line->get_stock_id(),
						'required_date'	=>	$order->get_issue_date(),
						'template'	=>	false
					);
					if($line->get_price_list_id()) {
						$query['price_list_id'] = $line->get_price_list_id();
					}
					$this->bo_order->get_detail_details($query, $pricing);

					// Go through the options and add the option component to the work order, then remove the kit
					foreach($pricing as $option) {
						if(!$option['options-value']) {
							continue;
						}
						$line_data = array(
							'stock_id'		=>	$option['options-value'][$option['value']]['component'],
							'quantity_ordered'	=>	$line->get_quantity_ordered(),
							'unit_price'		=>	$option['options-value'][$option['value']]['price'],
						);
						$new_line_no = $this->add_line($order, $line_data);
						$new_line =& $order->get_line($new_line_no);

						foreach($associated_lines as $detail) {
							$new_line->add_associated_line($detail->get_value());
						}
					}
					
					// Remove temporary work order
					$bo_wo->delete_work_order(-$order->get_order_id());

					// Remove the associated lines first, or they'll all go...
					if(is_array($associated_lines)) {
						foreach($associated_lines as $detail) {
							$line->remove_associated_line($detail->get_value());
						}
					}
					$order->remove_line($line->get_line_number());
				}
		
				if($GLOBALS['egw']->session->appsession("SalesOrder-" . $order->get_order_id() . '-' . $line_no . '-DAMAGE')) {
					$query[bo_perp_sales_order::DAMAGE_LINE] = $GLOBALS['egw']->session->appsession("SalesOrder-" . $order->get_order_id() . '-' . $line_no . '-DAMAGE');
					$GLOBALS['egw']->session->appsession("SalesOrder-" . $order->get_order_id() . '-' . $line_no . '-DAMAGE', 'perp_orders', null);
				}

				// Total is the last line
				$total = end($pricing);
				$line->set_unit_price($total['price']);

				// Look for any other sub-components
				foreach($pricing as $pricing_key => $category) {
					$component = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $category['options-value'][$category['value']]['component']);
					if($component['stock_type_flag'] == 'K') {
						// Found a sub-component, add it to sales order
						$component_price_query = array(
							'price_list_id'	=>      $line->get_price_list_id(),
							'effective_date'=>      $order->get_issue_date(),
							'kit_stock_id'	=>      $component['stock_id'],
							'stock_id'	=>      $line->get_stock_id(),
						);
						$component_prices = ExecMethod('perp_orders.so_perp_kit_price.get_kit_price_list', $component_price_query);
						foreach($component_prices as $component) {
							$line_data = array(
								'stock_id'		=>	$component['component_id'],
								'quantity_ordered'	=>	$line->get_quantity_ordered() * $component['quantity'],
								'unit_price'		=>	$component['price'] / $component['quantity'],
							);

							$new_line_no = $this->add_line($order, $line_data);
							$line->add_associated_line($new_line_no);
						}
					}
				}

				$GLOBALS['egw']->session->appsession('order', 'perp_orders', null);
				$order->write();

				// We're done
				$GLOBALS['egw']->js->set_onload('window.opener.location.reload(); ');
				if(count($this->messages) == 0) {
					if(!is_object($GLOBALS['egw']->js)) {
						$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
					}
					if(!$this->debug) {
						$GLOBALS['egw']->js->set_onload('self.close(); ');
					}
				}
			} elseif (isset($content['cancel'])) {
				$bo_wo->delete_work_order(-$order->get_order_id());
				$order->remove_line($line->get_line_number());
				$GLOBALS['egw']->session->appsession('order', 'perp_orders', null);
				$GLOBALS['egw']->js->set_onload('window.opener.location.reload(); self.close(); ');
				try {
					$order->write();
				} catch (Exception $e) {
					// Don't care about empty orders, we'll save it to the session instead
					$GLOBALS['egw']->session->appsession('order', 'perp_orders', $order);
				}
			}

			// Check for a work_order
			$work_orders = $line->get_details('WorkOrderDetail');
			if(count($work_orders)) {
				$work_order_id = $work_orders[0]->get_work_order_id();
				$detail_id = $work_orders[0]->get_work_order_detail_id();
				if(count($work_orders) > 1) {
					$this->messages[] = array(lang('Multiple work orders / line are not yet supported'), 'warning');
				}
			} elseif($required_date && !$content['cancel']) {
				// No associated work orders.  
				// First, create a temp order so options can be selected
				$wo['work_order_reference'] = $order->get_our_reference_number();
				$wo['created_date'] = time();
				$wo['required_date'] = $required_date;
				$wo['sales_order'] = $order->get_order_id();
				$wo['location'] = 0;
				$wo['details'] = array(
					array(
						'stock_id'	=>	$line->get_stock_id(),
						'quantity'	=>	$line->get_quantity_ordered()
					)
				);

				$detail_id = 1;
				try {
					$work_order_id = $bo_wo->set_work_order(-$order->get_order_id(), $wo);
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}

				// Default options to any instead of default so we can get rid of stuff already in stock
				$query = array(
					'work_order_id'	=>	$work_order_id,
					'detail_id'	=>	$detail_id,
					'stock_id'	=>	$line->get_stock_id(),
					'client_account_id'	=>	$order->get_client_account_id(),
					'effective_date'	=>	$order->get_issue_date(),
				);
				$detail = $bo_wo->get_work_order_detail_option_list($query, $options);
				if(!isset($content['options'])) {
					foreach($options as $option) {
						$bo_wo->set_work_order_detail_option($work_order_id, $detail_id, $line->get_stock_id(), $option['build_option_category_id'], null);
					}
				}
			}
			
			if($work_order_id) {
				$work_order = $bo_wo->get_work_order($work_order_id);
			}

			// Get data for display
			$data['stock_id'] = $line->get_stock_id();
			$data['quantity'] = $line->get_quantity_ordered();
			$data['required_date'] = (!$required_date ? $work_order['required_date'] : $required_date);

			if($data['stock_id'] && $data['required_date']) {
				//$data['options'] = array(
				$query = array(
					'work_order_id'	=>	$work_order_id,
					'detail_id'	=>	$detail_id,
					'stock_id'	=>	$line->get_stock_id(),
					'client_account_id'	=>	$order->get_client_account_id(),
					'date'		=>	$order->get_issue_date(),
					'effective_date'	=>	$order->get_issue_date(),
					'no_readonlys'	=>	true,
					'no_cat'        =>      True,
					'no_filter'     =>      True,
					'no_filter2'    =>      True,
					'get_rows'      =>      'perp_orders.bo_perp_sales_order.get_detail_details',
					'template'      =>      'perp_orders.sales_order_detail_edit.row',
					'include_discontinued_options' => true
				);
				if($GLOBALS['egw']->session->appsession("SalesOrder-" . $order->get_order_id() . '-' . $line_no . '-DAMAGE')) {
					$query[bo_perp_sales_order::DAMAGE_LINE] = $GLOBALS['egw']->session->appsession("SalesOrder-" . $order->get_order_id() . '-' . $line_no . '-DAMAGE');
				}
				$this->bo_order->get_detail_details($query, $data['options'], $readonly['options']);
				array_unshift($data['options'], false);
			}

			if($unclaimed['total'] > 0) {
				
				$data['unclaimed'] = array('total' => $unclaimed['total']);
				foreach($unclaimed as $unclaimed_work_order) {
					if(!is_array($unclaimed_work_order) || !$unclaimed_work_order['work_order_id']) {
						continue;
					}
					// Limit it to top 5 entries 
					if(count($data['unclaimed']) > 5) {
						break;
					}
					$unclaimed_work_order = array_merge($unclaimed_work_order, $bo_wo->get_work_order($unclaimed_work_order['work_order_id']));
					$unclaimed_work_order['options'] = $bo_wo->get_work_order_detail_option($unclaimed_work_order['work_order_id'], $unclaimed_work_order['detail_id']);
					$data['unclaimed'][] = $unclaimed_work_order;
				}
			}

			// Select box options
			$select_box_options = $this->get_line_detail_options($line, $data);

			$preserve['order_id'] = $order->get_order_id();
			$preserve['stock_id'] = $line->get_stock_id();
			$preserve['line_no'] = $line_no;
			
			$preserve['work_order_id'] = $work_order_id;
			$preserve['detail_id'] = $detail_id;
			$preserve['options'] = $data['options'];

			// Javascript
			$GLOBALS['egw']->js->validate_file('order', 'edit_line_detail', 'perp_orders');
			$GLOBALS['egw']->js->set_onload('add_handlers();');

			// Readonlys
			$config = config::read('perp_manufacturing');
			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $line->get_stock_id());
			$readonly['quantity'] = $config['one_serialized_per'] && $stock['serialized'];

			if(count($this->messages)) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Edit Item Details');
			$this->template->read($this->order_line_detail_edit);
			$this->template->exec('perp_orders.ui_perp_sales_order.edit_line_detail', $data, $select_box_options, $readonly, $preserve, 2);
		}

		/**
		*	Get selectbox options for displaying a list of Orders
		*/
		protected function order_list_get_options() {
			$options = parent::order_list_get_options();
			$options['filter2'] = array('' => lang('All')) + ExecMethod('perp_orders.bo_perp_orders.get_sales_categories');
			return $options;
		}

		/**
		*	Handle actions on a SalesOrder
		*	Actions can be one of new, edit, approve, deny, cancel which
		*	are handled by the parent, perp_api::ui_perp_order,
		*	or ship, invoice, which are handled here.
		*/
		public function handle_order_action($action, $order_id) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_orders';

			if($result) {
				return $result;
			}
			switch($action) {
				case 'cancel':
					$order = new SalesOrder($order_id);
					return $order->cancel($GLOBALS['egw_info']['user']['account_id']);
					break;
				case 'ship':
					$GLOBALS['egw']->redirect_link(
						'/index.php', array(
							'menuaction'    =>      'perp_ar.ui_perp_outgoing_shipment.new_shipment',
							'order_id'      =>      $order_id
						)
					);
					break;
				case 'invoice':
					$GLOBALS['egw']->redirect_link(
						'/index.php', array(
							'menuaction'    =>      'perp_ar.ui_perp_receivable_invoice.new_invoice',
							'order_id'      =>      $order_id
						)
					);
					break;
				default:
					$result = parent::handle_order_action($action, $order_id);
					return false;
			}
		}

		/**
		*	Update SalesOrder with the given data
		*	@see perp_api::ui_perp_order::update_order()
		*/
		protected function update_order(SalesOrder $sales_order, $content) {
			if(isset($content['client_account_id']) && is_numeric($content['client_account_id'])) {
				$result &= $this->check_result($sales_order->set_client_account_id((int)$content['client_account_id']));
			}
			if(isset($content['client_branch_id']) && is_numeric($content['client_branch_id'])) {
				$result &= $this->check_result($sales_order->set_client_branch_id((int)$content['client_branch_id']));
			}
			$result = parent::update_order($sales_order, $content);
			return $result;
		}

		/**
		*	Update an OrderLine with the given data
		*	@see perp_api::ui_perp_order::update_line()
		*/
		protected function update_line(SalesOrderLine $line, array $line_data) {
			$old_price_override_id = $line->get_price_override();

			$success = parent::update_line($line, $line_data);

			// see if there's a new price override
			$price_override_id = $line->get_price_override();
			if($price_override_id && $price_override_id != $old_price_override_id) {
				$edit_override_link = $GLOBALS['egw']->link('/index.php', 
					array(
						'menuaction'	=>	'infolog.infolog_ui.edit',
						'info_id'	=>	$price_override_id,
					)
				);
				$js = $GLOBALS['egw']->session->appsession('order_javascript');
				$js[] = $edit_override_js .="window.open('$edit_override_link', '_blank','dependent=yes,width=700,height=600,scrollbars=yes,status=yes'); "; 
				$GLOBALS['egw']->session->appsession('order_javascript', false, $js);
			}
			return $success;
		}

		/**
		*	Add SalesOrderLine(s) to the SalesOrder
		*	@see perp_api::ui_perp_order::add_line()
		*	@param order Order to add the line to
		*	@param line_data array of line data for the new line
		*
		*	@return the new line number
		*/
		protected function add_line(SalesOrder &$order, array &$line_data) {
			if($line_data['quantity_ordered'] == 0) {
				$this->messages[] = array(lang('Couldn\'t add line, quantity was 0'), 'warning');
				return false;
			}
			if(!$line_data['stock_id'] || !is_numeric($line_data['stock_id'])) {
				$this->messages[] = array(lang('Couldn\'t add line, no stock item selected'), 'warning');
				return false;
			}

			$stock = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $line_data['stock_id']);

			// Explode kits
			$new_lines = array();
			if($stock['stock_type_flag'] == 'K') {
				$bo_price = CreateObject('perp_orders.bo_perp_price');
				$query = array(
					'price_list_id'	=>	ExecMethod('perp_ar.bo_perp_client_account.get_price_lists', $order->get_client_account_id()),
					'effective_date'=>	$order->get_issue_date(),
					'stock_id'	=>	$line_data['stock_id'],
					'template'	=>	false
				);
				$components = so_perp_kit_price::get_kit_price_list($query);
				foreach($components as $component) {

					if($unit_price != PHP_INT_MAX) {
						$new_lines[] = array(
							'stock_id'		=>	$component['component_id'],
							'quantity_ordered'	=>	$line_data['quantity_ordered'] * $component['quantity'],
							'unit_price'		=>	$component['price'],
							'price_list_id'		=>	$component['price_list_id'],
						);
					} else {
						$this->messages[] = array(lang('Couldn\'t find price for %1', $detail['stock_num']), 'info');
					}
				}
			}
			if(count($new_lines) > 0) {
				$associated_lines = array();
				foreach($new_lines as $new_line_data) {
					$new_line_no = $this->add_line($order, $new_line_data);
					$associated_lines[] = $new_line_no;
					$new_line =& $order->get_line($new_line_no);
					$new_line->set_price_list_id($new_line_data['price_list_id']);
				}
			}

			// Add lines and check for options
			$line =& new SalesOrderLine($order, $line_data);
			$new_line_no = $order->add_line($line);
			if(is_array($associated_lines)) {
				foreach($associated_lines as $line_no) {
					$line->add_associated_line($line_no);
				}
			}

			// Check for a Client Reference, add one if missing and needed
			$config = config::read('perp_orders');
			if($config['use_client_reference']) {
				if(count($line->get_details('ClientReferenceDetail')) < 1) {
					$detail = new ClientReferenceDetail($line, array());
					$line->add_detail($detail);
				}
			}

			if($stock['stock_type_flag'] != 'R') {
				if($this->debug) {
					echo $stock['short_description'] . ' is not raw, I\'ll check for options.<br />';
				}
				$bo_wo = CreateObject('perp_manufacturing.bo_perp_build_option');
				
				$allowed_build_option_list_query['stock_id'] = $line_data['stock_id'];
				$allowed_build_option_list_query['value'] = array(-2,-3);
				$allowed_build_option_list_query['as_of_date'] = $order->get_issue_date();
				$options = ExecMethod2('perp_manufacturing.bo_perp_build_option.get_allowed_build_option_list', $allowed_build_option_list_query);

				if(count($options)) {
					if($this->debug) {
						echo count($options) . ' options found<br /';
					}
					if(!$order->get_order_id()) {
						// Need to save the sales order
						$result = $order->write();
						if(!is_numeric($result)) {
							$this->messages[] = array($result, 'warning');
							return false;
						}
					}
					if($order->get_order_id()) {
						$link = $GLOBALS['egw']->link('/index.php', 
							array(
								'menuaction'	=>	'perp_orders.ui_perp_sales_order.edit_line_detail',
								'order_id'	=>	$order->get_order_id(),
								'line_no'	=>	(string)$new_line_no
						));
						
						// Save this javascript in the session, because we need to reload the page to prevent re-POSTing 
						$js = $GLOBALS['egw']->session->appsession('order_javascript');
						$js[] = "window.open('$link', '_blank','dependent=yes,width=600,height=500,scrollbars=yes,status=yes'); ";
						$GLOBALS['egw']->session->appsession('order_javascript', false, $js);
						$GLOBALS['egw']->session->appsession('order_reload', false, true);
					} else {
						$this->messages[] = array($result, 'error');
					}
				} elseif ($stock['stock_type_flag'] == 'A' && !$stock['pseudo']) {
					if($this->debug) {
						echo 'No options found, so skip the options popup, but try to reserve a work order.<br />';
					}

					// This will find an unclaimed match and remove the temp, or turn the temp into a real work order
					$result = $this->bo_order->claim_work_orders($line, $line->get_owner()->get_requested_delivery_date());
					if($result !== true) {
						$this->messages += $result;
					}

					// Need to save the Order, so the reservation on this work order is available to others
					$line->get_owner()->write();
				} elseif ($stock['stock_type_flag'] == 'K') {
					// A kit without options.  We've already added all the components, so remove the kit

					// Remove the associated lines first, or they'll all go...
					if(is_array($associated_lines)) {
						foreach($associated_lines as $line_no) {
							$line->remove_associated_line($line_no);
						}
					}

					$order->remove_line($new_line_no);
				}
			}

			return $new_line_no;
		}

		/**
		*	Get data to display when editing (or viewing) a Sales Order
		*/
		protected function get_display_data(SalesOrder $sales_order) {
			$data = parent::get_display_data($sales_order);

			if($GLOBALS['egw']->acl->check('ledger', EGW_ACL_READ) &&
					($sales_order->can_invoice() === true || $sales_order->get_approval_required())) {
				// Check client credit limit - extra check for client account
				$bo_account = new bo_perp_client_account();
				$bo_client = new bo_perp_client();
				$client = $bo_client->get_client($sales_order->get_client_id());
				$account = $bo_account->get_client_account($sales_order->get_client_account_id());
				$balance = $bo_client->get_account_balance($sales_order->get_client_account_id());

				if($account['credit_limit'] != '' && ($sales_order->get_total() + $balance) > $account['credit_limit']) {
					$this->messages[] = array(
						lang('This order will exceed the credit limit of %1.  The current balance, including this order, is %2.',
							ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], $client['credit_limit']),
							ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $client['currency'], ($sales_order->get_total() + $balance))
						), 'warning'
					);
				}
			}

			$data['client_account_id']	=	array(
				'client_id'		=>	(string)$data['client_id'],
				'client_account_id'	=>	$sales_order->get_client_account_id(),
				// Active clients only
				'status'		=>	'1'
			);
			$data['client_branch_id']	=	array(
				'client_id'		=>	(string)$data['client_id'],
				'client_branch_id'	=>	$sales_order->get_client_branch_id()
			);

			/**
			*	Restrict the list of available clients to only active (status = 1) clients
			*/
			$data['client_id']	=	array(
				'client_id'		=>	$data['client_id'],
				'filter'	=>	'1'
			);

			// Restrict the price lists to buy from
			if($sales_order->get_client_id() && $sales_order->get_client_account_id()) {
				$price_lists = ExecMethod('perp_ar.bo_perp_client_account.get_price_lists', $sales_order->get_client_account_id());
			} elseif ($sales_order->get_client_id()) {
				$this->messages[] = array(lang('Client account is not set'), 'warning');
			}
			if(is_array($price_lists)) {
				$price_lists = implode(',', $price_lists);
			} elseif ($sales_order->get_client_account_id()) {
				$this->messages[] = array(lang('Unable to find any active price lists'), 'warning');
			}
			$data['new']['stock_id']	=	array(
				'price_list_id'		=>	$price_lists,
				'date'			=>	(string)$data['issue_date'],
				'include_options'	=>	false
			);

			/*
			*	Get order lines
			*	We do this because for some reason the nextmatch can't handle it.
			*	It only displays the first row, so instead the row template is used directly, and passed
			*	nextmatch[rows].  This works, but makes some long pages.
			*/
			$data['nextmatch']['start'] = 0;
			$data['nextmatch']['num_rows'] = -1;
			$this->get_order_lines($data['nextmatch'], $data['nextmatch']['rows'], $data['nextmatch']['readonlys']);

			if(count($data['nextmatch']['rows']) > 0) {
				array_unshift($data['nextmatch']['rows'], false);
			} else {
				unset($data['nextmatch']['rows']);
			}

			// Taxes
			$invoice = new ReceivableInvoice();
//			$invoice->read_from_object($sales_order);
			$taxes = $invoice->get_tax_lines();
			foreach($taxes as $tax) {
				$tax_array = $tax->toArray();
				$tax_array['formatted_amount'] = $GLOBALS['perp_api']['bo']->get_formatted_price($data['order_currency'], $tax_array['quantity'] * $tax_array['unit_price']);
				$data['taxes'][] = $tax_array;
			}
			$data['taxes'][] = array(
				'description'	=>	lang('total'),
				'formatted_amount'	=>	$GLOBALS['perp_api']['bo']->get_formatted_price($data['order_currency'], $invoice->get_total())
			);
			unset($invoice);

			return $data;
		}

		/**
		*	Get options for selectboxes for edit 
		*/
		protected function get_selectbox_options(SalesOrder $sales_order) {
			$options = parent::get_selectbox_options($sales_order);

			if($sales_order->get_client_id()) {
				$link_id = 'CLIENT-' . $sales_order->get_client_id();
				$contacts = egw_link::get_links('perp_ar', $link_id, 'addressbook');
				if($sales_order->get_client_branch_id()) {
					$link_id = 'CLIENT_BRANCH-' . $sales_order->get_client_branch_id();
					$contacts += egw_link::get_links('perp_ar', $link_id, 'addressbook');
				}
				foreach($contacts as $id => $contact) {
					$options['their_contact_person'][$contact] = egw_link::title('addressbook', $contact);
					$options['ship_to_address'][$contact] = $options['their_contact_person'][$contact];
				}
				if(count($contacts) == 0) {
					$this->messages[] = array(lang('No addresses set for client'), 'error');
				}
			}

			$options['ship_location'] = array(0 => lang('Any')) + ExecMethod('perp_inventory.bo_perp_location.get_location_list', array('filter2' => 'shippable'));

			return $options;
		}

		/**
		*	Get readonlys for sales order
		*/
		protected function get_readonlys(SalesOrder $order) {
			$readonly = parent::get_readonlys($order);

			$edit = $order->is_editor($GLOBALS['egw_info']['user']['account_id']);

			$readonly['client_account_id'] = array('search' => !$edit || $order->get_client_account_id());
			$readonly['client_branch_id'] = array('search' => !$edit || $order->get_client_branch_id());
			return $readonly;
		}
			
		/**
		*	Set up javascript and ajax
		*	@param ajax_callback appname.ajax_object Object to use for ajax callbacks
		*/
		protected function get_javascript($ajax_callback, $sales_order) {
			parent::get_javascript($ajax_callback);

			/*
			*	Javascript in the session was put there to preserve it across a 
			*	page reload.  The page reload is to prevent re-POSTing that would cause
			*	duplication.
			*/
			$session_js = $GLOBALS['egw']->session->appsession('order_javascript');
			if(!is_array($session_js)) {
				$session_js = array($session_js);
			}
			foreach($session_js as $js) {
				$GLOBALS['egw']->js->set_onload($js .' ');
			}
			$GLOBALS['egw']->session->appsession('order_javascript', false, false);
			$GLOBALS['egw']->js->set_onload("sales_order_edit_setup(" . $sales_order->is_editor($GLOBALS['egw_info']['user']['account_id']) . "); ");
			
		}

		/**
		*	Get selectbox options for line details.  This is where you can restrict which
		*	options are allowed for an OrderLineDetail.
		*/
		protected function get_line_detail_options(OrderLine &$line, $data) {
			if($line->get_stock_id()) {
				foreach($data['options'] as $key => $row) {
					if($row['options-value']) {
						$options[$key][''] = lang('Not important');
						foreach($row['options-value'] as $id => $option) {
							$options[$key][$id] = $option['price'] . ' : ' .  $option['build_option_name'];
						}
					}
				}
			}
			if($data['unclaimed']) {
				$options['option_category_id'] = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option_category_list');
				$options['build_option_id'] = ExecMethod('perp_manufacturing.bo_perp_build_option.get_build_option_list');
			}
			return $options;
		}

		/** 
		*	Update OrderLineDetails using data from the eTemplate
		*
		*	@todo This probably doesn't work correctly, as it just changes the work order ID, nothing else.
		*	@param detail OrderLineDetail to update
		*	@param data Array of data to update the detail with
		*/
		protected function update_line_detail(OrderLineDetail &$detail, Array $data) {
			if($detail instanceof WorkOrderDetail) {
				if($data['edit']) {
					// User wants to edit the work order
					$work_order_link = $GLOBALS['egw']->link('/index.php', 
						array(
							'menuaction'	=>	'perp_manufacturing.ui_perp_work_order.work_order_detail_edit',
							'work_order_id'	=>	$detail->get_work_order_id(),
							'detail_id'	=>	$detail->get_work_order_detail_id()
						)
					);
					$js = $GLOBALS['egw']->session->appsession('order_javascript');
					$js[] = "window.open('$work_order_link', '_blank','dependent=yes,width=700,height=600,scrollbars=yes,status=yes'); "; 
					$GLOBALS['egw']->session->appsession('order_javascript', false, $js);
				}

				/**
				*	Checking if the key work_order_reference exists before we mess 
				*	with the Detail lets us avoid clearing work orders if they're 
				*	read-only, because they've been shipped or invoiced
				*/
				if(array_key_exists('work_order_reference', $data)) {
					list($work_order_reference, $detail_id) = explode(':', $data['work_order_reference']);
					if(!$detail_id) {
						$detail_id = $detail->get_work_order_detail_id() ? $detail->get_work_order_detail_id() : 1;
					}
					if(!$work_order_reference) {
						$detail->get_order_line()->remove_detail($detail->get_number());
					}
					if($work_order_reference == $detail->get_work_order_reference() && $detail_id == $detail->get_work_order_detail_id()) {
						// Unchanged
						return;
					}
					$work_order_check = ExecMethod('perp_manufacturing.bo_perp_work_order.get_work_order_list', 
						array(
							'search'	=>	$work_order_reference,
							'order_id'	=>	null
						)
					);
					$work_order_id = null;
					if(count($work_order_check) == 1) {
						$work_order_id = key($work_order_check);
					} else {
						foreach($work_order_check as $id => $reference) {
							if ($id < 0 && $id == -($detail->get_order_line()->get_owner()->get_order_id())) {
								// Old temporary work order
								try {
									ExecMethod('perp_manufacturing.bo_perp_work_order.delete_work_order', $id);
								} catch (Exception $e) {
									// Just silence it
								}
							} elseif($reference == $work_order_reference) {
								$work_order_id = $id;
							}
						}
					}
				} else {
					$work_order_id = $detail->get_work_order_id();
					$detail_id = $detail->get_work_order_detail_id();
				}

				if(!is_null($work_order_id)) {
					$check = $detail->set_work_order_id($work_order_id);
					if($check === true) {
						$check = $detail->set_work_order_detail_id($detail_id);
					}
					if($check !== true) {
						return $check;
					}

					// Update quantity
					$wo_quantity = 0;
					$work_orders = $detail->get_order_line()->get_details('WorkOrderDetail');

					// Check to see if we've reserved more than we need
					foreach($work_orders as &$wo_detail) {
						$wo_quantity += $wo_detail->get_work_order_quantity();
						if($wo_quantity > $detail->get_order_line()->get_quantity_ordered()) {
							// Reduce reserved quantity
							$over_by = $wo_quantity - $detail->get_order_line()->get_quantity_ordered();
							$over_by = max(0, $over_by);
							$result = $wo_detail->set_work_order_quantity(max(0, $wo_detail->get_work_order_quantity() - $over_by));
							if($result !== true) {
								return $result;
							}
						}

						// Remove work order if we don't need it
						if($wo_detail->get_work_order_quantity() <= 0) {
							if($this->debug) {
								echo 'Removed work order ' . $wo_detail->__toString() . ' from line ' . $wo_detail->get_order_line()->__toString() . '<br />';
							}
							$detail->get_order_line()->remove_detail($wo_detail->get_number());
						}

						// Check for two details for the same work order, only one is needed, so merge them
						if($wo_detail != $detail && 
								$wo_detail->get_work_order_id() == $detail->get_work_order_id() && 
								$wo_detail->get_work_order_detail_id() == $detail->get_work_order_detail_id()) {
							$detail->set_work_order_quantity($detail->get_work_order_quantity() + $wo_detail->get_work_order_quantity());
							$detail->get_order_line()->remove_detail($wo_detail->get_number());
						}
					}

					// Check to see if we've reserved less than we need
					if($wo_quantity < $detail->get_order_line()->get_quantity_ordered()) {
						if($this->debug) {
							echo "Not enough stock from work orders ($wo_quantity).  Getting more...<br />";
						}
						$result = $this->bo_order->claim_work_orders(
							$detail->get_order_line(), 
							$detail->get_order_line()->get_owner()->get_requested_delivery_date(),
							$detail->get_work_order_id(),
							$detail->get_work_order_detail_id()
						);
					}
				} else {
					return lang('Could not find a match for work order %1, it may be claimed already.', $data['work_order_reference']);
				}
			} else {
				parent::update_line_detail($detail, $data);
			}
		}
	}
?>
