<?php

	include_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.perp_report.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_orders/inc/class.SalesOrder.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_orders/inc/class.bo_perp_sales_order.inc.php');

	/**
	*	Sales Order
	*/
	class sales_order extends perp_report {

		public function __construct(array &$parameters = array()) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_orders';
			parent::perp_report($parameters);

			$this->template = 'sales_order.odt';

			$config = config::read('perp_orders');

			// Default parameters
			if(isset($parameters['client_id']) && !isset($parameters['order_id'])) {
				$this->parameters['order_id']['client_id'] = (string)$parameters['client_id'];
			}
			if(!$this->parameters['order_id'] && !$this->parameters['order_date']) {
				$this->parameters['order_date'] = mktime(0,0,0,date('m'), date('d'), date('Y'));
			}

			$this->parameters['extra_comment'] = $this->parameters['extra_comment'] ? $this->parameters['extra_comment'] : $config['report']['sales_order']['extra_comment'];
			$config['report']['sales_order']['extra_comment'] = $this->parameters['extra_comment'];

			// These options are copied from perp_api.ui_perp_invoice
			$this->parameters['options-filter'] = array(false => 'No details', 1 => 'Some details', 2 => 'Full details');

			$this->parameters['order_id'] = $parameters['order_id'] ? $parameters['order_id'] : reg_var('order_id');
			$this->parameters['filter'] = $parameters['filter'] ? $parameters['filter'] : reg_var('filter');

			// If order_id is provided, just generate the report
			if($this->parameters['order_id']) {
				$parameters['generate_report'] = true;
			}

			config::save_value('report', $config['report'], 'perp_orders');
		}

		public function get_data($query = array()) {
			if(!(int)$query['order_id'] && !($query['order_date'] || $query['client_id'])) {
				return false;
			}

			// Get a list of orders dated on or after the order date
			if(!$query['order_id'] && ($query['client_id'] || $query['order_date'])) {
				$order_query = array(
					'client_id'	=>	$query['client_id'],
					'issue_date'	=>	array(
						'start'	=>	$query['order_date']
					),
					'template' 	=>	false,
				);
				$bo_order = new bo_perp_sales_order();
				$orders = array();
				$count = $bo_order->get_order_list($order_query, $orders);
				if($count == 0) {
					return false;
				}
				foreach($orders as $order) {
					$query['order_id'][] = $order['order_id'];
				}
			} else {
				$query['order_id'] = explode(',', $query['order_id']);
			}

			$data = array();
			$this->terms = ExecMethod('perp_api.bo_perp_api.get_payment_terms');

			foreach($query['order_id'] as $order_id) {
				
				$header = $this->get_header($order_id, $query);
				$data['order'][] = $header;
				$data[$order_id . '_taxes'] = $header['taxes'];
				$lines = $this->get_lines($order_id, $query);
				$data[$order_id . '_line'] = $lines['line'];
				if(is_array($lines['details'])) {
					$data += $lines['details'];
				}
			}

			// Logo
			$config = config::read('perp_api');
			$data['images'] = array();
			$data['images'][] = array(
				'name'	=>	'company_logo',
			) + $config['company_logo'];

			if(count($query['order_id']) == 1) {
				$this->filename = lang('Sales Order') . ' ' . $query['order']->get_our_reference_number();
			} else {
				$this->filename = lang('Multiple Sales Orders');
			}
//_debug_array($data); die();
			return $data;
		}

		protected function get_header($order_id, &$query) {
			$order =& new SalesOrder($order_id);
			$query['order'] =& $order;

			$data = array(
				'order_id'		=>	$order_id,
				'our_reference'		=>	$order->get_our_reference_number(),
				'their_reference'	=>	$order->get_their_reference_number(),
				'comment'		=>	$order->get_comments() . "\n" . $query['extra_comment'],
				'order_date'		=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $order->get_issue_date()),
				'requested_delivery_date'=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'] . ' H:i', $order->get_requested_delivery_date()),
				'currency'		=>	$order->get_order_currency(),
			);
			$account = ExecMethod('perp_ar.bo_perp_client_account.get_client_account', $order->get_client_account_id());
			$branch = ExecMethod('perp_ar.bo_perp_client_branch.get_client_branch', $order->get_client_branch_id());
			$data['client_account'] = $account['client_account_name'];
			$data['client_branch'] = $branch['branch_name'];

			// Create an invoice to get totals and taxes
			$invoice = new ReceivableInvoice();
			$invoice->read_from_object($order);

			$data['subtotal']	=	ExecMethod2(
				'perp_api.bo_perp_api.get_formatted_price', 
				$order->get_invoice_currency(), 
				$invoice->get_total(InvoiceLine::NORMAL) + $invoice->get_total(InvoiceLine::DISCOUNT)
			);
			$data['total']		=	ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $order->get_invoice_currency(), $invoice->get_total());
			$taxes			=	$invoice->get_tax_lines();
			$data['taxes']		=	array();
			foreach($taxes as $tax_line) {
				if($tax_line->get_unit_price()) {
					$data['taxes'][] = $tax_line->toArray();
				}
			}

			$invoice = null;

			$data['terms']		=	$this->terms[$order->get_payment_terms()];

			// Their billing address
			$data['bill'] = ExecMethod('addressbook.addressbook_bo.read', $order->get_bill_to_address());
			if(!$data['bill']) {
				$client = ExecMethod('perp_ar.bo_perp_client.get_client', $order->get_client_id());
				$data['bill'] = ExecMethod('addressbook.addressbook_bo.read', $client['address']);
			}

			// Their shipping address
			$data['ship'] = ExecMethod('addressbook.addressbook_bo.read', $order->get_ship_to_address());

			// Contact
			$data['our_contact'] = ExecMethod('addressbook.addressbook_bo.read', $order->get_our_contact_person());
			$data['their_contact'] = ExecMethod('addressbook.addressbook_bo.read', $order->get_their_contact_person());


			// Payable address
			if($order->get_bill_from_address()) {
				$data['pay'] = ExecMethod('addressbook.addressbook_bo.read', $order->get_bill_from_address());
			} else {
				$config = config::read('perp_ar');
				$data['pay'] = ExecMethod('addressbook.addressbook_bo.read', $config['default_bill_from']);
			}

			return $data;
		}

		/**
		*	Get the lines for the invoice
		*/
		protected function get_lines($order_id, $query) {

			$data = array();

			// Use the UI function to get all the nice details
			$ui = new ui_perp_sales_order();
			$ui->get_order_lines($query, $data['line'], $readonlys);

			foreach($data['line'] as $key => &$line) {
				if(!is_array($line)) {
					unset($data['line'][$key]);
					continue;
				}
				$line['line_price'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $query['order']->get_order_currency(), $line['unit_price'] * $line['quantity_ordered']);
				$line['unit_price'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $query['order']->get_order_currency(), $line['unit_price']);

				
				// Move details into a seperate array so they can be merged
				unset($line['details']['details']);
				$details = $query['order']->get_line($line['line_number'])->get_details();
				foreach($details as &$detail) {
					if($detail->get_display()) {
						$data['details'][$order_id . '_' . $line['line_number'] . '_detail'][] = array(
							'name'	=>	lang($detail->get_name()),
							'value'	=>	$detail->get_value()
						);
					}
				}
				unset($line['details']);
			}

			return $data;
		}
	}

?>
