<?php
// Call OrderTest::main() if this source file is executed directly.
if (!defined("PHPUnit_MAIN_METHOD")) {
    define("PHPUnit_MAIN_METHOD", "Simple_Pricing_Test::main");
}

require_once "PHPUnit/Framework/TestCase.php";
require_once "PHPUnit/Framework/TestSuite.php";

/**
 * Test class for Pricing
 * Generated by PHPUnit_Util_Skeleton on 2007-03-09 at 15:50:13.
 */

require_once 'Single_Price_List_Test.php';
require_once '../inc/class.bo_perp_price.inc.php';
require_once '../../perp_manufacturing/inc/class.bo_perp_build_option.inc.php';

class Option_Pricing_Test extends Single_Price_List_Test {
    /**
     * Runs the test methods of this class.
     *
     * @access public
     * @static
     */
    public static function main() {
        require_once "PHPUnit/TextUI/TestRunner.php";

        $suite  = new PHPUnit_Framework_TestSuite("Simple_Pricing_Test");
        $result = PHPUnit_TextUI_TestRunner::run($suite);
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     *
     * @access protected
     */
    protected function setUp() {
    	parent::setUp();

	// Also create a price list (we need that price list)
	parent::createNewPriceList();

	$this->bo = new bo_perp_price();
	$this->bo_option = new bo_perp_build_option();

	// Create test items
	$this->stock_items = array(
		195	// Use a known assembly, Rainier
	);

	// Put test stock items into test sales category
	foreach($this->stock_items as $stock_id) {
		ExecMethod2('perp_inventory.bo_perp_stock.add_sales_category', $stock_id, $this->sales_category_id);
	}

    }

	/**
	* Tears down the fixture, for example, close a network connection.
	* This method is called after a test is executed.
	*
	* @access protected
	*/
	protected function tearDown() {
		parent::tearDown();

		// Remove stock items from test sales category
		foreach($this->stock_items as $stock_id) {
			ExecMethod2('perp_inventory.bo_perp_stock.delete_sales_category', $stock_id, -1);
		}

		$this->bo_price_list->delete_price_list($this->price_list_id);

		/**
		*	Don't want to do this if we're using real stock items
		*	(this test will screw with data anyway, but let's not
		*	get carried away)
		foreach($this->stock_items as $stock_id) {
			ExecMethod('perp_inventory.bo_perp_stock.delete', $stock_id);
		}
		*/
	}

	/**
	*	Check and make sure test stock items are an assembly, and have build options
	*/
	function test_Verify_Test_Items() {
		foreach($this->stock_items as $stock_id) {
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);
			$this->assertEquals('A', $stock_item['stock_type_flag']);

			$option_query = array(
				'num_rows'	=>	-1,
				'stock_id'	=>	$stock_id,
				'order'		=>	'build_option_category, build_option_id',
			);
			$allowed_build_option_list_query['stock_id'] = $stock_id;
			$options = $this->bo_option->get_allowed_build_option_list($allowed_build_option_list_query);
			$this->assertNotEquals(0, count($options), 'Check for build options');
		}
	}

	/**
	*	Test setting option prices, effective immediately
	*/
	function test_Set_Pricing_Now() {
		$effective_date = time();

		$price = 5.00;

		$check = array();
		$options = array();

		$price_query = array(
			'price_list_id'	=>	$this->price_list_id,
			'num_rows'	=>	-1,
			'date'		=>	$effective_date,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id) {
			$price_query['stock_id']	=	$stock_id;
			
			$allowed_build_option_list_query['stock_id'] = $stock_id;
			$allowed_build_option_list_query['as_of_date'] = $effective_date;
			$options = $this->bo_option->get_allowed_build_option_list($allowed_build_option_list_query);

			foreach($options as $option) {
				$check[$stock_id][$option['build_option_id']] = $price * $option['build_option_id'];
				$this->bo->set_price($this->price_list_id, $stock_id, $price * $option['build_option_id'], $effective_date, $option['build_option_id']);
			}

			// Verify set prices
			$this->bo_option->get_price_list($price_query, $prices);
			foreach($prices as $price) {
				if(isset($check[$stock_id][$price['build_option_id']])) {
					$this->assertEquals($price['price'], $check[$stock_id][$price['build_option_id']], 'Checking prices, effective immediately');
					$this->assertEquals($price['effective_date'], $effective_date);
				}
			}
		}
	}


	/**
	*	Test setting prices, effective in the future
	*/
	function test_Set_Pricing_Future() {
		$now = time();
		$future = $now + 60*60;

		$price = 5.00;

		$check = array();
		$options = array();

		$price_query = array(
			'price_list_id'	=>	$this->price_list_id,
			'num_rows'	=>	-1,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id) {
			$price_query['stock_id']	=	$stock_id;
			
			$allowed_build_option_list_query['stock_id'] = $stock_id;
			$allowed_build_option_list_query['as_of_date'] = $effective_date;
			$options = $this->bo_option->get_allowed_build_option_list($allowed_build_option_list_query);

			foreach($options as $option) {
				$check[$stock_id][$option['build_option_id']][$now] = $price;
				$check[$stock_id][$option['build_option_id']][$future] = $price + 1.23;
				$this->bo->set_price($this->price_list_id, $stock_id, $price, $now, $option['build_option_id']);
				$this->bo->set_price($this->price_list_id, $stock_id, $price + 1.23, $future, $option['build_option_id']);
			}

			// Verify set prices (now)
			$price_query['date'] = $now;
			$this->bo_option->get_price_list($price_query, $prices);
			foreach($prices as $price) {
				if(isset($check[$stock_id][$price['build_option_id']])) {
					$this->assertEquals($price['price'], $check[$stock_id][$price['build_option_id']][$now]);
					$this->assertEquals($price['effective_date'], $now);
				}
			}

			// Verify set prices (future)
			$price_query['date'] = $future;
			$this->bo_option->get_price_list($price_query, $prices);
			foreach($prices as $price) {
				if(isset($check[$stock_id][$price['build_option_id']])) {
					$this->assertEquals($price['price'], $check[$stock_id][$price['build_option_id']][$future]);
					$this->assertEquals($price['effective_date'], $future);
				}
			}
		}
	}
}

// Call Simple_Pricing_Test::main() if this source file is executed directly.
if (PHPUnit_MAIN_METHOD == "Simple_Pricing_Test::main") {
    Simple_Pricing_Test::main();
}
?>
