<?php
// Call OrderTest::main() if this source file is executed directly.
if (!defined("PHPUnit_MAIN_METHOD")) {
    define("PHPUnit_MAIN_METHOD", "Simple_Pricing_Test::main");
}

require_once "PHPUnit/Framework/TestCase.php";
require_once "PHPUnit/Framework/TestSuite.php";

/**
 * Test class for Pricing
 * Generated by PHPUnit_Util_Skeleton on 2007-03-09 at 15:50:13.
 */

require_once 'Single_Price_List_Test.php';
require_once '../inc/class.bo_perp_price.inc.php';

class Simple_Pricing_Test extends Single_Price_List_Test {
    /**
     * Runs the test methods of this class.
     *
     * @access public
     * @static
     */
    public static function main() {
        require_once "PHPUnit/TextUI/TestRunner.php";

        $suite  = new PHPUnit_Framework_TestSuite("Simple_Pricing_Test");
        $result = PHPUnit_TextUI_TestRunner::run($suite);
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     *
     * @access protected
     */
    protected function setUp() {
    	parent::setUp();

	// Also create a price list (we need that price list)
	parent::createNewPriceList();


	$this->bo = new bo_perp_price();

	$this->stock_items = array(
		1	=>	5.00, 
		6	=>	10.00, 
		75	=>	11.11, 
		76	=>	2000,
		195	=>	3000,	// This one is a spa
	);

	// Put test stock items into test sales category
	foreach($this->stock_items as $stock_id) {
		ExecMethod2('perp_inventory.bo_perp_stock.add_sales_category', $stock_id, $this->sales_category_id);
	}

    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     *
     * @access protected
     */
    protected function tearDown() {
	// Remove stock items from test sales category
	foreach($this->stock_items as $stock_id) {
		ExecMethod2('perp_inventory.bo_perp_stock.delete_sales_category', $stock_id, -1);
	}

	$this->bo_price_list->delete_price_list($this->price_list_id);

    	parent::tearDown();
    }

	
	/**
	*	Test setting prices, effective immediately
	*/
	function test_Set_Pricing_Now() {
		$effective_date = time();
		$query = array(
			'price_list_id'	=>	$this->price_list_id,
			'date'		=>	$effective_date,
			'include_not_allowed'	=>	true,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id => $price) {
			$query['stock_id'] = $stock_id;
			$this->bo->set_price($this->price_list_id, $stock_id, $price, $effective_date);
			$this->bo->get_price_list($query, $prices);
			$this->assertEquals($price, $prices[0]['price']);
			$this->assertEquals($effective_date, $prices[0]['effective_date']);
		}
	}


	/**
	*	Test setting prices, effective in the future
	*/
	function test_Set_Pricing_Future() {
		// Current time
		$now = time();

		// One hour in the future
		$future = time() + 60*60;

		// Query to check prices
		$query = array(
			'price_list_id'	=>	$this->price_list_id,
			'include_not_allowed'	=>	true,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id => $price) {
			$this->bo->set_price($this->price_list_id, $stock_id, ($price + 1.23), $future);

			$query['stock_id']	=	$stock_id;

			// Check pricing NOW (Should be not set)
			$query['date']		=	$now;

			$this->bo->get_price_list($query, $prices);
			$this->assertNull($prices[0]['price']);
			$this->assertNull($prices[0]['effective_date']);

			// Check pricing in the future
			$query['date']		=	$future;

			$this->bo->get_price_list($query, $prices);
			$this->assertEquals(($price + 1.23), $prices[0]['price']);
			$this->assertEquals($future, $prices[0]['effective_date']);
		}
	}

	/**
	*	Test setting prices in the past.  Should fail.
	*/
	function test_Set_Pricing_Past() {
		// One hour in the past
		$past = time() - 60*60;

		// Query to check prices
		$query = array(
			'price_list_id'	=>	$this->price_list_id,
			'include_not_allowed'	=>	true,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id => $price) {
			$result = $this->bo->set_price($this->price_list_id, $stock_id, $price, $past);

			// Make sure it didn't return true
			$this->assertTrue(($result !== true));


			// Check pricing NOW (Should be not set)
			$query['stock_id']	=	$stock_id;
			$query['date']		=	time();

			$this->bo->get_price_list($query, $prices);
			$this->assertNull($prices[0]['price']);
			$this->assertNull($prices[0]['effective_date']);
		}
	}

	/**
	*	Test setting an item as not allowed
	*/
	function test_Not_Allowed() {
		$now = time();

		// Query to check prices
		$query = array(
			'price_list_id'	=>	$this->price_list_id,
			'date'		=>	$now,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id => $price) {
			$this->bo->set_price($this->price_list_id, $stock_id, so_perp_price::NOT_ALLOWED, $now);

			$query['stock_id']	=	$stock_id;
			$query['include_not_allowed']	=	false;
			$this->bo->get_price_list($query, $prices);
			$this->assertNull($prices[0]['price'], 'Price is N/A and excluded, but came back anyway.');
			$this->assertNull($prices[0]['effective_date']);

			$query['include_not_allowed']	=	true;
			$this->bo->get_price_list($query, $prices);
			$this->assertEquals(so_perp_price::NOT_ALLOWED, $prices[0]['price']);
			$this->assertEquals($now, $prices[0]['effective_date']);
		}
	}

	/**
	*	Test date cross-over
	*/
	function test_Date_Change() {
		// Current time
		$now = time();

		// One hour in the future
		$future = time() + 60*60;

		// Query to check prices
		$query = array(
			'price_list_id'	=>	$this->price_list_id,
			'include_not_allowed'	=>	true,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id => $price) {
			$this->bo->set_price($this->price_list_id, $stock_id, $price, $now);
			$this->bo->set_price($this->price_list_id, $stock_id, ($price + 1.23), $future);

			$query['stock_id']	=	$stock_id;

			// Check pricing right before $future 
			$query['date']		=	$future - 1;
			$this->bo->get_price_list($query, $prices);
			$this->assertEquals($price, $prices[0]['price']);
			$this->assertEquals($now, $prices[0]['effective_date']);

			// Check pricing in the future
			$query['date']		=	$future;
			$this->bo->get_price_list($query, $prices);
			$this->assertEquals(($price + 1.23), $prices[0]['price']);
			$this->assertEquals($future, $prices[0]['effective_date']);
		}
	}

	/**
	*	Test all prices for a stock item, over time
	*/
	function test_Stock_History() {
		$now = time();

		// History (time => price)
		$history = array(
			$now		=>	1.23,
			$now + 60	=>	2.34,
			$now + 180	=>	3.45,
			$now + 500	=>	so_perp_price::NOT_ALLOWED,
			$now + 60*60*24	=>	4.56
		);
			
		// Query to check prices
		$query = array(
			'price_list_id'	=>	$this->price_list_id,
			'include_not_allowed'	=>	true,
			'template'	=>	false,
		);
		foreach($this->stock_items as $stock_id => $_not_used) {
			foreach($history as $time => $price) {
				$this->bo->set_price($this->price_list_id, $stock_id, $price, $time);
			}

			$query['stock_id']	=	$stock_id;
			$count = $this->bo->get_price_list($query, $prices);
			$this->assertEquals($count, count($history));
			foreach($prices as $price_list) {
				if(!is_array($price_list)) {
					continue;
				}
				$this->assertEquals($history[$price_list['effective_date']], $price_list['price'], 'Historical price mis-match');
			}
		}
	}
}

// Call Simple_Pricing_Test::main() if this source file is executed directly.
if (PHPUnit_MAIN_METHOD == "Simple_Pricing_Test::main") {
    Simple_Pricing_Test::main();
}
?>
