<?php
// Call OrderTest::main() if this source file is executed directly.
if (!defined("PHPUnit_MAIN_METHOD")) {
    define("PHPUnit_MAIN_METHOD", "Single_Price_List_Test::main");
}

require_once "PHPUnit/Framework/TestCase.php";
require_once "PHPUnit/Framework/TestSuite.php";

/**
 * Test class for Pricing
 * Generated by PHPUnit_Util_Skeleton on 2007-03-09 at 15:50:13.
 */

require_once '../inc/class.bo_perp_price_list.inc.php';
require_once '../inc/class.bo_perp_orders.inc.php';

class Single_Price_List_Test extends PHPUnit_Framework_TestCase {
    /**
     * Runs the test methods of this class.
     *
     * @access public
     * @static
     */
    public static function main() {
        require_once "PHPUnit/TextUI/TestRunner.php";

        $suite  = new PHPUnit_Framework_TestSuite(__CLASS__);
        $result = PHPUnit_TextUI_TestRunner::run($suite);
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     *
     * @access protected
     */
    protected function setUp() {
	$this->bo_order = new bo_perp_orders();
	$this->bo_price_list = new bo_perp_price_list();

    	// Add new sales_category, sales_area, sales_type for testing

	// Sales category
	$cost_gl_account	=	'500000';
	$sales_gl_account	=	'400000';
	$category = array(
		'description'	=>	'Test sales category',
		'cost_of_sales_gl_account'	=>	$cost_gl_account,
		'sales_gl_account'		=>	$sales_gl_account
	);
	$this->sales_category_id = $this->bo_order->set_sales_category($category);

	// Sales area
	$area = array(
		'sales_area_id'		=>	-1,
		'sales_area_name'	=>	'Test Area A'
	);
	ExecMethod2('perp_api.bo_perp_api.change_lookup', 'perp_sales_area', 'perp_orders', null, $area);

	// Sales Type
	$type = array(
		'sales_type_id'		=>	-1,
		'sales_type_name'	=>	'Test Type A'
	);
	ExecMethod2('perp_api.bo_perp_api.change_lookup', 'perp_sales_type', 'perp_orders', null, $type);

    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     *
     * @access protected
     */
    protected function tearDown() {
	// Delete test category / area / type
	$this->bo_order->delete_sales_category($this->sales_category_id);
	ExecMethod2('perp_api.bo_perp_api.change_lookup', 'perp_sales_area', 'perp_orders', array('sales_area_id' => -1), null);
	ExecMethod2('perp_api.bo_perp_api.change_lookup', 'perp_sales_type', 'perp_orders', array('sales_type_id' => -1), null);

	$this->bo_price_list->delete_price_list($this->price_list_id);
    }

	/**
	*	Create a new price list for testing
	*
	*	@return new price list ID
	*/
	protected function createNewPriceList() {
		$config = config::read('perp_ledger');
		$this->price_list_id = -1;
		$new_list = array(
			'price_list_id'		=>	$this->price_list_id,
			'price_list_name'	=>	__METHOD__,
			'currency'		=>	$config['gl_currency'],
			'sales_category_id'	=>	$this->sales_category_id,
			'sales_area_id'		=>	-1,
			'sales_type_id'		=>	-1,
		);
		return $this->bo_price_list->set_price_list($this->price_list_id, $new_list);
	}

    /**
    *	Try to create a new price list
    */
    function testNewPriceList() {
    	$result = $this->createNewPriceList();
	$this->assertEquals($this->price_list_id, $result);
    }

    /**
    *	Test deleting the price list
    */
    function testDeletePriceList() {
	$this->bo_price_list->delete_price_list($this->price_list_id);

	// Check
	$empty = $this->bo_price_list->get_price_list($this->price_list_id);
	$this->assertType('array', $empty);
	$this->assertEquals(0, count($empty));
    }
}

// Call Single_Price_List_Test::main() if this source file is executed directly.
if (PHPUnit_MAIN_METHOD == "Single_Price_List_Test::main") {
    Single_Price_List_Test::main();
}
?>
