<?php

	/**
	*	Amounts based on time sheet information
	*
	*	Using a custom field in timesheet named staff_id that is set to the staff id of the 
	*	employee, this calculation searches for any timesheet entries for the employee in the set
	*	TimeSheet category that occur in this pay period.  
	*	The amounts for the paystub are pulled directly from the TimeSheet entry, 
	*	Unitprice * Quantity, for any matching entry.  For this reason, you may want to look into
	*	restricting who can edit these TimeSheet entries.
	*
	*	This Calculation is notable in that it actually changes the PaystubLine, setting both quantity 
	*	and rate as well as returning the rate.
	*
	*	@author Nathan Gray
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Calculation.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/timesheet/inc/class.timesheet_bo.inc.php');

	class TimeSheet extends Calculation {

		/**
		*	Access into the TimeSheet application
		*	Set after the class due to PHP limits
		*/
		public static $bo_timesheet = null;

		public static function calculate(PaystubLine &$line, $value) {
			// End date may end in 00:00, instead of 23:59, which will lose the last day
			if(date('H:i', $line->get_paystub()->get_pay_period()->end_date) == '00:00' && 
				date('H:i', $line->get_paystub()->get_pay_period()->start_date) == '00:00') {
				$end_date = strtotime('+1 day', $line->get_paystub()->get_pay_period()->end_date) - 1; 
			} else {
				$end_date = $line->get_paystub()->get_pay_period()->end_date;
			}

			// Earnings from timesheet
			$filter = array(
				'ts_extra_value = ' . $GLOBALS['egw']->db->quote('STAFF-' . $line->get_paystub()->get_staff_id()),
				'cat_id = ' . $value['timesheet_category_id']
			);

			$timesheet = self::$bo_timesheet->search(
				array(
					'ts_start >= ' . $line->get_paystub()->get_pay_period()->start_date,
					'ts_start < ' . $end_date,
					'perp_earning_type.type_id = ' . $line->get_type()->get_id()
				),
				false, 'GROUP BY staff_id, perp_earning_type.type_id, ts_unitprice ORDER BY sort_order ',
				array(
					'ts_extra_value AS staff_id',
					'SUM(ts_quantity) AS hours',
					'round(sum(ts_quantity)*ts_unitprice,2) AS amount',
					'perp_earning_type.type_id AS type_id',
					'name'
				),
				'', false, 'AND', false, $filter,
				' JOIN egw_timesheet_extra ON
					egw_timesheet_extra.ts_id = egw_timesheet.ts_id AND
					egw_timesheet_extra.ts_extra_name = \'staff_id\'
				JOIN perp_earning_type ON
					egw_timesheet.cat_id = perp_earning_type.timesheet_category_id
				JOIN perp_pay_stub_entry_type ON
					perp_earning_type.type_id = perp_pay_stub_entry_type.type_id'
			);

			if(is_array($timesheet)) {
				if(count($timesheet) == 1 && isset($timesheet[0])) {
					$line->set_quantity($timesheet[0]['hours']);

					// If the unit price isn't set in the timesheet entry, get it from the entry type
					if(!$timesheet[0]['ts_unitprice']) {
						$staff_value = $line->get_type()->get_value($line->get_paystub()->get_staff_id());

						if(!array_key_exists('amount', $staff_value)) {
							// Not their normal pay type.  Find it, in an hourly equivalent.
							$staff_value['amount'] = PayrollUtility::get_salaried_rate(
								$line->get_paystub()->get_staff(), 
								'h', 
								$line->get_paystub()->get_pay_period_id()
							);
						}
						$line->set_rate($staff_value['amount'] * $staff_value['rate_multiplier']);
					} else {
						$line->set_rate($timesheet[0]['ts_unitprice']);
					}

					// Check if this is an accrual draw down and check amounts
					if($line->get_type() instanceof EarningType) {
						$accrual_id = $line->get_type()->get_accrual_type();
						if($accrual_id !== false) {
							$accrual = PayStubEntryType::factory($accrual_id);
							$accrual->draw_check($line);
						}
					}
					return $line->get_rate();
				}
			}
		}

		public static function getName() {
			return 'TimeSheet based';
		}

		public static function getTemplate() {
			return 'perp_payroll.Calculation.TimeSheet';
		}
		public static function getOverrideTemplate() {
			return '';
		}
		public static function getOptions() {
			return array();
		}
	}

	if(is_null(TimeSheet::$bo_timesheet)) {
		TimeSheet::$bo_timesheet = new timesheet_bo();
	}
?>
