<?php

	/**
	*	Pay stub amendment
	*
	*	Amendments to a pay stub are for 1 time payments / withdrawals, such
	*	as expense re-imbursement.  They can (and should) be made before the paystub is generated,
	*	if they are created after, the stub(s) will have to be re-created to include the amendment.
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.PaystubEntryType.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.PayPeriod.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Paystub.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/etemplate/inc/class.so_sql2.inc.php');

	class Amendment extends so_sql2 {


		/**
		*	Overloaded constructor
		*
		*	new Amendment(amendment_id) will load an existing amendment
		*	new Amendment() will create a new amendment.
		*/
		public function __construct($amendment_id = false) {
			// Set up for so_sql
			$this->so_sql2('perp_payroll', 'perp_pay_amendment');
			$this->autoinc_id = 'amendment_id';
		
			if(!is_null($amendment_id) && (int)$amendment_id != 0 && $amendment_id !== false) {
				// Read an existing amendment
				$this->amendment_id = (int)$amendment_id;
				$this->read(array('amendment_id' => $amendment_id));
			}
		}

		public function save($recreate_stub = false) {
			if(!$this->data['staff_id']) {
				throw new Exception(lang('Amendment #%1 employee is not set', $this->amendment_id));
			}
			if(!$this->data['type_id']) {
				throw new Exception(lang('Amendment #%1 line type is not set', $this->amendment_id));
			}

			parent::save();

			// Re-create paystub, if it already exists
			if(!is_null($this->data['pay_period_id']) && $recreate_stub) {
				$stub = new Paystub($this->data['staff_id'], $this->data['pay_period_id']);
				$stub->delete();
				$stub = new Paystub($this->data['staff_id'], $this->data['pay_period_id']);
				$stub->save();
			}
		}

		public function data2db() {
			if(is_null($this->data['pay_period_id']) || (int)$this->data['pay_period_id'] == 0) {
				$this->data['pay_period_id'] = '';
			}
			return $this->data;
		}
		public function db2data() {
			if($this->data['pay_period_id'] == '' || !$this->data['pay_period_id']) {
				$this->data['pay_period_id'] = null;
			}
			return $this->data;
		}

		public function delete() {
			return parent::delete(array('amendment_id' => $this->amendment_id));
		}

		public function isEditable() {
			if(is_null($this->pay_period_id)) {
				return true;
			} else {
				$period = new PayPeriod($this->pay_period_id);
				return $period->isEditable();
			}
		}
		
	}
?>
