<?php

	abstract class Calculation {

		/**
		*	Get the human readable name for this calculation
		*	@return String
		*/
		public static abstract function getName();

		/**
		*	Perform the calculation to determine the amount for the PaystubLine.
		*	
		*	The calculation should not set the amount, but return it.
		*	If needed, the calculation can go through the PaystubLine, get the 
		*	Paystub and get the Staff person.  Calculations can then be done
		*	on all sorts of fun things.
		*
		*	@param line Paystub line
		*	@param value The value for this DeductionType, or for the staff person if
		*		overriden on a per person level.
		*
		*	@return double
		*/
		public static abstract function calculate(PaystubLine &$line, $value);

		/**
		*	Get the name of the eTemplate used for setting the value
		*/
		public static function getTemplate() {
			return 'perp_payroll.DeductionValue.value';
		}

		/**
		*	Get the name of the eTemplate used for setting per-staff
		*	override values.
		*/
		public static function getOverrideTemplate() {
			return self::getTemplate();
		}

		/**
		*	Get selectbox options for the etemplate
		*	@return array
		*/
		public static function getOptions() {
			return array();
		}

		/**
		*	Get a total of lines off a paystub
		*
		*	@param line The paystub line that was passed into the calculation.
		*	@param line_types A comma seperated list of type_ids to total.
		*
		*	@return float total of line amounts for the given types
		*/
		public static function total_lines(&$line, $line_types) {
			$total = 0;
			$lines = $line->get_paystub()->get_lines(explode(',',$line_types));
			foreach($lines as $_line) {
				$total = bcadd($total, $_line->get_amount());
			}
			return $total;
		}
	}
?>
