<?php

	/**
	*
	*	Calculations can be grouped into sub-directories and using a : as a seperator.  
	*	The name to use is then:
	*		dir_name:__CLASS__
	*
	*	The function to use is probably calculate().
	*/
	class CalculationFactory {
		
		public static function getList() {
			$list = array();
			return CalculationFactory::_getList($list, dir(EGW_INCLUDE_ROOT . '/perp_payroll/inc/calculation/'));
		}

		private static function _getList(&$list, $path, $prefix = '') {
			while(($file = $path->read()) !== false) {
				if($file == '.' || $file == '..' || strpos($file, '.') === 0) {
					continue;
				}

				if(is_dir($path->path.$file) == true) {
//					$list[$file] = array();
					CalculationFactory::_getList($list, dir($path->path.$file), $file . ':');
				} else {
					include_once($path->path . DIRECTORY_SEPARATOR .$file);
					$classname = basename($file, '.inc.php');
					$list[$prefix . $classname] = call_user_func(array($classname, 'getName'));
				}
			}
			$path->close();
			return $list;
		}

		/**
		*	Extract the classname from a given $name.
		*	If the name contains a :, we'll look in a subdirectory.
		*/
		private static function getClassname($name) {
			$path = explode(':', $name);
			return array_pop($path);
		}

		/**
		*	Include a class
		*/
		private static function includeClass($name) {
			if(trim($name) == '') {
				return false;
			}
			$path = explode(':', $name);
			$path = 'calculation' . DIRECTORY_SEPARATOR . implode(DIRECTORY_SEPARATOR, $path) . '.inc.php';
			return include_once($path);
		}

		/**
		*	Factory for getting calculation objects to calculate PaystubLine amounts.
		*
		*	Using the calculation name (__CLASS__, not the human readable name), the appropriate calculation 
		*	is returned.  
		*	
		*	@param name Name of the calculation you want
		*	@return Calculation
		*/
		public static function getCalculation($name) {
			$classname = self::getClassname($name);

			if(self::includeClass($name) ) {
				return new $classname;
			}
		}

		/**
		*	Perform the given calculation and return the result.
		*
		*	@param name The name of the calculation
		*	@param line PaystubLine the calculation is done for
		*	@param value Value for the calculation
		*
		*	@return double The result of the calculation, the amount for the line
		*/
		public static function calculate($name, PaystubLine &$line, $value) {
			$classname = self::getClassname($name);
			
			if(self::includeClass($name)) {
				return call_user_func(array($classname, 'calculate'), $line, $value);
			} else {
				throw new Exception(lang('Unable to find calculation %1' ,$name));
			}
		}

	}
?>
