<?php

	/**
	*	An entry in the Payroll ledger
	*
	*	PayrollTransactionLine encapsulates a single entry into the Payroll Ledger.
	*	It must be linked to a perp_ledger::TransactionLine in a perp_leger::GLTransaction.
	*
	*	@author Nathan Gray
	*	@ingroup perp_payroll
	*/

	require_once(EGW_SERVER_ROOT . '/perp_ledger/inc/class.GLTransaction.inc.php');

	class PayrollTransactionLine extends TransactionLine {

		/**
		*	PayStubEntryType
		*/
		protected $type_id; 

		/**
		*	Pay period
		*/
		protected $pay_period_id;

		/**
		*	Employee
		*/
		protected $staff_id;

		/**
		*	Pay stub entry number
		*/
		protected $entry_id;

		
		public function __construct(TransactionLine &$parent, PayStubLine &$paystub_line) {
			$this->transaction = $parent->transaction;
			if(!$this->transaction->get_transaction_number()) {
				$this->set_amount($paystub_line->get_amount());
				$this->type_id = $paystub_line->get_type_id();
				$this->pay_period_id = $paystub_line->get_paystub()->get_pay_period_id();
				$this->staff_id = $paystub_line->get_paystub()->get_staff_id();
				$this->entry_id = $paystub_line->get_entry_id();
				$parent->add_associated_line($this);
			}
			if($this->transaction->get_transaction_number()) {
				return $this->read($this->transaction->get_transaction_number(), $line_no);
			}
		}

		/**
		*	Can't really read them back in the same way because of 1:n relationship
		*/
		public function read() {}

		public function write() {
			$GLOBALS['egw']->db->insert(
				'perp_payroll_ledger',
				array(
					'transaction_id'	=>	$this->transaction->get_transaction_number(),
					'staff_id'		=>	$this->staff_id,
					'type_id'		=>	$this->type_id,
					'date'			=>	$this->transaction->get_date(),
					'amount'		=>	$this->amount,
					'entry_id'		=>	$this->entry_id,
				),
				array(),
				__LINE__, __FILE__, 'perp_payroll'
			);
		}

		public function __sleep() {
			return parent::sleep() + array(
				'type_id', 'staff_id', 'entry_id'
			);
		}

		/**
		*       Return a string representation of this line.
		*       Very useful for debugging purposes, not much else.
		*
		*       @return string Representation of this TransactionLine
		*/
		public function __toString() {
			return "PayrollTransactionLine \tStaff_id: " . $this->staff_id . "\tType_id: " . $this->type_id . "\tAmount: " . $this->amount . ")\n";
		}
	}
?>
