<?php

	/**
	*	Class for dealing with payment to employees
	*/

	require_once(EGW_INCLUDE_ROOT . '/perp_human_resources/inc/class.bo_perp_staff.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_api/inc/class.bo_perp_api.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/timesheet/inc/class.timesheet_bo.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.PaystubLine.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.EarningType.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.DeductionType.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.AccrualType.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.TotalType.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.CalculationFactory.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Amendment.inc.php');


	class Paystub {


		/**
		*	Frequency payments are made
		*/
		public static $pay_frequency = array(
			52	=>	'Weekly',
			26	=>	'Bi-Weekly',
			24	=>	'twice per Month',
			12	=>	'Monthly',
			6	=>	'every 2nd Month',
			4	=>	'Quarterly',
			2	=>	'Bi-Annually',
			1	=>	'Annually',
		);

		/**
		*	Pay units
		*	Pay type is measured in these units
		*/
		public static $pay_unit = array(
			'h'	=>	'hour',
			'm'	=>	'month',
			'y'	=>	'year'
		);

		/**
		*	The ID of the staff person this paystub is for
		*/
		protected $staff_id;
		protected $staff;

		/**
		*	The pay period id.  Together with the staff_id, this 
		*	forms the unique identifier for the paystub.
		*/
		protected $pay_period_id;
		protected $pay_period = null;

		/**
		*	The payment schedule this paystub is for
		*/
		protected $payment_schedule_id;

		/**
		*	Currency 
		*/
		protected $currency;

		/**
		*	Exchange rate is updated when the transactions are posted
		*	to the exchange rate at post time.
		*/
		protected $exchange_rate = 1;

		/**
		*	Paystub entry lines
		*/
		protected $paystub_lines = array();

		/**
		*	Notes on the paystub
		*/
		protected $notes = array();

		protected $processed = false;

		/**
		*	Data access objects for getting information from the system
		*	@{
		*/

		public static $bo_staff;

		public static $bo_timesheet;

		/**
		*	@}
		*/

		/**
		*	Construct a Paystub
		*	Create a new one, or read an already existing one.
		*/
		public function __construct($staff_id, &$pay_period_id = null, $currency = null, $exchange_rate = null, $create = false) {
			
			$this->staff_id	=	(int)$staff_id;

			if($currency == null) {
				$config = config::read('perp_ledger');
				$currency = $config['gl_currency'];
			}
			$this->currency = $currency;
			if($exchange_rate == null) {
				$bo_api = new bo_perp_api();
				$bo_api->get_currencies(array('currency_id' => $this->currency, 'template'=>false), $currency);
				$currency = $currency[0];
			}
			$this->exchange_rate = $currency['exchange_rate'];
			if(is_numeric($this->staff_id) && !is_null($pay_period_id)) {
				if($pay_period_id instanceof Payperiod) {
					$this->pay_period =& $pay_period_id;
					$this->pay_period_id = $this->pay_period->pay_period_id;
				} else {
					$this->pay_period_id	=	(int)$pay_period_id;
					$this->pay_period = new Payperiod($this->pay_period_id);
				}

				// Try for an existing pay period
				if($create || !$this->read()) {
					$this->build();
				}
			}
		}

		protected function build() {
			if(!$this->staff) {
				$this->staff = new Staff($this->staff_id);
			}
			$types = $this->get_line_types();
			foreach($types as $t) {
				$type = PayStubEntryType::factory($t['type_id']);
				if(strpos($t['type'], 'Amendment') === 0) {
					// Add in amendment lines, and update amendment to this pay period

					list(,$amendment_id) = explode(':', $t['type']);
					if($amendment_id) {
						$amendment = new Amendment($amendment_id);
						$line =& new PaystubLine($this, $amendment->quantity, $amendment->rate, $type->get_id());
						if($amendment->description != '') {
							$line->add_note(null, $amendment->description);
						}
						$this->add_line($line);

						$amendment->pay_period_id = $this->pay_period_id;
						$amendment->save(false);
					}
					unset($amendment);
					unset($line);
				} elseif ($type->get_calculation_name()) {
					$line =& new PaystubLine($this, 1, 0, $type->get_id());
					$line->set_rate(CalculationFactory::calculate($type->get_calculation_name(), $line, $type->get_value($this->staff_id)));
					$this->add_line($line);
					$total[$t['type']] += $line->get_amount();
				} else {
					$build_function = 'build_' . strtolower($t['type']);
					$total[$t['type']] += $this->$build_function($type);
				}

				$count[$t['type']]++;
			}

			// Sanity check
			if($count['Earning'] <= 0 || $total['Earning'] == 0) {
				$address = ExecMethod('addressbook.addressbook_bo.read', $this->staff->get_addressbook_id());
				$GLOBALS['egw']->log->error(array(
					'text'	=>	'W-Paystub, 0 earnings for %1. ',
					'p_1'	=>	$address['n_fn'],
					'file'	=>	__FILE__,
					'line'	=>	__LINE__
				));
			}
		}

		protected function get_line_types() {
			if(date('H:i', $this->get_pay_period()->end_date) == '00:00' &&
				date('H:i', $this->get_pay_period()->start_date) == '00:00') {
				$end_date = strtotime('+1 day', $this->get_pay_period()->end_date) - 1;
			} else {
				$end_date = $this->get_pay_period()->end_date;
			}

			// Get a list of line types
			$type_select = array(
				'table'	=>	'perp_pay_stub_entry_type',
				'cols'	=>	array(
					'perp_pay_stub_entry_type.type_id',
					"CASE
						WHEN perp_earning_type.type_id IS NOT NULL THEN 'Earning'
						WHEN perp_deduction_type.type_id IS NOT NULL THEN 'Deduction'
						WHEN perp_accrual_type.type_id IS NOT NULL THEN 'Accrual'
						WHEN perp_pay_stub_entry_type.calculation = 'Total' THEN 'Total'
					END AS type",
					"CASE
						WHEN perp_staff_pay_detail.type_id IS NOT NULL THEN perp_staff_pay_detail.staff_id
						WHEN perp_staff_entry_type.type_id IS NOT NULL THEN perp_staff_entry_type.staff_id
						WHEN perp_staff_accrual.type_id IS NOT NULL THEN perp_staff_accrual.staff_id
					END AS staff_id",
					'sort_order'
				),
				'where'	=>	array(),
				'append'=>	'HAVING staff_id = ' . $this->staff_id . " OR (staff_id IS NULL AND type = 'Total')",
				'app'	=>	'perp_payroll',
				'join'	=>'LEFT JOIN (
					perp_earning_type  
					JOIN perp_staff_pay_detail ON 
						perp_staff_pay_detail.type_id = perp_earning_type.type_id
				) ON
					perp_earning_type.type_id = perp_pay_stub_entry_type.type_id
				LEFT JOIN (
					perp_deduction_type  
					JOIN perp_staff_entry_type ON 
						perp_staff_entry_type.type_id = perp_deduction_type.type_id
					) ON
					perp_deduction_type.type_id = perp_pay_stub_entry_type.type_id
				LEFT JOIN (
					perp_accrual_type  
					JOIN perp_staff_accrual ON 
						perp_staff_accrual.type_id = perp_accrual_type.type_id AND
						perp_staff_accrual.accrues = true
					) ON
					perp_accrual_type.type_id = perp_pay_stub_entry_type.type_id'
			);

			$timesheet_select = array(
				'table'	=>	'egw_timesheet',
				'cols'	=>	array(
					'perp_earning_type.type_id AS type_id',
					'\'Earning\' AS type',
					// Chop off 'STAFF-' prefix
					'SUBSTRING(ts_extra_value, 7) AS staff_id',
					'sort_order',
				),
				'where'	=>	array(
					'ts_extra_value'	=>	'STAFF-' . $this->staff_id,
					'ts_start >= ' . $this->pay_period->start_date,
					'ts_start < ' . $end_date,
				),
				'append'=>	'GROUP BY type_id, staff_id, cat_id, ts_unitprice',
				'app'	=>	'egw_timesheet',
				'join'	=>	'JOIN egw_timesheet_extra ON
					egw_timesheet_extra.ts_id = egw_timesheet.ts_id AND
					egw_timesheet_extra.ts_extra_name = \'staff_id\' 
				JOIN perp_earning_type ON 
					egw_timesheet.cat_id = perp_earning_type.timesheet_category_id 
				JOIN perp_pay_stub_entry_type ON 
					perp_earning_type.type_id = perp_pay_stub_entry_type.type_id'
			);

			$amendment_select = array(
				'table'	=>	'perp_pay_amendment',
				'cols'	=>	array(
					'perp_pay_amendment.type_id',
					'CONCAT(\'Amendment:\', amendment_id) as type',
					'staff_id',
					'sort_order'
				),
				'where'	=>	array(
					'staff_id'	=>	$this->staff_id,
					'(pay_period_id = ' . (int)$this->pay_period_id . ' OR pay_period_id IS NULL)'
				),
				'append'	=>	'',
				'app'	=>	'perp_payroll',
				'join'	=>	'JOIN perp_pay_stub_entry_type ON 
				    perp_pay_amendment.type_id = perp_pay_stub_entry_type.type_id',
			);
			$GLOBALS['egw']->db->union(array($type_select, $timesheet_select, $amendment_select), __LINE__, __FILE__, 'ORDER BY sort_order', 0, -1);

			$types = array();
			while($row = $GLOBALS['egw']->db->row(true)) {
				$types[] = $row;
			}
			return $types;
		}

		/**
		*	Add an accrual line to the paystub
		*/
		protected function build_accrual(AccrualType $type) {
			$line = new PaystubLine($this, 1, 0, $type->get_id());
			$line->set_rate($type->get_amount($line, $this->staff));
			$this->add_line($line);
		}

		/**
		*	Add deductions to this pay stub for the given pay period
		*/
		protected function build_deduction($deduction) {
		/*
			$line = new PaystubLine($this, 1, 0, $deduction->get_id());
			$line->set_rate(CalculationFactory::calculate($deduction->get_calculation_name(), $line, $deduction->get_value($this->staff_id)));
			$this->add_line($line);
		*/
		}

		/**
		*	Add totals to this pay stub for the given pay period
		*/
		protected function build_total($type) {
	/*
			$line = new PaystubLine($this, 1, 0, $type->get_id());
			$line->set_rate(CalculationFactory::calculate($type->get_calculation_name(), $line, $type->get_value($this->staff_id)));
			$this->add_line($line);
	*/
		}

		/**
		*	Update Year to date amounts for all lines
		*/
		public function update_YTD() {
			foreach($this->paystub_lines as &$line) {
				PayStubEntryType::add_YTD($line->get_type_id(), $this->pay_period_id, $this->staff_id, $line->get_quantity(), $line->get_amount());
			}
		}

		/**
		*	Long term storage methods
		*	@{
		*/

		/**
		*	Read this paystub from the database using staff_id and period_id
		*/
		public function read() {
		
			$row = $GLOBALS['egw']->db->select(
				'perp_pay_stub',
				array('*'),
				array(
					'pay_period_id'	=>	$this->pay_period_id,
					'staff_id'	=>	$this->staff_id
				),
				__LINE__, __FILE__, false
			)->fetchRow();

			if(is_array($row)) {
				$this->currency = $row['currency'];
				$this->exchange_rate = $row['exchange_rate'];

				// Get lines
				$GLOBALS['egw']->db->select(
					'perp_pay_stub_entry',
					array('*'),
					array(
						'pay_period_id'	=>	$this->pay_period_id,
						'staff_id'	=>	$this->staff_id
					),
					__LINE__, __FILE__, false,
					'ORDER BY entry_id', 'perp_payroll', 0
				);
				$lines = array();
				while($row = $GLOBALS['egw']->db->row(true)) {
					$lines[$row['entry_id']] = $row;
				}

				// Get notes
				$GLOBALS['egw']->db->select(
					'perp_pay_stub_note',
					array(
						'entry_id',
						'note'
					),
					array(
						'pay_period_id'	=>	$this->pay_period_id,
						'staff_id'	=>	$this->staff_id
					),
					__LINE__, __FILE__, false,
					'ORDER BY entry_id', 'perp_payroll'
				);

				$this->notes = array();
				$note_count = 0;
				while($row = $GLOBALS['egw']->db->row(true)) {
					if($row['entry_id'] == null) {
						$this->notes[] = $row['note'];
					} elseif (isset($lines[$row['entry_id']])) {
						$lines[$row['entry_id']]['notes'][++$note_count] = $row['note'];
					}
				}

				// This is done in a seperate loop because we need to finish the other query before creating the stub.
				foreach($lines as $row) {
					$line =& new PaystubLine($this, $row['quantity'], $row['rate'], $row['type']);
					$line->set_entry_id($row['entry_id']);

					// Can't use add_line() after it's posted
					$this->paystub_lines[$row['entry_id']] = $line;

					if(is_array($row['notes'])) {
						foreach($row['notes'] as $ref => $note) {
							$line->add_note($ref, $note);
						}
					}
				}
				return true;
			} elseif (!$result) {
				return false;
			}
		}

		/**
		*	Write this paystub to the database using staff_id and pay_period_id
		*/
		public function save() {
			if(!$this->isEditable()) {
				return false;
			}
			$result =& $GLOBALS['egw']->db->insert(
				'perp_pay_stub',
				array(
					'pay_period_id'	=>	$this->pay_period_id,
					'staff_id'	=>	$this->staff_id,
					'currency'	=>	$this->currency,
					'exchange_rate'	=>	$this->exchange_rate
				),
				array(
					'pay_period_id'	=>	$this->pay_period_id,
					'staff_id'	=>	$this->staff_id,
				),
				__LINE__, __FILE__, 'perp_payroll'
			);
			if($result === false) {
				throw new Exception(lang('Unable to save paystub'));
			}
			$GLOBALS['egw']->db->delete(
				'perp_pay_stub_note',
				array(
					'pay_period_id'	=>	$this->get_pay_period_id(),
					'staff_id'	=>	$this->get_staff_id(),
					'entry_id'	=>	null
				),
				__LINE__, __FILE__,
				'perp_payroll'
			);

			foreach($this->notes as $note) {
				$GLOBALS['egw']->db->insert(
					'perp_pay_stub_note',
					array(
						'pay_period_id'	=>	$this->get_pay_period_id(),
						'staff_id'	=>	$this->get_staff_id(),
						'entry_id'	=>	null,
						'note'		=>	$note
					),
					array(
						'pay_period_id'	=>	$this->get_pay_period_id(),
						'staff_id'	=>	$this->get_staff_id(),
						'entry_id'	=>	null,
						'note'		=>	$note
					),
					__LINE__, __FILE__, 'perp_payroll'
				);
			}

			foreach($this->paystub_lines as $num => $line) {
				$line->save($num);
			}
		}

		public function delete() {
			if(!$this->isEditable()) {
				return false;
			}
			$GLOBALS['egw']->db->delete(
				'perp_pay_stub',
				array(
					'pay_period_id' =>      $this->pay_period_id,
					'staff_id'	=>	$this->staff_id
				),
				__LINE__,__FILE__
			);
			$GLOBALS['egw']->db->delete(
				'perp_pay_stub_entry',
				array(
					'pay_period_id' =>      $this->pay_period_id,
					'staff_id'	=>	$this->staff_id
				),
				__LINE__,__FILE__
			);
			$GLOBALS['egw']->db->delete(
				'perp_pay_stub_note',
				array(
					'pay_period_id' =>      $this->pay_period_id,
					'staff_id'	=>	$this->staff_id,
					'entry_id IS NOT NULL'
				),
				__LINE__, __FILE__
			);
			return true;
		}

		/**
		*	@}
		*/

		/**
		*	Access and permissions
		*	@{
		*/

		/**
		*	Can this paystub be edited?
		*	The answer is based on permissions of the current user and 
		*	whether or not the paystub has already been processed for 
		*	payment.
		*
		*	@return boolean 
		*/
		public function isEditable() {
			return $this->pay_period->isEditable();
		}

		/**
		*	Has this paystub been processed for payment?
		*	This means that either the check has been printed, or a electronic transfer made.
		*/
		public function is_processed() {
			throw new Exception(__METHOD__ . ' is not implemented yet (' . __FILE__ . ':' . __LINE__ .')');

		}

		/**
		*	@}
		*/

		/**
		*	Accessors
		*	@{
		*/
		public function get_staff_id() {
			return $this->staff_id;
		}

		public function set_staff_id($id) {
			if($this->is_editable() ) {
				if(count($this->bo_staff->get_staff(array('staff_id' => $id))) == 1 ) {
					$this->staff_id = $id;
					return true;
				} else {
					throw new Exception(lang('%1 is an invalid staff ID', $id));
				}
			} else {
				throw new Exception(lang('Unable to edit paystub'));
			}
			return false;
		}

		public function &get_staff() {
			if(!$this->staff) {
				$this->staff =& new Staff($this->get_staff_id());
			}
			return $this->staff;
		}

		public function get_pay_period_id() {
			return $this->pay_period_id;
		}

		/**
		*	Set the pay period ID for this paystub
		*/
		public function set_pay_period_id($id) {
			if(!is_numeric($id)) {
				throw new Exception(lang('%1 is an invalid pay period', $id));
			}
			if(!$this->is_editable()) {
				throw new Exception(lang('Unable to edit paystub'));
			}
			$this->pay_period_id = (int)$id;
		}

		public function &get_pay_period() {
			return $this->pay_period;
		}

		public function get_currency() {
			return $this->currency;
		}

		public function get_payment_schedule_id() {
			return $this->payment_schedule_id;
		}

		public function set_payment_schedule_id($id) {
			throw new Exception(__METHOD__ . ' is not implemented yet (' . __FILE__ . ':' . __LINE__ .')');
		}

		public function add_note($note) {
			$this->notes[] = $note;
			return true;
		}

		public function remove_note($reference) {
			unset($this->notes[$reference]);
		}

		/**
		*	Return a reference to all lines on the Paystub
		*/
		public function &get_lines($type = null) {
			if($type) {
				$result = array();
				if(!is_array($type)) {
					$type = array($type);
				}
				foreach($this->paystub_lines as &$line) {
					if(in_array($line->get_type_id(), $type)) {
						$result[] =& $line;
					}
				}
				return $result;
			}
			return $this->paystub_lines;
		}
		/**
		*	Get one line by line number
		*/
		public function &get_line($line_number) {
			return $this->paystub_lines[$line_number];
		}

	
		/**
		*	Add one line
		*/
		public function &add_line(PaystubLine &$line) {
			if($this->isEditable()) {
				if($line->get_entry_id() == null) {
					if(count($this->paystub_lines) > 0) {
						$line->set_entry_id(max(array_keys($this->paystub_lines))+1);
					} else {
						$line->set_entry_id(0);
					}
				}
				$this->paystub_lines[$line->get_entry_id()] =& $line;
				return $line;
			} else {
				throw new Exception(lang('Unable to edit paystub'));
			}
		}

		/**
		*	Remove one line by line number
		*
		*	@param line_number integer ID of the line number
		*
		*	@return PaystubLine removed line
		*/
		public function &remove_line($line_number) {
			if($this->is_editable()) {
				$line =& $this->paystub_lines[$line_number];
				$line->remove();
				unset($this->paystub_lines[$line_number]);
				return $line;
			} else {
				throw new Exception(lang('Unable to edit paystub'));
			}
		}

		/**
		*	Get the line for net pay.  This must be the last line on the stub.
		*	@return PaystubLine net pay line
		*/
		public function &get_net_pay_line() {
			return end($this->paystub_lines);
		}

		/**
		*	Get the net pay amount
		*/
		public function get_net_pay_amount() {
			if($this->get_net_pay_line() instanceof PaystubLine) {
				return $this->get_net_pay_line()->get_amount();
			} else {
				return 0;
			}
		}

		/**
		*	@}
		*/

		/**
		*	Utility and debugging functions
		*	@{
		*/
		public function __sleep() {
			throw new Exception(__METHOD__ . ' is not implemented yet (' . __FILE__ . ':' . __LINE__ .')');
		}
		public function toArray() {
			$data = array(
				'pay_period_id'	=>	$this->pay_period_id,
				'staff_id'	=>	$this->staff_id,
				'staff_name'	=>	ExecMethod('perp_human_resources.bo_perp_staff.link_title', $this->staff_id),
				'net_pay'	=>	$this->get_net_pay_amount(),
				'notes'		=>	$this->notes,
			);
			return $data;
		}
		public function __toString() {
			$string = 'Paystub for ' . ExecMethod('perp_human_resources.bo_perp_staff.link_title', $this->staff_id) . ' (';
			$string .= date('Y-m-d', $this->pay_period->start_date) . ' -> ' . date('Y-m-d', $this->pay_period->end_date) . ")\n<br >";
			$string .= sprintf("<pre>%-50s %8s %8s\t%12s\n", lang('Name'), lang('Rate'), lang('quantity'), lang('amount'));

			foreach($this->paystub_lines as $key => $line) {
				$string .=  $line->__toString();
			}
			return $string . "</pre>\n<br />";
		}
		/**
		*	@}
		*/
	}

	Paystub::$bo_staff = new bo_perp_staff();
	Paystub::$bo_timesheet = new timesheet_bo();
?>
