<?php

	/**
	*	One line on a pay stub
	*
	*	This represents one part of a specific paystub.  It can be a normal earning, an 
	*	Accrual or Deduction.  Each line has a PaystubLineType, which defines
	*	most of its common properties.
	*
	*	@package perp_payroll
	*	@author Nathan Gray
	*	@license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.PaystubEntryType.inc.php' );

	class PaystubLine {
		
		/**
		*	The paystub that this line is part of
		*/
		private $paystub = null;

		/**
		*	Line variables
		*/
		private $entry_id	=	null;

		private $quantity	=	1;
		private $rate		=	0;

		private $type_id	=	null;
		private $type		=	null;

		protected $notes	=	array();

		public function __construct(Paystub $paystub, $quantity, $rate, $type_id) {
			$this->paystub = $paystub;
			$this->quantity = $quantity;
			$this->rate = $rate;
			
			if($type_id == null || (int)$type_id == 0) {
				throw new Exception(lang('Could not find entry type with ID %1', $type_id));
			}
			$this->type_id = $type_id;
			$this->type = PayStubEntryType::factory($type_id);
			if($this->type == null) {
				throw new Exception(lang('Could not find entry type with ID %1', $type_id));
			}
		}

		/**
		*	Get the paystub this line is part of
		*/
		public function &get_paystub() {
			return $this->paystub;
		}

		public function get_rate() {
			return $this->rate;
		}

		public function set_rate($rate) {
			if($this->paystub->isEditable()) {
				$this->rate = (double)$rate;
			}
		}

		public function get_quantity() {
			return $this->quantity;
		}

		public function set_quantity($qty) {
			if($this->paystub->isEditable()) {
				$this->quantity = $qty;
			}
		}

		public function get_amount() {
			return $this->quantity * $this->rate;
		}

		public function get_type_id() {
			return $this->type_id;
		}

		public function &get_type() {
			return $this->type;
		}

		public function add_note($ref, $note) {
			if(!(int)$ref || is_null($ref)) {
				$ref = count($this->notes) > 0 ? max(array_keys($this->notes)) + 1 : 1;
			}
			$this->notes[$ref] = $note;
		}

		public function remove_note($ref) {
			unset($this->notes[$ref]);
		}

		public function set_entry_id($id) {
			$this->entry_id = $id;
		}
		public function get_entry_id() {
			return $this->entry_id;
		}

		/**
		*	Long term storage
		*/
		public function save() {
			$GLOBALS['egw']->db->insert(
				'perp_pay_stub_entry',
				array(
					'pay_period_id'	=>	$this->paystub->get_pay_period_id(),
					'staff_id'	=>	$this->paystub->get_staff_id(),
					'entry_id'	=>	$this->entry_id,
					'quantity'	=>	$this->quantity,
					'rate'		=>	$this->rate,
					'type'		=>	$this->type_id
				),
				array(
					'pay_period_id'	=>	$this->paystub->get_pay_period_id(),
					'staff_id'	=>	$this->paystub->get_staff_id(),
					'entry_id'	=>	$num
				),
				__LINE__, __FILE__, 'perp_payroll'
			);

			$GLOBALS['egw']->db->delete(
				'perp_pay_stub_note',
				array(
					'pay_period_id'	=>	$this->paystub->get_pay_period_id(),
					'staff_id'	=>	$this->paystub->get_staff_id(),
					'entry_id'	=>	(int)$this->entry_id,
				),
				__LINE__, __FILE__,
				'perp_payroll'
			);

			foreach($this->notes as $note) {
				$GLOBALS['egw']->db->insert(
					'perp_pay_stub_note',
					array(
						'pay_period_id'	=>	$this->paystub->get_pay_period_id(),
						'staff_id'	=>	$this->paystub->get_staff_id(),
						'entry_id'	=>	(int)$this->entry_id,
						'note'		=>	$note
					),
					array(
						'pay_period_id'	=>	$this->paystub->get_pay_period_id(),
						'staff_id'	=>	$this->paystub->get_staff_id(),
						'entry_id'	=>	(int)$this->entry_id,
						'note'		=>	$note
					),
					__LINE__, __FILE__, 'perp_payroll'
				);
			}
						
		}

		public function __toString() {
			$string =  sprintf("%-50s %8.2f %8.2f\t%12.2f\n",
				$this->type->get_name(),
				$this->rate,
				$this->quantity,
				$this->rate * $this->quantity
			);
			return $string . ($this->type instanceof TotalType ? "\n" : '');
		}

		public function toArray() {
			$data = array(
				'entry_id'	=>	$this->entry_id,
				'pay_period_id'	=>	$this->paystub->get_pay_period_id(),
				'staff_id'	=>	$this->paystub->get_staff_id(),
				'quantity'	=>	$this->quantity,
				'rate'		=>	$this->rate,
				'amount'	=>	$this->quantity * $this->rate,
				'type_id'	=>	$this->type_id,
				'type'		=>	get_class($this->type),
				'name'		=>	$this->type->get_name(),
				'notes'		=>	$this->notes,
			);
			$YTD = PayStubEntryType::get_YTD($this->type_id, $this->get_paystub()->get_pay_period_id(), $this->paystub->get_staff_id());
			if(is_array($YTD)) {
				$data += $YTD;
			} else {
				$data['ytd_amount'] = null;
				$data['ytd_quantity'] = null;
			}

			switch ($data['type']) {
				case 'DeductionType':
					$data['DeductionType'] = $this->type->get_type();
					break;
				case 'AccrualType':
					$data['balance'] = $this->type->get_balance($this->paystub->get_staff_id());
					break;
			}
			return $data;
		}
	}
?>
