<?php

	require_once('class.PaySchedule.inc.php');
	require_once('class.Paystub.inc.php');
	require_once('class.so_perp_payroll.inc.php');
	
	class bo_perp_payroll {

		/**
		*	Data access object
		*/
		protected $so;

		public function __construct() {
			$this->so = new so_perp_payroll();
		}

		/**
		*	Get a list of pay schedules
		*
		*	@nextmatch
		*/
		public function get_pay_schedules($query = array(), &$rows = array(), $readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('run', EGW_ACL_READ, 'perp_payroll')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!isset($query['order'])) {
				$query['order'] = 'schedule_id';
			}
			if(!$query['time']) {
				$query['time'] = time();
			}

			$count = PaySchedule::get_rows($query, $rows, $readonly);

			if(!isset($query['template'])) {
				// Reformat for selectbox & return
				$list = array();
				foreach($rows as $row) {
					$list[$row['schedule_id']] = $row['cal_title'];
				}
				return $list;
			}
			return $count;
		}

		/**
		*	Get a list of earning types (Regular, Overtime, ...)
		*
		*	@nextmatch
		*/
		public function get_earning_types($query = array(), &$rows=array(), &$readonly=array()) {
			if(!$GLOBALS['egw']->acl->check('earning_type', EGW_ACL_READ, 'perp_payroll')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!isset($query['template'])) {
				$query['num_rows'] = -1;
			}

			if(!$query['order']) {
				$query['order'] = 'perp_earning_type.type_id';
			}

			$list = $this->so->get_earning_types($query);
			if(!isset($query['template'])) {
				foreach($list as $item) {
					$rows[$item['type_id']] = $item['name'];
				}
				return $rows;
			}
			$rows = $list;
			return $query['count'];
		}

		/**
		*	Get a list of all line types
		*
		*	@nextmatch
		*/
		public function get_line_types($query = array(), &$rows = array(), &$readonlys=array()) {
			if(!$GLOBALS['egw']->acl->check('earning_type', EGW_ACL_READ, 'perp_payroll')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!isset($query['template'])) {
				$query['num_rows'] = -1;
			}
			if(!$query['order']) {
				$query['order'] = 'perp_pay_stub_entry_type.sort_order';
			}

			$list = array();
			PayStubEntryType::get_rows($query, $list, $readonlys);
			if(!isset($query['template'])) {
				foreach($list as $item) {
					$rows[$item['type_id']] = $item['name'];
				}
				return $rows;
			}
			$rows = $list;
			return $query['count'];
		}

		/**
		*	Get a list of pay periods
		*	
		*	@nextmatch
		*/
		public function get_pay_periods($query = array(), &$rows = array(), &$readonlys=array()) {
			if(!$GLOBALS['egw']->acl->check('pay_period', EGW_ACL_READ, 'perp_payroll')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}

			if(!$query['order']) {
				$query['order'] = 'perp_pay_period.pay_period_id';
				$query['sort'] = 'DESC';
			}

			$rows = $this->so->get_pay_periods($query);

			if(!isset($query['template'])) {
				$list = array();
				foreach($rows as $row) {
					$list[$row['pay_period_id']] = $row['pay_period_id'] . ': ' . 
					date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['start_date']) . ' - ' . 
					date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $row['end_date']);
				}
				return $list;
			} else {
				foreach($rows as $row) {
					$readonlys['delete[' . $row['pay_period_id'] . ']'] = $row['transaction_id'] && $GLOBALS['egw']->acl->check('pay_period', EGW_ACL_DELETE, 'perp_payroll');
					$readonlys['revert[' . $row['pay_period_id'] . ']'] = !is_null($row['reverted_id']) || !($readonlys['delete[' . $row['pay_period_id'] . ']'] && $GLOBALS['egw']->acl->check('pay_period', EGW_ACL_CUSTOM_1, 'perp_payroll'));
				}
			}
			return $query['count'];
		}

		/**
		*	Get a list of paystubs
		*	Additinal keys in the query can be pay_period_id and staff_id
		*/
		public function get_paystubs($query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('pay_stub', EGW_ACL_READ, 'perp_payroll')) {
			// Give staff a list of their own stubs
				$query['staff_id'] = $GLOBALS['egw_info']['user']['account_id'];
				if(!$query['staff_id']) {
					if(isset($query['template'])) {
						$rows = array();
						return 0;
					} else {
						return array();
					}
				}
			}

			if(!$query['order']) {
				$query['order'] = 'perp_pay_stub.pay_period_id';
			}
			$list = $this->so->get_paystubs($query);

			if(!isset($query['template'])) {
				$rows = array();
				if(isset($query['pay_period_id'])) {
					foreach($list as $stub) {
						$rows[ $stub['staff_id'] ] = $stub['start_date'] . ' -> ' . $stub['end_date'];
					}
				} elseif(isset($query['staff_id'])) {
					foreach($list as $stub) {
						$rows[ $stub['pay_period_id'] ] = ExecMethod('perp_human_resources.bo_perp_staff.link_title', $stub['staff_id']);
					}
				}
				return $rows;
			} else {
				$rows = $list;
			}
			return $query['count'];
		}

		/**
		*	Get a list of pay stub amendments
		*	Extra query keys: pay_period_id & staff_id
		*	filter => pay period ID
		*	filter2 => staff ID
		*/
		public function get_pay_amendments(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('amendment', EGW_ACL_READ, 'perp_payroll')) {
				if(isset($query['template'])) {
					$rows = array();
					return 0;
				} else {
					return array();
				}
			}
			if(!$query['order']) {
				$query['order'] = 'amendment_id';
			}
			if(isset($query['filter'])) {
				$query['pay_period_id'] = $query['filter'];
			}
			if(isset($query['filter2'])) {
				$query['staff_id'] = $query['filter2'];
			}
			if(is_array($query['col_filter'])) {
				foreach($query['col_filter'] as $name => $value) {
					$query[$name] = $value;
				}
			}

			$rows = $this->so->get_pay_amendments($query);

			$edit = $GLOBALS['egw']->acl->check('amendment', EGW_ACL_EDIT, 'perp_payroll');
			$delete = $GLOBALS['egw']->acl->check('amendment', EGW_ACL_DELETE, 'perp_payroll');
			foreach($rows as $key => $row) {
				if($edit || $delete) {
					$amendment = new Amendment($row['amendment_id']);
					$editable = $amendment->isEditable();
				}
				if(is_null($amendment) || !$editable) {
					$readonlys['edit[' . $row['amendment_id'] .']'] = !$edit || !$editable;
					$readonlys['delete[' . $row['amendment_id'] .']'] = !$delete || !$editable;
				}
			}
			return $query['count'];
		}

		/**
		*	Get a staff member's payroll information
		*
		*	Staff are allowed to see their own info.
		*	@param staff Staff object for staff member
		*
		*	@return array of payroll information
		*/
		public function get_staff_data(Staff $staff) {
			if(!$GLOBALS['egw']->acl->check('staff_data', EGW_ACL_READ, 'perp_payroll') &&
				$GLOBALS['egw_info']['user']['account_id'] != $staff->get_staff_id()) {

				return array();
			}
			// Pay information
			$data = $this->so->get_staff_data($staff->get_staff_id());
			return $data;
		}

		/**
		*	Set a staff member's payroll information.
		*	@param staff Staff object for a staff member
		*	@param data associative array of payroll infomation
		*	@return mixed true or error
		*/
		public function set_staff_data(Staff $staff, $data) {
			if(!$GLOBALS['egw']->acl->check('staff_data', EGW_ACL_EDIT, 'perp_payroll')) {
				return false;
			}
			if(!array_key_exists($data['type_id'], $this->get_earning_types())) {
				return lang('Invalid pay type');
			}
			if(!array_key_exists($data['pay_unit'], Paystub::$pay_unit)) {
				return lang('Invalid unit for pay rate');
			}
			
			if(!$data['staff_id']) {
				$data['staff_id'] = $staff->get_staff_id();
			}
			$earning = new EarningType($data['type_id']);
			$earning->set_staff(array($data));
		}

		/**
		*	Create a payperiod.  
		*
		*	This is pretty much just a wrapper so it can be called by async
		*
		*	@param data Array with the keys 'schedule_id' and 'start'
		*/
		public function make_payperiod(array $data) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_payroll';
			$GLOBALS['egw']->log->error(array(
				'text'	=>	'D-Payperiod, Starting to create pay period for schedule %1, %2 - %3', 
				'p_1'	=>	$data['schedule_id'],
				'p_2'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $data['start']),
				'p_3'	=>	date($GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], $data['end']),
				'file'	=>	__FILE__,
				'line'	=>	__LINE__
			));
			$GLOBALS['egw']->log->commit();
			$schedule = null;
	
			try {
				$schedule = new PaySchedule($data['schedule_id']);
				if(!$schedule) {
					throw new Exception(lang('Unable to find pay schedule'));
				}
				$id = $schedule->create_pay_period($data['start']);
				$GLOBALS['egw']->log->error(array('text'=>'I-Payperiod, Pay period %2 was created for pay schedule %1', 
					'p_1' => $schedule->get_name(),
					'p_2' => $id,
					'file'=>__FILE__,'line'=>__LINE__
				));
			} catch (Exception $e) {
				$GLOBALS['egw']->log->error(array('text'=>'E-Payperiod, %1','p_1'=>$e->getMessage(),'file'=>$e->getFile(),'line'=>$e->getLine()));
				$GLOBALS['egw']->log->error(array('text'=>'E-Payperiod, Pay period %1 was not generated!', 'p_1' => (!is_null($schedule) ? $schedule->get_name() : $data['schedule_id']), 'file'=>$e->getFile(),'line'=>$e->getLine()));
			}
			$GLOBALS['egw']->log->commit();
		}

		/**
		*	Create paystubs for a given pay period
		*	This is a wrapper so it can be called by async, but it also listens
		*	for exceptions and logs them to the error log.
		*
		*	@param data Array with the keys pay_schedule_id and pay_period_id
		*/
		public function create_paystubs(array $data) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_payroll';
			$GLOBALS['egw']->log->error(array('text'=>'D-Paystub, Starting to create paystubs for %1', 'p_1' => $data['pay_period_id'],'file'=>__FILE__,'line'=>__LINE__));

			$GLOBALS['egw']->db->transaction_begin();
			try {
				$pay_period = new PayPeriod($data['pay_period_id']);

				if(!$pay_period) {
					throw new Exception(lang('Unable to find pay period'));
				}
				if(!isset($data['pay_schedule_id'])) { 
					$pay_period->create_paystubs($null);
				} else {
					if(!$data['pay_schedule_id']) {
						throw new Exception(lang('pay_schedule_id was not set'));
					}
					$schedule = new PaySchedule($data['pay_schedule_id']);
					if(!$schedule) {
						throw new Exception(lang('Unable to find pay schedule'));
					}
					$pay_period->create_paystubs($schedule);
				}
				$GLOBALS['egw']->log->error(array('text'=>'I-Paystub, Paystubs were created for pay period %1', 'p_1' => $data['pay_period_id'],'file'=>__FILE__,'line'=>__LINE__));
			} catch (Exception $e) {
				$GLOBALS['egw']->log->error(array('text'=>'E-Paystub, %1','p_1'=>$e->getMessage(),'file'=>$e->getFile(),'line'=>$e->getLine()));
				$GLOBALS['egw']->log->error(array('text'=>'E-Paystub, Paystubs were not generated!','file'=>$e->getFile(),'line'=>$e->getLine()));
			$GLOBALS['egw']->db->transaction_abort();
			}
			$GLOBALS['egw']->db->transaction_commit();
			$GLOBALS['egw']->log->commit();
		}

		/**
		*	Post already created paystubs
		*	
		*	@param data Array with the key pay_period_id.
		*/
		public function process_payday(Array $data) {
			$GLOBALS['egw_info']['flags']['currentapp'] = 'perp_payroll';

			$GLOBALS['egw']->log->error(array('text'=>'D-Paystub, Starting to post paystubs for %1', 'p_1' => $data['pay_period_id'],'file'=>__FILE__,'line'=>__LINE__));
			$pay_period = new PayPeriod($data['pay_period_id']);
			
			$GLOBALS['egw']->db->transaction_begin();
			try {
				if(!$pay_period) {
					throw new Exception(lang('Unable to find pay period'));
				}

				$pay_period->post_paystubs();
				$GLOBALS['egw']->log->error(array('text'=>'I-Paystub, Payperiod %1 was posted', 'p_1' => $data['pay_period_id'], 'file'=>__FILE__,'line'=>__LINE__));
			} catch (Exception $e) {
				$GLOBALS['egw']->log->error(array('text'=>'E-Paystub, %1','p_1'=>$e->getMessage(),'file'=>$e->getFile(),'line'=>$e->getLine()));
				$GLOBALS['egw']->log->error(array('text'=>'E-Paystub, Payperiod was not posted!','file'=>$e->getFile(),'line'=>$e->getLine()));
				$GLOBALS['egw']->db->transaction_abort();
			}
			$GLOBALS['egw']->db->transaction_commit();
			$GLOBALS['egw']->log->commit();
		}
	}
?>
