<?php

	class so_perp_rate {

		public static function get_rates(&$query) {
			
			$where = array();

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(region LIKE $search OR locality LIKE $search OR income LIKE $search OR rate LIKE $search OR constant LIKE $search)";
			}

			foreach($query['col_filter'] as $column => $value) {
				if($value) {
					$where[$column] = $value;
				}
			}


			// Total number of rows
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_income_tax_rate',
				'count(*)',
				$where,
				__LINE__, __FILE__, 
				false, '', 'perp_payroll'
			)->fetchSingle();

			$result =& $GLOBALS['egw']->db->select(
				'perp_income_tax_rate',
				'*',
				$where,
				__LINE__, __FILE__,
				$query['start'], 
				'ORDER BY ' . $query['order'] .' ' . $query['sort'],
				'perp_payroll', $query['num_rows']
			);
			$list = array();
			foreach($result as $row) {
				$row['effective_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_date']);
				$list[] = $row;
			}
			return $list;
		}


		/**
		*	Get values for header filters 
		*/
		public static function get_header_filters() {
			$filters = array();

			$result =& $GLOBALS['egw']->db->select(
				'perp_income_tax_rate',
				array(
					'country',
					'region',
					'locality'
				),
				array(),
				__LINE__, __FILE__, false,
				'GROUP BY country, region, locality
				ORDER BY count(country) DESC, count(region) DESC, count(locality) DESC',
				'perp_payroll'
			);

			foreach($result as $row) {
				foreach($row as $col_name => $col_value) {
					if($col_name == 'country') {
						$filters[$col_name][$col_value] = $GLOBALS['egw']->country->get_full_name($col_value, true);
					} else {
						$filters[$col_name][$col_value] = $col_value;
					}
				}
			}
			return $filters;
		}

		/**
		*	Get tax rate settings, according to given query values
		*/
		public static function get_rate($query) {
			$where = array(
				'effective_date <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($query['effective_date'])),
				'income <= ' . (float)$query['income'],
			);
			if($query['locality']) {
				$where['locality'] = $query['locality'];
			}
			if($query['region']) {
				$where['region'] = $query['region'];
			}
			if($query['country']) {
				$where['country'] = $query['country'];
			}
			$result =& $GLOBALS['egw']->db->select(
				'perp_income_tax_rate',
				'*',
				$where,
				__LINE__, __FILE__, 0, 'ORDER BY effective_date DESC',
				'perp_payroll', 1
			);
			foreach($result as $row) {
				$row['effective_date'] = $GLOBALS['egw']->db->from_timestamp($row['effective_date']);
				return $row;
			}
			return array();
		}

		public static function set_rate($rate_info) {
			$GLOBALS['egw']->db->insert(
				'perp_income_tax_rate',
				array(
					'country'	=>	$rate_info['country'],
					'region'	=>	$rate_info['region'],
					'locality'	=>	$rate_info['locality'],
					'effective_date'	=>	$GLOBALS['egw']->db->to_timestamp($rate_info['effective_date']),
					'income'	=>	(float)$rate_info['income'],
					'rate'		=>	(float)$rate_info['rate'],
					'constant'	=>	(float)$rate_info['constant']
				),
				array(
					'country'	=>	$rate_info['country'],
					'region'	=>	$rate_info['region'],
					'locality'	=>	$rate_info['locality'],
					'effective_date'	=>	$GLOBALS['egw']->db->to_timestamp($rate_info['effective_date']),
					'income'	=>	(float)$rate_info['income'],
					'rate'		=>	(float)$rate_info['rate'],
					'constant'	=>	(float)$rate_info['constant']
				),
				__LINE__, __FILE__, 'perp_payroll'
			);
		}

		public static function delete_rate($rate_info) {
			// Info doesn't have string keys here
			$result =& $GLOBALS['egw']->db->delete(
				'perp_income_tax_rate',
				array(
					'country'	=>	$rate_info[0],
					'region'	=>	$rate_info[1],
					'locality'	=>	$rate_info[2],
					'effective_date'	=>	$GLOBALS['egw']->db->to_timestamp($rate_info[3]),
					'income'	=>	(float)$rate_info[4],
					'rate'		=>	(float)$rate_info[5],
					'constant'	=>	(float)$rate_info[6]
				),
				__LINE__, __FILE__, 'perp_payroll'
			);
			return $result != false;
		}
	}
?>
