<?php

	/**
	*	User interface for Pay stub amemdments.
	*
	*	@module perp_payroll
	*	@author Nathan Gray
	*	@license GPL
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Paystub.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Amendment.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.bo_perp_payroll.inc.php' );

	class ui_perp_pay_amendment {

		public $public_functions = array(
			'amendment_list'	=>	true,
			'edit'			=>	true,
		);

		public static $template;
		public static $bo;

		private $debug = false;

		public function __construct() {
		}

		/**
		*	View a list of amendments
		*/
		public function amendment_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_payroll.index'
				));
			} elseif(isset($content['nm']['rows']['delete'])) {
				if(!$GLOBALS['egw']->acl->check('amendment', EGW_ACL_DELETE, 'perp_payroll')) {
					$this->messages[] = array(lang('You do not have permission to delete amendments'), 'error');
				} else {
					$amendment_id = key($content['nm']['rows']['delete']);
					$amendment = new Amendment($amendment_id);
					if(is_null($amendment) || !$amendment->isEditable()) {
						$this->messages[] = array(lang('Pay period is not editable'), 'error');
					} else {
						if($amendment->delete() == 1) {
							$this->messages[] = array(lang('Amendment deleted'), 'success');
						} else {
							$this->messages[] = array('Unable to delete amendment', 'error');
						}
					}
				}
			}

			// Read data for display
			$data = $this->get_list_data();

			$select_box_options = $this->get_list_options();

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Pay stub Amendments');
			self::$template->read('perp_payroll.pay_amendment_list');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		protected function get_list_data() {
			$data = array();
			$data['nm'] = $GLOBALS['egw']->session->appsession('pay_amendment_list', 'perp_payroll');
			if(!is_array($data['nm'])) {
				$data['nm'] = array(
					'no_cat'	=>	true,
					'no_filter'	=>	true,
					'no_filter2'	=>	true,
					'no_search'	=>	true,
					'order'		=>	'amendment_id',
					'sort'		=>	'DESC',
					'template'	=>	'perp_payroll.pay_amendment_list.row',
					'get_rows'	=>	'perp_payroll.ui_perp_pay_amendment.get_rows'
				);
			}
			return $data;
		}

		/**
		*	Get selectbox options for list of amendments
		*/
		protected function get_list_options() {
			$options = array();
			$options['pay_period_id'] = array(null => 'Next pay period') + self::$bo->get_pay_periods(array(
				'order'	=>	'start_date',
				'sort'	=>	'DESC'
			));
			$options['staff_id'] = ExecMethod2('perp_human_resources.bo_perp_staff.get_staff_list', array('order'=>'n_family'));
			$options['type_id'] = self::$bo->get_line_types();
			return $options;
		}

		/**
		*	Get list of amendments
		*	@nextmatch
		*/
		public function get_rows(&$query = array(), &$rows = array(), &$readonly = array()) {
			$GLOBALS['egw']->session->appsession('pay_amendment_list', 'perp_payroll', $query);

			$count =  self::$bo->get_pay_amendments($query, $rows, $readonly);
			return $count;
		}

		/**
		*	Add a new amendment, or edit an existing one
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}

			
			$amendment_id = get_var('amendment_id', 'any', $content['amendment_id']);

			if($amendment_id) {
				$amendment = new Amendment($amendment_id);
			} else {
				$amendment = new Amendment();
				$amendment->pay_period_id = get_var('pay_period_id', 'any', $content['pay_period_id']);
				$amendment->staff_id = get_var('staff_id', 'any', $content['staff_id']);
				$amendment->type_id = get_var('type_id', 'any', $content['type_id']);
			}
			
			if(isset($content['cancel'])) {
				// Close the popup window
				if(!is_object($GLOBALS['egw']->js)) {
					$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
				}
				$GLOBALS['egw']->js->set_onload('window.close();');
			} else {
				if(isset($content['save'])) {
					$this->update_edit($amendment, $content);
				}

				$data = $this->get_edit_data($amendment);

				$select_box_options = $this->get_edit_options();

				$preserve['amendment_id'] = $amendment->amendment_id;
			}


			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = $amendment_id != 0 ? lang('Edit Amendment') : lang('New Amendment');
			self::$template->read('perp_payroll.pay_amendment_edit');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve, 2);
		}

		/**
		*	Update an amendment with data returned by the eTemplate
		*/
		protected function update_edit(Amendment &$amendment, Array &$content) {
			$no_errors = true;

			// Set pay period
			if(isset($content['pay_period_id']) && !is_null($content['pay_period_id'])) {
				$period = new PayPeriod($content['pay_period_id']);
				if(!$period->isEditable()) {
					$this->messages[] = array(lang('Pay period is not editable'), 'error');
					$no_errors = false;
				} else {
					$amendment->pay_period_id = (int)$content['pay_period_id'];
				}
			} else {
				// Leave null to get picked up by the next pay period to be created
				$amendment->pay_period_id = null;
			}

			// Employee / Staff person
			if(isset($content['staff_id']) && (int)$content['staff_id'] != 0) {
				$amendment->staff_id = (int)$content['staff_id'];
			} else {
				$this->messages[] = array(lang('Missing employee'), 'error');
				$no_errors = false;
			}

			// Paystub Line Type
			if(isset($content['type_id']) && (int)$content['type_id'] != 0) {
				$amendment->type_id = (int)$content['type_id'];
			} else {
				$this->messages[] = array(lang('Missing paystub entry type'), 'error');
				$no_errors = false;
			}

			// Quantity & rate
			if($content['quantity'] * $content['rate'] == 0) {
				$this->messages[] = array(lang('Amendment has an amount of 0.'), 'warning');
			}
			$amendment->quantity = (float)$content['quantity'];
			$amendment->rate = (float)$content['rate'];

			if(trim($content['description']) == '') {
				$this->messages[] = array(lang('Empty description'), 'warning');
			}
			$amendment->description = trim($content['description']);

			if(isset($content['save']) && $no_errors && (
					$GLOBALS['egw']->acl->check('amendment', EGW_ACL_ADD, 'perp_payroll') && $amendment->get_amendment_id == null ||
					$GLOBALS['egw']->acl->check('amendment', EGW_ACL_EDIT, 'perp_payroll')
			)) {
				// Passing true will re-create the paystub
				$amendment->save(true);

				// Close the popup window, reload parent
				if(!is_object($GLOBALS['egw']->js)) {
					$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
				}
				$GLOBALS['egw']->js->set_onload(
					'window.opener.location = \'' . 
					$GLOBALS['egw']->link('/index.php', 
					array(
						'menuaction'	=>	'perp_payroll.ui_perp_pay_amendment.amendment_list'
					)) . '\';'
				);
				if(count($this->messages) == 0) {
					$GLOBALS['egw']->js->set_onload('window.close();');
				}
			}
		}

		/**
		*	Get selectbox options for editing an amendment
		*/
		protected function get_edit_options() {
			$options = $this->get_list_options();
			unset($options['staff_id']);
			return $options;
		}

		/**
		*	Get display data for editing an amendment
		*/
		protected function get_edit_data(Amendment &$amendment) {
			$data = $amendment->data;
			return $data;
		}
	}
	ui_perp_pay_amendment::$template =& CreateObject('etemplate.etemplate');
	ui_perp_pay_amendment::$bo =& new bo_perp_payroll();
?>
