<?php
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.bo_perp_payroll.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.PayPeriod.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Paystub.inc.php' );

	class ui_perp_pay_period {

		public static $bo;
		public static $template;

		public $public_functions = array(
			'period_list'	=>	true,
			'new_period'	=>	true,
			'edit'		=>	true,

		);

		private $debug = false;
		
		/**
		*	Array of messages for the user
		*/
		private $messages = array();

		public function __construct() {
		}

		/**
		*	Display and deal with a list of pay periods
		*
		*	@param content Array of data returned from eTemplate
		*/
		public function period_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}

			if(isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_payroll.index'
				));
			} elseif (isset($content['new_period']) && $GLOBALS['egw']->acl->check('pay_period', EGW_ACL_ADD, 'perp_payroll')) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_period.new_period'
				));
			} elseif (isset($content['nm']['rows']['delete']) && $GLOBALS['egw']->acl->check('pay_period', EGW_ACL_DELETE, 'perp_payroll')) {
				$pay_period_id = (int)key($content['nm']['rows']['delete']);
				if($pay_period_id) {
					$pay_period = new PayPeriod($pay_period_id);
					if($pay_period) {
						try {
							$result = $pay_period->delete();
							if($result === true) {
								$this->messages[] = array(lang('Pay period %1 deleted', $pay_period_id), 'success');
							}
						} catch (Exception $e) {
							$this->messages[] = array(lang($result), 'error');
						}
					}
				}
			} elseif (isset($content['nm']['rows']['revert']) && $GLOBALS['egw']->acl->check('pay_period', EGW_ACL_CUSTOM_1, 'perp_payroll')) {
				$pay_period_id = (int)key($content['nm']['rows']['revert']);
				if($pay_period_id) {
					$pay_period = new PayPeriod($pay_period_id);
					if($pay_period) {
						try {
							$result = $pay_period->revert();
							if($result === true) {
								$this->messages[] = array(lang('Pay period %1 reverted', $pay_period_id), 'success');
							} else {
								$this->messages[] = array($result, 'error');
							}
						} catch (Exception $e) {
							$this->messages[] = array($e->getMessage(), 'error');
						}
					}
				}
			}

			// Read data for display
			$data = $this->get_list_data();
			$readonly['new_period'] = !$GLOBALS['egw']->acl->check('pay_period', EGW_ACL_ADD, 'perp_payroll');

			// Display
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			self::$template->read('perp_payroll.pay_period_list');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		protected function get_list_data() {
			$data = array();
			$data['nm'] = $GLOBALS['egw']->session->appsession('pay_period_list', 'perp_payroll');
			if(!is_array($data['nm'])) {
				$data['nm'] = array(
					'no_cat'        =>      true,
					'no_filter'     =>      true,
					'no_filter2'    =>      true,
					'template'      =>      'perp_payroll.pay_period_list.row',
					'get_rows'      =>      'perp_payroll.ui_perp_pay_period.get_rows',
					'order'		=>	'start_date',
					'sort'		=>	'DESC'
				);
			}
			

			return $data;
		}

		/**
		*	Create a new (unscheduled) pay period.
		*	
		*	Scheduled pay periods should be handled using a schedule and async services,
		*	but sometimes we need special pay periods that don't fit the schedule.
		*
		*	@param content Array of data returned from the eTemplate
		*/
		public function new_period($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}

			if($content['save']) {
				try {
					$pay_period = new PayPeriod(null, $content['start_date'], $content['end_date'], $content['transaction_date']);
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_payroll.ui_perp_pay_period.edit',
						'pay_period_id'	=>	$pay_period->pay_period_id
					));
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_period.period_list'
				));
			}

			$select_box_options['reverse'] = array(false => lang('None')) + ExecMethod('perp_payroll.bo_perp_payroll.get_pay_periods');
			
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$GLOBALS['egw_info']['flags']['app_header'] = lang('New Pay Period');
			self::$template->read('perp_payroll.pay_period_new');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	View or edit a pay period.  
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}

			$pay_period_id = get_var('pay_period_id', 'any', $content['pay_period_id']);
			$period = new PayPeriod($pay_period_id);
			if(!$period || $content['cancel'] || !$GLOBALS['egw']->acl->check('pay_period', EGW_ACL_READ, 'perp_payroll')) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_period.period_list'
				));
			}
			
			// Update
			$buttons = array('save', 'create_paystubs', 'note', 'add_staff');
			if((count(array_intersect($buttons, array_keys($content))) > 0 || $content['nm']['rows']['recreate']) && 
					$GLOBALS['egw']->acl->check('pay_period', EGW_ACL_EDIT, 'perp_payroll')) {
				if($period->isEditable()) {
					$this->update_edit($period, $content);
				} else {
					$this->messages[] = array(lang('Pay period is not editable'), 'warning');
				}
			}

			// Read data for display
			$data = $this->get_edit_data($period);

			// Supporting data
			$select_box_options = $this->get_edit_options($period);

			// Get readonlys
			$readonly = $this->get_edit_readonlys($period, $data);

			$preserve['pay_period_id'] = $pay_period_id;

			// Display
			self::$template->read('perp_payroll.pay_period_edit');

			// Disable link to transaction if not posted
			self::$template->set_cell_attribute('transaction_id', 'disabled', ($period->transaction_id == 0));
			$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);

		}

		/**
		*	Update PayPeriod with data from user
		*/
		public function update_edit(PayPeriod &$period, Array $content) {
			if(isset($content['transaction_date'])) {
				if((int)$content['transaction_date'] > time()) {
					try {
						$period->transaction_date = (int)$content['transaction_date'];
						$period->schedule_pay_day($content['transaction_date']);
					} catch (Exception $e) {
						$this->messages[] = array($e->getMessage(), 'error');
					}
				} else {
					$this->messages[] = array(lang('Transaction date must be in the future'), 'error');
				}
			}

			// Recreate paystub
			if(is_array($content['nm']['rows']['recreate']) && $staff_id = key($content['nm']['rows']['recreate'])) {
				try {
					$paystub =& new Paystub($staff_id, $period->pay_period_id);
					$paystub->delete();
					
					$paystub =& new Paystub($staff_id, $period->pay_period_id);
					$paystub->save();
					$this->messages[] = array(lang('Paystub recreated: %1', bo_perp_staff::link_title($staff_id)), 'success');
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
			}
			if(isset($content['note'])) {
				if($period->add_note($content['note'])) {
					$this->messages[] = array(lang('Note added to all paystubs: %1', $content['note']), 'success');
				}
			}
			if(isset($content['add_staff'])) {
				$result = $period->add_staff((int)$content['add_staff']);
				if($result !== true && is_numeric($result)) {
					$overlap = new PayPeriod($result);
					$this->messages[] = array(lang('Could not add %2.  Pay period overlap with %1', $overlap->__toString(), bo_perp_staff::link_title($content['add_staff'])), 'warning');
				}
			} elseif (isset($content['staff_list']['delete'])) {
				$staff_id = (int)key($content['staff_list']['delete']);
				$period->remove_staff($staff_id);
			}

			if(isset($content['create_paystubs']) && $period->isEditable()) {
				$this->messages[] = array(lang('Trying to create pay stubs...'), 'info');
				$data = array(
					'pay_period_id'		=>	$period->pay_period_id
				);
				if($content['schedule_id']) {
					$data['pay_schedule_id']	= 	(int)$content['schedule_id'];
				}
				self::$bo->create_paystubs($data);
			}
			$period->save();
		}

		/**
		*	Get data for displaying or editing a pay period
		*/
		public function get_edit_data(PayPeriod &$period) {
			$data = $period->toArray();
			if($data['transaction_id']) {
				$data['transaction_label'] = lang('transaction');
			} else {
				$data['transaction_label'] = lang('Not posted');
			}

			// Check to see if there are any paystubs
			$query = array(
				'make_paystubs' =>	false
			);
			$period->get_paystubs($query);

			if($query['count'] > 0) {
				// Paystubs have already been created

				// Setup nextmatch for paystubs
				$data['nm'] = $GLOBALS['egw']->session->appsession('pay_stub_list', 'perp_payroll');
				if(!is_array($data['nm'])) {
					$data['nm'] = array(
						'no_cat'        =>      true,
						'no_filter'     =>      true,
						'no_filter2'    =>      true,
						'template'      =>      'perp_payroll.pay_stub_list.row',
						'get_rows'      =>      'perp_payroll.ui_perp_pay_period.get_pay_stubs'
					);
				}
				$data['nm']['pay_period_id'] = $period->pay_period_id;
			} else {
				// Paystubs have not  been created

				// Check schedule
				$id = 'perp_payroll-' . $period->pay_period_id . '-stub_day';
				$stub_day = PayPeriod::$async->read($id);
				if($stub_day) {
					$this->messages[] = array(
						lang('Pay stubs will be created on %1', date(
							$GLOBALS['egw_info']['user']['preferences']['common']['dateformat'], 
							$stub_day[$id]['next']
						)), 
						'info'
					);
				} else {
					$this->messages[] = array(lang('Pay stubs are not scheduled.  They will have to be manually created at the end of the pay period.'), 'warning');
				}

				// Check if this is a 1 time pay period
				$staff_list = $period->get_staff();
				$staff = null;
				$data['staff_list'] = array();
				foreach($staff_list as $staff_id) {
					$data['staff_list'][] = array(
						'staff_id'	=>	$staff_id
					);
				}
			}
			$data['no_add'] = !$period->pay_period_id || !$data['staff_list'] && $query['count'] > 0;

			// Get any errors
			$ui_payroll = CreateObject('perp_payroll.ui_perp_payroll');
			$ui_payroll->get_stub_log($this->messages);

			return $data;
		}

		/**
		*	Get selectbox options for editing a pay period
		*/
		public function get_edit_options(PayPeriod &$period) {

			$options = array();

			if($period->isEditable()) {
				// Check to see if there are any paystubs
				$options['schedule_id'] = array();
				$query = array(
					'make_paystubs' =>	false
				);
				$period->get_paystubs($query);
				if($query['count'] == 0 || count($period->get_staff()) > 0 || count(self::$bo->get_pay_schedules()) == 0) {
					$options['schedule_id'][null] = lang('Use list');
				}
				$options['schedule_id'] += self::$bo->get_pay_schedules();
			}

			return $options;
		}

		/**
		*	Get readonly fields for a pay period
		*/
		public function get_edit_readonlys(PayPeriod &$period, &$data) {
			$readonlys = array();

			if(!$period->isEditable()) {
				foreach($data as $field => $value) {
					if($field == 'nm' || $field == 'staff_list') {
						continue;
					}
					$readonlys[$field] = true;
				}

				// Re-create paystubs
				$readonlys['schedule_id'] = true;
				$readonlys['create_paystubs'] = true;

				$data['no_add'] = true;
			}

			$readonlys['save'] = !($period->isEditable() && $GLOBALS['egw']->acl->check('pay_period', EGW_ACL_EDIT, 'perp_payroll'));
			$readonlys['note'] = $readonlys['save'];

			return $readonlys;
		}

		public function get_rows(&$query = array(), &$rows = array(), &$readonly = array()) {
			$GLOBALS['egw']->session->appsession('pay_period_list', 'perp_payroll', $query);

			$count =  self::$bo->get_pay_periods($query, $rows, $readonly);
			return $count;
		}

		public function get_pay_stubs(&$query = array(), &$rows = array(), &$readonly = array()) {
			$GLOBALS['egw']->session->appsession('pay_stub_list', 'perp_payroll', $query);

			return self::$bo->get_paystubs($query, $rows, $readonly);
		}
	}
	ui_perp_pay_period::$template =& CreateObject('etemplate.etemplate');
	ui_perp_pay_period::$bo =& new bo_perp_payroll();
?>
