<?php

	/**
	*	User interface for pay schedules
	*
	*	@package perp_payroll
	*	@author Nathan Gray
	*	@license GPL
	*/

	require_once('class.PaySchedule.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/etemplate/inc/class.etemplate.inc.php');

	class ui_perp_pay_schedule {

		/**
		*	Publicly accessable methods
		*/
		public $public_functions = array(
			'schedule_list'		=>	true,
			'schedule_new'		=>	true,
			'edit'		=>	true
		);

		private $debug = false;

		/**
		*	eTemplate used for display
		*/
		public static $template = null;

		public static $config = null;


		public function __construct() {
			if(self::$config == null) {
				self::$config = config::read('perp_payroll');
			}
		}

		/**
		*	Display a list of pay schedules
		*
		*	@param content Array of data returned from eTemplate
		*/
		public function schedule_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming<br />';
				_debug_array($content);
			}
			
			// Deal with incoming
			if($content['new']) {
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_schedule.schedule_new'
				)));
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/admin/index.php'));
			} elseif ($content['save'] || $content['rows']['delete']) {
				$this->update($content['rows']);
			}

			// Read data for display
			$query = $GLOBALS['egw_info']['user']['preferences']['perp_payroll'][__FUNCTION__]['nm'];
			if(!is_array($query)) {
				$query = array();
			}
			$this->get_rows($query, $data['rows'], $readonly);
			if(isset($data['rows'][0])) {
				array_unshift($data['rows'], false);
			}
			$preserve = $data;

			$this->messages[] = array(lang('Dates apply only to the schedule, pay periods may differ.'), 'info');

			if(count($this->messages) > 0) {
                                $data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
                        }

			self::$template->read('perp_payroll.pay_schedule_list');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Create a new pay schedule
		*/
		public function schedule_new($content = array()) {
			$set_recurrences = false;
			$calendar_id = ExecMethod2('calendar.calendar_so.save', array(
				'category'	=>	self::$config['pay_schedule_category'],
				'public'	=>	1,
				'title'	=>	lang('New schedule'),
				'non_blocking'	=>	1,
				'start'		=>	strtotime('next Monday'),
				'end'		=>	strtotime('+2 weeks', strtotime('next Monday')) - 1
			),$set_recurrences);
			if($calendar_id) {
				$schedule = new PaySchedule($calendar_id);
				$schedule->save();
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_schedule.schedule_list',
				)));
			}
		}

		/**
		*	Get a list of pay schedules for display
		*/
		public function get_rows(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$GLOBALS['egw_info']['user']['preferences']['perp_payroll']['schedule_list']['nm'] = $query;
			$count = PaySchedule::get_rows($query, $rows, $readonlys);
			return $count;
		}

		/**
		*	Update a list of schedules (not the calendar part, just the schedule part)
		*/
		protected function update(Array $schedules) {
			if($schedules['delete']) {
				$schedule_id = key($schedules['delete']);

				$s =& new PaySchedule($schedule_id);
				$s->delete();

				// Unset row so we don't try to save it
				unset($schedules['delete']);
				foreach($schedules as $key => $schedule) {
					if($schedule['schedule_id'] == $schedule_id) {
						unset($schedules[$key]);
					}
				}
			}

			$s = null;
			foreach($schedules as $key => $schedule) {
				if(is_array($schedule)) {
					$s =& new PaySchedule($schedule['schedule_id']);
					$s->set_paystub_delay($schedule['stub_delay_days']);
					$s->set_transaction_delay($schedule['pay_delay_days']);
					$s->save();

				}
			}

		}
	}

	ui_perp_pay_schedule::$template = new eTemplate();
?>
