<?php

	/**
	*	User interface for dealing with pay stubs.
	*
	*	@package perp_payroll
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.Paystub.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.bo_perp_payroll.inc.php' );

	class ui_perp_pay_stub {

		public $public_functions = array(
			'edit'		=>	true,
		);

		public static $template;
		public static $bo;

		private $debug = false;

		public function __construct() {
		}

		/**
		*	View or edit a paystub
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}

			$pay_period_id = get_var('pay_period_id', 'any', $content['pay_period_id']);
			$staff_id = get_var('staff_id', 'any', $content['staff_id']);
			if(!$staff_id) {
				$staff_id = $GLOBALS['egw_info']['user']['account_id'];
			}

			$stub = null;
			if($pay_period_id && $staff_id) {
				if($GLOBALS['egw']->acl->check('pay_stub', EGW_ACL_READ, 'perp_payroll') || $staff_id == $GLOBALS['egw_info']['user']['account_id']) {
					$stub = new Paystub($staff_id, $pay_period_id);
				} else {
					$this->messages[] = array(lang('Access is not allowed'), 'warning');
				}
			}
			if(is_null($stub)) {
				$this->messages[] = array(lang('Unable to find paystub'), 'error');
			}
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'    =>      'perp_payroll.ui_perp_pay_period.edit',
					'pay_period_id'	=>	$pay_period_id
				));
			}
			
			if($stub) {
				$this->edit_update($stub, $content);

				// Read data for display
				$data = $this->get_edit_data($stub);
				$readonly = $this->get_edit_readonly($stub, $data);
			}

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$preserve['pay_period_id'] = $pay_period_id;
			$preserve['staff_id'] = $staff_id;

			// Display
			self::$template->read('perp_payroll.pay_stub_edit');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		protected function edit_update(Paystub &$stub, $content = array()) {
			$buttons = array('save', 'lines', 'notes');
			if(!$stub->isEditable()) {
				$this->messages[] = array(lang('Pay stub is not editable'), 'error');
				return;
			}
			if(count(array_intersect($buttons, array_keys($content))) > 0 && $GLOBALS['egw']->acl->check('pay_stub', EGW_ACL_EDIT, 'perp_payroll')) {
				$reload = false;

				// Add a note
				if(is_array($content['lines']['note'])) {
					// We need note counts to properly footnote them
					foreach($content['lines']['note'] as $entry_id => $note) {
						$stub->get_line($entry_id)->add_note(null, $note);
					}
					$reload = true;
				} elseif (isset($content['notes']['note'])) {
					$stub->add_note($content['notes']['note']);
				}

				// Remove a note
				if(isset($content['notes']['delete'])) {
					list($entry_id, $note_ref) = explode(',', key($content['notes']['delete']));
					if((int)$entry_id) {
						$stub->get_line($entry_id)->remove_note($note_ref);
						$reload = true;
					} elseif(is_numeric($note_ref)) {
						$stub->remove_note($note_ref);
						$reload = true;
					}
				}
				$stub->save();

				if($reload) {
					// Re-loading the stub will correctly re-key the notes
					$stub =& new Paystub($stub->staff_id, $stub->pay_period_id);
				}
			}
		}

		protected function get_edit_data(Paystub &$stub) {
			$data = $stub->toArray();
			$notes = $data['notes'];
			unset($data['notes']);
			$data['notes'][] = false;
			foreach($notes as $key => $note) {
				$data['notes'][] = array(
					'key'	=>	$key,
					'note'		=>	$note
				);
			}
			$lines =& $stub->get_lines();
			$data['lines'][] = false;
			foreach($lines as &$line) {
				$line_data = $line->toArray();
				$line_data['quantity'] = $line_data['quantity'] * 1;
				$line_data['YTD'] = PayStubEntryType::get_YTD($line_data['type_id'], $stub->get_pay_period_id(), $stub->get_staff_id());
				$line_data['rate'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $line_data['rate']);
				$line_data['amount'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $line_data['amount']);
				$line_data['YTD'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', null, $line_data['YTD']['ytd_amount']);

				$line_data['style'] = $line_data['type'] . ' ' . $line_data['DeductionType'];
				if(count($line_data['notes'])) {
					$line_data['note_key'] = ' ' . implode(', ', array_keys($line_data['notes']));
					foreach($line_data['notes'] as $key => $note) {
						$data['notes'][] = array(
							'key'		=>	$key,
							'reference'	=>	$key,
							'entry_id'	=>	$line_data['entry_id'],
							'note'		=>	$note
						);
					}
				}
				$data['lines'][] = $line_data;
			}
			$period_id = $stub->get_pay_period_id();
			$period = new PayPeriod($period_id);
			$data['pay_period'] = $period->toArray();
			if(count($data['notes']) == 0) {
				unset($data['notes']);
			}
			return $data;
		}

		protected function get_edit_readonly(Paystub &$stub, Array &$data) {
			$readonly = array();
			if(!$stub->isEditable()) {
				$data['lines']['readonly'] = true;
				$readonly['save'] = true;
				$readonly['perp_payroll.pay_stub_edit.notes'] = true;
				$readonly['perp_payroll.pay_stub_edit.row'] = true;
			}
			return $readonly;
		}
	}

	ui_perp_pay_stub::$template =& CreateObject('etemplate.etemplate');
	ui_perp_pay_stub::$bo =& new bo_perp_payroll();
?>
