<?php

	/**
	*	User interface for pay stub entry types
	*
	*	@package perp_payroll
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.PaystubEntryType.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.EarningType.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.DeductionType.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.AccrualType.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.TotalType.inc.php' );
	require_once( EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.CalculationFactory.inc.php' );

	require_once(EGW_INCLUDE_ROOT . '/etemplate/inc/class.etemplate.inc.php');

	class ui_perp_pay_stub_entry_type {

		/**
		*	Publicly accessable methods
		*/
		public $public_functions = array(
			'type_list'	=>	true,
			'type_new'	=>	true,
			'edit'		=>	true,
		);

		private $debug = false;

		/**
		*       eTemplate used for display
		*/
		public static $template = null;

		public static $config = array();


		public function __construct() {
			self::$config = config::read('perp_payroll');
		}

		/**
		*	View a list of paystub entry types
		*
		*	@param content Array of data returned from etemplate
		*/
		public function type_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}
			
			// Deal with incoming
			if($content['new']) {
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_stub_entry_type.type_new'
				)));
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/admin/index.php'));
			} elseif ($content['save']) {
				$this->update($content['rows']);
			} elseif ($content['nm']['rows']['delete']) {
				$type = PayStubEntryType::factory(key($content['nm']['rows']['delete']));
				try {
					if(is_object($type)) {
						$result = $type->delete();
					}
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
				if($result === true) {
					$this->messages[] = array(lang('%1 removed', $type->get_name()), 'success');
				} else {
					// It might have been disabled...
					if($type->is_disabled()) {
						$this->messages[] = array(lang('%1 disabled', $type->get_name()), 'success');
					} else {
						$this->messages[] = array(lang('Unable to remove type'), 'error');
					}
				}
			}

			// Read data for display
			$data['nm'] = $GLOBALS['egw']->session->appsession('pay_stub_entry_type_list', 'perp_payroll');
			if(!is_array($data['nm'])) {
				$data['nm'] = array(
					'no_cat'	=>	true,
					'no_filter'	=>	true,
					'no_filter2'	=>	true,
					'order'		=>	'sort_order',
					'template'	=>	'perp_payroll.paystub_entry_list.row',
					'get_rows'	=>	'perp_payroll.ui_perp_pay_stub_entry_type.get_rows',
					'filter'	=>	false
				);
			}

			$select_box_options['calculation'] = CalculationFactory::getList();

			$preserve = $data;
			$this->messages[] = array(lang('Net Pay (amount paid to the employee) must be the bottom line, the last entry type.'), 'info');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Paystub Entry Types');
			self::$template->read('perp_payroll.paystub_entry_list');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	New paystub entry type
		*
		*	@param content Array of data returned from etemplate
		*/
		public function type_new($content = array()) {
			$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/index.php', array(
				'menuaction'	=>	'perp_payroll.ui_perp_pay_stub_entry_type.edit'
			)));
		}

		/**
		*	Edit an existing paystub entry type
		*
		*	@param content Array of data returned from etemplate
		*/
		public function edit($content = array()) {
			if($this->debug) {
				echo __METHOD__ . '() incoming:<br />';
				_debug_array($content);
			}
			
			$type_id = get_var('type_id', 'any', $content['type_id']);
			$class = $content['class'];
			$type = $this->get_entry_type($type_id, $class);
			if(is_null($type)) {
				$this->messages[] = array(lang('Unable to load entry'), 'error');
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect($GLOBALS['egw']->link('/index.php', array(
					'menuaction'	=>	'perp_payroll.ui_perp_pay_stub_entry_type.type_list'
				)));
			} else {
				// Deal with incoming
				if(count($content) > 0) {
					$this->update($type, $content);
				}
				if(isset($content['save'])) {
					unset($content['save']);
					$type->save();
				}
			}


			// Read data for display
			if($type instanceof PaystubEntryType) {
				$data = $this->get_edit_data($type);

			// Supporting data
				$select_box_options = $this->get_select_box_options($type);
			
				$preserve['type_id'] = $type->get_id();
			}
			$preserve['class'] = get_class($type);
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Paystub Entry Type');
			self::$template->read('perp_payroll.paystub_entry_edit');
			self::$template->exec('perp_payroll.' . __CLASS__ . '.' . __FUNCTION__, $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Get a list of pay stub entry types for display
		*/
		public function get_rows(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$GLOBALS['egw']->session->appsession('pay_stub_entry_type_list', 'perp_payroll', $query);
			$count = PaystubEntryType::get_rows($query, $rows, $readonlys);
			return $count;
		}

		/**
		*	Update a type
		*/
		protected function update(PaystubEntryType &$entry, Array $content) {
			$session_rows = $GLOBALS['egw']->session->appsession('entry_staff', 'perp_payroll');

			try {
				$entry->set_name($content['name']);
				$entry->set_debit_account($content['gl_debit_account']);
				$entry->set_credit_account($content['gl_credit_account']);
				$entry->set_sort_order($content['sort_order']);

				if(isset($content['calculation'])) {
					$entry->set_calculation_name($content['calculation']);
				} else {
					$entry->set_calculation_name(null);
				}

				// Add new staff
				if($content['add'] && $content['add']['button']) {
					unset($content['add']['button']);
					$content['add']['type_id'] = $entry->get_id();
					$entry->set_staff(array($content['add']));
					unset($content['add']);
				}

				// Set default data
				if(isset($content['value']) && $content['value']) {
					$entry->set_default_value($content['value']);
				}


				// Delete staff
				if($content['staff_list']['rows']['delete']) {
					$staff_id = key($content['staff_list']['rows']['delete']);
					if(method_exists($entry, 'remove_staff')) {
						$entry->remove_staff($staff_id);
					}
				} elseif(is_array($content['staff_list']['rows']) && count($content['staff_list']['rows']) > 0) {
				// Update existing staff
					if(is_array($session_rows)) {
						foreach($session_rows as $key => $row) {
							if(is_numeric($key) && is_array($row) ) {
								$content['staff_list']['rows'][$key] = array_merge($row, $content['staff_list']['rows'][$key]);
							}
						}
					}
					$entry->set_staff($content['staff_list']['rows']);
				}
			} catch (Exception $e) {
				$this->message[] = array($e->getMessage(), 'error');
			}

			switch (get_class($entry)) {
				case 'EarningType':
					if(isset($content['timesheet_category_id'])) {
						$entry->set_timesheet($content['timesheet_category_id']);
					}
					if(isset($content['rate_multiplier'])) {
						$entry->set_rate($content['rate_multiplier']);
					}
					if($content['pay_type']) {
						$entry->set_pay_type($content['pay_type']);
					}
					break;
				case 'DeductionType':
					if($content['type']) {
						$entry->set_type($content['type']);
					}
					break;
				case 'AccrualType':
					if($content['rate']) {
						// UI gives rate as a %
						$entry->set_rate($content['rate'] / 100);
					}
					if($content['base_types']) {
						$entry->set_base_types($content['base_types']);
					}
					$entry->set_draw_type($content['draw_type']);
					$entry->set_minimum($content['minimum']);
					$entry->set_maximum($content['maximum']);
	
					if($content['breakpoints']) {
						foreach($content['breakpoints'] as $key => &$breakpoint) {
							if($breakpoint['service_length'] == '' || $breakpoint['rate'] == '') {
								unset($content['breakpoints'][$key]);
							}
							$breakpoint['rate'] /= 100;
						}
						$entry->set_breakpoints($content['breakpoints']);
					}
					break;
				case 'TotalType':
					$entry->set_calculation_name('Total');
					break;
			}
		}

		/**
		*	Get the data for display
		*/
		protected function get_edit_data(PaystubEntryType &$type) {
			$data = array();
			$data += $type->toArray();

			if($data['type_id']) {
				$data['staff_list'] = array(
					'type'		=>	$type,
					'get_rows'	=>	'perp_payroll.ui_perp_pay_stub_entry_type.get_staff',
					'no_cat'	=>	true,
					'no_filter'	=>	true,
					'no_filter2'	=>	true,
					'no_search'	=>	true
				);
			}

			$data['class_template'] = 'perp_payroll.paystub_entry_type.' . get_class($type);
			$data['value'] = $data['default_value'];

			$data['gl_credit_account'] = array(
				'account_id'	=>	$data['gl_credit_account'],
				'account_group'	=>	self::$config[get_class($type).'_credit_group']
			);
			$data['gl_debit_account'] = array(
				'account_id'	=>	$data['gl_debit_account'],
				'account_group'	=>	self::$config[get_class($type).'_debit_group']
			);
			switch (get_class($type)) {
				case 'AccrualType':
					// UI shows rate as a %
					$data['rate'] *= 100;
					foreach($data['breakpoints'] as &$breakpoint) {
						$breakpoint['rate'] *= 100;
					}

					// Make room for header
					array_unshift($data['breakpoints'], false);

					// Add a blank line to the bottom
					$data['breakpoints'][] = array();
					break;
				case 'DeductionType':
					break;
				case 'TotalType':
					$data['calculation'] = 'Total';
					unset($data['staff_list']);
					break;
			}

			if($data['calculation']) {
				$calc = CalculationFactory::getCalculation($data['calculation']);
				if($calc instanceof Calculation) {
					$data['calculation_template'] = $calc->getTemplate();
				}
			}
			return $data;
		}

		/**
		*	Get supporting data
		*/
		public function get_select_box_options(PaystubEntryType &$type) {
			$select_box_options['class'] = array(
				'EarningType'		=>	lang('Earning'),
				'DeductionType'		=>	lang('Deduction'),
				'AccrualType'		=>	lang('Accrual'),
				'TotalType'		=>	lang('Total')
			);
			$select_box_options['calculation'] = CalculationFactory::getList();
			if($type->get_calculation_name()) {
				$calc = CalculationFactory::getCalculation($type->get_calculation_name());
				if($calc instanceof Calculation) {
					$select_box_options += $calc->getOptions();
				}
			}

			switch (get_class($type)) {
				case 'DeductionType':
					$select_box_options['type'] = DeductionType::$type_array;
					break;
				case 'AccrualType':
					$select_box_options['base_types'] = ExecMethod('perp_payroll.bo_perp_payroll.get_line_types');
					$select_box_options['draw_type'] = ExecMethod('perp_payroll.bo_perp_payroll.get_earning_types');
					$select_box_options['service_units'] = Paystub::$pay_unit;
					break;
				case 'EarningType':
					$select_box_options['pay_type'] = EarningType::$pay_type_array;
					break;
			} 

			return $select_box_options;
		}

		/**
		*	Get a paystub entry to work with
		*/
		protected function get_entry_type($type_id, $type = 'EarningType') {
			$entry_type = null;
			if($type) {
				$entry_type = new $type($type_id);
				if(!($entry_type instanceof PaystubEntryType)) {
					$entry_type = null;
					$type = null;
				} else {
					return $entry_type;
				}
			}
			if($type_id && !$type) {
				$entry_type = PaystubEntryType::factory($type_id);
				return $entry_type;
			} else {
				return new EarningType(null);
			}
		}

		/**
		*	Get a list of staff the current EntryType is applicable to.
		*	@nextmatch
		*/
		public function get_staff(&$query, &$rows = array(), &$readonlys = array()) {
			if(! ($query['type'] instanceof PayStubEntryType) ) {
				$rows = array();
				return 0;
			}

			$query['template'] = 'perp_payroll.paystub_entry_staff.' . get_class($query['type']);
			$rows = $query['type']->get_staff();

			if(isset($rows[0])) {
				array_unshift($rows, false);
				unset($rows[0]);
			}
			foreach($rows as $key => $row) {
				$readonlys["{$key}[staff_id]"]['search'] = true;
			}

			$GLOBALS['egw']->session->appsession('entry_staff', 'perp_payroll', $rows);
			if(isset($rows['override_template'])) {
				return count($rows) - 1;
			} else {
				return count($rows);
			}
		}
	}

	ui_perp_pay_stub_entry_type::$template = new etemplate();
?>
