<?php
	/**
	* @class generic_step
	* @brief generic_step is the parent step of all steps that use common function for pERP Setup Wizard
	* @author John Trudel
	* @version 1.0
	* @date September 2008
	*/
	require_once(EGW_INCLUDE_ROOT . '/wizard/inc/class.step.inc.php');
	
	class perp_setup_step extends step {
		
		public $step_name;

		protected $dependencies = array();
		private $debug = false;
		
		public function perp_setup_step() {
			$this->step_name = get_class($this);
		}
		
		/**
		*  get_step_data function
		*  
		* @param content the variables passed in from the wizard
		* @param data passed by reference from the wizard to pass in exec template call
		* @param select_box_options passed by reference from the wizard to pass in exec template call
		* @param readonly passed by reference from the wizard to pass in exec template call
		* @param preserve passed by reference from the wizard to pass in exec template call
		*/
		public function get_step_data($content = array(), &$data, &$select_box_options, &$readonly, &$preserve) {
						
			//template to be used for this step
			$data['template_name'] = 'perp_setup_wizard.generic_step';
			
			//read perp_setup_wizard config 
			$perp_setup_wizard_config = config::read($content['wizard_app_name']);
			//set $step_status_array with values from perp_setup_config
			$steps_status_array = $perp_setup_wizard_config['steps_status'];
			//send image name for status to template
			if($steps_status_array[$data['step_name']] == 1){
				$step_data['status_image'] = 'done.png';
			}else{
				$step_data['status_image'] = 'not_done.png';
			}
			
			$preserve['current_step'] = $content['current_step'];
			
		}
		
		/**
		*  get_permission function
		*  @brief this function is ran from wizards build_permissions function
		*  it calls the parent generic_step get_permission
		*/	
		public function get_permission(){		
			
			$step_permission = array();
			
			$perp_setup_wizard_config = config::read($GLOBALS['egw_info']['flags']['currentapp']);
			$step_status = $perp_setup_wizard_config['steps_status'];
			
			//check access rights and appropriate message if no access
			//first check acl for step, if user has rights to step, then check on dependencies, if steps dependencies are complete
			//then give access
			
			//check acl
			$isAllowed = true;
			if(count($this->acls) > 0){
				foreach($this->acls as $acl){
					if($GLOBALS['egw']->acl->check($acl, EGW_ACL_READ | EGW_ACL_EDIT, $this->app)){
						$isAllowed = true;
					}else{
						$isAllowed = false;
						$reasons[] = lang('perp_setup_wizard_no_privileges_error');
						break;
					}
				}
			}
			if(count($this->dependencies) > 0){
				$reason = lang('perp_setup_wizard_front_dependency_error');
				$x = 0;
				//check individual dependencies
				foreach($this->dependencies as $dependency){
					
					if(!isset($step_status[''.$dependency.''])){
						$isAllowed = false;
						if($x != 0){
							$reason .= ', ';
						}
						$reason .= lang($dependency);
						$x++;
					}else{
						$isAllowed = true;
					}
				}
				
				$reason .= lang('perp_setup_wizard_back_dependency_error');
				$reasons[] = $reason;
			}
				
			$step_permission['perms'] =  $isAllowed  ;
			if(!$isAllowed){
				$step_permission['reasons'] = $reasons;
			}
			if($this->debug){
				_debug_array($step_permission);
			}
			return $step_permission;			
		}
		
		/**
		*  complete_step function
		*  @brief this function is ran when it is the current working step in the session
		*  it determines if the step is complete from any location in eGroupware
		*  it is called from hook_topmenu_info, it then calls its parent complete step in class.step.inc.php
		*  @param content the variables passed in from the wizard
		*/
		public function complete_step($content){
			
			$data = $content;
			parent::complete_step($data);
			
		}
		
			
	}
?>