<?php
	/**
	*	TimeCard
	*	Used for keeping track of start and end times for employees
	*	Called by the TimeClock.
	*/

	require_once('class.so_perp_timecard.inc.php');

	class bo_perp_timecard {

		protected $so;

		public function __construct() {
			$this->so = new so_perp_timecard();
		}

		/**
		*	Get a list of timestamps 
		*/
		public function get_timestamps(&$query) {
			if(!$GLOBALS['egw']->acl->check('timestamp', EGW_ACL_READ, 'perp_timeclock')) {
				return array();
			} else {
				if(!$query['order']) {
					$query['order'] = 'perp_timecard.account_id';
				}
				return $this->so->get_timestamps($query);
			}
		}

		/**
		*	Create a timestamp for a staff person.  The timestamp is in server time, created by the server.
		*	@param user_id	User ID of the person to create the timestamp for.
		*/
		public function timestamp($user_id) {
			if(!$user_id || !$GLOBALS['egw']->acl->check('timestamp', EGW_ACL_ADD, 'perp_timeclock')) {
				return false;
			} else {
				return $this->so->timestamp($user_id);
			}
		}

		/**
		*	Get how many people are still in the building, according to timestamps.
		*	@param query Array of parameters
		*		'return'	=>	'int' for integer count, or 'string' for human friendly, translated string
		*
		*	@return int / string, depending on $query['return'] value
		*/
		public function get_people_in_building(Array $query = array()) {
			if(!$GLOBALS['egw']->acl->check('timestamp', EGW_ACL_READ, 'perp_timeclock')) {
				return -1;
			} else {
				$count = $this->so->people_in_building($query);
				if(!isset($query['return']) || $query['return'] == 'int') {
					return $count;
				} elseif ($query['return'] == 'string') {
					switch($count) {
						case 0:
							return lang('no_people_in_building');
							break;
						case 1:	
							return lang('one_person_in_building');
							break;
						default:
							return lang('%1_people_in_building', $count);
							break;
					}
				}
			}
		}


		/**
		 * handles introspection or discovery by the logged in client,
		 *  in which case the input might be an array.  The server always calls
		 *  this function to fill the server dispatch map using a string.
		 *
		 * @param string $_type='xmlrpc' xmlrpc or soap
		 * @return array
		 */
		public function list_methods($_type='xmlrpc') {
			if (is_array($_type))
			{
				$_type = $_type['type'] ? $_type['type'] : $_type[0];
			}

			switch($_type)
			{
				case 'xmlrpc':
					$xml_functions = array(
						'timestamp' => array(
							'function'  => 'timestamp',
							'signature' => array(array(xmlrpcStruct,xmlrpcInt)),
							'docstring' => lang('Create a timestamp for a staff person.  Used for the TimeClock.')
						),
						'people_in_building' => array(
							'function'  => 'get_people_in_building',
							'signature' => array(array(xmlrpcStruct,xmlrpcStruct)),
							'docstring' => lang('Get how many people are still in the building, according to the timestamps.  Used for the TimeClock.')
						),
					);
					return $xml_functions;
					break;
				case 'soap':
					return $this->soap_functions;
					break;
				default:
					return array();
					break;
			}
		}
	}
?>
