<?php
	/**
	*	Timeclock
	*	Functions for the timeclock, called by the TimeClock.
	*/

	
	require_once('class.so_perp_timeclock.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_human_resources/inc/class.Staff.inc.php');

	class bo_perp_timeclock {

		protected $so;

		/**
		*	Mapping of eGroupWare permissions to timeclock actions
		*/
		private $time_clock_actions = array(
			'perp_timeclock'	=>	array(
				'config'		=>	EGW_ACL_CUSTOM_1,
				'reboot'		=>	EGW_ACL_CUSTOM_1,
				'timestamp'		=>	EGW_ACL_ADD
			),
			'perp_payroll'		=>	array(
			),
			'perp_human_resources'	=>	array(
				'staff_authentication'	=>	EGW_ACL_ADD
			),
		);

		public function __construct() {
			$this->so = new so_perp_timeclock();
		}

		/**
		*	Get a list of actions allowed for the timeclock
		*/
		public function get_actions($user_id) {
			if(is_array($user_id)) {
				$user_id = current($user_id);
			}
			$staff = new Staff($user_id);

			// No staff person found by that ID, can't do anything
			if(is_null($staff)) {
				return array();
			}

			$actions = array();
			$rights = array();
			$department = $staff->get_department();

			// Start with the rights of the current user 
			foreach($this->time_clock_actions as $app => $locations) {
				// Actions that can actually be executed
				$location_rights = $GLOBALS['egw']->acl->get_all_location_rights($GLOBALS['egw_info']['user']['account_id'], $app);
				foreach($locations as $location => $needed) {
					if($location_rights[$location] & $needed) {
						$actions[] = $location;
					}
				}
				// Actions that are allowed to be executed
				if($staff) {
					$location_rights = $GLOBALS['egw']->acl->get_all_location_rights($user_id, $app);
					foreach($locations as $location => $needed) {
						if($location_rights[$location] & $needed) {
							$rights[] = $location;
						}
					}
				}
				if($department->get_group_id()) {
					$location_rights = $GLOBALS['egw']->acl->get_all_location_rights($department->get_group_id(), $app);
					foreach($locations as $location => $needed) {
						if($location_rights[$location] & $needed) {
							$rights[] = $location;
						}
					}
				}
			}
			$actions = array_intersect($actions, $rights);

			return array_values(array_unique($actions));
		}

		/**
		*	Get whether or not a person is still in the building, according to timestamps
		*/
		public function get_person_in_building($user_id) {
			if(is_array($user_id)) {
				if($user_id[0]) {
					$user_id = $user_id[0];
				} else {
					return (boolean)false;
				}
			}
			if(!$GLOBALS['egw']->acl->check('timestamp', EGW_ACL_READ, 'perp_timeclock')) {
				return (boolean)false;
			} else {
				return (boolean)$this->so->get_person_in_building($user_id);
			}
		}

		/**
		*	Get how many people are still in the building, according to timestamps.
		*	@param query Array of parameters
		*		'return'	=>	'int' for integer count, or 'string' for human friendly, translated string
		*
		*	@return int / string, depending on $query['return'] value
		*/
		public function get_people_in_building(Array $query = array()) {
			if(!$GLOBALS['egw']->acl->check('timestamp', EGW_ACL_READ, 'perp_timeclock')) {
				return -1;
			} else {
				$count = $this->so->people_in_building($query);
				if(!isset($query['return']) || $query['return'] == 'int') {
					return (int)$count;
				} elseif ($query['return'] == 'string') {
					switch($count) {
						case 0:
							return lang('no_people_in_building');
							break;
						case 1:	
							return lang('one_person_in_building');
							break;
						default:
							return lang('%1 people_in_building', $count);
							break;
					}
				}
			}
		}


		/**
		 * handles introspection or discovery by the logged in client,
		 *  in which case the input might be an array.  The server always calls
		 *  this function to fill the server dispatch map using a string.
		 *
		 * @param string $_type='xmlrpc' xmlrpc or soap
		 * @return array
		 */
		public function list_methods($_type='xmlrpc') {
			if (is_array($_type))
			{
				$_type = $_type['type'] ? $_type['type'] : $_type[0];
			}

			switch($_type)
			{
				case 'xmlrpc':
					$xml_functions = array(
						'person_in_building' => array(
							'function'  => 'get_person_in_building',
							'signature' => array(array(xmlrpcStruct,xmlrpcInt)),
							'docstring' => lang('Get if a person is still in the building, according to the timestamps.  ')
						),
						'people_in_building' => array(
							'function'  => 'get_people_in_building',
							'signature' => array(array(xmlrpcStruct,xmlrpcStruct)),
							'docstring' => lang('Get how many people are still in the building, according to the timestamps.  ')
						),
						'get_actions' => array(
							'function'  => 'get_actions',
							'signature' => array(array(xmlrpcStruct,xmlrpcInt)),
							'docstring' => lang('Returns a list of actions available to that staff person.')
						),
					);
					return $xml_functions;
					break;
				case 'soap':
					return $this->soap_functions;
					break;
				default:
					return array();
					break;
			}
		}
	}
?>
