<?php

	/**
	*	For converting timeclock punch data into timesheet entries.
	*
	*	This is interesting, because some people clock out for lunch, some people don't.
	*
	*	An example of the complexity is an 8 hour shift with 1/2 hour unpaid lunch.  If 
	*	the person doesn't clock out for lunch, it will show up as an 8.5 hour day (0.5h overtime).
	*	If someone else clocks out for lunch, and works 1/2 hour of overtime, it will show up
	*	as an 8.5 hour day.  Both of these need to be correctly put into the timesheet, the first
	*	as two 4 hour blocks, the second as two 4 hour blocks, and 1/2 hour of overtime.
	*/

	require_once('class.bo_perp_timecard.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_payroll/inc/class.bo_perp_payroll.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_human_resources/inc/class.bo_perp_staff.inc.php');
	require_once(EGW_INCLUDE_ROOT . '/perp_human_resources/inc/class.Staff.inc.php');

	require_once(EGW_INCLUDE_ROOT . '/timesheet/inc/class.timesheet_bo.inc.php');

	class bo_perp_timesheet {

		/**
		*	Enable for debug & trace information
		*/
		private $debug = false;

		/**
		*	Local access to timecard stuff
		*/
		protected $bo_timecard;

		public function __construct() {
			$this->bo_timecard = new bo_perp_timecard();
			$this->bo_payroll = new bo_perp_payroll();
			$this->bo_timesheet = new timesheet_bo();
			$this->bo_staff = new bo_perp_staff();

		}

		/**
		*	Import all timeclock data for all staff over the given date range into the timesheet.
		*/
		public function import($start_date = null, $end_date = null) {

			// Organizational settings
			$this->organization = config::read('perp_timeclock');
			$this->organization['time_resolution']		*=	60;	// Config stores minutes, we need seconds
			$this->organization['max_hours_per_week']	*=	3600;	// Config stores hours, we need seconds
			$this->organization['max_hours_per_day']	*=	3600;	// Config stores hours, we need seconds
			$this->organization['min_overtime_to_pay']	*=	60;	// Config stores minutes, we need seconds

			// Import only recognizes regular and overtime
			$this->bo_payroll->get_earning_types(array('template' => false), $time_categories);
			foreach($time_categories as $type) {
				if($type['rate_multiplier'] == 1) {
					$this->time_categories['regular_time'] = $type;
				} else {
					$this->time_categories['overtime'] = $type;
				}
				if(isset($this->time_categories['regular_time']) && isset($this->time_categories['overtime'])) {
					break;
				}
			}
			$timeclock = config::read('perp_timeclock');

			if(is_null($start_date)) {
				$start_date = $timeclock['timesheet_import_last_run'];
			} 
			if(is_null($end_date)) {
				$end_date = time();
			}

			$this->exceptions = array();

			// Set template to get the full results
			$query = array(
				'template'	=>	false
			);
			$this->bo_staff->get_staff_list($query, $staff_list);
			foreach($staff_list as $staff) {
				$this->import_staff($staff, $start_date, $end_date);
			}

			// Update last run time
			config::save_value('timesheet_import_last_run', $end_date, 'perp_timeclock');

			return $this->exceptions;
		}

		/**
		*	Import all timeclock data for a given staff person over the given range into the timesheet.
		*/
		public function import_staff(array $staff, $start_date, $end_date) {
			if($this->debug) {
				echo 'Importing timeclock data for staff ' . $staff['n_fn'] . ' (' . date('Y-m-d', $start_date) . ' -> ' . date('Y-m-d', $end_date) . ")<br />\n";
			}

			$this->staff = $staff;
			$this->staff['staff_object'] =& new Staff($staff['staff_id']);

			if(!$this->staff['staff_object']->get_staff_id()) {
				$this->exceptions[] = array(
					lang('Could not load Staff object for %1', $staff['n_fn']),
					'warning'
				);
				return false;
			}
			$this->staff['breaks']		=	array(
				array(
					'start'	=>	'12:00',
					'end'	=>	'12:30',
				),
				array(
					'start'	=>	'16:30',
					'end'	=>	'17:00'
				),
			);
			
			$pay_info = $this->staff['staff_object']->get_payroll_info();
			$this->staff['pay_rate'] = array(
				'regular_time'	=>	$pay_info['pay_rate'] * $this->time_categories['regular_time']['rate_multiplier'],
				'overtime'	=>	$pay_info['pay_rate'] * $this->time_categories['overtime']['rate_multiplier']
			);
			$this->staff['timesheet_title'] = 'Testing timesheet';
			$this->staff['exceptions'] = array();

			$query = array(
				'user_id'	=>	$this->staff['staff_id'],
				'start_date'	=>	$start_date,
				'end_date'	=>	$end_date,
				'num_rows'	=>	-1
			);
			$timestamps = $this->bo_timecard->get_timestamps($query);

			$timestamps = $this->get_totals($timestamps);

			$this->add_overtime($timestamps);

			$blocks = $this->get_blocks($timestamps);

			foreach($this->staff['exceptions'] as $exception) {
				$this->exceptions[] = array(
					lang(
						'%1 on %2: %3',
						$this->staff['n_fn'], 
						date($GLOBALS['egw_info']['user']['common']['dateformat'] . ' H:i', $exception['date']), 
						$exception['cause']
					), 
					$exception['severity']
				);
			}
		}

		/**
		*	Calculate daily and weekly totals for overtime, and group by day
		*/
		public function get_totals($timestamps) {
			$summary = array();

			if(count($timestamps) == 0) {
				return array();
			}

			// Group into days 
			foreach($timestamps as $timestamp) {
				$summary['day'][date('z', $timestamp['timestamp'])][] = $timestamp['timestamp'];
			}

			// Validate & total
			foreach($summary['day'] as $day => &$punches) {
				// Check for matching in/out for every day (graveyard shift may cause problems)
				$this->validate_day(&$punches);

				// Split at unpaid breaks
				$this->break_split(&$punches);

				// Calculate day totals
				$remove = array();
				for($i = 0; $i < count($punches); $i += 2) {
					if($punches[$i] && $punches[$i + 1]) {
						$time = ($punches[$i + 1] - $punches[$i]);
						if($this->debug) {
							echo 'Day ' . $day . ' block ' . ($i / 2) . ' ' . 
								date('H:i', $punches[$i]) . ' -> ' . date('H:i', $punches[$i + 1]) . 
								' = ' . floor($time / 3600) .':'.floor(($time % 3600)/60) . '<br />';
						}

						// Remove double punches (in for < 60 seconds)
						if($time < 60) {
							$remove[] = $i;
						} else {
							$punches['total'] += $punches[$i + 1] - $punches[$i];
						}
					}
				}

				// Remove double punches (in for < 60 seconds)
				foreach($remove as $id) {
					unset($punches[$id]);
					unset($punches[$id + 1]);
				}
				
				$summary['week'][date('W', $timestamp['timestamp'])]['total'] += $punches['total'];
			}
			return $summary;
		}

		/**
		*	Check to make sure every day has an even number of punches.
		*	Fix it if it doesn't
		*/
		protected function validate_day(&$punches) {
			if(count($punches) % 2 == 1) {
				// Missed a punch, need to have something
				$last_punch_time = $punches[count($punches) - 1];
				$this->get_shift_times($this->staff['staff_object'], $last_punch_time, $this->staff['shift_start'], $this->staff['shift_end']);
				if($last_punch_time >= $this->translate_time($this->staff['shift_start'], $last_punch_time)) {
					// Last punch of the day is after start of the shift, assume they missed the punch out
					$new_punch = $this->translate_time($this->staff['shift_end'], $last_punch_time);
				} elseif($last_punch_time <= $this->translate_time($this->staff['shift_start'], $last_punch_time)) {
					// Came in early, forgot to punch out
					$new_punch = $this->translate_time($this->staff['shift_end'], $last_punch_time);
				} else {
					// Assume they missed the punch in
					$new_punch = $this->translate_time($this->staff['shift_start'], $last_punch_time);
				}
				if($this->debug) {
					echo 'Missed a punch on ' . date('Y-m-d', $punches[count($punches) - 1]) . ', created one at ' . date('H:i', $new_punch) . '<br />';
				}
				$this->staff['exception'][] = array(
					'date'		=>	$last_punch_time,
					'severity'	=>	'warning',
					'cause'		=>	lang('missed a punch, created one at %1', date('H:i', $new_punch))
				);
				$punches[] = $new_punch;
			}

			// Remove double punches (in < 60 seconds)
			foreach($punches as $i => $punch) {
				if($punches[$i] && $punches[$i + 1]) {
					$time = ($punches[$i + 1] - $punches[$i]);
					if($time < 60 && $punches[$i + 2]) {
						unset($punches[$i + 1]);
						unset($punches[$i + 2]);
					}
				}
			}
			sort($punches);
		}

		/**
		*	Split a single day into multiple segments, based on the defined breaks
		*/
		protected function break_split(&$punches) {
			// Easy case: they only punched in and out once
			if(count($punches) == 2) {
				$in = $punches[0];
				$out = $punches[1];

				foreach($this->staff['breaks'] as $break) {
					$break_start = $this->translate_time($break['start'], $in);
					$break_end = $this->translate_time($break['end'], $in);
					if($in < ($break_start + $this->staff['tardy_tolerence']) && 
							($out + $this->staff['overtime_tolerence']) > $break_end) {
						$punches[] = $break_start;
						$punches[] = $break_end;
					}
					$in = $punches[count($punches) - 1];
				}
			}
			
			sort($punches);
		}

		/**
		*	Break out overtime into seperate blocks
		*/
		protected function add_overtime(&$time) {
			if(!is_array($time['day'])) {
				return;
			}
			foreach($time['day'] as $day_number => &$day) {
				$overtime = $day['total'] - $this->organization['max_hours_per_day'];
				if($time['week'][date('W', $day[0])]['total'] > $this->organization['max_hours_per_week'] || $day['total'] > $this->organization['max_hours_per_day']) {
					if($overtime < $this->organization['min_overtime_to_pay']) {
						continue;
					}

					$this->get_shift_times($this->staff['staff_object'], $day[0], $this->staff['shift_start'], $this->staff['shift_end']);

					// Break out overtime at early in and late out
					if($day[0] < $this->translate_time($this->staff['shift_start'], $day[0])) {
						$early_start = $day[0];
						$day[0] = $this->translate_time($this->staff['shift_start'], $day[0]);
						array_unshift($day, $day[0]);
						array_unshift($day, $early_start);
					}
					// count($day) - 2 because the total is at the end also
					if($day[count($day) - 2] > $this->translate_time($this->staff['shift_end'], $day[count($day) - 2])) {
						$late_end = $day[count($day) - 2];
						$shift_end = $this->translate_time($this->staff['shift_end'], $late_end);
						$day[count($day) - 1] = $shift_end;
						$day[] = $shift_end;
						$day[] = $late_end;
					}
					$day['overtime'] = true;
				}
			}
		}

		/**
		*	Break a list of timestamps into blocks with start & end
		*/
		protected function get_blocks($timestamps) {
			if(!is_array($timestamps['day'])) {
				return;
			}
			foreach($timestamps['day'] as $day_number => $punches) {
				$this->make_blocks($punches);
			}
		}

		protected function make_blocks($punches) {
			foreach($punches as $key => $punch) {
				if(is_numeric($key) && ($key % 2) == 0) {
					// Mangle time according to time resolution
					$start_time	=	(round($punch / $this->organization['time_resolution']) * $this->organization['time_resolution']);
					$end_time	=	(round($punches[$key + 1] / $this->organization['time_resolution']) * $this->organization['time_resolution']);

					$category	=	'regular_time';
					if($punches['overtime']) {
						if($punch < $this->translate_time($this->staff['shift_start'], $punch)) {
							$category = 'overtime';
						} elseif ($punch > $this->translate_time($this->staff['shift_end'], $punch)) {
							$category = 'overtime';
						}
					}

					$this->make_time_block($start_time, $end_time, $category);
				}
			}
		}

		protected function make_time_block($start_time, $end_time, $category) {
			// Total time in this block
			$duration = $end_time - $start_time;

			// Sanity check
			if($duration < 0) {
				return false;
			}

			// Minutes rounded according to the resolution allowed
			$quantity = (round($duration / $this->organization['time_resolution']) * $this->organization['time_resolution']) / 60;

			$data = array(
				'ts_title'	=>	$this->staff['n_fn'],
				'ts_start'	=>	$start_time,
				'ts_duration'	=>	$quantity,
				'ts_unitprice'	=>	$this->staff['pay_rate'][$category],
				'ts_quantity'	=>	$quantity / 60,
				'cat_id'	=>	$this->time_categories[$category]['timesheet_category_id'],
				'ts_owner'	=>	$GLOBALS['egw_info']['user']['account_id'],
				'#staff_id'	=>	'STAFF-' . $this->staff['staff_id']
			);

			$this->bo_timesheet->data = $data;

			if($data['ts_duration'] != 0) {
				$this->bo_timesheet->save($data, true, true);
			}
		}

		/**
		*	Translate a time in HH:mm[:ss] format into a timestamp on the given day
		*/
		public function translate_time($time, $timestamp) {
			list($hour, $minute, $second) = explode(':', $time);

			$translated = mktime(
				$hour,
				$minute,
				$second,
				date('m', $timestamp),
				date('d', $timestamp),
				date('y', $timestamp)
			);
			return $translated;
		}

		/**
		*	Get the start & end times of a shift for the given staff person on the given day
		*/
		protected function get_shift_times(Staff $staff, $day, &$start_time, &$end_time) {

			$day = mktime(0,0,0,date('m',$day),date('d',$day),date('Y',$day));

			// Work shift schedules                        
			$query = array(
				'start'         =>      $day,
				'end'		=>	$day + 24*3600,
				'date_format'   =>      'ts',
				'enum_recuring' =>      true,
				'ignore_acl'    =>      true,
				'cat_id'        =>      $this->organization['work_shift_category'],
				'users'         =>      $staff->get_staff_id(),
			);
			$work_shifts    =       ExecMethod('calendar.calendar_bo.search', $query);
			$schedules = array();
			foreach($work_shifts as $work_shift) {
				$shift_day = mktime(0,0,0,date('m',$work_shift['start']),date('d',$work_shift['start']),date('Y',$work_shift['start']));
				if($shift_day != $day) {
					continue;
				}
				if(isset($work_shift['participants'][$staff->get_staff_id()]) && $work_shift['participants'][$staff->get_staff_id()] == 'A') {
					$schedules[] = $work_shift;
				}
			}

			$start_time = date('H:i', (int)$schedules[0]['start']);
			$end_time = date('H:i', (int)$schedules[0]['end']);

			if($this->debug) {
				echo 'Shift for ' . $this->staff['n_fn'] . ' on ' . date('Y-m-d', $day) . ' is ' . $start_time . '->' . $end_time . '<br />';
			}

			// Get breaks from settings
			$this->staff['breaks'] = array();
			if(is_array($this->organization['breaks'])) {
				foreach($this->organization['breaks'] as $break) {
					$start = (int)$schedules[0]['start'];
					$end = (int)$schedules[0]['end'];
					
					do {
						$break_start = $start + ($break['work_length'] * 3600);
						$break_end = $break_start + ($break['break_length'] * 60);

						// If they're here for the break, add it in
						if($break_end < $end) {
							$this->staff['breaks'][] = array(
								'start'	=>	date('H:i', $break_start),
								'end'	=>	date('H:i', $break_end)
							);
						}
						$start = $break_start;
					} while($start < $break_start);
				}
			}
			if($this->debug) {
				echo 'Breaks:';
				_debug_array($this->staff['breaks']);
			}
		}
	}
?>
