<?php

	/**
	*	Detail that records an action taken with some goods on an RMA
	*
	*	Some overloading going on here.
	*	The detail_name is overloaded to refer to either a SerialNumberDetail's 
	*	detail_number if the item is serialized, or a quantity the action
	*	affects if the item is not serialized.
	*	Isn't weak typing fun?
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_api/inc/class.OrderLineDetail.inc.php');

	class ActionDetail extends OrderLineDetail {

		const GARBAGE = 'garbage';
		const MISSING = 'missing';
		const RETURN_TO_SUPPLIER = 'return';
		const REPAIR = 'repair';
		const RECEIVE_INTO_INVENTORY = 'inventory';

		public static $action_list = array(
			self::GARBAGE,
			self::MISSING,
			self::RETURN_TO_SUPPLIER,
			self::REPAIR,
			self::RECEIVE_INTO_INVENTORY
		);
		
		/**
		*	Get the quantity of items this action affects
		*/
		public function get_quantity() {
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $this->get_order_line()->get_stock_id());
			if($stock_item['controlled'] == 2) {
				return 1;
			} else {
				return (float)$this->detail_name;
			}
		}

		/**
		*	Set the quantity of items this action affects
		*
		*	Does nothing for serialized goods.
		*/
		public function set_quantity($quantity) {
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $this->get_order_line()->get_stock_id());
			if($stock_item['controlled'] == 2) {
				return false;
			} else {
				$this->detail_name = (float)$quantity;
				return true;
			}
		}

		/**
		*	Get the SerialNumberDetail this refers to
		*
		*	@return SerialNumberDetail
		*/
		public function get_serial_number_detail() {
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $this->get_order_line()->get_stock_id());
			if($stock_item['controlled'] == 2) {
				return $this->get_order_line()->get_detail((int)substr($this->detail_name, 3));
			} else {
				return null;
			}
		}

		/**
		*	Set the action taken
		*/
		public function set_value($value) {
			if(!in_array($value, self::$action_list)) {
				throw new egw_exception_wrong_parameter("$value is not a valid action");
			} else {
				parent::set_value($value);
			}
		}

		public function can_edit() {
			return ($this->get_order_line()->get_owner()->get_actual_delivery_date() &&
				!$this->get_order_line()->get_owner()->get_closed_date());
		}


		/**
		*	Do the action to the item
		*
		*	We'll pass the actual actions off the the BO layer, just in case we
		*	need to do the same things from somewhere else.
		*/
		public function perform_action() {
			$GLOBALS['egw']->db->transaction_begin();

			// Get a location_id for working with stock
			switch($this->get_value()) {
				// Locations from bo_perp_location
				case self::GARBAGE:
					$location_id = -3;
					break;
				case self::MISSING:
					$location_id = -1;
					break;
				case self::RETURN_TO_SUPPLIER:
				case self::REPAIR:
				case self::RECEIVE_INTO_INVENTORY:
					$location_id = $this->get_order_line()->get_owner()->get_location_id();
					break;
			}
			if(!$location_id) {
				$location_id = key(ExecMethod('perp_inventory.bo_perp_location.get_location_list'));
			}

			// Record what happened for serialized goods
			if($this->get_serial_number_detail()) {
				// Make sure the S/N has been recorded
				$location_check = array(
                                        'stock_id'	=>	$stock_id,
                                        'order'		=>	'stock_id',
                                        'serial_number'	=>	$this->get_serial_number_detail()->get_value()
                                );
                                $list = perp_inventory.bo_perp_inventory::get_controlled_inventory_list($location_check);

				// Add or move serial
				bo_perp_inventory::add_serial(
					$this->get_order_line()->get_stock_id(),
					$this->get_serial_number_detail()->get_value(),
					$this->get_quantity(),
					$location_id
				);
				bo_perp_stock_history::add_history(
					$this->get_order_line()->get_stock_id(),
					$this->get_serial_number_detail()->get_value(),
					lang($this->get_value()),
					array(
						'app'	=>	'perp_warranty',
						'id'	=>	'RMA-' . $this->get_order_line()->get_owner()->get_rma_id()
					),
					__LINE__, __FILE__
				);
			}
			
			switch ($this->get_value()) {
				case self::GARBAGE:
				case self::MISSING:
					// Nothing to do, just throw it out.
					break;
				case self::RETURN_TO_SUPPLIER:
					// Reserve it so nobody else thinks it's available
					ExecMethod2(
						'perp_inventory.bo_perp_inventory.reserve_stock',
						$location_id,
						$this->get_order_line()->get_stock_id(),
						$this->get_quantity(),
						lang('Damaged goods to be returned to supplier')
					);
					// Have to put it back into inventory so it's there when we return it.
					// Fall through
				case self::REPAIR:
				case self::RECEIVE_INTO_INVENTORY:
					// Not capable of recording the damage, or telling it apart
					if($this->get_serial_number_detail()) {
						$serials = array(
							$this->get_serial_number_detail()->get_value() => 1
						);
					} else {
						$serials = array();
					}
					$unit_cost = ExecMethod2(
						'perp_inventory.bo_perp_inventory.get_cost',
						$this->get_order_line()->get_stock_id(),
						$location_id,
						1
					);
					
					// GLTransaction
					$config = config::read('perp_warranty');
					$warranty_part_account = $config['part_account'];
					$inventory_account = ExecMethod2(
						'perp_inventory.bo_perp_stock.get_gl_accounts',
						$this->get_order_line()->get_stock_id(),
						bo_perp_inventory::ON_HAND
					);

					$trans = new GLTransaction();
					$narrative = lang('Part %1 returned from %2', 
						ExecMethod('perp_inventory.bo_perp_stock.link_title', $this->get_order_line()->get_stock_id()),
						bo_perp_rma::link_title($this->get_order_line()->get_owner()->get_rma_id())
					);
					$trans->add_line($inventory_account, $narrative, $unit_cost * $this->get_quantity());
					$trans->add_line($warranty_part_account, $narrative, -$unit_cost * $this->get_quantity());
					$trans->write();

					// Into inventory
					$stock_cost_id = ExecMethod2('perp_inventory.bo_perp_inventory.add_stock_cost',
						$this->get_order_line()->get_stock_id(),
						$this->get_quantity(),
						$unit_cost,
						0
					);
					ExecMethod2('perp_inventory.bo_perp_inventory.receive', 
						$this->get_order_line()->get_stock_id(),
						$location_id,
						$this->get_quantity(),
						$stock_cost_id,
						$serials,
						bo_perp_rma::link_title($this->get_order_line()->get_owner()->get_rma_id()),
						3 // Received
					);

					// If it's associated with a work order, make it re-sellable
					if($this->get_order_line()->get_owner()->get_claim_id()) {
						$claim = new ClientClaim($this->get_order_line()->get_owner()->get_claim_id());
						$parent = $claim->get_parent();
						$parent_item = $parent->get_component_by_stock_id($this->get_order_line()->get_stock_id());
						$work_order = $claim->get_work_order_keys();
						$serial_numbers = $parent_item->get_serial_numbers();
						if($work_order && $parent_item->get_stock_id() == $work_order['stock_id'] && $work_order['serial_number'] == $serial_numbers[0]['serial_number']) {
							bo_perp_sales_order::return_work_order(
								$this->get_order_line()->get_owner()->get_client_id(),
								$work_order['work_order_id'],
								$work_order['detail_id'],
								$this->get_quantity()
							);
						}
					}
					break;
			}

			$GLOBALS['egw']->db->transaction_commit();
		}

		public function toArray() {
			return parent::toArray() + array(
				'action'	=>	$this->get_value()
			);
		}
	}
?>
