<?php
	/**
	*	An item to be returned
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_api/inc/class.OrderLine.inc.php');

	// Make sure these are defined
	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.SerialNumberDetail.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.ActionDetail.inc.php');

	class RMALine extends OrderLine {
		
		protected $quantity_returned = null;

		public function __construct(RMA &$parent, Array $values) {
			parent::__construct($parent, $values);

			$this->set_quantity_returned($values['quantity_returned']);

			$this->set_serial_numbers($values['serial_numbers']);
		}

		public function &get_rma() {
			return $this->order;
		}

		public function get_stock_id() {
			return $this->stock_id;
		}

		public function get_quantity_returned() {
			return $this->quantity_returned;
		}
		public function set_quantity_returned($quantity) {
			$this->quantity_returned = $quantity;
		}

		public function set_serial_numbers($serial_numbers) {
			if(!is_array($serial_numbers)) {
				return false;
			}

			// Clear old
			$serials =& $this->get_details('SerialNumberDetail');
			foreach($serials as &$detail) {
				$key = array_search($detail->get_value(), $serial_numbers);
				if($key === false) {
					$this->remove_detail($detail->get_number());
				} else {
					// Already there, don't add it again
					unset($serial_numbers[$key]);
				}
			}

			foreach($serial_numbers as $number) {
				$detail =& new SerialNumberDetail($this, array(
					'detail_name'	=>	'serial_number',
					'detail_value'	=>	$number,
				));
				$this->add_detail($detail);
			}
		}

		public function get_serial_numbers() {
			$serials = array();
			$serial_details =& $this->get_details('SerialNumberDetail');
			foreach($serial_details as &$detail) {
				$serials[] = $detail->get_value();
			}
			return $serials;
		}

		/**
		*	Can this Line be cancelled?
		*
		*	Overridden so we can check the correct ACL
		*/
		public function can_cancel() {
			if(!$GLOBALS['egw']->acl->check('rma', EGW_ACL_EDIT | EGW_ACL_CUSTOM_1, 'perp_warranty')) {
				return false;
			}

			if($this->get_quantity_invoiced() != 0 || $this->get_quantity_shipped() != 0) {
				// Already shipped or invoiced
				return false;
			}
			/*
			* Now we check for shipped + returned, or invoice + credit.  We still need to keep
			* the line for reference. 
			*/
			if(count($this->get_status())) {
				// Something happened to this line.  Keep it.
				return false;
			}
			
			return true;
		}

		/**
		*	Is this line complete?
		*/
		public function is_complete() {
			return ($this->get_quantity_ordered() == $this->get_quantity_shipped() && $this->get_rma()->get_supplier_id() != 0) ||
				($this->get_quantity_ordered() == $this->get_quantity_received() && $this->get_rma()->get_client_id() != 0);
				
		}

		/**
		*	Override parent to remove restriction on having to be shipped first
		*/
		public function can_receive() {
			return !$this->is_complete();
		}

		public function __toString() {
			$line = sprintf("%-50.50s \t %2.3f / %2.3f\n", 
				ExecMethod('perp_inventory.bo_perp_stock.link_title', $this->stock_id), 
				$this->get_quantity_ordered(),
				$this->get_quantity_returned()
			);
			if($this->get_serial_numbers()) {
				foreach($this->get_serial_numbers() as $number) {
					$line .= $number . "\n";
				}
			}
			return $line;
		}

		public function toArray() {
			$data = parent::toArray();
			$data['quantity_returned'] = $this->quantity_returned;
			return $data;
		}
	}
?>
