<?php

	/**
	*	Logic layer for warranty claims.
	*
	*	This is the base class for warranty, and covers the commonalities between
	*	claims from our clients, and claims to our suppliers.
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.so_perp_claim.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.Claim.inc.php');
	
	require_once( EGW_INCLUDE_ROOT . '/tracker/inc/class.tracker_bo.inc.php');

	class bo_perp_claim {
	
		public static $resolutions = array(
			Claim::UNRESOLVED	=>	'None',
			Claim::ACCEPTED		=>	'Accepted',
			Claim::REJECTED		=>	'Rejected',
			Claim::DUPLICATE	=>	'Duplicate',
			Claim::INVALID		=>	'Invalid',
		);

		/**
		*	Get a list of claims, according to the conditions provided in the query array.
		*
		*	@nextmatch
		*
		*	Extra filters:
		*		- client_id
		*		- supplier_id
		*/
		public static function get_claim_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				$rows = array();
				return 0;
			}

			// Restrict clients to only their claims
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids)) {
				$query['client_id'] = $client_ids;
			}
		
			// Restrict suppliers to only their claims
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids)) {
				$query['supplier_ids'] = $supplier_ids;
			}

			if(!$query['order']) {
				$query['order'] = 'perp_warranty_claim.claim_id';
			}

			$rows = so_perp_claim::get_claim_list($query);
			$count = $query['total'];

			$issue_query = $query;
			$issue_query['order'] = 'perp_warranty_claim_issue.claim_id';

			foreach($rows as &$row) {
				// Add in issues
				$issue_query['claim_id'] = $row['claim_id'];
				bo_perp_claim_issue::get_claim_issues($issue_query, $row['issues']);
				$row['parent'] = so_perp_claim::get_parent($row['claim_id']);
			}

			return $count;
		}

		/**
		*	Get a list of possible resolutions for a claim (issue)
		*
		*/
		public static function get_resolution_list() {
			return self::$resolutions;
		}

		/**
		*	If the claim is for one or more issues on a product (PARENT), get that ClaimIssue
		*
		*	@param claim_id ID of a claim
		*
		*	@return array
		*/
		public static function get_parent($claim_id) {
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				return null;
			}
			$data = so_perp_claim::get_parent($claim_id);
			$data['components'][] = array(
				'stock_id'	=>	$data['stock_id'],
			);
			return new ClaimIssue($data);
		}

		/**
		*	Participate in linking
		*/
		public static function link_title($entry) {
			return lang('Claim #%1', $entry);
		}

		/**
		*	Create a {Purchase|Sales}Order for the parts
		*
		*	@param claim_id
		*	@param order
		*/
		protected static function create_order(&$claim, Order &$order) {

			$line_class = get_class($order) . 'Line';

			$order->set_client_id((int)$claim->get_client_id());
			$order->set_supplier_id((int)$claim->get_supplier_id());
			$order->set_comments(lang('Warranty replacements from %1', self::link_title($claim->get_claim_id())));
			$order->set_our_reference_number($order->suggest_order_number());

			// Get list of issues
			$query = array(
				'claim_id'	=>	$claim->get_claim_id(),
				'num_rows'	=>	-1,
			);
			bo_perp_claim_issue::get_claim_issues($query, $issues);
			foreach($issues as &$issue) {
				if($issue['issue_id'] && $issue['resolution'] == Claim::ACCEPTED) {
					$issue['replacements'] = bo_perp_claim_issue::get_replacements($issue['claim_issue_id']);

					foreach($issue['replacements'] as $replacement) {
						$line_data = array(
							'stock_id'		=>	$replacement['replacement_stock_id'],
							'quantity_ordered'	=>	$replacement['replacement_quantity'],
						);
						$line =& new $line_class($order, $line_data);
						$order->add_line($line);
					}
				}
			}

			return $order->write();
		}

		/**
		*	Create a [credit] invoice for the claim
		*
		*	This should be called from the extending child, which will provide the correct type of invoice
		*/
		protected static function enter_credit(&$claim, Invoice &$invoice) {

			$invoice->read_from_object($claim);
			$invoice->set_bill_date($claim->get_claim_date());
			$invoice->set_due_date(time());

			$invoice->set_comments(lang('Warranty credit from %1', self::link_title($claim->get_claim_id())));

			// Don't write here, and don't return the invoice ID, we can just re-direct to the unsaved invoice
		}

		/**
		*	Get the [credit] invoice for a claim
		*/
		public static function get_claim_invoice_id($claim_id) {
			return so_perp_claim::get_claim_invoice_id($claim_id);
		}
	}
?>
