<?php

	/**
	*	Logic layer for dealing with issues on a warranty claim
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.ClaimIssue.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/tracker/inc/class.tracker_bo.inc.php');

	class bo_perp_claim_issue {

		protected static $so;
		public function __construct() {
			self::static_init();
		}

		protected function static_init() {
			if(!is_object(self::$so)) {
				self::$so = new so_perp_claim_issue();
			}
		}

		public static function get_claim_issues(&$query, &$rows = array(), &$readonly = array()) {
			self::static_init();
			
			$rows =& self::$so->get_claim_issues($query);
			
			// Shift one to allow for header and keep the readonly in sync)
			foreach($rows as $key => $row) {
			//	$readonly[$key + 1]['status'] = false;
			}

			return $query['total'];
		}

		/**
		*	Read and return information to populate a ClaimIssue
		*
		*	@param claim_issue_id integer
		*
		*	@return array
		*/
		public static function read($claim_issue_id) {
			// Check permissions
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				throw new egw_exception_no_permission();
			}
			
			$claim_issue = so_perp_claim_issue::read($claim_issue_id);
			
			// Get components
			$claim_issue['components'] = self::get_components($claim_issue);

			return $claim_issue;
		}

		/**
		*	Store a ClaimIssue
		*
		*	@param issue A ClaimIssue to store
		*
		*	@return int the claim_issue_id
		*/
		public static function write(ClaimIssue $issue) {
			// Check permissions
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_EDIT, 'perp_warranty')) {
				throw new egw_exception_no_permission();
			}
			
			// Do some checks

			$claim_issue_id = so_perp_claim_issue::write($issue);

			// Tracker
			if($issue->get_issue_id() && !$issue->get_tracker_id()) {
				self::create_tracker($issue);
			}

			self::set_components($claim_issue_id, $issue);

			return $claim_issue_id;
		}

		/**
		*	Remove a claim issue
		*/
		public static function delete($claim_issue_id) {
			// Check permissions
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_EDIT, 'perp_warranty')) {
				throw new egw_exception_no_permission();
			}

			$claim_issue = self::read($claim_issue_id);
			so_perp_claim_issue::delete($claim_issue_id);

			
			$tracker_bo = new tracker_bo();
			$tracker_bo->delete($claim_issue['tracker_id']);
		}

		/**
		*	Create a new tracker entry for this claim issue
		*/
		protected static function create_tracker(ClaimIssue &$claim_issue) {
			// Get settings
			$config = config::read('perp_warranty');

			$tracker = array(
				'tr_tracker'	=>	$config['tracker_queue'],
				'tr_summary'	=>	self::link_title($claim_issue->get_claim_issue_id()),
				'tr_status'	=>	tracker_bo::STATUS_OPEN,
				'tr_private'	=>	0,
				
			);
			$tracker_bo = new tracker_bo();
			$tracker_bo->save($tracker);

			$tracker_id = $tracker_bo->data['tr_id'];
			
			// Create link
			$link_id = 'CLAIM_ISSUE-' . $claim_issue->get_claim_issue_id();
			egw_link::link('perp_warranty', $link_id, 'tracker', $tracker_id);
		}		

		/**
		*	Get a list of components
		*/
		public static function get_components(Array $claim_issue) {
			$components = so_perp_claim_issue::get_components($claim_issue['claim_issue_id']);

			return $components;
		} 

		/**
		*	Set the components
		*/
		protected static function set_components($claim_issue_id, ClaimIssue &$claim_issue) {
			$components =& $claim_issue->get_components();

			foreach($components as &$component) {
				so_perp_claim_issue::set_component($claim_issue_id, $component);
			}
		}

		/**
		*	Get a list of replacement components
		*/
		public static function get_replacements($claim_issue_id) {
			return so_perp_claim_issue::get_replacements($claim_issue_id);
		}

		/**
		*	Return a title to participate in linking
		*
		*/
		public static function link_title($id) {
			list($claim_issue_id, $stock_id) = explode('-', $id);
			$claim_issue = self::read($claim_issue_id);
			$title = '#' . $claim_issue['claim_issue_id'] . ' - ' . bo_perp_issue::link_title($claim_issue['issue_id']);

			if($stock_id) {
				$title .= ' - ' . ExecMethod('perp_inventory.bo_perp_stock.link_title', $stock_id);
			}

			return $title;
		}
	}
?>
