<?php

	/**
	*	Logic layer for Return Materials Authorizations (RMAs)
	*
	*	An RMA is an authorization to return some good (for credit).
	*	Goods returned without authorization are usually discarded or returned.
	*
	*	@package perp_warranty
	*	@author Nathan Gray
	*/

	class bo_perp_rma {

		/**
		*	Stub to work with the existing Order framework
		*/
		public static function get_order_list(&$query, &$rows, &$readonlys) {
			$query['order_class'] = 'RMA';
			return self::get_rma_list($query, $rows, $readonlys);
		}

		/**
		*	Get a list of RMA information
		*/
		public static function get_rma_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$query['order_class'] = 'RMA';
		
			if(!$GLOBALS['egw']->acl->check('rma', EGW_ACL_READ, 'perp_warranty')) {
				$rows = array();
			}

			if(!$query['order']) {
				$query['order'] = 'rma_id';
			}

			$list =& so_perp_rma::get_rma_list($query);

			$user = $GLOBALS['egw_info']['user']['account_id'];
			foreach($list as &$row) {
				$row['formatted_total'] = ExecMethod2('perp_api.bo_perp_api.get_formatted_price', $row['order_currency'], $row['total']);

				$rows[] =& $row;
				$order = new $query['order_class']($row['rma_id']);
				$complete = $order->is_complete();

				$readonlys['read[' .$row['rma_id'].']'] = !$read;
				$readonlys['edit[' .$row['rma_id'].']'] = !($order->is_editor($user)) || $complete;
				$readonlys['approve[' . $row['rma_id'] . ']'] = !$order->is_admin($user) || $order->get_approval_date() || $order->approval_check() !== true || !$order->get_approval_required();
				$readonlys['deny[' . $row['rma_id'] . ']'] = $readonlys['approve[' . $row['rma_id'] . ']'];
				$readonlys['ship[' . $row['rma_id'] . ']'] = ($order->can_ship() !== true);
				$readonlys['receive[' . $row['rma_id'] . ']'] =!$order->can_receive();
				$readonlys['invoice[' . $row['rma_id'] . ']'] =!($order->can_invoice() === true);
				$readonlys['cancel[' . $row['rma_id'] . ']'] = $order->can_cancel($user) !== true;

				if(!$complete && ($order->requires_approval() && $order->get_approval_date() || !$order->requires_approval())) {
					$ship_warnings = $order->can_ship();
					$readonlys['ship_warning_icon[' . $row['rma_id'] . ']'] = !($order->is_editor($user)) || is_bool($ship_warnings);
					if(!is_bool($ship_warnings)) {
						$row['ship_warning'] = $ship_warnings;
					}
				} else {
					$readonlys['ship_warning_icon[' . $row['rma_id'] . ']'] = true;
				}
			}

			return $query['total'];
		}

		/**
		*	Get the RMA ID for a particular claim
		*/
		public static function get_claim_rma_id($claim_id) {
			$query = array(
				'claim_id'	=>	$claim_id
			);
			self::get_rma_list($query, $rows);
			return $rows[0]['rma_id'];
		}

		/**
		*	Create a RMA from a Claim
		*
		*	All components / items that are to be returned will be added to a new RMA.
		*	The RMA will be returned.
		*
		*	@param claim_id ID of the claim to base the RMA on
		*
		*	@return RMA
		*/
		public static function &create_from_claim($claim_id) {
			$data = array(
				'claim_id'	=>	$claim_id
			);

			$rma =& new RMA($data);
			$claim =& new Claim($claim_id);
			$rma->set_supplier_id((int)$claim->get_supplier_id());
			$rma->set_client_id((int)$claim->get_client_id());

			// These are opposite!  The ship to is the one that the bill is from, it's a return
			$rma->set_ship_to_address((int)$claim->get_bill_from_address());
			$rma->set_ship_from_address((int)$claim->get_bill_to_address());
			$rma->set_our_reference_number($rma->suggest_order_number());

			$query = array(
				'claim_id'	=>	$claim_id,
				'num_rows'	=>	-1
			);
			bo_perp_claim_issue::get_claim_issues($query, $issues);

			foreach($issues as $issue) {
				$issue = bo_perp_claim_issue::read($issue['claim_issue_id']);

				foreach($issue['components'] as $component) {
					if($component['return_parts']) {
						// Using the stock_id as the key lets us merge multiple components with the same stock id together
						$lines[$component['stock_id']]['stock_id'] = $component['stock_id'];
						$lines[$component['stock_id']]['description'] = ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['stock_id']);
						$lines[$component['stock_id']]['quantity_ordered'] += $component['quantity_claimed'];
						
						if(is_array($component['serial_numbers'])) {
							foreach($component['serial_numbers'] as $values) {
								$lines[$component['stock_id']]['serial_numbers'][] = $values['serial_number'];
							}
						}
					}
				}
			}
			foreach($lines as $line_data) {
				$rma_line =& new RMALine($rma, $line_data);
				$rma->add_line($rma_line);
			}
			return $rma;
		}

		/**
		*	Get a list of items on an RMA
		*
		*	@nextmatch
		*
		*	Extra keys: 
		*	- rma_id
		*/
		public static function get_rma_stock(&$query = array(), &$rows = array(), &$readonly = array()) {
			if(!$GLOBALS['egw']->acl->check('rma', EGW_ACL_READ, 'perp_warranty')) {
				$rows = array();
				return 0;
			}
			
			$rows =& so_perp_rma::get_rma_stock($query);
	
			return $query['total'];
		}

		/**
		*	Handy link title
		*/
		public static function link_title($entry) {
			$rma = so_perp_rma::read($entry);
			return lang('RMA %1', $rma['our_reference_number']);
		}
	}
?>
