<?php
	class so_perp_claim {

		public static function get_claim_list(&$query) {
			$field_list = array(
				'claim_id',
				'supplier_id',
				'client_id',
				'perp_warranty_claim_issue.claim_issue_id'
			);

			$where = array();
			$join = 'JOIN perp_warranty_claim_issue ON
					perp_warranty_claim_issue.claim_id = perp_warranty_claim.claim_id
				LEFT JOIN egw_links ON (
						CONCAT(\'CLAIM_ISSUE-\', perp_warranty_claim_issue.claim_issue_id) = egw_links.link_id1
						AND egw_links.link_app1 = \'perp_warranty\' 
						AND egw_links.link_app2 = \'tracker\'
					) OR (
						CONCAT(\'CLAIM_ISSUE-\', perp_warranty_claim_issue.claim_issue_id) = egw_links.link_id2
						AND egw_links.link_app2 = \'perp_warranty\' 
						AND egw_links.link_app1 = \'tracker\'
					)
				LEFT JOIN egw_tracker ON
					(egw_links.link_app2 = \'tracker\' AND egw_links.link_id2 = egw_tracker.tr_id )
					OR (egw_links.link_app1 = \'tracker\' AND egw_links.link_id1 = egw_tracker.tr_id) 
				LEFT JOIN egw_tracker_assignee ON
					egw_tracker.tr_id = egw_tracker_assignee.tr_id
			';

			$order_by = "ORDER BY {$query['order']} {$query['sort']}";
			$group_by = array('perp_warranty_claim.claim_id');

			// Mess with the field list to change it into database columns
			if($query['selectcols']) {
//				$field_list = array_merge($field_list, explode(',',$query['selectcols']));
			}

			// claim_id is ambiguous, replace it with tablename.claim_id
			$field_list[array_search('claim_id', $field_list)] = 'perp_warranty_claim.claim_id';
			
			// Nextmatch header runs these together
			if(array_search('status_resolution', $field_list)) {
				unset($field_list[array_search('status_resolution', $field_list)]);
				$field_list[] = 'tr_status AS status';
				$field_list[] = 'tr_resolution AS resolution';
			}
			if(array_search('created_by_assigned_to', $field_list)) {
				unset($field_list[array_search('created_by_assigned_to', $field_list)]);
				$field_list[] = 'created_by';
				$field_list[] = 'tr_assigned AS assigned_to';
			}
			if(array_search('claim_date_last_contact', $field_list)) {
				unset($field_list[array_search('claim_date_last_contact', $field_list)]);
				$field_list[] = 'claim_date';
				$field_list[] = 'last_contact';
			}

			// Find out the last comment on the tracker ticket
			if(array_search('last_contact', $field_list)) {
				unset($field_list[array_search('last_contact', $field_list)]);
				$field_list[] = 'MAX(reply_created) AS last_contact';
				$join .= 'LEFT JOIN egw_tracker_replies ON
						egw_tracker_replies.tr_id = egw_tracker.tr_id ';
				$group_by[] = 'egw_tracker.tr_id';
			}
			
			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%'.$query['search'].'%');
				$where[] = "(perp_warranty_claim.claim_id LIKE $search 
					OR perp_warranty_claim_issue.claim_issue_id LIKE $search)";
			}
			

			if($query['filter'] == 'supplier') {
				$where[] = 'supplier_id != 0';
			} elseif($query['filter'] == 'client') {
				$where[] = 'client_id != 0';
			}
			if($query['col_filter']['supplier_id']) {
				$where[] = 'supplier_id = ' . (int)$query['col_filter']['supplier_id'];
			}
			if($query['col_filter']['client_id']) {
				$where[] = 'client_id = ' . (int)$query['col_filter']['client_id'];
			}
			if($query['col_filter']['issue_id']) {
				$where[] = 'issue_id = ' . (int)$query['col_filter']['issue_id'];
			}
			if($query['col_filter']['status']) {
				$where[] = 'tr_status = ' . (int)$query['col_filter']['status'];
			}
			if($query['col_filter']['resolution']) {
				$where[] = 'tr_resolution = ' . $GLOBALS['egw']->db->quote($query['col_filter']['resolution']);
			}
			if($query['col_filter']['created_by']) {
				$where[] = 'egw_tracker.tr_creator = ' . (int)$query['col_filter']['created_by'];
			}
			if($query['col_filter']['assigned_to']) {
				$where[] = 'egw_tracker_assignee.tr_assigned = ' . (int)$query['col_filter']['assigned_to'];
			}

			// User wants to restrict by stock category, join up to components / replacements
			if($query['cat_id']) {
				$join .=' JOIN (
						perp_stock 
						LEFT JOIN perp_warranty_claim_component ON
							perp_warranty_claim_component.component_id = perp_stock.stock_id
							AND perp_stock.stock_category = ' . (int)$query['cat_id'] . '
						LEFT JOIN perp_warranty_claim_replacement ON
							perp_warranty_claim_replacement.stock_id = perp_stock.stock_id
							AND perp_stock.stock_category = ' . (int)$query['cat_id'] . '
					) ON
						perp_warranty_claim_component.claim_issue_id = perp_warranty_claim_issue.claim_issue_id
						OR perp_warranty_claim_replacement.claim_issue_id = perp_warranty_claim_issue.claim_issue_id ';
			}

			// Get count
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_warranty_claim',
				'count(perp_warranty_claim.claim_id)',
				$where,
				__LINE__, __FILE__, false,
				($group_by ? 'GROUP BY ' . implode(',', $group_by) : '') . ' ' . $order_by,
				'perp_warranty', 1,
				$join
			)->fetchSingle();

			// Get results
			$result =& $GLOBALS['egw']->db->select(
				'perp_warranty_claim',
				$field_list,
				$where,
				__LINE__, __FILE__, $query['start'],
				($group_by ? 'GROUP BY ' . implode(',', $group_by) : '') . ' ' . $order_by,
				'perp_warranty', $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				$row['claim_date'] = $GLOBALS['egw']->db->from_timestamp($row['claim_date']);
				$row['last_contact'] = $GLOBALS['egw']->db->from_timestamp($row['last_contact']);
				$list[] = $row;
			}
			
			return $list;			
		}

		/**
		*	If a claim has multiple issues on a single stock item (something we made),
		*	get the single stock item
		*/
		public static function get_parent($claim_id) {
			$row = $GLOBALS['egw']->db->select(
				'perp_warranty_claim_issue',
				array(
					'perp_warranty_claim_issue.*',
					'perp_warranty_claim_component.*',
					'perp_warranty_claim_serial.serial_number',
					'perp_stock.*'
				),
				array(
					'claim_id'	=>	(int)$claim_id,
					'issue_id'	=>	bo_perp_issue::PARENT,
				),
				__LINE__, __FILE__, false,
				'', 'perp_manufacturing', 1,
				'JOIN perp_warranty_claim_component ON
					perp_warranty_claim_component.claim_issue_id = perp_warranty_claim_issue.claim_issue_id
				LEFT JOIN perp_warranty_claim_serial ON
					perp_warranty_claim_component.claim_issue_id = perp_warranty_claim_serial.claim_issue_id
					AND perp_warranty_claim_component.component_id = perp_warranty_claim_serial.stock_id
					AND perp_warranty_claim_serial.sent_or_received = 0
				JOIN perp_stock ON
					perp_warranty_claim_component.component_id = perp_stock.stock_id'
			)->fetch();
			return $row;
		}


		/**
		*	Get the invoice ID for a claim
		*
		*	Only 1 invoice per claim
		*
		*	@param claim_id
		*
		*	@return int Invoice ID, or null if not invoiced yet
		*/
		public static function get_claim_invoice_id($claim_id) {
			return $GLOBALS['egw']->db->select(
				'perp_warranty_credit',
				array(
					'invoice_id',
				),
				array(
					'claim_id'		=>	$claim_id,
				),
				__LINE__, __FILE__, false,
				'', 'perp_warranty', 1, 
				'JOIN ' . InvoiceLine::invoice_line_table . ' ON
					' . InvoiceLine::invoice_line_table . '.invoicable_id = perp_warranty_credit.invoicable_id'
			)->fetchSingle();
		}
	}
?>
