<?php

	/**
	*	Storage layer for dealing with a claim's issues
	*/


	class so_perp_claim_issue {

		public function __construct() {
		}

		public function get_claim_issues(&$query) {
			$fields = array(
				'claim_issue_id',
				'claim_id',
				'perp_warranty_claim_issue.issue_id',
				'claim_amount',
				'tr_modified AS modified',
				'tr_modifier AS modified_by',
				'tr_status AS status',
				'tr_resolution AS resolution',
				'tr_creator AS created_by',
				'tr_assigned AS assigned_to',
			);

			$where = array(
				'claim_id'	=>	$query['claim_id']
			);
			$join = 'LEFT JOIN perp_warranty_issue ON
					perp_warranty_issue.issue_id = perp_warranty_claim_issue.issue_id
				LEFT JOIN egw_links ON (
						CONCAT(\'CLAIM_ISSUE-\', perp_warranty_claim_issue.claim_issue_id) = egw_links.link_id1
						AND egw_links.link_app1 = \'perp_warranty\'
					AND egw_links.link_app2 = \'tracker\'
					) OR (
						CONCAT(\'CLAIM_ISSUE-\', perp_warranty_claim_issue.claim_issue_id) = egw_links.link_id2
						AND egw_links.link_app2 = \'perp_warranty\'
						AND egw_links.link_app1 = \'tracker\'
					)
				LEFT JOIN egw_tracker ON
					(egw_links.link_app2 = \'tracker\' AND egw_links.link_id2 = egw_tracker.tr_id )
					OR (egw_links.link_app1 = \'tracker\' AND egw_links.link_id1 = egw_tracker.tr_id)
				LEFT JOIN egw_tracker_assignee ON
					egw_tracker.tr_id = egw_tracker_assignee.tr_id
			';
			$order = 'ORDER BY ' . ($query['order'] ? $query['order'] : 'claim_issue_id') . ' ' . $query['sort'];

			$group_by = 'GROUP BY perp_warranty_claim_issue.claim_issue_id';

			if($query['hide_parent']) {
				$where[] = 'perp_warranty_claim_issue.issue_id != ' . bo_perp_issue::PARENT;
			}

			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_warranty_claim_issue',
				'count(claim_issue_id)',
				$where,
				__LINE__, __FILE__, false,
				$group_by, 'perp_warranty', -1,
				$join
			)->NumRows();

			$result =& $GLOBALS['egw']->db->select(
				'perp_warranty_claim_issue',
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				$group_by . ' ' . $order, 'perp_warranty', $query['num_rows'],
				$join
			);

			$list = array();
			foreach($result as $row) {
				if(!$row['resolution']) {
					$row['resolution'] = Claim::UNRESOLVED;
				}
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Read a ClaimIssue from the DB
		*/
		public static function read($id) {
			$join = 'LEFT JOIN egw_links ON (
						CONCAT(\'CLAIM_ISSUE-\', perp_warranty_claim_issue.claim_issue_id) = egw_links.link_id1
						AND egw_links.link_app1 = \'perp_warranty\'
					AND egw_links.link_app2 = \'tracker\'
					) OR (
						CONCAT(\'CLAIM_ISSUE-\', perp_warranty_claim_issue.claim_issue_id) = egw_links.link_id2
						AND egw_links.link_app2 = \'perp_warranty\'
						AND egw_links.link_app1 = \'tracker\'
					)
				LEFT JOIN egw_tracker ON
					(egw_links.link_app2 = \'tracker\' AND egw_links.link_id2 = egw_tracker.tr_id )
					OR (egw_links.link_app1 = \'tracker\' AND egw_links.link_id1 = egw_tracker.tr_id)';

			$row = $GLOBALS['egw']->db->select(
				'perp_warranty_claim_issue',
				array(
					'perp_warranty_claim_issue.*',
					'egw_tracker.tr_id AS tracker_id'
				),
				array(
					'claim_issue_id'	=>	$id
				),
				__LINE__, __FILE__, false,
				'', 'perp_warranty', 1,
				$join
			)->fetch();

			$row['components'] = self::get_components($id);

			return $row;
		}

		public static function write(ClaimIssue &$issue) {
			
			$GLOBALS['egw']->db->insert(
				'perp_warranty_claim_issue',
				array(
					'claim_id'	=>	$issue->get_claim_id(),
					'issue_id'	=>	$issue->get_issue_id(),
					'claim_amount'	=>	$issue->get_claim_amount()
				),
				array(
					'claim_issue_id'	=>	$issue->get_claim_issue_id()
				),
				__LINE__, __FILE__,
				'perp_warranty'
			);
			if(!$issue->get_claim_issue_id()) {
				$id = $GLOBALS['egw']->db->get_last_insert_id('perp_warranty_claim_issue', 'claim_issue_id');
			} else {
				$id = $issue->get_claim_issue_id();
			}

			return $id;
		}

		public static function delete($claim_issue_id) {
			$GLOBALS['egw']->db->delete(
				'perp_warranty_claim_issue',
				array(
					'claim_issue_id'	=>	$claim_issue_id
				),
				__LINE__, __FILE__,
				'perp_warranty'
			);
				
		}

		public static function get_components($claim_issue_id) {
			$join = 'JOIN perp_stock ON
				perp_stock.stock_id = perp_warranty_claim_component.component_id
				JOIN perp_unit_of_measure ON
					perp_unit_of_measure.uom_id = perp_stock.unit_of_measure';

			$result =& $GLOBALS['egw']->db->select(
				'perp_warranty_claim_component',
				array(
					'claim_issue_id',
					'component_id AS stock_id',
					'quantity_claimed',
					'return_parts',
					'replace_parts',
					'retrofits_allowed',
					'image_required',
					'coverage',
					'credit',
					'decimal_places',
					'uom_name AS unit_of_measure_name'
				),
				array(
					'claim_issue_id'	=>	$claim_issue_id
				),
				__LINE__, __FILE__, false,
				'ORDER BY component_id', 'perp_warranty', -1,
				$join
			);
			$list = array();
			foreach($result as $row) {
				$row['quantity_claimed'] = round($row['quantity_claimed'], $row['decimal_places']);
				$list[] = $row;
			}

			// Look for serial numbers
			$join = 'LEFT JOIN perp_warranty_claim_replacement ON
				perp_warranty_claim_serial.claim_issue_id = perp_warranty_claim_replacement.claim_issue_id
				AND perp_warranty_claim_serial.stock_id = perp_warranty_claim_replacement.original_stock_id
			LEFT JOIN perp_warranty_claim_serial AS replacement_serial ON
				perp_warranty_claim_replacement.stock_id = replacement_serial.stock_id 
				AND perp_warranty_claim_serial.claim_issue_id = replacement_serial.claim_issue_id
				AND replacement_serial.sent_or_received = 1';

			foreach($list as &$row) {
				$result =& $GLOBALS['egw']->db->select(
					'perp_warranty_claim_serial',
					array(
						'perp_warranty_claim_serial.serial_number',
						'perp_warranty_claim_serial.serial_quantity',
						'replacement_serial.serial_number AS replacement'
					),
					array(
						'perp_warranty_claim_serial.claim_issue_id = ' . (int)$claim_issue_id,
						'perp_warranty_claim_serial.stock_id = ' . (int)$row['stock_id'],
						'perp_warranty_claim_serial.sent_or_received = 0'
					),
					__LINE__, __FILE__, false,
					'', 'perp_warranty', -1,
					$join
				);
				foreach($result as $serial) {
					$row['serial_numbers'][] = $serial;
				}
			}
			return $list;
		}
				

		public static function set_component($claim_issue_id, &$component) {
			$component_array = $component->toArray();
			$GLOBALS['egw']->db->insert(
				'perp_warranty_claim_component',
				array(
					'claim_issue_id'	=>	$claim_issue_id,
					'component_id'		=>	$component->get_stock_id(),
					'quantity_claimed'	=>	$component->get_quantity_claimed(),
					'return_parts'		=>	$component_array['return_parts'],
					'replace_parts'		=>	$component_array['replace_parts'],
					'retrofits_allowed'	=>	$component_array['retrofits_allowed'],
					'image_required'	=>	$component_array['image_required'],
					'coverage'		=>	$component_array['coverage'],
					'credit'		=>	$component_array['credit']
				),
				array(
					'claim_issue_id'	=>	$claim_issue_id,
					'component_id'		=>	$component->get_stock_id(),
				),
				__LINE__, __FILE__,
				'perp_warranty'
			);

			if($component->get_serial_numbers()) {
				foreach($component->get_serial_numbers() as $values) {
					if(!$value['serial_number']) {
						continue;
					}
					$GLOBALS['egw']->db->insert(
						'perp_warranty_claim_serial',
						array(
							'claim_issue_id'	=>	$claim_issue_id,
							'stock_id'		=>	$component->get_stock_id(),
							'serial_number'		=>	$values['serial_number'],
							'serial_quantity'	=>	$values['quantity'],
							'sent_or_received'	=>	0
						),
						array(
							'claim_issue_id'	=>	$claim_issue_id,
							'stock_id'		=>	$component->get_stock_id(),
							'serial_number'		=>	$values['serial_number']
						),
						__LINE__, __FILE__, 'perp_warranty'
					);
					if($values['replacement']) {
						$GLOBALS['egw']->db->insert(
							'perp_warranty_claim_replacement',
							array(
								'claim_issue_id'	=>	$claim_issue_id,
								'stock_id'		=>	$component->get_stock_id(),
								'original_stock_id'	=>	$component->get_stock_id(),
							),
							array(
								'claim_issue_id'	=>	$claim_issue_id,
								'stock_id'		=>	$component->get_stock_id(),
								'original_stock_id'	=>	$component->get_stock_id(),
							),
							__LINE__, __FILE__, 'perp_warranty'
						);


						$GLOBALS['egw']->db->insert(
							'perp_warranty_claim_serial',
							array(
								'claim_issue_id'	=>	$claim_issue_id,
								'stock_id'		=>	$component->get_stock_id(),
								'serial_number'		=>	$values['replacement'],
								'serial_quantity'	=>	$values['quantity'],
								'sent_or_received'	=>	1
							),
							array(
								'claim_issue_id'	=>	$claim_issue_id,
								'stock_id'		=>	$component->get_stock_id(),
								'serial_number'		=>	$values['replacement']
							),
							__LINE__, __FILE__, 'perp_warranty'
						);
					}
				}
			}
		}

		public static function get_replacements($claim_issue_id) {

			$where = array(
				'perp_warranty_claim_issue.claim_issue_id = ' . (int)$claim_issue_id,
				'replace_parts'
			);

			$join = 'JOIN perp_warranty_claim_issue ON
				    perp_warranty_claim_issue.claim_issue_id = perp_warranty_claim_component.claim_issue_id
				LEFT JOIN perp_warranty_replacement_parts ON
				    perp_warranty_replacement_parts.stock_id = perp_warranty_claim_component.component_id';
				
			$result = $GLOBALS['egw']->db->select(
				'perp_warranty_claim_component',
				array(
					'component_id AS stock_id',
					'IF(replacement_id IS NULL, component_id, replacement_id) AS replacement_stock_id',
					'IF(quantity IS NULL, perp_warranty_claim_component.quantity_claimed, quantity * perp_warranty_claim_component.quantity_claimed) AS replacement_quantity'
				),
				$where,
				__LINE__, __FILE__, false,
				'', 'perp_warranty', -1,
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			return $list;
		}

	}
?>
