<?php
	/**
	*	Storage object for dealing with warranty issues
	*
	*	@author Nathan Gray
	*/

	class so_perp_issue {

		/**
		*	Get a list of known issues
		*/
		public static function get_issue_list(&$query) {

			$columns = array(
				'perp_warranty_issue.issue_id',
			);
			if($query['default_cols']) {
				$columns[] = $query['default_cols'];
			}
			if($query['selectcols']) {
				$columns += explode(',',$query['selectcols']);
			}

			$where = array();
			$join = '';

			if(!$query['include_exceptions']) {
				$where[] = 'perp_warranty_issue_exception.stock_id IS NULL';
				$join = 'LEFT JOIN perp_warranty_issue_exception ON
					perp_warranty_issue.issue_id = perp_warranty_issue_exception.exception_issue_id';
			}

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(perp_warranty_issue.name LIKE $search OR
					perp_warranty_issue.description LIKE $search)";
			}

			if($query['cat_id']) {
				$join .= 'JOIN perp_warranty_issue_stock_cat ON
					perp_warranty_issue_stock_cat.issue_id = perp_warranty_issue.issue_id ';
				$where[] = 'perp_warranty_issue_stock_cat.stock_category_id IN (' . implode(',', (array)$query['cat_id']) . ')';
			}

			if($query['col_filter']) {
				foreach($query['col_filter'] as $column => $value) {
					$where[$column] = $value;
				}
			}
			if(strpos($query['selectcols'], 'open_claims') !== false) {
				$columns[] = 'count(claim_issue_id) AS open_claims';
				$group_by[] = 'perp_warranty_claim_issue.issue_id';

				unset($columns[array_search('open_claims', $columns)]);
				$join .= ' LEFT JOIN perp_warranty_claim_issue ON
					perp_warranty_claim_issue.issue_id = perp_warranty_issue.issue_id
					LEFT JOIN egw_links ON
						egw_links.link_app1 = \'perp_warranty\' AND
						egw_links.link_id1 = CONCAT(\'CLAIM_ISSUE-\', perp_warranty_issue.issue_id)
					LEFT JOIN egw_tracker ON
						egw_links.link_app2 = \'tracker\' AND
						egw_links.link_id2 = egw_tracker.tr_id';
				
			}
	

			if(count($group_by) > 0) {
				$group_by = 'GROUP BY ' . implode(',', $group_by);
			}

			// Get number of rows
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_warranty_issue',
				'count(*) as count',
				$where,
				__LINE__, __FILE__, false, 
				$group_by, 'perp_warranty', false,
				$join
			)->fetchSingle();

			// Get results
			$list = array();
			$results =& $GLOBALS['egw']->db->select(
				'perp_warranty_issue',
				$columns,
				$where,
				__LINE__, __FILE__, $query['start'],
				$group_by . ' ORDER BY ' . $query['order'] . ' ' . $query['sort'], 
				'perp_warranty', $query['num_rows'],
				$join
			);
			foreach($results as $row) {
				$list[] = $row;
			}
			return $list;
		}

		public static function read($issue_id) {
			$issue = $GLOBALS['egw']->db->select(
				'perp_warranty_issue',
				array('*'),
				array('issue_id' => $issue_id),
				__LINE__, __FILE__, false,
				'', 'perp_manufacturing'
			)->fetchRow();

			return $issue;
		}

		/**
		*	Get a list of stock categories, and their warranty settings.
		*	Optionally, include stock categories that are not affected by the
		*	given issue.
		*
		*	@param issue_id
		*	@param include_all
		*
		*	@return array of settings, each category in a sub-array
		*/
		public static function get_stock_categories($issue_id, $include_all) {
			$fields = array(
				'egw_categories.cat_id AS stock_category_id',
				'perp_warranty_issue_stock_cat.return_parts',
				'perp_warranty_issue_stock_cat.replace_parts',
				'perp_warranty_issue_stock_cat.retrofits_allowed',
				'perp_warranty_issue_stock_cat.image_required',
				'perp_warranty_issue_stock_cat.image_description',
				'(perp_warranty_issue_stock_cat.stock_category_id IS NOT NULL) AS included',
				'count(perp_warranty_issue_exception.exception_issue_id) > 0 AS exceptions'
			);

			$where = array(
				'egw_categories.cat_appname = \'perp_inventory\'',
				'(perp_warranty_issue_stock_cat.issue_id = ' . $issue_id .' OR perp_warranty_issue_stock_cat.issue_id IS NULL)',
			);
			if(!$include_all) {
				$where[] = 'perp_warranty_issue_stock_cat.stock_category_id IS NOT NULL';
			}

			$join = 'LEFT JOIN perp_warranty_issue_stock_cat ON
					egw_categories.cat_id = perp_warranty_issue_stock_cat.stock_category_id
				LEFT JOIN (
					perp_warranty_issue_exception 
					JOIN perp_stock ON
						perp_warranty_issue_exception.stock_id = perp_stock.stock_id
				) ON
					perp_warranty_issue_stock_cat.issue_id = perp_warranty_issue_exception.base_issue_id 
					AND perp_stock.stock_category = perp_warranty_issue_stock_cat.stock_category_id';

			$result = $GLOBALS['egw']->db->select(
				'egw_categories',
				$fields,
				$where,
				__LINE__, __FILE__, false,
				'GROUP BY egw_categories.cat_id, perp_warranty_issue_exception.base_issue_id ORDER BY cat_name', 
				'perp_manufacturing',-1,
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Get a list of stock items that are exceptions to the given issue,
		*	and their settings.
		*
		*	@param issue_id
		*
		*	@return array of settings, each stock item in a sub-array
		*/
		public static function get_stock_exception_list($issue_id, $stock_id = '') {
			$fields = array(
				'perp_warranty_issue_exception.*',
				'perp_stock.stock_num',
				'perp_stock.short_description',
				'perp_warranty_issue.*'
			);
			$where = array(
				'perp_warranty_issue_exception.base_issue_id = ' . $issue_id
			);
			if($stock_id) {
				$where[] = 'perp_warranty_issue_exception.stock_id = ' . (int)$stock_id;
			}

			$join = 'JOIN perp_warranty_issue ON
				perp_warranty_issue.issue_id = perp_warranty_issue_exception.exception_issue_id
				JOIN perp_stock ON
					perp_stock.stock_id = perp_warranty_issue_exception.stock_id';

			$result = $GLOBALS['egw']->db->select(
				'perp_warranty_issue_exception',
				$fields,
				$where,
				__LINE__, __FILE__, false,
				'ORDER BY stock_id', 'perp_manufacturing', -1,
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}
			
			return $list;
		}
		

		/**
		*	Save an issue to the database
		*
		*	@param issue_id
		*	@param issue Array of issue settings
		*
		*	@return true or error
		*/
		public static function save($issue_id, Array $issue) {
			$GLOBALS['egw']->db->insert(
				'perp_warranty_issue',
				array(
					'name'			=>	$issue['name'],
					'description'		=>	$issue['description'],
					'expected_time_claimed'	=>	$issue['expected_time_claimed'],
					'coverage'		=>	$issue['coverage'],
					'coverage_limit'	=>	$issue['coverage_limit'],
					'credit_calculation'	=>	$issue['credit_calculation']
				),
				array(
					'issue_id'	=>	$issue_id
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
			$issue_id = $issue_id ? $issue_id : $GLOBALS['egw']->db->get_last_insert_id('perp_warranty_issue', 'issue_id');
			return $issue_id;
		}

		/**
		*	Delete an issue from the database
		*
		*	@param issue_id
		*/
		public static function delete($issue_id) {
			$GLOBALS['egw']->db->transaction_begin();
			$GLOBALS['egw']->db->delete(
				'perp_warranty_issue',
				array(
					'issue_id'	=>	$issue_id
				),
				__LINE__, __FILE__, 'perp_warranty'
			);

			$GLOBALS['egw']->db->delete(
				'perp_warranty_issue_stock_cat',
				array(
					'issue_id'	=>	$issue_id
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
			$GLOBALS['egw']->db->delete(
				'perp_warranty_replacement_parts',
				array(
					'issue_id'	=>	$issue_id
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
			$GLOBALS['egw']->db->delete(
				'perp_warranty_issue_exception',
				array(
					'base_issue_id'	=>	$issue_id
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
			$GLOBALS['egw']->db->transaction_commit();
		}

		/**
		*	Check to see if an issue can be safely deleted from the database
		*/
		public static function can_delete(Array $issue_ids) {
			$result = $GLOBALS['egw']->db->select(
				'perp_warranty_claim_issue',
				array('issue_id'),
				array('issue_id' => $issue_ids),
				__LINE__, __FILE__, false,
				'GROUP BY issue_id', 
				'perp_manufacturing',-1
			);
			$list = array_fill_keys($issue_ids, true);
			foreach($result as $row) {
				$list[$row['issue_id']] = false;
			}
			return $list;
		}

		/**
		*	Update the stock category settings for an issue
		*
		*	@param issue_id
		*	@param stock_categories Array of stock category settings, one sub-array per stock category
		*
		*	@return true or error
		*/
		public static function set_stock_categories($issue_id, Array $stock_categories) {
			$GLOBALS['egw']->db->transaction_begin();

			foreach($stock_categories as $category) {
				if(!$category['included']) {
					$GLOBALS['egw']->db->delete(
						'perp_warranty_issue_stock_cat',
						array(
							'issue_id'	=>	$issue_id,
							'stock_category_id'	=>	$category['stock_category_id']
						),
						__LINE__, __FILE__, 'perp_warranty'
					);
					continue;
				} 
				$GLOBALS['egw']->db->insert(
					'perp_warranty_issue_stock_cat',
					array(
						'issue_id'		=>	$issue_id,
						'stock_category_id'	=>	$category['stock_category_id'],
						'return_parts'		=>	$category['return_parts'],
						'replace_parts'		=>	$category['replace_parts'],
						'retrofits_allowed'	=>	$category['retrofits_allowed'],
						'image_required'	=>	$category['image_required'],
						'image_description'	=>	$category['image_description']
					),
					array(
						'issue_id'	=>	$issue_id,
						'stock_category_id'	=>	$category['stock_category_id']
					),
					__LINE__, __FILE__, 'perp_warranty'
				);
			}
			$GLOBALS['egw']->db->transaction_commit();
			return true;
		}

		/**
		*	Update the settings for an exception for a particular stock item
		*
		*	@param exception
		*/
		public static function set_stock_exception(Array $exception) {
			$GLOBALS['egw']->db->transaction_begin();

			$issue = $exception;
			$issue['issue_id'] = $exception['exception_issue_id'];
			self::save($issue['issue_id'], $issue);

			$GLOBALS['egw']->db->insert(
				'perp_warranty_issue_exception',
				array(
					'base_issue_id'		=>	$exception['base_issue_id'],
					'stock_id'		=>	$exception['stock_id'],
					'exception_issue_id'	=>	$exception['exception_issue_id'],
					'return_parts'		=>	$exception['return_parts'],
					'replace_parts'		=>	$exception['replace_parts'],
					'retrofits_allowed'	=>	$exception['retrofits_allowed'],
					'image_required'	=>	$exception['image_required'],
					'image_description'	=>	$exception['image_description']
				),
				array(
					'base_issue_id'		=>	$exception['base_issue_id'],
					'stock_id'		=>	$exception['stock_id'],
				),
				__LINE__, __FILE__, 'perp_warranty'
			);

			$GLOBALS['egw']->db->transaction_commit();
		}

		/**
		*	Remove an exception on an issue
		*
		*	@param base_issue_id
		*	@param stock_id
		*/
		public static function delete_stock_exception($base_issue_id, $stock_id) {
			$GLOBALS['egw']->db->transaction_begin();
			$exception = self::get_stock_exception_list($base_issue_id, $stock_id);
			$exception_id = $exception[0]['exception_issue_id'];

			$GLOBALS['egw']->db->delete(
				'perp_warranty_issue_exception',
				array(
					'base_issue_id'		=>	$base_issue_id,
					'stock_id'		=>	$stock_id,
				),
				__LINE__, __FILE__, 'perp_warranty'
			);

			$GLOBALS['egw']->db->delete(
				'perp_warranty_issue',
				array(
					'issue_id'		=>	$exception_id,
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
		}

		/**
		*	Normally damaged parts are replaced with the same thing.  Add an exception to that rule.
		*
		*	@param issue_id
		*	@param stock_id
		*	@param replacement_id
		*	@param quantity
		*/
		public static function add_replacement_part($issue_id, $stock_id, $replacement_id, $quantity) {
			$GLOBALS['egw']->db->insert(
				'perp_warranty_replacement_parts',
				array(
					'issue_id'		=>	$issue_id,
					'stock_id'		=>	$stock_id,
					'replacement_id'	=>	$replacement_id,
					'quantity'		=>	$quantity
				),
				array(
					'issue_id'		=>	$issue_id,
					'stock_id'		=>	$stock_id,
					'replacement_id'	=>	$replacement_id,
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
		}

		/**
		*	Remove an exception for a replacement part, send the same thing as was damaged.
		*
		*	@param issue_id
		*	@param stock_id
		*	@param replacement_id
		*/
		public static function delete_replacement_part($issue_id, $stock_id, $replacement_id) {
			$GLOBALS['egw']->db->delete(
				'perp_warranty_replacement_parts',
				array(
					'issue_id'		=>	$issue_id,
					'stock_id'		=>	$stock_id,
					'replacement_id'	=>	$replacement_id
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
		}

		/**
		*	Get a list of replacement parts for a given issue.
		*
		*	If the stock_id parameter is set, it will return the replacement parts for that
		*	stock item, which may be the same stock item.
		*
		*	If stock_id parameter is not set, it will return only replacement parts where
		*	the stock item is replaced with (a) different item(s).
		*
		*	@nextmatch
		*/
		public static function get_replacement_parts(&$query) {
			$fields = array(
				'perp_warranty_replacement_parts.issue_id',
				'perp_warranty_replacement_parts.stock_id',
				'perp_warranty_replacement_parts.replacement_id',
				'perp_warranty_replacement_parts.quantity',
				'perp_stock.stock_num',
				'perp_stock.short_description',
			);
			$where = array();
			$join = 'LEFT JOIN perp_warranty_replacement_parts ON
				perp_stock.stock_id = perp_warranty_replacement_parts.stock_id ';

			if($query['issue_id']) {
				$where[] = '(perp_warranty_replacement_parts.issue_id = ' . (int)$query['issue_id'] . ' OR
					/* Include issue-independent replacements */
					perp_warranty_replacement_parts.issue_id = 0)';
			}
			if(!$query['stock_id']) {
				$where[] = 'perp_warranty_replacement_parts.stock_id IS NOT NULL';
			} else {
				$where[] = 'perp_stock.stock_id = ' . (int)$query['stock_id'];
			}
			if($query['cat_id']) {
				$where[] = 'perp_stock.stock_category = ' . (int)$query['cat_id'];
			}

			// Get number of rows
			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_stock',
				'count(*) as count',
				$where,
				__LINE__, __FILE__, false, 
				'', 'perp_warranty', false,
				$join
			)->fetchSingle();

			// Get results
			$list = array();
			$result = $GLOBALS['egw']->db->select(
				'perp_stock',
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'], 
				'ORDER BY ' . $query['order'] . ' ' . $query['sort'], 'perp_warranty', $query['num_rows'],
				$join
			);
			foreach($result as $row) {
				$list[] = $row;
			}

			return $list;
		}

		/**
		*	Settings for one stock item
		*/
		public static function get_stock_settings($issue_id, $stock_id) {
			$fields = array(
				'return_parts',
				'replace_parts',
				'retrofits_allowed',
				'image_required',
				'image_description'
			);
			$where = array(
				'issue_id'	=>	$issue_id,
				"stock_id = $stock_id"
			);
			return $GLOBALS['egw']->db->union(
				array(
					array(
						'table'	=>	'perp_warranty_issue_stock_cat',
						'cols'	=>	$fields,
						'where'	=>	$where,
						'app'	=>	'perp_warranty',
						'join'	=>	'JOIN perp_stock ON
						perp_stock.stock_category = perp_warranty_issue_stock_cat.stock_category_id'
					),
					array(
						'table'	=>	'perp_warranty_issue_exception',
						'cols'	=>	$fields,
						'where'	=>	array(
							'base_issue_id'	=>	$issue_id,
							"stock_id = $stock_id"
						),
						'app'	=>	'perp_warranty',
					)
				),
				__LINE__, __FILE__
			)->fetch();
		}
	}
?>
