<?php

	/**
	*	Storage layer for warranty registrations
	*	These are tied fairly closely into addressbook, which is where
	*	most of the information is kept, via the owner column.
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*/

	class so_perp_registration {

		/**
		*	Get a list of registration information, including addresses of owners
		*
		*	@nextmatch
		*
		*/
		public static function get_registration_list(&$query) {
			$fields = array(
				'registration_id',
				'client_id',
				'stock_id',
				'serial_number',
				'owner',
				'registration_date',
				'registered_by',
				'egw_addressbook.*',
			);

			$where = array();

			$join = 'LEFT JOIN egw_addressbook ON
				perp_warranty_registration.owner = egw_addressbook.contact_id';

			$order = 'ORDER BY ' . ($query['order'] ? $query['order'] : 'registration_id') . ' ' . $query['sort'];

			if($query['search']) {
				$search = $GLOBALS['egw']->db->quote('%' . $query['search'] . '%');
				$where[] = "(serial_number LIKE $search OR
					egw_addressbook.n_fn LIKE $search OR
					egw_addressbook.n_fileas LIKE $search OR
					egw_addressbook.adr_one_street LIKE $search OR
					egw_addressbook.adr_one_street2 LIKE $search OR
					egw_addressbook.adr_one_locality LIKE $search)";
			}
			
			if($query['client_id']) {
				$where['client_id'] = (int)$query['client_id'];
			}
			if($query['stock_id']) {
				$where['stock_id'] = (int)$query['stock_id'];
			}
			if($query['serial_number']) {
				$where['serial_number'] = $query['serial_number'];
			}
			if($query['col_filter']['adr_one_countryname']) {
				$where[] = 'egw_addressbook.adr_one_countryname = ' . $GLOBALS['egw']->db->quote($query['col_filter']['adr_one_countryname']);
			}

			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_warranty_registration',
				array('count(registration_id)'),
				$where,
				__LINE__, __FILE__, false,
				$order, 'perp_warranty', 1,
				$join
			)->fetchSingle();

			$list = array();
			$result =& $GLOBALS['egw']->db->select(
				'perp_warranty_registration',
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				$order, 'perp_warranty', $query['num_rows'],
				$join
			);
			foreach($result as $row) {
				$row['registration_date'] = $GLOBALS['egw']->db->from_timestamp($row['registration_date']);
				$list[] = $row;
			}

			return $list;
		}

		public static function save($registration) {
			$GLOBALS['egw']->db->insert(
				'perp_warranty_registration',
				array(
					'registration_id'	=>	(int)$registration['registration_id'],
					'stock_id'		=>	(int)$registration['stock_id'],
					'serial_number'		=>	$registration['serial_number'],
					'owner'			=>	(int)$registration['owner'],
					'start_date'		=>	$registration['start_date'] ? $GLOBALS['egw']->db->to_timestamp($registration['start_date']) : null,
					'end_date'		=>	$registration['end_date'] ? $GLOBALS['egw']->db->to_timestamp($registration['end_date']) : null,
					'notes'			=>	$registration['notes'],
					'registered_by'		=>	(int)$registration['registered_by'],
					'registration_date'	=>	$GLOBALS['egw']->db->to_timestamp($registration['registration_date']),
					'client_id'		=>	(int)$registration['client_id']
				),
				array(
					'registration_id'	=>	(int)$registration['registration_id'],
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
			
			if(!$registration['registration_id']) {
				// First time
				$registration['registration_id'] = $GLOBALS['egw']->db->get_last_insert_id('perp_warranty_registration', 'registration_id');
			}

			return $registration['registration_id'];
		}

		public static function read($registration_id) {
			$result = $GLOBALS['egw']->db->select(
				'perp_warranty_registration',
				array('*'),
				array(
					'registration_id'	=>	(int)$registration_id,
				),
				__LINE__, __FILE__, false,
				'', 'perp_warranty', 1
			)->fetch();
			$result['start_date'] = $GLOBALS['egw']->db->from_timestamp($result['start_date']);
			$result['end_date'] = $GLOBALS['egw']->db->from_timestamp($result['end_date']);
			$result['registration_date'] = $GLOBALS['egw']->db->from_timestamp($result['registration_date']);

			return $result;
		}
	}
?>
