<?php
	/**
	*	Storage for Return Material Authorizations (RMA)
	*
	*	@package perp_warranty
	*	@author Nathan Gray
	*/

	class so_perp_rma {

		/**
		*	Get some RMA info
		*/
		public static function get_rma_list(Array &$query) {

			$fields = array(
				'rma_id',
				'our_reference_number',
			);

			$where = array();
			$join = 'LEFT JOIN perp_warranty_claim ON
					perp_rma.claim_id = perp_warranty_claim.claim_id
				JOIN perp_order ON
                                        ' . Order::order_table .'.order_id = perp_rma.rma_id 
				JOIN perp_order_line ON
                                        ' . Order::order_table .'.order_id = perp_order_line.order_id '.
                                'LEFT JOIN perp_client ON
                                        perp_client.client_id = ' . Order::order_table . '.client_id '.
                                'LEFT JOIN perp_supplier ON
                                        perp_supplier.supplier_id = ' . Order::order_table . '.supplier_id ' .
                                'LEFT JOIN (
                                        SELECT
                                                sum(quantity_shipped) AS quantity_shipped,
                                                perp_shipment_line.shippable_id,
                                                perp_shipment_line.shippable_item_line
                                        FROM perp_shipment_line
                                        GROUP by shippable_id, shippable_item_line
                                ) AS perp_shipment_line ON
                                        perp_shipment_line.shippable_id = ' . Order::order_table . '.order_id AND
                                        perp_shipment_line.shippable_item_line = ' . OrderLine::order_line_table . '.line_no
                                LEFT JOIN (
                                        SELECT
                                                sum(quantity) AS quantity_invoiced,
                                                perp_invoice_line.invoicable_id,
                                                perp_invoice_line.invoicable_item_line
                                        FROM perp_invoice_line
                                        GROUP BY invoicable_id, invoicable_item_line
                                ) AS perp_invoice_line ON
                                        perp_invoice_line.invoicable_id = ' . Order::order_table . '.order_id AND
                                        perp_invoice_line.invoicable_item_line = ' . OrderLine::order_line_table . '.line_no';

			$order = "GROUP BY perp_rma.rma_id
				ORDER BY {$query['order']} {$query['sort']}";
			
			
			// Mess with the field list to change it into database columns
			if($query['selectcols']) {
				// Exclude action column
				$selectcols = str_replace(',actions', '', $query['selectcols']);
				$fields = array_merge($fields, explode(',',$selectcols));
			}

			if($query['search']) {
                                $search = $GLOBALS['egw']->db->quote('%'.$query['search'].'%');
                                $where[] = "(perp_rma.rma_id LIKE $search
                                        OR perp_order.our_reference_number LIKE $search)";
                        }


                        if($query['filter'] == 'supplier') {
                                $where[] = 'supplier_id != 0';
                        } elseif($query['filter'] == 'client') {
                                $where[] = 'client_id != 0';
                        }

			if($query['claim_id']) {
				$where[] = 'perp_rma.claim_id = ' . (int)$query['claim_id'];
			}

			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_rma',
				array('count(rma_id) AS count'),
				$where,
				__LINE__, __FILE__, false,
				$order, 'perp_manufacturing', 1,
				$join
			)->NumRows();

			$result =& $GLOBALS['egw']->db->select(
				'perp_rma',
				$fields,
				$where,
				__LINE__, __FILE__, $query['start'],
				$order, 'perp_manufacturing', $query['num_rows'],
				$join
			);

			$list = array();
			foreach($result as $row) {
				$row['originating_date'] = $GLOBALS['egw']->db->from_timestamp($row['originating_date']);
				$row['closed_date'] = $GLOBALS['egw']->db->from_timestamp($row['closed_date']);

				$list[] = $row;
			}

			return $list;
		}


		public static function read($rma_id) {
			$rma = $GLOBALS['egw']->db->select(
				'perp_rma',
				array(
					'*',
				),
				array(
					'rma_id'	=>	$rma_id
				),
				__LINE__, __FILE__, false,
				'', 'perp_warranty', 1,
				'JOIN perp_order ON
					perp_order.order_id = perp_rma.rma_id'
			)->fetch();

			$rma['closed_date'] = $GLOBALS['egw']->db->from_timestamp($rma['closed_date']);

			return $rma;
		}

		public static function write(RMA $rma) {
			$rma_array = $rma->toArray();
			
throw new Exception('Deprecated');
			$GLOBALS['egw']->db->transaction_begin();

			// Insert RMA
			$GLOBALS['egw']->db->insert(
				'perp_rma',
				array(
					'rma_id'	=>	$rma_array['rma_id'],
					'claim_id'	=>	$rma_array['claim_id']
				),
				array(
					'rma_id'	=>	$rma_array['rma_id']
				),
				__LINE__, __FILE__, 'perp_warranty'
			);
	

			if(!$rma->get_rma_id()) {
				$rma_array['rma_id'] = $GLOBALS['egw']->db->get_last_insert_id('perp_rma', 'rma_id');
				
			}
			
			
			// Insert lines
			$GLOBALS['egw']->db->delete(
				'perp_rma_line',
				array('rma_id'	=>	$rma_array['rma_id']),
				__LINE__, __FILE__, 'perp_warranty'
			);
			$GLOBALS['egw']->db->delete(
				'perp_rma_serial_number',
				array('rma_id'	=>	$rma_array['rma_id']),
				__LINE__, __FILE__, 'perp_warranty'
			);

			foreach($rma->get_lines() as $line) {
				$line_array = $line->toArray();
				$GLOBALS['egw']->db->insert(
					'perp_rma_line',
					array(
						'rma_id'		=>	$rma_array['rma_id'],
						'stock_id'		=>	$line_array['stock_id'],
						'quantity_authorized'	=>	$line_array['quantity_authorized'],
						'quantity_received'	=>	$line_array['quantity_recieved']
					),
					array(
						'rma_id'	=>	$rma_array['rma_id'],
						'stock_id'	=>	$line_array['stock_id']
					),
					__LINE__, __FILE__, 'perp_warranty'
				);
				if(is_array($line_array['serial_numbers'])) {
					foreach($line_array['serial_numbers'] as $serial_number => $quantity) {
						$GLOBALS['egw']->db->insert(
							'perp_rma_serial_number',
							array(
								'rma_id'		=>	$rma_array['rma_id'],
								'stock_id'		=>	$line_array['stock_id'],
								'serial_number'		=>	$serial_number,
								'serial_quantity'	=>	$quantity
							),
							array(
								'rma_id'		=>	$rma_array['rma_id'],
								'stock_id'		=>	$line_array['stock_id'],
								'serial_number'		=>	$serial_number
							),
							__LINE__, __FILE__, 'perp_warranty'
						);
					}
				}
			}

			$GLOBALS['egw']->db->transaction_commit();

			return $rma_array['rma_id'];
		}

		/**
		*	Get a list of RMA items
		*/
		public static function get_rma_stock(&$query) {

			$where = array();
			$join = 'JOIN perp_stock ON
				perp_stock.stock_id = perp_rma_line.stock_id';

			if($query['rma_id']) {
				$where[] = 'rma_id = ' . (int)$query['rma_id'];
			}
			if($query['cat_id']) {
				$where[] = 'stock_category = ' . (int)$query['cat_id'];
			}

			if($query['search']) {
				$search = $this->db->quote('%' . $query['search'] . '%');
				$search_clause = "(stock_num LIKE $search OR short_description LIKE $search OR
					perp_stock_barcode.barcode = " . $this->db->quote($query['search']);
				if(strpos($query['selectcols'], 'supplier_code') !== false || strpos($query['selectcols'], 'preferred') !== false) {
					$query['selectcols'] .= ',perp_supplier_stock.supplier_id,supplier_name';
					$search_clause .= ' OR ' .
					"perp_supplier_stock.supplier_code LIKE $search";
				}
				$search_clause .= ")";
				$where[] = $search_clause;
			}

			$query['total'] = $GLOBALS['egw']->db->select(
				'perp_rma_line',
				array('count(*)'),
				$where,
				__LINE__, __FILE__, false,
				'', 'perp_warranty', 1,
				$join
			)->fetchSingle();

			$result =& $GLOBALS['egw']->db->select(
				'perp_rma_line',
				array(
					'*',
				),
				$where,
				__LINE__, __FILE__, $query['start'],
				'', 'perp_warranty', $query['num_rows'],
				$join
			);
			$list = array();
			foreach($result as $row) {
				$list[] = $row;
			}

			foreach($list as &$line) {
				$result =& $GLOBALS['egw']->db->select(
					'perp_rma_serial_number',
					array(
						'serial_number',
						'serial_quantity'
					),
					array(
						'rma_id'	=>	$line['rma_id'],
						'stock_id'	=>	$line['stock_id']
					),
					__LINE__, __FILE__, false,
					'', 'perp_warranty'
				);
				foreach($result as $row) {
					$line['serial_numbers'][$row['serial_number']] = $row['serial_quantity'];
				}
			}
			return $list;
		}
	}
?>
