<?php
	/**
	*	User interface for entering and processing warranty claims
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.bo_perp_claim.inc.php');

	class ui_perp_claim {
		
		// Functions we allow to be called directly via a URL
		public $public_functions = array(
			'claim_list'	=>	true,
			'new_claim'	=>	true,
			'edit'		=>	true,
		);

		// Turn on handy debug messages
		protected static $debug = false;

		// eTemplate oject used for page generation
		protected static $template;

		/**
		* List of messages for the user.  Each message is in the form
		* array(message, severity)
		*/
		public static $messages = array();

		/**
		*	A list of eTemplates used for claims
		*/
		protected static $list_template		=	'perp_warranty.claim_list';
		public static $list_row_template	=	'perp_warranty.claim_list.row';
		protected static $edit_template		=	'perp_warranty.claim_edit';


		protected static $tabs = 'summary|links|history';

		public function __construct() {
			if(!is_object(self::$template)) {
				self::$template = new etemplate();
			}
		}

		/**
		*	Get a list of claims.  This covers all statuses, claims from other people
		*	on us, and claims from us on our suppliers.
		*
		*	@param content Array of data returned from the eTemplate
		*/
		public function claim_list($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}

			// Deal with incoming
			self::list_update($content);
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_warranty.index'
				));
			}

			// Set up for outgoing
			$data = self::list_get_display_data();

			$select_box_options = self::list_get_options();
			$readonly = self::list_get_readonlys($data);

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Claim List');
			self::$template->read(self::$list_template);
			self::$template->exec('perp_warranty.ui_perp_claim.claim_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Deal with data returned from the list
		*/
		protected function list_update($content) {
			// Action buttons
			if($content['new_client']) {
				self::handle_action('new_client', array());
			} elseif ($content['new_supplier']) {
				self::handle_action('new_supplier', array());
			}
			if(is_array($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as $action => $key) {
					// Just data?
					if(is_numeric($action)) {
						if(is_array($key['issues'])) {
							if($key['issues']['selected'] && $content['update']) {
								// Doing an update on a bunch of issues
								$tracker_bo = new tracker_bo();
								foreach($key['issues']['selected'] as $issue_id) {
									$issue_obj =& new ClaimIssue($issue_id);
									$tracker_bo->read($issue_obj->get_tracker_id());
									if($content['new_resolution'] ) {
										$tracker_bo->data['tr_resolution'] = $content['new_resolution'];
									}
									if($content['status']) {
										$tracker_bo->data['tr_status'] = $content['status'];
									}
									if($content['assigned'] != '') {
										$tracker_bo->data['tr_assigned'] = $content['assigned'];
									}
									$tracker_bo->save();
								}
							}
						}
						continue;
					}

					$key = key($key);
					self::handle_action($action, array($key));
				}
			}

		}

		/**
		*	Get the list of claims to display
		*/
                protected function list_get_display_data() {
                        $data = array();

                        $data['nextmatch'] = $GLOBALS['egw']->session->appsession('claim_list', 'perp_warranty');
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array(
					'cat_app'		=>	'perp_inventory',
					'filter_label'		=>	lang('Claims from'),
					'filter'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter'],
					'cat_id'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['cat_id'],
					'no_filter2'		=>	true,
					'get_rows'		=>	$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.get_claim_list',
					'template'		=>	self::$list_row_template,
					'col_filter'		=>	array(
						'assigned_to'	=>	$GLOBALS['egw_info']['user']['account_id'],
					),
					// If the claim is 1 or more issues on a single product, hide the single product
					'hide_parent'		=>	true,
				);
			}
			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}
			return $data;
		}

		/**
                *       Get the selectbox options for the issue list
                */
		public function list_get_options() {
			$config = config::read('perp_warranty');

                        $options = array();
			$options['filter'] = array(
				''		=>	lang('All'),
				'supplier'	=>	lang('Suppliers'),
				'client'	=>	lang('Clients'),
			);
			$options['supplier_id']	=	ExecMethod('perp_ap.bo_perp_supplier.get_suppliers');
			$options['client_id']	=	ExecMethod('perp_ar.bo_perp_client.get_client_list');
			$options['issue_id']	=	bo_perp_issue::get_issue_list();
			$options['status']	=	ExecMethod('tracker.tracker_bo.get_tracker_stati', $config['warranty_tracker']);

			$options['resolution']	=	bo_perp_claim::get_resolution_list();
			$options['new_resolution'] = 	$options['resolution'];
			$options['assigned_to']	=	array(0 => lang('Not assigned')) + ExecMethod('tracker.tracker_bo.get_staff', $config['warranty_tracker']);
			$options['assigned']	=	$options['assigned_to'];

			return $options;
		}

		/**
		*	Get additional readonlys for the claim list
		*/
		protected function list_get_readonlys($data) {
			$readonly = array();

			if($data['nextmatch']['filter'] == 'supplier') {
				$readonly['new_client'] = true;
			} elseif ($data['nextmatch']['filter'] == 'client') {
				$readonly['new_supplier'] = true;
			}


			// Check for tracker being set
			$config = config::read('perp_warranty');
			if(!$config['tracker_queue']) {
				self::$messages[] = array(lang('perp_warranty has not been properly configured.  Missing tracker queue.'), 'error');
				$readonly['new_supplier'] = true;
				$readonly['new_client'] = true;
			}

			return $readonly;
		}

		public function get_claim_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			// Put it in the session to preserve where they are when they come back
			$GLOBALS['egw']->session->appsession('claim_list', 'perp_warranty', $query);

			// Save changes to preferences
			if($query['cat_id'] != $GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['cat_id'] ||
					$query['filter'] != $GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter']) {
				$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['cat_id'] = $query['cat_id'];
				$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter'] = $query['filter'];

				$GLOBALS['egw']->preferences->add('perp_warranty', 'claim_list', $GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']);
				$GLOBALS['egw']->preferences->save_repository(true);
			}

			// Reset supplier / client columns
			if($query['filter'] == 'client') {
				$query['supplier_id'] = 0;
			} elseif ($query['filter'] == 'supplier') {
				$query['client_id'] = 0;
			}

			$total = bo_perp_claim::get_claim_list($query, $rows, $readonlys);

			// Hide supplier / client columns
			if($query['filter'] == 'client') {
				$rows['no_supplier_id'] = true;
			} elseif ($query['filter'] == 'supplier') {
				$rows['no_client_id'] = true;
			}

			return $total;
		}

		/**
		*       Handle an action on a list of claims
		*
		*       @param action String name of action to do
		*       @param claim_id Array of claim IDs to apply the action to
		*/
		protected static function handle_action($action, array $claim_id) {
			switch ($action) {
				case 'new_client':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_warranty.ui_perp_client_claim.new_claim'
					));
				case 'new_supplier':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_warranty.ui_perp_supplier_claim.new_claim'
					));
				case 'delete':
					foreach($claim_id as $id) {
						bo_perp_claim::delete($id);
					}
					break;
				case 'close':

					// Close all issues on the claim
					$query = array(
						'claim_id'	=>	$claim_id
					);
					bo_perp_claim_issue::get_claim_issues($query, $issues);
					$tracker_bo = new tracker_bo();
					foreach($issues as $issue) {
						$issue_obj =& new ClaimIssue($issue['claim_issue_id']);
						if($issue_obj->get_tracker_id()) {
							$tracker_bo->read($issue_obj->get_tracker_id());
							$tracker_bo->data['tr_status'] = tracker_bo::STATUS_CLOSED;
							$tracker_bo->save();
						}
					}
			}
		}

		/**
		*	Stub that redirects to either client.edit() or supplier.edit()
		*/
		public static function edit($content = array()) {
			$claim_id = reg_var('claim_id');
			$claim = new Claim($claim_id);
			
			if($claim->get_client_id()) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_client_claim.edit',
					'claim_id'	=>	$claim_id
				));
			} elseif ($claim->get_supplier_id()) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_supplier_claim.edit',
					'claim_id'	=>	$claim_id
				));
			} else {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_claim.claim_list'
				));
			}
		}

		/**
		*       Internal function to get a Claim object to work with.
		*       First, we check the session for a claim in progress.  Then we
		*       check the provided claim ID and read from the database, if required.
		*       Extending classes should override this function to create the correct
		*       claim type, or provide the class.
		*
		*	@param claim_id The ID number of the claim to fetch
		*	@param class If the claim is fetched from the database, the class to use
		*
		*	@return A reference to a Claim, or and extending class
		*/
		protected static function &get_claim($claim_id, $class = 'Claim') {
			$temp = $GLOBALS['egw']->session->appsession('claim_' . $claim_id);
			if(
				($claim_id && $temp instanceof $class && $claim_id != $temp->get_claim_id()) ||
				$claim_id && !($temp instanceof $class)
			) {
				$claim = new $class($claim_id);
			} elseif ($temp) {
				$claim = $temp;
			} else {
				$claim = null;
			}

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($claim->get_supplier_id(), $supplier_ids)) {
				$claim = null;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($claim->get_client_id(), $client_ids)) {
				$claim = null;
			}

			return $claim;
		}

		/**
		*	Get common display data for claims
		*/
		protected static function get_display_data(Claim &$claim, &$data, &$preserve = array()) {

			// General
			$data = $claim->toArray();

			$parent = $claim->get_parent();
			if($parent) {
				$parent_array = $parent->toArray();
				$parent_array += $parent_array['components'][0];
				$parent_array += ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $parent_array['stock_id']);
				unset($parent_array['components']);
				$parent_array['serial_number'] = $parent_array['serial_numbers'][0]['serial_number'];
				$data['item'] = $parent_array;
			}

			// Links
			$data['links'] = array(
				'to_app'	=>	'perp_warranty',
				'to_id'		=>	'CLAIM-' . $claim->get_claim_id()
			);

			// Issues nextmatch
			$issue_session_key = 'claim_' . $claim->get_claim_id() . '_issues';
			if($GLOBALS['egw']->session->appsession($issue_session_key)) {
				$data['issues'] = $GLOBALS['egw']->session->appsession($issue_session_key);
			} else {
				// Template & get_rows need to be set in overriding class
				$data['issues'] = array(
					'claim_id'	=>	$claim->get_claim_id(),
					'no_filter'	=>	true,
					'no_filter2'	=>	true,
					'no_cat'	=>	true,
					// The nested grids in the template cause issues with the how this works.
					'no_columnselection'	=>	true, 
					'hide_parent'	=>	true
				);
			}

			return $data;
		}

		/**
		*	Get common selectbox options
		*/
		protected static function get_select_box_options(Claim &$claim) {
			$options = array();
			
			$options['issue_id']	=	bo_perp_issue::get_issue_list();
			$options['new_issue']	=	$options['issue_id'];
			$options['status']	=	ExecMethod('tracker.tracker_bo.get_tracker_stati', $config['warranty_tracker']);
			$options['resolution']	=	bo_perp_claim::get_resolution_list();


			return $options;
		}

		/**
		*	Get common readonlys
		*/
		protected static function get_readonlys(Claim &$claim) {
			$readonlys = array();
			return $readonlys;
		}

		/**
		*	Get common javascript
		*/
		protected static function get_javascript() {
		}

		/**
		*	Get a list of issues for a claim.  Parameters are stored in the session so
		*	the list is the same when you come back to it
		*
		*	@nextmatch
		*
		*	Extra parameters:
		*		claim_id: ID number of the claim to get the issues for
		*/
		public static function get_claim_issues(&$query = array(), &$rows = array(), &$readonly = array()) {
			
			// Store parameters in session
			$issue_session_key = 'claim_' . $query['claim_id'] . '_issues';
			$GLOBALS['egw']->session->appsession($issue_session_key, 'perp_warranty', $query);

			// Get issues
			$count = bo_perp_claim_issue::get_claim_issues($query, $rows, $readonly);

			return $count;
		}
	}

?>
