<?php
	/**
	*	User interface for entering and processing warranty claims
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.bo_perp_claim_issue.inc.php');

	class ui_perp_claim_issue {
		
		// Functions we allow to be called directly via a URL
		public $public_functions = array(
			'edit'		=>	true,
		);

		// Turn on handy debug messages
		protected static $debug = false;

		// eTemplate oject used for page generation
		protected static $template;

		/**
		* List of messages for the user.  Each message is in the form
		* array(message, severity)
		*/
		public static $messages = array();

		/**
		*	A list of eTemplates used for claims
		*/
		protected static $edit_template		=	'perp_warranty.claim_issue_edit';


		protected static $tabs = 'summary|add_comment|comments|links|history';

		public function __construct() {
			if(!is_object(self::$template)) {
				self::$template = new etemplate();
			}
		}


		/**
		*	Deal with data returned from the list
		*/
		protected function list_update($content) {

			// Action buttons
			if(is_array($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as $action => $key) {
					// Just data
					if(is_numeric($action)) continue;

					$key = key($key);
					self::handle_action($action, array($key));
				}
			}

		}

		/**
		*       Handle an action on a list of claim_issues
		*
		*       @param action String name of action to do
		*       @param claim_id Array of claim_issue_IDs to apply the action to
		*/
		protected static function handle_action($action, $claim_issue_id) {
			if(!is_array($claim_issue_ids)) {
				$claim_issue_ids = array($claim_issue_ids);
			}

			switch ($action) {
				case 'delete':
					foreach($claim_issue_id as $id) {
						bo_perp_claim_issue::delete($id);
					}
					break;
				case 'issue_RMA':
				case 'received_RMA':
				case 'returned':
				case 'create_order':
				case 'enter_credit':
				case 'apply_credit':
				default:
					echo __METHOD__ . ": $action not implemented. " . __FILE__ . ':' . __LINE__ . '<br />';
			}
		}

		/**
		*	Edit a claim issue
		*/
		public function edit($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}

			// Get claim issue
			$claim_issue_id = get_var('claim_issue_id', 'any', $content['claim_issue_id']);
			$claim_issue =& self::get_claim_issue($claim_issue_id);
			if(!$claim_issue) {
				self::$messages[] = array(lang('Unable to find issue'), 'error');
die();
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_claim.claim_list'
				));
			}

			if(self::$debug) {
				echo 'ClaimIssue at top of ' . __METHOD__ . '<br />';
				_debug_array($claim_issue->__toString());
			}

			// Save / Update
			//$admin = true; 
			$edit = $admin = $GLOBALS['egw']->acl->check('claim', EGW_ACL_EDIT, 'perp_warranty');

                        $buttons = array(
                                'save'          =>      true,
				'add'		=>	true,
                        );
                        if($edit && $claim_issue && $content) {
                                try {
                                        self::update($claim_issue, $content);
                                } catch (Exception $e) {
                                        self::$messages[] = array($e->getMessage(), 'error');
                                }
                        }

                        if(isset($content['cancel']) || !$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				// Clear from session
				$GLOBALS['egw']->session->appsession('claim_issue_' . $claim_issue_id, false, null);
				$GLOBALS['egw']->session->appsession('claim_' . $claim_issue->get_claim_id(), false, null);

                                $GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_claim.claim_list'
				));
                        }
                        if($content['save']) {
                                if($edit) {
                                        try {
                                                $result = bo_perp_claim_issue::write($claim_issue);
                                        } catch (Exception $ex) {
                                                self::$messages[] = array($ex->getMessage(), 'error');
                                        }
                                        if(!is_numeric($result )) {
                                                self::$messages[] = array($result, 'error');
                                        } else {
						self::$messages[] = array(lang('Issue saved'), 'success');
					}

                                        $GLOBALS['egw']->session->appsession('claim_issue_' . $claim_issue_id, false, null);
					$GLOBALS['egw']->session->appsession('claim_' . $claim_issue->get_claim_id(), false, null);
                                } else {
                                        self::$messages[] = array(lang('Cannot save issue'), 'error');
                                }
                        }
			// Handle actions
			if($content['actions']) {
				foreach($content['actions'] as $action => $value) {
					if($action == 'resolve') {
					} else {
						self::handle_action($action, $value != 'pressed' ? $value : array($claim_issue_id));
					}
				}
			}

			// Save to session
			$GLOBALS['egw']->session->appsession('claim_issue_' . $claim_issue_id, false, $claim_issue);
			
			// Get data
                        $data = array();
                        self::get_display_data($claim_issue, $data, $preserve);

                        // Supporting data
			$select_box_options = self::get_select_box_options($claim_issue);

			// Read onlys
			$readonly = self::get_readonlys($claim_issue, $data);

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			self::get_javascript();

			$preserve = $data;

			if(self::$debug) {
				echo __METHOD__ . ' outgoing data:';
				_debug_array($data);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Edit Claim #%1: %2 (#%3)', 
				$claim_issue->get_claim_id(),
				ExecMethod('perp_warranty.bo_perp_issue.link_title', $claim_issue->get_issue_id()),
				$claim_issue->get_claim_issue_id()
			);
			self::$template->read(self::$edit_template);
			self::$template->exec('perp_warranty.ui_perp_claim_issue.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*       Internal function to get a ClaimIssue object to work with.
		*       First, we check the session for a claim issue in progress.  Then we
		*       check the provided claim_issue ID and read from the database, if required.
		*
		*	@param claim_issue_id The ID number of the claim to fetch
		*	@param class If the data is fetched from the database, the class to use
		*
		*	@return A reference to a ClaimIssue, or an extending class
		*/
		protected static function &get_claim_issue($claim_issue_id, $class = 'ClaimIssue') {
			$temp = $GLOBALS['egw']->session->appsession('claim_issue_' . $claim_issue_id);
			if(
				($claim_issue_id && $temp instanceof $class && $claim_issue_id != $temp->get_claim_issue_id()) ||
				$claim_issue_id && !($temp instanceof $class)
			) {
				$claim_issue = new $class($claim_issue_id);
			} elseif ($temp) {
				$claim_issue = $temp;
			} else {
				$claim_issue = null;
			}

			// Check if the current user is a supplier
			$supplier_ids = ExecMethod('perp_ap.bo_perp_supplier.get_supplier_by_user_account');
			if(is_array($supplier_ids) && count($supplier_ids) > 0 && !in_array($claim_issue->get_claim()->get_supplier_id(), $supplier_ids)) {
				$claim_issue = null;
			}

			// Check if the current user is a client
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0 && !in_array($claim_issue->get_claim()->get_client_id(), $client_ids)) {
				$claim_issue = null;
			}

			return $claim_issue;
		}

		/**
		*	Get common display data for claim issues
		*/
		protected static function get_display_data(ClaimIssue &$claim_issue, &$data, &$preserve = array()) {
			$data = $claim_issue->toArray();

			// Linking on the claim_issue
			$data['links'] = array(
				'to_app'	=>	'perp_warranty',
				'to_id'		=>	'CLAIM_ISSUE-' . $claim_issue->get_claim_issue_id(),
				// Exclude tracker, it will almost always have a link, but we don't want people messing with it
				'only_app'	=>	'!tracker'
			);

			if($data['components']) {
				// Shift to allow room for header
				array_unshift($data['components'], false);

				// Flag to see if we can hide the entire images column
				$data['components']['no_images'] = true;

				// Set link ID
				foreach($data['components'] as &$component) {
					if(!is_array($component)) {
						continue;
					}

					// The readonly AJAX select will overwrite stock_id, so we copy it here
					$component['_stock_id'] = $component['stock_id'];

					$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $component['stock_id']);
						
					$component['images'] = array(
						'to_app'	=>	'perp_warranty',
						'to_id'		=>	'CLAIM_ISSUE-' . $claim_issue->get_claim_issue_id() . '-' . $component['stock_id'],
						'only_app'	=>	egw_link::VFS_APPNAME
					);

					$component['unit_of_measure_id'] = $stock_item['unit_of_measure'];
					// We'll need this for readonly checking
					$component['stock_category_id'] = $stock_item['stock_category'];
					$issue_settings = bo_perp_issue::get_stock_settings($claim_issue->get_issue_id(), $component['stock_id']);
					$component['image_description'] = $issue_settings['image_description'];
					$data['components']['no_images'] = $data['components']['no_images'] && 
						($issue_settings['image_required'] == bo_perp_issue::NEVER);

					// Serialized serial numbers so they can be passed to the popup
					if($stock_item['controlled']) {
						$component['serials'] = $component['serial_numbers'];
						$component['serial_numbers'] = array(
							'serials'	=>	$component['serials'],
							'stock_id'	=>	$component['stock_id'],
							'required'	=>	true,
							'replacement_needed'	=>	$component['replace_parts']
						);
					}
				}
			} else {
				unset($data['components']);
			}
			$data += bo_perp_issue::read($claim_issue->get_issue_id());

			$data['new_component'] = array(
				'claim_issue_id'	=>	$claim_issue->get_claim_issue_id(),
				'issue_id'		=>	$claim_issue->get_issue_id(),
				'supplier_id'		=>	$claim_issue->get_claim()->get_supplier_id(),
				'client_id'		=>	$claim_issue->get_claim()->get_client_id(),
			);


			// Comments from tracker
			$data['tracker']['tr_description_options'] = 'ascii,230px,540px';
                        $data['tracker']['tr_reply_options'] = 'ascii,205px,540px';

			$tracker_bo = new tracker_bo();
			$tracker_bo->read($claim_issue->get_tracker_id());
			$data['tracker'] += $tracker_bo->data;
			if ($data['tracker']['num_replies']) array_unshift($data['tracker']['replies'],false);
			
			return $data;
		}

		/**
		*	Get common selectbox options
		*/
		protected static function get_select_box_options(ClaimIssue &$claim) {

			$options = array();

			$options['resolution'] = bo_perp_claim::$resolutions;
			$options['coverage_limit'] = bo_perp_issue::get_coverage_limit_list();
			$options['coverage'] = bo_perp_issue::get_coverage_list();
			$options['unit_of_measure_id'] = ExecMethod('perp_api.bo_perp_api.get_unit_of_measures');
			return $options;
		}

		/**
		*	Get common readonlys
		*/
		protected static function get_readonlys(ClaimIssue &$claim, &$data) {
			$readonlys = array();

			$edit = $admin = $GLOBALS['egw']->acl->check('claim', EGW_ACL_EDIT, 'perp_warranty');

			// Hide Comments tab if there are no comments
			if(!$data['tracker']['num_replies']) {
				$readonlys[self::$tabs]['comments'] = true;
			}

			// Hide credit column from all but admins
			$data['no_credit'] = !$admin;

			// Set columns to readonly if there is no choice
			$columns = array('return_parts', 'replace_parts');
			if(is_array($data['components'])) {
				foreach($data['components'] as $key => $component) {
					if(!is_array($component)) {
						continue;
					}

					// Check the Component cache
					$issue_settings = bo_perp_issue::get_stock_settings($claim->get_issue_id(), $component['stock_id']);
					foreach($columns as $column) {
						// No choice, or not admin
						if(in_array($issue_settings[$column], array(bo_perp_issue::ALWAYS, bo_perp_issue::NEVER)) || !$admin) {
							$readonlys["{$key}[{$column}]"] = true;
						}
						if($issue_settings['image_required'] == bo_perp_issue::NEVER || !$edit) {
							$readonlys["{$key}[new_image]"] = true;
						}
					}

					// Disable serial number button for non-controlled items
					if(!$component['serial_numbers']) {
						$readonlys["{$key}[serial_numbers]"] = true;
					}

					// Disable credit field for non-admins
					$readonlys["{$key}[credit]"] = !$admin;
				}
			}

			// Hide Add component line
			$data['no_add'] = !$admin;

			// History not implemented yet
			$readonlys[self::$tabs]['history'] = true;

			return $readonlys;
		}

		/**
		*	Get common javascript
		*/
		protected static function get_javascript() {
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->validate_file('.', 'warranty', 'perp_warranty', false);
		}

		/**
		*	Get the list of components that are claimable
		*	
		*	If this is a claim on one of our assemblies, and we can find the bill of materials,
		*	restrict to that bill of materials.  If we can't find the BOM, leave it wide open.
		*	If this is a claim to our suppliers, restrict to what we get from that supplier.
		*/
		public function get_component_list(&$query = array(), &$rows = array()) {

			// Limit to stock categories the issue applies to
			$stock_categories = bo_perp_issue::get_stock_categories($query['issue_id']);
			if(is_array($stock_categories)) {
				foreach($stock_categories as $category) {
					$query['cat_id'][] = $category['stock_category_id'];
				}
			}
			if(!$query['claim_issue']) {
				$claim_issue =& self::get_claim_issue($query['claim_issue_id']);
			}
		
			// Supplier?
			if($query['supplier_id']) {
				$count = bo_perp_catalog::get_supplier_catalog_list($query, $rows);
			} else {
				if(method_exists($claim_issue->get_claim(), 'get_work_order_keys')) {
					$work_order_data = $claim_issue->get_claim()->get_work_order_keys();
				}
				if($work_order_data) {
					// Found everything needed, direct the query to the work order's bill of materials
					$query['work_order_id']=       $work_order_data['work_order_id'];
					$query['detail_id']    =       $work_order_data['detail_id'];
					$bo_perp_work_order = new bo_perp_work_order();
					$count = $bo_perp_work_order->get_work_order_detail_bom($query, $rows);

					// Add in the stock item the claim is on, so they can return the whole thing?
					$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $work_order_data['stock_id']);
					if(is_array($query['cat_id']) && in_array($stock_item['stock_category'], $query['cat_id'])) {
						$count++;
						$rows[] = $stock_item;
					}

				} else {

					/**
					* Default / fallback settings.  
					* This is pretty poor, as it allows access to our entire stock list,
					* but it's better than an empty list.
					*/			
					$query['pseudo'] = false;
					$bo_perp_stock = new bo_perp_stock();
					$count = $bo_perp_stock->get_stock_list($query, $rows);
				}
			}

			return $count;
		}

		/**
		*	Update the ClaimIssue with information from the user
		*
		*	@param $claim_issue ClaimIssue
		*	@param $content
		*/
		protected static function update(ClaimIssue &$claim_issue, Array &$content) {
			
			// Update the tracker
			if($claim_issue->get_tracker_id()) {
				$result = ExecMethod('tracker.tracker_bo.save', $content['tracker']);
			}

			// Add a new component to a ClaimIssue
			if($content['new_component']) {
				$data['parent'] =& $claim_issue;
				$data['stock_id'] = $content['new_component'];
				$data['quantity_claimed'] = $content['quantity_claimed'] ? $content['quantity_claimed'] : 1;

				$component =& new Component($data);
				$claim_issue->add_component($component);
			}

			// Update components
			if($content['components']) {
				foreach($content['components'] as $row => $component) {
					if((string)$row == 'remove') {
						$component_obj =& $claim_issue->get_component_by_stock_id(key($component));
						$claim_issue->remove_component($component_obj);
						continue;
					}
					if(!is_array($component)) {
						continue;
					}

					// The read-only AJAX select will overwrite the stock_id with null, even if it's in preserve
					if(!$component['stock_id'] && $component['_stock_id']) {
						$component['stock_id'] = $component['_stock_id'];
					}

					$component_obj =& $claim_issue->get_component_by_stock_id($component['stock_id']);

					if(!$component_obj instanceof Component) {
						self::$messages[] = array(lang('Unable to find %1.  Weird.', ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['stock_id'])), 'warning');
						continue;
					}
					
					// Attach image
					if($component['new_image']) {
						$link_id = 'CLAIM_ISSUE-' . $claim_issue->get_claim_issue_id() . '-' . $component['stock_id'];
						$result = egw_link::link('perp_warranty', $link_id, egw_link::VFS_APPNAME, $component['new_image']);
					}

					// Update other numbers
					$component_obj->set_quantity_claimed($component['quantity_claimed']);
					$component_obj->set_return_parts($component['return_parts']);
					$component_obj->set_replace_parts($component['replace_parts']);
					$component_obj->set_credit($component['credit']);

					// Serial Numbers
					$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $component['stock_id']);
					if($stock_item['controlled']) {
						if(!($component['serial_numbers'])) {
							self::$messages[] = array(lang(
								'Missing serial number(s) for %1', 
								ExecMethod('perp_inventory.bo_perp_stock.link_title', $component['stock_id'])
							), 'warning');
							$component['serial_numbers'] = array();
						}
						try {
							$component_obj->set_serial_numbers($component['serial_numbers']);
						} catch (Exception $e) {
							self::$messages[] = array($e->getMessage, 'error');
						}
					}
				}
			}
		}
	}

?>
