<?php

	/**
	*	User interface for claims from clients
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.ui_perp_claim.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.ClientClaim.inc.php');
	require_once( EGW_INCLUDE_ROOT . '/perp_ar/inc/class.ui_perp_client.inc.php');

	include_once( EGW_INCLUDE_ROOT . '/perp_manufacturing/inc/class.bo_perp_work_order.inc.php');

	class ui_perp_client_claim extends ui_perp_claim {

		// Name of tabs, so we can disable some
		protected static $tabs = '';

		protected static $claim_issue_list_row = 'perp_warranty.client_claim_edit.row';

		public static function new_claim($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			// Get stock_id
			$stock_id = get_var('stock_id', 'ANY', $content['stock_id']);
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $stock_id);

			// Required fields
			$required_fields = array(
				'claim_date'		=>	lang('Claim Date'),
				'technician'		=>	lang('Technician'),
				'stock_id'		=>	lang('Stock Code'),
				'quantity_claimed'	=>	lang('Quantity Claimed'),
				'issue_id'		=>	lang('Issue'),
			);
			if($stock_item['controlled']) {
				$required_fields['serial_number'] = lang('Serial number');
			}

			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_claim.claim_list'
				));
			} elseif ($content['next']) {
				$got_required = true;
				foreach($required_fields as $field => $label) {
					if(!$content[$field]) {
						self::$messages[] = array(lang('Missing required field: %1', $label), 'error');
						$got_required = false;
					}
				}
				if($got_required) {
					$claim = $content;
			
					// If client is stuck using miles, we'll convert to km.
					if($content['travel_units'] == 'mi') {
						$claim['travel_claimed'] *= 1.6;
					}

					$claim['issues'] = array(
						// The product they have a problem with
						array(
							'issue_id'	=>	bo_perp_issue::PARENT,
							'components'	=>	array(
								array(
									'stock_id'		=>	$content['stock_id'],
									'quantity_claimed'	=>	$content['quantity_claimed'],
									'serial_numbers'	=>	array(
										array(
											'serial_number'	=>	$content['serial_number'],
											'quantity'	=>	$content['quantity_claimed']
										)
									)
								)
							)
						),
						// Damaged components that suffer from this problem
						array(
							'issue_id'	=>	$content['issue_id'],
							'components'	=>	array()
						)
					);
					try {
						$new_claim = new ClientClaim($claim);
						$result = $new_claim->write();
					} catch (egw_exception_no_permission $e) {
						self::$messages[] = array(lang('You do not have access to create new claims', 'error'));
					} catch (egw_exception $e) {
						self::$messages[] = array($e, 'error');
					}
					if(is_numeric($result) && $result > 0) {
						// Got a new claim number
						$GLOBALS['egw']->session->appsession('claim_' . $result, 'perp_warranty', $new_claim);

						// Record claim in history
						if($content['serial_number']) {
							bo_perp_stock_history::add_history(
								$content['stock_id'], 
								$content['serial_number'], 
								lang('Warranty claim #%1', $result),
								array('app' => 'perp_warranty', 'id' => 'CLAIM-' . $result),
								__LINE__, __FILE__
							);
						}

						if($stock_item['stock_type_flag'] == 'A') {
							// Redirect to the issue so user can add components
							$issues = $new_claim->get_issues();
							$claim_issue_id = $issues[1]->get_claim_issue_id();
							$GLOBALS['egw']->redirect_link('/index.php', array(
								'menuaction'	=>	'perp_warranty.ui_perp_claim_issue.edit',
								'claim_issue_id'=>	$claim_issue_id
							));
						} else {
							$GLOBALS['egw']->redirect_link('/index.php', array(
								'menuaction'	=>	'perp_warranty.ui_perp_client_claim.edit',
								'claim_id'	=>	$result
							));
						}
					}
				}
			}
			
			$data = $content;


			// Check for client ID
			$client_id = get_var('client_id', 'ANY', $content['client_id']);
			if($client_id) {
				$data['client_id'] = $client_id;
				$client_settings = bo_perp_client_warranty::get_client_settings($client_id);
				$data['travel_units'] = $client_settings['travel_units'];
			}
			if(!$data['travel_units']) {
				$data['travel_units'] = 'km';
			}
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			if(is_array($client_ids) && count($client_ids) > 0) {
				$data['client_id'] = array(
					'client_id'	=>	$client_ids
				);
			}

			// Claim date
			if(!$content['claim_date']) {
				$data['claim_date'] = time();
			} else {
				$data['claim_date'] = $content['claim_date'];
			}

			// Who did the work?
			if(!$content['technician']) {
				$data['technician'] = $GLOBALS['egw_info']['user']['fullname'];
			} else {
				$data['technician'] = trim($data['technician']);
			}

			// Check serial number(s)
			if($content['serial_number']) {
				$query = array(
					'stock_id'	=>	$stock_id,
					'serial_number'	=>	$content['serial_number']
				);
				$count = bo_perp_inventory::get_controlled_inventory_list($query, $result);
				if($count == 0) {
					self::$messages[] = array(lang('Unable to find serial number %1', $content['serial_number']), 'error');
				} elseif ($count == 1) {
					$data['serial_number'] = $content['serial_number'];
					$data['registration']['id'] = bo_perp_registration::get_registration($data['stock_id'], $data['serial_number']);
					$data['registration']['link'] = array(
						'app'	=>	'perp_warranty',
						'id'	=>	'REGISTRATION-' . $data['registration']['id']
					);
					$data['registration'] += bo_perp_registration::read($data['registration']['id']);
	
					if($result[0]['work_order_id']) {
						$data['options'] = self::get_build_options($result[0]);
					}
				} else {
					self::$messages[] = array(lang('Dangerous error.  Duplicate serial number "%1" for %2.', 
						$content['serial_number'],
						$stock_item['short_description']),
						'error'
					);
				}
			}
			
			$data['stock_id'] = array(
				'stock_id'	=>	$stock_id,
				'pseudo'	=>	false
			);


			// Toggle relevant sections
			if($data['stock_id']['stock_id']) {
				$readonly['stock_id'] = true;
			}
			if($stock_item['stock_type_flag'] == 'R') {
				$data['raw'] = true;
			} else {
				$data['quantity_claimed'] = 1;
			}
			if($stock_item['controlled'] == 2) {
				$data['serialized'] = true;
				$data['quantity_claimed'] = 1;
			}

			$select_box_options['issue_id'] = bo_perp_issue::get_issue_list();

			$preserve = $data;
			$preserve['stock_id'] = $stock_id;

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('New Claim');
			self::$template->read('perp_warranty.new_client_claim');
			self::$template->exec('perp_warranty.ui_perp_client_claim.new_claim', $data, $select_box_options, $readonly, $preserve);
		}

		protected static function get_build_options($controlled) {
			// Start at 1, so there's room for the title
			$build_options = array(0 => false);

			$bo_work_order = new bo_perp_work_order();
			$query = array(
				'work_order_id'	=>	$controlled['work_order_id'],
				'detail_id'	=>	$controlled['detail_id'],
				'stock_id'	=>	$controlled['stock_id'],
				'no_status'	=>	true,
				'no_readonlys'	=>	true,
				'no_options'	=>	true,
			);
			$options = array();
			$bo_work_order->get_work_order_detail_option_list($query, $options);
			foreach($options as $option) {
				$build_options[] = array(
					'name'	=>	$option['build_option_category_name'],
					'value'	=>	$option['options-value'][$option['value']]['build_option_name']
				);
			}
			return $build_options;
		}

		/**
		*	Handle an action on a claim or list of claims
		*/
		public static function handle_action($action, $claim_ids) {
			switch($action) {
				case 'issue_RMA':
					foreach($claim_ids as $claim_id) {
						$rma = bo_perp_rma::create_from_claim($claim_id);
						$rma->write();
						$result[$claim_id] = $rma->get_rma_id();
					}
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_warranty.ui_perp_rma.edit',
						'rma_id'	=>	$result[$claim_id]
					));
					break;
				case 'close_RMA':
					foreach($claim_ids as $claim_id) {
						$rma =& new RMA(bo_perp_rma::get_claim_rma_id($claim_id));
						$rma->set_closed_date(time());
						$result = $rma->write();
						self::$messages[] = array(lang('Closed %1', bo_perp_rma::link_title($rma->get_rma_id())), 'success');
					}
					break;

				case 'create_order':
					$redirect = true;
					foreach($claim_ids as $claim_id) {
						$claim =& self::get_claim($claim_id);
						$result[$claim_id] = bo_perp_client_claim::create_order($claim);
						if(!is_numeric($result[$claim_id])) {
							$redirect = false;
						}
					}
					if($redirect) {
						$GLOBALS['egw']->redirect_link('/index.php', array(
							'menuaction'	=>	'perp_orders.ui_perp_sales_order.edit',
							'order_id'	=>	$result[$claim_id]
						));
					} else {
						foreach($result as $claim_id => $message) {
							if(is_numeric($message)) {
								self::$messages[] = array(lang('Order created for claim %1', $claim_id), 'success');
							} else {
								self::$messages[] = array($message, 'error');
							}
						}
					}
					break;
				case 'enter_credit':
					foreach($claim_ids as $claim_id) {
						$claim =& self::get_claim($claim_id);
						$result[$claim_id] =& bo_perp_client_claim::enter_credit($claim);
						if (count($claim_ids) == 1) {
							// Store the unsaved claim in the session
							$GLOBALS['egw']->session->appsession('invoice', 'perp_ar', $result[$claim_id]);
							$GLOBALS['egw']->redirect_link('/index.php', array(
								'menuaction'	=>	'perp_ar.ui_perp_receivable_invoice.edit',
							));
						} else {
							$result[$claim_id] = $result[$claim_id]->write();
						}
					}
					foreach($result as $claim_id => $message) {
						if(is_numeric($message)) {
							self::$messages[] = array(lang('Invoice created for claim %1', $claim_id), 'success');
						} else {
							self::$messages[] = array($message, 'error');
						}
					}
				default:
					return parent::handle_action($action, (Array)$claim_ids);
			}
		}

		/**
		*	Edit an existing claim
		*/
		public static function edit($content = array()) {
			// Get claim
			$claim_id = get_var('claim_id', 'any', $content['claim_id']);
			$claim =& self::get_claim($claim_id, 'ClientClaim');
			if(!$claim) {
				self::$messages[] = array(lang('Unable to find claim'), 'error');
				return self::claim_list();
			}

			if(self::$debug) {
				echo 'Claim at top of ' . __METHOD__ . '<br />';
				_debug_array($claim->__toString());
			}

			// Save / Update
			$edit = $admin = $GLOBALS['egw']->acl->check('claim', EGW_ACL_EDIT, 'perp_warranty');

                        $buttons = array(
                                'save'          =>      true,
                        );
                        if($edit && $claim && $content) {
                                try {
                                        self::update_claim($claim, $content);
                                } catch (Exception $e) {
                                        self::$messages[] = array($e->getMessage(), 'error');
                                }
                        }

                        if(isset($content['cancel']) || !$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
                                $GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_claim.claim_list'
				));
                        }
                        if($content['save']) {
                                if($edit) {
                                        try {
                                                $result = $claim->write();
                                        } catch (Exception $ex) {
                                                self::$messages[] = array($ex->getMessage(), 'error');
                                        }
                                        if(!is_numeric($result )) {
                                                self::$messages[] = array($result, 'error');
                                        }

                                        $GLOBALS['egw']->session->appsession('claim_' . $claim_id, false, null);
                                } else {
                                        self::$messages[] = array(lang('Cannot save claim'), 'error');
                                }
                        }
			// Handle actions
			if($content['actions']) {
				foreach($content['actions'] as $action => $value) {
					if($action == 'resolve') {
					} else {
						self::handle_action($action, $value != 'pressed' ? $value : array($claim_id));
					}
				}
			}

			// Save to session
			$GLOBALS['egw']->session->appsession('claim_' . $claim_id, false, $claim);
			
			// Get data
                        $data = array();
                        self::get_display_data($claim, $data, $preserve);

                        // Supporting data
			$select_box_options = self::get_select_box_options($claim);

			// Read onlys
			$readonly = self::get_readonlys($claim, $data);

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			self::get_javascript();

			if(self::$debug) {
				echo __METHOD__ . ' outgoing data:';
				_debug_array($data);
			}

			$preserve = $data;

			if(count(self::$messages) > 0) {
                                $data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
                        }

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang(
				'Edit Claim %1 - %2', 
				$claim->get_claim_id(), 
				ExecMethod('perp_ar.bo_perp_client.link_title', $claim->get_client_id())
			);
			self::$template->read('perp_warranty.client_claim_edit');
			self::$template->exec('perp_warranty.ui_perp_client_claim.edit', $data, $select_box_options, $readonly, $preserve);
		}

		protected static function get_display_data(&$claim, &$data, &$preserve = array()) {
			parent::get_display_data($claim, $data, $preserve);

			if(!$data['registration_id'] && $data['item']['stock_id'] && $data['item']['serial_number']) {
				$data['registration_id'] = bo_perp_registration::get_registration($data['item']['stock_id'], $data['item']['serial_number']);
			}
			if($data['registration_id']) {
				$data['registration'] = array(
					'app'	=>	'perp_warranty',
					'id'	=>	'REGISTRATION-' . $data['registration_id']
				);
			}

			$data['issues']['template'] = self::$claim_issue_list_row;
			$data['issues']['get_rows'] = 'perp_warranty.ui_perp_client_claim.get_claim_issues';
		}

		protected static function get_select_box_options(&$claim) {
			$options = parent::get_select_box_options($claim);

			$options['assigned_to']	=	array(0 => lang('Not assigned')) + ExecMethod('tracker.tracker_bo.get_staff', $config['warranty_tracker']);

			return $options;
		}

		protected static function get_readonlys(ClientClaim &$claim, &$data) {
			$readonlys = parent::get_readonlys($claim);

			$edit = $admin = $GLOBALS['egw']->acl->check('claim', EGW_ACL_EDIT, 'perp_warranty');

			// Default everything to disabled
			$readonlys['issue_RMA']		=	true;
			$readonlys['close_RMA']		=	true;
			$readonlys['create_order']	=	true;
			$readonlys['enter_credit']	=	true;
			$readonlys['apply_credit']	=	true;
			$readonlys['close']		=	true;

			// If all the issues have a resolution, we can continue
			$all_resolved = true;
			$all_closed = true;
			$issues =& $claim->get_issues();
			foreach($issues as &$issue) {
				if($issue->get_issue_id() && $issue->get_tracker_id()) {
					$tracker = ExecMethod('tracker.tracker_bo.read', $issue->get_tracker_id());

					$all_closed = $all_closed && ($tracker['tr_status'] == tracker_bo::STATUS_CLOSED);
						
					if(trim($tracker['tr_resolution']) == '') {
						self::$messages[] = array(lang('Issue %1 needs a resolution before you can do any actions.', $issue->get_claim_issue_id()), 'warning');
						$all_resolved = false;
						break;
					} elseif($issue->get_issue_id() == bo_perp_issue::OTHER) {
						self::$messages[] = array(lang('Issue %1 needs an actual issue assigned before you can do any actions.', $issue->get_claim_issue_id()), 'warning');
						$all_resolved = false;
						break;
					}
				}
			}

			if($all_resolved) {
				$rma_id = bo_perp_rma::get_claim_rma_id($claim->get_claim_id());
				if($rma_id) {
					$rma = new RMA($rma_id);
					$data['actions']['rma_issue_date'] = $rma->get_issue_date();
					$data['actions']['rma_closed_date'] = $rma->get_closed_date();

					$readonlys['close_RMA'] = !$admin || $rma->get_closed_date();
				} else {
					$readonlys['issue_RMA'] = !$admin;
				}

				// Can create a sales order for replacement parts if there isn't already one
				$link_id = 'CLAIM-' . $claim->get_claim_id();
				$links = egw_link::get_links('perp_warranty', $link_id, 'perp_orders');
				if($links) {
					$link = egw_link::get_link(key($links));
			
					$data['actions']['order_date'] = $link['link_lastmod'];
				} else {
					$readonlys['create_order'] = !$admin;
				}

				// Can we give a credit?  Only if it hasn't been done already.
				$invoice_id = bo_perp_claim::get_claim_invoice_id($claim->get_claim_id());
				if($invoice_id) {
					$invoice = new Invoice($invoice_id);
					$data['actions']['credit_date'] = $invoice->get_originating_date();
				} else {
					$readonlys['enter_credit'] = false;
				}
			}

			if(!$all_closed) {
				$readonlys['close'] = !$admin;
			} else {
				// Claim is closed, everything is readonly
				$readonlys['issues']['rows']['__ALL__'] = true;
			}
			
			return $readonlys;
		}

		protected static function update_claim(ClientClaim &$claim, Array &$content) {

			// Update issues
			$tracker_bo = new tracker_bo();
			if(is_array($content['issues']['rows'])) {
				foreach($content['issues']['rows'] as $claim_issue) {
					if($claim_issue['claim_issue_id']) {
						$claim_issue_obj =& $claim->get_issue($claim_issue['claim_issue_id']);
					}
					if(!$claim_issue_obj) {
						$claim_issue_obj =& new ClaimIssue($claim_issue['claim_issue_id']);
						$claim->add_issue($claim_issue_obj);
					}

					if(isset($claim_issue['issue_id'])) {
						$claim_issue_obj->set_issue_id((int)$claim_issue['issue_id']);
					}
					if(isset($claim_issue['claim_amount'])) {
						$claim_issue_obj->set_claim_amount($claim_issue['claim_amount']);
					}

					// Some data actually belongs to the tracker, it will handle it
					$tracker_bo->read($claim_issue_obj->get_tracker_id());
					$tracker = array(
						'tr_id'		=>	$claim_issue_obj->get_tracker_id(),
						'tr_status'	=>	$claim_issue['status'],
						'tr_resolution'	=>	$claim_issue['resolution'],
						'tr_assigned'	=>	$claim_issue['assigned_to'],
					);
					$tracker_bo->save($tracker);

					if($claim_issue['delete']) {
						bo_perp_claim_issue::delete($claim_issue['claim_issue_id']);
					}
				}
			}

			// New issue?
			// This has to be after all the other updates because it will redirect
			if($content['new_issue']) {
				$new_issue = array(
					'claim'		=>	$claim,
					'issue_id'	=>	$content['new_issue'],
				);
				$claim_issue = new ClaimIssue($new_issue);
				$claim_issue->write();

				if($claim_issue->get_claim_issue_id()) {
					$claim->write();
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_warranty.ui_perp_claim_issue.edit',
						'claim_issue_id'	=>	$claim_issue->get_claim_issue_id()
					));
				}
			}
		}
	}
					
?>
