<?php

	class ui_perp_client_warranty {

		public $public_functions = array( 
			'category_list'	=>	true,
			'edit'		=>	true,
		);

		protected static $template;

		protected static $list_category_template = 'perp_warranty.client_category_list';
		protected static $edit_category_template = 'perp_warranty.client_category_edit';

		private static $debug = false;

		public function __construct() {
			self::$template = CreateObject('etemplate.etemplate');
		}

		/**
		*	Display an indented list of categories
		*	
		*	@param content array of data returned from the eTemplate
		*/
		public function category_list($content = array()) {
			$this->handle_category_actions($content);

			$data = $this->get_category_list_display_data();

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Client Warranty Categories');
			self::$template->read(self::$list_category_template);
			self::$template->exec(
				$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.category_list',
				$data, $select_box_options, $readonly, $preserve
			);
		}

		/**
		*	Handle action and button clicks from the list of stock
		*	categories.
		*/
		protected function handle_category_actions($content) {
			if(isset($content['add'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_warranty.ui_perp_client_warranty.edit'));
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/admin/index.php');
			} elseif (isset($content['nextmatch']['rows']['delete'])) {
				$category_id = key($content['nextmatch']['rows']['delete']);
				$message = ExecMethod('perp_warranty.bo_perp_client_warranty.delete_category', $category_id);
				if($message !== true) {
					$this->messages[] = array($message, 'error');
				}
			}
		}

		/**
		*	Get data required for displaying a list of sales categories
		*/
		protected function get_category_list_display_data() {

			// Set up nextmatch
			$data['nextmatch'] = array(
				'get_rows'	=>	'perp_warranty.bo_perp_client_warranty.get_category_list',
				'template'	=>	self::$list_category_template . '.row',
				'never_hide'	=>	false,
				'no_cat'	=>	true,
				'no_filter'	=>	true,
				'no_filter2'	=>	true,
			);
			return $data;
		}

		public function edit($content = array()) {
			$content['data']['color'] = $_POST['exec']['data']['color'];
			if(self::$debug) {
				echo __METHOD__ . ' incoming data:';
				_debug_array($content);
			}

			$category_id = reg_var('category_id');
			if(!$category_id && isset($content['category_id'])) {
				$category_id = $content['category_id'];
			}
			$content['category_id'] = $category_id;

			$parent_id = reg_var('parent_id');

			// Update
			if(isset($content['save'])) {
				$content_id = bo_perp_client_warranty::set_category($content);
				if(is_numeric($content_id)) {
					$this->messages[] = array(lang('Category saved'), 'success');
				} else {
					$this->messages[] = array($content_id, 'error');
				}
			} elseif (isset($content['delete'])) {
				ExecMethod('perp_warranty.bo_perp_client_warranty.delete_category', $category_id);
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_warranty.ui_perp_client_warranty.category_list'));
			} elseif (isset($content['cancel'])) {
				$GLOBALS['egw']->redirect_link('/index.php', array('menuaction'=>'perp_warranty.ui_perp_client_warranty.category_list'));
			}

			// Read category for display
			$data = $this->get_category_display_data($category_id);
			if(!$category_id && $parent_id) {
				$data['parent'] = $parent_id;
			}
		
			$select_box_options['travel_units'] = array(
				'km'	=>	'km',
				'mi'	=>	'mi'
			);

			$readonly = $this->get_category_readonlys($category_id);

			$preserve = $category;
			$preserve['category_id'] = $category_id;
			$preserve['old_parent'] = $category['parent'];

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$GLOBALS['egw_info']['flags']['app_header'] = ($category_id ? lang('Edit category'):lang('Add category'));
			self::$template->read(self::$edit_category_template);
			self::$template->exec(
				$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.edit', 
				$data, $select_box_options, $readonly, $preserve
			);
		}

		/**
		*	Get data for displaying or editing a stock category
		*	@param category_id The id number of the category to display
		*	@return array of data for display
		*/
		protected function get_category_display_data($category_id) {
			if($category_id) {
				$category = ExecMethod('perp_warranty.bo_perp_client_warranty.get_category', $category_id);
			} else {
				$category = array();
				$category['options-parent'] = ExecMethod('perp_warranty.bo_perp_client_warranty.get_category_list');
			}

			$data = $category;
			
			$data['data']['color'] = $GLOBALS['egw']->html->inputColor('exec[data][color]', $data['data']['color'], lang('Click to select a color'));

			$config = config::read('perp_warranty');
			$data['pdi']['no_pdi'] = !$config['use_pdi'];

			return $data;
		}

		/**
		*	Get readonlys for viewing or editing a category
		*/
		protected function get_category_readonlys($category_id) {
			$readonly['delete'] = !$GLOBALS['egw']->acl->check('client', EGW_ACL_DELETE, 'perp_warranty');

			$no_edit = !$GLOBALS['egw']->acl->check('client', EGW_ACL_EDIT, 'perp_warranty');

			$readonly['parent'] = $no_edit;
			$readonly['name'] = $no_edit;
			$readonly['description'] = $no_edit;
			$readonly['data']['color'] = $no_edit;
			$readonly['data']['icon'] = $no_edit;
			$readonly['save'] = $no_edit;
			
			return $readonly;
		}
	}
?>
