<?php

	/**
	*	User interface for dealing with known issues.
	*
	*	These are a list of things that you know can go wrong with your
	*	product, or the components of your product.
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_warranty/inc/class.bo_perp_issue.inc.php');

	class ui_perp_issue {

		// Functions we allow to be called directly by URL
		public $public_functions = array(
			'issue_list'	=>	true,
			'new_issue'	=>	true,
			'edit'		=>	true,
		);

		// Turn on debug information
		protected static $debug = false;

		// eTemplate object used for page generation
		protected static $template;

		/**
		* List of messages for the user.  Each message is in the form
		* array(message, severity)
		*/
		public static $messages = array();

		/**
		* A list of eTemplates used for issues
		*/
		protected $list_template	=	'perp_warranty.issue_list';
		protected $list_row_template	=	'perp_warranty.issue_list.row';
		protected $edit_template	=	'perp_warranty.issue_edit';

		protected static $tabs = 'details|claims|stock_categories|stock|replacement_parts|links';


		public function __construct() {
			if(!is_object($this->template)) {
				$this->template = new etemplate();
			}
		}

		/**
		*	Display a list of known issues
		*
		*	@param content Array of information returned by the eTemplate
		*/
		public function issue_list($content = array()) {
			
			// Deal with incoming
			$this->list_update($content);
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_warranty.index'
				));
			}
			
			// Set up for outgoing
			$data = $this->list_get_display_data();

			if(!isset($data['nextmatch']['filter'])) {
				$data['nextmatch']['filter'] = 'open';
			}

			$select_box_options = $this->list_get_options();
			$readonly = $this->list_get_readonlys();

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Issue List');
			$this->template->read($this->list_template);
			$this->template->exec('perp_warranty.ui_perp_issue.issue_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Create a new issue
		*
		*	@param content Array of information returned by the eTemplate
		*/
		public function new_issue($content = array()) {
			if(!$GLOBALS['egw']->acl->check('issue', EGW_ACL_ADD, 'perp_warranty')) {
				self::$messages[] = array('You do not have permission to add warranty issues', 'error');
				return $this->issue_list();
			}

			// Any special stuff needed here?  Not yet.

			return $this->edit($content);
				
		}

		/**
		*	Modify an existing issue
		*
		*	@param content Array of information returned by the eTemplate
		*/
		public function edit($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming:<br />';
				_debug_array($content);
			}

			if(!$GLOBALS['egw']->acl->check('issue', EGW_ACL_READ, 'perp_warranty')) {
				self::$messages[] = array(lang('You do not have access to warranty issues'), 'error');
				return $this->issue_list();
			}

			// Get the issue_id
			$issue_id = (int)get_var('issue_id', array('POST', 'GET'), $content['issue_id']);
			
			// Deal with incoming
			$this->update_issue($issue_id, $content);
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_issue.issue_list'
				));
			}

			// Get data for display
			$data = $this->get_display_data($issue_id);
			$data[self::$tabs] = $content[self::$tabs];

			$select_box_options = $this->get_edit_options();
			$readonly = $this->get_edit_readonly($data);
			$preserve = $data;
			$preserve['issue_id'] = $issue_id;
			
			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang($data['name']);
			$this->template->read($this->edit_template);
			$this->template->exec('perp_warranty.ui_perp_issue.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Get the list of known issues to display
		*/
		protected function list_get_display_data() {
			$data = array();

			$data['nextmatch'] = $GLOBALS['egw']->session->appsession('issue_list', $GLOBALS['egw_info']['flags']['currentapp']);
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array(
					'cat_app'		=>	'perp_inventory',
					'filter_label'		=>	'Status',
					'filter_no_lang'	=>	true,
					'no_filter2'		=>	true,
					'get_rows'		=>	$GLOBALS['egw_info']['flags']['currentapp'] . '.' . get_class($this) . '.get_issue_list',
					'template'		=>	$this->list_row_template,
					'default_cols'		=>	'perp_warranty_issue.issue_id,name,coverage,coverage_limit',
				);
			}
			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}
			return $data;
		}

		/**
		*	Get the selectbox options for the issue list
		*/
		protected function list_get_options() {
			$options = array();

			$options['coverage'] = bo_perp_issue::get_coverage_list();
			$options['coverage_limit'] = bo_perp_issue::get_coverage_limit_list();

			return $options;
		}

		/**
		*	Get the readonly restrictions for the issue list
		*/
		protected function list_get_readonlys() {
		}

		/**
		*	Deal with data returned from the list
		*
		*	@param content Array of information to deal with
		*/
		protected function list_update($content) {

			// Action buttons
			if($content['new_issue']) {
				self::handle_action('new', array());
			}
			if(is_array($content['nextmatch']['rows'])) {
				foreach($content['nextmatch']['rows'] as $action => $key) {
					// Just data
					if(is_numeric($action)) continue;

					$key = key($key);
					self::handle_action($action, array($key));
				}
			}
		}


		/**
		*	Handle an action on a list of issues
		*
		*	@param action String name of action to do
		*	@param issue_id Array of issues to apply the action to
		*/
		protected static function handle_action($action, array $issue_id) {
			switch ($action) {
				case 'new':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'	=>	'perp_warranty.ui_perp_issue.new_issue'
					));
				case 'delete':
					foreach($issue_id as $id) {
						bo_perp_issue::delete($id);
					}
					break;
			}
		}

		/**
		*	Store nextmatch parameters in the session, so the list is the same when the user comes back to it
		*
		*	@nextmatch
		*/
		public function get_issue_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			$GLOBALS['egw']->session->appsession('issue_list', 'perp_warranty', $query);
                        return bo_perp_issue::get_issue_list($query, $rows, $readonly);
		}

		/**
		*	Update an issue with new information
		*/
		protected function update_issue(&$issue_id, $data) {
			// Save
			$buttons = array('save');
			if(array_intersect(array_keys($data), $buttons)) {

				// Save details
				$result = bo_perp_issue::save($issue_id, $data);
				if(is_numeric($result)) {
					$issue_id = (int)$result;
				} else {
					self::$messages[] = array($result, 'error');
					return false;
				}

				// Stock categories
				if(is_array($data['stock_categories'])) {
					$result = bo_perp_issue::set_stock_categories($issue_id, $data['stock_categories']);
				}

				// Exceptions
				if(is_array($data['exceptions'])) {
					foreach($data['exceptions'] as $exception) {
						if($exception['base_issue_id']) {
							bo_perp_issue::set_stock_exception($exception);
						}
					}
				}
			}

			// Store 'all categories' filter in session
			$GLOBALS['egw']->session->appsession('issue_all_categories', 'perp_warranty', $data['stock_categories']['include_all']);

			// Exceptions
			if($data['new_exception']['add_exception']) {
				$result = bo_perp_issue::add_stock_exception($issue_id, $data['new_exception']['stock_id']);
			}
			if($data['exceptions']['delete_exception']) {
				$stock_id = key($data['exceptions']['delete_exception']);
				bo_perp_issue::delete_stock_exception($issue_id, $stock_id);
			}

			// Replacement Parts
			if($data['new_replacement']['add']) {
				$result = bo_perp_issue::add_replacement_part(
					$issue_id, 
					$data['new_replacement']['stock_id'],
					$data['new_replacement']['replacement_id'],
					$data['new_replacement']['quantity']
				);
			}
			if($data['replacements']['rows']['delete']) {
				list($replace_issue_id, $stock_id, $replacement_id) = explode('-', key($data['replacements']['rows']['delete']));
				bo_perp_issue::delete_replacement_part($replace_issue_id, $stock_id, $replacement_id);
			}

		}

		/**
		*	Get information for displaying an issue
		*
		*	@param issue_id
		*
		*	@return array of data for display
		*/
		protected function get_display_data($issue_id) {
			$data = bo_perp_issue::read($issue_id);
			
			$config = config::read('perp_ledger');
			$data['gl_currency'] = $config['gl_currency'];

			// Claims tab
			if($GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				$data['claims'] = array(
					'template'		=>	ui_perp_claim::$list_row_template,
					'get_rows'		=>	'perp_warranty.ui_perp_claim.get_claim_list',
					'cat_app'		=>	'perp_inventory',
					'filter_label'		=>	lang('Claims from'),
					'filter'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter'],
					'cat_id'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['cat_id'],
					'no_filter2'		=>	true,
					'col_filter'		=>	array(
						'issue_id'	=>	$issue_id
					),
					'hide_parent'		=>	true
				);
			}

			// Stock Categories tab
			
			$stock_categories = bo_perp_issue::get_stock_categories($issue_id, $GLOBALS['egw']->session->appsession('issue_all_categories'));
			if(count($stock_categories)) {
				// Shift one to allow for heading
				array_unshift($stock_categories, false);
				$data['stock_categories'] = $stock_categories;
			}
		
			if($GLOBALS['egw']->session->appsession('issue_all_categories')) {
				$data['stock_categories']['include_all'] = true;
			}

			// Exceptions tab
			$exceptions = bo_perp_issue::get_stock_exception_list($issue_id);
			if(count($exceptions)) {
				// Shift one to allow for heading
				array_unshift($exceptions, false);
				$data['exceptions'] = $exceptions;
			}
			$data['new_exception']['stock_id'] = array(
				'status'	=>	'E,P'
			);

			// Replacement parts
			$data['replacements'] = array(
				'issue_id'	=>	$issue_id,
				'cat_app'	=>	'perp_inventory',
				'no_filter'	=>	true,
				'no_filter2'	=>	true,
				'template'	=>	'perp_warranty.issue_edit.replacement_parts.row',
				'get_rows'	=>	'perp_warranty.bo_perp_issue.get_replacement_parts',
			);
			$data['new_replacement']['stock_id'] = array(
				'pseudo'	=>	false
			);
			$data['new_replacement']['replacement_id'] = array(
				'status'	=>	'E,P'
			);

			// Links
			$data['links']['to_app'] = 'perp_warranty';
			$data['links']['to_id'] = 'ISSUE-' . $issue_id;


			return $data;
		}

		/**
		*	Get selectbox options for displaying an issue
		*/
		protected function get_edit_options() {
			$options = array();

			$options['coverage'] = bo_perp_issue::get_coverage_list();
			$options['coverage_limit'] = bo_perp_issue::get_coverage_limit_list();
			
			// Claims
			$options += ui_perp_claim::list_get_options();

			// Stock categories tab
			$options['include_all'] = array(
				false	=>	lang('Only Included'),
				true	=>	lang('All')
			);
			$defaults = bo_perp_issue::get_default_option_list();
			$options['return_parts'] = $defaults;
			$options['replace_parts'] = $defaults;
			$options['retrofits_allowed'] = $defaults;
			$options['image_required'] = $defaults;

			return $options;
		}
	
		protected function get_edit_readonly(&$data) {
			$readonlys = array();

			// New issue
			if(!$data['issue_id']) {
				// Turn off all tabs except details
				foreach(explode('|', self::$tabs) as $tab_name) {
					$readonlys[self::$tabs][$tab_name] = true;
				}
				$readonlys[self::$tabs]['details'] = false;
			}
				
			
			
			if(!$GLOBALS['egw']->acl->check('issue', EGW_ACL_EDIT, 'perp_warranty')) {
				foreach($data as $key => $value) {
					$readonlys[$key] = true;
				}
				$data['no_add_exception'] = true;
			}

			// Claims tab
			if(!$GLOBALS['egw']->acl->check('claim', EGW_ACL_READ, 'perp_warranty')) {
				$readonlys[self::$tabs]['claims'] = true;
			}

			return $readonlys;
		}
	}
?>
