<?php

	/**
	*	User interface for registration of warrantied product
	*
	*	@author Nathan Gray
	*	@package perp_warranty
	*/


	class ui_perp_registration {
		
		public $public_functions = array(
			'registration_list'	=>	true,
			'edit'			=>	true
		);

		// Turn on handy debug messages
		protected static $debug = false;

		/**
		* List of messages for the user.  Each message is in the form
		* array(message, severity)
		*/
		public static $messages = array();

		protected static $template;

		/**
		*	A list of eTemplates used for registration
		*/
		protected static $list_template		=	'perp_warranty.registration_list';
		public static $list_row_template	=	'perp_warranty.registration_list.row';
		protected static $edit_template		=	'perp_warranty.registration_edit';

		public function __construct() {
			if(!is_object(self::$template)) {
				self::$template = new etemplate();
			}
		}

		/**
		*	Display a list of warranty registrations
		*
		*	@content array of information returned from the etemplate
		*/
		public function registration_list($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}


			// Deal with incoming
			self::list_update($content);
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_warranty.index'
				));
			}

			// Set up for outgoing
			$data = self::list_get_display_data();

			$select_box_options = self::list_get_options();
			$readonly = self::list_get_readonlys($data);

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Warranty Registrations');
			self::$template->read(self::$list_template);
			self::$template->exec('perp_warranty.ui_perp_registration.registration_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Deal with data returned from the list
		*/
		protected function list_update($content) {
			if($content['new_registration']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_registration.edit'
				));
			}
		}

		/**
		*	Get the list of claims to display
		*/
                protected function list_get_display_data() {
                        $data = array();

                        $data['nextmatch'] = $GLOBALS['egw']->session->appsession('registration_list', 'perp_warranty');
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array(
					'cat_app'		=>	'perp_inventory',
					'filter_label'		=>	'Client',
					'filter2_label'		=>	'Stock Item',
					'get_rows'		=>	'perp_warranty.ui_perp_registration.get_list',
					'template'		=>	self::$list_row_template,
					'col_filter'		=>	array(
					),
				);
			}

			return $data;
		}

		/**
                *       Get the selectbox options for the issue list
                */
		public function list_get_options() {

                        $options = array();
			$clients	=	array('' => lang('All')) + ExecMethod('perp_ar.bo_perp_client.get_client_list');
			$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
			$options['client_id'] = $clients;
			if($client_ids) {
				foreach($client_ids as $id) {
					$options['filter'][$id] = $clients[$id];
				}
			} else {
				$options['filter'] = $clients;
			}

			return $options;
		}

		/**
		*	Get additional readonlys for the claim list
		*/
		protected function list_get_readonlys($data) {
			$readonly = array();

			// Check for tracker being set
			$config = config::read('perp_warranty');
			if(!$config['tracker_queue']) {
				self::$messages[] = array(lang('perp_warranty has not been properly configured.  Missing tracker queue.'), 'error');
				$readonly['new_registration'] = true;
			}
		}

		/**
		*	Get a list of registrations
		*/
		public function get_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			// Put it in the session to preserve where they are when they come back
			$GLOBALS['egw']->session->appsession('registration_list', 'perp_warranty', $query);

			$query['options-filter2']	=	array('' => lang('all')) + ExecMethod('perp_inventory.bo_perp_stock.get_stock_list', array(
				'cat_id'	=>	$query['cat_id'],
				'pseudo'	=>	false,
				'status'	=>	array('P','R','D'),
				'controlled'	=>	'2',
			));

			$total = bo_perp_registration::get_registration_list($query, $rows, $readonlys);

			return $total;
		}

		/**
		*	View / Edit a warranty registration
		*
		*	@param content Array returned from the eTemplate on POST
		*/
		public static function edit($content = array()) {
			if(self::$debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}

			$data = array();
			$data['registration_id'] = reg_var('registration_id') ? reg_var('registration_id') : $content['registration_id'];

			// This is needed for the addressbook popup to correctly reload this page
			if(!$data['registration_id'] && $GLOBALS['egw']->session->appsession('registration_id', 'perp_warranty')) {
				$data['registration_id'] = $GLOBALS['egw']->session->appsession('registration_id', 'perp_warranty');
				$GLOBALS['egw']->session->appsession('registration_id', 'perp_warranty', null);
			}

			// Deal with incoming
			self::update($data, $content);
		
			if($content['save']) {
				unset($content['save']);
				try {
					$result = bo_perp_registration::save($content);
					if(is_numeric($result)) {
						$data['registration_id'] = $result;
						self::$messages[] = array(lang('Registered'), 'success');
					} else {
						self::$messages[] = array($result, 'error');
					}
				} catch (egw_exception $e) {
					self::$messages[] = array($e->getMessage(), 'error');
				}
			} elseif ($content['cancel']) {
				$GLOBALS['egw']->session->appsession('registration_id', 'perp_warranty', null);
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'	=>	'perp_warranty.ui_perp_registration.registration_list'
				));
			}

			// Keep what the user entered
			$data = array_merge($content, $data);

			// Set up for outgoing
			self::get_display_data($data);
			$select_box_options = self::get_select_box_options($data);
			$readonly = self::get_readonlys($data);

			if(count(self::$messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', self::$messages);
			}

			$preserve = $data;

			// This is needed for the addressbook popup to correctly reload this page
			$GLOBALS['egw']->session->appsession('registration_id', 'perp_warranty', $data['registration_id']);

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Warranty Registration');
			self::$template->read(self::$edit_template);
			self::$template->exec('perp_warranty.ui_perp_registration.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Update registration
		*/
		protected static function update($data, &$content) {
			// Check serial number
			if($content['stock_id'] && $content['serial_number']) {
				$previous_registration_id = bo_perp_registration::get_registration($content['stock_id'], $content['serial_number']);
				if($previous_registration_id != $data['registration_id']) {
					self::$messages[] = array(lang('%1 serial number %2 is already registered',
						ExecMethod('perp_inventory.bo_perp_stock.link_title', $content['stock_id']),
						$content['serial_number']
					), 'error');
					unset($content['serial_number']);
					unset($content['save']);
					return;
				}
			}

			// Create the addressbook entry, if it doesn't exist yet
			if(!$content['owner'] && $content['serial_number']) {
				$content['owner'] = self::create_new_contact($content);
			}
		}

		/**
		*	Get common display data for registration
		*/
		protected static function get_display_data(&$data = array(), &$preserve = array()) {
			$data = array_merge($data, bo_perp_registration::read($data['registration_id']));

			if(!$data['registration_id']) {
				$data['stock_id'] = array(
					'stock_id'	=>	$data['stock_id'],
					'pseudo'	=>	false,
					'status'	=>	'P,R,D',
					'controlled'	=>	'2',
				);

				$client_ids = ExecMethod('perp_ar.bo_perp_client.get_client_by_user_account');
				if($client_ids) {
					$data['client_id']['client_id'] = $client_ids;
				}

				$data['save_label'] = lang('Next');
			} else {
				$data['save_label'] = lang('Save');
			}
					
			return $data;
		}

		/**
		*	Get common selectbox options
		*/
		protected static function get_select_box_options(&$data) {
			$options = array();
			return $options;
		}

		/**
		*	Get common readonlys
		*/
		protected static function get_readonlys(&$data) {
			$readonlys = array();

			if($data['registration_id']) {
				$readonlys = array(
					'client_id'	=>	array('search' => true),
					'stock_id'	=>	array('search' => true),
					'registration_date'	=>	true,
					'start_date'	=>	!$GLOBALS['egw']->acl->check('registration', EGW_ACL_CUSTOM_1, 'perp_warranty'),
					'end_date'	=>	!$GLOBALS['egw']->acl->check('registration', EGW_ACL_CUSTOM_1, 'perp_warranty'),
				);
			}
			$readonlys['save'] = !$GLOBALS['egw']->acl->check('registration', EGW_ACL_EDIT, 'perp_warranty');

			return $readonlys;
		}

		/**
		*	Create a new addressbook record for the registrar to fill out
		*/
		protected static function create_new_contact($data) {
			// Create new contact
			$config = config::read('perp_warranty');
			$contact = array(
				'cat_id'	=>	$config['address_category'],
				'contact_tid'	=>	$config['contact_tid'],
			);
			$contact_id = ExecMethod('addressbook.addressbook_bo.save', $contact);

			// Link to new entry
			//$link = egw_link::link('perp_warranty', $link_id, 'addressbook', $contact_id);

			// Open contact for editing (won't work if we redirect for a new client)
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}

			$href = $GLOBALS['egw']->link('/index.php',array(
				'menuaction'    =>      'addressbook.addressbook_ui.edit',
				'contact_id'    =>      $contact_id
			));
			$GLOBALS['egw']->js->set_onload("window.open('$href','_blank','dependent=yes,width=850x440,height=450,scrollbars=yes,status=yes'); ");

			return $contact_id;
		}
	}
?>
