<?php

	/**
	*	User interface for Return Materials Authorization (RMA)
	*
	*
	*	@package perp_warranty
	*	@author Nathan Gray
	*/

	require_once( EGW_INCLUDE_ROOT . '/perp_api/inc/class.ui_perp_order.inc.php');

	class ui_perp_rma extends ui_perp_order {

		// Functions we allow to be called directly by URLs
		public $public_functions = array(
			'rma_list'	=>	true,
			'edit'		=>	true,
			'new_rma'	=>	true,
			'form'		=>	true,
			'receive'	=>	true
		);

		/**
		*       A list of eTemplates used for claims
		*/
		protected $order_list_template		=	'perp_warranty.rma_list';
		protected $order_list_row_template	=	'perp_warranty.rma_list.row';
		protected $order_line_template		=	'perp_warranty.rma_edit.row';

		protected $edit_template		=	'perp_warranty.rma_edit';

		public function __construct() {
			parent::__construct();

			$this->bo_order = new bo_perp_rma();
		}

		/**
		*	Get a list of RMAs.  This covers RMAs from clients to us as well as from us
		*	to our suppliers.
		*
		*	@param content Array of form information from the eTemplate
		*/
		public function rma_list($content = array()) {
			if($this->debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}

			// Deal with incoming
			self::order_list_update($content);
			if($content['cancel']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'    =>      'perp_warranty.ui_perp_warranty.index'
				));
			}

			// Set up for outgoing
			$data = self::list_get_display_data();

			$select_box_options = self::list_get_options();
			$readonly = self::list_get_readonlys($data);

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Return Materials Authorization List');
			$this->template->read($this->order_list_template);
			$this->template->exec('perp_warranty.ui_perp_rma.rma_list', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Deal with data returned from the list
		*
		*	@param content Array of form information from the eTemplate
		*/
		protected function order_list_update($content) {
			if($content['new_rma']) {
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'    =>      'perp_warranty.ui_perp_rma.new_rma'
				));
			}

			return parent::order_list_update($content);
		}

		/**
		*	Deal with actions on RMAs
		*/
		protected function handle_order_action($action, $rma_id) {
			switch($action) {
				case 'new':
					$GLOBALS['egw']->redirect_link('/index.php', array(
						'menuaction'    =>      'perp_warranty.ui_perp_rma.new_rma'
					));
					break;
				case 'cancel':
					$rma = new RMA($rma_id);
					return $rma->cancel($GLOBALS['egw_info']['user']['account_id']);
					break;
				default:
					return parent::handle_order_action($action, $rma_id);
					break;
			}
		}

		/**
		*	Set up outgoing data
		*/
		protected function list_get_display_data() {
			$data = parent::order_list_get_display_data();

			$data['nextmatch'] = $GLOBALS['egw']->session->appsession('rma_list', 'perp_warranty');
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array(
					'no_cat'		=>	true,
					'filter_label'		=>	lang('RMAs from'),
					'filter'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter'],
					'no_filter2'		=>	true,
					'get_rows'		=>	'perp_warranty.ui_perp_rma.get_rma_list',
					'selectcols'		=>	$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['nextmatch-' . $this->order_list_row_template],
					'template'		=>	$this->order_list_row_template,
				);
			}

			return $data;
		}

		/**
		*	Get the selectbox options for the RMA
		*/
		protected function list_get_options() {
			$options = array();
			$options['filter'] = array(
				''              =>      lang('All'),
				'supplier'      =>      lang('Suppliers'),
				'client'        =>      lang('Clients'),
			);
			$options['supplier_id'] =       ExecMethod('perp_ap.bo_perp_supplier.get_suppliers');
			$options['client_id']   =       ExecMethod('perp_ar.bo_perp_client.get_client_list');
			$options['issue_id']    =       bo_perp_issue::get_issue_list();
		
			return $options;
		}

		/**
		*	Get additional readonlys for the RMA list
		*/
		protected function list_get_readonlys($data) {
			$readonlys = parent::order_list_get_readonlys();
			
			return $readonly;
		}

		/**
		*	Get a list of RMAs, store settings for later
		*
		*	@nextmatch
		*/
		public function get_rma_list(&$query = array(), &$rows = array(), &$readonlys = array()) {
			// Put it in the session to preserve where they are when they come back
			$GLOBALS['egw']->session->appsession('rma_list', 'perp_warranty', $query);

			// Save changes to preferences
			if($query['filter'] != $GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter']) {
				$GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']['filter'] = $query['filter'];

				$GLOBALS['egw']->preferences->add('perp_warranty', 'claim_list', $GLOBALS['egw_info']['user']['preferences']['perp_warranty']['claim_list']);
				$GLOBALS['egw']->preferences->save_repository(true);
			}

			// Reset supplier / client columns
			if($query['filter'] == 'client') {
				$query['supplier_id'] = 0;
			} elseif ($query['filter'] == 'supplier') {
				$query['client_id'] = 0;
			}

			$total = parent::get_order_list($query, $rows, $readonlys);

			// Hide supplier / client columns
			if($query['filter'] == 'client') {
				$rows['no_supplier_id'] = true;
			} elseif ($query['filter'] == 'supplier') {
				$rows['no_client_id'] = true;
			}

			return $total;
		}

		/**
		*	Create a new, unattached RMA.  This is for non-warranty returns.
		*
		*	@param content Array of form information from the eTemplate
		*/
		public function new_rma($content = array()) {
			$rma = new RMA();
			$rma->set_our_reference_number($rma->suggest_order_number());
			
			// Default for a return to us
			$config = config::read('perp_warranty');
			$rma->set_supplier_id(0);
			$rma->set_ship_to_address($config['warranty_address']);
			$GLOBALS['egw']->session->appsession('order', 'perp_warranty', $rma);

			return $this->edit($content);
		}

		/**
		*	Edit a RMA list
		*
		*	@param content Array of form information from the eTemplate
		*/
		public function edit($content = array()) {

			if($this->debug) {
				echo __METHOD__ . ' incoming:';
				_debug_array($content);
			}

			$rma_id = get_var('rma_id', 'any', get_var('order_id', 'any', $content['rma_id']));
			$rma = $this->get_order('RMA', $rma_id);

			if(!$rma || !($rma instanceof RMA)) {
				$this->messages[] = array(lang('Cannot find RMA'), 'error');
				return self::rma_list();
			}

			if($content['cancel']) {
				$GLOBALS['egw']->session->appsession('order', 'perp_warranty', null);
				$GLOBALS['egw']->redirect_link('/index.php', array(
					'menuaction'    =>      'perp_warranty.ui_perp_rma.rma_list'
				));
			}

			// Deal with incoming
			$this->update_order($rma, $content);

			if ($content['save']) {
				try {
					$rma->write();
				} catch (Exception $e) {
					$this->messages[] = array($e->getMessage(), 'error');
				}
				$GLOBALS['egw']->session->appsession('order', 'perp_warranty', false);
			}

			// Store in session.  'order' because ui_perp_rma is extending ui_perp_order
			$GLOBALS['egw']->session->appsession('order', 'perp_warranty', $rma);

			// Set up for outgoing
			$data = self::get_display_data($rma);

			$select_box_options = self::get_selectbox_options($rma);
			$readonly = self::get_readonlys($rma);

			$this->get_javascript('perp_warranty.ajax_perp_rma');

			if(count($this->messages) > 0) {
				$data['message'] = ExecMethod('perp_api.ui_perp_api.make_message', $this->messages);
			}

			$preserve['rma_id'] = $rma_id;

			// Display
			$GLOBALS['egw_info']['flags']['app_header'] = lang('Return Materials Authorization %1', $data['our_reference_number']);
			$this->template->read($this->edit_template);
			$this->template->exec('perp_warranty.ui_perp_rma.edit', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Update an RMA with information from the eTemplate
		*/
		protected function update_order(RMA &$rma, &$content) {
			parent::update_order($rma, $content);

			if($content['closed_date']) {
				$rma->set_closed_date($content['closed_date']);
			}
		}

		/**
		*	Add a new line to the RMA
		*
		*	This is only allowed for non-warranty returns.
		*/
		protected function add_line(RMA &$rma, &$line_data) {
			$stock_item = ExecMethod('perp_inventory.bo_perp_stock.get_stock_item', $line_data['stock_id']);
			
			if(!$stock_item) {
				return;
			}
			$line_data['description'] = $stock_item['short_description'];

			// Serial numbers from $_POST
			if($_POST['exec']['new']['serial_numbers']) {
				$line_data['serial_numbers'] = array_keys(unserialize(stripslashes(urldecode($_POST['exec']['new']['serial_numbers']))));
			}

			$line =& new RMALine($rma, $line_data);
			$rma->add_line($line);
		}

		/**
		*	Update an RMA line
		*/
		protected function update_line(RMALine &$line, $content) {
			parent::update_line($line, $content);

			if(isset($content['quantity_authorized'])) {
				$result =& $this->check_result($line->set_quantity_authorized($content['quantity_authorized']));
			}

			if($content['serial_numbers']) {
				$content['serial_numbers'] = array_keys(unserialize(stripslashes(urldecode($content['serial_numbers']))));
				$line->set_serial_numbers($content['serial_numbers']);
			}
		}

		/**
		*	Update a LineDetail
		*/
		protected function update_line_detail(OrderLineDetail &$detail, Array $data) {
			switch (get_class($detail)) {
				case 'ActionDetail':
					$detail->set_quantity((float)$data['detail_name']);
					$detail->set_value($data['action']);

					// Remove ActionDetail with 0 quantity
					if($detail->get_quantity() == 0) {
						$detail->get_order_line()->remove_detail($detail->get_number());
					}

					// Check if we need to add another ActionDetail 
					$action_details =& $detail->get_order_line()->get_details('ActionDetail');
					foreach($action_details as &$detail) {
						$action_total += $detail->get_quantity();
					}
					if($action_total < $detail->get_order_line()->get_quantity_ordered()) {
						$new_detail =& new ActionDetail($detail->get_order_line(), array(
							'detail_name'   =>      $detail->get_order_line()->get_quantity_ordered() - $action_total,
						));
						$detail->get_order_line()->add_detail($new_detail);
					}
					break;
				case 'SerialNumberDetail':
					parent::update_line_detail($detail, $data);
					$action_detail =& $detail->get_action();

					if($action_detail && ($action_detail instanceof ActionDetail)) {
						$action_detail->set_value($data['action']);
					}
					break;
			}
		}

		/**
		*	Get data for display for editing an RMA
		*/
		protected function get_display_data(RMA &$rma) {
			$data = parent::get_display_data($rma);

			$data['rma_id'] = $rma->get_rma_id();

                        $data['nextmatch'] = $GLOBALS['egw']->session->appsession('rma_stock_' . $rma->get_rma_id(), 'perp_warranty');
			if(!is_array($data['nextmatch'])) {
				$data['nextmatch'] = array(
					'rma_id'		=>	$rma->get_rma_id(),
					'no_cat'		=>	true,
					'no_filter'		=>	true,
					'no_filter2'		=>	true,
					'get_rows'		=>	'perp_warranty.ui_perp_rma.get_rma_stock',
					'template'		=>	$this->order_line_template,
				);
			}

			$data['nextmatch']['order'] = $rma;
			return $data;
		}

		/**
		*	Get selectbox options for editing an RMA
		*/
		protected function get_selectbox_options(RMA $rma) {
			$options = parent::get_selectbox_options($rma);

			$options['action'] = array_fill_keys(ActionDetail::$action_list, false);
			foreach($options['action'] as $key => &$value) {
				$value = lang($key);
			}

			return $options;
		}

		/**
		*	Get readonlys for editing an RMA
		*/
		protected function get_readonlys(RMA &$rma) {
			$readonly = parent::get_readonlys($rma);
			
			if(!$GLOBALS['egw']->acl->check('rma', EGW_ACL_EDIT, 'perp_warranty') || $rma->get_closed_date()) {
				$readonly = array(
					'our_reference_number'	=>	true,
					'closed_date'	=>	true,
					'issue_date'	=>	true,
					'ship_to_address'	=>	true,
					'ship_from_address'	=>	true,
					'client_id'	=>	array('search' => true),
					'save'		=>	true
				);
			}

			if(!$GLOBALS['egw']->acl->check('report_rma_form', EGW_ACL_READ, 'perp_warranty')) {
				$data['no_form'] = true;
			}

			return $readonly;
		}

		/**
		*	Get a list of items on an RMA
		*/
		public function get_rma_stock(&$query = array(), &$rows = array(), &$readonlys = array()) {

			// Store parameters in session
			$GLOBALS['egw']->session->appsession('rma_stock_' . $query['rma_id'], 'perp_warranty', $query);

			$count = parent::get_order_lines($query, $rows, $readonlys);

			$no_actions = !($query['order']->get_actual_delivery_date());

			foreach($rows as $number => &$row) {
				$item =& $query['order']->get_line($row['line_number']);

				// Screw around with serial numbers to get them into a form ready for the popup
				if($item && $item->get_serial_numbers()) {
					$row['serial_numbers'] = urlencode(serialize(array_fill_keys($item->get_serial_numbers(), 1)));
				}

				// Hide ActionDetails if there are SerialNumberDetails
				if($row['details']['SerialNumberDetail']) {
					unset($row['details']['ActionDetail']);
					$row['details']['SerialNumberDetail']['no_actions'] = $no_actions;
				}

				// This causes everything in the row to be readonly
				if($query['order']->get_closed_date()) {
					$readonlys[$number + 1] = true;
				} else {
					unset($readonlys[$number + 1]);
				}
			}

			// Can't change serial numbers for an RMA based on a claim
			$readonlys['serial_number_button'] = $query['order']->get_claim_id() || $query['order']->get_actual_delivery_date();

			return $count;
		}

		protected function get_javascript($ajax_callback) {
			parent::get_javascript($ajax_callback);

			$GLOBALS['egw']->js->validate_file('', 'rma', 'perp_warranty');
			$GLOBALS['egw']->js->set_onload('rma_setup();');
		}

		/**
		*	Mark an RMA as received.  It will be set aside for verification and inspection.
		*/
		public function receive() {
			$rma_id = get_var('rma_id', 'any');
			$box_count = (int)get_var('box_count', 'any');
			$rma = $this->get_order('RMA', $rma_id);

			if($rma->can_receive() && $rma->receive($box_count)) {
				$this->messages[] = array(lang('Return Materials Authorization %1 received.', $rma->get_our_reference_number()), 'success');
			} else {
				$this->messages[] = array(lang('Unable to receive Return Materials Authorization %1.', $rma->get_our_reference_number()), 'error');
			}

			return $this->rma_list();
		}
	}
?>
