<?php
	/**
	* @class ui_wizard
	* @brief Wizard is an application that is built to be added to eGroupware. It currently resides
	* in eGroupware addons as part of pERP. It allows others to extend the Wizard to build there own
	* wizards for what ever perpose. It was developed initially for the creation of pERP Setup Wizard 
	* @author John Trudel
	* @version 1.0
	* @date September 2008
	*/
	
	class ui_wizard {

		public $public_functions = array( 
			'index'		=>	true
		);

		public $debug = false;
		public $wizard_title;
		public $wizard_app_name;
		public $wizard_class_name;
		public $steps_dir;
		
		private $tpl;
				
		/**
		*  Wizard Constructor
		*  
		*/
		public function __construct() {
			$GLOBALS['egw_info']['flags']['app_header'] = $this->wizard_title;
			$this->tpl =& CreateObject('etemplate.etemplate', 'wizard.index');			
			
			//create the wizard template object
			$this->tpl =& CreateObject('etemplate.etemplate', 'wizard.index');			
			
			//****DOCS "explain in the docs that the steps of the wizard must be in a folders named -steps- in the inc folder of the application" DOCS****//
			$this->steps_dir = EGW_INCLUDE_ROOT.'/'.$this->wizard_app_name.'/inc/steps/';
		}
				
		/**
		*  Wizard Index Function
		*  
		* @param content content holds reload data
		*/
		public function wizard_index($content = array()) {
			
			if($this->debug){
				echo '<br />--BEGIN "wizard_index" debug--<br /><br />';
				echo 'content:';
				_debug_array($content);
				echo '<br />';
			}
			
			if(isset($_GET['step'])){
				$current_step = $_GET['step'];
			}elseif(isset($content['current_step'])){
				$current_step = $content['current_step'];
			}else{
				$current_step = 1;
			}
			
			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->validate_file('wizard', 'common', 'wizard');			
			
			//add the calling child wizards translation file
			$GLOBALS['egw']->translation->add_app($this->wizard_app_name);
			
			$all_steps = $this->get_steps_list();
			
			if($this->debug){
				echo 'current_step:';
				_debug_array($current_step);
				echo '<br />';
				echo 'all_steps:';
				_debug_array($all_steps);
				echo '<br />';
			}
			
			//set the session with the step count - the step count does not include section splash pages
			$GLOBALS['egw']->session->appsession('step_count', $this->wizard_app_name, $this->get_step_count($all_steps));	
			
			//set the session with the current working step to later check if completed
			$GLOBALS['egw']->session->appsession('current_working_step', $this->wizard_app_name, $all_steps[$current_step]);
					
			//set the session with a set of steps for the sidebox hooks
			$GLOBALS['egw']->session->appsession('steps', 'wizard', $all_steps);
			
			//template vars
			$data['step_title'] = $all_steps[$current_step];
			$data['step_out_steps'] = $current_step.'/'.count($all_steps);
			$data['step_forward'] = $this->wizard_app_name.'.'.$this->wizard_class_name.'.index&step='.($current_step+1);
			$data['step_back'] = $this->wizard_app_name.'.'.$this->wizard_class_name.'.index&step='.($current_step-1);
			$data['right'] = (($current_step+1) > count($all_steps))?'1rightarrow_grey.png':'1rightarrow.png';
			$data['left'] = (($current_step-1) <= 0)?'1leftarrow_grey.png':'1leftarrow.png';
			if($this->debug){
				echo 'template vars:';
				_debug_array($data);
				echo '<br />';
			}
			
			
			//disable step arrow if at begining or end
			if(($current_step+1) > count($all_steps)){
				unset($data['step_forward']);
			}
			if(($current_step-1) <= 0){
				unset($data['step_back']);
			}
			
			
			$permissions = $GLOBALS['egw']->session->appsession('permissions', $GLOBALS['egw_info']['flags']['currentapp']);
			
			$class_exploded = explode("_",$all_steps[$current_step]);
			if(($class_exploded[1]) != 0){
				if(($class_exploded[2]) != 0){
					if(!$permissions[$all_steps[$current_step]]['perms']){
						$permission_fail = true;
					}				
				}
			}
			
			
			//Build The Current Step
			
			//get required child wizard step class file
			//check to see if current wizard has steps, if not, require_once class.no_steps.php
			if($all_steps[$current_step] != 'no_steps'){
				//if the step's permissions have failed, then require_once class.permission_fail.php
				($permission_fail)?require_once(EGW_INCLUDE_ROOT.'/wizard/inc/class.permission_fail.php'):require_once($this->steps_dir.$all_steps[$current_step].'.php');
				$class = ($permission_fail)?'permission_fail':$all_steps[$current_step];
							
			}else{
				require_once(EGW_INCLUDE_ROOT.'/wizard/inc/class.no_steps.php');
				$class = 'no_steps';
			}		
			$current_step_object = new $class();
				
			//get steps info
			$content['wizard_app_name'] = $this->wizard_app_name;
			$content['all_steps'] = $all_steps;
			$content['current_step'] = $current_step;
			
			//create step//
			$step_data = array();
			$current_step_object->get_step_data($content, $data, $select_box_options, $readonly, $preserve);			
			if($this->debug){
				echo 'select_box_options vars:';
				_debug_array($select_box_options);
				echo '<br />';
				echo 'readonly vars:';
				_debug_array($readonly);
				echo '<br />';
				echo 'preserve vars:';
				_debug_array($preserve);
				echo '<br />';
				echo '<br />--END "wizard_index" debug--<br /><br />';
			}
			
			//call template to render step
			$this->tpl->exec($this->wizard_app_name.'.'.$this->wizard_class_name.'.index', $data, $select_box_options, $readonly, $preserve);
			
		}
		
		
		
		/**
		* Wizard Complete Step Function
		* 
		* @brief this function calls the complete_step function in the current working step, the
		* step's complete_step function runs to see if the criteria has been met to complete the step
		* upon return the step_status session is set to store the completed steps
		*/
		public function  complete_step($content = array()){
			
			$process_this_current_step = $GLOBALS['egw']->session->appsession('current_working_step', $content['wizard_app_name']);
			$exploded_step = explode("_",$process_this_current_step);
			if($exploded_step[2] != 0){			
				
				if($process_this_current_step){
					require_once($content['steps_dir'].$process_this_current_step.'.php');
					$class = $process_this_current_step;
					$process_this_current_step_object = new $class();
					$process_this_current_step_object->complete_step($content);				
				}				
			}
			$wizard_config = config::read($content['wizard_app_name']);
			$steps_status_array = $wizard_config['steps_status'];
			
			$GLOBALS['egw']->session->appsession('steps_status', $content['wizard_app_name'], $steps_status_array);
		}
		
		/**
		* Wizard Build Permission
		* 
		* @brief asks all steps what their permissions are, the results are an array
		* that contains ['perms'](boolean) and ['reasons'](string) if ['perms'] is false for each step
		* the array is saved in a session called 'permissions' under the wizard app name
		*/
		public function  build_permissions(){
			
			$all_steps = $this->get_steps_list();
			
			if(count($all_steps)){
				foreach($all_steps as $step) {
					$exploded_step = explode("_",$step);
					if($exploded_step[2] != 0){
						require_once($this->steps_dir.$step.'.php');
						$class = $step;
						$current_step_object = new $class();
						
						if($step_premission = $current_step_object->get_permission()){
							$perms_array[''.$step.''] = $step_premission;	
						}
						
						
					}
				}
				
				$GLOBALS['egw']->session->appsession('permissions', $this->wizard_app_name, $perms_array);
			}		
		}		
		
		/**
		* Wizard Generate Next Step Function
		* 
		* @brief this function returns the number for the next step
		*/
		public function  generate_next_step($content = array()){
			$all_steps = $content['all_steps'];
			
			$steps_status = $GLOBALS['egw']->session->appsession('steps_status', $content['wizard_app_name']);
			$permissions = $GLOBALS['egw']->session->appsession('permissions', $content['wizard_app_name']);
								
			if(!empty($steps_status)){
				$keys = array_keys($steps_status);
				foreach($all_steps as $step) {
					$exploded_step = explode("_",$step);
					if($exploded_step[2] != 0){
						if(!in_array($step,$keys)){							
							$perms = $permissions[$step];							
							if($perms['perms']){
								$flipped = array_flip($all_steps);
								$next_step = $flipped[$step];
								break;
							}							
						}											
					}
				}
				if(empty($next_step)){
					$flipped = array_flip($all_steps);
					$next_step = $flipped['_0_0_wizard'];
				}
				
			}else{
				$flipped = array_flip($all_steps);
				$next_step = 1;
			}
			
			return $next_step;
		}		
		
		/**
		* Wizard Blink Function
		* 
		* @brief this function renders a blinking cursor next to the date of eGroupware
		* the cursor is a link to the next permissible step
		* 
		* @param content content needs to include wizard_icon, next_step_link, wizard_title
		* @todo needs to be looked at to fix a double blink problem. Currently implemented a hack to function correctly 
		*/
		public function wizard_blink_method($content = array()) {
			/*
			$is_blink = $GLOBALS['egw']->session->appsession('is_blink', $content['wizard_app_name']);
			if(!$is_blink){
				$is_blink = true;
				$GLOBALS['egw']->session->appsession('is_blink', $content['wizard_app_name'], $is_blink);
			}elseif($is_blink){
				$GLOBALS['egw']->framework->topmenu_info_icon($content['wizard_app_name'],$content['wizard_icon'],$content['next_step_link'],true,$content['wizard_title']);
			}
			*/
		}
					
		/**
		* Wizard Step List Function
		* 
		* @brief this function looks in the step folder and returns an array of the names of the steps
		*/
		public function get_steps_list($content = array()){
			
			//generate array of steps
			foreach(scandir(isset($content['steps_dir'])?$content['steps_dir']:$this->steps_dir) as $file) {
			 if ($file[0]=='.') continue;
			 	if(substr($file, 0,1) == '_'){
			 		if($GLOBALS['egw']->session->appsession('is_pre_step', isset($content['wizard_app_name'])?$content['wizard_app_name']:$this->wizard_app_name)){
			 			if(substr($file, 1,1) == '0' && substr($file, 3,1) != '0' ){
			 				$all_steps[] = strstr($file, '.', true);
			 			}	
					}else{
						
						if(substr($file, 1,1) == '0' && substr($file, 3,1) != '0' ){
			 				
			 			}else{
			 				$all_steps[] = strstr($file, '.', true);
			 			}	
					}
			 		
			 	}
				
			}
			
			if(count($all_steps)){
				natsort($all_steps);		
				array_unshift($all_steps, false);
				unset($all_steps[0]);
			}else{
				$all_steps[1] = 'no_steps';
			}
			
			return $all_steps;
		}	
		
		/**
		* Wizard Step Count Function
		* 
		* @brief this function returns the count of actionable steps
		* actionable steps do not include the 0 splash steps
		* 
		* @param all_steps array of all the steps names
		*/
		public function get_step_count($all_steps){
			if(count($all_steps)){
				foreach($all_steps as $step) {
					$exploded_step = explode("_",$step);
					if($exploded_step[2] != 0){
						$count++;									
					}
				}
				return $count;
			}else{
				return 0;
			}			
		}	
		
		/**
		* Wizard Array Flipper Function
		* 
		* @brief this function flips the $all_steps array and runs it against the next allowable step
		* this is done to return the step number
		* 
		* @param array array of all the steps names
		* @param step next allowable step
		*/
		function array_flipper($array,$step) {
			$array = array_flip($array);
			return $array[$step];
	    }
		
		/**
		* Wizard Make Message Function
		* 
		* @brief this function creates a formatted message for no_steps and permission_fail
		* 
		* @param message messages to format
		* @param type type of error
		*/
		public function make_message($message, $type = '') {
			
			if(file_exists(EGW_SERVER_ROOT . '/' . 'wizard' . '/'. 'templates' . '/' . 'default'. '/' . 'app.css')) {
				$GLOBALS['egw_info']['flags']['css'] .= '</style><link href="'.$GLOBALS['egw_info']['server']['webserver_url'] . "/wizard/templates/default/app.css" . '" type=text/css rel=StyleSheet><style type="text/css">';
			}
			
			if(is_array($message)) {
				$msg = '';
				$list = array();
				foreach($message as $text => $type) {
					if(is_array($type)) {
						$text = $type[0];
						$type = $type[1];
					} 
					if(trim($text) != '') {
						$list[$type][] = '<li>' . $text . '</li>';
					}
				}
				foreach($list as $type => $messages) {
					$msg .= "<ul class='message' type='$type'>";
					$msg .= implode(' ', $messages);
					$msg .= '</ul>';
				}
				return $msg;
			} else {
				return "<ul class='message' type='$type'><li>$message</li></div>";
			}
		}
		
	}
?>
